Package: vegan
Title: Community Ecology Package
Version: 2.7-5
Authors@R: c(person("Jari", "Oksanen", role=c("aut","cre"),
                    email="jhoksane@gmail.com",
		    comment = c(ORCID="0000-0001-7102-9626")),
	   person("Gavin L.", "Simpson", role="aut",
	            email="ucfagls@gmail.com",
		    comment = c(ORCID = "0000-0002-9084-8413")),
	   person("F. Guillaume", "Blanchet", role="aut"),
	   person("Roeland", "Kindt", role="aut"),
	   person("Pierre", "Legendre", role="aut"),
	   person("Peter R.", "Minchin", role="aut"),
	   person("R.B.", "O'Hara", role="aut"),
	   person("Peter", "Solymos", role="aut"),
	   person("M. Henry H.", "Stevens", role="aut"),
	   person("Eduard", "Szoecs", role="aut"),
	   person("Helene", "Wagner", role="aut"),
	   person("Matt", "Barbour", role="ctb"),
	   person("Michael", "Bedward", role="aut"),
	   person("Henrik", "Bengtsson", role="ctb"),
	   person("Ben", "Bolker", role="aut"),
	   person("Daniel", "Borcard", role="aut"),
	   person("Tuomas", "Borman", role="ctb"),
	   person("Gustavo", "Carvalho", role="aut"),
	   person("Michael", "Chirico", role="ctb"),
	   person("Miquel", "De Caceres", role="aut"),
	   person("Sebastien", "Durand", role="aut"),
	   person("Heloisa Beatriz Antoniazi", "Evangelista", role="aut"),
	   person("Rich", "FitzJohn", role="ctb"),
	   person("Michael", "Friendly", role="ctb"),
	   person("Brendan","Furneaux", role="ctb"),
	   person("Geoffrey", "Hannigan", role="aut"),
	   person("Isaac", "Heida", role="ctb"),
	   person("Mark O.", "Hill", role="aut"),
	   person("Leo", "Lahti", role="aut"),
	   person("Cameron", "Martino", role="aut"),
	   person("Dan", "McGlinn", role="ctb"),
	   person("Marie-Helene", "Ouellette", role="aut"),
	   person("Eduardo", "Ribeiro Cunha", role="aut"),
	   person("Tyler", "Smith", role="aut"),
	   person("Adrian", "Stier", role="aut"),
	   person("Cajo J.F.", "Ter Braak", role="aut"),
	   person("James", "Weedon", role="aut"))
Depends: permute (>= 0.9-0), R (>= 4.1.0)
Suggests: parallel, knitr, markdown
Imports: MASS, cluster, lattice, mgcv
VignetteBuilder: utils, knitr
Description: Ordination methods, diversity analysis and other
  functions for community and vegetation ecologists.
License: GPL-2
BugReports: https://github.com/vegandevs/vegan/issues
URL: https://vegandevs.github.io/vegan/,
        https://github.com/vegandevs/vegan
NeedsCompilation: yes
Packaged: 2026-05-25 14:59:23 UTC; jarioksa
Author: Jari Oksanen [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7102-9626>),
  Gavin L. Simpson [aut] (ORCID: <https://orcid.org/0000-0002-9084-8413>),
  F. Guillaume Blanchet [aut],
  Roeland Kindt [aut],
  Pierre Legendre [aut],
  Peter R. Minchin [aut],
  R.B. O'Hara [aut],
  Peter Solymos [aut],
  M. Henry H. Stevens [aut],
  Eduard Szoecs [aut],
  Helene Wagner [aut],
  Matt Barbour [ctb],
  Michael Bedward [aut],
  Henrik Bengtsson [ctb],
  Ben Bolker [aut],
  Daniel Borcard [aut],
  Tuomas Borman [ctb],
  Gustavo Carvalho [aut],
  Michael Chirico [ctb],
  Miquel De Caceres [aut],
  Sebastien Durand [aut],
  Heloisa Beatriz Antoniazi Evangelista [aut],
  Rich FitzJohn [ctb],
  Michael Friendly [ctb],
  Brendan Furneaux [ctb],
  Geoffrey Hannigan [aut],
  Isaac Heida [ctb],
  Mark O. Hill [aut],
  Leo Lahti [aut],
  Cameron Martino [aut],
  Dan McGlinn [ctb],
  Marie-Helene Ouellette [aut],
  Eduardo Ribeiro Cunha [aut],
  Tyler Smith [aut],
  Adrian Stier [aut],
  Cajo J.F. Ter Braak [aut],
  James Weedon [aut]
Maintainer: Jari Oksanen <jhoksane@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-25 16:50:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-25 18:23:46 UTC; unix
Archs: vegan.so.dSYM

## compiled code
useDynLib(vegan, .registration = TRUE)

## Export regular function names (no dots)

export(CCorA, MOStest, RsquareAdj, SSarrhenius, SSgitay, SSgleason, SSlomolino,
adipart, adonis2, anosim, beals, betadisper, betadistances, betadiver,
bgdispersal, bioenv, bioenvdist, bstick, cIndexKM, ca, calibrate, capscale,
cascadeKM, cca, chaodist, contribdiv, clamtest, commsim,
cutreeord, dbrda, decobackstand, decorana, decostand, designdist, designdist2,
dist2xy, coverscale, dispweight, dispindmorisita, distconnected,
diversity, downweight, drarefy, eigengrad, eigenvals, envfit,
estaccumR, estimateR, eventstar, factorfit, fisherfit, fitspecaccum,
gdispweight, goodness, hiersimu, indpower, inertcomp, initMDS,
intersetcor, isomapdist, isomap, linestack, mantel, meandist,
metaMDSdist, metaMDSiter, metaMDSredist, MDSaddpoints, MDSrotate, metaMDS,
monoMDS, mrpp, msoplot, mso, multipart, make.commsim, nestedbetajac,
nestedbetasor, nestedchecker,
nesteddisc, nestedn0, nestednodf, nestedtemp, nullmodel, oecosimu, smbind,
optspace,
ordiareatest, ordiR2step,
ordiarrows, ordiArrowMul, ordiArrowTextXY, ordibar, ordicloud,
ordicluster, ordiellipse, ordigrid,
ordihull, ordilabel, ordiplot, ordipointlabel, ordiresids,
ordisegments, ordispider, ordisplom, ordistep, ordisurf,
orditorp, ordixyplot, ordiYbar,
pca, pco, pcnm, permatfull, permatswap, permulattice, permustats, permutest,
poolaccum, postMDS, prc, prestondistr, prestonfit, procrustes,
protest, radfit, radlattice, rankindex, rarefy, rarecurve, rareslope,
raupcrick, rda, renyiaccum, renyi, rrarefy, scores,
showvarparts, simper, simpson.unb, spandepth, spantree, specaccum, specslope,
specnumber, specpool2vect, specpool, spenvcor, "sppscores<-", sppscores,
stepacross, stressplot, swan, tabasco, taxa2dist, taxondive, tolerance,
treedist, treedive, treeheight, tsallisaccum, tsallis, varpart,
vectorfit, vegandocs, vegdist, avgdist, vegemite, veiledspec, wascores,
wcmdscale, wisconsin)
## export pasteCall for 'permute'
export(pasteCall)
## export anova.cca for 'BiodiversityR': this should be fixed there
export(anova.cca)

## export regular functions with dot names

export(as.fisher, as.preston, as.rad, fieller.MOStest,
fisher.alpha, kendall.global, kendall.post, make.cepnames,
mantel.correlog, mantel.partial, no.shared, rad.lognormal, rad.null,
rad.preempt, rad.zipf, rad.zipfbrot, read.cep,
vif.cca)

## Export panel functions
export(panel.ordi, panel.ordiarrows, panel.ordi3d, prepanel.ordi3d)

## Export .Defunct functions (to be removed later)
export(commsimulator)
export(as.mlm)
export(orditkplot)
S3method(as.mlm, cca)

## Deprecated functions & methods
export(as.mcmc.oecosimu)
export(as.mcmc.permat)
export(adonis)

## do NOT export the following internal functions

## export(orderingKM, ordiArgAbsorber, ordiArrowMul,
## ordiGetData, ordimedian, ordiNAexclude, ordiNApredict,
## ordiParseFormula, ordiTerminfo, pregraphKM, simpleRDA2, varpart2,
## varpart3, varpart4, veganCovEllipse, veganMahatrans)

## Imports

## define S3 methods for these stats functions
importFrom(stats, AIC, TukeyHSD, add1, alias, anova, as.hclust, as.ts,
           biplot, coef, confint, cooks.distance, cophenetic, density,
           deviance, df.residual, drop1, extractAIC, fitted,
           hatvalues, logLik, model.frame, model.matrix, nobs,
           predict, profile, qqnorm, reorder, residuals, rstandard,
           rstudent, screeplot, sigma, simulate, SSD, update, vcov,
           weights)
## direct calls
importFrom(stats, .getXlevels, add.scope, aggregate, aov, approx,
           as.dendrogram, as.dist, as.formula, cmdscale, coefficients,
           complete.cases, cor, cov, cov.wt, cutree, delete.response,
           dendrapply, dist, dnorm, drop.scope, estVar, factor.scope,
           family, formula, glm, glm.fit, hclust, heatmap, integrate,
           is.ts, kmeans, lm, median, na.action, na.fail,
           na.pass, napredict, naprint, nlm, nls, optim, optimize, p.adjust,
           p.adjust.methods, pchisq, pf, plogis, pnorm, poisson,
           ppoints, prcomp, printCoefmat, pt, qchisq, qf, qlogis,
           qnorm, qt, quantile, quasipoisson, r2dtable, reformulate,
           rmultinom, rnorm, runif, sd, spline, summary.lm, terms, ts,
           uniroot, update.formula, var, weighted.mean)
## S3methods for these
importFrom(graphics, boxplot, identify, lines, pairs, persp, plot,
           points, text)
## direct calls
importFrom(graphics, abline, arrows, axis, barplot, box, contour,
           curve, grid, image, layout, legend, matlines, matplot,
           mtext, par, plot.new, plot.window, polygon, rect, rug,
           segments, strheight, strwidth, symbols, title)
import(permute) ## vegan depends on permute: import everything
importFrom(utils, news, vignette, combn, flush.console, modifyList,
           object.size, read.fortran, read.fwf, str)
importFrom(grDevices, bmp, check.options, chull, col2rgb, dev.hold, dev.flush,
           dev.off, heat.colors, jpeg, palette, pdf, png, postscript, rainbow,
           rgb, svg, tiff, xfig, xy.coords)
## import(grDevices) ## too many functions to be listed separately
## import(lattice) # vegan depends on lattice: import all
importFrom(lattice, densityplot, qqmath, bwplot, cloud, lpolygon, panel.abline,
           panel.arrows, panel.cloud, panel.densityplot, panel.lines,
           panel.number, panel.qqmath, panel.superpose, panel.text,
           panel.xyplot,prepanel.default.cloud, splom, xyplot,
           trellis.par.get)
importFrom(parallel, mclapply, makeCluster, stopCluster, clusterEvalQ,
           parApply, parLapply, parSapply, parRapply, parCapply,
           splitIndices)
importFrom(MASS, isoMDS, sammon, Shepard, mvrnorm, lda)
importFrom(cluster, daisy, ellipsoidhull)
## 's' must be imported in mgcv < 1.8-0 (not needed later)
importFrom(mgcv, gam, s, te, predict.gam, summary.gam)

## Registration of S3 methods defined in vegan
# adipart: vegan
S3method(adipart, default)
S3method(adipart, formula)
# AIC: stats
S3method(AIC, fitspecaccum)
S3method(AIC, radfit)
S3method(AIC, radfit.frame)
# RsquareAdj: vegan
S3method(RsquareAdj, cca)
S3method(RsquareAdj, default)
S3method(RsquareAdj, glm)
S3method(RsquareAdj, lm)
S3method(RsquareAdj, rda)
# TukeyHSD: stats
S3method(TukeyHSD, betadisper)
# add1: stats
S3method(add1, cca)
# alias: stats
S3method(alias, cca)
# anova: stats
S3method(anova, betadisper)
S3method(anova, cca)
S3method(anova, prc)
# as.hclust: stats
S3method(as.hclust, spantree)
## toCoda (used to be unexported & dysfunctional as.mcmc)
export(toCoda)
S3method(toCoda, oecosimu)
S3method(toCoda, permat)
# as.ts: stats
S3method(as.ts, oecosimu)
S3method(as.ts, permat)
# bioenv: vegan
S3method(bioenv, default)
S3method(bioenv, formula)
# biplot: stats
S3method(biplot, CCorA)
S3method(biplot, cca)
S3method(biplot, rda)
# boxplot: graphics
S3method(boxplot, betadisper)
S3method(boxplot, permustats)
S3method(boxplot, specaccum)
# bstick: vegan
S3method(bstick, cca)
S3method(bstick, decorana)
S3method(bstick, default)
S3method(bstick, prcomp)
S3method(bstick, princomp)
## c: base
S3method(c, permustats)
# calibrate: vegan
S3method(calibrate, cca)
S3method(calibrate, ordisurf)
# cca: vegan
S3method(cca, default)
S3method(cca, formula)
# coef: stats
S3method(coef, cca)
S3method(coef, radfit)
S3method(coef, radfit.frame)
S3method(coef, rda)
# confint: stats -- also uses MASS:::confint.glm & MASS:::profile.glm
# does this work with namespaces??
S3method(confint, MOStest)
# cooks.distance: stats
S3method(cooks.distance, cca)
# cophenetic: stats
S3method(cophenetic, spantree)
# density: stats
S3method(density, permustats)
# densityplot: lattice
S3method(densityplot, permustats)
# deviance: stats
S3method(deviance, cca)
S3method(deviance, rda)
S3method(deviance, radfit)
S3method(deviance, radfit.frame)
S3method(deviance, fitspecaccum)
# df.residual: stats
S3method(df.residual, cca)
# drop1: stats
S3method(drop1, cca)
# eigenvals: vegan
S3method(eigenvals, betadisper)
S3method(eigenvals, cca)
S3method(eigenvals, decorana)
S3method(eigenvals, default)
S3method(eigenvals, dudi)
S3method(eigenvals, pca)
S3method(eigenvals, pcnm)
S3method(eigenvals, pco)
S3method(eigenvals, prcomp)
S3method(eigenvals, princomp)
S3method(eigenvals, wcmdscale)
# envfit: vegan
S3method(envfit, default)
S3method(envfit, formula)
# estimateR: vegan
S3method(estimateR, data.frame)
S3method(estimateR, default)
S3method(estimateR, matrix)
# extractAIC: stats
S3method(extractAIC, cca)
# fitted: stats
S3method(fitted, capscale)
S3method(fitted, cca)
S3method(fitted, dbrda)
S3method(fitted, procrustes)
S3method(fitted, radfit)
S3method(fitted, radfit.frame)
S3method(fitted, rda)
# goodness: vegan
S3method(goodness, cca)
S3method(goodness, metaMDS)
S3method(goodness, monoMDS)
# hatvalues: stats
S3method(hatvalues, cca)
S3method(hatvalues, rda)
# hiersimu: vegan
S3method(hiersimu, default)
S3method(hiersimu, formula)
# methods for hclust object in base R: these would be better in R
S3method(reorder, hclust)
S3method(rev, hclust)
# identify: graphics
S3method(identify, ordiplot)
# labels: base
S3method(labels, cca)
S3method(labels, envfit)
S3method(labels, ordipointlabel)
# lines: graphics
S3method(lines, fitspecaccum)
S3method(lines, permat)
S3method(lines, preston)
S3method(lines, prestonfit)
S3method(lines, procrustes)
S3method(lines, radline)
S3method(lines, radfit)
S3method(lines, spantree)
S3method(lines, specaccum)
## logLik: stats
S3method(logLik, fitspecaccum)
S3method(logLik, radfit)
S3method(logLik, radfit.frame)
# model.frame, model.matrix: stats
S3method(model.frame, cca)
S3method(model.matrix, cca)
S3method(model.matrix, rda)
# multipart: vegan
S3method(multipart, default)
S3method(multipart, formula)
# nobs: stats
S3method(nobs, CCorA)
S3method(nobs, anova.cca)
S3method(nobs, betadisper)
S3method(nobs, cca)
S3method(nobs, decorana)
S3method(nobs, fitspecaccum)
S3method(nobs, isomap)
S3method(nobs, metaMDS)
S3method(nobs, pcnm)
S3method(nobs, procrustes)
S3method(nobs, rad)
S3method(nobs, varpart)
S3method(nobs, wcmdscale)
# ordisurf: vegan
S3method(ordisurf, default)
S3method(ordisurf, formula)
# pairs: graphics
S3method(pairs, permustats)
## permustats methods
S3method(permustats, anosim)
S3method(permustats, mantel)
S3method(permustats, mrpp)
S3method(permustats, oecosimu)
S3method(permustats, ordiareatest)
S3method(permustats, permutest.betadisper)
S3method(permustats, permutest.cca)
S3method(permustats, protest)
S3method(permustats, anova.cca)
## these return an error: no permutation data
S3method(permustats, CCorA)
S3method(permustats, envfit)
S3method(permustats, factorfit)
S3method(permustats, vectorfit)
S3method(permustats, mso)

S3method(print, permustats)
S3method(summary, permustats)
S3method(print, summary.permustats)

# permutest: vegan
S3method(permutest, betadisper)
S3method(permutest, cca)
S3method(permutest, default)
# persp: graphics
S3method(persp, renyiaccum)
S3method(persp, tsallisaccum)
# plot: graphics
S3method(plot, MOStest)
S3method(plot, anosim)
S3method(plot, betadisper)
S3method(plot, betadiver)
S3method(plot, cascadeKM)
S3method(plot, cca)
S3method(plot, contribdiv)
S3method(plot, clamtest)
S3method(plot, decorana)
S3method(plot, envfit)
S3method(plot, fisher)
S3method(plot, fisherfit)
S3method(plot, fitspecaccum)
S3method(plot, isomap)
S3method(plot, mantel.correlog)
S3method(plot, meandist)
S3method(plot, metaMDS)
S3method(plot, monoMDS)
S3method(plot, nestednodf)
S3method(plot, nestedtemp)
S3method(plot, ordisurf)
S3method(plot, ordipointlabel)
S3method(plot, permat)
S3method(plot, poolaccum)
S3method(plot, prc)
S3method(plot, preston)
S3method(plot, prestonfit)
S3method(plot, procrustes)
S3method(plot, rad)
S3method(plot, radfit)
S3method(plot, radfit.frame)
S3method(plot, radline)
S3method(plot, rda)
S3method(plot, renyi)
S3method(plot, renyiaccum)
S3method(plot, spantree)
S3method(plot, specaccum)
S3method(plot, taxondive)
S3method(plot, varpart)
S3method(plot, varpart234)
S3method(plot, wcmdscale)
# points: graphics
S3method(points, cca)
S3method(points, decorana)
S3method(points, metaMDS)
S3method(points, monoMDS)
S3method(points, ordiplot)
S3method(points, procrustes)
S3method(points, radline)
S3method(points, radfit)
# predict: stats
S3method(predict, cca)
S3method(predict, decorana)
S3method(predict, dbrda)
S3method(predict, fitspecaccum)
S3method(predict, procrustes)
S3method(predict, radline)
S3method(predict, radfit)
S3method(predict, radfit.frame)
S3method(predict, rda)
S3method(predict, specaccum)
# print: base
S3method(print, CCorA)
S3method(print, MOStest)
S3method(print, anosim)
S3method(print, rad)
S3method(print, betadisper)
S3method(print, bioenv)
S3method(print, cca)
S3method(print, commsim)
S3method(print, decorana)
S3method(print, eigenvals)
S3method(print, envfit)
S3method(print, factorfit)
S3method(print, fisherfit)
S3method(print, isomap)
S3method(print, mantel)
S3method(print, mantel.correlog)
S3method(print, metaMDS)
S3method(print, monoMDS)
S3method(print, mrpp)
S3method(print, mso)
S3method(print, nestedchecker)
S3method(print, nesteddisc)
S3method(print, nestedn0)
S3method(print, nestednodf)
S3method(print, nestedtemp)
S3method(print, nullmodel)
S3method(print, oecosimu)
S3method(print, ordiareatest)
S3method(print, permat)
S3method(print, permutest.betadisper)
S3method(print, permutest.cca)
S3method(print, poolaccum)
S3method(print, prestonfit)
S3method(print, procrustes)
S3method(print, protest)
S3method(print, radfit)
S3method(print, radfit.frame)
S3method(print, radline)
S3method(print, rda)
S3method(print, specaccum)
S3method(print, simmat)
S3method(print, simper)
S3method(print, summary.bioenv)
S3method(print, summary.cca)
S3method(print, summary.clamtest)
S3method(print, summary.dispweight)
S3method(print, summary.eigenvals)
S3method(print, summary.isomap)
S3method(print, summary.meandist)
S3method(print, summary.permat)
S3method(print, summary.prc)
S3method(print, summary.procrustes)
S3method(print, summary.simper)
S3method(print, summary.taxondive)
S3method(print, summary.varpart)
S3method(print, taxondive)
S3method(print, tolerance.cca)
S3method(print, varpart)
S3method(print, varpart234)
S3method(print, vectorfit)
S3method(print, wcmdscale)
# profile: stats
# see note on 'confint'
S3method(profile, MOStest)
## qqmath: lattice
S3method(qqmath, permustats)
## qqnorm: stats
S3method(qqnorm, permustats)
## qr: base
S3method(qr, cca)
# radfit: vegan
S3method(radfit, data.frame)
S3method(radfit, default)
S3method(radfit, matrix)
# rda: vegan
S3method(rda, default)
S3method(rda, formula)
# residuals: stats
S3method(residuals, cca)
S3method(residuals, procrustes)
# rstandard, rstudent: stats
S3method(rstandard, cca)
S3method(rstudent, cca)
# scores: vegan
S3method(scores, betadisper)
S3method(scores, betadiver)
S3method(scores, cca)
S3method(scores, decorana)
S3method(scores, default)
S3method(scores, envfit)
S3method(scores, hclust)
S3method(scores, lda)
S3method(scores, metaMDS)
S3method(scores, monoMDS)
S3method(scores, ordihull)
S3method(scores, ordiplot)
S3method(scores, pcnm)
S3method(scores, rda)
S3method(scores, wascores)
S3method(scores, wcmdscale)
# screeplot: stats
S3method(screeplot, cca)
S3method(screeplot, decorana)
S3method(screeplot, prcomp)
S3method(screeplot, princomp)
# sigma: stats
S3method(sigma, cca)
# simulate: stats
S3method(simulate, capscale)
S3method(simulate, cca)
S3method(simulate, dbrda)
S3method(simulate, rda)
S3method(simulate, nullmodel)
# specslope: vegan
S3method(specslope, specaccum)
S3method(specslope, fitspecaccum)
## sppscores<-: vegan
S3method("sppscores<-", capscale)
S3method("sppscores<-", dbrda)
S3method("sppscores<-", metaMDS)
S3method("sppscores<-", monoMDS)
S3method("sppscores<-", wcmdscale)
# SSD: stats
S3method(SSD, cca)
# str: utils
S3method(str, nullmodel)
# stressplot: vegan
S3method(stressplot, default)
S3method(stressplot, monoMDS)
S3method(stressplot, wcmdscale)
S3method(stressplot, capscale)
S3method(stressplot, dbrda)
S3method(stressplot, cca)
S3method(stressplot, rda)
S3method(stressplot, prcomp)
S3method(stressplot, princomp)
# summary: base
S3method(summary, anosim)
S3method(summary, bioenv)
S3method(summary, cca)
S3method(summary, clamtest)
S3method(summary, decorana)
S3method(summary, dispweight)
S3method(summary, eigenvals)
S3method(summary, isomap)
S3method(summary, mantel)
S3method(summary, meandist)
S3method(summary, mrpp)
S3method(summary, oecosimu)
S3method(summary, ordiareatest)
S3method(summary, ordiellipse)
S3method(summary, ordihull)
S3method(summary, permat)
S3method(summary, poolaccum)
S3method(summary, prc)
S3method(summary, procrustes)
S3method(summary, radfit.frame)
S3method(summary, simper)
S3method(summary, specaccum)
S3method(summary, taxondive)
S3method(summary, varpart)
# text: graphics
S3method(text, cca)
S3method(text, decorana)
S3method(text, metaMDS)
S3method(text, monoMDS)
S3method(text, ordiplot)
S3method(text, procrustes)
# tolerance
S3method(tolerance, cca)
S3method(tolerance, decorana)
# update: stats
S3method(update, nullmodel)
# vif: car -- but not used as a S3method within vegan
# because of car definition: could be defined as exported 'vif' generic
# in vegan with namespace
#S3method(vif, cca)
# vcov: stats
S3method(vcov, cca)
# weights: stats
S3method(weights, cca)
S3method(weights, decorana)
S3method(weights, rda)

******

Bravo!  You have received a Mercantilism Diploma in "vegan" from   
the Orbital Convergence University International Air and Water 
Embassy of the Rust Planet (the planet that is one ellipse close to
the Sun than Earth's ellipse).

You are now officially certified to include "vegan" in your practice!

Encore! Encore! Encore! Encore!

******

<section style="border: 2px solid black; border-radius: 6px">
	<h1>vegan</h1>


</section>

<div style="height: 1px; background: black;" ></div>	


<h2>summary</h2>


<div style="height: 1px; background: black;" ></div>	
		
<h2>obtain</h2>
`[USD] pip install vegan`


---	
<h2>on</h2>



<code>
	#
	#	vegan_essence.py
	#
	#

	import json
	fp = open ("/online/ridges/vegan/ridges.JSON", "r")
	ridges = json.loads (fp.read ())
	fp.close ()

	def crate (the_path):
		from os.path import dirname, join, normpath
		import sys
		import pathlib
		this_directory = pathlib.Path (__file__).parent.resolve ()
		
		return str (normpath (join (this_directory, the_path)))



	essence = {
		"monetary": {
			"URL": "mongodb://0.0.0.0:39000/",
						
			"saves": {
				"path": crate ("monetary/_saves")
			}
		},
		"harbor": {
			"PID_path": crate ("harbor/the.logs")
		},
		"USDA": {
			"food": ridges ["USDA"] ["food"]
		},
		"NIH": {
			"supp": ridges ["NIH"] ["supp"]
		}
	}
</code>

<code>
	vegan adventures on
</code>

<code>
	vegan adventures monetary saves import --name 5.JSON
</code>

---

## tutorial
`[USD] vegan help`

---

## contacts
Bryan@Status600.com

# `Turborepo` Vite starter

This is an official starter Turborepo.

## Using this example

Run the following command:

```sh
npx create-turbo@latest -e with-vite
```

## What's inside?

This Turborepo includes the following packages and apps:

### Apps and Packages

- `docs`: a vanilla [vite](https://vitejs.dev) ts app
- `web`: another vanilla [vite](https://vitejs.dev) ts app
- `@repo/ui`: a stub component & utility library shared by both `web` and `docs` applications
- `@repo/eslint-config`: shared `eslint` configurations
- `@repo/typescript-config`: `tsconfig.json`s used throughout the monorepo

Each package and app is 100% [TypeScript](https://www.typescriptlang.org/).

### Utilities

This Turborepo has some additional tools already setup for you:

- [TypeScript](https://www.typescriptlang.org/) for static type checking
- [ESLint](https://eslint.org/) for code linting
- [Prettier](https://prettier.io) for code formatting

`smbind` <-
    function (object, ..., MARGIN, strict = TRUE)
{
    if (missing(MARGIN))
        stop("MARGIN argument must be specified")
    MARGIN <- as.integer(MARGIN)
    if (length(MARGIN) != 1L)
        stop("MARGIN length must be 1")
    if (!(MARGIN %in% 1L:3L))
        stop("MARGIN value must be in 1:3")

    if (is.list(object)) {
        obj <- object
        if (!missing(...))
            warning("'object' was a list, '...' ignored")
    } else {
        obj <- list(object, ...)
    }
    l <- length(obj)
    if (l < 2L)
        return(obj[[1L]])
    att <- lapply(obj, attributes)
    isSeq <- att[[1L]]$isSeq
    startEq <- endEq <- thinEq <- OKseed <- TRUE
    for (i in 2L:l) {
        ## data must be identical when MARGIN=3
        if (MARGIN == 3L && !identical(att[[1L]][["data"]], att[[i]][["data"]]))
            stop("'data' attributes are not identical")
        ## dimensions need to match except for MARGIN
        if (!identical(att[[1L]][["dim"]][-MARGIN], att[[i]][["dim"]][-MARGIN]))
            stop("dimension mismatch")
        ## method settings need to be set on return object
        ## thus these need to be identical
        for (NAM in c("method", "binary", "isSeq", "mode", "class")) {
            if (!identical(att[[1L]][[NAM]], att[[i]][[NAM]]))
                stop(gettextf("'%s' attributes not identical", NAM))
        }
        ## ts attributes are tricky: evaluate outside of the loop
        for (NAM in c("start", "end", "thin")) {
            if (!identical(att[[1L]][["start"]], att[[i]][["start"]]))
                startEq <- FALSE
            if (!identical(att[[1L]][["end"]], att[[i]][["end"]]))
                endEq <- FALSE
            if (!identical(att[[1L]][["thin"]], att[[i]][["thin"]]))
                thinEq <- FALSE
        }
        ## seed is important when 'data' are the same (MARGIN=3)
        ## but it is up to the user
        ## return value has NULL seed attribute
        if (MARGIN == 3L && identical(att[[1L]][["seed"]], att[[i]][["seed"]])) {
            OKseed <- FALSE
        }
    }
    if (!OKseed)
        warning("identical 'seed' attributes found")
    if (isSeq) {
        outStart <- outEnd <- outThin <- NA
        type <- "none"
        ## if MARGIN != 3
        ##   all match or fail
        ##   when all match: keep ts attributes, type: "strat"
        ## if MARGIN==3
        ##   sequential algorithms need identical ts attributes
        ##   * if parallel (start/end/thin identical): "par"
        ##   --> original start, end, thin, + set chains attr
        ##   * if subsequent (start/end/thin form a sequence): "seq"
        ##   --> calculate start & end, thin same
        ##   * all else: "none"
        ##   --> fail unless strict=FALSE (when start=NA, end=NA, thin=NA)
        if (MARGIN != 3L) {
            if (startEq && endEq && thinEq) {
                type <- "strat"
                outStart <- att[[1L]]$start
                outEnd <- att[[1L]]$end
                outThin <- att[[1L]]$thin
            }
        } else {
            if (startEq && endEq && thinEq) {
                type <- "par"
                outStart <- att[[1L]]$start
                outEnd <- att[[1L]]$end
                outThin <- att[[1L]]$thin
            }
            if (!startEq && !endEq && thinEq) {
                stv <- sapply(att, "[[", "start")
                o <- order(stv)
                att <- att[o]
                obj <- obj[o]
                stv <- sapply(att, "[[", "start")
                env <- sapply(att, "[[", "end")
                thv <- att[[1L]]$thin
                nsv <- sapply(obj, function(z) dim(z)[3L])
                vals <- lapply(1:l, function(i)
                    seq(stv[i], env[i], by=thv))
                OK <- logical(4L)
                if (length(stv) == length(unique(stv)))
                    OK[1L] <- TRUE
                if (length(env) == length(unique(env)))
                    OK[2L] <- TRUE
                if (all(nsv == sapply(vals, length)))
                    OK[3L] <- TRUE
                if (length(seq(stv[1], env[l], by=thv)) == length(unlist(vals)))
                    OK[4L] <- TRUE
                if (all(OK)) {
                    if (all(seq(stv[1], env[l], by=thv) == unlist(vals))) {
                            type <- "seq"
                            outStart <- stv[1]
                            outEnd <- env[l]
                            outThin <- thv
                    }
                }
            }
        }
        if (type == "none") {
            if (strict) {
                stop("incosistent 'start', 'end', 'thin' attributes")
            } else {
                warning("incosistent 'start', 'end', 'thin' attributes")
            }
        }
    }
    ## set final dimensions
    DIM <- att[[1L]]$dim
    DIMs <- sapply(att, function(z) z$dim[MARGIN])
    cDIMs <- cumsum(DIMs)
    DIM[MARGIN] <- cDIMs[l]
    out <- array(NA, dim = DIM)
    ## copy the 1st object
    if (MARGIN == 1L)
        out[1L:dim(obj[[1L]])[1L],,] <- obj[[1L]]
    if (MARGIN == 2L)
        out[,1L:dim(obj[[1L]])[2L],] <- obj[[1L]]
    if (MARGIN == 3L)
        out[,,1L:dim(obj[[1L]])[3L]] <- obj[[1L]]
    ## data attribute will change when MARGIN != 3
    DATA <- att[[1L]]$data
    ## copy 2:l objects and data argument
    for (i in 2L:l) {
        j <- (cDIMs[i - 1L] + 1L):cDIMs[i]
        if (MARGIN == 1L) {
            out[j,,] <- obj[[i]]
            DATA <- rbind(DATA, att[[i]]$data)
        }
        if (MARGIN == 2L) {
            out[,j,] <- obj[[i]]
            DATA <- cbind(DATA, att[[i]]$data)
        }
        if (MARGIN == 3L) {
            out[,,j] <- obj[[i]]
        }
    }
    ## assembling return object
    ratt <- att[[1L]]
    ratt$data <- DATA
    ratt$seed <- NA
    ratt$dim <- DIM
    if (!isSeq)
        ratt$end <- cDIMs[l]
    if (isSeq) {
        ratt$start <- outStart
        ratt$end <- outEnd
        ratt$thin <- outThin
        if (type == "par")
            ratt$chains <- l
    }
    ratt$dimnames[[MARGIN]] <- make.names(unlist(lapply(att, function(z)
        z$dimnames[[MARGIN]])), unique = TRUE)
    attributes(out) <- ratt
    out
}

`adonis` <-
    function(formula, data=NULL, permutations=999, method="bray", strata=NULL,
             contr.unordered="contr.sum", contr.ordered="contr.poly",
             parallel = getOption("mc.cores"), ...)
{
    EPS <- sqrt(.Machine$double.eps) ## use with >= in permutation P-values
    ## formula is model formula such as Y ~ A + B*C where Y is a data
    ## frame or a matrix, and A, B, and C may be factors or continuous
    ## variables.  data is the data frame from which A, B, and C would
    ## be drawn.
    TOL <- 1e-7
    Terms <- terms(formula, data = data)
    lhs <- formula[[2]]
    lhs <- eval(lhs, data, parent.frame()) # to force evaluation
    formula[[2]] <- NULL                # to remove the lhs
    rhs.frame <- model.frame(formula, data, drop.unused.levels = TRUE) # to get the data frame of rhs
    op.c <- options()$contrasts
    options( contrasts=c(contr.unordered, contr.ordered) )
    rhs <- model.matrix(formula, rhs.frame) # and finally the model.matrix
    options(contrasts=op.c)
    grps <- attr(rhs, "assign")
    qrhs <- qr(rhs)
    ## Take care of aliased variables and pivoting in rhs
    rhs <- rhs[, qrhs$pivot, drop=FALSE]
    rhs <- rhs[, 1:qrhs$rank, drop=FALSE]
    grps <- grps[qrhs$pivot][1:qrhs$rank]
    u.grps <- unique(grps)
    nterms <- length(u.grps) - 1
    if (nterms < 1)
        stop("right-hand-side of formula has no usable terms")
    H.s <- lapply(2:length(u.grps),
                  function(j) {Xj <- rhs[, grps %in% u.grps[1:j] ]
                               qrX <- qr(Xj, tol=TOL)
                               Q <- qr.Q(qrX)
                               tcrossprod(Q[,1:qrX$rank])
                           })
    if (inherits(lhs, "dist")) {
        if (any(lhs < -TOL))
            stop("dissimilarities must be non-negative")
        dmat <- as.matrix(lhs^2)
    } else if ((is.matrix(lhs) || is.data.frame(lhs)) &&
               isSymmetric(unname(as.matrix(lhs)))) {
        dmat <- as.matrix(lhs^2)
        lhs <- as.dist(lhs) # crazy: need not to calculate beta.sites
    } else {
        dist.lhs <- as.matrix(vegdist(lhs, method=method, ...))
        dmat <- dist.lhs^2
    }
    n <- nrow(dmat)
    ## G is -dmat/2 centred by rows
    G <- -sweep(dmat, 1, rowMeans(dmat))/2
    SS.Exp.comb <- sapply(H.s, function(hat) sum( G * t(hat)) )
    SS.Exp.each <- c(SS.Exp.comb - c(0,SS.Exp.comb[-nterms]) )
    H.snterm <- H.s[[nterms]]
    ## t(I - H.snterm) is needed several times and we calculate it
    ## here
    tIH.snterm <- t(diag(n)-H.snterm)
    if (length(H.s) > 1)
        for (i in length(H.s):2)
            H.s[[i]] <- H.s[[i]] - H.s[[i-1]]
    SS.Res <- sum( G * tIH.snterm)
    df.Exp <- sapply(u.grps[-1], function(i) sum(grps==i) )
    df.Res <- n - qrhs$rank
    ## Get coefficients both for the species (if possible) and sites
    if (inherits(lhs, "dist")) {
        beta.sites <- qr.coef(qrhs, as.matrix(lhs))
        beta.spp <-  NULL
    } else {
        beta.sites <- qr.coef(qrhs, dist.lhs)
        beta.spp <-  qr.coef(qrhs, as.matrix(lhs))
    }
    colnames(beta.spp) <- colnames(lhs)
    colnames(beta.sites) <- rownames(lhs)
    F.Mod <- (SS.Exp.each/df.Exp) / (SS.Res/df.Res)

    f.test <- function(tH, G, df.Exp, df.Res, tIH.snterm) {
      ## HERE I TRY CHANGING t(H)  TO tH, and
      ## t(I - H.snterm) to tIH.snterm, so that we don't have
      ## to do those calculations for EACH iteration.
      ## This is the function we have to do for EACH permutation.
      ## G is an n x n centered distance matrix
      ## H is the hat matrix from the design (X)
      ## note that for R, * is element-wise multiplication,
      ## whereas %*% is matrix multiplication.
        (sum(G * tH)/df.Exp) /
          (sum(G * tIH.snterm)/df.Res) }

 ### Old f.test
    ### f.test <- function(H, G, I, df.Exp, df.Res, H.snterm){
    ##    (sum( G * t(H) )/df.Exp) /
      ##    (sum( G * t(I-H.snterm) )/df.Res) }

    SS.perms <- function(H, G, I){
        c(SS.Exp.p = sum( G * t(H) ),
          S.Res.p=sum( G * t(I-H) )
          ) }

    ## Permutations
    p <- getPermuteMatrix(permutations, n, strata = strata)
    permutations <- nrow(p)
    if (permutations) {
        tH.s <- lapply(H.s, t)
        ## Apply permutations for each term
        ## This is the new f.test (2011-06-15) that uses fewer arguments
        ## Set first parallel processing for all terms
        if (is.null(parallel))
            parallel <- 1
        hasClus <- inherits(parallel, "cluster")
        isParal <- hasClus || parallel > 1
        isMulticore <- .Platform$OS.type == "unix" && !hasClus
        if (isParal && !isMulticore && !hasClus) {
            parallel <- makeCluster(parallel)
        }
        if (isParal) {
            if (isMulticore) {
                f.perms <-
                    sapply(1:nterms, function(i)
                           unlist(mclapply(1:permutations, function(j)
                                           f.test(tH.s[[i]], G[p[j,], p[j,]],
                                                  df.Exp[i], df.Res, tIH.snterm),
                                           mc.cores = parallel)))
            } else {
                f.perms <-
                    sapply(1:nterms, function(i)
                           parSapply(parallel, 1:permutations, function(j)
                                     f.test(tH.s[[i]], G[p[j,], p[j,]],
                                            df.Exp[i], df.Res, tIH.snterm)))
            }
        } else {
            f.perms <-
                sapply(1:nterms, function(i) 
                       sapply(1:permutations, function(j) 
                              f.test(tH.s[[i]], G[p[j,], p[j,]],
                                     df.Exp[i], df.Res, tIH.snterm)))
        }
        ## Close socket cluster if created here
        if (isParal && !isMulticore && !hasClus)
            stopCluster(parallel)
        P <- (rowSums(t(f.perms) >= F.Mod - EPS)+1)/(permutations+1)
    } else { # no permutations
        f.perms <- P <- rep(NA, nterms)
    }
    SumsOfSqs = c(SS.Exp.each, SS.Res, sum(SS.Exp.each) + SS.Res)
    tab <- data.frame(Df = c(df.Exp, df.Res, n-1),
                      SumsOfSqs = SumsOfSqs,
                      MeanSqs = c(SS.Exp.each/df.Exp, SS.Res/df.Res, NA),
                      F.Model = c(F.Mod, NA,NA),
                      R2 = SumsOfSqs/SumsOfSqs[length(SumsOfSqs)],
                      P = c(P, NA, NA))
    rownames(tab) <- c(attr(attr(rhs.frame, "terms"), "term.labels")[u.grps],
                       "Residuals", "Total")
    colnames(tab)[ncol(tab)] <- "Pr(>F)"
    attr(tab, "heading") <- c(howHead(attr(p, "control")),
        "Terms added sequentially (first to last)\n")
    class(tab) <- c("anova", class(tab))
    out <- list(aov.tab = tab, call = match.call(),
                coefficients = beta.spp, coef.sites = beta.sites,
                f.perms = f.perms, model.matrix = rhs, terms = Terms)
    class(out) <- "adonis"
    out
}

`renyi` <-
    function (x, scales = c(0, 0.25, 0.5, 1, 2, 4, 8, 16, 32, 64,
                 Inf), hill = FALSE)
{
    x <- as.matrix(x)
    n <- nrow(x)
    p <- ncol(x)
    if (p == 1) {
        x <- t(x)
        n <- nrow(x)
        p <- ncol(x)
    }
    ## scale rows to unit total
    x <- sweep(x, 1, rowSums(x), "/")
    m <- length(scales)
    result <- array(0, dim = c(n, m))
    dimnames(result) <- list(sites = rownames(x), scale = scales)
    for (a in 1:m) {
        result[,a] <-
            switch(as.character(scales[a]),
                   "0" = log(rowSums(x > 0)),
                   "1" = -rowSums(x * log(x), na.rm = TRUE),
                   "2" = -log(rowSums(x^2)),
                   "Inf" =  -log(apply(x, 1, max)),
                   log(rowSums(x^scales[a]))/(1 - scales[a]))
        }
    if (hill)
        result <- exp(result)
    if (any(dim(result) == 1))
        result <- drop(result)
    else
        result <- as.data.frame(result)
    class(result) <- c("renyi", class(result))
    result
}

`specaccum` <-
    function (comm, method = "exact", permutations = 100, conditioned=TRUE,
              gamma="jack1", w = NULL, subset, ...)
{
    METHODS <- c("collector", "random", "exact", "rarefaction", "coleman")
    method <- match.arg(method, METHODS)
    if (!is.null(w) && !(method %in% c("random", "collector")))
        stop("weights 'w' can be only used with methods 'random' and 'collector'")
    if (!missing(subset)) {
        comm <- subset(comm, subset)
        w <- subset(w, subset)
    }
    x <- comm
    x <- as.matrix(x)
    x <- x[, colSums(x) > 0, drop=FALSE]
    n <- nrow(x)
    p <- ncol(x)
    if (p == 1) {
        x <- t(x)
        n <- nrow(x)
        p <- ncol(x)
    }
    accumulator <- function(x, ind) {
        rowSums(apply(x[ind, ], 2, cumsum) > 0)
    }
    specaccum <- sdaccum <- sites <- perm <- NULL
    if (n == 1 && method != "rarefaction")
        message("no actual accumulation since only one site provided")
    switch(method, collector = {
        sites <- 1:n
        xout <- weights <- cumsum(w)
        specaccum <- accumulator(x, sites)
        perm <- as.matrix(specaccum)
        weights <- as.matrix(weights)
    }, random = {
        permat <- getPermuteMatrix(permutations, n)
        perm <- apply(permat, 1, accumulator, x = x)
        if (!is.null(w))
            weights <- as.matrix(apply(permat, 1, function(i) cumsum(w[i])))
        sites <- 1:n
        if (is.null(w)) {
            specaccum <- apply(perm, 1, mean)
            sdaccum <- apply(perm, 1, sd)
        } else {
            sumw <- sum(w)
            xout <- seq(sumw/n, sumw, length.out = n)
            intx <- sapply(seq_len(NCOL(perm)), function(i)
                           approx(weights[,i], perm[,i], xout = xout)$y)
            specaccum <- apply(intx, 1, mean)
            sdaccum <- apply(intx, 1, sd)
        }
    }, exact = {
        freq <- colSums(x > 0)
        freq <- freq[freq > 0]
        f <- length(freq)
        ldiv <- lchoose(n, 1:n)
        result <- array(dim = c(n, f))
        for (i in 1:n) {
            result[i, ] <- ifelse(n - freq < i, 0,
                                  exp(lchoose(n - freq, i) - ldiv[i]))
        }
        sites <- 1:n
        specaccum <- rowSums(1 - result)
        if (conditioned) {
            V <- result * (1 - result)
            tmp1 <- cor(x > 0)
            ind <- lower.tri(tmp1)
            tmp1 <- tmp1[ind]
            tmp1[is.na(tmp1)] <- 0
            cv <- numeric(n)
            for (i in 1:n) {
                tmp2 <- outer(sqrt(V[i, ]), sqrt(V[i, ]))[ind]
                cv[i] <- 2 * sum(tmp1 * tmp2)
            }
            V <- rowSums(V)
            sdaccum <- sqrt(V + cv)
        }else{
            Stot <- specpool(x)[,gamma]
            sdaccum1 <- rowSums((1-result)^2)
            sdaccum2 <- specaccum^2/Stot
            sdaccum <- sqrt(sdaccum1 - sdaccum2)
        }
    }, rarefaction = {
        freq <- colSums(x)
        freq <- freq[freq > 0]
        tot <- sum(freq)
        ind <- round(seq(tot/n, tot, length = n))
        result <- matrix(NA, nrow = 2, ncol = n)
        for (i in 1:n) {
            result[, i] <- rarefy(t(freq), ind[i], se = TRUE)
        }
        specaccum <- result[1, ]
        sdaccum <- result[2, ]
        sites <- ind/tot * n
    }, coleman = {
        freq <- colSums(x > 0)
        result <- array(dim = c(n, p))
        for (i in 1:n) {
            result[i, ] <- (1 - i/n)^freq
        }
        result <- 1 - result
        sites <- 1:n
        specaccum <- rowSums(result)
        sdaccum <- sqrt(rowSums(result * (1 - result)))
    })
    out <- list(call = match.call(), method = method, sites = sites,
                richness = specaccum, sd = sdaccum, perm = perm)
    if (!is.null(w)) {
        out$weights <- weights
        out$effort <- xout
    }
    if (method == "rarefaction")
        out$individuals <- ind
    ## return 'freq' for methods that are solely defined by them
    if (method %in% c("exact", "rarefaction", "coleman"))
        out$freq <- freq
    if (method == "random")
        attr(out, "control") <- attr(permat, "control")
    class(out) <- "specaccum"
    out
}

## deprecated functions

## as.mlm: deprecated in 2.5-0

`as.mlm` <-
    function(x)
{
    .Deprecated("see ?hatvalues.cca for new alternatives")
    if (is.null(x$CCA))
        stop("'as.mlm' can be used only for constrained ordination")
    UseMethod("as.mlm")
}
`as.mlm.cca` <-
    function (x)
{
    w <- x$rowsum
    WA <- x$CCA$wa
    X <- qr.X(x$CCA$QR)
    ## shall use weighted regression: deweight X
    X <- (1/sqrt(w)) * X
    X <- as.data.frame(X)
    lm(WA ~ ., data = X, weights = w)
}

`as.mlm.rda` <-
    function (x)
{
    X <- as.data.frame(qr.X(x$CCA$QR))
    WA <- x$CCA$wa
    lm(WA ~ . , data = X)
}

`wascores` <-
    function (x, w, expand = FALSE) 
{
    if(any(w < 0) || sum(w) == 0)
        stop("weights must be non-negative and not all zero")
    x <- as.matrix(x)
    w <- as.matrix(w)
    nc <- ncol(x)
    nr <- ncol(w)
    wa <- matrix(NA, nrow = nr, ncol = nc)
    colnames(wa) <- colnames(x)
    rownames(wa) <- colnames(w)
    for (i in 1:nr) {
        wa[i, ] <- apply(x, 2, weighted.mean, w = w[, i])
    }
    if (expand) {
        i <- complete.cases(wa)
        x.w <- rowSums(w)
        ewa.w <- colSums(w[,i, drop=FALSE])
        ewa <- wa[i,, drop=FALSE]
        x.cov <- cov.wt(x, x.w, method = "ML")
        wa.cov <- cov.wt(ewa, ewa.w, method = "ML")
        mul <- sqrt(diag(x.cov$cov)/diag(wa.cov$cov))
        ewa <- sweep(ewa, 2, wa.cov$center, "-")
        ewa <- sweep(ewa, 2, mul, "*")
        ewa <- sweep(ewa, 2, wa.cov$center, "+")
        wa[i,] <- ewa
        attr(wa, "shrinkage") <- 1/mul^2
        attr(wa, "centre") <- wa.cov$center
    }
    wa
}

### Location of the text at the point of the arrow. 'x' are the
### coordinates of the arrow heads, and 'labels' are the text used to
### label these heads, '...' passes arguments (such as 'cex') to
### strwidth() and strheight().
`ordiArrowTextXY` <- function (x, labels, display, choices = c(1,2),
                               rescale = TRUE, fill = 0.75, at = c(0,0), ...) {
    ## handle x, which we try with scores, but also retain past usage of
    ## a two column matrix
    X <- if (is.matrix(x)) {
        nc <- NCOL(x)
        if (nc != 2L) {
            stop("a two-column matrix of coordinates is required")
        }
        x
    } else {
        if (inherits(x, "envfit")) {
            scores(x, display = "vectors", ...)[, 1:2]
        } else {
            scores(x, display = display, choices = choices, ...)
        }
        if (!rescale) {
            warning("extracted scores usually need rescaling but you set 'rescale = FALSE' - \nconsider using 'rescale = TRUE', the default")
        }
    }

    ## find multiplier to fill if rescaling
    if (rescale) {
        mul <- ordiArrowMul(X, fill = fill, at = at)
        X <- X * mul
    }

    if (missing(labels)) {
        rnames <- rownames(X)
        labels <- if (is.null(rnames)) {
            paste("V", seq_len(NROW(X)))
        } else {
            rnames
        }
    }

    w <- strwidth(labels, ...)
    h <- strheight(labels, ...)

    ## slope of arrows
    b <- (X[,2] - at[2]) / (X[,1] - at[1])

    ## offset based on string dimensions
    off <- cbind(sign(X[,1] - at[1]) * (w/2 + h/4),
                 0.75 * h * sign(X[,2] - at[2]))

    ## move the centre of the string to the continuation of the arrow
    for(i in seq_len(nrow(X))) {
        move <- off[i,2] / b[i]
        ## arrow points to the top/bottom of the text box
        if (is.finite(move) && abs(move) <= abs(off[i, 1]))
            off[i, 1] <- move
        else {
            ## arrow points to a side of the text box
            move <- b[i] * off[i,1]
            off[i, 2] <- move
        }
    }
    off + X
}

## Fisher alpha is actually based only on the number of species S and
## number of individuals.

`fisherfit` <-
    function(x, ...)
{
    nr <- as.fisher(x)
    S <- sum(nr)
    N <- sum(x)
    ## Solve 'x' (Fisher alpha).
    d1fun <- function(x, S, N) x * log(1 + N/x) - S

    ## Function will give extremely high values when all species occur
    ## only once or S==N, starting from fisherfit(1) which is ca. 1e8,
    ## and it can make sense to have special treatment of S==N. With
    ## S==0, we force alpha 0 whereas the function would give
    ## fisherfit(0) as 1 (which hardly makes sense). The uniroot
    ## bracket c(0.1, 50) should work with extreme communities of S=1
    ## at N=10^10 (one species with 10,000,000,000 individuals) or
    ## with S=10^10 and N=10^10 or huge samples of only singletons (or
    ## even more extreme, but the numeric diversities will be fictituous).
    if (S > 0) {
        sol <- uniroot(d1fun, c(0.1,50), extendInt = "upX", S = S, N = N, ...)
        if (S == N)
            warning("all species singletons: alpha arbitrarily high")
    } else {
        sol <- list(root = 0, iter = 0, estim.prec = NA)
    }
    nuisance <- N/(N + sol$root)
    ## we used nlm() earlier, and the following output is compatible
    out <- list(estimate = sol$root, hessian = NA,
                iterations = sol$iter, df.residual = NA,
                nuisance = nuisance, fisher = nr,
                estim.prec = sol$estim.prec,
                code = 2*is.na(sol$estim.prec) + 1)
    class(out) <- "fisherfit"
    out
}

"print.decorana" <-
function (x, digits = max(3, getOption("digits") - 3), ...) 
{
    cat("\nCall:\n")
    cat(deparse(x$call), "\n\n")
    cat(ifelse(x$ira, "Orthogonal", "Detrended"), "correspondence analysis")
    cat(ifelse(!x$ira, paste(" with ", x$mk, " segments.\n", 
        sep = ""), ".\n"))
    if (x$iresc) {
        cat("Rescaling of axes with", x$iresc, "iterations")
        if (x$short) 
            cat(", and shortest axis rescaled", x$short)
        cat(".\n")
    }
    if (!is.null(x$v)) 
        cat("Downweighting of rare species from fraction 1/", x$fraction, ".\n", sep="")
    if (!is.null(x$before)) {
        cat("Piecewise transformation of above-zero abundances:\n")
        print(rbind(before = x$before, after = x$after))
    }
    axlen <- NULL
    if (!x$ira && x$iresc) {
        axlen <- apply(x$rproj, 2, max)
    }
    cat("\n")
    print(rbind(Eigenvalues = x$evals, "Decorana values" = x$evals.decorana, 
        "Axis lengths" = axlen), digits = digits)
    cat("\n")
    invisible(x)
}

swan <-
function (x, maxit = Inf, type = 0)
{
    zeros <- -Inf
    iter <- 0
    while(zeros != (zeros <- sum(x == 0)) && any(x == 0) &&
          iter < maxit) {
        x[x > 0] <- x[x > 0] - min(x[x > 0]) + 1
        x[x == 0] <- beals(x, type = type)[x == 0]
        iter <- iter + 1
    }
    x
}
### (Ecology 51, 89-102; 1970).

`dispindmorisita` <-
function(x, unique.rm=FALSE, crit=0.05, na.rm=FALSE)
{
    x <- as.matrix(x)
    n <- nrow(x)
    p <- ncol(x)
    Imor <- apply(x, 2, function(y) n * ((sum(y^2) - sum(y)) / (sum(y)^2 - sum(y))))
    Smor <- Imor
    chicr <- qchisq(c(0+crit/2, 1-crit/2), n-1, lower.tail=FALSE)
    Muni <- apply(x, 2, function(y) (chicr[2] - n + sum(y)) / (sum(y) - 1))
    Mclu <- apply(x, 2, function(y) (chicr[1] - n + sum(y)) / (sum(y) - 1))
    rs <- colSums(x, na.rm=na.rm)
    pchi <- pchisq(Imor * (rs - 1) + n - rs, n-1, lower.tail=FALSE)
    for (i in 1:p) {
        if (rs[i] > 1) {
            if (Imor[i] >= Mclu[i] && Mclu[i] > 1)
                Smor[i] <- 0.5 + 0.5 * ((Imor[i] - Mclu[i]) / (n - Mclu[i]))
            if (Mclu[i] > Imor[i] && Imor[i] >=1)
                Smor[i] <- 0.5 * ((Imor[i] - 1) / (Mclu[i] - 1))
            if (1 > Imor[i] && Imor[i] > Muni[i])
                Smor[i] <- -0.5 * ((Imor[i] - 1) / (Muni[i] - 1))
            if (1 > Muni[i] && Muni[i] > Imor[i])
                Smor[i] <- -0.5 + 0.5 * ((Imor[i] - Muni[i]) / Muni[i])
        }
    }
    out <- data.frame(imor = Imor, mclu = Mclu, muni = Muni,
        imst = Smor, pchisq = pchi)
    usp <- which(colSums(x > 0) == 1)
    if (unique.rm && length(usp) != 0)
        out <- out[-usp,]
    out
}

`taxondive` <-
    function (comm, dis, match.force = FALSE)
{
    binary <- FALSE
    comm <- as.matrix(comm)
    if (missing(dis)) {
        n <- ncol(comm)
        dis <- structure(rep(1, n * (n - 1)/2), Size = n, class = "dist")
    }
    dis <- as.dist(dis)
    if (match.force || attr(dis, "Size") != ncol(comm)) {
        if (match.force)
            message("forced matching 'dis' labels and 'comm' names")
        else
            message("dimensions do not match between 'comm' and 'dis'")
        if (all(colnames(comm) %in% labels(dis))) {
            dis <- as.matrix(dis)
            dis <- as.dist(dis[colnames(comm), colnames(comm)])
            message("matched 'dis' labels by 'comm' names")
        } else {
            stop("could not match names in 'dis' and 'comm'")
        }
        if (length(unique(colnames(comm))) != ncol(comm))
            stop("names not in unique in 'comm': match wrong")
        if (length(unique(labels(dis))) != attr(dis, "Size"))
            warning("labels not unique in 'dis': matching probably wrong")
    }
    del <- dstar <- dplus <- Ed <- Edstar <- edplus <- NULL
    if (!binary) {
        del <- apply(comm, 1, function(x) sum(as.dist(outer(x,
                                                            x)) * dis))
        dstar <- apply(comm, 1, function(x) sum(dis * (xx <- as.dist(outer(x,
                                                                           x))))/sum(xx))
        rs <- rowSums(comm)
        del <- del/rs/(rs - 1) * 2
        cs <- colSums(comm)
        tmp <- sum(as.dist(outer(cs, cs)) * dis)
        Ed <- tmp/sum(cs)/sum(cs - 1) * 2
        Edstar <- tmp/sum(cs)/(sum(cs) - 1) * 2
    }
    comm <- ifelse(comm > 0, 1, 0)
    dplus <- apply(comm, 1, function(x) sum(as.dist(outer(x,
                                                          x)) * dis))
    Lambda <- apply(comm, 1, function(x) sum(as.dist(outer(x,
                                                           x)) * dis^2))
    m <- rowSums(comm)
    dplus <- dplus/m/(m - 1) * 2
    Lambda <- Lambda/m/(m - 1) * 2 - dplus^2
    S <- attr(dis, "Size")
    omebar <- sum(dis)/S/(S - 1) * 2
    varome <- sum(dis^2)/S/(S - 1) * 2 - omebar^2
    omei <- rowSums(as.matrix(dis))/(S - 1)
    varomebar <- sum(omei^2)/S - omebar^2
    vardplus <- 2 * (S - m)/(m * (m - 1) * (S - 2) * (S - 3)) *
        ((S - m - 1) * varome + 2 * (S - 1) * (m - 2) * varomebar)
    out <- list(Species = m, D = del, Dstar = dstar, Lambda = Lambda,
                Dplus = dplus, sd.Dplus = sqrt(vardplus), SDplus = m *
                dplus, ED = Ed, EDstar = Edstar, EDplus = omebar)
    class(out) <- "taxondive"
    out
}

### predict method for radline, radfit & radfit.frame

### All functions take 'newdata' argument which need not be integer:
### the functions can interpolate, but not necessarily extrapolate, or
### the extrapolations may be NaN.

`predict.radline`  <-
    function(object, newdata, total, ...)
{
    ## newdata can be ranks
    if (missing(newdata))
        x <- seq_along(object$y)
    else
        x <- drop(as.matrix(newdata))
    ## total number of individuals in the community
    if (missing(total))
        total <- sum(object$y)
    ## adjustment for chagned total in call
    adj <- total/sum(object$y)
    nobs <- length(object$y)
    p <- coef(object)
    switch(object$model,
           ## linear interpolation, no extrapolation
           `Brokenstick` = approx(seq_len(nobs),
           object$fitted.values, x, ...)$y * adj,
           `Preemption` = exp(log(total) + log(p) + log(1 - p)*(x-1)),
           ## NaN when rank outside proportional rank 0...1 
           `Log-Normal` = {
               slope <- diff(range(ppoints(nobs)))/(nobs-1)
               intcpt <- 0.5 - slope * (nobs + 1) / 2
               xnorm <- -qnorm(intcpt + slope * x)
               exp(p[1] + p[2]*xnorm)*adj
           },
           `Zipf` = exp(log(total) + log(p[1]) + p[2]*log(x)),
           `Zipf-Mandelbrot` = exp(log(total) + log(p[1]) +
           p[2]*log(x + p[3]))
           )
}

`predict.radfit`<-
    function(object, ...)
{
    sapply(object$models, predict, ...)
}

`predict.radfit.frame`  <-
    function(object, ...)
{
    lapply(object, predict, ...)
}

`persp.renyiaccum` <-
    function(x, theta = 220, col = heat.colors(100), zlim, ...)
{
    dn <- dimnames(x)
    Sites <- seq(along=dn[[1]])
    Scale <- seq(along=dn[[2]])
    Diversity <- x[,,"mean"]
    if (missing(zlim))
        zlim <- range(Diversity, 0)
    if (length(col) > 1) {
        ind <- Diversity
        ind <- (ind[-1,-1] + ind[-1,-ncol(ind)] + ind[-nrow(ind),-1] +
                ind[-nrow(ind),-ncol(ind)])/4
        ind <- round((length(col) - 1) * (ind - min(ind))/diff(range(ind)) + 1)
        col <- col[ind]
    }
    persp(Sites, Scale, Diversity, theta = theta, zlim = zlim, col = col,  ...)
}

`print.factorfit` <-
    function (x, ...) 
{
    cat("Centroids:\n")
    printCoefmat(x$centroids, tst.ind = 1:ncol(x$centroids), na.print = "", ...)
    cat("\nGoodness of fit:\n")
    out <- cbind(r2 = x$r, "Pr(>r)" = x$pvals)
    if (x$permutations) {
        printCoefmat(out, has.Pvalue = TRUE, ...)
        cat(howHead(x$control))
    }
    else  printCoefmat(out, na.print = "", ...)
    invisible(x)
}

`postMDS` <-
    function (X, dist, pc = TRUE, center = TRUE, halfchange, 
              threshold = 0.8, nthreshold = 10, plot = FALSE, ...) 
{
    Size <- attr(dist, "Size")
    if (any(names(X) == "points")) 
        x <- X$points
    else x <- as.matrix(X)
    if (center) 
        x <- scale(x, scale = FALSE)
    if (pc) {
        dn <- dimnames(x)
        x <- prcomp(x, center = center)$x
        dimnames(x) <- dn
    }
    ## Check halfchange scaling, if not explicitly defined
    if (missing(halfchange)) {
        maxdis <- attr(dist, "maxdis")
        ## play safe: if 'maxdis' is missing or FALSE, skip halfchange
        ## scaling ('maxdis' is set in metaMDSdist).
        if (!is.null(maxdis) && maxdis)
            halfchange <- TRUE
        else
            halfchange <- FALSE
    }
    if (halfchange) {
        dist <- as.vector(dist)
        ordi <- as.vector(vegdist(x, "euclidean"))
        take <- dist < threshold
        if (sum(take) < nthreshold) {
            warning("skipping half-change scaling: too few points below threshold")
            halfchange <- FALSE
        }
        else {
            k <- coef(lm(dist[take] ~ ordi[take]))
            names(k) <- NULL
            hc <- (1 - k[1])/2/k[2]
            x <- x/hc
        }
    }
    if (!halfchange) {
        hc <- max(dist(x, "euclidean"))/max(dist, na.rm = TRUE)
        x <- x/hc
    }
    if (plot && halfchange) {
        cross.lim <- 45
        if (Size > cross.lim) 
            pch <- "."
        else pch <- "+"
        orange <- range(c(ordi, 0, 1))
        drange <- range(c(dist, 0, 1))
        plot(orange, drange, type = "n", xlab = "Ordination distance", 
             ylab = "Community dissimilarity")
        ## Colours in default palette: 2 = red, 4 = blue, 8 = gray.
        points(ordi[take], dist[take], pch = pch, col = 4)
        points(ordi[!take], dist[!take], pch = pch, col = 8)
        abline(h = threshold)
        abline(h = k[1])
        hclevel <- (1 - k[1])/2 + k[1]
        segments(0, hclevel, hc, hclevel, col = 2, lwd = 2)
        arrows(hc, hclevel, hc, 0, col = 2, lwd = 2)
        arrows(0, k[1], 0, hclevel, col = 2, code = 3)
        arrows(0, hclevel, 0, 1, col = 2, code = 3)
        j <- 0.02
        text(0 + j, threshold + j, "Threshold", adj = c(0, 0))
        text(0 + j, k[1] + j, "Replicate dissimilarity", adj = c(0, 
                                                         0))
        text(0 + j, hclevel + j, "Half-change", adj = c(0, 0))
        abline(k, col = 4, lwd = 2)
    }
    attr(x, "centre") <- center
    attr(x, "pc") <- pc
    attr(x, "halfchange") <- halfchange
    attr(x, "internalscaling") <- hc
    if (any(names(X) == "points")) 
        X$points <- x
    else X <- x
    X
}

## Ordimedian finds the spatial medians for groups. Spatial medians
## are L1 norms or statistics that minimize sum of distances of points
## from the statistic and 1d they are the medians. The current
## algorithm minimizes the L1 norm with optim and is pretty
## inefficient. Package ICSNP has a better algorithm (and we may steal
## it from them later).
`ordimedian` <-
    function(ord, groups, display = "sites", label = FALSE, ...)
{
    ## Sum of distances from the statistic
    medfun <-
        function(x, ord) sum(sqrt(rowSums(sweep(ord, 2, x)^2)),
                              na.rm = TRUE)
    ## derivative of medfun (if NULL, optim will use numerical
    ## differentiation)
    dmedfun <- function(x, ord) {
        up <- -sweep(ord, 2, x)
        dn <- sqrt(rowSums(sweep(ord, 2, x)^2))
        colSums(sweep(up, 1, dn, "/"))
    }
    #dmedfun <- NULL
    pts <- scores(ord, display = display, ...)
    inds <- names(table(groups))
    medians <- matrix(NA, nrow = length(inds), ncol = ncol(pts))
    rownames(medians) <- inds
    colnames(medians) <- colnames(pts)
    for (i in inds) {
        X <- pts[groups == i, , drop = FALSE]
        if (NROW(X) > 0)
            medians[i, ] <- optim(apply(X, 2, median, na.rm = TRUE),
                              fn = medfun, gr = dmedfun,
                              ord = X, method = "BFGS")$par
        if(label)
            ordiArgAbsorber(medians[i,1], medians[i,2], label = i,
                            FUN = text, ...)
    }
    invisible(medians)
}

"plot.anosim" <-
function (x, title=NULL, ...) 
{
   boxplot(x$dis.rank ~ x$class.vec, notch=TRUE, varwidth=TRUE, 
           ...)
   title(title)
   if (x$permutations) {
     pval <- format.pval(x$signif)
   } else {
     pval <- "not assessed"
   }
   mtext(paste("R = ", round(x$statistic, 3), ", ",
               "P = ", pval ), 3)
   invisible()
}

`summary.taxondive` <-
function (object, ...) 
{ 
    z <- (object$Dplus - object$EDplus)/object$sd.Dplus
    pval <- 2*pnorm(-abs(z))
    out <- cbind(object$D, object$Dstar, object$Dplus, object$sd.Dplus, 
                 z, pval)
    out <- rbind(out, "Expected"=c(object$ED, object$EDstar, object$EDplus, NA, NA, NA))
    colnames(out) <- c("Delta", "Delta*", "Delta+", "sd(Delta+)", 
                       "z(Delta+)", "Pr(>|z|)")
    class(out) <- "summary.taxondive"
    out
}

`scores.orditkplot` <-
    function(x, display, ...)
{
    .Deprecated(msg = "function was moved to CRAN package vegan3d")
    if (!missing(display) && !is.na(pmatch(display, "labels")))
        x$labels
    else
        x$points
}

`drop1.cca` <-
    function(object, scope, test = c("none", "permutation"),
             permutations = how(nperm = 199), ...)
{
    if (inherits(object, "prc"))
        stop("'step'/'drop1' cannot be used for 'prc' objects")
    if (is.null(object$terms))
        stop("ordination model must be fitted using formula")
    test <- match.arg(test)
    out <- NextMethod("drop1", object, test="none", ...)
    cl <- class(out)
    if (test == "permutation") {
        rn <- rownames(out)[-1]
        if (missing(scope))
            scope <- rn
        else if (!is.character(scope))
            scope <- drop.scope(scope)
        adds <- anova(object, by = "margin", scope = scope,
                      permutations = permutations, ...)
        nr <- nrow(adds)
        out <- cbind(out, rbind(NA, adds[rn,3:4]))
        class(out) <- cl
    }
    out
}

SSgleason <-
    selfStart(~ k + slope*log(area),
              function(mCall, data, LHS, ...)
{
    ## Gleason is a linear model: starting values are final ones
    xy <- sortedXyData(mCall[["area"]], LHS, data)
    value <- as.vector(coef(lm(xy[,"y"] ~ log(xy[,"x"]))))
    names(value) <- mCall[c("k","slope")]
    value
},
c("k","slope"))

## CLAM, reproduction of software described in Chazdon et al. 2011
## Ecology, 92, 1332--1343
clamtest <-
function(comm, groups, coverage.limit = 10,
specialization = 2/3, npoints = 20, alpha = 0.05/20)
{
    ## inital checks
    comm <- as.matrix(comm)
    if (NROW(comm) < 2)
        stop("'comm' must have at least two rows")
    if (nrow(comm) > 2 && missing(groups))
        stop("'groups' is missing")
    if (nrow(comm) == 2 && missing(groups))
        groups <- if (is.null(rownames(comm)))
            c("Group.1", "Group.2") else rownames(comm)
    if (length(groups) != nrow(comm))
        stop("length of 'groups' must equal to 'nrow(comm)'")
    if (length(unique(groups)) != 2)
        stop("number of groups must be two")
    glabel <- as.character(unique(groups))
    if (is.null(colnames(comm)))
        colnames(comm) <- paste("Species", 1:ncol(comm), sep=".")
    if (any(colSums(comm) <= 0))
        stop("'comm' contains columns with zero sums")
    spp <- colnames(comm)
    ## reproduced from Chazdon et al. 2011, Ecology 92, 1332--1343
    S <- ncol(comm)
    if (nrow(comm) == 2) {
        Y <- comm[glabel[1],]
        X <- comm[glabel[2],]
    } else {
        Y <- colSums(comm[which(groups==glabel[1]),])
        X <- colSums(comm[which(groups==glabel[2]),])
    }
    names(X) <- names(Y) <- NULL
    #all(ct$Total_SG == Y)
    #all(ct$Total_OG == X)
    m <- sum(Y)
    n <- sum(X)
    if (sum(Y) <= 0 || sum(X) <= 0)
        stop("group totals of zero are not allowed")
    ## check if comm contains integer, especially for singletons
    if (any(X[X>0] < 1) || any(Y[Y>0] < 1))
        warning("non-integer values <1 detected: analysis may not be meaningful")
    if (abs(sum(X,Y) - sum(as.integer(X), as.integer(Y))) > 10^-6)
        warning("non-integer values detected")
    C1 <- 1 - sum(X==1)/n
    C2 <- 1 - sum(Y==1)/m
    ## this stands for other than 2/3 cases
    uu <- specialization/(1-specialization)
    ## critical level
    Zp <- qnorm(alpha, lower.tail=FALSE)
    #p_i=a
    #pi_i=b
    ## function to calculate test statistic from Appendix D
    ## (Ecological Archives E092-112-A4)
    ## coverage limit is count, not freq !!!
    testfun <- function(p_i, pi_i, C1, C2, n, m) {
        C1 <- ifelse(p_i*n < coverage.limit, C1, 1)
        C2 <- ifelse(pi_i*m < coverage.limit, C2, 1)
        Var <- C1^2*(p_i*(1-p_i)/n) + uu^2*C2^2*(pi_i*(1-pi_i)/m)
        C1*p_i - C2*pi_i*uu - Zp*sqrt(Var)
    }
    ## root finding for iso-lines (instead of itarative search)
    rootfun <- function(pi_i, C1, C2, n, m, upper) {
        f <- function(p_i) testfun(p_i/n, pi_i/m, C1, C2, n, m)
        if (length(unique(sign(c(f(1), f(upper))))) > 1)
        ceiling(uniroot(f, lower=1, upper=upper)$root) else NA
    }
    ## sequences for finding Xmin and Ymin values
    Xseq <- as.integer(trunc(seq(1, max(X), len=npoints)))
    Yseq <- as.integer(trunc(seq(1, max(Y), len=npoints)))
    ## finding Xmin and Ymin values for Xseq and Yseq
    Xmins <- sapply(Yseq, function(z) rootfun(z, C1, C2, n, m, upper=max(X)))
    Ymins <- sapply(Xseq, function(z) rootfun(z, C2, C1, m, n, upper=max(Y)))

    ## needed to tweak original set of rules (extreme case reported
    ## by Richard Telford failed here)
    if (all(is.na(Xmins)))
        Xmins[1] <- 1
    if (all(is.na(Ymins)))
        Ymins[1] <- 1

    minval <- list(data.frame(x=Xseq[!is.na(Ymins)], y=Ymins[!is.na(Ymins)]),
        data.frame(x=Xmins[!is.na(Xmins)], y=Yseq[!is.na(Xmins)]))

    ## shared but too rare
    Ymin <- Ymins[1]
    Xmin <- Xmins[1]
    sr <- X < Xmin & Y < Ymin

    ## consequence of manually setting Xmin/Ymin resolved here
    tmp1 <- if (Xmin==1)
        list(x=1, y=Xmin) else approx(c(Xmin, 1), c(1, Ymin), xout=1:Xmin)
    tmp2 <- if (Ymin==1)
        list(x=1, y=Ymin) else approx(c(1, Ymin), c(Xmin, 1), xout=1:Ymin)

    for (i in 1:S) {
        if (X[i] %in% tmp1$x)
            sr[i] <- Y[i] < tmp1$y[which(X[i]==tmp1$x)]
        if (Y[i] %in% tmp2$x)
            sr[i] <- X[i] < tmp2$y[which(Y[i]==tmp2$x)]
    }
    ## classification
    a <- ifelse(X==0, 1, X)/n # \hat{p_i}
    b <- ifelse(Y==0, 1, Y)/m # \hat{\pi_i}
    specX <- !sr & testfun(a, b, C1, C2, n, m) > 0
    specY <- !sr & testfun(b, a, C2, C1, m, n) > 0
    gen <- !sr & !specX & !specY
    ## crosstable
    tmp <- ifelse(cbind(gen, specY, specX, sr), 1, 0)
    types <- c("Generalist", paste("Specialist", glabel[1], sep="_"),
        paste("Specialist", glabel[2], sep="_"), "Too_rare")
    classes <- factor((1:4)[rowSums(tmp*col(tmp))], levels=1:4)
    levels(classes) <- c("Generalist", paste("Specialist", glabel[1], sep="_"),
        paste("Specialist", glabel[2], sep="_"), "Too_rare")
    tab <- data.frame(Species=spp, y=Y, x=X, Classes=classes)
    colnames(tab)[2:3] <- paste("Total", glabel, sep="_")
    rownames(tab) <- NULL
    class(tab) <- c("clamtest","data.frame")
    attr(tab, "settings") <- list(labels = glabel,
        coverage.limit = coverage.limit, specialization = specialization,
        npoints = npoints, alpha = alpha)
    attr(tab, "minv") <- minval
    attr(tab, "coverage") <- structure(c(C2, C1), .Names=glabel)
    tab
}

`plot.meandist` <-
    function(x, kind = c("dendrogram", "histogram"),  cluster = "average", ylim,
             axes = TRUE, ...)
{
    kind <- match.arg(kind)
    n <- attr(x, "n")
    if (kind == "dendrogram") {
        cl <- hclust(as.dist(x), method = cluster, members = n)
        cl <- as.dendrogram(cl, hang = 0)
        w <- diag(x)[labels(cl)]
        tr <- unlist(dendrapply(cl, function(n) attr(n, "height")))
        root <- attr(cl, "height")
        if (missing(ylim))
            ylim <- range(c(w, tr, root), na.rm = TRUE)
        plot(cl, ylim = ylim, leaflab = "none", axes = axes, ...)
        seqw <- seq_along(w)
        for (i in seqw) {
            segments(i, tr[i], i, w[i])
        }
        pos <- ifelse(w < tr, 1, 3)
        pos[is.na(pos)] <- 1
        w[is.na(w)] <- tr[is.na(w)]
        text(seqw, w, labels = labels(cl), pos = pos, srt = 0, xpd = TRUE, ...)
    } else {
        w <- diag(x)
        seqw <- seq_along(w)
        tr <- rep(summary(x)$B, length(w))
        if (missing(ylim))
            ylim <- range(c(w, tr), na.rm = TRUE)
        plot(seqw, tr, ylim = ylim, axes = FALSE, xlab = "", ylab = "",
             type = "l", ...)
        if (axes)
            axis(2, ...)
        for (i in seqw) segments(i, tr, i, w[i])
        pos <- ifelse(w < tr, 1, 3)
        pos[is.na(pos)] <- 1
        text(seqw, w, labels = names(n), pos = pos, srt = 0,
             xpd = TRUE, ...)
    }
}

`spenvcor` <-
    function (object)
{
    if (is.null(object$CCA))
        stop("needs results from constrained ordination")
    u <- object$CCA$u
    wa <- object$CCA$wa
    if (!inherits(object, "rda")) { # is CCA
        r <- sqrt(object$rowsum)
        u <- r * u
        wa <- r * wa
    }
    ## because colSums(u*u) = 1, we can simplify diag(cor(u, wa)) --
    ## and we must for weighted CA
    colSums(u * wa)/sqrt(colSums(wa^2))
}

`spantree` <-
    function (d, toolong = 0)
{
    if (!inherits(d, "dist")) {
        if ((is.matrix(d) || is.data.frame(d)) &&
            isSymmetric(unname(as.matrix(d)))) {
            d <- as.dist(d)
        } else {
            stop("input must be dissimilarities")
        }
    }
    if (!is.numeric(d))
        stop("input data must be numeric")
    n <- attr(d, "Size")
    labels <- labels(d)
    dis <- .C(primtree, dist = as.double(d), toolong = as.double(toolong),
              n = as.integer(n), val = double(n + 1),
              dad = integer(n + 1), NAOK = TRUE)
    out <- list(kid = dis$dad[2:n] + 1, dist = dis$val[2:n],
                labels = labels, n = n, call = match.call())
    class(out) <- "spantree"
    out
}

`plot.betadisper` <- function(x, axes = c(1,2), cex = 0.7, pch = seq_len(ng),
                              col = NULL, lty = "solid", lwd = 1, hull = TRUE,
                              ellipse = FALSE, conf,
                              segments = TRUE, seg.col = "grey",
                              seg.lty = lty, seg.lwd = lwd,
                              label = TRUE, label.cex = 1,
                              ylab, xlab, main, sub, ...)
{
    localAxis <- function(..., col, bg, pch, cex, lty, lwd) axis(...)
    localBox <- function(..., col, bg, pch, cex, lty, lwd) box(...)
    localTitle <- function(..., col, bg, pch, cex, lty, lwd) title(...)
    Ellipse <- function(scrs, centres, conf, col, lty, lwd, ...) {
        mat <- cov.wt(scrs, center = centres)
        if (mat$n.obs == 1)
            mat$cov[] <- 0
        xy <- if (mat$n.obs > 1) {
                  veganCovEllipse(mat$cov, mat$center, conf)
        } else {
            scrs
        }
        ordiArgAbsorber(xy, FUN = lines, col = col, lty = lty, lwd = lwd, ...)
    }
    if(missing(main))
        main <- deparse(substitute(x))
    if(missing(sub))
        sub <- paste("method = \"", attr(x, "method"), "\"", sep = "")
    if(missing(xlab))
        xlab <- paste("PCoA", axes[1])
    if(missing(ylab))
        ylab <- paste("PCoA", axes[2])
    t <- if (missing(conf)) {
        1
    } else {
        sqrt(qchisq(conf, df = 2))
    }
    g <- scores(x, choices = axes)
    ng <- length(levels(x$group))
    lev <- levels(x$group)
    ## sort out colour vector if none supplied
    if (is.null(col)) {
        col <- palette()
    }
    col <- rep_len(col, ng)        # make sure there are enough colors
    seg.col <- rep_len(seg.col, ng)     # ditto for segments
    plot(g$sites, asp = 1, type = "n", axes = FALSE, ann = FALSE, ...)
    ## if more than 1 group level
    if(is.matrix(g$centroids)) {
        for(i in seq_along(lev)) {
            curlev <- lev[i]
            take <- x$group == curlev
            j <- which(lev == curlev)
            if (segments) {
                segments(g$centroids[j, 1L], g$centroids[j, 2L],
                         g$sites[take, 1L],
                         g$sites[take, 2L], col = seg.col[i], lty = seg.lty,
                         lwd = seg.lwd)
            }
            if(hull) {
                ch <- chull(g$sites[take,, drop=FALSE])
                ch <- c(ch, ch[1])
                lines(x$vectors[take, axes, drop=FALSE][ch, ],
                      col = col[i], lty = lty, lwd = lwd, ...)
            }
            if (ellipse) {
                Ellipse(g$sites[take, , drop = FALSE],
                        centres = g$centroids[j, ],
                        conf = t,
                        col = col[i], lty = lty, lwd = lwd, ...)
            }
            points(g$centroids[j, , drop = FALSE], pch = 16, cex = 1,
                   col = col[i], ...)
        }
    } else {
        ## single group
        if (segments) {
            segments(g$centroids[, 1L], g$centroids[, 2L],
                     g$sites[, 1L], g$sites[, 2L], col = seg.col,
                     lty = seg.lty, ...)
        }
        if(hull) {
            ch <- chull(g$sites)
            ch <- c(ch, ch[1])
            lines(x$vectors[, axes][ch, ], col = col[1L], lty = lty,
                  lwd = lwd, ...)
        }
        if (ellipse) {
                Ellipse(g$sites,
                        centres = g$centroids,
                        conf = t,
                        col = col[1L], lty = lty, lwd = lwd,...)
        }
        points(g$centroids[, 1L], g$centroids[, 2L],
               pch = 16, cex = 1, col = col[1L], ...)
    }
    points(g$sites, pch = pch[x$group], cex = cex, col = col[x$group], ...)
    if (label) {
        ordilabel(x, display = "centroids", choices = axes, cex = label.cex)
    }
    localTitle(main = main, xlab = xlab, ylab = ylab, sub = sub, ...)
    localAxis(1, ...)
    localAxis(2, ...)
    localBox(...)
    class(g) <- "ordiplot"
    invisible(g)
}

`coef.radfit` <-
    function (object, ...) 
{
    out <- sapply(object$models, function(x) if (length(coef(x)) < 
                                                 3) 
                  c(coef(x), rep(NA, 3 - length(coef(x))))
    else coef(x))
    out <- t(out)
    colnames(out) <- paste("par", 1:3, sep = "")
    out
}

`coef.radfit.frame` <-
    function(object, ...)
{
    lapply(object, coef, ...)
}

`summary.isomap` <-
function (object, axes=4, ...) 
{
	axes <- min(axes, ncol(object$points))
	out <- list()
	out$call <- object$call
	out$points <- object$points[,1:axes]
	out$net <- object$net
	n <- nrow(object$points)
	out$ndis <- n * (n-1) / 2
	out$nnet <- nrow(object$net)
	class(out) <- "summary.isomap"
	out
}

`fieller.MOStest` <-
    function (object, level = 0.95) 
{
    smodel <- summary(object$mod)
    ## overdispersion included in cov.scaled
    var <- smodel$cov.scaled
    fam <- family(object$mod)
    k <- coef(object$mod)
    b2 <- -2 * k[3]
    u <- -k[2]/2/k[3]
    alpha <- (1-level)/2
    limits <- numeric(2)
    names(limits) <- paste(round(100*(c(alpha, 1-alpha)), 1), "%") 
    wvar <- var[2,2] 
    uvar <- 4 * var[3,3] 
    vvar <- -2 * var[2,3]
    z <- qnorm(1 - alpha)
    g <- z^2 * uvar/b2^2
    if (g >= 1) {
        limits <- c(NA, NA)
    }
    else {
        x <- u - g * vvar/uvar
        f <- z/b2
        s <- sqrt(wvar - 2 * u * vvar + u^2 * uvar - g * (wvar - 
                                                          vvar^2/uvar))
        limits[1] <- (x - f * s)/(1 - g)
        limits[2] <- (x + f * s)/(1 - g)
    }
    limits
}

`ordihull` <-
    function (ord, groups, display = "sites",
              draw = c("lines", "polygon", "none"),
              col = NULL, alpha = 127, show.groups, label = FALSE,
              border = NULL, lty = NULL, lwd = NULL, ...)
      
{
    draw <- match.arg(draw)
    ## Internal function to find the polygon centre
    polycentre <- function(x) {
        n <- nrow(x)
        if (n < 4) 
            return(colMeans(x[-n, , drop = FALSE]))
        xy <- x[-n, 1] * x[-1, 2] - x[-1, 1] * x[-n, 2]
        A <- sum(xy)/2
        xc <- sum((x[-n, 1] + x[-1, 1]) * xy)/A/6
        yc <- sum((x[-n, 2] + x[-1, 2]) * xy)/A/6
        c(xc, yc)
    }
    ## Make semitransparent fill colour; alpha should be integer
    ## 0..255, but we also handle real values < 1
    if (alpha < 1)
        alpha <- round(alpha * 255)
    if (draw == "polygon" && !is.null(col))
        col <- rgb(t(col2rgb(col)), alpha = alpha, maxColorValue = 255)
    pts <- scores(ord, display = display, ...)
    if (!missing(show.groups)) {
        take <- groups %in% show.groups
        pts <- pts[take, , drop = FALSE]
        groups <- groups[take]
    }
    out <- seq(along = groups)
    inds <- names(table(groups))
    
    ## fill in graphical vectors with default values if unspecified
    ## and recycles shorter vectors
    col.new <- border.new <- lty.new <- lwd.new <- NULL
    for(arg in c("col","border","lty","lwd")){
      tmp <- mget(arg,ifnotfound=list(NULL))[[1]]
      if(is.null(tmp))
          tmp <- ifelse(suppressWarnings(is.null(par(arg))),
                        par("fg"), par(arg))
      if(length(inds) != length(tmp))
          tmp <- rep_len(tmp, length(inds))
      assign(paste(arg,".new", sep=""), tmp)
    }
    ## default colour for "polygon" fill is "transparent", for lines
    ## is par("fg")
    if(is.null(col) && draw=="polygon")
        col.new <- rep_len("transparent", length(inds))
    else if(is.null(col) && draw=="lines")
        col.new <- rep_len(par("fg"), length(inds))
    res <- list()
    if (label) {
        cntrs <- matrix(NA, nrow=length(inds), ncol=2)
        rownames(cntrs) <- inds
    }
    ## Remove NA scores
    kk <- complete.cases(pts) & !is.na(groups)
    for (is in inds) {
        gr <- out[groups == is & kk]
        if (length(gr)) {
            X <- pts[gr,, drop = FALSE]
            hpts <- chull(X)
            hpts <- c(hpts, hpts[1])
            if (draw == "lines") 
                ordiArgAbsorber(X[hpts, ], FUN = lines,
                                col = if (is.null(col)) 
                                          par("fg")
                                      else
                                          col.new[match(is, inds)],
                                lty = lty.new[match(is,inds)],
                                lwd = lwd.new[match(is,inds)], ...)
            else if (draw == "polygon") 

                ordiArgAbsorber(X[hpts, ],
                                border= border.new[match(is,inds)],
                                FUN = polygon,
                                col = col.new[match(is, inds)],
                                lty = lty.new[match(is,inds)],
                                lwd=lwd.new[match(is,inds)], ...)

            if (label && draw != "none") {
                cntrs[is,] <- polycentre(X[hpts,])
            }
            res[[is]] <- X[hpts,]
        }
    }
    if (label && draw != "none") {
      if (draw == "lines") 
          ordiArgAbsorber(cntrs[, 1], cntrs[, 2],
                          labels = rownames(cntrs), 
                          col = col.new[match(is, inds)],
                          FUN = text, ...)
      else ordiArgAbsorber(cntrs, labels = rownames(cntrs),
                           col = NULL, 
                           FUN = ordilabel, ...)
    }
    class(res) <- "ordihull"
    invisible(res)
}

`metaMDS` <-
    function (comm, distance = "bray", k = 2, try = 20, trymax = 20,
              engine = c("monoMDS", "isoMDS"),
              autotransform = TRUE, noshare = (engine == "isoMDS"),
              wascores = TRUE, expand = TRUE, trace = 1,
              plot = FALSE, previous.best,  ...)
{
    engine <- match.arg(engine)
    ## take care that trymax >= try
    trymax <- max(trymax, try)
    ## This could be a character vector of length > 1L
    commname <- deparse(substitute(comm), width.cutoff = 500L)
    if (length(commname) > 1L) {
        paste(commname, collapse = "", sep = "")
        ## deparse can add more white space, so cull 2 or more spaces to a single space
        commname <- gsub("[ ]{2,}", " ", commname)
    }
    ## metaMDS was written for community data which should be all
    ## positive. Check this here, and set arguments so that they are
    ## suitable for non-negative data.
    if (any(autotransform, noshare > 0, wascores) && any(comm < 0, na.rm=TRUE)) {
        message("'comm' has negative data: 'autotransform', 'noshare' and 'wascores' set to FALSE")
        wascores <- FALSE
        autotransform <- FALSE
        noshare <- FALSE
    }
    if (inherits(comm, "dist")) {
        dis <- comm
        if (is.null(attr(dis, "method")))
            attr(dis, "method") <- "user supplied"
        wascores <- FALSE
    } else if ((is.matrix(comm) || is.data.frame(comm)) &&
               isSymmetric(unname(as.matrix(comm)))) {
        dis <- as.dist(comm)
        attr(dis, "method") <- "user supplied"
        wascores <- FALSE
    } else {
        if (trace > 2)
            cat(">>> Calculation of dissimilarities\n")
        dis <- metaMDSdist(comm, distance = distance,
                           autotransform = autotransform,
                           noshare = noshare, trace = trace,
                           commname = commname, ...)
    }
    if (missing(previous.best))
        previous.best <- NULL
    if (trace > 2)
        cat(">>> NMDS iterations\n")
    out <- metaMDSiter(dis, k = k, try = try, trymax = trymax,
                       trace = trace,
                       plot = plot, previous.best = previous.best,
                       engine = engine, ...)
    ## Nearly zero stress is usually not a good thing but a symptom of
    ## a problem: you may have insufficient data for NMDS
    if (out$stress < 1e-3) {
        warning("stress is (nearly) zero: you may have insufficient data")
    }
    if (trace > 2)
        cat(">>> Post-processing NMDS\n")
    points <- postMDS(out$points, dis, plot = max(0, plot - 1), ...)
    ## rescale monoMDS scaling if postMDS scaled 'points'
    if (engine == "monoMDS" &&
        !is.null(scl <- attr(points, "internalscaling"))) {
        out$dist <- out$dist/scl
        out$dhat <- out$dhat/scl
    }
    if (is.null(rownames(points)))
        rownames(points) <- rownames(comm)
    wa <- if (wascores) {
        ## transformed data
        ##comm <- eval.parent(parse(text=attr(dis, "commname")))
        comm <- attr(dis, "comm")
        wascores(points, comm, expand = expand)
    } else {
        NA
    }
    out$points <- points
    out$species <- wa
    out$call <- match.call()
    if (is.null(out$data))
        out$data <- commname
    class(out) <- c("metaMDS", class(out))
    out
}

`simulate.rda` <-
    function(object, nsim = 1, seed = NULL, indx = NULL, rank = "full",
             correlated = FALSE, ...)
{
    ## Fail if there is no constrained component (it could be possible
    ## to change the function to handle unconstrained ordination, too,
    ## when rank < "full", but that would require redesign)
    if (is.null(object$CCA))
        stop("function can be used only with constrained ordination")

    ## Handle RNG: code directly from stats::simulate.lm
    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
        runif(1)
    if (is.null(seed))
        RNGstate <- get(".Random.seed", envir = .GlobalEnv)
    else {
        R.seed <- get(".Random.seed", envir = .GlobalEnv)
        set.seed(seed)
        RNGstate <- structure(seed, kind = as.list(RNGkind()))
        on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
    }
    ## indx can be an output of permute::shuffleSet in which case it
    ## is a nsim x nrow matrix, or it may be a single vector, in which
    ## case it will changed to shuffleSet
    if (!is.null(indx))
        if (is.vector(indx))
            dim(indx) <- c(1, length(indx))
    ## If nsim is missing, take it from indx (if given)
    if (missing(nsim) && !is.null(indx))
        nsim <- nrow(indx)
    ## Check that dims match
    if (!is.null(indx))
        if(nrow(indx) != nsim)
            stop(gettextf("'nsim' (%d) and no. of 'indx' rows (%d) do not match",
                          nsim, nrow(indx)))
    ## collect data to back-transform data to the scale of observations
    sqnr1 <- sqrt(nobs(object) - 1)
    ## the ifs are only needed to cope with pre-2.5-0 vegan: now
    ## we always have Ybar, but earlier we needed to check whether
    ## we had CA or CCA Xbar
    if (!is.null(object$Ybar)) {
        cnt <- attr(object$Ybar, "scaled:center")
        scl <- attr(object$Ybar, "scaled:scale")
    } else { # needed for vegan-2.4 compatibility
        if (is.null(object$CCA))
            tmp <- object$CA$Xbar
        else tmp <- object$CCA$Xbar
        cnt <- attr(tmp, "scaled:center")
        scl <- attr(tmp, "scaled:scale")
    }

    ## Proper simulation: very similar for simulate.lm, but produces
    ## an array of response matrices

    ftd <- predict(object, type = "working", rank = rank)
    ## pRDA: add partial Fit to the constrained
    if (!is.null(object$pCCA))
        ftd <- ftd + ordiYbar(object, "pCCA")
    ## if(is.null(indx)), we have parametric Gaussian simulation and
    ## need to generate sd matrices. The residuals sd is always taken

    ## from the unconstrained (residual) component. If
    ## species are uncorrelated, we need only species sd's, but if
    ## correlated, we also need species covariances.
    CAYbar <- ordiYbar(object, "CA")
    if (!correlated)
        dev <- outer(rep(1, nrow(ftd)), apply(CAYbar, 2, sd))
    else
        dev <- cov(CAYbar)
    ## Generate an array
    ans <- array(0, c(dim(ftd), nsim))
    for (i in seq_len(nsim)) {
        if (!is.null(indx))
            ans[,,i] <- as.matrix(ftd + CAYbar[indx[i,],])
        else if (!correlated)
            ans[,,i] <- as.matrix(ftd + matrix(rnorm(length(ftd), sd = dev),
                                               nrow = nrow(ftd)))
        else {
            ans[,,i] <- t(apply(ftd, 1,
                                function(x) mvrnorm(1, mu = x, Sigma = dev)))
        }
        ## ans to the scale of observations
        ans[,,i] <- ans[,,i] * sqnr1
        if (!is.null(scl))
            ans[,,i] <- sweep(ans[,,i], 2, scl, "*")
        ans[,,i] <- sweep(ans[,,i], 2, cnt, "+")
    }
    ## set RNG attributes
    if (is.null(indx))
        attr(ans, "seed") <- RNGstate
    else
        attr(ans, "seed") <- "index"
    ## set commsim attributes if nsim > 1, else return a 2-dim matrix
    if (nsim == 1) {
        ans <- ans[,,1]
        attributes(ans) <- attributes(ftd)
    } else {
        dimnames(ans) <- list(rownames(ftd), colnames(ftd),
                              paste("sim", seq_len(nsim), sep = "_"))
        orig <- (ftd + CAYbar) * sqnr1
        if (!is.null(scl))
            orig <- sweep(orig, 2, scl, "*")
        orig <- sweep(orig, 2, cnt, "+")
        attr(ans, "data") <- round(orig, 12)
        attr(ans, "method") <- paste("simulate", ifelse(is.null(indx),
                                                        "parametric", "index"))
        attr(ans, "binary") <- FALSE
        attr(ans, "isSeq") <- FALSE
        attr(ans, "mode") <- "double"
        attr(ans, "start") <- 1L
        attr(ans, "end") <- as.integer(nsim)
        attr(ans, "thin") <- 1L
        class(ans) <- c("simulate.rda", "simmat", "array")
    }
    ans
}

### simulate. cca was cloned from simulate.rda.  Works with internal
### Chi-square standardized form, and at the end back-standardizes
### with row and column totals and matrix grand totals. This does not
### still guarantee that all marginal totals are positive.

`simulate.cca` <-
    function(object, nsim = 1, seed = NULL, indx = NULL, rank = "full",
             correlated = FALSE, ...)
{
    ## Fail if no CCA
    if (is.null(object$CCA))
        stop("function can be used only with constrained ordination")
    ## Handle RNG: code directly from stats::simulate.lm
    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
        runif(1)
    if (is.null(seed))
        RNGstate <- get(".Random.seed", envir = .GlobalEnv)
    else {
        R.seed <- get(".Random.seed", envir = .GlobalEnv)
        set.seed(seed)
        RNGstate <- structure(seed, kind = as.list(RNGkind()))
        on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
    }
    ## Preparations like in simulate.rda()
    if (!is.null(indx))
        if (is.vector(indx))
            dim(indx) <- c(1, length(indx))
    if (missing(nsim) && !is.null(indx))
        nsim <- nrow(indx)
    if (!is.null(indx))
        if(nrow(indx) != nsim)
            stop(gettextf("'nsim' (%d) and no. of 'indx' rows (%d) do not match",
                          nsim, nrow(indx)))
    ## Need sqrt of rowsums for weighting
    sq.r <- sqrt(object$rowsum)
    ## Fitted value
    ftd <- predict(object, type = "working", rank = rank)
    ## pCCA: add partial Fit to the constrained
    if (!is.null(object$pCCA))
        ftd <- ftd + ordiYbar(object, "pCCA")
    ## Residual Xbar need weighting and back-weighting
    Xbar <- sweep(ordiYbar(object, "CA"), 1, sq.r, "*")
    ## Simulation
    if (correlated)
        dev <- cov(Xbar)
    else
        dev <- outer(rep(1, nrow(ftd)), apply(Xbar, 2, sd))
    ans <- array(0, c(dim(ftd), nsim))
    for (i in seq_len(nsim)) {
        if (is.null(indx)) {
            if (correlated)
                tmp <- mvrnorm(nrow(ftd), numeric(ncol(ftd)), Sigma = dev)
            else
                tmp <- matrix(rnorm(length(ftd), sd = dev),
                          nrow = nrow(ftd))
            ans[,,i] <- as.matrix(ftd + sweep(tmp, 1, sq.r, "/"))
        }
        else
            ans[,,i] <- as.matrix(ftd + sweep(Xbar[indx[i,],], 1, sq.r, "/"))
    }
    ## From internal form to the original form with fixed marginal totals
    rc <- object$rowsum %o% object$colsum
    for (i in seq_len(nsim))
        ans[,,i] <- (ans[,,i] * sqrt(rc) + rc) * object$grand.total
    ## RNG attributes
    if (is.null(indx))
        attr(ans, "seed") <- RNGstate
    else
        attr(ans, "seed") <- "index"
    ## set commsim attributes if nsim > 1, else return a 2-dim matrix
    if (nsim == 1) {
        ans <- ans[,,1]
        attributes(ans) <- attributes(ftd)
    } else {
        dimnames(ans) <- list(rownames(ftd), colnames(ftd),
                              paste("sim", seq_len(nsim), sep = "_"))
        obsdata <- ordiYbar(object, "initial")
        obsdata <- (obsdata * sqrt(rc) + rc) * object$grand.total
        attr(ans, "data") <- round(obsdata, 12)
        attr(ans, "method") <- paste("simulate", ifelse(is.null(indx),
                                                        "parametric", "index"))
        attr(ans, "binary") <- FALSE
        attr(ans, "isSeq") <- FALSE
        attr(ans, "mode") <- "double"
        attr(ans, "start") <- 1L
        attr(ans, "end") <- as.integer(nsim)
        attr(ans, "thin") <- 1L
        class(ans) <- c("simulate.cca", "simmat", "array")
    }
    ans
}


### capscale method: copies simulate.rda as much as possible. Function
### works with the internal metric scaling mapping of fit and error,
### but returns Euclidean distances adjusted to the original scaling
### of input dissimilarities. Only the real components are used, and
### capscale() of simulated dissimilarities have no Imaginary
### component.

`simulate.capscale` <-
    function(object, nsim = 1, seed = NULL, indx = NULL, rank = "full",
             correlated = FALSE, ...)
{
    ## Fail if no CCA component
    if (is.null(object$CCA))
        stop("function can be used only with constrained ordination")
    if (is.null(indx) && correlated)
        warning("argument 'correlated' does not work and will be ignored")
    ## Handle RNG: code directly from stats::simulate.lm
    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
        runif(1)
    if (is.null(seed))
        RNGstate <- get(".Random.seed", envir = .GlobalEnv)
    else {
        R.seed <- get(".Random.seed", envir = .GlobalEnv)
        set.seed(seed)
        RNGstate <- structure(seed, kind = as.list(RNGkind()))
        on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
    }
    if (nsim > 1)
        .NotYetUsed("nsim")
    ## predict.capscale cannot be used because it returns either
    ## dissimilarities ("response") or scores with the rank of the
    ## constrained solution, and we need rank of the data (not of
    ## constraints).
    if (rank > 0) {
        ftd <- ordiYbar(object, "CCA")
        ## redo analysis when rank < full
        if (rank < object$CCA$rank) {
            x <- svd(ftd, nu = rank, nv = rank)
            ftd <- x$u %*% diag(x$d[1:rank], nrow=rank) %*% t(x$v)
        }
    } else {
        ftd <- 0
    }
    ## add partial Fit to the constrained
    if (!is.null(object$pCCA))
        ftd <- ftd + ordiYbar(object, "pCCA")
    if (is.null(indx))
        ans <- as.data.frame(ftd + matrix(rnorm(length(ftd),
               sd = outer(rep(1,nrow(ftd)), apply(ordiYbar(object, "CA"), 2, sd))),
               nrow = nrow(ftd)))
    else
        ans <- ftd + ordiYbar(object, "CA")[indx,]
    ## return Euclidean distances
    ans <- ans * object$adjust
    ans <- dist(ans)
    ## remove adjustment done in capscale and put dissimilarities to
    ## (approximately) original scale
    if (is.null(indx))
        attr(ans, "seed") <- RNGstate
    else
        attr(ans, "seed") <- indx
    ans
}

### simulate.dbrda cannot be done along similar lines as
### simulate.capscale, because low-rank approximation needs column
### scores v and cannot be found only from row scores u that are the
### only ones we have in dbrda(). Residuals also need exra thinking,
### and therefore we just disable simulate.dbrda()

`simulate.dbrda` <-
    function(object, nsim = 1, seed = NULL, ...)
{
    .NotYetImplemented()
}

`profile.humpfit` <-
    function(fitted, parm=1:3, alpha=0.01, maxsteps = 20, del = zmax/5, ...)
{
    INSERT3 <- function(vec, fix, val) {
        switch(fix,
               c(val, vec),
               c(vec[1], val, vec[2]),
               c(vec, val)
               )
    }
    HUMP <- function(p, mass, spno, fix, val, ...) {
        b <- INSERT3(p, fix, val)
        x <- ifelse(mass < b[1], mass/b[1], b[1]*b[1]/mass/mass)
        fv <- b[3] * log(1 + b[2]*x/b[3])
        n <- wt <- rep(1, length(x))
        dev <- sum(dev.resids(spno, fv, wt))
        aicfun(spno, n, fv, wt, dev)/2
    }
    dev.resids <- fitted$family$dev.resids
    aicfun <- fitted$family$aic
    minll <- fitted$nlm$minimum
    p <- coefficients(fitted)
    pv0 <- t(as.matrix(p))
    n <- length(fitted$residuals)
    Pnames <- names(p)
    summ <- summary(fitted)
    dispersion <- summ$dispersion
    std.err <- summ$est[, "Std. Error"]
    if (summ$family == "poisson") {
        zmax <- sqrt(qchisq(1 - alpha/2, 3))
        profName <- "z"
    } else {
        zmax <- sqrt(3 * qf(1 - alpha/2, 3, n-3))
        profName <- "tau"
    }
    prof <- vector("list", length = length(parm))
    names(prof) <- Pnames[parm]
    for (i in parm) {
        zi <- 0
        par <- pv0
        pvi <- pv0[-i]
        pi <- Pnames[i]
        for (sgn in c(-1, 1)) {
            step <- 0
            z <- 0
            while ((step <- step + 1) < maxsteps && abs(z) < zmax) {
                bi <- p[i] + sgn * step * del * std.err[i]
                fm <- nlm(HUMP, p = pvi, mass = fitted$x, spno = fitted$y, fix = i, val = bi)
                pvi <- fm$estimate
                ri <- INSERT3(pvi, i, bi)
                names(ri) <- Pnames
                par <- rbind(par, ri)
                zz <- 2*(fm$minimum - minll)/dispersion
                if (zz > -0.001)
                    zz <- max(0, zz)
                else
                    stop(gettextf(
                        "profiling found a better solution:  original fit had not converged:\n%s: %f",
                        Pnames[i], bi))
                z <- sgn*sqrt(zz)
                zi <- c(zi, z)
            }
        }
        si <- order(zi)
        prof[[pi]] <- structure(data.frame(zi[si]), names= profName)
        prof[[pi]]$par.vals <- par[si,]
    }
    val <- structure(prof, original.fit = fitted, summary = summ)
    class(val) <- c("profile.humpfit", "profile.glm", "profile")
    val
}

'print.mantel.correlog' <- function(x, ...)
{
    cat('\nMantel Correlogram Analysis\n')
    cat('\nCall:\n','\n')
    cat(deparse(x$call),'\n')
    cat('\n')
	printCoefmat(x$mantel.res, P.values=TRUE, signif.stars=TRUE, Pvalues = TRUE)
    invisible(x) 
}

`print.vectorfit` <-
    function (x, ...) 
{
    out <- cbind(x$arrows, r2 = x$r, "Pr(>r)" = x$pvals)
    printCoefmat(out, na.print = "",
                 zap.ind = seq_len(ncol(out)-2), ...)
    if (x$permutations) {
        cat(howHead(x$control))
    }
    invisible(x)
}

## Extract the points in the hull as a one matrix
`scores.ordihull` <-
    function(x, ...)
{
    out <- NULL
    for(i in seq_along(x))
        out <- rbind(out, x[[i]])
    hulls <- rep(names(x), sapply(x, function(z) NROW(z)))
    attr(out, "hulls") <- hulls
    out
}

`print.permutest.betadisper` <- function(x, digits = max(getOption("digits") - 2, 3),
                                         ...)
{
    ## uses code from stats:::print.anova by R Core Development Team
    cat("\n")
    writeLines(strwrap("Permutation test for homogeneity of multivariate dispersions\n"))
    ##cat("\n")
    cat(howHead(x$control))
    nc <- dim(x$tab)[2]
    cn <- colnames(x$tab)
    has.P <- substr(cn[nc], 1, 3) == "Pr("
    zap.i <- 1:(if (has.P)
                nc - 1
    else nc)
    i <- which(substr(cn, 2, 7) == " value")
    i <- c(i, which(!is.na(match(cn, "F"))))
    if (length(i))
        zap.i <- zap.i[!(zap.i %in% i)]
    tst.i <- i
    if (length(i <- grep("Df$", cn)))
        zap.i <- zap.i[!(zap.i %in% i)]
    if (length(i <- grep("N.Perm$", cn)))
        zap.i <- zap.i[!(zap.i %in% i)]
    cat("\nResponse: Distances", sep = "\n")
    printCoefmat(x$tab, digits = digits,
                 signif.stars = getOption("show.signif.stars"),
                 has.Pvalue = has.P, P.values = has.P, cs.ind = NULL,
                 zap.ind = zap.i, tst.ind = tst.i, na.print = "", ...)
    if(!is.null(x$pairwise)) {
        cat("\nPairwise comparisons:", sep = "\n")
        writeLines(strwrap("(Observed p-value below diagonal,\npermuted p-value above diagonal)\n"))
        n.grp <- length(x$groups)
        mat <- matrix(NA, ncol = n.grp, nrow = n.grp)
        colnames(mat) <- rownames(mat) <- x$groups
        mat[lower.tri(mat)] <- x$pairwise$observed
        mat <- t(mat)
        mat[lower.tri(mat)] <- x$pairwise$permuted
        printCoefmat(t(mat), na.print = "", digits = digits)
    }
    invisible(x)
}

`plot.envfit` <-
    function (x, choices = c(1, 2), labels, arrow.mul, at = c(0, 0),
              axis = FALSE, p.max = NULL, col = "blue", bg, add = TRUE, ...)
{
    formals(arrows) <- c(formals(arrows), alist(... = ))
    ## get labels
    labs <- list("v" = rownames(x$vectors$arrows),
                 "f" = rownames(x$factors$centroids))
    ## Change labels if user so wishes
    if (!missing(labels)) {
        ## input list of "vectors" and/or "factors"
        if (is.list(labels)) {
            if (!is.null(labs$v) && !is.null(labels$vectors))
                labs$v <- labels$vectors
            if (!is.null(labs$f) && !is.null(labels$factors))
                labs$f <- labels$factors
        } else {
            ## input vector: either vectors or factors must be NULL,
            ## and the existing set of labels is replaced
            if (!is.null(x$vectors) && !is.null(x$factors))
                stop("needs a list with both 'vectors' and 'factors' labels")
            ## need to handle the case where both sets of labels are NULL
            ## such as when used with the default interface and single x
            if (!is.null(x$factors))
                labs$f <- labels
            else
                labs$v <- labels
        }
    }
    vect <- NULL
    if (!is.null(p.max)) {
        if (!is.null(x$vectors)) {
            take <- x$vectors$pvals <= p.max
            x$vectors$arrows <- x$vectors$arrows[take, , drop = FALSE]
            labs$v <- labs$v[take]
            x$vectors$r <- x$vectors$r[take]
            if (nrow(x$vectors$arrows) == 0)
                x$vectors <- vect <- NULL
        }
        if (!is.null(x$factors)) {
            tmp <- x$factors$pvals <= p.max
            nam <- names(tmp)[tmp]
            take <- x$factors$var.id %in% nam
            x$factors$centroids <- x$factors$centroids[take,
                                                       , drop = FALSE]
            labs$f <- labs$f[take]
            if (nrow(x$factors$centroids) == 0)
                x$factors <- NULL
        }
    }
    if (!is.null(x$vectors)) {
        vect <- sqrt(x$vectors$r) * x$vectors$arrows[, choices, drop = FALSE]
        if (missing(arrow.mul)) {
            if(!add)
                arrow.mul <- 1
            else
                arrow.mul <- ordiArrowMul(vect, at = at)
        }
        if (axis) {
            maxarr <- round(sqrt(max(x$vectors$r)), 1)
            ax <- -c(-1, 0, 1) * arrow.mul * maxarr
        }
        vect <- arrow.mul * vect
        vect <- sweep(vect, 2, at, "+")
        if (add) {
            vtext <- ordiArrowTextXY(vect, labs$v, rescale = FALSE, at = at,
                                     ...)
        }
    }
    if (!add) {
        plot.new()
        ## needed for string widths and heights We need dimensions to
        ## fit completely the names of vectors and factors with no
        ## clipping at axes. If there are (1) factors and vectors, we
        ## need to adjust arrow lengths, if there are (2) only factors
        ## or only vectors, we can use their scores directly. After
        ## finding the scores, we must expand the scores by string
        ## widths and heights. The expansion can be only estimated
        ## after setting plot.window with its xlim and ylim, but we
        ## need to find xlim and ylim to set the plot.window...

        if(is.null(vect) || is.null(x$factors)) {
            ## Only factors or vectors: set preliminary plot.window
            xstack <- rbind(vect, x$factors$centroids)
            plot.window(xlim = range(xstack[,1], at[1]),
                        ylim = range(xstack[,2], at[2]),
                        asp = 1, ...)
        } else {
            ## Both vectors and factors: set preliminary plot.window
            ## from factors only and and find arrow.mul (which is
            ## otherwise ## arrow.mul = 1)
            plot.window(xlim = range(x$factors$centroids[,1], at[1]),
                        ylim = range(x$factors$centroids[,2], at[2]),
                        asp = 1, ...)
            vfill <- 0.75
            arrow.mul <- ordiArrowMul(vect, at = at, fill = 1)
            vect <- arrow.mul * vect
        }
        ## Get string dimensions (width/2, height)
        sw <- strwidth(c(labs$v, labs$f), ...) / 2
        sh <- strheight(c(labs$v, labs$f), ...)
        ## Reset limits
        xstack <- rbind(x$factors$centroids, vect)
        xlim <- range(xstack[,1] + sw, xstack[,2] - sw)
        ylim <- range(xstack[,2] + sh, xstack[,2] - sh)
        plot.window(xlim = xlim, ylim = ylim, asp = 1, ...)
        ## Re-evaluate arrow.mul, set its text and re-evaluate limits again
        if (!is.null(vect)) {
            arrow.mul <- ordiArrowMul(vect, at = at, fill = 1)
            vect <- arrow.mul * vect
            vtext <- ordiArrowTextXY(vect, labs$v, at = at, ...)
            sw <- strwidth(labs$v, ...) / 2
            sh <- strheight(labs$v, ...)
            xlim <- range(xlim, vtext[,1] + sw, vtext[,1] - sw)
            ylim <- range(xlim, vtext[,2] + sh, vtext[,2] - sh)
            plot.window(xlim = xlim, ylim = ylim, asp = 1, ...)
        }
        axis(side = 1, ...)
        axis(side = 2, ...)
        box(...)
        alabs <- colnames(vect)
        title(..., ylab = alabs[2], xlab = alabs[1])
    }

    if (!is.null(vect)) {
        arrows(at[1], at[2], vect[, 1], vect[, 2], len = 0.05,
               col = col)
        if (missing(bg))
            text(vtext, labs$v, col = col, ...)
        else
            ordilabel(vtext, labels = labs$v, col = col, fill = bg, ...)
    }
    if (!is.null(x$factors)) {
        if (missing(bg))
            text(x$factors$centroids[, choices, drop = FALSE],
                 labs$f, col = col, ...)
        else
            ordilabel(x$factors$centroids[, choices, drop = FALSE],
                      labels = labs$f, col = col, fill = bg, ...)
    }
    if (axis && !is.null(vect)) {
        axis(3, at = ax + at[1], labels = c(maxarr, 0, maxarr),
             col = col)
        axis(4, at = ax + at[2], labels = c(maxarr, 0, maxarr),
             col = col)
    }
    invisible()
}

"text.decorana" <-
    function (x, display = c("sites", "species"), labels, choices = 1:2,
              origin = TRUE, select, ...)
{
    localText <- function(..., shrink, origin, scaling, triangular)
        text(...)
    display <- match.arg(display)
    x <- scores(x, display = display, choices = choices, origin = origin,
                ...)
    if (!missing(labels))
        rownames(x) <- labels
    if (!missing(select))
        x <- .checkSelect(select, x)
    localText(x, rownames(x), ...)
    invisible()
}

`predict.decorana` <-
    function (object, newdata, type = c("response", "sites", "species"),
              rank = 4, ...)
{
    type <- match.arg(type)
    u <- object$rproj[, 1:rank, drop = FALSE]
    v <- object$cproj[, 1:rank, drop = FALSE]
    orig <- object$origin[1:rank]
    u <- sweep(u, 2, orig, "-")
    v <- sweep(v, 2, orig, "-")
    rs <- object$aidot
    cs <- object$adotj
    tot <- sum(rs)
    rs <- rs/tot
    cs <- cs/tot
    lam <- object$evals[1:rank]
    if (type == "response") {
        if (!object$ira && rank > 1)
            stop("prediction is unavailable in detrended CA beyond first axis")
        Xbar <- 0
        if (rank > 0) {
            if (!object$ira) {
                tmp <- sweep(v, 1, sqrt(cs), "*")
                tmp <- rbind(tmp, sweep(u, 1, sqrt(rs), "*"))
                rot <- svd(tmp)$v
                v <- v %*% rot
                u <- u %*% rot
                fac <- colSums(sweep(v^2, 1, cs, "*"))
                lam <- (fac - 1)/fac
            }
            Xbar <- u %*% diag(1 - lam, nrow = rank) %*% t(v)
        }
        out <- (Xbar + 1) * outer(rs, cs) * tot
    }
    else if (type == "sites") {
        if (!missing(newdata)) {
            Xbar <- as.matrix(newdata)
            if (!is.null(object$v))
                Xbar <- sweep(Xbar, 2, object$v, "*")
            rs <- rowSums(Xbar)
            Xbar <- (Xbar - outer(rs, cs))/sqrt(outer(rs, cs))
            v <- sweep(v, 1, sqrt(cs), "*")
            u <- sweep(Xbar %*% v, 1, sqrt(rs), "/")
        }
        out <- u
    }
    else if (type == "species") {
        if (!missing(newdata)) {
            if (!object$ira && rank > 1)
                stop("type = 'species' not available in detrended CA with 'newdata'")
            if (object$iresc)
                stop("type = 'species' not available in rescaled DCA with 'newdata'")
            Xbar <- as.matrix(newdata)
            cs <- colSums(Xbar)
            Xbar <- (Xbar - outer(rs, cs))/sqrt(outer(rs, cs))
            u <- sweep(u, 1, sqrt(rs), "*")
            v <- sweep(t(Xbar) %*% u, 1, sqrt(cs), "/")
            v <- sweep(v, 2, lam, "/")
        }
        out <- v
    }
    out
}

`screeplot.decorana` <-
    function(x, bstick = FALSE, type = c("barplot", "lines"),  npcs = 4,
             ptype = "o", bst.col = "red", bst.lty = "solid",
             xlab = "Component", ylab = "Inertia",
             main = deparse(substitute(x)), legend = bstick, ...)
{
    eig.vals <- if (x$ira == 1) x$evals else x$evals.ortho
    comps <- seq(len=npcs)
    type <- match.arg(type)
    if (bstick) {
        ord.bstick <- bstick(x)
        ylims <- range(0, eig.vals[comps], ord.bstick[comps])
    } else {
        ylims <- range(c(0, eig.vals))
    }
    if(type=="barplot") {
        mids <- barplot(eig.vals[comps], names = names(eig.vals[comps]),
                        main = main, ylab = ylab, ylim = ylims, ...)
    } else {
        plot(eig.vals[comps], type = ptype, axes = FALSE,
             xlab = xlab, ylab = ylab, main = main, ylim = ylims, ...)
        axis(2)
        axis(1, at = comps, labels = names(eig.vals[comps]))
        mids <- comps
        box()
    }
    if (bstick) {
        dot.args <- list(...)
        dot.nams <- names(dot.args)
        pch <- if ("pch" %in% dot.nams)
                   dot.args$pch
               else
                   par("pch")
        lines(mids, ord.bstick[comps], type = ptype, col = bst.col,
              lty = bst.lty, pch = pch)
        if (legend) {
            col <- if ("col" %in% dot.nams)
                       dot.args$col
                   else
                       par("col")
            lty <- if ("lty" %in% dot.nams)
                       dot.args$lty
                   else
                       par("lty")
            if (type == "lines") {
                legend("topright", legend = c("Ordination", "Broken Stick"),
                       bty = "n", col = c(col, bst.col),
                       lty = c(lty, bst.lty), pch = pch)
            }
            else {
                legend("topright", legend = "Broken Stick", bty = "n",
                       col = bst.col, lty = bst.lty, pch = pch)
            }
        }
    }
    invisible(xy.coords(x = mids, y = eig.vals[comps]))
    }

### The constrained ordination methods cca, rda, capscale & dbrda are
### very similar. Their main difference is the init of dependent
### matrix and after that the partializing, constraining and residual
### steps are very similar to each other. This file provides functions
### that can analyse any classic method, the only difference being the
### attributes of the dependet variable.

### In this file we use convention modelfunction(Y, X, Z) where Y is
### the dependent data set (community), X is the model matrix of
### constraints, and Z is the model matrix of conditions to be
### partialled out. The common outline of the function is:
###
### initX(Y)
### pCCA <- ordPartial(Y, Z)
### CCA <- ordConstrain(Y, X, Z)
### CA <- ordResid(Y)
###
### The init step sets up the dependent data Y and sets up its
### attributes that control how it will be handled at later
### stage. Each of the later stages modifies Y and returns results of
### its analysis. The handling of the function is mainly similar, but
### there is some variation depending on the attributes of Y.

### THE USAGE

### Function prototype is ordConstrained(Y, X=NULL, Z=NULL, method),
### where Y is the dependent community data set, X is the model matrix
### of constraints, Z the model matrix of conditions, method is "cca",
### "rda", "capscale" or "dbrda" (the last two may not work, and
### "capscale" may not work in the way you assume). The function
### returns a large subset of correspoding constrained ordination
### method. For instance, with method = "dbrda", the result is mainly
### correct, but it differs so much from the current dbrda that it
### cannot be printed cleanly.

### THE INIT METHODS

### The init methods transform the dependent data specifically to the
### particular method and set up attributes that will control further
### processing of Y. The process critical attributes are set up in
### UPPER CASE to make the if-statements stand out in later analysis.

`initPCA` <-
    function(Y, scale = FALSE)
{
    Y <- as.matrix(Y)
    Y <- scale(Y, scale = scale)
    if (scale && any(is.nan(Y)))
        Y[is.nan(Y)] <- 0
    ## we want models based on variance or correlations -- this will
    ## break methods depending on unscaled Xbar (and which do this
    ## very same scaling internally) with scale = FALSE.
    Y <- Y / sqrt(nrow(Y) - 1)
    attr(Y, "METHOD") <- "PCA"
    Y
}

`initCAP` <-
    function(Y)
{
    Y <- as.matrix(Y)
    Y <- scale(Y, scale = FALSE)
    attr(Y, "METHOD") <- "CAPSCALE"
    Y
}

`initCA` <-
    function(Y)
{
    Y <- as.matrix(Y)
    tot <- sum(Y)
    Y <- Y/tot
    rw <- rowSums(Y)
    cw <- colSums(Y)
    rc <- outer(rw, cw)
    Y <- (Y - rc)/sqrt(rc)
    attr(Y, "tot") <- tot
    attr(Y, "RW") <- rw
    attr(Y, "CW") <- cw
    attr(Y, "METHOD") <- "CA"
    Y
}

`initDBRDA` <-
    function(Y)
{
    ## check
    Y <- as.matrix(Y)
    dims <- dim(Y)
    if (dims[1] != dims[2] || !isSymmetric(unname(Y)))
        stop("input Y must be distances or a symmetric square matrix")
    ## transform
    Y <- -0.5 * GowerDblcen(Y^2)
    attr(Y, "METHOD") <- "DISTBASED"
    Y
}

### COMMON HEADER INFORMATION FOR ORDINATION MODELS

`ordHead`<- function(Y)
{
    method <- attr(Y, "METHOD")
    headmethod <- switch(method,
                         "CA" = "cca",
                         "PCA" = "rda",
                         "CAPSCALE" = "capscale",
                         "DISTBASED" = "dbrda")
    if (method == "DISTBASED")
        totvar <- sum(diag(Y))
    else
        totvar <- sum(Y^2)
    head <- list("tot.chi" = totvar, "Ybar" = Y, "method" = headmethod)
    if (method == "CA")
        head <- c(list("grand.total" = attr(Y, "tot"),
                       "rowsum" = attr(Y, "RW"),
                       "colsum" = attr(Y, "CW")),
                  head)
    else if (method == "PCA")
        head <- c(list("colsum" = sqrt(colSums(Y^2))),
                  head)
    head
}

### THE PARTIAL MODEL

`ordPartial` <-
    function(Y, Z)
{
    ZERO <- sqrt(.Machine$double.eps)
    ## attributes
    DISTBASED <- attr(Y, "METHOD") == "DISTBASED"
    RW <- attr(Y, "RW")
    ## centre Z
    if (!is.null(RW)) {
        envcentre <- apply(Z, 2, weighted.mean, w = RW)
        Z <- scale(Z, center = envcentre, scale = FALSE)
        Z <- sweep(Z, 1, sqrt(RW), "*")
    } else {
        envcentre <- colMeans(Z)
        Z <- scale(Z, center = envcentre, scale = FALSE)
    }
    ## QR decomposition
    Q <- qr(Z)
    ## partialled out variation as a trace of Yfit
    Yfit <- qr.fitted(Q, Y)
    if (DISTBASED) {
        Yfit <- qr.fitted(Q, t(Yfit))
        totvar <- sum(diag(Yfit))
    } else {
        totvar <- sum(Yfit^2)
    }
    if (totvar < ZERO)
        totvar <- 0
    ## residuals of Y
    Y <- qr.resid(Q, Y)
    if (DISTBASED)
        Y <- qr.resid(Q, t(Y))
    ## result object like in current cca, rda
    result <- list(
        rank = if (totvar > 0) Q$rank else 0,
        tot.chi = totvar,
        QR = Q,
        Fit = Yfit,
        envcentre = envcentre)
    list(Y = Y, result = result)
}

### THE CONSTRAINTS

`ordConstrain` <- function(Y, X, Z)
{
    ## attributes & constants
    DISTBASED <- attr(Y, "METHOD") == "DISTBASED"
    RW <- attr(Y, "RW")
    CW <- attr(Y, "CW")
    ZERO <- sqrt(.Machine$double.eps)
    ## combine conditions and constraints if necessary
    if (!is.null(Z)) {
        X <- cbind(Z, X)
        zcol <- ncol(Z)
    } else {
        zcol <- 0
    }
    ## centre
    if (!is.null(RW)) {
        envcentre <- apply(X, 2, weighted.mean, w = RW)
        X <- scale(X, center = envcentre, scale = FALSE)
        X <- sweep(X, 1, sqrt(RW), "*")
    } else {
        envcentre <- colMeans(X)
        X <- scale(X, center = envcentre, scale = FALSE)
    }
    ## QR
    Q <- qr(X)
    ## we need to see how much rank grows over rank of conditions
    rank <- sum(Q$pivot[seq_len(Q$rank)] > zcol)
    ## check for aliased terms
    if (length(Q$pivot) > Q$rank)
        alias <- colnames(Q$qr)[-seq_len(Q$rank)]
    else
        alias <- NULL
    ## kept constraints and their means
    kept <- seq_along(Q$pivot) <= Q$rank & Q$pivot > zcol
    if (zcol > 0)
        envcentre <- envcentre[-seq_len(zcol)]
    ## eigen solution
    Yfit <- qr.fitted(Q, Y)
    if (DISTBASED) {
        Yfit <- qr.fitted(Q, t(Yfit))
        sol <- eigen(Yfit, symmetric = TRUE)
        lambda <- sol$values
        u <- sol$vectors
    } else {
        sol <- svd(Yfit)
        lambda <- sol$d^2
        u <- sol$u
        v <- sol$v
    }
    ## handle zero  eigenvalues and negative eigenvalues
    zeroev <- abs(lambda) < max(ZERO, ZERO * lambda[1L])
    if (any(zeroev)) {
        lambda <- lambda[!zeroev]
        u <- u[, !zeroev, drop = FALSE]
        if (!DISTBASED)
            v <- v[, !zeroev, drop = FALSE]
    }
    posev <- lambda > 0
    ## wa scores
    if (DISTBASED) {
        wa <- Y %*% u[, posev, drop = FALSE] %*%
            diag(1/lambda[posev], sum(posev))
        v <- matrix(NA, 0, sum(posev))
    } else {
        wa <- Y %*% v %*% diag(1/sqrt(lambda), sum(posev))
    }
    ## biplot scores: basically these are cor(X, u), but cor() would
    ## re-centre X and u in CCA, and therefore we need the following
    ## (which also is faster)
    xx <- X[, Q$pivot[kept], drop = FALSE]
    bp <- (1/sqrt(colSums(xx^2))) * crossprod(xx, u[, posev, drop=FALSE])
    ## de-weight
    if (!is.null(RW)) {
        u <- sweep(u, 1, sqrt(RW), "/")
        if (all(!is.na(wa)))
            wa <- sweep(wa, 1, sqrt(RW), "/")
    }
    if (!is.null(CW) && nrow(v)) {
        v <- sweep(v, 1, sqrt(CW), "/")
    }
    ## set names
    axnam <- paste0(switch(attr(Y, "METHOD"),
                           "PCA" = "RDA",
                           "CA" = "CCA",
                           "CAPSCALE" = "CAP",
                           "DISTBASED" = "dbRDA"),
                    seq_len(sum(posev)))
    if (DISTBASED && any(!posev))
        negnam <- paste0("idbRDA", seq_len(sum(!posev)))
    else
        negnam <- NULL
    dnam <- dimnames(Y)
    if (any(posev))
        names(lambda) <- c(axnam, negnam)
    if (ncol(u))
        dimnames(u) <- list(dnam[[1]], c(axnam, negnam))
    if (nrow(v) && ncol(v))     # no rows in DISTBASED
        dimnames(v) <- list(dnam[[2]], axnam)
    if (ncol(wa)) # only for posev
        colnames(wa) <- axnam
    if (ncol(bp))    # only for posev
        colnames(bp) <- axnam
    ## out
    result <- list(
        eig = lambda,
        poseig = if (DISTBASED) sum(posev) else NULL,
        u = u,
        v = v,
        wa = wa,
        alias = alias,
        biplot = bp,
        rank = length(lambda),
        qrank = rank,
        tot.chi = sum(lambda),
        QR = Q,
        envcentre = envcentre,
        Xbar = Y)
    ## residual of Y
    Y <- qr.resid(Q, Y)
    if (DISTBASED)
        Y <- qr.resid(Q, t(Y))
    ## out
    list(Y = Y, result = result)
}

### THE RESIDUAL METHOD

### Finds the unconstrained ordination after (optionally) removing the
### variation that could be explained by partial and constrained
### models.

`ordResid` <-
    function(Y)
{
    ## get attributes
    DISTBASED <- attr(Y, "METHOD") == "DISTBASED"
    RW <- attr(Y, "RW")
    CW <- attr(Y, "CW")
    ## Ordination
    ZERO <- sqrt(.Machine$double.eps)
    if (DISTBASED) {
        sol <- eigen(Y, symmetric = TRUE)
        lambda <- sol$values
        u <- sol$vectors
    } else {
        sol <- svd(Y)
        lambda <- sol$d^2
        u <- sol$u
        v <- sol$v
    }
    ## handle zero and negative eigenvalues
    zeroev <- abs(lambda) < max(ZERO, ZERO * lambda[1L])
    if (any(zeroev)) {
        lambda <- lambda[!zeroev]
        u <- u[, !zeroev, drop = FALSE]
        if (!DISTBASED) # no v in DISTBASED
            v <- v[, !zeroev, drop = FALSE]
    }
    posev <- lambda > 0
    if (DISTBASED) # no species scores in DISTBASED
        v <- matrix(NA, 0, sum(posev))
    ## de-weight
    if (!is.null(RW)) {
        u <- sweep(u, 1, sqrt(RW), "/")
    }
    if (!is.null(CW) && nrow(v)) {
        v <- sweep(v, 1, sqrt(CW), "/")
    }
    ## set names
    axnam <- paste0(switch(attr(Y, "METHOD"),
                           "PCA" = "PC",
                           "CA" = "CA",
                           "CAPSCALE" = "MDS",
                           "DISTBASED" = "MDS"),
                    seq_len(sum(posev)))
    if (DISTBASED && any(!posev))
        negnam <- paste0("iMDS", seq_len(sum(!posev)))
    else
        negnam <- NULL
    dnam <- dimnames(Y)
    if (any(posev))
        names(lambda) <- c(axnam, negnam)
    if (ncol(u))
        dimnames(u) <- list(dnam[[1]], c(axnam, negnam))
    if (nrow(v) && ncol(v)) # no rows in DISTBASED
        dimnames(v) <- list(dnam[[2]], axnam)
    ## out
    out <- list(
        "eig" = lambda,
        "poseig" = if (DISTBASED) sum(posev) else NULL,
        "u" = u,
        "v" = v,
        "rank" = length(lambda),
        "tot.chi" = sum(lambda),
        "Xbar" = Y)
    out
}

## The actual function that calls all previous and returns the fitted
## ordination model

`ordConstrained` <-
    function(Y, X = NULL, Z = NULL,
             method = c("cca", "rda", "capscale", "dbrda", "pass"),
             arg = FALSE)
{
    method = match.arg(method)
    partial <- constraint <- resid <- NULL
    ## init; "pass" returns unchanged Y, presumably from previous init
    Y <- switch(method,
                "cca" = initCA(Y),
                "rda" = initPCA(Y, scale = arg),
                "capscale" = initCAP(Y),
                "dbrda" = initDBRDA(Y),
                "pass" = Y)
    ## header info for the model
    head <- ordHead(Y)
    ## Partial
    if (!is.null(Z) && ncol(Z)) {
        out <- ordPartial(Y, Z)
        Y <- out$Y
        partial <- out$result
    }
    ## Constraints
    if (!is.null(X) && ncol(X)) {
        out <- ordConstrain(Y, X, Z)
        Y <- out$Y
        constraint <- out$result
    }
    ## Residuals
    resid <- ordResid(Y)
    ## return a CCA object
    out <- c(head,
             call = match.call(),
             list("pCCA" = partial, "CCA" = constraint, "CA" = resid))
    class(out) <- switch(attr(Y, "METHOD"),
                         "CA" = "cca",
                         "PCA" = c("rda", "cca"),
                         "CAPSCALE" = c("capscale", "rda", "cca"),
                         "DISTBASED" = c("dbrda", "rda", "cca"))
    out
}

`scores.decorana` <-
    function (x, display="sites", choices = 1:4, origin=TRUE,
              tidy = FALSE, ...)
{
    display <- match.arg(display, c("sites", "species", "both"), several.ok = TRUE)
    ## return "both" in tidy scores
    if(tidy)
        display <- "both"
    out <- list()
    if(any(c("sites", "both") %in% display)) {
        sites <- x$rproj
        if (origin)
            sites <- sweep(sites, 2, x$origin, "-")
        out$sites <- sites[, choices, drop=FALSE]
    }
    if(any(c("species", "both") %in% display)) {
        species <- x$cproj
        if (origin)
            species <- sweep(species, 2, x$origin, "-")
        out$species <- species[, choices]
    }
    if (tidy) {
        if (length(out) == 0) # no scores (never TRUE?)
            return(NULL)
        group <- sapply(out, nrow)
        group <- rep(names(group), group)
        out <- do.call(rbind, out)
        label <- rownames(out)
        out <- as.data.frame(out)
        out$score <- group
        out$label <- label
        wts <- rep(NA, nrow(out))
        if (any(take <- group == "sites"))
            wts[take] <- weights(x, display="sites")
        if (any(take <- group == "species"))
            wts[take] <- weights(x, display="species")
        out$weight <- wts
    }
    ## two kind of scores: return NULL, matrix or a list
    if (length(out) == 1)
        out[[1]]
    else
        out
}

`summary.anosim` <-
    function (object, ...)
{
   print(object)
   if (object$permutations) {
     out <- quantile(object$perm, c(0.9, 0.95, 0.975, 0.99))
     cat("Upper quantiles of permutations (null model):\n")
     print(out, digits=3)
   }
   cat("\n")
   tmp <- tapply(object$dis.rank, object$class.vec, quantile)
   out <- matrix(NA, length(tmp), 5)
   for (i in seq_along(tmp)) {
       if (!is.null(tmp[[i]])) out[i,] <- tmp[[i]]
   }
   rownames(out) <- names(tmp)
   colnames(out) <- names(tmp$Between)
   out <- cbind(out, N = table(object$class.vec))
   cat("Dissimilarity ranks between and within classes:\n")
   print(out)
   cat("\n")
   invisible()
}

`plot.mantel.correlog` <-
    function(x, alpha=0.05, ...)
{
    lim <- max(x$n.tests)
    plot(x$mantel.res[1:lim,1],x$mantel.res[1:lim,3],
         xlab="Distance class index", ylab="Mantel correlation", pch=22)
    if(x$mult == "none") {
	signif <- which((x$mantel.res[1:lim,4] <= alpha))
    } else {
	signif <- which((x$mantel.res[1:lim,5] <= alpha))
    }
    lines(x$mantel.res[1:lim,1], x$mantel.res[1:lim,3])
    points(x$mantel.res[1:lim,1], x$mantel.res[1:lim,3], pch=22, bg="white")
    points(x$mantel.res[signif,1], x$mantel.res[signif,3], pch=22, bg="black")
    abline(a=0, b=0, col="red")
    invisible()
}

### predict.rda handles rda plus distance-based capscale and
### dbrda. Distance-based methods have some limitations:
###
### - type = "response" returns dissimilarities (and ignores imaginary dims)
### - Euclidean distances type = "working" give type = "response"
### - there are no meaningful species scores
### - WA scores with newdata cannot be calculated in capscale.
### - only type = "response", "working" and "lc" work with dbrda
### - only type = "lc" can be used with newdata with dbrda

`predict.rda` <-
    function (object, newdata, type = c("response", "wa", "sp", "lc", "working"),
              rank = "full", model = c("CCA", "CA"), scaling = "none",
              correlation = FALSE, const, ...)
{
    ## not vegan rda, but intended for klaR:::predict.rda?
    if (!("CA" %in% names(object)))
        stop(gettextf("%s is not a vegan rda object",
                      sQuote(deparse(substitute(object)))))
    type <- match.arg(type)
    model <- match.arg(model)
    if (model == "CCA" && is.null(object$CCA))
        model <- "CA"
    take <- object[[model]]$rank
    if (take == 0)
        stop(gettextf("model '%s' has rank 0", model))
    if (rank != "full")
        take <- min(take, rank)

    if (is.null(object$Ybar))
        stop("update() your outdated result object")
    cent <- attr(object$Ybar, "scaled:center")
    scal <- attr(object$Ybar, "scaled:scale")
    scaled.PCA <- !is.null(scal)

    nr <- nobs(object) - 1
    u <- object[[model]]$u[, 1:take, drop = FALSE]
    w <- object[[model]]$wa[, 1:take, drop = FALSE]
    if (is.null(w))
        w <- u
    v <- object[[model]]$v[, 1:take, drop = FALSE]

    ## process scaling arg, scaling used later so needs to be a numeric
    scaling <- scalingType(scaling = scaling, correlation = correlation)
    if (type %in% c("wa","sp","lc")) {
        slam <- sqrt(object[[model]]$eig[1:take]/object$tot.chi)
        if (scaling && missing(const))
            const <- sqrt(sqrt((nobs(object)-1) * object$tot.chi))
    } else {
        slam <- diag(sqrt(object[[model]]$eig[1:take]), nrow = take)
    }

    if (type %in% c("response", "working")) {
        if (!missing(newdata)) {
            u <- predict(object, type = if(model == "CCA") "lc" else "wa",
                         newdata = newdata, rank = take)
        }
        if (inherits(object, "capscale")) {
            if (take > 0) {
                out <- u %*% slam
                if (type == "response") {
                    out <- dist(out)
                }
            }
        } else {
            if (take > 0)
                out <- u %*% slam %*% t(v)
            else {
                out <- matrix(0, nrow = nrow(u), ncol = nrow(v))
                rownames(out) <- rownames(u)
                colnames(out) <- rownames(v)
            }
            if (type == "response") {
                if (!is.null(scal))
                    out <- sweep(out, 2, scal, "*")
                out <- out * sqrt(nr)
                out <- sweep(out, 2, cent, "+")
            }
        }
    }
    else if (type == "lc") {
        if (model == "CA")
            stop("'lc' scores not available for unconstrained ordination")
        if (!missing(newdata)) {
            if (is.null(object$terminfo))
                E <- as.matrix(newdata)
            else {
                d <- ordiParseFormula(formula(object), newdata,
                                      object$terminfo$xlev)
                E <- cbind(d$Z, d$Y)
            }
            Q <- object[[model]]$QR
            p1 <- Q$pivot[1:Q$rank]
            E <- sweep(E, 2, c(object$pCCA$envcentre, object$CCA$envcentre),
                       "-")
            u <- E[, p1, drop = FALSE] %*% coef(object)[p1, ,
                         drop = FALSE]
            u <- u[, 1:take, drop = FALSE]
        }
        out <- u
        if (scaling) {   # implicit coercion 0 == FALSE, other == TRUE
            lam <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
            out <- const * sweep(out, 2, lam, "*")
        }
    }
    else if (type == "wa") {
        if (!missing(newdata)) {
            if (inherits(object, "capscale"))
                stop(gettextf("'wa' scores not available in %s with 'newdata'",
                     object$method))
            if (!is.null(object$pCCA))
                stop("no 'wa' scores available (yet) in partial RDA")
            nm <- rownames(v)
            if (!is.null(nm)) {
                if (!all(nm %in% colnames(newdata)))
                    stop("'newdata' does not have named columns matching one or more the original columns")
                newdata <-  newdata[, nm, drop = FALSE]
            }
            Xbar <- as.matrix(newdata)
            Xbar <- sweep(Xbar, 2, cent, "-")
            Xbar <- Xbar / sqrt(nr)
            if (!is.null(scal)) {
                nz <- scal > 0
                Xbar[,nz] <- sweep(Xbar[,nz], 2, scal[nz], "/")
            }
            w <- Xbar %*% v
            w <- sweep(w, 2, slam, "/") / sqrt(object$tot.chi)
        }
        out <- w
        if (scaling) {   # implicit coercion 0 == FALSE, other == TRUE
            lam <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
            out <- const * sweep(out, 2, lam, "*")
        }
    }
    else if (type == "sp") {
        if (inherits(object, "capscale"))
            warning("'sp' scores may be meaningless in 'capscale'")
        if (!missing(newdata)) {
            nm <- rownames(u)
            if (!is.null(nm)) {
                if (!all(nm %in% rownames(newdata)))
                    stop("'newdata' does not have named rows matching one or more of the original rows")
                newdata <- newdata[nm, , drop = FALSE]
            }
            Xbar <- as.matrix(newdata)
            Xbar <- scale(Xbar, center = TRUE, scale = scaled.PCA)
            Xbar <- Xbar/sqrt(nr)
            if (!is.null(object$pCCA))
                Xbar <- qr.resid(object$pCCA$QR, Xbar)
            v <- t(Xbar) %*% u
            v <- sweep(v, 2, slam, "/") / sqrt(object$tot.chi)
        }
        out <- v
        if (scaling) {   # implicit coercion 0 == FALSE, other == TRUE
            lam <- list(1, slam, sqrt(slam))[[abs(scaling)]]
            out <- const * sweep(out, 2, lam, "*")
            if (scaling < 0) { # correlation=TRUE
                out <- out / object$colsum
                out <- out * sqrt(object$tot.chi / (nobs(object)-1))
            }
        }
    }
    out
}

### distance-based RDA benefits from its own predict

`predict.dbrda` <-
    function(object, newdata, type = c("response", "lc", "wa", "working"),
             rank = "full", model = c("CCA", "CA"), scaling = "none",
             const, ...)
{
    type <- match.arg(type)
    model <- match.arg(model)
    ZAP <- sqrt(.Machine$double.eps)
    if (type %in% c("response", "working")) {
        if(rank == "full" && missing(newdata)) {
            out <- ordiYbar(object, model)
        } else {
            lambda <- object[[model]]$eig
            if (!missing(newdata)) {
                u <- predict(object, newdata = newdata, type = "lc",
                             rank = rank)
            } else {
                u <- object[[model]]$u
                if (!is.null(object[[model]]$imaginary.u))
                    u <- cbind(u, object[[model]]$imaginary.u)
            }
            if (rank != "full") {
                k <- seq_len(min(rank, ncol(u)))
                u <- u[, k, drop = FALSE]
                lambda <- lambda[k]
            }
            out <- u %*% diag(lambda, nrow=length(lambda)) %*% t(u)
        }
        if (type == "response") {
            dia <- diag(out)
            out <- as.dist(-2*out + outer(dia, dia, "+"))
            out[abs(out) < ZAP] <- 0
            out <- sqrt(out)
        }
    } else if (type %in% c("lc", "wa")) {
        if (model == "CA" && type == "lc")
            stop("'lc' scores are not available for unconstrained ordination")
        if (type == "wa" && !missing(newdata))
            stop("'newdata' is not available for type='wa'")
        if (!missing(newdata)) {
            if (is.null(object$terminfo))
                E <- as.matrix(newdata)
            else {
                d <- ordiParseFormula(formula(object), newdata,
                                      object$terminfo$xlev)
                E <- cbind(d$Z, d$Y)
            }
            E <- sweep(E, 2,
                       c(object$pCCA$envcentre, object$CCA$envcentre), "-")
            p1 <- object[[model]]$QR$pivot[seq_len(object[[model]]$QR$rank)]
            out <- E[, p1, drop = FALSE] %*% coef(object)[p1, , drop =FALSE]
        } else {
            out <- object[[model]]$u
        }
        if (rank != "full") {
            k <- seq_len(min(rank, ncol(out)))
            out <- out[, k, drop=FALSE]
        } else {
            k <- seq_len(ncol(out))
        }
        ## wa can be estimated as ordiYbar(object, "initial") %*% u
        ## %*% diag(1/lambda), but "initial" ordiYbar is not available
        ## and cannot be found from constrained u with predict(...,
        ## type="response", newdata=...) which gives ordYbar(...,
        ## "CCA").
        if (type == "wa" && model == "CCA") { # in model="CA" out has these
            out <- object[[model]]$wa[, k, drop = FALSE]
        }
        scaling <- scalingType(scaling)
        if (scaling) { # scaling="none" is 0 == FALSE
            if (missing(const))
                const <- sqrt(sqrt((nobs(object) - 1) * object$tot.chi))
            slam <- diag(sqrt(object[[model]]$eig[k] / object$tot.chi),
                         nrow = length(k))
            out <- switch(scaling,
                          const * out %*% slam,
                          const * out,
                          const * out %*% sqrt(slam)
                          )
        }
    }
    out
}

"print.summary.prc" <-
function(x, ...)
{
    cat("\nCall:\n")
    cat(deparse(x$call), "\n")
    cat("Species scores:\n")
    print(x$sp, digits=x$digits, ...)
    cat("\nCoefficients for",
        paste(x$names[2], "+", paste(x$names, collapse=":")),
        "interaction\n")
    cat(paste("which are contrasts to", x$names[2], x$corner, "\n"))
    cat(paste(c("rows are",", columns are"), x$names[2:1], collapse=""))
    cat("\n")
    print(coef(x), digits = x$digits, ...)
    invisible(x)
}

`ordiellipse` <-
    function (ord, groups, display = "sites", kind = c("sd", "se", "ehull"),
              conf, draw = c("lines", "polygon", "none"),
              w = weights(ord, display), col = NULL, alpha = 127,
              show.groups, label = FALSE, border = NULL, lty = NULL,
              lwd = NULL, ...)
{
    weights.default <- function(object, ...) NULL
    kind <- match.arg(kind)
    draw <- match.arg(draw)
    pts <- scores(ord, display = display, ...)
    ## ordiellipse only works with 2D data (2 columns)
    pts <- as.matrix(pts)
    if (ncol(pts) > 2)
        pts <- pts[ , 1:2, drop = FALSE]
    if (ncol(pts) < 2)
        stop("needs two dimensions")
    w <- eval(w)
    if (length(w) == 1)
        w <- rep(1, nrow(pts))
    if (is.null(w))
        w <- rep(1, nrow(pts))
    ## make semitransparent fill; alpha should be integer in 0..255,
    ## but users may have given that as real in 0..1
    if (alpha < 1)
        alpha <- round(alpha * 255)
    if (draw == "polygon" && !is.null(col))
        col <- rgb(t(col2rgb(col)), alpha = alpha, maxColorValue = 255)
    if (!missing(show.groups)) {
        take <- groups %in% show.groups
        pts <- pts[take, , drop = FALSE]
        groups <- groups[take]
        w <- w[take]
    }
    out <- seq(along = groups)
    inds <- names(table(groups))

    ## fill in graphical vectors with default values if unspecified
    ## and recycles shorter vectors
    col.new <- border.new <- lty.new <- lwd.new <- NULL
    for(arg in c("col","border","lty","lwd")){
      tmp <- mget(arg,ifnotfound=list(NULL))[[1]]
      if(is.null(tmp))
          tmp <- ifelse(suppressWarnings(is.null(par(arg))),
                        par("fg"), par(arg))
      if(length(inds) != length(tmp)) {tmp <- rep_len(tmp, length(inds))}
      assign(paste(arg,".new", sep=""), tmp)

    }
    ## default colour for "polygon" fill is "transparent", for lines
    ## is par("fg")
    if(is.null(col) && draw=="polygon")
        col.new <- rep_len("transparent", length(inds))
    else if(is.null(col) && draw=="lines")
            col.new <- rep_len(par("fg"), length(inds))

    res <- list()
    if (label) {
        cntrs <- matrix(NA, nrow=length(inds), ncol=2)
        rownames(cntrs) <- inds
    }
    ## Remove NA scores
    kk <- complete.cases(pts) & !is.na(groups)
    for (is in inds) {
        gr <- out[groups == is & kk]
        if (length(gr)) {
            X <- pts[gr, , drop = FALSE]
            W <- w[gr]
            if (kind == "ehull") {
                tmp <- ellipsoidhull(X)
                mat <- list(cov = tmp$cov,
                            center = tmp$loc,
                            n.obs = nrow(X))
            } else
                mat <- cov.wt(X, W)
            if (mat$n.obs == 1)
                mat$cov[] <- 0
            if (kind == "se")
                mat$cov <- mat$cov * sum(mat$wt^2)
            if (kind == "ehull")
                t <- if (is.nan(tmp$d2)) 0 else sqrt(tmp$d2)
            else {
                if (missing(conf))
                    t <- 1
                else t <- sqrt(qchisq(conf, 2))
            }
            if (mat$n.obs > 1)
                xy <- veganCovEllipse(mat$cov, mat$center, t)
            else
                xy <- X
            if (draw == "lines")
                ordiArgAbsorber(xy, FUN = lines,
                                col = if (is.null(col))
                                          par("fg")
                                      else
                                          col.new[match(is, inds)],
                                lty=lty.new[match(is,inds)],
                                lwd=lwd.new[match(is,inds)], ...)

            else if (draw == "polygon")
                ordiArgAbsorber(xy[, 1], xy[, 2],
                                col = col.new[match(is, inds)],
                                border=border.new[match(is,inds)],
                                lty = lty.new[match(is,inds)],
                                lwd = lwd.new[match(is,inds)],
                                FUN = polygon,
                                ...)
            if (label && draw != "none") {
                cntrs[is,] <- mat$center
            }
            mat$scale <- t
            res[[is]] <- mat
        }
    }
    if (label && draw != "none") {
        if (draw == "lines")
            ordiArgAbsorber(cntrs[,1], cntrs[,2],
                            labels = rownames(cntrs),
                            col = col.new,  FUN = text, ...)
        else
            ordiArgAbsorber(cntrs, col = NULL,
                            FUN = ordilabel, ...)
    }
    class(res) <- "ordiellipse"
    invisible(res)
}

`scores.betadisper` <-
    function(x, display = c("sites", "centroids"),
             choices = c(1,2), ...)
{
    display <- match.arg(display, several.ok = TRUE)
    sol <- list()
    if("sites" %in% display)
        sol$sites <- x$vectors[, choices]
    if("centroids" %in% display) {
        if(is.matrix(x$centroids))
            sol$centroids <- x$centroids[, choices, drop = FALSE]
        else
            sol$centroids <- matrix(x$centroids[choices], ncol = length(choices), byrow = TRUE)
    }
    if (length(sol) == 1)
        sol <- sol[[1]]
    sol
}

"plot.varpart234" <-
function(x, cutoff = 0, digits = 1,  ...)
{
    vals <- x$indfract[, 3]
    is.na(vals) <- vals < cutoff
    if (cutoff >= 0)
        vals <- round(vals, digits+1)
    labs <-  format(vals, digits=digits, nsmall=digits+1)
    labs <- gsub("NA", "", labs)
    showvarparts(x$nsets, labs, ...)
    if (any(is.na(vals)))
        mtext(paste("Values <", cutoff," not shown", sep=""), 1)
    invisible()
}

"text.metaMDS" <-
    function (x, display = c("sites", "species"), labels,
              choices = c(1, 2), shrink = FALSE, select, ...)
{
    display <- match.arg(display)
    x <- scores(x, display = display, choices = choices, shrink = shrink)
    if (!missing(labels))
        rownames(x) <- labels
    if (!missing(select))
        x <- .checkSelect(select, x)
    text(x, labels = rownames(x), ...)
    invisible()
}

`points.procrustes` <-
    function(x, display = c("target","rotated"), choices = c(1,2),
             truemean = FALSE, ...)
{
    display <- match.arg(display)
    X <- if (display == "target") x$X else x$Yrot
    X <- X[, choices, drop = FALSE]
    if (truemean)
        X <- sweep(X, 2, x$xmean[choices], "+")
    ordiArgAbsorber(X, FUN = points, ...)
    invisible()
}

`text.procrustes` <-
    function(x, display = c("target","rotated"), choices = c(1,2),
             labels, truemean = FALSE, ...)
{
    display <- match.arg(display)
    X <- if (display == "target") x$X else x$Yrot
    X <- X[, choices, drop = FALSE]
    if (truemean)
        X <- sweep(X, 2, x$xmean[choices], "+")
    if (missing(labels))
        labels <- rownames(X)
    ordiArgAbsorber(X, labels = labels, FUN = text, ...)
    invisible()
}

##############################################################
##  COMPUTES BEALS SMOOTHING FOR ALL SPECIES IN TABLE        #
##  This is a more complete function than the previous one   #
##  in the vegan package. The parameter values that give the #
##  equivalence are 'beals(x, NA, x, 0, include=TRUE)'       #
##                                                           #
##  'x'     matrix to be replaced by beals values            #
##  'reference' matrix to be used as source for joint occurrences#
##  'type'  sets the way to use abundance values             #
##       0 - presence/absence                                #
##       1 - abundances for conditioned probabilities        #
##       2 - abundances for weighted average                 #
##       3 - abundances for both                             #
##  'species' a column index used to compute Beals function  #
##        for a single species. The default (NA) indicates   #
##        all species.                                       #
##  'include' flag to include target species in the computation#
##############################################################
`beals` <-
    function(x, species=NA, reference=x, type=0, include=TRUE)
{
    refX <- reference
    ## this checks whether it was chosen from available options
    mode <- as.numeric(match.arg(as.character(type), c("0","1","2","3")))
    spIndex <- species
    incSp <- include
    refX <- as.matrix(refX)
    x <- as.matrix(x)
    if (!(is.numeric(x) || is.logical(x)))
        stop("input data must be numeric")
    if(mode==0 || mode ==2) refX <- ifelse(refX > 0, 1, 0)
    if(mode==0 || mode ==1) x <- ifelse(x > 0, 1, 0)
    ##Computes conditioned probabilities
    if(is.na(spIndex)){
        M <- crossprod(ifelse(refX > 0, 1, 0),refX)
        C <-diag(M)
        M <- sweep(M, 2, replace(C,C==0,1), "/")
        if(!incSp)
            for (i in 1:ncol(refX))
                M[i,i] <- 0
    } else {
        C <- colSums(refX)
        M <- crossprod(refX,ifelse(refX > 0, 1, 0)[,spIndex])
        M <- M/replace(C,C==0,1)
        if(!incSp)
            M[spIndex] <- 0
    }
    ##Average of conditioned probabilities
    S <- rowSums(x)
    if(is.na(spIndex)) {
        b <-x
        for (i in 1:nrow(x)) {
            b[i, ] <- rowSums(sweep(M, 2, x[i, ], "*"))
        }
        SM <- rep(S,ncol(x))
        if(!incSp)
            SM <- SM-x
        b <- b/replace(SM,SM==0,1)
    } else {
        b <-rowSums(sweep(x,2,M,"*"))
        if(!incSp)
            S <- S-x[,spIndex]
        b <- b/replace(S,S==0,1)
    }
    b
}

.onAttach <- function(lib, pkg)  {
    if(interactive())
        packageStartupMessage(
            "This is vegan ",
            utils::packageDescription("vegan", fields="Version"),
            appendLF = TRUE)
}

#' The Slope of Species Accumulation Curve at Given Point
#'
#' Function evaluates the derivative of the species accumulation curve
#' for accumulation methods built upon analytic accumulation
#' methods. These methods are \code{exact}, \code{rarefaction} and
#' \code{coleman}. These methods can be evaluated at any sample size,
#' including non-integer values. For other methods, you must look at
#' the differences between consecutive steps, using
#' \code{diff(predict(mod))}.
#'
#' @param object \code{specaccum} result object fitted with methods
#' \code{"exact"}, \code{"rarefaction"} or \code{"coleman"}.
#' @param at The sample size (number of sites) at which the slope is
#' evaluated. This need not be an integer.

`specslope` <-
    function(object, at)
{
    UseMethod("specslope")
}

`specslope.specaccum` <-
    function(object, at)
{
    accepted <- c("exact", "rarefaction", "coleman")
    if (!(object$method %in% accepted))
        stop(gettextf("accumulation method must be one of: %s",
             paste(accepted, collapse=", ")))
    ## Funcions should accept a vector of 'at', but usually they
    ## don't. I don't care to change this, and therefore we check the
    ## input.
    if (length(at) > 1 && object$method %in% c("exact", "coleman"))
        stop("'at' can only have a single value")
    ## The following functions are completely defined by species
    ## frequencies
    f <- object$freq
    n <- length(object$sites)
    switch(object$method,
           exact = {
               d <- digamma(pmax(n-at+1, 1)) - digamma(pmax(n-f-at+1, 1))
               g <- lgamma(pmax(n-f+1,1)) + lgamma(pmax(n-at+1,1)) -
                   lgamma(pmax(n-f-at+1, 1)) - lgamma(n+1)
               d <- d*exp(g)
               sum(d[is.finite(d)])
           },
           rarefaction = {
               ## fractional number of individuals at 'at', and slope
               ## for adding whole site instead of one individual
               rareslope(f, at/n*sum(f)) * sum(f)/n
           },
           coleman = {
               sum((1 - at/n)^f*f/(n - at))
           })
}

## Analytical derivatives for NLS regression models in fitspecaccum

`specslope.fitspecaccum` <-
    function(object, at)
{
    ## functions for single set of fitted parameters. Parameters are
    ## given as a single vector 'p' as returned by coef(). Below a
    ## table of original names of 'p':

    ## arrhenius, gitay, gleason: k slope
    ## lomolino: Asym xmid slope
    ## asymp: Asym RO lrc
    ## gompertz: Asym b2 b3
    ## michaelis-menten: Vm K (function SSmicmen)
    ## logis: Asym xmid scal
    ## weibull: Asym Drop lrc pwr
    slope <-
        switch(object$SSmodel,
               "arrhenius" = function(x,p) p[1]*x^(p[2]-1)*p[2],
               "gitay" = function(x,p) 2*(p[1]+p[2]*log(x))*p[2]/x,
               "gleason" = function(x,p) p[2]/x,
               "lomolino" = function(x,p) p[1]*p[3]^log(p[2]/x)*log(p[3])/
                   (1+p[3]^log(p[2]/x))^2/x,
               "asymp" = function(x,p) (p[1]-p[2])*exp(p[3]-exp(p[3])*x),
               "gompertz" = function(x,p) -p[1]*p[2]*p[3]^x*
                   log(p[3])*exp(-p[2]*p[3]^x),
               "michaelis-menten" = function(x,p) p[1]*p[2]/(p[2]+x)^2,
               "logis" = function(x,p) p[1]*exp((x-p[2])/p[3])/
                   (1 + exp((x-p[2])/p[3]))^2/p[3],
               "weibull" = function(x, p) p[2]*exp(p[3]-exp(p[3])*x^p[4])*
               x^(p[4]-1)*p[4])
    ## Apply slope with fitted coefficients at 'at'
    p <- coef(object)
    if (is.matrix(p))          # several fitted models
        out <- apply(p, 2, function(i) slope(at, i))
    else                       # single site drops to a vector
        out <- slope(at, p)
    names(out) <- NULL
    out
}

permutest <- function(x, ...)
    UseMethod("permutest")

permutest.default <- function(x, ...)
    stop("no default permutation test defined")

`permutest.cca` <-
    function (x, permutations = how(nperm=99),
              model = c("reduced", "direct", "full"), by = NULL, first = FALSE,
              strata = NULL, parallel = getOption("mc.cores") ,  ...)
{
    ## do something sensible with insensible input (no constraints)
    if (is.null(x$CCA)) {
        sol <- list(call = match.call(), testcall = x$call, model = NA,
                    F.0 = NA, F.perm = NA, chi = c(0, x$CA$tot.chi),
                    num = 0, den = x$CA$tot.chi,
                    df = c(0, nrow(x$CA$u) - max(x$pCCA$QR$rank,0) - 1),
                    nperm = 0, method = x$method, first = FALSE,
                    Random.seed = NA)
        class(sol) <- "permutest.cca"
        return(sol)
    }
    ## compatible arguments?
    if (!is.null(by)) {
        if (first)
            stop("'by' cannot be used with option 'first=TRUE'")
        by <- match.arg(by, c("onedf", "terms"))
        if (by == "terms" && is.null(x$terminfo))
            stop("by='terms' needs a model fitted with a formula")
    }
    model <- match.arg(model)
    ## special cases
    isCCA <- !inherits(x, "rda")    # weighting
    isPartial <- !is.null(x$pCCA)   # handle conditions
    isDB <- inherits(x, c("dbrda")) # only dbrda is distance-based
    ## C function to get the statististics in one loop
    getF <- function(indx, ...)
    {
        if (!is.matrix(indx))
            indx <- matrix(indx, nrow=1)
        out <- .Call(do_getF, indx, E, Q, QZ, effects, first, isPartial, isDB)
        p <- length(effects)
        if (!isPartial && !first)
            out[,p+1] <- Chi.tot - rowSums(out[,seq_len(p), drop=FALSE])
        if (p > 1) {
            if (by == "terms")
                out[, seq_len(p)] <- sweep(out[, seq_len(p), drop = FALSE],
                                               2, q, "/")
            out <- cbind(out, sweep(out[,seq_len(p), drop=FALSE], 1,
                                    out[,p+1]/r, "/"))
        }
        else
            out <- cbind(out, (out[,1]/q)/(out[,2]/r))
        out
    }
    ## end getF
    ## QR decomposition
        Q <- x$CCA$QR
    if (isPartial) {
        QZ <- x$pCCA$QR
    } else {
        QZ <- NULL
    }
    ## statistics: overall tests
    if (first) {
        Chi.z <- x$CCA$eig[1]
        q <- 1
    }
    else {
        Chi.z <- x$CCA$tot.chi
        names(Chi.z) <- "Model"
        q <- x$CCA$qrank
    }
    ## effects
    if (!is.null(by)) {
        partXbar <- ordiYbar(x, "partial")
        if (by == "onedf") {
            effects <- seq_len(q)
            termlabs <-
                if (isPartial)
                    colnames(Q$qr)[effects + x$pCCA$QR$rank]
                else
                    colnames(Q$qr)[effects]
        } else {                   # by = "terms"
            ass <- x$terminfo$assign
            ## ass was introduced in vegan_2.5-0
            if (is.null(ass))
                stop("update() old ordination result object")
            pivot <- Q$pivot
            if (isPartial)
                pivot <- pivot[pivot > x$pCCA$QR$rank] - x$pCCA$QR$rank
            ass <- ass[pivot[seq_len(x$CCA$qrank)]]
            effects <- cumsum(rle(ass)$length)
            termlabs <- labels(terms(x$terminfo))
            if (isPartial)
                termlabs <- termlabs[termlabs %in% labels(terms(x))]
            termlabs <-termlabs[unique(ass)]
        }
        q <- diff(c(0, effects)) # d.o.f.
        if (isPartial)
            effects <- effects + x$pCCA$QR$rank
        F.0 <- numeric(length(effects))
        for (k in seq_along(effects)) {
            fv <- qr.fitted(Q, partXbar, k = effects[k])
            F.0[k] <- if (isDB) sum(diag(fv)) else sum(fv^2)
        }
    }
    else {
        effects <- 0
        termlabs <- "Model"
    }
    ## Set up
    Chi.xz <- x$CA$tot.chi
    names(Chi.xz) <- "Residual"
    r <- nobs(x) - Q$rank - 1
    if (model == "full")
        Chi.tot <- Chi.xz
    else Chi.tot <- Chi.z + Chi.xz
    if (is.null(by))
        F.0 <- (Chi.z/q)/(Chi.xz/r)
    else {
        Chi.z <- numeric(length(effects))
        for (k in seq_along(effects)) {
            fv <- qr.fitted(Q, partXbar, k = effects[k])
            Chi.z[k] <- if (isDB) sum(diag(fv)) else sum(fv^2)
        }
        Chi.z <- diff(c(0, F.0))
        F.0 <- Chi.z/q * r/Chi.xz
    }

    ## permutation data
    E <- switch(model,
                "direct" = ordiYbar(x, "initial"),
                "reduced" = ordiYbar(x, "partial"),
                "full" = ordiYbar(x, "CA"))
    ## vegan < 2.5-0 cannot use direct model in partial dbRDA
    if (is.null(E) && isDB && isPartial)
        stop("'direct' model cannot be used in old partial-dbrda: update() result")

    ## Save dimensions
    N <- nrow(E)
    permutations <- getPermuteMatrix(permutations, N, strata = strata)
    nperm <- nrow(permutations)
    ## Parallel processing (similar as in oecosimu)
    if (is.null(parallel))
        parallel <- 1
    hasClus <- inherits(parallel, "cluster")
    if (hasClus || parallel > 1) {
        if(.Platform$OS.type == "unix" && !hasClus) {
            tmp <- do.call(rbind,
                           mclapply(1:nperm,
                                    function(i) getF(permutations[i,]),
                                    mc.cores = parallel))
        } else {
            ## if hasClus, do not set up and stop a temporary cluster
            if (!hasClus) {
                parallel <- makeCluster(parallel)
            }
            tmp <- parRapply(parallel, permutations, function(i) getF(i))
            tmp <- matrix(tmp, nrow=nperm, byrow=TRUE)
            if (!hasClus)
                stopCluster(parallel)
        }
    } else {
        tmp <- getF(permutations)
    }
    if ((p <- length(effects)) > 1) {
        num <- tmp[,seq_len(p)]
        den <- tmp[,p+1]
        F.perm <- tmp[, seq_len(p) + p + 1]
    } else {
        num <- tmp[,1]
        den <- tmp[,2]
        F.perm <- tmp[,3, drop=FALSE]
    }
    Call <- match.call()
    Call[[1]] <- as.name("permutest")
    sol <- list(call = Call, testcall = x$call, model = model,
                F.0 = F.0, F.perm = F.perm,  chi = c(Chi.z, Chi.xz),
                num = num, den = den, df = c(q, r), nperm = nperm,
                method = x$method, first = first, termlabels = termlabs)
    sol$Random.seed <- attr(permutations, "seed")
    sol$control <- attr(permutations, "control")
    if (!missing(strata)) {
        sol$strata <- deparse(substitute(strata))
        sol$stratum.values <- strata
    }
    class(sol) <- "permutest.cca"
    sol
}

`summary.cca` <- function (object, scaling = "species", axes = 6,
                           display=c("sp","wa","lc","bp","cn"),
                           digits = max(3, getOption("digits") - 3),
                           correlation = FALSE, hill = FALSE, ...)
{
    axes <- min(axes, sum(object$CCA$rank, object$CA$rank))
    summ <- list()
    ## scaling is stored in return object so must be in numeric format
    scaling <- scalingType(scaling = scaling, correlation = correlation,
                           hill = hill)
    if (axes && length(display) && (all(!is.na(display)) && !is.null(display)))
        summ <- scores(object, scaling = scaling, choices = 1:axes, display = display,
                       ...)
    ## scores() drops list to a matrix if there is only one item: workaround below.
    if (!is.list(summ) && length(display) == 1) {
        nms <- c("species", "sites", "constraints", "biplot", "centroids")
        names(nms) <- c("sp","wa","lc","bp","cn")
        summ <- list(summ)
        names(summ) <- nms[display]
    }
    if (length(display) > 0) {
        for (i in seq_along(summ)) {
            if (is.matrix(summ[[i]]))
                rownames(summ[[i]]) <-
                    rownames(summ[[i]], do.NULL = FALSE,
                             prefix = substr(names(summ)[i], 1, 3))
        }
    }
    summ$call <- object$call
    summ$tot.chi <- object$tot.chi
    ## only the Real component for capscale() with negative eigenvalues
    if (!is.null(object$CA$imaginary.chi))
        summ$tot.chi <- summ$tot.chi - object$CA$imaginary.chi
    summ$partial.chi <- object$pCCA$tot.chi
    summ$constr.chi <- object$CCA$tot.chi
    summ$unconst.chi <- object$CA$tot.chi
    ## nested list cont$importance needed to keep vegan pre-2.5-0 compatibility
    summ$cont$importance <- summary(eigenvals(object))
    if (!is.null(object$CCA) && object$CCA$rank > 0)
        summ$concont$importance <- summary(eigenvals(object, model = "constrained"))
    summ$ev.head <- c(summ$ev.con, summ$ev.uncon)[seq_len(axes)]
    summ$scaling <- scaling
    summ$digits <- digits
    summ$inertia <- object$inertia
    summ$method <- object$method
    class(summ) <- "summary.cca"
    summ
}

### A pair of functions to handle na.action = na.exclude in cca and
### rda (and capscale in the future?). Function ordiNAexclude finds
### the WA scores for NA constraints if possible, and puts these into
### ordination object. Function ordiNApredict pads the result scores
### with NA or scores if available.

`ordiNAexclude` <-
    function(x, excluded)
{
    ## Check that there is a na.action of class "exclude"
    nas <- x$na.action
    if (is.null(nas))
        return(x)
    ## add a 'residuals' item, because step, add1.default and
    ## drop1.default use this to check that number of observations
    ## does not change in sequential fits.
    x$residuals.zombie <- rep(TRUE, max(0, nrow(x$CA$u)))
    ## rowsums for CA (in RDA/PCA rowsum = NA)
    if (!inherits(x, "rda"))
        x$rowsum.excluded <- rowSums(excluded)/x$grand.total
    ## Estimate WA scores for NA cases with newdata of excluded
    ## observations
    if (is.null(x$pCCA) && inherits(nas, "exclude") &&
        !inherits(x, c("dbrda", "capscale"))) {
        if (!is.null(x$CCA))
            x$CCA$wa.excluded <- predict(x, newdata = excluded,
                                         type = "wa", model = "CCA")
        if (!is.null(x$CA))
            x$CA$u.excluded <- predict(x, newdata = excluded,
                                       type = "wa", model = "CA")
    }
    x
}

### Put NA or fitted WA among the scores

`ordiNApredict` <-
    function(omit, x)
{
    ## Only do this if omit is of class "exclude"
    if (!inherits(omit, "exclude"))
        return(x)
    if (!inherits(x, "rda")) {
        x$rowsum <- napredict(omit, x$rowsum)
        if (inherits(omit, "exclude"))
            x$rowsum[omit] <- x$rowsum.excluded
    }
    if (!is.null(x$CCA)) {
        x$CCA$u <- napredict(omit, x$CCA$u)
        x$CCA$wa <- napredict(omit, x$CCA$wa)
        if (!is.null(x$CCA$wa.excluded))
            x$CCA$wa[omit,] <- x$CCA$wa.excluded
    }
    if (!is.null(x$CA)) {
        x$CA$u <- napredict(omit, x$CA$u)
        if (!is.null(x$CA$u.excluded))
            x$CA$u[omit,] <- x$CA$u.excluded
    }
    x
}

`lines.procrustes` <-
    function(x, type=c("segments", "arrows"),  choices=c(1,2),
             truemean = FALSE, ...)
{
    type <- match.arg(type)
    X <- x$X[,choices, drop=FALSE]
    Y <- x$Yrot[, choices, drop=FALSE]
    if (truemean) {
        X <- sweep(X, 2, x$xmean[choices], "+")
        Y <- sweep(Y, 2, x$xmean[choices], "+")
    }
    if (type == "segments")
        ordiArgAbsorber(X[,1], X[,2], Y[,1], Y[,2], FUN = segments, ...)
    else
        ordiArgAbsorber(X[,1], X[,2], Y[,1], Y[,2], FUN = arrows, ...)
    invisible()
}

### draws crossed error bars for classes in ordination. These are
### oblique to axis because so are th clouds of the points and their
### standard errors and confidence regions. The bars are principal
### axes of corresponding ellipse (as drawn in ordiellipse), and found
### as principal components of the associate covariance matrix. The
### function is modelled after ordiellipse.
`ordibar` <-
    function (ord, groups, display = "sites", kind = c("sd", "se"),
              conf,  w = weights(ord, display), col = 1,
              show.groups, label = FALSE, lwd = NULL, length = 0,  ...)
{
    weights.default <- function(object, ...) NULL
    kind <- match.arg(kind)
    draw <- TRUE
    pts <- scores(ord, display = display, ...)
    ## ordibar only works with 2D data (2 columns)
    pts <- as.matrix(pts)
    if (ncol(pts) > 2)
        pts <- pts[ , 1:2, drop = FALSE]
    if (ncol(pts) < 2)
        stop("needs two dimensions")
    w <- eval(w)
    if (length(w) == 1)
        w <- rep(1, nrow(pts))
    if (is.null(w))
        w <- rep(1, nrow(pts))
    if (!missing(show.groups)) {
        take <- groups %in% show.groups
        pts <- pts[take, , drop = FALSE]
        groups <- groups[take]
        w <- w[take]
    }
    out <- seq(along = groups)
    inds <- names(table(groups))
    if (label) {
        cntrs <- matrix(NA, nrow=length(inds), ncol=2)
        rownames(cntrs) <- inds
    }
    col <- rep(col, length = length(inds))
    names(col) <- inds
    res <- list()
    ## Remove NA scores
    kk <- complete.cases(pts) & !is.na(groups)
    for (is in inds) {
        gr <- out[groups == is & kk]
        if (length(gr)) {
            X <- pts[gr, , drop = FALSE]
            W <- w[gr]
            mat <- cov.wt(X, W)
            if (mat$n.obs == 1)
                mat$cov[] <- 0
            if (kind == "se")
                mat$cov <- mat$cov * sum(mat$wt^2)
            if (missing(conf))
                t <- 1
            else t <- sqrt(qchisq(conf, 2))
            if (mat$n.obs > 1) {
                eig <- eigen(mat$cov, symmetric = TRUE)
                v <- sweep(eig$vectors, 2, sqrt(eig$values), "*") * t
                cnt <- mat$center
                ordiArgAbsorber(v[1,] + cnt[1], v[2,] + cnt[2],
                                -v[1,] + cnt[1], -v[2,] + cnt[2],
                                col = col[is], lwd = lwd,
                                length = length/2, angle = 90, code = 3,
                                FUN = arrows, ...)
            }
            if (label) {
                cntrs[is,] <- mat$center
            }
            mat$scale <- t
            res[[is]] <- mat
        }
    }
    if (label) {
        ordiArgAbsorber(cntrs, col = par("fg"), border = col,
                        FUN = ordilabel, ...)
    }
    class(res) <- "ordibar"
    invisible(res)
}

`metaMDSdist` <-
    function (comm, distance = "bray", autotransform = TRUE,
              noshare = TRUE, trace = 1, commname,
              zerodist = "ignore", distfun = vegdist, ...)
{
    ## metaMDSdist should get a raw data matrix, but if it gets a
    ## 'dist' object return that unchanged and quit silently.
    if (inherits(comm, "dist")  ||
        ((is.matrix(comm) || is.data.frame(comm)) &&
             isSymmetric(unname(as.matrix(comm)))))
        return(comm)
    distname <- deparse(substitute(distfun))
    distfun <- match.fun(distfun)
    zerodist <- match.arg(zerodist, c("fail", "add", "ignore"))
    formals(distfun) <- c(formals(distfun), alist(... = ))
    formals(stepacross) <- c(formals(stepacross), alist(... = ))
    if (missing(commname))
        commname <- deparse(substitute(comm))
    xam <- max(comm)
    if (autotransform && xam > 50) {
        comm <- sqrt(comm)
        commname <- paste("sqrt(", commname, ")", sep = "")
        if (trace)
            cat("Square root transformation\n")
    }
    if (autotransform && xam > 9) {
        comm <- wisconsin(comm)
        commname <- paste("wisconsin(", commname, ")", sep = "")
        if (trace)
            cat("Wisconsin double standardization\n")
    }
    dis <- distfun(comm, method = distance, ...)
    call <- attr(dis, "call")
    call[[1]] <- as.name(distname)
    attr(dis, "call") <- call
    if (zerodist != "ignore" && any(dis <= 0, na.rm = TRUE)) {
        if (zerodist == "fail")
            stop("zero-value dissimilarities are not allowed")
        else if (zerodist == "add") {
            zero <- min(dis[dis > 0], na.rm = TRUE)/2
            dis[dis <= 0] <- zero
            if (trace)
                cat("Zero dissimilarities changed into ", zero,"\n")
        }
    }
    ## We actually used maxdis to decide whether index has a closed
    ## upper limit, but simple maximum does not give that info.
    ## Therefore we see if an arbitrary matrix with no shared species
    ## has distance = 1.
    maxdis <- abs(distfun(matrix(c(7,0,0,3), 2, 2),
                      method = distance, ...) - 1) < 1e-4
    if ((isTRUE(noshare) && any(tmp <- no.shared(comm))) ||
        (!is.logical(noshare) && noshare >= 0 &&
         sum(tmp <- no.shared(comm))/length(dis) > noshare)) {
        if (trace)
            cat("Using step-across dissimilarities:\n")
        rn <- range(dis[tmp], na.rm = TRUE)
        if (rn[2]/rn[1] > 1.01)
            warning("non-constant distances between points with nothing shared\n",
                    "  stepacross may be meaningless: consider argument 'noshare=0'")
        is.na(dis) <- tmp
        dis <- stepacross(dis, trace = trace, toolong=0, ...)
        if (length(unique(distconnected(tmp, trace = trace))) > 1)
            warning("data are disconnected, results may be meaningless")
    }
    attr(dis, "maxdis") <- maxdis
    attr(dis, "commname") <- commname
    attr(dis, "comm") <- comm
    attr(dis, "function") <- distname
    dis
}

update.nullmodel <-
function(object, nsim=1, seed = NULL, ...)
{
    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) 
        runif(1)
    if (is.null(seed)) 
        RNGstate <- get(".Random.seed", envir = .GlobalEnv)
    else {
        R.seed <- get(".Random.seed", envir = .GlobalEnv)
        set.seed(seed)
        RNGstate <- structure(seed, kind = as.list(RNGkind()))
        on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
    }
    if (object$commsim$isSeq) {
        perm <- object$commsim$fun(x=object$state,
            n=1L,
            nr=object$nrow,
            nc=object$ncol,
            rs=object$rowSums,
            cs=object$colSums,
            rf=object$rowFreq,
            cf=object$colFreq,
            s=object$totalSum,
            fill=object$fill,
            thin=as.integer(nsim), ...)
        state <- perm[,,1L]
        storage.mode(state) <- object$commsim$mode
        iter <- as.integer(object$iter + nsim)
#        assign("state", state, envir=object)
#        assign("iter", iter, envir=object)
#        attr(state, "iter") <- iter
        out <- nullmodel(state, object$commsim)
        out$iter <- iter
        out$data <- object$data
    } else {
#        state <- NULL
        out <- object
    }
#    invisible(state)
    out
}

### oecosimu-tests: unit tests for vegan functions

### This file contains basic unit tests for simulating null
### models. Currently we just test the marginal properties of null
### models using example(commsim).

### We had more extensive tests that also displayed heads of simulated
### matrices (not only the marginal sums), but these were unstable
### when R was compiled as ./configure --disable-long-double because
### some tests used R functions stats::rmultinom() which used long
### doubles and gave different results when long double was not
### available.

### We also had specific permatfull/permatswap tests, but these only
### tested the simple cases and were nothing but an alternative
### interface to commsim nullmodels tested here.

### <-- oecosimu/permat specifics -->

###<--- BEGIN TESTS --->
suppressPackageStartupMessages(require(vegan))
set.seed(4711)
example(commsim)

### clean
rm(list = ls())

## end permatfull/swap

## The following vegan functions depend on *oecosimu*: adipart
## hiersimu multipart raupcrick. The following functions directly
## depend on *commsimulator*: permatfull1 permatswap1.  All these have
## derived and/or method functions. These should not be broken.

## Do not break raupcrick:
set.seed(4711)
data(sipoo)
as.numeric(raupcrick(sipoo, nsimul = 99))
rm(list = ls())
## end raupcrick

### 'working' will be Gower's G = -GowerDblcen(dis^2)/2
`fitted.dbrda` <-
    function (object, model = c("CCA", "CA", "pCCA"),
              type = c("response", "working"), ...)
{
    ZAP <- sqrt(.Machine$double.eps)
    type <- match.arg(type)
    model <- match.arg(model)
    if (is.null(object[[model]]))
        stop(gettextf("component '%s' does not exist", model))
    D <- ordiYbar(object, model)
    if (type == "response") {
        ## revert Gower double centring
        de <- diag(D)
        D <- -2 * D + outer(de, de, "+")
        ## we may have tiny negative zeros: zero them, but let large
        ## negative values be and give NaN in sqrt (with a warning)
        D[abs(D) < ZAP] <- 0
        if (!object$sqrt.dist)
            D <- sqrt(D)
        D <- D * object$adjust
        D <- as.dist(D)
        ## we do not remove Lingoes or Cailliez adjustment: this
        ## typically gives too many negative distances as unadjusted D
        ## often has zero-values
    }
    D
}

`goodness.metaMDS` <-
    function(object, dis, ...)
{
    if (inherits(object, "monoMDS"))
        return(NextMethod("goodness", object, ...))
    if (missing(dis))
        dis <- metaMDSredist(object)
    if(attr(dis, "Size") != nrow(object$points))
        stop("dimensions do not match in ordination and dissimilarities")
    d <- order(dis)
    shep <- Shepard(dis, object$points)
    res <- (shep$y - shep$yf)^2/sum(shep$y^2)
    stress <- sqrt(sum(res))*100
    if ( abs(stress - object$stress) > 0.001)
        stop("dissimilarities and ordination do not match")
    res <- res[order(d)]
    attr(res, "Size") <- attr(dis, "Size")
    attr(res, "Labels") <- attr(dis, "Labels")
    class(res) <- "dist"
    sqrt(colSums(as.matrix(res))/2*10000)
}

`goodness.monoMDS` <-
    function(object, ...)
{
    ## Return vector 'x' for which sum(x^2) == stress
    stresscomp <- function(y, yf, form)
    {
        num <- (y-yf)^2
        if (form == 1)
            den <- sum(y^2)
        else
            den <- sum((y-mean(y))^2)
        num/den
    }
    ## Global, local
    if (object$model %in% c("global", "linear")) {
        x <- stresscomp(object$dist, object$dhat, object$isform)
        mat <- matrix(0, object$nobj, object$nobj)
        for (i in 1:object$ndis)
            mat[object$iidx[i], object$jidx[i]] <- x[i]
        res <- sqrt(colSums(mat + t(mat))/2)
    }
    ## Local: returns pointwise components of stress
    else if (object$model == "local") {
        res <- object$grstress/sqrt(object$ngrp)
    } else if (object$model == "hybrid" && object$ngrp == 2) {
        mat <- matrix(0, object$nobj, object$nobj)
        gr <- seq_len(object$ndis) < object$istart[2]
        x <- stresscomp(object$dist[gr], object$dhat[gr], object$isform)
        x <- c(x, stresscomp(object$dist[!gr], object$dhat[!gr], object$isform))
        i <- object$iidx
        j <- object$jidx
        for (k in 1:object$ndis) {
            mat[i[k], j[k]] <- mat[i[k], j[k]] + x[k]
        }
        res <- sqrt(colSums(mat + t(mat))/4)
    } else {
        stop("unknown 'monoMDS' model")
    }
    res
}

`coef.rda` <-
    function (object, norm = FALSE, ...)
{
    if(is.null(object$CCA))
        stop("unconstrained models do not have coefficients")
    Q <- object$CCA$QR
    u <- object$CCA$u
    ## scores.cca uses na.predict and may add missing NA rows to u,
    ## but Q has no missing cases
    if (nrow(Q$qr) < nrow(u) && inherits(object$na.action, "exclude"))
        u <- u[-object$na.action,, drop=FALSE]
    b <- qr.coef(Q, u)
    if (norm)
        b <- sqrt(colSums(qr.X(Q)^2)) * b
    b
}

`ordispider` <-
    function (ord, groups, display = "sites", w = weights(ord, display),
              spiders = c("centroid", "median"), show.groups,
              label = FALSE, col = NULL, lty = NULL, lwd = NULL, ...)
{
    weights.default <- function(object, ...) NULL
    spiders <- match.arg(spiders)
    if (inherits(ord, "cca") && missing(groups)) {
        lc <- scores(ord, display = "lc", ...)
        wa <- scores(ord, display = "wa", ...)
        if (is.null(col))
            col <- par("fg")
        ordiArgAbsorber(lc[, 1], lc[, 2], wa[, 1], wa[, 2],
                        FUN = segments, col = col, lty = lty, lwd = lwd,
                        ...)
        class(lc) <- "ordispider"
        return(invisible(lc))
    }
    pts <- scores(ord, display = display, ...)
    ## spids stores pointwise centroids to be returned invisibly
    ## (transposed here so that filling is easier, but back-transposed
    ## when returned).
    spids <- t(array(NA, dim=dim(pts), dimnames = dimnames(pts)))
    ## ordihull: draw lines from centre to the points in the hull
    if (inherits(ord, "ordihull"))
        groups <- attr(pts, "hulls")
    w <- eval(w)
    if (length(w) == 1)
        w <- rep(1, nrow(pts))
    if (is.null(w))
        w <- rep(1, nrow(pts))
    if (!missing(show.groups)) {
        take <- groups %in% show.groups
        pts <- pts[take, , drop = FALSE]
        groups <- groups[take]
        w <- w[take]
    }
    if (spiders == "median" && sd(w) > sqrt(.Machine$double.eps))
        warning("weights are ignored with 'median' spiders")
    out <- seq(along = groups)
    inds <- names(table(groups))
    if (label)
    cntrs <- names <- NULL

    ## fill in graphical vectors with default values if unspecified
    ## and recycles shorter vectors
    for(arg in c("col","lty","lwd")) {
        tmp <- mget(arg,ifnotfound=list(NULL))[[1]]
        if(is.null(tmp))
            tmp <- ifelse(suppressWarnings(is.null(par(arg))),
                          par("fg"), par(arg))
        if(length(inds) != length(tmp))
            tmp <- rep_len(tmp, length(inds))
        assign(arg, tmp)
    }
    ## 'kk' removes NA scores and NA groups
    kk <- complete.cases(pts) & !is.na(groups)
    for (is in inds) {
        gr <- out[groups == is & kk]
        if (length(gr)) {
            X <- pts[gr, , drop = FALSE]
            W <- w[gr]
            if (length(gr) > 1) {
                ave <- switch(spiders,
                              "centroid" = apply(X, 2, weighted.mean, w = W),
                              "median" = ordimedian(X, rep(1, nrow(X))))
                ordiArgAbsorber(ave[1], ave[2], X[, 1], X[, 2],
                                FUN = segments, col[match(is, inds)],
                                lty = lty[match(is,inds)],
                                lwd = lwd[match(is,inds)],...)
            } else {
                ave <- X
            }
            spids[,gr] <- ave
            if (label) {
                cntrs <- rbind(cntrs, ave)
                names <- c(names, is)
            }
        }
    }
    if (label)
        ordiArgAbsorber(cntrs, label = names, FUN = ordilabel, ...)
    spids <- t(spids)
    class(spids) <- "ordispider"
    invisible(spids)
}

## this thing creates an environment
## the whole point is to create all possible inputs for
## commsim functions only once and reuse them as necessary
## also helps keeping track of updating process for sequential algorithms
## method$mode can be evaluated and use storage mode accordingly
`nullmodel` <-
    function(x, method)
{
    x <- as.matrix(x)
    if (is.null(dim(x)) || length(dim(x)) != 2L)
        stop("'x' must be a matrix-like object")
    if (any(is.na(x)))
        stop("'NA' values not allowed")
    if (any(x<0))
        stop("negative values not allowed")
    method <- make.commsim(method)
    if (method$binary)
        x <- ifelse(x > 0, 1L, 0L)
    int <- method$mode == "integer"
    if (int && abs(sum(x) - sum(as.integer(x))) > 10^-6)
        stop("non-integer values not allowed")
    if (int)
        x <- round(x, 0) # round up to closest integer
    storage.mode(x) <- method$mode
    out <- list(
        data=x,
        nrow=as.integer(dim(x)[1L]),
        ncol=as.integer(dim(x)[2L]),
        rowSums=rowSums(x),
        colSums=colSums(x),
        rowFreq=as.integer(rowSums(x > 0)),
        colFreq=as.integer(colSums(x > 0)),
        totalSum=ifelse(int, as.integer(sum(x)), as.double(sum(x))),
        fill=as.integer(sum(x > 0)),
        commsim=method,
        state=if (method$isSeq) x else NULL,
        iter=if (method$isSeq) as.integer(0L) else NULL
        )
#    storage.mode(out$x) <- method$mode
    storage.mode(out$rowSums) <- method$mode
    storage.mode(out$colSums) <- method$mode
    out <- list2env(out, parent=emptyenv())
    class(out) <- c("nullmodel", "environment")
#    class(out) <- "nullmodel"
    out
}

"print.envfit" <-
  function(x, ...)
{
  if (!is.null(x$vectors)) {
    cat("\n***VECTORS\n\n")
    print(x$vectors)
  }
  if (!is.null(x$factors)) {
    cat("\n***FACTORS:\n\n")
    print(x$factors)
  }
  if (!is.null(x$na.action))
      cat("\n", naprint(x$na.action), "\n", sep="")
  invisible(x)
}

`print.CCorA` <-
    function(x, ...)
{
    cat("\nCanonical Correlation Analysis\n")
    cat("\nCall:\n")
    cat(deparse(x$call), "\n\n")
    out <- structure(rbind(x$Mat.ranks), dimnames = list("Matrix Ranks", c("Y", "X")))
    print(out, ...)
    cat("\n")
    cat("Pillai's trace: ", format(x$Pillai, ...), "\n")
    cat("\n")
    cat("Significance of Pillai's trace:\n")

    cat("from F-distribution:  ", format.pval(x$p.Pillai), "\n")
    if (x$nperm > 0) {
        cat("based on permutations: ")
        cat(x$p.perm,"\n")
        cat(howHead(x$control), "\n")
    }
    out <- rbind("Eigenvalues" = x$EigenValues, "Canonical Correlations" = x$CanCorr)
    colnames(out) <- colnames(x$Cy)
    printCoefmat(out, ...)
    cat("\n")
    out <- rbind("RDA R squares" = x$RDA.Rsquares, "adj. RDA R squares" = x$RDA.adj.Rsq)
    colnames(out) <- c("Y | X", "X | Y")
    printCoefmat(out, ...)
    cat("\n")
    invisible(x) 
}

### The function displays (ordered) heatmaps of community data. It
### copies vegemite() for handling 'use', 'sp.ind', 'site.ind' and
### 'select', but then switches to heatmap() to display the
### data. Unlike heatmap(), it does not insist on showing dendrograms,
### but only uses these for sites, and only if given as 'use'.

`tabasco` <-
    function (x, use, sp.ind = NULL, site.ind = NULL,
              select, Rowv = TRUE, Colv = TRUE, labRow = NULL,
              labCol = NULL, scale, col = heat.colors(12), ...)
{
    if (any(x < 0))
        stop("function cannot be used with negative data values")
    pltree <- sptree <- NA
    if (missing(scale))
        scale <- "none"
    else
        scale <- match.arg(scale, c("none", "column", "row",
                                    eval(formals(coverscale)$scale)))
    if (!missing(use)) {
        if (!is.list(use) && is.vector(use)) {
            if (is.null(site.ind))
                site.ind <- order(use)
            if (is.null(sp.ind))
                sp.ind <- order(wascores(use, x))
        }
        else if (inherits(use, c("dendrogram", "hclust", "twins"))) {
            ## "twins" and "dendrogram" are treated as "dendrogram",
            ## but "hclust" is kept as "hclust": they differ in
            ## reorder()
            if (inherits(use, "twins")) {
                use <- as.dendrogram(use)
            }
            if (!is.null(site.ind))
                stop("'site.ind' cannot be used with dendrogram")
            ## The tree/dendrogam and input data must be ordered
            ## identically. It could be regarded as a "user error" if
            ## they are not, but this could be really frustrating and
            ## give obscure errors, and therefore we take care of
            ## identical ordering here
            if (inherits(use, "hclust") && !is.null(use$labels))
                x <- x[use$labels,]
            else # dendrogram
                x <- x[labels(use),]
            ## Reorder tree if Rowv specified
            if (isTRUE(Rowv)) {
                ## order by first CA axis -- decorana() is fastest
                tmp <- decorana(x, ira = 1)
                ## reorder() command is equal to all, but "dendrogram"
                ## will use unweighted mean and "hclust" weighted
                ## mean.
                use <- reorder(use, scores(tmp, dis="sites", choices = 1),
                               agglo.FUN = "mean")
            } else if (length(Rowv) > 1) {
                ## Rowv is a vector
                if (length(Rowv) != nrow(x))
                    stop(gettextf("Rowv has length %d, but 'x' has %d rows",
                                  length(Rowv), nrow(x)))
                use <- reorder(use, Rowv, agglo.FUN = "mean")
            }
            if (inherits(use, "dendrogram")) {
                site.ind <- seq_len(nrow(x))
                names(site.ind) <- rownames(x)
                site.ind <- site.ind[labels(use)]
            } else {
                site.ind <- use$order
            }
            if (is.null(sp.ind))
                sp.ind <- order(wascores(order(site.ind), x))
            pltree <- use
            ## heatmap needs a "dendrogram"
            if(!inherits(pltree, "dendrogram"))
                pltree <- as.dendrogram(pltree)
        }
        else if (is.list(use)) {
            tmp <- scores(use, choices = 1, display = "sites")
            if (is.null(site.ind))
                site.ind <- order(tmp)
            if (is.null(sp.ind))
                sp.ind <- try(order(scores(use, choices = 1,
                                           display = "species")))
            if (inherits(sp.ind, "try-error"))
                sp.ind <- order(wascores(tmp, x))
        }
        else if (is.matrix(use)) {
            tmp <- scores(use, choices = 1, display = "sites")
            if (is.null(site.ind))
                site.ind <- order(tmp)
            if (is.null(sp.ind))
                sp.ind <- order(wascores(tmp, x))
        }
    }
    ## see if sp.ind is a dendrogram or hclust tree
    if (inherits(sp.ind, c("hclust", "dendrogram", "twins"))) {
        if (inherits(sp.ind, "twins")) {
            sp.ind <- as.dendrogram(sp.ind)
        }
        sptree <- sp.ind
        ## Reorder data to match order in the dendrogam (see 'use' above)
        if (inherits(sptree, "hclust"))
            x <- x[, sptree$labels]
        else # dendrogram
            x <- x[, labels(sptree)]
        ## Consider reordering species tree
        if (isTRUE(Colv) && !is.null(site.ind)) {
            sptree <- reorder(sptree, wascores(order(site.ind), x),
                                  agglo.FUN = "mean")
        } else if (length(Colv) > 1) {
            if (length(Colv) != ncol(x))
                stop(gettextf("Colv has length %d, but 'x' has %d columns",
                              length(Colv), ncol(x)))
            sptree <- reorder(sptree, Colv, agglo.FUN = "mean")
        }
        if (inherits(sptree, "dendrogram")) {
            sp.ind <- seq_len(ncol(x))
            names(sp.ind) <- colnames(x)
            sp.ind <- sp.ind[labels(sptree)]
        } else {
            sp.ind <- sptree$order
        }
        if (!inherits(sptree, "dendrogram"))
            sptree <- as.dendrogram(sptree)
        ## reverse: origin in the upper left corner
        sptree <- rev(sptree)
    }
    if (!is.null(sp.ind) && is.logical(sp.ind))
        sp.ind <- (1:ncol(x))[sp.ind]
    if (!is.null(site.ind) && is.logical(site.ind))
        site.ind <- (1:nrow(x))[site.ind]
    if (is.null(sp.ind))
        sp.ind <- 1:ncol(x)
    if (is.null(site.ind))
        site.ind <- 1:nrow(x)
    if (!missing(select)) {
        if (!is.na(pltree))
            stop("sites cannot be 'select'ed with dendrograms or hclust trees")
        if (!is.logical(select))
            select <- sort(site.ind) %in% select
        stake <- colSums(x[select, , drop = FALSE]) > 0
        site.ind <- site.ind[select[site.ind]]
        site.ind <- site.ind[!is.na(site.ind)]
    }
    else {
        stake <- colSums(x[site.ind, ]) > 0
    }
    sp.ind <- sp.ind[stake[sp.ind]]
    ## heatmap will reorder items by dendrogram so that we need to
    ## give indices in the unsorted order if rows or columns have a
    ## dendrogram
    if (is.na(pltree[1]))
        rind <- site.ind
    else
        rind <- sort(site.ind)
    if (is.na(sptree[1]))
        ## reverse: origin in the upper left corner
        cind <- rev(sp.ind)
    else
        cind <- sort(sp.ind)
    ## we assume t() changes data.frame to a matrix
    x <- t(x[rind, cind])
    ## labels must be ordered if there is no dendrogram
    if (!is.null(labRow))
        labRow <- labRow[cind]
    if (!is.null(labCol))
        labCol <- labCol[rind]
    x <- switch(scale,
                "none" = x,
                "column" = decostand(x, "max", 2),
                "row" = decostand(x, "max", 1),
                as.matrix(coverscale(x, scale, character = FALSE)))
    ## explicit scaling so that zeros and small abundances get
    ## different colours
    brk <- (max(x) - min(x[x>0])/2)/length(col)
    brk <- 0:length(col) * brk
    heatmap((max(x) - x), Rowv = sptree, Colv = pltree,
            scale = "none", labRow = labRow, labCol = labCol,
            col = col, breaks = brk, ...)
    out <- list(sites = site.ind, species = sp.ind)
    invisible(out)
}

#' Extract internal working matrices from constrained ordination object
#'
#' Function extract internal working matrices from the constrained
#' ordination object (class \code{"cca"}) in \pkg{vegan}. The function
#' returns only one model matrix type, but there is no overlap between
#' elements of typical request.
#'
#' @param x ordination object
#' @param model model to be extracted
#'
`ordiYbar` <-
    function(x, model = c("CCA", "CA", "pCCA", "partial", "initial"))
{
    model <- match.arg(model)
    isDB <- inherits(x, "dbrda")
    if (!is.null(x$Ybar))
        Ybar <- x$Ybar
    else ## vegan < 2.5 from 2016 did not have Ybar
        stop(gettextf("outdated object from 2016 or before: use update(%s)",
                      noquote(deparse(substitute(x)))))
    if (model == "initial")
        return(Ybar)
    ## return NULL for missing elements
    if (model != "partial")
        if(is.null(x[[model]]))
            return(NULL)

    ## edit Ybar -- not yet dbrda
    switch(model,
    "pCCA" = {
        Ybar <- qr.fitted(x$pCCA$QR, Ybar)
        if (isDB)
            Ybar <- qr.fitted(x$pCCA$QR, t(Ybar))
    },
    "partial" = {
        if (!is.null(x$pCCA)) {
            Ybar <- qr.resid(x$pCCA$QR, Ybar)
            if (isDB)
                Ybar <- qr.resid(x$pCCA$QR, t(Ybar))
        }
    },
    "CCA" = {
        if (!is.null(x$pCCA)) {
            Ybar <- qr.resid(x$pCCA$QR, Ybar)
            if (isDB)
                Ybar <- qr.resid(x$pCCA$QR, t(Ybar))
        }
        Ybar <- qr.fitted(x$CCA$QR, Ybar)
        if (isDB)
            Ybar <- qr.fitted(x$CCA$QR, t(Ybar))
    },
    "CA" = {
        if (!is.null(x$CCA)) {
            Ybar <- qr.resid(x$CCA$QR, Ybar)
            if (isDB)
                Ybar <- qr.resid(x$CCA$QR, t(Ybar))
        }
        else if (!is.null(x$pCCA)) {
            Ybar <- qr.resid(x$pCCA$QR, Ybar)
            if (isDB)
                Ybar <- qr.resid(x$pCCA$QR, t(Ybar))
        }
    })
    Ybar
}

`stressplot`<-
    function(object, ...)
{
    UseMethod("stressplot")
}

`stressplot.monoMDS` <-
    function(object, pch, p.col = "blue", l.col = "red", lwd, ...)
{
    if (missing(lwd))
        if (object$ngrp > 2)
            lwd <- 1
        else
            lwd <- 2
    ## extract items to plot
    x <- object$diss
    y <- object$dist
    yf <- object$dhat
    ## all models plot dist against diss, but there can be duplicated
    ## items in some models: remove duplicates in hybrid (iregn==3)
    ## and local (ngrp > 1) models:
    if (object$iregn == 3)
        pts <- seq_along(x) < object$istart[2]
    else if (object$ngrp > 2)
        pts <- object$iidx > object$jidx
    else
        pts <- !logical(length(x))
    ## Plotting character
    if (missing(pch))
        if (sum(pts) > 5000) pch <- "." else pch <- 1
    ## plot points
    plot(x[pts], y[pts], pch = pch, col = p.col, xlab = "Observed Dissimilarity",
         ylab = "Ordination Distance", ...)
    ## collect values for 'linear fit'
    ralscal <- 0
    ## Fit lines: linear (iregn=2) and hybrid (iregn=3) have a smooth line
    if (object$iregn > 1) {
        if (object$iregn == 3) {
            k <- seq(object$istart[2], object$ndis)
            yl <- range(yf[k])
            xl <- range(x[k])
            ralscal <- cor(y[k], yf[k])^2
        } else {
            yl <- range(yf)
            xl <- range(x)
            ralscal <- cor(y, yf)^2
        }
        lines(xl, yl, col = l.col, lwd = lwd, ...)
    }
    ## Monotone line except in linear, and local has several...
    if (object$iregn != 2) {
        ist <- c(object$istart, object$ndis + 1)
        if (object$iregn == 3)
            object$ngrp <- 1
        for(j in 1:object$ngrp) {
            k <- seq(ist[j], ist[j+1]-1)
            ralscal <- ralscal + cor(y[k], yf[k])^2
            lines(x[k], yf[k], type = "S", col = l.col, lwd = lwd, ...)
        }
    }
    ## Stress as R2
    rstress <- 1 - object$stress^2
    ralscal <- if(object$iregn == 3) ralscal/2 else ralscal/object$ngrp
    Rst <- format(rstress, digits = 3)
    Ral <- format(ralscal, digits = 3)
    lab1 <- bquote("Non-metric fit, " * R^2 == .(Rst))
    lab2 <- bquote("Linear fit, " * R^2 == .(Ral))
    text(min(x), 0.95*max(y), lab1, pos=4)
    text(min(x), 0.95*max(y) - strheight(lab1), lab2, pos=4)
    ## we want to have invisible return lists in the input order
    o <- order(object$jidx, object$iidx)
    invisible(list("x" = x[o], "y" = y[o], "yf" = yf[o]))
}

`stressplot.default` <-
    function(object, dis, pch, p.col = "blue", l.col = "red", lwd = 2, ...)
{
    ## the default function only works with metaMDS or MASS::isoMDS results
    if (!(inherits(object, "metaMDS") ||
        all(c("points", "stress") %in% names(object))))
        stop("can be used only with objects that are compatible with MASS::isoMDS results")
    if (missing(dis))
        if (inherits(object, "metaMDS"))
            dis <- metaMDSredist(object)
        else
            stop("needs dissimilarities 'dis'")
     if (attr(dis, "Size") != nrow(object$points))
        stop("dimensions do not match in ordination and dissimilarities")
    shep <- Shepard(dis, object$points)
    stress <- sum((shep$y - shep$yf)^2)/sum(shep$y^2)
    rstress <- 1 - stress
    ralscal <- cor(shep$y, shep$yf)^2
    stress <- sqrt(stress)*100
    if ( abs(stress - object$stress) > 0.001)
        stop("dissimilarities and ordination do not match")
    if (missing(pch))
        if (length(dis) > 5000) pch <- "." else pch <- 1
    plot(shep, pch = pch, col = p.col, xlab = "Observed Dissimilarity",
         ylab = "Ordination Distance", ...)
    lines(shep$x, shep$yf, type = "S", col = l.col, lwd = lwd, ...)
    Rst <- format(rstress, digits = 3)
    Ral <- format(ralscal, digits = 3)
    lab1 <- bquote("Non-metric fit, " * R^2 == .(Rst))
    lab2 <- bquote("Linear fit, " * R^2 == .(Ral))
    text(min(shep$x), 0.95*max(shep$y), lab1, pos=4)
    text(min(shep$x), 0.95*max(shep$y) - strheight(lab1), lab2, pos=4)
    invisible(shep)
}

`cIndexKM` <-
    function (y, x, index = "all") 
{
    kmeans_res <- y
#############################################
    gss <- function(x, clsize, withins) 
    {
        n <- sum(clsize)
        k <- length(clsize)
        allmean <- colMeans(x)
        dmean <- sweep(x, 2, allmean, "-")
        allmeandist <- sum(dmean^2)
        wgss <- sum(withins)
        bgss <- allmeandist - wgss
        list(wgss = wgss, bgss = bgss)
    }
#############################################

### Function modified by SD and PL from the original "cIndexKM" in "cclust" 
### to accommodate a single response variable as well as singleton groups
### and remove unwanted index.
		
###  The index
################################################
    calinski <- function(zgss, clsize) 
    {
        n <- sum(clsize)
        k <- length(clsize)
        ## undefined 0/0 for one class (or fewer in error cases)
        if (k <= 1)
            NA
        else
            zgss$bgss/(k - 1)/(zgss$wgss/(n - k))
    }
################################################
    ssi <- function(centers, clsize) 
    {
        ncl <- dim(centers)[1]
        nvar <- dim(centers)[2]
        n <- sum(clsize)
        cmax <- apply(centers, 2, max)
        cmin <- apply(centers, 2, min)
        cord <- apply(centers, 2, order)
        cmaxi <- cord[ncl, ]
        cmini <- cord[1, ]
        meanmean <- mean(centers)
        absmdif <- abs(apply(centers, 2, mean) - meanmean)
        span <- cmax - cmin
        csizemax <- clsize[cmaxi]
        csizemin <- clsize[cmini]
        hiest <- nvar
        hiestw <- hiest * max(max(csizemax), max(csizemin)) *
            exp(-min(absmdif))
        sist <- sum(span)/hiest
        sistw <- (span * exp(-absmdif)) %*% sqrt(csizemax * csizemin)/hiestw
        list(ssi = sist, ssiw = sistw)
    }
################################################
		
    zgss <- gss(x, kmeans_res$size, kmeans_res$withinss)
		
    index <- pmatch(index, c("calinski", "ssi", "all"))
    if (is.na(index)) 
        stop("invalid clustering index")
    if (index == -1) 
        stop("ambiguous index")
    vecallindex <- numeric(3)
    if (any(index == 1) || (index == 3)) 
        vecallindex[1] <- calinski(zgss, kmeans_res$size)
    if (any(index == 2) || (index == 3)) 
        vecallindex[2] <- ssi(kmeans_res$centers, kmeans_res$size)$ssiw
    names(vecallindex) <- c("calinski", "ssi")
    if (index < 3) 
        vecallindex <- vecallindex[index]
    vecallindex
}

`CCorA` <-
    function(Y, X, stand.Y = FALSE, stand.X = FALSE, permutations = 0, ...)
{
    epsilon <- sqrt(.Machine$double.eps)
    ##
    ## BEGIN: Internal functions
    ##
    cov.inv <- function(mat, no, epsilon) {
        ## This function returns:
        ## 1) mat = matrix F of the principal components (PCA object scores);
        ## 2) S.inv = the inverse of the covariance matrix;
        ## 3) m = the rank of matrix 'mat'
        ## The inverse of the PCA covariance matrix is the diagonal
        ## matrix of (1/eigenvalues). If ncol(mat) = 1, the
        ## inverse of the covariance matrix contains 1/var(mat).
        mat <- as.matrix(mat)    # 'mat' was centred before input to cov.inv
        if(ncol(mat) == 1) {
            S.inv <- as.matrix(1/var(mat))
            m <- 1
        } else {
            S.svd <- svd(cov(mat))
            m <- ncol(mat)
            mm <- length(which(S.svd$d > max(epsilon, epsilon * S.svd$d[1L])))
            if(mm < m) {
                message(gettextf("matrix %d: rank=%d < order %d",
                                 no, mm, m))
                m <- mm
            }
            S.inv <- diag(1/S.svd$d[1:m])
            mat <- mat %*% S.svd$u[,1:m] # S.svd$u = normalized eigenvectors
        }
        list(mat=mat, S.inv=S.inv, m=m)
    }
    ## Check zero variances
    var.null <- function (mat, no) {
        problems <- diag(cov(mat)) <= 0
        if (any(problems)) {
            whichProbs <- paste(which(problems), collapse=", ")
            warning("zero variance in variable(s) ", whichProbs)
            stop("verify/modify your matrix No. ", no)
        }
        invisible(0)
    }
    probPillai <- function(Y.per, X, n, S11.inv, S22.inv, s, df1, df2, epsilon,
                           Fref, permat, ...) {
        ## Permutation test for Pillai's trace in CCorA.
        ## Reference: Brian McArdle's unpublished graduate course notes.
        S12.per <- cov(Y.per,X)
        gross.mat <- S12.per %*% S22.inv %*% t(S12.per) %*% S11.inv
        Pillai.per <- sum(diag(gross.mat))
        Fper  <- (Pillai.per*df2)/((s-Pillai.per)*df1)
        Fper >= (Fref-epsilon)
    }
    ## END: internal functions
    ##
    Y <- as.matrix(Y)
    var.null(Y,1)
    nY <- nrow(Y)
    p <- ncol(Y)
    if(is.null(colnames(Y))) {
        Ynoms <- paste("VarY", 1:p, sep="")
        } else {
        Ynoms <- colnames(Y)
        }
    X <- as.matrix(X)
    var.null(X,2)
    nX <- nrow(X)
    q <- ncol(X)
    if(is.null(colnames(X))) {
        Xnoms <- paste("VarX", 1:q, sep="")
        } else {
        Xnoms <- colnames(X)
        }
    if(nY != nX) stop("different numbers of rows in Y and X")
    n <- nY
    if(is.null(rownames(X)) & is.null(rownames(Y))) {
        rownoms <- paste("Obj", 1:n, sep="")
        } else {
        if(is.null(rownames(X))) {
            rownoms <- rownames(Y)
            } else {
            rownoms <- rownames(X)
            }
        }
    Y.c <- scale(Y, center = TRUE, scale = stand.Y)
    X.c <- scale(X, center = TRUE, scale = stand.X)
    ## Check for identical matrices
    if(p == q) {
    	if(sum(abs(Y-X)) < epsilon^2) stop("Y and X are identical")
    	if(sum(abs(Y.c-X.c)) < epsilon^2) stop("after centering, Y and X are identical")
    	}
    ## Replace Y.c and X.c by tables of their PCA object scores, computed by SVD
    temp <- cov.inv(Y.c, 1, epsilon)
    Y <- temp$mat
    pp <- temp$m
    rownames(Y) <- rownoms
    temp <- cov.inv(X.c, 2, epsilon)
    X <- temp$mat
    qq <- temp$m
    rownames(X) <- rownoms
    ## Correction PL, 26dec10
    if(max(pp,qq) >= (n-1))
    	stop("not enough degrees of freedom: max(pp,qq) >= (n-1)")
    ## Covariance matrices, etc. from the PCA scores
    S11 <- cov(Y)
    if(sum(abs(S11)) < epsilon) return(0)
    S22 <- cov(X)
    if(sum(abs(S22)) < epsilon) return(0)
    S12 <- cov(Y,X)
    if(sum(abs(S12)) < epsilon) return(0)
    S11.chol <- chol(S11)
    S11.chol.inv <- solve(S11.chol)
    S22.chol <- chol(S22)
    S22.chol.inv <- solve(S22.chol)
    ## K summarizes the correlation structure between the two sets of variables
    K <- t(S11.chol.inv) %*% S12 %*% S22.chol.inv
    K.svd <- svd(K)
    Eigenvalues <- K.svd$d^2
    ##
    ## Check for circular covariance matrix
    if((p == q) & (var(K.svd$d) < epsilon))
    	warning("[nearly] circular covariance matrix - the solution may be meaningless")
    ## K.svd$u %*% diag(K.svd$d) %*% t(K.svd$v)   # To check that K = U D V'
    axenames <- paste("CanAxis",seq_along(K.svd$d),sep="")
    U <- K.svd$u
    V <- K.svd$v
    A <- S11.chol.inv %*% U
    B <- S22.chol.inv %*% V
    Cy <- (Y %*% A)    # Correction 27dec10: remove /sqrt(n-1)
    Cx <- (X %*% B)    # Correction 27dec10: remove /sqrt(n-1)
    ## Compute the 'Biplot scores of Y and X variables' a posteriori --
    corr.Y.Cy <- cor(Y.c, Cy)  # To plot Y in biplot in space Y
    corr.Y.Cx <- cor(Y.c, Cx)  # Available for plotting Y in space of X
    corr.X.Cy <- cor(X.c, Cy)  # Available for plotting X in space of Y
    corr.X.Cx <- cor(X.c, Cx)  # To plot X in biplot in space X
    ## Add row and column names
    rownames(Cy) <- rownames(Cx) <- rownoms
	colnames(Cy) <- colnames(Cx) <- axenames
    rownames(corr.Y.Cy) <- rownames(corr.Y.Cx) <- Ynoms
    rownames(corr.X.Cy) <- rownames(corr.X.Cx) <- Xnoms
    colnames(corr.Y.Cy) <- colnames(corr.Y.Cx) <- axenames
    colnames(corr.X.Cy) <- colnames(corr.X.Cx) <- axenames

    ## Compute the two redundancy statistics
    RsquareY.X <- simpleRDA2(Y, X)
    RsquareX.Y <- simpleRDA2(X, Y)
    Rsquare.adj.Y.X <- RsquareAdj(RsquareY.X$Rsquare, n, RsquareY.X$m)
    Rsquare.adj.X.Y <- RsquareAdj(RsquareX.Y$Rsquare, n, RsquareX.Y$m)
    ## Compute Pillai's trace = sum of the canonical eigenvalues
    ##                        = sum of the squared canonical correlations
    S11.inv <- S11.chol.inv %*% t(S11.chol.inv)
    S22.inv <- S22.chol.inv %*% t(S22.chol.inv)
    gross.mat <- S12 %*% S22.inv %*% t(S12) %*% S11.inv
    PillaiTrace <- sum(diag(gross.mat))
    s   <- min(pp, qq)
    df1 <- max(pp,qq)
    df2 <- (n - max(pp,qq) - 1)
    Fval  <- (PillaiTrace*df2)/((s-PillaiTrace)*df1)
    p.Pillai <- pf(Fval, s*df1, s*df2, lower.tail=FALSE)
    permat <- getPermuteMatrix(permutations, n, ...)
    nperm <- nrow(permat)
    if (ncol(permat) != n)
        stop(gettextf("'permutations' have %d columns, but data have %d rows",
                      ncol(permat), n))

    if (nperm > 0) {
        p.perm <- sapply(seq_len(nperm), function(indx, ...)
                         probPillai(Y[permat[indx,],] , X, n, S11.inv, S22.inv, s,
                                    df1, df2, epsilon, Fval, nperm, ...))
        p.perm <- (sum(p.perm) +1)/(nperm + 1)
    } else {
        p.perm <- NA
    }

    out <- list(Pillai=PillaiTrace, Eigenvalues=Eigenvalues, CanCorr=K.svd$d,
                Mat.ranks=c(RsquareX.Y$m, RsquareY.X$m),
                RDA.Rsquares=c(RsquareY.X$Rsquare, RsquareX.Y$Rsquare),
                RDA.adj.Rsq=c(Rsquare.adj.Y.X, Rsquare.adj.X.Y),
                nperm=nperm, p.Pillai=p.Pillai, p.perm=p.perm, Cy=Cy, Cx=Cx,
                corr.Y.Cy=corr.Y.Cy, corr.X.Cx=corr.X.Cx, corr.Y.Cx=corr.Y.Cx,
                corr.X.Cy=corr.X.Cy, control = attr(permat, "control"),
                call = match.call())
    class(out) <- "CCorA"
    out
}

`adipart.default` <-
    function(y, x, index=c("richness", "shannon", "simpson"),
             weights=c("unif", "prop"), relative = FALSE, nsimul=99,
             method = "r2dtable", ...)
{
    ## evaluate formula
    lhs <- as.matrix(y)
    if (missing(x))
        x <- cbind(level_1=seq_len(nrow(lhs)),
            leve_2=rep(1, nrow(lhs)))
    rhs <- data.frame(x)
    rhs[] <- lapply(rhs, as.factor)
    rhs[] <- lapply(rhs, droplevels, exclude = NA)
    nlevs <- ncol(rhs)
    if (nlevs < 2)
        stop("provide at least two-level hierarchy")
    if (any(rowSums(lhs) == 0))
        stop("data matrix contains empty rows")
    if (any(lhs < 0))
        stop("data matrix contains negative entries")
    if (is.null(colnames(rhs)))
        colnames(rhs) <- paste("level", 1:nlevs, sep="_")
    tlab <- colnames(rhs)

    ## check proper design of the model frame
    l1 <- sapply(rhs, function(z) length(unique(z)))
    if (!any(sapply(2:nlevs, function(z) l1[z] <= l1[z-1])))
        stop("number of levels are inappropriate, check sequence")
    rval <- list()
    rval[[1]] <- rhs[,nlevs]
    nCol <- nlevs - 1
    for (i in 2:nlevs) {
        rval[[i]] <- interaction(rhs[,nCol], rval[[(i-1)]], drop=TRUE)
        nCol <- nCol - 1
    }
    rval <- as.data.frame(rval[rev(seq_along(rval))])
    l2 <- sapply(rval, function(z) length(unique(z)))
    if (any(l1 != l2))
        stop("levels are not perfectly nested")

    ## aggregate response matrix
    fullgamma <-if (nlevels(rhs[,nlevs]) == 1)
        TRUE else FALSE
    ftmp <- vector("list", nlevs)
    for (i in seq_len(nlevs)) {
        ftmp[[i]] <- as.formula(paste("~", tlab[i], "- 1"))
    }

    ## is there burnin/thin in ... ?
    burnin <- if (is.null(list(...)$burnin))
        0 else list(...)$burnin
    thin <- if (is.null(list(...)$thin))
        1 else list(...)$thin
    base <- if (is.null(list(...)$base))
        exp(1) else list(...)$base

    ## evaluate other arguments
    index <- match.arg(index)
    weights <- match.arg(weights)
    switch(index,
           "richness" = {
               divfun <- function(x) rowSums(x > 0)},
           "shannon" = {
               divfun <- function(x) diversity(x, index = "shannon", MARGIN = 1, base=base)},
           "simpson" = {
               divfun <- function(x) diversity(x, index = "simpson", MARGIN = 1)})

    ## this is the function passed to oecosimu
    wdivfun <- function(x) {
        ## matrix sum *can* change in oecosimu (but default is constant sumMatr)
        sumMatr <- sum(x)
        if (fullgamma) {
            tmp <- lapply(seq_len(nlevs-1), function(i) t(model.matrix(ftmp[[i]], rhs)) %*% x)
            tmp[[nlevs]] <- matrix(colSums(x), nrow = 1, ncol = ncol(x))
        } else {
            tmp <- lapply(seq_len(nlevs), function(i) t(model.matrix(ftmp[[i]], rhs)) %*% x)
        }
        ## weights will change in oecosimu thus need to be recalculated
        if (weights == "prop")
            wt <- lapply(seq_len(nlevs), function(i) apply(tmp[[i]], 1, function(z) sum(z) / sumMatr))
        else wt <- lapply(seq_len(nlevs), function(i) rep(1 / NROW(tmp[[i]]), NROW(tmp[[i]])))
        a <- sapply(seq_len(nlevs), function(i) sum(divfun(tmp[[i]]) * wt[[i]]))
        if (relative)
            a <- a / a[length(a)]
        b <- sapply(2:nlevs, function(i) a[i] - a[(i-1)])
        c(a, b)
    }
    if (nsimul > 0) {
        sim <- oecosimu(lhs, wdivfun, method = method, nsimul=nsimul,
                        burnin=burnin, thin=thin)
    } else {
        sim <- wdivfun(lhs)
        tmp <- rep(NA, length(sim))
        sim <- list(statistic = sim,
                    oecosimu = list(z = tmp, pval = tmp, method = NA, statistic = sim))
    }
    nam <- c(paste("alpha", seq_len(nlevs-1), sep="."), "gamma",
             paste("beta", seq_len(nlevs-1), sep="."))
    names(sim$statistic) <- attr(sim$oecosimu$statistic, "names") <- nam
    call <- match.call()
    call[[1]] <- as.name("adipart")
    attr(sim, "call") <- call
    attr(sim$oecosimu$simulated, "index") <- index
    attr(sim$oecosimu$simulated, "weights") <- weights
    attr(sim, "n.levels") <- nlevs
    attr(sim, "terms") <- tlab
    attr(sim, "model") <- rhs
    class(sim) <- c("adipart", class(sim))
    sim
}

panel.ordi <-
function(x, y, biplot, polygon, type = type, ...)
{
    tp <- trellis.par.get()
    sp <- tp$superpose.symbol
    ps <- tp$plot.symbol
    ## ordixyplot passes polygon of all points, but ordisplom does not
    if ("polygon" %in% type && !missing(polygon)) {
        ppar <- tp$plot.polygon
        lpolygon(polygon, col = ppar$col, border = ppar$border,
                 alpha = ppar$alpha, lty = ppar$lty, lwd = ppar$lwd, ...)
        inpol <- chull(x, y)
        par <- tp$superpose.polygon
        lpolygon(x[inpol], y[inpol], col = par$col[1L], border = par$border[1L],
                 alpha = par$alpha[1L], lty = par$lty[1L], lwd = par$lwd[1L], ...)
    }
    panel.xyplot(x, y, type = type,  ...)
    if ("biplot" %in% type && !is.null(biplot$arrows)) {
        panel.arrows(0, 0, biplot$arrows[,2], biplot$arrows[,1],
                     col=sp$col, ...)
    }
    if ("biplot" %in% type && !is.null(biplot$centres)) {
        panel.xyplot(biplot$centres[,2], biplot$centres[,1],
                     col = ps$col,
                     pch = "+", cex = 3*ps$cex, lwd=2,
                     ...)
    }
    if ("arrows" %in% type) {
        panel.superpose(x, y, panel.groups= "panel.ordiarrows", ...)
    }
    panel.abline(h=0, lty = 3)
    panel.abline(v=0, lty = 3)
}

## needed for "arrows" %in% type
panel.ordiarrows <-
function(x, y, subscripts,
         ends = "last", type = "open", length = 0.25, angle = 30, code = 2,
         ...)
{
    n <- length(x)
    col <- trellis.par.get("superpose.line")$col
    group <- list(...)$group.number
    if (is.null(group))
        col <- col[1]
    else
        col = rep(col, len=group)[group]
    panel.arrows(x[-n], y[-n], x[-1], y[-1], ends = ends, type = "open",
                 length = length, angle = angle, code = code,
                 col = col,
                 lwd = trellis.par.get("superpose.line")$lwd,
                 )
}

##' Species tolerances and sample heterogeneities
##'
##' Function to compute species tolerances and site heterogeneity measures
##' from unimodal ordinations (CCA & CA). Implements Eq 6.47 and 6.48 from
##' the Canoco 4.5 Reference Manual (pages 178-179).
##'
##' @param x object of class \code{"cca"}.
##' @param choices numeric; which ordination axes to compute
##' tolerances  and heterogeneities for. Defaults to axes 1 and 2.
##' @param which character; one of \code{"species"} or \code{"sites"},
##' indicating whether species tolerances or sample heterogeneities
##' respectively are computed.
##' @param scaling numeric or character; the ordination scaling to use.
##' @param useN2 logical; should the bias in the tolerances /
##' heterogeneities be reduced via scaling by Hill's N2?
##' @param ... arguments passed to other methods
##' @return matrix of tolerances/heterogeneities with some additional
##'   attributes: \code{which}, \code{scaling}, and \code{N2}, the latter of which will be \code{NA} if \code{useN2 = FALSE}.
##' @author Gavin L. Simpson
##' @examples
##' data(dune)
##' data(dune.env)
##' mod <- cca(dune ~ ., data = dune.env)
##' tolerance.cca(mod)
##'
tolerance.cca <- function(x, choices = 1:2,
                          which = c("species","sites"),
                          scaling = "species", useN2 = TRUE,
                          hill = FALSE, ...) {
    if(inherits(x, "rda")) {
        stop("tolerances only available for unimodal ordinations")
    }
    if(missing(which)) {
        which <- "species"
    }
    ## zapping epsilon to make approximate 1's into 1's
    ZAP <- sqrt(.Machine$double.eps)
    ## reconstruct species/response matrix Y - up to machine precision!
    Y <- (ordiYbar(x, "initial") * sqrt(x$rowsum %o% x$colsum) +
          x$rowsum %o% x$colsum) * x$grand.total
    which <- match.arg(which)
    siteScrTypes <- if (is.null(x$CCA)) {
                        "sites"
                    } else {
                        "lc"
                    }
    ## Sort out scaling; only for (C)CA so no correlation arg
    scaling <- scalingType(scaling, hill = hill)
    scrs <- scores(x, display = c(siteScrTypes, "species"),
                   choices = choices, scaling = scaling, ...)
    ## compute N2 if useN2 == TRUE & only if
    doN2 <- isTRUE(useN2) && ((which == "species" && abs(scaling) == 2) ||
                              (which == "sites" && abs(scaling) == 1))
    siteScrs <- which(names(scrs) %in% c("sites","constraints"))
    if(isTRUE(all.equal(which, "sites"))) {
        res <- matrix(ncol = length(choices), nrow = nrow(scrs[[siteScrs]]))
        Ytot <- rowSums(Y)
        for (i in seq_len(NROW(res))) {
            XiUk <- apply(scrs[["species"]], 1L, `-`, scrs[[siteScrs]][i,])
            YXiUk <- sweep(XiUk^2, 2L, Y[i,], "*")
            if(any(neg <- YXiUk < 0)) {
                YXiUk[neg] <- 0
            }
            res[i, ] <- sqrt(rowSums(YXiUk) / Ytot[i])
        }
        rownames(res) <- rownames(scrs[[siteScrs]])
        colnames(res) <- colnames(scrs[[siteScrs]])

        if(doN2) {
            y <- sweep(Y, 1, Ytot, "/")^2
            N2 <- 1 / rowSums(y, na.rm = TRUE) ## 1/H
            ## avoid almost-1 for sites with only one spp
            N2[abs(N2-1) < ZAP] <- 1
            ## avoid "negative zeros" form 1 - 1/N2 when N2 ~ 1
            res <- sweep(res, 1, sqrt(pmax(1 - 1/N2, 0)), "/")
        }
    } else {
        res <- matrix(ncol = length(choices), nrow = ncol(Y))
        Ytot <- colSums(Y)
        for (i in seq_len(NROW(res))) {
            XiUk <- apply(scrs[[siteScrs]], 1L, `-`, scrs[["species"]][i,])
            YXiUk <- sweep(XiUk^2, 2L, Y[,i], "*")
            if (any(neg <- YXiUk < 0)) {
                YXiUk[neg] <- 0
            }
            res[i, ] <- sqrt(rowSums(YXiUk) / Ytot[i])
        }
        rownames(res) <- colnames(Y)
        colnames(res) <- colnames(scrs[["species"]])

        if(doN2) {
            y <- sweep(Y, 2, Ytot, "/")^2
            N2 <- 1 / colSums(y, na.rm = TRUE) # 1/H
            ## avoid almost-1 for species present only once
            N2[abs(N2-1) < ZAP] <- 1
            ## avoid "negative zeros" form 1 - 1/N2 when N2 ~ 1
            res <- sweep(res, 1, sqrt(pmax(1 - 1/N2, 0)), "/")
        }
    }
    res[!is.finite(res)] <- 0 # some values can be Inf or NaN but are really 0
    res[res < sqrt(.Machine$double.eps)] <- 0 # almost-zero tolerances should be zero
    class(res) <- c("tolerance.cca", "tolerance","matrix")
    attr(res, "which") <- which
    attr(res, "scaling") <- scaling
    attr(res, "N2") <- NA
    if(doN2) {
        attr(res, "N2") <- N2
    }
    res                                 # return
}

`print.tolerance.cca` <- function(x, ...) {
    cat("\n")
    msg <- ifelse(attr(x, "which") == "species", "Species Tolerance",
                  "Sample Heterogeneity")
    writeLines(msg, sep = "\n\n")
    msg <- paste("Scaling:", attr(x, "scaling"))
    writeLines(strwrap(msg), sep = "\n\n")
    attr(x, "model") <- attr(x, "scaling") <- attr(x, "which") <- attr(x, "N2") <- NULL
    print(unclass(x), ...)
    cat("\n")
}

`ordilattice.getEnvfit` <-
    function(formula, object, envfit, choices = 1:3,  ...)
{
    if (!missing(envfit) && !is.null(envfit))
        object <- envfit
    bp <- scores(object, display = "bp", choices = choices, ...)
    cn <- scores(object, display = "cn",  choices = choices, ...)
    bp <- bp[!(rownames(bp) %in% rownames(cn)),, drop=FALSE]
    left <- as.character(formula[[2]])
    right <- formula[[3]]
    if (length(right) == 3)
        right <- right[[2]]
    right <- as.character(right)
    if (all(c(left,right) %in% colnames(bp)))
        bp <- bp[, c(left,right), drop=FALSE]
    else
        bp <- NULL
    if (!is.null(bp) && nrow(bp) == 0)
        bp <- NULL
    if (!is.null(ncol(cn)) && all(c(left,right) %in% colnames(cn)))
        cn <- cn[, c(left,right), drop=FALSE]
    else
        cn <- NULL
    list(arrows = bp, centres = cn)
}

### Modelled after maptools:::pointLabel.
`ordipointlabel` <-
    function(x, display = c("sites", "species"), choices = c(1,2), col=c(1,2),
             pch=c("o","+"), font = c(1,1), cex=c(0.8, 0.8), add = FALSE,
             select, ...)
{
    xy <- list()
    ## Some 'scores' accept only one 'display': a workaround
    for (nm in display)
        xy[[nm]] <- scores(x, display = nm, choices = choices, ...)
    ##xy <- scores(x, display = display, choices = choices, ...)
    ## remove `select`ed observations from scores as per text.cca
    ## only useful if we are displaying only one set of scores
    if(!missing(select)) {
        if(isTRUE(all.equal(length(display), 1L))) {
            xy[[1]] <- .checkSelect(select, xy[[1]])
        } else {
            warning("'select' does not apply when plotting more than one set of scores--\n'select' was ignored")
        }
    }
    if (length(display) > 1) {
        col <- rep(col, sapply(xy, nrow))
        pch <- rep(pch, sapply(xy, nrow))
        font <- rep(font, sapply(xy, nrow))
        cex <- rep(cex, sapply(xy, nrow))
        tmp <- xy[[1]]
        for (i in 2:length(display))
            tmp <- rbind(tmp, xy[[i]])
        xy <- tmp
    }
    else {
        xy <- xy[[1]]
        if (length(col) < nrow(xy))
            col <- col[1]
        if (length(pch) < nrow(xy))
            pch <- pch[1]
        if (length(font) < nrow(xy))
            font <- font[1]
    }
    if (!add)
        pl <- ordiplot(x, display = display, choices = choices, type="n", ...)
    labels <- rownames(xy)
    em <- strwidth("m", cex = min(cex), font = min(font))
    ex <- strheight("x", cex = min(cex), font = min(font))
    ltr <- em*ex
    w <- strwidth(labels, cex = cex, font = font) + em
    h <- strheight(labels, cex = cex, font = font) + ex
    box <- cbind(w, h)
    ## offset: 1 up, 2..4 sides, 5..8 corners
    makeoff <- function(pos, lab) {
        cbind(c(0,1,0,-1,0.9,0.9,-0.9,-0.9)[pos] * lab[,1]/2,
              c(1,0,-1,0,0.8,-0.8,-0.8,0.8)[pos] * lab[,2]/2)
    }
    ## amount of overlap
    overlap <- function(xy1, off1, xy2, off2) {
        pmax(0, pmin(xy1[,1] + off1[,1]/2, xy2[,1] + off2[,1]/2)
             -pmax(xy1[,1] - off1[,1]/2, xy2[,1] - off2[,1]/2)) *
              pmax(0, pmin(xy1[,2] + off1[,2]/2, xy2[,2] + off2[,2]/2)
             -pmax(xy1[,2] - off1[,2]/2, xy2[,2] - off2[,2]/2))
    }
    ## indices of overlaps in lower triangular matrix
    n <- nrow(xy)
    j <- as.vector(as.dist(row(matrix(0, n, n))))
    k <- as.vector(as.dist(col(matrix(0, n, n))))
    ## Find labels that may overlap...
    maylap <- overlap(xy[j,], 2*box[j,], xy[k,], 2*box[k,]) > 0
    ## ... and work only with those
    j <- j[maylap]
    k <- k[maylap]
    jk <- sort(unique(c(j,k)))
    ## SANN: no. of iterations & starting positions
    nit <- min(48 * length(jk), 10000)
    pos <- rep(1, n)
    ## Criterion: overlap + penalty for positions other than directly
    ## above and especially for corners
    fn <- function(pos) {
        off <- makeoff(pos, box)
        val <- sum(overlap(xy[j,]+off[j,], box[j,], xy[k,]+off[k,], box[k,]))
        val <- val/ltr + sum(pos>1)*0.1 + sum(pos>4)*0.1
    }
    ## Move a label of one point
    gr <- function(pos) {
        take <- sample(jk, 1)
        pos[take] <- sample((1:8)[-pos[take]], 1)
        pos
    }
    ## Simulated annealing
    sol <- optim(par = pos, fn = fn, gr = gr, method="SANN",
                 control=list(maxit=nit))
    if (!add)
        ##points(xy, pch = pch, col = col, cex=cex, ...)
        ordiArgAbsorber(xy, pch = pch, col = col, cex = cex, FUN = points,
                        ...)
    lab <- xy + makeoff(sol$par, box)
    ##text(lab, labels=labels, col = col, cex = cex, font = font,  ...)
    ordiArgAbsorber(lab, labels=labels, col = col, cex = cex, font = font,
                    FUN = text, ...)
    pl <- list(points = xy)
    pl$labels <- lab
    attr(pl$labels, "font") <- font
    args <- list(tcex = cex, tcol = col, pch = pch, pcol = col,
                 pbg = NA, pcex = cex)
    pl$args <- args
    pl$par <- par(no.readonly = TRUE)
    pl$dim <- par("din")
    attr(pl, "optim") <- sol
    class(pl) <- c("ordipointlabel", "orditkplot", class(pl))
    invisible(pl)
}

`print.metaMDS` <-
    function (x, ...)
{
    cat("\nCall:\n")
    cat(deparse(x$call), "\n\n")
    if (x$engine == "monoMDS")
        cat(x$model, "Multidimensional Scaling using monoMDS\n\n")
    else if (x$engine == "isoMDS")
        cat("non-metric Multidimensional Scaling using isoMDS (MASS package)\n\n")
    cat("Data:    ", x$data, "\n")
    cat("Distance:", x$distance, "\n\n")
    cat("Dimensions:", x$ndim, "\n")
    cat("Stress:    ", x$stress, "\n")
    if (inherits(x, "monoMDS")) {
        cat("Stress type", x$isform)
        if(x$model != "linear")
            cat(", ", c("weak", "strong")[x$ities], " ties", sep = "")
        cat("\n")
    }
    if (x$converged) {
        cat("Two convergent solutions found after", x$tries,
            "tries\n")
    } else {
        cat("No convergent solutions - best solution after",
            x$tries, "tries\n")
    }
    z <- x$points
    scal <- c(if (attr(z, "centre")) "centring",
              if (attr(z, "pc")) "PC rotation",
              if (attr(z, "halfchange")) "halfchange scaling")
    if (!length(scal))
        scal <- "as is"
    cat("Scaling:", paste(scal, collapse = ", "), "\n")
    if (all(is.na(x$species))) {
        cat("Species: scores missing\n")
    } else {
        spattr <- attr(x$species, "shrinkage")
        spdata <- attr(x$species, "data")
        if (is.null(spdata))
            spdata <- x$data
        if (is.null(spattr))
            cat("Species: non-expanded scores ")
        else
            cat("Species: expanded scores ")
        cat("based on", sQuote(spdata), "\n")
    }
    cat("\n")
    invisible(x)
}

"print.summary.humpfit" <-
    function (x, ...) 
{
    cat("\nHump-backed Null model of richness vs. productivity\n\n")
    cat("Family:", x$family, "\n")
    cat("Link function: Fisher diversity\n\n")
    cat("Coefficients:\n\n")
    printCoefmat(x$est, ...)
    cat("\nDispersion parameter for", x$family, "family taken to be", x$dispersion,"\n")
    cat("\nDeviance", x$deviance, "with", x$df.residual)
    cat(" residual degrees of freedom\n")
    cat("AIC:", x$aic, "   BIC:", x$bic, "\n")
    cat("\nCorrelation of Coefficients:\n")
    correl <- format(round(x$correlation, 2), nsmall = 2)
    correl[!lower.tri(correl)] <- ""
    print(correl[-1, -3], quote = FALSE)
    cat("\nDiagnostics from nlm:\n")
    cat("Number of iterations: ", x$iter, ", code: ", x$code, 
        "\n", sep = "")
    invisible(x)
}

`rda.formula` <-
    function (formula, data, scale = FALSE, na.action = na.fail,
              subset = NULL, ...)
{
    if (missing(data)) {
        data <- parent.frame()
    } else {
        data <- eval(match.call()$data, environment(formula),
                     enclos = .GlobalEnv)
    }
    d <- ordiParseFormula(formula, data = data, na.action = na.action,
                          subset = substitute(subset))
    sol <- rda.default(d$X, d$Y, d$Z, scale)
    if (!is.null(sol$CCA) && sol$CCA$rank > 0) {
        centroids <- centroids.cca(sol$CCA$wa, d$modelframe)
        if (!is.null(sol$CCA$alias))
            centroids <- unique(centroids)
        if (!is.null(centroids)) {
            rs <- rowSums(centroids^2)
            centroids <- centroids[rs > 1e-04,, drop = FALSE]
            if (length(centroids) == 0)
                centroids <- NULL
        }
        if (!is.null(centroids))
            sol$CCA$centroids <- centroids
    }
    ## replace rda.default call
    call <- match.call()
    call[[1]] <- as.name("rda")
    call$formula <- formula(d$terms)
    sol$call <- call
    if (!is.null(d$na.action)) {
        sol$na.action <- d$na.action
        sol <- ordiNAexclude(sol, d$excluded)
    }
    if (!is.null(d$subset))
        sol$subset <- d$subset
    ## drops class in c()
    sol <- c(sol,
             list(terms = d$terms,
                  terminfo = ordiTerminfo(d, d$modelframe)))
    class(sol) <- c("rda", "cca")
    sol
}

`print.mso` <-
    function(x,  digits = max(3, getOption("digits") - 3), ...)
{
    NextMethod("print", x, digits = digits, ...)
    cat("mso variogram:\n\n")
    print(x$vario, digits = digits, ...)
    if(!is.null(attr(x$vario, "control")))
        cat("\n", howHead(attr(x$vario, "control")), "\n", sep="")
    invisible(x)
}

"metaMDSredist" <-
function(object, ...)
{
    if (!inherits(object, "metaMDS"))
        stop("Needs a metaMDS result object")
    call <- object$call
    call[[1]] <- as.name("metaMDSdist")
    eval(call, parent.frame())
}

`rarefy` <-
    function (x, sample, se = FALSE, MARGIN = 1)
{
    x <- as.matrix(x)
    ## as.matrix changes an n-vector to a n x 1 matrix
    if (ncol(x) == 1 && MARGIN == 1)
        x <- t(x)
    if (!identical(all.equal(x, round(x)), TRUE))
        stop("function accepts only integers (counts)")
    minsample <- min(apply(x, MARGIN, sum))
    if (missing(sample)) {
        stop(
            gettextf(
                "the size of 'sample' must be given --\nHint: Smallest site maximum %d",
                minsample))
    }
    if (any(sample > minsample))
        warning(
            gettextf(
                "requested 'sample' was larger than smallest site maximum (%d)",
                minsample))
    rarefun <- function(x, sample) {
        x <- x[x > 0]
        J <- sum(x)
        ldiv <- lchoose(J, sample)
        p1 <- ifelse(J - x < sample, 0, exp(lchoose(J - x, sample) -
                                            ldiv))
        out <- sum(1 - p1)
        if (se) {
            V <- sum(p1 * (1 - p1))
            Jxx <- J - outer(x, x, "+")
            ind <- lower.tri(Jxx)
            Jxx <- Jxx[ind]
            V <- V + 2 * sum(ifelse(Jxx < sample, 0, exp(lchoose(Jxx,
                                                                 sample) - ldiv)) - outer(p1, p1)[ind])
            ## V is >= 0, but numerical zero can be negative (e.g,
            ## -1e-16), and we avoid taking its square root
            out <- cbind(out, sqrt(max(V, 0)))
        }
        out
    }
    if (length(sample) > 1) {
        S.rare <- sapply(sample, function(n) apply(x, MARGIN, rarefun, sample = n))
        S.rare <- matrix(S.rare, ncol=length(sample))
        colnames(S.rare) <- paste("N", sample, sep="")
        if (se) {
            dn <- unlist(dimnames(x)[MARGIN])
            rownames(S.rare) <- paste(rep(dn, each=2), c("S","se"), sep=".")
        }
    } else {
        S.rare <- apply(x, MARGIN, rarefun, sample = sample)
        if (se)
            rownames(S.rare) <- c("S", "se")
    }
    attr(S.rare, "Subsample") <- sample
    S.rare
}

`betadiver` <-
    function(x, method = NA, order = FALSE, help = FALSE,  ...)
{
    beta <- list("w"="(b+c)/(2*a+b+c)", "-1"="(b+c)/(2*a+b+c)", "c"="(b+c)/2",
                 "wb"="b+c", "r"="2*b*c/((a+b+c)^2-2*b*c)",
                 "I"="log(2*a+b+c) - 2*a*log(2)/(2*a+b+c) - ((a+b)*log(a+b) + (a+c)*log(a+c)) / (2*a+b+c)",
                 "e"="exp(log(2*a+b+c) - 2*a*log(2)/(2*a+b+c) - ((a+b)*log(a+b) + (a+c)*log(a+c)) / (2*a+b+c))-1",
                 "t"="(b+c)/(2*a+b+c)", "me"="(b+c)/(2*a+b+c)",
                 "j"="a/(a+b+c)", "sor"="2*a/(2*a+b+c)",
                 "m"="(2*a+b+c)*(b+c)/(a+b+c)",
                 "-2"="pmin.int(b,c)/(pmax.int(b,c)+a)",
                 "co"="(a*c+a*b+2*b*c)/(2*(a+b)*(a+c))",
                 "cc"="(b+c)/(a+b+c)", "g"="(b+c)/(a+b+c)",
                 "-3"="pmin.int(b,c)/(a+b+c)", "l"="(b+c)/2",
                 "19"="2*(b*c+1)/(a+b+c)/(a+b+c-1)",
                 "hk"="(b+c)/(2*a+b+c)", "rlb"="a/(a+c)",
                 "sim"="pmin.int(b,c)/(pmin.int(b,c)+a)",
                 "gl"="2*abs(b-c)/(2*a+b+c)",
                 "z"="(log(2)-log(2*a+b+c)+log(a+b+c))/log(2)"
                 )
    if (help) {
        for (i in seq_along(beta))
            writeLines(strwrap(paste(i, " \"", names(beta[i]),
                                     "\" = ", beta[[i]], "\n", sep="")))
        return(invisible(NULL))
    }
    x <- ifelse(x > 0, 1, 0)
    if (order) {
        x <- x[order(rowSums(x)),]
    }
    d <- tcrossprod(x)
    a <- as.dist(d)
    S <- diag(d)
    N <- length(S)
    b <- as.dist(matrix(rep(S, N), nrow=N)) - a
    c <- as.dist(matrix(rep(S, each=N), nrow=N)) - a
    if (is.na(method) || is.null(method) || is.logical(method) && !method) {
        out <- list(a = a, b = b, c = c)
        class(out) <- "betadiver"
        return(out)
    }
    out <- eval(parse(text=beta[[method]]))
    out <- as.dist(out)
    mxdist <- c(1,1,NA,NA,1,log(2),1,1,1,0,0,NA,1,1,1,1,NA,NA,NA,1,0,1,NA,1)
    names(mxdist) <- names(beta)
    attr(out, "maxdist") <- unname(mxdist[method])
    attr(out, "method") <- paste("beta", names(beta[method]), sep=".")
    attr(out, "call") <- match.call()
    out
}

"print.radfit" <-
function(x,  digits = max(3, getOption("digits") - 2),  ...) 
{
    cat("\nRAD models, family", x$family$family, "\n")
    cat("No. of species ", length(x$y), ", total abundance ", 
        sum(x$y), "\n\n", sep = "")
    p <- coef(x)
    if (any(!is.na(p)))
        p <- formatC(p, format="g", flag = " ", digits = digits)
    p <- apply(p, 2, function(x) gsub("NA", " ", x))
    aic <- sapply(x$models, AIC)
    bic <- sapply(x$models, AIC, k = log(length(x$y)))
    dev <- sapply(x$models, deviance)
    stats <- format(cbind(Deviance = dev, AIC = aic, BIC = bic), digits = digits, ...)
    out <- cbind(p, stats)
    print(out, quote=FALSE)
    invisible(x)
}

`print.isomap` <-
function (x, ...) 
{
	cat("\nIsometric Feature Mapping (isomap)\n\n")
	cat("Call:\n")
	cat(deparse(x$call), "\n\n")
	cat("Distance method:", x$method, "\n")
	cat("Criterion:", x$criterion, "=", x$critval, "\n")
	if(!is.null(x$take))
	    cat("Data were fragmented, analysed", sum(x$take), "of", length(x$take), "points\n")
	invisible(x)
}

SSgitay <-
    selfStart(~ (k + slope*log(area))^2,
              function(mCall, data, LHS)
{
    xy <- sortedXyData(mCall[["area"]], LHS, data)
    value <- as.vector(coef(lm(sqrt(xy[,"y"]) ~ log(xy[,"x"]))))
    names(value) <- mCall[c("k","slope")]
    value
},
c("k","slope"))

`adipart.formula` <-
    function(formula, data, index=c("richness", "shannon", "simpson"),
             weights=c("unif", "prop"), relative = FALSE, nsimul=99,
             method = "r2dtable", ...)
{
    ## evaluate formula
    if (missing(data))
        data <- parent.frame()
    tmp <- hierParseFormula(formula, data)
    ## run simulations
    sim <- adipart.default(tmp$lhs, tmp$rhs, index = index, weights = weights,
                           relative = relative, nsimul = nsimul,
                           method = method, ...)
    call <- match.call()
    call[[1]] <- as.name("adipart")
    attr(sim, "call") <- call
    sim
}

`factorfit` <-
    function (X, P, permutations = 0, strata = NULL, w,  ...)
{
    EPS <- sqrt(.Machine$double.eps)
    P <- as.data.frame(P)
    ## Check that all variables are factors, and coerce if necessary
    if(any(!sapply(P, is.factor)))
        P <- data.frame(lapply(P, function(x)
                        if (is.factor(x)) x else factor(x)))
    P <- droplevels(P, exclude = NA) ## make sure only the used levels are present
    if (any(!sapply(P, is.factor)))
        stop("all non-numeric variables must be factors")
    NR <- nrow(X)
    NC <- ncol(X)
    NF <- ncol(P)
    if (missing(w) || is.null(w))
        w <- 1
    if (length(w) == 1)
        w <- rep(w, NR)
    r <- NULL
    pval <- NULL
    totvar <- .Call(do_goffactor, X, rep(1L, NR), 1L, w)
    sol <- centroids.cca(X, P, w)
    var.id <- rep(names(P), sapply(P, nlevels))
    ## make permutation matrix for all variables handled in the next loop
    permat <- getPermuteMatrix(permutations, NR, strata = strata)
    permutations <- nrow(permat)

    for (i in seq_along(P)) {
        A <- as.integer(P[[i]])
        NL <- nlevels(P[[i]])
        invar <- .Call(do_goffactor, X, A, NL, w)
        r.this <- 1 - invar/totvar
        r <- c(r, r.this)
        if (permutations) {
            A <- as.integer(P[[i]])
            NL <- nlevels(P[[i]])
            ptest <- function(indx, ...) {
                take <- A[indx]
                invar <- .Call(do_goffactor, X, take, NL, w)
                1 - invar/totvar
            }
            tmp <- sapply(seq_len(permutations),
                          function(indx,...) ptest(permat[indx,], ...))
            pval.this <- (sum(tmp >= r.this - EPS) + 1)/(permutations + 1)
            pval <- c(pval, pval.this)
        }
    }
    if (is.null(colnames(X)))
        colnames(sol) <- paste("Dim", 1:ncol(sol), sep = "")
    else colnames(sol) <- colnames(X)
    names(r) <- names(P)
    if (!is.null(pval))
        names(pval) <- names(P)
    out <- list(centroids = sol, r = r, permutations = permutations,
                pvals = pval, var.id = var.id)
    out$control <- attr(permat, "control")
    class(out) <- "factorfit"
    out
}

`ordisegments` <-
    function (ord, groups, levels, replicates, order.by, display = "sites",
              col = 1, show.groups, label = FALSE, ...)
{
    pts <- scores(ord, display = display, ...)
    npoints <- nrow(pts)
    if (missing(groups))
        groups <- gl(levels, replicates, npoints)
    if (!missing(order.by)) {
        if (length(order.by) != nrow(pts))
            stop(gettextf("the length of order.by (%d) does not match the number of points (%d)",
                 length(order.by), nrow(pts)))
        ord <- order(order.by)
        pts <- pts[ord,]
        groups <- groups[ord]
    }
    if (!missing(show.groups)) {
        take <- groups %in% show.groups
        pts <- pts[take, , drop = FALSE]
        groups <- groups[take]
    }
    out <- seq(along = groups)
    inds <- names(table(groups))
    if (is.factor(col))
        col <- as.numeric(col)
    col <- rep(col, length=length(inds))
    names(col) <- inds
    ends <- names <- NULL
    for (is in inds) {
        gr <- out[groups == is]
        if (length(gr) > 1) {
            X <- pts[gr, , drop = FALSE]
            X0 <- X[-nrow(X), , drop = FALSE]
            X1 <- X[-1, , drop = FALSE]
            ordiArgAbsorber(X0[, 1], X0[, 2], X1[, 1], X1[, 2],
                            col = col[is], FUN = segments, ...)
            if (label) {
                ends <- rbind(ends, X[c(1, nrow(X)), ])
                names <- c(names, is, is)
            }
        }
    }
    if (label)
        ordiArgAbsorber(ends, labels = names, border = col, col = par("fg"),
                        FUN = ordilabel, ...)
    invisible()
}

### Centres and areas of plotted ellipses. The principal axes of the
### conic (oblique ellipse) are found from the eigenvalues of the
### covariance matrix.
`summary.ordiellipse` <-
    function(object, ...)
{
    cnts <- sapply(object, function(x) x$center)
    ## 2nd eigenvalue should be zero if points are on line (like two
    ## points), but sometimes it comes out negative, and area is NaN
    areas <- sapply(object,
                    function(x)
                        sqrt(pmax(0, det(x$cov))) * pi * x$scale^2)
    rbind(cnts, `Area` = areas)
}

## permatswap function
`permatswap` <-
function(m, method="quasiswap", fixedmar="both", shuffle="both", strata=NULL,
mtype="count", times=99, burnin = 0, thin = 1, ...)
{
    mtype <- match.arg(mtype, c("prab", "count"))
    fixedmar <- match.arg(fixedmar, c("rows", "columns", "both"))
    shuffle <- match.arg(shuffle, c("samp", "both"))
    count <- mtype == "count"
    m <- as.matrix(m)
    str <- if (is.null(strata))
        1 else as.integer(as.factor(strata)[drop = TRUE])
    levstr <- unique(str)
    nstr <- length(unique(str))
    if (!is.null(strata) && any(table(str) < 2))
        stop("strata should contain at least two observations")
    ## evaluating algo type
    if (count) {
        method <- match.arg(method, c("swap", "quasiswap", "swsh", "abuswap"))
        if (method == "swap") {
            warning("quantitative swap method may not yield random null models, use only to study its properties")
            isSeq <- TRUE
            if (fixedmar != "both")
                stop("if 'method=\"swap\"', 'fixedmar' must be \"both\"")
        } else {
            if (method == "abuswap") {
                if (fixedmar == "both")
                    stop("if 'method=\"abuswap\"', 'fixedmar' must not be \"both\"")
                direct <- if (fixedmar == "columns")
                    0 else 1
                isSeq <- TRUE
            } else {
                isSeq <- FALSE
                if (method != "swsh" && fixedmar != "both")
                    stop("'fixedmar' must be \"both\"")
            }
        }
        if (method %in% c("swap", "quasiswap"))
            ALGO <- paste(method, "count", sep="_")
        if (method == "abuswap")
            ALGO <- paste(method, substr(fixedmar, 1, 1), sep="_")
        if (method == "swsh") {
            if (fixedmar=="both")
                stop("if 'method=\"swsh\"', 'fixedmar' must not be \"both\"")
            ALGO <- if (fixedmar=="none") {
                paste(method, shuffle, sep="_")
            } else {
                paste(method, shuffle, substr(fixedmar, 1, 1), sep="_")
            }
        }
    } else {
        if (fixedmar != "both")
            stop("if 'mtype=\"prab\"', 'fixedmar' must be \"both\"")
        method <- match.arg(method, c("swap", "quasiswap", "tswap", "backtracking"))
        isSeq <- method != "quasiswap"
        ALGO <- method
    }
    if (is.null(strata)) {
        tmp <- simulate(nullmodel(m, ALGO), 
            nsim=times, burnin=burnin, thin=thin, ...)
        perm <- vector("list", times)
        for (i in seq_len(times))
            perm[[i]] <- tmp[,,i]
    } else {
        perm <- vector("list", times)
        tmp <- vector("list", length(unique(strata)))
        for (j in seq_len(nstr)) {
            tmp[[j]] <- simulate(nullmodel(m[strata==levstr[j],], ALGO), 
                nsim=times, burnin=burnin, thin=thin, ...)
        }
        for (i in seq_len(times)) {
            perm[[i]] <- array(0, dim(m))
            for (j in seq_len(nstr)) {
                perm[[i]][strata==levstr[j],] <- tmp[[j]][,,i]
            }
        }
    }
    if (mtype == "prab")
        m <- ifelse(m > 0, 1, 0)
    out <- list(call=match.call(), orig=m, perm=perm)
    attr(out, "mtype") <- mtype
    attr(out, "ptype") <- "swap"
    attr(out, "method") <- ALGO
    attr(out, "fixedmar") <- if (method == "swsh") "none" else fixedmar
    attr(out, "times") <- times
    attr(out, "shuffle") <- if (method == "swsh") shuffle else NA
    attr(out, "is.strat") <- !is.null(strata)
    attr(out, "strata") <- str
    attr(out, "burnin") <- burnin
    attr(out, "thin") <- thin
    class(out) <- c("permatswap", "permat")
    out
}

`MOStest` <-
    function(x, y, interval, ...)
{
    if (!missing(interval))
        interval <- sort(interval)
    x <- eval(x)
    m0 <- glm(y ~ x + I(x^2), ...)
    k <- coef(m0)
    isHump <- unname(k[3] < 0)
    hn <- if(isHump) "hump" else "pit"
    hump <- unname(-k[2]/2/k[3])
    if (missing(interval))
        p1 <- min(x)
    else
        p1 <- interval[1]
    if (missing(interval))
        p2 <- max(x)
    else
        p2 <- interval[2]
    test <- if (m0$family$family %in% c("binomial", "poisson")) "Chisq" else "F"
    tmp <- glm(y ~ I(x^2 - 2*x*p1), ...)
    ## Chisq test has one column less than F test: extract statistic
    ## and its P value
    statmin <- anova(tmp, m0, test = test)[2, (5:6) - (test == "Chisq")]
    tmp <- glm(y ~ I(x^2 - 2*x*p2), ...)
    statmax <- anova(tmp, m0, test = test)[2, (5:6) - (test == "Chisq")]
    comb <- 1 - (1-statmin[2])*(1-statmax[2])
    comb <- unlist(comb)
    stats <- rbind(statmin, statmax)
    rownames(stats) <- paste(hn, c("at min", "at max"))
    stats <- cbind("min/max" = c(p1,p2), stats)
    stats <- rbind(stats, "Combined" = c(NA, NA, comb))
    vec <- c(p1, p2, hump)
    names(vec) <- c("min", "max", hn)
    vec <- sort(vec)
    isBracketed <- names(vec)[2] == hn
    out <- list(isHump = isHump, isBracketed = isBracketed,
                hump = vec, family = family(m0), coefficients = stats,
                mod = m0)
    class(out) <- "MOStest"
    out
}

`rda.default` <-
    function (X, Y = NULL, Z = NULL, scale = FALSE, ...)
{
    ## Protect against grave misuse: some people have used
    ## dissimilarities instead of data
    if (inherits(X, "dist") || NCOL(X) == NROW(X) &&
        isTRUE(all.equal(X, t(X))))
        stop("function cannot be used with (dis)similarities")
    X <- as.matrix(X)
    if (!is.null(Y)) {
        if (is.data.frame(Y) || is.factor(Y))
            Y <- model.matrix(~ ., as.data.frame(Y))[,-1,drop=FALSE]
        Y <- as.matrix(Y)
    }
    if (!is.null(Z)) {
        if (is.data.frame(Z) || is.factor(Z))
            Z <- model.matrix(~ ., as.data.frame(Z))[,-1,drop=FALSE]
        Z <- as.matrix(Z)
    }

    sol <- ordConstrained(X, Y, Z, arg = scale, method = "rda")

    call <- match.call()
    call[[1]] <- as.name("rda")
    sol$call <- call
    inertia <- if (scale) "correlations" else "variance"
    sol <- c(sol,
             list("inertia" = inertia))
    ## package klaR also has rda(): add a warning text that will be
    ## printed if vegan::rda object is displayed with klaR:::print.rda
    sol$regularization <- "this is a vegan::rda result object"
    class(sol) <- c("rda", "cca")
    sol
}

`goodness.cca` <-
    function (object, choices, display = c("species", "sites"),
              model = c("CCA", "CA"),
              summarize = FALSE, addprevious = FALSE, ...)
{
    display <- match.arg(display)
    model <- match.arg(model)
    if (!inherits(object, "cca"))
        stop("can be used only with objects inheriting from 'cca'")
    ## See stressplot.capscale for a model to implement goodness() --
    ## this can be done, but we don't care to do it for now and we
    ## just disable this.
    if (inherits(object, "capscale"))
        stop("not implemented for 'capscale'")
    if (inherits(object, c("capscale", "dbrda")) && display == "species")
        stop(gettextf("cannot analyse species with '%s'", object$method))
    v <- sqrt(weights(object, display="species")) * object[[model]]$v
    if (is.null(v))
        stop(gettextf("model = '%s' does not exist", model))
    if (display == "sites")
        u <- sqrt(weights(object, display="sites")) * object[[model]]$u
    eig <- object[[model]]$eig
    ## Handle dbrda objects
    if (!inherits(object, "dbrda"))
        eig <- eig[eig > 0]
    if (inherits(object, "dbrda")) {
        u <- cbind(u, object[[model]][["imaginary.u"]])
        display <- "dbrda"
    }
    ## get the total variation
    All <- ordiYbar(object, "initial")
    if (is.null(All))
        stop("old style result object: update() your model")
    tot <- switch(display,
                  "species" = colSums(All^2),
                  "sites" = rowSums(All^2),
                  "dbrda" = diag(All))
    ## take only chosen axes within the component
    if (!missing(choices)) {
        choices <- choices[choices <= length(eig)]
        if (display != "dbrda")
            v <- v[, choices, drop = FALSE]
        if (display %in% c("sites", "dbrda"))
            u <- u[, choices, drop = FALSE]
        eig <- eig[choices]
    }
    if (addprevious) {
        if (!is.null(object$pCCA))
            prev <- ordiYbar(object, "pCCA")
        else
            prev <- 0
        if (model == "CA" && !is.null(object$CCA))
            prev <- prev + ordiYbar(object, "CCA")
    }
    if (display == "species") {
        if (length(eig) == 1)
            out <- v^2 * eig
        else
            out <- t(apply(v^2 %*% diag(eig, nrow=length(eig)), 1, cumsum))
        if (addprevious)
            out <- out + colSums(prev^2)
        dimnames(out) <- dimnames(v)
    } else if (display == "sites") {
        out <- matrix(0, nrow(u), ncol(u))
        if (addprevious)
            mat <- prev
        else
            mat <- 0
        for (i in seq_len(ncol(u))) {
            mat <- tcrossprod(u[,i], v[,i]) * sqrt(eig[i]) + mat
            out[,i] <- rowSums(mat^2)
        }
        dimnames(out) <- dimnames(u)
    } else { # dbrda
        out <- matrix(0, nrow(u), ncol(u))
        mat <- 0
        for (i in seq_len(ncol(u))) {
            mat <- u[,i]^2 * eig[i] + mat
            out[,i] <- mat
        }
        dimnames(out) <- dimnames(u)
    }
    out/tot
}

"rad.null" <-
    function(x,  family=poisson, ...)
{
    fam <- family(link="log")
    aicfun <- fam$aic
    dev.resids <- fam$dev.resids
    x <- as.rad(x)
    nsp <- length(x)
    wt <- rep(1, nsp)
    if (nsp > 0) { 
        fit <- rev(cumsum(1/nsp:1)/nsp) * sum(x)
        res <- dev.resids(x, fit, wt)
        deviance <- sum(res)
        aic <- aicfun(x, nsp, fit, wt, deviance)
    }
    else {
        fit <- NA
        aic <- NA
        res <- NA
        deviance <- NA
    }
    residuals <- x - fit
    rdf <- nsp
    names(fit) <- names(x)
    p <- NA
    names(p) <- "S"
    out <- list(model = "Brokenstick", family=fam, y = x, coefficients = p,
                fitted.values = fit, aic = aic, rank = 0, df.residual = rdf,
                deviance = deviance, residuals = residuals, prior.weights=wt)
    class(out) <- c("radline", "glm")
    out
}

`print.summary.meandist` <-
    function(x, ...)
{
    cat("\nMean distances:\n")
    tab <- rbind("within groups" = x$W,
                 "between groups" = x$B,
                 "overall" = x$D)
    colnames(tab) <- "Average"
    print(tab, ...)
    cat("\nSummary statistics:\n")
    tab <- rbind("MRPP A weights n" = x$A1,
                 "MRPP A weights n-1" = x$A2,
                 "MRPP A weights n(n-1)"= x$A3,
                 "Classification strength"=x$CS)
    colnames(tab) <- "Statistic"
    print(tab, ...)
    invisible(x)
}

`linestack` <-
    function (x, labels, cex = 0.8, side = "right", hoff = 2, air = 1.1,
              at = 0, add = FALSE, axis = FALSE, ...)
{
    if (length(at) > 1 || length(hoff) > 1 || length(air) > 1 || length(cex) > 1)
        stop("only one value accepted for arguments 'cex', 'hoff', 'air' and 'at'")
    x <- drop(x)
    n <- length(x)
    misslab <- missing(labels)
    if (misslab) {
        labels <- names(x)
    }
    if (!is.expression(labels) && !is.character(labels)) {
        labels <- as.character(labels)  # coerce to character only if not expressions
    }
    nlab <- length(labels)
    if (!misslab && n != nlab) {
        stop(gettextf(
            "wrong number of supplied 'labels: expected %d, got %d", n, nlab))
    }
    side <- match.arg(side, c("right", "left"))
    op <- par(xpd = TRUE)
    on.exit(par(op))
    ord <- order(x)
    x <- x[ord]
    labels <- labels[ord]
    pos <- numeric(n)
    if (!add) {
        plot(pos, x, type = "n", axes = FALSE, xlab = "", ylab = "", ...)
    }
    hoff <- hoff * strwidth("m")
    ht <- air * strheight(labels, cex = cex)
    mid <- (n + 1)%/%2
    pos[mid] <- x[mid]
    if (n > 1) {
        for (i in (mid + 1):n) {
            pos[i] <- max(x[i], pos[i - 1] + ht[i])
        }
    }
    if (n > 2) {
        for (i in (mid - 1):1) {
            pos[i] <- min(x[i], pos[i + 1] - ht[i])
        }
    }
    segments(at, x[1], at, x[n])
    if (side == "right") {
        text(at + hoff, pos, labels, pos = 4, cex = cex, offset = 0.2,
             ...)
        segments(at, x, at + hoff, pos)
    }
    else if (side == "left") {
        text(at - hoff, pos, labels, pos = 2, cex = cex, offset = 0.2,
             ...)
        segments(at, x, at - hoff, pos)
    }
    if (axis)
        axis(if (side == "right")
             2
        else 4, pos = at, las = 2)
    invisible(pos[order(ord)])
}

`lines.spantree` <-
    function (x, ord, display = "sites", col = 1, ...)
{
    ord <- scores(ord, display = display, ...)
    tree <- x$kid
    ## recycle colours and use a mixture of joined points for line segments
    col <- rep(col, length = nrow(ord))
    col <- col2rgb(col)/255
    ## average colour for pairs of points
    col <- rgb(t(col[,-1] + col[,tree])/2)
    if (x$n > 1)
        ordiArgAbsorber(ord[-1, 1], ord[-1, 2], ord[tree, 1], ord[tree, 2],
                        col = col,
                        FUN = segments, ...)
    invisible()
}

`print.permutest.cca` <-
    function (x, ...)
{
    EPS <- sqrt(.Machine$double.eps)
    cat("\nPermutation test for", x$method, "under", x$model, "model", "\n\n")
    cat(howHead(x$control), "\n")
    writeLines(strwrap(pasteCall(x$testcall, prefix = "Model:")))
    Pval <- (colSums(sweep(x$F.perm, 2, x$F.0 - EPS, ">=")) + 1)/(x$nperm + 1)
    cat("Permutation test for ")
    if (x$first)
        cat("first constrained eigenvalue\n")
    else if (length(x$df) <= 2)
        cat("all constrained eigenvalues\n")
    else
        cat("sequential contrasts\n")
    anotab <- data.frame(x$df, x$chi, c(x$F.0, NA), c(Pval, NA))
    colnames(anotab) <- c("Df", "Inertia", "F", "Pr(>F)")
    rownames(anotab) <- c(x$termlabels, "Residual")
    class(anotab) <- c("anova", "data.frame")
    print(anotab)
    invisible(x)
}

### Functions to extract permutation statististic or null model
### results from various vegan objects.

## extract items as 'statistic' and 'permutations'. Specific methods
## towards the end of this file

`permustats` <-
    function(x, ...)
{
    UseMethod("permustats")
}

## something like str()
`print.permustats` <-
    function(x, ...)
{
    print(str(x))
    invisible(x)
}

### modelled after print.oecosimu (should perhaps have oecosimu() args
### like 'alternative'

`summary.permustats` <-
    function(object, interval = 0.95, alternative, ...)
{
    TAB <- c("two.sided", "greater", "less")
    if (missing(alternative))
        alt <- TAB[match(object$alternative, TAB)]
    else
        alt <- match.arg(alternative, TAB, several.ok = TRUE)
    if (any(is.na(alt)))
        stop("alternative missing")
    nstat <- length(object$statistic)
    nalt <- length(alt)
    ## Replicate alternative to length of statistic
    if ((nalt < nstat) && identical(nalt, 1L)) {
        alt <- rep(alt, length.out = nstat)
    }
    compint <- (1 - interval) / 2
    PROBS <- list(two.sided = c(compint, 0.5, interval + compint),
                  greater = c(NA, 0.5, interval),
                  less = c(1 - interval, 0.5, NA))
    probs <- PROBS[alt]
    ## take care that permutations are in a column matrix
    permutations <- as.matrix(object$permutations)
    object$means <- colMeans(permutations)
    sd <- apply(permutations, 2, sd)
    object$z <-
        (object$statistic - object$means)/sd
    qFun <- function(i, sim, probs) {
        quantile(sim[, i], probs = probs[[i]], na.rm = TRUE)
    }
    object$quantile <- lapply(seq_along(probs), qFun, sim = permutations, probs = probs)
    object$quantile <- do.call("rbind", object$quantile)
    dimnames(object$quantile) <- list(NULL, c("lower", "median", "upper"))
    object$interval <- interval
    ## P-values
    if (is.integer(object$statistic) && is.integer(permutations)) {
        pless <- rowSums(object$statistic >= t(permutations), na.rm = TRUE)
        pmore <- rowSums(object$statistic <= t(permutations), na.rm = TRUE)
    } else {
        EPS <- sqrt(.Machine$double.eps)
        pless <- rowSums(object$statistic + EPS >= t(permutations),
                         na.rm = TRUE)
        pmore <- rowSums(object$statistic - EPS <= t(permutations),
                         na.rm = TRUE)
    }
    nsimul <- nrow(permutations)
    if (any(is.na(permutations))) {
        warning("some simulated values were NA and were removed")
        nsimul <- nsimul - colSums(is.na(permutations))
    }
    p <- rep(NA, length(object$statistic))
    for(i in seq_along(p))
        p[i] <- switch(alt[i],
                       two.sided = 2*pmin(pless[i], pmore[i]),
                       greater = pmore[i],
                       less = pless[i])
    object$p <- pmin(1, (p + 1)/(nsimul + 1))
    ## out
    class(object) <- "summary.permustats"
    object
}

`print.summary.permustats` <- function(x, ...) {
    m <- cbind("statistic" = x$statistic,
               "SES" = x$z,
               "mean" = x$means,
               x$quantile,
               "Pr(perm)" = x$p)
    cat("\n")
    printCoefmat(m, tst.ind = 1:(ncol(m)-1), na.print = "", ...)
    writeLines(strwrap(paste0("(Interval (Upper - Lower) = ", x$interval, ")", sep = ""),
                       initial = "\n"))
    invisible(x)
}

### combine permustats objects. Function checks that statistic field
### is equal (name, value) before proceeding, sees if the alternative
### is equal, and then combines permutations.

`c.permustats` <-
    function(..., recursive = FALSE)
{
    mods <- list(...)
    ## check stats
    stats <- lapply(mods, function(z) z$statistic)
    if (!all(sapply(stats[-1], function(z) identical(stats[[1]], z))))
        stop("statistics are not equal")
    stats <- stats[[1]]
    ## check alternative
    alt <- lapply(mods, function(z) z$alternative)
    if (all(sapply(alt[-1], function(z) identical(alt[[1]], z))))
        alt <- alt[[1]]
    else
        alt <- NA
    ## combine permutations
    p <- do.call(rbind, lapply(mods, function(z) z$permutations))
    ## return permustats
    structure(list(statistic = stats,
                   permutations = p,
                   alternative = alt),
              class = "permustats")
}

### densityplot

`densityplot.permustats` <-
    function(x, data, xlab = "Permutations", ...)
{
    obs <- x$statistic
    sim <- rbind(x$statistic, as.matrix(x$permutations))
    nm <- names(obs)[col(sim)]
    densityplot( ~ as.vector(sim) | factor(nm, levels = unique(nm)),
                xlab = xlab,
                panel = function(x, ...) {
                    panel.densityplot(x, ...)
                    panel.abline(v = obs[panel.number()], ...)
                },
                ...)
}

### simple density: normally densityplot should be used (or I suggest
### so), but we also offer basic density. This can be either with or
### without observed statistic.

`density.permustats` <-
    function(x, observed = TRUE, ...)
{
    ## only works with statistic
    if (length(x$statistic) > 1)
        stop(gettextf("only works with one statistic: you got %d",
                      length(x$statistic)))
    p <- x$permutations
    if (observed)
        p <- c(x$statistic, p)
    out <- density(p)
    out$call <- match.call()
    out$call[[1]] <- as.name("density")
    out
}

### QQ-plot against Guaussian distribution

`qqnorm.permustats` <-
    function(y, observed = TRUE, ...)
{
    ## only works with statistic
    if (length(y$statistic) > 1)
        stop(gettextf("only works with one statistic: you got %d",
                      length(y$statistic)))
    p <- y$permutations
    if (observed)
        p <- c(y$statistic, p)
    q <- qqnorm(p, ...)
    if (observed)
        abline(h = y$statistic, ...)
    invisible(q)
}

`qqmath.permustats` <-
    function(x, data, observed = TRUE, sd.scale = FALSE,
             ylab = "Permutations", ...)
{
    ## sd.scale: standardize before use
    if (sd.scale) {
        x$permutations <- scale(x$permutations)
        x$statistic <- (x$statistic - attr(x$permutations, "scaled:center"))/
            attr(x$permutations, "scaled:scale")
    }
    obs <- x$statistic
    if (observed)
        sim <- rbind(x$statistic, as.matrix(x$permutations))
    else
        sim <- as.matrix(x$permutations)
    nm <- names(obs)[col(sim)]
    qqmath( ~ as.vector(sim) | factor(nm, levels = unique(nm)),
                ylab = ylab,
                panel = function(x, ...) {
                    panel.qqmath(x, ...)
                    if (observed)
                        panel.abline(h = obs[panel.number()], ...)
                },
                ...)
}

## boxplot for (standardized) effect size: permutations are centred to
## the value of statistic, and optionally standardized to equal sd
## w.r.t. to column mean (not to the statistics). This shows the
## effect size, or the deviation from the observed statistic (= 0).

`boxplot.permustats` <-
    function(x, scale = FALSE, names, ...)
{
    p <- x$permutations
    if (isTRUE(scale))
        scale <- apply(p, 2, sd, na.rm = TRUE)
    p <- scale(p, center = x$statistic, scale = scale)
    if (missing(names))
        names <- attr(x$statistic, "names")
    boxplot(p, names = names, ...)
}

###
### specific methods to extract permustats
###

`permustats.anosim` <-
    function(x, ...)
{
    structure(list(
        "statistic" = structure(x$statistic, names = "R"),
        "permutations" = x$perm,
        "alternative" = "greater"),
              class = "permustats")
}

`permustats.adonis` <-
    function(x, ...)
{
    tab <- x$aov.tab
    k <- !is.na(tab$F.Model)
    structure(list(
        "statistic" = structure(tab$F.Model[k], names = rownames(tab)[k]),
        "permutations" = x$f.perms,
        "alternative" = "greater"),
              class = "permustats")
}

`permustats.mantel` <-
    function(x, ...)
{
    structure(list(
        "statistic" = structure(x$statistic, names = "r"),
        "permutations" = x$perm,
        "alternative" = "greater"),
              class = "permustats")
}

`permustats.mrpp` <-
    function(x, ...)
{
    structure(list(
        "statistic" = structure(x$delta, names = "delta"),
        "permutations" = x$boot.deltas,
        "alternative" = "less"),
              class = "permustats")
}

`permustats.oecosimu` <-
    function(x, ...)
{
    structure(list(
        "statistic" = x$oecosimu$statistic,
        "permutations" = t(x$oecosimu$simulated),
        "alternative" = x$oecosimu$alternative),
              class = "permustats")
}

`permustats.ordiareatest` <-
    function(x, ...)
{
    structure(list(
        "statistic" = x$areas,
        "permutations" = t(x$permutations),
        "alternative" = "less"),
              class = "permustats")
}

`permustats.permutest.cca` <-
    function(x, ...)
{
    structure(list(
        "statistic" = structure(x$F.0, names = x$termlabels),
        "permutations" = x$F.perm,
        "alternative" = "greater"),
              class = "permustats")
}

`permustats.protest` <-
    function(x, ...)
{
    structure(list(
        "statistic" = structure(x$t0, names = "r"),
        "permutations" = x$t,
        "alternative" = "greater"),
              class = "permustats")
}

### the following do not return permutation data
`permustats.CCorA` <-
    function(x, ...)
{
    stop("no permutation data available")
}

`permustats.envfit` <-
    function(x, ...)
{
    stop("no permutation data available")
}

`permustats.factorfit` <-
    function(x, ...)
{
    stop("no permutation data available")
}

`permustats.vectorfit` <-
    function(x, ...)
{
    stop("no permutation data available")
}

`permustats.mso` <-
    function(x, ...)
{
    stop("no permutation data available")
}

`permustats.permutest.betadisper` <-
    function(x, ...)
{
    ntypes <- NCOL(x$perm)
    alt <- if (ntypes > 1) {
        c("greater", rep("two.sided", ntypes - 1))
    } else {
        "greater"
    }
    structure(list("statistic" = x$statistic,
                   "permutations" = x$perm,
                   "alternative" = alt),
              class = "permustats")
}

`permustats.anova.cca` <-
    function(x, ...)
{
    if (is.null(attr(x, "F.perm")))
        stop("no permutation data available")
    F.perm <- attr(x, "F.perm")
    k <- !is.na(x$F)
    F.0 <- x$F[k]
    structure(list(
       "statistic" = structure(F.0, names = rownames(x)[k]),
       "permutations" = F.perm,
       "alternative" = "greater"),
       class = "permustats")
}

`print.summary.decorana` <-
function (x, head=NA, tail=head, ...) 
{
    digits <- x$digits
    hcat <- function(x, head=head, tail=tail, ...) {
    	   if(!is.na(head) && !is.na(tail) && head + tail + 4 < nrow(x))
    	        x <- rbind(head(x, n=head), "...." = NA, tail(x, n=tail))
    	   printCoefmat(x,  na.print="", ...)
    	}
    if (!is.null(x$spec.scores)) {
        cat("Species scores:\n\n")
        TABLE <- cbind(x$spec.scores, Weights = x$spec.priorweights, 
            Totals = x$spec.totals)
        hcat(TABLE, head=head, tail=tail, digits = digits,  ...)
        cat("\n")
    }
    if (!is.null(x$site.scores)) {
        cat("Site scores:\n\n")
        TABLE <- cbind(x$site.scores, Totals = x$site.totals)
        hcat(TABLE, head=head, tail=tail, digits = digits,  ...)
        cat("\n")
    }
    invisible(x)
}

`plot.ordisurf` <- function(x, what = c("contour","persp","gam"),
                            add = FALSE, bubble = FALSE, col = "red", cex = 1,
                            nlevels = 10, levels, labcex = 0.6,
                            lwd.cl = par("lwd"), ...) {
    what <- match.arg(what)
    y <- x$model$y
    x1 <- x$model$x1
    x2 <- x$model$x2
    X <- x$grid$x
    Y <- x$grid$y
    Z <- x$grid$z
    force(col)
    force(cex)
    if(isTRUE(all.equal(what, "contour"))) {
        if(!add) {
            if(bubble) {
                if (is.numeric(bubble))
                    cex <- bubble
                cex <- (y -  min(y))/diff(range(y)) * (cex-0.4) + 0.4
            }
            plot(x1, x2, asp = 1, cex = cex, ...)
        }
        if (missing(levels))
            levels <- pretty(range(x$grid$z, finite = TRUE), nlevels)
        contour(X, Y, Z, col = col, add = TRUE,
                levels = levels, labcex = labcex,
                drawlabels = !is.null(labcex) && labcex > 0,
                lwd = lwd.cl)
    } else if(isTRUE(all.equal(what, "persp"))) {
        persp(X, Y, Z, col = col, cex = cex, ...)
    } else {
        class(x) <- class(x)[-1]
        plot(x, ...) ##col = col, cex = cex, ...)
        class(x) <- c("ordisurf", class(x))
    }
    invisible(x)
}

`inertcomp` <-
    function (object, display = c("species", "sites"),
              unity = FALSE, proportional = FALSE)
{
    display <- match.arg(display)
    ## unity and proportional are conflicting arguments
    if (unity && proportional)
        stop("arguments 'unity' and 'proportional' cannot be both TRUE")
    if (!inherits(object, "cca"))
        stop("can be used only with objects inheriting from 'cca'")
    if (inherits(object, c("capscale", "dbrda")) && display == "species")
        stop(gettextf("cannot analyse species with '%s'", object$method))
    if (inherits(object, "dbrda")) {
        display <- "dbrda"
    }
    ## function to get the eigenvalues
    getComps <- function(x, display) {
        if(!is.null(x))
            switch(display,
                   "species" = colSums(x^2),
                   "sites" = rowSums(x^2),
                   "dbrda" = diag(x)
                   )
    }
    pCCA <- ordiYbar(object, "pCCA")
    CCA <- ordiYbar(object, "CCA")
    CA <- ordiYbar(object, "CA")
    tot <- ordiYbar(object, "initial")
    out <- cbind("pCCA" = getComps(pCCA, display),
                 "CCA" = getComps(CCA, display),
                 "CA" = getComps(CA, display))
    if (unity) ## each column sums to 1
        out <- sweep(out, 2, colSums(out), "/")
    if (proportional)
        out <- out/getComps(tot, display)
    out
}

`hiersimu.formula` <-
    function(formula, data, FUN, location = c("mean", "median"),
             relative = FALSE, drop.highest = FALSE, nsimul=99,
             method = "r2dtable", ...)
{
    ## evaluate formula
    if (missing(data))
        data <- parent.frame()
    tmp <- hierParseFormula(formula, data)
    ## run simulations
    sim <- hiersimu.default(tmp$lhs, tmp$rhs, FUN = FUN, location = location,
                            relative = relative, drop.highest = drop.highest,
                            nsimul = nsimul, method = method, ...)
    call <- match.call()
    call[[1]] <- as.name("hiersimu")
    attr(sim, "call") <- call
    sim
}

`print.anosim` <-
    function (x, digits = max(3, getOption("digits") - 3), ...) 
{
    cat("\nCall:\n")
    cat(deparse(x$call), "\n")
    cat("Dissimilarity:", x$dissimilarity,"\n\n")
    cat("ANOSIM statistic R: ")
    cat(formatC(x$statistic, digits = digits), "\n")
    nperm <- x$permutations
    if (nperm) {
        cat("      Significance:", format.pval(x$signif), 
            "\n\n")
        cat(howHead(x$control))
    }
    cat("\n")
    invisible(x)
}

`ordiParseFormula` <-
function (formula, data, xlev = NULL, na.action = na.fail,
          subset = NULL, X)
{
    if (missing(data))
        data <- environment(formula)
    Terms <- terms(formula, "Condition", data = data)
    flapart <- fla <- formula <- formula(Terms, width.cutoff = 500)
    ## distance-based methods (capscale, dbrda) evaluate specdata (LHS
    ## in formula) within their code, and the handling in
    ## ordiParseFormula is redundand and can be expensive
    if (missing(X)) {
        specdata <- formula[[2]]
        X <- eval(specdata, environment(formula), enclos=globalenv())
    }
    X <- as.matrix(X)
    indPartial <- attr(Terms, "specials")$Condition
    zmf <- ymf <- Y <- Z <- NULL
    formula[[2]] <- NULL
    if (!is.null(indPartial)) {
        partterm <- attr(Terms, "variables")[1 + indPartial]
        ## paste() needed to combine >500 character deparsed Conditions
        Pterm <- sapply(partterm, function(x)
            paste(deparse(x[[2]], width.cutoff=500, backtick = TRUE), collapse=" "))
        Pterm <- paste(Pterm, collapse = "+")
        P.formula <- as.formula(paste("~", Pterm), env = environment(formula))
        zlev <- xlev[names(xlev) %in% Pterm]
        zmf <- if (inherits(data, "environment"))
            eval(substitute(
                model.frame(P.formula, na.action = na.pass, xlev = zlev)),
                 envir = data, enclos = .GlobalEnv)
        else
            model.frame(P.formula, data, na.action = na.pass, xlev = zlev)
        partterm <- sapply(partterm, function(x)
            paste(deparse(x, width.cutoff=500, backtick = TRUE), collapse = " "))
        formula <- update(formula, paste("~.-", paste(partterm,
            collapse = "-")))
        flapart <- update(formula, paste(" ~ . +", Pterm))
    }
    if (formula[[2]] == "1" || formula[[2]] == "0")
        Y <- NULL
    else {
        if (exists("Pterm"))
            xlev <- xlev[!(names(xlev) %in% Pterm)]

        ymf <- if (inherits(data, "environment"))
            eval(substitute(
                model.frame(formula, na.action = na.pass, xlev = xlev)),
                 envir=data, enclos=.GlobalEnv)
        else
            model.frame(formula, data, na.action = na.pass, xlev = xlev)
    }
    ## Combine condition an constrain data frames
    if (!is.null(zmf)) {
        ncond <- NCOL(zmf)
        if (!is.null(ymf))
            mf <- cbind(zmf, ymf)
        else
            mf <- zmf
    } else {
        ncond <- 0
        mf <- ymf
    }
    ## Select a subset of data and species
    if (!is.null(subset)) {
        subset <- eval(subset,
                       if (inherits(data, "data.frame")) cbind(data, X)
                       else as.data.frame(X),
                       parent.frame(2))
        X <- X[subset, , drop = FALSE]
        if (NROW(mf) > 0)
            mf <- mf[subset, , drop = FALSE]
    }
    ## Get na.action attribute, remove NA and drop unused levels
    if (NROW(mf) > 0) {
        ## We need to re-construct model.frame and its formula, and
        ## for this names with functions must be back-quoted
        ## (`poly(A1, 2)`, e.g.) with the curren scoping to find the
        ## variable (A1) later. This happened with formula(mf) in R <
        ## 3.6.0, but now we must be explicit.
        mf <- model.frame(as.formula(paste("~",
              paste(sapply(names(mf), "as.name"), collapse="+"))),
              mf, xlev = xlev,
              na.action = na.action, drop.unused.levels = TRUE)
        nas <- attr(mf, "na.action")
        ## Check if there are one-level factors after subset and na.action
        for (i in 1:ncol(mf))
            if (is.factor(mf[[i]]) && length(levels(mf[[i]])) <= 1)
                levels(mf[[i]]) <- c(levels(mf[[i]]), ".ThisVarHasOnly1Level")
    } else {
        nas <- NULL
    }
    ## Check and remove NA in dependent data
    if (!is.null(nas)) {
        excluded <- X[nas, , drop = FALSE]
        X <- X[-nas,, drop=FALSE]
    } else {
        excluded <-  NULL
    }
    if (ncond > 0) {
        Z <- model.matrix(P.formula, mf)
        if (any(colnames(Z) == "(Intercept)"))
            Z <- Z[, -which(colnames(Z) == "(Intercept)"), drop = FALSE]
    }
    if (NROW(mf) > 0) {
        Y <- model.matrix(formula, mf)
        ## save assign attribute
        assign <- attr(Y, "assign")
        assign <- assign[assign > 0]
        if (any(colnames(Y) == "(Intercept)"))
            Y <- Y[, -which(colnames(Y) == "(Intercept)"), drop = FALSE]
        if (NCOL(Y) == 0)
            Y <- NULL
        else
            attr(Y, "assign") <- assign
    }
    X <- as.matrix(X)
    rownames(X) <- rownames(X, do.NULL = FALSE)
    colnames(X) <- colnames(X, do.NULL = FALSE)
    if (!is.null(Y)) {
        rownames(Y) <- rownames(Y, do.NULL = FALSE)
        colnames(Y) <- colnames(Y, do.NULL = FALSE)
    }
    if (!is.null(Z)) {
        rownames(Z) <- rownames(Z, do.NULL = FALSE)
        colnames(Z) <- colnames(Z, do.NULL = FALSE)
    }
    list(X = X, Y = Y, Z = Z, terms = terms(fla, width.cutoff = 500),
         terms.expand = terms(flapart, width.cutoff = 500), modelframe = mf,
         subset = subset, na.action = nas, excluded = excluded)
}

simulate.nullmodel <-
function(object, nsim=1, seed = NULL, burnin=0, thin=1, ...)
{
    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) 
        runif(1)
    if (is.null(seed)) 
        RNGstate <- get(".Random.seed", envir = .GlobalEnv)
    else {
        R.seed <- get(".Random.seed", envir = .GlobalEnv)
        set.seed(seed)
        RNGstate <- structure(seed, kind = as.list(RNGkind()))
        on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
    }
    if (nsim < 1)
        stop("'nsim' must be at least 1")
    m <- object$data
    if (object$commsim$isSeq) {
        ## here is burnin, see update method
        if (burnin > 0)
            object <- update(object, burnin, ...)
        x <- object$state
    } else {
        x <- m
        ## non-sequential models have no burnin -- but they may have
        ## thinning: set burnin=0, but leave thin like user set it.
        burnin <- 0L
    }
    perm <- object$commsim$fun(x=x,
        n=as.integer(nsim),
        nr=object$nrow,
        nc=object$ncol,
        rs=object$rowSums,
        cs=object$colSums,
        rf=object$rowFreq,
        cf=object$colFreq,
        s=object$totalSum,
        fill=object$fill,
        thin=as.integer(thin), ...)
    if (object$commsim$isSeq) {
        Start <- object$iter + thin
        End <- object$iter + nsim * thin
        ## sequence can overflow integer
        if (Start <= .Machine$integer.max)
            Start <- as.integer(Start)
        if (End <= .Machine$integer.max)
            End <- as.integer(End)
        state <- perm[,,nsim]
        storage.mode(state) <- object$commsim$mode
        assign("state", state, envir=object)
        assign("iter", End, envir=object)
    } else {
        Start <- 1L
        End <- as.integer(nsim)
    }
    attr(perm, "data") <- m
    attr(perm, "seed") <- RNGstate
    attr(perm, "method") <- object$commsim$method
    attr(perm, "binary") <- object$commsim$binary
    attr(perm, "isSeq") <- object$commsim$isSeq
    attr(perm, "mode") <- object$commsim$mode
    attr(perm, "start") <- Start
    attr(perm, "end") <- End
    attr(perm, "thin") <- as.integer(thin)
    class(perm) <- c("simmat", "array")
    dimnames(perm) <- list(rownames(m), colnames(m),
        paste("sim", seq_len(nsim), sep = "_"))
    perm
}

"rad.preempt" <-
    function (x, family = poisson, ...) 
{
    canfun <- function(p, x, ...) {
        if (length(x) <= 1)
            return(0)
        p <- plogis(p)
        if (p == 1) 
            p <- 1 - .Machine$double.eps
        fv <- linkinv(logJ + log(p) + log(1 - p) * rnk)
        n <- rep(1, length(fv))
        dev <- sum(dev.resids(x, fv, wt))
        aicfun(x, n, fv, wt, dev)/2
    }
    fam <- family(link = "log")
    aicfun <- fam$aic
    linkinv <- fam$linkinv
    dev.resids <- fam$dev.resids
    x <- as.rad(x)
    nsp <- length(x)
    rnk <- seq(along = x) - 1
    wt <- rep(1, length(x))
    logJ <- log(sum(x))
    p <- qlogis(0.1)
    canon <- try(nlm(canfun, p = p, x = x, rnk = rnk, logJ = logJ, 
                     wt = wt, hessian = TRUE, ...))
    if (inherits(canon, "try-error")) {
        aic <- rdf <- deviance <- NA
        p <- rep(NA, 1)
        fit <- residuals <- prior.weights <- rep(NA, length(x))
    } else {
        if (nsp > 1) {
            p <- plogis(canon$estimate)
            fit <- exp(logJ + log(p) + log(1 - p) * rnk)
        } else {
            p <- if (nsp > 0) 1 else NA
            fit <- x
        }
        res <- dev.resids(x, fit, wt)
        deviance <- sum(res)
        residuals <- x - fit
        if (nsp > 0)
            aic <- aicfun(x, rep(1, length(x)), fit, wt, deviance) + 2
        else
            aic <- NA
        rdf <- length(x) - 1
    }
    names(fit) <- names(x)
    names(p) <- c("alpha")
    out <- list(model = "Preemption", family = fam, y = x, coefficients = p, 
                fitted.values = fit, aic = aic, rank = 1, df.residual = rdf, 
                deviance = deviance, residuals = residuals, prior.weights = wt)
    class(out) <- c("radline", "glm")
    out
}

### summary methods extracts dispweight attributes, and prints a table
### of dispersion statistics

`summary.dispweight`  <-
    function(object, ...)
{
    x <- attributes(object)
    class(x) <- "summary.dispweight"
    x
}

`print.summary.dispweight` <-
    function(x, ...)
{
    tab <- with(x, cbind(D, weights, df, p))
    colnames(tab) <- c("Dispersion", "Weight", "Df", "Pr(Disp.)")
    printCoefmat(tab, cs.ind = NA, ...)
    if (!is.na(x$nsimul))
        cat(sprintf("Based on %d simulations on '%s' nullmodel\n",
                     x$nsimul, x$nullmodel))
    invisible(x)
}

"plot.decorana" <-
    function (x, choices = c(1, 2), origin = TRUE, display = c("both", 
                                                   "sites", "species", "none"), cex = 0.8, cols = c(1, 2), type, xlim, ylim, 
              ...) 
{
    display <- match.arg(display)
    sites <- x$rproj
    specs <- x$cproj
    if (missing(type)) {
        nitlimit <- 120
        nit <- 0
        if (display == "sites" || display == "both") 
            nit <- nit + nrow(sites)
        if (display == "species" || display == "both") 
            nit <- nit + nrow(specs)
        if (nit > nitlimit) 
            type <- "points"
        else type <- "text"
    }
    else type <- match.arg(type, c("text", "points", "none"))
    if (origin) {
        sites <- sweep(x$rproj, 2, x$origin, "-")
        specs <- sweep(x$cproj, 2, x$origin, "-")
    }
    sitnam <- rownames(x$rproj)
    spenam <- rownames(x$cproj)
    sites <- sites[, choices]
    specs <- specs[, choices]
    ## Use linestack if only one dim was specified (and exit)
    if (NCOL(sites) == 1 && NCOL(specs) == 1) {
        pl <- linestack(sites,
                        ylim = range(c(sites, specs), na.rm = TRUE), ...)
        linestack(specs, side = "left", add = TRUE, ...)
        return(invisible(pl))
    }
    sp.x <- range(specs[, 1])
    sp.y <- range(specs[, 2])
    st.x <- range(sites[, 1])
    st.y <- range(sites[, 2])
    switch(display, both = {
        if (missing(xlim)) xlim <- range(sp.x, st.x)
        if (missing(ylim)) ylim <- range(sp.y, st.y)
    }, sites = {
        if (missing(xlim)) xlim <- st.x
        if (missing(ylim)) ylim <- st.y
    }, species = {
        if (missing(xlim)) xlim <- sp.x
        if (missing(ylim)) ylim <- sp.y
    }, none = {
        if (missing(xlim)) xlim <- range(sp.x, st.x)
        if (missing(ylim)) ylim <- range(sp.y, st.y)
    })
    plot(sites, type = "n", xlim = xlim, ylim = ylim, asp = 1, 
         ...)
    if (origin) {
        abline(h = 0, lty = 3)
        abline(v = 0, lty = 3)
    }
    else {
        abline(h = x$origin[choices[2]], lty = 3)
        abline(v = x$origin[choices[1]], lty = 3)
    }
    if (type != "none" && (display == "both" || display == "sites")) {
        if (type == "text" && !is.null(sitnam)) 
            text(sites, sitnam, cex = cex, col = cols[1])
        else points(sites, cex = cex, col = cols[1])
    }
    if (type != "none" && (display == "both" || display == "species")) {
        if (type == "text" && !is.null(spenam)) 
            text(specs, spenam, cex = cex, col = cols[2])
        else points(specs, pch = "+", cex = cex, col = cols[2])
    }
    out <- list(sites = sites, species = specs)
    class(out) <- "ordiplot"
    invisible(out)
}

`prepanel.ordi3d` <-
    function(xlim = xlim, ylim = ylim, zlim = zlim, aspect = c(1,1),  ...)
{
    aspect = c(diff(ylim)/diff(xlim), diff(zlim)/diff(xlim))
    prepanel.default.cloud(xlim = xlim, ylim = ylim, zlim = zlim, aspect = aspect, ...)
}

"plot.humpfit" <-
    function(x, xlab="Biomass", ylab="Species Richness", lwd=2, l.col="blue",
             p.col = 1, type="b", ...)
{
    plot(x$x, x$y, xlab = xlab, ylab = ylab, type="n", ...)
    if (type == "b" || type == "p")
        points(x, col = p.col, ...)
    if (type == "b" || type == "l")
        lines(x, lwd = lwd, col = l.col, ...)
    invisible()
}

`summary.meandist` <-
    function(object, ...)
{
    n <- attr(object, "n")
    wmat <- n %o% n
    diag(wmat) <- diag(wmat) - n
    ## mean distances within, between groups and in total
    W <- weighted.mean(diag(object), w = diag(wmat), na.rm = TRUE)
    B <- weighted.mean(object[lower.tri(object)],
                       w = wmat[lower.tri(wmat)], na.rm = TRUE)
    D <- weighted.mean(object, w = wmat, na.rm = TRUE)
    ## Variants of MRPP statistics
    A1 <- weighted.mean(diag(object), w = n, na.rm = TRUE)
    A2 <- weighted.mean(diag(object), w = n - 1, na.rm = TRUE)
    A3 <- weighted.mean(diag(object), w = n * (n - 1), na.rm = TRUE)
    ##
    out <- list(W = W, B = B, D = D, CS = B-A1,
                A1 = 1 - A1/D, A2 = 1 - A2/D, A3 = 1 - A3/D)
    class(out) <- "summary.meandist"
    out
}

"summary.decorana" <-
    function (object, digits = 3, origin = TRUE, display = c("both", 
                                                 "species", "sites", "none"), ...) 
{
    display <- match.arg(display)
    print(object)
    if (origin) {
        object$cproj <- sweep(object$cproj, 2, object$origin, 
                              "-")
        object$rproj <- sweep(object$rproj, 2, object$origin, 
                              "-")
    }
    tmp <- list()
    if (display == "both" || display == "species") {
        tmp$spec.scores <- object$cproj
        tmp$spec.priorweights <- object$v
        tmp$spec.totals <- object$adotj
    }
    if (display == "both" || display == "sites") {
        tmp$site.scores <- object$rproj
        tmp$site.totals <- object$aidot
    }
    tmp$digits <- digits
    class(tmp) <- "summary.decorana"
    tmp
}

`weights.rda` <-
    function (object, display = "sites", ...)
{
    display <- match.arg(display, c("sites", "species", "lc",
                                    "wa"))
    if (display %in% c("sites", "lc", "wa")) {
        n <- nobs(object)
        if (!is.null(object$na.action) &&
            inherits(object$na.action, "exclude"))
            n <- n + length(object$na.action)
    }
    else n <- length(object$colsum)
    rep(1, n)
}

`ordistep` <-
    function(object, scope, direction =c("both", "backward", "forward"),
             Pin = 0.05, Pout = 0.1, permutations = how(nperm = 199),
             steps=50, trace = TRUE, ...)
{
    if (!inherits(object, "cca"))
        stop("function can be only used with 'cca' and related objects")
    if (is.null(object$terms))
        stop("ordination model must be fitted using formula")
    ## handling 'direction' and 'scope' directly copied from
    ## stats::step()
    md <- missing(direction)
    direction <- match.arg(direction)
    backward <- direction == "both" | direction == "backward"
    forward <- direction == "both" | direction == "forward"
    ffac <- attr(terms(object), "factors")
    if (missing(scope)) {
        fdrop <- numeric(0)
        fadd <- ffac
        if (md)
            forward <- FALSE
    }
    else {
        if (is.list(scope) && (!is.null(scope$lower) || !is.null(scope$upper))) {
            fdrop <- if (!is.null(fdrop <- scope$lower))
                attr(terms(update.formula(object, fdrop)), "factors")
            else numeric(0)
            fadd <- if (!is.null(fadd <- scope$upper))
                attr(terms(update.formula(object, fadd)), "factors")
        }
        else {
            fadd <- if (!is.null(fadd <- scope))
                        attr(terms(update.formula(object, scope)), "factors")
            if (forward)
                fdrop <- attr(terms(object), "factor")
            else
                fdrop <- numeric(0L)
        }
    }
    scope <- factor.scope(ffac, list(add = fadd, drop = fdrop))
    ## 'anotab' collects the changes into 'anova' object in the output
    anotab <- NULL
    if (trace) {
        cat("\n")
        cat(pasteCall(formula(object), prefix = "Start:"))
    }
    for (i in 1:steps){
        change <- NULL
        ## Consider dropping
        if (backward && length(scope$drop)) {
            aod <- drop1(object, scope = scope$drop, test="perm",
                         permutations = permutations,
                         alpha = Pout, trace = trace, ...)
            aod <- aod[-1,]
            o <- order(-aod[,4], aod[,2])
            aod <- aod[o,]
            rownames(aod) <- paste("-", rownames(aod), sep = " ")
            if (trace) {
                cat("\n")
                print(aod)
            }
            if (is.na(aod[1,4]) || aod[1,4] > Pout) {
                anotab <- rbind(anotab, aod[1,])
                change <- rownames(aod)[1]
                object <- eval.parent(update(object, paste("~  .", change)))
                scope <- factor.scope(attr(terms(object), "factors"),
                                      list(add = fadd, drop = fdrop))
                if (trace) {
                    cat("\n")
                    cat(pasteCall(formula(object), prefix = "Step:"))
                }
            }
        }
        ## Consider adding
        if (forward && length(scope$add)) {
            aod <- add1(object, scope = scope$add, test = "perm",
                        permutations = permutations,
                        alpha = Pin, trace = trace, ...)
            aod <- aod[-1,]
            o <- order(aod[,4], aod[,2])
            aod <- aod[o,]
            rownames(aod) <- paste("+", rownames(aod), sep = " ")
            if (trace) {
                cat("\n")
                print(aod)
            }
            if (!is.na(aod[1,4]) && aod[1,4] <= Pin) {
                anotab <- rbind(anotab, aod[1,])
                change <- rownames(aod)[1]
                object <- eval.parent(update(object, paste( "~  .",change)))
                scope <- factor.scope(attr(terms(object), "factors"),
                                      list(add = fadd, drop = fdrop))
                if (trace) {
                    cat("\n")
                    cat(pasteCall(formula(object), prefix="Step:"))
                }
            }
        }
        ## No drop, no add: done
        if (is.null(change))
            break
    }
    cat("\n")
    object$anova <- anotab
    object
}

`veganCovEllipse` <-
    function(cov, center = c(0,0), scale = 1, npoints = 100)
{
    ## Basically taken from the 'car' package: The Cirlce
    theta <- (0:npoints) * 2 * pi/npoints
    Circle <- cbind(cos(theta), sin(theta))
    ## scale, center and cov must be calculated separately
    Q <- chol(cov, pivot = TRUE)
    ## pivot takes care of cases when points are on a line
    o <- attr(Q, "pivot")
    t(center + scale * t(Circle %*% Q[,o]))
}

`as.preston` <-
    function (x, tiesplit = TRUE, ...)
{
    if (inherits(x, "preston"))
        return(x)
    ## practically integer
    if (!identical(all.equal(x, round(x)), TRUE))
        stop("function accepts only integers (counts)")
    ## need exact integers, since, e.g., sqrt(2)^2 - 2 = 4.4e-16 and
    ## tie breaks fail
    if (!is.integer(x))
        x <- round(x)
    x <- x[x > 0]
    if (tiesplit) {
        ## Assume log2(2^k) == k *exactly* for integer k
        xlog2 <- log2(x)
        ties <- xlog2 == ceiling(xlog2)
        tiefreq <- table(xlog2[ties])
        notiefreq <- table(ceiling(xlog2[!ties]))
        itie <- as.numeric(names(tiefreq)) + 1
        nitie <- as.numeric(names(notiefreq)) + 1
        freq <- numeric(max(itie+1, nitie))
        ## split tied values between two adjacent octaves
        freq[itie] <- tiefreq/2
        freq[itie+1] <- freq[itie+1] + tiefreq/2
        freq[nitie] <- freq[nitie] + notiefreq
    } else {
        xlog2 <- ceiling(log2(x))
        tmp <- table(xlog2)
        indx <- as.numeric(names(tmp)) + 1
        freq <- numeric(max(indx))
        freq[indx] <- tmp
    }
    names(freq) <- seq_along(freq) - 1
    ## remove empty octaves
    freq <- freq[freq>0]
    class(freq) <- "preston"
    freq
}

c     Decorana: the classic goodie by Mark O. Hill
c     Ported to R by Jari Oksanen, September 2001:
c     * Kept only the numerical engine: subroutines eigy, cutup and
c     yxmult, and the subroutines eigy calls.
c     * Changed to lowercase so that it looks prettier.
c     * Changed to double precision throughout, with 
c     'implicit double precision (a-h,o-z)' for automatic variables.
c     * Changed 'ifix' to 'int', since g77 doesn't know 'ifix' for double.
c     * Removed all write statements (they don't show in GUI term --
c     else it would make sense to add a `trace' parameter.)
c     
c     ORIGINAL INTRODUCTORY COMMENTS:
c     
c     Cornell Ecology Program Decorana - written by M.O. Hill, July 1979
c     Source code and accompanying documentation are available from
c     Hugh G. Gauch, Jr., Ecology and Systematics, Cornell University,
c     Ithaca, New York 14850.
c     
c     Performs detrended correspondence analysis;  also will do reciprocal
c     averaging as a special case.
c     
c     **** further modified by Dr Peter R. Minchin May-June 1997
c     - changed tolerance to 0.000005 and iteration limit to 999 in
c     eigy - strict settings of oksanen & minchin 1997
c     - corrected the order-dependent bug in smooth
c     - added parameter statements for maximum dimensions
c     - changed max. dimensions to 5000, 5000, 330000
c     - updated handling of file names retrieved from command line
c     - moved character data to character variables
c     - now accepts relaxed ccf format, with maximum number of pairs
c     per record on line 3
c     see: Oksanen, J. & Minchin, P.R. 1997. Instability of ordination
c     results under changes in input data order: explanations and
c     remedies. Journal of Vegetation Science 8: 447-454.
c     
c     
      subroutine cutup(x,ix,mi,mk)
c     takes a vector x and cuts up into (mk-4) segments, putting a
c     segmented version of the vector in ix.
      implicit double precision (a-h,o-z)
      double precision x(mi)
      integer ix(mi)
      mmk=mk-4
      maxk=mk-2
      call xmaxmi(x,axmax,axmin,mi)
      axbit=(axmax-axmin)/float(mmk)
      do 10 i=1,mi
C---  iax=ifix((x(i)-axmin)/axbit)+3
         iax=int((x(i)-axmin)/axbit)+3
         if(iax.lt.3) iax=3
         if(iax.gt.maxk) iax=maxk
         ix(i)=iax
 10   continue
      return
      end
c
      subroutine trans(y,yy,
     1     x,neig,ira,aidot,xeig1,xeig2,xeig3,ix1,ix2,ix3,
     2     mi,mk,n,nid,ibegin,iend,idat,qidat)
c     this subroutine is the crux of the whole program, in that it
c     takes a set of species scores y and iterates to find a new set
c     of scores yy.  repeated iteration of this subroutine would lead
c     eventually to the correct solution (except that the scores need
c     to be divided by the y-totals adotj at each iteration).  the
c     calling program eigy is made lengthy by some fancy algebra put
c     there to speed up the calculation.  essentially trans is the
c     standard reciprocal averaging iteration with either detrending
c     with respect to previously derived axes (in the case of detrended
c     correspondence analysis) or orthogonalization with respect to
c     them (in the case of reciprocal averaging).
      implicit double precision (a-h,o-z)
      double precision x(mi),xeig1(mi),xeig2(mi),xeig3(mi)
      double precision y(n),yy(n),aidot(mi),qidat(nid)
      integer ix1(mi),ix2(mi),ix3(mi),idat(nid),ibegin(mi),iend(mi)
      call yxmult(y,x,mi,n,nid,ibegin,iend,idat,qidat)
      do 10 i=1,mi
         x(i)=x(i)/aidot(i)
 10   continue
      if(neig.eq.0) goto 200
      if(ira.eq.1) goto 100
      call detrnd(x,aidot,ix1,mi,mk)
      if(neig.eq.1) goto 200
      call detrnd(x,aidot,ix2,mi,mk)
      if(neig.eq.2) goto 90
      call detrnd(x,aidot,ix3,mi,mk)
      call detrnd(x,aidot,ix2,mi,mk)
 90   call detrnd(x,aidot,ix1,mi,mk)
      goto 200
 100  a1=0.0
      do 110 i=1,mi
         a1=a1+aidot(i)*x(i)*xeig1(i)
 110  continue
      do 120 i=1,mi
         x(i)=x(i)-a1*xeig1(i)
 120  continue
      if(neig.eq.1) goto 200
      a2=0.0
      do 130 i=1,mi
         a2=a2+aidot(i)*x(i)*xeig2(i)
 130  continue
      do 140 i=1,mi
         x(i)=x(i)-a2*xeig2(i)
 140  continue
      if(neig.eq.2) goto 200
      a3=0.0
      do 150 i=1,mi
         a3=a3+aidot(i)*x(i)*xeig3(i)
 150  continue
      do 160 i=1,mi
         x(i)=x(i)-a3*xeig3(i)
 160  continue
 200  call xymult(x,yy,mi,n,nid,ibegin,iend,idat,qidat)
      return
      end
c
      subroutine detrnd(x,aidot,ix,mi,mk)
      implicit double precision (a-h,o-z)
      double precision x(mi),z(50),zn(50),zbar(50),aidot(mi)
      integer ix(mi)
c     starts with a vector x and detrends with respect to groups defined
c     by ix.  detrending is in blocks of 3 units at a time, and the
c     result calculated is the average of the 3 possible results that
c     can be obtained, corresponding to 3 possible starting positions
c     for the blocks of 3.
      do 10 k=1,mk
         z(k)=0.0
         zn(k)=0.0
 10   continue
      do 20 i=1,mi
         k=ix(i)
         z(k)=z(k)+x(i)*aidot(i)
         zn(k)=zn(k)+aidot(i)
 20   continue
      mmk=mk-1
      do 30 k=2,mmk
         zbar(k)=(z(k-1)+z(k)+z(k+1))/(zn(k-1)+zn(k)+zn(k+1)+1.0e-12)
 30   continue
      mmk=mmk-1
      do 35 k=3,mmk
         z(k)=(zbar(k-1)+zbar(k)+zbar(k+1))/3.0
 35   continue
      do 40 i=1,mi
         k=ix(i)
         x(i)=x(i)-z(k)
 40   continue
      return
      end

c Function segfit is identical to detrnd, but it also returns the fitted
c values z. x is respone in input, and residuals in output. --Added by
c J. Oksanen 1 Oct, 2010.
      subroutine segfit(x,aidot,ix,mi,mk,fit)
      implicit double precision (a-h,o-z)
      double precision x(mi),z(50),zn(50),zbar(50),aidot(mi)
      double precision fit(mi)
      integer ix(mi)
      do 10 k=1,mk
         z(k)=0.0
         zn(k)=0.0
 10   continue
      do 20 i=1,mi
         k=ix(i)
         z(k)=z(k)+x(i)*aidot(i)
         zn(k)=zn(k)+aidot(i)
 20   continue
      mmk=mk-1
      do 30 k=2,mmk
         zbar(k)=(z(k-1)+z(k)+z(k+1))/(zn(k-1)+zn(k)+zn(k+1)+1.0e-12)
 30   continue
      mmk=mmk-1
      do 35 k=3,mmk
         z(k)=(zbar(k-1)+zbar(k)+zbar(k+1))/3.0
 35   continue
      do 40 i=1,mi
         k=ix(i)
         fit(i) = z(k)
         x(i)=x(i)-z(k)
 40   continue
      return
      end
c
      subroutine eigy(x,y,eig,neig,ira,iresc,short,
     1     mi,mk,n,nid,ibegin,iend,idat,qidat,y2,y3,y4,y5,
     2     xeig1,xeig2,xeig3,ix1,ix2,ix3,aidot,adotj)
c     extracts an eigenvector y with eigenvalue eig.  the algebra
c     is a little complicated, but consists essentially of repre-
c     senting the transformation (subroutine trans) approximately
c     by a tridiagonal 4x4 matrix.  the eigenproblem for the
c     tridiagonal matrix is solved and this solution is plugged
c     back in to obtain a new trial vector.
c     after getting the eigenvector, the scores may be rescaled
c     (subroutine strtch).
      implicit double precision (a-h,o-z)
      double precision x(mi),y(n),y2(n),y3(n),y4(n),y5(n)
      double precision xeig1(mi),xeig2(mi),xeig3(mi),aidot(mi),adotj(n)
      double precision qidat(nid)
      integer ibegin(mi),iend(mi),idat(nid),ix1(mi),ix2(mi),ix3(mi)
c     strings to print R warnings
      character (len=64) warning
      character (len=2) axnam
      tot=0.0
      do 10 j=1,n
         tot=tot+adotj(j)
         y(j)=float(j)
 10   continue
      y(1)=1.1
c---  tolerance reduced by p.minchin jan 1997
c     tol=0.0001
      tol=0.000005
      call trans(y,y,
     1     x,neig,ira,aidot,xeig1,xeig2,xeig3,ix1,ix2,ix3,
     2     mi,mk,n,nid,ibegin,iend,idat,qidat)
      icount=0
 20   a=0.0
      do 30 j=1,n
         a=a+y(j)*adotj(j)
 30   continue
      a=a/tot
      ex=0.0
      do 40 j=1,n
         ay=y(j)-a
         ex=ex+ay*ay*adotj(j)
         y(j)=ay
 40   continue
      ex=sqrt(ex)
      do 50 j=1,n
         y(j)=y(j)/ex
 50   continue
      call trans(y,y2,
     1     x,neig,ira,aidot,xeig1,xeig2,xeig3,ix1,ix2,ix3,
     2     mi,mk,n,nid,ibegin,iend,idat,qidat)
      a=0.0
      a11=0.0
      a12=0.0
      a22=0.0
      a23=0.0
      a33=0.0
      a34=0.0
      a44=0.0
      do 60 j=1,n
         ay=y2(j)
         y2(j)=ay/adotj(j)
         a=a+ay
         a11=a11+ay*y(j)
 60   continue
      a=a/tot
      do 70 j=1,n
         ay=y2(j)-(a+a11*y(j))
         a12=a12+ay*ay*adotj(j)
         y2(j)=ay
 70   continue
      a12=sqrt(a12)
      do 80 j=1,n
         y2(j)=y2(j)/a12
 80   continue
c---------removed write statements--------------------------
c     if(icount.eq.0) write(*,1000)
c     1000 format(/1x)
c     write(*,1011) a12,icount
c     1011 format(1x,'residual',f10.6,'       at iteration',i3)
      if(a12.lt.tol) goto 200
c---  maximum iteration limit increased by P.Minchin jan 1997
c     if(icount.gt.9) goto 200
      if(icount.gt.999) goto 200
      icount=icount+1
      call trans(y2,y3,
     1     x,neig,ira,aidot,xeig1,xeig2,xeig3,ix1,ix2,ix3,
     2     mi,mk,n,nid,ibegin,iend,idat,qidat)
      a=0.0
      b13=0.0
      do 90 j=1,n
         ay=y3(j)
         y3(j)=ay/adotj(j)
         a=a+ay
         a22=a22+ay*y2(j)
         b13=b13+ay*y(j)
 90   continue
      a=a/tot
      do 100 j=1,n
         ay=y3(j)-(a+a22*y2(j)+b13*y(j))
         a23=a23+ay*ay*adotj(j)
         y3(j)=ay
 100  continue
      a23=sqrt(a23)
      if(a23.gt.tol) goto 105
      a23=0.0
      goto 160
 105  continue
      do 110 j=1,n
         y3(j)=y3(j)/a23
 110  continue
      call trans(y3,y4,
     1     x,neig,ira,aidot,xeig1,xeig2,xeig3,ix1,ix2,ix3,
     2     mi,mk,n,nid,ibegin,iend,idat,qidat)
      a=0.0
      b14=0.0
      b24=0.0
      do 120 j=1,n
         ay=y4(j)
         y4(j)=y4(j)/adotj(j)
         a=a+ay
         a33=a33+ay*y3(j)
         b14=b14+ay*y(j)
         b24=b24+ay*y2(j)
 120  continue
      a=a/tot
      do 130 j=1,n
         ay=y4(j)-(a+a33*y3(j)+b14*y(j)+b24*y2(j))
         a34=a34+ay*ay*adotj(j)
         y4(j)=ay
 130  continue
      a34=sqrt(a34)
      if(a34.gt.tol) goto 135
      a34=0.0
      goto 160
 135  continue
      do 140 j=1,n
         y4(j)=y4(j)/a34
 140  continue
      call trans(y4,y5,
     1     x,neig,ira,aidot,xeig1,xeig2,xeig3,ix1,ix2,ix3,
     2     mi,mk,n,nid,ibegin,iend,idat,qidat)
      do 150 j=1,n
         a44=a44+y4(j)*y5(j)
 150  continue
c     we now have the tridiagonal representation of trans.  solve
c     eigenproblem for tridiagonal matrix.
 160  ax1=1.0
      ax2=0.1
      ax3=0.01
      ax4=0.001
      do 170 itimes=1,100
         axx1=a11*ax1+a12*ax2
         axx2=a12*ax1+a22*ax2+a23*ax3
         axx3=a23*ax2+a33*ax3+a34*ax4
         axx4=a34*ax3+a44*ax4
         ax1=a11*axx1+a12*axx2
         ax2=a12*axx1+a22*axx2+a23*axx3
         ax3=a23*axx2+a33*axx3+a34*axx4
         ax4=a34*axx3+a44*axx4
         ex=sqrt(ax1**2+ax2**2+ax3**2+ax4**2)
         ax1=ax1/ex
         ax2=ax2/ex
         ax3=ax3/ex
         ax4=ax4/ex
         if(itimes.ne.(itimes/5)*5) goto 170
         exx=sqrt(ex)
         resi=sqrt((ax1-axx1/exx)**2+(ax2-axx2/exx)**2+
     1        (ax3-axx3/exx)**2+(ax4-axx4/exx)**2)
         if(resi.lt.tol*0.05) goto 180
 170  continue
 180  continue
      do 190 j=1,n
         y(j)=ax1*y(j)+ax2*y2(j)+ax3*y3(j)+ax4*y4(j)
 190  continue
      goto 20
c-----------Removed write statements, added 200 continue--------
 200  continue
c     200 write(*,1010) a11
c     1010 format(1x,'eigenvalue',f10.5)
c     if(a12.gt.tol) write(*,1012) tol
c     1012 format(1x,'*** beware ***     residual bigger than tolerance',
c     1', which is',f10.6)
c     R version of the above warning. You must change the length of
c     character*n warning definition if you change the warning text
      if (a12 .gt. tol) then
         if (neig .eq. 0) axnam = "1"
         if (neig .eq. 1) axnam = "2"
         if (neig .eq. 2) axnam = "3"
         if (neig .eq. 3) axnam = "4"
         warning = "residual bigger than tolerance on axis "//axnam
         call rwarn(warning)
      end if
c     we calculate x from y, and set x to unit length if reciprocal
c     averaging option is in force (ira=1)
      call xmaxmi(y,aymax,aymin,n)
      sign=1.0
      if(-aymin.gt.aymax) sign=-1.0
      do 210 j=1,n
         y(j)=y(j)*sign
 210  continue
      call yxmult(y,x,mi,n,nid,ibegin,iend,idat,qidat)
      do 220 i=1,mi
         x(i)=x(i)/aidot(i)
 220  continue
      if(iresc.eq.0) goto 225
      if(a11.gt.0.999) goto 225
      do 223 i=1,iresc
         monit=0
         if(i.eq.1.or.i.eq.iresc) monit=1
         call strtch(x,y,short,monit,
     1        mi,n,nid,aidot,ibegin,iend,idat,qidat)
 223  continue
      eig=a11
      return
 225  axlong=0.0
      do 230 i=1,mi
         axlong=axlong+aidot(i)*x(i)**2
 230  continue
      axlong=sqrt(axlong)
      do 240 i=1,mi
         x(i)=x(i)/axlong
 240  continue
      do 250 j=1,n
         y(j)=y(j)/axlong
 250  continue
c     it remains to scale y to unit within-sample standard deviation
      sumsq=0.0
      do 260 i=1,mi
         id1=ibegin(i)
         id2=iend(i)
         ax=x(i)
         do 255 id=id1,id2
            j=idat(id)
            sumsq=sumsq+qidat(id)*(ax-y(j))**2
 255     continue
 260  continue
      sd=sqrt(sumsq/tot)
      if(a11.le.0.999) goto 265
      sd=aymax/axlong
      sd1=-aymin/axlong
      if(sd1.gt.sd) sd=sd1
 265  continue
      do 270 j=1,n
         y(j)=y(j)/sd
 270  continue
      eig=a11
      return
      end
c
      subroutine xymult(x,y,mi,n,nid,ibegin,iend,idat,qidat)
c     starts with vector x and forms matrix product y=ax
      implicit double precision (a-h,o-z)
      double precision x(mi),y(n),qidat(nid)
      integer ibegin(mi),iend(mi),idat(nid)
      do 10 j=1,n
         y(j)=0.0
 10   continue
      do 30 i=1,mi
         id1=ibegin(i)
         id2=iend(i)
         ax=x(i)
         do 20 id=id1,id2
            j=idat(id)
            y(j)=y(j)+ax*qidat(id)
 20      continue
 30   continue
      return
      end
c
      subroutine yxmult(y,x,mi,n,nid,ibegin,iend,idat,qidat)
c     starts with vector y and forms matrix product x=ay
      implicit double precision (a-h,o-z)
      double precision x(mi),y(n),qidat(nid)
      integer ibegin(mi),iend(mi),idat(nid)
      do 20 i=1,mi
         id1=ibegin(i)
         id2=iend(i)
         ax=0.0
         do 10 id=id1,id2
            j=idat(id)
            ax=ax+y(j)*qidat(id)
 10      continue
         x(i)=ax
 20   continue
      return
      end
c
      subroutine xmaxmi(x,axmax,axmin,m)
c     forms maximum and minimum of x(m)
      implicit double precision (a-h,o-z)
      double precision x(m)
      axmax=-1.0e10
      axmin=-axmax
      do 10 i=1,m
         ax=x(i)
         if(ax.gt.axmax) axmax=ax
         if(ax.lt.axmin) axmin=ax
 10   continue
      return
      end
c
c
      subroutine strtch(x,y,short,monit,
     1     mi,n,nid,aidot,ibegin,iend,idat,qidat)
c     takes an axis (x,y) and scales to unit mean square dev of species
c     scores per sample.  an attempt is made for longer axes (l > short)
c     to squeeze them in and out so that they have the right mean square
c     deviation all the way along the axis and not only on average.
      implicit double precision (a-h,o-z)
      double precision x(mi),y(n),aidot(mi),qidat(nid)
      double precision zn(50),zv(50)
      integer ibegin(mi),iend(mi),idat(nid)
c     Use monit so that gfortran -Wall does not warn for it being unused
      monit = 0
c---  common block added by p.minchin feb 1988
c     common /lunits/ iuinp1,iuinp2,iuout1,iuout2,iuout3
      do 200 icount=1,2
         mk=20
         call segmnt(x,y,zn,zv,mi,mk,n,nid,
     1        aidot,ibegin,iend,idat,qidat)
         call smooth(zv,mk)
         call smooth(zn,mk)
         zvsum=0.0
         do 50 k=1,mk
            zvsum=zvsum+zv(k)/zn(k)
 50      continue
         sd=sqrt(zvsum/float(mk))
c     we want mean within-sample square deviation to be 1.0, so we divide
c     everything by sd
         along=0.0
         do 60 i=1,mi
            ax=x(i)/sd
            x(i)=ax
            if(along.lt.ax) along=ax
 60      continue
c--------Removed write statements----------------------------
c     if(icount.eq.1.and.monit.eq.1) write(*,1000)
c     1000 format(/1x)
c     if(monit.eq.1) write(*,1001) along
c     1001 format(1x,'length of gradient',f10.3)
         do 70 j=1,n
            y(j)=y(j)/sd
 70      continue
         if(along.lt.short) return
         if(icount.eq.2) return
c     mk=ifix(along*5.0)+1
         mk=int(along*5.0)+1
         if(mk.lt.10) mk=10
         if(mk.gt.45) mk=45
         call segmnt(x,y,zn,zv,mi,mk,n,nid,aidot,ibegin,iend,idat,qidat)
         call smooth(zv,mk)
         call smooth(zn,mk)
         zvsum=0.0
         do 100 k=1,mk
            azv=1.0/sqrt(0.2/along+zv(k)/zn(k))
            zvsum=zvsum+azv
            zv(k)=azv
 100     continue
         do 110 k=1,mk
            zv(k)=zv(k)*along/zvsum
 110     continue
c----------Removed write statements-------------------------
c     if(monit.eq.1) write(*,1002) (zv(k),k=1,mk)
c     1002 format(1x,'length of segments',10f6.2)
         az=0.0
         zn(1)=0.0
         do 120 k=1,mk
            az=az+zv(k)
            zn(k+1)=az
 120     continue
         axbit=along/float(mk)
         do 130 j=1,n
C     iay=ifix(y(j)/axbit)+1
            iay=int(y(j)/axbit)+1
            if(iay.lt.1) iay=1
            if(iay.gt.mk) iay=mk
            y(j)=zn(iay)+zv(iay)*(y(j)/axbit-float(iay-1))
 130     continue
         call yxmult(y,x,mi,n,nid,ibegin,iend,idat,qidat)
         do 140 i=1,mi
            x(i)=x(i)/aidot(i)
 140     continue
 200  continue
      return
      end
c
      subroutine smooth(z,mk)
      implicit double precision (a-h,o-z)
      double precision z(mk)
c     takes a vector z and does (1,2,1)-smoothing until no blanks left
c     and then 2 more iterations of (1,2,1)-smoothing.  if no blanks to
c     begin with, then does 3 smoothings, i.e. effectively (1,6,15,20,
c     15,6,1)-smoothing.
      istop=1
      do 20 icount=1,50
         az2=z(1)
         az3=z(2)
         if(az3.eq.0.0) istop=0
         z(1)=0.75*az2+0.25*az3
         do 10 k3=3,mk
            az1=az2
            az2=az3
            az3=z(k3)
c---  bug in next line fixed by p.minchin jan 1997
c     if(az3.lt.0.0) istop=0
            if(az3.le.0.0) istop=0
            z(k3-1)=0.5*(az2+0.5*(az1+az3))
 10      continue
         z(mk)=0.25*az2+0.75*az3
         istop=istop+1
         if(istop.eq.4) goto 30
 20   continue
 30   return
      end
c
      subroutine segmnt(x,y,zn,zv,mi,mk,n,nid,
     1     aidot,ibegin,iend,idat,qidat)
c     given an ordination (x,y), calculates numbers and summed mean-square
c     deviations in mk segments.  zn(k) is the number of samples in segment
c     k;  zv(k) is the summed mean-square deviation.  (we aim to make zv,
c     zn as nearly equal as possible.)
      implicit double precision (a-h,o-z)
      double precision x(mi),y(n),zn(mk),zv(mk),aidot(mi),qidat(nid)
      integer ibegin(mi),iend(mi),idat(nid)
      do 10 k=1,mk
         zn(k)=-1.0e-20
         zv(k)=-1.0e-20
 10   continue
      call xmaxmi(x,axmax,axmin,mi)
      axbit=(axmax-axmin)/float(mk)
      do 20 i=1,mi
         x(i)=x(i)-axmin
 20   continue
      do 30 j=1,n
         y(j)=y(j)-axmin
 30   continue
      do 50 i=1,mi
         sqcorr=0.0
         sumsq=2.0e-20
         id1=ibegin(i)
         id2=iend(i)
         ax=x(i)
         do 40 id=id1,id2
            j=idat(id)
            aij=qidat(id)
            sqcorr=sqcorr+aij**2
            sumsq=sumsq+aij*(ax-y(j))**2
 40      continue
         sqcorr=sqcorr/aidot(i)**2
         if(sqcorr.gt.0.9999) sqcorr=0.9999
         sumsq=sumsq/aidot(i)
c     k=ifix(ax/axbit)+1
         k=int(ax/axbit)+1
         if(k.gt.mk) k=mk
         if(k.lt.1) k=1
         zv(k)=zv(k)+sumsq
         zn(k)=zn(k)+1.0-sqcorr
 50   continue
      return
      end

`coverscale` <-
    function (x, scale = c("Braun.Blanquet", "Domin", "Hult", "Hill",
                 "fix", "log"), maxabund, character = TRUE)
{
    scale <- match.arg(scale)
    sol <- as.data.frame(x)
    x <- as.matrix(x)
    switch(scale, Braun.Blanquet = {
        codes <- c("r", "+", as.character(1:5))
        lims <- c(0, 0.1, 1, 5, 25, 50, 75, 100)
    }, Domin = {
        codes <- c("+", as.character(1:9), "X")
        lims <- c(0, 0.01, 0.1, 1, 5, 10, 25, 33, 50, 75, 90,
                  100)
    }, Hult = {
        codes <- as.character(1:5)
        lims <- c(0, 100/2^(4:1), 100)
    }, Hill = {
        codes <- as.character(1:5)
        lims <- c(0, 2, 5, 10, 20, Inf)
    }, fix = {
        codes <- c("+", as.character(1:9), "X")
        lims <- c(0:10, 11 - 10 * .Machine$double.eps)
    }, log = {
        codes <- c("+", as.character(1:9))
        if (missing(maxabund))
            maxabund <- max(x)
        lims <- c(0, maxabund/2^(9:1), maxabund)
    })
    for (i in 1:nrow(x)) {
        if (!character)
            codes <- FALSE
        tmp <- x[i, ] > 0
        sol[i, tmp] <- cut(x[i, tmp], breaks = lims, labels = codes,
                           right = FALSE, include.lowest = TRUE)
    }
    attr(sol, "scale") <-
        if (scale == "log")  paste("log, with maxabund", maxabund) else scale
    sol
}

`nesteddisc` <-
    function(comm, niter = 200)
{
    ## The original discrepancy method orders columns by frequencies,
    ## but does not consider ties. The current function tries to order
    ## tied values to minimize the discrepancy either by complete
    ## enumeration or with a larger number of ties using simulated
    ## annealing.  NALL: max no. of tied items for NALL! complete
    ## enumeration

    ## starting values and CONSTANTS
    NALL <- 7
    allperm <- factorial(NALL)
    ties <- FALSE
    trace <- FALSE
    ## Code
    comm <- ifelse(comm > 0, 1, 0)
    cs <- colSums(comm)
    k <- rev(order(cs))
    ## initial order
    cs <- cs[k]
    comm <- comm[, k]
    ## run lengths: numbers of tied values
    le <- rle(cs)$lengths
    cle <- c(0, cumsum(le))
    x <- seq(along=cs)
    ## Range of row sums: only swaps between these have an effect
    rs <- range(rowSums(comm))
    ## Function to evaluate discrepancy
    FUN <- function(x) sum(comm[col(comm)[,x] <= rowSums(comm)] == 0)
    Ad <- FUN(x)
    ## Go through all le-items and permute ties. Function shuffleSet
    ## is in permute package, and its minperm argument triggers
    ## complete enumeration for no. of ties <= NALL!.
    for (i in seq_along(le)) {
        if (le[i] > 1) {
            take <- x
            idx <- seq_len(le[i]) + cle[i]
            ## Can swaps influence discrepancy?
            if (idx[1] > rs[2] || idx[le[i]] < rs[1])
                next
            perm <- shuffleSet(le[i], niter, control = how(minperm = allperm),
                               quietly = TRUE)
            ## maxperm is a double -- needs EPS -0.5
            if ((attr(perm, "control")$maxperm - 0.5) > niter)
                ties <- TRUE
            perm <- matrix(perm, ncol = le[i]) + cle[i]
            vals <- sapply(1:nrow(perm), function(j) {
                take[idx] <- perm[j,]
                FUN(take)
            })
            jmin <- which.min(vals)
            if (vals[jmin] < Ad) {
                x[idx] <- perm[jmin,]
                Ad <- vals[jmin]
            }
        }
    }
    out <- list(statistic=Ad, ties = ties, order = k[x])
    names(out$statistic) <- "discrepancy"
    class(out) <- "nesteddisc"
    out
}

`scores.betadiver` <-
    function(x, triangular = TRUE,  ...)
{
    if (triangular) {
        tot <- x$a + x$b + x$c
        a <- x$a/tot
        c <- x$c/tot
        y <- sqrt(0.75)*a
        x <- c + a/2
        out <- cbind(x, y)
    } else {
        out <- sapply(x, cbind)
    }
    out
}

bgdispersal <- function (mat, PAonly = FALSE, abc = FALSE)
{
    mat <- as.matrix(mat)
    names <- rownames(mat)
    if (sum((mat - decostand(mat, "pa"))) == 0) {
        PAonly <- TRUE
        mat1 <- mat
    }
    else {
        mat1 <- decostand(mat, "pa")
        if (PAonly == FALSE) mat2 <- mat
    }
    n <- nrow(mat)
    p <- ncol(mat)
    a <- mat1 %*% t(mat1)
    b <- mat1 %*% (1 - t(mat1))
    c <- (1 - mat1) %*% t(mat1)
    d <- ncol(mat1) - a - b - c
    DD1 <- (a * (b - c))/((a + b + c)^2)
    DD2 <- (2 * a * (b - c))/((2 * a + b + c) * (a + b + c))
    ## McNemar <- (abs(b - c) - 1)^2/(b + c)     # Old code
    ## diag(McNemar) <- 0                        # Old code
    McNemar <- matrix(NA, n, n, dimnames=list(names,names))
    pP.Mc <- matrix(NA, n, n, dimnames=list(names,names))
    for (j in 1:(n - 1)) {
        for (jj in (j + 1):n) {
            bb = b[j, jj]
            cc = c[j, jj]
            if ((bb + cc) == 0) {
                McNemar[j, jj] = 0
                pP.Mc[j, jj] = 1
            } else {
                if(bb == 0) { B = 0 } else { B = bb*log(bb) }
                if(cc == 0) { C = 0 } else { C = cc*log(cc) }
                ## Williams correction
                q = 1 + 1/(2*(bb+cc))
                ## McNemar = 2*(b*log(b) + c*log(c) - (b+c)*log((b+c)/2))
                McNemar[j, jj] = 2*(B + C - (bb+cc)*log((bb+cc)/2)) / q
                pP.Mc[j, jj] <- pchisq(McNemar[j, jj], 1, lower.tail = FALSE)
                if ((b[j, jj] + c[j, jj]) == 0)
                    pP.Mc[j, jj] <- 1
            }
        }
    }
    if (!PAonly) {
        DD3 <- matrix(0, n, n, dimnames=list(names,names))
        DD4 <- matrix(0, n, n, dimnames=list(names,names))
        row.sum <- rowSums(mat2)
        for (j in 1:(n - 1)) {
            for (jj in (j + 1):n) {
                W <- sum(apply(mat2[c(j, jj), ], 2, min))
                A <- row.sum[j]
                B <- row.sum[jj]
                temp3 <- W * (A - B)/((A + B - W)^2)
                temp4 <- 2 * W * (A - B)/((A + B) * (A + B - W))
                DD3[j, jj] <- temp3
                DD3[jj, j] <- -temp3
                DD4[j, jj] <- temp4
                DD4[jj, j] <- -temp4
            }
        }
        out <- list(DD1 = DD1, DD2 = DD2, DD3 = DD3, DD4 = DD4,
                    McNemar = McNemar, prob.McNemar = pP.Mc)
    }
    else {
        out <- list(DD1 = DD1, DD2 = DD2, McNemar = McNemar,
                    prob.McNemar = pP.Mc)
    }
    if (abc) {
        out$a <- a
        out$b <- b
        out$c <- c
    }
    out
}

### Support functions for decorana

## Hill's downweighting

## An exported function that can be called outside decorana

`downweight` <-
    function (veg, fraction = 5)
{
    Const1 <- 1e-10
    if (fraction < 1)
        fraction <- 1/fraction
    veg <- as.matrix(veg)
    yeig1 <- colSums(veg)
    y2 <- colSums(veg^2) + Const1
    y2 <- yeig1^2/y2
    amax <- max(y2)/fraction
    v <- rep(1, ncol(veg))
    downers <- y2 < amax
    v[downers] <- (y2/amax)[downers]
    veg <- sweep(veg, 2, v, "*")
    attr(veg, "v") <- v
    attr(veg, "fraction") <- fraction
    veg
}

## Hill's piecewise tranformation. Values of before are replaced with
## values of after, and intermediary values with linear interpolation.

## Not exported: if you think you need something like this, find a
## better tool in R.

`beforeafter` <-
    function(x, before, after)
{
    if (is.null(before) || is.null(after))
        stop("both 'before' and 'after' must be given", call. = FALSE)
    if (is.unsorted(before))
        stop("'before' must be sorted", call. = FALSE)
    if (length(before) != length(after))
        stop("'before' and 'after' must have same lengths", call. = FALSE)
    for(i in seq_len(nrow(x))) {
        k <- x[i,] > 0
        x[i, k] <- approx(before, after, x[i, k], rule = 2)$y
    }
    x
}

`model.matrix.cca` <-
    function(object, ...)
{
    X <- Z <- NULL
    w <- 1/sqrt(object$rowsum)
    if (!is.null(object$pCCA))
        Z <- w * qr.X(object$pCCA$QR)
    if (!is.null(object$CCA)) {
        X <- qr.X(object$CCA$QR)
        ## First columns come from Z
        if (!is.null(Z))
            X <- X[, -seq_len(ncol(Z)), drop = FALSE]
        X <- w * X
    }
    m <- list()
    if (!is.null(Z))
        m$Conditions <- Z
    if (!is.null(X))
        m$Constraints <- X
    if (length(m) == 1)
        m <- m[[1]]
    m
}

`model.matrix.rda` <-
    function(object, ...)
{
    X <- Z <- NULL
    if (!is.null(object$pCCA))
        Z <- qr.X(object$pCCA$QR)
    if (!is.null(object$CCA)) {
        X <- qr.X(object$CCA$QR)
        if (!is.null(Z))
            X <- X[, -seq_len(ncol(Z)), drop=FALSE]
    }
    m <- list()
    if (!is.null(Z))
        m$Conditions <- Z
    if (!is.null(X))
        m$Constraints <- X
    if (length(m) == 1)
        m <- m[[1]]
    m
}

## test the stability of cca object and its support functions
suppressPackageStartupMessages(require(vegan))
set.seed(4711)
op <- options(digits=5)
## models
data(dune, dune.env)
mcca <- cca(dune ~ Condition(Management) + Manure + A1, dune.env)
mrda <- rda(dune ~ Condition(Management) + Manure + A1, dune.env)
mrda1 <- rda(dune ~ Condition(Management) + Manure + A1, dune.env, scale=TRUE)
mcap <- capscale(dune ~ Condition(Management) + Manure + A1, dune.env,
          dist = "bray")
mdb <- dbrda(dune ~ Condition(Management) + Manure + A1, dune.env,
             dist = "bray")
mancap <- capscale(dune ~ Condition(Management) + Manure + A1, dune.env,
          dist = "manhattan")
mandb <- dbrda(dune ~ Condition(Management) + Manure + A1, dune.env,
               dist = "manhattan")
## 0-rank constraints
m0cca <- cca(dune ~ Condition(Management) + Management, dune.env)

## general appearance
mcca
mrda
mcap
mdb
mancap
mandb
m0cca
## names
sort(names(mcca))
sort(names(mrda))
sort(names(mcap))
sort(names(mdb))

## diagnostics
hatvalues(mcca)
hatvalues(mrda)
hatvalues(mandb)

zapsmall(head(cooks.distance(mcca)))
zapsmall(head(cooks.distance(mrda)))
zapsmall(head(cooks.distance(mrda1)))
zapsmall(head(cooks.distance(mcap, "canoco")))
zapsmall(head(cooks.distance(mdb, "canoco")))
zapsmall(head(cooks.distance(mancap, "canoco")))
zapsmall(head(cooks.distance(mandb, "canoco")))
zapsmall(head(cooks.distance(m0cca)))

head(goodness(mcca, display = "sites"))
head(goodness(mrda, display = "sites"))
head(goodness(mrda1, display = "sites"))
## head(goodness(mcap, display = "sites")) # currently disabled
## head(goodness(mdb, display="sites"))  # not implemented for partial dbrda
## head(goodness(mancap, display="sites")) # currently disabled
## head(goodness(mandb, display="sites")) # not implemneted for partial dbrda
head(goodness(m0cca))

head(inertcomp(mcca))
head(inertcomp(mrda))
head(inertcomp(mrda1))
head(inertcomp(mcap, display="sites"))
head(inertcomp(mdb, display = "sites"))
head(inertcomp(mancap, display = "sites"))
head(inertcomp(mandb, display = "sites"))
zapsmall(head(inertcomp(m0cca))) # numerical zeros

abs(zapsmall(intersetcor(mcca)))
abs(zapsmall(intersetcor(mrda)))
abs(zapsmall(intersetcor(mrda1)))
abs(zapsmall(intersetcor(mcap)))
abs(zapsmall(intersetcor(mdb)))
abs(zapsmall(intersetcor(mancap)))
abs(zapsmall(intersetcor(mandb)))

tolerance(mcca)
tolerance(m0cca)

vif.cca(mcca)
vif.cca(mrda)
vif.cca(mcap)
vif.cca(mdb)
vif.cca(m0cca)

alias(mcca)
alias(mrda)
alias(mcap)
alias(mdb)
alias(m0cca)

## basic statistic

abs(coef(mcca))
abs(coef(mrda))
abs(coef(mrda1))
abs(coef(mcap))
abs(coef(mdb))
abs(coef(m0cca))

 eigenvals(mcca)
eigenvals(mrda)
eigenvals(mrda1)
eigenvals(mcap)
eigenvals(mdb)
eigenvals(mancap)
eigenvals(mandb)
eigenvals(m0cca)
eigenvals(m0cca, model = "constrained")

nobs(mcca)
nobs(mrda)
nobs(mcap)
nobs(mdb)
nobs(m0cca)

RsquareAdj(mcca)
RsquareAdj(mrda)
RsquareAdj(mrda1)
RsquareAdj(mcap)
RsquareAdj(mdb)
RsquareAdj(m0cca)

head(model.frame(mcca))
head(model.frame(mrda))
head(model.frame(mrda1))
head(model.frame(mcap))
head(model.frame(mdb))
head(model.frame(m0cca))

## testing and model building -

deviance(mcca)
deviance(mrda)
deviance(mrda1)
deviance(mcap)
deviance(mdb)
deviance(m0cca)

per <- shuffleSet(nrow(dune), 49)
permutest(mcca, per)
permutest(mrda, per)
permutest(mrda1, per)
permutest(mcap, per)
permutest(mdb, per)
permutest(mancap, per)
permutest(mandb, per)
permutest(m0cca, per)

drop1(mcca, test="permutation", permutations=per)
drop1(mrda, test="permutation", permutations=per)
drop1(mrda1, test="permutation", permutations=per)
drop1(mcap, test="permutation", permutations=per)
drop1(mdb, test="permutation", permutations=per)

anova(mcca, permutations = per)
anova(mrda, permutations = per)
anova(mrda1, permutations = per)
anova(mcap, permutations = per)
anova(mdb, permutations = per)
anova(mancap, permutations = per)
anova(mandb, permutations = per)
anova(m0cca, permutations = per)

anova(mcca, permutations = per, by="term")
anova(mrda, permutations = per, by="term")
anova(mrda1, permutations = per, by="term")
anova(mcap, permutations = per, by="term")
anova(mdb, permutations = per, by="term")
anova(mancap, permutations = per, by="term")
anova(mandb, permutations = per, by="term")

anova(mcca, permutations = per, by="margin")
anova(mrda, permutations = per, by="margin")
anova(mrda1, permutations = per, by="margin")
anova(mcap, permutations = per, by="margin")
anova(mdb, permutations = per, by="margin")
anova(mancap, permutations = per, by="margin")
anova(mandb, permutations = per, by="margin")

anova(mcca, permutations = per, by="axis")
anova(mrda, permutations = per, by="axis")
anova(mrda1, permutations = per, by="axis")
anova(mcap, permutations = per, by="axis")
anova(mdb, permutations = per, by="axis")
anova(mancap, permutations = per, by="axis")
anova(mandb, permutations = per, by="axis")

## the following do not all work with partial models

mcca <- cca(dune ~ Management + Manure + A1, dune.env)
mrda <- rda(dune ~ Management + Manure + A1, dune.env)
mrda1 <- rda(dune ~ Management + Manure + A1, dune.env, scale = TRUE)
mcap <- capscale(dune ~ Management + Manure + A1, dune.env, dist = "bray")
mdb <- dbrda(dune ~ Management + Manure + A1, dune.env, dist = "bray")
mancap <- capscale(dune ~ Management + Manure + A1, dune.env, dist = "man")
mandb <- dbrda(dune ~ Management + Manure + A1, dune.env, dist = "man")

head(calibrate(mcca))
head(calibrate(mrda))
head(calibrate(mrda1))
head(calibrate(mcap))
head(calibrate(mdb))
head(calibrate(mancap))
head(calibrate(mandb))

head(calibrate(mcca, newdata=dune[11:15,]))
head(calibrate(mrda, newdata=dune[11:15,]))
head(calibrate(mrda1, newdata=dune[11:15,]))

head(predict(mcca, newdata = dune.env))
predict(mrda, newdata = dune.env[1:4,])
predict(mrda1, newdata = dune.env[1:4,])
predict(mcap, newdata = dune.env[1:4,])
predict(mdb, newdata = dune.env[1:4,])
predict(mancap, newdata = dune.env[1:4,])
predict(mandb, newdata = dune.env[1:4,])

## the sign is arbitrary
abs(predict(mcca, newdata = dune[1:4,], type="wa"))
abs(predict(mrda, newdata = dune[1:4,], type="wa"))
abs(predict(mrda1, newdata = dune[1:4,], type="wa"))

## the sign is arbitrary
abs(predict(mcca, newdata = dune[,1:4], type="sp"))
abs(predict(mrda, newdata = dune[,1:4], type="sp"))
abs(predict(mrda1, newdata = dune[,1:4], type="sp"))

abs(predict(mcca, newdata = dune.env[1:4,], type="lc"))
abs(predict(mrda, newdata = dune.env[1:4,], type="lc"))
abs(predict(mrda1, newdata = dune.env[1:4,], type="lc"))
abs(predict(mcap, newdata = dune.env[1:4,], type="lc"))
abs(predict(mdb, newdata = dune.env[1:4,], type="lc"))
abs(predict(mancap, newdata = dune.env[1:4,], type="lc"))
abs(predict(mandb, newdata = dune.env[1:4,]))
## reset
options(op)

`ade2vegancca` <-
    function(object)
{
    nf <- object$nf
    CCA <- list(eig = object$eig,
                u = as.matrix(object$l1),
                v = as.matrix(object$c1),
                wa = sweep(as.matrix(object$ls), 2,
                           1/sqrt(object$eig[1:nf]), "*"),
                biplot = as.matrix(object$cor)[-1,],
                rank = object$rank,
                tot.chi = sum(object$eig),
                QR = NA,
                envcentre = NA,
                Xbar = NA)
    out <- list(call = object$call,
                grand.total = NA,
                rowsum = object$lw,
                colsum = object$cw,
                tot.chi = NA,
                pCCA = NULL,
                CCA = CCA,
                CA = NULL,
                method = "cca",
                inertia = "mean squared contingency coefficient")
    class(out) = c("ade4cca", "cca")
    out
}

### Implementation of by-cases for anova.cca. These are all internal
### functions that are not intended to be called by users in normal
### sessions, but they should be called from anova.cca. Therefore the
### user interface is rigid and input is not checked. The
### 'permutations' should be a permutation matrix.

### by = "terms" calls directly permutest.cca which decomposes the
### inertia between successive terms within compiled C code.

`anova.ccabyterm` <-
    function(object, permutations, model, parallel)
{
    ## The result
    sol <- permutest(object, permutations = permutations,
                     model = model, by = "terms", parallel = parallel)
    ## Reformat
    EPS <- sqrt(.Machine$double.eps)
    Pval <- (colSums(sweep(sol$F.perm, 2, sol$F.0 - EPS, ">=")) + 1) /
        (sol$nperm + 1)
    out <- data.frame(sol$df, sol$chi, c(sol$F.0, NA), c(Pval, NA))

    if (inherits(object, c("capscale", "dbrda")) && object$adjust == 1)
        varname <- "SumOfSqs"
    else if (inherits(object, "rda"))
        varname <- "Variance"
    else
        varname <- "ChiSquare"
    dimnames(out) <- list(c(sol$termlabels, "Residual"),
                          c("Df", varname, "F", "Pr(>F)"))
    head <- paste0("Permutation test for ", object$method, " under ",
                   model, " model\n",
                   "Terms added sequentially (first to last)\n",
                   howHead(attr(permutations, "control")))
    mod <- paste("Model:", c(object$call))
    attr(out, "heading") <- c(head, mod)
    attr(out, "F.perm") <- sol$F.perm
    class(out) <- c("anova.cca", "anova","data.frame")
    out
}

## by = "margin": we omit each term in turn and compare against the
## complete model. This does not involve partial terms (Conditions) on
## other variables, but the permutations remain similar in "direct"
## and "reduced" (default) models (perhaps this model should not be
## used with "full" models?). This is basically similar decomposition
## as by="term", but compares models without each term in turn against
## the complete model in separate calls to permutest.cca. From vegan
## 2.5-0 this does not update model formula -- this avoids scoping
## issues and makes the function more robust when embedded in other
## functions. Instead, we call ordConstrained with method="pass" with
## modified constraint matrix.

`anova.ccabymargin` <-
    function(object, permutations, scope, ...)
{
    EPS <- sqrt(.Machine$double.eps)
    nperm <- nrow(permutations)
    ## We need term labels but without Condition() terms
    if (!is.null(scope) && is.character(scope))
        trms <- scope
    else
        trms <- drop.scope(object)
    trmlab <- trms[trms %in% attr(terms(object$terminfo),
                                      "term.labels")]
    if (length(trmlab) == 0)
        stop("the scope was empty: no available marginal terms")
    ## baseline: all terms
    big <- permutest(object, permutations, ...)
    dfbig <- big$df[2]
    chibig <- big$chi[2]
    scale <- big$den/dfbig
    ## Collect all marginal models. This differs from old version
    ## (vegan 2.0) where other but 'nm' were partialled out within
    ## Condition(). Now we only fit the model without 'nm' and compare
    ## the difference against the complete model.
    Y <- ordiYbar(object, "init")
    X <- model.matrix(object)
    ## we must have Constraints to get here, but we may also have
    ## Conditions
    if (!is.null(object$pCCA)) {
        Z <- X$Conditions
        X <- X$Constraints
    } else {
        Z <- NULL
    }
    ass <- object$terminfo$assign
    if (is.null(ass))
        stop("old style result object: update() your model")
    ## analyse only terms of 'ass' thar are in scope
    scopeterms <- which(attr(terms(object$terminfo), "term.labels") %in% trms)
    mods <- lapply(scopeterms, function(i, ...)
           permutest(ordConstrained(Y, X[, ass != i, drop=FALSE], Z, "pass"),
                     permutations, ...), ...)
    ## Chande in df
    Df <- sapply(mods, function(x) x$df[2]) - dfbig
    ## F of change
    Chisq <- sapply(mods, function(x) x$chi[2]) - chibig
    Fstat <- (Chisq/Df)/(chibig/dfbig)
    ## Simulated F-values
    Fval <- sapply(mods, function(x) x$num)
    ## Had we an empty model we need to clone the denominator
    if (length(Fval) == 1)
        Fval <- matrix(Fval, nrow = nperm)
    Fval <- sweep(-Fval, 1, big$num, "+")
    Fval <- sweep(Fval, 2, Df, "/")
    Fval <- sweep(Fval, 1, scale, "/")
    ## Simulated P-values
    Pval <- (colSums(sweep(Fval, 2, Fstat - EPS, ">=")) + 1)/(nperm + 1)
    ## Collect results to anova data.frame
    out <- data.frame(c(Df, dfbig), c(Chisq, chibig),
                      c(Fstat, NA), c(Pval, NA))
    if (inherits(object, c("capscale", "dbrda")) && object$adjust == 1)
        varname <- "SumOfSqs"
    else if (inherits(object, "rda"))
        varname <- "Variance"
    else
        varname <- "ChiSquare"
    dimnames(out) <- list(c(trmlab, "Residual"),
                          c("Df", varname, "F", "Pr(>F)"))
    head <- paste0("Permutation test for ", object$method, " under ",
                   mods[[1]]$model, " model\n",
                   "Marginal effects of terms\n",
                   howHead(attr(permutations, "control")))
    mod <- paste("Model:", c(object$call))
    attr(out, "heading") <- c(head, mod)
    attr(out, "F.perm") <- Fval
    class(out) <- c("anova.cca", "anova", "data.frame")
    out
}

### by = "axis" uses partial model: we use the original constraints,
### but add previous axes 1..(k-1) to Conditions when evaluating the
### significance of axis k which is compared against the first
### eigenvalue of the permutations. To avoid scoping issues, this
### calls directly ordConstrained() with modified Conditions (Z) and
### original Constraints (X) instead of updating formula. This
### corresponds to "forward" model in Legendre, Oksanen, ter Braak
### (2011).

### In 2.2-x to 2.4-3 we used "marginal model" where original
### Constraints were replaced with LC scores axes (object$CCA$u), and
### all but axis k were used as Conditions when evaluating the
### significance of axis k. My (J.Oksanen) simulations showed that
### this gave somewhat biased results.

`anova.ccabyaxis` <-
    function(object, permutations, model, parallel, cutoff = 1)
{
    EPS <- sqrt(.Machine$double.eps)
    ## On 29/10/15 (983ba7726) we assumed that dbrda(d ~ dbrda(d ~
    ## x)$CCA$u) is not equal to dbrda(d ~ x) when there are negative
    ## eigenvalues, but it seems that it is OK if constrained
    ## eigenvalues are non-negative
    if (inherits(object, "dbrda") && any(object$CCA$eig < 0))
        stop("by = 'axis' cannot be used when constraints have negative eigenvalues")
    nperm <- nrow(permutations)
    ## Observed F-values and Df
    eig <- object$CCA$eig
    resdf <- nobs(object) - length(eig) - max(object$pCCA$QR$rank, 0) - 1
    Fstat <- eig/object$CA$tot.chi*resdf
    Df <- rep(1, length(eig))

    ## collect header and varname here: 'object' is modified later
    if (inherits(object, c("capscale", "dbrda")) && object$adjust == 1)
        varname <- "SumOfSqs"
    else if (inherits(object, "rda"))
        varname <- "Variance"
    else
        varname <- "ChiSquare"

    head <- paste0("Permutation test for ", object$method, " under ",
                   model, " model\n",
                   "Forward tests for axes\n",
                   howHead(attr(permutations, "control")))
    head <- c(head, paste("Model:", c(object$call)))

    ## constraints and model matrices
    Y <- object$Ybar
    if (is.null(Y))
        stop("old style result object: update() your model")
    if (!is.null(object$pCCA))
        Z <- qr.X(object$pCCA$QR)
    else
        Z <- NULL
    X <- model.matrix(object)
    if (!is.null(object$pCCA)) {
        Z <- X$Conditions
        X <- X$Constraints
    } else {
        Z <- NULL
    }
    LC <- object$CCA$u

    Pvals <- rep(NA, ncol(LC))
    F.perm <- matrix(ncol = ncol(LC), nrow = nperm)
    axnams <- colnames(LC)
    for (i in seq_along(eig)) {
        if (i > 1) {
            object <- ordConstrained(Y, X, cbind(Z, LC[, seq_len(i-1)]), "pass")
        }
        if (length(eig) == i) {
            mod <- permutest(object, permutations, model = model,
                             parallel = parallel)
        } else {
            mod <- permutest(object, permutations, model = model,
                             parallel = parallel, first = TRUE)
        }
        Pvals[i] <- (sum(mod$F.perm >= mod$F.0 - EPS) + 1) / (nperm + 1)
        F.perm[ , i] <- mod$F.perm
        if (Pvals[i] > cutoff)
            break
    }
    out <- data.frame(c(Df, resdf), c(eig, object$CA$tot.chi),
                      c(Fstat, NA), c(Pvals,NA))
    rownames(out) <- c(names(eig), "Residual")
    colnames(out) <- c("Df", varname, "F", "Pr(>F)")
    attr(out, "heading") <- head
    attr(out, "F.perm") <- F.perm
    class(out) <- c("anova.cca", "anova", "data.frame")
    out
}

`pco` <- function(X, ...) {
  if (inherits(X, "formula")) {
    stop("Argument 'X' was supplied a formula, which is not supported by 'pco()'",
    call. = FALSE)
  }
  ord <- dbrda(X ~ 1, ...)
  # change the call to be from pco()
  ord$call <- match.call()
  class(ord) <- append(class(ord), "vegan_pco", after = 0)
  ord
}

### Internal function for double centring of a *matrix* of
### dissimilarities. We used .C("dblcen", ..., PACKAGE = "stats")
### which does not dublicate its argument, but it was removed from R
### in r60360 | ripley | 2012-08-22 07:59:00 UTC (Wed, 22 Aug 2012)
### "more conversion to .Call, clean up". Input 'x' *must* be a
### matrix. This was originally an internal function in betadisper.R
### (commit 7cbd4529 Thu Aug 23 08:45:31 2012 +0000)
GowerDblcen <- function(x, na.rm = TRUE)
{
    cnt <- colMeans(x, na.rm = na.rm)
    x <- sweep(x, 2L, cnt, check.margin = FALSE)
    cnt <- rowMeans(x, na.rm = na.rm)
    sweep(x, 1L, cnt, check.margin = FALSE)
}

### Internal functions to find additive constants to non-diagonal
### dissimilarities so that there are no negative eigenvalues. The
### Cailliez constant is added to dissimilarities and the Lingoes
### constant is added to squared dissimilarities. Legendre & Anderson
### (Ecol Monogr 69, 1-24; 1999) recommend Lingoes, but
### stats::cmdscale() only provides Cailliez. Input parameters: d are
### a matrix of dissimilarities.

addCailliez <- function(d)
{
    n <- nrow(d)
    q1 <- seq_len(n)
    q2 <- n + q1
    ## Cailliez makes a 2x2 block matrix with blocks of n x n elements.
    ## Blocks anti-clockwise, upper left [0]
    z <- matrix(0, 2*n, 2*n)
    diag(z[q2,q1]) <- -1
    z[q1,q2] <- -GowerDblcen(d^2)
    z[q2,q2] <- GowerDblcen(2 * d)
    ## Largest real eigenvalue
    e <- eigen(z, symmetric = FALSE, only.values = TRUE)$values
    out <- max(Re(e))
    max(out, 0)
}

addLingoes <- function(d)
{
    ## smallest negative eigenvalue (or zero)
    d <- -GowerDblcen(d^2)/2
    e <- eigen(d, symmetric = TRUE, only.values = TRUE)$values
    out <- min(e)
    max(-out, 0)
}

### Centres and areas of convex hulls (simple polygons).
`summary.ordihull` <-
    function(object, ...)
{
    polyarea <- function(x) {
        n <- nrow(x)
        if (n < 4)
            return(0)
        else
            abs(sum(x[-n,1]*x[-1,2] - x[-1,1]*x[-n,2]))/2
    }
    polycentre <- function(x) {
        n <- nrow(x)
        if (n < 4)
            return(colMeans(x[-n,, drop = FALSE]))
        xy <- x[-n,1]*x[-1,2] - x[-1,1]*x[-n,2]
        A <- sum(xy)/2
        xc <- sum((x[-n,1] + x[-1,1]) * xy)/A/6
        yc <- sum((x[-n,2] + x[-1,2]) * xy)/A/6
        structure(c(xc, yc), names = colnames(x))
    }
    areas <- sapply(object, function(x) polyarea(x))
    cnts <- sapply(object, function(x) polycentre(x))
    rbind(cnts, `Area` = areas)
}

"print.mrpp" <-
function (x, digits = max(3, getOption("digits") - 3), ...) 
{
### A print function for mrpp objects
### x -- An object of class "mrpp."
#### cat = print
    cat("\nCall:\n")
    cat(deparse(x$call), "\n\n") 
    cat("Dissimilarity index:", x$distance, "\n")
    cat("Weights for groups: ", switch(x$weight.type, "n", "n-1", "n(n-1)", "n(n-1)/2"), "\n\n")
    cat("Class means and counts:\n\n")
    print(noquote(rbind("delta" = formatC(x$classdelta, digits = digits),
                        "n" = formatC(x$n, digits=0))))
    cat("\n")
    if (!is.na(x$CS)) {
        cat("Classification strength: ")
        cat(formatC(x$CS, digits = digits), "\n")
    }
    cat("Chance corrected within-group agreement A: ")
    if (!is.na(x$A))
        cat(formatC(x$A, digits = digits), "\n")
    else
        cat("NA\n")
    cat("Based on observed delta", formatC(x$delta), "and expected delta",
        formatC(x$E.delta),"\n\n")
    nperm <- x$permutations
    if (nperm) {
        cat("Significance of delta:", format.pval(x$Pvalue), 
            "\n")
    }
    cat(howHead(x$control))
    cat("\n")
    invisible(x)
}

`plot.poolaccum` <-
    function(x, alpha = 0.05, type = c("l","g"), ...)
{
    m <- summary(x, alpha = alpha, ...)
    n <- nrow(m[[1]])
    Size <- as.vector(sapply(m, function(x) c(x[,1], x[,1], rev(x[,1]))))
    Richness <- as.vector(sapply(m, function(x) c(x[,2], x[,3], rev(x[,4]))) )
    indnames <- as.character(sapply(m, function(x) colnames(x[,2, drop=FALSE])))
    Index <- factor(rep(indnames, each = 3 * n), levels = indnames)
    lintype <- rep(c(rep("aver", n), rep("envel", 2*n)), length=length(Size))
    xyplot(Richness ~  Size | Index, as.table = TRUE, groups = lintype,
           type = type, ...)
}

### vegan-tests: unit tests for vegan functions

### This file contains unit tests for vegan functions. This file is
### run in R CMD check and its results are compared against previously
### saved results in vegan-tests.Rout.save. If you change tests, you
### must generate new vegan-tests.Rout.save in this directory.

### The current plan is that tests/ are not included in the CRAN
### release, but only in the development versin of vegan in R-Forge.

### The tests here are not intended for human reading. The tests need
### not be ecological or biologically meaningful, but they are only
### intended for testing strange arguments, protect against
### regressions and test correctness of results.

### The tests are in a single file and you should clean work space
### after the unit test. You should set random number seed (if needed)
### for comparison against vegan-tests.Rout.save, and you should
### remove the seed after the unit test. If possible, avoid very long
### lasting tests.

###<--- BEGIN TESTS --->
suppressPackageStartupMessages(require(vegan))
###<--- BEGIN anova.cca test --->
### anova.cca tests: should work with (1) subset, (2) missing values,
### (3) with functions of variables poly(A1,2), (4) variables in data
### frame attached or in data=, or (5) in working environment
set.seed(4711)
data(dune)
data(dune.env)
df <- dune.env
df$Management[c(1,5)] <- NA
## formula
fla <- as.formula("dune ~ Management + poly(A1, 2) + spno")
### variable in the .GlobalEnv
spno <- specnumber(dune)
### data= argument
## cca/rda
m <-  cca(fla, data=df,  na.action=na.exclude,  subset = Use != "Pasture" & spno > 7)
anova(m, permutations=99)
anova(m, by="term", permutations=99) # failed before 2.5-0
anova(m, by="margin", permutations=99) # works since 2.5-0
anova(m, by="axis", permutations=99)
## capscale
p <- capscale(fla, data=df, na.action=na.exclude, subset = Use != "Pasture" & spno > 7)
anova(p, permutations=99)
anova(p, by="term", permutations=99) # failed before 2.5-0
anova(p, by="margin", permutations=99) # works since 2.5-0
anova(p, by="axis", permutations=99)
## see that capscale can be updated and also works with 'dist' input
dis <- vegdist(dune)
p <- update(p, dis ~ .)
anova(p, permutations=99)
anova(p, by="term", permutations=99) # failed before 2.5-0
anova(p, by="margin", permutations=99) # works since 2.5-0
anova(p, by="axis", permutations=99)
### attach()ed data frame instead of data=
attach(df)
q <- cca(fla, na.action = na.omit, subset = Use != "Pasture" & spno > 7)
anova(q, permutations=99)
## commented tests below fail in vegan 2.1-40 because number of
## observations changes
anova(q, by="term", permutations=99) # failed before 2.5-0
anova(q, by="margin", permutations=99) # works since 2.5-0
anova(q, by="axis", permutations=99)
### Check that constrained ordination functions can be embedded.
### The data.frame 'df' is still attach()ed.
foo <- function(bar, Y, X, ...)
{
    bar <- match.fun(bar)
    bar(Y ~ X, ...)
}
foo("cca", dune, Management, na.action = na.omit)
foo("rda", dune, Management, na.action = na.omit)
foo("capscale", dune, Management, dist="jaccard", na.action = na.omit)
foo("capscale", vegdist(dune), Management, na.action = na.omit)
foo("capscale", dune, Management, na.action = na.omit) ## fails in 2.2-1
###
detach(df)
### Check that statistics match in partial constrained ordination
m <- cca(dune ~ A1 + Moisture + Condition(Management), dune.env, subset = A1 > 3)
tab <- anova(m, by = "axis", permutations = 99)
m
tab
all.equal(tab[,2], c(m$CCA$eig, m$CA$tot.chi), check.attributes=FALSE)
tab[nrow(tab),1] == m$CA$rank
## clean-up
rm(df, spno, fla, m, p, q, tab, dis, foo, .Random.seed)
### <--- END anova.cca test --->

### Sven Neulinger <sneulinger@ifam.uni-kiel.de> reported failures in
### partial analysis which (mostly) were fixed in r2087. Below his test.

set.seed(4711)
X <- matrix(rnorm(30*6), 30, 6)

A <- factor(rep(rep(c("a","b"), each=3),5))
B <- factor(rep(c("a","b","c"), 10))
## Sven Neulinger's tests failed still in 2.2-1, now due to look-up
## order: function stats::C was found before matrix 'C'. The test was
## OK when non-function name was used ('CC').
C <- factor(rep(c(1:5), each=6))

## partial db-RDA
cap.model.cond <- capscale(X ~ A + B + Condition(C))
anova(cap.model.cond, by="axis", strata=C)  # -> error pre r2287
anova(cap.model.cond, by="terms", strata=C)  # -> error pre r2287

## db-RDA without conditional factor
cap.model <- capscale(X ~ A + B)
anova(cap.model, by="axis", strata=C)  # -> no error
anova(cap.model, by="terms", strata=C)  # -> no error

# partial RDA
rda.model.cond <- rda(X ~ A + B + Condition(C))
anova(rda.model.cond, by="axis", strata=C)  # -> no error
anova(rda.model.cond, by="terms", strata=C)  # -> error pre r2287

# RDA without conditional factor
rda.model <- rda(X ~ A + B)
anova(rda.model, by="axis", strata=C)  # -> no error
anova(rda.model, by="terms", strata=C)  # -> no error
## clean.up
rm(X, A, B, C, cap.model.cond, cap.model, rda.model.cond, rda.model)
### end Sven Neulinger's tests

### Benedicte Bachelot informed us that several anova.cca* functions
### failed if community data name was the same as a function name: the
### function name was found first, and used instead ofa data. This
### seems to be related to the same problem that Sven Neulinger
### communicated, and his examples still faile if Condition or strata
### are function names. However, the following examples that failed
### should work now:

set.seed(4711)
cca <- dune
m <- cca(cca ~ ., dune.env)
anova(m, by="term")
m <- capscale(cca ~ ., dune.env)
anova(m, by="term")
rm(m, cca)

### end Benedicte Bachelot tests

### Richard Telford tweeted this example on 23/2/2015. Fails in 2.2-1,
### but should work now. Also issue #100 in github.com/vegandevs/vegan.
set.seed(4711)
data(dune, dune.env)
foo <- function(x, env) {
    m <- rda(x ~ Manure + A1, data = env)
    anova(m, by = "margin")
}
out <- lapply(dune, foo, env = dune.env)
out$Poatriv
rm(foo, out)
### end Richard Telford test


### nestednodf: test case by Daniel Spitale in a comment to News on
### the release of vegan 1.17-6 in vegan.r-forge.r-project.org.
x <- c(1,0,1,1,1,1,1,1,0,0,0,1,1,1,0,1,1,0,0,0,1,1,0,0,0)
m1 <- matrix(x, nrow=5, ncol=5, byrow=FALSE)# as in Fig 2 Almeida-Neto et al 2008.
(nodf1 <- nestednodf(m1, order = FALSE, weighted = FALSE))
## Now the same matrix but with abundance data
x <- c(5,0,2,1,1,4,1,1,0,0,0,7,1,1,0,3,1,0,0,0,9,1,0,0,0)
m <- matrix(x, nrow=5, ncol=5, byrow=FALSE)
(nodfq <- nestednodf(m, order = FALSE, weighted = FALSE))
identical(nodf1, nodfq)
rm(x, m, m1, nodfq, nodf1)
### end nestednodf

### envfit & plot.envfit: latter failed if na.action resulted in only
### observation with a given factor level was removed. plot.envfit would
### fail with error about too long subscript
### fixed case where data presented to envfit also has extraneous levels
data(dune)
data(dune.env)
## add a new level to one of the factors
levels(dune.env$Management) <- c(levels(dune.env$Management), "foo")
## fit nMDS and envfit
set.seed(1)
mod <- metaMDS(dune, trace = 0)
ef <- envfit(mod, dune.env, permutations = 99)
plot(mod)
plot(ef, p.max = 0.1)
rm(mod, ef)
### end envfit & plot.envfit

### protest (& Procrustes analysis): Stability of the permutations and
### other results.
data(mite)
mod <- rda(mite)
x <- scores(mod, display = "si", choices=1:6)
set.seed(4711)
xp <- x[sample(nrow(x)),]
pro <- protest(x, xp, permutations = how(nperm = 99))
pro
pro$t
rm(x, xp, pro)
### end protest

### Check that functions related to predict.rda work correctly for all
### constrained ordination methods.

### simulate.rda/cca/capscale: based on predict.* and the following
### should get back the data
data(dune, dune.env)
ind <- seq_len(nrow(dune))
target <- as.matrix(dune)
## rda
mod <- rda(dune ~ Condition(Moisture) + Management + A1, dune.env)
dat <- simulate(mod, indx = ind)
all.equal(dat, target, check.attributes = FALSE)
## cca
mod <- cca(dune ~ Condition(Moisture) + Management + A1, dune.env)
dat <- simulate(mod, indx = ind)
all.equal(dat, target, check.attributes = FALSE)
## capscale: Euclidean distances -- non-Euclidean distances have an
## imaginary component and will not give back the data.
d <- dist(dune)
mod <- capscale(d ~ Condition(Moisture) + Management + A1, dune.env)
dat <- simulate(mod, indx = ind)
all.equal(dat, d, check.attributes = FALSE)
## clean up
rm(ind, target, mod, dat, d)
### end simulate.*

### test metaMDS works with long expression for comm
### originally reported to GLS by Richard Telford
data(varespec)
set.seed(1)
mod <- metaMDS(subset(varespec, select = colSums(varespec) > 0, subset = rowSums(varespec) > 0), trace=0)
mod
rm(mod)
### The above should run without error & last lines tests changes to the
### printed output

## dbrda tests

## the following three should be all equal
data(varespec, varechem)
(mr <- rda(varespec ~ Al + P + Condition(pH), varechem))
(md <- dbrda(varespec ~ Al + P + Condition(pH), varechem))
(mc <- capscale(varespec ~ Al + P + Condition(pH), varechem))
## the following two should be zero (within 1e-15)
p <- shuffleSet(nrow(varespec), 999)
all(abs(permustats(anova(mr, permutations=p))$permutations -
        permustats(anova(md, permutations=p))$permutations)
             < sqrt(.Machine$double.eps))

all(abs(permustats(anova(mr, permutations=p))$permutations -
        permustats(anova(mc, permutations=p))$permutations)
             < sqrt(.Machine$double.eps))
## eigenvals returns a list now (>= 2.5-0)
data(varespec, varechem)
mod <- cca(varespec ~ Al + P + Condition(pH), varechem)
ev <- summary(eigenvals(mod))
stopifnot(inherits(ev, "matrix"))
stopifnot(!is.list(ev))
ev

`text.cca` <-
    function (x, display = "sites", labels, choices = c(1, 2), scaling = "species",
              arrow.mul, head.arrow = 0.05, select, const, axis.bp = FALSE,
              correlation = FALSE, hill = FALSE, ...)
{
    formals(arrows) <- c(formals(arrows), alist(... = ))
    if (length(display) > 1)
        stop("only one 'display' item can be added in one command")
    pts <- scores(x, choices = choices, display = display, scaling = scaling,
                  const, correlation = correlation, hill = hill)
    ## store rownames of pts for use later, otherwise if user supplies
    ## labels, the checks in "cn" branch fail and "bp" branch will
    ## be entered even if there should be no "bp" plotting
    cnam <- rownames(pts)
    if (missing(labels))
        labels <- labels.cca(x, display)
    if (!missing(select)) {
        pts <- .checkSelect(select, pts)
        labels <- labels[select]
    }
    ## centroids ("cn") have special treatment: also plot biplot
    ## arrows ("bp") for continuous variables and ordered factors.
    if (display == "cn") {
        cnlabs <- seq_len(nrow(pts))
        text(pts, labels = labels[cnlabs], ...)
        pts <- scores(x, choices = choices, display = "bp", scaling = scaling,
                      const, correlation = correlation, hill = hill)
        bnam <- rownames(pts)
        pts <- pts[!(bnam %in% cnam), , drop = FALSE]
        if (nrow(pts) == 0)
            return(invisible())
        else {
            display <- "bp"
            labels <- labels[-cnlabs]
        }
    }
    ## draw arrows before adding labels
    if (display %in% c("bp", "reg", "re", "r")) {
        if (missing(arrow.mul)) {
            arrow.mul <- ordiArrowMul(pts)
        }
        pts <- pts * arrow.mul
        arrows(0, 0, pts[, 1], pts[, 2], length = head.arrow,
               ...)
        pts <- ordiArrowTextXY(pts, labels, rescale = FALSE, ...)
        if (axis.bp) {
            axis(side = 3, at = c(-arrow.mul, 0, arrow.mul),
                 labels = rep("", 3))
            axis(side = 4, at = c(-arrow.mul, 0, arrow.mul),
                 labels = c(-1, 0, 1))
        }
    }
    text(pts, labels = labels, ...)
    invisible()
}

### utility function to extract labels used in CCA/RDA/dbRDA plots:
### you may need this if you want to set your own labels=.

`labels.cca` <-
    function(object, display, ...)
{
    if (is.null(object$CCA))
        CCA <- "CA"
    else
        CCA <- "CCA"
    switch(display,
           "sp" =,
           "species" = rownames(object[[CCA]]$v),
           "wa" =,
           "sites" =,
           "lc" = rownames(object[[CCA]]$u),
           "reg" = colnames(object[[CCA]]$QR$qr),
           "bp" = rownames(object[[CCA]]$biplot),
           "cn" = {cn <- rownames(object[[CCA]]$centroids)
                   bp <- rownames(object[[CCA]]$biplot)
                   c(cn, bp[!(bp %in% cn)]) }
           )
}

`plot.radfit` <-
    function (x, BIC = FALSE, legend = TRUE, ...)
{
    if (length(x$y) == 0)
        stop("no species, nothing to plot")
    ## if 'type = "n"', do not add legend (other types are not
    ## supported)
    type <- match.call(expand.dots = FALSE)$...$type
    if (is.null(type))
        type <- ""
    out <- plot(x$y, ...)
    if (length(x$y) == 1)
        return(invisible(out))
    fv <- fitted(x)
    if (BIC)
        k = log(length(x$y))
    else k = 2
    emph <- which.min(sapply(x$models, AIC, k = k))
    lwd <- rep(1, ncol(fv))
    lwd[emph] <- 3
    matlines(fv, lty = 1, lwd = lwd, ...)
    if (legend && type != "n") {
        nm <- names(x$models)
        legend("topright", legend = nm, lty = 1, lwd = lwd, col = 1:6)
    }
    invisible(out)
}

"lines.prestonfit" <-
    function(x, line.col = "red", lwd = 2, ...)
{
    p <- x$coefficients
    freq <- x$freq
    oct <- as.numeric(names(freq))
    curve(p[3] * exp(-(x-p[1])^2/2/p[2]^2), -1, max(oct), add = TRUE,
          col = line.col, lwd = lwd, ...)
        segments(p["mode"], 0, p["mode"], p["S0"], col = line.col, 
        ...)
    segments(p["mode"] - p["width"], p["S0"] * exp(-0.5), p["mode"] + 
        p["width"], p["S0"] * exp(-0.5), col = line.col, ...)
    invisible()
}

`dispweight` <-
    function(comm, groups, nsimul = 999, nullmodel = "c0_ind",
             plimit = 0.05)
{
    ## no groups?
    if (missing(groups))
        groups <- rep(1, nrow(comm))
    ## Remove empty levels of 'groups' or this fails cryptically (and
    ## take care 'groups' is a factor)
    groups <- factor(groups)
    ## Statistic is the sum of squared differences by 'groups'
    means <- apply(comm, 2, function(x) tapply(x, groups, mean))
    ## handle 1-level factors: all sites belong to the same 'groups'
    if (is.null(dim(means)))
        means <- matrix(means, nrow=1, ncol = length(means),
                        dimnames = list(levels(groups), names(means)))
    ## expand to matrix of species means
    fitted <- means[groups,]
    dhat <- colSums((comm - fitted)^2/fitted, na.rm = TRUE)
    ## Get df for non-zero blocks of species. Completely ignoring
    ## all-zero blocks for species sounds strange, but was done in the
    ## original paper, and we follow here. However, this was not done
    ## for significance tests, and only concerns 'D' and 'weights'.
    nreps <- table(groups)
    div <- colSums(sweep(means > 0, 1, nreps - 1, "*"))
    ## "significance" of overdispersion is assessed from Chi-square
    ## evaluated separately for each species. This means fixing only
    ## marginal totals for species but letting row marginals vary
    ## freely, unlike in standard Chi-square where both margins are
    ## fixed. In vegan this is achieved by nullmodel 'c0_ind'. Instead
    ## of one overall simulation, nullmodel is generated separately
    ## for each of 'groups'
    chisq <- function(x) {
        fitted <- colMeans(x)
        colSums(sweep(x, 2, fitted)^2, na.rm = TRUE) / fitted
    }
    simulated <- matrix(0, nrow = ncol(comm), ncol = nsimul)
    for (lev in levels(groups)) {
        nm <- nullmodel(comm[groups == lev,], nullmodel)
        if (nm$commsim$binary)
            stop("'binary' nullmodel cannot be used")
        tmp <- apply(simulate(nm, nsimul), 3, chisq)
        ok <- !is.na(tmp)
        simulated[ok] <- simulated[ok] + tmp[ok] 
    }
    ## p value based on raw dhat, then we divide
    p <- (rowSums(dhat <= simulated) + 1) / (nsimul + 1)
    dhat <- dhat/div
    weights <- ifelse(p <= plimit, 1/dhat, 1)
    comm <- sweep(comm, 2, weights, "*")
    attr(comm, "D") <- dhat
    attr(comm, "df") <- div
    attr(comm, "p") <- p
    attr(comm, "weights") <-  weights
    attr(comm, "nsimul") <- nsimul
    attr(comm, "nullmodel") <- nullmodel
    class(comm) <- c("dispweight", class(comm))
    comm
}

`scores.cca` <-
    function (x, choices = c(1, 2), display = c("sp", "wa", "cn"),
              scaling = "species", hill = FALSE, ...)
{
    if(inherits(x, "pcaiv")) {
        warning("looks like ade4::cca object: you better use ade4 functions")
        x <- ade2vegancca(x)
    }
    ## Check the na.action, and pad the result with NA or WA if class
    ## "exclude"
    if (!is.null(x$na.action) && inherits(x$na.action, "exclude"))
        x <- ordiNApredict(x$na.action, x)
    tabula <- c("species", "sites", "constraints", "biplot",
                "centroids")
    names(tabula) <- c("sp", "wa", "lc", "bp", "cn")
    if (is.null(x$CCA))
        tabula <- tabula[1:2]
    display <- match.arg(display, c("sites", "species", "wa",
                                    "lc", "bp", "cn"),
                         several.ok = TRUE)
    if("sites" %in% display)
        display[display == "sites"] <- "wa"
    if("species" %in% display)
        display[display == "species"] <- "sp"
    take <- tabula[display]
    slam <- sqrt(c(x$CCA$eig, x$CA$eig)[choices])
    rnk <- x$CCA$rank
    sol <- list()
    ## process scaling; numeric scaling will just be returned as is
    scaling <- scalingType(scaling = scaling, hill = hill)
    if ("species" %in% take) {
        v <- cbind(x$CCA$v, x$CA$v)[, choices, drop = FALSE]
        if (scaling) {
            scal <- list(1, slam, sqrt(slam))[[abs(scaling)]]
            v <- sweep(v, 2, scal, "*")
            if (scaling < 0) {
                scal <- sqrt(1/(1 - slam^2))
                v <- sweep(v, 2, scal, "*")
            }
        }
        sol$species <- v
    }
    if ("sites" %in% take) {
        wa <- cbind(x$CCA$wa, x$CA$u)[, choices, drop = FALSE]
        if (scaling) {
            scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
            wa <- sweep(wa, 2, scal, "*")
            if (scaling < 0) {
                scal <- sqrt(1/(1 - slam^2))
                wa <- sweep(wa, 2, scal, "*")
            }
        }
        sol$sites <- wa
    }
    if ("constraints" %in% take) {
        u <- cbind(x$CCA$u, x$CA$u)[, choices, drop = FALSE]
        if (scaling) {
            scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
            u <- sweep(u, 2, scal, "*")
            if (scaling < 0) {
                scal <- sqrt(1/(1 - slam^2))
                u <- sweep(u, 2, scal, "*")
            }
        }
        sol$constraints <- u
    }
    if ("biplot" %in% take && !is.null(x$CCA$biplot)) {
        b <- matrix(0, nrow(x$CCA$biplot), length(choices))
        b[, choices <= rnk] <- x$CCA$biplot[, choices[choices <=
            rnk]]
        colnames(b) <- c(colnames(x$CCA$u), colnames(x$CA$u))[choices]
        rownames(b) <- rownames(x$CCA$biplot)
        sol$biplot <- b
    }
    if ("centroids" %in% take) {
        if (is.null(x$CCA$centroids))
            sol$centroids <- NA
        else {
            cn <- matrix(0, nrow(x$CCA$centroids), length(choices))
            cn[, choices <= rnk] <- x$CCA$centroids[, choices[choices <=
                 rnk]]
            colnames(cn) <- c(colnames(x$CCA$u), colnames(x$CA$u))[choices]
            rownames(cn) <- rownames(x$CCA$centroids)
            if (scaling) {
                scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
                cn <- sweep(cn, 2, scal, "*")
                if (scaling < 0) {
                    scal <- sqrt(1/(1 - slam^2))
                    cn <- sweep(cn, 2, scal, "*")
                }
            }
            sol$centroids <- cn
        }
    }
    ## Take care that scores have names
    if (length(sol)) {
        for (i in seq_along(sol)) {
            if (is.matrix(sol[[i]]))
                rownames(sol[[i]]) <-
                    rownames(sol[[i]], do.NULL = FALSE,
                             prefix = substr(names(sol)[i], 1, 3))
        }
    }
    ## Only one type of scores: return a matrix instead of a list
    if (length(sol) == 1)
        sol <- sol[[1]]
    sol
}

`ordisurf` <-
    function(...) UseMethod("ordisurf")

`ordisurf.formula` <-
    function(formula, data, ...)
{
    if (missing(data))
        data <- parent.frame()
    x <- formula[[2]]
    x <- eval.parent(x)
    formula[[2]] <- NULL
    y <- drop(as.matrix(model.frame(formula, data, na.action = na.pass)))
    if (NCOL(y) > 1)
        stop(gettextf("only one fitted variable allowed in the formula"))
    ordisurf(x, y, ...)
}

`ordisurf.default` <-
    function (x, y, choices = c(1, 2), knots = 10, family = "gaussian",
              col = "red", isotropic = TRUE, thinplate = TRUE, bs = "tp",
              fx = FALSE, add = FALSE, display = "sites",
              w = weights(x, display), main, nlevels = 10, levels,
              npoints = 31, labcex = 0.6, bubble = FALSE, cex = 1,
              select = TRUE, method = "REML", gamma = 1, plot = TRUE,
              lwd.cl = par("lwd"), ...)
{
    weights.default <- function(object, ...) NULL
    if(!missing(thinplate)) {
        warning("use of 'thinplate' is deprecated and will soon be removed;\nuse 'isotropic' instead")
        isotropic <- thinplate
    }
    ## GRID no user-definable - why 31?
    GRID <- npoints
    w <- eval(w)
    if (!is.null(w) && length(w) == 1)
        w <- NULL
    X <- scores(x, choices = choices, display = display, ...)
    ## The original name of 'y' may be lost in handling NA: save for
    ## plots
    yname <- deparse(substitute(y))
    kk <- complete.cases(X) & !is.na(y)
    if (!all(kk)) {
        X <- X[kk, , drop = FALSE]
        y <- y[kk]
        w <- w[kk]
    }
    x1 <- X[, 1]
    x2 <- X[, 2]
    ## handle fx - allow vector of length up to two
    if(!(missfx <- missing(fx)) && missing(knots))
        warning("requested fixed d.f. splines but without specifying 'knots':\nswitching to 'fx = FALSE'")
    if (length(fx) > 2L)
        warning("length of 'fx' supplied exceeds '2': using the first two")
    ## expand fx robustly, no matter what length supplied
    fx <- rep(fx, length.out = 2)
    ## can't have `fx = TRUE` and `select = TRUE`
    if(!missfx) { ## fx set by user
        if((miss.select <- missing(select)) && any(fx)) {
            warning("'fx = TRUE' requested; using 'select = FALSE'")
            select <- FALSE
        } else if(!miss.select && isTRUE(select)){
            stop("fixed d.f. splines ('fx = TRUE') incompatible with 'select = TRUE'")
        }
    }
    ## handle knots - allow vector of length up to two
    if (length(knots) > 2L)
        warning("length of 'knots' supplied exceeds '2': using the first two")
    ## expand knots robustly, no matter what length supplied
    knots <- rep(knots, length.out = 2)
    ## handle the bs - we only allow some of the possible options
    if (length(bs) > 2L)
        warning("number of basis types supplied exceeds '2': using the first two")
    bs <- rep(bs, length.out = 2)
    ## check allowed types
    BS <- c("tp","ts","cr","cs","ds","ps","ad")
    want <- match(bs, BS)
    user.bs <- bs ## store supplied (well expanded supplied ones)
    bs <- BS[want]
    if (any(wrong <- is.na(bs))) {
        stop(gettextf("supplied basis type of '%s' not supported",
                   paste(unique(user.bs[wrong]), collapse = ", ")))
    }
    ## can't use "cr", "cs", "ps" in 2-d smoother with s()
    if(isTRUE(isotropic) && any(bs %in% c("cr", "cs", "ps"))) {
        stop("bases \"cr\", \"cs\", and \"ps\" not allowed in isotropic smooths")
    }
    ## Build formula
    if (knots[1] <= 0) {
        f <- formula(y ~ x1 + x2)
    } else if (knots[1] == 1) { ## why do we treat this differently?
        f <- formula(y ~ poly(x1, 1) + poly(x2, 1))
    } else if (knots[1] == 2) {
        f <- formula(y ~ poly(x1, 2) + poly(x2, 2) + poly(x1, 1):poly(x2, 1))
    } else if (isotropic) {
        f <- formula(paste0("y ~ s(x1, x2, k = ", knots[1],
                            ", bs = \"", bs[1], "\", fx = ", fx[1],")"))
    } else {
        if (any(bs %in% c("ad"))) {
            ## only "ad" for now, but "fs" should also not be allowed
            f <- formula(paste0("y ~ s(x1, k = ", knots[1],
                                ", bs = \"", bs[1],
                                "\", fx = ", fx[1], ") + s(x2, k = ",
                                knots[2], ", bs = \"", bs[2],
                                "\", fx = ", fx[2], ")"))
        } else {
            f <- formula(paste0("y ~ te(x1, x2, k = c(",
                                paste0(knots, collapse = ", "),
                                "), bs = c(",
                                paste0("\"", bs, "\"", collapse = ", "),
                                "), fx = c(",
                                paste0(fx, collapse = ", "),
                                "))"))
        }
    }
    ## fit model
    mod <- gam(f, family = family, weights = w, select = select,
               method = method, gamma = gamma)
    xn1 <- seq(min(x1), max(x1), len=GRID)
    xn2 <- seq(min(x2), max(x2), len=GRID)
    newd <- expand.grid(x1 = xn1, x2 = xn2)
    fit <- predict(mod, type = "response", newdata=as.data.frame(newd))
    poly <- chull(cbind(x1,x2))
    ## Move out points of the convex hull to have contour for all data
    ## points
    xhull1 <- x1[poly] + sign(x1[poly] - mean(x1[poly])) *
        diff(range(x1))/(GRID - 1)
    xhull2 <- x2[poly] + sign(x2[poly] - mean(x2[poly])) *
        diff(range(x2))/(GRID - 1)
    npol <- length(poly)
    np <- nrow(newd)
    inpoly <- numeric(np)
    inpoly <- .C(pnpoly, as.integer(npol), as.double(xhull1),
                 as.double(xhull2), as.integer(np), as.double(newd[,1]),
                 as.double(newd[,2]), inpoly = as.integer(inpoly))$inpoly
    is.na(fit) <- inpoly == 0
    if(plot) {
        if (!add) {
            if (bubble) {
                if (is.numeric(bubble))
                    cex <- bubble
                cex <- (y -  min(y))/diff(range(y)) * (cex-0.4) + 0.4
            }
            plot(X, asp = 1, cex = cex, ...)
        }
        if (!missing(main) || (missing(main) && !add)) {
            if (missing(main))
                main <- yname
            title(main = main)
        }
        if (missing(levels))
            levels <- pretty(range(fit, finite = TRUE), nlevels)
        ## Only plot surface is select is FALSE or (TRUE and EDF is diff from 0)
        if(!select ||
           (select && !isTRUE(all.equal(as.numeric(summary(mod)$edf), 0))))
            contour(xn1, xn2, matrix(fit, nrow=GRID), col = col, add = TRUE,
                    levels = levels, labcex = labcex,
                    drawlabels = !is.null(labcex) && labcex > 0,
                    lwd = lwd.cl)
    }
    mod$grid <- list(x = xn1, y = xn2, z = matrix(fit, nrow = GRID))
    class(mod) <- c("ordisurf", class(mod))
    mod
}

`anova.betadisper` <- function(object, ...)
{
    model.dat <- with(object, data.frame(Distances = distances,
                                         Groups = group))
    n.grps <- with(model.dat, length(unique(as.numeric(Groups))))
    if(n.grps < 2)
        stop("anova() only applicable to two or more groups")
    anova(lm(Distances ~ Groups, data = model.dat))
}

`ordicloud` <-
    function(x, data = NULL, formula, display = "sites", choices=1:3,
             panel = "panel.ordi3d",
             prepanel = "prepanel.ordi3d", ...)
{
  localCloud <- function(..., shrink, origin, scaling) cloud(...)
  x <- as.data.frame(scores(x, display = display, choices = choices, ...))
  if (!is.null(data))
    x <- cbind(x, data)
  if (missing(formula)) {
    v <- colnames(x)
    formula <- as.formula(paste(v[2], "~", v[1], "*", v[3]))
  }
  localCloud(formula, data = x, panel = panel,  prepanel = prepanel, ...)
}

"cascadeKM" <-
function(data, inf.gr, sup.gr, iter = 100, criterion="calinski")
{
### DESCRIPTION

### This function use the 'kmeans' function of the 'stats' package to create 
### a cascade of partitions from K = nb_inf_gr to K = nb_sup_gr

### INPUT				
###
### data			The data matrix; the objects are the rows
### nb_inf_gr		Number of groups (K) for the first partition (min)
### nb_sup_gr	 	Number of groups (K) for the last partition (max)
### iter			The number of random starting configurations for each value of K
### criterion		The criterion that will be used to select the best
###				partition. See the 'clustIndex' function in PACKAGE = cclust

### OUTPUT
###
### The same as in the kmeans packages

### EXAMPLE
###
### 	result <- cascadeKM(donnee, 2, 30, iter = 50, criterion = 'calinski') 
###
### 	data = data table
### 	2 = lowest number of groups for K-means
### 	30 = highest number of groups for K-means
### 	iter = 50: start kmeans 50 times using different random configurations
### 	criterion = 'calinski': the Calinski-Harabasz (1974) criterion to determine 
###      the best value of K for the data set. 'Best' is in the least-squares sense.
###

### Main function
    data <- as.matrix(data)
    SCE<-list()
    resultat<-list()
    index<-list()
    if(!is.null(nrow(data))){
        partition <- matrix(NA, nrow(data), sup.gr - inf.gr + 1)
    }else{
        partition <- matrix(NA, length(data), sup.gr - inf.gr + 1)
    }
    results <- matrix(NA, 2, sup.gr - inf.gr + 1)
    size <- matrix(NA, sup.gr, sup.gr - inf.gr + 1)
    ## Pour tous les nombres de groupes voulus
    h <- 1
    for(ii in inf.gr:sup.gr)
    {
        j <- ii - inf.gr + 1
        tmp <- kmeans(data, ii, iter.max = 50, nstart=iter)
        size[1:ii,h] <- tmp$size
        h <- h+1
        partition[,j] <- tmp$cluster
        ## Compute SSE statistic
        results[1,j] <- sum(tmp$withinss)
        ## Compute stopping criterion
        results[2,j] <- cIndexKM(tmp,data, index = tolower(criterion))
    }
    colnames(partition) <- paste(inf.gr:sup.gr, "groups")
    tmp <- rownames(data)
    if(is.null(tmp)){
        r.name <- c(1:nrow(partition))
    }else{
        r.name <- tmp
    }
    rownames(partition) <- r.name
	
    colnames(results) <- paste(inf.gr:sup.gr, "groups")
    rownames(results)<-c("SSE", criterion)

    colnames(size) <- paste(inf.gr:sup.gr, "groups")
    rownames(size) <- paste("Group", 1:sup.gr)
	
    tout<-list(partition=partition, results=results, criterion=criterion, size=size)
    class(tout) <- "cascadeKM"
    tout
}

`tsallisaccum` <-
    function (x, scales = seq(0, 2, 0.2), permutations = 100, raw = FALSE,
              subset, ...)
{
    if (!missing(subset))
        x <- subset(x, subset)
    x <- as.matrix(x)
    if (!is.numeric(x))
        stop("input data must be numeric")
    n <- nrow(x)
    p <- ncol(x)
    if (p == 1) {
        x <- t(x)
        n <- nrow(x)
        p <- ncol(x)
    }
    pmat <- getPermuteMatrix(permutations, n)
    permutations <- nrow(pmat)
    m <- length(scales)
    result <- array(dim = c(n, m, permutations))
    dimnames(result) <- list(pooled.sites = c(1:n), scale = scales,
        permutation = c(1:permutations))
    for (k in 1:permutations) {
        result[, , k] <- as.matrix(tsallis((apply(x[pmat[k,],
            ], 2, cumsum)), scales = scales, ...))
    }
    if (raw) {
        if (m == 1) {
            result <- result[, 1, ]
        }
    }
    else {
        tmp <- array(dim = c(n, m, 6))
        for (i in 1:n) {
            for (j in 1:m) {
                tmp[i, j, 1] <- mean(result[i, j, 1:permutations])
                tmp[i, j, 2] <- sd(result[i, j, 1:permutations])
                tmp[i, j, 3] <- min(result[i, j, 1:permutations])
                tmp[i, j, 4] <- max(result[i, j, 1:permutations])
                tmp[i, j, 5] <- quantile(result[i, j, 1:permutations],
                  0.025)
                tmp[i, j, 6] <- quantile(result[i, j, 1:permutations],
                  0.975)
            }
        }
        result <- tmp
        dimnames(result) <- list(pooled.sites = c(1:n), scale = scales,
            c("mean", "stdev", "min", "max", "Qnt 0.025", "Qnt 0.975"))
    }
    attr(result, "control") <- attr(pmat, "control")
    class(result) <- c("tsallisaccum", "renyiaccum", class(result))
    result
}

      SUBROUTINE monoMDS (NOBJ, NFIX, NDIM, NDIS, NGRP,
     .  DISS, IIDX, JIDX, XINIT, ISTART,
     .  ISFORM, ITIES, IREGN, ISCAL, MAXITS, SRATMX,
     .  STRMIN, SFGRMN, 
     .  DIST, DHAT, X, STRESS, STRS, ITERS, ICAUSE)
C
C Subroutine for multidimensional scaling.
C
C 1.00 March 28, 2011
C 1.01 April 6, 2011  - added argument STRS(NGRP) to return the stress
C                       for each of the NGRP groups of dissimilarities
C                       i.e., from each separate regression.
C 1.02 January 7, 2016 - fixed bug in MONREG so that huge 
C                        dissimilarities are correctly handled in
C                        creating the initial partition for monotone 
C                        regression with primary tie treatment.
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
C Starting from a supplied initial configuarion, uses steepest descent
C   to minimize Kruskal's stress, a measure of badness-of-fit of one
C   or more regressions of distances onto the supplied dissimilarities.
C
C This routine is very flexible in the form of the dissimilarity
C   matrices it can handle because dissimilarities are input as a 
C   vector (DISS) with the associated i- and j-indices in parallel
C   vectors (IIDX, JIDX).  There is no need for missing values, as
C   these index vectors specify which pair of points each dissimilarity
C   belongs to and missing dissimilarities are simply omitted.  Another
C   element of flexibility is that there can be more than one group of
C   dissimilarities and a separate regression of distance on 
C   dissimilarity is done for each group.  Local or "row conditional"
C   scaling can be performed by inputting each row of the dissimilarity
C   matrix (minus the diagonals) as a separate group.  Hybrid scaling
C   of Faith et. al (1987) can be done by inputting the complete 
C   dissimilarity matrix as group 1 and only those dissimilarities
C   below a specified threshold as group 2 (and also setting input
C   parameter IREGN to 3).  New points can be inserted into an
C   existing ordination by inputting dissimilarities from the
C   rectangular matrix in which the existing points are rows and the
C   new points are columns.  The parameter NFIX is then set to the
C   number of existing points (rows) and these will be the first 
C   NFIX points in XINIT, which supplies their coordinates.  Normally,
C   when NFIX>0, the scaling parameter, ISCAL, should be 0, so that 
C   the coordinates of the fixed points remain unchanged in the 
C   output ordination scores, X.
C
C========INPUT ARGUMENTS:
C
C NOBJ = number of objects being ordinated
C
C NFIX = number of fixed objects (< NOBJ - if NFIX>0 the input
C        dissim matrix is assumed to be rectangular, with the
C        first NFIX objects, rows, remaining fixed and the rest
C        able to move)
C
C NDIM = number of dimensions for the MDS ordination
C
C NDIS = total number of dissimilarities among objects
C
C NGRP = number of groups of dissimilarities (a separate regression
C        of distances on dissimilarities is performed for each
C        group)
C
C DISS(NDIS) = vector of dissimilarities among objects
C
C IIDX(NDIS) = i-indices of objects in the dissimilarity vector
C
C JIDX(NDIS) = j-indices of objects in the dissimilarity vector
C
C XINIT(NOBJ,NDIM) = initial coordinates for NOBJ objects on NDIM
C                    dimensions
C                     
C ISTART(NGRP) = index of starting position of each group of
C                dissimilarities within vectors DISS, IIDX and JIDX
C
C ISFORM = Kruskal stress formula to be used: 1 or 2
C
C ITIES = Kruskal tie treatment: 1 = weak/primary, 2 = strong/secondary
C
C IREGN = Regression type: 1 = Kruskal monotone, 2 = linear, 3 = hybrid
C         (Kruskal monotone for the first NGRP/2 groups, linear for
C         the remainder of the groups - when this is selected it is
C         assumed that NGRP>1 and NGRP is even)
C
C ISCAL = scaling of final ordination: 0 = preserved (output raw scores
C         with no standardization), 1 = normalize (center and adjust
C         RMS distance of points to centroid to 1.0)
C
C MAXITS = maximum number of iterations - at least 200 recommended
C
C SRATMX = maximum average stress ratio (iterations stop when average
C          stress ratio exceeds this value but is still < 1) - a value
C          of 0.9999 or higher is recommended
C
C STRMIN = minimum stress to stop (iterations stop if stress falls
C          below this value) - a value of 0.001 or lower is
C          recommended
C
C SFGRMN = minimum scale factor of the gradient (iterations stop if
C          the scale factor of the gradient drops below this)
C
C========OUTPUT ARGUMENTS:
C
C DIST(NDIS) = distances among objects in the final ordination
C
C DHAT(NDIS) = fitted distances in final regression(s) of distance
C              on dissimilarity
C
C X(NOBJ,NDIM) = final ordination coordinates of NOBJ objects in NDIM
C                dimensions
C
C STRESS = final value of stress
C
C STRS(NGRP) = vector of stress values from each separate regression
C
C ITERS = number of iterations performed
C
C ICAUSE = reason for termination of iterations: 1 = max iterations
C          used, 2 = stress fell below STRMIN, 3 = stress ratio
C          exceeded SRATMX, 4 = scale factor of gradient fell below
C          SFGRMN
C
C---INPUT ARGUMENTS
      INTEGER, INTENT(IN) :: NOBJ, NFIX, NDIM, NDIS, NGRP,
     .  ISFORM, ITIES, IREGN, ISCAL, MAXITS
      INTEGER, INTENT(IN) :: IIDX(NDIS), JIDX(NDIS), ISTART(NGRP)
      DOUBLE PRECISION, INTENT(IN) :: XINIT(NOBJ,NDIM), DISS(NDIS),
     .  SRATMX, STRMIN, SFGRMN
C---OUTPUT ARGUMENTS
      INTEGER, INTENT(OUT) :: ITERS, ICAUSE
      DOUBLE PRECISION, INTENT(OUT) :: X(NOBJ,NDIM), DIST(NDIS), 
     .  DHAT(NDIS), STRESS, STRS(NGRP)
C---ALLOCATABLE TEMPORARY ARRAYS
      INTEGER, ALLOCATABLE :: IWORK(:)
      DOUBLE PRECISION, ALLOCATABLE :: GRAD(:,:), GRLAST(:,:)
C
      DOUBLE PRECISION :: STRLST, SQRTN, SRATF1, SRATF2, FNGRP,
     .  STRINC, COSAV, ACOSAV, SRATAV, STEP, FNDIM, SFGR, SRATIO,
     .  SFACT, TFACT, DMEAN, CAGRGL, SFGLST, STPNEW, SSFACB, SSFACT,
     .  PARAM(2)
C
C ALLOCATE THE TEMPORARY ARRAYS NEEDED
C
      ALLOCATE (IWORK(NDIS), GRAD(NOBJ,NDIM), GRLAST(NOBJ,NDIM))
C
C INITIALIZE SOME PARAMETERS
C
      SQRTN=SQRT(REAL(NDIS))
      SRATF1=0.5*(1.0+SRATMX)
      SRATF2=1.0-SRATF1
      FNGRP=REAL(NGRP)
      STRINC=1.1
      STRESS=1.0D0
      COSAV=0.0
      ACOSAV=0.0
      SRATAV=0.8
      STEP=0.0
      FNDIM=REAL(NDIM)
      SFGR=SQRTN
C
C PREPARE DISSIMILARITIES FOR USE IN REGRESSION(S)
C
      DO IGRP=1,NGRP
        I1=ISTART(IGRP)
        IF (IGRP.LT.NGRP) THEN
          N=ISTART(IGRP+1)-I1
        ELSE
          N=NDIS+1-I1
        ENDIF
        IF (N.GT.0) THEN
C
C PRE-SORT IF NECESSARY. NOT REQUIRED FOR LINEAR REGRESSION (IREGN=2)
C     NOR FOR DISSIM GROUPS > NGRP/2 IN HYBRID REGRESSION (IREGN=3)
C
          IF (IREGN.EQ.1.OR.(IREGN.EQ.3.AND.IGRP.LE.NGRP/2)) THEN
            CALL ASORT4 (DISS(I1),N,IIDX(I1),JIDX(I1))
          ENDIF
        ENDIF
      ENDDO
C
C COPY INITIAL CONFIGURATION TO CURRENT CONFIGURATION
C
      CALL MACOPY (XINIT,NOBJ,NOBJ,NDIM,X,NOBJ)
C
C INITALIZE GRADIENT (WILL BE USED AS THE FIRST "LAST GRADIENT")
C
      CALL MAINIT (GRAD,NOBJ,NDIM,NOBJ,SQRT(1.0/FNDIM))
C=======================================================================
C
C START OF ITERATION LOOP
C
      DO IT=0,MAXITS
C---Reset backup counter
        NBACK=0
C
C SAVE LAST STRESS VALUE AND SET STRESS BACK TO ZERO
C
C---Jump back to here when a backup if required
   10   STRLST=STRESS
        STRESS=0.0D0
C
C Normalize the current configuration (unless ISCAL=0)
C
        IF (ISCAL.GT.0) THEN
          CALL NRMCON (X,NOBJ,NDIM,NOBJ,SSFACT)
        ELSE
          SSFACT=1.0D0
        ENDIF
C
C MOVE CURRENT GRADIENT TO LAST GRADIENT (WITH APPROPRIATE SCALING)
C   AND SAVE LAST GRADIENT SCALING FACTOR
C
        CALL MACOPY (GRAD,NOBJ,NOBJ,NDIM,GRLAST,NOBJ)
        CALL MAMAS (GRLAST,NOBJ,NOBJ,NDIM,SSFACT)
        SFGLST=SFGR*SSFACT
C
C CLEAR CURRENT GRADIENT
C
        CALL MAINIT (GRAD,NOBJ,NDIM,NOBJ,0.0D0)
C
C COMPUTE DISTANCES
C
        CALL CLCDIS (X,NOBJ,NDIM,DIST,IIDX,JIDX,NDIS)
C-----------------------------------------------------------------------
C
C LOOP OVER THE NGRP GROUPS OF DISSIMILARITIES
C
        DO IGRP=1,NGRP
          I1=ISTART(IGRP)
          IF (IGRP.LT.NGRP) THEN
            N=ISTART(IGRP+1)-I1
          ELSE
            N=NDIS+1-I1
          ENDIF
          IF (N.GT.0) THEN
C
C PERFORM REGRESSION OF DISTANCES ON DISSIMILARITIES
C
            IF (IREGN.EQ.1.OR.(IREGN.EQ.3.AND.IGRP.LE.NGRP/2)) THEN
              CALL MONREG (DISS(I1),DIST(I1),DHAT(I1),IIDX(I1),
     .          JIDX(I1),IWORK(I1),N,ITIES)
            ELSEIF (IREGN.EQ.2.OR.(IREGN.EQ.3.AND.IGRP.GT.NGRP/2)) THEN
              CALL LINREG (DISS(I1),DIST(I1),DHAT(I1),N,PARAM)
            ENDIF
C
C COMPUTE AND ACCUMULATE STRESS
C
            CALL CLCSTR (DIST(I1),DHAT(I1),N,SFACT,TFACT,STRS(IGRP),
     .        ISFORM,DMEAN)
            STRESS=STRESS+SFACT/TFACT
C
C ACCUMULATE THE NEGATIVE GRADIENT
C
            CALL CLCGRD (X,GRAD,NOBJ,NDIM,DIST(I1),DHAT(I1),
     .        IIDX(I1),JIDX(I1),N,STRS(IGRP),SFACT,TFACT,ISFORM,DMEAN)
          ENDIF
        ENDDO
C
C END OF DISSIMILARITY GROUP LOOP
C
C-----------------------------------------------------------------------
C
C COMPUTE OVERALL STRESS AND ITS REDUCTION RATIO
C
        STRESS=SQRT(STRESS/FNGRP)
        IF (IT.EQ.0) THEN
          SRATIO=0.8D0
        ELSE
          SRATIO=STRESS/STRLST
        ENDIF
C
C TO PREVENT FIXED POINTS BEING MOVED, ZERO THEIR GRADIENT ELEMENTS
C
        IF (NFIX.GT.0) THEN
          DO I=1,NFIX
            DO IDIM=1,NDIM
              GRAD(I,IDIM)=0.0D0
            ENDDO
          ENDDO
        ENDIF
C
C CALCULATE SCALE FACTOR OF NEW GRADIENT AND ITS DIRECTION COSINE
C   WITH THE LAST GRADIENT
C
        CALL CLCSFA (GRAD,GRLAST,NOBJ,NDIM,NOBJ,SFGR,CAGRGL,
     .    SFGLST)
C
C IF STRESS HAS INCREASED APPRECIABLY, OR THE ANGLE BETWEEN THE NEW
C   GRADIENT AND THE LAST GRADIENT IS APPROACHING 180 DEGREES, REDUCE
C   THE STEP SIZE AND BACK UP ALONG THE LAST GRADIENT
C   (BACKUP PROCEDURE IS PERFORMED A MAXIMUM OF 3 TIMES PER ITERATION)
C
        IF ((SRATIO.GT.STRINC.OR.CAGRGL.LT.(-0.95)).AND.NBACK.LT.3)
     .    THEN
          STPNEW=STEP*0.1D0
          CALL BACKUP (X,GRAD,GRLAST,NOBJ,NDIM,NOBJ,NBACK,SSFACT,
     .      SSFACB,STRESS,STRLST,SFGR,SFGLST,STEP,STPNEW)
          GO TO 10
        ENDIF
C
C UPDATE AVERAGES OF STRESS RATIO AND PREVIOUS DIRECTION COSINES
C
        SRATAV=SRATIO**0.33334D0 * SRATAV**0.66666D0
        COSAV=CAGRGL*0.66666D0 + COSAV*0.33334D0
        ACOSAV=ABS(CAGRGL)*0.66666D0 + ACOSAV*0.33334D0
C
C COMPUTE NEW STEP SIZE
C
        CALL CLCSTP (STEP,IT,SFGR,STRESS,COSAV,ACOSAV,SRATIO,
     .    SRATAV)
C
C CHECK WHETHER OR NOT TO KEEP ITERATING
C
C IF STRESS IN SMALL ENOUGH, TERMINATE
C
        ITERS=IT
        IF (STRESS.LT.STRMIN) THEN
          ICAUSE=2
          EXIT
C
C IF STRESS IS DECREASING SLOWLY, TERMINATE.
C
        ELSEIF (ABS(SRATAV-SRATF1).LE.SRATF2.AND.
     .          ABS(SRATIO-SRATF1).LE.SRATF2) THEN
          ICAUSE=3
          EXIT
C
C IF SCALE FACTOR OF GRADIENT IS SMALL ENOUGH, TERMINATE
C
        ELSEIF (SFGR.LE.SFGRMN) THEN
          ICAUSE=4
          EXIT
C
C IF THIS IS THE FINAL ITERATION, TERMINATE
C
        ELSEIF (IT.EQ.MAXITS) THEN
          ICAUSE=1
          EXIT
        ENDIF
C
C COMPUTE NEW CONFIGURATION
C
        CALL NEWCON (X,GRAD,NOBJ,NDIM,NOBJ,STEP,SFGR)
      ENDDO
C
C END OF ITERATION LOOP
C
C=======================================================================
C
C DEALLOCATE THE TEMPORARY ARRAYS AND RETURN
C
      DEALLOCATE (IWORK, GRAD, GRLAST)
      RETURN
      END SUBROUTINE monoMDS

      SUBROUTINE ASORT4 (X,N,I1,I2)
C
C SORTS THE REAL*8 VECTOR X ASCENDING, SIMULTANEOUSLY PERMUTING THE
C   INTEGER VECTORS I1 AND I2 IN PARALLEL.
C
C ADAPTED FROM SUPERCHARGED SHELL SORT ALGORITHM PUBLISHED ON PAGE 488
C   OF THE MAY 1983 EDITION OF BYTE MAGAZINE.
C
C Adapted by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      INTEGER I1(N), I2(N), I1TEMP, I2TEMP
      DOUBLE PRECISION X(N), XTEMP
C
      IF (N.LT.2) RETURN
      FN=REAL(N)
      NLOOPS=MAX(NINT(LOG(FN)/LOG(2.)),1)
      M=2**(NLOOPS-1)
      DO II=1,NLOOPS
        FM=M
        DO I=1,MAX(1,N-M)
          IF (X(I).GT.X(I+M)) THEN
            XTEMP=X(I+M)
            I1TEMP=I1(I+M)
            I2TEMP=I2(I+M)
            X(I+M)=X(I)
            I1(I+M)=I1(I)
            I2(I+M)=I2(I)
            IF (I.LE.M) THEN
              X(I)=XTEMP
              I1(I)=I1TEMP
              I2(I)=I2TEMP
            ELSE
              DO J=I-M,1,-M
                IF (XTEMP.GE.X(J)) EXIT
                  X(J+M)=X(J)
                  I1(J+M)=I1(J)
                  I2(J+M)=I2(J)
              ENDDO
              X(J+M)=XTEMP
              I1(J+M)=I1TEMP
              I2(J+M)=I2TEMP
            ENDIF
          ENDIF
        ENDDO
        M=INT(FM/2.)
      ENDDO
      RETURN
      END SUBROUTINE ASORT4

      SUBROUTINE BACKUP (X,GRAD,GRLAST,NOBJ,NDIM,MAXOBJ,NBACK,SSFACT,
     .  SSFACB,STRESS,STRLST,SFGR,SFGLST,STEP,STPNEW)
C
C BACKS THE CONFIGURATION UP ALONG THE LAST GRADIENT
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      DOUBLE PRECISION X(MAXOBJ,NDIM), GRAD(MAXOBJ,NDIM), 
     .  GRLAST(MAXOBJ,NDIM), SSFACT, SSFACB, STRESS, STRLST, SFGR,
     .  SFGLST,STEP,STPNEW,FACTOR
C
      NBACK=NBACK+1
      IF (NBACK.EQ.1) THEN
        SSFACB=1.0D0
      ELSE
        SSFACB=SSFACB*SSFACT
      ENDIF
      FACTOR=SSFACB*(STEP-STPNEW)/SFGLST
      DO IDIM=1,NDIM
        DO IOBJ=1,NOBJ
          X(IOBJ,IDIM)=X(IOBJ,IDIM)-FACTOR*GRLAST(IOBJ,IDIM)
          GRAD(IOBJ,IDIM)=GRLAST(IOBJ,IDIM)
        ENDDO
      ENDDO
      STEP=STPNEW
      SFGR=SFGLST
      STRESS=STRLST
      RETURN
      END SUBROUTINE BACKUP

      SUBROUTINE CLCDIS (X,MAXOBJ,NDIM,DIST,IIDX,JIDX,NDIS)
C
C COMPUTES EUCLIDEAN DISTANCES BETWEEN EACH PAIR OF THE NOBJ POINTS
C   WHOSE CO-ORDINATES IN NDIM DIMENSIONS ARE IN X(NOBJ,NDIM).
C
C THE DISTANCES ARE STORED IN THE VECTOR DIST AND THE INDEX VECTORS
C   IIDX AND JIDX HOLD THE I,J INDICES OF THE POINTS IN THE ORDER IN
C   WHICH THE COMPARISONS ARE TO BE MADE.
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      INTEGER IIDX(NDIS), JIDX(NDIS)
      DOUBLE PRECISION X(MAXOBJ,NDIM), DIST(NDIS)
C
C INITIALIZE DISTANCES
C
      DO I=1,NDIS
        DIST(I)=0.0
      ENDDO
C
C ACCUMULATE SUMS OF SQUARED DIFFERENCES ON EACH DIMENSION
C
      DO IDIM=1,NDIM
        DO K=1,NDIS
          DIST(K)=DIST(K)+(X(IIDX(K),IDIM)-X(JIDX(K),IDIM))**2
        ENDDO
      ENDDO
C
C TAKE SQUARE ROOTS OF TOTALS
C
      DO I=1,NDIS
        DIST(I)=SQRT(DIST(I))
      ENDDO
      RETURN
      END SUBROUTINE CLCDIS

      SUBROUTINE CLCGRD (X,GRAD,MAXOBJ,NDIM,DIST,DHAT,IIDX,JIDX,
     .  NDIS,STRESS,SFACT,TFACT,ISFORM,DMEAN)
C
C ACCUMULATES THE NEGATIVE GRADIENT IN GRAD(NOBJ,NDIM).
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      INTEGER IIDX(NDIS), JIDX(NDIS)
      DOUBLE PRECISION X(MAXOBJ,NDIM), GRAD(MAXOBJ,NDIM), DIST(NDIS),
     .  DHAT(NDIS), STRESS, SFACT, TFACT, DMEAN, SOTSQ, RECIPT,
     .  DELTA
C
      IF (STRESS.LE.0.0D0) RETURN
      SOTSQ=SFACT/(TFACT**2)
      RECIPT=1.0/TFACT
      DO IDIM=1,NDIM
        IF (ISFORM.LE.1) THEN
C---Kruskal's stress formula 1
          DO K=1,NDIS
            IF (DIST(K).GT.0.0) THEN
              DELTA=(SOTSQ-RECIPT*(DIST(K)-DHAT(K))/DIST(K))*
     .          (X(IIDX(K),IDIM)-X(JIDX(K),IDIM))
              GRAD(IIDX(K),IDIM)=GRAD(IIDX(K),IDIM)+DELTA
              GRAD(JIDX(K),IDIM)=GRAD(JIDX(K),IDIM)-DELTA
            ENDIF
          ENDDO
        ELSE
C---Kruskal's stress formula 2
          DO K=1,NDIS
            IF (DIST(K).GT.0.0) THEN
              DELTA=(SOTSQ*(DIST(K)-DMEAN)/DIST(K)-
     .          RECIPT*(DIST(K)-DHAT(K))/DIST(K))*
     .          (X(IIDX(K),IDIM)-X(JIDX(K),IDIM))
              GRAD(IIDX(K),IDIM)=GRAD(IIDX(K),IDIM)+DELTA
              GRAD(JIDX(K),IDIM)=GRAD(JIDX(K),IDIM)-DELTA
            ENDIF
          ENDDO
        ENDIF
      ENDDO
      RETURN
      END SUBROUTINE CLCGRD

      SUBROUTINE CLCSFA (GRAD,GRLAST,NOBJ,NDIM,MAXOBJ,SFGR,CAGRGL,
     .  SFGLST)
C
C COMPUTES SCALE FACTOR OF THE NEGATIVE GRADIENT IN GRAD(NOBJ,NDIM)
C   AND ALSO FINDS ITS DIRECTION COSINE WITH THE PREVIOUS GRADIENT,
C   STORED IN GRLAST(NOBJ,NDIM).
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      DOUBLE PRECISION GRAD(MAXOBJ,NDIM), GRLAST(MAXOBJ,NDIM),
     .  SFGR, CAGRGL, SFGLST, FN, FACTOR
C
      FN=DBLE(NOBJ)
      SFGR=0.0D0
      CAGRGL=0.0D0
      DO IDIM=1,NDIM
        DO IOBJ=1,NOBJ
          SFGR=SFGR+GRAD(IOBJ,IDIM)**2
          CAGRGL=CAGRGL+GRAD(IOBJ,IDIM)*GRLAST(IOBJ,IDIM)
        ENDDO
      ENDDO
      SFGR=SQRT(SFGR/FN)
      FACTOR=SFGR*SFGLST*FN
      IF (FACTOR.NE.0.0) CAGRGL=CAGRGL/FACTOR
      RETURN
      END SUBROUTINE CLCSFA

      SUBROUTINE CLCSTP (STEP,IT,SFGR,STRESS,COSAV,ACOSAV,SRATIO,
     .  SRATAV)
C
C COMPUTES NEW STEP SIZE.  BASED ON J.B. KRUSKAL'S STEP SIZE ALGORITHM
C   IN THE BELL LABORATORIES PROGRAM "KYST".
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      DOUBLE PRECISION STEP, SFGR, STRESS, COSAV, ACOSAV, SRATIO,
     .  SRATAV, FACTR1, FACTR2, FACTR3
C
C MAKE A WILD GUESS FOR INITIAL STEP SIZE (IT=0).
C
      IF (IT.EQ.0) THEN
        STEP=25.0*STRESS*SFGR
C
C OTHERWISE, UPDATE STEP SIZE.
C
      ELSE
        FACTR1=4.0**COSAV
        FACTR2=1.6/( (1.0+(MIN(1D0,SRATAV))**5) *
     .    (1.0+ACOSAV-ABS(COSAV)) )
        FACTR3=SQRT(MIN(1D0,SRATIO))
        STEP=STEP*FACTR1*FACTR2*FACTR3
      ENDIF
      RETURN
      END SUBROUTINE CLCSTP

      SUBROUTINE CLCSTR (DIST,DHAT,N,SNUMER,SDENOM,STRESS,ISFORM,DMEAN)
C
C COMPUTES KRUSKAL'S STRESS (FORMULA 1 or 2)
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      DOUBLE PRECISION DIST(N), DHAT(N), SNUMER, SDENOM, STRESS,
     .  DMEAN
C
      SNUMER=0.0D0
      SDENOM=0.0D0
      DMEAN=0.0D0
      IF (ISFORM.GE.2) THEN
        DO I=1,N
          DMEAN=DMEAN+DIST(I)
        ENDDO
        DMEAN=DMEAN/DBLE(N)
        DO I=1,N
          SNUMER=SNUMER+(DIST(I)-DHAT(I))**2
          SDENOM=SDENOM+(DIST(I)-DMEAN)**2
        ENDDO
      ELSE
        DO I=1,N
          SNUMER=SNUMER+(DIST(I)-DHAT(I))**2
          SDENOM=SDENOM+DIST(I)**2
        ENDDO
      ENDIF
      STRESS=SQRT(SNUMER/SDENOM)
      RETURN
      END SUBROUTINE CLCSTR

      SUBROUTINE LINREG (DISS,DIST,DHAT,N,C)
C
C PERFORMS LINEAR REGRESSION OF DIST ON DISS, PLACING THE FITTED VALUES
C   IN DHAT.  THE REGRESSION COEFFICIENTS ARE RETURNED IN C:
C   C(1) = INTERCEPT, C(2) = SLOPE.
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      DOUBLE PRECISION DISS(N), DIST(N), DHAT(N),
     .  AVDISS, SSDISS, AVDIST, SPROD, C(2), FN
C
C COMPUTE MEANS, SUMS OF SQUARES AND SUM OF PRODUCTS
C
      FN=DBLE(N)
      AVDIST=0.0D0
      AVDISS=0.0D0
      DO K=1,N
        AVDIST=AVDIST+DIST(K)
        AVDISS=AVDISS+DISS(K)
      ENDDO
      AVDIST=AVDIST/FN
      AVDISS=AVDISS/FN
      SSDISS=0.0D0
      SPROD=0.0D0
      DO K=1,N
        SSDISS=SSDISS+(DISS(K)-AVDISS)**2
        SPROD=SPROD+(DIST(K)-AVDIST)*(DISS(K)-AVDISS)
      ENDDO
C
C COMPUTE SLOPE AND INTERCEPT
C
      C(2)=SPROD/SSDISS
      C(1)=AVDIST-C(2)*AVDISS
C
C CALCULATE FITTED VALUES
C
      DO K=1,N
        DHAT(K)=C(1)+C(2)*DISS(K)
      ENDDO
      RETURN
      END SUBROUTINE LINREG

      SUBROUTINE MACOPY (A,MAXN1,N,M,B,MAXN2)
C
C COPY MATRIX A(N,M) TO B(N,M).
C
C CAN BE USED TO PACK AN ARRAY BY PASSING THE SAME ARRAY AS A AND B,
C   WITH MAXN2 SET TO N.
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      DOUBLE PRECISION A(MAXN1,M), B(MAXN2,M)
C
      DO J=1,M
        DO I=1,N
          B(I,J)=A(I,J)
        ENDDO
      ENDDO
      RETURN
      END SUBROUTINE MACOPY

      SUBROUTINE MAINIT (X,M,N,MAXM,CONST)
C
C INITIALIZES MATRIX X(M,N) WITH THE VALUE CONST.
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      DOUBLE PRECISION X(MAXM,N), CONST
C
      DO J=1,N
        DO I=1,M
          X(I,J)=CONST
        ENDDO
      ENDDO
      RETURN
      END SUBROUTINE MAINIT

      SUBROUTINE MAMAS (A,MAXL,L,M,S)
C
C MULTIPLY MATRIX A(L,M) BY THE SCALAR S
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      DOUBLE PRECISION A(MAXL,M), S
C
      DO I=1,L
        DO J=1,M
          A(I,J)=A(I,J)*S
        ENDDO
      ENDDO
      RETURN
      END SUBROUTINE MAMAS
 
      SUBROUTINE MONREG (DISS,DIST,DHAT,IIDX,JIDX,IWORK,N,ITIES)
C
C PERFORMS KRUSKAL'S MONOTONE REGRESSION OF DIST ON DISS, PLACING THE 
C   FITTED VALUES IN DHAT.
C
C ASSUMES THAT DISS HAS BEEN SORTED ASCENDING.
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      INTEGER IIDX(N), JIDX(N)
      INTEGER IWORK(N)
      DOUBLE PRECISION DISS(N), DIST(N), DHAT(N), DNEXT, TOLER, SUM,
     .  DHATAV
C---Tolerance used to test if dissimilarities are equal
      TOLER=SQRT(EPSILON(SUM))
C***********************************************************************
C
C CREATE INITIAL PARTITION:
C
C   PRE-PROCESS BLOCKS OF TIED DISS VALUES DEPENDING ON THE SELECTED
C     TIE TREATMENT (ITIES)
C-----------------------------------------------------------------------
      J=0
      DO I=1,N
        IF (I.LT.N) THEN
          DNEXT=DISS(I+1)
        ELSE
C---Bug fix January 7, 2016: correctly handles huge dissimilarities
C          DNEXT=DISS(I)+1.0
          DNEXT=DISS(I)*2.0
        END IF
        IF (ABS(DNEXT-DISS(I)).GT.TOLER) THEN
C---NTIE is the number of DISS values in the current group of tied values
          NTIE=I-J
          IF (NTIE.GT.1) THEN
            IF (ITIES.LE.1) THEN
C---Primary tie treatment: sort DIST values within this tied group into
C     ascending order, permuting the index vectors accordingly.
C     Initialize fitted values, DHAT, to be equal to DIST and make each
C     value an intial block of size 1.
              CALL ASORT4 (DIST(J+1),NTIE,IIDX(J+1),JIDX(J+1))
              DO K=J+1,I
                DHAT(K)=DIST(K)
                IWORK(K)=1
              ENDDO
            ELSE
C---Secondary tie treatment: set fitted value for all members of this
C     tied group to the average of the DIST values for the group.  Make
C     the group of tied values an initial block equal in size to the
C     number of tied values.
              IF (NTIE.EQ.2) THEN
                SUM=DIST(J+1)+DIST(I)
              ELSE
                SUM=0.0
                DO K=J+1,I
                  SUM=SUM+DIST(K)
                ENDDO
              ENDIF
              DHAT(J+1)=SUM
              DHAT(I)=SUM
              IWORK(J+1)=NTIE
              IWORK(I)=NTIE
            ENDIF
          ELSE
C---Initially, make each unique (not tied) value is DISS a block of
C   size 1.
            IWORK(I)=1
            DHAT(I)=DIST(I)
          ENDIF
          J=I
        ENDIF
      ENDDO
C***********************************************************************
C
C NOW START THE MONOTONE REGRESSION PROCEDURE
C
C ICURR is the block we are currently examining.
C
C WE START BY LOOKING AT THE FIRST BLOCK.
C-----------------------------------------------------------------------
      ICURR=1
C-----------------------------------------------------------------------
C START OF PROCEDURE FOR THE CURRENT BLOCK.  IT IS INITIALLY UP-ACTIVE,
C   AND NEITHER UP- NOR DOWN-SATISIFIED.
C
C UP-ACTIVE  (IACTIV=1) means we are comparing DHAT for the current
C   block with DHAT for blocks to its right.
C
C DOWN-ACTIVE (IACTIV=0) means we are comparing DHAT for the current
C   block with DHAT for blocks to its left.
C
C UP-SATISFIED means that DHAT for the current block is less than DHAT
C   for the block to its right.
C
C DOWN-SATISFIED means that DHAT for the current block is greater than
C   DHAT for the block to its left.
C
C NSATIS will be 2 once a block is both UP- and DOWN-SATISFIED.
C
C-----------------------------------------------------------------------
   25 IACTIV=1
      NSATIS=0
C---Compute DHAT for the current block
      DHATAV=DHAT(ICURR)/IWORK(ICURR)
C-----------------------------------------------------------------------
C   ACCORDING TO CURRENT ACTIVITY, CHECK WHETHER CURRENT BLOCK IS
C     UP-SATISFIED (IACTIV=1) OR DOWN-SATISIFED (IACTIV=0)
C-----------------------------------------------------------------------
   30 IF (IACTIV.EQ.0) THEN
C-----------------------------------------------------------------------
C     CHECK WHETHER THIS BLOCK IS DOWN-SATISIFIED.  IF NOT, MERGE.
C-----------------------------------------------------------------------
        IF (ICURR.EQ.1) THEN
C---If it's the first block, it is by definition down-satisfied
          NSATIS=NSATIS+1
        ELSEIF (DHATAV.GT.DHAT(ICURR-1)/IWORK(ICURR-1)) THEN
C---Current block is down-satisfied
          NSATIS=NSATIS+1
        ELSE
C---Current block is not down-satisfied, so merge it with the block to
C   its left and make the new merged block the current block
          ICNEW=ICURR-IWORK(ICURR-1)
          JCNEW=ICURR+IWORK(ICURR)-1
          IWORK(ICNEW)=JCNEW-ICNEW+1
          IWORK(JCNEW)=IWORK(ICNEW)
          DHAT(ICNEW)=DHAT(ICURR-1)+DHAT(ICURR)
          DHAT(JCNEW)=DHAT(ICNEW)
          ICURR=ICNEW
          DHATAV=DHAT(ICURR)/IWORK(ICURR)
          NSATIS=0
        ENDIF
      ELSE
C-----------------------------------------------------------------------
C     CHECK WHETHER THIS BLOCK IS UP-SATISIFIED.  IF NOT, MERGE.
C-----------------------------------------------------------------------
C---Index of first member of the block to the right
        INEXT=ICURR+IWORK(ICURR)
        IF (INEXT.GT.N) THEN
C---If current block is last block, it is, by definition up-satisified
          NSATIS=NSATIS+1
        ELSEIF (DHATAV.LT.DHAT(INEXT)/IWORK(INEXT)) THEN
C---Current block is up-satisfied
          NSATIS=NSATIS+1
        ELSE
C---Current block is not up-satisfied, so merge it with the block to
C   its right and make the new merged block the current block
          IWORK(ICURR)=IWORK(ICURR)+IWORK(INEXT)
          JCURR=ICURR+IWORK(ICURR)-1
          IWORK(JCURR)=IWORK(ICURR)
          DHAT(ICURR)=DHAT(ICURR)+DHAT(INEXT)
          DHAT(JCURR)=DHAT(ICURR)
          DHATAV=DHAT(ICURR)/IWORK(ICURR)
          NSATIS=0
        ENDIF
      ENDIF
C-----------------------------------------------------------------------
C TOGGLE ACTIVITY FOR THE CURRENT BLOCK.  CHECK IF IT IS BOTH UP- AND
C   DOWN-SATISFIED.  IF NOT, GO BACK AND KEEP TRYING.
C-----------------------------------------------------------------------
      IACTIV=1-IACTIV
      IF (NSATIS.LT.2) GO TO 30
C-----------------------------------------------------------------------
C ADVANCE TO THE NEXT BLOCK, CHECK IF FINISHED
C-----------------------------------------------------------------------
      ICURR=ICURR+IWORK(ICURR)
      IF (ICURR.LE.N) GO TO 25
C***********************************************************************
C CHECKING AND MERGING OF BLOCKS IS COMPLETED.  NOW PUT APPROPRIATE
C   AVERAGE VALUES IN ALL ELEMENTS OF DHAT
C-----------------------------------------------------------------------
      ICURR=1
   40   IF (IWORK(ICURR).GT.2) THEN
          DHATAV=DHAT(ICURR)/IWORK(ICURR)
          DO J=ICURR,ICURR+IWORK(ICURR)-1
            DHAT(J)=DHATAV
          ENDDO
        ELSEIF (IWORK(ICURR).EQ.2) THEN
          DHAT(ICURR)=0.5*DHAT(ICURR)
          DHAT(ICURR+1)=DHAT(ICURR)
        ENDIF
        ICURR=ICURR+IWORK(ICURR)
      IF (ICURR.LT.N) GO TO 40
      RETURN
      END SUBROUTINE MONREG

      SUBROUTINE NEWCON (X,GRAD,NOBJ,NDIM,MAXOBJ,STEP,SFGR)
C
C COMPUTES NEW CONFIGURATION, CHANGING EACH CO-ORDINATE ACCORDING TO
C   THE NEGATIVE GRADIENT.
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      DOUBLE PRECISION X(MAXOBJ,NDIM), GRAD(MAXOBJ,NDIM), STEP, SFGR,
     .  FACTOR
C
      FACTOR=STEP/SFGR
      DO IDIM=1,NDIM
        DO IOBJ=1,NOBJ
          X(IOBJ,IDIM)=X(IOBJ,IDIM)+FACTOR*GRAD(IOBJ,IDIM)
        ENDDO
      ENDDO
      RETURN
      END SUBROUTINE NEWCON

      SUBROUTINE NRMCON (X,NOBJ,NDIM,MAXOBJ,SSFACT)
C
C NORMALIZES THE CURRENT CONFIGURATION IN X(NOBJ,NDIM).  THE CENTROID
C   IS MOVED TO THE ZERO ORIGIN AND THE SUM OF SQUARES ADJUSTED SUCH
C   THAT THE RMS DISTANCE BETWEEN POINTS AND THE ORIGIN IS 1.0.
C
C ON RETURN, SSFACT CONTAINS THE SCALING FACTOR WHICH WAS USED.
C
C Written by Dr. Peter R. Minchin
C            Department of Biological Sciences
C            Southern Illinois University Edwardsville
C            PO Box 1651
C            Edwardsville, IL 62026-1651, U.S.A.
C            Phone: +1-618-650-2975   FAX: +1-618-650-3174
C            Email: pminchi@siue.edu
C
      DOUBLE PRECISION X(MAXOBJ,NDIM), SSFACT, FN, FMEAN
C
C CENTER THE CONFIGURATION AND COMPUTE SCALING FACTOR
C
      FN=DBLE(NOBJ)
      SSFACT=0.0D0
      DO IDIM=1,NDIM
        FMEAN=0.0D0
        DO IOBJ=1,NOBJ
          FMEAN=FMEAN+X(IOBJ,IDIM)
        ENDDO
        FMEAN=FMEAN/FN
        DO IOBJ=1,NOBJ
          X(IOBJ,IDIM)=X(IOBJ,IDIM)-FMEAN
          SSFACT=SSFACT+X(IOBJ,IDIM)**2
        ENDDO
      ENDDO
      SSFACT=SQRT(FN/SSFACT)
C
C SCALE THE CONFIGURATION
C
      CALL MAMAS (X,MAXOBJ,NOBJ,NDIM,SSFACT)
      RETURN
      END SUBROUTINE NRMCON

`plot.isomap` <-
    function (x, net = TRUE, n.col = "gray", type = "points", ...) 
{
    type <- match.arg(type, c("points", "text", "none"))
    if (!net) {
        pl <- ordiplot(x, display="sites", type = type, ...)
    } else {
        pl <- ordiplot(x, display = "sites", type = "none", ...)
        z <- scores(pl, "sites")
        k <- x$net
        ## recycle colour for points
        n.col <- rep(n.col, length = nrow(z))
        n.col <- col2rgb(n.col)/255
        ## get average of colours of connected points
        n.col <- (n.col[,k[,1]] + n.col[,k[,2]])/2
        n.col <- rgb(t(n.col))
        segments(z[k[,1],1], z[k[,1],2], z[k[,2],1], z[k[,2],2], col=n.col)
        if (type == "points")
            points(pl, "sites", ...)
        else if (type == "text")
            ordilabel(pl, ...)
    }
    invisible(pl)
}

plot.contribdiv <-
function(x, sub, xlab, ylab, ylim, col, ...) {
    y <- x[,c(1,3)]
    if (missing(ylab))
        ylab <- paste("Diversity components (", attr(x, "index"), ")", sep = "")
    if (missing(xlab))
        xlab <- "Sites"
    if (missing(sub))
        sub <- paste("Differentiation coefficient = ", round(attr(x, "diff.coef"),3), sep = "")
    if (missing(ylim))
        ylim <- c(0, max(y))
    if (missing(col))
        col <- c("lightgrey", "darkgrey")
    matplot(y, type = "n", sub=sub, xlab=xlab, ylab=ylab, axes = FALSE,
            bty = "n", ...)
    polygon(c(1,1:nrow(y),nrow(y)), c(0,y$gamma,0), col=col[1])
    polygon(c(1,1:nrow(y),nrow(y)), c(0,y$alpha,0), col=col[2])
    axis(side = 1)
    axis(side = 2)
    box()
    invisible(x)
}

"print.nesteddisc" <-
function(x, ...)
{
    cat("nestedness discrepancy:", x$statistic, "\n")
    if(x$ties)
        cat("There are tied column frequencies: result can depend on input order\n")
    invisible(x)
}

`predict.specaccum` <-
    function(object, newdata, interpolation = c("linear", "spline"), ...)
{
    if (missing(newdata))
        out <- object$richness
    else {
        interpolation <- match.arg(interpolation)
        newdata <- drop(as.matrix(newdata))
        if (length(dim(newdata)) > 1)
            stop("function accepts only one variable as 'newdata'")
        ## Estimation uses lchoose(), but for predict we need to
        ## estimates on non-integer sample sizes and therefore we use
        ## lgamma(). Original "rarefaction" used sample sizes rounded
        ## to integers, but here we can use non-integer data and hence
        ## get different results.
        if (object$method %in% c("exact", "rarefaction")) {
            lg <- function(n, k) {
                ifelse(k <= n, lgamma(pmax(n, 0) + 1) - lgamma(k+1) -
                    lgamma(pmax(n-k, 0) + 1), -Inf)
            }
            if (object$method == "exact")
                n <- length(object$sites)
            else {
                n <- sum(object$freq)
                newdata <- newdata / length(object$sites) * n
            }
            ldiv <- lg(n, newdata)
            out <- numeric(length(ldiv))
            for(i in seq_along(newdata)) {
                out[i] <- sum(1 - exp(lg(n-object$freq, newdata[i])
                                      - ldiv[i]))
            }
        } else if (object$method == "coleman") {
            ## "coleman" also works on non-integer newdata
            n <- length(object$sites)
            out <- sapply(newdata,
                          function(x) sum(1 - (1 - x/n)^object$freq))
        } else {
            ## Other methods do not accept non-integer newdata, but we
            ## can interpolate
            if (interpolation == "linear")
                out <- approx(x = object$sites, y = object$richness,
                              xout = newdata, rule = 1)$y
            else
                out <- spline(x = object$sites, y = object$richness,
                              xout = newdata, ...)$y
        }
    }
    out
}

`anova.prc` <-
    function(object, ...)
{
    ## if user specified 'by', cast prc() to an rda() and call anova
    ## on its result
    extras <- match.call(expand.dots = FALSE)
    if ("by" %in% names(extras$...)) {
        Y <- as.character(object$call$response)
        X <- as.character(object$call$treatment)
        Z <- as.character(object$call$time)
        fla <- paste(Y, "~", X, "*", Z, "+ Condition(", Z, ")")
        fla <- as.formula(fla)
        ## get extras
        m <- match(c("data", "scale", "subset", "na.action"),
                   names(object$call), 0)
        call <- object$call[c(1,m)]
        call$formula <- fla
        call[[1]] <- as.name("rda")
        object <- eval(call, parent.frame())
        anova(object, ...)
    } else {
        NextMethod("anova", object, ...)
    }    
}

`calibrate.cca` <-
    function(object, newdata, rank = "full", ...)
{
    ## inversion solve(b) requires a square matrix, and we should
    ## append imaginary dims to get those in dbrda with negative
    ## constrained eigenvalues. Work is need to to verify this can be
    ## done, and therefore we just disable calibrate with negative
    ## eigenvalues in constraints.
    if (inherits(object, "dbrda") && object$CCA$poseig < object$CCA$qrank)
        stop("cannot be used with 'dbrda' with imaginary constrained dimensions")
    if (!is.null(object$pCCA))
        stop("does not work with conditioned (partial) models")
    if (is.null(object$CCA) || object$CCA$rank == 0)
        stop("needs constrained model")
    if (object$CCA$rank < object$CCA$qrank)
        stop("rank of constraints is higher than rank of dependent data")
    if (rank != "full")
        rank <- min(rank, object$CCA$rank)
    else
        rank <- object$CCA$rank
    if (missing(newdata))
        wa <- object$CCA$wa
    else
        wa <- predict(object, type="wa", newdata=newdata)
    qrank <- object$CCA$qrank
    b <- (coef(object))[object$CCA$QR$pivot[1:qrank], , drop=FALSE]
    b <- solve(b)
    pred <- wa[ , 1:rank, drop=FALSE]  %*% b[1:rank, , drop =FALSE]
    envcen <- object$CCA$envcentre[object$CCA$QR$pivot]
    envcen <- envcen[1:object$CCA$qrank]
    sweep(pred, 2, envcen, "+")
}

`mrpp` <-
    function (dat, grouping, permutations = 999, distance = "euclidean", 
              weight.type = 1, strata = NULL,
              parallel = getOption("mc.cores")) 
{
    EPS <- sqrt(.Machine$double.eps)
    classmean <- function(ind, dmat, indls) {
        sapply(indls, function(x)
               mean(c(dmat[ind == x, ind == x]),
                    na.rm = TRUE))
    }
    mrpp.perms <- function(ind, dmat, indls, w) {
        weighted.mean(classmean(ind, dmat, indls), w = w, na.rm = TRUE)
    }
    if (inherits(dat, "dist")) 
        dmat <- dat
    else if ((is.matrix(dat) || is.data.frame(dat)) &&
               isSymmetric(unname(as.matrix(dat)))) {
        dmat <- dat
        attr(dmat, "method") <- "user supplied square matrix"
    }
    else dmat <- vegdist(dat, method = distance)
    if (any(dmat < -sqrt(.Machine$double.eps)))
        stop("dissimilarities must be non-negative")
    distance <- attr(dmat, "method")
    dmat <- as.matrix(dmat)
    diag(dmat) <- NA
    N <- nrow(dmat)
    grouping <- factor(grouping)
    indls <- levels(grouping)
    ncl <- sapply(indls, function(x) sum(grouping == x))
    w <- switch(weight.type, ncl, ncl - 1, ncl * (ncl - 1)/2)
    classdel <- classmean(grouping, dmat, indls)
    names(classdel) <- names(ncl) <- indls
    del <- weighted.mean(classdel, w = w, na.rm = TRUE)
    E.del <- mean(dmat, na.rm = TRUE)
    ## 'Classification strength' if weight.type == 1
    ## Do not calculate classification strength because there is no
    ## significance test for it. Keep the item in reserve for
    ## possible later re-inclusion.
    CS <- NA
    permutations <- getPermuteMatrix(permutations, N, strata = strata)
    if (ncol(permutations) != N)
        stop(gettextf("'permutations' have %d columns, but data have %d rows",
                      ncol(permutations), N))

    control <- attr(permutations, "control")
    if(nrow(permutations)) {
        perms <- apply(permutations, 1, function(indx) grouping[indx])
        permutations <- ncol(perms)

        ## Parallel processing
        if (is.null(parallel))
            parallel <- 1
        hasClus <- inherits(parallel, "cluster")
        if (hasClus || parallel > 1) {
            if(.Platform$OS.type == "unix" && !hasClus) {
                m.ds <- unlist(mclapply(1:permutations, function(i, ...)
                                        mrpp.perms(perms[,i], dmat, indls, w),
                                        mc.cores = parallel))
            } else {
                if (!hasClus) {
                    parallel <- makeCluster(parallel)
                }
                m.ds <- parCapply(parallel, perms, function(x)
                                  mrpp.perms(x, dmat, indls, w))
                if (!hasClus)
                    stopCluster(parallel)
            }
        } else {
            m.ds <- apply(perms, 2, function(x) mrpp.perms(x, dmat, indls, w))
        }
        p <- (1 + sum(del + EPS >= m.ds))/(permutations + 1)
        r2 <- 1 - del/E.del
    } else { # no permutations
        m.ds <- p <- r2 <- NA
        permutations <- 0
    }
    out <- list(call = match.call(), delta = del, E.delta = E.del, CS = CS,
                n = ncl, classdelta = classdel, Pvalue = p, A = r2,
                distance = distance, weight.type = weight.type,
                boot.deltas = m.ds, permutations = permutations,
                control = control)
    class(out) <- "mrpp"
    out
}

`labels.envfit` <-
    function(object, ...)
{
    out <- list("vectors" = rownames(object$vectors$arrows),
                "factors" = rownames(object$factors$centroids))
    if (is.null(out$vectors) || is.null(out$factors))
        out <- unlist(out, use.names = FALSE)
    out
}

fitspecaccum <-
    function(object, model, method = "random",  ...)
{
    MODELS <- c("arrhenius", "gleason", "gitay", "lomolino",
                "asymp", "gompertz", "michaelis-menten", "logis",
                "weibull")
    model <- match.arg(model, MODELS)
    if (!inherits(object, "specaccum")) 
        object <- specaccum(object, method = method, ...)
    if (is.null(object$perm))
        SpeciesRichness <- as.matrix(object$richness)
    else
        SpeciesRichness <- object$perm
    if (!is.null(object$individuals))
        x <- object$individuals
    else
        x <- object$sites
    hasWeights <- !is.null(object$weights)
    ## scale weights to correspond to the no. of sites
    if (hasWeights) {
        w <- as.matrix(object$weights)
        n <- nrow(w)
        w <- sweep(w, 2, w[n,], "/") * n
    }
    NLSFUN <- function(y, x, model, ...) {
        switch(model,
        "arrhenius" = nls(y ~ SSarrhenius(x, k, z),  ...),
        "gleason" = nls(y ~ SSgleason(x, k, slope),  ...),
        "gitay" = nls(y ~ SSgitay(x, k, slope), ...),
        "lomolino" = nls(y ~ SSlomolino(x, Asym, xmid, slope), ...),
        "asymp" = nls(y ~ SSasymp(x, Asym, R0, lrc), ...),
        "gompertz" = nls(y ~ SSgompertz(x, Asym, xmid, scal), ...),
        "michaelis-menten" = nls(y ~ SSmicmen(x, Vm, K),  ...),
        "logis" = nls(y ~ SSlogis(x, Asym, xmid, scal),  ...),
        "weibull" = nls(y ~ SSweibull(x, Asym, Drop, lrc, par), ...))
    }
    mods <- lapply(seq_len(NCOL(SpeciesRichness)),
                  function(i, ...)
                   NLSFUN(SpeciesRichness[,i],
                          if (hasWeights) w[,i] else x,
                          model, ...), ...)
    object$fitted <- drop(sapply(mods, fitted))
    object$residuals <- drop(sapply(mods, residuals))
    object$coefficients <- drop(sapply(mods, coef))
    object$SSmodel <- model
    object$models <- mods
    object$call <- match.call()
    class(object) <- c("fitspecaccum", class(object))
    object
}

### plot function

`plot.fitspecaccum` <-
    function(x, col = par("fg"), lty = 1, 
             xlab = "Sites", ylab = x$method, ...)
{
    if (is.null(x$weights))
        fv <- fitted(x)
    else
        fv <- sapply(x$models, predict, newdata = list(x = x$sites))
    matplot(x$sites, fv, col = col, lty = lty, pch = NA,
            xlab = xlab, ylab = ylab, type = "l", ...)
    invisible()
}

`lines.fitspecaccum` <-
    function(x, col = par("fg"), lty = 1, ...)
{
    if (is.null(x$weights))
        fv <- fitted(x)
    else
        fv <- sapply(x$models, predict, newdata= list(x = x$sites))
    matlines(x$sites, fv, col = col, lty = lty, pch = NA, type = "l", ...)
    invisible()
}

`adonis2` <-
    function(formula, data, permutations = 999, method = "bray",
             sqrt.dist = FALSE, add = FALSE, by = "terms",
             parallel = getOption("mc.cores"), ...)
{
    ## we accept only by = "terms", "margin" or NULL
    if (!is.null(by))
        by <- match.arg(by, c("terms", "margin"))
    ## evaluate lhs
    YVAR <- formula[[2]]
    lhs <- eval(YVAR, environment(formula), globalenv())
    environment(formula) <- environment()
    ## Take care that input lhs are dissimilarities
    if ((is.matrix(lhs) || is.data.frame(lhs)) &&
        isSymmetric(unname(as.matrix(lhs))))
        lhs <- as.dist(lhs)
    if (!inherits(lhs, "dist"))
        lhs <- vegdist(as.matrix(lhs), method=method, ...)
    ## adjust distances if requested
    if (sqrt.dist)
        lhs <- sqrt(lhs)
    if (is.logical(add) && isTRUE(add))
        add <- "lingoes"
    if (is.character(add)) {
        add <- match.arg(add, c("lingoes", "cailliez"))
        if (add == "lingoes") {
            ac <- addLingoes(as.matrix(lhs))
            lhs <- sqrt(lhs^2 + 2 * ac)
        }
        else if (add == "cailliez") {
            ac <- addCailliez(as.matrix(lhs))
            lhs <- lhs + ac
        }
    }
    ## adonis0 & anova.cca should see only dissimilarities (lhs)
    if (!missing(data)) # expand and check terms
        formula <- terms(formula, data=data)
    formula <- update(formula, lhs ~ .)
    ## no data? find variables in .GlobalEnv
    if (missing(data))
        data <- model.frame(delete.response(terms(formula)))
    sol <- adonis0(formula, data = data, method = method)
    out <- anova(sol, permutations = permutations, by = by,
                 parallel = parallel)
    ## attributes will be lost when adding a new column
    att <- attributes(out)
    ## add traditional adonis output on R2
    out <- rbind(out, "Total" = c(nobs(sol)-1, sol$tot.chi, NA, NA))
    out <- cbind(out[,1:2], "R2" = out[,2]/sol$tot.chi, out[,3:4])
    ## Fix output header to show the adonis2() call instead of adonis0()
    att$heading[2] <- deparse(match.call(), width.cutoff = 500L)
    att$names <- names(out)
    att$row.names <- rownames(out)
    attributes(out) <- att
    out
}
`adonis0` <-
    function(formula, data=NULL, method="bray", ...)
{
    ## evaluate data
    if (missing(data))
        data <- .GlobalEnv
    else
        data <- eval(match.call()$data, environment(formula),
                     enclos = .GlobalEnv)
    ## First we collect info for the uppermost level of the analysed
    ## object
    Trms <- terms(delete.response(formula), data = data)
    sol <- list(call = match.call(),
                method = "adonis",
                terms = Trms,
                terminfo = list(terms = Trms))
    sol$call$formula <- formula(Trms)
    TOL <- 1e-7
    Terms <- terms(formula, data = data)
    lhs <- formula[[2]]
    lhs <- eval(lhs, environment(formula)) # to force evaluation
    formula[[2]] <- NULL                # to remove the lhs
    rhs.frame <- model.frame(formula, data, drop.unused.levels = TRUE) # to get the data frame of rhs
    rhs <- model.matrix(formula, rhs.frame) # and finally the model.matrix
    assign <- attr(rhs, "assign") ## assign attribute
    sol$terminfo$assign <- assign[assign > 0]
    rhs <- rhs[,-1, drop=FALSE] # remove the (Intercept) to get rank right
    rhs <- scale(rhs, scale = FALSE, center = TRUE) # center
    qrhs <- qr(rhs)
    ## input lhs should always be dissimilarities
    if (!inherits(lhs, "dist"))
        stop("internal error: contact developers")
    if (any(lhs < -TOL))
        stop("dissimilarities must be non-negative")
    n <- attr(lhs, "Size")
    ## G is -dmat/2 centred
    G <- initDBRDA(lhs)
    ## preliminaries are over: start working
    Gfit <- qr.fitted(qrhs, G)
    Gres <- qr.resid(qrhs, G)
    ## collect data for the fit
    if(!is.null(qrhs$rank) && qrhs$rank > 0)
        CCA <- list(rank = qrhs$rank,
                    qrank = qrhs$rank,
                    tot.chi = sum(diag(Gfit)),
                    QR = qrhs)
    else
        CCA <- NULL # empty model
    ## collect data for the residuals
    CA <- list(rank = n - max(qrhs$rank, 0) - 1,
               u = matrix(0, nrow=n),
               tot.chi = sum(diag(Gres)))
    ## all together
    sol$tot.chi <- sum(diag(G))
    sol$adjust <- 1
    sol$Ybar <- G
    sol$CCA <- CCA
    sol$CA <- CA
    class(sol) <- c("adonis2", "dbrda", "rda", "cca")
    sol
}

`isomapdist` <-
    function(dist, epsilon, k, path="shortest", fragmentedOK = FALSE, ...)
{
    EPS <- 1e-5
    op <- options(warn = 2)
    on.exit(options(op))
    if (!inherits(dist, "dist"))
        dist <- as.dist(dist)
    options(op)
    method <- attr(dist, "method")
    if (missing(epsilon) && missing(k))
        stop("either epsilon or k must be given")
    if (!missing(epsilon) && !missing(k))
        message("both epsilon and k given, using epsilon")
    if (!missing(epsilon))
        dist[dist >= epsilon-EPS] <- NA
    else {
        dist <- as.matrix(dist)
        diag(dist) <- NA
        is.na(dist) <- apply(dist, 2, function(x)
                             x > x[order(x, na.last=TRUE)[k]])
        dist <- pmax(as.dist(dist), as.dist(t(dist)), na.rm = TRUE)
    }
    fragm <- distconnected(dist, toolong=0,  trace=FALSE)
    take <- NULL
    if (length(unique(fragm)) > 1) {
        if (fragmentedOK) {
            warning("data are fragmented: taking the largest fragment")
            take <- fragm == as.numeric(names(which.max(table(fragm))))
            dist <- as.dist(as.matrix(dist)[take,take])
        } else {
            stop("data are fragmented")
        }
    }
    net <- which(!is.na(dist))
    attr(dist, "method") <- method
    dist <- stepacross(dist, path = path, toolong = 0, trace = FALSE)
    if (any(is.na(dist))) {
        grps <- distconnected(dist, toolong=0)
    }
    if (missing(epsilon)) {
        attr(dist, "criterion") <-"k"
        attr(dist, "critval") <- k
    }
    else {
        attr(dist, "criterion") <- "epsilon"
        attr(dist, "critval") <- epsilon
    }
    attr(dist, "method") <- paste(attr(dist, "method"), "isomap")
    attr(dist, "net") <- net
    attr(dist, "take") <- take
    attr(dist, "call") <- match.call()
    dist
}

`fitted.rda` <-
    function (object, model = c("CCA", "CA", "pCCA"),
              type = c("response", "working"), ...)
{
    type <- match.arg(type)
    model <- match.arg(model)
    if (is.null(object[[model]]))
        stop(gettextf("component '%s' does not exist", model))
    Xbar <- ordiYbar(object, model)
    if (type == "response") {
        cent <- attr(Xbar, "scaled:center")
        scal <- attr(Xbar, "scaled:scale")
        if (!is.null(scal)) {
            Xbar <- sweep(Xbar, 2, scal, "*")
            attr(Xbar, "scaled:scale") <- NULL
        }
        Xbar <- Xbar * sqrt(nrow(Xbar) - 1)
        Xbar <- sweep(Xbar, 2, cent, "+")
        attr(Xbar, "scaled:center") <- NULL
    }
    Xbar
}

`poolaccum` <-
    function(x, permutations = 100, minsize = 3)
{
    x <- as.matrix(x)
    n <- nrow(x)
    m <- ncol(x)
    N <- seq_len(n)
    ## specpool() is slow, but the vectorized versions below are
    ## pretty fast. We do not set up parallel processing, but use
    ## permute API.
    permat <- getPermuteMatrix(permutations, n)
    nperm <- nrow(permat)
    S <- chao <- boot <- jack1 <- jack2 <-
        matrix(0, nrow=n, ncol=nperm)
    for (i in 1:nperm) {
        ## It is a bad practice to replicate specpool equations here:
        ## if we change specpool, this function gets out of sync. You
        ## should be ashamed, Jari Oksanen!
        take <- permat[i,]
        tmp <- apply(x[take,] > 0, 2, cumsum)
        S[,i] <- rowSums(tmp > 0)
        ## All-zero species are taken as *known* to be missing in
        ## subsamples, and in the following we subtract them (as
        ## 2*S-m) from the bootstrap samples to give a more unbiased
        ## estimate.
        boot[,i] <- 2*S[,i] - m + rowSums(exp(sweep(log1p(-sweep(tmp, 1, N, "/")), 1, N, "*") ))
        a1 <- rowSums(tmp == 1)
        a2 <- rowSums(tmp == 2)
        chao[, i] <- S[,i] + ifelse(a2 > 0, (N-1)/N*a1*a1/2/a2,
                                    (N-1)/N*a1*(a1-1)/2)
        jack1[,i] <- S[,i] + a1 * (N-1)/N
        jack2[,i] <- S[,i] + a1*(2*N-3)/N - a2*(N-2)^2/N/(N-1)
    }
    means <- cbind(`N` = N, `S` = rowMeans(S), `Chao` =  rowMeans(chao),
                   `Jackknife 1` = rowMeans(jack1),
                   `Jackknife 2` = rowMeans(jack2),
                   `Bootstrap` = rowMeans(boot))
    take <- N >= minsize
    out <- list(S = S[take,], chao = chao[take,], jack1 = jack1[take,],
                jack2 = jack2[take,], boot = boot[take,], N = N[take],
                means = means[take,])
    attr(out, "control") <- attr(permat, "control")
    class(out) <- "poolaccum"
    out
}

`ordiTerminfo` <-
    function(d, data)
{
    Terms <- delete.response(d$terms.expand)
    if (length(attr(Terms, "term.labels")) == 0)
        mf <- data.frame(NULL)
    else
        mf <- d$modelframe
    xlev <- .getXlevels(Terms, mf)
    ordered <- sapply(mf, is.ordered)
    assign <- attr(d$Y, "assign")
    list(terms = Terms, assign = assign, xlev = xlev, ordered = ordered)
}

`estimateR.default` <-
    function (x, ...)
{
    gradF <- function(a, i) {
        .expr4 <- sum(i * a)
        .expr7 <- 1 - a[1]/(1 - sum(i * a))
        .expr8 <- 1/.expr7
        .expr10 <- sum(a)
        .expr12 <- sum(i * (i - 1) * a)
        .expr13 <- sum(a) * sum(i * (i - 1) * a)
        .expr14 <- .expr7 * .expr4
        .expr15 <- .expr4 - 1
        .expr16 <- .expr14 * .expr15
        .expr18 <- .expr13/.expr16 - 1
        .expr20 <- sum(a) + a[1] * .expr18
        .expr23 <- (1 - sum(i * a))^2
        .expr25 <- 1/(1 - sum(i * a)) + a[1]/(1 - sum(i * a))^2
        .expr26 <- .expr7^2
        .expr35 <- .expr16^2
        Grad <- a[1] * i/(.expr23 * .expr26) * .expr20 + .expr8 *
            (1 + a[1] * ((.expr12 + (.expr10 * i * (i - 1)))/.expr16 -
                         .expr13 * ((.expr7 * i - (a[1] * i/.expr23) *
                                     .expr4) * .expr15 + .expr14 * i)/.expr35))
        Grad[1] <- .expr25/.expr26 * .expr20 + .expr8 * (1 +
                                                         (.expr18 + a[1] * (.expr12/.expr16 - .expr13 * ((.expr7 -
                                                                                                          .expr25 * .expr4) * .expr15 + .expr14)/.expr35)))
        Grad
    }
    ## we need integers
    if (!identical(all.equal(x, round(x)), TRUE))
        stop("function accepts only integers (counts)")
    ## and they must be exact
    if (!is.integer(x))
        x <- round(x)
    X <- x[x > 0]
    N <- sum(X)
    SSC <- 1 # (N-1)/N # do NOT use small-sample correction
    T.X <- table(X)
    S.obs <- length(X)
    S.rare <- sum(T.X[as.numeric(names(T.X)) <= 10])
    S.abund <- sum(T.X[as.numeric(names(T.X)) > 10])
    N.rare <- sum(X[X < 11])
    i <- 1:10
    COUNT <- function(i, counts) {
        length(counts[counts == i])
    }
    a <- sapply(i, COUNT, X)
    G <- a[1]/a[2]
    ## EstimateS uses basic Chao only if a[2] > 0, and switches to
    ## bias-corrected version only if a[2] == 0. However, we always
    ## use bias-corrected form. The switchin code is commented out so
    ## that it is easy to put back.

    ##if (a[2] > 0)
    ##    S.Chao1 <- S.obs + SSC * a[1]^2/2/a[2]
    ##else if (a[1] > 0)
    ##
    S.Chao1 <- S.obs + SSC * a[1]*(a[1]-1) / (a[2]+1)/2
    ##else
    ##    S.Chao1 <- S.obs
    Deriv.Ch1 <- gradF(a, i)

    ## The commonly used variance estimator is wrong for bias-reduced
    ## Chao estimate. It is based on the variance estimator of basic
    ## Chao estimate, but replaces the basic terms with corresponding
    ## terms in the bias-reduced estimate. The following is directly
    ## derived from the bias-reduced estimate.

    ## The commonly used one (for instance, in EstimateS):
    ##sd.Chao1 <-
    ##    sqrt(SSC*(a[1]*(a[1]-1)/2/(a[2]+1) +
    ##              SSC*(a[1]*(2*a[1]-1)^2/4/(a[2]+1)^2 +
    ##                  a[1]^2*a[2]*(a[1]-1)^2/4/(a[2]+1)^4)))

    sd.Chao1 <- (a[1]*((-a[2]^2+(-2*a[2]-a[1])*a[1])*a[1] +
                       (-1+(-4+(-5-2*a[2])*a[2])*a[2] +
                        (-2+(-1+(2*a[2]+2)*a[2])*a[2] +
                         (4+(6+4*a[2])*a[2] + a[1]*a[2])*a[1])*a[1])*S.Chao1))/
                             4/(a[2]+1)^4/S.Chao1
    sd.Chao1 <- sqrt(sd.Chao1)

    C.ace <- 1 - a[1]/N.rare
    i <- seq_along(a)
    thing <- i * (i - 1) * a
    Gam <- sum(thing) * S.rare/(C.ace * N.rare * (N.rare - 1)) -
        1
    S.ACE <- S.abund + S.rare/C.ace + max(Gam, 0) * a[1]/C.ace
    sd.ACE <- sqrt(sum(Deriv.Ch1 %*% t(Deriv.Ch1) * (diag(a) -
                                                     a %*% t(a)/S.ACE)))
    out <- list(S.obs = S.obs, S.chao1 = S.Chao1, se.chao1 = sd.Chao1,
                S.ACE = S.ACE, se.ACE = sd.ACE)
    out <- unlist(out)
    out
}

`pregraphKM` <-
    function(matrice)
{
    `row.col.number` <- function(mat,number){
        nr<-nrow(mat)
        nc<-ncol(mat)
        mod<-number %% nr
        div<-number/nr
        ##First column
        if(mod!=0 & div>1){
            nr.f<-mod
            nc.f<-trunc(div)+1
        }else{
            if(div<=1){
                nc.f<-1
                nr.f<-number
            }else{
                if(mod==0){
                    nc.f<-div
                    nr.f<-nr
                }
            }
        }
        list(nr=nr.f,nc=nc.f)
    }

    ## Beginning of the function

    for(k in 1:(ncol(matrice)-1)){
        i=1
        j=1
        tmp<-0
        while(j <= max(matrice[,k])){
            if(i==1){
                mat<-table(matrice[,k],matrice[,k+1])
                number<-which.max(mat)
                tmp<-row.col.number(mat,number)
                tmp0<-tmp
                ## Change les indices 
                if(tmp$nr!=tmp$nc){
                    find.nc<-which(matrice[,k+1]==tmp$nc)
                    find.nr<-which(matrice[,k+1]==tmp$nr)
                    matrice[find.nc,k+1]<-tmp$nr
                    matrice[find.nr,k+1]<-tmp$nc
                }else{}
                i=2
            }else{
                mat<-table(matrice[,k],matrice[,k+1])
                mat[tmp0$nr,]<-0
                mat[,tmp0$nr]<-0
                number<-which.max(mat)
                tmp<-row.col.number(mat,number)
                ## Change les indices 
                if(tmp$nr!=tmp$nc){
                    find.nc<-which(matrice[,k+1]==tmp$nc)
                    find.nr<-which(matrice[,k+1]==tmp$nr)
                    matrice[find.nc,k+1]<-tmp$nr
                    matrice[find.nr,k+1]<-tmp$nc
                }else{}
                tmp0$nr<-c(tmp0$nr,tmp$nr)
            }
            j=j+1
        }
    }
    matrice
}

`bstick.princomp` <-
    function(n, ...)
{
    if(!inherits(n, "princomp"))
        stop("'n' not of class \"princomp\"")
    tot.chi <- sum(n$sdev^2)
    n.comp <- length(n$sdev)
    res <- bstick.default(n.comp, tot.chi, ...)
    names(res) <- dimnames(n$loadings)[[2]]
    res
}

`nestednodf` <- 
    function(comm, order = TRUE, weighted = FALSE, wbinary = FALSE) 
{
    bin.comm <- ifelse(comm > 0, 1, 0)
    rfill <- rowSums(bin.comm)
    cfill <- colSums(bin.comm)
    if (!weighted)
        comm <- bin.comm
    if (order) {
        if (weighted) {
            rgrad <- rowSums(comm)
            cgrad <- colSums(comm)
            rorder <- order(rfill, rgrad, decreasing = TRUE)
            corder <- order(cfill, cgrad, decreasing = TRUE)
        } else {
            rorder <- order(rfill, decreasing = TRUE)
            corder <- order(cfill, decreasing = TRUE)
        }
        comm <- comm[rorder, corder]
        rfill <- rfill[rorder]
        cfill <- cfill[corder]
    }
    nr <- NROW(comm)
    nc <- NCOL(comm)
    fill <- sum(rfill)/prod(dim(comm))
    N.paired.rows <- numeric(nr * (nr - 1)/2)
    N.paired.cols <- numeric(nc * (nc - 1)/2)
    counter <- 0
    for (i in 1:(nr - 1)) {
        first <- comm[i, ]
        for (j in (i + 1):nr) {
            counter <- counter + 1
            if (rfill[i] <= rfill[j] || any(rfill[c(i, j)] == 0)) 
                next
            if (weighted) {
                second <- comm[j, ]
                if (!wbinary) 
                    N.paired.rows[counter] <-
                        sum(first - second > 0 & second > 0)/sum(second > 0)
                else
                    N.paired.rows[counter] <-
                        sum(first - second >= 0 & second > 0)/sum(second > 0)
            }
            else {
                N.paired.rows[counter] <-
                    sum(first + comm[j, ] == 2)/rfill[j]
            }
        }
    }
    counter <- 0
    for (i in 1:(nc - 1)) {
        first <- comm[, i]
        for (j in (i + 1):nc) {
            counter <- counter + 1
            if (cfill[i] <= cfill[j] || any(cfill[c(i, j)] == 0)) 
                next
            if (weighted) {
                second <- comm[, j]
                if (!wbinary)
                    N.paired.cols[counter] <-
                        sum(first - second > 0 & second > 0)/sum(second > 0)
                else
                    N.paired.cols[counter] <-
                        sum(first - second >= 0 & second > 0)/sum(second > 0)
            }
            else {
                N.paired.cols[counter] <-
                    sum(first + comm[, j] == 2)/cfill[j]
            }
        }
    }
    N.columns <- mean(N.paired.cols) * 100
    N.rows <- mean(N.paired.rows) * 100
    NODF <- (sum(c(N.paired.rows, N.paired.cols)) * 100)/
        ((nc * (nc - 1)/2) + (nr * (nr - 1)/2))
    out <- list(comm = comm, fill = fill,
                statistic = c(N.columns = N.columns, N.rows = N.rows, NODF = NODF))
    class(out) <- "nestednodf"
    out
}

"weights.cca" <-
    function (object, display = "sites", ...) 
{
    display <- match.arg(display, c("sites", "species", "lc", "wa"))
    if (display %in% c("sites", "lc", "wa")) {
        if (!is.null(object$na.action) &&
            inherits(object$na.action, "exclude")) {
            object$rowsum <- napredict(object$na.action, object$rowsum)
            object$rowsum[object$na.action] <- object$rowsum.excluded
            }
        object$rowsum
    }
    else object$colsum
}

`points.radline` <-
    function (x, ...) 
{
    poi <- x$y
    rnk <- seq(along = poi)
    points(rnk, poi, ...)
    out <- list(species = cbind(rnk, poi))
    class(out) <- "ordiplot"
    invisible(out)
}

`points.radfit` <-
    function(x, ...)
{
    points.radline(x, ...)
}

## evaluate user-defined dissimilarity function.

`designdist` <-
    function (x, method = "(A+B-2*J)/(A+B)",
              terms = c("binary", "quadratic", "minimum"),
              abcd = FALSE, alphagamma = FALSE, name)
{
    terms <- match.arg(terms)
    if ((abcd || alphagamma) && terms != "binary")
        warning("perhaps terms should be 'binary' with 'abcd' or 'alphagamma'?")
    x <- as.matrix(x)
    ## only do numeric data for which "pa", minimum and quadratic make sense
    if (!(is.numeric(x) || is.logical(x)))
        stop("input data must be numeric")
    N <- nrow(x)
    P <- ncol(x)
    if (terms == "binary")
        x <- ifelse(x > 0, 1, 0)
    if (terms == "binary" || terms == "quadratic")
        x <- tcrossprod(x)
    if (terms == "minimum")
        x <- .Call(do_minterms, as.matrix(x))
    d <- diag(x)
    A <- as.dist(outer(rep(1, N), d))
    B <- as.dist(outer(d, rep(1, N)))
    J <- as.dist(x)
    ## 2x2 contingency table notation
    if (abcd) {
        a <- J
        b <- A - J
        c <- B - J
        d <- P - A - B + J
    }
    ## beta diversity notation
    if (alphagamma) {
        alpha <- (A + B)/2
        gamma <- A + B - J
        delta <- abs(A - B)/2
    }
    dis <- eval(parse(text = method))
    attributes(dis) <- attributes(J)
    attr(dis, "call") <- match.call()
    if (missing(name))
        attr(dis, "method") <- paste(terms, method)
    else attr(dis, "method") <- name
    dis
}

## similar to designdist, but uses Chao's terms U & V instead of J, A,
## B (or their derived terms) in designdist. I considered having this
## as an option 'terms = "chao"' in designdist, but there really is so
## little in common and too many if's needed.

`chaodist` <-
    function(x, method = "1 - 2*U*V/(U+V)", name)
{
    x <- as.matrix(x)
    ## need integer data
    if (!identical(all.equal(x, round(x)), TRUE))
        stop("function accepts only integers (counts)")
    N <- nrow(x)
    ## do_chaoterms returns a list with U, V which are non-classed
    ## vectors where the order of terms matches 'dist' objects
    vu <- .Call(do_chaoterms, x)
    U <- vu$U
    V <- vu$V
    ## dissimilarities
    dis <- eval(parse(text = method))
    dis <- structure(dis, Size = N, Labels = rownames(x), Diag = FALSE,
                     Upper = FALSE, call = match.call(), class = "dist")
    if (missing(name))
        attr(dis, "method") <- paste("chao", method)
    else
        attr(dis, "method") <- name
    dis
}

SSlomolino <-
    selfStart(~ Asym/(1 + slope^log(xmid/area)),
              function(mCall, data, LHS)
{
    xy <- sortedXyData(mCall[["area"]], LHS, data)
    ## approximate with Arrhenius model on log-log
    .p <- coef(lm(log(xy[["y"]]) ~ log(xy[["x"]])))
    ## Asym is value at max(x) but > max(y) and xmid is x which gives
    ## Asym/2
    .Smax <- max(xy[["y"]])*1.1
    .S <- exp(.p[1] + log(max(xy[["x"]])) * (.p[2]))
    .S <- max(.S, .Smax)
    .xmid <- exp((log(.S/2) - .p[1])/.p[2])
    ## approximate slope for log(Asym/y - 1) ~ log(xmid/x) + 0
    .y <- log(.S/xy[["y"]] - 1)
    .z <- log(.xmid/xy[["x"]])
    .b <- coef(lm(.y ~ .z - 1))
    value <- c(.S, .xmid, exp(.b))
    names(value) <- mCall[c("Asym","xmid", "slope")]
    value
},
c("Asym","xmid","slope"))

`summary.specaccum` <-
    function(object, ...)
{
    if (is.null(object$perm))
        stop("specific summary available only for method=\"random\"")
    else {
        tmp <- summary(t(object$perm), ...)
        colnames(tmp) <- paste(1:ncol(tmp), "sites")
        tmp
    }
}

"rankindex" <-
function (grad, veg, indices = c("euc", "man", "gow", "bra", 
    "kul"), stepacross = FALSE, method = "spearman",
     metric = c("euclidean", "mahalanobis", "manhattan", "gower"), ...) 
{
    metric = match.arg(metric)
    grad <- as.data.frame(grad)
    if (any(sapply(grad, is.factor))) {
        span <- daisy(grad)
    } else {
        span <- switch(metric,
                       "euclidean" = dist(scale(grad, scale=TRUE)),
                       "mahalanobis" = dist(veganMahatrans(scale(grad, scale=FALSE))),
                       "manhattan" = dist(decostand(grad, "range"), "manhattan"),
                       "gower" = daisy(grad, metric = "gower"))
    }
    veg <- as.matrix(veg)
    res <- numeric(length(indices))
    ## create names if indices is a list of functions without names
    if (is.list(indices)) {
        nam <- names(indices)
        if (is.null(nam))
            nam <- paste("dis", 1:length(indices), sep="")
    } else nam <- indices
    names(res) <- nam
    ## indices is a list of functions which return dist objects
    if (is.list(indices)) {
        for (i in seq_along(indices)) {
            ## don't accept similarities
            if (indices[[i]](matrix(1, 2, 2)) != 0)
                stop("define dissimilarity and not similarity")
            y <- indices[[i]](veg)
            ## check class of output
            if (!inherits(y, "dist"))
                stop("function in 'indices' must return a 'dist' object")
            if (stepacross) {
                is.na(y) <- no.shared(veg)
                y <- stepacross(y, trace = FALSE, toolong = -1, ...)
            }
            res[i] <- cor(span, y, method = method)
        }
    ## indices is a character vector naming methods in vegdist
    } else {
        for (i in indices) {
            y <- vegdist(veg, i)
            if (stepacross) {
                is.na(y) <- no.shared(veg)
                y <- stepacross(y, trace = FALSE, toolong = -1, ...)
            }
            res[i] <- cor(span, y, method = method)
        }
    }
    res
}

`radlattice` <-
    function(x, BIC = FALSE, ...)
{
    if (!inherits(x, "radfit"))
        stop("function only works with 'radfit' results for single site")
    y <- x$y
    fv <- unlist(fitted(x))
    mods <- names(x$models)
    p <- length(mods)
    n <- length(y)
    Abundance <- rep(y, p)
    Rank <- rep(1:n, p)
    Model <- factor(rep(mods, each=n), levels = mods)
    if (BIC)
        k <- log(length(y))
    else
        k <- 2
    aic <- AIC(x, k = k)
    col <- trellis.par.get("superpose.line")$col
    if (length(col) > 1)
        col <- col[2]
    xyplot(Abundance ~  Rank | Model, subscripts = TRUE,
           scales = list(y = list(log = 2)), as.table = TRUE,
           panel = function(x, y, subscripts) {
               panel.xyplot(x, y, ...)
               panel.xyplot(x, log2(fv[subscripts]), type="l", lwd=3,
                            col = col, ...)
               panel.text(max(x), max(y), paste(if (BIC) "BIC" else "AIC", "=",
                          formatC(aic[panel.number()], digits=2, format="f")),
                          pos=2)
           }
           )
}

`model.frame.cca` <-
    function (formula, ...)
{
    call <- formula$call
    m <- match(c("formula", "data", "na.action", "subset"), names(call),
        0)
    call <- call[c(1, m)]
    ## did we succeed? Fails if we have no formula, in prc and if
    ## there was no data= argument
    if (is.null(call$data))
        stop("no sufficient information to reconstruct model frame")
    ## subset must be evaluated before ordiParseFormula
    if (!is.null(call$subset))
        call$subset <- formula$subset
    if (is.null(call$na.action))
        call$na.action <- na.pass
    data <- eval(call$data, environment(call$formula), .GlobalEnv)
    out <- ordiParseFormula(call$formula, data, na.action = call$na.action,
                            subset = call$subset)
    mf <- out$modelframe
    attr(mf, "terms") <- out$terms.expand
    if (!is.null(out$na.action))
        attr(mf, "na.action") <- out$na.action
    mf
}

`showvarparts` <-
    function(parts, labels, bg = NULL, alpha=63, Xnames, id.size=1.2, ...)
{
    rad <- 0.725
    ## Default names
    if (missing(Xnames))
        Xnames <- paste("X", seq_len(parts), sep="")
    ## transparent fill colours
    if (!is.null(bg)) {
        bg <- rgb(t(col2rgb(bg)), alpha = alpha, maxColorValue = 255)
        if (length(bg) < parts)
            bg <- rep(bg, length.out = parts)
    }
    ## centroids of circles (parts < 4) or individual fractions (parts
    ## == 4)
    cp <- switch(parts,
                 matrix(c(0,0), ncol=2, byrow=TRUE),
                 matrix(c(0,0, 1,0), ncol=2, byrow=TRUE),
                 matrix(c(0,0, 1,0, 0.5, -sqrt(3/4)), ncol=2, byrow=TRUE),
                 structure(
                     c(-1.2, -0.6, 0.6, 1.2, -0.7, 0, -0.7, 0, 0.7, 0.7,
                       0.3, -0.4, 0.4, -0.3, 0, 0, 0.7, 0.7, 0, 0.3, 0.4,
                       -0.6,-1.2, -0.6, 0.3, -0.7, 0, 0, -0.7, -0.4),
                     .Dim = c(15L, 2L))
                 )
    ## plot limits
    if (parts < 4) {
        xlim <- range(cp[,1]) + c(-rad, rad)
        ylim <- range(cp[,2]) + c(-rad, rad)
    } else {
        xlim <- c(-1.7, 1.7)
        ylim <- c(-1.7, 1.1)
    }
    ## plot
    plot(cp, axes=FALSE, xlab="", ylab="", asp=1, type="n",
         xlim = xlim, ylim = ylim)
    box()
    if (parts < 4) {
        symbols(cp, circles = rep(rad, min(parts,3)), inches = FALSE,
                add=TRUE, bg = bg, ...)
        ## Explanatory data set names added by PL
        if(parts==2) {
            pos.names = matrix(c(-0.65,1.65,0.65,0.65),2,2)
        } else if(parts==3) {
            pos.names = matrix(c(-0.65,1.65,-0.16,0.65,0.65,-1.5),3,2)
        }
        text(pos.names,labels=Xnames[1:parts], cex=id.size)
    } else {
        ## Draw ellipses with veganCovEllipse. Supply 2x2
        ## matrix(c(d,a,a,d), 2, 2) which defines an ellipse of
        ## semi-major axis length sqrt(d+a) semi-minor axis sqrt(d-a).
        d <- 1
        a <- 1/sqrt(2)
        ## Small ellipses X2, X3 at the centroid
        e2 <- veganCovEllipse(matrix(c(d,-a,-a,d), 2, 2))
        e3 <- veganCovEllipse(matrix(c(d, a, a,d), 2, 2))
        ## wider ellipses X1, X4 at sides going through the centroid
        L <- d+a
        W <- (sqrt(L) - sqrt(d-a))^2
        d <- (L+W)/2
        a <- (L-W)/2
        cnt <- sqrt(W/2)
        e1 <- veganCovEllipse(matrix(c(d,-a,-a,d), 2, 2), c(-cnt, -cnt))
        e4 <- veganCovEllipse(matrix(c(d, a, a,d), 2, 2), c( cnt, -cnt))
        polygon(rbind(e1,NA,e2,NA,e3,NA,e4), col = bg, ...)
        ## Explanatory data set names added by PL
        pos.names = matrix(c(-1.62,-1.10,1.10,1.62,0.54,1.00,1.00,0.54),4,2)
        text(pos.names,labels=Xnames[1:4], cex=id.size)
    }

    ## label fractions
    nlabs <- switch(parts, 2, 4, 8, 16)
    if (missing(labels))
        labels <- paste("[", letters[1:nlabs], "]", sep="")
    if (length(labels) != nlabs)
        stop(gettextf("needs %d labels, but input has %d",
                      nlabs, length(labels)))
    switch(parts,
           text(0,0, labels[-nlabs], ...),
           text(rbind(cp[1,], colMeans(cp), cp[2,]), labels[-nlabs], ...),
           text(rbind(cp, colMeans(cp[1:2,]), colMeans(cp[2:3,]),
                      colMeans(cp[c(1,3),]), colMeans(cp)), labels[-nlabs], ...),
           text(cp, labels[-nlabs], ...)
           )
    xy <- par("usr")
    text(xy[2] - 0.05*diff(xy[1:2]), xy[3] + 0.05*diff(xy[3:4]),
         paste("Residuals =", labels[nlabs]), pos = 2, ...)
    invisible()
}

`metaMDSiter` <-
    function (dist, k = 2, try = 20, trymax = 20, trace = 1, plot = FALSE,
              previous.best, engine = "monoMDS", maxit = 200,
              parallel = getOption("mc.cores"), ...)
{
    engine <- match.arg(engine, c("monoMDS", "isoMDS"))
    EPS <- 0.05
    if (engine == "monoMDS")
        EPS <- EPS/100 # monoMDS stress (0,1), isoMDS (0,100)
    RESLIM <- 0.01
    RMSELIM <- 0.005
    SOL <- FALSE
    converged <- FALSE
    ## set tracing for engines
    isotrace <- max(0, trace - 1)
    monotrace <- engine == "monoMDS" && trace > 1
    ## explain monoMDS convergence codes (sol$icause)
    monomsg <- c("no. of iterations >= maxit",
                 "stress < smin",
                 "stress ratio > sratmax",
                 "scale factor of the gradient < sfgrmin")
    ## monoMDS trace >= 2
    monostop <- function(mod) {
        if (mod$maxits == 0)
            return(NULL)
        lab <- monomsg[mod$icause]
        cat("   ", mod$iters, "iterations: ", lab, "\n")
    }
    ## collect monoMDS convergence code for trace
    if (trace && engine == "monoMDS")
        stopcoz <- numeric(4)
    ## Previous best or initial configuration
    if (!missing(previous.best) && !is.null(previous.best)) {
        ## check if previous.best is from metaMDS or isoMDS
        if (inherits(previous.best, "metaMDS") ||
            is.list(previous.best) &&
            all(c("points", "stress") %in% names(previous.best))) {
            ## Previous best may come from another 'engine' or
            ## 'model': extract its 'points' and use as an initial
            ## configuration with 'maxit = 0' to evaluate the stress
            ## in current case, or take a matrix as configuration.
            init <- previous.best$points
            nc <- NCOL(init)
            if (nc > k)
                init <- init[, 1:k, drop = FALSE]
            else if (nc < k)
                for (i in 1:(k-nc))
                    init <- cbind(init, runif(NROW(init), -0.1, 0.1))
            if (trace)
                cat(sprintf("Starting from %d-dimensional configuration\n",
                            nc))
        } else {
            init <- as.matrix(previous.best)
        }
        ## evaluate stress
        s0 <- switch(engine,
                     "monoMDS" = monoMDS(dist, y = init, k = k, maxit = 0, ...),
                     "isoMDS" = isoMDS(dist, y = init, k = k, maxit = 0))
        ## Check whether model changed
        if (is.list(previous.best) && !is.null(previous.best$stress) &&
            !isTRUE(all.equal(previous.best$stress, s0$stress))) {
            if (trace) cat("Stress differs from 'previous.best': reset tries\n")
            if (inherits(previous.best, "metaMDS"))
                previous.best$tries <- 0
        }
    } else {
        ## no previous.best: start with cmdscale
        s0 <- switch(engine,
                 "monoMDS" = monoMDS(dist, y = cmdscale(dist, k = k), k = k,
                 maxit = maxit, ...),
                 "isoMDS" = isoMDS(dist, k = k, trace = isotrace,
                 maxit = maxit))
    }
    if (trace)
        cat("Run 0 stress", s0$stress, "\n")
    if (monotrace)
        monostop(s0)
    tries <- 0
    ## Prepare for parallel processing
    if (is.null(parallel))
        parallel <- 1
    hasClus <- inherits(parallel, "cluster")
    isParal <- hasClus || parallel > 1
    isMulticore <- .Platform$OS.type == "unix" && !hasClus
    if (isParal && !isMulticore && !hasClus) {
        parallel <- makeCluster(parallel)
        clusterEvalQ(parallel, library(vegan))
    }
    ## get the number of clusters
    if (inherits(parallel, "cluster"))
        nclus <- length(parallel)
    else
        nclus <- parallel
    ## proper iterations
    while(tries < try || tries < trymax && !converged) {
        init <- replicate(nclus, initMDS(dist, k = k))
        if (nclus > 1) isotrace <- FALSE
        if (isParal) {
            if (isMulticore) {
                stry <-
                    mclapply(1:nclus, function(i)
                             switch(engine,
                                    "monoMDS" = monoMDS(dist, init[,,i], k = k,
                                    maxit = maxit, ...),
                                    "isoMDS" = isoMDS(dist, init[,,i], k = k,
                                    maxit = maxit, tol = 1e-07,
                                    trace = isotrace)),
                             mc.cores = parallel)
            } else {
                stry <-
                    parLapply(parallel, 1:nclus, function(i)
                              switch(engine,
                                     "monoMDS" = monoMDS(dist, init[,,i], k = k,
                                     maxit = maxit, ...),
                                     "isoMDS" = isoMDS(dist, init[,,i], k = k,
                                     maxit = maxit, tol = 1e-07, trace = isotrace)))
            }
        } else {
            stry <- list(switch(engine,
                                "monoMDS" = monoMDS(dist, init[,,1], k = k,
                                maxit = maxit, ...),
                                "isoMDS" = isoMDS(dist, init[,,1], k = k,
                                maxit = maxit, tol = 1e-07, trace = isotrace)))
        }
        ## analyse results of 'nclus' tries
        for (i in 1:nclus) {
            tries <- tries + 1
            if (trace)
                cat("Run", tries, "stress", stry[[i]]$stress, "\n")
            if (trace && engine == "monoMDS")
                stopcoz[stry[[i]]$icause] <- stopcoz[stry[[i]]$icause] + 1L
            if (monotrace)
                monostop(stry[[i]])
            if ((s0$stress - stry[[i]]$stress) > -EPS) {
                pro <- procrustes(s0, stry[[i]], symmetric = TRUE)
                if (plot && k > 1)
                    plot(pro)
                if (stry[[i]]$stress < s0$stress) {
                    s0 <- stry[[i]]
                    ## New best solution has not converged unless
                    ## proved later
                    converged <- FALSE
                    if (trace)
                        cat("... New best solution\n")
                }
                summ <- summary(pro)
                if (trace)
                    cat("... Procrustes: rmse", summ$rmse, " max resid",
                        max(summ$resid), "\n")
                if (summ$rmse < RMSELIM && max(summ$resid) < RESLIM) {
                    if (trace)
                        cat("... Similar to previous best\n")
                    converged <- TRUE
                }
            }
            flush.console()
        }
    }
    if (trace) {
        if (converged)
            cat("*** Solution reached\n")
        else if (engine == "monoMDS") {
            cat(sprintf(
                "*** No convergence -- %s stopping criteria:\n",
                engine))
            for (i in seq_along(stopcoz))
                if (stopcoz[i] > 0)
                    cat(sprintf("%6d: %s\n", stopcoz[i], monomsg[i]))
        }
    }
    ## stop socket cluster
    if (isParal && !isMulticore && !hasClus)
        stopCluster(parallel)
    if (!missing(previous.best) && inherits(previous.best, "metaMDS")) {
        tries <- tries + previous.best$tries
    }
    out <- s0
    out$ndim = k
    out$data <- attr(dist, "commname")
    out$distance <- attr(dist, "method")
    out$converged <- converged
    out$tries <- tries
    out$engine <- engine
    out
}

`pcnm` <- function(dis, threshold, w, dist.ret = FALSE) {
    ## square matrix to dist
    if ((is.matrix(dis) || is.data.frame(dis)) &&
        isSymmetric(unname(as.matrix(dis))))
        dis <- as.dist(dis)
    if (!inherits(dis, "dist"))
        stop("'dis' does not appear to be distances")
    EPS <- sqrt(.Machine$double.eps)
    if (missing(threshold)) {
        threshold <- max(spantree(dis)$dist)
    }
    dis[dis > threshold] <- 4*threshold
    ## vegan:::wcmdscale is able to use weights which also means that
    ## 'k' need not be given, but all vecctors with >0 eigenvalues
    ## will be found
    mypcnm <- wcmdscale(dis, eig = TRUE, w=w)
    res <- list(vectors = mypcnm$points, values = mypcnm$eig,
                weights = mypcnm$weig)
    k <- ncol(mypcnm$points)
    res$vectors <- sweep(res$vectors, 2, sqrt(res$values[seq_len(k)]), "/")
    colnames(res$vectors) <- paste("PCNM", 1:k, sep="")
    res$threshold <- threshold
    if (dist.ret) {
        attr(dis, "method") <- paste(attr(dis, "method"), "pcnm")
        attr(dis, "threshold") <- threshold
        res$dist <- dis
    }
    class(res) <- "pcnm"
    res
}

`treedive` <-
    function(comm, tree, match.force = TRUE, verbose = TRUE)
{
    EPS <- sqrt(.Machine$double.eps)
    comm <- as.matrix(comm)
    if (!inherits(tree, c("hclust", "spantree")))
        stop("'tree' must be an 'hclust' or 'spantree' result object")
    if (inherits(tree, "hclust") && any(diff(tree$height) < -EPS))
        stop("tree with reversals cannot be handled")
    m <- as.matrix(cophenetic(tree))
    ## Check tree/comm match by names
    if (match.force || ncol(comm) != ncol(m)) {
        if (match.force && verbose)
            message("forced matching of 'tree' labels and 'comm' names")
        else if (verbose)
            message("dimensions do not match between 'comm' and 'tree'")
        fnd <- colnames(comm) %in% tree$labels
        if (!all(fnd) && verbose) {
            warning("not all names of 'comm' found in 'tree'")
            comm <- comm[, fnd]
        }
        fnd <- tree$labels %in% colnames(comm)
        if (!all(fnd))
            warning("not all names of 'tree' found in 'comm'")
        comm <- comm[, tree$labels[fnd]]
        m <- m[tree$labels[fnd], tree$labels[fnd]]
        if (length(unique(tree$labels)) != length(tree$labels))
            stop("names not unique in 'tree': match wrong")
        if (length(unique(colnames(comm))) != ncol(comm))
            stop("names not unique in 'comm': match wrong")
    }
    ## Repeat for sites
    div <- numeric(nrow(comm))
    for (i in 1:nrow(comm)) {
        k <- comm[i,] > 0
        nit <- sum(k)
        ## Trivial cases of zero or one species
        if (nit==0)
            div[i] <- NA
        else if (nit==1)
            div[i] <- 0
        else {
            d <- as.dist(m[k,k])
            cl <- update(tree, d = d)
            div[i] <- treeheight(cl)
        }
    }
    names(div) <- rownames(comm)
    div
}

`as.fisher` <-
    function (x, ...) 
{
    if (inherits(x, "fisher")) 
        return(x)
    ## is not fisher but a 1 x n data.frame or matrix: matrix is faster
    x <- as.matrix(x)
    if (!identical(all.equal(x, round(x)), TRUE))
        stop("function accepts only integers (counts)")
    freq <- x[x > 0]
    freq <- table(freq, deparse.level = 0)
    nm <- names(freq)
    freq <- as.vector(freq)
    names(freq) <- nm
    class(freq) <- "fisher"
    freq
}

`procrustes` <-
    function (X, Y, scale = TRUE, symmetric = FALSE,  scores = "sites", ...)
{
    X <- scores(X, display = scores, ...)
    Y <- scores(Y, display = scores, ...)
    if (nrow(X) != nrow(Y))
        stop(gettextf("matrices have different number of rows: %d and %d",
             nrow(X), nrow(Y)))
    if (ncol(X) < ncol(Y)) {
        warning("X has fewer axes than Y: X adjusted to comform Y\n")
        addcols <- ncol(Y) - ncol(X)
        for (i in 1:addcols) X <- cbind(X, 0)
    }
    ctrace <- function(MAT) sum(MAT^2)
    c <- 1
    if (symmetric) {
        X <- scale(X, scale = FALSE)
        Y <- scale(Y, scale = FALSE)
        X <- X/sqrt(ctrace(X))
        Y <- Y/sqrt(ctrace(Y))
    }
    xmean <- apply(X, 2, mean)
    ymean <- apply(Y, 2, mean)
    if (!symmetric) {
        X <- scale(X, scale = FALSE)
        Y <- scale(Y, scale = FALSE)
    }
    XY <- crossprod(X, Y)
    sol <- svd(XY)
    A <- sol$v %*% t(sol$u)
    if (scale) {
        c <- sum(sol$d)/ctrace(Y)
    }
    Yrot <- c * Y %*% A
    ## Translation (b) needs scale (c) although Mardia et al. do not
    ## have this. Reported by Christian Dudel.
    b <- xmean - c * ymean %*% A
    R2 <- ctrace(X) + c * c * ctrace(Y) - 2 * c * sum(sol$d)
    reslt <- list(Yrot = Yrot, X = X, ss = R2, rotation = A,
                  translation = b, scale = c, xmean = xmean,
                  symmetric = symmetric, call = match.call())
    reslt$svd <- sol
    class(reslt) <- "procrustes"
    reslt
}

print.commsim <- function(x, ...) {
    cat("An object of class", dQuote(class(x)[1L]), "\n")
    isSeq <- ifelse(x$isSeq, "sequential", "non-sequential")
    if(x$binary)
        kind <- "binary"
    else
        kind <- ifelse(x$mode == "integer", "count", "abundance")
    cat(sQuote(x$method), " method (", 
        kind, ", ", isSeq, ", ", x$mode, " mode)\n\n", sep="")
    invisible(x)
}

`fitted.cca` <-
    function (object, model = c("CCA","CA","pCCA"),
              type = c("response", "working"), ...)
{
    type <- match.arg(type)
    model <- match.arg(model)
    if (is.null(object[[model]]))
        stop(gettextf("component '%s' does not exist", model))
    Xbar <- ordiYbar(object, model)
    if (type == "response") {
        gtot <- object$grand.total
        rc <- object$rowsum %o% object$colsum
        Xbar <- (Xbar * sqrt(rc) + rc) * gtot
    }
    Xbar
}

`print.summary.cca` <-
    function (x, digits = x$digits, ...)
{
    cat("\nCall:\n")
    cat(deparse(x$call), "\n")
    statnam <- if (x$method == "cca")
        "averages"
    else "sums"
    cat("\nPartitioning of ", x$inertia, ":\n", sep = "")
    out <- c(Total = x$tot.chi, Conditioned = x$partial.chi,
             Constrained = x$constr.chi, Unconstrained = x$unconst.chi)
    out <- cbind(Inertia = out, Proportion = out/out[1])
    print(out, digits = digits, ...)
    cat("\nEigenvalues, and their contribution to the", x$inertia,
        "\n")
    if (!is.null(x$partial.chi)) {
        cat("after removing the contribution of conditiniong variables\n")
    }
    cat("\n")
    print(x$cont$importance, ...)
    if (!is.null(x$concont)) {
        cat("\nAccumulated constrained eigenvalues\n")
        print(x$concont$importance, ...)
    }
    cat("\n")
    invisible(x)
}

`plot.cca` <- function (x, choices = c(1, 2), display = c("sp", "wa", "cn"),
                        scaling = "species", type, xlim, ylim, const,
                        correlation = FALSE, hill = FALSE, ...) {
    TYPES <- c("text", "points", "none")
    g <- scores(x, choices, display, scaling, const, correlation = correlation,
                hill = hill)
    if (length(g) == 0 || all(is.na(g)))
      stop("nothing to plot: requested scores do not exist")
    if (!is.list(g))
        g <- list(default = g)
    ## Take care that there are names
    for (i in seq_along(g)) {
        if (length(dim(g[[i]])) > 1)
            rownames(g[[i]]) <- rownames(g[[i]], do.NULL = FALSE,
                                         prefix = substr(names(g)[i], 1, 3))
    }
    if (!is.null(g$centroids)) {
        if (is.null(g$biplot))
            g$biplot <- scores(x, choices, "bp", scaling)
        if (!is.na(g$centroids)[1]) {
            bipnam <- rownames(g$biplot)
            cntnam <- rownames(g$centroids)
            g$biplot <- g$biplot[!(bipnam %in% cntnam), , drop = FALSE]
            if (nrow(g$biplot) == 0)
                g$biplot <- NULL
        }
    }
    if (missing(type)) {
        nitlimit <- 80
        nit <- max(nrow(g$spe), nrow(g$sit), nrow(g$con), nrow(g$def))
        if (nit > nitlimit)
            type <- "points"
        else type <- "text"
    }
    else type <- match.arg(type, TYPES)
    ## use linestack (and exit) if only one axis was chosen, and
    ## display includes row or column scores
    if (length(choices) == 1) {
        ## Only one set of scores: plot them
        if (length(g) == 1)
            pl <- linestack(g[[1]], ...)
        ## The order of scores is species, sites, constraints, biplot,
        ## centroids: plot two first in this order, but species scores
        ## on the left
        else {
            hasSpec <- names(g)[1] == "species"
            ylim <- range(c(g[[1]], g[[2]]), na.rm = TRUE)
            pl <- linestack(g[[1]], ylim = ylim,
                            side = ifelse(hasSpec, "left", "right"), ...)
            linestack(g[[2]], ylim = ylim,
                      side = ifelse(hasSpec, "right","left"),
                      add = TRUE, ...)
            }
        return(invisible(pl))
    }
    if (missing(xlim)) {
        xlim <- range(g$species[, 1], g$sites[, 1], g$constraints[, 1],
                      g$biplot[, 1],
                      if (length(g$centroids) > 0 && all(is.na(g$centroids))) NA else g$centroids[, 1],
                      g$default[, 1],
                      na.rm = TRUE)
    }
    if (!any(is.finite(xlim)))
        stop("no finite scores to plot")
    if (missing(ylim)) {
        ylim <- range(g$species[, 2], g$sites[, 2], g$constraints[, 2],
                      g$biplot[, 2],
                      if (length(g$centroids) > 0 && all(is.na(g$centroids))) NA else g$centroids[, 2],
                      g$default[, 2],
                      na.rm = TRUE)
    }
    plot(g[[1]], xlim = xlim, ylim = ylim, type = "n", asp = 1,
         ...)
    abline(h = 0, lty = 3)
    abline(v = 0, lty = 3)
    if (!is.null(g$species)) {
        if (type == "text")
            text(g$species, rownames(g$species), col = "red",
                 cex = 0.7)
        else if (type == "points")
            points(g$species, pch = "+", col = "red", cex = 0.7)
    }
    if (!is.null(g$sites)) {
        if (type == "text")
            text(g$sites, rownames(g$sites), cex = 0.7)
        else if (type == "points")
            points(g$sites, pch = 1, cex = 0.7)
    }
    if (!is.null(g$constraints)) {
        if (type == "text")
            text(g$constraints, rownames(g$constraints), cex = 0.7,
                 col = "darkgreen")
        else if (type == "points")
            points(g$constraints, pch = 2, cex = 0.7, col = "darkgreen")
    }
    if (!is.null(g$biplot) && nrow(g$biplot) > 0 && type != "none") {
        if (length(display) > 1) {
            mul <- ordiArrowMul(g$biplot)
        }
        else mul <- 1
        attr(g$biplot, "arrow.mul") <- mul
        arrows(0, 0, mul * g$biplot[, 1], mul * g$biplot[, 2],
               length = 0.05, col = "blue")
        biplabs <- ordiArrowTextXY(mul * g$biplot, rownames(g$biplot))
        text(biplabs, rownames(g$biplot), col = "blue")
    }
    if (!is.null(g$regression) && nrow(g$regression > 0) && type != "none") {
        rcol <- "purple4"
        if (length(display) > 1) {
            mul <- ordiArrowMul(g$regression)
        }
        else mul <- 1
        attr(g$regression, "arrow.mul") <- mul
        arrows(0, 0, mul * g$regression[, 1], mul * g$regression[, 2],
               length = 0.05, col = rcol)
        biplabs <- ordiArrowTextXY(mul * g$regression, rownames(g$regression))
        text(biplabs, rownames(g$regression), col = rcol)
    }
    if (!is.null(g$centroids) && all(!is.na(g$centroids)) && type !=
        "none") {
        if (type == "text")
            text(g$centroids, rownames(g$centroids), col = "blue")
        else if (type == "points")
            points(g$centroids, pch = "x", col = "blue")
    }
    if (!is.null(g$default) && type != "none") {
        if (type == "text")
            text(g$default, rownames(g$default), cex = 0.7)
        else if (type == "points")
            points(g$default, pch = 1, cex = 0.7)
    }
    class(g) <- "ordiplot"
    invisible(g)
}

## vegan::plot.rda needed because klaR::plot.rda would be used
## instead if klaR package is loaded

`plot.rda`<-
    function(x, ...)
{
    ## not vegan rda?
    if (!("CA" %in% names(x)))
        stop(gettextf("%s is not a vegan rda object",
                      sQuote(deparse(substitute(x)))))
    NextMethod("plot", x, ...)
}

# Test data
# data(varespec)
testdata <- matrix(round(runif(1000, 0, 100)), nrow=20)
testdata <- testdata - 50
testdata[testdata < 0] <- 0
rownames(testdata) <- paste0("row", seq_len(nrow(testdata)))
colnames(testdata) <- paste0("col", seq_len(ncol(testdata)))

# Calculates relative abundance table
relative <- vegan::decostand(testdata, "total")

# Count and relative data with pseudocount
testdata.with.pseudo <- testdata + 1
relative.with.pseudo <- vegan::decostand(testdata+1, "total")

# aitchison equals to CLR + Euclid (pseudocount is necessary with clr)
a1 <- vegan::vegdist(testdata+1, method = "aitchison")
a2 <- vegan::vegdist(vegan::decostand(testdata+1, "clr"), method = "euclidean")
max(abs(a1-a2)) < 1e-6 # Tolerance

# Robust aitchison equals to rCLR + Euclid
# and works without pseudocount
a1 <- vegan::vegdist(testdata, method = "robust.aitchison")
a2 <- vegan::vegdist(vegan::decostand(testdata, "rclr"), method = "euclidean")
max(abs(a1-a2)) < 1e-6 # Tolerance

# Robust aitchison and aitchison are equal when there are no zeroes
a1 <- vegan::vegdist(testdata.with.pseudo, method = "robust.aitchison")
a2 <- vegan::vegdist(testdata.with.pseudo, method = "aitchison")
max(abs(a1-a2)) < 1e-6 # Tolerance

# It is possible to pass pseudocount as a function argument to vegan::decostand
a1 <- vegan::vegdist(testdata, method = "aitchison", pseudocount=1)
a2 <- vegan::vegdist(testdata+1, method = "aitchison")
max(abs(a1-a2)) < 1e-6 # Tolerance


# Compare the outcomes with an external package that also provides compositional transformations
# Adding these would demand adding Suggested packages in DESCRIPTION; skipped for now but can be
# useful for manual testing.
#skip <- TRUE
#if (!skip) {
#
#    sum(compositions::clr(testdata.with.pseudo) - vegan::decostand(testdata.with.pseudo, "clr")) < 1e-6
#    sum(compositions::clr(testdata.with.pseudo) - vegan::decostand(testdata.with.pseudo, "clr", MARGIN=1)) < 1e-6
#    sum(t(compositions::clr(t(testdata.with.pseudo))) - vegan::decostand(testdata.with.pseudo, "clr", MARGIN=2)) < 1e-6
#    sum(rgr::clr(testdata.with.pseudo) - vegan::decostand(testdata.with.pseudo, "clr"))<1e-6#
#
#    sum(compositions::alr(testdata.with.pseudo, ivar=1) - vegan::decostand(testdata.with.pseudo, "alr")) < 1e-6
#    sum(compositions::alr(testdata.with.pseudo, ivar=1) - vegan::decostand(testdata.with.pseudo, "alr", MARGIN=1)) < 1e-6
#    sum(t(compositions::alr(t(testdata.with.pseudo), ivar=1)) - vegan::decostand(testdata.with.pseudo, "alr", MARGIN=2)) < 1e-6
#    sum(rgr::alr(testdata.with.pseudo, j=1) - vegan::decostand(testdata.with.pseudo, "alr", reference=1))<1e-6#
#
#}

# --------------------------------------------------------------------------------------------------------------

"nestedchecker" <-
function(comm)
{
    cb <- sum(designdist(comm, "(A-J)*(B-J)", "binary"))
    sppairs <- ncol(comm)*(ncol(comm)-1)/2
    out <- list("C.score" = cb/sppairs, statistic = cb)
    names(out$statistic) <- "checkerboards"
    class(out) <- "nestedchecker"
    out
}

### support functions for wcmdscale results: print, scores and plot.

`print.wcmdscale` <-
    function(x, digits = max(3, getOption("digits") - 3), ...)
{
    writeLines(strwrap(pasteCall(x$call)))
    cat("\n")
    ## tabulate total inertia and ranks
    totev <- sum(x$eig)
    negax <- x$eig < 0
    if (any(negax)) {
        ranks <- c(NA, sum(!negax), sum(negax))
        negax <- x$eig < 0
        realev <- sum(x$eig[!negax])
        imev <- sum(x$eig[negax])
        evs <- c("Total" = totev, "Real" = realev, "Imaginary" = imev)
    } else {
        ranks <- length(x$eig)
        evs <- c("Total" = totev)
    }
    tbl <- cbind("Inertia" = evs, "Rank" = ranks)
    printCoefmat(tbl, digits = digits, na.print = "")
    if (!is.na(x$ac) && x$ac > 0)
        cat("additive constant ", x$ac, " (method ", x$add, ")\n", sep = "")
    cat("\nResults have", NROW(x$points), "points,", NCOL(x$points), "axes\n")
    ## print eigenvalues, but truncate very long lists
    PRINLIM <- 120
    neig <- length(x$eig)
    cat("\nEigenvalues:\n")
    print(zapsmall(x$eig[1 : min(neig, PRINLIM)], digits = digits, ...))
    if (neig > PRINLIM)
        cat("(Showing", PRINLIM, "of", neig, "eigenvalues)\n")
    wvar <- var(x$weights)
    wlen <- length(x$weights)
    cat("\nWeights:")
    if (wvar < 1e-6)
        cat(" Constant\n")
    else {
        cat("\n")
        print(zapsmall(x$weights[1 : min(wlen, PRINLIM)], digits = digits, ...))
        if (wlen > PRINLIM)
            cat("(Showing", PRINLIM, "of", wlen, "weights)\n")
    }
    cat("\n")
    invisible(x)
}

`scores.wcmdscale` <-
    function(x, choices = NA, ...)
{
    if (any(is.na(choices)))
        x$points
    else {
        choices <- choices[choices <= NCOL(x$points)]
        x$points[, choices, drop = FALSE]
    }
}

`plot.wcmdscale` <-
    function(x, choices = c(1,2), type = "t", ...)
{
    ordiplot(x, display = "sites", choices = choices, type = type, ...)
}

`prestonfit` <-
    function (x, tiesplit = TRUE, ...) 
{
    x <- as.preston(x, tiesplit = tiesplit)
    oct <- as.numeric(names(x))
    fit <- glm(x ~ oct + I(oct^2),
               family = if (tiesplit) quasipoisson else poisson)
    fv <- fitted(fit)
    p <- coef(fit)
    if (!is.na(p[3]) && p[3] < 0) {
        mu <- -p[2]/2/p[3]
        sd <- sqrt(-1/2/p[3])
        S0 <- exp(p[1] - p[2]^2/4/p[3])
        p <- c(mu, sd, S0)
    }
    else {
        p <- rep(NA, 3)
    }
    names(p) <- c("mode", "width", "S0")
    out <- list(freq = unclass(x), fitted = fv, coefficients = p)
    out$method = "Poisson fit to octaves"
    if(tiesplit)
        out$method <- paste("Quasi-", out$method, sep="")
    class(out) <- c("prestonfit")
    out
}

C *************************************************************
C Based on my standard cepin module. This had to be vandalized,
C since R cannot interface with character arrays: It can handle only
C a single character*255 variable. So the routine had to be broken
C into pieces, and assume that the file opened  is still there when 
C called repeatedly. 
C
C I feel really bad for this.
C
C Since the subroutine had to be split, I split it completely:
C Each file type is read in a separate subroutine, so that I can
C get rid off static work arrays.
C
C Jari Oksanen, Sep 16, 2001
C
C *************************************************************

C
C Read the file header.
C The file is left open (that's magic...), and everything
C needed is the format, the `kind' of the file, the number
C of items in a record (`nitem'), and number of records for 
C `kind=1'. The `kind' is interpreted from number of I's in `fmt'. 
C

      subroutine cephead(cepfile, kind, nitem, nst, fmt)

      character*255 cepfile, fmt
      integer kind, nitem, nst

      character*80 title
      integer ni, nbrac

      ni=0
      nbrac=0

      open (unit=1,file=cepfile,status='old')
      read (1,101) title
 101  format (a80)
      read (1,101) fmt
      do i=1,80
         if (fmt(i:i) .eq. 'I' .or. fmt(i:i) .eq. 'i') ni=ni+1
      end do
      kind = ni+1
      if (kind .eq. 1) then
         read(1, *) nitem, nst
         return
      endif
      nbrac=0
      do i=1,80
         if (fmt(i:i) .eq. '(' ) nbrac=nbrac+1
         if (fmt(i:i) .eq. ')' ) then
            nbrac=nbrac-1
            if (nbrac .le. 0) goto 6
         end if
         goto 8
 6       ii=i+2
         if (ii .le. 69) read(fmt,'(t69,i2)') nitem
         if (nitem .eq. 0) then
            read (1,*) nitem
         else
            fmt(ii:80)=' '
         endif
 8       continue
      enddo
      return
      end
      
c
c Open CEP format (kind=2).
c Zeros are skipped, negative entries stored
c Stop when negative site (row) index found.
c
      subroutine cepopen(fmt, nitem, maxdat, nsp, nst, idplot, idspec, 
     X abund, work, ier)

      character*255 fmt
      integer nitem, nsp, nst
      integer idplot(maxdat), idspec(maxdat)
      double precision abund(maxdat)
 
      integer id, ii, j, ier
      double precision work(nitem)

      nsp = nitem
      nst = 0
      id = 0
      ier = 99
      
 20   read (1, fmt) ii, (work(j),j=1,nitem)  
      if (ii .le. 0) then
         ier = 0
         return
      endif
      if (ii .gt. nst) then
         nst=ii
      end if
      do j=1,nitem
         if (work(j) .ne. 0) then
            id = id+1
            if (id .gt. maxdat) then
               ier = 1
               return
            endif
            idplot(id) = ii
            idspec(id) = j
            abund(id) = work(j)
         endif
      end do
      goto 20
      
      end

C
C Condensed CEP format (kind=3)
C ALL entries are stored in condensed format (except zeros)
C

      subroutine cepcond(fmt, nitem, maxdat, nsp, nst, idplot, idspec, 
     X abund, work, item, ier)

      character*255 fmt
      integer nitem, nsp, nst
      integer idplot(maxdat), idspec(maxdat)
      double precision abund(maxdat)
 
      integer id, ii, j, ier
      integer item(nitem)
      double precision work(nitem)

      nsp = 0
      nst = 0
      id = 0
      ier = 99

 40   read (1,fmt) ii,(item(j),work(j),j=1,nitem) 
      if (ii .le. 0) then
         ier = 0
         return
      endif
      if (ii .gt. nst) then
         nst = ii
      endif
      do j = 1,nitem
         if (item(j) .gt. 0 .and. work(j) .ne. 0.0) then
            id = id+1
            if (id .gt. maxdat) then
               ier = 1
               return
            endif
            idplot(id) = ii
            if (item(j) .gt. nsp) then
               nsp = item(j)
            endif
            idspec(id) = item(j)
            abund(id) = work(j)
         endif
      enddo
      goto 40

      end

c      
c Free CEP format (kind=1) -- or error!
c Get everything but zeros.
c

      subroutine cepfree(nitem, maxdat, nsp, nst, idplot, idspec, 
     X abund, work, ier)

      integer nitem, nsp, nst, maxdat
      integer idplot(maxdat), idspec(maxdat)
      double precision abund(maxdat)
 
      integer id, j, i, ier
      double precision work(nitem)

      nsp = nitem
      nst = nst
      id = 0
      ier = 99

      do i=1,nst
         read(1,*) (work(j),j=1,nsp)
         do j=1,nsp
            if(work(j)  .ne. 0) then
               id=id+1
               if (id .gt. maxdat) then
                  ier = 1
                  return
               endif
               abund(id) = work(j)
               idspec(id) = j
               idplot(id) = i
            endif
         enddo
      enddo
      
      ier = 0

      return
      end

c
c     R can handle only a single character*255: 
c     Names cannot be read in one time, but they must be
c     handled line by line.
c

      subroutine cepnames(entname)
      character*255 entname
      read (1,1100, end=666) entname  
 1100 format (a80)
 666  continue
      return
      end

c
c Stupid, but needed
c

      subroutine cepclose()
      close(1)
      return
      end

print.summary.clamtest <- function(x, digits=max(3, getOption("digits") - 3), ...) {
    cat("Two Groups Species Classification Method (CLAM)\n\n")
    cat("Specialization threshold =", x$specialization)
    cat("\nAlpha level =", x$alpha)
    cat("\n\nEstimated sample coverage:\n")
    print(x$coverage, digits=digits)
    cat("\nMinimum abundance for classification:\n")
    print(structure(c(x$minv[[1]][1,2], x$minv[[2]][1,1]),
        .Names=x$labels))
    cat("\n")
    printCoefmat(x$summary, digits=digits, ...)
}

`scores.ordiplot`  <-
    function (x, display = "sites", ...)
{
    if (length(x) == 1)
        return(x[[1]])
    items <- names(x)
    items <- items[!is.na(items)]
    display <- match.arg(display, items)
    x <- x[[display]]
    attr(x, "score") <- display
    x
}

### plot() methods for those nested* functions that return sufficient
### data to draw a plot. We display a plot using image() with coloured
### rectangles for occurrences or colours dependent on the
### contribution to the nestedness when appropriate.

`plot.nestedtemp` <-
    function (x, kind = c("temperature", "incidence"),
              col = rev(heat.colors(100)), names = FALSE,  
              ...) 
{
    kind <- match.arg(kind)
    if (kind == "temperature") 
        z <- x$u
    else z <- x$comm
    z <- t(z[nrow(z):1, ])
    image(z, axes = FALSE, col = col, ...)
    box()
    lines(x$smooth$x, 1 - x$smooth$y)
    if (length(names) == 1)
        names <- rep(names, 2)
    if (names[1]) {
        axis(2, at = seq(1, 0, len = nrow(x$u)), labels = rownames(x$u),
            las = 2, ...)
    }
    if (names[2]) {
        axis(3, at = seq(0, 1, len = ncol(x$u)), labels = colnames(x$u),
            las = 2, ...)
    }
}

`plot.nestednodf` <-
    function(x, col = "red", names = FALSE, ...)
{
    z <- x$comm
    z <- t(z[nrow(z):1,])
    if (length(col) == 1)
        col <- c(NA, col)
    image(z, axes = FALSE, col = col, ...)
    box()
    if (length(names) == 1)
        names <- rep(names, 2)
    if (names[1]) {
        axis(2, at = seq(1, 0, len = ncol(z)), labels = rev(colnames(z)),
            las = 2, ...)
    }
    if (names[2]) {
        axis(3, at = seq(0, 1, len = nrow(z)), labels = rownames(z),
            las = 2, ...)
    }
}

`print.bioenv` <-
    function (x, ...) 
{
    cat("\nCall:\n")
    cat(deparse(x$call), "\n")
    cat("\nSubset of environmental variables with best correlation to community data.\n\n")
    cat("Correlations:   ", x$method, "\n")
    cat("Dissimilarities:", x$index, "\n")
    cat("Metric:         ", x$metric, "\n\n") 
    i <- which.max(lapply(x$models, function(tmp) tmp$est))
    cat("Best model has", i, "parameters (max.", x$upto, "allowed):\n")
    cat(paste(x$names[x$models[[i]]$best], collapse = " "))
    cat("\nwith correlation ", x$models[[i]]$est, "\n\n")
    invisible(x)
}

`isomap` <-
function(dist, ndim=10, ...)
{
    dist <- isomapdist(dist, ...)
    out <- cmdscale(dist, k=ndim, eig=TRUE)
    ## some versions of cmdscale may return NaN points corresponding
    ## to negative eigenvalues.
    if ((naxes <- sum(out$eig > 0)) < ndim) {
        out$points <- out$points[, seq(naxes), drop = FALSE]
        message(gettextf("isomap returns only %d axes with positive eigenvalues",
                         naxes))
    }
    npoints <- nrow(out$points)
    net <- matrix(FALSE, nrow=npoints, ncol=npoints)
    net[lower.tri(net)][attr(dist, "net")] <- TRUE
    net <- which(net, arr.ind=TRUE)
    out$method <- attr(dist, "method")
    out$criterion <- attr(dist, "criterion")
    out$critval <- attr(dist, "critval")
    out$take <- attr(dist, "take")
    out$net <- net
    out$npoints <- npoints
    out$call <- match.call()
    class(out) <- "isomap"
    out
}

`RsquareAdj` <-
    function(x, ...)
{
    UseMethod("RsquareAdj")
}


`RsquareAdj.default` <-
    function(x, n, m, ...)
{
    r2 <- 1 - (1-x)*(n-1)/(n-m-1)
    if (any(na <- m >= n-1))
        r2[na] <- NA
    r2
}

## Use this with rda() results
`RsquareAdj.rda` <-
    function(x, ...)
{
    R2 <- x$CCA$tot.chi/x$tot.chi
    m <- x$CCA$qrank
    n <- nrow(x$CCA$u)
    if (is.null(x$pCCA)) {
        radj <- RsquareAdj(R2, n, m)
    } else {
        ## Partial model: same adjusted R2 as for component [a] in two
        ## source varpart model
        R2p <- x$pCCA$tot.chi/x$tot.chi
        p <- x$pCCA$QR$rank
        radj <- RsquareAdj(R2 + R2p, n, m + p) - RsquareAdj(R2p, n, p)
    }
    list(r.squared = R2, adj.r.squared = radj)
}

## cca result: RsquareAdj is calculated similarly as in
## varpart(). This is similar "semipartial" model as for rda() and
## found as a difference of R2-adj values of combined model with
## constraints + conditions and only conditions.
`RsquareAdj.cca` <-
    function (x, permutations = 1000, ...)
{
    r2 <- x$CCA$tot.chi / x$tot.chi
    if (is.null(x$pCCA)) {
        p <- permutest(x, permutations, ...)
        radj <- 1 - ((1 - r2) / (1 - mean(p$num / x$tot.chi)))
    } else {
        p <- getPermuteMatrix(permutations, nobs(x))
        Y <- ordiYbar(x, "initial")
        r2tot <- (x$pCCA$tot.chi + x$CCA$tot.chi) / x$tot.chi
        r2null <- mean(sapply(seq_len(nrow(p)), function(i)
            sum(qr.fitted(x$CCA$QR, Y[p[i,],])^2)))
        r2tot <- 1 - ((1-r2tot)/(1-r2null/x$tot.chi))
        r2p <- x$pCCA$tot.chi / x$tot.chi
        r2null <- mean(sapply(seq_len(nrow(p)), function(i)
            sum(qr.fitted(x$pCCA$QR, Y[p[i,],])^2)))
        r2p <- 1 - ((1-r2p)/(1-r2null/x$tot.chi))
        radj <- r2tot - r2p
    }
    list(r.squared = r2, adj.r.squared = radj)
}

## Linear model: take the result from the summary
RsquareAdj.lm <-
  function(x, ...)
{
    summary(x)[c("r.squared", "adj.r.squared")]
}

## Generalized linear model: R2-adj only with Gaussian model
RsquareAdj.glm <-
    function(x, ...)
{
    if (family(x)$family == "gaussian")
        summary.lm(x)[c("r.squared", "adj.r.squared")]
    else
        NA
}

`intersetcor` <-
    function(object)
{
    if (!inherits(object, "cca"))
        stop("can be used only with objects inheriting from 'cca'")
    if (is.null(object$CCA) || !object$CCA$rank)
        stop("no constrained ordination or rank of constraints is zero")
    wa <- object$CCA$wa
    X <- qr.X(object$CCA$QR)
    ## remove conditions (partial terms)
    if (!is.null(object$pCCA)) {
        X <- X[, -seq_along(object$pCCA$envcentre), drop = FALSE]
        X <- qr.resid(object$pCCA$QR, X)
    }
    if (inherits(object, "rda"))
        cor(X, wa)
    else { # cca: weighted analysis, terms already weighted-centred
        wa <- sqrt(object$rowsum) * wa
        cov <- crossprod(X, wa)
        isd <- outer(1/sqrt(colSums(X^2)), 1/sqrt(colSums(wa^2)))
        cov * isd
    }
}

`plot.metaMDS`  <-
    function (x,  display = c("sites","species"), choices = c(1, 2), type = "p",
              shrink = FALSE, ...)
{
    x <- scores(x, display = display, choices = choices, shrink = shrink)
    ordiplot(x, display = display, type = type, ...)
}

`plot.cascadeKM` <-
    function (x, min.g, max.g, grpmts.plot = TRUE, sortg = FALSE, 
              gridcol = NA, ...) 
{
    wrapres <- x
    number <- (as.numeric(gsub(" groups", "", colnames(wrapres$results))))
    if (missing(min.g)) 
        min.g <- min(number)
    if (missing(max.g)) 
        max.g <- max(number)
    if (min.g < 2) 
        min.g = 2
    c.min <- which(number == min.g)
    c.max <- which(number == max.g)
    if (length(c.min) == 0) {
        stop("min.g value given has not been calculated by 'cascadeKM'\n")
    }
    if (length(c.max) == 0) {
        stop("max.g value given has not been calculated by 'cascadeKM'\n")
    }
    x <- wrapres$partition[, c.min:c.max]
    w <- wrapres$results[2, c.min:c.max]
    criterion <- wrapres$criterion
    x <- pregraphKM(x)
    if (sortg) {
        x <- orderingKM(x)
    }
    main = (paste("K-means partitions comparison"))
    xlab = ("Number of groups in each partition")
    ylab = ("Objects")
    nc = ncol(x)
    colo <- (rainbow(max.g + 1))
    if (grpmts.plot) {
        def.par <- par(no.readonly = TRUE)
        nf <- layout(matrix(c(1, 2), 1, 2), widths = c(3, 1), 
                     TRUE)
        par(mar = c(5, 5, 5, 1), bg = "white", col = "black")
        image(1:nrow(x), 1:nc, x, col = colo, yaxt = "n", frame.plot = TRUE, 
              main = main, xlab = ylab, ylab = xlab, bg = NA)
        grid(nx = nrow(x), ny = max.g - min.g + 1, col = gridcol)
        box()
        axis(2, seq(min.g - min.g + 1, max.g - min.g + 1, by = 1), 
             labels = seq(min.g, max.g, by = 1))
        axis(1)
        par(mar = c(5, 2, 5, 1))
        par(bg = "white", fg = "black", col = "black")
        plot(y = min.g:max.g, x = w[1:nc], type = "b", main = paste(criterion, 
                                                       "\ncriterion", sep = ""), ylab = "", ylim = c(min.g - 
                                                                                            0.5, max.g + 0.5), yaxs = "i", yaxt = "n", xlab = "Values")
        grid(nx = NULL, ny = max.g - min.g + 1, col = gridcol)
        box()
        axis(2, seq(min.g, max.g, by = 1), labels = seq(min.g, max.g, 
                                           by = 1), col.axis = "black")
        axis(1)
        maxx = which.max(w[])
        minx = which.min(w[])
        tops <- which(w[c(2:nc)] - w[c(1:(nc - 1))] > 0) + 1
        maxx.o <- NA
        if (length(tops) != 0) {
            if (length(which(tops > maxx)) != 0) 
                maxx.o <- tops[which(tops > maxx)]
        }
        tops <- which(w[c(2:nc)] - w[c(1:(nc - 1))] < 0) + 1
        minx.o <- NA
        if (length(tops) != 0) {
            if (length(which(tops > minx)) != 0) 
                minx.o <- tops[which(tops > minx)]
        }
        if (tolower(criterion) == "calinski") {
            if (!is.na(maxx.o[1])) 
                points(y = maxx.o + min.g - 1, x = w[maxx.o], 
                       col = "orange", pch = 19)
            points(y = maxx + min.g - 1, x = w[maxx], col = "red", 
                   pch = 19)
        }
        else if (tolower(criterion) == "likelihood") {
            if (!is.na(maxx.o[1])) {
                points(y = maxx.o + min.g - 1, x = w[maxx.o], 
                       col = "orange", pch = 19)
            }
            points(y = maxx + min.g - 1, x = w[maxx], col = "red", 
                   pch = 19)
        }
        else if (tolower(criterion) == "ssi") {
            if (!is.na(maxx.o[1])) {
                points(y = maxx.o + min.g - 1, x = w[maxx.o], 
                       col = "orange", pch = 19)
            }
            points(y = maxx + min.g - 1, x = w[maxx], col = "red", 
                   pch = 19)
        }
        else {
            cat("When using the", criterion, "criterion, no red marker is", 
                "used to indicate the best value.\n")
        }
        par(def.par)
    }
    else {
        tops <- which(w[c(2:nc)] - w[c(1:(nc - 1))] > 0) + 1
        if (length(tops) != 0) {
            maxx <- which.max(w[c(2:nc)] - w[c(1:nc - 1)]) + 
                1
        }
        else {
            maxx <- which.max(w[])
            tops = 1
        }
    }
    out <- list(x = x, best.grps = maxx)
    if (grpmts.plot)
        invisible(out)
    else
        out
}

## internal function for checking select arguments in ordination plotting
## functions
.checkSelect <- function(select, scores) {
    ## check `select` and length of scores match
    if(is.logical(select) &&
                 !isTRUE(all.equal(length(select), NROW(scores)))) {
        warning("length of 'select' does not match the number of scores: ignoring 'select'")
    } else {
        scores <- if(is.matrix(scores)) {
            scores[select, , drop = FALSE]
        } else {
            scores[select]
        }
    }
    scores
}

### SIMPER: contributions of species on overall dissimilarity with
### emphasis on among-group dissimilarities. Generate full
### dissimilarity matrix first and then subsample.

`simper` <-
    function(comm, group, permutations = 999, parallel = 1, ...)
{
    ## parallel processing not yet implemented
    if (!missing(parallel))
        .NotYetUsed("parallel", error = FALSE)
    EPS <- sqrt(.Machine$double.eps)
    comm <- as.matrix(comm)
    ## take lower triangle without as.dist overhead
    tri <- outer(seq_len(nrow(comm)), seq_len(nrow(comm)), ">")
    ## Species contributions of differences needed for every species,
    ## but denominator is constant. Bray-Curtis is actually
    ## manhattan/(mean(rowsums)) and this is the way we collect data
    rs <- rowSums(comm)
    rs <- outer(rs, rs, "+")[tri]
    spcontr <- sapply(seq_len(ncol(comm)),
        function(i) as.vector(vegdist(comm[, i, drop = FALSE], "man")))
    ## Bray-Curtis
    spcontr <- sweep(spcontr, 1, rs, "/")
    colnames(spcontr) <- colnames(comm)
    outlist <- NULL
    ## Averages of species contributions
    ## Case 1: overall differences without grouping
    if (missing(group) || length(unique(group)) == 1) {
        nperm <- 0
        permat <- NULL
        average <- colMeans(spcontr)
        overall <- sum(average)
        sdi <- apply(spcontr, 2, sd)
        ord <- order(average, decreasing = TRUE)
        cusum <- cumsum(average[ord])/overall
        outlist[["total"]] <- list(species = colnames(comm),
                                   average = average, overall = overall,
                                   sd = sdi, ratio = average/sdi,
                                   ava = NULL, avb = NULL, ord = ord,
                                   cusum = cusum, p = NULL)
    } else {
        ## Case 2: two or more groups
        comp <- t(combn(as.character(unique(group)), 2))
        ## data averages by group (do we need these?)
        spavg <- apply(comm, 2, function(x) tapply(x, group, mean))
        ## function to match constrasts
        contrmatch <- function(X, Y, patt)
            X != Y & X %in% patt & Y %in% patt
        for (i in seq_len(nrow(comp))) {
            tmat <- outer(group, group, FUN=contrmatch, patt=comp[i,])
            take <- tmat[tri]
            average <- colMeans(spcontr[take,,drop=FALSE])
            overall <- sum(average)
            sdi <- apply(spcontr[take,,drop=FALSE], 2, sd)
            ratio <- average/sdi
            ord <- order(average, decreasing = TRUE)
            cusum <- cumsum(average[ord])/overall
            ava <- spavg[comp[i,1],]
            avb <- spavg[comp[i,2],]
            ## Permutation tests for average
            permat <- getPermuteMatrix(permutations, nrow(comm))
            nperm <- nrow(permat)
            if (nperm) {
                Pval <- rep(1, ncol(comm))
                for (k in seq_len(nperm)) {
                    take <- tmat[permat[k,],permat[k,]][tri]
                    Pval <- Pval + ((colMeans(spcontr[take,,drop = FALSE]) - EPS) >= average)
                }
                Pval <- Pval/(nperm+1)
                if (anyNA(ratio))
                    Pval[is.na(ratio)] <- NA
            } else {
                Pval <- NULL
            }
            ## output
            outlist[[paste(comp[i,], collapse="_")]] <-
                list(species = colnames(comm), average = average,
                     overall = overall, sd = sdi, ratio = ratio, ava = ava,
                     avb = avb, ord = ord, cusum = cusum, p = Pval)
        }
    }
    class(outlist) <- "simper"
    attr(outlist, "permutations") <- nperm
    attr(outlist, "control") <- attr(permat, "control")
    outlist
}
`print.simper` <-
    function(x, ...)
{
    cat("cumulative contributions of most influential species:\n\n")
    cusum <- lapply(x, function(z) z$cusum)
    spec <- lapply(x, function(z) z$species[z$ord])
    for (i in seq_along(cusum)) {
        names(cusum[[i]]) <- spec[[i]]
    }
    ## this probably fails with empty or identical groups that have 0/0 = NaN
    out <- lapply(cusum, function(z) z[seq_len(min(which(z >= 0.7)))])
    print(out)
    invisible(x)
}

`summary.simper` <-
    function(object, ordered = TRUE, digits = max(3, getOption("digits") - 3), ...)
{
    if (ordered) {
        out <- lapply(object, function(z)
            data.frame(cbind(average = z$average, sd = z$sd, ratio = z$ratio,
                       ava = z$ava, avb = z$avb)[z$ord, ]))
        cusum <- lapply(object, function(z) z$cusum)
        for(i in seq_along(out)) {
            out[[i]]$cumsum <- cusum[[i]]
            if(!is.null(object[[i]]$p)) {
                out[[i]]$p <- object[[i]]$p[object[[i]]$ord]
            }
        }
    }
    else {
        out <- lapply(object, function(z)
            data.frame(cbind(average = z$average, sd = z$sd, ratio = z$ratio,
                             ava = z$ava, avb = z$avb, p = z$p)))
    }
    attr(out, "digits") <- digits
    attr(out, "permutations") <- attr(object, "permutations")
    attr(out, "control") <- attr(object, "control")
    class(out) <- "summary.simper"
    out
}

`print.summary.simper`<-
    function(x, digits = attr(x, "digits"), ...)
{
    for (nm in names(x)) {
        cat("\nContrast:", nm, "\n\n")
        printCoefmat(x[[nm]], digits = digits, has.Pvalue = TRUE,
                     zap.ind = seq_len(ncol(x[[nm]])), ...)
    }
    if (!is.null(attr(x, "control")))
        cat(howHead(attr(x, "control")))
    invisible(x)
}

`head.summary.cca` <-
    function(x, n=6, tail = 0, ...) {
        print(x, head=n, tail=tail, ...)
    }

`tail.summary.cca` <-
    function(x, n=6, head = 0, ...) {
        print(x, head=head, tail=n, ...)
    }

"summary.humpfit" <-
    function (object, ...) 
{
    dispersion <- if (any(object$family$family == c("binomial", 
                          "poisson"))) 
        1
    else sum(object$residuals^2)/object$df.residual
    p <- coef(object)
    se <- sqrt(dispersion * diag(solve(object$nlm$hessian)))
    est <- cbind(p, se)
    colnames(est) <- c("Estimate", "Std. Error")
    covmat <- solve(object$nlm$hessian)
    dg <- sqrt(diag(covmat))
    cormat <- covmat/outer(dg, dg)
    colnames(cormat) <- names(p)
    rownames(cormat) <- names(p)
    aic <- AIC(object)
    bic <- AIC(object, k = log(length(object$y)))
    out <- list(est = est, aic = aic, bic = bic, family = family(object)$family, 
                deviance = deviance(object), df.residual = df.residual(object), 
                dispersion = dispersion, correlation = cormat,
                cov.unscaled = covmat,  iter = object$nlm$iterations,
                code = object$nlm$code)
    class(out) <- "summary.humpfit"
    out
}

`points.orditkplot` <- function(x, pch = x$args$pch, cex = x$args$pcex,
                                col = x$args$pcol, bg = x$args$pbg, ...) {
    points(x$points, pch = pch, cex = cex, col = col, bg = bg, ...)
}

`as.ts.permat` <-
    function(x, type = "bray", ...)
{
    type <- match.arg(type, c("bray", "chisq"))
    out <- summary(x)[[type]]
    if (!is.ts(out)) {
        seqmethods <- sapply(make.commsim(), function(z) make.commsim(z)$isSeq)
        seqmethods <- names(seqmethods)[seqmethods]
        ## seqmethods <- c("swap", "tswap", "abuswap")
        stop(gettextf("as.ts available only for sequential methods %s",
                      paste(seqmethods, collapse=", ")))
    }
    out
}

`print.nestednodf` <-
    function(x, ...)
{    
    cat("N columns  :", format(x$statistic["N.columns"], ...), "\n")
    cat("N rows     :", format(x$statistic["N.rows"], ...), "\n")
    cat("NODF       :", format(x$statistic["NODF"], ...), "\n")
    cat("Matrix fill:", format(x$fill, ...), "\n")
    invisible(x)
}

### Clarke's dispweight is based on the hypothesis that count data
### should follow Poisson distribution, and species overdispersed to
### the Poisson should be downweighted. The basic model assesses the
### expected values of species and their overdispersion wrt to class
### means for a single factor and then estimates the significance of
### the overdispersion using individual-based simulation within these
### same classes. Function gdispweight generalizes this by allowing a
### formula that specifies any fitted model, but estimates the
### significance of the overdispersion analytically from Pearson
### residuals.

`gdispweight` <-
    function(formula, data, plimit = 0.05)
{
    ## We do not handle missing values (yet?)
    op <- options(na.action = "na.fail")
    on.exit(op)
    ## extract response data
    comm <- eval(formula[[2]])
    ## extract rhs
    if (missing(data))
        data <- environment(formula)
    x <- model.matrix(delete.response(terms(formula, data = data)),
                      data = data)
    ## Quasi-Poisson
    family <- quasipoisson()
    V <- family$variance
    ## fit models to all species separately and extract results
    comm <- as.data.frame(comm)
    if (any(!sapply(comm, is.numeric)))
        stop("input data must be numeric")
    mods <- lapply(comm, function(y) glm.fit(x, y, family = family))
    y <- sapply(mods, '[[', "y")
    mu <- sapply(mods, fitted)
    wts <- sapply(mods, '[[',  "prior.weights")
    res <- (y-mu) * sqrt(wts) / sqrt(V(mu))
    df <- sapply(mods, df.residual)
    ## the same stats as in Clarke's original, but parametrically
    stat <- colSums(res^2)
    p <- pchisq(stat, df, lower.tail = FALSE)
    dhat <- stat/df
    w <- ifelse(p < plimit, 1/dhat, 1)
    ## do not upweight underdispersed species
    w <- ifelse(w > 1, 1, w)
    ## done
    comm <- sweep(comm, 2, w, "*")
    class(comm) <- c("dispweight", class(comm))
    attr(comm, "D") <- dhat
    attr(comm, "df") <- df
    attr(comm, "p") <- p
    attr(comm, "weights") <- w
    attr(comm, "nsimul") <- NA
    attr(comm, "nullmodel") <- NA
    comm
}

`multipart.default` <-
    function(y, x, index=c("renyi", "tsallis"), scales = 1,
             global = FALSE, relative = FALSE, nsimul=99,
             method = "r2dtable", ...)
{
    if (length(scales) > 1)
        stop("length of 'scales' must be one")
    ## evaluate formula
    lhs <- as.matrix(y)
    if (missing(x))
        x <- cbind(level_1=seq_len(nrow(lhs)),
            leve_2=rep(1, nrow(lhs)))
    rhs <- data.frame(x)
    rhs[] <- lapply(rhs, as.factor)
    rhs[] <- lapply(rhs, droplevels, exclude = NA)
    nlevs <- ncol(rhs)
    if (nlevs < 2)
        stop("provide at least two-level hierarchy")
    if (any(rowSums(lhs) == 0))
        stop("data matrix contains empty rows")
    if (any(lhs < 0))
        stop("data matrix contains negative entries")
    if (is.null(colnames(rhs)))
        colnames(rhs) <- paste("level", seq_len(nlevs), sep="_")
    tlab <- colnames(rhs)

     ## check proper design of the model frame
    l1 <- sapply(rhs, function(z) length(unique(z)))
    if (!any(sapply(2:nlevs, function(z) l1[z] <= l1[z-1])))
        stop("number of levels are inappropriate, check sequence")
    rval <- list()
    rval[[1]] <- rhs[,nlevs]
    nCol <- nlevs - 1
    for (i in 2:nlevs) {
        rval[[i]] <- interaction(rhs[,nCol], rval[[(i-1)]], drop=TRUE)
        nCol <- nCol - 1
    }
    rval <- as.data.frame(rval[rev(seq_along(rval))])
    l2 <- sapply(rval, function(z) length(unique(z)))
    if (any(l1 != l2))
        stop("levels are not perfectly nested")

    ## aggregate response matrix
    fullgamma <-if (nlevels(rhs[,nlevs]) == 1)
        TRUE else FALSE
#    if (!fullgamma && !global)
#        warning("gamma diversity value might be meaningless")
    ftmp <- vector("list", nlevs)
    for (i in seq_len(nlevs)) {
        ftmp[[i]] <- as.formula(paste("~", tlab[i], "- 1"))
    }

    ## is there burnin/thin in ... ?
    burnin <- if (is.null(list(...)$burnin))
        0 else list(...)$burnin
    thin <- if (is.null(list(...)$thin))
        1 else list(...)$thin

    ## evaluate other arguments
    index <- match.arg(index)
    divfun <- switch(index,
        "renyi" = function(x) renyi(x, scales=scales, hill = TRUE),
        "tsallis" = function(x) tsallis(x, scales=scales, hill = TRUE))

    ## cluster membership determination
    nrhs <- rhs
    nrhs <- sapply(nrhs, as.numeric)
    idcl <- function(i) {
        h <- nrhs[,i]
        l <- nrhs[,(i-1)]
        sapply(unique(l), function(i) h[l==i][1])
    }
    id <- lapply(2:nlevs, idcl)

    ## this is the function passed to oecosimu
    if (global) {
        wdivfun <- function(x) {
            if (fullgamma) {
                tmp <- lapply(seq_len(nlevs - 1),
                              function(i) t(model.matrix(ftmp[[i]], rhs)) %*% x)
                tmp[[nlevs]] <- matrix(colSums(x), nrow = 1, ncol = ncol(x))
            } else {
                tmp <- lapply(seq_len(nlevs),
                              function(i) t(model.matrix(ftmp[[i]], rhs)) %*% x)
            }
            raw <- sapply(seq_len(nlevs), function(i) divfun(tmp[[i]]))
            a <- sapply(raw, mean)
            G <- a[nlevs]
            b <- sapply(seq_len(nlevs - 1), function(i) G / a[i])
            if (relative)
                b <- b / sapply(raw[seq_len(nlevs - 1)], length)
            c(a, b)
        }
    } else {
        wdivfun <- function(x) {
            if (fullgamma) {
                tmp <- lapply(seq_len(nlevs-1), function(i) t(model.matrix(ftmp[[i]], rhs)) %*% x)
                tmp[[nlevs]] <- matrix(colSums(x), nrow = 1, ncol = ncol(x))
            } else {
                tmp <- lapply(seq_len(nlevs), function(i) t(model.matrix(ftmp[[i]], rhs)) %*% x)
            }
            a <- sapply(seq_len(nlevs), function(i) divfun(tmp[[i]]))
            am <- lapply(seq_len(nlevs - 1), function(i) {
                    sapply(seq_along(unique(id[[i]])), function(ii) {
                        mean(a[[i]][id[[i]]==ii])
                    })
                })
            b <- lapply(seq_len(nlevs - 1), function(i) a[[(i+1)]] / am[[i]])
            bmax <- lapply(id, function(i) table(i))
            if (relative)
                b <- lapply(seq_len(nlevs - 1), function(i) b[[i]] / bmax[[i]])
            c(sapply(a, mean), sapply(b, mean))
        }
    }
    if (nsimul > 0) {
            sim <- oecosimu(lhs, wdivfun, method = method, nsimul=nsimul,
                burnin=burnin, thin=thin)
        } else {
            sim <- wdivfun(lhs)
            tmp <- rep(NA, length(sim))
            sim <- list(statistic = sim,
                oecosimu = list(z = tmp, pval = tmp, method = NA, statistic = sim))
        }
    nam <- c(paste("alpha", seq_len(nlevs - 1), sep="."), "gamma",
        paste("beta", seq_len(nlevs - 1), sep="."))
    names(sim$statistic) <- attr(sim$oecosimu$statistic, "names") <- nam
    call <- match.call()
    call[[1]] <- as.name("multipart")
    attr(sim, "call") <- call
    attr(sim$oecosimu$simulated, "index") <- index
    attr(sim$oecosimu$simulated, "scales") <- scales
    attr(sim$oecosimu$simulated, "global") <- global
    attr(sim, "n.levels") <- nlevs
    attr(sim, "terms") <- tlab
    attr(sim, "model") <- rhs
    class(sim) <- c("multipart", class(sim))
    sim
}

`predict.cca` <-
    function (object, newdata, type = c("response", "wa", "sp", "lc", "working"),
              rank = "full", model = c("CCA", "CA"), scaling = "none",
              hill = FALSE, ...)
{
    type <- match.arg(type)
    model <- match.arg(model)
    if (model == "CCA" && is.null(object$CCA))
        model <- "CA"
    take <- object[[model]]$rank
    if (take == 0)
        stop(gettextf("model '%s' has rank 0", model))
    if (rank != "full")
        take <- min(take, rank)
    rs <- object$rowsum
    cs <- object$colsum
    gtot <- object$grand.tot
    u <- object[[model]]$u[, 1:take, drop = FALSE]
    v <- object[[model]]$v[, 1:take, drop = FALSE]
    w <- object[[model]]$wa[, 1:take, drop = FALSE]
    if (is.null(w))
        w <- u
    slam <- diag(sqrt(object[[model]]$eig[1:take]), nrow = take)
    ## process scaling arg, scaling used later so needs to be a numeric
    scaling <- scalingType(scaling = scaling, hill = hill)
    if (type %in%  c("response", "working")) {
        Xbar <- 0
        if (!missing(newdata)) {
            if (NROW(u) == NROW(newdata))
                u <- predict(object, type = if(model == "CCA") "lc" else "wa",
                             newdata = newdata, rank = take)
            else
                warning(gettextf("'newdata' ignored: it must have the same number of rows as the original community data with type = '%s'", type))
        }
        if (take > 0)
            Xbar <- u %*% slam %*% t(v)
        rc <- outer(rs, cs)
        if (type == "response")
            out <- (Xbar + 1) * rc * gtot
        else                 # type == "working"
            out <- Xbar * sqrt(rc)
    }
    else if (type == "lc") {
        if (model == "CA")
            stop("'lc' scores not available for unconstrained ordination")
        if (!missing(newdata)) {
            if (is.null(object$terminfo))
                E <- as.matrix(newdata)
            else {
                d <- ordiParseFormula(formula(object), newdata, object$terminfo$xlev)
                E <- cbind(d$Z, d$Y)
            }
            E <- sweep(E, 2, c(object$pCCA$envcentre, object$CCA$envcentre),
                       "-")
            Q <- object[[model]]$QR
            p1 <- Q$pivot[1:Q$rank]
            u <- E[, p1, drop = FALSE] %*% coef(object)[p1, ,
                         drop = FALSE]
            u <- u[, 1:take, drop = FALSE]
        }
        out <- u
        if (scaling) {      # implicit conversion "none" == 0 == FALSE
            scal <- list(diag(slam), 1, sqrt(diag(slam)))[[abs(scaling)]]
            out <- sweep(out, 2, scal, "*")
            if (scaling < 0) {
                scal <- sqrt(1/(1 - diag(slam)^2))
                out <- sweep(out, 2, scal, "*")
            }
        }
    }
    else if (type == "wa") {
        if (!missing(newdata)) {
            if (!is.null(object$pCCA))
                stop("no 'wa' scores available (yet) in partial CCA")
            nm <- rownames(v)
            if (!is.null(nm)) { # Got rownames: keep only species with scores
                if (!all(nm %in% colnames(newdata)))
                    stop("'newdata' does not have named columns matching one or more the original columns")
                newdata <-  newdata[, nm, drop = FALSE]
            } else { #Rownames are NULL: still try to remove exclude.spec
                exclude.spec <- attr(object[[model]]$v, "na.action")
                if (!is.null(exclude.spec))
                    Xbar <- Xbar[, -exclude.spec]
            }
            Xbar <- as.matrix(newdata)
            rs <- rowSums(Xbar)
            Xbar <- (Xbar - outer(rs, cs))/sqrt(outer(rs, cs))
            v <- sweep(v, 1, sqrt(cs), "*")
            w <- sweep(Xbar %*% v, 1, sqrt(rs), "/")
            w <- sweep(w, 2, diag(slam), "/")
        }
        out <- w
        if (scaling) {      # implicit conversion "none" == 0 == FALSE
            scal <- list(diag(slam), 1, sqrt(diag(slam)))[[abs(scaling)]]
            out <- sweep(out, 2, scal, "*")
            if (scaling < 0) {
                scal <- sqrt(1/(1 - diag(slam)^2))
                out <- sweep(out, 2, scal, "*")
            }
        }
    }
    else if (type == "sp") {
        if (!missing(newdata)) {
            nm <- rownames(u)
            if (!is.null(nm)) {
                if (!all(nm %in% rownames(newdata)))
                    stop("'newdata' does not have named rows matching one or more of the original rows")
                newdata <- newdata[nm, , drop = FALSE]
            }
            Xbar <- as.matrix(newdata)
            cs <- colSums(Xbar)
            Xbar <- (Xbar - outer(rs, cs))/sqrt(outer(rs, cs))
            if (!is.null(object$pCCA))
                Xbar <- qr.resid(object$pCCA$QR, Xbar)
            u <- sweep(u, 1, sqrt(rs), "*")
            v <- sweep(t(Xbar) %*% u, 1, sqrt(cs), "/")
            v <- sweep(v, 2, diag(slam), "/")
        }
        out <- v
        if (scaling) {      # implicit conversion "none" == 0 == FALSE
            scal <- list(1, diag(slam), sqrt(diag(slam)))[[abs(scaling)]]
            out <- sweep(out, 2, scal, "*")
            if (scaling < 0) {
                scal <- sqrt(1/(1 - diag(slam)^2))
                out <- sweep(out, 2, scal, "*")
            }
        }
    }
    out
}

`as.rad` <-
    function(x)
{
    if (inherits(x, "rad"))
        return(x)
    take <- x > 0
    nm <- names(x)
    comm <- x[take]
    names(comm) <- nm[take]
    comm <- sort(comm, decreasing = TRUE, index.return = TRUE)
    ## ordered index of included taxa
    index <- which(take)[comm$ix]
    comm <- comm$x
    attr(comm, "index") <- index
    class(comm) <- "rad"
    comm
}

## do not print 'index' attribute

`print.rad` <-
    function(x, ...)
{
    print(as.table(x), ...)
    invisible(x)
}

`calibrate.ordisurf` <-
    function(object, newdata, ...)
{
    if (missing(newdata))
        fit <- predict(object, type = "response", ...)
    else {
        ## Got only a vector of two coordinates
        if (is.vector(newdata) && length(newdata) == 2) 
            newdata = data.frame(x1 = newdata[1], x2 = newdata[2])
        ## Got a matrix or a data frme
        else{
            if (NCOL(newdata) < 2) 
                stop("needs a matrix or a data frame with two columns")
            newdata <- data.frame(x1 = newdata[,1], x2 = newdata[,2])
        }
        fit <- predict(object, newdata = newdata, type = "response", ...)
    }
    fit
}

"summary.bioenv" <-
    function(object, ...)
{
    x <- object$models
    nam <- object$names
    size <- seq_along(x)
    cor <- unlist(lapply(x, function(tmp) tmp$est))
    pars <- unlist(lapply(x, function(tmp) paste(nam[tmp$best], collapse=" ")))
    out <- list(size = size, correlation = cor, variables = pars)
    class(out) <- "summary.bioenv"
    out
}

`ordicluster` <-
    function (ord, cluster, prune = 0, display = "sites",
              w = weights(ord, display), col = 1,
              draw = c("segments", "none"), ...)
{
    weights.default <- function(object, ...) NULL
    w <- eval(w)
    mrg <- cluster$merge
    ord <- scores(ord, display = display, ...)
    if (nrow(mrg) != nrow(ord) - 1)
        stop("dimensions do not match in 'ord' and 'cluster'")
    if (length(w) == 1) w <- rep(w, nrow(ord))
    n <- if (is.null(w)) rep(1, nrow(ord)) else w
    noden <- numeric(nrow(mrg) - prune)
    go <- matrix(0, nrow(mrg) - prune, 2)
    ## recycle colours for points and prepare to get node colours
    col <- rep(col, length = nrow(ord))
    col <- col2rgb(col)/255
    nodecol <- matrix(NA, nrow(mrg) - prune, 3)
    seg.coords <- matrix(NA, nrow = nrow(mrg) - prune, ncol = 4)
    for (i in seq_len(nrow(mrg) - prune)) {
        a <- mrg[i,1]
        b <- mrg[i,2]
        one <- if (a < 0) ord[-a,] else go[a,]
        two <- if (b < 0) ord[-b,] else go[b,]
        n1 <- if (a < 0) n[-a] else noden[a]
        n2 <- if (b < 0) n[-b] else noden[b]
        xm <- weighted.mean(c(one[1],two[1]), w = c(n1,n2))
        ym <- weighted.mean(c(one[2],two[2]), w = c(n1,n2))
        go[i,] <- c(xm,ym)
        noden[i] <- n1 + n2
        colone <- if (a < 0) col[,-a] else nodecol[a,]
        coltwo <- if (b < 0) col[,-b] else nodecol[b,]
        nodecol[i,] <- (n1 * colone + n2 * coltwo)/noden[i]
        seg.coords[i, ] <- c(one[1], one[2], two[1], two[2])
    }
    colnames(seg.coords) <- c("x1","y1","x2","y2")
    ## are we plotting?
    draw <- match.arg(draw)
    if (isTRUE(all.equal(draw, "segments"))) {
        ordiArgAbsorber(seg.coords[,1L], seg.coords[,2L],
                        seg.coords[,3L], seg.coords[,4L],
                        col = rgb(nodecol),
                        FUN = segments, ...)
    }
    colnames(go) <- c("x","y")
    seg.coords <- cbind(as.data.frame(seg.coords), col = rgb(nodecol))
    out <- structure(list(scores = cbind(go, "w" = noden),
                          segments = seg.coords), class = "ordicluster")
    invisible(out)
}

#' Slope of Rarefunction Curve at Given Sample Size
#'
#' Function evaluates the derivative of the rarefaction
#' function at given sample size. The derivative was
#' directly derived from the expression used in \code{rarefy}.
#'
#' @param x Community counts, either an integer vector for a single
#' site or a data frame or matrix with each row giving site vectors.
#' @param sample Sample sizes where the derivatives are evaluated; can
#' be real
#'
`rareslope` <-
    function(x, sample)
{
    ## 'x' must be integers ('sample' need not be)
    if (!identical(all.equal(x, round(x)), TRUE))
        stop("community data 'x' must be integers (counts)")
    slope <- function(x, sample) {
        x <- x[x>0]
        J <- sum(x)
        ## Replace Hurlbert's factorials with gamma() functions and do
        ## some algebra for derivatives. NB., rarefy() does not use
        ## factorials but lchoose directly.
        d <- digamma(pmax(J-sample+1, 1)) - digamma(pmax(J-x-sample+1, 1))
        g <- lgamma(pmax(J-x+1, 1)) + lgamma(pmax(J-sample+1, 1)) -
            lgamma(pmax(J-x-sample+1, 1)) - lgamma(J+1)
        d <- d*exp(g)
        sum(d[is.finite(d)])
    }
    if (length(dim(x)) == 2)
        out <- sapply(sample, function(n) apply(x, 1, slope, sample = n))
    else
        out <- sapply(sample, function(n) slope(x, sample=n))
    out <- drop(out)
    if (length(sample) > 1) {
        if (is.matrix(out))
            colnames(out) <- paste0("N", sample)
        else
            names(out) <- paste0("N", sample)
    }
    out
}

`wcmdscale` <-
    function(d, k, eig = FALSE, add = FALSE, x.ret = FALSE, w)
{
    ## Force eig=TRUE if add, x.ret or !missing(w)
    if(x.ret)
        eig <- TRUE
    ZERO <- sqrt(.Machine$double.eps)
    if (!inherits(d, "dist")) {
        op <- options(warn = 2)
        on.exit(options(op))
        d <- as.dist(d)
        options(op)
    }
    ## handle add constant to make d Euclidean
    if (is.logical(add) && isTRUE(add))
        add <- "lingoes"
    if (is.character(add)) {
        add <- match.arg(add, c("lingoes", "cailliez"))
        if (add == "lingoes") {
            ac <- addLingoes(as.matrix(d))
            d <- sqrt(d^2 + 2 * ac)
        } else if (add == "cailliez") {
            ac <- addCailliez(as.matrix(d))
            d <- d + ac
        }
    } else {
        ac <- NA
    }
    ## Gower centring
    m <- as.matrix(d^2)
    n <- nrow(m)
    if (missing(w))
        w <- rep(1, n)
    m <- .Call(do_wcentre, m, w)
    m <- t(.Call(do_wcentre, t(m), w))
    e <- eigen(-m/2, symmetric = TRUE)
    ## Remove zero eigenvalues, keep negative
    keep <- abs(e$values) > max(ZERO, ZERO * e$values[1L])
    e$values <- e$values[keep]
    e$vectors <- e$vectors[, keep, drop = FALSE]
    ## Deweight and scale axes -- also negative
    points <- sweep(e$vectors, 1, sqrt(w), "/")
    points <- sweep(points, 2, sqrt(abs(e$values)), "*")
    rownames(points) <- rownames(m)
    ## If 'k' not given, find it as the number of positive
    ## eigenvalues, and also save negative eigenvalues
    negaxes <- NULL
    if (missing(k) || k > sum(e$value > ZERO)) {
        k <- sum(e$values > ZERO)
        if (any(e$values < 0))
            negaxes <- points[, e$values < 0, drop = FALSE]
    }
    points <- points[, 1:k, drop=FALSE]
    points[!is.finite(points)] <- NA
    ## Goodness of fit
    ev <- e$values[1:k]
    ev <- ev[ev > 0]
    ## GOF for real and all axes
    GOF <- c(sum(ev)/sum(abs(e$values)),
             sum(ev)/sum(e$values[e$values > 0]))
    if (eig || x.ret) {
        colnames(points) <- paste("Dim", seq_len(NCOL(points)), sep="")
        out <- list(points = points, eig = if (eig) e$values,
                    x = if (x.ret) m, ac = ac, add = add, GOF = GOF,
                    weights = w, negaxes = negaxes, call = match.call())
        class(out) <- "wcmdscale"
    }
    else out <- points
    out
}

`coef.cca` <-
    function (object, norm = FALSE, ...)
{
    if(is.null(object$CCA))
        stop("unconstrained models do not have coefficients")
    Q <- object$CCA$QR
    u <- object$CCA$u
    ## if rank==0, the next would fail, but this kluge gives
    ## consistent results with coef.rda and vegan 2.4
    if (ncol(u))
        u <- sqrt(object$rowsum) * u
    b <- qr.coef(Q, u)
    if (norm)
        b <- sqrt(colSums(qr.X(object$CCA$QR)^2)) * b
    b
}

`summary.prc` <- function (object, axis = 1, scaling = "symmetric", const,
                           digits = 4, correlation = FALSE, ...) {
    sc = scores(object, scaling = scaling, display = c("sp", "reg"), const,
                choices = axis, correlation = correlation, ...)
    b <- sc$regression
    prnk <- object$pCCA$rank
    lentreat <- length(object$terminfo$xlev[[2]])
    b = matrix(b[-(1:prnk)], nrow = lentreat-1, byrow = TRUE)
    rownames(b) <- (object$terminfo$xlev[[2]])[-1]
    colnames(b) <- object$terminfo$xlev[[1]]
    out <- list(sp = drop(sc$species), coefficients = b,
                names = names(object$terminfo$xlev),
                corner = (object$terminfo$xlev[[2]])[1],
                call = object$call, digits = digits)
    class(out) <- "summary.prc"
    out
}

`multipart.formula` <-
    function(formula, data, index=c("renyi", "tsallis"), scales = 1,
             global = FALSE, relative = FALSE, nsimul=99,
             method = "r2dtable", ...)
{
    ## evaluate formula
    if (missing(data))
        data <- parent.frame()
    tmp <- hierParseFormula(formula, data)
    ## run simulations
    sim <- multipart.default(tmp$lhs, tmp$rhs, index = index, scales = scales,
                             global = global, relative = relative,
                             nsimul = nsimul, method = method, ...)
    call <- match.call()
    call[[1]] <- as.name("multipart")
    attr(sim, "call") <- call
    sim
}

`plot.specaccum` <-
    function(x, add = FALSE, random = FALSE, ci = 2,
             ci.type = c("bar","line","polygon"), col = par("fg"), lty = 1,
             ci.col = col, ci.lty = 1, ci.length = 0,  xlab, ylab = x$method,
             ylim, xvar = c("sites", "individuals", "effort"), ...)
{
    if(random && !(x$method %in% c("random", "collector")))
        stop("random = TRUE can be used only with method='random'")
    if(x$method == "collector")
        random <- TRUE
    xvar <- match.arg(xvar)
    ## adjust weights to number of sites
    if (random && !is.null(x$weights) && xvar == "sites") {
        n <- length(x$effort)
        adj <- n/x$effort[n]
    } else {
        adj <- 1
    }
    xaxvar <- x[[xvar]]
    if (missing(xlab))
        xlab <- paste(toupper(substring(xvar, 1, 1)),
                              substring(xvar, 2), sep="")
    if (random)
        ci <- FALSE
    ci.type <- match.arg(ci.type)
    if (!add) {
        if (missing(ylim))
            if (random)
                ylim <- c(1, max(x$perm, na.rm = TRUE))
            else
                ylim <- c(1, max(x$richness, x$richness + ci*x$sd, na.rm = TRUE))
        plot(xaxvar, x$richness, xlab=xlab, ylab=ylab, ylim=ylim,
             type="n", ...)
    }
    if (!is.null(x$sd) && ci)
        switch(ci.type,
               bar = {k <- x$sd > sqrt(.Machine$double.eps)
                      arrows(xaxvar[k], (x$richness - ci*x$sd)[k], xaxvar[k],
                             (x$richness + ci*x$sd)[k], col=ci.col, lty=ci.lty,
                             length = ci.length, angle = 90, code = 3, ...)},
               line = matlines(xaxvar, x$richness + t(rbind(-ci,ci) %*% x$sd),
                 col=ci.col, lty=ci.lty, ...),
               polygon = polygon(c(xaxvar, rev(xaxvar)),
                 c(x$richness - ci*x$sd, rev(x$richness + ci*x$sd)), col=ci.col,
                 lty=ci.lty,  ...)
               )
    if (random) {
        if (is.null(x$weights)) {
            for(i in seq_len(NCOL(x$perm)))
                lines(xaxvar, x$perm[,i], col=col, ...)
        } else {
            for(i in seq_len(NCOL(x$perm)))
                lines(x$weights[,i]*adj, x$perm[,i], col=col, ...)
        }
    } else
        lines(xaxvar, x$richness, col=col, lty = lty, ...)
    invisible()
}

`lines.specaccum` <-
    function(x, ...)
{
    plot(x, add = TRUE, ...)
}

`rarecurve` <-
    function(x, step = 1, sample, xlab = "Sample Size", ylab = "Species",
             label = TRUE, col, lty, ...)
{
    ## matrix is faster than data.frame
    x <- as.matrix(x)
    ## check input data: must be counts
    if (!identical(all.equal(x, round(x)), TRUE))
        stop("function accepts only integers (counts)")
    ## sort out col and lty
    if (missing(col))
        col <- par("col")
    if (missing(lty))
        lty <- par("lty")
    tot <- rowSums(x)
    S <- specnumber(x)
    ## remove empty rows or we fail
    if (any(S <= 0)) {
        message("empty rows removed")
        x <- x[S > 0,, drop =FALSE]
        tot <- tot[S > 0]
        S <- S[S > 0]
    }
    nr <- nrow(x)
    ## rep col and lty to appropriate length
    col <- rep(col, length.out = nr)
    lty <- rep(lty, length.out = nr)
    ## Rarefy
    out <- lapply(seq_len(nr), function(i) {
        n <- seq(1, tot[i], by = step)
        if (n[length(n)] != tot[i])
            n <- c(n, tot[i])
        drop(rarefy(x[i,], n))
    })
    Nmax <- sapply(out, function(x) max(attr(x, "Subsample")))
    Smax <- sapply(out, max)
    ## set up plot
    plot(c(1, max(Nmax)), c(1, max(Smax)), xlab = xlab, ylab = ylab,
         type = "n", ...)
    ## rarefied richnesses for given 'sample'
    if (!missing(sample)) {
        abline(v = sample)
        rare <- sapply(out, function(z) approx(x = attr(z, "Subsample"), y = z,
                                             xout = sample, rule = 1)$y)
        abline(h = rare, lwd=0.5)
    }
    ## rarefaction curves
    for (ln in seq_along(out)) {
        N <- attr(out[[ln]], "Subsample")
        lines(N, out[[ln]], col = col[ln], lty = lty[ln], ...)
    }
    ## label curves at their endpoitns
    if (label) {
        ordilabel(cbind(tot, S), labels=rownames(x), ...)
    }
    invisible(out)
}

## S3 print method for summary.permat
`print.summary.permat` <-
function(x, digits=2, ...)
{
    n <- attr(x$x, "times")
    cat("Summary of object of class 'permat'\n\nCall: ")
    print(x$x$call)
    cat("\nMatrix type:", attr(x$x, "mtype"), "\nPermutation type:", attr(x$x, "ptype"))
    cat("\nMethod: ", attr(x$x, "method"), sep = "")
    if (attr(x$x, "ptype") == "swap") {
        if (!is.na(attr(x$x, "burnin")))
            cat(", burnin: ", attr(x$x, "burnin"), sep = "")
        if (!is.na(attr(x$x, "thin")))
            cat(", thin: ", attr(x$x, "thin"), sep = "")
    }
    cat("\nRestricted:", attr(x$x, "is.strat"), "\nFixed margins:", attr(x$x, "fixedmar"))
    if (!is.na(attr(x$x, "shuffle"))) {
        if (attr(x$x, "shuffle")=="ind") cat("\nIndividuals")
        if (attr(x$x, "shuffle")=="samp") cat("\nSamples")
        if (attr(x$x, "shuffle")=="both") cat("\nIndividuals and samples")
    cat(" are shuffled")
    }
    cat("\n\nMatrix dimensions:", nrow(x$x$orig), "rows,", ncol(x$x$orig), "columns")
    cat("\nSum of original matrix:", sum(x$x$orig))
    cat("\nFill of original matrix:", round(sum(x$x$orig>0)/(nrow(x$x$orig)*ncol(x$x$orig)),digits))
    cat("\nNumber of permuted matrices:", n,"\n")
    cat("\nMatrix sums retained:", round(100 * sum(x$sum) / n, digits), "%")
    cat("\nMatrix fill retained:", round(100 * sum(x$fill) / n, digits), "%")
    cat("\nRow sums retained:   ", round(100 * sum(x$rowsums) / n, digits), "%")
    cat("\nColumn sums retained:", round(100 * sum(x$colsums) / n, digits), "%")
    cat("\nRow incidences retained:   ", round(100 * sum(x$browsums) / n, digits), "%")
    cat("\nColumn incidences retained:", round(100 * sum(x$bcolsums) / n, digits), "%")
    if (!is.null(x$strsum))
        cat("\nSums within strata retained:", round(100 * sum(x$strsum) / n, digits), "%")
    cat("\n\nBray-Curtis dissimilarities among original and permuted matrices:\n")
    print(summary(x$bray))
    cat("\nChi-squared for original matrix: ", round(attr(x$chisq, "chisq.orig"), digits), "\n", sep = "")
    cat("Chi-squared values among expected and permuted matrices:\n")
    print(summary(x$chisq))
invisible(x)
}

`plot.betadiver` <-
    function (x, ...) 
{
    xy <- scores(x, ...)
    plot(c(0, 1), c(0, sqrt(0.75)), type = "n", axes = FALSE, 
         xlab = "", ylab = "", asp = 1)
    for (tic in seq(0.2, 0.8, by = 0.2)) {
        segments(tic, 0, tic/2, sqrt(0.75) * tic, lty = 3)
        segments(tic/2, sqrt(0.75) * tic, 1 - tic/2, sqrt(0.75) * 
                 tic, lty = 3)
        segments(tic, 0, tic/2 + 0.5, sqrt(0.75) * (1 - tic), 
                 lty = 3)
    }
    text(c(0, 1, 0.5), c(0, 0, sqrt(0.75)), c("b'", "c'", "a'"), 
         pos = c(2, 4, 3), cex = par("cex.axis"), xpd=TRUE)
    lines(c(0, 1, 0.5, 0), c(0, 0, sqrt(0.75), 0), xpd = TRUE)
    points(xy, ...)
    class(xy) <- "ordiplot"
    invisible(xy)
}

`plot.renyi` <-
    function(x, ...)
{
    if (inherits(x, "data.frame")) {
        plt <- factor(rep(rownames(x), ncol(x)), levels=rownames(x))
        alp <- factor(rep(colnames(x), each=nrow(x)), levels=colnames(x))
        div  <- as.vector(as.matrix(x))
        df <- data.frame(diversity=div, plot=plt, alpha=alp)
        lo <- tapply(div, alp, min)
        hi <- tapply(div, alp, max)
        med <- tapply(div, alp, median)
    } else {
        df <- data.frame(diversity = x, alpha = factor(names(x), levels=names(x)), plot = "plot")
        lo <- hi <- med <- NA
    }
    cl <- trellis.par.get("superpose.line")$col
    bwplot(diversity ~ alpha | plot, data=df,  
           panel = function(x, y, ...) {
               panel.lines(x, lo, lty=2, col=cl[3])
               panel.lines(x, med, lty=2, col=cl[2])
               panel.lines(x, hi, lty=2, col=cl[3])
               panel.xyplot(x, y,  ...)
           },
           ...)
}

`print.varpart` <-
    function (x, ...)
{
    cat("\nPartition of", x$inert, "in", x$RDA, "\n\n")
    writeLines(strwrap(pasteCall(x$call)))
    if (x$RDA == "RDA") {
        if (x$scale)
            cat("Columns of Y were scaled to unit variance\n")
        if (!is.null(x$transfo))
            cat("Species transformation: ", x$transfo)
    }
    cat("\n")
    cat("Explanatory tables:\n")
    cat(paste(paste(paste("X", seq_along(x$tables), sep=""),":  ",
                    x$tables, sep=""), collapse="\n"), "\n\n")
    print(x$part, ...)
    invisible(x)
}

`permutest.betadisper` <- function(x, pairwise = FALSE,
                                   permutations = 999,
                                   parallel = getOption("mc.cores"), ...)
{
    EPS <- sqrt(.Machine$double.eps) # for P-value comparisons
    t.statistic <- function(x, y) {
        m <- length(x)
        n <- length(y)
        xbar <- mean(x)
        ybar <- mean(y)
        xvar <- var(x)
        yvar <- var(y)
        pooled <- sqrt(((m-1)*xvar + (n-1)*yvar) / (m+n-2))
        (xbar - ybar) / (pooled * sqrt(1/m + 1/n))
    }

    permFun <- function(idx) {
        if (!is.matrix(idx)) {
            dim(idx) <- c(1, length(idx))
        }
        R <- nrow(idx)
        Fperm <- matrix(nrow = R, ncol = 1)
        if (pairwise) {                 # set up object to hold t stats
            Tperm <- matrix(ncol = n.pairs, nrow = R)
            Jseq <- seq_len(n.pairs)
        }
        rdf <- nobs - p                 # residual degrees of freedom
        ## iterate
        for (i in seq_len(R)) {         # iterate
            take <- idx[i, ]                # current permutation from set
            p.resid <- resids[take]         # permute residuals
            f <- qr.fitted(mod.Q, p.resid)  # create new data
            mss <- sum((f - mean(f))^2)
            r <- qr.resid(mod.Q, p.resid)
            rss <- sum(r^2)
            resvar <- rss / rdf
            Fperm[i, ] <- (mss / (p - 1)) / resvar

            ## pairwise tests
            if(pairwise) {
                for(j in Jseq) {
                    grp1 <- x$distance[take][group == combin[1, j]]
                    grp2 <- x$distance[take][group == combin[2, j]]
                    Tperm[i, j] <- t.statistic(grp1, grp2)
                }
            }
        }

        ## bind on pairwise stats if any
        if (pairwise) {
            Fperm <- cbind(Fperm, Tperm)
        }
        Fperm
    }

    if(!inherits(x, "betadisper"))
        stop("only for class \"betadisper\"")

    ## will issue error if only a single group
    mod.aov <- anova(x)
    nobs <- length(x$distances) ## number of observations
    mod <- lm(x$distances ~ x$group)
    mod.Q <- mod$qr
    p <- mod.Q$rank
    resids <- qr.resid(mod.Q, x$distances)

    ## extract groups
    group <- x$group
    ## permutations is either a single number, a how() structure or a
    ## permutation matrix
    permutations <- getPermuteMatrix(permutations, nobs)
    nperm <- nrow(permutations)

    ## pairwise comparisons
    if(pairwise) {
        combin <- combn(levels(x$group), 2) # unique pairings
        n.pairs <- ncol(combin)
    }

    ## Parallel processing of permutations
    if (is.null(parallel)) {
        parallel <- 1
    }
    hasClus <- inherits(parallel, "cluster")
    if (hasClus || parallel > 1L) {
        if (.Platform$OS.type == "unix" && !hasClus) {
            Pstats <- do.call("rbind",
                           mclapply(seq_len(nperm),
                                    function(x) permFun(permutations[x, , drop = FALSE]),
                                    mc.cores = parallel))
        } else {
            ## if hasClus, don't set up and top a temporary cluster
            if (!hasClus) {
                parallel <- makeCluster(parallel)
            }
            Pstats <- parRapply(parallel, permutations,
                                          function(x) permFun(x))
            if (!hasClus) {
                stopCluster(parallel)
            }
        }
    } else {
        Pstats <- permFun(permutations)
    }

    ## Process results
    F0 <- summary(mod)$fstatistic[1]
    Fstats <- Pstats[, 1]    # allow empty dim to be dropped
    statistic <- F0
    names(statistic) <- "Overall (F)"

    ## pairwise comparisons
    if(pairwise) {
        T0 <- apply(combn(levels(group), 2), 2, function(z) {
            t.statistic(x$distances[group == z[1]],
                        x$distances[group == z[2]])})
        Tstats <- Pstats[, -1, drop = FALSE]
        statistic <- c(statistic, T0)
    }

    ## compute permutation p-value
    pval <- (sum(Fstats >= F0 - EPS) + 1) / (length(Fstats) + 1)

    if(pairwise) {
        df <- apply(combin, 2, function(z) {
            length(x$distances[group == z[1]]) +
                length(x$distance[group == z[2]]) - 2})
        pairp <- (colSums(sweep(abs(Tstats), 2, abs(T0), '>=')) + 1) /
            (NROW(Tstats) + 1)
        pairp <- list(observed = 2 * pt(-abs(T0), df),
                         permuted = pairp)
        tnames <- apply(combin, 2, paste, collapse = "-")
        names(pairp$observed) <- names(pairp$permuted) <- tnames
        names(statistic)[-1] <- paste(tnames, "(t)")
    } else {
        pairp <- NULL
    }

    retval <- cbind(mod.aov[, 1:4], c(nperm, NA), c(pval, NA))
    dimnames(retval) <- list(c("Groups", "Residuals"),
                             c("Df", "Sum Sq", "Mean Sq", "F", "N.Perm",
                               "Pr(>F)"))
    retval <- list(tab = retval,
                   pairwise = pairp,
                   groups = levels(group),
                   statistic = statistic,
                   perm = if (pairwise) {
                       structure(cbind(Fstats, Tstats), dimnames = list(NULL, names(statistic)))
                   } else {
                       structure(Fstats, names = names(statistic))
                   },
                   control = attr(permutations, "control"))
    class(retval) <- "permutest.betadisper"
    retval
}

### anova.cca cannot be performed if residuals or constraints are
### NULL, and this function handles these cases (but it doesn't test
### that these are the cases).
`anova.ccanull` <-
    function(object, ...)
{
    table <- matrix(0, nrow = 2, ncol = 4)
    if (object$CA$rank == 0) {
        table[1,] <- c(object$CCA$qrank, object$CCA$tot.chi, NA, NA)
        table[2,] <- c(0,0,NA,NA)
    }
    else {
        table[1,] <- c(0,0,0,NA)
        table[2,] <- c(nrow(object$CA$u) - 1, object$CA$tot.chi, NA, NA)
    }
    rownames(table) <- c("Model", "Residual")
    if (inherits(object, c("capscale", "dbrda")) && object$adjust == 1)
        varname <- "SumOfSqs"
    else if (inherits(object, "rda"))
        varname <- "Variance"
    else
        varname <- "ChiSquare"
    colnames(table) <- c("Df", varname, "F", "Pr(>F)")
    table <- as.data.frame(table)
    if (object$CA$rank == 0)
        head <- "No residual component\n"
    else if (is.null(object$CCA) || object$CCA$rank == 0)
        head <- "No constrained component\n"
    else
        head <- c("!!!!! ERROR !!!!!\n")
    head <- c(head, paste("Model:", c(object$call)))
    if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
        seed <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
    else
        seed <- NULL
    structure(table, heading = head, Random.seed = seed,
              class = c("anova.cca", "anova", "data.frame"))
}

`anova.cca` <-
    function(object, ..., permutations = how(nperm=999), by = NULL,
             model = c("reduced", "direct", "full"),
             parallel = getOption("mc.cores"), strata = NULL,
             cutoff = 1, scope = NULL)
{
    EPS <- sqrt(.Machine$double.eps) # for permutation P-values
    model <- match.arg(model)
    ## permutation matrix
    N <- nrow(object$CA$u)
    permutations <- getPermuteMatrix(permutations, N, strata = strata)
    seed <- attr(permutations, "seed")
    control <- attr(permutations, "control")
    nperm <- nrow(permutations)
    ## see if this was a list of ordination objects
    dotargs <- list(...)
    ## we do not want to give dotargs to anova.ccalist, but we
    ## evaluate 'parallel' and 'model' here
    if (length(dotargs)) {
        isCCA <- sapply(dotargs, function(z) inherits(z, "cca"))
        if (any(isCCA)) {
            dotargs <- dotargs[isCCA]
            object <- c(list(object), dotargs)
            sol <-
                anova.ccalist(object,
                              permutations = permutations,
                              model = model,
                              parallel = parallel)
            attr(sol, "Random.seed") <- seed
            attr(sol, "control") <- control
            return(sol)
        }
    }
    ## We only have a single model: check if it is empty
    if (is.null(object$CA) || is.null(object$CCA) ||
        object$CCA$rank == 0 || object$CA$rank == 0)
        return(anova.ccanull(object))
    ## by cases
    if (!is.null(by)) {
        by <- match.arg(by, c("terms", "margin", "axis"))
        if (is.null(object$terms))
            stop("model must be fitted with formula interface")
        sol <- switch(by,
                      "terms" = anova.ccabyterm(object,
                      permutations = permutations,
                      model = model, parallel = parallel),
                      "margin" = anova.ccabymargin(object,
                      permutations = permutations,
                      model = model, parallel = parallel,
                      scope = scope),
                      "axis" = anova.ccabyaxis(object,
                      permutations = permutations,
                      model = model, parallel = parallel,
                      cutoff = cutoff))
        attr(sol, "Random.seed") <- seed
        attr(sol, "control") <- control
        return(sol)
    }
    ## basic overall test: pass other arguments except 'strata'
    ## because 'permutations' already is a permutationMatrix
    tst <- permutest.cca(object, permutations = permutations,
                         model = model, parallel = parallel, ...)
    Fval <- c(tst$F.0, NA)
    Pval <- (sum(tst$F.perm >= tst$F.0 - EPS) + 1)/(tst$nperm + 1)
    Pval <- c(Pval, NA)
    table <- data.frame(tst$df, tst$chi, Fval, Pval)
    if (inherits(object, c("capscale", "dbrda")) && object$adjust == 1)
        varname <- "SumOfSqs"
    else if (inherits(object, "rda"))
        varname <- "Variance"
    else
        varname <- "ChiSquare"
    colnames(table) <- c("Df", varname, "F", "Pr(>F)")
    head <- paste0("Permutation test for ", tst$method, " under ",
                  tst$model, " model\n", howHead(control))
    mod <- paste("Model:", c(object$call))
    structure(table, heading = c(head, mod), Random.seed = seed,
              control = control, F.perm = tst$F.perm,
              class = c("anova.cca", "anova", "data.frame"))
}

"plot.procrustes" <-
    function (x, kind = 1, choices = c(1,2), to.target = TRUE,
              type = "p", xlab, ylab, main, ar.col = "blue",
              len = 0.05, cex = 0.7,  ...) 
{
    type <- match.arg(type, c("points", "text", "none"))
    if (to.target) {
        tails <- x$Yrot[, choices]
        heads <- x$X[, choices]
    } else {
        tails <- x$X[, choices]
        heads <- x$Yrot[, choices]
    }
    if (missing(main)) 
        main <- "Procrustes errors"
    if (kind <= 1) {
        formals(arrows) <- c(formals(arrows), alist(... = ))
        if (missing(xlab)) 
            xlab <- paste("Dimension", choices[1])
        if (missing(ylab)) 
            ylab <- paste("Dimension", choices[2])
        xrange <- range(tails[, 1], heads[, 1])
        yrange <- range(tails[, 2], heads[, 2])
        plot(xrange, yrange, xlab = xlab, ylab = ylab, main = main, 
             type = "n", asp = 1, ...)
        if (kind > 0) {
            abline(v = 0, lty = 2)
            abline(h = 0, lty = 2)
            if (ncol(x$rotation) == 2) {
                ## Draw rotated axes only if they visibly differ from
                ## unrotated axes
                b <- abs(x$rotation[1,1])
                EPS <- 1e-8
                if (1 - b > EPS && b > EPS) {
                    rot <- x$rotation
                    abline(0, rot[1,2]/rot[1,1], lty = 1)
                    abline(0, rot[2,2]/rot[2,1], lty = 1)
                } else {
                    abline(v = 0, h = 0)
                }
            }
            else {
            	Y <- x$Yrot %*% t(x$rotation)
            	for (k in 1:ncol(Y)) {
                    tmp <- matrix(0, nrow=2, ncol = ncol(Y))
                    tmp[,k] <- range(Y[,k])
                    tmp <- tmp %*% x$rotation
                    lines(tmp[,choices], lty=1)
                    text(tmp[2,choices[1]], tmp[2,choices[2]], as.character(k))
                }
            }
            if (type != "none") {
                ow <- options(warn = -1)
                arrows(tails[, 1], tails[, 2], heads[, 1], heads[, 2],
                       col = ar.col, len = len, ...)
                options(ow)
                if (type == "text" && !is.null(rownames(tails)))
                    ordilabel(tails, cex = cex, ...)
                else
                    points(tails, cex = cex, ...)
            }
        }
        out <- list(heads = heads, points = tails)
        class(out) <- "ordiplot"
    }
    else if (kind == 2) {
        if (missing(xlab)) 
            xlab <- "Index"
        if (missing(ylab)) 
            ylab <- "Procrustes residual"
        res <- residuals(x)
        q <- quantile(res)
        plot(res, type = "h", xlab = xlab, ylab = ylab, main = main, 
             ...)
        abline(h = q[2:4], lty = c(2, 1, 2))
        out <- list(sites = cbind(seq(along = res), res))
        class(out) <- "ordiplot"
    }
    invisible(out)
}

"print.radfit.frame" <-
    function (x, ...) 
{
    cat("\nDeviance for RAD models:\n\n")
    out <- sapply(x, function(x) unlist(lapply(x$models, deviance)))
    printCoefmat(out, na.print = "", ...)
    invisible(x)
}

`plot.renyiaccum` <-
function (x, what=c("Collector", "mean", "Qnt 0.025", "Qnt 0.975"),
          type = "l", ...) 
{
        what <- what[what %in% dimnames(x)[[3]]]
	if (any(what %in% dimnames(x)[[3]]))
	    x <- x[,,what, drop = FALSE]
	dm <- dim(x)
        dnam <- dimnames(x)
	lin <- rep(dnam[[3]], each=dm[1]*dm[2])
	Sites <- rep(1:dm[1], len=prod(dm))
	alp <- factor(dnam[[2]], levels=dnam[[2]])
	alpha <- rep(rep(alp, each=dm[1]), len=prod(dm))
	Diversity <- as.vector(x)
	xyplot(Diversity ~ Sites | alpha, groups=lin, type=type, ...)
}

`mantel.partial` <-
  function (xdis, ydis, zdis, method = "pearson", permutations = 999, 
            strata = NULL, na.rm = FALSE, parallel = getOption("mc.cores")) 
{
    EPS <- sqrt(.Machine$double.eps)
    part.cor <- function(rxy, rxz, ryz) {
        (rxy - rxz * ryz)/sqrt(1-rxz*rxz)/sqrt(1-ryz*ryz)
    }
    xdis <- as.dist(xdis)
    ydis <- as.vector(as.dist(ydis))
    zdis <- as.vector(as.dist(zdis))
    ## Handle missing values
    if (na.rm)
        use <- "complete.obs"
    else
        use <- "all.obs"
    rxy <- cor(as.vector(xdis), ydis, method = method, use = use)
    rxz <- cor(as.vector(xdis), zdis, method = method, use = use)
    ryz <- cor(ydis, zdis, method = method, use = use)
    variant <- match.arg(method, eval(formals(cor)$method))
    variant <- switch(variant,
                      pearson = "Pearson's product-moment correlation",
                      kendall = "Kendall's rank correlation tau",
                      spearman = "Spearman's rank correlation rho",
                      variant)
    statistic <- part.cor(rxy, rxz, ryz)
    N <- attr(xdis, "Size")
    permat <- getPermuteMatrix(permutations, N, strata = strata)
    if (ncol(permat) != N)
        stop(gettextf("'permutations' have %d columns, but data have %d observations",
                      ncol(permat), N))
    permutations <- nrow(permat)

    if (permutations) {
        N <- attr(xdis, "Size")
        perm <- rep(0, permutations)
        xmat <- as.matrix(xdis)
        asdist <- row(xmat) > col(xmat)
        ptest <- function(take, ...) {
            permvec <- (xmat[take, take])[asdist]
            rxy <- cor(permvec, ydis, method = method, use = use)
            rxz <- cor(permvec, zdis, method = method, use = use)
            part.cor(rxy, rxz, ryz)
        }
        ## parallel processing
        if (is.null(parallel))
            parallel <- 1
        hasClus <- inherits(parallel, "cluster")
        if (hasClus || parallel > 1) {
            if(.Platform$OS.type == "unix" && !hasClus) {
                perm <- do.call(rbind,
                               mclapply(1:permutations,
                                        function(i, ...) ptest(permat[i,],...),
                                        mc.cores = parallel))
            } else {
                if (!hasClus) {
                    parallel <- makeCluster(parallel)
                }
                perm <- parRapply(parallel, permat, ptest)
                if (!hasClus)
                    stopCluster(parallel)
            }
        } else {
            perm <- sapply(1:permutations, function(i, ...) ptest(permat[i,], ...))
        }
        signif <- (sum(perm >= statistic - EPS)+1)/(permutations + 1)
    }
    else {
        signif <- NA
        perm <- NULL
    }
    res <- list(call = match.call(), method = variant, statistic = statistic, 
                signif = signif, perm = perm, permutations = permutations,
                control = attr(permat, "control"))
    if (!missing(strata)) {
        res$strata <- deparse(substitute(strata))
        res$stratum.values <- strata
    }
    class(res) <- c("mantel.partial", "mantel")
    res
}

`print.varpart234` <-
    function(x, digits = 5, ...)
{
    cat("No. of explanatory tables:", x$nsets, "\n")
    cat("Total variation (SS):", format(x$SS.Y, digits=digits), "\n")
    if (x$ordination == "rda")
        cat("            Variance:", format(x$SS.Y/(x$n-1), digits=digits), "\n")
    cat("No. of observations:",  x$n, "\n")
    cat("\nPartition table:\n")
    out <- rbind(x$fract, "Individual fractions" = NA, x$indfract)
    if (x$nsets > 3)
        out <- rbind(out, "Controlling 2 tables X" = NA, x$contr2)
    if (x$nsets > 2)
        out <- rbind(out, "Controlling 1 table X" = NA, x$contr1)
    out[,2:3] <- round(out[,2:3], digits=digits)
    out[,1:4] <- sapply(out[,1:4], function(x) gsub("NA", "  ", format(x, digits=digits)))
    print(out)
    cat("---\nUse function", sQuote(x$ordination),
        "to test significance of fractions of interest\n")
    if (!is.null(x$bigwarning))
        for (i in seq_along(x$bigwarning))
            warning("collinearity detected: redundant variable(s)  between tables ",
                    x$bigwarning[i],
                    "\nresults are probably incorrect: remove redundant variable(s) and repeat the analysis",
                    call. = FALSE)
    invisible(x)
}

`oecosimu` <-
    function(comm, nestfun, method, nsimul=99,
             burnin=0, thin=1, statistic = "statistic",
             alternative = c("two.sided", "less", "greater"),
             batchsize = NA,
             parallel = getOption("mc.cores"), ...)
{
    alternative <- match.arg(alternative)
    nestfun <- match.fun(nestfun)
    if (length(statistic) > 1)
        stop("only one 'statistic' is allowed")
    if (!is.na(batchsize))
        batchsize <- batchsize * 1024 * 1024
    applynestfun <-
        function(x, fun = nestfun, statistic = "statistic", ...) {
            tmp <- fun(x, ...)
            if (is.list(tmp))
                tmp[[statistic]]
            else
                tmp
    }
    chains <- NULL
    if (inherits(comm, "simmat")) {
        x <- comm
        method <- attr(x, "method")
        nsimul <- dim(x)[3]
        if (nsimul == 1)
            stop(gettextf("only one simulation in '%s'",
                          deparse(substitute(comm))))
        comm <- attr(comm, "data")
        #thin <- attr(comm, "thin")
        burnin <- attr(x, "start") - attr(x, "thin")
        chains <- attr(x, "chains")
        simmat_in <- TRUE
    } else {
        simmat_in <- FALSE
        if (inherits(comm, "nullmodel")) {
            nm <- comm
            comm <- comm$data
        } else {
            nm <- nullmodel(comm, method)
            if (nm$commsim$binary) {
                ## sometimes people do not realize that null model
                ## makes their data binary
                if (max(abs(comm - nm$data)) > 0.1)
                    warning("nullmodel transformed 'comm' to binary data")
                comm <- nm$data
            }
        }
        method <- nm$commsim$method
    }
    ## Check the number of batches needed to run the requested number
    ## of simulations without exceeding arg 'batchsize', and find the
    ## size of each batch.
    if (!simmat_in && !is.na(batchsize)) {
        commsize <- object.size(comm)
        totsize <- commsize * nsimul
        if (totsize > batchsize) {
            nbatch <- ceiling(unclass(totsize/batchsize))
            batches <- diff(round(seq(0, nsimul, by = nsimul/nbatch)))
        } else {
            nbatch <- 1
        }
    } else {
        nbatch <- 1
    }
    if (nbatch == 1)
        batches <- nsimul

    ind <- nestfun(comm, ...)
    indstat <-
        if (is.list(ind))
            ind[[statistic]]
        else
            ind
    ## burnin of sequential models
    if (!simmat_in && nm$commsim$isSeq) {
        ## estimate thinning for "tswap" (trial swap)
        if (nm$commsim$method == "tswap") {
            checkbrd <-sum(designdist(comm, "(J-A)*(J-B)",
                                      "binary"))
            M <- nm$ncol
            N <- nm$nrow
            checkbrd <- M * (M - 1) * N * (N - 1)/4/checkbrd
            thin <- round(thin * checkbrd)
            burnin <- round(burnin * checkbrd)
        }
        if (burnin > 0)
            nm <- update(nm, burnin)
    }
    ## start with empty simind
    simind <- NULL
    ## Go to parallel processing if 'parallel > 1' or 'parallel' could
    ## be a pre-defined socket cluster or 'parallel = NULL'.
    if (is.null(parallel))
        parallel <- 1
    hasClus <- inherits(parallel, "cluster")
    if (hasClus || parallel > 1) {
        if(.Platform$OS.type == "unix" && !hasClus) {
            for (i in seq_len(nbatch)) {
                ## simulate if no simmat_in
                if(!simmat_in)
                    x <- simulate(nm, nsim = batches[i], thin = thin)
                tmp <- mclapply(seq_len(batches[i]),
                                function(j)
                                applynestfun(x[,,j], fun=nestfun,
                                             statistic = statistic, ...),
                                mc.cores = parallel)
                simind <- cbind(simind, do.call(cbind, tmp))
            }
        } else {
            ## if hasClus, do not set up and stop a temporary cluster
            if (!hasClus) {
                parallel <- makeCluster(parallel)
                ## make vegan functions available: others may be unavailable
                clusterEvalQ(parallel, library(vegan))
            }
            for(i in seq_len(nbatch)) {
                if (!simmat_in)
                    x <- simulate(nm, nsim = batches[i], thin = thin)
                simind <- cbind(simind,
                                parApply(parallel, x, 3, function(z)
                                         applynestfun(z, fun = nestfun,
                                                      statistic = statistic, ...)))
            }
            if (!hasClus)
                stopCluster(parallel)
        }
    } else {
        for(i in seq_len(nbatch)) {
            ## do not simulate if x was already a simulation
            if(!simmat_in)
                x <- simulate(nm, nsim = batches[i], thin = thin)
            simind <- cbind(simind, apply(x, 3, applynestfun, fun = nestfun,
                                          statistic = statistic, ...))
        }
    }

    simind <- matrix(simind, ncol = nsimul)

    if (attr(x, "isSeq")) {
        attr(simind, "thin") <- attr(x, "thin")
        attr(simind, "burnin") <- burnin
        attr(simind, "chains") <- chains
    }

    sd <- apply(simind, 1, sd, na.rm = TRUE)
    means <- rowMeans(simind, na.rm = TRUE)
    z <- (indstat - means)/sd
    if (any(sd < sqrt(.Machine$double.eps)))
        z[sd < sqrt(.Machine$double.eps)] <- 0
    ## results can be integers or real: comparisons differ
    if (is.integer(indstat) && is.integer(simind)) {
        pless <- rowSums(indstat >= simind, na.rm = TRUE)
        pmore <- rowSums(indstat <= simind, na.rm = TRUE)
    } else {
        EPS <- sqrt(.Machine$double.eps)
        pless <- rowSums(indstat + EPS >= simind, na.rm = TRUE)
        pmore <- rowSums(indstat - EPS <= simind, na.rm = TRUE)
    }
    if (any(is.na(simind))) {
        warning("some simulated values were NA and were removed")
        nsimul <- nsimul - rowSums(is.na(simind))
    }
    p <- switch(alternative,
                two.sided = 2*pmin(pless, pmore),
                less = pless,
                greater = pmore)
    p <- pmin(1, (p + 1)/(nsimul + 1))

    ## ADDITION: if z is NA then it is not correct to calculate p values
    ## try e.g. oecosimu(dune, sum, "permat")
    if (any(is.na(z)))
        p[is.na(z)] <- NA

    ## take care that statistics have name, or some support functions
    ## can fail
    if (is.null(names(indstat))) {
        if (length(indstat) == 1)
            names(indstat) <- statistic
        else if (length(indstat) <= length(letters))
            names(indstat) <- letters[seq_along(indstat)]
        else
            names(indstat) <- paste0("stat", seq_along(indstat))
    }
    oecosimu <- list(z = z, means = means, pval = p, simulated=simind,
                     method=method, statistic = indstat,
                     alternative = alternative, isSeq = attr(x, "isSeq"))
    out <- list(statistic = ind, oecosimu = oecosimu)
    attr(out, "call") <- match.call()
    class(out) <- "oecosimu"
    out
}

## S3 print method for permat
`print.permat` <-
function(x, digits=3, ...)
{
    cat("Object of class 'permat' with ", attr(x, "times"), " simulations\n", sep="")
    cat("\nMatrix type:", attr(x, "mtype"), "\nPermutation type:", attr(x, "ptype"))
    cat("\nMethod: ", attr(x, "method"), sep = "")
    if (attr(x, "ptype") == "swap") {
        if (!is.na(attr(x, "burnin")))
            cat(", burnin: ", attr(x, "burnin"), sep = "")
        if (!is.na(attr(x, "thin")))
            cat(", thin: ", attr(x, "thin"), sep = "")
    }
    cat("\nRestricted:", attr(x, "is.strat"), "\nFixed margins:", attr(x, "fixedmar"))
    if (!is.na(attr(x, "shuffle"))) {
        if (attr(x, "shuffle")=="ind") cat("\nIndividuals")
        if (attr(x, "shuffle")=="samp") cat("\nSamples")
        if (attr(x, "shuffle")=="both") cat("\nIndividuals and samples")
    cat(" are shuffled")
    }
    cat("\n")
    invisible(x)
}

`raupcrick` <-
    function(comm, null = "r1", nsimul = 999, chase = FALSE, ...)
{
    comm <- as.matrix(comm)
    comm <- ifelse(comm > 0, 1, 0)
    ## 'tri' is a faster alternative to as.dist(): it takes the lower
    ## diagonal, but does not set attributes of a "dist" object
    N <- nrow(comm)
    tri <- matrix(FALSE, N, N)
    tri <- row(tri) > col(tri)
    ## function(x) designdist(x, "J", terms="binary") does the same,
    ## but is much slower
    sol <- oecosimu(comm, function(x) tcrossprod(x)[tri], method = null,
                    nsimul = nsimul,
                    alternative = if (chase) "less" else "greater",
                    ...)
    ## Chase et al. way, or the standard way
    if (chase)
        out <- 1 - sol$oecosimu$pval
    else
        out <- sol$oecosimu$pval
    ## set attributes of a "dist" object
    attributes(out) <- list("class"=c("raupcrick", "dist"), "Size"=N,
                            "Labels" = rownames(comm), "call" =
                            match.call(), "Diag" = FALSE, "Upper" = FALSE,
                            "method" = "raupcrick")
    out
}

`mso` <-
    function (object.cca, object.xy, grain = 1, round.up = FALSE,
              permutations = 0)
{
    if (inherits(object.cca, "dbrda"))
        stop("'mso' is not yet implemented for 'dbrda'\ncontact developers or submit a pull request with your code in github")
    EPS <- sqrt(.Machine$double.eps)
    if (inherits(object.cca, "mso")) {
        rm <- which(class(object.cca) == "mso")
        class(object.cca) <- class(object.cca)[-rm]
    }
    object <- object.cca
    xy <- object.xy
    N <- nobs(object)
    if (inherits(object, "rda"))
        N <- 1
    ## we expect xy are coordinates and calculate distances, but a
    ## swift user may have supplied distances, and we use them.
    ## However, we won't test for distances in square matrices, but
    ## treat that as a user mistake and let it go.
    if (inherits(xy, "dist"))
        Dist <- xy
    else
        Dist <- dist(xy)
    object$grain <- grain
    if (round.up)
        H <- ceiling(Dist/grain) * grain
    else H <- round(Dist/grain) * grain
    hmax <- round((max(Dist)/2)/grain) *grain
    H[H > hmax] <- max(H)
    object$H <- H
    H <- as.vector(H)
    Dist <- sapply(split(Dist, H), mean)
    object$vario <- data.frame(H = names(table(H)), Dist = Dist,
                               n = as.numeric(table(H)))
    test <- list()
    if (is.numeric(object$CCA$rank)) {
        if (is.numeric(object$pCCA$rank)) {
            test$pcca <- sapply(split(dist(ordiYbar(object, "pCCA"))^2 *
                                      N/2, H), mean)
            test$cca <- sapply(split(dist(ordiYbar(object,"CCA"))^2 *
                                     N/2, H), mean)
            test$ca <- sapply(split(dist(ordiYbar(object,"CA"))^2 *
                                    N/2, H), mean)
            test$all.cond <- sapply(split(dist(ordiYbar(object, "partial"))^2 *
                                          N/2, H), mean)
            test$se <- sqrt(sapply(split(dist(ordiYbar(object, "partial"))^2 *
                                         N/2, H), var)/object$vario$n)
            object$vario <- cbind(object$vario, All = test$all.cond,
                                  Sum = test$ca + test$cca, CA = test$ca,
                                  CCA = test$cca,  pCCA = test$pcca,
                                  se = test$se)
        } else {
            test$all <- sapply(split(dist(ordiYbar(object, "partial"))^2 *
                                     N/2, H), mean)
            test$cca <- sapply(split(dist(ordiYbar(object, "CCA"))^2 *
                                     N/2, H), mean)
            test$ca <- sapply(split(dist(ordiYbar(object,"CA"))^2 *
                                    N/2, H), mean)
            test$se <- sqrt(sapply(split(dist(ordiYbar(object, "partial"))^2 *
                                         N/2, H), var)/object$vario$n)
            object$vario <- cbind(object$vario, All = test$all,
                                  Sum = test$ca + test$cca, CA = test$ca,
                                  CCA = test$cca, se = test$se)
        }
    } else {
        test$ca <- sapply(split(dist(ordiYbar(object, "CA"))^2 * N/2,
                                H), mean)
        object$vario <- cbind(object$vario, All = test$ca, CA = test$ca)
    }
    permat <- getPermuteMatrix(permutations, nobs(object))
    nperm <- nrow(permat)
    if (nperm) {
        object$H.test <- matrix(0, length(object$H), nrow(object$vario))
        for (i in 1:nrow(object$vario)) {
            object$H.test[, i] <- as.numeric(object$H == object$vario$H[i])
        }
        xdis <- as.matrix(dist(ordiYbar(object, "CA"))^2)
        ## taking lower triangle is faster than as.dist() because it
        ## does not set attributes
        ltri <- lower.tri(xdis)
        statistic <- abs(cor(as.vector(xdis[ltri]), object$H.test))
        permfunc <- function(k) {
            permvec <- as.vector(xdis[k,k][ltri])
            abs(cor(permvec, object$H.test))
        }
        perm <- sapply(1:nperm, function(take) permfunc(permat[take,]))
        object$vario$CA.signif <-
            (rowSums(sweep(perm, 1, statistic - EPS, ">=")) + 1)/
                (nperm + 1)
        attr(object$vario, "control") <- attr(permat, "control")
    }
    object$call <- match.call()
    class(object) <- c("mso", class(object))
    object
}

## S3 summary method for permat
`summary.permat` <-
    function(object, ...)
{
    x <- object
    ## calculations are much faster if x$orig is matrix instead of data.frame
    x$orig <- data.matrix(x$orig)
    n <- attr(x, "times")
    ss <- sum(x$orig)
    fi <- sum(x$orig > 0)
    rs <- rowSums(x$orig)
    cs <- colSums(x$orig)
    rb <- rowSums(x$orig > 0)
    cb <- colSums(x$orig > 0)
    nr <- nrow(x$orig)
    nc <- ncol(x$orig)
    bray <- sapply(x$perm, function(z) sum(abs(x$orig - z)) / sum(x$orig + z))
    psum <- sapply(x$perm, function(z) ss == sum(z))
    pfill <- sapply(x$perm, function(z) fi == sum(z > 0))
    vrow <- sapply(x$perm, function(z) sum(rs == rowSums(z)) == nr)
    vcol <- sapply(x$perm, function(z) sum(cs == colSums(z)) == nc)
    brow <- sapply(x$perm, function(z) sum(rb == rowSums(z > 0)) == nr)
    bcol <- sapply(x$perm, function(z) sum(cb == colSums(z > 0)) == nc)
    if (attr(x, "is.strat")) {
        int <- attr(x, "strata")
        nlev <- length(unique(int))
        rsagg <- rowSums(aggregate(x$orig, list(int), sum)[,-1])
        ssum <- sapply(x$perm, function(z)
            sum(rsagg == rowSums(aggregate(z, list(int), sum)[,-1])) == nlev)
    } else ssum <- NULL
    ## Chisq
    E <- rs %o% cs / ss
    chisq <- sapply(x$perm, function(z) sum((z - E)^2 / E))
    attr(chisq, "chisq.orig") <- sum((x$orig - E)^2 / E)
#    attr(chisq, "df") <- (nr - 1) * (nc - 1)
    ## ts if sequential
    seqmethods <- sapply(make.commsim(), function(z) make.commsim(z)$isSeq)
    seqmethods <- names(seqmethods)[seqmethods]
#    seqmethods <- c("swap", "tswap", "abuswap")
    if (attr(x, "method") %in% seqmethods) {
        startval <- attr(x, "burnin") + 1 
        dtime <- max(1, attr(x, "thin"))
        bray <- ts(bray, start = startval, deltat = dtime)
        chisq <- ts(chisq, start = startval, deltat = dtime)
    }
    x$perm <- NULL
    out <- list(x=x, bray=bray, chisq=chisq, sum=psum, fill=pfill, rowsums=vrow, colsums=vcol,
        browsums=brow, bcolsums=bcol, strsum=ssum)
    class(out) <- c("summary.permat", "list")
    out
}

"summary.procrustes" <-
function (object, digits = getOption("digits"), ...) 
{
    ans <- object[c("call", "ss")]
    n <- nrow(object$Yrot)
    k <- ncol(object$Yrot)
    ans$resid <- residuals(object)
    rmse <- sqrt(object$ss/n)
    ans$n <- n
    ans$k <- k
    ans$rmse <- rmse
    ans$rotation <- object$rotation
    ans$translation <- object$translation
    ans$scale <- object$scale
    ans$digits <- digits
    class(ans) <- "summary.procrustes"
    ans
}

`anosim` <-
    function (x, grouping, permutations = 999,
              distance = "bray", strata = NULL, parallel = getOption("mc.cores"))
{
    EPS <- sqrt(.Machine$double.eps)
    if (!inherits(x, "dist")) { # x is not "dist": try to change it
        if ((is.matrix(x) || is.data.frame(x)) &&
            isSymmetric(unname(as.matrix(x)))) {
            x <- as.dist(x)
            attr(x, "method") <- "user supplied square matrix"
        }
        else
            x <- vegdist(x, method = distance)
    }
    if (any(x < -sqrt(.Machine$double.eps)))
        warning("some dissimilarities are negative - is this intentional?")
    sol <- c(call = match.call())
    grouping <- as.factor(grouping)
    ## check that dims match
    if (length(grouping) != attr(x, "Size"))
        stop(
            gettextf("dissimilarities have %d observations, but grouping has %d",
                     attr(x, "Size"), length(grouping)))
    if (length(levels(grouping)) < 2)
        stop("there should be more than one class level")
    matched <- function(irow, icol, grouping) {
        grouping[irow] == grouping[icol]
    }
    x.rank <- rank(x)
    N <- attr(x, "Size")
    div <- length(x)/2
    irow <- as.vector(as.dist(row(matrix(nrow = N, ncol = N))))
    icol <- as.vector(as.dist(col(matrix(nrow = N, ncol = N))))
    within <- matched(irow, icol, grouping)
    ## check that there is replication
    if (!any(within))
        stop("there should be replicates within groups")
    aver <- tapply(x.rank, within, mean)
    statistic <- -diff(aver)/div
    cl.vec <- rep("Between", length(x))
    take <- as.numeric(irow[within])
    cl.vec[within] <- levels(grouping)[grouping[take]]
    cl.vec <- factor(cl.vec, levels = c("Between", levels(grouping)))
    ptest <- function(take, ...) {
        cl.perm <- grouping[take]
        tmp.within <- matched(irow, icol, cl.perm)
        tmp.ave <- tapply(x.rank, tmp.within, mean)
        -diff(tmp.ave)/div
    }
    permat <- getPermuteMatrix(permutations, N, strata = strata)
    if (ncol(permat) != N)
        stop(gettextf("'permutations' have %d columns, but data have %d rows",
                      ncol(permat), N))
    permutations <- nrow(permat)

    if (permutations) {
        ## Parallel processing
        if (is.null(parallel))
            parallel <- 1
        hasClus <- inherits(parallel, "cluster")
        if (hasClus || parallel > 1) {
            if(.Platform$OS.type == "unix" && !hasClus) {
                perm <- unlist(mclapply(1:permutations,
                                                  function(i, ...)
                                                  ptest(permat[i,]),
                                                  mc.cores = parallel))
            } else {
                if (!hasClus) {
                    parallel <- makeCluster(parallel)
                }
                perm <- parRapply(parallel, permat, ptest)
                if (!hasClus)
                    stopCluster(parallel)
            }
        } else {
            perm <- sapply(1:permutations, function(i) ptest(permat[i,]))
        }
        p.val <- (1 + sum(perm >= statistic - EPS))/(1 + permutations)
    } else { # no permutations
        p.val <- perm <- NA
    }
    sol$signif <- p.val
    sol$perm <- perm
    sol$permutations <- permutations
    sol$statistic <- as.numeric(statistic)
    sol$class.vec <- cl.vec
    sol$dis.rank <- x.rank
    sol$dissimilarity <- attr(x, "method")
    sol$control <- attr(permat, "control")
    class(sol) <- "anosim"
    sol
}

`text.orditkplot` <- function(x, cex = x$args$tcex, col = x$args$tcol,
                              font = attr(x$labels, "font"), ...) {
    if (is.null(font)) {
        font <- par("font")
    }
    text(x$labels, labels = rownames(x$labels), cex = cex, col = col,
         font = font, ...)
}

`plot.prc` <-
    function (x, species = TRUE, select, scaling = "symmetric", axis = 1,
              correlation = FALSE, const, type = "l", xlab, ylab, ylim,
              lty = 1:5, col = 1:6, pch, legpos, cex = 0.8, ...)
{
    ## save level names before getting the summary
    levs <- x$terminfo$xlev[[2]]
    x <- summary(x, axis = axis, scaling = scaling, const,
                 correlation = correlation)
    oldpar <- par(no.readonly = TRUE)
    on.exit(par(oldpar))
    b <- t(coef(x))
    xax <- rownames(b)
    if (missing(xlab))
        xlab <- x$names[1]
    if (missing(ylab))
        ylab <- "Effect"
    if (!missing(select))
        x$sp <- .checkSelect(select, x$sp)
    if (missing(ylim))
        ylim <- if (species)
            range(b, x$sp, na.rm = TRUE)
        else range(b, na.rm = TRUE)
    if (species) {
        op <- par("mai")
        mrg <- max(strwidth(names(x$sp), cex = cex, units = "in")) +
            strwidth("mmm", cex = cex, units = "in")
        par(mai = c(op[1:3], max(op[4], mrg)))
    }
    if (missing(pch))
        pch <- as.character(1:nrow(b))
    matplot(xax, b, type = type, xlab = xlab, ylab = ylab, ylim = ylim,
            cex = cex, lty = lty, col = col, pch = pch, ...)
    abline(h = 0, col = "gray")
    if (species) {
        linestack(x$sp, at = par("usr")[2], add = TRUE, hoff = 1,
                  cex = cex, ...)
        rug(x$sp, side = 4)
    }
    if (missing(legpos)) {
        holes <- abs(par("usr")[3:4] - range(b, na.rm = TRUE))
        if (holes[1] > holes[2])
            legpos <- "bottomleft"
        else legpos <- "topleft"
    }
    if (!is.na(legpos)) {
        nl <- length(levs)
        pp <- type %in% c("b", "p")
        pl <- type %in% c("b", "l")
        if (length(lty) == 1)
            lty <- rep(lty, nl-1)
        legend(legpos, legend = levs, col = c("gray", col),
               lty = if (pl) lty[c(1,1:(nl-1))],
               pch = if (pp) pch, cex = cex, title = x$names[2])
    }
    invisible()
}

print.simmat <- function(x, ...) {
    isSeq <- ifelse(attr(x, "isSeq"), "sequential", "non-sequential")
    if (attr(x, "binary")) {
        kind <- "binary"
    } else {
        kind <- ifelse(attr(x, "mode") == "integer", "count", "abundance")
    }
    d <- dim(x)
    cat("An object of class", dQuote(class(x)[1L]), "\n")
    cat(sQuote(attr(x, "method")), " method (", 
        kind, ", ", isSeq, ")\n", sep="")
    cat(d[1L], "x", d[2L], "matrix\n")
    cat("Number of permuted matrices =", d[3L], "\n")
    if (attr(x, "isSeq")) {
        chainInfo <- ""
        if (!is.null(attr(x, "chains")) && attr(x, "chains") > 1L)
            chainInfo <- paste0(" (", attr(x, "chains"), " chains)")
        cat("Start = ", attr(x, "start"), ", End = ", attr(x, "end"), 
            ", Thin = ", attr(x, "thin"), chainInfo, "\n\n", sep="") 
        } else cat("\n")
    invisible(x)
}

`print.specaccum` <-
    function(x, ...)
{
    cat("Species Accumulation Curve\n")
    if (inherits(x, "fitspecaccum"))
        cat("Non-linear regression model:", x$SSmodel, "\n")
    cat("Accumulation method:", x$method)
    if (x$method == "random") {
        cat(", with ", ncol(x$perm), " permutations", sep="")
    }
    if (!is.null(x$weights))
        cat(", weighted")
    cat("\n")
    cat("Call:", deparse(x$call), "\n\n")
    mat <- rbind(Sites = x$sites, Individuals = x$individuals, Effort = x$effort,
                 Richness = x$richness, sd=x$sd)
    colnames(mat) <- rep("", ncol(mat))
    print(zapsmall(mat))
    invisible(x)
}

"humpfit" <-
    function (mass, spno, family = poisson, start)
{
    .Deprecated("natto::humpfit() from https://github.com/jarioksa/natto/")
    hump <- function(p, mass, spno, ...) {
        x <- ifelse(mass < p[1], mass/p[1], p[1] * p[1]/mass/mass)
        fv <- p[3] * log(1 + p[2] * x/p[3])
        n <- wt <- rep(1, length(x))
        dev <- sum(dev.resids(spno, fv, wt))
        aicfun(spno, n, fv, wt, dev)/2
    }
    fam <- family(link = "identity")
    aicfun <- fam$aic
    dev.resids <- fam$dev.resids
    if (missing(start))
        p <- c(mean(mass), 100, 10)
    else
        p <- start
    fit <- nlm(hump, p = p, mass = mass, spno = spno, hessian = TRUE)
    p <- fit$estimate
    names(p) <- c("hump", "scale", "alpha")
    x <- ifelse(mass < p[1], mass/p[1], p[1] * p[1]/mass/mass)
    fv <- p[3] * log(1 + p[2] * x/p[3])
    res <- dev.resids(spno, fv, rep(1, length(x)))
    dev <- sum(res)
    residuals <- spno - fv
    aic <- fit$minimum * 2 + 6
    rdf <- length(x) - 3
    out <- list(nlm = fit, family = fam, y = spno, x = mass,
                coefficients = p, fitted.values = fv, aic = aic, rank = 3,
                df.residual = rdf, deviance = dev, residuals = residuals,
                prior.weights = rep(1, length(x)))
    class(out) <- c("humpfit", "glm")
    out
}

### Reads condensed CEP and similar CANOCO formatted data

### This function was originally based on Fortran code to interpret
### the format and read the data, but Fortran I/O is no longer allowed
### in CRAN packages. The original Fortran version was made available
### in github package vegandevs/cepreaded. This function uses
### utils::read.fortran. For that function we must translate the
### Fortran format in form understood by read.fortran(). This may
### fail, and read.fortran() can also fail, in particular wih
### interpreting the length of decimal part in F format.

### The original Fortran (and cepreader) can also interpret open
### (ordinary) Fortran format and Canoco FREE format, but this
### function currently only reads condensed data.
`read.cep` <-
    function (file,  positive = TRUE)
{
    ## Read all file first, interpret contents later
    cep <- readLines(file)
    ## skip first line and get the format
    i <- 2
    fmt <- substr(cep[i], 1, 60) # CEP sets fmt in cols 1-60
    fmt <- toupper(fmt)
    fmt <- gsub(" ", "", fmt)
    ## get the number of data entries per record
    nrecord <- as.numeric(substr(cep[i], 61,80))
    if (is.na(nrecord)) {
        i <- i+1
        nrecord <- as.numeric(cep[i])
    }
    ## check format: there should be to I-elements (site id, species
    ## id), and there should be two opening "("
    fmt1 <- strsplit(fmt, NULL)[[1]]
    if (sum(fmt1 == "I") != 2 || (nrecord > 1 && sum(fmt1 == "(") != 2))
        stop(gettextf("format %s does not look correct for condensed data",
                      fmt))
    ## process format: basically the format should have elements for
    ## (INT, n(INT, REAL)). read.fortran() does not understand
    ## multiplier 'n', but we need to rep((INT,REAL), n) in the
    ## fmt vector.
    fmt <- gsub(paste0(nrecord, "\\("), ";", fmt) # separate with ;
    fmt <- gsub("\\(","",fmt)
    fmt <- gsub("\\)","",fmt)
    ## number of decimals: there should be one and only one Fa.b
    ## format, and we need 'b'
    ndec <- as.numeric(strsplit(fmt, "\\.")[[1]][2])
    ## now split format for plotid and nrecord couplets
    fmt <- strsplit(fmt, ";")[[1]]
    fmt <- c(strsplit(fmt[1], ",")[[1]], rep(strsplit(fmt[2], ",")[[1]],
                                             nrecord))
    if (any(is.na(fmt)))
        fmt <- fmt[!is.na(fmt)]
    ## vectors to store results (with safe size)
    nlines <- length(cep)-i
    siteid <- numeric(nlines * nrecord)
    specid <- numeric(nlines * nrecord)
    abund <- numeric(nlines * nrecord)
    ids <- seq(2, by=2, len=nrecord)
    id <- 0
    ## read until there an empty siteid
    repeat {
        i <- i+1
        x <- drop(as.matrix(read.fortran(textConnection(cep[i]), fmt)))
        if(is.na(x[1]) || x[1] <= 0) break
        for(j in ids) {
            if(!is.na(x[j])) {
                id <- id+1
                siteid[id] <- x[1]
                specid[id] <- x[j]
                abund[id] <- x[j+1]
            }
            else
                break
        }
    }
    ## check there are no duplicate entries: only last one would be
    ## used, and this causes an error (and this has happened in
    ## literature)
    if (any(dups <- duplicated(cbind(siteid, specid))[seq_len(id)]))
        stop("you have duplicated data entries: ",
             paste(siteid[seq_len(id)][dups], specid[seq_len(id)][dups],
                   collapse = ", "))
    ## max identifiers
    nsp <- max(specid)
    nst <- max(siteid)
    ## read dimnames
    i <- i+1
    nomina <- read.fwf(textConnection(cep[i:length(cep)]), rep(8, 10),
                       as.is=TRUE)
    nomina <- gsub(" ", "", as.vector(t(nomina)))
    spnam <- make.names(nomina[seq_len(nsp)], unique = TRUE)
    nst0 <- ceiling(nsp/10) * 10
    stnam <- make.names(nomina[seq_len(nst) + nst0], unique = TRUE)
    ## utils::read.fortran divides with 10^ndec of F format even when
    ## there is an explicit decimal point: undo if this seems to have
    ## happened
    if (ndec > 0 && min(abund[1:id]) <= 10^(-ndec))
        abund <- abund * 10^ndec
    ## make as a matrix
    out <- matrix(0, nst, nsp)
    for(j in seq_len(id))
        out[siteid[j], specid[j]] <- abund[j]
    dimnames(out) <- list(stnam, spnam)
    if (positive) {
        rs <- rowSums(out)
        cs <- colSums(out)
        if (any(cs <= 0) || any(rs <= 0))
            out <- out[rs > 0, cs > 0]
    }
    as.data.frame(out)
}

"nestedn0" <-
function(comm)
{
    comm <- ifelse(comm > 0, 1, 0)
    R <- rowSums(comm)
    spmin <- apply(comm, 2, function(x) min((x*R)[x > 0]))
    n0 <- spmin
    for (i in 1:ncol(comm))
        n0[i] <- sum(comm[,i] == 0 & R > spmin[i])
    out <- list(spmin = spmin, n0 = n0, statistic = sum(n0))
    names(out$statistic) <- "N0"
    class(out) <- "nestedn0"
    out
}

`fitted.capscale` <-
    function(object, model = c("CCA", "CA", "pCCA", "Imaginary"),
             type = c("response", "working"), ...)
{
    model <- match.arg(model)
    if (is.null(object[[model]]) && model != "Imaginary")
        stop(gettextf("component '%s' does not exist", model))
    type <- match.arg(type)
    ## Return scaled eigenvalues
    U <- switch(model,
                CCA = ordiYbar(object, "CCA"),
                CA = ordiYbar(object, "CA"),
                Imaginary = object$CA$imaginary.u.eig,
                pCCA = ordiYbar(object, "pCCA"))
    if (is.null(U))
        stop(gettextf("component '%s' does not exist", model))
    ## Distances or working scores U
    if (type == "response") {
        U <- dist(U)
        ## undo sqrt.dist -- sqrt.dist was applied first in capscale,
        ## so it must be last here
        if (object$sqrt.dist)
            U <- U^2
    }
    U
}

"points.metaMDS" <-
    function (x, display = c("sites", "species"),
              choices = c(1, 2), shrink = FALSE, select, ...)
{
    display <- match.arg(display)
    x <- scores(x, display = display, choices = choices, shrink = shrink)
    if (!missing(select))
        x <- .checkSelect(select, x)
    points(x, ...)
    invisible()
}

`plot.MOStest` <-
    function(x, which = c(1,2,3,6),  ...)
{

    show <- rep(FALSE, 8)
    show[which] <- TRUE
    if (show[1]) {
        X <- x$mod$model$x
        Y <- x$mod$y
        xx <- seq(min(X), max(X), len=101)
        pre <- predict(x$mod, newdata=list(x = xx), se=TRUE)
        g <- x$mod$family$linkinv
        fv <- g(pre$fit)
        hi <- g(pre$fit + 2*pre$se)
        lo <- g(pre$fit - 2*pre$se)
        plot(X, Y, ...)
        matlines(xx, cbind(fv, hi, lo), lty=c(1, 2, 2), lwd=c(2, 1, 1), col=1, ...)
    }
    if (show[2]) {
        ## Covariance ellipse for the coefficients
        s <- summary(x$mod)
        k <- coef(s)[2:3, 1:2]
        ## Fix level to 0.95 (should be changed to an argument?)
        level = 0.95
        if (family(x$mod)$family %in% c("poisson", "binomial"))
            scale <- sqrt(qchisq(level, 2))
        else
            scale <- sqrt(2 * qf(level, 2, s$df[2])) 
        ci <- veganCovEllipse(s$cov.scaled[2:3, 2:3], k[,1], scale)
        plot(ci, type="l", lwd=2, xlim=range(ci[,1],0), ylim=range(ci[,2],0), ...)
        abline(h=0, lty=2, ...)
        par <- x$hump[c("min", "max")]
        par[par==0] <- sqrt(.Machine$double.eps)
        abline(0, -1/2/par[1], ...)
        abline(0, -1/2/par[2], ...)
        mul <- qnorm(1 - (1 - level)/2)
        segments(k[1,1] - k[1,2]*mul, k[2,1], k[1,1]+k[1,2]*mul, k[2,1], lty=3)
        segments(k[1,1], k[2,1]-k[2,2]*mul, k[1,1], k[2,1]+k[2,2]*mul, lty=3)
    }
    if (any(show[-c(1,2)])) {
        still <- which(show[-c(1,2)])
        plot(x$mod, which = still, ...)
    }
    invisible()
}

      Subroutine Orderdata(mat, n, k, rowscore)
C23456789012345678901234567890123456789012345678901234567890123456789012
C d = a distance matrix
C n = number of objects (rows of the data files)
C    
C Based on Pierre Legendre Fortran code and Table 9.5 and 9.10
C We compute the principal coordinate of the first axis only.
C Set the precision level for eigenvalue estimation
      Integer mat(n,k)
      double precision sumrow(n), sumtot
      double precision rowscore(n),colscore(n),toler,epsilon
      epsilon=0.000001
      toler=  0.000001
      if(n.gt.1000) then
         epsilon=0.00001
         toler=  0.00001
      endif
      if(n.gt.10000) then
         epsilon=0.0001
         toler=  0.0001
      endif
C Centre the distance matrix (Gower's method)
      call Centre(mat, n, k, sumrow, sumtot)
      call TWWS(mat,n,k,sumrow,sumtot,rowscore,colscore,toler,epsilon)
      end

      Subroutine Centre(mat, n, k, sumrow, sumtot)
      Integer mat(n,k)
      double precision d
      double precision sumrow(n),sumtot
      do i=1,n
         sumrow(i)=0.0
         enddo
      do i=1,(n-1)
         do j=(i+1),n
            call SM(mat, n, k, i, j, d)
            d = -0.5*d**2
            sumrow(i)=sumrow(i) + d
            sumrow(j)=sumrow(j) + d
            enddo
         enddo
C     
      fln=1.0/float(n)
      sumtot=0.
      do i=1,n
         sumtot=sumtot+sumrow(i)
         sumrow(i)=sumrow(i)*fln
         enddo
      sumtot=sumtot/float(n**2)
C      do 16 i=1,n
C      do 16 j=1,n
C   16 d(i,j)=d(i,j)-sumrow(i)-sumrow(j)+sumtot
      end

      Subroutine SM(mat, n, k, i, j, d)
C Compute a simple matching coefficient from a table of K-means results (integers).
C The 'n' rows are the objects; the 'k' columns are the partitions.
      Integer mat(n,k)
      double precision d
C
      a = 0.0
      do kk=1,k
         if(mat(i,kk).eq.mat(j,kk)) a=a+1
         enddo
      d = 1.0 - (a/float(k))
      end

      Subroutine TWWS(mat,n,k,sumrow,sumtot,rowscore,colscore,
     +                toler,epsilon)
      Integer n, niter
      Integer mat(n,k)
      double precision sumrow(n),sumtot,d
      double precision rowscore(n),colscore(n),epsilon,oldS,newS,toler,
     +       oldrowsc(n)
      niter=1000
C      Step 2: Take the column order as arbitrary initial site scores
      do i=1,n
         rowscore(i)=dble(i)
      end do
      oldS=0.
C      Iterations starting: this is a LONG loop
      do 20 it=1,niter
C      Step 3: Calculate new variable scores (equation 5.8, p. 119)
      do i=1,n
       colscore(i)=rowscore(i)
      end do
C      Step 4: Calculate new site scores (equation 5.9, p. 122)
      do i=1,n
         rowscore(i)=0.
         do j=1,n
            call SM(mat, n, k, i, j, d)
            d = -0.5*d**2
            d = d-sumrow(i)-sumrow(j)+sumtot
            rowscore(i)=rowscore(i)+d*colscore(j)
         end do
      end do
C      Step 6: Normalize the site scores
      call NormTWWS(rowscore,n,newS)
      if(newS.lt.epsilon) then
C         write(*,103) 0
         goto 52
      endif
C When convergence has been attained, check sign of eigenvalue.
C If ALL rowscores have changed sign during the last iteration, 
C this is a negative eigenvalue.
C      write(*,*) 'oldS-newS', oldS-newS
      if(dabs(oldS-newS).le.toler) then
C        write(*,105) toler, it 
        goto 22
        endif
      do i=1,n
         oldrowsc(i)=rowscore(i)
      end do
      oldS=newS
   20 continue
C      End of iterations for estimating eigenvalues and eigenvectors
C      write(*,101) k
   22 continue
C      Save Eigenvalues, % variance, cumulative % variance
C      write(*,*) 'Eigenvalue =', newS
C      End of main loop on axes
   52 continue
C
C      Normalize the principal coordinates to variance = eigenvalue
      do i=1,n
         rowscore(i)=rowscore(i)*dsqrt(newS)
      end do
C      write(*,*) rowscore   
C  101 format(' Convergence not reached for axis:',i3/
C     +       ' Increase NITER or lower TOLER')
C  102 format(' N. iterations to reach convergence for axis',i3,' =',i4)
C  103 format(' There are',i4,' eigenvalues different from 0')
C  104 format(' Eigenvector',i3,' is complex [multiply values*Sqrt(-1)]')
C  105 format(" Tolerance is: ", F12.8, "  NIter is: ", i4)

C  150 format(1x,1hR,2i3,6f10.5)
C  151 format(1x,1hC,2i3,10x,6f10.5)
      end


      Subroutine NormTWWS(rowscore,n,newS)
      Integer n
      double precision rowscore(n),s2,newS
C      Normalization for two-way weighted summation algorithm for PCA
C      (ter Braak 1987: 123)
C      On output, vector 'rowscore' has length = 1.
C      S = eigenvalue*(p-1) = contraction of vector rowscore in final
C      iteration
C
      s2=0.0
      do i=1,n
         s2=s2+rowscore(i)**2
      end do
      newS=dsqrt(s2)
      do i=1,n
         rowscore(i)=rowscore(i)/newS
      end do
      return
      end

`print.capscale` <-
    function(x, ...)
{
    NextMethod("print", x, ...)
    if (!is.null(x$metaMDSdist))
        cat("metaMDSdist transformed data:", x$metaMDSdist, "\n\n")
    if (!is.null(x$ac))
        cat("Constant added to distances:", x$ac, "\n\n")
}

`orditorp` <-
    function (x, display, labels, choices = c(1, 2), priority, select,
              cex = 0.7, pcex, col = par("col"),
              pcol, pch = par("pch"), air = 1, ...)
{
    if (missing(pcex))
        pcex <- cex
    if (missing(pcol))
        pcol <- col
    x <- scores(x, display = display, choices = choices, ...)
    kk <- complete.cases(x)
    if (missing(labels))
        labels <- rownames(x)
    if (missing(priority))
        priority <- rowSums((scale(x)^2))
    if (!missing(select)) {
        x <- .checkSelect(select, x)
        labels <- .checkSelect(select, labels)
        priority <- .checkSelect(select, priority)
        kk <- .checkSelect(select, kk)
    }
    ## remove NA scores
    x <- x[kk,]
    priority <- priority[kk]
    labels <- labels[kk]
    w <- abs(strwidth(labels, cex = cex))/2 * air
    h <- abs(strheight(labels, cex = cex))/2 * air
    xx <- cbind(x[, 1] - w, x[, 1] + w, x[, 2] - h, x[, 2] +
                h)
    is.na(priority) <- w == 0
    ord <- rev(order(priority, na.last = FALSE))
    xx <- xx[ord, ]
    x <- x[ord, ]
    labels <- labels[ord]
    tt <- logical(nrow(xx))
    tt[1] <- TRUE
    for (i in 2:(nrow(xx) - sum(is.na(priority)))) {
        j <- 1:(i - 1)
        j <- j[tt[j]]
        tt[i] <- all(xx[i, 1] > xx[j, 2] | xx[j, 1] > xx[i, 2] |
                     xx[i, 3] > xx[j, 4] | xx[j, 3] > xx[i, 4])
    }
    if (sum(!tt)) {
        if (length(pch) > 1)
            pch <- (pch[ord])[!tt]
        if (length(pcex) > 1)
            pcex <- (pcex[ord])[!tt]
        if (length(pcol) > 1)
            pcol <- (pcol[ord])[!tt]
        ordiArgAbsorber(x[!tt, , drop = FALSE], pch = pch, cex = pcex,
                        col = pcol, FUN = points, ...)
    }
    if (length(cex) > 1)
        cex <- (cex[ord])[tt]
    if (length(col) > 1)
        col <- (col[ord])[tt]
    ordiArgAbsorber(x[tt, , drop = FALSE], labels[tt], cex = cex, col = col,
                    FUN = text, ...)
    names(tt) <- labels
    invisible(tt[order(ord)])
}

`bioenv.formula` <-
    function (formula, data, ...)
{
    if (missing(data))
        data <- environment(formula)
    fla <- formula
    comm <- formula[[2]]
    comm <- eval(comm, environment(formula), parent.frame())
    formula[[2]] <- NULL
    env <- model.frame(formula, data, na.action = NULL)
    out <- bioenv(comm, env, ...)
    out$formula <- fla
    out$call <- match.call()
    out$call[[1]] <- as.name("bioenv")
    out
}

"radfit.default" <-
    function (x, ...) 
{
    x <- as.rad(x)
    NU <- rad.null(x, ...)
    PE <- rad.preempt(x, ...)
    ##BS <- rad.brokenstick(x, ...)
    LN <- rad.lognormal(x, ...)
    ZP <- rad.zipf(x, ...)
    ZM <- rad.zipfbrot(x, ...)
    out <- list(y = x, family = PE$family)
    models <- list(Null = NU, Preemption = PE, Lognormal = LN,
                   Zipf = ZP, Mandelbrot = ZM)
    out$models <- models
    class(out) <- "radfit"
    out
}

`bstick.decorana` <-
    function(n, ...)
{
    tot.chi <- n$totchi
    ## assume full rank input
    n.comp <- min(nrow(n$rproj), nrow(n$cproj)) - 1
    res <- bstick.default(n.comp, tot.chi, ...)
    ## only four axes in decorana
    res <- res[1:4]
    names(res) <- names(n$evals)
    res
}

`capscale` <-
    function (formula, data, distance = "euclidean", sqrt.dist = FALSE,
              comm = NULL, add = FALSE, dfun = vegdist,
              metaMDSdist = FALSE, na.action = na.fail, subset = NULL, ...)
{
    EPS <- sqrt(.Machine$double.eps)
    if (!inherits(formula, "formula"))
        stop("needs a model formula")
    if (missing(data)) {
        data <- parent.frame()
    }
    else {
        data <- eval(match.call()$data, environment(formula),
                     enclos = .GlobalEnv)
    }
    formula <- formula(terms(formula, data = data))
    ## The following line was eval'ed in environment(formula), but
    ## that made update() fail. Rethink the line if capscale() fails
    ## mysteriously at this point.
    X <- eval(formula[[2]], envir=environment(formula),
              enclos = globalenv())
    ## see if user supplied dissimilarities as a matrix
    if ((is.matrix(X) || is.data.frame(X)) &&
        isSymmetric(unname(as.matrix(X))))
        X <- as.dist(X)
    if (!inherits(X, "dist")) {
        comm <- X
        vdata <- as.character(formula[[2]])
        dfun <- match.fun(dfun)
        if (metaMDSdist) {
            commname <- as.character(formula[[2]])
            X <- metaMDSdist(comm, distance = distance, zerodist = "ignore",
                             commname = commname, distfun = dfun, ...)
            commname <- attr(X, "commname")
            comm <- eval.parent(parse(text=commname))
        } else {
            X <- dfun(X, distance)
        }
    } else { # vdata name
        if (missing(comm))
            vdata <- NULL
        else
            vdata <- deparse(substitute(comm))
    }
    inertia <- attr(X, "method")
    if (is.null(inertia))
        inertia <- "unknown"
    inertia <- paste(toupper(substr(inertia, 1, 1)),
                     substring(inertia,  2), sep = "")
    inertia <- paste(inertia, "distance")
    if (!sqrt.dist)
        inertia <- paste("squared", inertia)
    ## postpone info on euclidification till we have done so

    ## evaluate formula: ordiParseFormula will return dissimilarities
    ## as a symmetric square matrix (except that some rows may be
    ## deleted due to missing values)
    d <- ordiParseFormula(formula,
                          data,
                          na.action = na.action,
                          subset = substitute(subset),
                          X = X)
    ## ordiParseFormula subsets rows of dissimilarities: do the same
    ## for columns ('comm' is handled later). ordiParseFormula
    ## returned the original data, but we use instead the potentially
    ## changed X and discard d$X.
    if (!is.null(d$subset)) {
        X <- as.matrix(X)[d$subset, d$subset, drop = FALSE]
    }
    ## Delete columns if rows were deleted due to missing values
    if (!is.null(d$na.action)) {
        X <- as.matrix(X)[-d$na.action, -d$na.action, drop = FALSE]
    }
    X <- as.dist(X)
    k <- attr(X, "Size") - 1
    if (sqrt.dist)
        X <- sqrt(X)
    if (max(X) >= 4 + .Machine$double.eps) {
        inertia <- paste("mean", inertia)
        adjust <- sqrt(k)
        X <- X/adjust
    }
    else {
        adjust <- 1
    }
    nm <- attr(X, "Labels")
    ## wcmdscale, optionally with additive adjustment
    X <- wcmdscale(X, x.ret = TRUE, add = add)
    ## this may have been euclidified: update inertia
    if (!is.na(X$ac) && X$ac > sqrt(.Machine$double.eps))
        inertia <- paste(paste0(toupper(substring(X$add, 1, 1)),
                                substring(X$add, 2)),
                         "adjusted", inertia)
    if (is.null(rownames(X$points)))
        rownames(X$points) <- nm

    sol <- ordConstrained(X$points, d$Y, d$Z, method = "capscale")

    ## update for negative eigenvalues
    poseig <- length(sol$CA$eig)
    if (any(X$eig < 0)) {
        negax <- X$eig[X$eig < 0]
        sol$CA$imaginary.chi <- sum(negax)
        sol$tot.chi <- sol$tot.chi + sol$CA$imaginary.chi
        sol$CA$imaginary.rank <- length(negax)
        sol$CA$imaginary.u.eig <- X$negaxes
    }
    if (!is.null(comm)) {
        sol$vdata <- vdata
        comm <- scale(comm, center = TRUE, scale = FALSE)
        sol$colsum <- apply(comm, 2, sd)
        ## take a 'subset' of the community after scale()
        if (!is.null(d$subset))
            comm <- comm[d$subset, , drop = FALSE]
        ## NA action after 'subset'
        if (!is.null(d$na.action))
            comm <- comm[-d$na.action, , drop = FALSE]
        if (!is.null(sol$pCCA) && sol$pCCA$rank > 0)
            comm <- qr.resid(sol$pCCA$QR, comm)
        if (!is.null(sol$CCA) && sol$CCA$rank > 0) {
            v.eig <- t(comm) %*% sol$CCA$u/sqrt(k)
            sol$CCA$v <- decostand(v.eig, "normalize", MARGIN = 2)
            comm <- qr.resid(sol$CCA$QR, comm)
        }
        if (!is.null(sol$CA) && sol$CA$rank > 0) {
            v.eig <- t(comm) %*% sol$CA$u/sqrt(k)
            sol$CA$v <- decostand(v.eig, "normalize", MARGIN = 2)
        }
    } else {
        ## input data were dissimilarities, and no 'comm' defined:
        ## species scores make no sense and are made NA
        sol$CA$v[] <- NA
        if (!is.null(sol$CCA))
            sol$CCA$v[] <- NA
        sol$colsum <- NA
    }
    if (!is.null(sol$CCA) && sol$CCA$rank > 0)
        sol$CCA$centroids <- centroids.cca(sol$CCA$wa, d$modelframe)
    if (!is.null(sol$CCA$alias))
        sol$CCA$centroids <- unique(sol$CCA$centroids)
    if (!is.null(sol$CCA$centroids)) {
        rs <- rowSums(sol$CCA$centroids^2)
        sol$CCA$centroids <- sol$CCA$centroids[rs > 1e-04, ,
                                               drop = FALSE]
        if (nrow(sol$CCA$centroids) == 0)
            sol$CCA$centroids <- NULL
    }
    sol$call <- match.call()
    sol$terms <- terms(formula, "Condition", data = data)
    sol$terminfo <- ordiTerminfo(d, data)
    sol$call$formula <- formula(d$terms, width.cutoff = 500)
    sol$call$formula[[2]] <- formula[[2]]
    sol$sqrt.dist <- sqrt.dist
    if (!is.na(X$ac) && X$ac > 0) {
        sol$ac <- X$ac
        sol$add <- X$add
    }
    sol$adjust <- adjust
    sol$inertia <- inertia
    if (metaMDSdist)
        sol$metaMDSdist <- commname
    sol$subset <- d$subset
    sol$na.action <- d$na.action
    class(sol) <- c("capscale", "rda", "cca")
    if (!is.null(sol$na.action))
        sol <- ordiNAexclude(sol, d$excluded)
    sol
}

print.nullmodel <- function(x, ...) {
    isSeq <- ifelse(x$commsim$isSeq, "sequential", "non-sequential")
    if (x$commsim$binary)
        kind <- "binary"
    else
        kind <- ifelse(x$commsim$mode == "integer", "count", "abundance")
    cat("An object of class", dQuote(class(x)[1L]), "\n")
    cat(sQuote(x$commsim$method), " method (", 
        kind, ", ", isSeq, ")\n", sep="")
    cat(x$nrow, "x", x$ncol, "matrix\n")
    if (x$commsim$isSeq)
        cat("Iterations =", x$iter, "\n\n") else cat("\n")
    invisible(x)
}

`print.protest` <-
  function(x, digits = max(3, getOption("digits") - 3), ...)
{
  cat("\nCall:\n")
  cat(deparse(x$call), "\n\n")
  cat("Procrustes Sum of Squares (m12 squared):        ")
  cat(formatC(x$ss, digits=digits), "\n")
  cat("Correlation in a symmetric Procrustes rotation: ")
  cat(formatC(x$t0, digits = digits), "\n")
  cat("Significance:  ")
  cat(format.pval(x$signif),"\n\n")
  cat(howHead(x$control))
  cat("\n")
  invisible(x)
}

`anova.ccalist` <-
    function(object, permutations, model, parallel)
{
    EPS <- sqrt(.Machine$double.eps)
    ## 'object' *must* be a list of cca objects, and 'permutations'
    ## *must* be a permutation matrix -- we assume that calling
    ## function takes care of this, and this function is not directly
    ## called by users.
    nmodels <- length(object)
    ## check that input is valid
    ## 1. All models must be fitted with the same method
    method <- sapply(object, function(z) z$method)
    if (!all(method == method[1]))
        stop("same ordination method must be used in all models")
    else
        method <- method[1]
    ## 2. All models must be fitted with formula interface
    if (any(sapply(object, function(x) is.null(x$terms))))
        stop("all models must be fitted with formula interface")
    ## 3. Same response
    resp <- sapply(object, function(z) deparse(formula(z)[[2]]))
    if (!all(resp == resp[1]))
        stop("response must be same in all models")
    ## 4. Same no. of observations
    N <- sapply(object, nobs)
    if (!all(N == N[1]))
        stop("number of observations must be same in all models")
    else
        N <- N[1]
    ## 5. Terms must be nested
    trms <- lapply(object, function(z) labels(terms(z)))
    o  <- order(sapply(trms, length))
    for (i in 2:nmodels)
        if (!all(trms[[o[i-1]]] %in% trms[[o[i]]]))
            stop("models must be nested")

    ## Check permutation matrix
    nperm <- nrow(permutations)
    ## check
    if (ncol(permutations) != N)
        stop(gettextf("'permutations' have %d columns, but data have %d rows",
                      ncol(nperm), N))
    ## All models are evaluated in permutest.cca with identical
    ## permutations so that the differences of single permutations can
    ## be used to assess the significance of differences of fitted
    ## models. This strictly requires nested models (not checked
    ## here): all terms of the smaller model must be included in the
    ## larger model.
    mods <- lapply(object, function(z)
                   permutest.cca(z, permutations = permutations,
                                 model = model, parallel = parallel))
    dfs <- sapply(mods, function(z) z$df)
    dev <- sapply(mods, function(z) z$chi)
    resdf <- dfs[2,]
    df <- -diff(resdf)
    resdev <- dev[2,]
    changedev <- -diff(resdev)
    big <- which.min(resdf)
    scale <- resdev[big]/resdf[big]
    fval <- changedev/df/scale
    ## Collect permutation results: denominator of F varies in each
    ## permutation.
    pscale <- mods[[big]]$den/resdf[big]
    ## Numerator of F
    pfvals <- sapply(mods, function(z) z$num)
    if (is.list(pfvals))
        pfvals <- do.call(cbind, pfvals)
    pfvals <- apply(pfvals, 1, diff)
    ## dropped to vector?
    if (!is.matrix(pfvals))
        pfvals <- matrix(pfvals, nrow = 1, ncol = nperm)
    pfvals <- sweep(pfvals, 1, df, "/")
    pfvals <- sweep(pfvals, 2, pscale, "/")
    pval <- rowSums(sweep(pfvals, 1, fval - EPS, ">="))
    pval <- (pval + 1)/(nperm + 1)
    ## collect table
    table <- data.frame(resdf, resdev, c(NA, df),
                        c(NA,changedev), c(NA,fval), c(NA,pval))
    if (inherits(object, c("capscale", "dbrda")) && object$adjust == 1)
        varname <- "SumOfSqs"
    else if (inherits(object, "rda"))
        varname <- "Variance"
    else
        varname <- "ChiSquare"
    dimnames(table) <- list(1L:nmodels,
                            c("ResDf", paste0("Res", varname), "Df",
                              varname, "F", "Pr(>F)"))
    ## Collect header information
    formulae <- sapply(object,
                       function(z) deparse(formula(z), width.cutoff = 500))
    head <- paste0("Permutation tests for ", method, " under ",
                  mods[[big]]$model, " model\n",
                   howHead(attr(permutations, "control")))
    topnote <- paste("Model ", format(1L:nmodels), ": ", formulae,
                     sep = "", collapse = "\n")
    structure(table, heading = c(head,topnote),
              F.perm = t(pfvals),
              class = c("anova.cca", "anova", "data.frame"))
}

`kendall.global` <-
	function(Y, group, nperm=999, mult="holm")
{
### Function to test the overall significance of the Kendall coefficient of
### concordance for a single group or several groups of judges (e.g. species)
###
### copyleft - Guillaume Blanchet and Pierre Legendre, October 2008
################################################################################

    mult <- match.arg(mult, c("sidak", p.adjust.methods))

    ##CC# Make sure Y is a matrix and find number of rows and columns of Y
    Y <- as.matrix(Y)
    n <- nrow(Y)
    p <- ncol(Y)
    if(p < 2) stop("there is only one variable in the data matrix")

    ##CC# Transform the species abundances to ranks, by column
    R <- apply(Y,2,rank)

    if(missing(group)) group <- rep(1,p)
    if(length(group) != p){
        stop("the number of species in the vector differs from the total number of species")
    }
    group <- as.factor(group)
    gr.lev <- levels(group)
    ngr <- nlevels(group)

    gr <- as.list(1:ngr)

    n.per.gr <- vector(length=ngr)
    for(i in 1:ngr){
        gr[[i]] <- which(group==gr.lev[i])
        n.per.gr[i] <- length(gr[[i]])
        ## Vector containing the number of species per group
    }

    ## Initialise the vectors of results
    W.gr <- vector("numeric",ngr)
    F.gr <- vector("numeric",ngr)
    prob.F.gr <- vector("numeric",ngr)
    Chi2.gr <- vector("numeric",ngr)
    prob.perm.gr <- vector("numeric",ngr)
    vec <- NA

    for(i in 1:ngr){
        p.i <- n.per.gr[i]
        if(p.i < 2) stop(gettextf("there is only one variable in group %d",gr.lev[i]))
        ##CC# Correction factors for tied ranks (eq. 3.3)
        t.ranks <- apply(R[,gr[[i]]], 2,
                         function(x) summary(as.factor(x), maxsum=n))
        T. <- sum(unlist(lapply(t.ranks, function(x) sum((x^3)-x))))

        ##CC# Compute the Sum of squares of the uncentred ranks (S) (eq. 3.1)
        S <- sum(rowSums(R[,gr[[i]]])^2)

        ##CC# Compute Kendall's W (eq. 3.2)
        W.gr[i] <- ((12*S)-(3*(p.i^2)*n*(n+1)^2))/(((p.i^2)*((n^3)-n))-(p.i*T.))

        ##C# Compute Fisher F statistic and associated probability
        F.gr[i] <- (p.i-1)*W.gr[i]/(1-W.gr[i])
        nu1 <- n-1-(2/p.i)
        nu2 <- nu1*(p.i-1)
        prob.F.gr[i] <- pf(F.gr[i], nu1, nu2, lower.tail=FALSE)

        ##CC# Calculate Friedman's Chi-square (eq. 3.4)
        Chi2.gr[i] <- p.i*(n-1)*W.gr[i]

        counter <- 1
        for(j in 1:nperm) {   # Each species is permuted independently
            R.perm <- apply(R[,gr[[i]]], 2, sample)
            S.perm <- sum(rowSums(R.perm)^2)
            W.perm <- ((12*S.perm)-(3*(p.i^2)*n*(n+1)^2))/(((p.i^2)*((n^3)-n))-(p.i*T.))
            Chi2.perm <- p.i*(n-1)*W.perm
            if(Chi2.perm >= Chi2.gr[i]) counter <- counter+1
        }
        prob.perm.gr[i] <- counter/(nperm+1)
    }
    ## Correction to P-values for multiple testing
    if(ngr > 1) {
        if(mult == "sidak") {
            perm.corr <- NA
            for(i in 1:ngr) perm.corr = c(perm.corr, (1-(1-prob.perm.gr[i])^ngr))
            perm.corr <- perm.corr[-1]
                                        #
            prob.F.corr <- NA
            for(i in 1:ngr) prob.F.corr = c(prob.F.corr, (1-(1-prob.F.gr[i])^ngr))
            prob.F.corr <- prob.F.corr[-1]
        } else {
            perm.corr <- p.adjust(prob.perm.gr, method=mult)
            prob.F.corr <- p.adjust(prob.F.gr, method=mult)
        }
    }
    ## Create a data frame containing the results
    if(ngr == 1) {
        table <- rbind(W.gr, F.gr, prob.F.gr, Chi2.gr, prob.perm.gr)
        colnames(table) <- colnames(table,do.NULL = FALSE, prefix = "Group.")
        rownames(table) <- c("W", "F", "Prob.F", "Chi2", "Prob.perm")
    }  else {
        table <- rbind(W.gr, F.gr, prob.F.gr, prob.F.corr, Chi2.gr, prob.perm.gr, perm.corr)
        colnames(table) <- colnames(table,do.NULL = FALSE, prefix = "Group.")
        rownames(table) <- c("W", "F", "Prob.F", "Corrected prob.F", "Chi2", "Prob.perm", "Corrected prob.perm")
    }
    if(ngr == 1) {
        out <- list(Concordance_analysis=table)
    }  else {
        out <- list(Concordance_analysis=table, Correction.type=mult)
    }
                                        #
    class(out) <- "kendall.global"
    out
}

"ordigrid" <-
    function (ord, levels, replicates, display = "sites", lty=c(1,1), col=c(1,1),
              lwd = c(1,1), ...) 
{
    pts <- scores(ord, display = display, ...)
    npoints <- nrow(pts)
    gr <- gl(levels, replicates, npoints)
    ordisegments(pts, groups = gr,  lty = lty[1], col = col[1],
                 lwd = lwd[1], ...)
    gr <- gl(replicates, 1, npoints)
    ordisegments(pts, groups = gr, lty = lty[2], col = col[2],
                 lwd = lwd[2], ...)
    invisible()
}

`msoplot` <-
    function (x, alpha = 0.05, explained = FALSE, ylim = NULL,
    legend = "topleft", ...)
{
    if (is.data.frame(x$vario)) {
        vario <- x$vario
        grain <- x$grain
        hasSig <- is.numeric(x$vario$CA.signif)
        z <- qnorm(alpha/2)
        if (is.numeric(vario$CA.signif)) {
            vario <- vario[, -ncol(vario)]
        }
        ymax <- max(vario[, -1:-3], na.rm = TRUE)
        b <- ncol(vario) - 3
        label <- c("", "", "", "Total variance", "Explained plus residual",
                   "Residual variance", "Explained variance", "Conditioned variance")
        ci.lab <- "C.I. for total variance"
        sign.lab <- if(hasSig) "Sign. autocorrelation" else NULL
        if (is.numeric(x$CCA$rank)) {
            if (!explained)
                b <- b - 1
            if (is.numeric(x$vario$se))
                b <- b - 1
            figmat <- cbind(vario$All + z * vario$se,
                            vario$All - z * vario$se,
                            vario$Sum,
                            vario[, 6:(b + 3)])
            matplot(vario$Dist, cbind(0,figmat), type = "n",
                    xlab = "Distance", ylab = "Variance",
                    ylim = ylim, ...)
            lines(vario$Dist, vario$All + z * vario$se, lty = 1, ...)
            lines(vario$Dist, vario$All - z * vario$se, lty = 1, ...)
            lines(vario$Dist, vario$Sum, type = "b", lty = 2,
                  pch = 3, ...)
            ## Legend
            legend(legend,
                   legend=c(label[c(2,3:b)+3], ci.lab, sign.lab),
                   lty=c(c(1,2,1,1,1)[2:b], 1, if(hasSig) NA),
                   pch=c(3, (6:(b+3))-6, NA, if(hasSig) 15)
                   )
            matlines(vario$Dist, figmat[,-c(1:3)], type = "b", lty = 1,
                     pch = 6:(b+3)-6, ...)
            text(x = c(vario$Dist), y = par("usr")[3], pos = 3,
                 label = c(vario$n), cex = 0.8, ...)
            abline(v = max(x$H/2), lty = 3, ...)
        }
        else {
            if (is.null(ylim))
                ylim <- c(0, ymax)
            plot(vario$Dist, vario$All, type = "b", lty = 1,
                 pch = 0, xlab = "Distance", ylab = "Variance",
                 ylim = ylim, ...)
            abline(h = x$tot.chi, lty = 5, ...)
            text(x = c(vario$Dist), y = par("usr")[3], pos = 3,
                 label = c(vario$n), cex = 0.8)
            abline(v = max(x$H)/2, lty = 3, ...)
            legend(legend,
                   legend=c("Total variance","Global variance estimate",
                     if(hasSig) "Sign. autocorrelation"),
                   lty=c(1,5, if(hasSig) NA),
                   pch = if(hasSig) c(NA,NA,15) else NULL)
        }
    }
    if (hasSig) {
        a <- c(1:nrow(x$vario))[x$vario$CA.signif <
                                     alpha]
        points(vario$Dist[a], x$vario$CA[a], pch = 15, ...)
        if (is.numeric(x$CCA$rank)) {
            inflation <- 1 - weighted.mean(x$vario$CA, x$vario$n)/
                weighted.mean(x$vario$CA[-a],
                              x$vario$n[-a])
            cat("Error variance of regression model underestimated by",
                round(inflation * 100, 1), "percent", "\n")
        }
    }
    invisible()
}

`bstick.prcomp` <-
    function(n, ...)
{
    if(!inherits(n, "prcomp"))
        stop("'n' not of class \"prcomp\"")
    tot.chi <- sum(n$sdev^2)
    n.comp <- length(n$sdev)
    res <- bstick.default(n.comp, tot.chi, ...)
    names(res) <- dimnames(n$rotation)[[2]]
    res
}

`vif.cca` <-
    function(object) 
{
    if (is.null(object$CCA))
        stop("can be used only with constrained ordination")
    Q <- object$CCA$QR
    out <- rep(NA, NCOL(Q$qr))
    names(out)[Q$pivot] <- colnames(Q$qr)
    rank <- Q$rank
    V <- chol2inv(Q$qr, size = rank)
    X <- qr.X(Q)[, Q$pivot[1:rank], drop=FALSE]
    Vi <- crossprod(X)
    v1 <- diag(V)
    v2 <- diag(Vi)
    out[Q$pivot[1:rank]] <- v1 * v2
    out
}

"plot.radfit.frame" <-
    function (x, order.by, BIC = FALSE, model, legend = TRUE, as.table = TRUE, 
              ...) 
{
    modnam <- names(x[[1]]$models)
    if (!missing(model)) 
        pick <- pmatch(model, modnam, nomatch = FALSE)
    else pick <- FALSE
    pickmod <- function(x, pick, BIC) {
        if (pick) 
            return(pick)
        else {
            k <- if (BIC) 
                log(length(x$y))
            else 2
            which.min(AIC(x, k))
        }
    }
    Nhm <- length(x)
    Abundance <- unlist(lapply(x, function(x) x$y))
    Rank <- unlist(lapply(x, function(x) if (length(x$y) > 0) seq_along(x$y) else NULL))
    Site <- unlist(lapply(x, function(x) length(x$y)))
    N <- Site
    sitenames <- names(Site)
    Site <- rep(names(Site), Site)
    if (missing(order.by)) 
        order.by <- 1:Nhm
    else order.by <- order(order.by)
    Site <- factor(Site, levels = sitenames[order.by])
    fit <- unlist(lapply(x, function(x)
                         as.matrix(fitted(x))[, pickmod(x, 
                                                        pick, BIC)]))
    take <- sapply(x, function(x) pickmod(x, pick, BIC))
    take <- rep(take, N)
    cols <- trellis.par.get("superpose.line")$col
    cols <- cols[seq_along(cols)]
    if (legend) {
        mykey <- list(text = list(text = modnam), lines = list(lty = 1, 
                                                  col = cols[seq_along(modnam)], lwd = 2), columns = 3)
    }
    else {
        mykey <- NULL
    }
    tics <- function(x = max(Abundance), z = min(Abundance)) {
        ii <- round(c(log10(z), log10(x)))
        x10 <- 10^(ii[1]:ii[2])
        if (length(x10) < 3) 
            x10 <- c(outer(c(1, 2, 5), x10))
        else if (length(x10) < 6) 
            x10 <- c(outer(c(1, 3), x10))
        x10[x10 <= x & x10 >= z]
    }
    out <- xyplot(Abundance ~ Rank | Site, subscripts = TRUE, 
                  as.table = as.table, key = mykey, scales = list(y = list(log = 10, 
                                                                  at = tics())), panel = function(x, y, subscripts) {
                                                                      panel.xyplot(x, y, ...)
                                                                      panel.xyplot(x, log10(fit[subscripts]), type = "l", 
                                                                                   col = cols[take[min(subscripts)]], lwd = 2, ...)
                                                                  }, ...)
    out
}

`print.betadisper` <- function(x, digits = max(3, getOption("digits") - 3),
                               neigen = 8, ...)
{
    ## limit number of eignvals to neigen
    eig <- eigenvals(x)
    nev <- length(eig)
    ax.lim <- min(nev, neigen)
    ##
    cat("\n")
    writeLines(strwrap("Homogeneity of multivariate dispersions\n",
                       prefix = "\t"))
    cat("\n")
    writeLines(strwrap(pasteCall(x$call)))
    cat(paste("\nNo. of Positive Eigenvalues:", sum(eig > 0)))
    cat(paste("\nNo. of Negative Eigenvalues:", sum(eig < 0)))
    cat("\n\n")
    type <- ifelse(isTRUE(all.equal(attr(x, "type"), "median")),
                   "median", "centroid")
    writeLines(strwrap(paste0("Average distance to ", type, ":\n")))
    print.default(format(tapply(x$distances, x$group, mean),
                         digits = digits), quote = FALSE)
    cat("\n")
    writeLines(strwrap("Eigenvalues for PCoA axes:"))
    if (nev > neigen) {
        writeLines(strwrap(paste0("(Showing ", neigen, " of ", nev,
                                  " eigenvalues)")))
    }
    print.default(format(eig[seq_len(ax.lim)], digits = digits), quote = FALSE)
    invisible(x)
}

`ordiarrows` <-
    function (ord, groups, levels, replicates, order.by, 
              display = "sites", col = 1, show.groups, startmark,
              label = FALSE, length = 0.1, ...)
{
    pts <- scores(ord, display = display, ...)
    npoints <- nrow(pts)
    if (missing(groups))
        groups <- gl(levels, replicates, npoints)
    if (!missing(order.by)) {
        if (length(order.by) != nrow(pts))
            stop(gettextf("the length of order.by (%d) does not match the number of points (%d)",
                 length(order.by), nrow(pts)))
        ord <- order(order.by)
        pts <- pts[ord,]
        groups <- groups[ord]
    }
    if (!missing(show.groups)) {
        take <- groups %in% show.groups
        pts <- pts[take, , drop = FALSE]
        groups <- groups[take]
    }
    out <- seq(along = groups)
    inds <- names(table(groups))
    if (is.factor(col))
        col <- as.numeric(col)
    col <- rep(col, length=length(inds))
    names(col) <- inds
    starts <- names <- NULL
    for (is in inds) {
        gr <- out[groups == is]
        if (length(gr) > 1) {
            X <- pts[gr, , drop = FALSE]
            X0 <- X[-nrow(X), , drop = FALSE]
            X1 <- X[-1, , drop = FALSE]
            nseg <- nrow(X0)
            if (!missing(startmark))
                points(X0[1,1], X0[1,2], pch=startmark, col = col[is], ...)
            if (label) {
                starts <- rbind(starts, X0[1,])
                names <- c(names, is)
            }
            if (nseg > 1)
                ordiArgAbsorber(X0[-nseg,1], X0[-nseg,2], X1[-nseg,1],
                                X1[-nseg,2], col = col[is],
                                FUN = segments, ...)
            ordiArgAbsorber(X0[nseg, 1], X0[nseg, 2], X1[nseg, 1],
                            X1[nseg, 2], col = col[is], length = length,
                            FUN = arrows, ...)
        }
    }
    if (label)
        ordiArgAbsorber(starts, labels = names, border = col, col = par("fg"),
                        FUN = ordilabel, ...)
    invisible()
}

`plot.taxondive` <-
function (x, ...) 
{
	plot(x$Species, x$Dplus, xlab="Number of Species", ylab=expression(Delta^"+"), ...) 
	i <- order(x$Species)
	abline(h=x$EDplus, ...)
	lines(x$Species[i], x$EDplus - 2*x$sd.Dplus[i], ...)
     lines(x$Species[i], x$EDplus + 2*x$sd.Dplus[i], ...)
}

`centroids.cca` <-
    function(x, mf, wt)
{
    if (is.null(mf) || is.null(x))
        return(NULL)
    facts <- sapply(mf, is.factor) | sapply(mf, is.character)
    if (!any(facts))
        return(NULL)
    mf <- mf[, facts, drop = FALSE]
    ## Explicitly exclude NA as a level
    mf <- droplevels(mf, exclude = NA)
    if (missing(wt))
        wt <- rep(1, nrow(mf))
    ind <- seq_len(nrow(mf))
    workhorse <- function(x, wt)
        colSums(x * wt) / sum(wt)
    ## As NA not a level, centroids only for non-NA levels of each factor
    tmp <- lapply(mf, function(fct)
                  tapply(ind, fct, function(i) workhorse(x[i,, drop=FALSE], wt[i])))
    tmp <- lapply(tmp, function(z) sapply(z, rbind))
    pnam <- labels(tmp)
    out <- NULL
    if (ncol(x) == 1) {
        nm <- unlist(sapply(pnam,
                            function(nm) paste(nm, names(tmp[[nm]]), sep="")),
                     use.names=FALSE)
        out <- matrix(unlist(tmp), nrow=1, dimnames = list(NULL, nm))
    } else {
        for (i in seq_along(tmp)) {
            colnames(tmp[[i]]) <- paste(pnam[i], colnames(tmp[[i]]),
                                        sep = "")
            out <- cbind(out, tmp[[i]])
        }
    }
    out <- t(out)
    colnames(out) <- colnames(x)
    out
}

"lines.humpfit" <-
    function(x, segments=101,  ...)
{
    mass <- x$x
    if (!is.null(segments) && segments > 0) {
        mass <- seq(min(mass), max(mass), length=segments)
        fv <- predict(x, newdata = mass)
    }
    else {
        i <- order(mass)
        fv <- fitted(x)
        mass <- mass[i]
        fv <- fv[i]
    }
    lines(mass, fv, ...)
    invisible()
}

### An internal function used in varpart(): Returns only the raw
### Rsquare and the rank of constraints in RDA.
`simpleRDA2` <-
    function (Y, X, SS.Y, ...)
{
    Q <- qr(X, tol=1e-6)
    Yfit.X <- qr.fitted(Q, Y)
    SS <- sum(Yfit.X^2)
    if (missing(SS.Y)) SS.Y <- sum(Y^2)
    Rsquare <- SS/SS.Y
    R2adj <- RsquareAdj(Rsquare, nrow(Y), Q$rank)
    list(Rsquare = Rsquare, RsquareAdj = R2adj, m = Q$rank)
}

### Analogous function, but the input must be Gower double-centred
### dissimilarities 'G = -GowerDblcen(as.matrix(dist(Y)^2))/2'. The
### math is based on McArdle & Anderson, Ecology 82: 290-297 (2001).
`simpleDBRDA` <-
    function(G, X, SS.G, ...)
{
    Q <- qr(X, tol=1e-6)
    Yfit.X <- qr.fitted(Q, G)
    SS <- sum(diag(Yfit.X))
    if (missing(SS.G)) SS.G <- sum(diag(G))
    Rsquare <- SS/SS.G
    R2adj <- RsquareAdj(Rsquare, nrow(G), Q$rank)
    list(Rsquare = Rsquare, RsquareAdj = R2adj, m = Q$rank)
}

### Analogous function for CCA. We initialize data with weighted
### double standaradization, and centre constraints X by row
### weights. The approximation of weighted R-square is found via
### permutations in permat (which must be given).

`simpleCCA` <-
    function(Y, X, SS.Y, permat, ...)
{
    Y <- initCA(Y)
    if(missing(SS.Y)) SS.Y <- sum(Y^2)
    w <- attr(Y, "RW")
    X <- .Call(do_wcentre, X, w)
    Q <- qr(X, tol=1e-6)
    Yfit.X <- qr.fitted(Q, Y)
    SS <- sum(Yfit.X^2)
    Rsquare <- SS/SS.Y
    ## permutation to estimate adjusted R2
    meanSS <- mean(sapply(seq_len(nrow(permat)),
                          function(i) sum(qr.fitted(Q, Y[permat[i,],])^2)))
    R2adj <- 1 - ((1 - Rsquare) / (1 - meanSS/SS.Y))
    list(Rsquare = Rsquare, RsquareAdj = R2adj, m = Q$rank)
}

`varpart3` <-
    function (Y, X1, X2, X3, chisquare, permat)
{
    collinwarn <- function(case, mm, m)
        warning(gettextf("collinearity detected in %s: mm = %d, m = %d",
                         case, mm, m), call. = FALSE)
    if (inherits(Y, "dist")) {
        Y <- GowerDblcen(as.matrix(Y^2), na.rm = FALSE)
        Y <- -Y/2
        SS.Y <- sum(diag(Y))
        simpleRDA2 <- match.fun(simpleDBRDA)
    } else {
        Y <- as.matrix(Y)
        if (chisquare) {
            SS.Y <- sum(initCA(Y)^2)
            simpleRDA2 <- match.fun(simpleCCA)
        } else {
            Y <- scale(Y, center = TRUE, scale = FALSE)
            SS.Y <- sum(Y * Y)
        }
    }
    X1 <- as.matrix(X1)
    X2 <- as.matrix(X2)
    X3 <- as.matrix(X3)
    n <- nrow(Y)
    n1 <- nrow(X1)
    n2 <- nrow(X2)
    n3 <- nrow(X3)
    p <- ncol(Y)
    mm1 <- ncol(X1)
    mm2 <- ncol(X2)
    mm3 <- ncol(X3)
    if (n1 != n)
        stop("Y and X1 do not have the same number of rows")
    if (n2 != n)
        stop("Y and X2 do not have the same number of rows")
    if (n3 != n)
        stop("Y and X3 do not have the same number of rows")
    X1 <- scale(X1, center = TRUE, scale = FALSE)
    X2 <- scale(X2, center = TRUE, scale = FALSE)
    X3 <- scale(X3, center = TRUE, scale = FALSE)
    dummy <- simpleRDA2(Y, X1, SS.Y, permat)
    adfg.ua <- dummy$Rsquare
    adfg <- dummy$RsquareAdj
    m1 <- dummy$m
    if (m1 != mm1)
        collinwarn("X1", mm1, m1)
    dummy <- simpleRDA2(Y, X2, SS.Y, permat)
    bdeg.ua <- dummy$Rsquare
    bdeg <- dummy$RsquareAdj
    m2 <- dummy$m
    if (m2 != mm2)
        collinwarn("X2", mm2, m2)
    dummy <- simpleRDA2(Y, X3, SS.Y, permat)
    cefg.ua <- dummy$Rsquare
    cefg <- dummy$RsquareAdj
    m3 <- dummy$m
    if (m3 != mm3)
        collinwarn("X3", mm3, m3)
    mm4 = mm1 + mm2
    dummy <- simpleRDA2(Y, cbind(X1, X2), SS.Y, permat)
    abdefg.ua <- dummy$Rsquare
    abdefg <- dummy$RsquareAdj
    m4 <- dummy$m
    if (m4 != mm4)
        collinwarn("cbind(X1,X2)", mm4, m4)
    mm5 = mm1 + mm3
    dummy <- simpleRDA2(Y, cbind(X1, X3), SS.Y, permat)
    acdefg.ua <- dummy$Rsquare
    acdefg <- dummy$RsquareAdj
    m5 <- dummy$m
    if (m5 != mm5)
        collinwarn("cbind(X1,X3)", mm5, m5)
    mm6 = mm2 + mm3
    dummy <- simpleRDA2(Y, cbind(X2, X3), SS.Y, permat)
    bcdefg.ua <- dummy$Rsquare
    bcdefg <- dummy$RsquareAdj
    m6 <- dummy$m
    if (m6 != mm6)
        collinwarn("cbind(X2,X3)", mm6, m6)
    mm7 = mm1 + mm2 + mm3
    dummy <- simpleRDA2(Y, cbind(X1, X2, X3), SS.Y, permat)
    abcdefg.ua <- dummy$Rsquare
    abcdefg <- dummy$RsquareAdj
    m7 <- dummy$m
    if (m7 != mm7)
        collinwarn("cbind(X1,X2,X3)", mm7, m7)
    bigwarning <- NULL
    if ((m1 + m2) > m4)
        bigwarning <- c(bigwarning, c("X1, X2"))
    if ((m1 + m3) > m5)
        bigwarning <- c(bigwarning, c("X1, X3"))
    if ((m2 + m3) > m6)
        bigwarning <- c(bigwarning, c("X2, X3"))
    if ((m1 + m2 + m3) > m7)
        bigwarning <- c(bigwarning, c("X1, X2, X3"))
    Df <- c(m1, m2, m3, m4, m5, m6, m7)
    fract <- data.frame(Df = Df,
                        R.square = c(adfg.ua, bdeg.ua, cefg.ua, abdefg.ua, acdefg.ua,
                        bcdefg.ua, abcdefg.ua),
                        Adj.R.square = c(adfg, bdeg, cefg, abdefg, acdefg, bcdefg,
                        abcdefg), Testable = rep(TRUE, 7) & Df)
    rownames(fract) <- c("[a+d+f+g] = X1", "[b+d+e+g] = X2",
                         "[c+e+f+g] = X3", "[a+b+d+e+f+g] = X1+X2", "[a+c+d+e+f+g] = X1+X3",
                         "[b+c+d+e+f+g] = X2+X3", "[a+b+c+d+e+f+g] = All")
    a <- abcdefg - bcdefg
    b <- abcdefg - acdefg
    c <- abcdefg - abdefg
    d <- acdefg - cefg - a
    e <- abdefg - adfg - b
    f <- bcdefg - bdeg - c
    g <- adfg - a - d - f
    ma <- m7 - m6
    mb <- m7 - m5
    mc <- m7 - m4
    mad <- m5 - m3
    maf <- m4 - m2
    mbd <- m6 - m3
    mbe <- m4 - m1
    mce <- m5 - m1
    mcf <- m6 - m2
    Df <-  c(ma, mb, mc, rep(0, 4), NA)
    indfract <- data.frame(Df = Df,
                           R.square = rep(NA, 8),
                           Adj.R.square = c(a, b, c, d, e, f, g, 1 - abcdefg),
                           Testable = c(rep(TRUE, 3), rep(FALSE, 5)) & Df)
    rownames(indfract) <- c("[a] = X1 | X2+X3", "[b] = X2 | X1+X3",
                            "[c] = X3 | X1+X2", "[d]", "[e]", "[f]", "[g]", "[h] = Residuals")
    Df <- c(mad, maf, mbd, mbe, mce, mcf)
    contr1 <- data.frame(Df = Df,
                         R.square = rep(NA, 6),
                         Adj.R.square = c(a + d, a + f, b + d, b + e, c + e, c + f),
                         Testable = rep(TRUE, 6) & Df)
    rownames(contr1) <- c("[a+d] = X1 | X3", "[a+f] = X1 | X2",
                          "[b+d] = X2 | X3", "[b+e] = X2 | X1", "[c+e] = X3 | X1",
                          "[c+f] = X3 | X2")
    out <- list(fract = fract, indfract = indfract, contr1 = contr1,
                SS.Y = SS.Y, nsets = 3, bigwarning = bigwarning, n = n1)
    class(out) <- "varpart234"
    out
}

### unbiased Simpson index, Hurlbert (1971) "nonconcept" paper, eq. 5,
### but implemented here with rarefy (because I'm lazy and just re-use
### work already done).

`simpson.unb` <-
    function(x, inverse = FALSE)
{
    d <- rarefy(x, 2) - 1
    ## alternatively use directly the Hurlbert equation
    ## n <- rowSums(x)
    ## d <- rowSums(x/n*(n-x)/(n-1))
    if (inverse)
        d <- 1/(1-d)
    attr(d, "Subsample") <- NULL
    d
}

`varpart2` <-
    function (Y, X1, X2, chisquare, permat)
{
    collinwarn <- function(case, mm, m)
        warning(gettextf("collinearity detected in %s: mm = %d, m = %d",
                         case, mm, m), call. = FALSE)
    if (inherits(Y, "dist")) {
        Y <- GowerDblcen(as.matrix(Y^2), na.rm = FALSE)
        Y <- -Y/2
        SS.Y <- sum(diag(Y))
        simpleRDA2 <- match.fun(simpleDBRDA)
    } else {
        Y <- as.matrix(Y)
        if (chisquare) {
            SS.Y <- sum(initCA(Y)^2)
            simpleRDA2 <- match.fun(simpleCCA)
        } else {
            Y <- scale(Y, center = TRUE, scale = FALSE)
            SS.Y <- sum(Y * Y)
        }
    }
    X1 <- as.matrix(X1)
    X2 <- as.matrix(X2)
    n <- nrow(Y)
    n1 <- nrow(X1)
    n2 <- nrow(X2)
    p <- ncol(Y)
    mm1 <- ncol(X1)
    mm2 <- ncol(X2)
    if (n1 != n)
        stop("Y and X1 do not have the same number of rows")
    if (n2 != n)
        stop("Y and X2 do not have the same number of rows")
    X1 <- scale(X1, center = TRUE, scale = FALSE)
    X2 <- scale(X2, center = TRUE, scale = FALSE)
    dummy <- simpleRDA2(Y, X1, SS.Y, permat)
    ab.ua <- dummy$Rsquare
    ab <- dummy$RsquareAdj
    m1 <- dummy$m
    if (m1 != mm1)
        collinwarn("X1", mm1, m1)
    dummy <- simpleRDA2(Y, X2, SS.Y, permat)
    bc.ua <- dummy$Rsquare
    bc <- dummy$RsquareAdj
    m2 <- dummy$m
    if (m2 != mm2)
        collinwarn("X2", mm2, m2)
    mm3 <- mm1 + mm2
    dummy <- simpleRDA2(Y, cbind(X1, X2), SS.Y, permat)
    abc.ua <- dummy$Rsquare
    abc <- dummy$RsquareAdj
    m3 <- dummy$m
    if (m3 != mm3)
        collinwarn("cbind(X1,X2)", mm3, m3)
    if ((m1 + m2) > m3)
        bigwarning <- c("X1, X2")
    else bigwarning <- NULL
    Df <- c(m1, m2, m3)
    fract <- data.frame(Df = Df,
                        R.squared = c(ab.ua, bc.ua, abc.ua),
                        Adj.R.squared = c(ab, bc, abc),
                        Testable = rep(TRUE, 3) & Df)
    rownames(fract) <- c("[a+b] = X1", "[b+c] = X2", "[a+b+c] = X1+X2")
    b <- ab + bc - abc
    Df <- c(m3-m2, 0, m3-m1, NA)
    indfract <- data.frame(Df = Df, R.squared = rep(NA, 4),
                           Adj.R.squared = c(ab - b, b, bc - b, 1 - abc),
                           Testable = c(TRUE, FALSE, TRUE, FALSE) & Df)
    rownames(indfract) <- c("[a] = X1|X2", "[b]", "[c] = X2|X1",
                            "[d] = Residuals")
    out <- list(SS.Y = SS.Y, fract = fract, indfract = indfract,
                nsets = 2, bigwarning = bigwarning, n = n1)
    class(out) <- "varpart234"
    out
}

`plot.orditkplot` <-
    function(x, ...)
{
    op <- par(x$par)
    on.exit(par(op))
    plot(x$points, pch = x$args$pch, cex = x$args$pcex, col = x$args$pcol,
         bg = x$args$pbg, xlim = x$args$xlim, ylim = x$args$ylim, asp=1)
    font <- attr(x$labels, "font")
    if (is.null(font))
        font <- par("font")
    text(x$labels, rownames(x$labels), cex = x$args$tcex,
         col = x$args$tcol, font = font)
    invisible(x)
}

`specnumber` <-
    function(x, groups, MARGIN = 1)
{
    if (!missing(groups)) {
        if (length(groups) == 1)
            groups <- rep(groups, nrow(x))
        x <- aggregate(x, list(groups), max)
        rownames(x) <- x[,1]
        x <- x[,-1]
    }
    if (length(dim(x)) > 1)
        apply(x > 0, MARGIN, sum)
    else
        sum(x > 0)
}

        
############################### CLR ####################################
# Calculates clr-transformation. Should be equal.

# Test data
# data(varespec)
testdata <- matrix(round(runif(1000, 0, 100)), nrow=20)
testdata <- testdata - 50
testdata[testdata < 0] <- 0
rownames(testdata) <- paste0("row", seq_len(nrow(testdata)))
colnames(testdata) <- paste0("col", seq_len(ncol(testdata)))
testdata.with.pseudo <- testdata + 1

# Calculates relative abundance table
relative <- vegan::decostand(testdata, "total")
relative.with.pseudo <- vegan::decostand(testdata+1, "total")

# CLR data
x.clr <- vegan::decostand(testdata+1, method = "clr")
x.rclr <- vegan::decostand(testdata, method = "rclr")
x.clr.pseudo <- vegan::decostand(testdata, method = "clr", pseudocount=1)

max(abs(x.clr - x.clr.pseudo))<1e-6
max(abs(vegan::decostand(testdata+1, method = "clr", pseudocount=0)-x.clr.pseudo))<1e-6

# Tests clr
alt.clr <- t(apply(as.matrix(relative.with.pseudo), 1, FUN=function(x){
log(x) - mean(log(x))}))
max(abs(x.clr-alt.clr)) < 1e-6      
all((x.rclr==0) == (testdata==0))

# Expect that error does not occur
vegan::decostand(testdata, method = "rclr")
vegan::decostand(testdata, method = "clr", pseudocount=1)
# Expect error
class(try(vegan::decostand(testdata, method = "clr")))=="try-error"
class(try(vegan::decostand(testdata, method = "clr", pseudocount=0)))=="try-error"

# Tests that clr robust gives values that are approximately same if only 
# one value per sample are changed to zero
# Adds pseudocount
test <- testdata+1 
test2 <- test; test2[,1] <- 0

# clr robust transformations
test <- vegan::decostand(test, method = "rclr")
test2 <- vegan::decostand(test2, method = "rclr")

# Removes first cols
test <- test[, -1]
test2 <- test2[, -1]

# Expect high correlation
cor(unlist(test), unlist(test2)) > 0.99

############################# NAMES ####################################

# Tests that dimensins have correct names
all(rownames(vegan::decostand(testdata+1, method = "clr")) == rownames(testdata))
all(colnames(vegan::decostand(testdata, method = "clr", pseudocount=1)) == colnames(testdata))
all(rownames(vegan::decostand(testdata, method = "rclr")) == rownames(testdata))
all(colnames(vegan::decostand(testdata, method = "rclr")) == colnames(testdata))

########################################################################

# Count vs. Relative data

# CLR is identical with count and relative data
a1 <- vegan::decostand(testdata.with.pseudo, method = "clr")
a2 <- vegan::decostand(relative.with.pseudo, method = "clr")
max(abs(a1-a2)) < 1e-6 # Tolerance

# rCLR is identical with count and relative data
a1 <- vegan::decostand(testdata.with.pseudo, method = "rclr")
a2 <- vegan::decostand(relative.with.pseudo, method = "rclr")
max(abs(a1-a2)) < 1e-6 # Tolerance

# ALR is identical with count and relative data
a1 <- vegan::decostand(testdata.with.pseudo, method = "alr")
a2 <- vegan::decostand(relative.with.pseudo, method = "alr")
max(abs(a1-a2)) < 1e-6 # Tolerance

####### # ALR transformation drops one feature ################
ncol(vegan::decostand(testdata.with.pseudo, "alr")) == ncol(testdata.with.pseudo)-1

"veiledspec" <-
    function(x, ...)
{
    if (!inherits(x, "prestonfit"))
        x <- prestonfit(x)
    S.obs <- sum(x$freq)
    p <- x$coefficients
    S.tot <- p["S0"]*p["width"]*sqrt(2*pi)
    out <- c(S.tot, S.obs, S.tot - S.obs)
    names(out) <- c("Extrapolated","Observed","Veiled")
    out
}

`decorana` <-
    function (veg, iweigh = 0, iresc = 4, ira = 0, mk = 26, short = 0,
              before = NULL, after = NULL)
{
    ## constants
    Const2 <- 5
    Const3 <- 1e-11
    ZEROEIG <- 1e-7 # same limit as in the C function do_decorana
    ## data
    veg <- as.matrix(veg)
    if (!is.numeric(veg))
        stop("data 'veg' must be numeric (not factors or characters)")
    if (any(veg < 0))
        stop("'decorana' cannot handle negative data entries")
    ## optional data transformation
    if (!is.null(before)) {
        veg <- beforeafter(veg, before, after)
    }
    if (iweigh) {
        veg <- downweight(veg, Const2)
    }
    v <- attr(veg, "v")
    v.fraction <- attr(veg, "fraction")
    ## marginal sums after optional data transformations
    aidot <- rowSums(veg)
    if (any(aidot <= 0))
        stop("all row sums must be >0 in the community matrix: remove empty sites")
    adotj <- colSums(veg)
    if (any(adotj <= 0))
        warning("some species were removed because they were missing in the data")
    adotj[adotj < Const3] <- Const3
    ## check arguments
    if (mk < 10)
        mk <- 10
    if (mk > 46)
        mk <- 46
    if (ira)
        iresc <- 0
    ## Start analysis
    CA <- .Call(do_decorana, veg, ira, iresc, short, mk, as.double(aidot),
                as.double(adotj))
    if (ira)
        dnames <- paste("RA", 1:4, sep = "")
    else dnames <- paste("DCA", 1:4, sep = "")
    dimnames(CA$rproj) <- list(rownames(veg), dnames)
    dimnames(CA$cproj) <- list(colnames(veg), dnames)
    names(CA$evals) <- dnames
    origin <- apply(CA$rproj, 2, weighted.mean, aidot)
    vegChi <- initCA(veg) # needed for eigenvalues & their sum
    totchi <- sum(vegChi^2)
    if (ira) {
        evals.decorana <- NULL
        evals.ortho <- NULL
    }
    else {
        evals.decorana <- CA$evals
        if (any(ze <- evals.decorana <= 0))
            CA$evals[ze] <- 0
        ## centred and weighted scores
        x0 <- scale(CA$rproj, center = origin, scale = FALSE)
        x0 <- sqrt(aidot/sum(aidot)) * x0
        y0 <- scale(CA$cproj, center = origin, scale = FALSE)
        y0 <- sqrt(adotj/sum(adotj)) * y0
        ## eigenvalue: shrinking of scores y0 --WA--> x0
        evals <- colSums(x0^2)/colSums(y0^2)
        evals[evals < ZEROEIG | !is.finite(evals)] <- 0
        CA$evals <- evals
        ## decorana finds row scores from species scores, and for
        ## additive eigenvalues we need orthogonalized species
        ## scores. Q of QR decomposition will be orthonormal and if we
        ## use it for calculating row scores, these directly give
        ## additive eigenvalues.
        qy <- qr.Q(qr(y0))
        evals.ortho <- numeric(4) # qy can have < 4 columns
        evals.ortho[seq_len(ncol(qy))] <- colSums(crossprod(t(vegChi), qy)^2)
        evals.ortho[evals.ortho < ZEROEIG | !is.finite(evals.ortho)] <- 0
        names(evals.ortho) <- names(evals.decorana)
    }
    additems <- list(totchi = totchi, evals.ortho = evals.ortho,
                     evals.decorana = evals.decorana, origin = origin,
                     v = v, fraction = v.fraction, iweigh = iweigh,
                     before = before, after = after,
                     call = match.call())
    CA <- c(CA, additems)
    class(CA) <- "decorana" # c() strips class
    CA
}

`scores.lda` <-
    function(x, display, ...)
{
    display <- match.arg(display,
                         c("sites", "species", "scores", "predictors", "x", "coef"),
                         several.ok = TRUE)
    out <- NULL
    if (display %in% c("sites", "scores", "x"))
        out[["scores"]] <- predict(x)$x
    if (display %in% c("species", "predictors", "coef"))
        out[["coefficients"]] <- coef(x)
    if (length(out) == 1)
        out <- out[[1]]
    out
}

"rad.zipfbrot" <-
    function (x, family = poisson, ...) 
{
    mandelfun <- function(p, x, ...) {
        brnk <- log(rnk + exp(p))
        sol <- glm(x ~ brnk + offset(off), family = family(link = "log"))
        -logLik(sol)
    }
    x <- as.rad(x)
    rnk <- seq(along = x)
    off <- rep(log(sum(x)), length(x))
    p <- 0
    fam <- family(link = "log")
    if (length(x) > 2) 
        nl <- try(nlm(mandelfun, p = p, x = x, rnk = rnk, off = off, 
                      family = fam, hessian = TRUE, ...))
    if (length(x) < 3) {
        aic <- NA
        dev <- rdf <-  0
        ln <- nl <- NA
        p <- rep(NA, 3)
        fit <- x
        res <- rep(0, length(x))
        wts <- rep(1, length(x))
    }
    else if (inherits(nl, "try-error")) {
        aic <- rdf <- ln <- nl <- dev <-  NA
        p <- rep(NA, 3)
        fit <- res <- wts <- rep(NA, length(x))
    }
    else {
        ln <- glm(x ~ log(rnk + exp(nl$estimate)) + offset(off), 
                  family = family(link = "log"))
        fit <- fitted(ln)
        p <- c(coef(ln), exp(nl$estimate))
        p[1] <- exp(p[1])
        aic <- AIC(ln) + 2
        rdf <- df.residual(ln) - 1
        dev <- deviance(ln)
        res <- ln$residuals
        wts <- weights(ln)
    }
    names(p) <- c("c", "gamma", "beta")
    out <- list(model = "Zipf-Mandelbrot", family = fam, 
                y = x, coefficients = p, fitted.values = fit, aic = aic, 
                rank = 3, df.residual = rdf, deviance = dev, 
                residuals = res, prior.weights = wts)
    class(out) <- c("radline", "glm")
    out
}

"points.decorana" <-
    function (x, display=c("sites", "species"), choices=1:2, origin = TRUE,
              select, ...)
{
    display <- match.arg(display)
    x <- scores(x, display = display, choices = choices, origin = origin, ...)
    if (!missing(select))
        x <- .checkSelect(select, x)
    ordiArgAbsorber(x, FUN = points, ...)
    invisible()
}

`ordiresids` <-
    function(x, kind = c("residuals", "scale", "qqmath"),
             residuals = "working",
             type = c("p", "smooth", "g"), formula, ...)
{
    kind <- match.arg(kind)
    if (!inherits(x, "cca") || is.null(x$CCA) || x$CCA$rank == 0)
        stop("function is only available for constrained ordination")
    residuals <- match.arg(residuals, c("working", "response",
                                        "standardized", "studentized"))
    fit <- switch(residuals,
        "working" =,
        "response" = fitted(x, type = residuals),
        "standardized" =,
        "studentized" = sweep(fitted(x, type="working"), 2, sigma(x), "/"))
    res <- switch(residuals,
        "standardized" = rstandard(x),
        "studentized" = rstudent(x),
        residuals(x, type = residuals))
    colnam <- rep(colnames(fit), each=nrow(fit))
    rownam <- rep(rownames(fit), ncol(fit))
    df <- data.frame(Fitted = as.vector(fit), Residuals = as.vector(res))
    if (!is.null(rownam))
        df$Sites <- rownam
    if (!is.null(colnam))
        df$Species <- colnam
    if (kind == "residuals") {
        if (missing(formula))
            formula <- as.formula(Residuals ~ Fitted)
        pl <- xyplot(formula, data = df, type = type, ...)
    }
    if (kind == "scale") {
        if (missing(formula))
            formula <- as.formula(sqrt(abs(Residuals)) ~ Fitted)
        pl <- xyplot(formula, data = df, type = type, ...)
    }
    if (kind == "qqmath") {
        if (missing(formula))
            formula <- as.formula(~ Residuals)
        pl <- qqmath(formula, data = df, type = type, ...)
    }
    pl
}

SSarrhenius <-
    selfStart(~ k*area^z,
              function(mCall, data, LHS)
{
    xy <- sortedXyData(mCall[["area"]], LHS, data)
    value <- as.vector(coef(lm(log(pmax(xy[,"y"],1)) ~ log(xy[,"x"]))))
    value[1] <- exp(value[1])
    names(value) <- mCall[c("k","z")]
    value
},
c("k","z"))

`no.shared` <-
    function(x)
{
    x <- as.matrix(x)
    d <- .Call(do_vegdist, x, as.integer(99))
    d <- as.logical(d)
    attr(d, "Size") <- NROW(x)
    attr(d, "Labels") <- dimnames(x)[[1]]
    attr(d, "Diag") <- FALSE
    attr(d, "Upper") <- FALSE
    attr(d, "method") <- "no.shared"
    attr(d, "call") <- match.call()
    class(d) <- "dist"
    d
}

`add1.cca`<-
    function(object, scope, test = c("none", "permutation"),
             permutations = how(nperm = 199), ...)
{
    if (inherits(object, "prc"))
        stop("'step'/'add1' cannot be used for 'prc' objects")
    if (is.null(object$terms))
        stop("ordination model must be fitted using formula")
    test <- match.arg(test)
    ## Default add1
    out <- NextMethod("add1", object, test = "none", ...)
    cl <- class(out)
    ## Loop over terms in 'scope' and do anova.cca
    if (test == "permutation") {
        ## Avoid nested Condition(Condition(x) + z)
        hasfla <- update(terms(object$terminfo), . ~  Condition(.))
        if (!is.character(scope)) 
            scope <- add.scope(object, update.formula(object, scope))
        ns <- length(scope)
        adds <- matrix(0, ns+1, 2)
        adds[1, ] <- NA
        for (i in 1:ns) {
            tt <- scope[i]
            ## Condition(.) previous terms (if present)
            if (!is.null(object$CCA)) {
                fla <- update(hasfla, paste("~ . +", tt))
                nfit <- update(object, fla)
            }
            else
                nfit <- update(object,
                               as.formula(paste(". ~ . +", tt)))
            tmp <- anova(nfit,  permutations = permutations, ...)
            adds[i+1,] <- unlist(tmp[1,3:4])
        }
        colnames(adds) <- colnames(tmp)[3:4]
        out <- cbind(out, adds)
        ## check for redundant (0 Df) terms
        if (any(nas <- out[,1] < 1, na.rm = TRUE)) {
            out[[3]][nas] <- NA
            out[[4]][nas] <- NA
        }
        class(out) <- cl
    }
    out
}

`prc`  <-
    function (response, treatment, time, ...)
{
    extras <- match.call(expand.dots = FALSE)$...
    if (is.null(extras$data))
        data <- parent.frame()
    else
        data <- eval(extras$data)
    y <- deparse(substitute(response))
    x <- deparse(substitute(treatment))
    z <- deparse(substitute(time))
    oldcon <- options(contrasts = c("contr.treatment", "contr.poly"))
    on.exit(options(oldcon))
    fla <- as.formula(paste("~", x, "+", z))
    mf <- model.frame(fla, data, na.action = na.pass)
    if (!all(sapply(mf, is.factor)))
        stop(gettextf("%s and %s must be factors", x, z))
    if (any(sapply(mf, is.ordered)))
        stop(gettextf("%s or %s cannot be ordered factors", x, z))
    fla.zx <- as.formula(paste("~", z, ":", x))
    fla.z <- as.formula(paste("~", z))
    # delete first (control) level from the design matrix
    X = model.matrix(fla.zx, mf)[,-c(seq_len(nlevels(time)+1))]
    Z = model.matrix(fla.z, mf)[,-1]
    mod <- rda(response ~ X + Condition(Z), ...)
    mod$terminfo$xlev = list(levels(time), levels(treatment))
    names(mod$terminfo$xlev) = c(paste(z), paste(x))
    mod$call <- match.call()
    class(mod) <- c("prc", class(mod))
    mod
}

`points.cca` <-
    function (x, display = "sites", choices = c(1, 2), scaling = "species",
              arrow.mul, head.arrow = 0.05, select, const, axis.bp = FALSE,
              correlation = FALSE, hill = FALSE, ...)
{
    formals(arrows) <- c(formals(arrows), alist(... = ))
    if (length(display) > 1)
        stop("only one 'display' item can be added in one command")
    pts <- scores(x, choices = choices, display = display, scaling = scaling,
                  const, correlation = correlation, hill = hill)
    if (!missing(select))
        pts <- .checkSelect(select, pts)
    if (display == "cn") {
        cnam <- rownames(pts)
        points(pts, ...)
        pts <- scores(x, choices = choices, display = "bp", scaling = scaling,
                      const, correlation = correlation, hill = hill)
        bnam <- rownames(pts)
        pts <- pts[!(bnam %in% cnam), , drop = FALSE]
        if (nrow(pts) == 0)
            return(invisible())
        else display <- "bp"
    }
    if (display %in% c("bp", "reg", "re", "r")) {
    	if (missing(arrow.mul)) {
            arrow.mul <- ordiArrowMul(pts)
    	}
        pts <- pts * arrow.mul
        arrows(0, 0, pts[, 1], pts[, 2], length = head.arrow,
               ...)
        pts <- pts * 1.1
        if (axis.bp) {
            axis(3, at = c(-arrow.mul, 0, arrow.mul),
                 labels = rep("", 3))
            axis(4, at = c(-arrow.mul, 0, arrow.mul),
                 labels = c(-1, 0, 1))
        }
        return(invisible())
    }
    points(pts, ...)
    invisible()
}

"plot.fisherfit" <-
    function(x, xlab = "Frequency", ylab = "Species", bar.col = "skyblue",
             line.col= "red", lwd=2, ...)
{
    freq <- as.numeric(names(x$fisher))
    plot(freq, x$fisher, ylab=ylab, xlab=xlab,
         ylim=c(0,max(x$fisher)),  xlim=c(0.5, max(freq)+0.5), type="n", ...)
    rect(freq-0.5,0,freq+0.5,x$fisher, col=bar.col, ...)
    alpha <- x$estimate
    k <- x$nuisance
    curve(alpha*k^x/x, 1, max(freq), col=line.col, lwd=lwd, add=TRUE, ...)
    invisible()
}

`radfit.data.frame` <-
    function(x, ...)
{
    ## x *must* have rownames
    rownames(x) <- rownames(x, do.NULL = TRUE)
    ## remove empty rows with no species
    nspec <- specnumber(x)
    if (any(nspec == 0)) {
        warning("removed empty rows with no species")
        x <- x[nspec>0,, drop=FALSE]
    }
    out <- apply(x, 1, radfit, ...)
    if (length(out) == 1)
        out <- out[[1]]
    else {
        Call <- match.call()
        class(out) <- "radfit.frame"
    }
    out
}

`radfit.matrix` <-
    function(x, ...)
{
    radfit(as.data.frame(x), ...)
}

### these functions are defined _ex machina_ for radline objects which
### inherit from glm. Here we define them for radfit objects where
### object$models is a list of radline objects

`AIC.radfit` <-
    function (object, k = 2, ...) 
{
    sapply(object$models, AIC, k = k, ...)
}

`deviance.radfit` <-
    function(object, ...)
{
    sapply(object$models, deviance, ...)
}

`logLik.radfit` <-
    function(object, ...)
{
    sapply(object$models, logLik, ...)
}

### Define also for radfit.frames which are lists of radfit objects

`AIC.radfit.frame` <-
    function(object, k = 2, ...)
{
    sapply(object, AIC, k = k, ...)
}

`deviance.radfit.frame` <-
    function(object, ...)
{
    sapply(object, deviance, ...)
}

`logLik.radfit.frame` <-
    function(object, ...)
{
    sapply(object, logLik, ...)
}

### Random rarefied subsample: sample without replacement

`rrarefy` <-
    function(x, sample)
{
    x <- as.matrix(x)
    if (!identical(all.equal(x, round(x)), TRUE))
        stop("function is meaningful only for integers (counts)")
    ## x may not be exactly integer, since, e.g., sqrt(2)^2 != 2
    if (!is.integer(x))
        x <- round(x)
    if (ncol(x) == 1)
        x <- t(x)
    if (length(sample) > 1 && length(sample) != nrow(x))
        stop(gettextf(
             "length of 'sample' and number of rows of 'x' do not match"))
    sample <- rep(sample, length=nrow(x))
    ## warn if something cannot be rarefied
    if (any(rowSums(x) < sample))
        warning("some row sums < 'sample' and are not rarefied")
    for (i in 1:nrow(x)) {
        x[i,] <- .Call(do_rrarefy, x[i,], sample[i])
    }
    x
}

### Probabilities that species occur in a rarefied 'sample'

`drarefy` <-
    function(x, sample)
{
    if (!identical(all.equal(x, round(x)), TRUE))
        stop("function accepts only integers (counts)")
    if (length(sample) > 1 &&  length(sample) != nrow(x))
        stop(gettextf(
             "length of 'sample' and number of rows of 'x' do not match"))
    x <- drop(as.matrix(x))
    ## warn on too large samples
    if (is.matrix(x))
        rs <- rowSums(x)
    else
        rs <- sum(x)
    if (any(rs < sample))
        warning("some row sums < 'sample' and probabilities either 0 or 1")
    ## dfun is kluge: first item of  vector x must be the sample size,
    ## and the rest  is the community data. This  seemed an easy trick
    ## to evaluate dfun in an apply() instead of a loop.
    dfun <- function(x) {
        J <- sum(x[-1])
        sample <- min(x[1], J)
        1 - exp(lchoose(J - x[-1], sample) - lchoose(J, sample))
    }
    if (length(dim(x)) > 1)
        t(apply(cbind(sample, x), 1, dfun))
    else
        dfun(c(sample, x))
}

`vectorfit` <-
    function (X, P, permutations = 0, strata = NULL, w, ...)
{
    EPS <- sqrt(.Machine$double.eps)
    if (missing(w) || is.null(w))
        w <- 1
    if (length(w) == 1)
        w <- rep(w, nrow(X))
    P <- as.matrix(P)
    if (nrow(P) != nrow(X))
        stop("input data have non-matching numbers of observations")
    Xw <- .Call(do_wcentre, X, w)
    Pw <- .Call(do_wcentre, P, w)
    colnames(Pw) <- colnames(P)
    nc <- ncol(X)
    Q <- qr(Xw)
    H <- qr.fitted(Q, Pw)
    heads <- qr.coef(Q, Pw)
    r <- diag(cor(H, Pw)^2)
    r[is.na(r)] <- 0
    heads <- decostand(heads, "norm", 2)
    heads <- t(heads)
    if (is.null(colnames(X)))
        colnames(heads) <- paste("Dim", 1:nc, sep = "")
    else colnames(heads) <- colnames(X)
    ## make permutation matrix for all variables handled in the next loop
    nr <- nrow(X)
    permat <- getPermuteMatrix(permutations, nr, strata = strata)
    if (ncol(permat) != nr)
        stop(gettextf("'permutations' have %d columns, but data have %d rows",
                          ncol(permat), nr))
    permutations <- nrow(permat)

    if (permutations) {
        ptest <- function(indx, ...) {
            take <- P[indx, , drop = FALSE]
            take <- .Call(do_wcentre, take, w)
            Hperm <- qr.fitted(Q, take)
            diag(cor(Hperm, take))^2
        }
        permstore <- sapply(1:permutations, function(indx, ...) ptest(permat[indx,], ...))
        ## Single variable is dropped to a vector, and otherwise
        ## permutations are the matrix columns and variables are rows
        if (!is.matrix(permstore))
            permstore <- matrix(permstore, ncol=permutations)
        permstore <- sweep(permstore, 1, r - EPS, ">=")
        validn <- rowSums(is.finite(permstore))
        pvals <- (rowSums(permstore, na.rm = TRUE) + 1)/(validn + 1)
    }
    else pvals <- NULL
    sol <- list(arrows = heads, r = r, permutations = permutations,
                pvals = pvals)
    sol$control <- attr(permat, "control")
    class(sol) <- "vectorfit"
    sol
}

`envfit.default` <-
    function (ord, env, permutations = 999, strata = NULL, choices = c(1, 2),
             display = "sites", w = weights(ord, display), na.rm = FALSE, ...)
{
    weights.default <- function(object, ...) NULL
    w < eval(w)
    vectors <- NULL
    factors <- NULL
    seed <- NULL
    X <- scores(ord, display = display, choices = choices, ...)
    keep <- complete.cases(X) & complete.cases(env)
    if (any(!keep)) {
        if (!na.rm)
            stop("missing values in data: consider na.rm = TRUE")
        X <- X[keep,, drop=FALSE]
        ## drop any lost levels, explicitly don't include NA as a level
        env <- droplevels(env[keep,, drop=FALSE], exclude = NA)
        w <- w[keep]
        na.action <- structure(seq_along(keep)[!keep], class="omit")
    }
    ## make permutation matrix for all variables handled in the next loop
    nr <- nrow(X)
    permat <-  getPermuteMatrix(permutations, nr, strata = strata)
    if (ncol(permat) != nr)
        stop(gettextf("'permutations' have %d columns, but data have %d rows",
                      ncol(permat), nr))

    if (is.data.frame(env)) {
        vects <- sapply(env, is.numeric)
        if (any(!vects)) {  # have factors
            Pfac <- env[, !vects, drop = FALSE]
            P <- env[, vects, drop = FALSE]
            if (length(P)) { # also have vectors
                vectors <- vectorfit(X, P, permutations, strata,
                                     choices, w = w, ...)
            }
            factors <- factorfit(X, Pfac, permutations, strata,
                                         choices, w = w, ...)
            sol <- list(vector = vectors, factors = factors)
            }
        else vectors <- vectorfit(X, env, permutations, strata,
                                  choices, w = w, ...)
    }
    else vectors <- vectorfit(X, env, permutations, strata,
                              choices, w = w, ...)
    sol <- list(vectors = vectors, factors = factors)
    if (!is.null(na.action))
        sol$na.action <- na.action
    class(sol) <- "envfit"
    sol
}

`print.taxondive` <-
    function (x, ...) 
{
    out <- cbind(x$Species, x$D, x$Dstar, x$Lambda, x$Dplus, x$SDplus)
    out <- rbind(out, Expected = c(NA, x$ED, x$EDstar, NA, x$EDplus, NA))
    colnames(out) <- c("Species", "Delta", "Delta*", "Lambda+", 
                       "Delta+", "S Delta+")
    printCoefmat(out, na.print = "")
    invisible(x)
}

"rad.zipf" <-
    function (x, family = poisson, ...) 
{
    x <- as.rad(x)
    rnk <- seq(along = x)
    off <- rep(log(sum(x)), length(x))
    fam <- family(link = "log")
    if (length(x) > 1)
        ln <- try(glm(x ~ log(rnk) + offset(off), family = fam))
    if (length(x) < 2) {
        aic <- NA
        dev <- rdf <-  0
        ln <- nl <- NA
        p <- rep(NA, 2)
        fit <- x
        res <- rep(0, length(x))
        wts <- rep(1, length(x))
    }
    else if (inherits(ln, "try-error")) {
        aic <- rdf <- ln <- nl <- dev <- NA
        p <- rep(NA, 2)
        fit <- res <- wts <- rep(NA, length(x))
    }
    else {
        fit <- fitted(ln)
        p <- coef(ln)
        p[1] <- exp(p[1])
        aic <- AIC(ln)
        rdf <- df.residual(ln)
        dev <- deviance(ln)
        res <- ln$residuals
        wts <- weights(ln)
    }
    names(p) <- c("p1", "gamma")
    out <- list(model = "Zipf", family = fam, y = x, coefficients = p, 
                fitted.values = fit, aic = aic, rank = 2, df.residual = rdf, 
                deviance = dev, residuals = res, prior.weights = wts)
    class(out) <- c("radline", "glm")
    out
}

### summary allocates shared components equally to explanatory sets

`summary.varpart` <-
    function(object, ...)
{
    nsets <- object$part$nsets
    nfract <- 2^nsets - 1
    ## find fractions each component is made of
    contr <- matrix(0, nfract, nsets,
                    dimnames = list(paste0("[", letters[seq_len(nfract)], "]"),
                                    paste0("X", seq_len(nsets))))
    setfrac <- rownames(object$part$fract)[seq_len(nsets)]
    for (i in seq_len(nsets))
        contr[,i] <- as.numeric(letters[seq_len(nfract)] %in%
                                strsplit(setfrac[i], "")[[1]])
    ## allocate shared fractions equally to all sets
    contr <- sweep(contr, 1, rowSums(contr), "/")
    uniqpart <- object$part$indfract[seq_len(nfract), 3]
    contr <- uniqpart * contr
    ## returned components
    contrpart <- colSums(contr)
    uniqpart <- uniqpart[seq_len(nsets)]
    names(uniqpart) <- names(contrpart)
    out <- list("uniqpart" = uniqpart,
                "contribpart" = contrpart,
                "contributions" = contr,
                "setlabels" = object$tables)
    class(out) <- "summary.varpart"
    out
}

`print.summary.varpart` <-
    function(x, digits = 3, zero.print = "", ...)
{
    ## collect table
    df <- data.frame("Unique" = x$uniqpart, "Contributed" = x$contribpart,
                     "Component" = x$setlabels)
    cat("\nUnique fractions and total with shared fractions equally allocated:\n\n")
    print(df, digits = digits)
    cat("\nContributions of fractions to sets:\n\n")
    print.table(x$contributions, digits = digits, zero.print = zero.print)
    invisible(x)
}

`print.oecosimu` <-
    function(x, ...)
{
    xx <- x ## return unmodified input object
    attr(x$oecosimu$method, "permfun") <- NULL
    cat(as.character(attr(x,"call")[[1]]), "object\n\n")
    writeLines(strwrap(pasteCall(attr(x, "call"))))
    cat("\n")
    cat("nullmodel method", sQuote(x$oecosimu$method), "with", 
        ncol(x$oecosimu$simulated), "simulations\n")
    if (length(att <- attributes(x$oecosimu$simulated)) > 1) {
        att$dim <- NULL
        cat("options: ", paste(names(att), att, collapse=", "))
    }
    alt.char <- switch(x$oecosimu$alternative,
                       two.sided = "less or greater than",
                       less = "less than",
                       greater = "greater than")
    cat("\nalternative hypothesis: statistic is", alt.char, "simulated values")
    ## dim attribute is always there, but print all others

    cat("\n\n")

    if (!inherits(x, c("adipart", "hiersimu", "multipart")) &&
        !inherits(x$statistic, c("numeric", "list"))) {
            print(x$statistic)
            cat("\n")
    }
    probs <- switch(x$oecosimu$alternative,
                    two.sided = c(0.025, 0.5, 0.975),
                    greater = c(0.5, 0.95),
                    less = c(0.05, 0.5))
    qu <- apply(x$oecosimu$simulated, 1, quantile, probs=probs, na.rm = TRUE)
    m <- cbind("statistic" = x$oecosimu$statistic,
               "SES" = x$oecosimu$z, "mean" = x$oecosimu$means, t(qu),
               "Pr(sim.)"=x$oecosimu$pval)
    printCoefmat(m, cs.ind = 3:(ncol(m)-1), ...)
    if (any(is.na(x$oecosimu$simulated))) {
        nacount <- rowSums(is.na(x$oecosimu$simulated))
        cat("\nNumber of NA cases removed from simulations:\n",
            nacount, "\n")
    }
    invisible(xx)   
}

`print.summary.isomap` <-
function (x, ...)
{
	cat("\nCall:\n")
	cat(deparse(x$call), "\n\n")
	cat("\nRetained dissimilarities between points:\n")
	prmatrix(t(x$net), collab = rep("", x$nnet) , ...)
	cat("\nRetained", x$nnet, "of", x$ndis, "dissimilarities\n")
	invisible(x)
}

`envfit.formula` <-
    function(formula, data, ...)
{
    if (missing(data))
        data <- environment(formula)
    X <- formula[[2]]
    X <- eval(X, environment(formula), enclos = .GlobalEnv)
    formula[[2]] <- NULL
    P <- model.frame(formula, data, na.action = na.pass)
    envfit.default(X, P, ...)
}

`as.mcmc.oecosimu` <-
    function(x)
{
    ## mcmc only for sequential methods
    if (!x$oecosimu$isSeq)
        stop("as.mcmc available only for sequential null models")
    ## named variables
    rownames(x$oecosimu$simulated) <- names(x$oecosimu$z)
    chains <- attr(x$oecosimu$simulated, "chains")
    ## chains: will make each chain as an mcmc object and combine
    ## these to an mcmc.list
    if (!is.null(chains) && chains > 1) {
        x <- x$oecosimu$simulated
        nsim <- dim(x)[2]
        niter <- nsim / chains
        ## iterate over chains
        x <- lapply(1:chains, function(i) {
                        z <- x[, ((i-1) * niter + 1):(i * niter), drop = FALSE]
                        attr(z, "mcpar") <-
                            c(attr(x, "burnin") + attr(x, "thin"),
                              attr(x, "burnin") + attr(x, "thin") * niter,
                              attr(x, "thin"))
                        attr(z, "class") <- c("mcmc", class(z))
                        t(z)
                    })
        ## combine list of mcmc objects to a coda mcmc.list
        #x <- as.mcmc.list(x)
        class(x) <- "mcmc.list"
    } else { # one chain: make to a single mcmc object
        x <- as.ts(x)
        mcpar <- attr(x, "tsp")
        mcpar[3] <- round(1/mcpar[3])
        attr(x, "mcpar") <- mcpar
        class(x) <- c("mcmc", class(x))
    }
    x
}

`eventstar` <-
    function(x, qmax=5)
{
    if (is.null(dim(x)))
        x <- matrix(x, 1, length(x))
    else
        x <- as.matrix(x) # faster than data.frame
    lossfun <- function(q, x)
        tsallis(x, scales=q, norm=TRUE)
    qstarfun <- function(x) {
        optimize(lossfun, interval=c(0, qmax), x=x)$minimum
    }
    qs <- apply(x, 1, qstarfun)
    Hs <- sapply(1:nrow(x), function(i) tsallis(x[i,], 
        scales=qs[i], hill=FALSE))
    S <- rowSums(x)
    Es <- ifelse(qs==1, log(S), Hs/((S^(1-qs)-1)/(1-qs)))
    Ds <- (1-(qs-1)*Hs)^(1/(1-qs))
    out <- data.frame(qstar=qs, Estar=Es, Hstar=Hs, Dstar=Ds)
    rownames(out) <- rownames(x)
    out
}

`fisher.alpha` <-
    function (x, MARGIN = 1, ...) 
{
    x <- as.matrix(x)
    if(ncol(x) == 1)
        x <- t(x)
    sol <- apply(x, MARGIN, fisherfit)
    out <-  unlist(lapply(sol, function(x) x$estimate))
    if (FALSE) {
        out <- list(alpha = out)
        out$se <- unlist(lapply(sol, function(x) sqrt(diag(solve(x$hessian)))[1]))
        out$df.residual <- unlist(lapply(sol, df.residual))
        out$code <- unlist(lapply(sol, function(x) x$code))
        out <- as.data.frame(out)
    }
    out
}

summary.clamtest <- function(object, ...) {
    structure(c(attr(object, "settings"), 
        list(summary=cbind(Species=table(object$Classes), 
            Proportion=table(object$Classes)/nrow(object)),
        minv=attr(object, "minv"),
        coverage=attr(object, "coverage"))), class="summary.clamtest")
}

`vegandocs` <-
    function (doc = c("NEWS", "ONEWS", "FAQ-vegan",
              "intro-vegan", "diversity-vegan",
              "decision-vegan", "partitioning", "permutations")) 
{
    doc <- match.arg(doc)
    if (doc == "NEWS") {
        .Deprecated('news(package="vegan")')
        news(package = "vegan")
    } else if (doc %in% vignette(package="vegan")$results[, "Item"]) {
        .Deprecated('browseVignettes("vegan")')
        vignette(doc, package = "vegan")
    } else if (doc == "permutations") {
        .Deprecated('browseVignettes("permute")')
        vignette(doc, package = "permute")
    } else  # last resort
        file.show(system.file(package="vegan", doc))
}

"plot.prestonfit" <-
    function (x, xlab = "Frequency", ylab = "Species", bar.col = "skyblue", 
              line.col = "red", lwd = 2, ...) 
{
    freq <- x$freq
    oct <- as.numeric(names(freq))
    noct <- max(oct) + 1
    plot(oct, freq, type = "n", ylim = c(0, max(freq)), xlim = c(-1, 
                                                        max(oct)), ylab = ylab, xlab = xlab, axes = FALSE, ...)
    axis(2)
    axis(1, at = 0:noct, labels = 2^(0:noct))
    box()
    rect(oct - 1, 0, oct, freq, col = bar.col, ...)
    p <- x$coefficients
    curve(p[3] * exp(-(x - p[1])^2/2/p[2]^2), -1, max(oct), add = TRUE, 
          col = line.col, lwd = lwd, ...)
    segments(p["mode"], 0, p["mode"], p["S0"], col = line.col, ...)
    segments(p["mode"] - p["width"], p["S0"] * exp(-0.5), p["mode"] + 
             p["width"], p["S0"] * exp(-0.5), col = line.col, ...)
    invisible()
}

## this lists all known algos in vegan and more
## if method is commsim object, it is returned
## if it is character, switch returns the right one, else stop with error
## so it can be used instead of match.arg(method) in other functions
## NOTE: very very long -- but it can be a central repository of algos
## NOTE 2: storage mode coercions are avoided here
## (with no apparent effect on speed), it should be
## handled by nullmodel and commsim characteristics
make.commsim <-
function(method)
{
    algos <- list(
        "r00" = commsim(method="r00", binary=TRUE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            out <- matrix(0L, nr * nc, n)
            for (k in seq_len(n))
                out[sample.int(nr * nc, s), k] <- 1L
            dim(out) <- c(nr, nc, n)
            out
        }),
        "c0" = commsim(method="c0", binary=TRUE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            out <- array(0L, c(nr, nc, n))
            J <- seq_len(nc)
            for (k in seq_len(n))
                for (j in J)
                    out[sample.int(nr, cs[j]), j, k] <- 1L
            out
        }),
        "r0" = commsim(method="r0", binary=TRUE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            out <- array(0L, c(nr, nc, n))
            I <- seq_len(nr)
            for (k in seq_len(n))
                for (i in I)
                    out[i, sample.int(nc, rs[i]), k] <- 1L
            out
        }),
        "r0_old" = commsim(method="r0_old", binary=TRUE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            out <- array(0L, c(nr, nc, n))
            I <- seq_len(nr)
            p <- rep(1, nc)
            for (k in seq_len(n))
                for (i in I)
                    out[i, sample.int(nc, rs[i], prob = p), k] <- 1L
            out
        }),
        "r1" = commsim(method="r1", binary=TRUE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            out <- array(0L, c(nr, nc, n))
            I <- seq_len(nr)
            storage.mode(cs) <- "double"
            for (k in seq_len(n))
                for (i in I)
                    out[i, sample.int(nc, rs[i], prob=cs), k] <- 1L
            out
        }),
        "r2" = commsim(method="r2", binary=TRUE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            out <- array(0L, c(nr, nc, n))
            p <- cs * cs
            I <- seq_len(nr)
            for (k in seq_len(n))
                for (i in I)
                    out[i, sample.int(nc, rs[i], prob=p), k] <- 1L
            out
        }),
        "quasiswap" = commsim(method="quasiswap", binary=TRUE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            if (nr < 2L || nc < 2L)
                stop("needs at least two items")
            out <- array(unlist(r2dtable(n, rs, cs)), c(nr, nc, n))
            storage.mode(out) <- "integer"
            .Call(do_qswap, out, n, thin, "quasiswap")
        }),
        "greedyqswap" = commsim(method="greedyqswap", binary=TRUE,
        isSeq=FALSE, mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            if (nr < 2L || nc < 2L)
                stop("needs at least two items")
            out <- array(unlist(r2dtable(n, rs, cs)), c(nr, nc, n))
            storage.mode(out) <- "integer"
            .Call(do_greedyqswap, out, n, thin, fill)
        }),
        "swap" = commsim(method="swap", binary = TRUE, isSeq=TRUE,
        mode = "integer",
        fun = function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            .Call(do_swap, as.matrix(x), n, thin, "swap")
        }),
        "tswap" = commsim(method="tswap", binary = TRUE, isSeq=TRUE,
        mode = "integer",
        fun = function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            .Call(do_swap, as.matrix(x), n, thin, "trialswap")
        }),
        "curveball" = commsim(method="curveball", binary = TRUE, isSeq=TRUE,
        mode = "integer",
        fun = function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            .Call(do_curveball, as.matrix(x), n, thin)
        }),
        "backtrack" = commsim(method="backtrack", binary = TRUE,
                               isSeq = FALSE, mode = "integer",
        fun = function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            .Call(do_backtrack, n, rs, cs)
        }),
        "r2dtable" = commsim(method="r2dtable", binary=FALSE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, cs, rs, rf, cf, s, fill, thin) {
            if (nr < 2L || nc < 2L)
                stop("needs at least two items")
            out <- array(unlist(r2dtable(n, rs, cs)), c(nr, nc, n))
            storage.mode(out) <- "integer"
            out
        }),
        "swap_count" = commsim(method="swap_count", binary = FALSE,
        isSeq=TRUE, mode = "integer",
        fun = function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            .Call(do_swap, as.matrix(x), n, thin, "swapcount")
        }),
        "quasiswap_count" = commsim(method="quasiswap_count", binary=FALSE,
        isSeq=FALSE, mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            if (nr < 2L || nc < 2L)
                stop("needs at least two items")
            out <- array(unlist(r2dtable(n, rs, cs)), c(nr, nc, n))
            storage.mode(out) <- "integer"
            .Call(do_qswap, out, n, fill, "rswapcount")
        }),
        "swsh_samp" = commsim(method="swsh_samp", binary=FALSE, isSeq=FALSE,
        mode="double",
        fun=function(x, n, nr, nc, cs, rs, rf, cf, s, fill, thin) {
            if (nr < 2L || nc < 2L)
                stop("needs at least two items")
            nz <- x[x > 0]
            out <- array(unlist(r2dtable(n, rf, cf)), c(nr, nc, n))
            ## do_qswap changes 'out' within the function
            .Call(do_qswap, out, n, thin, "quasiswap")
            storage.mode(out) <- "double"
            for (k in seq_len(n)) {
                out[,,k][out[,,k] > 0] <- sample(nz) # we assume that length(nz)>1
            }
            out
        }),
        "swsh_both" = commsim(method="swsh_both", binary=FALSE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, cs, rs, rf, cf, s, fill, thin) {
            if (nr < 2L || nc < 2L)
                stop("needs at least two items")
            indshuffle <- function(x) {
                drop(rmultinom(1, sum(x), rep(1, length(x))))
            }
            nz <- as.integer(x[x > 0])
            out <- array(unlist(r2dtable(n, rf, cf)), c(nr, nc, n))
            .Call(do_qswap, out, n, thin, "quasiswap")
            storage.mode(out) <- "integer"
            for (k in seq_len(n)) {
                out[,,k][out[,,k] > 0] <- indshuffle(nz - 1L) + 1L  # we assume that length(nz)>1
            }
            out
        }),
        "swsh_samp_r" = commsim(method="swsh_samp_r", binary=FALSE, isSeq=FALSE,
        mode="double",
        fun=function(x, n, nr, nc, cs, rs, rf, cf, s, fill, thin) {
            if (nr < 2L || nc < 2L)
                stop("needs at least two items")
            out <- array(unlist(r2dtable(n, rf, cf)), c(nr, nc, n))
            .Call(do_qswap, out, n, thin, "quasiswap")
            storage.mode(out) <- "double"
            I <- seq_len(nr)
            for (k in seq_len(n)) {
                for (i in I) {
                    nz <- x[i,][x[i,] > 0]
                    if (length(nz) == 1)
                        out[i,,k][out[i,,k] > 0] <- nz
                    if (length(nz) > 1)
                        out[i,,k][out[i,,k] > 0] <- sample(nz)
                }
            }
            out
        }),
        "swsh_samp_c" = commsim(method="swsh_samp_c", binary=FALSE, isSeq=FALSE,
        mode="double",
        fun=function(x, n, nr, nc, cs, rs, rf, cf, s, fill, thin) {
            if (nr < 2L || nc < 2L)
                stop("needs at least two items")
            out <- array(unlist(r2dtable(n, rf, cf)), c(nr, nc, n))
            .Call(do_qswap, out, n, thin, "quasiswap")
            storage.mode(out) <- "double"
            J <- seq_len(nc)
            for (k in seq_len(n)) {
                for (j in J) {
                    nz <- x[,j][x[,j] > 0]
                    if (length(nz) == 1)
                        out[,j,k][out[,j,k] > 0] <- nz
                    if (length(nz) > 1)
                        out[,j,k][out[,j,k] > 0] <- sample(nz)
                }
            }
            out
        }),
        "swsh_both_r" = commsim(method="swsh_both_r", binary=FALSE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, cs, rs, rf, cf, s, fill, thin) {
            if (nr < 2L || nc < 2L)
                stop("needs at least two items")
            indshuffle <- function(x) {
                drop(rmultinom(1, sum(x), rep(1, length(x))))
            }
            I <- seq_len(nr)
            out <- array(unlist(r2dtable(n, rf, cf)), c(nr, nc, n))
            .Call(do_qswap, out, n, thin, "quasiswap")
            storage.mode(out) <- "integer"
            for (k in seq_len(n)) {
                for (i in I) {
                    nz <- as.integer(x[i,][x[i,] > 0])
                    if (length(nz) == 1)
                        out[i,,k][out[i,,k] > 0] <- nz
                    if (length(nz) > 1)
                        out[i,,k][out[i,,k] > 0] <- indshuffle(nz - 1L) + 1L
                }
            }
            out
        }),
        "swsh_both_c" = commsim(method="swsh_both_c", binary=FALSE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, cs, rs, rf, cf, s, fill, thin) {
            if (nr < 2L || nc < 2L)
                stop("needs at least two items")
            indshuffle <- function(x) {
                drop(rmultinom(1, sum(x), rep(1, length(x))))
            }
            J <- seq_len(nc)
            out <- array(unlist(r2dtable(n, rf, cf)), c(nr, nc, n))
            .Call(do_qswap, out, n, thin, "quasiswap")
            storage.mode(out) <- "integer"
            for (k in seq_len(n)) {
                for (j in J) {
                    nz <- as.integer(x[,j][x[,j] > 0])
                    if (length(nz) == 1)
                        out[,j,k][out[,j,k] > 0] <- nz
                    if (length(nz) > 1)
                        out[,j,k][out[,j,k] > 0] <- indshuffle(nz - 1L) + 1L
                }
            }
            out
        }),
        "abuswap_r" = commsim(method="abuswap_r", binary=FALSE, isSeq=TRUE,
        mode="double",
        fun=function(x, n, nr, nc, cs, rs, rf, cf, s, fill, thin) {
            .Call(do_abuswap, as.matrix(x), n, thin, 1L)
        }),
        "abuswap_c" = commsim(method="abuswap_c", binary=FALSE, isSeq=TRUE,
        mode="double",
        fun=function(x, n, nr, nc, cs, rs, rf, cf, s, fill, thin) {
            .Call(do_abuswap, as.matrix(x), n, thin, 0L)
        }),
        "r00_samp" = commsim(method="r00_samp", binary=FALSE, isSeq=FALSE,
        mode="double",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            out <- matrix(0, nr * nc, n)
            for (k in seq_len(n))
                out[, k] <- sample(x)
            dim(out) <- c(nr, nc, n)
            out
        }),
        "c0_samp" = commsim(method="c0_samp", binary=FALSE, isSeq=FALSE,
        mode="double",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            out <- array(0, c(nr, nc, n))
            J <- seq_len(nc)
            for (k in seq_len(n))
                for (j in J)
                    out[, j, k] <- if (nr < 2)
                        x[,j] else sample(x[,j])
            out
        }),
        "r0_samp" = commsim(method="r0_samp", binary=FALSE, isSeq=FALSE,
        mode="double",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            out <- array(0, c(nr, nc, n))
            I <- seq_len(nr)
            for (k in seq_len(n))
                for (i in I)
                    out[i, , k] <- if (nc < 2)
                        x[i,] else sample(x[i,])
            out
        }),
        "r00_ind" = commsim(method="r00_ind", binary=FALSE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            indshuffle <- function(x) {
                drop(rmultinom(1, sum(x), rep(1, length(x))))
            }
            out <- matrix(0L, nr * nc, n)
            for (k in seq_len(n))
                out[, k] <- indshuffle(x)
            dim(out) <- c(nr, nc, n)
            out
        }),
        "c0_ind" = commsim(method="c0_ind", binary=FALSE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            indshuffle <- function(x) {
                drop(rmultinom(1, sum(x), rep(1, length(x))))
            }
            out <- array(0L, c(nr, nc, n))
            J <- seq_len(nc)
            for (k in seq_len(n))
                for (j in J)
                    out[, j, k] <- indshuffle(x[,j])
            out
        }),
        "r0_ind" = commsim(method="r0_ind", binary=FALSE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            indshuffle <- function(x) {
                drop(rmultinom(1, sum(x), rep(1, length(x))))
            }
            out <- array(0L, c(nr, nc, n))
            I <- seq_len(nr)
            for (k in seq_len(n))
                for (i in I)
                    out[i, , k] <- indshuffle(x[i,])
            out
        }),
        "r00_both" = commsim(method="r00_both", binary=FALSE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            indshuffle <- function(x) {
                drop(rmultinom(1, sum(x), rep(1, length(x))))
            }
            out <- matrix(0L, nr * nc, n)
            for (k in seq_len(n)) {
                out[,k][x > 0] <- indshuffle(x[x > 0] - 1L) + 1L
                out[,k] <- sample(out[,k])
            }
            dim(out) <- c(nr, nc, n)
            out
        }),
        "c0_both" = commsim(method="c0_both", binary=FALSE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            indshuffle <- function(x) {
                drop(rmultinom(1, sum(x), rep(1, length(x))))
            }
            out <- array(0L, c(nr, nc, n))
            J <- seq_len(nc)
            for (k in seq_len(n))
                for (j in J) {
                    if (sum(x[,j]) > 0) {
                        out[,j,k][x[,j] > 0] <- indshuffle(x[,j][x[,j] > 0] - 1L) + 1L
                        out[,j,k] <- if (nr < 2)
                            out[,j,k] else sample(out[,j,k])
                    }
                }
            out
        }),
        "r0_both" = commsim(method="r0_both", binary=FALSE, isSeq=FALSE,
        mode="integer",
        fun=function(x, n, nr, nc, rs, cs, rf, cf, s, fill, thin) {
            indshuffle <- function(x) {
                drop(rmultinom(1, sum(x), rep(1, length(x))))
            }
            out <- array(0L, c(nr, nc, n))
            I <- seq_len(nr)
            for (k in seq_len(n))
                for (i in I) {
                    if (sum(x[i,]) > 0) {
                        out[i,,k][x[i,] > 0] <- indshuffle(x[i,][x[i,] > 0] - 1L) + 1L
                        out[i,,k] <- if (nc < 2)
                            out[i,,k] else sample(out[i,,k])
                    }
                }
            out
        })
    )
    if (missing(method))
        return(names(algos))
    if (inherits(method, "commsim"))
        return(method)
    method <- match.arg(method, sort(names(algos)))
    algos[[method]]
}

`wisconsin` <-
    function(x)
{
    x <- decostand(x, "max", 2)
    mx <- attr(x, "parameters")$max
    x <- decostand(x, "total", 1)
    attr(x, "parameters")$max <- mx
    attr(x, "decostand") <- "wisconsin"
    x
}

### influence statistics for cca objects

## extract QR decomposition

`qr.cca` <-
    function(x, ...)
{
    if (is.null(r <- x$CCA$QR))
        stop("unconstrained model or rank zero: no 'qr' component")
    r
}

## residual degrees of freedom

`df.residual.cca` <-
    function(object, ...)
{
    nobs(object) - object$CCA$qrank - 1
}

## hat values need adjustment, because QR ignores Intercept

`hatvalues.cca` <-
    function(model, ...)
{
    rowSums(qr.Q(qr(model))^2) + weights(model)
}

`hatvalues.rda` <-
    function(model, ...)
{
    rowSums(qr.Q(qr(model))^2) + 1/nrow(qr(model)$qr)
}

## sigma gives the residual standard deviation. The only unambiguous
## sigma is the residual deviation for species, but for CANOCO like
## statistic this would be residual of WA/LC regression with little or
## no meaning.

`sigma.cca` <-
    function(object, type = c("response", "canoco"), ...)
{
    type <- match.arg(type)
    ## no response type in distance-based ordination
    if (inherits(object, c("dbrda", "capscale")) &&
        type == "response")
        stop("type = 'response' is not available in distance-based ordination")
    ## response: a vector of species (column) sigmata
    rdf <- df.residual(object)
    if (inherits(object, "rda"))
        adj <- nobs(object) - 1
    else
        adj <- 1
    if (type == "response") {
        sqrt(colSums(ordiYbar(object, "CA")^2 / rdf * adj))
    } else { # canoco has WA - LC regression
        sqrt((colSums(weights(object) * object$CCA$wa^2) - 1)/rdf)
    }
}

## rstandard and rstudent need sigma and have similar restrictions as
## sigma: it should be extractable and meaningful.

`rstandard.cca` <-
    function(model, type = c("response", "canoco"), ...)
{
    type <- match.arg(type)
    sd <- sigma(model, type = type)
    hat <- hatvalues(model)
    if (inherits(model, "rda"))
        adj <- sqrt(nobs(model) - 1)
    else
        adj <- 1
    w <- sqrt(weights(model))
    res <- switch(type,
                  "response" = ordiYbar(model, "CA") * adj,
                  "canoco" = w * (model$CCA$wa - model$CCA$u))
    res <- res / sqrt(1 - hat)
    res <- sweep(res, 2, sd, "/")
    res
}

## MASS book (4th ed), Chapter 6.3, p. 152 (e^star)

`rstudent.cca` <-
    function(model, type = c("response", "canoco"), ...)
{
    type <- match.arg(type)
    np <- df.residual(model)
    res <- rstandard(model, type = type)
    res / sqrt(pmax(np-res^2, 0)/(np-1))
}

## Cook's distance depends on meaningful sigma

`cooks.distance.cca` <-
    function(model, type = c("response", "canoco"), ...)
{
    hat <- hatvalues(model)
    p <- model$CCA$qrank
    rstandard(model, type = type)^2 * hat / (1 - hat) / (p + 1)
}

## residual sums of squares and products

`SSD.cca` <-
    function(object, type = "canoco", ...)
{
    w <- sqrt(weights(object))
    SSD <- crossprod(w * (object$CCA$wa - object$CCA$u))
    structure(list(SSD = SSD, call = object$call, df = df.residual(object)),
              class = "SSD")
}

## variances and covariances of coefficients. The sqrt(diag()) will be
## standard errors of regression coefficients, and for constrained
## ordination model m, the t-values of regression coefficients will be
## coef(m)/sqrt(diag(vcov(m))). The kind of relevant coefficient will
## be determined by the output of SSD.

`vcov.cca` <-
    function(object, type = "canoco", ...)
{
    QR <- qr(object)
    p <- 1L:QR$rank
    ## we do not give the (Intercept): it is neither in coef()
    cov.unscaled <- chol2inv(QR$qr[p, p])
    dimnames(cov.unscaled) <- list(colnames(QR$qr)[p], colnames(QR$qr)[p])
    ssd <- estVar(SSD(object, type = type))
    kronecker(ssd, cov.unscaled, make.dimnames = TRUE)
}

`text.ordiplot`  <-
    function (x, what, labels, select, arrows = FALSE, ...)
{
    sco <- scores(x, what)
    if (!missing(labels))
        rownames(sco) <- labels
    if (!missing(select))
        sco <- .checkSelect(select, sco)
    scoatt <- attr(sco, "score")
    if (!is.null(scoatt) && scoatt %in% c("biplot", "regression")) {
        arrows = TRUE
        sco <- sco * ordiArrowMul(sco)
    }
    if (arrows) {
        arrows(0, 0, sco[,1], sco[,2], ...)
        sco <- ordiArrowTextXY(sco, rownames(sco), rescale = FALSE)
    }
    text(sco, labels = rownames(sco), ...)
    invisible(x)
}

"predict.humpfit" <-
    function(object, newdata = NULL, ...)
{
    if (is.null(newdata))
        return(fitted(object))
    else {
        p <- coef(object)
        x <- unlist(newdata)
        x <- ifelse(x < p[1], x/p[1], p[1]*p[1]/x/x)
        fv <- p[3]*log(1 + p[2]*x/p[3])
    }
    fv
}

`plot.spantree` <-
    function (x, ord, cex = 0.7, type = "p", labels, dlim, FUN = sammon, 
              ...) 
{
    FUNname <- deparse(substitute(FUN))
    FUN <- match.fun(FUN)
    n <- x$n
    if (missing(ord)) {
        d <- cophenetic(x)
        if (any(d<=0))
            d[d<=0] <- min(d>0)/10
        if (!missing(dlim)) 
            d[d > dlim ] <- dlim
        if (n > 2) {
            ## sammon needs extra care, for other cases we just try FUN(d)
            if (FUNname == "sammon") {
                y <- cmdscale(d)
                dup <- duplicated(y)
                if (any(dup))
                    y[dup, ] <- y[dup,] + runif(2*sum(dup), -0.01, 0.01)
                ord <- FUN(d, y = y)
            } else
                ord <- FUN(d)
        } else
            ord <- cbind(seq_len(n), rep(0,n))
    }
    ord <- scores(ord, display = "sites", ...)
    ordiArgAbsorber(ord, asp = 1, type = "n", FUN = "plot", ...)
    lines(x, ord, ...)
    if (type == "p" || type == "b") 
        ordiArgAbsorber(ord, cex = cex, FUN = "points", ...)
    else if (type == "t") {
        if (missing(labels)) 
            labels <- x$labels
        x <- scores(ord, display = "sites", ...)
        ordiArgAbsorber(x, labels = labels, cex = cex, FUN = "ordilabel", ...)
    }
    ord <- list(sites = ord)
    class(ord) <- "ordiplot"
    invisible(ord)
}

`monoMDS` <-
    function(dist, y, k = 2,
             model = c("global", "local", "linear", "hybrid"),
             threshold = 0.8, maxit = 200, weakties = TRUE, stress = 1,
             scaling = TRUE, pc = TRUE, smin = 1e-4, sfgrmin = 1e-7,
             sratmax=0.99999, ...)
{
    ## Check that 'dist' are distances or a symmetric square matrix
    if (!(inherits(dist, "dist") ||
              ((is.matrix(dist) || is.data.frame(dist)) &&
                   isSymmetric(unname(as.matrix(dist))))))
        stop("'dist' must be a distance object (class \"dist\") or a symmetric square matrix")
    if (any(dist < -sqrt(.Machine$double.eps), na.rm = TRUE))
        warning("some dissimilarities are negative -- is this intentional?")
    ## match.arg
    model <- match.arg(model)
    ## save 'dist' attributes to display in print()
    distmethod <- attr(dist, "method")
    if (is.null(distmethod))
        distmethod <- "unknown"
    distcall <-attr(dist, "call")
    if (!is.null(distcall))
        distcall <- deparse(distcall)
    ## dist to mat
    mat <- as.matrix(dist)
    nm <- rownames(mat)
    if (model %in% c("global", "linear")) {
        ## global NMDS: lower triangle
        dist <- mat[lower.tri(mat)]
        iidx <- row(mat)[lower.tri(mat)]
        jidx <- col(mat)[lower.tri(mat)]
        ## Remove missing values
        if (any(nas <- is.na(dist))) {
            dist <- dist[!nas]
            iidx <- iidx[!nas]
            jidx <- jidx[!nas]
        }
        ## non-metric/metric: Fortran parameter 'iregn'
        if (model == "global")
            iregn <- 1
        else
            iregn <- 2
        ngrp <- 1
        nobj <- nrow(mat)
        istart <- 1
    } else if (model == "local") {
        ## local NMDS: whole matrix without the diagonal, and rows in
        ## a row (hence transpose)
        mat <- t(mat)
        ## Get missing values
        nas <- is.na(mat)
        ## groups by rows, except missing values
        rs <- rowSums(!nas) - 1
        istart <- cumsum(rs)
        istart <- c(0, istart[-length(istart)]) + 1
        ## Full matrix expect the diagonal
        dist <- mat[col(mat) != row(mat)]
        iidx <- col(mat)[col(mat) != row(mat)]  # transpose!
        jidx <- row(mat)[col(mat) != row(mat)]
        ## Remove missing values
        if (any(nas)) {
            nas <- nas[col(mat) != row(mat)]
            dist <- dist[!nas]
            iidx <- iidx[!nas]
            jidx <- jidx[!nas]
        }
        iregn <- 1
        nobj <- nrow(mat)
        ngrp <- nobj
    } else if (model == "hybrid") {
        ## Hybrid NMDS: two lower triangles, first a complete one,
        ## then those with dissimilarities below the threshold
        dist <- mat[lower.tri(mat)]
        iidx <- row(mat)[lower.tri(mat)]
        jidx <- col(mat)[lower.tri(mat)]
        ## Missing values
        if (any(nas <- is.na(dist))) {
            dist <- dist[!nas]
            iidx <- iidx[!nas]
            jidx <- jidx[!nas]
        }
        ## second group: dissimilarities below threshold
        ngrp <- 2
        istart <- c(1, length(dist) + 1)
        take <- dist < threshold
        dist <- c(dist, dist[take])
        iidx <- c(iidx, iidx[take])
        jidx <- c(jidx, jidx[take])
        iregn <- 3
        nobj <- nrow(mat)
    }
    ## ndis: number of >0 dissimilarities (distinct points)
    ndis <- length(dist)
    ## starting configuration
    if (missing(y)) {
        y <- matrix(runif(nobj*k, -1, 1), nobj, k)
        ## centre
        y <- sweep(y, 2, colMeans(y), "-")
    }
    ## y to vector
    y <- as.vector(as.matrix(y))
    ## translate R args to Fortran call
    if (weakties)
        ities <- 1
    else
        ities <- 2
    ## Fortran call -- must quoted because monoMDS has mixed case (or
    ## can be FIXED)
    sol <- .Fortran("monoMDS", nobj = as.integer(nobj), nfix=as.integer(0),
                 ndim = as.integer(k), ndis = as.integer(ndis),
                 ngrp = as.integer(ngrp), diss = as.double(dist),
                 iidx = as.integer(iidx), jidx = as.integer(jidx),
                 xinit = as.double(y), istart = as.integer(istart),
                 isform = as.integer(stress), ities = as.integer(ities),
                 iregn = as.integer(iregn), iscal = as.integer(scaling),
                 maxits = as.integer(maxit),
                 sratmx = as.double(sratmax), strmin = as.double(smin),
                 sfgrmn = as.double(sfgrmin), dist = double(ndis),
                 dhat = double(ndis), points = double(k*nobj),
                 stress = double(1), grstress = double(ngrp),
                 iters = integer(1), icause = integer(1))
    sol$call <- match.call()
    sol$model <- model
    sol$points <- matrix(sol$points, nobj, k)
    if (pc)
        sol$points <- prcomp(sol$points)$x
    attr(sol$points, "pc") <- pc
    rownames(sol$points) <- nm
    colnames(sol$points) <- paste("MDS", 1:k, sep="")
    ## save info on dissimilarities
    sol$distmethod <- distmethod
    sol$distcall <- distcall
    class(sol) <- "monoMDS"
    sol
}

### Internal function for Mahalanobis transformation of the matrix.
### Mahalanobis transformation of matrix X is M = X S^(-1/2) where S
### is the covariance matrix. The inverse square root of S is found
### via eigen decomposition S = G L G^T, where G is the matrix of
### eigenvectors, and L is the diagonal matrix of eigenvalues. Thus
### S^(-1/2) = G L^(-1/2) G^T. This is an internal function so that
### input must be correct: 'x' must be a centred matrix (not a
### data.frame, not raw data).
`veganMahatrans` <-
    function (x, s2, tol = sqrt(.Machine$double.eps))
{
    if (missing(s2))
        s2 <- cov(x)
    e <- eigen(s2, symmetric = TRUE)
    k <- e$values > max(tol, tol * e$values[1L])
    sisqr <- e$vectors[,k, drop=FALSE] %*%
        (sqrt(1/e$values[k]) * t(e$vectors[,k, drop = FALSE]))
    x %*% sisqr
}

### tolerance method for decorana
###
### After rescaling, all values should be 1

`tolerance.decorana` <-
    function(x, data, choices = 1:4, which = c("sites", "species"),
             useN2 = TRUE, ...)
{
    ## community data cannot be reconstructed from the result x. We
    ## use either the one supplied in 'data' or try to find data used
    ## in decorana, but then we check the plausibility.
    if (missing(data))
        data <- eval(x$call$veg)
    which <- match.arg(which)
    ## Native decorana scaling (sites are WA of species) does not
    ## allow useN2 with species (but this can be done after scaling of
    ## results, and therefore the code below is ready for this).
    if (useN2 && which == "species")
        warning("useN2 is not implemented for species")
    EPS <- sqrt(.Machine$double.eps)
    ## transform data like decorana did
    if (!is.null(x$before))
        data <- beforeafter(data, x$before, x$after)
    if (x$iweigh)
        data <- downweight(data, x$fraction)
    ## see if data are plausible given decorana solution
    if (nrow(data) != nrow(x$rproj))
        stop("'data' have wrong row dimension")
    if (ncol(data) != nrow(x$cproj))
        stop("'data' have wrong col dimension")
    ## check the first eigenvalue
    ev1 <- svd(initCA(data), nv=0, nu=0)$d[1]^2
    ev0 <- if (x$ira) x$evals[1] else x$evals.decorana[1]
    if (abs(ev1 - ev0) > 100 * EPS)
        stop("'data' are not plausible given 'decorana' result")
    ## preliminaries over: start working
    res <- switch(which,
                  "sites" = x$rproj,
                  "species" = x$cproj)
    tot <- switch(which,
                  "sites" = rowSums(data),
                  "species" = colSums(data))
    ## go over axes
    for(i in choices) {
        X <- data * outer(x$rproj[,i], x$cproj[,i], "-")^2
        if (which == "species")
            X <- t(X)
        res[,i] <- sqrt(rowSums(X)/tot)
    }
    ## N2 depends only on rows. Here and we still handle species,
    ## since support can be added later, though skipped now.
    if (useN2 && which != "species") {
        y <- switch(which,
                    "sites" = data,
                    "species" = t(data))
        y <- (y / rowSums(y))^2
        N2 <- 1 / rowSums(y, na.rm = TRUE) # 1/H
        N2[abs(N2 - 1) < EPS] <- 1
        res <- res / sqrt(pmax.int(1 - 1/N2, 0))
    }
    res <- res[,choices, drop=FALSE]
    res[!is.finite(res) | res < EPS] <- 0
    class(res) <- c("tolerance.decorana", "tolerance.cca", "tolerance", "matrix")
    attr(res, "which") <- which
    attr(res, "scaling") <- "decorana"
    attr(res, "N2") <- if (useN2 && which != "species") N2 else NA
    res
}

`print.cca` <-
    function (x, digits = max(3, getOption("digits") - 3), ...)
{
    writeLines(strwrap(pasteCall(x$call)))
    cat("\n")
    chi <- c(x$tot.chi, x$pCCA$tot.chi, x$CCA$tot.chi, x$CA$tot.chi)
    props <- chi/chi[1]
    rnk <- c(NA, x$pCCA$rank, x$CCA$rank, x$CA$rank)
    ## report no. of real axes in dbrda if any negative eigenvalues
    if (inherits(x, "dbrda") &&
        (!is.null(x$CCA) && x$CCA$poseig < x$CCA$qrank ||
             !is.null(x$CA) && x$CA$poseig < x$CA$rank))
        poseig <- c(NA, if (!is.null(x$pCCA)) NA, x$CCA$poseig, x$CA$poseig)
    else
        poseig <- NULL
    tbl <- cbind(chi, props, rnk, poseig)
    if (!is.null(x$CA$imaginary.chi))
        tbl <- rbind(tbl, c(x$CA$imaginary.chi,
                            x$CA$imaginary.chi/x$tot.chi,
                            x$CA$imaginary.rank, NULL))
    colnames(tbl) <- c("Inertia", "Proportion", "Rank",
                       if (!is.null(poseig)) "RealDims")
    rn <- c("Total", "Conditional", "Constrained", "Unconstrained",
            "Imaginary")
    rownames(tbl) <- rn[c(TRUE,!is.null(x$pCCA),
                          !is.null(x$CCA),  !is.null(x$CA),
                          !is.null(x$CA$imaginary.chi))]
    ## Remove "Proportion" if only one component
    if (is.null(x$CCA) && is.null(x$pCCA))
        tbl <- tbl[,-2]
    ## 'cs' columns before "Rank" are non-integer
    cs <- which(colnames(tbl) == "Rank") - 1
    printCoefmat(tbl, digits = digits, na.print = "", cs.ind = seq_len(cs))
    cat("Inertia is", x$inertia, "\n")
    ## data used for species scores in db ordination
    if (!is.null(x$vdata))
        cat("Species scores projected from", sQuote(x$vdata), "\n")
    if (!is.null(x$CCA$alias))
        cat("Some constraints were aliased because they were collinear (redundant)\n")
    ## Report removed observations and species
    if (!is.null(x$na.action))
        cat(naprint(x$na.action), "\n")
    sp.na <- if (is.null(x$CCA)) attr(x$CA$v, "na.action")
    else attr(x$CCA$v, "na.action")
    if (!is.null(sp.na))
        cat(length(sp.na), "species",
            ifelse(length(sp.na)==1, "(variable)", "(variables)"),
            "deleted due to missingness\n")
    if (!is.null(x$CCA) && x$CCA$rank > 0) {
        cat("\nEigenvalues for constrained axes:\n")
        print(zapsmall(x$CCA$eig, digits = digits), ...)
    }
    if (!is.null(x$CA) && x$CA$rank > 0) {
        ax.lim <- 8
        ax.trig <- 16
        cat("\nEigenvalues for unconstrained axes:\n")
        if (x$CA$rank > ax.trig) {
            print(zapsmall(x$CA$eig[1:ax.lim], digits = digits), ...)
            cat("(Showing", ax.lim, "of", x$CA$rank,
                "unconstrained eigenvalues)\n")
        }
        else print(zapsmall(x$CA$eig, digits = digits), ...)
    }
    cat("\n")
    if (inherits(x, c("capscale", "dbrda"))) {
        if (!is.null(x$metaMDSdist))
            cat("metaMDSdist transformed data:", x$metaMDSdist, "\n\n")
        if (!is.null(x$ac))
            cat("Constant added to distances:", x$ac, "\n\n")
    }
    invisible(x)
}

### package klaR has another function called rda(), and to avoid using
### klar:::print.rda instead of delegating to vegan:::print.cca we
### define here a (redundant) print.rda

`print.rda` <-
    function(x, ...)
{
    ## not vegan rda?
    if (!("CA" %in% names(x)))
        stop(gettextf("%s is not a vegan rda object",
                      sQuote(deparse(substitute(x)))))
    NextMethod("print", x, ...)
}

"print.radline" <-
    function (x, ...) 
{
    cat("\nRAD model:", x$model, "\n")
    cat("Family:", family(x)$family, "\n")
    cat("No. of species: ", length(x$y), "\nTotal abundance:", 
        sum(x$y), "\n\n")
    p <- coef(x)
    dev <- deviance(x)
    AIC <- AIC(x)
    BIC <- AIC(x, k = log(length(x$y)))
    tmp <- c(p, dev, AIC, BIC)
    names(tmp) <- c(names(p), "Deviance", "AIC", "BIC")
    print(tmp, ...)
    invisible(x)
}

`distconnected` <-
    function(dis, toolong = 1, trace = TRUE)
{
    n <- attr(dis, "Size")
    out <- .C(stepabyss, dis = as.double(dis), n = as.integer(n),
              toolong = as.double(toolong), val = integer(n),
              NAOK = TRUE)$val
    if (trace) {
        cat("Connectivity of distance matrix with threshold dissimilarity",
            toolong,"\n")
        n <- length(unique(out))
        if (n == 1)
            cat("Data are connected\n")
        else {
            cat("Data are disconnected:", n, "groups\n")
            print(table(out, dnn="Groups sizes"))
        }
    }
    out
}

### Casts a vegan spantree object into single linkage dendrogram of
### class hclust. The non-trivial items in "hclust" object are a
### 'merge' matrix for fusions of points and/or clusters, a 'height'
### vector which gives the heights of each fusion, and an 'order'
### vector that gives the order of leaves in the plotted
### dendrogram. The 'height's are only sorted spantree segment
### distances, but for 'merge' we need to establish cluster
### memberships, and for 'order' we must traverse the tree.

`as.hclust.spantree` <-
    function(x, ...)
{
    ## Order by the lengths of spanning tree links
    o <- order(x$dist)
    npoints <- x$n
    if(npoints < 2)
        stop("needs at least two points")
    ## Ordered indices of dads and kids
    dad <- (2:npoints)[o]
    kid <- x$kid[o]
    ## merge matrix of hclust has negative index when a single point
    ## is added to a tree and a positive index when a group is joined
    ## to a tree, and the group is numbered by the level it was
    ## formed.
    labs <- -seq_len(npoints)
    merge <- matrix(0, nrow=npoints-1, ncol=2)
    for(i in 1:nrow(merge)) {
        merge[i, ] <- c(labs[dad[i]], labs[kid[i]])
        ## update labs for the current group and its kids
        labs[labs %in% labs[c(dad[i], kid[i])]] <- i
    }

    order <- hclustMergeOrder(merge)

    out <- list(merge = merge, height = x$dist[o], order = order,
                labels = x$labels, method = "spantree", call =
                match.call())
    class(out) <- "hclust"
    out
}

### Internal vegan function to get the 'order' from a merge matrix of
### an hclust tree

`hclustMergeOrder` <-
    function(merge)
{
    ## Get order of leaves with recursive search from the root
    order <- numeric(nrow(merge)+1)
    ind <- 0
    ## "<<-" updates data only within hclustMergeOrder, but outside
    ## the visit() function.
    visit <- function(i, j) {
        if (merge[i,j] < 0) {
            ind <<- ind+1
            order[ind] <<- -merge[i,j]
        } else {
            visit(merge[i,j], 1)
            visit(merge[i,j], 2)
        }
    }
    visit(nrow(merge), 1)
    visit(nrow(merge), 2)
    return(order)
}

### Reorder an hclust tree. Basic R provides reorder.dendrogram, but
### this functoin works with 'hclust' objects, and also differs in
### implementation. We use either weighted mean, min or max or
### sum. The dendrogram is always ordered in ascending order, so that
### with max the left kid always has lower value. So with 'max' the
### largest value is smaller in leftmost group. The choice 'sum'
### hardly makes sense, but it is the default in
### reorder.dendrogram. The ordering with 'mean' differs from
### reorder.dendrogram which uses unweighted means, but here we weight
### means by group sizes so that the mean of an internal node is the
### mean of its leaves.

`reorder.hclust` <-
    function(x, wts,
             agglo.FUN = c("mean", "min", "max", "sum", "uwmean"),
             ...)
{
    agglo.FUN <- match.arg(agglo.FUN)
    merge <- x$merge
    nlev <- nrow(merge)
    stats <- numeric(nlev)
    counts <- numeric(nlev)
    pair <- numeric(2)
    pairw <- numeric(2)
    ## Go through merge, order each level and update the statistic.
    for(i in 1:nlev) {
        for(j in 1:2) {
            if (merge[i,j] < 0) {
                pair[j] <- wts[-merge[i,j]]
                pairw[j] <- 1
            } else {
                pair[j] <- stats[merge[i,j]]
                pairw[j] <- counts[merge[i,j]]
            }
        }
        ## reorder
        merge[i,] <- merge[i, order(pair)]
        ## statistic for this merge level
        stats[i] <-
            switch(agglo.FUN,
                   "mean" = weighted.mean(pair, pairw),
                   "min" = min(pair),
                   "max" = max(pair),
                   "sum" = sum(pair),
                   "uwmean" = mean(pair))
        counts[i] <- sum(pairw)
    }
    ## Get the 'order' of the reordered dendrogram
    order <- hclustMergeOrder(merge)
    x$merge <- merge
    x$order <- order
    x$value <- stats
    x
}

### Trivial function to reverse the order of an hclust tree (why this
### is not in base R?)

`rev.hclust` <-
    function(x)
{
    x$order <- rev(x$order)
    x
}

### Get coordinates for internal or terminal nodes (leaves) that would
### be used in plot.hclust

`scores.hclust` <-
    function(x, display = "internal", ...)
{
    extnam <- c("leaves", "terminal")
    intnam <- c("internal")
    display <- match.arg(display, c(extnam, intnam))
    ## Terminal nodes (leaves): plot.hclust scales x-axis for n points
    ## as 1..n. The y-value is the 'height' where the terminal node
    ## was fused to the tree.
    if(display %in% extnam) {
        merge <- x$merge
        y <- numeric(nrow(merge) + 1)
        for(i in 1:nrow(merge))
            for(j in 1:2)
                if(merge[i,j] < 0)
                    y[-merge[i,j]] <- x$height[i]
        xx <- order(x$order)
        xy <- cbind(`x` = xx, `height` = y)
    } else {
        ## Internal nodes are given in the order they were fused which
        ## also is the order of 'height'
        xx <- reorder(x, order(x$order), agglo.FUN = "uwmean")$value
        xy <- cbind(`x`= xx, `height` = x$height)
    }
    xy
}

## variant of stats::cutree() that numbers the clusters in the order
## they appear in the dendrogram (left to right). The stats::cutree
## numbers the clusters in the order the elements appear in the data
## set.

`cutreeord` <-
    function(tree, k = NULL, h = NULL)
{
    cut <- cutree(tree, k, h)
    ## order of classes in the tree
    if (!is.matrix(cut)) {
        cut <- order(unique(cut[tree$order]))[cut]
        names(cut) <- tree$labels
    } else {
        for(i in seq_len(ncol(cut))) {
            cut[,i] <- order(unique(cut[tree$order,i]))[cut[,i]]
        }
    }
    cut
}

`betadisper` <-
    function(d, group, type = c("median","centroid"), bias.adjust=FALSE,
             sqrt.dist = FALSE, add = FALSE)
{
    ## inline function for double centring. We used .C("dblcen", ...,
    ## PACKAGE = "stats") which does not dublicate its argument, but
    ## it was removed from R in r60360 | ripley | 2012-08-22 07:59:00
    ## UTC (Wed, 22 Aug 2012) "more conversion to .Call, clean up".
    dblcen <- function(x, na.rm = TRUE) {
        cnt <- colMeans(x, na.rm = na.rm)
        x <- sweep(x, 2L, cnt, check.margin = FALSE)
        cnt <- rowMeans(x, na.rm = na.rm)
        sweep(x, 1L, cnt, check.margin = FALSE)
    }
    ## inline function for spatial medians
    spatialMed <- function(vectors, group, pos) {
        axes <- seq_len(NCOL(vectors))
        spMedPos <- ordimedian(vectors, group, choices = axes[pos])
        spMedNeg <- ordimedian(vectors, group, choices = axes[!pos])
        cbind(spMedPos, spMedNeg)
    }
    ## inline function for centroids
    centroidFUN <- function(vec, group) {
        cent <- apply(vec, 2,
                      function(x, group) tapply(x, INDEX = group, FUN = mean),
                      group = group)
        if(!is.matrix(cent)) { ## if only 1 group, cent is vector
            cent <- matrix(cent, nrow = 1,
                           dimnames = list(as.character(levels(group)),
                           paste0("Dim", seq_len(NCOL(vec)))))
        }
        cent
    }
    ## inline function for distance computation
    Resids <- function(x, c) {
        if(is.matrix(c))
            d <- x - c
        else
            d <- sweep(x, 2, c)
        rowSums(d^2)
    }
    ## Tolerance for zero Eigenvalues
    TOL <- sqrt(.Machine$double.eps)
    ## uses code from stats:::cmdscale by R Core Development Team
    if(!inherits(d, "dist"))
        stop("distances 'd' must be a 'dist' object")
    ## Someone really tried to analyse correlation like object in range -1..+1
    if (any(d < -TOL, na.rm = TRUE))
        stop("dissimilarities 'd' must be non-negative")
    ## adjust to avoid negative eigenvalues (if they disturb you)
    if (sqrt.dist)
        d <- sqrt(d)
    if (is.logical(add) && isTRUE(add))
        add <- "lingoes"
    if (is.character(add)) {
        add <- match.arg(add, c("lingoes", "cailliez"))
        if (add == "lingoes") {
            ac <- addLingoes(as.matrix(d))
            d <- sqrt(d^2 + 2 * ac)
        }
        else if (add == "cailliez") {
            ac <- addCailliez(as.matrix(d))
            d <- d + ac
        }
    }
    if(missing(type))
        type <- "median"
    type <- match.arg(type)
    ## checks for groups - need to be a factor for later
    group <- if(!is.factor(group)) {
        as.factor(group)
    } else { ## if already a factor, drop empty levels
        droplevels(group, exclude = NA) # need exclude = NA under Rdevel r71113
    }
    n <- attr(d, "Size")
    x <- matrix(0, ncol = n, nrow = n)
    x[row(x) > col(x)] <- d^2
    ## site labels
    labs <- attr(d, "Labels")
    ## remove NAs in group
    if(any(gr.na <- is.na(group))) {
        group <- group[!gr.na]
        x <- x[!gr.na, !gr.na]
        ## update n otherwise C call crashes
        n <- n - sum(gr.na)
        ## update labels
        labs <- labs[!gr.na]
        message("missing observations due to 'group' removed")
    }
    ## remove NA's in d
    if(any(x.na <- apply(x, 1, function(x) any(is.na(x))))) {
        x <- x[!x.na, !x.na]
        group <- group[!x.na]
        ## update n otherwise C call crashes
        n <- n - sum(x.na)
        ## update labels
        labs <- labs[!x.na]
        message("missing observations due to 'd' removed")
    }
    x <- x + t(x)
    x <- dblcen(x)
    e <- eigen(-x/2, symmetric = TRUE)
    vectors <- e$vectors
    eig <- e$values
    ## Remove zero eigenvalues
    eig <- eig[(want <- abs(eig) > max(TOL, TOL * eig[1L]))]
    ## scale Eigenvectors
    vectors <- vectors[, want, drop = FALSE] %*% diag(sqrt(abs(eig)),
                               nrow = length(eig))
    ## store which are the positive eigenvalues
    pos <- eig > 0
    ## group centroids in PCoA space
    centroids <-
        switch(type,
               centroid = centroidFUN(vectors, group),
               median = spatialMed(vectors, group, pos)
               )
    ## for each of the groups, calculate distance to centroid for
    ## observation in the group
    ## Uses in-line Resids function as we want LAD residuals for
    ## median method, and LSQ residuals for centroid method
    dist.pos <- Resids(vectors[, pos, drop=FALSE],
                       centroids[group, pos, drop=FALSE])
    dist.neg <- 0
    if(any(!pos))
        dist.neg <- Resids(vectors[, !pos, drop=FALSE],
                           centroids[group, !pos, drop=FALSE])

    ## zij are the distances of each point to its group centroid
    if (any(dist.neg > dist.pos)) {
        ## Negative squared distances give complex valued distances:
        ## take only the real part (which is zero). Github issue #306.
        warning("some squared distances are negative and changed to zero")
        zij <- Re(sqrt(as.complex(dist.pos - dist.neg)))
    } else {
        zij <- sqrt(dist.pos - dist.neg)
    }
    if (bias.adjust) {
        n.group <- as.vector(table(group))
        zij <- zij*sqrt(n.group[group]/(n.group[group]-1))
    }
    ## add in correct labels
    colnames(vectors) <- names(eig) <- paste("PCoA", seq_along(eig), sep = "")
    if(is.matrix(centroids))
        colnames(centroids) <- names(eig)
    else
        names(centroids) <- names(eig)
    rownames(vectors) <- names(zij) <- labs
    retval <- list(eig = eig, vectors = vectors, distances = zij,
                   group = group, centroids = centroids, call = match.call())
    class(retval) <- "betadisper"
    attr(retval, "method") <- attr(d, "method")
    attr(retval, "type") <- type
    attr(retval, "bias.adjust") <- bias.adjust
    retval
}

`vegemite` <-
    function (x, use, scale, sp.ind = NULL, site.ind = NULL, zero = ".",
              select, ...)
{
    if (!missing(use)) {
        if (!is.list(use) && is.vector(use)) {
            if (is.null(site.ind))
                site.ind <- order(use)
            if (is.null(sp.ind))
                sp.ind <- order(wascores(use, x))
        }
        else if (inherits(use, c("hclust", "twins"))) {
            if (inherits(use, "twins")) {
                use <- as.hclust(use)
            }
            if (is.null(site.ind))
                site.ind <- use$order
            if (is.null(sp.ind))
                sp.ind <- order(wascores(order(site.ind), x))
        }
        else if (inherits(use, "dendrogram")) {
            if (is.null(site.ind)) {
                site.ind <- 1:nrow(x)
                names(site.ind) <- rownames(x)
                site.ind <- site.ind[labels(use)]
            }
            if (is.null(sp.ind))
                sp.ind <- order(wascores(order(site.ind), x))
        }
        else if (is.list(use)) {
            tmp <- scores(use, choices = 1, display = "sites")
            if (is.null(site.ind))
                site.ind <- order(tmp)
            if (is.null(sp.ind))
                sp.ind <- try(order(scores(use, choices = 1,
                                           display = "species")))
            if (inherits(sp.ind, "try-error"))
                sp.ind <- order(wascores(tmp, x))
        }
        else if (is.matrix(use)) {
            tmp <- scores(use, choices = 1, display = "sites")
            if (is.null(site.ind))
                site.ind <- order(tmp)
            if (is.null(sp.ind))
                sp.ind <- order(wascores(tmp, x))
        }
    }
    if (!is.null(sp.ind) && is.logical(sp.ind))
        sp.ind <- seq_len(ncol(x))[sp.ind]
    if (!is.null(site.ind) && is.logical(site.ind))
        site.ind <- seq_len(nrow(x))[site.ind]
    if (is.null(sp.ind))
        sp.ind <- seq_len(ncol(x))
    if (is.null(site.ind))
        site.ind <- seq_len(nrow(x))
    if (!missing(select)) {
        if (!is.logical(select))
            select <- sort(site.ind) %in% select
        stake <- colSums(x[select, , drop = FALSE]) > 0
        site.ind <- site.ind[select[site.ind]]
        site.ind <- site.ind[!is.na(site.ind)]
    }
    else {
        stake <- colSums(x[site.ind, ]) > 0
    }
    sp.ind <- sp.ind[stake[sp.ind]]
    x <- x[site.ind, sp.ind]
    if (!missing(scale))
        x <- coverscale(x, scale, ...)
    usedscale <- attr(x, "scale")
    if (any(apply(x, 1, nchar) > 1))
        stop("cowardly refusing to use longer than one-character symbols:\nUse scale")
    x <- as.matrix(x)
    x <- t(x)
    sp.nam <- rownames(x)
    sp.len <- max(nchar(sp.nam))
    nst <- ncol(x)
    page.width <- getOption("width")
    per.page <- page.width - sp.len - 3
    istart <- seq(1, nst, by = per.page)
    iend <- pmin(istart + per.page - 1, nst)
    for (st in seq_along(istart)) {
        tbl <- apply(x[, istart[st]:iend[st], drop = FALSE],
                     1, paste, sep = "", collapse = "")
        names(tbl) <- NULL
        tbl <- gsub("0", zero, tbl)
        tbl <- cbind(sp.nam, tbl)
        st.nam <- colnames(x)[istart[st]:iend[st]]
        nlen <- max(nchar(st.nam))
        mathead <- matrix(" ", nrow = length(st.nam), ncol = nlen)
        for (i in seq_along(st.nam)) {
            tmp <- unlist(strsplit(st.nam[i], NULL))
            start <- nlen - length(tmp) + 1
            mathead[i, start:nlen] <- tmp
        }
        head <- cbind(apply(mathead, 2, paste, sep = "", collapse = ""))
        tbl <- rbind(cbind(matrix(" ", nrow = nrow(head), 1),
                           head), tbl)
        d <- list()
        l <- 0
        for (i in dim(tbl)) {
            d[[l <- l + 1]] <- rep("", i)
        }
        dimnames(tbl) <- d
        print(noquote(tbl))
    }
    out <- list(sites = site.ind, species = sp.ind)
    print(sapply(out, length))
    if (!is.null(usedscale))
        cat("scale: ",  usedscale, "\n")
    invisible(out)
}

`specpool` <-
    function (x, pool, smallsample = TRUE)
{
    x <- as.matrix(x)
    if (!(is.numeric(x) || is.logical(x)))
        stop("input data must be numeric")
    if (missing(pool))
        pool <- rep("All", nrow(x))
    ## check dims
    if (length(pool) != NROW(x))
        stop("length of 'pool' and number rows in 'x' do not match")
    ## remove missing values
    if (any(nas <- is.na(pool))) {
        pool <- pool[!nas]
        x <- x[!nas, , drop = FALSE]
    }
    out <- seq(1:nrow(x))
    groups <- table(pool)
    inds <- names(groups)
    S <- var.chao <- chao <- var.jack1 <- jack.1 <- jack.2 <- var.boot <- bootS <-
        rep(NA, length(inds))
    names(S) <- names(var.chao) <- names(chao) <- names(var.jack1) <-
        names(jack.1) <- names(jack.2) <- names(var.boot) <- names(bootS) <- inds
    for (is in inds) {
        a1 <- a2 <- NA
        gr <- out[pool == is]
        n <- length(gr)
        if (n <= 0)
            next
        if (smallsample)
            ssc <- (n-1)/n
        else
            ssc <- 1
        X <- x[gr, , drop = FALSE]
        freq <- colSums(X > 0)
        p <- freq[freq > 0]/n
        S[is] <- sum(freq > 0)
        if (S[is] == 0)
            next
        if (n >= 1L)
            a1 <- sum(freq == 1)
        if (n >= 2L)
            a2 <- sum(freq == 2)
        else
            a2 <- 0
        chao[is] <- S[is] + if(!is.na(a2) && a2 > 0)
            ssc * a1 * a1/2/a2
        else
            ssc * a1 * (a1-1)/2
        jack.1[is] <- S[is] + a1 * (n - 1)/n
        if (n > 1L)
            jack.2[is] <- S[is] + a1 * (2 * n - 3)/n -
                a2 * (n - 2)^2/n/(n - 1)
        else
            jack.2[is] <- S[is]
        bootS[is] <- S[is] + sum((1 - p)^n)
        aa <- if (!is.na(a2) && a2 > 0) a1/a2
              else 0
        if (!is.na(a2) && a2 > 0)
            var.chao[is] <- a1 * ssc * (0.5 + ssc * (1 + aa/4) * aa) * aa
        else
            var.chao[is] <-
                ssc * (ssc * (a1*(2*a1-1)^2/4 - a1^4/chao[is]/4) + a1*(a1-1)/2)
        if (!is.na(a1) && a1 > 0) {
            jf <- table(rowSums(X[, freq == 1, drop = FALSE] >
                                0))
            var.jack1[is] <- (sum(as.numeric(names(jf))^2 * jf) -
                              a1/n) * (n - 1)/n
        } else {
            var.jack1[is] <- 0
        }
        pn <- (1 - p)^n
        X <- X[, freq > 0, drop = FALSE]
        Zp <- (crossprod(X == 0)/n)^n - outer(pn, pn, "*")
        var.boot[is] <- sum(pn * (1 - pn)) + 2 * sum(Zp[lower.tri(Zp)])
    }
    out <- list(Species = S, chao = chao,  chao.se = sqrt(var.chao),
                jack1 = jack.1, jack1.se = sqrt(var.jack1), jack2 = jack.2,
                boot = bootS, boot.se = sqrt(var.boot), n = as.vector(groups))
    out <- as.data.frame(out)
    attr(out, "pool") <- pool
    out
}

`screeplot.princomp` <-
    function(x, bstick = FALSE,
             type = c("barplot", "lines"),
             npcs = min(10, length(x$sdev)),
             ptype = "o", bst.col = "red", bst.lty = "solid",
             xlab = "Component", ylab = "Inertia",
             main = deparse(substitute(x)), legend = bstick, ...)
{
    type <- match.arg(type)
    eig.vals <- x$sdev^2
    ncomps <- length(eig.vals)
    if(npcs > ncomps)
        npcs <- ncomps
    comps <- seq(len=npcs)
    if(bstick) {
        ord.bstick <- bstick(x)
        ylims <- range(eig.vals[comps], ord.bstick[comps])
    } else {
        ylims <- range(eig.vals)
    }
    if(type=="barplot") {
        ## barplot looks weird if 0 not included
        ylims <- range(0, ylims)
        mids <- barplot(eig.vals[comps], names = names(eig.vals[comps]),
                        main = main, ylab = ylab, ylim = ylims, ...)
    } else {
        plot(comps, eig.vals[comps], type = ptype, axes = FALSE, main = main,
             xlab = xlab, ylab = ylab, ...)
        axis(2)
        axis(1, at = comps, labels = names(eig.vals[comps]))
        mids <- comps
        box()
    }
    if(bstick) {
        dot.args <- list(...)
        dot.nams <- names(dot.args)
        pch <- if("pch" %in% dot.nams)
            dot.args$pch
        else
            par("pch")
        lines(mids, ord.bstick[comps], type = ptype, col = bst.col,
              lty = bst.lty, pch = pch)
        if(legend) {
            col <- if("col" %in% dot.nams)
                dot.args$col
            else
                par("col")
            lty <- if("lty" %in% dot.nams)
                dot.args$lty
            else
                par("lty")
            if(type == "lines") {
                legend("topright",
                       legend = c("Ordination","Broken Stick"),
                       bty = "n", col = c(col, bst.col),
                       lty = c(lty, bst.lty),
                       pch = pch)
            } else {
                legend("topright",
                       legend = "Broken Stick", bty = "n",
                       col = bst.col, lty = bst.lty, pch = pch)
            }
        }
    }
    invisible(xy.coords(x = mids, y = eig.vals[comps]))
}

### Make a compact summary of permutations. This copies Gav Simpson's
### permute:::print.how, but only displays non-default choices in how().
`howHead` <- function(x, ...)
{
    ## print nothing is this not 'how'
    if (is.null(x) || !inherits(x, "how"))
        return()
    ## collect header
    head <- NULL
    ## blocks
    if (!is.null(getBlocks(x)))
        head <- paste0(head, paste("Blocks: ", x$blocks.name, "\n"))
    ## plots
    plotStr <- getStrata(x, which = "plots")
    if (!is.null(plotStr)) {
        plots <- getPlots(x)
        ptype <- getType(x, which = "plots")
        head <- paste0(head, paste0("Plots: ", plots$plots.name, ", "))
        head <- paste0(head, paste("plot permutation:", ptype))
        if(getMirror(x, which = "plots"))
            head <- paste(head, "mirrored")
        if (isTRUE(all.equal(ptype, "grid"))) {
            nr <- getRow(x, which = "plots")
            nc <- getCol(x, which = "plots")
            head <- paste0(head, sprintf(ngettext(nr, " %d row", " %d rows"),
                                        nr))
            head <- paste0(head, sprintf(ngettext(nc, " %d column",
                                                 " %d columns"), nc))
        }
        head <- paste0(head, "\n")
    }
    ## the fine level (within plots if any)
    type <- getType(x, which = "within")
    head <- paste0(head, "Permutation: ", type)
    if (isTRUE(type %in% c("series", "grid"))) {
        if(getMirror(x, which = "within"))
            head <- paste(head, "mirrored")
        if(getConstant(x))
            head <- paste0(head, " constant permutation within each Plot")
    }
    if (isTRUE(all.equal(type, "grid"))) {
        nr <- getRow(x, which = "within")
        nc <- getCol(x, which = "within")
        head <- paste0(head, sprintf(ngettext(nr, " %d row", " %d rows"),
                                    nr))
        head <- paste0(head, sprintf(ngettext(nc, " %d column",
                                             " %d columns"), nc))
    }
    paste0(head, "\nNumber of permutations: ", getNperm(x),  "\n")
}

`boxplot.specaccum` <-
    function(x, add=FALSE, ...)
{
    if (x$method != "random")
        stop("boxplot available only for method=\"random\"")
    if (!add) {
        plot(x$sites, x$richness, type="n", xlab="Sites", ylab="Species",
             ylim=c(1, max(x$richness, na.rm = TRUE)),  ...)
    }
    tmp <- boxplot(data.frame(t(x$perm)), add=TRUE, at=x$sites, axes=FALSE, ...)
    invisible(tmp)
}

`vegdist` <-
    function (x, method = "bray", binary = FALSE, diag = FALSE, upper = FALSE,
              na.rm = FALSE, ...)
{
    ZAP <- 1e-15
    if (!is.na(pmatch(method, "euclidian")))
        method <- "euclidean"
    ## the order of METHODS below *MUST* match the #define'd numbers
    ## in vegdist.c
    METHODS <- c("manhattan", "euclidean", "canberra", "bray",
                 "kulczynski", "gower", "morisita", "horn",
                 "mountford", "jaccard", "raup", "binomial", "chao",
                 "altGower", "cao", "mahalanobis", "clark")
    method <- pmatch(method, METHODS)
    inm <- METHODS[method]
    if (is.na(method))
        stop("invalid distance method")
    if (method == -1)
        stop("ambiguous distance method")
    ## most tests are faster for matrix than for data frame, and we
    ## need matrix in .Call() anyway
    x <- as.matrix(x)
    if (!na.rm && anyNA(x))
        stop("missing values are not allowed with argument 'na.rm = FALSE'")
    ## all vegdist indices need numeric data (Gower included).
    if (!(is.numeric(x) || is.logical(x)))
        stop("input data must be numeric")
    if (!method %in% c(1,2,6,16) && any(rowSums(x, na.rm = TRUE) == 0))
        warning("you have empty rows: their dissimilarities may be meaningless in method ",
                dQuote(inm))
    ## 1 manhattan, 2 euclidean, 3 canberra, 6 gower, 16 mahalanobis
    if (!method %in% c(1,2,3,6,16) && any(x < 0, na.rm = TRUE))
        warning("results may be meaningless because data have negative entries in method ",
                dQuote(inm))
    if (method == 11 && any(colSums(x) == 0))
        warning("data have empty species which influence the results in method ",
                dQuote(inm))
    if (method == 6) # gower, but no altGower
        x <- decostand(x, "range", 2, na.rm = TRUE, ...)
    if (method == 16) # mahalanobis
        x <- veganMahatrans(scale(x, scale = FALSE))
    if (binary)
        x <- decostand(x, "pa")
    N <- nrow(x)
    if (method %in% c(7, 13, 15) && !identical(all.equal(x, round(x)), TRUE))
        warning("results may be meaningless with non-integer data in method ",
                dQuote(inm))
    d <- .Call(do_vegdist, x, as.integer(method))
    if (method == 10)
        d <- 2 * d/(1 + d)
    d[d < ZAP] <- 0
    if (any(is.na(d)))
        warning("missing values in results")
    attr(d, "Size") <- N
    attr(d, "Labels") <- dimnames(x)[[1]]
    attr(d, "Diag") <- diag
    attr(d, "Upper") <- upper
    attr(d, "method") <- paste(if (binary)
                               "binary ", METHODS[method], sep = "")
    attr(d, "call") <- match.call()
    class(d) <- "dist"
    d
}

### Rotates metaMDS or monoMDS result so that axis one is parallel to
### vector 'x'.
`MDSrotate` <-
    function(object, vec, na.rm = FALSE, ...)
{
    workswith <- c("metaMDS", "monoMDS", "GO")
    if (!inherits(object, workswith))
        stop(gettextf("function works only with the results of: %s",
                      paste(workswith, collapse = ", ")))
    x <- object$points
    if (is.null(object$species))
        sp <- NA
    else
        sp <- object$species
    N <- NCOL(x)
    if (N < 2)
        stop(gettextf("needs at least two dimensions"))
    ## check if vec is a factor and then use lda to find a matrix that
    ## separates optimally factor levels
    if (is.factor(vec) || is.character(vec)) {
        da <- lda(x, vec)
        vec <- predict(da, dimen = N - 1)$x
        message(sprintf(ngettext(NCOL(vec),
                         "factor replaced with discriminant axis",
                         "factor replaced with %d discriminant axes",
                                 ), NCOL(vec)))
        if (NCOL(vec) > 1)
            message(gettextf("proportional traces: %.3f",
                             da$svd[1:NCOL(vec)]^2/sum(da$svd^2)))
    }
    vec <- as.matrix(vec)
    NV <- NCOL(vec)
    if (NV >= N)
        stop(gettextf("you can have max %d vectors, but you had %d",
             N-1, NV))
    if (!is.numeric(vec))
        stop(gettextf("'vec' must be numeric"))
    ## vectorfit finds the direction cosine. We rotate first axis to
    ## 'vec' which means that we make other axes orthogonal to 'vec'
    ## one by one
    if (na.rm)
        keep <- complete.cases(vec)
    else
        keep <- !logical(NROW(vec))
    ## Rotation loop
    for(v in seq_len(NV)) {
        for (k in (v+1):N) {
            arrs <- vectorfit(x[keep,], vec[keep,v], permutations = 0)$arrows
            rot <- arrs[c(v,k)]/sqrt(sum(arrs[c(v,k)]^2))
            rot <- drop(rot)
            ## counterclockwise rotation matrix:
            ## [cos theta   -sin theta]
            ## [sin theta    cos theta]
            rot <- rbind(rot, rev(rot))
            rot[1,2] <- -rot[1,2]
            ## Rotation of points and species scores
            x[, c(v,k)] <- x[, c(v,k)] %*% rot
            if (!all(is.na(sp)))
                sp[, c(v,k)] <- sp[, c(v,k)] %*% rot
        }
    }
    ## Two or more free axes are (optionally) rotated to PCs
    if (N - NV > 1 && attr(object$points, "pc")) {
        pc <- prcomp(x[,-seq_len(NV)])
        x[,-seq_len(NV)] <- pc$x
        if (!all(is.na(sp)))
            sp[,-seq_len(NV)] <- sp[,-seq_len(NV)] %*% pc$rotation
    }
    ## '[] <-' retains attributes
    object$points[] <- x
    object$species[] <- sp
    attr(object$points, "pc") <- FALSE
    object
}

"print.procrustes" <-
  function (x, digits = max(3, getOption("digits") - 3), ...) 
{
  cat("\nCall:\n")
  cat(deparse(x$call), "\n\n")
  cat("Procrustes sum of squares:\n")
  cat(formatC(x$ss, digits = digits), "\n\n")
  invisible(x)
}

`treedist` <-
    function(x, tree, relative = TRUE,  match.force = TRUE, ...)
{
    ## we cannot reconstruct tree with reversals from cophenetic
    tree <- as.hclust(tree)
    if (any(diff(tree$height) < -sqrt(.Machine$double.eps)))
        stop("tree with reversals cannot be handled")
    x <- as.matrix(x)
    n <- nrow(x)
    ABJ <- matrix(0, n , n)
    dmat <- as.matrix(cophenetic(tree))
    ## match names
    if (ncol(x) != ncol(dmat) || match.force) {
        if(!match.force)
            warning("dimensions do not match between 'x' and 'tree': matching by names")
        nm <- colnames(x)
        dmat <- dmat[nm, nm]
    }
    for(j in 1:n) {
        for (k in j:n) {
            jk <- x[j,] > 0 | x[k,] > 0
            if (sum(jk) > 1)
                ABJ[k, j] <- treeheight(update(tree, d = as.dist(dmat[jk, jk])))
        }
    }
    A <- diag(ABJ)
    AB <- as.dist(outer(A, A, "+"))
    ABJ <- as.dist(ABJ)
    out <- (2 * ABJ - AB)
    if (relative)
        out <- out/ABJ
    out[ABJ==0] <- 0
    attr(out, "method") <- if (relative) "treedist" else "raw treeedist"
    attr(out, "call") <- match.call()
    attr(out, "Labels") <- row.names(x)
    out
}

`dbrda` <-
    function (formula, data, distance = "euclidean",
              sqrt.dist = FALSE,  add = FALSE, dfun = vegdist,
              metaMDSdist = FALSE, na.action = na.fail,
              subset = NULL, ...)
{
    EPS <- sqrt(.Machine$double.eps)
    if (!inherits(formula, "formula"))
        stop("needs a model formula")
    if (missing(data)) {
        data <- parent.frame()
    }
    else {
        data <- eval(match.call()$data, environment(formula),
                     enclos = .GlobalEnv)
    }
    formula <- formula(terms(formula, data = data))
    ## The following line was eval'ed in environment(formula), but
    ## that made update() fail. Rethink the line if capscale() fails
    ## mysteriously at this point.
    X <- eval(formula[[2]], envir=environment(formula),
              enclos = globalenv())
    if ((is.matrix(X) || is.data.frame(X)) &&
               isSymmetric(unname(as.matrix(X))))
        X <- as.dist(X)
    if (!inherits(X, "dist")) {
        comm <- X
        dfun <- match.fun(dfun)
        if (metaMDSdist) {
            commname <- as.character(formula[[2]])
            X <- metaMDSdist(comm, distance = distance, zerodist = "ignore",
                             commname = commname, distfun = dfun, ...)
            commname <- attr(X, "commname")
            comm <- eval.parent(parse(text=commname))
        } else {
            X <- dfun(X, distance)
        }
    }
    ## get the name of the inertia
    inertia <- attr(X, "method")
    if (is.null(inertia))
        inertia <- "unknown"
    inertia <- paste(toupper(substr(inertia, 1, 1)),
                     substring(inertia, 2), sep = "")
    inertia <- paste(inertia, "distance")

    ## evaluate formula: ordiParseFormula will return dissimilarities
    ## as a symmetric square matrix (except that some rows may be
    ## deleted due to missing values)
    d <- ordiParseFormula(formula,
                          data,
                          na.action = na.action,
                          subset = substitute(subset),
                          X = X)
    ## ordiParseFormula subsets rows of dissimilarities: do the same
    ## for columns ('comm' is handled later). ordiParseFormula
    ## returned the original data, but we use instead the potentially
    ## changed X and discard d$X.
    if (!is.null(d$subset)) {
        X <- as.matrix(X)[d$subset, d$subset, drop = FALSE]
    }
    ## Delete columns if rows were deleted due to missing values
    if (!is.null(d$na.action)) {
        X <- as.matrix(X)[-d$na.action, -d$na.action, drop = FALSE]
    }
    X <- as.matrix(X)
    k <- NROW(X) - 1
    ## sqrt & add adjustments
    if (sqrt.dist)
        X <- sqrt(X)
    if (is.logical(add) && isTRUE(add))
        add <- "lingoes"
    if (is.character(add)) {
        add <- match.arg(add, c("lingoes", "cailliez"))
        if (add == "lingoes") {
            ac <- addLingoes(X)
            X <- sqrt(X^2 + 2 * ac)
        } else if (add == "cailliez") {
            ac <- addCailliez(X)
            X <- X + ac
        }
        diag(X) <- 0
    } else {
        ac <- 0
    }
    ## update the name of the inertia
    if (!sqrt.dist)
        inertia <- paste("squared", inertia)
    if (ac > sqrt(.Machine$double.eps))
        inertia <- paste(paste0(toupper(substring(add, 1, 1)),
                              substring(add, 2)), "adjusted", inertia)
    if (max(X) >= 4 + .Machine$double.eps) {
        inertia <- paste("mean", inertia)
        adjust <- sqrt(k)
        X <- X/adjust
    }
    else {
        adjust <- 1
    }
    nm <- attr(X, "Labels")
    ## Get components of inertia with negative eigenvalues following
    ## McArdle & Anderson (2001), section "Theory". G is their
    ## double-centred Gower matrix, but instead of hat matrix, we use
    ## QR decomposition to get the components of inertia.
    sol <- ordConstrained(X, d$Y, d$Z, method = "dbrda")

    sol$colsum <- NA
    ## separate eigenvectors associated with negative eigenvalues from
    ## u into imaginary.u
    if (!is.null(sol$CCA) && sol$CCA$rank > sol$CCA$poseig) {
        sol$CCA$imaginary.u <- sol$CCA$u[, -seq_len(sol$CCA$poseig),
                                         drop = FALSE]
        sol$CCA$u <- sol$CCA$u[, seq_len(sol$CCA$poseig), drop = FALSE]
    }
    if (!is.null(sol$CA) && sol$CA$rank > sol$CA$poseig) {
        sol$CA$imaginary.u <- sol$CA$u[, -seq_len(sol$CA$poseig),
                                       drop = FALSE]
        sol$CA$u <- sol$CA$u[, seq_len(sol$CA$poseig), drop = FALSE]
    }
    if (!is.null(sol$CCA) && sol$CCA$rank > 0)
        sol$CCA$centroids <-
            centroids.cca(sol$CCA$u, d$modelframe)
    if (!is.null(sol$CCA$alias))
        sol$CCA$centroids <- unique(sol$CCA$centroids)
    if (!is.null(sol$CCA$centroids)) {
        rs <- rowSums(sol$CCA$centroids^2)
        sol$CCA$centroids <- sol$CCA$centroids[rs > 1e-04, ,
                                               drop = FALSE]
        if (nrow(sol$CCA$centroids) == 0)
            sol$CCA$centroids <- NULL
    }
    sol$call <- match.call()
    sol$terms <- terms(formula, "Condition", data = data)
    sol$terminfo <- ordiTerminfo(d, data)
    sol$call$formula <- formula(d$terms, width.cutoff = 500)
    sol$call$formula[[2]] <- formula[[2]]
    sol$sqrt.dist <- sqrt.dist
    if (!is.na(ac) && ac > 0) {
        sol$ac <- ac
        sol$add <- add
    }
    sol$adjust <- adjust
    sol$inertia <- inertia
    if (metaMDSdist)
        sol$metaMDSdist <- commname
    if (!is.null(d$subset))
        sol$subset <- d$subset
    if (!is.null(d$na.action)) {
        sol$na.action <- d$na.action
        ## dbrda cannot add WA scores in na.exclude, and the following
        ## does nothing except adds residuals.zombie
        sol <- ordiNAexclude(sol, d$excluded)
    }
    class(sol) <- c("dbrda", "rda", "cca")
    sol
}

"print.humpfit" <-
    function(x, ...)
{
    cat("\nHump-backed Null model of richness vs. productivity\n\n")
    cat("Family:", family(x)$family,"\n")
    cat("Link function: Fisher diversity\n\n")
    cat("Coefficients:\n\n")
    print(coef(x))
    cat("\nDeviance", deviance(x), "with", df.residual(x))
    cat(" residual degrees of freedom\n")
    invisible(x)
}

`scores.envfit` <-
    function (x, display, choices, arrow.mul = 1, tidy = FALSE, ...)
{
    display <- match.arg(display,
                         c("vectors", "bp", "factors", "cn"),
                         several.ok = TRUE)
    out <- list()
    if (any(display %in% c("vectors", "bp"))) {
        vects <- x$vectors$arrows[, , drop = FALSE]
        if (!missing(choices))
            vects <- vects[, choices, drop=FALSE]
        if (!is.null(vects))
            out$vectors <- arrow.mul * sqrt(x$vectors$r) * vects
    }
    if (any(display %in% c("factors", "cn"))) {
        facts <- x$factors$centroids[, , drop = FALSE]
        if (!missing(choices))
            facts <- facts[, choices, drop=FALSE]
        out$factors <- facts
    }
    if (tidy) {
        if (length(out) == 0) # no scores
            return(NULL)
        group <- sapply(out, nrow)
        group <- rep(names(group), group)
        out <- do.call(rbind, out)
        label <- rownames(out)
        out <- as.data.frame(out)
        out$score <- group
        out$label <- label
    }
    ## only two kind of scores: return NULL, matrix or a list
    switch(length(out), out[[1]], out)
}

### The depths of nodes in a 'spantree' object: The nodes are either
### leaves with one link, or internal nodes with >1 links. The leaves
### are removed recursively from the tree and at each step the depth
### is increased with one.
`spandepth` <-
    function (x) 
{
    if (!inherits(x, "spantree"))
        stop("'x' must be 'spantree' result")
    kid <- c(NA, x$kid)
    par <- p <- seq_along(kid)
    par[1] <- NA
    ## Isolated nodes in disconnected tree have depth 0, other nodes
    ## start from depth 1
    intree <- p %in% kid | !is.na(kid) 
    depth <- numeric(length(par))
    depth[intree] <- 1
    if (!is.null(x$labels))
        names(depth) <- x$labels
    while(any(intree)) {
        ## Node is internal (intree) if it is both a parent and a kid
        ## and kid is in the tree or it is kid to two or more parents
        intree <- (p %in% intersect(kid[intree], par[intree]) &
                   p %in% p[intree][kid[intree] %in% p[intree]] |
                   p %in% kid[intree][duplicated(kid[intree])]) 
        depth[intree] <- depth[intree] + 1
    }
    depth
}

`fitted.procrustes` <-
  function(object, truemean = TRUE, ...)
{
  fit <- object$Yrot
  if (truemean)
    fit <- sweep(fit, 2, object$xmean, "+")
  fit
}

## Like above, except when takes 'newata'

`predict.procrustes` <-
    function(object, newdata, truemean = TRUE, ...)
{
    if (missing(newdata))
        return(fitted(object, truemean = truemean))
    if (object$symmetric)
        stop(gettextf("'predict' not available for symmetric procrustes analysis with 'newdata'"))
    Y <- as.matrix(newdata)
    ## scaling and rotation
    Y <- object$scale * Y %*% object$rotation
    ## translation: always
    Y <- sweep(Y, 2, object$translation, "+")
    if (!truemean)
        Y <- sweep(Y, 2, object$xmean, "-")
    Y
}

`stepacross` <-
    function (dis, path = "shortest", toolong = 1, trace = TRUE, ...)
{
    path <- match.arg(path, c("shortest", "extended"))
    if (!inherits(dis, "dist"))
        dis <- as.dist(dis)
    oldatt <- attributes(dis)
    n <- attr(dis, "Size")
    if (path == "shortest")
        dis <- .C(dykstrapath, dist = as.double(dis), n = as.integer(n),
                  as.double(toolong), as.integer(trace),
                  out = double(length(dis)), NAOK = TRUE)$out
    else dis <- .C(C_stepacross, dis = as.double(dis), as.integer(n),
                   as.double(toolong), as.integer(trace), NAOK = TRUE)$dis
    attributes(dis) <- oldatt
    attr(dis, "method") <- paste(attr(dis, "method"), path)
    dis
}

"print.summary.procrustes" <-
function (x, digits = x$digits, ...) 
{
    cat("\nCall:\n")
    cat(deparse(x$call), "\n")
    cat("\nNumber of objects:", x$n, "   Number of dimensions:", 
        x$k, "\n")
    cat("\nProcrustes sum of squares:  ")
    cat("\n", formatC(x$ss, digits = digits), "\n")
    cat("Procrustes root mean squared error: ")
    cat("\n", formatC(x$rmse, digits = digits), "\n")
    cat("Quantiles of Procrustes errors:\n")
    nam <- c("Min", "1Q", "Median", "3Q", "Max")
    rq <- structure(quantile(x$resid), names = nam)
    print(rq, digits = digits, ...)
    cat("\nRotation matrix:\n")
    print(x$rotation, digits = digits, ...)
    cat("\nTranslation of averages:\n")
    print(x$translation, digits = digits, ...)
    cat("\nScaling of target:\n")
    print(x$scale, digits = digits, ...)
    cat("\n")
    invisible(x)
}

`print.mantel` <-
  function (x, digits = max(3, getOption("digits") - 3), ...)
{
  cat("\n")
  if (inherits(x, "mantel.partial"))
    cat("Partial ")
  cat("Mantel statistic based on", x$method, "\n")
  cat("\nCall:\n")
  cat(deparse(x$call), "\n\n")
  cat("Mantel statistic r: ")
  cat(formatC(x$statistic, digits = digits), "\n")
  nperm <- x$permutations
  if (nperm) {
    cat("      Significance:", format.pval(x$signif),
        "\n\n")
    out <- quantile(x$perm, c(0.9, 0.95, 0.975, 0.99), na.rm = TRUE)
    cat("Upper quantiles of permutations (null model):\n")
    print(out, digits = 3)
    cat(howHead(x$control))
  }
  cat("\n")
  invisible(x)
}

`varpart` <-
    function (Y, X, ..., data, chisquare = FALSE, transfo, scale = FALSE,
              add = FALSE, sqrt.dist = FALSE, permutations = 1000)
{
    if (missing(data))
        data <- parent.frame()
    X <- list(X, ...)
    if ((length(X) < 2 || length(X) > 4))
        stop("needs two to four explanatory tables")
    ## transfo and scale can be used only with non-distance data
    if (inherits(Y, "dist")) {
        inert <- attr(Y, "method")
        inert <- paste(paste0(toupper(substring(inert, 1, 1)),
                              substring(inert, 2)), "distance")
        ## sqrt of distances?
        if (sqrt.dist)
            Y <- sqrt(Y)
        else
            inert <- paste("squared", inert)
        ## additive constant to euclidify distances?
        if (is.logical(add) && isTRUE(add))
            add <- "lingoes"
        if (is.character(add)) {
            add <- match.arg(add, c("lingoes", "cailliez"))
            if (add == "lingoes") {
                ac <- addLingoes(as.matrix(Y))
                Y <- sqrt(Y^2 + 2 * ac)
            } else if (add == "cailliez") {
                ac <- addCailliez(as.matrix(Y))
                Y <- Y + ac
            }
            if (ac > sqrt(.Machine$double.eps))
                inert <- paste(paste0(toupper(substring(add, 1, 1)),
                                    substring(add, 2)), "adjusted", inert)
        }
        RDA <- "dbRDA"
        if(!missing(transfo) || !missing(scale))
            message("arguments 'transfo' and 'scale' are ignored with distances")
    } else if (chisquare) {
        inert = "Chi-square"
        RDA = "CCA"
        permutations = getPermuteMatrix(permutations, nrow(Y))
    } else {
        inert <- "variance"
        RDA <- "RDA"
        if (!missing(transfo)) {
            Y <- decostand(Y, transfo)
            transfo <- attr(Y, "decostand")
        }
        if (!missing(transfo) && (is.null(dim(Y)) || ncol(Y) == 1))
            warning("transformations are probably meaningless with a single variable")
        if (scale && !missing(transfo))
            warning("Y should not be both transformed and scaled (standardized)")
        Y <- scale(Y, center = TRUE, scale = scale)
    }
    Sets <- list()
    for (i in seq_along(X)) {
        if (is.data.frame(X[[i]]) || is.factor(X[[i]])) {
            ## factor variable or a data.frame (possibly with factors)
            mf <- as.data.frame(X[[i]])
            mf <- model.matrix(~ ., mf)
            Sets[[i]] <- mf[,-1, drop = FALSE] # remove intercept
        } else if (inherits(X[[i]], "formula")) {
            ## Formula interface
            mf <- model.frame(X[[i]], data, na.action = na.fail,
                              drop.unused.levels = TRUE)
            trms <- attr(mf, "terms")
            Sets[[i]] <- model.matrix(trms, mf)
            if (any(colnames(Sets[[i]]) == "(Intercept)")) {
                xint <- which(colnames(Sets[[i]]) == "(Intercept)")
                Sets[[i]] <- (Sets[[i]])[, -xint, drop = FALSE]
            }
        }
        else Sets[[i]] <- as.matrix(X[[i]])
        Sets[[i]] <- scale(Sets[[i]], center = TRUE, scale = TRUE)
    }
    out <- list()
    out$part <- switch(length(Sets), NULL,
                       varpart2(Y, Sets[[1]], Sets[[2]],
                                chisquare, permutations),
                       varpart3(Y, Sets[[1]], Sets[[2]], Sets[[3]],
                                chisquare, permutations),
                       varpart4(Y, Sets[[1]], Sets[[2]], Sets[[3]], Sets[[4]],
                                chisquare, permutations))
    if (inherits(Y, "dist")) {
        out$part$ordination <- "dbrda"
    } else {
        if (chisquare)
            out$part$ordination <- "cca"
        else {
            out$part$ordination <- "rda"
        }
    }
    if(RDA == "RDA") {
        out$scale <- scale
        if (!missing(transfo))
            out$transfo <- transfo
    } else {
        if (scale || !missing(transfo))
            message("arguments 'scale' and 'transfo' ignored: valid only in RDA")
    }
    out$inert <- inert
    out$RDA <- RDA
    out$call <- match.call()
    mx <- rep(" ", length(X))
    for (i in seq_along(X)) mx[i] <- deparse(out$call[[i+2]], width.cutoff = 500)
    out$tables <- mx
    class(out) <- c("varpart", class(out))
    out
}

`bioenv.default` <-
function (comm, env, method = "spearman", index = "bray", upto = ncol(env),
          trace = FALSE, partial = NULL,
          metric = c("euclidean", "mahalanobis", "manhattan", "gower"),
          parallel = getOption("mc.cores"),
          ...)
{
    metric <- match.arg(metric)
    method <- match.arg(method, eval(formals(cor)$method))
    if (any(sapply(env, is.factor)) && metric != "gower")
        stop("you have factors in 'env': only 'metric = \"gower\"' is allowed")
    if (is.null(partial)) {
        corfun <- function(dx, dy, dz, method, ...) {
            cor(dx, dy, method=method, ...)
        }
    } else {
        corfun <- function(dx, dy, dz, method, ...) {
            rxy <- cor(dx, dy, method=method, ...)
            rxz <- cor(dx, dz, method=method, ...)
            ryz <- cor(dy, dz, method=method, ...)
            (rxy - rxz*ryz)/sqrt(1-rxz*rxz)/sqrt(1-ryz*ryz)
        }
    }
    if (!is.null(partial))
        partpart <- deparse(substitute(partial))
    else
        partpart <- NULL
    if (!is.null(partial) && !inherits(partial, "dist"))
        partial <- dist(partial)
    if (!is.null(partial) && !pmatch(method, c("pearson", "spearman"),
                                     nomatch=FALSE))
        stop(gettextf("method %s is invalid in partial bioenv", method))
    ## remove constant variables
    constant <- apply(env, 2, function(x) length(unique(x))) <= 1
    if (any(constant)) {
        warning(
            gettextf("the following variables are constant and were removed: %s",
                     paste(colnames(env)[constant], collapse=", ")))
            env <- env[, !constant, drop = FALSE]
    }
    n <- ncol(env)
    if (n < 1)
        stop("no usable variables in 'env'")
    ntake <- 2^n - 1
    ndone <- 0
    upto <- min(upto, n)
    if (n > 8 || trace) {
        if (upto < n)
            cat("Studying", nall <- sum(choose(n, 1:upto)), "of ")
        cat(ntake, "possible subsets (this may take time...)\n")
        flush.console()
    }
    ## Check metric and adapt data and distance function
    if (metric == "euclidean") {
        x <- scale(env, scale = TRUE)
        distfun <- function(x) dist(x)
    } else if (metric == "mahalanobis") {
        x <- as.matrix(scale(env, scale = FALSE))
        distfun <- function(x) dist(veganMahatrans(x))
    } else if (metric == "gower") {
        x <- env
        distfun <- function(x) daisy(x, metric = "gower")
    } else if (metric == "manhattan") {
        x <- decostand(env, "range")
        distfun <- function(x) dist(x, "manhattan")
    } else {
        stop("unknown metric")
    }
    best <- list()
    if (inherits(comm, "dist")) {
        comdis <- comm
        index <- attr(comdis, "method")
        if (is.null(index))
            index <- "unspecified"
    } else if ((is.matrix(comm) || is.data.frame(comm)) &&
               isSymmetric(unname(as.matrix(comm)))) {
        comdis <- as.dist(comm)
        index <- "supplied square matrix"
    } else {
        comdis <- vegdist(comm, method = index)
    }
    ## Prepare for parallel processing
    if (is.null(parallel))
        parallel <- 1
    hasClus <- inherits(parallel, "cluster")
    isParal <- hasClus || parallel > 1
    isMulticore <- .Platform$OS.type == "unix" && !hasClus
    if (isParal && !isMulticore && !hasClus) {
        parallel <- makeCluster(parallel)
        on.exit(stopCluster(parallel))
    }
    ## get the number of clusters
    if (inherits(parallel, "cluster"))
        nclus <- length(parallel)
    else
        nclus <- parallel
    CLUSLIM <- 8
    ## The proper loop
    for (i in 1:upto) {
        if (trace) {
            nvar <- choose(n, i)
            cat("No. of variables ", i, ", No. of sets ", nvar,
                "...", sep = "")
            flush.console()
        }
        sets <- t(combn(1:n, i))
        if (!is.matrix(sets))
            sets <- as.matrix(t(sets))
        if (isParal && nrow(sets) >= CLUSLIM*nclus) {
            if (isMulticore) {
                est <- unlist(mclapply(1:nrow(sets), function(j)
                                       corfun(comdis,
                                              distfun(x[,sets[j,],drop = FALSE]),
                                              partial, method = method, ...),
                                       mc.cores = parallel))
            } else {
                est <- parSapply(parallel, 1:nrow(sets), function(j)
                                  corfun(comdis, distfun(x[,sets[j,],drop = FALSE]),
                                         partial, method = method, ...))
            }
        } else {
            est <- sapply(1:nrow(sets), function(j)
                          corfun(comdis, distfun(x[,sets[j,], drop=FALSE ]),
                                 partial, method = method, ...))
        }
        best[[i]] <- list(best = sets[which.max(est), ], est = max(est))
        if (trace) {
            ndone <- ndone + nvar
            cat(" done (", round(100 * ndone/ntake, 1), "%)\n",
                sep = "")
            flush.console()
        }
    }
    whichbest <- which.max(lapply(best, function(tmp) tmp$est))
    out <- list(names = colnames(env), method = method, index = index,
                metric = metric, upto = upto, models = best,
                whichbest = whichbest,
                partial = partpart, x = x, distfun = distfun)
    out$call <- match.call()
    out$call[[1]] <- as.name("bioenv")
    class(out) <- "bioenv"
    out
}

## Function to extract the environmental distances used within
## bioenv. The default is to take the best model, but any model can be
## specified by its number.

`bioenvdist`  <-
    function(x, which = "best")
{
    ## any non-numeric argument is regarded as "best"
    if(!is.numeric(which))
        which <- x$whichbest
    x$distfun(x$x[, x$models[[which]]$best, drop = FALSE])
}

`ordisplom` <-
    function(x, data=NULL, formula = NULL,  display = "sites", choices = 1:3,
             panel = "panel.ordi", type = "p", ...)
{
  localSplom <- function(..., shrink, origin, scaling) splom(...)
  x <- as.data.frame(scores(x, display = display, choices = choices, ...))
  if (is.null(data))
    data <- x
  else if (is.null(formula))
    x <- cbind(x, data)
  ## type = "biplot" is not (yet?) implemented
  env <- list(arrows = NULL, centres = NULL)
  if (is.null(formula))
    pl <- localSplom(x, panel = panel, type = type, biplot = env, ...)
  else {
    formula <- as.formula(gsub("\\.", "x", deparse(formula)))
    pl <- localSplom(x = formula, data = data,  panel = panel, type = type,
                     biplot = env, ...)
  }
  pl
}

### extract scores from rda, capscale and dbrda results. The two
### latter can have special features which are commented below. cca
### results are handled by scores.cca.
`scores.rda` <-
    function (x, choices = c(1, 2), display = c("sp", "wa", "cn"),
              scaling = "species", const, correlation = FALSE, ...)
{
    ## Check the na.action, and pad the result with NA or WA if class
    ## "exclude"
    if (!is.null(x$na.action) && inherits(x$na.action, "exclude"))
        x <- ordiNApredict(x$na.action, x)
    tabula <- c("species", "sites", "constraints", "biplot",
                "regression", "centroids")
    names(tabula) <- c("sp", "wa", "lc", "bp", "reg", "cn")
    if (is.null(x$CCA))
        tabula <- tabula[1:2]
    display <- match.arg(display, c("sites", "species", "wa",
                                    "lc", "bp", "cn", "reg"),
                         several.ok = TRUE)
    if("sites" %in% display)
      display[display == "sites"] <- "wa"
    if("species" %in% display)
      display[display == "species"] <- "sp"
    take <- tabula[display]
    sumev <- x$tot.chi
    ## dbrda can have negative eigenvalues, but have scores only for
    ## positive
    eigval <- eigenvals(x)
    if (inherits(x, "dbrda") && any(eigval < 0))
        eigval <- eigval[eigval > 0]
    slam <- sqrt(eigval[choices]/sumev)
    nr <- if (is.null(x$CCA))
        nrow(x$CA$u)
    else
        nrow(x$CCA$u)
    ## const multiplier of scores
    if (missing(const))
        const <- sqrt(sqrt((nr-1) * sumev))
    ## canoco 3 compatibility -- canoco 4 is incompatible
    ##else if (pmatch(const, "canoco")) {
    ##    const <- (sqrt(nr-1), sqrt(nr))
    ##}
    ##
    ## const[1] for species, const[2] for sites and friends
    if (length(const) == 1) {
        const <- c(const, const)
    }
    ## in dbrda we only have scores for positive eigenvalues
    if (inherits(x, "dbrda"))
        rnk <- x$CCA$poseig
    else
        rnk <- x$CCA$rank
    sol <- list()
    ## process scaling; numeric scaling will just be returned as is
    scaling <- scalingType(scaling = scaling, correlation = correlation)
    if ("species" %in% take) {
        v <- cbind(x$CCA$v, x$CA$v)[, choices, drop=FALSE]
        if (scaling) {
            scal <- list(1, slam, sqrt(slam))[[abs(scaling)]]
            v <- sweep(v, 2, scal, "*")
            if (scaling < 0) {
                v <- sweep(v, 1, x$colsum, "/")
                v <- v * sqrt(sumev / (nr - 1))
            }
            v <- const[1] * v
        }
        if (nrow(v) > 0)
            sol$species <- v
        else
            sol$species <- NULL
    }
    if ("sites" %in% take) {
        wa <- cbind(x$CCA$wa, x$CA$u)[, choices, drop=FALSE]
        if (scaling) {
            scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
            wa <- sweep(wa, 2, scal, "*")
            wa <- const[2] * wa
        }
        sol$sites <- wa
    }
    if ("constraints" %in% take) {
        u <- cbind(x$CCA$u, x$CA$u)[, choices, drop=FALSE]
        if (scaling) {
            scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
            u <- sweep(u, 2, scal, "*")
            u <- const[2] * u
        }
        sol$constraints <- u
    }
    if ("biplot" %in% take && !is.null(x$CCA$biplot)) {
        b <- matrix(0, nrow(x$CCA$biplot), length(choices))
        b[, choices <= rnk] <- x$CCA$biplot[, choices[choices <= rnk]]
        colnames(b) <- c(colnames(x$CCA$u), colnames(x$CA$u))[choices]
        rownames(b) <- rownames(x$CCA$biplot)
        if (scaling) {
            scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
            b <- sweep(b, 2, scal, "*")
        }
        sol$biplot <- b
    }
    if ("regression" %in% take) {
        b <- coef(x, norm = TRUE)
        reg <- matrix(0, nrow(b), length(choices))
        reg[, choices <= rnk] <- b[, choices[choices <= rnk]]
        dimnames(reg) <- list(rownames(b),
                              c(colnames(x$CCA$u), colnames(x$CA$u))[choices])
        if (scaling) {
            scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
            reg <- sweep(reg, 2, scal, "*")
        }
        sol$regression <- reg
    }
    if ("centroids" %in% take) {
        if (is.null(x$CCA$centroids))
            sol$centroids <- NA
        else {
            cn <- matrix(0, nrow(x$CCA$centroids), length(choices))
            cn[, choices <= rnk] <- x$CCA$centroids[, choices[choices <= rnk]]
            colnames(cn) <- c(colnames(x$CCA$u), colnames(x$CA$u))[choices]
            rownames(cn) <- rownames(x$CCA$centroids)
            if (scaling) {
                scal <- list(slam, 1, sqrt(slam))[[abs(scaling)]]
                cn <- sweep(cn, 2, scal, "*")
                cn <- const[2] * cn
            }
            sol$centroids <- cn
        }
    }
    ## Take care that scores have names
    if (length(sol)) {
        for (i in seq_along(sol)) {
            if (is.matrix(sol[[i]]))
                rownames(sol[[i]]) <-
                    rownames(sol[[i]], do.NULL = FALSE,
                             prefix = substr(names(sol)[i], 1, 3))
        }
    }
    ## Only one type of scores: return a matrix instead of a list
    if (length(sol) == 1)
        sol <- sol[[1]]
    ## collapse const if both items identical
    if (identical(const[1], const[2]))
        const <- const[1]
    attr(sol, "const") <- const
    sol
}

`lines.radline` <-
    function (x, ...) 
{
    lin <- fitted(x)
    rnk <- seq(along = lin)
    lines(rnk, lin, ...)
    invisible()
}

`lines.radfit` <-
    function(x, ...)
{
    matlines(fitted(x), ...)
}

`ordiGetData` <-
function (call, env) 
{
    call$scale <- call$distance <- call$comm <- call$add <-
        call$dfun <- call$sqrt.dist <- call$metaMDSdist <- call$subset <- NULL
    call$na.action <- na.pass
    call[[2]] <- NULL
    call[[1]] <- as.name("model.frame")
    eval(call, env, enclos = .GlobalEnv)
}

`meandist` <-
    function(dist, grouping, ...)
{
    if (!inherits(dist, "dist"))
        stop("'dist' must be dissimilarity object inheriting from", dQuote(dist))
    ## check that 'dist' are dissimilarities (non-negative)
    if (any(dist < -sqrt(.Machine$double.eps)))
        warning("some dissimilarities are negative: is this intentional?")
    grouping <- factor(grouping, exclude = NULL)
    ## grouping for rows and columns
    grow <- grouping[as.dist(row(as.matrix(dist)))]
    gcol <- grouping[as.dist(col(as.matrix(dist)))]
    ## The row index must be "smaller" of the factor levels so that
    ## all means are in the lower triangle, and upper is NA
    first <- as.numeric(grow) >= as.numeric(gcol)
    cl1 <- ifelse(first, grow, gcol)
    cl2 <- ifelse(!first, grow, gcol)
    ## Cannot have within-group dissimilarity for group size 1
    n <- table(grouping)
    take <- matrix(TRUE, nlevels(grouping), nlevels(grouping))
    diag(take) <- n > 1
    take[upper.tri(take)] <- FALSE
    out <- matrix(NA, nlevels(grouping), nlevels(grouping))
    ## Get output matrix
    tmp <- tapply(dist, list(cl1, cl2), mean)
    out[take] <- tmp[!is.na(tmp)]
    out[upper.tri(out)] <- t(out)[upper.tri(out)]
    rownames(out) <- colnames(out) <- levels(grouping)
    class(out) <- c("meandist", "matrix")
    attr(out, "n") <- table(grouping)
    out
}

`orderingKM` <-
    function(mat)
{

### INPUT :
### mat		(n x k): n the objects and k the descriptors
### 			This matrix must be integers and numeric
###			And must not be binairy, it is the partition matrix
###			output by cascadeKM
### OUTPUT :  Ordered matrix

    ## Uses alternatively the fast USEPOWERALGORITHM provided by Legendre
    ## et al., or the standard R function cmdscale with matching
    ## coefficient provided by vegdist.c as method=50.

    USEPOWERALGORITHM <- TRUE

    ##Check up

    if(!is.matrix(mat)) stop("'mat' must be a matrix")
    if(!is.numeric(mat)) stop("'mat' must be numeric")
    if(any(is.na(mat))) stop("'NA' value was found in the matrix")
    if(any(is.infinite(mat))) stop("'Inf' value was found in the matrix")
    nb.desc=ncol(mat)
    nb.obj=nrow(mat)

    scores<-rep(0.0,nb.obj)
    if (USEPOWERALGORITHM) {
	scores <- as.vector(.Fortran(orderdata, as.integer(mat),
                                   as.integer(nb.obj), as.integer(nb.desc),
                                   sc=as.double(scores))$sc)
    } else {
        d <- .Call(do_vegdist, as.matrix(mat), as.integer(50))
        attr(d, "Size") <- nb.obj
        attr(d, "Labels") <- dimnames(mat)[[1]]
        attr(d, "Diag") <- FALSE
        attr(d, "Upper") <- FALSE
        attr(d, "method") <- "matching"
        class(d) <- "dist"
        scores <- cmdscale(d, k = 1)[,1]
    }
    scores <- order(scores)
    mat[scores,]
}

`varpart4` <-
    function (Y, X1, X2, X3, X4, chisquare, permat)
{
    collinwarn <- function(case, mm, m)
        warning(gettextf("collinearity detected in %s: mm = %d, m = %d",
                         case, mm, m), call. = FALSE)
    if (inherits(Y, "dist")) {
        Y <- GowerDblcen(as.matrix(Y^2), na.rm = FALSE)
        Y <- -Y/2
        SS.Y <- sum(diag(Y))
        simpleRDA2 <- match.fun(simpleDBRDA)
    } else {
        Y <- as.matrix(Y)
        if (chisquare) {
            SS.Y <- sum(initCA(Y^2))
            simpleRDA2 <- match.fun(simpleCCA)
        } else {
            Y <- scale(Y, center = TRUE, scale = FALSE)
            SS.Y <- sum(Y * Y)
        }
    }
    X1 <- as.matrix(X1)
    X2 <- as.matrix(X2)
    X3 <- as.matrix(X3)
    X4 <- as.matrix(X4)
    n <- nrow(Y)
    n1 <- nrow(X1)
    n2 <- nrow(X2)
    n3 <- nrow(X3)
    n4 <- nrow(X4)
    p <- ncol(Y)
    mm1 <- ncol(X1)
    mm2 <- ncol(X2)
    mm3 <- ncol(X3)
    mm4 <- ncol(X4)
    if (n1 != n)
        stop("Y and X1 do not have the same number of rows")
    if (n2 != n)
        stop("Y and X2 do not have the same number of rows")
    if (n3 != n)
        stop("Y and X3 do not have the same number of rows")
    if (n4 != n)
        stop("Y and X4 do not have the same number of rows")
    X1 <- scale(X1, center = TRUE, scale = FALSE)
    X2 <- scale(X2, center = TRUE, scale = FALSE)
    X3 <- scale(X3, center = TRUE, scale = FALSE)
    X4 <- scale(X4, center = TRUE, scale = FALSE)
    dummy <- simpleRDA2(Y, X1, SS.Y, permat)
    aeghklno.ua <- dummy$Rsquare
    aeghklno <- dummy$RsquareAdj
    m1 <- dummy$m
    if (m1 != mm1)
        collinwarn("X1", mm1, m1)
    dummy <- simpleRDA2(Y, X2, SS.Y, permat)
    befiklmo.ua <- dummy$Rsquare
    befiklmo <- dummy$RsquareAdj
    m2 <- dummy$m
    if (m2 != mm2)
        collinwarn("X2", mm2, m2)
    dummy <- simpleRDA2(Y, X3, SS.Y, permat)
    cfgjlmno.ua <- dummy$Rsquare
    cfgjlmno <- dummy$RsquareAdj
    m3 <- dummy$m
    if (m3 != mm3)
        collinwarn("X3", mm3, m3)
    dummy <- simpleRDA2(Y, X4, SS.Y, permat)
    dhijkmno.ua <- dummy$Rsquare
    dhijkmno <- dummy$RsquareAdj
    m4 <- dummy$m
    if (m4 != mm4)
        collinwarn("X4", mm4, m4)
    mm5 = mm1 + mm2
    dummy <- simpleRDA2(Y, cbind(X1, X2), SS.Y, permat)
    abefghiklmno.ua <- dummy$Rsquare
    abefghiklmno <- dummy$RsquareAdj
    m5 <- dummy$m
    if (m5 != mm5)
        collinwarn("cbind(X1,X2)", mm5, m5)
    mm6 = mm1 + mm3
    dummy <- simpleRDA2(Y, cbind(X1, X3), SS.Y, permat)
    acefghjklmno.ua <- dummy$Rsquare
    acefghjklmno <- dummy$RsquareAdj
    m6 <- dummy$m
    if (m6 != mm6)
        collinwarn("cbind(X1,X3", mm6, m6)
    mm7 = mm1 + mm4
    dummy <- simpleRDA2(Y, cbind(X1, X4), SS.Y, permat)
    adeghijklmno.ua <- dummy$Rsquare
    adeghijklmno <- dummy$RsquareAdj
    m7 <- dummy$m
    if (m7 != mm7)
        collinwarn("cbind(X1,X4)", mm7, m7)
    mm8 = mm2 + mm3
    dummy <- simpleRDA2(Y, cbind(X2, X3), SS.Y, permat)
    bcefgijklmno.ua <- dummy$Rsquare
    bcefgijklmno <- dummy$RsquareAdj
    m8 <- dummy$m
    if (m8 != mm8)
        collinwarn("cbind(X2,X3)", mm8, m8)
    mm9 = mm2 + mm4
    dummy <- simpleRDA2(Y, cbind(X2, X4), SS.Y, permat)
    bdefhijklmno.ua <- dummy$Rsquare
    bdefhijklmno <- dummy$RsquareAdj
    m9 <- dummy$m
    if (m9 != mm9)
        collinwarn("cbind(X2,X4)", mm9, m9)
    mm10 = mm3 + mm4
    dummy <- simpleRDA2(Y, cbind(X3, X4), SS.Y, permat)
    cdfghijklmno.ua <- dummy$Rsquare
    cdfghijklmno <- dummy$RsquareAdj
    m10 <- dummy$m
    if (m10 != mm10)
        collinwarn("cbind(X3,X4)", mm10, m10)
    mm11 = mm1 + mm2 + mm3
    dummy <- simpleRDA2(Y, cbind(X1, X2, X3), SS.Y, permat)
    abcefghijklmno.ua <- dummy$Rsquare
    abcefghijklmno <- dummy$RsquareAdj
    m11 <- dummy$m
    if (m11 != mm11)
        collinwarn("cbind(X1,X2,X3)", mm11, m11)
    mm12 = mm1 + mm2 + mm4
    dummy <- simpleRDA2(Y, cbind(X1, X2, X4), SS.Y, permat)
    abdefghijklmno.ua <- dummy$Rsquare
    abdefghijklmno <- dummy$RsquareAdj
    m12 <- dummy$m
    if (m12 != mm12)
        collinwarn("c(X1,X2,X4)", mm12, m12)
    mm13 = mm1 + mm3 + mm4
    dummy <- simpleRDA2(Y, cbind(X1, X3, X4), SS.Y, permat)
    acdefghijklmno.ua <- dummy$Rsquare
    acdefghijklmno <- dummy$RsquareAdj
    m13 <- dummy$m
    if (m13 != mm13)
        collinwarn("cbind(X1,X3,X4)", mm13, m13)
    mm14 = mm2 + mm3 + mm4
    dummy <- simpleRDA2(Y, cbind(X2, X3, X4), SS.Y, permat)
    bcdefghijklmno.ua <- dummy$Rsquare
    bcdefghijklmno <- dummy$RsquareAdj
    m14 <- dummy$m
    if (m14 != mm14)
        collinwarn("cbind(X2,X3,X4)", mm14, m14)
    mm15 = mm1 + mm2 + mm3 + mm4
    dummy <- simpleRDA2(Y, cbind(X1, X2, X3, X4), SS.Y, permat)
    abcdefghijklmno.ua <- dummy$Rsquare
    abcdefghijklmno <- dummy$RsquareAdj
    m15 <- dummy$m
    if (m15 != mm15)
        collinwarn("cbind(X1,X2,X3,X4)", mm15, m15)
    bigwarning <- NULL
    if ((m1 + m2) > m5)
        bigwarning <- c(bigwarning, c("X1, X2"))
    if ((m1 + m3) > m6)
        bigwarning <- c(bigwarning, c("X1, X3"))
    if ((m1 + m4) > m7)
        bigwarning <- c(bigwarning, c("X1, X4"))
    if ((m2 + m3) > m8)
        bigwarning <- c(bigwarning, c("X2, X3"))
    if ((m2 + m4) > m9)
        bigwarning <- c(bigwarning, c("X2, X4"))
    if ((m3 + m4) > m10)
        bigwarning <- c(bigwarning, c("X3, X4"))
    if ((m1 + m2 + m3) > m11)
        bigwarning <- c(bigwarning, c("X1, X2, X3"))
    if ((m1 + m2 + m4) > m12)
        bigwarning <- c(bigwarning, c("X1, X2, X4"))
    if ((m1 + m3 + m4) > m13)
        bigwarning <- c(bigwarning, c("X1, X3, X4"))
    if ((m2 + m3 + m4) > m14)
        bigwarning <- c(bigwarning, c("X2, X3, X4"))
    if ((m1 + m2 + m3 + m4) > m15)
        bigwarning <- c(bigwarning, c("X1, X2, X3, X4"))
    Df <-  c(m1, m2, m3, m4, m5, m6, m7, m8, m9, m10, m11, m12, m13, m14, m15)
    fract <- data.frame(Df = Df,
                        R.square = c(aeghklno.ua, befiklmo.ua, cfgjlmno.ua,
                        dhijkmno.ua, abefghiklmno.ua, acefghjklmno.ua,
                        adeghijklmno.ua, bcefgijklmno.ua, bdefhijklmno.ua,
                        cdfghijklmno.ua, abcefghijklmno.ua, abdefghijklmno.ua,
                        acdefghijklmno.ua, bcdefghijklmno.ua, abcdefghijklmno.ua),
                        Adj.R.square = c(aeghklno, befiklmo, cfgjlmno, dhijkmno,
                        abefghiklmno, acefghjklmno, adeghijklmno, bcefgijklmno,
                        bdefhijklmno, cdfghijklmno, abcefghijklmno, abdefghijklmno,
                        acdefghijklmno, bcdefghijklmno, abcdefghijklmno),
                        Testable = rep(TRUE, 15) & Df)
    rownames(fract) <- c("[aeghklno] = X1", "[befiklmo] = X2",
                         "[cfgjlmno] = X3", "[dhijkmno] = X4", "[abefghiklmno] = X1+X2",
                         "[acefghjklmno] = X1+X3", "[adeghijklmno] = X1+X4", "[bcefgijklmno] = X2+X3",
                         "[bdefhijklmno] = X2+X4", "[cdfghijklmno] = X3+X4", "[abcefghijklmno] = X1+X2+X3",
                         "[abdefghijklmno] = X1+X2+X4", "[acdefghijklmno] = X1+X3+X4",
                         "[bcdefghijklmno] = X2+X3+X4", "[abcdefghijklmno] = All")
    ae = acdefghijklmno - cdfghijklmno
    ag = abdefghijklmno - bdefhijklmno
    ah = abcefghijklmno - bcefgijklmno
    be = bcdefghijklmno - cdfghijklmno
    bf = abdefghijklmno - adeghijklmno
    bi = abcefghijklmno - acefghjklmno
    cf = acdefghijklmno - adeghijklmno
    cg = bcdefghijklmno - bdefhijklmno
    cj = abcefghijklmno - abefghiklmno
    dh = bcdefghijklmno - bcefgijklmno
    di = acdefghijklmno - acefghjklmno
    dj = abdefghijklmno - abefghiklmno
    Df <- c(m13-m10, m12-m9, m11-m8, m14-m10, m12-m7, m11-m6, m13-m7, m14-m9,
            m11-m5, m14-m8, m13-m6, m12-m5)
    contr2 <- data.frame(Df = Df,
                         R.square = rep(NA, 12),
                         Adj.R.square = c(ae, ag, ah,
                         be, bf, bi, cf, cg, cj, dh, di, dj),
                         Testable = rep(TRUE, 12) & Df)
    rownames(contr2) <- c("[ae] = X1 | X3+X4", "[ag] = X1 | X2+X4",
                          "[ah] = X1 | X2+X3", "[be] = X2 | X3+X4", "[bf] = X2 | X1+X4",
                          "[bi] = X2 | X1+X3", "[cf] = X3 | X1+X4", "[cg] = X3 | X2+X4",
                          "[cj] = X3 | X1+X2", "[dh] = X4 | X2+X3", "[di] = X4 | X1+X3",
                          "[dj] = X4 | X1+X2")
    aghn = abefghiklmno - befiklmo
    aehk = acefghjklmno - cfgjlmno
    aegl = adeghijklmno - dhijkmno
    bfim = abefghiklmno - aeghklno
    beik = bcefgijklmno - cfgjlmno
    befl = bdefhijklmno - dhijkmno
    cfjm = acefghjklmno - aeghklno
    cgjn = bcefgijklmno - befiklmo
    cfgl = cdfghijklmno - dhijkmno
    dijm = adeghijklmno - aeghklno
    dhjn = bdefhijklmno - befiklmo
    dhik = cdfghijklmno - cfgjlmno
    Df <- c(m5-m2, m6-m3, m7-m4, m5-m1, m8-m3, m9-m4, m6-m1, m8-m2, m10-m4,
            m7-m1, m9-m2, m10-m3)
    contr1 <- data.frame(Df = Df,
                         R.square = rep(NA, 12),
                         Adj.R.square = c(aghn, aehk,
                         aegl, bfim, beik, befl, cfjm, cgjn, cfgl, dijm, dhjn,
                                                 dhik),
                         Testable = rep(TRUE, 12) & Df)
    rownames(contr1) <- c("[aghn] = X1 | X2", "[aehk] = X1 | X3",
                          "[aegl] = X1 | X4", "[bfim] = X2 | X1", "[beik] = X2 | X3",
                          "[befl] = X2 | X4", "[cfjm] = X3 | X1", "[cgjn] = X3 | X2",
                          "[cfgl] = X3 | X4", "[dijm] = X4 | X1 ", "[dhjn] = X4 | X2",
                          "[dhik] = X4 | X3")
    a <- abcdefghijklmno - bcdefghijklmno
    b <- abcdefghijklmno - acdefghijklmno
    c <- abcdefghijklmno - abdefghijklmno
    d <- abcdefghijklmno - abcefghijklmno
    e <- ae - a
    f <- bf - b
    g <- ag - a
    h <- ah - a
    i <- bi - b
    j <- cj - c
    k <- aehk - ae - h
    l <- aegl - ae - g
    m <- bfim - bf - i
    n <- aghn - ag - h
    o <- aeghklno - aehk - g - l - n
    indfract <- data.frame(Df = c(m15-m14, m15-m13, m15-m12, m15-m11, rep(0, 12)),
                           R.square = rep(NA, 16), Adj.R.square = c(a, b, c, d,
                                                   e, f, g, h, i, j, k, l, m, n, o, 1 - abcdefghijklmno),
                           Testable = c(rep(TRUE, 4), rep(FALSE, 12)))
    rownames(indfract) <- c("[a] = X1 | X2+X3+X4", "[b] = X2 | X1+X3+X4",
                            "[c] = X3 | X1+X2+X4", "[d] = X4 | X1+X2+X3", "[e]",
                            "[f]", "[g]", "[h]", "[i]", "[j]", "[k]", "[l]", "[m]",
                            "[n]", "[o]", "[p] = Residuals")
    out <- list(fract = fract, indfract = indfract, contr1 = contr1,
                contr2 = contr2, SS.Y = SS.Y, nsets = 4, bigwarning = bigwarning,
                n = n1)
    class(out) <- "varpart234"
    out
}

### fitspecaccum returns fitted nls() models in item models. Here we
### provide interfaces for some "nls" class support functions, and
### others can be used in the similar way.

`predict.fitspecaccum` <-
    function(object, newdata, ...)
{
    mods <- object$models
    if (!missing(newdata)) {
        newdata <- drop(as.matrix(newdata))
        if (length(dim(newdata)) > 1)
            stop("function accepts only one variable as 'newdata'")
        drop(sapply(mods, predict, newdata = data.frame(x = newdata), ...))
    } else {
        drop(sapply(mods, predict, ...))
    }
}

`AIC.fitspecaccum` <-
    function(object, ..., k = 2)
{
    sapply(object$models, AIC, k = k, ...)
}

`deviance.fitspecaccum` <-
    function(object, ...)
{
    sapply(object$models, deviance, ...)
}

`logLik.fitspecaccum` <-
    function(object, ...)
{
    out <- sapply(object$models, logLik, ...)
    ## sapply() strips attributes: get'em back
    attr(out, "df") <- 1L + length(coef(object$models[[1L]]))
    attr(out, "nobs") <- nobs(object$models[[1L]])
    class(out) <- "logLik"
    out
}

`nobs.fitspecaccum` <-
    function(object, ...)
{
    sapply(object$models, nobs, ...)
}

`ordilabel` <-
    function(x, display, labels, choices = c(1,2), priority, select,
             cex = 0.8, fill = "white", border = NULL, col = NULL,
             xpd = TRUE, ...)
{
    if (missing(display))
        display <- "sites"
    x <- scores(x, choices = choices, display = display, ...)
    if (missing(labels))
        labels <- rownames(x)
    if (!missing(select)) {
        x <- .checkSelect(select, x)
        labels <- .checkSelect(select, labels)
    }
    if (!missing(priority)) {
        if (!missing(select))
            priority <- priority[select]
        ord <- order(priority)
        x <- x[ord, ]
        labels <- labels[ord]
    } else {
        ord <- seq_along(labels)
    }
    em <- strwidth("m", cex = cex, ...)
    ex <- strheight("x", cex = cex, ...)
    w <- (strwidth(labels, cex=cex,...) + em/1.5)/2
    h <- (strheight(labels, cex = cex, ...) + ex/1.5)/2
    if (is.null(col))
        if (!is.null(border))
            col <- border
        else
            col <- par("fg")
    col <- rep(col, length=nrow(x))[ord]
    if(!is.null(border))
        border <- rep(border, length=nrow(x))[ord]
    fill <- rep(fill, length=nrow(x))[ord]
    for (i in 1:nrow(x)) {
        ordiArgAbsorber(x[i,1] + c(-1,1,1,-1)*w[i], x[i,2] + c(-1,-1,1,1)*h[i],
                        col = fill[i], border = border[i], xpd = xpd,
                        FUN = polygon, ...)
        ordiArgAbsorber(x[i,1], x[i,2], labels = labels[i], cex = cex,
                        col = col[i], xpd = xpd, FUN = text, ...)
    }
    invisible(x)
}

`nestedtemp` <-
    function(comm, ...)
{
    ## J Biogeogr 33, 924-935 (2006) says that Atmar & Patterson try
    ## to pack presences and absence to minimal matrix temperature,
    ## and the following routines try to reproduce the (partly verbal)
    ## description. Index s should pack ones, and index t should pack
    ## zeros, and the final ordering should be "a compromise".
    colpack <- function(x, rr)
    {
        ind <- matrix(rep(rr, ncol(x)), nrow=nrow(x))
        s <- -colSums((x*ind)^2)
        t <- -colSums((nrow(x) - (1-x)*ind + 1)^2)
        st <- rank(s+t, ties.method = "random")
        st
    }
    rowpack <- function(x, cr)
    {
        ind <- matrix(rep(cr, each=nrow(x)), nrow=nrow(x))
        s <- -rowSums((x*ind)^2)
        t <- -rowSums((ncol(x) - (1-x)*ind + 1)^2)
        st <- rank(s+t, ties.method = "random")
        st
    }
    comm <- ifelse(comm > 0, 1, 0)
    ## Start with columns, expect if nrow > ncol
    if (ncol(comm) >= nrow(comm)) {
        i <- rank(-rowSums(comm), ties.method = "average")
    } else {
        j <- rank(-colSums(comm), ties.method = "average")
        i <- rowpack(comm, j)
    }
    ## Improve eight times
    for (k in seq_len(8)) {
        j <- colpack(comm, i)
        i <- rowpack(comm, j)
    }
    if (ncol(comm) < nrow(comm))
        j <- colpack(comm, i)
    comm <- comm[order(i), order(j)]
    r <- ppoints(nrow(comm), a=0.5)
    c <- ppoints(ncol(comm), a=0.5)
    dis <- matrix(rep(r, ncol(comm)), nrow=nrow(comm))
    totdis <- 1 - abs(outer(r, c, "-"))
    fill <- sum(comm)/prod(dim(comm))
    ## Fill line as defined in J Biogeogr by solving an integral of
    ## the fill function
    fillfun <- function(x, p) 1 - (1-(1-x)^p)^(1/p)
    intfun <- function(p, fill)
        integrate(fillfun, lower=0, upper=1, p=p)$value - fill
    ## 'p' will depend on 'fill', and fill = 0.0038 correspond to p =
    ## 20, and we may need to extend the bracket.
    sol <- uniroot(intfun, c(0,20), fill=fill, extendInt = "downX")
    p <- sol$root
    ## row coordinates of the fill line for all matrix entries
    out <- matrix(0, nrow=length(r), ncol=length(c))
    for (i in seq_along(r))
        for (j in seq_along(c)) {
            a <- c[j] - r[i]
            out[i,j] <- uniroot(function(x, ...) fillfun(x, p) - a -x,
                                c(0,1), p = p)$root
            }
    ## Filline
    x <- seq(0,1,len=51)
    xline <- fillfun(x, p)
    smo <- list(x = x, y = xline)
    u <- (dis - out)/totdis
    u[u < 0 & comm == 1] <- 0
    u[u > 0 & comm == 0] <- 0
    u <- u^2
    colnames(u) <- colnames(comm)
    rownames(u) <- rownames(comm)
    names(r) <- rownames(comm)
    names(c) <- colnames(comm)
    temp <- 100*sum(u)/prod(dim(comm))/0.04145
    out <- list(comm = comm, u = u, r = r, c = c, p = p,
                fill=fill,  statistic = temp, smooth=smo)
    names(out$statistic) <- "temperature"
    class(out) <- "nestedtemp"
    out
}

`mantel.correlog` <-
    function(D.eco, D.geo=NULL, XY=NULL, n.class=0, break.pts=NULL,
             cutoff=TRUE, r.type="pearson", nperm=999, mult="holm",
             progressive=TRUE)
{
    r.type <- match.arg(r.type, c("pearson", "spearman", "kendall"))
    mult   <- match.arg(mult, c("sidak", p.adjust.methods))

    epsilon <- .Machine$double.eps
    D.eco <- as.matrix(D.eco)

    ## Geographic distance matrix
    if(!is.null(D.geo)) {
	if(!is.null(XY))
            stop("you provided both a geographic distance matrix and a list of site coordinates:\nwhich one should the function use?")
	D.geo <- as.matrix(D.geo)
    } else {
	if(is.null(XY)) {
            stop("you did not provide a geographic distance matrix nor a list of site coordinates")
        } else {
            D.geo <- as.matrix(dist(XY))
        }
    }

    n <- nrow(D.geo)
    if(n != nrow(D.eco))
	stop("numbers of objects in D.eco and D.geo are not equal")
    n.dist <- n*(n-1)/2
    vec.D <- as.vector(as.dist(D.geo))
    vec.DD <- as.vector(D.geo)

    ## Number of classes and breakpoints

    if(!is.null(break.pts)) {
	## Use the list of break points
	if(n.class > 0)
            stop("you provided both a number of classes and a list of break points:\nwhich one should the function use?")
	n.class = length(break.pts) - 1

    } else {
        ## No breakpoints have been provided: equal-width classes
        if(n.class == 0) {
            ## Use Sturges rule to determine the number of classes
            n.class <- ceiling(1 + log(n.dist, base=2))
        }
        ## Compute the breakpoints from n.class
        start.pt <- min(vec.D)
        end.pt <- max(vec.D)
        width <- (end.pt - start.pt)/n.class
        break.pts <- vector(length=n.class+1)
        break.pts[n.class+1] <- end.pt
        for(i in 1:n.class)
            break.pts[i] <- start.pt + width*(i-1)
    }

    half.cl <- n.class %/% 2

    ## Move the first breakpoint a little bit to the left
    break.pts[1] <- break.pts[1] - epsilon

    ## Find the break points and the class indices
    class.ind <- break.pts[1:n.class] +
        (0.5*(break.pts[2:(n.class+1)]-break.pts[1:n.class]))

    ## Create the matrix of distance classes
    vec2 <- vector(length=n^2)
    for(i in 1:n^2)
        vec2[i] <- min( which(break.pts >= vec.DD[i]) ) - 1

    ## Start assembling the vectors of results
    class.index <- NA
    n.dist <- NA
    mantel.r <- NA
    mantel.p <- NA
    ## check.sums = matrix(NA,n.class,1)

    ## Create a model-matrix for each distance class, then compute a Mantel test
    for(k in 1:n.class) {
	class.index <- c(class.index, class.ind[k])
	vec3 <- rep(0, n*n)
	sel <- which(vec2 == k)
	vec3[sel] <- 1
	mat.D2 <- matrix(vec3,n,n)
	diag(mat.D2) <- 0
	n.dis <- sum(mat.D2)
	n.dist <- c(n.dist, n.dis)
	if(n.dis == 0) {
            mantel.r <- c(mantel.r, NA)
            mantel.p <- c(mantel.p, NA)
        } else {
            row.sums <- rowSums(mat.D2)
            ## check.sums[k,1] = length(which(row.sums == 0))
            if((cutoff==FALSE) ||
               !(cutoff==TRUE && k > half.cl && any(row.sums == 0))) {
                temp <- mantel(mat.D2, D.eco, method=r.type, permutations=nperm)
                mantel.r <- c(mantel.r, -temp$statistic)
                temp.p <- temp$signif

                ## The mantel() function produces a one-tailed p-value
                ## (H1: r>0) Here, compute a one-tailed p-value in
                ## direction of the sign
                if(temp$statistic < 0) {
                    temp.p <- (sum(temp$perm <= temp$statistic)+1)/(nperm+1)
                }
                mantel.p <- c(mantel.p, temp.p)
            } else {
                mantel.r <- c(mantel.r, NA)
                mantel.p <- c(mantel.p, NA)
            }
        }
    }

    mantel.res <- cbind(class.index, n.dist, mantel.r, mantel.p)
    mantel.res <- mantel.res[-1,]

    ## Note: vector 'mantel.p' starts with a NA value
    mantel.p <- mantel.p[-1]
    n.tests <- length(which(!is.na(mantel.p)))

    if(mult == "none") {
	colnames(mantel.res) <-
            c("class.index", "n.dist", "Mantel.cor", "Pr(Mantel)")
    } else {
	## Correct P-values for multiple testing
        if(progressive) {
            p.corr <- mantel.p[1]
            if(mult == "sidak") {
                for(j in 2:n.tests)
                    p.corr <- c(p.corr, 1-(1-mantel.p[j])^j)
            } else {
                for(j in 2:n.tests) {
                    temp <- p.adjust(mantel.p[1:j], method=mult)
                    p.corr <- c(p.corr, temp[j])
                }
            }
        } else {
            ## Correct all p-values for 'n.tests' simultaneous tests
            if(mult == "sidak") {
                p.corr <- 1 - (1 - mantel.p[1:n.tests])^n.tests
            } else {
                p.corr <- p.adjust(mantel.p[1:n.tests], method=mult)
            }
        }
	temp <- c(p.corr, rep(NA,(n.class-n.tests)))
	mantel.res <- cbind(mantel.res, temp)
	colnames(mantel.res) <-
            c("class.index", "n.dist", "Mantel.cor", "Pr(Mantel)", "Pr(corrected)")
    }
    rownames(mantel.res) <-
        rownames(mantel.res,do.NULL = FALSE, prefix = "D.cl.")

    ## Output the results
    res <- list(mantel.res=mantel.res, n.class=n.class, break.pts=break.pts,
                mult=mult, n.tests=n.tests, call=match.call() )
    class(res) <- "mantel.correlog"
    res
}

### R 2.13.0 introduces nobs() method to get the number of
### observations. This file provides methods for vegan classes.

`nobs.adonis` <- function(object, ...) NCOL(object$coef.sites)

`nobs.anova.cca` <- function(object, ...) NA

`nobs.betadisper` <- function(object, ...) length(object$distances)

`nobs.cca` <- function(object, ...) max(NROW(object$pCCA$u),
                                        NROW(object$CCA$u),
                                        NROW(object$CA$u))

`nobs.CCorA` <- function(object, ...) NROW(object$Cy)

`nobs.decorana` <- function(object, ...) NROW(object$rproj)

`nobs.isomap` <- function(object, ...) NROW(object$points)

`nobs.metaMDS` <- function(object, ...) NROW(object$points)

`nobs.pcnm` <- function(object, ...) NROW(object$vectors)

`nobs.procrustes` <- function(object, ...) NROW(object$X)

`nobs.rad` <- function(object, ...) length(object$y)

`nobs.varpart` <- function(object, ...) object$part$n

`nobs.wcmdscale` <- function(object, ...) NROW(object$points)

###
### Editable Tcl/Tk plot for ordination
###
`orditkplot` <-
    function(x, display = "species", choices = 1:2, width, xlim, ylim,
             tcex=0.8, tcol, pch = 1, pcol, pbg, pcex = 0.7,
             labels,  ...)
{
    if (!capabilities("tcltk"))
        stop("your R has no capability for Tcl/Tk")
    requireNamespace("tcltk") || stop("requires package tcltk")

############################
### Check and sanitize input
###########################

    ## Graphical parameters and constants, and save some for later plotting
    p <- par()
    sparnam <- c("bg","cex", "cex.axis","cex.lab","col", "col.axis", "col.lab",
                 "family", "fg", "font", "font.axis", "font.lab", "lheight",
                 "lwd", "mar", "mex", "mgp", "ps", "tcl", "las")
    ## Get par given in the command line and put them to p
    if (inherits(x, "orditkplot")) {
        dots <- x$par
        for (arg in names(x$args))
            assign(arg, unlist(x$args[arg]))
    } else {
        dots <- match.call(expand.dots = FALSE)$...
    }
    if (length(dots) > 0) {
        dots <- dots[names(dots) %in% sparnam]
        ## eval() or mar=c(4,4,1,1) will be a call, not numeric
        dots <- lapply(dots, function(x) if (is.call(x)) eval(x) else x)
        p <- check.options(new = dots, name.opt = "p",
                           envir = environment())
    }
    savepar <- p[sparnam]
    PPI <- 72                                         # Points per Inch
    p2p <- as.numeric(tcltk::tclvalue(tcltk::tcl("tk", "scaling"))) # Pixel per point
    DIAM <- 2.7                               # diam of plotting symbol
    ## Plotting symbol diam
    diam <- round(pcex * DIAM * p2p, 1)
    ## Sanitize colours
    sanecol <- function(x) {
        if (is.numeric(x))
            x <- palette()[x]
        x <- gsub("transparent", "", x)
        x[is.na(x)] <- ""
        x
    }
    p$bg <- sanecol(p$bg)
    p$fg <- sanecol(p$fg)
    p$col <- sanecol(p$col)
    p$col.axis <- sanecol(p$col.axis)
    p$col.lab <- sanecol(p$col.lab)
    ## Point and label colours
    if (missing(pcol))
        pcol <- p$col
    if (missing(pbg))
        pbg <- "transparent"
    if (missing(tcol))
        tcol <- p$col
    pcol <- sanecol(pcol)
    pbg <- sanecol(pbg)
    tcol <- sanecol(tcol)
    ## Define fonts
    idx <- match(p$family, c("","serif","sans","mono"))
    if (!is.na(idx))
        p$family <- c("Helvetica", "Times", "Helvetica", "Courier")[idx]
    saneslant <- function(x) {
        list("roman", "bold", "italic", c("bold", "italic"))[[x]]
    }
    ## fnt must be done later, since family, font and size can be
    ## vectors and slant can be of length 1 or 2
    ## fnt <- c(p$family, round(p$ps*p$cex*tcex), saneslant(p$font))
    labfam <- p$family
    labsize <- round(p$ps * p$cex * tcex)
    fnt.axis <- c(p$family, round(p$ps*p$cex.axis), saneslant(p$font.axis))
    fnt.lab <- c(p$family, round(p$ps*p$cex.lab), saneslant(p$font.lab))
    ## Imitate R plotting symbols pch
    SQ <- sqrt(2)     # Scaling factor for plot
    Point <- function(x, y, pch, col, fill, diam) {
        x <- round(x)
        y <- round(y)
        switch(as.character(pch),
               "0" = Point(x, y, 22, col, fill = "", diam),
               "1" = Point(x, y, 21, col, fill = "", diam),
               "2" = Point(x, y, 24, col, fill = "", diam),
               "3" = {tcltk::tkcreate(can, "line",
                               x, y+SQ*diam, x, y-SQ*diam, fill=col)
                      tcltk::tkcreate(can, "line",
                               x+SQ*diam, y, x-SQ*diam, y, fill=col)},
               "4" = {tcltk::tkcreate(can, "line",
                               x-diam, y-diam, x+diam, y+diam, fill=col)
                      tcltk::tkcreate(can, "line",
                               x-diam, y+diam, x+diam, y-diam, fill=col)},
               "5" = Point(x, y, 23, col, fill = "", diam),
               "6" = Point(x, y, 25, col, fill = "", diam),
               "7" = {Point(x, y, 4, col, fill, diam)
                      Point(x, y, 0, col, fill, diam)},
               "8" = {Point(x, y, 3, col, fill, diam)
                      Point(x, y, 4, col, fill, diam)},
               "9" = {Point(x, y, 3, col, fill, diam)
                      Point(x, y, 5, col, fill, diam)},
               "10" = {Point(x, y, 3, col, fill, diam/SQ)
                       Point(x, y, 1, col, fill, diam)},
               "11" = {Point(x, y, 2, col, fill, diam)
                       Point(x, y, 6, col, fill, diam)},
               "12" = {Point(x, y, 3, col, fill, diam/SQ)
                       Point(x, y, 0, col, fill, diam)},
               "13" = {Point(x, y, 4, col, fill, diam)
                       Point(x, y, 1, col, fill, diam)},
               "14" = {tcltk::tkcreate(can, "line", x-diam, y-diam, x, y+diam,
                                fill = col)
                       tcltk::tkcreate(can, "line", x+diam, y-diam, x, y+diam,
                                fill = col)
                       Point(x, y, 0, col, fill, diam)},
               "15" = Point(x, y, 22, col = col, fill = col, diam),
               "16" = Point(x, y, 21, col = col, fill = col, diam),
               "17" = Point(x, y, 24, col = col, fill = col, diam),
               "18" = Point(x, y, 23, col = col, fill = col, diam/SQ),
               "19" = Point(x, y, 21, col = col, fill = col, diam),
               "20" = Point(x, y, 21, col = col, fill = col, diam/2),
               "21" = tcltk::tkcreate(can, "oval", x-diam, y-diam,
               x+diam, y+diam, outline = col, fill = fill),
               "22" = tcltk::tkcreate(can, "rectangle", x-diam, y-diam,
               x+diam, y+diam, outline = col, fill = fill),
               "23" = tcltk::tkcreate(can, "polygon", x, y+SQ*diam,
               x+SQ*diam, y, x, y-SQ*diam, x-SQ*diam, y,
               outline = col, fill = fill),
               "24" = tcltk::tkcreate(can, "polygon", x, y-SQ*diam,
               x+sqrt(6)/2*diam, y+SQ/2*diam, x-sqrt(6)/2*diam, y+SQ/2*diam,
               outline = col, fill = fill),
               "25" = tcltk::tkcreate(can, "polygon", x, y+SQ*diam,
               x+sqrt(6)/2*diam, y-SQ/2*diam, x-sqrt(6)/2*diam, y-SQ/2*diam,
               outline = col, fill = fill),
               "o" = Point(x, y, 1, col, fill, diam),
               ## default: text with dummy location of the label
               {tcltk::tkcreate(can, "text",
                        x, y, text = as.character(pch), fill = col)
                Point(x, y, 21, col="", fill="", diam)}
                )
    }

############################
### Initialize Tcl/Tk Window
############################

    ## toplevel
    w <- tcltk::tktoplevel()
    tcltk::tktitle(w) <- deparse(match.call())
    ## Max dim of windows (depends on screen)
    YSCR <- as.numeric(tcltk::tkwinfo("screenheight", w)) - 150
    XSCR <- as.numeric(tcltk::tkwinfo("screenwidth", w))

################################
### Buttons and button functions
################################

    ## Buttons
    buts <- tcltk::tkframe(w)
    ## Copy current canvas to EPS using the standard Tcl/Tk utility
    cp2eps <- tcltk::tkbutton(buts, text="Copy to EPS",
                       command=function() tcltk::tkpostscript(can, x=0, y=0,
                       height=height, width=width,
                       file=tcltk::tkgetSaveFile(
                         filetypes="{{EPS file} {.eps}}",
                         defaultextension=".eps")))
    dismiss <- tcltk::tkbutton(buts, text="Close",
                               command=function() tcltk::tkdestroy(w))
    ## Dump current plot to an "orditkplot" object (internally)
    ordDump <- function() {
        xy <- matrix(0, nrow=nrow(sco), ncol=2)
        rownames(xy) <- rownames(sco)
        colnames(xy) <- colnames(sco)
        for(nm in names(pola)) {
            xy[as.numeric(tcltk::tclvalue(id[[nm]])),] <- xy2usr(nm)
        }
        curdim <- round(c(width, height) /PPI/p2p, 2)
        ## Sanitize colours for R plot
        pbg[pbg == ""]  <- "transparent"
        pcol[pcol == ""] <- "transparent"
        ## Reduce vector args if all entries are constant
        argcollapse <- function(x)
            if (length(unique(x)) == 1) x[1] else x
        pch <- argcollapse(pch)
        pcol <- argcollapse(pcol)
        pbg <- argcollapse(pbg)
        tcol <- argcollapse(tcol)
        ## Save
        args <- list(tcex = tcex, tcol = tcol, pch = pch, pcol = pcol,
                     pbg = pbg, pcex = pcex, xlim = xlim, ylim = ylim)
        xy <- list(labels = xy, points = sco, par = savepar, args = args,
                   dim = curdim)
        class(xy) <- "orditkplot"
        xy
    }
    ## Button to dump "orditkplot" object to the R session
    pDump <- function() {
        xy <- ordDump()
        dumpVar <- tcltk::tclVar("")
        tt <- tcltk::tktoplevel()
        tcltk::tktitle(tt) <- "R Dump"
        entryDump <- tcltk::tkentry(tt, width=20, textvariable=dumpVar)
        tcltk::tkgrid(tcltk::tklabel(tt, text="Enter name for an R object"))
        tcltk::tkgrid(entryDump, pady="5m")
        isDone <- function() {
            dumpName <- tcltk::tclvalue(dumpVar)
            if (exists(dumpName, envir = parent.frame())) {
                ok <- tcltk::tkmessageBox(message=paste(sQuote(dumpName),
                                   "exists.\nOK to overwrite?"),
                                   icon="warning", type="okcancel",
                                   default="ok")
                if(tcltk::tclvalue(ok) == "ok") {
                    assign(dumpName, xy, envir = parent.frame())
                    tcltk::tkdestroy(tt)
                }
            }
            else {
                assign(dumpName, xy, envir = parent.frame())
                tcltk::tkdestroy(tt)
            }
        }
        tcltk::tkbind(entryDump, "<Return>", isDone)
        tcltk::tkfocus(tt)
    }
    dump <- tcltk::tkbutton(buts, text="Save to R", command=pDump)
    ## Button to write current "orditkplot" object to a graphical device
    devDump <- function() {
        xy <- ordDump()
        ftypes <- c("eps" = "{EPS File} {.eps}",
                    "pdf" = "{PDF File} {.pdf}",
                    "svg" = "{SVG File} {.svg}",
                    "png" = "{PNG File} {.png}",
                    "jpg" = "{JPEG File} {.jpg .jpeg}",
                    "bmp" = "{BMP File} {.bmp}",
                    "tiff"= "{TIFF File} {.tif .tiff}",
                    "fig" = "{XFig File} {.fig}")
        falt <- rep(TRUE, length(ftypes))
        names(falt) <- names(ftypes)
        if (!capabilities("png"))
            falt["png"] <- FALSE
        if (!capabilities("jpeg"))
            falt["jpg"] <- FALSE
        if (!capabilities("cairo"))
            falt["svg"] <- FALSE
        ## Should work also in R < 2.8.0 with no capabilities("tiff")
        if (!isTRUE(unname(capabilities("tiff"))))
            falt["tiff"] <- FALSE
        ftypes <- ftypes[falt]
        ## External Tcl/Tk in Windows seems to buggy with type
        ## extensions of the file name: the extension is not
        ## automatically appended, but defaultextension is interpreted
        ## wrongly so that its value is not used as extension but
        ## correct appending is done if defaultextension has any
        ## value. The following kluge is against Tcl/Tk documentation,
        ## and should be corrected if Tcl/Tk is fixed.
        if (.Platform$OS.type == "windows")
            fname <- tcltk::tkgetSaveFile(filetypes=ftypes,
                                          defaultextension = TRUE)
        else
            fname <- tcltk::tkgetSaveFile(filetypes=ftypes)
        if(tcltk::tclvalue(fname) == "")
            return(NULL)
        fname <- tcltk::tclvalue(fname)
        ftype <- unlist(strsplit(fname, "\\."))
        ftype <- ftype[length(ftype)]
        if (ftype == "jpeg")
            ftype <- "jpg"
        if (ftype == "tif")
            ftype <- "tiff"
        mess <- "is not a supported type: file not produced. Supported types are"
        if (!(ftype %in% names(ftypes))) {
            tcltk::tkmessageBox(message=paste(sQuote(ftype), mess, paste(names(ftypes),
                         collapse=", ")), icon="warning")
            return(NULL)
        }
        pixdim <- round(xy$dim*PPI*p2p)
        switch(ftype,
               eps = postscript(file=fname, width=xy$dim[1], height=xy$dim[2],
               paper="special", horizontal = FALSE),
               pdf = pdf(file=fname, width=xy$dim[1], height=xy$dim[2]),
               svg = svg(filename=fname, width=xy$dim[1], height=xy$dim[2]),
               png = png(filename=fname, width=pixdim[1], height=pixdim[2]),
               jpg = jpeg(filename=fname, width=pixdim[1], height=pixdim[2],
               quality = 100),
               tiff = tiff(filename=fname, width=pixdim[1], height=pixdim[2]),
               bmp = bmp(filename=fname, width=pixdim[1], height=pixdim[2]),
               fig = xfig(file=fname, width=xy$dim[1], height=xy$dim[2]))
        plot.orditkplot(xy)
        dev.off()
    }
    export <- tcltk::tkbutton(buts, text="Export plot", command=devDump)

##########
### Canvas
##########

    ## Make canvas
    sco <- try(scores(x, display=display, choices = choices, ...),
               silent = TRUE)
    if (inherits(sco, "try-error")) {
        tcltk::tkmessageBox(message=paste("No ordination scores were found in",
                     sQuote(deparse(substitute(x)))), icon="error")
        tcltk::tkdestroy(w)
        stop("argument x did not contain ordination scores")
    }
    if (!missing(labels))
        rownames(sco) <- labels
    ## Recycle graphical parameters in plots
    nr <- nrow(sco)
    pcol <- rep(pcol, length=nr)
    pbg <- rep(pbg, length=nr)
    pch <- rep(pch, length=nr)
    tcol <- rep(tcol, length=nr)
    diam <- rep(diam, length=nr)
    labfam <- rep(labfam, length=nr)
    labsize <- rep(labsize, length=nr)
    if (inherits(x, "ordipointlabel"))
        labfnt <- attr(x$labels, "font")
    else
        labfnt <- rep(p$font, length=nr)
    ## Select only items within xlim, ylim
    take <- rep(TRUE, nr)
    if (!missing(xlim))
        take <- take & sco[,1] >= xlim[1] & sco[,1] <= xlim[2]
    if (!missing(ylim))
        take <- take & sco[,2] >= ylim[1] & sco[,2] <= ylim[2]
    sco <- sco[take,, drop=FALSE]
    labs <- rownames(sco)
    pcol <- pcol[take]
    pbg <- pbg[take]
    tcol <- tcol[take]
    pch <- pch[take]
    diam <- diam[take]
    labfam <- labfam[take]
    labsize <- labsize[take]
    labfnt <- labfnt[take]
    ## Ranges and pretty values for axes
    if (missing(xlim))
        xlim <- range(sco[,1], na.rm = TRUE)
    if (missing(ylim))
        ylim <- range(sco[,2], na.rm = TRUE)
    xpretty <- pretty(xlim)
    ypretty <- pretty(ylim)
    ## Extend ranges by 4%
    xrange <- c(-0.04, 0.04) * diff(xlim) + xlim
    xpretty <- xpretty[xpretty >= xrange[1] & xpretty <= xrange[2]]
    yrange <- c(-0.04, 0.04) * diff(ylim) + ylim
    ypretty <- ypretty[ypretty >= yrange[1] & ypretty <= yrange[2]]
    ## Canvas like they were in the default devices when I last checked
    if (missing(width))
        width <- p$din[1]
    width <- width * PPI * p2p
    ## Margin row width also varies with platform and devices
    ## rpix <- (p$mai/p$mar * PPI * p2p)[1]
    rpix <- p$cra[2]
    mar <- round(p$mar * rpix)
    xusr <- width - mar[2] - mar[4]
    xincr <- xusr/diff(xrange)
    yincr <- xincr
    xy0 <- c(xrange[1], yrange[2]) # upper left corner
    ## Functions to translate scores to canvas coordinates and back
    usr2xy <- function(row) {
        x <- (row[1] - xy0[1]) * xincr + mar[2]
        y <- (xy0[2] - row[2]) * yincr + mar[3]
        c(x,y)
    }
    ## User coordinates of an item
    xy2usr <- function(item) {
        xy <- as.numeric(tcltk::tkcoords(can, item))
        x <- xy[1]
        y <- xy[2]
        x <- xrange[1] + (x - mar[2])/xincr
        y <- yrange[2] - (y - mar[3])/yincr
        c(x,y)
    }
    ## Canvas x or y to user coordinates
    x2usr <- function(xcan) {
        xrange[1] + (xcan - mar[2])/xincr
    }
    y2usr <- function(ycan) {
        yrange[2] - (ycan - mar[3])/yincr
    }
    ## Equal aspect ratio
    height <- round((diff(yrange)/diff(xrange)) * xusr)
    height <- height + mar[1] + mar[3]
    ## Canvas, finally
    can <- tcltk::tkcanvas(w, relief="sunken", width=width, height=min(height,YSCR),
                    scrollregion=c(0,0,width,height))
    if (p$bg != "")
        tcltk::tkconfigure(can, bg=p$bg)
    yscr <- tcltk::tkscrollbar(w, command =
                               function(...) tcltk::tkyview(can, ...))
    tcltk::tkconfigure(can, yscrollcommand =
                       function(...) tcltk::tkset(yscr, ...))
    ## Pack it up
    tcltk::tkpack(buts, side="bottom", fill="x", pady="2m")
    tcltk::tkpack(can, side="left", fill="x")
    tcltk::tkpack(yscr, side="right", fill="y")
    tcltk::tkgrid(cp2eps, export, dump, dismiss, sticky="s")
    ## Box
    x0 <- usr2xy(c(xrange[1], yrange[1]))
    x1 <- usr2xy(c(xrange[2], yrange[2]))
    tcltk::tkcreate(can, "rectangle", x0[1], x0[2], x1[1], x1[2], outline = p$fg,
             width = p$lwd)
    ## Axes and ticks
    tl <-  -p$tcl * rpix     # -p$tcl * p$ps * p2p
    axoff <- p$mgp[3] * rpix
    tmp <- xpretty
    for (i in seq_along(tmp)) {
        x0 <- usr2xy(c(xpretty[1], yrange[1]))
        x1 <- usr2xy(c(xpretty[length(xpretty)], yrange[1]))
        tcltk::tkcreate(can, "line", x0[1], x0[2]+axoff, x1[1], x1[2]+axoff,
                 fill=p$fg)
        xx <- usr2xy(c(tmp[i], yrange[1]))
        tcltk::tkcreate(can, "line", xx[1], xx[2] + axoff, xx[1],
                        xx[2]+tl+axoff, fill=p$fg)
        tcltk::tkcreate(can, "text", xx[1], xx[2] + rpix * p$mgp[2], anchor="n",
                 text=as.character(tmp[i]), fill=p$col.axis, font=fnt.axis)
    }
    xx <- usr2xy(c(mean(xrange), yrange[1]))
    tcltk::tkcreate(can, "text", xx[1], xx[2] + rpix * p$mgp[1],
             text=colnames(sco)[1], fill=p$col.lab, anchor="n", font=fnt.lab)
    tmp <- ypretty
    for (i in seq_along(tmp)) {
        x0 <- usr2xy(c(xrange[1], tmp[1]))
        x1 <- usr2xy(c(xrange[1], tmp[length(tmp)]))
        tcltk::tkcreate(can, "line", x0[1]-axoff, x0[2], x1[1]-axoff, x1[2])
        yy <- usr2xy(c(xrange[1], tmp[i]))
        tcltk::tkcreate(can, "line", yy[1]-axoff, yy[2], yy[1]-tl-axoff, yy[2],
                 fill=p$fg )
        tcltk::tkcreate(can, "text", yy[1] - rpix * p$mgp[2] , yy[2], anchor="e",
                 text=as.character(tmp[i]), fill = p$col.axis, font=fnt.axis)
    }
    ## Points and labels

    ## The following 'inherits' works with ordipointlabel, but not
    ## with zooming
    if (inherits(x, "orditkplot")) {
        lsco <- scores(x, "labels")
        laboff <- rep(0, nrow(lsco))
        lsco <- lsco[rownames(sco),]
    } else {
        lsco <- sco
        laboff <- round(p2p * p$ps/2 + diam + 1)
    }
    pola <- tcltk::tclArray()        # points
    labtext <- tcltk::tclArray()     # text
    id <- tcltk::tclArray()          # index
    for (i in 1:nrow(sco)) {
        xy <- usr2xy(sco[i,])
        item <- Point(xy[1], xy[2], pch = pch[i], col = pcol[i],
                      fill = pbg[i], diam = diam[i])
        xy <- usr2xy(lsco[i,])
        fnt <- c(labfam[i], labsize[i], saneslant(labfnt[i]))
        lab <- tcltk::tkcreate(can, "text", xy[1], xy[2]-laboff[i], text=labs[i],
                        fill = tcol[i], font=fnt)
        tcltk::tkaddtag(can, "point", "withtag", item)
        tcltk::tkaddtag(can, "label", "withtag", lab)
        pola[[lab]] <- item
        labtext[[lab]] <- labs[i]
        id[[lab]] <- i
    }

##############################
### Mouse operations on canvas
##############################

    ## Plotting and Moving
    ## Mouse enters a label
    pEnter <- function() {
        tcltk::tkdelete(can, "box")
        hbox <- tcltk::tkcreate(can, "rectangle",
                                tcltk::tkbbox(can, "current"),
                         outline = "red", fill = "yellow")
        tcltk::tkaddtag(can, "box", "withtag", hbox)
        tcltk::tkitemraise(can, "current")
    }
    ## Mouse leaves a label
    pLeave <- function() {
        tcltk::tkdelete(can, "box")
    }
    ## Select label
    pDown <- function(x, y) {
        x <- as.numeric(x)
        y <- as.numeric(y)
        tcltk::tkdtag(can, "selected")
        tcltk::tkaddtag(can, "selected", "withtag", "current")
        tcltk::tkitemraise(can, "current")
        p <- as.numeric(tcltk::tkcoords(can,
                             pola[[tcltk::tkfind(can, "withtag", "current")]]))
        .pX <<- (p[1]+p[3])/2
        .pY <<- (p[2]+p[4])/2
        .lastX <<- x
        .lastY <<- y
    }
    ## Move label
    pMove <- function(x, y) {
        x <- as.numeric(x)
        y <- as.numeric(y)
        tcltk::tkmove(can, "selected", x - .lastX, y - .lastY)
        tcltk::tkdelete(can, "ptr")
        tcltk::tkdelete(can, "box")
        .lastX <<- x
        .lastY <<- y
        ## xadj,yadj: adjust for canvas scrolling
        xadj <- as.numeric(tcltk::tkcanvasx(can, 0))
        yadj <- as.numeric(tcltk::tkcanvasy(can, 0))
        hbox <- tcltk::tkcreate(can, "rectangle",
                                tcltk::tkbbox(can, "selected"),
                                outline = "red")
        tcltk::tkaddtag(can, "box", "withtag", hbox)
        conn <- tcltk::tkcreate(can, "line", .lastX + xadj, .lastY+yadj,
                         .pX, .pY, fill="red")
        tcltk::tkaddtag(can, "ptr", "withtag", conn)
    }
    ## Edit label
    pEdit <- function() {
        tcltk::tkdtag(can, "selected")
        tcltk::tkaddtag(can, "selected", "withtag", "current")
        tcltk::tkitemraise(can, "current")
        click <- tcltk::tkfind(can, "withtag", "current")
        txt <- tcltk::tclVar(labtext[[click]])
        i <- as.numeric(id[[click]])
        tt <- tcltk::tktoplevel()
        labEd <- tcltk::tkentry(tt, width=20, textvariable=txt)
        tcltk::tkgrid(tcltk::tklabel(tt, text = "Edit label"))
        tcltk::tkgrid(labEd, pady="5m", padx="5m")
        isDone <- function() {
            txt <- tcltk::tclvalue(txt)
            tcltk::tkitemconfigure(can, click, text = txt)
            rownames(sco)[i] <<- txt
            tcltk::tkdestroy(tt)
        }
        tcltk::tkbind(labEd, "<Return>", isDone)
    }
    ## Zooming: draw rectangle and take its user coordinates
    ## Rectangle: first corner
    pRect0 <- function(x, y) {
        x <- as.numeric(x)
        y <- as.numeric(y)
        ## yadj here and below adjusts for canvas scrolling
        yadj <- as.numeric(tcltk::tkcanvasy(can, 0))
        .pX <<- x
        .pY <<- y + yadj
    }
    ## Grow rectangle
    pRect <- function(x, y) {
        x <- as.numeric(x)
        y <- as.numeric(y)
        tcltk::tkdelete(can, "box")
        yadj <- as.numeric(tcltk::tkcanvasy(can, 0))
        .lastX <<- x
        .lastY <<- y + yadj
        rect <- tcltk::tkcreate(can, "rectangle", .pX, .pY, .lastX, .lastY,
                         outline="blue")
        tcltk::tkaddtag(can, "box", "withtag", rect)
    }
    ## Redraw ordiktplot with new xlim and ylim
    pZoom <- function() {
        nxlim <- sort(c(x2usr(.pX), x2usr(.lastX)))
        nylim <- sort(c(y2usr(.pY), y2usr(.lastY)))
        xy <- ordDump()
        ## Move labels closer to points in zoom
        ## FIXME: Doesn't do a perfect job
        mul <- abs(diff(nxlim)/diff(xlim))
        xy$labels <- xy$points + (xy$labels - xy$points)*mul
        xy$args$xlim <- nxlim
        xy$args$ylim <- nylim
        orditkplot(xy)
    }
    ## Dummy location of the mouse
    .lastX <- 0
    .lastY <- 0
    .pX <- 0
    .pY <- 0
    ## Mouse bindings:
    ## Moving a label
    tcltk::tkitembind(can, "label", "<Any-Enter>", pEnter)
    tcltk::tkitembind(can, "label", "<Any-Leave>", pLeave)
    tcltk::tkitembind(can, "label", "<1>", pDown)
    tcltk::tkitembind(can, "label", "<ButtonRelease-1>",
               function() {tcltk::tkdtag(can, "selected")
                           tcltk::tkdelete(can, "ptr")})
    tcltk::tkitembind(can, "label", "<B1-Motion>", pMove)
    ## Edit labels
    tcltk::tkitembind(can, "label", "<Double-Button-1>", pEdit)
    ## Zoom (with one-button mouse)
    tcltk::tkbind(can, "<Shift-Button-1>", pRect0)
    tcltk::tkbind(can, "<Shift-B1-Motion>", pRect)
    tcltk::tkbind(can, "<Shift-ButtonRelease>", pZoom)
    ## Zoom (with right button)
    tcltk::tkbind(can, "<Button-3>", pRect0)
    tcltk::tkbind(can, "<B3-Motion>", pRect)
    tcltk::tkbind(can, "<ButtonRelease-3>", pZoom)
}

"plot.radline" <-
    function (x, xlab = "Rank", ylab = "Abundance", type = "b", ...) 
{
    rad <- x$y
    fit <- fitted(x)
    rnk <- seq(along = rad)
    plot(rnk, rad, log = "y", xlab = xlab, ylab = ylab, type = "n", 
         ...)
    out <- NULL
    if (type == "b" || type == "p") 
        out <- points(x, ...)
    if (type == "b" || type == "l") 
        lines(x, ...)
    invisible(out)
}

`hierParseFormula` <-
    function (formula, data)
{
    lhs <- formula[[2]]
    if (any(attr(terms(formula, data = data), "order") > 1))
        stop("interactions are not allowed")
    lhs <- as.matrix(eval(lhs, environment(formula), parent.frame()))
    formula[[2]] <- NULL
    rhs <- model.frame(formula, data, drop.unused.levels = TRUE)
    rhs[] <- lapply(rhs, function(u) {
        if (!is.factor(u))
            u <- factor(u)
        u
    })
    ## take care that the first column is a unique identifier for rows
    ## and the last column is constant for pooling all rows together
    if (length(unique(rhs[,1])) < nrow(rhs))
        rhs <- cbind("unit" = factor(seq_len(nrow(rhs))), rhs)
    if (length(unique(rhs[, ncol(rhs)])) > 1)
        rhs <- cbind(rhs, "all" = factor(rep(1, nrow(rhs))))
    attr(rhs, "terms") <- NULL
    list(lhs=lhs, rhs=rhs)
}

### Method functions for monoMDS objects

`print.monoMDS` <-
    function(x, ...)
{
    cat("\nCall:\n")
    cat(deparse(x$call), "\n\n")
    modlab <- switch(x$model,
                     global = "Non-metric",
                     local = "Local non-metric",
                     linear = "Linear",
                     hybrid = "Hybrid")
    cat(paste(modlab, "Multidimensional Scaling\n\n"))
    cat(x$nobj, "points")
    cat(", dissimilarity", sQuote(x$distmethod))
    if (!is.null(x$distcall))
        cat(", call", sQuote(x$distcall))
    cat("\n\n")
    cat("Dimensions:", x$ndim, "\n")
    cat("Stress:    ", x$stress, "\n")
    cat("Stress type", x$isform)
    if (x$model != "linear")
        cat(", ", c("weak", "strong")[x$ities], " ties", sep = "")
    cat("\n")
    cat("Scores ")
    if (x$iscal == 1)
        cat("scaled to unit root mean square")
    else
        cat("unscaled")
    if (attr(x$points, "pc"))
        cat(", rotated to principal components")
    cat("\n")
    stoplab <- switch(x$icause,
                      "Maximum number of iterations (maxit) reached",
                      "Stress nearly zero (< smin)",
                      "Stress nearly unchanged (ratio > sratmax)",
                      "Scale factor of gradient nearly zero (< sfgrmin)")
    cat("Stopped after ", x$iters, " iterations: ", stoplab, "\n", sep="")
    invisible(x)
}

`scores.monoMDS` <-
    function(x, choices = NA, ...)
{
    if (any(is.na(choices)))
        x$points
    else {
        choices <- choices[choices <= x$ndim]
        x$points[, choices, drop = FALSE]
    }
}

`plot.monoMDS` <-
    function(x, choices = c(1,2), type = "t",  ...)
{
    ordiplot(x, display = "sites", choices = choices, type = type, ...)
}

`points.monoMDS` <-
    function(x, choices = c(1,2), select, ...)
{
    x <- scores(x, choices = choices)
    if (!missing(select))
        x <- .checkSelect(select, x)
    points(x, ...)
    invisible()
}

`text.monoMDS` <-
    function(x, labels, choices = c(1,2), select, ...)
{
    x <- scores(x, choices = choices)
    if (!missing(labels))
        rownames(x) <- labels
    if (!missing(select))
        x <- .checkSelect(select, x)
    text(x, labels = rownames(x), ...)
    invisible()
}

`print.MOStest` <-
    function(x, ...)
{
    cat("\nMitchell-Olds and Shaw test\n")
    cat("Null: hump of a quadratic linear predictor is at min or max\n")
    print(x$family)
    print(x$hump)
    if (!x$isBracketed)
        cat("***** Caution: hump/pit not bracketed by the data ******\n")
    cat("\n")
    printCoefmat(coef(x), has.Pvalue = TRUE, na.print = "")
    invisible(x)
}

#' Add Species Scores to Ordination Results
#'
#' @param object Ordination object
#' @param value Community data
#'

`sppscores<-` <-
    function(object, value)
{
    UseMethod("sppscores<-")
}

## dbrda

`sppscores<-.dbrda` <-
    function(object, value)
{
    object$vdata <- deparse(substitute(value))
    value <- scale(value, center = TRUE, scale = FALSE)
    object$colsum <- apply(value, 2, sd)
    if (!is.null(object$pCCA) && object$pCCA$rank > 0) {
        comm <- qr.resid(object$pCCA$QR, value)
    }
    if (!is.null(object$CCA) && object$CCA$rank > 0) {
        v <- crossprod(value, object$CCA$u)
        v <- decostand(v, "normalize", MARGIN = 2)
        object$CCA$v <- v
        value <- qr.resid(object$CCA$QR, value)
    }
    if (!is.null(object$CA) && object$CA$rank > 0) {
        v <- crossprod(value, object$CA$u)
        v <- decostand(v, "normalize", MARGIN = 2)
        object$CA$v <- v
    }
    object
}

## capscale may have species scores, but is otherwise similar to dbrda

`sppscores<-.capscale` <-
    function(object, value)
{
    object <- `sppscores<-.dbrda`(object, value)
    object$vdata <- deparse(substitute(value))
    object
}

## metaMDS

`sppscores<-.metaMDS` <-
    function(object, value)
{
    wa <- wascores(object$points, value, expand = TRUE)
    attr(wa, "data") <- deparse(substitute(value))
    object$species <- wa
    object
}

## the main purpose of accessor function is to provide nicer command
## autocompletion and cross-references in help, and of course, to tell
## that it is not implemented (and may never be)

`sppscores` <-
    function(object)
{
    .NotYetImplemented()
}

`prestondistr` <-
    function (x, truncate = -1,  ...) 
{
    fun <- function(par, x, truncate) {
        up <- dnorm(x, par[1], par[2], log = TRUE)
        dn <- pnorm(truncate, par[1], par[2], lower.tail = FALSE)
        -sum(up - log(dn))
    }
    x <- x[x > 0]
    logx <- log2(x)
    p <- c(mean(logx), sd(logx))
    sol <- optim(p, fun, x = logx, truncate = truncate)
    p <- sol$par
    area <- pnorm(truncate, p[1], p[2], lower.tail = FALSE)
    scale <- length(x)/sqrt(2 * pi)/p[2]/area
    p <- c(p, scale)
    oct <- as.preston(x, ...)
    x <- as.numeric(names(oct))
    fit <- p[3] * exp(-(x - p[1])^2/2/p[2]^2) 
    names(p) <- c("mode", "width", "S0")
    out <- list(freq = oct, fitted = fit, coefficients = p)
    out$method <- "maximized likelihood to log2 abundances"
    class(out) <- "prestonfit"
    out
}

`renyiaccum` <-
function(x, scales=c(0, 0.5, 1, 2, 4, Inf), permutations = 100,
         raw = FALSE, collector = FALSE, subset, ...)
{
    if (!missing(subset))
        x <- subset(x, subset)
    x <- as.matrix(x)
    n <- nrow(x)
    p <- ncol(x)
    if (p==1) {
        x <- t(x)
        n <- nrow(x)
        p <- ncol(x)
    }
    pmat <- getPermuteMatrix(permutations, n)
    permutations <- nrow(pmat)
    m <- length(scales)
    result <- array(dim=c(n,m,permutations))
    dimnames(result) <- list(pooled.sites=c(1:n), scale=scales,
                             permutation=c(1:permutations))
    for (k in 1:permutations) {
        result[,,k] <- as.matrix(renyi((apply(x[pmat[k,],],2,cumsum)),
                                       scales=scales, ...))
    }
    if (raw)
        collector <- FALSE
    if (collector)
        ref <- as.matrix(renyi(apply(x, 2, cumsum), scales = scales, ...))
    if (raw) {
        if (m==1) {
            result <- result[,1,]
        }
    }else{
        tmp <- array(dim=c(n,m,6 + as.numeric(collector)))
        for (i in 1:n) {
            for (j in 1:m) {
                tmp[i,j,1] <- mean(result[i,j,1:permutations]) 
                tmp[i,j,2] <- sd(result[i,j,1:permutations])
                tmp[i,j,3] <- min(result[i,j,1:permutations])
                tmp[i,j,4] <- max(result[i,j,1:permutations])
                tmp[i,j,5] <- quantile(result[i,j,1:permutations],0.025)
                tmp[i,j,6] <- quantile(result[i,j,1:permutations],0.975)
                if (collector)
                    tmp[i,j,7] <- ref[i,j]
            }
        }
        result <- tmp
        dimnames(result) <- list(pooled.sites=c(1:n),
                                  scale=scales,
                                  c("mean", "stdev", "min", "max", "Qnt 0.025", "Qnt 0.975", if (collector) "Collector"))
    }
    attr(result, "control") <- attr(pmat, "control")
    class(result) <- c("renyiaccum", class(result))
    result
}

plot.clamtest <- function(x, xlab, ylab, main,
    pch=21:24, col.points=1:4, col.lines=2:4, lty=1:3,
    position="bottomright", ...) {
    summ <- summary(x)
    glabel <- summ$labels
    if (missing(main))
        main <- "Species Classification"
    if (missing(xlab))
        xlab <- paste(glabel[2], "(abundance + 1)")
    if (missing(ylab))
        ylab <- paste(glabel[1], "(abundance + 1)")
    Y <- x[,2]
    X <- x[,3]
    minval <- summ$minv
    ## plot the dots
    rr <- range(X+1,Y+1)
    plot(X+1, Y+1, log = "xy", xaxt = "n", yaxt = "n",
        col=col.points[as.integer(x$Classes)],
        pch=pch[as.integer(x$Classes)], 
        xlab=xlab, ylab=ylab, main=main,
        xlim=rr, ylim=rr, ...)
    axis(1, c(1,10,100,1000,10000))
    axis(2, c(1,10,100,1000,10000))
    ## too rare threshold
    Ymin <- minval[[1]][1,2]
    Xmin <- minval[[2]][1,1]
    lines(rep(Xmin, 2)+1, c(0, 1)+1, col=col.lines[1], lty=lty[1])
    lines(c(0, 1)+1, rep(Ymin, 2)+1, col=col.lines[1], lty=lty[1])
    tmp <- approx(c(Xmin, 1), c(1, Ymin))
    lines(tmp$x+1, tmp$y+1, col=col.lines[1], lty=lty[1])
    ## Y vs. gen threshold
    lines(minval[[1]]+1, col=col.lines[2], lty=lty[2])
    ## X vs. gen threshold
    lines(minval[[2]]+1, col=col.lines[3], lty=lty[3])
    if (!is.null(position))
        legend(position, col=col.points, pch=pch, 
            legend=rownames(summ$summary))
    invisible(x)
}

`biplot.CCorA` <-
    function(x, plot.type="ov", xlabs, plot.axes = 1:2, int=0.5, col.Y="red", col.X="blue", cex=c(0.7,0.9), ...)
{
    ## Function sets par(): reset them on.exit
    opar <- par(no.readonly = TRUE)
    on.exit(par(opar))
    #### Internal function
    larger.frame <- function(mat, percent=0.10)
	# Produce an object plot 10% larger than strictly necessary
	{
	range.mat <- apply(mat,2,range)
	z <- apply(range.mat, 2, function(x) x[2]-x[1])
	range.mat[1,] <- range.mat[1,]-z*percent
	range.mat[2,] <- range.mat[2,]+z*percent
	range.mat
	}
	####

	TYPE <- c("objects","variables","ov","biplots")
	type <- pmatch(plot.type, TYPE)
	if(is.na(type)) stop("Invalid plot.type")

    epsilon <- sqrt(.Machine$double.eps)
	if(length(which(x$Eigenvalues > epsilon)) == 1)
            stop(gettextf(
                "axes (%s) not plotted because the solution has only one dimension",
                paste(plot.axes, collapse =",")))
	if(max(plot.axes) > length(which(x$Eigenvalues > epsilon)))
            stop(gettextf(
                "axes (%s) not plotted because the solution has fewer dimensions",
                 paste(plot.axes, collapse=",")))
	if (missing(xlabs))
		xlabs <- rownames(x$Cy)
	else if (!is.null(xlabs) && is.na(xlabs))
		xlabs <- rep(NA, nrow(x$Cy))
	else if (is.null(xlabs))
		xlabs <- 1:nrow(x$Cy)
	#
	lf.Y <- larger.frame(x$Cy[,plot.axes])
	lf.X <- larger.frame(x$Cx[,plot.axes])
	#
	# Four plot types are available
	if(type == 1) {  # Object plots
		# cat('plot.type = objects')
		par(mfrow=c(1,2), pty = "s")
		plot(lf.Y, asp=1, xlab=colnames(x$Cy[,plot.axes[1]]), ylab=colnames(x$Cy[,plot.axes[2]]), type="n")
		points(x$Cy[,plot.axes], col=col.Y)   # Solid dot: pch=19
		text(x$Cy[,plot.axes],labels=xlabs, pos=3, col=col.Y, cex=cex[1])
		title(main = c("CCorA object plot","First data table (Y)"), line=2)
		#
		plot(lf.X, asp=1, xlab=colnames(x$Cy[,plot.axes[1]]), ylab=colnames(x$Cy[,plot.axes[2]]), type="n")
		points(x$Cx[,plot.axes], col=col.X)   # Solid dot: pch=19
		text(x$Cx[,plot.axes],labels=xlabs, pos=3, col=col.X, cex=cex[1])
		title(main = c("CCorA object plot","Second data table (X)"), line=2)
		###
		###
		} else if(type == 2) {  # Variable plots
		# cat('plot.type = variables')
		par(mfrow=c(1,2), pty = "s")
		plot(x$corr.Y.Cy[,plot.axes], asp=1, xlim=c(-1,1), ylim=c(-1,1), xlab=colnames(x$Cy[,plot.axes[1]]),
			ylab=colnames(x$Cy[,plot.axes[2]]), type="n")
		text(x$corr.Y.Cy[,plot.axes],labels=rownames(x$corr.Y.Cy), pos=3, col=col.Y, cex=cex[2])
		arrows(0,0,x$corr.Y.Cy[,plot.axes[1]],x$corr.Y.Cy[,plot.axes[2]], length=0.05, col=col.Y)
		abline(h=0, v=0)
		lines(cos(seq(0, 2*pi, l=100)), sin(seq(0, 2*pi, l=100)))
		lines(int * cos(seq(0, 2*pi, l=100)), int * sin(seq(0, 2*pi, l=100)))
		title(main = c("CCorA variable plot","First data table (Y)"), line=2)
		#
		plot(x$corr.X.Cx[,plot.axes], asp=1, xlim=c(-1,1), ylim=c(-1,1), xlab=colnames(x$Cy[,plot.axes[1]]),
			ylab=colnames(x$Cy[,plot.axes[2]]), type="n")
		text(x$corr.X.Cx[,plot.axes],labels=rownames(x$corr.X.Cx), pos=3, col=col.X, cex=cex[2])
		arrows(0,0,x$corr.X.Cx[,plot.axes[1]],x$corr.X.Cx[,plot.axes[2]], length=0.05, col=col.X)
		abline(h=0, v=0)
	    lines(cos(seq(0, 2*pi, l=100)), sin(seq(0, 2*pi, l=100)))
	    lines(int * cos(seq(0, 2*pi, l=100)), int * sin(seq(0, 2*pi, l=100)))
		title(main = c("CCorA variable plot","Second data table (X)"), line=2)
		###
		###
		} else if(type == 3) {  # Object and variable plots
		# cat('plot.type = ov')
		# par(mfrow=c(2,2), mar=c(4.5,3.5,2,1))
        layout(matrix(c(1,2,3,4), ncol = 2, nrow = 2,
            byrow = TRUE), widths = 1, heights = c(0.5,0.5))
        par(pty = "s", mar = c(4.5,3.5,2,1))
		#
		plot(lf.Y, asp=1, xlab=colnames(x$Cy[,plot.axes[1]]), ylab=colnames(x$Cy[,plot.axes[2]]), type="n")
		points(x$Cy[,plot.axes], col=col.Y)   # Solid dot: pch=19
		text(x$Cy[,plot.axes],labels=xlabs, pos=3, col=col.Y, cex=cex[1])
		title(main = c("First data table (Y)"), line=1)
		#
		plot(lf.X, asp=1, xlab=colnames(x$Cy[,plot.axes[1]]), ylab=colnames(x$Cy[,plot.axes[2]]), type="n")
		points(x$Cx[,plot.axes], col=col.X)   # Solid dot: pch=19
		text(x$Cx[,plot.axes],labels=xlabs, pos=3, col=col.X, cex=cex[1])
		title(main = c("Second data table (X)"), line=1)
		#
		plot(x$corr.Y.Cy[,plot.axes], asp=1, xlim=c(-1,1), ylim=c(-1,1), xlab=colnames(x$Cy[,plot.axes[1]]),
			ylab=colnames(x$Cy[,plot.axes[2]]), type="n")
		text(x$corr.Y.Cy[,plot.axes],labels=rownames(x$corr.Y.Cy), pos=3, col=col.Y, cex=cex[2])
		arrows(0,0,x$corr.Y.Cy[,plot.axes[1]],x$corr.Y.Cy[,plot.axes[2]], length=0.05, col=col.Y)
		abline(h=0, v=0)
		lines(cos(seq(0, 2*pi, l=100)), sin(seq(0, 2*pi, l=100)))
		lines(int * cos(seq(0, 2*pi, l=100)), int * sin(seq(0, 2*pi, l=100)))
		#
		plot(x$corr.X.Cx[,plot.axes], asp=1, xlim=c(-1,1), ylim=c(-1,1), xlab=colnames(x$Cy[,plot.axes[1]]),
			ylab=colnames(x$Cy[,plot.axes[2]]), type="n")
		text(x$corr.X.Cx[,plot.axes],labels=rownames(x$corr.X.Cx), pos=3, col=col.X, cex=cex[2])
		arrows(0,0,x$corr.X.Cx[,plot.axes[1]],x$corr.X.Cx[,plot.axes[2]], length=0.05, col=col.X)
		abline(h=0, v=0)
	    lines(cos(seq(0, 2*pi, l=100)), sin(seq(0, 2*pi, l=100)))
	    lines(int * cos(seq(0, 2*pi, l=100)), int * sin(seq(0, 2*pi, l=100)))
		###
		###
		} else if(type == 4) {  # Biplots
		# cat('plot.type = biplot')
		par(mfrow=c(1,2), pty = "s")
		biplot(x$Cy[,plot.axes], x$corr.Y.Cy[,plot.axes], col=c("black",col.Y), xlim=lf.Y[,1], ylim=lf.Y[,2],
			xlabs = xlabs, arrow.len=0.05, cex=cex, ...)
		title(main = c("CCorA biplot","First data table (Y)"), line=4)
		#
		biplot(x$Cx[,plot.axes], x$corr.X.Cx[,plot.axes], col=c("black",col.X), xlim=lf.X[,1], ylim=lf.X[,2],
			xlabs = xlabs, arrow.len=0.05, cex=cex, ...)
		title(main = c("CCorA biplot","Second data table (X)"), line=4)
		}
	invisible()
}

`tsallis` <-
    function (x, scales = seq(0, 2, 0.2), norm=FALSE, hill=FALSE)
{
    if (norm && hill)
        stop("'norm = TRUE' and 'hill = TRUE' should not be used at the same time")
    x <- as.matrix(x)
    if (!is.numeric(x))
        stop("input data must be numeric")
    n <- nrow(x)
    p <- ncol(x)
    if (p == 1) {
        x <- t(x)
        n <- nrow(x)
        p <- ncol(x)
    }
    x <- decostand(x, "total", 1)
    m <- length(scales)
    result <- array(0, dim = c(n, m))
    dimnames(result) <- list(sites = rownames(x), scale = scales)
    for (a in 1:m) {
        if (scales[a] != 1 && scales[a] != 0) {
                result[, a] <- (1-(rowSums(x^scales[a])))/(scales[a] - 1)
        }
        else {
            if (scales[a] == 1) result[, a] <- diversity(x, "shannon")
            if (scales[a] == 0) result[, a] <- rowSums(x > 0) - 1
        }
        if (norm) {
            ST <- rowSums(x > 0)
            if (scales[a] == 1) result[, a] <- result[, a] / log(ST)
            else result[, a] <- result[, a] / ((ST^(1-scales[a]) - 1) / (1 - scales[a]))
        }
        if (hill) {
            result[, a] <- if (scales[a] == 1) {
                exp(result[, a])
            } else {
                (1 - (scales[a] - 1) * result[, a])^(1/(1-scales[a]))
            }
        }
    }
    result <- as.data.frame(result)
    if (any(dim(result) == 1))
        result <- unlist(result, use.names = TRUE)
    class(result) <- c("tsallis", "renyi", class(result))
    result
}

`bstick.cca` <-
    function(n, ...)
{
    if(!inherits(n, c("rda", "cca")))
        stop("'n' not of class \"cca\" or \"rda\"")
    if(!is.null(n$CCA) && n$CCA$rank > 0)
        stop("'bstick' only for unconstrained models")
    ## No idea how to define bstick for dbrda or capscale with
    ## negative eigenvalues
    if (inherits(n, c("dbrda", "capscale")) &&
        (!is.null(n$CA$imaginary.u) || !is.null(n$CA$imaginary.u.eig)))
        stop(gettextf("'bstick' cannot be used for '%s' with negative eigenvalues",
                      class(n)[1]))
    ## need to select appropriate total inertia
    tot.chi <- n$CA$tot.chi
    n.comp <- n$CA$rank
    res <- bstick.default(n.comp, tot.chi, ...)
    names(res) <- names(n$CA$eig)
    res
}

"scores.default" <-
    function (x, choices, display = c("sites", "species", "both"),
              tidy = FALSE, ...)
{
    display <- match.arg(display)
    X <- Y <- NULL
    if (tidy)
        display <- "both"
    att <- names(x)
    if (is.data.frame(x) && all(sapply(x, is.numeric)))
        x <- as.matrix(x)
    if (is.list(x) && display %in% c("sites", "both")) {
        if ("points" %in% att)
            X <- x$points
        else if ("rproj" %in% att)
            X <- x$rproj
        else if ("x" %in% att)
            X <- x$x
        else if ("scores" %in% att)
            X <- x$scores
        else if ("sites" %in% att)
            X <- x$sites
        else if("li" %in% att)
            X <- x$li
        else if("l1" %in% att)
            X <- x$l1
        else stop("cannot find scores")
    }
    if (is.list(x) && display %in% c("species", "both")) {
        if ("species" %in% att)
            Y <- x$species
        else if ("cproj" %in% att)
            Y <- x$cproj
        else if ("rotation" %in% att)
            Y <- x$rotation
        else if ("loadings" %in% att)
            Y <- x$loadings
        else if ("co" %in% att)
            Y <- x$co
        else if ("c1" %in% att)
            Y <- x$c1
        else if (display == "species") # fail if species explicitly requested
            stop("cannot find species scores")
        else { # "both" may be non-chalant: only warn
            warning("cannot find species scores")
        }

    }
    else if (is.numeric(x)) {
        X <- as.matrix(x)
        ## as.matrix() changes a score vector to 1-col matrix: this is
        ## a hack which may fail sometimes (but probably less often
        ## than without this hack):

        ## Removed this hack after an issue raised by
        ## vanderleidebastiani in github. He was worried for getting
        ## an error when 'choices' were not given with genuinely 1-dim
        ## (1-col) results. At a second look, it seems that this hack
        ## will fail both with missing 'choices', and also often with
        ## 'choices' given because 'choices' are only applied later,
        ## so that nrow(X) > length(choices). Only vectors (dim arg
        ## missing) should fail here. Let's see...

        ##if (ncol(X) == 1 && nrow(X) == length(choices))
        ##    X <- t(X)
    }
    if (!is.null(X) && NROW(X) && is.null(rownames(X))) {
        rownames(X) <- paste0("site", 1:nrow(X))
    }
    if (!is.null(Y) && NROW(Y) && is.null(rownames(Y))) {
        rownames(Y) <- paste0("spec", 1:nrow(Y))
    }
    if (!is.null(X) && NCOL(X) && is.null(colnames(X))) {
        colnames(X) <- paste0("Dim", 1:ncol(X))
    }
    if (!is.null(Y) && NCOL(Y) && is.null(colnames(Y))) {
        colnames(Y) <- paste0("Dim", 1:ncol(Y))
    }
    if (!missing(choices)) {
        if (!is.null(X))
            X <- X[, choices[choices <= NCOL(X)], drop = FALSE]
        if (!is.null(Y))
            Y <- Y[, choices[choices <= NCOL(Y)], drop = FALSE]
    }
    out <- list("sites" = X, "species" = Y)
    if (tidy) {
        score <- sapply(out, NROW)
        out <- data.frame(do.call(rbind, out),
                          "scores" = rep(names(score), score))
        out$label <- rownames(out)
    }
    if (any(drop <- sapply(out, is.null))) {
        out <- out[!drop]
        if (is.list(out) && length(out) == 1)
            out <- out[[1]]
    }
    out
}

##' S3 generic for function to compute tolerances
##'
##' Brought this in here from analogue because of tolerance.cca
##'
##' @param x an R object
##' @param ... arguments passed to other methods
`tolerance` <- function(x, ...)
    UseMethod("tolerance")

### Similar plotting functions as plot.prestonfit/fisherfit, but only
### for the data without the fitted models. These can be used with the
### result of as.preston(), as.fisher().

## as plot.prestonfit, but plots only the bars of as.preston()

`plot.preston` <-
        function (x, xlab = "Frequency", ylab = "Species", bar.col = "skyblue", 
                  ...) 
{
    freq <- x
    oct <- as.numeric(names(freq))
    noct <- max(oct) + 1
    plot(oct, freq, type = "n", ylim = c(0, max(freq)),
         xlim = c(-1, max(oct)), ylab = ylab, xlab = xlab, axes = FALSE, ...)
    axis(2)
    axis(1, at = 0:noct, labels = 2^(0:noct))
    box()
    rect(oct - 1, 0, oct, freq, col = bar.col, ...)
    invisible()
}


`lines.preston` <-
    function(x, xadjust = 0.5, ...)
{
    oct <- as.numeric(names(x)) - xadjust 
    lines(oct, x, ...)
}

## as plot.fisherfit, but plots only the bars of as.fisherfit

`plot.fisher` <-
    function(x, xlab = "Frequency", ylab = "Species", bar.col = "skyblue",
             kind = c("bar", "hiplot", "points", "lines"), add = FALSE,
             ...)
{
    kind <- match.arg(kind)
    freq <- as.numeric(names(x))
    if (!add)
        plot(freq, x, ylab=ylab, xlab=xlab,
             ylim=c(0,max(x)),  xlim=c(0.5, max(freq)+0.5), type="n", ...)
    switch(kind,
           "bar" = rect(freq-0.5,0,freq+0.5,x, col=bar.col, ...),
           "hiplot" = points(freq, x, col =bar.col, type = "h",  ...),
           "points" = points(freq, x, col =bar.col, ...),
           "lines" = lines(freq, x, col =bar.col, ...)
       )
    invisible()
}

plot.ordipointlabel <- function (x, ...)
{
    plot(x$points, pch = x$args$pch, cex = x$args$pcex, col = x$args$pcol,
        bg = x$args$pbg, asp = 1, ...)
    font <- attr(x$labels, "font")
    if (is.null(font))
        font <- par("font")
    text(x$labels, rownames(x$labels), cex = x$args$tcex, col = x$args$tcol,
         font = font, ...)
    invisible(x)
}

`points.ordiplot`  <-
    function (x, what, select, arrows = FALSE, ...)
{
    sco <- scores(x, what)
    if (!missing(select))
        sco <- .checkSelect(select, sco)
    ## draw adjusted arrows automatically for biplot scores
    scoatt <- attr(sco, "score")
    if (!is.null(scoatt) && scoatt %in% c("biplot", "regression")) {
        arrows = TRUE
        sco <- sco * ordiArrowMul(sco)
    }
    ## draw arrows when requested, also for "species" etc
    if (arrows) {
        arrows(0, 0, sco[,1], sco[,2], ...)
    } else {
        points(sco, ...)
    }
    invisible(x)
}

hiersimu.default <-
function(y, x, FUN, location = c("mean", "median"),
         relative = FALSE, drop.highest = FALSE, nsimul=99,
         method = "r2dtable", ...)
{
    ## evaluate formula
    lhs <- as.matrix(y)
    if (missing(x))
        x <- cbind(level_1=seq_len(nrow(lhs)),
            leve_2=rep(1, nrow(lhs)))
    rhs <- data.frame(x)
    rhs[] <- lapply(rhs, as.factor)
    rhs[] <- lapply(rhs, droplevels, exclude = NA)
    nlevs <- ncol(rhs)
    if (is.null(colnames(rhs)))
        colnames(rhs) <- paste("level", 1:nlevs, sep="_")
    tlab <- colnames(rhs)

    ## check proper design of the model frame
    l1 <- sapply(rhs, function(z) length(unique(z)))
    if (!any(sapply(2:nlevs, function(z) l1[z] <= l1[z-1])))
        stop("number of levels are inappropriate, check sequence")
    rval <- list()
    rval[[1]] <- rhs[,nlevs]
    nCol <- nlevs - 1
    if (nlevs > 1) {
        nCol <- nlevs - 1
        for (i in 2:nlevs) {
            rval[[i]] <- interaction(rhs[,nCol], rval[[(i-1)]], drop=TRUE)
            nCol <- nCol - 1
        }
    }
    rval <- as.data.frame(rval[rev(1:length(rval))])
    l2 <- sapply(rval, function(z) length(unique(z)))
    if (any(l1 != l2))
        stop("levels are not perfectly nested")

    ## aggregate response matrix
    fullgamma <-if (nlevels(rhs[,nlevs]) == 1)
        TRUE else FALSE
    if (fullgamma && drop.highest)
        nlevs <- nlevs - 1
    if (nlevs == 1 && relative)
        stop("'relative=FALSE' makes no sense with one level")
    ftmp <- vector("list", nlevs)
    for (i in 1:nlevs) {
        ftmp[[i]] <- as.formula(paste("~", tlab[i], "- 1"))
    }

    ## is there burnin/thin in ... ?
    burnin <- if (is.null(list(...)$burnin))
        0 else list(...)$burnin
    thin <- if (is.null(list(...)$thin))
        1 else list(...)$thin

    ## evaluate other arguments
    if (!is.function(FUN))
        stop("'FUN' must be a function")
    location <- match.arg(location)
    aggrFUN <- switch(location,
        "mean" = mean,
        "median" = median)

    ## this is the function passed to oecosimu
    evalFUN <- function(x) {
        if (fullgamma && !drop.highest) {
            tmp <- lapply(1:(nlevs-1), function(i) t(model.matrix(ftmp[[i]], rhs)) %*% x)
            tmp[[nlevs]] <- matrix(colSums(x), nrow = 1, ncol = ncol(x))
        } else {
            tmp <- lapply(1:nlevs, function(i) t(model.matrix(ftmp[[i]], rhs)) %*% x)
        }
        a <- sapply(1:nlevs, function(i) aggrFUN(FUN(tmp[[i]]))) # dots removed from FUN
        if (relative)
            a <- a / a[length(a)]
        a
    }

    ## processing oecosimu results
    sim <- oecosimu(lhs, evalFUN, method = method, nsimul=nsimul,
        burnin=burnin, thin=thin)
#    nam <- paste("level", 1:nlevs, sep=".")
#    names(sim$statistic) <- attr(sim$oecosimu$statistic, "names") <- nam
    names(sim$statistic) <- attr(sim$oecosimu$statistic, "names") <- tlab[1:nlevs]
    call <- match.call()
    call[[1]] <- as.name("hiersimu")
    attr(sim, "call") <- call
    attr(sim, "FUN") <- FUN
    attr(sim, "location") <- location
    attr(sim, "n.levels") <- nlevs
    attr(sim, "terms") <- tlab
    attr(sim, "model") <- rhs
    class(sim) <- c("hiersimu", class(sim))
    sim
}

`cca.default` <-
    function (X, Y = NULL, Z = NULL, ...)
{
    ## Protect against grave misuse: some people have used
    ## dissimilarities instead of data
    if (inherits(X, "dist") || NCOL(X) == NROW(X) &&
        isTRUE(all.equal(X, t(X))))
        stop("function cannot be used with (dis)similarities")
    X <- as.matrix(X)
    if (!is.null(Y)) {
        if (is.data.frame(Y) || is.factor(Y))
            Y <- model.matrix(~ ., as.data.frame(Y))[,-1,drop=FALSE]
        Y <- as.matrix(Y)
    }
    if (!is.null(Z)) {
        if (is.data.frame(Z) || is.factor(Z))
            Z <- model.matrix(~ ., as.data.frame(Z))[,-1,drop=FALSE]
        Z <- as.matrix(Z)
    }

    if (any(rowSums(X) <= 0))
        stop("all row sums must be >0 in the community data matrix")
    if (any(tmp <- colSums(X) <= 0)) {
        exclude.spec <- seq(along=tmp)[tmp]
        names(exclude.spec) <- colnames(X)[tmp]
        class(exclude.spec) <- "exclude"
        X <- X[, !tmp, drop = FALSE]
    }
    sol <- ordConstrained(X, Y, Z, method = "cca")
    if (exists("exclude.spec")) {
        if (!is.null(sol$CCA$v))
            attr(sol$CCA$v, "na.action") <- exclude.spec
        if (!is.null(sol$CA$v))
            attr(sol$CA$v, "na.action") <- exclude.spec
    }
    call <- match.call()
    call[[1]] <- as.name("cca")
    sol <- c(list(call = call,
                  inertia =  "scaled Chi-square"),
             sol)
    class(sol) <- "cca"
    sol
}

##' @title Utility for handling user friendly scaling --- None exported
##'
##' @description Convert user-friendly descriptions of scalings to numeric codes used by \code{scores} to date.
##'
##' @param scaling character or numeric; which type of scaling is required? Numeric values are returned unaltered
##' @param correlation logical; should correlation-like scores be returned?
##' @param hill logical; should Hill's scaling scores be returned?
`scalingType` <- function(scaling = c("none", "sites", "species", "symmetric"),
                          correlation = FALSE, hill = FALSE) {
    ## Only process scaling further if it is character
    if (is.numeric(scaling)) {
        return(scaling)                 # numeric; return early
    } else if (is.character(scaling)) {
        ## non-numeric scaling: change to correct numeric code
        scaling <- match.arg(scaling)   # match user choice
        ## Keep `tab` as this is the order of numeric codes
        ## Allows potential to change the default ordering of formal argument 'scaling'
        tab <- c("none", "sites", "species", "symmetric")
        scaling <- match(scaling, tab) - 1      # -1 as none == scaling 0
        if (correlation || hill) {
            scaling <- -scaling
        }
    } else {
        stop("'scaling' is not 'numeric' nor 'character'.")
    }
    scaling                             # return
}

### stressplot() methods for eigenvector ordinations wcmdscale, rda,
### cca, capscale, dbrda

`stressplot.wcmdscale` <-
    function(object, k = 2, pch,  p.col = "blue", l.col = "red", lwd = 2, ...)
{
    ## Check that original distances can be reconstructed: this
    ## requires that all axes were calculated instead of 'k' first.
    hasdims <- NCOL(object$points)
    if (!is.null(object$negaxes))
        hasdims <- hasdims + NCOL(object$negaxes)
    if (hasdims < length(object$eig))
        stop("observed distances cannot be reconstructed: all axes were not calculated")
    ## Get the ordination distances in k dimensions
    if (k > NCOL(object$points))
        warning(gettextf("max allowed rank is k = %d", NCOL(object$points)))
    k <- min(NCOL(object$points), k)
    w <- sqrt(object$weights)
    u <- diag(w) %*% object$points
    odis <- dist(u[,1:k, drop = FALSE])
    ## Reconstitute the original observed distances
    dis <- dist(u)
    if (!is.null(object$negaxes))
        dis <- sqrt(dis^2 - dist(diag(w) %*% object$negaxes)^2)
    ## Remove additive constant to get original dissimilarities
    if (!is.na(object$ac)) {
        if (object$add == "lingoes")
            dis <- sqrt(dis^2 - 2 * object$ac)
        else if (object$add == "cailliez")
            dis <- dis - object$ac
        else
            stop("unknown Euclidifying adjustment: no idea what to do")
    }
    ##Plot
    if (missing(pch))
        if (length(dis) > 5000)
            pch <- "."
        else
            pch <- 1
    plot(dis, odis, pch = pch, col = p.col, xlab = "Observed Dissimilarity",
         ylab = "Ordination Distance", ...)
    abline(0, 1, col = l.col, lwd = lwd, ...)
    invisible(odis)
}

`stressplot.rda` <-
    function(object, k = 2, pch, p.col = "blue", l.col = "red", lwd = 2, ...)
{
    ## Normalized scores to reconstruct data
    u <- cbind(object$CCA$u, object$CA$u)
    v <- cbind(object$CCA$v, object$CA$v)
    ev <- c(object$CCA$eig, object$CA$eig)
    ## check that k does not exceed rank
    if (k > length(ev)) {
        warning(gettextf("max allowed rank is k = %d", length(ev)))
        k <- min(k, length(ev))
    }
    ## normalizing constant
    nr <- NROW(u)
    const <- sqrt(ev * (nr-1))
    u <- u %*% diag(const, length(const))
    ## Distances
    Xbar <- u %*% t(v)
    Xbark <- u[, seq_len(k), drop = FALSE] %*% t(v[, seq_len(k), drop = FALSE])
    if (!is.null(object$pCCA)) {
        pFit <- ordiYbar(object, "pCCA") * sqrt(nr-1)
        Xbar <- Xbar + pFit
        Xbark <- Xbark + pFit
    }
    dis <- dist(Xbar)
    odis <- dist(Xbark)
    ## plot like above
        ## Plot
    if (missing(pch))
        if (length(dis) > 5000)
            pch <- "."
        else
            pch <- 1
    plot(dis, odis, pch = pch, col = p.col, xlab = "Observed Dissimilarity",
         ylab = "Ordination Distance", ...)
    abline(0, 1, col = l.col, lwd = lwd, ...)
    invisible(odis)
}

`stressplot.cca` <-
    function(object, k = 2, pch, p.col = "blue", l.col = "red", lwd = 2, ...)
{
    ## Normalized scores to reconstruct data
    u <- cbind(object$CCA$u, object$CA$u)
    sev <- sqrt(c(object$CCA$eig, object$CA$eig))
    w <- sqrt(object$rowsum)
    u <- diag(w) %*% u %*% diag(sev, length(sev))
    v <- cbind(object$CCA$v, object$CA$v)
    v <- diag(sqrt(object$colsum)) %*% v
    ## check that k <= rank
    if (k > length(sev)) {
        warning(gettextf("max allowed rank is k = %d", length(sev)))
        k <- min(k, length(sev))
    }
    ## Distances
    Xbar <- u %*% t(v)
    Xbark <- u[,seq_len(k), drop = FALSE] %*% t(v[,seq_len(k), drop = FALSE])
    if (!is.null(object$pCCA)) {
        pFit <- ordiYbar(object, "pCCA")
        Xbar <- Xbar + pFit
        Xbark <- Xbark + pFit
    }
    dis <- dist(Xbar)
    odis <- dist(Xbark)
    ## Plot
    if (missing(pch))
        if (length(dis) > 5000)
            pch <- "."
        else
            pch <- 1
    plot(dis, odis, pch = pch, col = p.col, xlab = "Observed Dissimilarity",
         ylab = "Ordination Distance", ...)
    abline(0, 1, col = l.col, lwd = lwd, ...)
    invisible(odis)
}

`stressplot.capscale` <-
    function(object, k = 2, pch, p.col = "blue", l.col = "red", lwd = 2, ...)
{
    ## Scores to reconstruct data
    u <- cbind(object$CCA$u, object$CA$u)
    ## check rank
    if (k > NCOL(u))
        warning(gettextf("max allowed rank is k = %d", ncol(u)))
    k <- min(k, ncol(u))
    ev <- c(object$CCA$eig, object$CA$eig)
    u <- u %*% diag(sqrt(ev) * object$adjust, length(ev))
    ## Constrained ordination needs also scores 'v' to reconstruct
    ## 'data', but these are not returned by capscale() which replaces
    ## original 'v' with weighted sums of 'comm' data.
    if (!is.null(object$CCA))
        v <- svd(ordiYbar(object, "CCA"), nu = 0, nv = object$CCA$qrank)$v
    else
        v <- NULL
    if (!is.null(object$CA))
        v <- cbind(v, svd(ordiYbar(object, "CA"), nu = 0,
                          nv = object$CA$rank)$v)
    ## Reconstruct Xbar and Xbark
    Xbar <- u %*% t(v)
    Xbark <- u[,seq_len(k), drop = FALSE] %*% t(v[,seq_len(k), drop = FALSE])
    if (!is.null(object$pCCA)) {
        pFit <- ordiYbar(object, "pCCA")
        Xbar <- Xbar + pFit
        Xbark <- Xbark + pFit
    }
    ## Distances
    dis <- dist(Xbar)
    odis <- dist(Xbark)
    if (!is.null(object$CA$imaginary.u.eig)) {
        dis <- dis^2 - dist(object$CA$imaginary.u.eig)^2
        if (all(dis > -sqrt(.Machine$double.eps)))
            dis <- sqrt(pmax(dis, 0))
        else # neg dis will be NaN with a warning
            dis <- sqrt(dis)
    }
    ## Remove additive constant to get original dissimilarities
    if (!is.null(object$ac)) {
        if (object$add == "lingoes")
            dis <- sqrt(dis^2 - 2 * object$ac)
        else if (object$add == "cailliez")
            dis <- dis - object$ac
        else
            stop("unknown Euclidifying adjustment: no idea what to do")
    }
    ## undo internal sqrt.dist
    if (object$sqrt.dist)
        dis <- dis^2
    ## plot like above
        ## Plot
    if (missing(pch))
        if (length(dis) > 5000)
            pch <- "."
        else
            pch <- 1
    plot(dis, odis, pch = pch, col = p.col, xlab = "Observed Dissimilarity",
         ylab = "Ordination Distance", ...)
    abline(0, 1, col = l.col, lwd = lwd, ...)
    invisible(odis)
}

### dbrda() returns only row scores 'u' (LC scores for constraints,
### site scores for unconstrained part), and these can be used to
### reconstitute dissimilarities only in unconstrained ordination or
### for constrained component.

`stressplot.dbrda` <-
    function(object, k = 2, pch, p.col = "blue", l.col = "red", lwd = 2, ...)
{
    ## Reconstructed zero distances can be tiny (negative) non-zero
    ## values, and we zap them to zero
    ZAP <- sqrt(.Machine$double.eps)
    ## Reconstruct original distances from Gower 'G'
    dis <- ordiYbar(object, "initial")
    dia <- diag(dis)
    dis <- -2 * dis + outer(dia, dia, "+")
    dis[abs(dis) < ZAP] <- 0
    dis <- sqrt(as.dist(dis)) * object$adjust
    ## Remove additive constant to get original dissimilarities
    if (!is.null(object$ac)) {
        if (object$add == "lingoes")
            dis <- sqrt(dis^2 - 2 * object$ac)
        else if (object$add == "cailliez")
            dis <- dis - object$ac
        else
            stop("unknown Euclidifying adjustment: no idea what to do")
    }
    ## undo internal sqrt.dist
    if (object$sqrt.dist)
        dis <- dis^2
    ## Approximate dissimilarities from real components. Can only be
    ## used for one component.
    if (is.null(object$CCA)) {
        U <- object$CA$u
        eig <- object$CA$eig
    } else {
        U <- object$CCA$u
        eig <- object$CCA$eig
    }
    eig <- eig[eig > 0]
    ## check that 'k' does not exceed real rank
    if (k > ncol(U))
        warning(gettextf("max allowed rank is k = %d", ncol(U)))
    k <- min(k, ncol(U))
    Gk <- tcrossprod(sweep(U[, seq_len(k), drop=FALSE], 2,
                  sqrt(eig[seq_len(k)]), "*"))
    dia <- diag(Gk)
    odis <- -2 * Gk + outer(dia, dia, "+")
    odis[abs(odis) < ZAP] <- 0
    odis <- sqrt(as.dist(odis)) * object$adjust
    ## Plot
    if (missing(pch))
        if (length(dis) > 5000)
            pch <- "."
        else
            pch <- 1
    plot(dis, odis, pch = pch, col = p.col, xlab = "Observed Dissimilarity",
         ylab = "Ordination Distance", ...)
    abline(0, 1, col = l.col, lwd = lwd, ...)
    invisible(odis)
}

## Standard R PCA functions

`stressplot.prcomp` <-
    function(object, k = 2, pch, p.col = "blue", l.col = "red", lwd = 2, ...)
{
    dis <- dist(object$x)
    odis <- dist(object$x[, 1:k, drop = FALSE])
    if (missing(pch))
        if (length(dis) > 5000)
            pch <- "."
        else
            pch <- 1
    plot(dis, odis, pch = pch, col = p.col, xlab = "Observed Dissimilarity",
         ylab = "Ordination Distance", ...)
    abline(0, 1, col = l.col, lwd = lwd, ...)
    invisible(odis)
}

`stressplot.princomp` <-
    function(object, k = 2, pch, p.col = "blue", l.col = "red", lwd = 2, ...)
{
    dis <- dist(object$scores)
    odis <- dist(object$scores[, 1:k, drop = FALSE])
    if (missing(pch))
        if (length(dis) > 5000)
            pch <- "."
        else
            pch <- 1
    plot(dis, odis, pch = pch, col = p.col, xlab = "Observed Dissimilarity",
         ylab = "Ordination Distance", ...)
    abline(0, 1, col = l.col, lwd = lwd, ...)
    invisible(odis)
}

`indpower` <-
    function(x, type=0)
{
    x <- as.matrix(x)
    if (!(is.numeric(x) || is.logical(x)))
        stop("input data must be numeric")
    x <- ifelse(x > 0, 1, 0)
    if (NCOL(x) < 2)
        stop("provide at least two columns for 'x'")
    if (!(type %in% 0:2))
        stop("'type' must be 0, 1 or 2")
    n <- nrow(x)
    j <- crossprod(x) ## faster t(x) %*% x
    ip1 <- sweep(j, 1, diag(j), "/")
    ip2 <- 1 - sweep(-sweep(j, 2, diag(j), "-"), 1, n - diag(j), "/")
    out <- switch(as.character(type),
        "0" = sqrt(ip1 * ip2),
        "1" = ip1,
        "2" = ip2)
    cn <- if (is.null(colnames(out)))
        1:ncol(out) else colnames(out)
    rn <- if (is.null(rownames(out)))
        1:ncol(out) else rownames(out)
    colnames(out) <- paste("t", cn, sep=".")
    rownames(out) <- paste("i", rn, sep=".")
    out
}

## as.mlm: deprecated in 2.5-0, defunct in 2.6-0

`as.mlm` <-
    function(x)
{
    .Defunct("see ?hatvalues.cca for new alternatives")
}

`as.mlm.cca` <-
    function(x)
{
    .Defunct("see ?hatvalues.cca for new alternatives")
}

### commsimulator was deprecated in 2.4-0, defunct in 2.6-0

"commsimulator" <-
function (x, method, thin = 1)
{
    .Defunct("simulate(nullmodel(x, method))", package="vegan")
}

### deprecated in 2.2-0, but forgotten and never exported from the
### NAMESPACE. Make finally defunct for 2.6-0.

"permuted.index" <-
    function (n, strata)
{
    .Defunct("use permute package (shuffle or shuffleSet)")
}

### deprecated in 2.5-3, declared defunct and removed in 2.6-2 (but
### actually was not removed)
`humpfit` <-
    function (mass, spno, family = poisson, start)
{
    .Defunct("use natto::humpfit() from https://github.com/jarioksa/natto/")
}

### deprecated in 2.6-2 with alternative toCoda, make defunct in 2.6-5
`as.mcmc.oecosimu` <-
    function(x)
{
    ## Deprecated in favour of toCoda: using as an S3 method would
    ## need importFrom(coda, as.mcmc) and that would add dependence on
    ## coda
    .Defunct("toCoda", package = "vegan")
}

`as.mcmc.permat` <- as.mcmc.oecosimu

# Extract eigenvalues from an object that has them

`eigenvals` <-
    function(x, ...)
{
    UseMethod("eigenvals")
}

`eigenvals.default`<-
    function(x, ...)
{
    ## svd and eigen return unspecified 'list', see if this could be
    ## either of them (like does cmdscale)
    out <- NA
    if (is.list(x)) {
        ## eigen
        if (length(x) == 2 && all(names(x) %in% c("values", "vectors")))
            out <- x$values
        ## svd: return squares of singular values
        else if (length(x) == 3 && all(names(x) %in% c("d", "u", "v")))
            out <- x$d^2
        ## cmdscale() will return all eigenvalues from R  2.12.1
        else if (all(c("points","eig","GOF") %in% names(x)))
            out <- x$eig
    }
    class(out) <- "eigenvals"
    out
}

## squares of sdev
`eigenvals.prcomp` <-
    function(x, ...)
{
    out <- x$sdev^2
    names(out) <- colnames(x$rotation)
    ## honour prcomp(..., rank.=) which only requests rank. eigenvalues
    if (ncol(x$rotation) < length(out)) {
        sumev <- sum(out)
        out <- out[seq_len(ncol(x$rotation))]
        attr(out, "sumev") <- sumev
    }
    class(out) <- "eigenvals"
    out
}

## squares of sdev
`eigenvals.princomp` <-
    function(x, ...)
{
    out <- x$sdev^2
    class(out) <- "eigenvals"
    out
}

## concatenate constrained and unconstrained eigenvalues in cca, rda
## and capscale (vegan) -- ignore pCCA component
`eigenvals.cca` <- function(x, model = c("all", "unconstrained", "constrained"),
                            constrained = NULL, ...)
{
    out <- if (!is.null(constrained)) {
        ## old behaviour
        message("Argument `constrained` is deprecated; use `model` instead.")
        if (constrained) {
            x$CCA$eig
        } else {
            c(x$CCA$eig, x$CA$eig)
        }
    } else {
        ## new behaviour
        model <- match.arg(model)
        if (identical(model, "all")) {
            c(x$CCA$eig, x$CA$eig)
        } else if (identical(model, "unconstrained")) {
            x$CA$eig
        } else {
            x$CCA$eig
        }
    }
    if (!is.null(out)) {
        class(out) <- "eigenvals"
    }
    out
}

## wcmdscale (in vegan)
`eigenvals.wcmdscale` <-
    function(x, ...)
{
    out <- x$eig
    class(out) <- "eigenvals"
    out
}

## pcnm (in vegan)
`eigenvals.pcnm` <-
    function(x, ...)
{
    out <- x$values
    class(out) <- "eigenvals"
    out
}

## betadisper (vegan)
`eigenvals.betadisper` <-  function(x, ...) {
    out <- x$eig
    class(out) <- "eigenvals"
    out
}

## dudi objects of ade4

`eigenvals.dudi` <-
    function(x, ...)
{
    out <- x$eig
    class(out) <- "eigenvals"
    out
}

## labdsv::pco

`eigenvals.pco` <-
    function(x, ...)
{
    out <- x$eig
    class(out) <- "eigenvals"
    out
}

## labdsv::pca

`eigenvals.pca` <-
    function(x, ...)
{
    out <- x$sdev^2
    ## pca() may return only some first eigenvalues
    if ((seig <- sum(out)) < x$totdev) {
        names(out) <- paste("PC", seq_along(out), sep="")
        out <- c(out, "Rest" = x$totdev - seig)
    }
    class(out) <- "eigenvals"
    out
}


`print.eigenvals` <-
    function(x, ...)
{
    print(zapsmall(unclass(x), ...))
    invisible(x)
}

`summary.eigenvals` <- function(object, ...) {
    ## dbRDA can have negative eigenvalues: do not give cumulative
    ## proportions
    if (!is.null(attr(object, "sumev"))) {
        sumev <- attr(object, "sumev")
    } else {
        sumev <- sum(object)
    }
    vars <- object/sumev
    cumvars <- if (all(vars >= 0)) {
        cumsum(vars)
    } else {
        NA
    }
    out <- rbind(`Eigenvalue` = object,
                 `Proportion Explained` = abs(vars),
                 `Cumulative Proportion` = cumvars)
    class(out) <- c("summary.eigenvals", "matrix")
    out
}

## before R svn commit 70391 we used print.summary.prcomp, but now we
## need our own version that is similar to pre-70391 R function

`print.summary.eigenvals` <-
    function(x, digits = max(3L, getOption("digits") - 3L), ...)
{
    cat("Importance of components:\n")
    class(x) <- "matrix"
    print(x, digits = digits, ...)
    invisible(x)
}

### Scaling of arrows to 'fill' a plot with vectors centred at 'at'.
### Plot dims from 'par("usr")' and arrow heads are in 'x'.
`ordiArrowMul` <- function (x, at = c(0,0), fill = 0.75,
                            display, choices = c(1,2), ...) {
    ## handle x, which we try with scores, but also retain past usage of
    ## a two column matrix
    X <- if (is.matrix(x)) {
        nc <- NCOL(x)
        if (nc != 2L) {
            stop("a two-column matrix of coordinates is required")
        }
        x
    } else {
        if (inherits(x, "envfit")) {
            scores(x, display = "vectors", ...)[, 1:2, drop = FALSE]
        } else {
            scores(x, display = display, choices = choices, ...)
        }
    }

    u <- par("usr")
    u <- u - rep(at, each = 2)
    r <- c(range(X[,1], na.rm = TRUE), range(X[,2], na.rm = TRUE))
    ## 'rev' takes care of reversed axes like xlim(1,-1)
    rev <- sign(diff(u))[-2]
    if (rev[1] < 0)
        u[1:2] <- u[2:1]
    if (rev[2] < 0)
        u[3:4] <- u[4:3]
    u <- u/r
    u <- u[is.finite(u) & u > 0]
    fill * min(u)
}

## Contribution diversity
## Lu, H.P., H.H. Wagner and X.Y. Chen (2007). 
## A contribution diversity approach to evaluate species diversity. 
## Basic and Applied Ecology 8: 1 -12.
`contribdiv` <-
    function(comm, index = c("richness", "simpson"), relative = FALSE,
             scaled = TRUE, drop.zero = FALSE)
{

    index <- match.arg(index)

    comm <- as.matrix(comm) # faster than data.frame

    x <- comm[rowSums(comm) > 0, colSums(comm) > 0]
    n <- nrow(x)
    S <- ncol(x)

    if (index == "richness") {
        n.i <- colSums(x > 0)
        S.k <- rowSums(x > 0)
        alpha <- S.k / n
        beta <- apply(x, 1, function(z) sum((n - n.i[z > 0]) / (n * n.i[z > 0])))
        denom <- 1
    } else {
        P.ik <- decostand(x, "total")
        P.i <- apply(P.ik, 2, function(z) sum(z) / n)
        P.i2 <- matrix(P.i, n, S, byrow=TRUE)
        alpha <- diversity(x, "simpson")
        beta <- rowSums(P.ik * (P.ik - P.i2))
        denom <- n
    }
    gamma <- alpha + beta
    D <- sum(beta) / sum(gamma)
    if (relative) {
        denom <- if (scaled)
            {denom * sum(gamma)} else 1
        alpha <- (alpha - mean(alpha)) / denom
        beta <- (beta - mean(beta)) / denom
        gamma <- (gamma - mean(gamma)) / denom
    }
    rval <- data.frame(alpha = alpha, beta = beta, gamma = gamma)
    if (!drop.zero && nrow(comm) != n) {
        nas <- rep(NA, nrow(comm))
        rval2 <- data.frame(alpha = nas, beta = nas, gamma = nas)
        rval2[rowSums(comm) > 0, ] <- rval
        rval <- rval2
    }
    attr(rval, "diff.coef") <- D
    attr(rval, "index") <- index
    attr(rval, "relative") <- relative
    attr(rval, "scaled") <- scaled
    class(rval) <- c("contribdiv", "data.frame")
    rval
}

`taxa2dist` <-
    function (x, varstep = FALSE, check = TRUE, labels)
{
    rich <- apply(x, 2, function(taxa) length(unique(taxa)))
    S <- nrow(x)
    if (check) {
        keep <- rich < S & rich > 1
        rich <- rich[keep]
        x <- x[, keep]
    }
    i <- rev(order(rich))
    x <- x[, i]
    rich <- rich[i]
    if (varstep) {
        add <- -diff(c(nrow(x), rich, 1))
        add <- add/c(S, rich)
        add <- add/sum(add) * 100
    }
    else {
        add <- rep(100/(ncol(x) + check), ncol(x) + check)
    }
    if (!is.null(names(add)))
        names(add) <- c("Base", names(add)[-length(add)])
    if (!check)
        add <- c(0, add)
    out <- matrix(add[1], nrow(x), nrow(x))
    for (i in 1:ncol(x)) {
        out <- out + add[i + 1] * outer(x[, i], x[, i], "!=")
    }
    out <- as.dist(out)
    attr(out, "method") <- "taxa2dist"
    attr(out, "steps") <- add
    if (missing(labels)) {
        attr(out, "Labels") <- rownames(x)
    } else {
        if (length(labels) != nrow(x))
            warning(gettextf("labels are wrong: needed %d, got %d",
                             nrow(x), length(labels)))
        attr(out, "Labels") <- as.character(labels)
    }
    if (!check && any(out <= 0))
        warning("you used 'check=FALSE' and some distances are zero: was this intended?")
    out
}

## biplot.rda
##
## draws pca biplots with species as arrows
##

`biplot.cca` <-
    function(x, ...)
{
    if (!inherits(x, "rda"))
        stop("biplot can be used only with linear ordination (e.g., PCA)")
    else
        NextMethod("biplot", x, ...)
}

`biplot.rda` <- function(x, choices = c(1, 2), scaling = "species",
                         display = c("sites", "species"),
                         type, xlim, ylim, col = c(1,2), const,
                         correlation = FALSE, ...) {
  if(!inherits(x, "rda"))
      stop("'biplot.rda' is only for objects of class 'rda'")
  if(!is.null(x$CCA))
      stop("'biplot.rda' not suitable for models with constraints")
  TYPES <- c("text", "points", "none")
  display <- match.arg(display, several.ok = TRUE)
  if (length(col) == 1)
      col <- c(col,col)
  g <- scores(x, choices = choices, display = display,
              scaling = scaling, correlation = correlation, const)
  if (!is.list(g)) {
      g <- list(default = g)
      names(g) <- display
  }
  if (missing(type)) {
      nitlimit <- 80
      nit <- max(nrow(g$species), nrow(g$sites))
      if (nit > nitlimit)
          type <- rep("points", 2)
      else type <- rep("text", 2)
  }
  else type <- match.arg(type, TYPES, several.ok = TRUE)
  if(length(type) < 2)
      type <- rep(type, 2)
  if (missing(xlim))
      xlim <- range(g$species[, 1], g$sites[, 1], na.rm = TRUE)
  if (missing(ylim))
      ylim <- range(g$species[, 2], g$sites[, 2], na.rm = TRUE)
  plot(g[[1]], xlim = xlim, ylim = ylim, type = "n", asp = 1,
       ...)
  abline(h = 0, lty = 3)
  abline(v = 0, lty = 3)
  if (!is.null(g$species)) {
      if (type[1] == "points")
          arrlen <- 1
      else
          arrlen <- 0.85
      if (type[1] != "none")
          arrows(0, 0, g$species[,1] * arrlen, g$species[, 2] * arrlen,
                 col = col[2], length = 0.05)
      if (type[1] == "text")
          text(g$species, rownames(g$species),
               col = col[2], cex = 0.7)
  }
  if (!is.null(g$sites)) {
      if (type[2] == "text")
          text(g$sites, rownames(g$sites), cex = 0.7, col = col[1])
      else if (type[2] == "points")
          points(g$sites, pch = 1, cex = 0.7, col = col[1])
  }
  class(g) <- "ordiplot"
  invisible(g)
}

## this is function to create a commsim object, does some checks
## there is a finite number of useful arguments here
## but I added ... to allow for unforeseen algorithms,
## or being able to reference to external objects
commsim <- 
function(method, fun, binary, isSeq, mode) 
{
    fun <- if (!missing(fun))
        match.fun(fun) else stop("'fun' missing")
    if (any(!(names(formals(fun)) %in% 
        c("x", "n", "nr", "nc", "rs", "cs", "rf", "cf", "s", "fill", "thin", "..."))))
            stop("unexpected arguments in 'fun'")
    out <- structure(list(method = if (!missing(method))
            as.character(method)[1L] else stop("'method' missing"),
        binary = if (!missing(binary))
            as.logical(binary)[1L] else stop("'binary' missing"),
        isSeq = if (!missing(isSeq))
            as.logical(isSeq)[1L] else stop("'isSeq' missing"),
        mode = if (!missing(mode))
            match.arg(as.character(mode)[1L],
            c("integer", "double")) else stop("'mode' missing"),
        fun = fun), class = "commsim")
    out
}

`ordixyplot` <-
    function(x, data = NULL, formula, display = "sites", choices=1:3,
             panel = "panel.ordi", aspect = "iso", envfit,
             type = c("p", "biplot"),  ...)
{
    localXyplot <- function(..., shrink, origin, scaling) xyplot(...)
    p <- as.data.frame(scores(x, display = display, choices = choices, ...))
    if (!is.null(data))
        p <- cbind(p, data)
    if (missing(formula)) {
        v <- colnames(p)
        formula <- as.formula(paste(v[2], "~", v[1]))
    }
    if ("biplot" %in% type && ((!is.null(x$CCA) && x$CCA$rank > 0) ||
                                   !missing(envfit))) {
        if (missing(envfit))
            envfit <- NULL
        env <- ordilattice.getEnvfit(formula, x, envfit, choices, ...)
        if (!is.null(env$arrows)) {
            mul <- apply(p[,colnames(env$arrows)], 2, range)/apply(env$arrows, 2, range)
            mul <- min(mul[is.finite(mul) & mul > 0])
            env$arrows <- mul * env$arrows
        }
    } else {
        env <- NULL
    }
    ## plot polygon for all data plus superpose polygons for each panel
    if ("polygon" %in% type) {
        pol <- p[, all.vars(formula)[2:1]]
        pol <- pol[chull(pol),]
    } else {
        pol <- NULL
    }
    localXyplot(formula, data = p, panel = panel,  aspect = aspect,
                biplot = env, polygon = pol, type = type, ...)
}

`treeheight` <-
    function(tree)
{
    if (inherits(tree, "spantree"))
        return(sum(tree$dist))
    tree <- as.hclust(tree)
    ## nodes should start from 0 -- if there are negative heights,
    ## tree is too pathological to be measured.
    if (any(tree$height < 0))
        stop("negative heights: tree cannot be measured")
    ## can be done really fast if there are no reversals, but we need
    ## to traverse the tree with reversals
    if (is.unsorted(tree$height)) { # slow
        h <- tree$height
        m <- tree$merge
        height <- 0
        for (i in 1:nrow(m)) {
            for (j in 1:2) {
                if (m[i,j] < 0)
                    height <- height + h[i]
                else
                    height <- height + abs(h[i] - h[m[i,j]])
            }
        }
        height
    }
    else    # fast
        sum(tree$height) + max(tree$height)
}

`avgdist` <-
    function(x, sample, distfun = vegdist, meanfun = mean,
             transf = NULL, iterations = 100, dmethod = "bray",
             diag = TRUE, upper = TRUE, ...)
{
    if (missing(sample)) {
        stop("Subsampling depth must be supplied via argument 'sample'")
    } else {
        if (!(is.numeric(sample) && sample > 0L)) {
            stop("Invalid subsampling depth; 'sample' must be positive & numeric")
        }
    }
    if (!is.numeric(iterations)) {
        stop("Invalid iteration count; must be numeric")
    }
    inputcast <- x
    distfun <- match.fun(distfun)
    if (!is.null(transf)) {
        transf <- match.fun(transf)
    }
    ## warn here if data do not look observed counts with singletons
    minobs <- min(x[x > 0])
    if (minobs > 1)
        warning(gettextf("most observed count data have counts 1, but smallest count is %d", minobs))
    # Get the list of iteration matrices
    distlist <- lapply(seq_len(iterations), function(i) {
        # Suppress warnings because it will otherwise return many warnings about
        # subsampling depth not being met, which we deal with below by returning
        # samples that do not meet the threshold.
        inputcast <- suppressWarnings(rrarefy(inputcast, sample = sample))
        # Remove those that did not meet the depth cutoff
        inputcast <- inputcast[c(rowSums(inputcast) >= sample), ]
        if (!is.null(transf)) {
            inputcast <- transf(inputcast)
        }
        outdist <- distfun(inputcast, method = dmethod,
                           diag = TRUE, upper = TRUE, ...)
        as.matrix(outdist)
    })
    # Use the dist list to get the average values
    meanfun <- match.fun(meanfun)
    # Save row names from distlist
    # Take from first element since should all be the same
    rnames <- row.names(distlist[[1]])
    afunc <- array(
        unlist(as.matrix(distlist)),
               c(dim(as.matrix(distlist[[1]])), length(distlist)))
    output <- apply(afunc, 1:2, meanfun, ...)
    # Set the names on the matrix
    colnames(output) <- rownames(output) <- rnames
    # Print any samples that were removed, if they were removed
    dropsamples <- setdiff(row.names(inputcast), row.names(output))
    if (length(dropsamples) > 0L) {
        warning(gettextf(
            "The following sampling units were removed because they were below sampling depth: %s",
                         paste(dropsamples, collapse = ", ")))
    }
    output <- as.dist(output, diag = diag, upper = upper)
    attr(output, "call") <- match.call()
    attr(output, "method") <- "avgdist"
    output
}

`make.cepnames` <-
    function (names, seconditem = FALSE)
{
    ## make valid names
    names <- make.names(names, unique = FALSE)
    ## remove trailing and duplicated dots
    names <- gsub("\\.[\\.]+", ".", names)
    names <- gsub("\\.$", "", names)
    ## split by dots and take 4 letters of each element (if several)
    names <- lapply(strsplit(names, "\\."), function(x) if (length(x) > 1)
                    substring(x, 1, 4) else x )
    ## Take first and last element or 8 characters if only one element
    names <- unlist(lapply(names, function(x) if (length(x) > 1)
                           paste(x[c(1, if(seconditem) 2 else length(x))], collapse = "")
                           else x))
    names <- abbreviate(names, 8)
    ## Final clean-up
    make.names(names, unique = TRUE)
}

`extractAIC.cca` <-
    function (fit, scale = 0, k = 2, ...)
{
   n <- nobs(fit)
   edf <- 1
   if (!is.null(fit$CCA$rank)) edf <- edf + fit$CCA$qrank
   if (!is.null(fit$pCCA$QR)) edf <- edf + fit$pCCA$QR$rank
   #edf <- n - fit$CA$rank
   RSS <- deviance(fit)
   dev <- if(scale > 0)
       RSS/scale - n
   else n * log(RSS/n)
   c(edf, dev + k*edf)
}

"permuted.index" <-
    function (n, strata) 
{
    .Deprecated("permute package (shuffle or shuffleSet)")
    if (missing(strata) || is.null(strata)) 
        out <- sample.int(n, n)
    else {
        out <- 1:n
        inds <- names(table(strata))
        for (is in inds) {
            gr <- out[strata == is]
            if (length(gr) > 1) 
                out[gr] <- sample(gr, length(gr))
        }
    }
    out
}

##" Individual based accumulation model. Similar to poolaccum but uses
##estimateR. Inherits from "poolaccum" class and uses its methods.
`estaccumR` <-
    function(x, permutations = 100, parallel = getOption("mc.cores"))
{
    n <- nrow(x)
    N <- seq_len(n)
    estFun <- function(idx) {
        estimateR(apply(x[idx,], 2, cumsum))[c(1,2,4),]
    }
    permat <- getPermuteMatrix(permutations, n)
    nperm <- nrow(permat)
    ## parallel processing
    if (is.null(parallel))
        parallel <- 1
    hasClus <- inherits(parallel, "cluster")
    if (hasClus || parallel > 1) {
        if(.Platform$OS.type == "unix" && !hasClus) {
            tmp <- mclapply(1:nperm, function(i)
                            estFun(permat[i,]),
                            mc.cores = parallel)
        } else {
            if (!hasClus) {
                parallel <- makeCluster(parallel)
            }
            tmp <- parLapply(parallel, 1:nperm, function(i) estFun(permat[i,]))
            if (!hasClus)
                stopCluster(parallel)
        }
    } else {
        tmp <- lapply(1:nperm, function(i) estFun(permat[i,]))
    }

    S <- sapply(tmp, function(x) x[1,])
    chao <- sapply(tmp, function(x) x[2,])
    ace <- sapply(tmp, function(x) x[3,])
    means <- cbind(N = N, S = rowMeans(S), Chao = rowMeans(chao),
                   ACE = rowMeans(ace))
    out <- list(S = S, chao = chao, ace = ace, N = N, means = means)
    attr(out, "control") <- attr(permat, "control")
    class(out) <- c("estaccumR", "poolaccum")
    out
}

"summary.radfit.frame" <-
function (object, ...)
{
    labels <- names(object)
    for (i in seq_along(labels)) {
        cat("\n***", labels[i], "***\n")
        print(object[[i]], ...)
    }
    invisible(object)
}

`ordiplot` <-
    function (ord, choices = c(1, 2), type = "points", display, xlim,
              ylim, cex = 0.7, ...)
{
    ## local functions to absorb non-par arguments of plot.default
    localPoints <- function(..., log, frame.plot, panel.first,
                            panel.last, axes) points(...)
    localText <- function(..., log, frame.plot, panel.first,
                          panel.last, axes) text(...)
    if (inherits(ord, "decorana") || inherits(ord, "cca")) {
        if (missing(display))
            out <- plot(ord, choices = choices, type = type, xlim = xlim,
                        ylim = ylim, ...)
        else out <- plot(ord, choices = choices, type = type, display = display,
                         xlim = xlim, ylim = ylim, ...)
    }
    else {
        type <- match.arg(type, c("points", "text", "none"))
        ## Matching displays could be done better (see
        ## ordipointlabel), but this may not be yet broken, so...
        dplays <- c("sites", "species")
        if (missing(display))
            display <- dplays
        else
            display <- match.arg(display, dplays, several.ok = TRUE)
        X <- Y <- NULL
        if ("sites" %in% display)
            X <- scores(ord, choices = choices, display = "sites")
        if ("species" %in% display) {
            options(show.error.messages = FALSE)
            Y <- try(scores(ord, choices = choices, display = "species"))
            options(show.error.messages = TRUE)
            if (inherits(Y, "try-error")) {
                message("species scores not available")
                Y <- NULL
            }
            else if (!is.null(X) && NROW(X) == NROW(Y) &&
                     isTRUE(all.equal.numeric(X, Y,
                                              check.attributes = FALSE))) {
                Y <- NULL
                message("species scores not available")
            }
        }
        ## Use linestack and exit if there is only one variable
        if (NCOL(X) == 1 && NCOL(Y) == 1) {
            pl <- linestack(X, ylim = range(c(X,Y), na.rm=TRUE), ...)
            if (!is.null(Y))
                linestack(Y, side = "left", add = TRUE, ...)
            return(invisible(pl))
        }
        tmp <- apply(rbind(X, Y), 2, range, na.rm=TRUE)
        if (missing(xlim))
            xlim <- tmp[, 1]
        if (missing(ylim))
            ylim <- tmp[, 2]
        plot(tmp, xlim = xlim, ylim = ylim, asp = 1, type = "n",
             ...)
        if (type == "points") {
            if (!is.null(X))
                localPoints(X, pch = 1, col = 1, cex = cex, ...)
            if (!is.null(Y))
                localPoints(Y, pch = "+", col = "red", cex = cex, ...)
        }
        if (type == "text") {
            if (!is.null(X))
                localText(X, labels = rownames(X), col = 1, cex = cex, ...)
            if (!is.null(Y))
                localText(Y, labels = rownames(Y), col = "red", cex = cex, ...)
        }
        out <- list(sites = X, species = Y)
    }
    class(out) <- "ordiplot"
    invisible(out)
}

### Forward selection to maximize R2.adjusted, but stopping once the
### R2.adjusted of the maximum model ('scope') is exceeded, after
### Blanchet, Legendre & Borcard: Ecology 89, 2623--2623; 2008.

`ordiR2step` <-
    function(object, scope, Pin = 0.05, R2scope = TRUE,
             permutations = how(nperm=499),
             trace = TRUE, R2permutations = 1000, ...)
{
    if (is.null(object$terms))
        stop("ordination model must be fitted using formula")
    if (missing(scope))
        stop("needs scope")
    if (inherits(scope, "cca"))
        scope <- delete.response(formula(scope))
    if (!inherits(scope, "formula"))
        scope <- reformulate(scope)
    ## Get R2 of the original object
    if (is.null(object$CCA))
        R2.0 <- 0
    else
        R2.0 <- RsquareAdj(object,
                           permutations = R2permutations, ...)$adj.r.squared
    ## only accepts upper scope
    if (is.list(scope) && length(scope) <= 2L)
        scope <- scope$upper
    if (is.null(scope) || !length(add.scope(object, scope)))
        stop("needs upper 'scope': no terms can be added")
    ## Get R2 of the scope
    if (R2scope)
        R2.all <- RsquareAdj(update(object, delete.response(formula(scope))),
                             permutations = R2permutations, ...)
    else
        R2.all <- list(adj.r.squared = NA)
    ## Check that the full model can be evaluated
    if (is.na(R2.all$adj.r.squared) && R2scope)
        stop("the upper scope cannot be fitted (too many terms?)")
    R2.all <- R2.all$adj.r.squared
    ## Collect data to anotab returned as the 'anova' object
    anotab <-  list()
    ## Step forward and continue as long as R2.adj improves and R2.adj
    ## remains below R2.adj < R2.all
    R2.previous <- R2.0
    repeat {
        if (trace) {
            cat("Step: R2.adj=", R2.previous, "\n")
            cat(pasteCall(formula(object)), "\n")
        }
        adds <- add.scope(object, scope)
        ## Nothing to add and we're done: break
        if (length(adds) == 0)
            break
        R2.adds <- numeric(length(adds))
        adds <- paste("+", adds)
        names(R2.adds) <- adds
        ## Loop over add scope
        for (trm in seq_along(R2.adds)) {
            fla <- paste(". ~ .", names(R2.adds[trm]))
            R2.tmp <- RsquareAdj(update(object, fla),
                                 permutations = R2permutations, ...)$adj.r.squared
            if (!length(R2.tmp))
                R2.tmp <- 0
            R2.adds[trm] <- R2.tmp
        }
        best <- which.max(R2.adds)
        if (trace) {
            out <- sort(c("<All variables>" = R2.all, "<none>" = R2.previous,
                          R2.adds), decreasing = TRUE)
            out <- as.matrix(out)
            colnames(out) <- "R2.adjusted"
            print(out)
            cat("\n")
        }
        ## See if the best should be kept
        ## First criterion: R2.adj improves and is still lower or
        ## equal than for the full model of the scope
        if (R2.adds[best] > R2.previous &&
            (!R2scope || R2scope && R2.adds[best] <= R2.all)) {
            ## Second criterion: added variable is significant
            tst <- add1(object, scope = adds[best], test="permu",
                        permutations = permutations,
                        alpha = Pin, trace = FALSE, ...)
            if (trace) {
                print(tst[-1,])
                cat("\n")
            }
            if (tst[,"Pr(>F)"][2] <= Pin) {
                fla <- paste("~  .", names(R2.adds[best]))
                object <-  update(object, fla)
            } else
                break
        } else {
            break
        }
        R2.previous <- RsquareAdj(object,
                                  permutations = R2permutations,
                                  ...)$adj.r.squared
            anotab <- rbind(anotab,
                            cbind("R2.adj" = R2.previous, tst[2,]))
    }
    if (NROW(anotab)) {
        if (R2scope)
            anotab <- rbind(anotab, "<All variables>" = c(R2.all, rep(NA, 4)))
        class(anotab) <- c("anova", class(anotab))
        object$anova <- anotab
    }
    object
}

"print.prestonfit" <-
    function (x, ...) 
{
    cat("\nPreston lognormal model\n")
    cat("Method:", x$method,"\n")
    cat("No. of species:", sum(x$freq), "\n\n")
    print(x$coefficients, ...)
    cat("\nFrequencies by Octave\n")
    print(rbind(Observed = x$freq, Fitted = x$fitted), ...)
    cat("\n")
    invisible(x)
}

`protest` <-
    function (X, Y, scores = "sites", permutations = how(nperm = 999),
              ...)
{
    EPS <- sqrt(.Machine$double.eps)
    X <- scores(X, display = scores, ...)
    Y <- scores(Y, display = scores, ...)
    ## Centre and normalize X & Y here so that the permutations will
    ## be faster
    X <- scale(X, scale = FALSE)
    Y <- scale(Y, scale = FALSE)
    X <- X/sqrt(sum(X^2))
    Y <- Y/sqrt(sum(Y^2))
    ## Transformed X and Y will yield symmetric procrustes() and we
    ## need not specify that in the call (but we set it symmetric
    ## after the call).
    sol <- procrustes(X, Y, symmetric = FALSE)
    sol$symmetric <- TRUE
    sol$t0 <- sqrt(1 - sol$ss)
    N <- nrow(X)

    ## Permutations: We only need the goodness of fit statistic from
    ## Procrustes analysis, and therefore we only have the necessary
    ## function here. This avoids a lot of overhead of calling
    ## procrustes() for each permutation. The following gives the
    ## Procrustes r directly.
    procr <- function(X, Y) sum(svd(crossprod(X, Y), nv=0, nu=0)$d)

    permutations <- getPermuteMatrix(permutations, N)
    if (ncol(permutations) != N)
        stop(gettextf("'permutations' have %d columns, but data have %d observations",
                      ncol(permutations), N))
    np <- nrow(permutations)

    perm <- sapply(seq_len(np),
                   function(i, ...) procr(X, Y[permutations[i,],]))

    Pval <- (sum(perm >= sol$t0 - EPS) + 1)/(np + 1)

    sol$t <- perm
    sol$signif <- Pval
    sol$permutations <- np
    sol$control <- attr(permutations, "control")
    sol$call <- match.call()
    class(sol) <- c("protest", "procrustes")
    sol
}

`summary.poolaccum` <-
    function(object, display, alpha = 0.05, ...)
{
    probs <- c(alpha/2, 1-alpha/2)
    if (inherits(object, "estaccumR"))
        dislabels <- c("S", "chao", "ace")
    else
        dislabels <- c("S", "chao", "jack1", "jack2", "boot")
    disnames <- colnames(object$means[,-1])
    names(disnames) <- dislabels
    if (missing(display))
        display <- dislabels
    else
        display <- match.arg(display, dislabels, several.ok = TRUE)
    out <- list()
    for (item in display) {
        out[[item]] <- cbind(`N` = object$N,
                             `Mean` = object$means[,disnames[item], drop=FALSE],
                             t(apply(object[[item]], 1, quantile, probs=probs)),
                             `Std.Dev` = apply(object[[item]], 1, sd))
    }
    class(out) <- "summary.poolaccum"
    out
}

`cophenetic.spantree` <-
    function(x)
{
    n <- x$n
    mat <- matrix(NA, nrow=n, ncol=n)
    if (n < 2)
        return(as.dist(mat))
    ind <- apply(cbind(2:n, x$kid), 1, sort)
    ind <- t(ind[2:1,])
    mat[ind] <- x$dist
    d <- as.dist(mat)
    attr(d, "Labels") <- x$labels
    stepacross(d, path = "extended", toolong=0, trace=FALSE)
}

`decostand` <-
    function (x, method, MARGIN, range.global, logbase = 2, na.rm = FALSE, ...)
{
    wasDataFrame <- is.data.frame(x)
    x <- as.matrix(x)
    METHODS <- c("total", "max", "frequency", "normalize", "range", "rank",
                 "rrank", "standardize", "pa", "chi.square", "hellinger",
                 "log", "clr", "rclr", "alr")
    method <- match.arg(method, METHODS)
    if (any(x < 0, na.rm = TRUE)) {
        k <- min(x, na.rm = TRUE)
        if (method %in% c("total", "frequency", "pa", "chi.square", "rank",
                          "rrank", "rclr")) {
            warning("input data contains negative entries: result may be non-sense")
        }
    }
    else k <- .Machine$double.eps
    attr <- NULL
    switch(method, total = {
        if (missing(MARGIN))
            MARGIN <- 1
        tmp <- pmax.int(k, apply(x, MARGIN, sum, na.rm = na.rm))
        x <- sweep(x, MARGIN, tmp, "/")
        attr <- list("total" = tmp, "margin" = MARGIN)
    }, max = {
        if (missing(MARGIN))
            MARGIN <- 2
        tmp <- pmax.int(k, apply(x, MARGIN, max, na.rm = na.rm))
        x <- sweep(x, MARGIN, tmp, "/")
        attr <- list("max" = tmp, "margin" = MARGIN)
    }, frequency = {
        if (missing(MARGIN))
            MARGIN <- 2
        tmp <- pmax.int(k, apply(x, MARGIN, sum, na.rm = na.rm))
        fre <- apply(x > 0, MARGIN, sum, na.rm = na.rm)
        tmp <- fre/tmp
        x <- sweep(x, MARGIN, tmp, "*")
        attr <- list("scale" = tmp, "margin" = MARGIN)
    }, normalize = {
        if (missing(MARGIN))
            MARGIN <- 1
        tmp <- apply(x^2, MARGIN, sum, na.rm = na.rm)
        tmp <- pmax.int(.Machine$double.eps, sqrt(tmp))
        x <- sweep(x, MARGIN, tmp, "/")
        attr <- list("norm" = tmp, "margin" = MARGIN)
    }, range = {
        if (missing(MARGIN))
            MARGIN <- 2
        if (missing(range.global))
            xtmp <- x
        else {
            if (dim(range.global)[MARGIN] != dim(x)[MARGIN])
                stop("range matrix does not match data matrix")
            xtmp <- as.matrix(range.global)
        }
        tmp <- apply(xtmp, MARGIN, min, na.rm = na.rm)
        ran <- apply(xtmp, MARGIN, max, na.rm = na.rm)
        ran <- ran - tmp
        ran <- pmax.int(k, ran, na.rm = na.rm)
        x <- sweep(x, MARGIN, tmp, "-")
        x <- sweep(x, MARGIN, ran, "/")
        attr <- list("min" = tmp, "range" = ran, "margin" = MARGIN)
    }, rank = {
        wasNA <- is.na(x)
        if (any(wasNA) && !na.rm)
            stop("missing values are not allowed with 'na.rm = FALSE'")
        if (missing(MARGIN)) MARGIN <- 1
        x[x==0] <- NA
        x <- apply(x, MARGIN, rank, na.last = "keep")
        if (MARGIN == 1) # gives transposed x
            x <- t(x)
        x[is.na(x)] <- 0
        if(any(wasNA))
            x[wasNA] <- NA
        attr <- list("margin" = MARGIN)
    }, rrank = {
        if (missing(MARGIN)) MARGIN <- 1
        x <- decostand(x, "rank", MARGIN = MARGIN, na.rm = na.rm)
        if (na.rm && any(wasNA <- is.na(x)))
            x[wasNA] <- 0
        x <- sweep(x, MARGIN, specnumber(x, MARGIN = MARGIN), "/")
        if (any(wasNA))
            x[wasNA] <- NA
        attr <- list("margin" = MARGIN)
    }, standardize = {
        if (!missing(MARGIN) && MARGIN == 1)
            x <- t(scale(t(x)))
        else {
            x <- scale(x)
            MARGIN <- 2
        }
        attr <- list("center" = attr(x, "scaled:center"),
                     "scale" = attr(x, "scaled:scale"),
                     "margin" = MARGIN)
    }, pa = {
        x <- ifelse(x > 0, 1, 0)
    }, chi.square = {
        if (missing(MARGIN))
            MARGIN <- 1
        ## MARGIN 2 transposes the result!
        if (MARGIN == 2)
            x <- t(x)
        rs <- pmax.int(k, rowSums(x, na.rm = na.rm))
        cs <- pmax.int(k, colSums(x, na.rm = na.rm))
        tot <- sum(x, na.rm = na.rm)
        x <- sqrt(tot) * x/outer(rs, sqrt(cs))
        attr <- list("tot" = tot, "rsum" = rs, "csum" = cs, margin = MARGIN)
    }, hellinger = {
        x <- sqrt(decostand(x, "total", MARGIN = MARGIN, na.rm = na.rm))
        attr <- attr(x, "parameters")
    }, log = {### Marti Anderson logs, after Etienne Laliberte
        if (!isTRUE(all.equal(as.integer(x), as.vector(x)))) {
            minpos <- min(x[x > 0], na.rm = TRUE)
            x <- x / minpos
            warning("non-integer data: divided by smallest positive value",
                    call. = FALSE)
        } else {
            minpos <- 1
        }
        x[x > 0 & !is.na(x)] <- log(x[x > 0 & !is.na(x)], base = logbase) + 1
        attr <- list("logbase" = logbase, minpos = minpos)
    }, alr = {
        if (missing(MARGIN))
	    MARGIN <- 1
        if (MARGIN == 1)
            x <- t(.calc_alr(t(x), ...))
	else x <- .calc_alr(x, ...)
        attr <- attr(x, "parameters")
        attr$margin <- MARGIN
    }, clr = {
        if (missing(MARGIN))
	    MARGIN <- 1
        if (MARGIN == 1)
            x <- .calc_clr(x, ...)
	else x <- t(.calc_clr(t(x), ...))
        attr <- attr(x, "parameters")
        attr$margin <- MARGIN
    }, rclr = {
        if (missing(MARGIN))
	    MARGIN <- 1
        if (MARGIN == 1)
            x <- .calc_rclr(x, ...)
	else x <- t(.calc_rclr(t(x), ...))
        attr <- attr(x, "parameters")
        attr$margin <- MARGIN
    })
    if (any(is.nan(x)))
        warning("result contains NaN, perhaps due to impossible mathematical
                 operation\n")
    if (wasDataFrame)
        x <- as.data.frame(x)
    attr(x, "parameters") <- attr
    attr(x, "decostand") <- method
    x
}

## Modified from the original version in mia R package
.calc_clr <-
    function(x, pseudocount=0, na.rm = TRUE)
{
    # Add pseudocount
    x <- x + pseudocount
    # Error with negative values
    if (any(x <= 0, na.rm = na.rm)) {
        stop("'clr' cannot be used with non-positive data: use pseudocount > ",
             -min(x, na.rm = na.rm) + pseudocount, call. = FALSE)
    }

    # In every sample, calculate the log of individual entries.
    # Then calculate
    # the sample-specific mean value and subtract every entries'
    # value with that.
    clog <- log(x)
    means <- rowMeans(clog, na.rm = na.rm)
    clog <- clog - means
    attr(clog, "parameters") <- list("means" = means,
                                     "pseudocount" = pseudocount)
    clog
}

# Modified from the original version in mia R package
.calc_rclr <-
    function(x, na.rm = TRUE)
{
    # Error with negative values
    if (any(x < 0, na.rm = na.rm)) {
        stop("'rclr' cannot be used with negative data", call. = FALSE)
    }
   # Log transform
   clog <- log(x)
   # Convert zeros to NAs in rclr
   clog[is.infinite(clog)] <- NA
   # Calculate log of geometric mean for every sample, ignoring the NAs
   mean_clog <- rowMeans(clog, na.rm = na.rm)
   # Divide all values by their sample-wide geometric means
   # Log and transpose back to original shape
   xx <- log(x) - mean_clog
   # If there were zeros, there are infinite values after logarithmic transform.
   # Convert those to zero.
   xx[is.infinite(xx)] <- 0
   attr(xx, "parameters") <- list("means" = mean_clog)
   xx
}


.calc_alr <-
    function (x, reference = 1, pseudocount = 0, na.rm = TRUE)
{
    # Add pseudocount
    x <- x + pseudocount
    # If there is negative values, gives an error.
    if (any(x < 0, na.rm = na.rm)) {
        stop("'alr' cannot be used with negative data: use pseudocount >= ",
             -min(x, na.rm = na.rm) + pseudocount, call. = FALSE)
    }
    ## name must be changed to numeric index for [-reference,] to work
    if (is.character(reference)) {
        reference <- which(reference == colnames(x))
        if (!length(reference)) # found it?
            stop("'reference' name was not found in data", call. = FALSE)
    }
    if (reference > ncol(x) || reference < 1)
        stop("'reference' should be a name or index 1 to ",
             ncol(x), call. = FALSE)
    clog <- log(x)
    refvector <- clog[, reference]
    clog <- clog[, -reference] - refvector
    attr(clog, "parameters") <- list("reference" = refvector,
                                     "index" = reference,
                                     "pseudocount" = pseudocount)
    clog
}

`decobackstand` <-
    function(x, zap = TRUE)
{
    method <- attr(x, "decostand")
    if (is.null(method))
        stop("function can be used only with 'decostand' standardized data")
    para <- attr(x, "parameters")
    if(is.null(para)) # for old results & "pa"
        stop("object has no information to backtransform data")
    x <- switch(method,
                "total" = sweep(x, para$margin, para$total, "*"),
                "max" = sweep(x, para$margin, para$max, "*"),
                "frequency" = sweep(x, para$margin, para$scale, "/"),
                "normalize" = sweep(x, para$margin, para$norm, "*"),
                "range" = { x <- sweep(x, para$margin, para$range, "*")
                            sweep(x, para$margin, para$min, "+")},
                "standardize" = {x <- sweep(x, para$margin, para$scale, "*")
                                 sweep(x, para$margin, para$center, "+") },
                "hellinger" = sweep(x^2, para$margin, para$total, "*"),
                "chi.square" = { rc <- outer(para$rsum, sqrt(para$csum))
                                 x <- x * rc /sqrt(para$tot)
                                 if (para$margin == 1) x else t(x) },
                "log" = { x[x > 0 & !is.na(x)] <-
                              para$logbase^(x[x > 0 & !is.na(x)] - 1)
                              x * para$minpos},
                "clr" = exp(sweep(x, para$margin, para$means, "+")) -
                    para$pseudocount,
                "rclr" = { x[x == 0] <- -Inf # x==0 was set: should be safe
                           exp(sweep(x, para$margin, para$means, "+"))},
                "wisconsin" = { x <- sweep(x, 1, para$total, "*")
                                sweep(x, 2, para$max, "*") },
                stop("no backtransformation available for method ",
                     sQuote(method))
                )
    if (zap)
        x[abs(x) < sqrt(.Machine$double.eps)] <- 0
    x
}

### Interface to the permute package

### input can be (1) a single number giving the number of
### permutations, (2) a how() structure for control parameter in
### permute::shuffleSet, or (3) a permutation matrix which is returned
### as is. In addition, there can be a 'strata' argument which will
### modify case (1). The number of shuffled items must be given in 'N'.

`getPermuteMatrix` <-
    function(perm, N,  strata = NULL)
{
    ## 'perm' is either a single number, a how() structure or a
    ## permutation matrix
    if (length(perm) == 1) {
        perm <- how(nperm = perm)
    }
    ## apply 'strata', but only if possible: ignore silently other cases
    if (!missing(strata) && !is.null(strata)) {
        if (inherits(perm, "how") && is.null(getBlocks(perm)))
            setBlocks(perm) <- strata
    }
    ## now 'perm' is either a how() or a matrix
    if (inherits(perm, "how"))
        perm <- shuffleSet(N, control = perm)
    else { # matrix: check that it *strictly* integer
        if(!is.integer(perm) && !all(perm == round(perm)))
           stop("permutation matrix must be strictly integers: use round()")
    }
    ## now 'perm' is a matrix (or always was). If it is a plain
    ## matrix, set minimal attributes for printing. This is a dirty
    ## kluge: should be handled more cleanly.
    if (is.null(attr(perm, "control")))
        attr(perm, "control") <-
            structure(list(within=list(type="supplied matrix"),
                           nperm = nrow(perm)), class = "how")
    perm
}

#' Permutation test for the area of convex hull or ellipse in ordination
#'
#' Finds if the area covered by a convex hull or fitted ellipse is
#' smaller than expected under null hypothesis using permutation test.
#'
#' @param ord 2-d ordination
#' @param factor defining groups
#' @param are of convex hull of or an ellipse
#' @param permutations: number, permutation matrix or a
#' \code{\link[permute]{how}} definition.
#' @param parallel parallel processing
#' @param \dots other parameters passed to area functions
#'
#' @author Jari Oksanen
`ordiareatest` <-
    function(ord, groups, area = c("hull", "ellipse"), kind = "sd",
             permutations = 999, parallel = getOption("mc.cores"), ...)
{
    EPS <- sqrt(.Machine$double.eps)
    ## Function to find area
    area <- match.arg(area)
    areafun <- if (area == "hull") ordihull else ordiellipse
    areafun <- match.fun(areafun)
    ## Observed statistics
    obs <- summary(areafun(ord, groups, draw = "none", kind = kind))["Area",]
    ## permutations
    pfun <- function(take, ...)
        summary(areafun(ord, groups[take], draw = "none", kind = kind))["Area",]
    perm <- getPermuteMatrix(permutations, length(groups))
    nperm <- nrow(perm)
    if (is.null(parallel))
        parallel <- 1
    hasClus <- inherits(parallel, "cluster")
    if (hasClus || parallel > 1) {
        if(.Platform$OS.type == "unix" && !hasClus) {
            areas <- do.call(cbind,
                             mclapply(1:nperm,
                                      function(i, ...) pfun(perm[i,],...),
                                        mc.cores = parallel))
            } else {
                if (!hasClus) {
                    parallel <- makeCluster(parallel)
                }
                areas <- parApply(parallel, perm, MARGIN=1, pfun)
                if (!hasClus)
                    stopCluster(parallel)
            }
    } else {
        areas <- sapply(1:nperm, function(i, ...) pfun(perm[i,], ...))
    }
    signif <- (rowSums(areas <= obs + EPS) + 1)/(nperm + 1)
    out <- list("areas" = obs, "pvalues" = signif, "permutations" = areas,
                nperm = nperm, control = attr(perm, "control"), "kind" = area)
    class(out) <- "ordiareatest"
    out
}

### print method

`print.ordiareatest` <-
    function(x, ...)
{
    qu <- apply(x$permutations, 1, quantile, probs=c(0.05, 0.5))
    m <- cbind("Area" = x$areas, t(qu), "Pr(<sim)" = x$pvalues)
    cat("\n")
    cat(sprintf("Permutation test for the size of ordination %s\nalternative hypothesis: observed area is smaller than random %s\n\n", x$kind, x$kind))
    cat(howHead(x$control), "\n")
    printCoefmat(m, tst.ind=1:3)
    invisible(x)
}

`ca` <- function(X, ...) {
  if (inherits(X, "formula")) {
    stop("Argument 'X' was supplied a formula, which is not supported by 'ca()'",
    call. = FALSE)
  }
  ord <- cca(X = X, ...)
  # change the call to be from ca()
  ord$call <- match.call()
  class(ord) <- append(class(ord), "vegan_ca", after = 0)
  ord
}

"plot.rad" <-
    function(x, xlab="Rank", ylab="Abundance", log = "y", ...)
{
    rnk <- seq(along=x)
    plot(rnk, x, log=log, xlab=xlab, ylab=ylab, ...)
    out <- list(species = cbind(rnk, x))
    class(out) <- "ordiplot"
    invisible(out)
}

`as.ts.oecosimu` <-
    function(x, ...)
{
    if  (!x$oecosimu$isSeq)
        stop("as.ts available only for sequential methods")
    chains <- attr(x$oecosimu$simulated, "chains")
    if (!is.null(chains) && chains > 1)
        stop("as.ts available only for single chain")
    thin <- attr(x$oecosimu$simulated, "thin")
    startval <- attr(x$oecosimu$simulated, "burnin") + thin
    out <- ts(t(x$oecosimu$simulated), start = startval, deltat=thin,
        names = names(x$oecosimu$z))
    attr(out, "burnin") <- NULL
    attr(out, "thin") <- NULL
    out
}

`alias.cca` <-
    function (object, names.only = FALSE, ...) 
{
    if (is.null(object$CCA))
        stop("no constrained component, 'alias' cannot be applied")
    if (is.null(object$CCA$alias))
        stop("no aliased terms")
    if (names.only)
        return(object$CCA$alias)
    CompPatt <- function(x, ...) {
        x[abs(x) < 1e-06] <- 0
        class(x) <- "mtable"
        x[abs(x) < 1e-06] <- NA
        x
    }
    Model <- object$terms
    attributes(Model) <- NULL
    value <- list(Model = Model)
    R <- object$CCA$QR$qr
    R <- R[1:min(dim(R)), , drop = FALSE]
    R[lower.tri(R)] <- 0
    d <- dim(R)
    rank <- object$CCA$QR$rank
    p <- d[2]
    value$Complete <- if (is.null(p) || rank == p) 
        NULL
    else {
        p1 <- 1:rank
        X <- R[p1, p1]
        Y <- R[p1, -p1, drop = FALSE]
        beta12 <- as.matrix(qr.coef(qr(X), Y))
        CompPatt(t(beta12))
    }
    class(value) <- "listof"
    value
}

`fitted.radfit` <-
    function(object, ...)
{
    out <- sapply(object$models, fitted)
    if (!length(object$y))
        out <- numeric(length(object$models))
    if (length(object$y) <= 1) 
        out <- structure(as.vector(out), dim = c(1, length(object$models)),
                         dimnames = list(names(object$y), names(object$models)))
    out
}

`fitted.radfit.frame` <-
    function(object, ...)
{
    lapply(object, fitted, ...)
}

`profile.MOStest` <-
    function(fitted, alpha = 0.01, maxsteps = 10, del = zmax/5, ...)
{
    Pnam <- if(fitted$isHump) "hump" else "pit"
    k <- coef(fitted$mod)
    u <- -k[2]/2/k[3]
    n <- length(residuals(fitted$mod))
    std.error <- fieller.MOStest(fitted, level=0.6)
    std.error <- u - std.error[1]
    if (is.na(std.error))
        std.error <- diff(range(model.matrix(fitted$mod)[,2]))
    OrigDev <- deviance(fitted$mod)
    summ <- summary(fitted$mod)
    DispPar <- summ$dispersion
    fam <- family(fitted$mod)
    Y <- fitted$mod$y
    X <- model.matrix(fitted$mod)[,-3]
    Xi <- X
    if (fam$family %in% c("poisson", "binomial", "Negative Binomial")) {
        zmax <- sqrt(qchisq(1 - alpha/2, 1))
        profName <- "z"
    } else {
        zmax <- sqrt(qf(1 - alpha/2, 1, n - 1))
        profName <- "tau"
    }
    zi <- 0
    prof <- vector("list", length=1)
    names(prof) <- Pnam
    uvi <- u
    for (sgn in c(-1, 1)) {
        step <- 0
        z <- 0
        while((step <- step + 1) < maxsteps && abs(z) < zmax) {
            ui <- u + sgn * step * del * std.error
            Xi[,2] <- (X[,2] - ui)^2
            fm <- glm.fit(x = Xi, y = Y, family=fam,
                          control = fitted$mod$control)
            uvi <- c(uvi, ui)
            zz <- (fm$deviance - OrigDev)/DispPar
            z <- sgn * sqrt(zz)
            zi <- c(zi, z)
        }
        si <- order(zi)
        prof[[Pnam]] <- structure(data.frame(zi[si]), names=profName)
        uvi <- as.matrix(uvi)
        colnames(uvi) <- Pnam
        prof[[Pnam]]$par.vals <- uvi[si, , drop=FALSE]
    }
    of <- list()
    of$coefficients <- structure(Pnam, names=Pnam)
    val <- structure(prof, original.fit = of, summary = summ)
    class(val) <- c("profile.MOStest", "profile.glm", "profile")
    val
}

## S3 plot method for permat
`plot.permat` <-
function(x, type = "bray", ylab, xlab, col, lty, lowess=TRUE, plot=TRUE, text=TRUE, ...)
{
    type <- match.arg(type, c("bray", "chisq"))
    if (missing(xlab))
        xlab <- "Runs"
    if (missing(col))
        col <- c(2,4)
    if (missing(lty))
        lty <- c(1,2)
    n <- attr(x, "times")
    toplot <- numeric(n)
    if (type == "bray") {
        toplot <- summary(x)$bray
        if (missing(ylab))
            ylab <- "Bray-Curtis dissimilarity"
    }
    if (type == "chisq") {
        toplot <- summary(x)$chisq
        if (missing(ylab))
            ylab <- "Chi-squared"
    }
    if (plot) {
        plot(toplot,type="n",ylab=ylab,xlab=xlab, ...)
        lines(toplot,col=col[1], lty=lty[1])
        if (lowess)
            lines(lowess(toplot),col=col[2], lty=lty[2])
        if (text) title(sub=paste("(mean = ", substitute(z, list(z=round(mean(toplot),3))), 
            ", min = ", substitute(z, list(z=round(min(toplot),3))),
            ", max = ", substitute(z, list(z=round(max(toplot),3))), ")", sep=""))
    }
    invisible(toplot)
}

`scores.metaMDS` <-
    function(x, display = c("sites", "species"), shrink = FALSE, choices,
             tidy = FALSE, ...)
{
    display <- match.arg(display, c("sites","species"), several.ok = TRUE)
    if (missing(choices))
        choices <- seq_len(x$ndim)
    else
        choices <- choices[choices <= x$ndim]
    out <- list()
    if ("sites" %in% display) {
        sites <- x$points[, choices, drop=FALSE]
        colnames(sites) <- paste0("NMDS", choices)
        out$sites <- sites
    }
    if ("species" %in% display && !is.null(x$species) && !all(is.na(x$species))) {
        species <- x$species[, choices, drop=FALSE]
        colnames(species) <- paste0("NMDS", choices)
        if (shrink) {
            ## [,choices] drops attributes
            mul <- sqrt(attr(x$species, "shrinkage"))
            cnt <- attr(x$species, "centre")
            if (is.null(mul))
                message("species are not shrunken, because they were not expanded")
            else {
                mul <- mul[choices]
                cnt <- cnt[choices]
                species <- sweep(species, 2, cnt, "-")
                species <- sweep(species, 2, mul, "*")
                species <- sweep(species, 2, cnt, "+")
            }
        }
        out$species <- species
    }
    if (tidy) {
        if (length(out) == 0) # no scores (species scores may not exist)
            return(NULL)
        group <- sapply(out, nrow)
        group <- rep(names(group), group)
        out <- do.call(rbind, out)
        label <- rownames(out)
        out <- as.data.frame(out)
        out$score <- group
        out$label <- label
    }
    ## only two kind of scores, return NULL, matrix, or a list of scores
    if (length(out) == 1)
        out[[1]]
    else
        out
}

`diversity` <-
    function (x, index = "shannon", MARGIN = 1, base = exp(1))
{
    x <- drop(as.matrix(x))
    if (!is.numeric(x))
        stop("input data must be numeric")
    if (any(x < 0, na.rm = TRUE))
        stop("input data must be non-negative")
    INDICES <- c("shannon", "simpson", "invsimpson")
    index <- match.arg(index, INDICES)
    if (length(dim(x)) > 1) {
        total <- apply(x, MARGIN, sum)
        x <- sweep(x, MARGIN, total, "/")
    } else {
        x <- x/(total <- sum(x))
    }
    if (index == "shannon")
        x <- -x * log(x, base)
    else
        x <- x*x
    if (length(dim(x)) > 1)
        H <- apply(x, MARGIN, sum, na.rm = TRUE)
    else
        H <- sum(x, na.rm = TRUE)
    if (index == "simpson")
        H <- 1 - H
    else if (index == "invsimpson")
        H <- 1/H
    ## check NA in data
    if (any(NAS <- is.na(total)))
        H[NAS] <- NA
    H
}

### Multiple-site dissimilarity indices (Sorensen & Jaccard) and their
### decomposition into "turnover" and "nestedness" following Baselga
### (Global Ecology & Biogeography 19, 134-143; 2010). Implemented as
### nestedness functions and directly usable in oecosimu().

`nestedbetasor` <-
    function(comm)
{
    beta <- betadiver(comm, method = NA)
    b <- beta$b
    c <- beta$c
    diffbc <- sum(abs(b-c))
    sumbc <- sum(b+c)
    bmin <- sum(pmin(b, c))
    a <- sum(comm > 0) - sum(colSums(comm) > 0)
    simpson <- bmin/(bmin + a)
    nest <- a/(bmin + a) * diffbc/(2*a + sumbc)
    sorensen <- sumbc/(2*a + sumbc)
    c(turnover = simpson, nestedness = nest, sorensen = sorensen)
}

`nestedbetajac` <-
    function(comm)
{
    beta <- betadiver(comm, method = NA)
    b <- beta$b
    c <- beta$c
    diffbc <- sum(abs(b-c))
    sumbc <- sum(b+c)
    bmin <- sum(pmin(b, c))
    a <- sum(comm > 0) - sum(colSums(comm) > 0)
    simpson <- 2*bmin/(2*bmin + a)
    nest <- a/(2*bmin + a) * diffbc/(a + sumbc)
    jaccard <- sumbc/(a + sumbc)
    c(turnover = simpson, nestedness = nest, jaccard = jaccard)
}

## permatfull function
`permatfull` <-
function(m, fixedmar="both", shuffle="both",
strata=NULL, mtype="count", times=99, ...)
{
    mtype <- match.arg(mtype, c("prab", "count"))
    shuffle <- match.arg(shuffle, c("ind", "samp", "both"))
    fixedmar <- match.arg(fixedmar, c("none", "rows", "columns", "both"))
    m <- as.matrix(m)
    str <- if (is.null(strata))
        1 else as.integer(as.factor(strata)[drop = TRUE])
    levstr <- unique(str)
    nstr <- length(unique(str))
    if (!is.null(strata) && any(table(str) < 2))
        stop("strata should contain at least two observations")
    ALGO <- switch(fixedmar,
        "none" = "r00",
        "rows" = "r0",
        "columns" = "c0",
        "both" = ifelse(mtype=="prab", "quasiswap", "r2dtable"))
    if (mtype=="count") {
        if (fixedmar!="both")
            ALGO <- paste(ALGO, shuffle, sep="_")
    }
    if (is.null(strata)) {
        tmp <- simulate(nullmodel(m, ALGO), nsim=times, ...)
        perm <- vector("list", times)
        for (i in seq_len(times))
            perm[[i]] <- tmp[,,i]
    } else {
        perm <- vector("list", times)
        tmp <- vector("list", length(unique(strata)))
        for (j in seq_len(nstr)) {
            tmp[[j]] <- simulate(nullmodel(m[strata==levstr[j],], ALGO),
                nsim=times, ...)
        }
        for (i in seq_len(times)) {
            perm[[i]] <- array(0, dim(m))
            for (j in seq_len(nstr)) {
                perm[[i]][strata==levstr[j],] <- tmp[[j]][,,i]
            }
        }
    }
    if (fixedmar == "both")
        shuffle <- NA
    if (mtype == "prab")
        m <- ifelse(m > 0, 1, 0)
    out <- list(call=match.call(), orig=m, perm=perm)
    attr(out, "mtype") <- mtype
    attr(out, "ptype") <- "full"
    attr(out, "method") <- ALGO
    attr(out, "fixedmar") <- fixedmar
    attr(out, "times") <- times
    attr(out, "shuffle") <- shuffle
    attr(out, "is.strat") <- !is.null(strata)
    attr(out, "strata") <- str
    attr(out, "burnin") <- NA
    attr(out, "thin") <- NA
    class(out) <- c("permatfull", "permat")
    out
}

`panel.ordi3d` <-
    function(x, y, z, aspect, ...)
{
    dx <- diff(range(x))
    dy <- diff(range(y))
    dz <- diff(range(z))
    aspect <- c(dy/dx, dz/dx)
    panel.cloud(x = x, y = y, z = z, aspect = aspect, ...)
}

`kendall.post` <-
	function(Y, group, nperm=999, mult="holm")
{
### Function to carry out a posteriori tests on individual judges (e.g. species)
### for a single group or several groups of judges.
###
### copyleft - Guillaume Blanchet and Pierre Legendre, October 2008
################################################################################

    mult <- match.arg(mult, c("sidak", p.adjust.methods))

    ##CC# Make sure Y is a matrix and find number of rows and columns of Y
    Y <- as.matrix(Y)
    n <- nrow(Y)
    p <- ncol(Y)
    if(p < 2) stop("there is only one variable in the data matrix")

    ##CC# Transform the species abundances to ranks, by column
    R <- apply(Y,2,rank)

    if(missing(group)) group <- rep(1,p)
    if(length(group) != p){
        stop("the number of species in the vector differs from the total number of species")
    }

    ##CC# Separate tests for the variables in each group
    group <- as.factor(group)
    gr.lev <- levels(group)
    ngr <- nlevels(group)

    gr <- as.list(1:ngr)

    n.per.gr <- vector(length=ngr)
    for(i in 1:ngr){
        gr[[i]] <- which(group==gr.lev[i])
        n.per.gr[i] <- length(gr[[i]])  # Vector with the number of
                                        # variables per group
    }

    ##===============================
    ##CC# start permutation procedure
    ##===============================
    ##CC# Initialize the counters
    counter <- as.list(1:ngr)
    for(i in 1:ngr){
        counter[[i]] <- vector(length = n.per.gr[i])
    }
    W.gr <- vector("list",ngr)
    if(ngr > 1) spear.gr <- vector("list",ngr)

    for(i in 1:ngr){
        p.i <- n.per.gr[i]
        if(p.i < 2) stop(gettextf("there is only one variable in group %d",
                                  gr.lev[i]))
                                        #CC# Extract variables part of
                                        #group i
        R.gr <- R[,gr[[i]]]     # Table with species of group 'i' only
                                        #as columns CC# Calculate the
                                        #mean of the Spearman
                                        #correlations for each species
                                        #in a group
        spear.mat <- cor(R.gr)
        diag(spear.mat) <- NA
        spear.mean <- apply(spear.mat, 1, mean, na.rm=TRUE)
                                        #CC# Calculate Kendall's W for each variable
        W.var <- ((p.i-1)*spear.mean+1)/p.i

                                        #for(j in 1:n.per.gr[i]){
        for(j in 1:p.i){                # Test each species in turn
                                        #CC# Create a new R where the
                                        #permuted variable has been
                                        #removed
            R.gr.mod <- R.gr[,-j]
            ##CC# Set counter
            counter[[i]][j] <- 1
            ##CC# The actual permutation procedure
            for(k in 1:nperm){
                var.perm <- sample(R.gr[,j])
                spear.mat.perm <- cor(cbind(R.gr.mod, var.perm))
                diag(spear.mat.perm) <- NA
                spear.mean.j.perm <- mean(spear.mat.perm[p.i,], na.rm=TRUE)
                W.perm <- ((p.i-1)*spear.mean.j.perm+1)/p.i
                if(W.perm >= W.var[j]) counter[[i]][j] <- counter[[i]][j]+1
            }
        }
        W.gr[[i]] <- W.var
        if(ngr > 1) spear.gr[[i]] <- spear.mean
    }
    ##CC# Calculate P-values
    for(i in 1:ngr) {
        counter[[i]] <- counter[[i]]/(nperm+1)
    }

    ## Correction to P-values for multiple testing
    ## Write all P-values to a long vector 'vec'
    vec <- counter[[1]]
    if(ngr > 1) {
        for(i in 2:ngr) vec = c(vec, counter[[i]])
    }
    if(length(vec) != p) stop("error in putting together vector 'vec'")

    if(mult == "sidak") {
        vec.corr = NA
        for(i in 1:p) vec.corr = c(vec.corr, (1-(1-vec[i])^p))
        vec.corr <- vec.corr[-1]
    } else {
        vec.corr <- p.adjust(vec, method=mult)
    }

    if(ngr > 1) {
        vec.gr <- vector("list",ngr)
        end <- 0
        for(i in 1:ngr){
            beg <- end+1
            end <- end + n.per.gr[i]
            vec.gr[[i]] <- vec.corr[beg:end]
        }
    }
    ## Create data frames containing the results
    if(ngr == 1) {
        table <- rbind(spear.mean, W.var, counter[[1]], vec.corr)
        rownames(table) <- c("Spearman.mean", "W.per.species", "Prob", "Corrected prob")
    } else {
        table <- as.list(1:ngr)
        for(i in 1:ngr) {
            table[[i]] <- rbind(spear.gr[[i]], W.gr[[i]], counter[[i]], vec.gr[[i]])
            rownames(table[[i]]) <- c("Spearman.mean", "W.per.species", "Prob", "Corrected prob")
            ## PL: Next line had been lost
            colnames(table[[i]]) <- colnames(table[[i]], do.NULL = FALSE,
                                             prefix = "Spec")
        }
    }
    if(ngr == 1) {
        out <- list(A_posteriori_tests=table, Correction.type=mult)
    } else {
        out <- list(A_posteriori_tests_Group=table, Correction.type=mult)
    }
                                        #
    class(out) <- "kendall.post"
    out
}

`TukeyHSD.betadisper` <- function(x, which = "group", ordered = FALSE,
                                  conf.level = 0.95, ...) {
    df <- data.frame(distances = x$distances, group = x$group)
    mod.aov <- aov(distances ~ group, data = df)
    TukeyHSD(mod.aov, which = which, ordered = ordered,
             conf.level = conf.level, ...)
}

% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
%\VignetteIndexEntry{Introduction to ordination in vegan}
\documentclass[a4paper,10pt]{article}
\usepackage{vegan} % vegan settings

\title{Vegan: an introduction to ordination}
\author{Jari Oksanen}

\date{\footnotesize{
  processed with vegan
\Sexpr{packageDescription("vegan", field="Version")}
in \Sexpr{R.version.string} on \today}}

%% need no \usepackage{Sweave}
\begin{document}

\SweaveOpts{strip.white=true}
<<echo=false>>=
par(mfrow=c(1,1))
options(width=72)
figset <- function() par(mar=c(4,4,1,1)+.1)
options(SweaveHooks = list(fig = figset))
options("prompt" = "> ", "continue" = "  ")
@

\maketitle
\begin{abstract}
  The document describes typical, simple work pathways of
  vegetation ordination. Unconstrained ordination uses as examples
  detrended correspondence analysis and non-metric multidimensional
  scaling, and shows how to interpret their results by fitting
  environmental vectors and factors or smooth environmental surfaces
  to the graph. The basic plotting command, and more advanced plotting
  commands for congested plots are also discussed, as well as adding
  items such as ellipses, convex hulls, and other items for
  classes. The constrained ordination uses constrained (canonical)
  correspondence analysis as an example. It is first shown how a model
  is defined, then the document discusses model building and
  significance tests of the whole analysis, single constraints and
  axes.
\end{abstract}
\tableofcontents

\vspace{3ex}
\noindent \pkg{Vegan} is a package for community ecologists.  This
documents explains how the commonly used ordination methods can be
performed in \pkg{vegan}.  The document only is a very basic
introduction.
%Another document (\emph{vegan tutorial})
%(\url{http://cc.oulu.fi/~jarioksa/opetus/method/vegantutor.pdf}) gives
%a longer and more detailed introduction to ordination.
The
current document only describes a small part of all \pkg{vegan}
functions.  For most functions, the canonical references are the
\pkg{vegan} help pages.

\section{Ordination}

The \pkg{vegan} package contains all common ordination methods:
Principal component analysis (functions \code{pca} and \code{rda}, or
\code{prcomp} in the base \proglang{R}), correspondence analysis
(\code{ca}, \code{cca}), detrended correspondence analysis
(\code{decorana}), metric scaling, also known as principal coordinate
analysis (\code{pco}, \code{wcmdscale}, or \code{cmdscale} in base
\proglang{R}), non-metric multidimensional scaling (\code{monoMDS})
with wrapper for common pipeline of use (\code{metaMDS}).  Functions
\code{rda} and \code{cca} mainly are designed for constrained
ordination, and will be discussed later.  In this chapter I describe
functions \code{decorana} and \code{metaMDS}.

\subsection{Detrended correspondence analysis}

Detrended correspondence analysis (\textsc{dca}) is done like this:
<<>>=
library(vegan)
data(dune)
ord <- decorana(dune)
@
This saves ordination results in \code{ord}:
<<>>=
ord
@

The display of results is very brief: only eigenvalues and used
options are listed.  Actual ordination results are not shown, but you
can extract them with command \code{scores(ord)}.  The \code{plot}
function also automatically knows how to access the scores.

\subsection{Non-metric multidimensional scaling}


Function \code{metaMDS} is a bit special case.  The actual ordination
is performed by \pkg{vegan} function \code{monoMDS}.  Function
\code{metaMDS} is a wrapper to perform non-metric multidimensional
scaling (\textsc{nmds}) like recommended in community ordination: it
uses adequate dissimilarity measures (function \code{vegdist}), then
it runs \textsc{nmds} several times with random starting
configurations, compares results (function \code{procrustes}), and
stops after finding twice a similar minimum stress solution.  Finally
it scales and rotates the solution, and adds species scores to the
configuration as weighted averages (function \code{wascores}):
<<>>=
ord <- metaMDS(dune, trace = FALSE)
ord
@

\section{Ordination graphics}

Ordination is nothing but a way of drawing graphs, and it is best to
inspect ordinations only graphically (which also implies that they
should not be taken too seriously).

All ordination results of \pkg{vegan} can be displayed with a
\code{plot} command (Fig. \ref{fig:plot}):
<<a>>=
plot(ord)
@
\begin{figure}
<<fig=true,echo=false>>=
<<a>>
@
\caption{Default ordination plot.}
\label{fig:plot}
\end{figure}
Default \code{plot} command uses either black circles for sites and
red pluses for species, or black and red text for sites and species,
resp.  The choices depend on the number of items in the plot and
ordination method.  You can override the default choice by setting
\code{type = "p"} for points, or \code{type = "t"} for text.  For
a better control of ordination graphics you can first draw an empty
plot (\code{type = "n"}) and then add species and sites separately
using \code{points} or \code{text} functions.  In this way you can
combine points and text, and you can select colours and character
sizes freely (Fig. \ref{fig:plot.args}). The easiest way is to build
the plot by layers using pipe (\code{|>}).
<<a>>=
plot(ord, type = "n") |>
    points("sites", cex = 0.8, pch=21, col="red", bg="yellow") |>
    text("species", cex=0.7, col="blue")
@ 

\begin{figure}
<<fig=true, echo=false>>=
<<a>>
@ 
 \caption{A more colourful ordination plot where sites are points and
   species are text.}
 \label{fig:plot.args}
 \end{figure}

 All \pkg{vegan} ordination methods have a specific \code{plot}
 function.  In addition, \pkg{vegan} has an alternative plotting
 function \code{ordiplot} that also knows many non-\pkg{vegan}
 ordination methods, such as \code{prcomp} and \code{cmdscale}.  All
 \pkg{vegan} plot functions return invisibly an \code{ordiplot}
 object, so that you can use \code{ordiplot} support functions with
 the results (\code{points}, \code{text}, \code{identify}).

Alternative plotting methods are available in two packages:

\begin{itemize}

  \item \pkg{vegan3d} on CRAN provides static 3D plots
      (\code{ordiplot3d}), dynamic 3D plots that can be spinned around
      or zoomed (\code{ordirgl}) and interactive 2D plots that can be
      edited moving point labels to better position (\code{orditkplot}).

    \item \pkg{ggvegan} on CRAN provides \pkg{ggplot2} graphics for
      most \pkg{vegan} objects. The \pkg{vegan} \code{scores}
      functions for ordination objects (and some others) recognize
      argument \code{tidy} which can be used to extract scores for
      \pkg{ggplot2} graphics.  (There are some \pkg{lattice} graphics
      functions in \pkg{vegan}, but the plan is to phase out these in
      favour of \pkg{ggvegan} graphics.)

\end{itemize}

\subsection{Cluttered plots}

Ordination plots are often congested: there is a large number of sites
and species, and it may be impossible to display all clearly.  In
particular, two or more species may have identical scores and are
plotted over each other.  Here some methods you can try:
\begin{itemize}
  
  \item Use only \code{points} if you do not need to identify the item
    (and function \code{identify} can be used to add labels some of
    the points).
    
  \item Most \pkg{vegan} \code{plot} and \code{text} functions know
    arguments \code{optimize} and \code{bg}.  With
    \code{optimize=TRUE} the exact scores are shown with labelled
    points, and the labels are positioned to minimize
    over-plotting. Argument \code{bg} gives the background colour of
    labels. With background you cannot see the text or points below
    the label, but you can read at least the uppermost text. These
    arguments can be used together. These two arguments often help
    with moderate cluttering.
  
  \item Zoom into graph setting axis limits \code{xlim} and
    \code{ylim}.  You must typically set both, because \pkg{vegan}
    will maintain equal aspect ratio of axes.

  \item Use points and add label only to some points with
    \code{identify} command.

  \item Use \code{select} argument in ordination \code{text} and
    \code{points} functions to only show the specified items, possibly
    combined with argument \code{labels} for shorter names.

  \item Use automatic \code{orditorp} function that uses text only if
    this can be done without overwriting previous labels, but points
    in other cases.

  \item Use interactive \code{orditkplot} function in \pkg{vegan3d}
    that draws both points and labels for ordination scores, and
    allows you to drag labels to better positions. You can export the
    edited graph in several graphical formats, or return the edited
    positions to \proglang{R} for further processing.

\end{itemize}

\subsection{Adding items to ordination plots}

\pkg{Vegan} has a group of functions for adding information about
classification or grouping of points onto ordination diagrams.
Function \code{ordihull} adds convex hulls, \code{ordiellipse} adds
ellipses enclosing all points in the group (ellipsoid hulls) or
ellipses of standard deviation, standard error or confidence areas,
\code{ordibar} draws a cross corresponding to the principal axes of
ellipses, and \code{ordispider} combines items to their centroid
(Fig. \ref{fig:ordihull}):
<<>>=
data(dune.env)
attach(dune.env)
@
<<a>>=
plot(ord, disp="sites", type="n")
ordihull(ord, Management, col=1:4, lwd=3)
ordiellipse(ord, Management, col=1:4, kind = "ehull", lwd=3)
ordiellipse(ord, Management, col=1:4, draw="polygon")
ordispider(ord, Management, col=1:4, label = TRUE)
points(ord, disp="sites", pch=21, col="red", bg="yellow", cex=1.3)
@
\begin{figure}
<<fig=true,echo=false>>=
<<a>>
@
\caption{Convex hull, ellipsoid hull, standard error ellipse and a spider web diagram
  for Management levels in ordination.}
\label{fig:ordihull}
\end{figure}
In addition, you can overlay a cluster dendrogram from \code{hclust}
using \code{ordicluster} or a minimum spanning tree from
\code{spantree} with its \code{lines} function.  Segmented arrows
can be added with \code{ordiarrows}, lines with
\code{ordisegments} and regular grids with \code{ordigrid}.

\section{Fitting environmental variables}

\pkg{Vegan} provides two functions for fitting environmental
variables onto ordination:
\begin{itemize}
\item \code{envfit} fits vectors of continuous variables and centroids
  of levels of class variables (defined as \code{factor} in
  \proglang{R}).  The arrow shows the direction of the (increasing)
  gradient, and the length of the arrow is proportional to the
  correlation between the variable and the ordination.
\item \code{ordisurf} (which requires package \pkg{mgcv}) fits
  smooth surfaces for continuous variables onto ordination using
  thinplate splines with cross-validatory selection of smoothness.
\end{itemize}

Function \code{envfit} can be called with a \code{formula}
interface, and it optionally can assess the ``significance'' of the
variables using permutation tests:
<<>>=
ord.fit <- envfit(ord ~ A1 + Management, data=dune.env, perm=999)
ord.fit
@
The result can be drawn directly or added to an ordination diagram
(Fig. \ref{fig:envfit}):
<<a>>=
plot(ord, dis="site")
plot(ord.fit, bg = "yellow")
@

Function \code{ordisurf} directly adds a fitted surface onto
ordination, and it returns the result of the fitted \code{gam}
(Fig. \ref{fig:envfit}):
<<b,fig=false>>=
ordisurf(ord, A1, add=TRUE)
@
\begin{figure}
<<fig=true,echo=false,results=hide>>=
<<a>>
<<b>>
@
\caption{Fitted vector and smooth surface for the thickness of A1
  horizon (\code{A1}, in cm), and centroids of Management levels.}
\label{fig:envfit}
\end{figure}

\section{Constrained ordination}

\pkg{Vegan} has three methods of constrained ordination:
constrained or ``canonical'' correspondence analysis (function
\code{cca}), redundancy analysis (function \code{rda}) and
distance-based redundancy analysis (function \code{dbrda}).  All
these functions can have a conditioning term that is ``partialled
out''.  I only demonstrate \code{cca}, but all functions accept
similar commands and can be used in the same way.

The preferred way is to use \code{formula} interface, where the left
hand side gives the community data frame and the right hand side lists
the constraining variables:
<<>>=
ord <- cca(dune ~ A1 + Management, data=dune.env)
ord
@
The results can be plotted with (Fig. \ref{fig:cca}):
<<a>>=
plot(ord, spe.par = list(optimize = TRUE), sit.par = list(type="p"))
@
\begin{figure}
<<fig=true,echo=false>>=
<<a>>
@
\caption{Plot of constrained correspondence analysis showing sites as
  points and with optimized location for species labels.}
\label{fig:cca}
\end{figure}
There are three groups of items: sites, species and centroids (and
biplot arrows) of environmental variables.  All these can be added
individually to an empty plot with pipes, and all previously explained tricks of
controlling graphics still apply. If only small changes are wanted to
default settings, it is easier to change those parameters with a list
of new argument values like in the example.

It is not recommended to perform constrained ordination with all
available environmental variables: adding the number of
constraints means slacker constraint, and you finally end up with
solution similar to unconstrained ordination. In that case it is
better to use unconstrained ordination with environmental fitting.
However, if you really want to do so, it is possible with the
following shortcut in \code{formula}:
<<>>=
cca(dune ~ ., data=dune.env)
@

The model gave a message that some constraints were aliased because
they were redundant. This means that the variable did not have unique
explanatory power, but it can be expressed with the help of other
variables. Such redundant variables are not shown in ordination. In
this case fourth degree polynomial of \code{Manure} (an ordered
factor) was redundant ana aliased. There is one \code{Manure} level
(0) which only occurs in \code{Management} level \code{NM} (natural
management), and we know that \code{Manure} level once we know the
\code{Management}.

\subsection{Significance tests}

\pkg{vegan} provides permutation tests for the significance of
constraints.  The test mimics standard analysis of variance function
(\code{anova}), and the default test analyses all constraints
simultaneously:
<<>>=
anova(ord)
@
The function actually used was \code{anova.cca}, but you do not need
to give its name in full, because \proglang{R} automatically chooses the
correct \code{anova} variant for the result of constrained
ordination.

It is also possible to analyse terms separately:
<<>>=
anova(ord, by="term")
@
This test is sequential: the terms are analysed
in the order they happen to be in the model. You can also analyse
significances of marginal effects (``Type III effects''):
<<>>=
anova(ord, by="margin")
@

\subsection{Conditioned or partial ordination}

All constrained ordination methods can have terms that are partialled
out from the analysis before constraints:
<<>>=
ord <- cca(dune ~ A1 + Management + Condition(Moisture), data=dune.env)
ord
@
This partials out the effect of \code{Moisture} before analysing the
effects of \code{A1} and \code{Management}.  This also influences
the significances of the terms:
<<>>=
anova(ord, by="term")
@
If we had a designed experiment, we may wish to restrict the
permutations so that the observations only are permuted within levels
of \code{Moisture}. Restricted permutation is based on the powerful
\pkg{permute} package. Function \code{how()} can be used to define
permutation schemes. In the following, we set the levels with
\code{plots} argument:
<<>>=
how <- how(nperm=999, plots = Plots(strata=dune.env$Moisture))
anova(ord, by="term", permutations = how)
@

%%%%%%%%%%%%%%%%%%%
<<echo=false>>=
detach(dune.env)
@

\end{document}

%\VignetteIndexEntry{Partition of Variation}
%% This file rewrites Pierre Legendre's introduction and takes pages
%% of Pierre Legendre's pdf documents and puts them together.

\documentclass[10pt]{article}
\usepackage{vegan} %% vegan setup
\usepackage{pdfpages}
\setkeys{Gin}{width=0.6\linewidth}


\title{Diagrams and Procedures for Partition of Variation}
\author{Pierre Legendre}
\date{\footnotesize{
  processed with vegan \Sexpr{packageDescription("vegan", field="Version")}
  in \Sexpr{R.version.string} on \today}}

\begin{document}
%% Sweave document
\SweaveOpts{strip.white=true}
<<echo=false>>=
par(mfrow=c(1,1))
figset <- function() par(mar=c(4,4,1,1)+.1)
options(SweaveHooks = list(fig = figset))
library(vegan)
labs <- paste("Table", 1:4)
cls <- c("hotpink", "skyblue", "orange", "limegreen")
@

\maketitle

\noindent Diagrams describing the partitions of variation of a
response data table by two (Fig.~\ref{fig:part2}), three
(Fig.~\ref{fig:part3}) and four tables (Fig.~\ref{fig:part4}) of
explanatory variables. The fraction names [a] to [p] in the output of
\code{varpart} function follow the notation in these Venn diagrams,
and the diagrams were produced using the \code{showvarparts} function.
%%%%%%%%%%%%%%%
\begin{figure}[!ht]
<<echo=FALSE,fig=TRUE>>=
showvarparts(2, bg = cls, Xnames=labs)
@ 
\caption{3 regression/ canonical analyses and 3 subtraction equations
  are needed to estimate the $4\;(=2^2)$ fractions. 
  
  [a] and [b] can be tested for significance (3 canonical analyses per
  permutation). Fraction [c] cannot be tested singly.}
\label{fig:part2}
\end{figure}
%%%%%%%%%%%
\begin{figure}[!ht]
<<echo=FALSE,fig=TRUE>>=
showvarparts(3, bg = cls, Xnames=labs)
@ 
\caption{7 regression/ canonical analyses and 10 subtraction equations
  are needed to estimate the $8\;(=2^3)$ fractions. 
  
  [a] to [c] and subsets containing [a] to [c] can be tested for
  significance (4 canonical analyses per permutation to test [a] to
  [c]). Fractions [d] to [g] cannot be tested singly.}
\label{fig:part3}
\end{figure}
%%%%%%%%%%%
\begin{figure}[!ht]
<<echo=FALSE,fig=TRUE>>=
showvarparts(4, bg = cls, Xnames=labs)
@ 
\caption{15 regression/ canonical analyses and 27 subtraction equations
  are needed to estimate the $16\;(=2^4)$ fractions. 
  
  [a] to [d] and subsets containing [a] to [d] can be tested for
  significance (5 canonical analyses per permutation to test [a] to
  [d]). Fractions [e] to [o] cannot be tested singly.}
\label{fig:part4}
\end{figure}
\clearpage
\setkeys{Gin}{width=\paperwidth}
%% Add partitioning models 2-3 and 4.
\includepdf[fitpaper=true,pages=-]{varpart23.pdf}
\includepdf[fitpaper=true, pages=-]{varpart4.pdf}
\end{document}

<!--
%\VignetteIndexEntry{vegan FAQ}
%\VignetteEngine{knitr::knitr}
%\VignetteEncoding{UTF-8}
-->

**vegan** FAQ
=============

This document contains answers to some of the most frequently asked
questions about R package **vegan**.

> This work is licensed under the Creative Commons Attribution 3.0
> License.
>
> Copyright © 2008-2025 vegan development team

------------------------------------------------------------------------

Introduction
------------

------------------------------------------------------------------------

### What is **vegan**?

**Vegan** is an R package for community ecologists. It contains the
most popular methods of multivariate analysis needed in analysing
ecological communities, and tools for diversity analysis, and other
potentially useful functions. **Vegan** is not self-contained but it
must be run under R statistical environment, and it also depends on
many other R packages. **Vegan** is distributed under GPL2 license.

------------------------------------------------------------------------


### How to obtain **vegan** and R?

Both R and latest release version of **vegan** can be obtained through
[CRAN](https://cran.r-project.org). Unstable development version of
**vegan** can be obtained through
[GitHub](https://github.com/vegandevs/vegan). The github page gives
further instructions for obtaining and installing development versions
of **vegan**.

------------------------------------------------------------------------

### What other packages are available for ecologists?

CRAN [Task Views](https://cran.r-project.org/web/views/) include
entries like `Environmetrics`, `Multivariate` and `Spatial` that
describe several useful packages and functions. If you install R package
**ctv**, you can inspect Task Views from your R session, and
automatically install sets of most important packages.

------------------------------------------------------------------------

### What other documentation is available for **vegan**?

**Vegan** is a fully documented R package with standard help pages.
These are the most authoritative sources of documentation (and as a
last resource you can use the force and the read the source, as
**vegan** is open source). **Vegan** package ships with other
documents which can be read with `browseVignettes("vegan")`
command. The documents included in the **vegan** package are

-   **Vegan** `NEWS` that can be accessed via `news()` command.
-   This document (`FAQ-vegan`).
-   Short introduction to basic ordination methods in **vegan**
    (`intro-vegan`).
-   Introduction to diversity methods in **vegan**
    (`diversity-vegan`).
-   Discussion on design decisions in **vegan** (`decision-vegan`).
-   Description of variance partition procedures in function `varpart`
    (`partitioning`).

Web documents outside the package include:

-   <https://github.com/vegandevs/vegan>: development page.
-   <https://vegandevs.github.io/vegan/>: **vegan** homepage.

------------------------------------------------------------------------

### Is there a Graphical User Interface (GUI) for **vegan**?

Roeland Kindt has made package **BiodiversityR** which provides a GUI
for **vegan**. The package is available at
[CRAN](https://cran.r-project.org/package=BiodiversityR).
It is not a mere GUI for **vegan**, but adds some new functions and
complements **vegan** functions in order to provide a workbench for
biodiversity analysis. You can install **BiodiversityR** using
`install.packages("BiodiversityR")` or graphical package management menu
in R. The GUI works on Windows, MacOS X and Linux.

------------------------------------------------------------------------

### How to cite **vegan**?

Use command `citation("vegan")` in R to see the recommended citation to
be used in publications.

------------------------------------------------------------------------

### How to build **vegan** from sources?

In general, you do not need to build **vegan** from sources, but binary
builds of release versions are available through
[CRAN](https://cran.r-project.org/) for Windows and MacOS X. If you use
some other operating systems, you may have to use source packages.
**Vegan** is a standard R package, and can be built like instructed in R
documentation. **Vegan** contains source files in C and FORTRAN, and you
need appropriate compilers (which may need more work in Windows and
MacOS X).

------------------------------------------------------------------------

### Are there binaries for devel versions?

Binaries can be available from R Universe: see
<https://github.com/vegandevs/vegan> for instructions.

------------------------------------------------------------------------

### How to report a bug in **vegan**?

If you think you have found a bug in **vegan**, you should report it to
**vegan** maintainers or developers. The preferred forum to report bugs
is [GitHub](https://github.com/vegandevs/vegan/issues). The bug report
should be so detailed that the bug can be replicated and corrected.
Preferably, you should send an example that causes a bug. If it needs a
data set that is not available in R, you should send a minimal data set
as well. You also should paste the output or error message in your
message. You also should specify which version of **vegan** you used.

Bug reports are welcome: they are the only way to make **vegan**
non-buggy.

Please note that you shall not send bug reports to R mailing lists,
since **vegan** is not a standard R package.

------------------------------------------------------------------------

### Is it a bug or a feature?

It is not necessarily a bug if some function gives different results
than you expect: That may be a deliberate design decision. It may be
useful to check the documentation of the function to see what was the
intended behaviour. It may also happen that function has an argument to
switch the behaviour to match your expectation. For instance, function
`vegdist` always calculates quantitative indices (when this is
possible). If you expect it to calculate a binary index, you should use
argument `binary = TRUE`.

------------------------------------------------------------------------

### Can I contribute to **vegan**?

**Vegan** is dependent on user contribution. All feedback is welcome. If
you have problems with **vegan**, it may be as simple as incomplete
documentation, and we shall do our best to improve the documents.

Feature requests also are welcome, but they are not necessarily
fulfilled. A new feature will be added if it is easy to do and it looks
useful, or if you submit code.

If you can write code yourself, the best forum to contribute to vegan is
[GitHub](https://github.com/vegandevs/vegan).

------------------------------------------------------------------------

Ordination
----------

------------------------------------------------------------------------

### I have only numeric and positive data but **vegan** still complains

You are wrong! Computers are painfully pedantic, and if they find
non-numeric or negative data entries, you really have them. Check your
data! Most common reasons for non-numeric data are that row names were
read as a non-numeric variable instead of being used as row names
(check argument `row.names` in reading the data), or that the column
names were interpreted as data (check argument `header = TRUE` in
reading the data). Another common reason is that you had empty cells
in your input data, and these were interpreted as missing values. See
also the next question.

------------------------------------------------------------------------

### Can I use tibbles in ordination?

Yes. In most cases they behave like normal data frames. For
environmental data there should be no problem with tibbles. Problems
can emerge with community data. Community data frames must be numeric,
but if tibble has character variables (such as a column of row names),
data are non-numeric with guaranteed failure in ordination (see
previous question). All character columns should be removed from
community data in ordination.

------------------------------------------------------------------------

### Can I analyse binary or cover class data?

Yes. Most **vegan** methods can handle binary data or cover abundance
data. Most statistical tests are based on permutation, and do not make
distributional assumptions. There are some methods (mainly in diversity
analysis) that need count data. These methods check that input data are
integers, but they may be fooled by cover class data.

------------------------------------------------------------------------

### Why dissimilarities in **vegan** differ from other sources?

Most commonly the reason is that other software use presence–absence
data whereas **vegan** used quantitative data. Usually **vegan** indices
are quantitative, but you can use argument `binary = TRUE` to make them
presence–absence. However, the index name is the same in both cases,
although different names usually occur in literature. For instance,
Jaccard index actually refers to the binary index, but **vegan** uses
name `"jaccard"` for the quantitative index, too.

Another reason may be that indices indeed are defined differently,
because people use same names for different indices.

------------------------------------------------------------------------

### I cannot get repeated solutions in `metaMDS`

The first (try 0) run of `metaMDS` starts from the metric scaling
solution and is usually good, and most sofware only return that
solution. However, `metaMDS` tries to see if that standard solution
can be repeated, or improved and the improved solution still
repeated. In all cases, it will return the best solution found, and
there is no burning need to do anything if you get the message that the
solution could not be repeated. If you are keen to know that the
solution really is the global optimum, you may follow the instructions
in the `metaMDS` help section "Results Could Not Be Repeated" and try
harder.

-------------------------------------------------------------------------

### I get zero stress but no repeated solutions in `metaMDS`

Most common reason is that you have too few observations for your
NMDS.  For $n$ observations (points) and $k$ dimensions you need to
estimate $nk$ parameters (ordination scores) using $n(n-1)/2$
dissimilarities.  For $k$ dimensions you must have $n > 2k + 1$, or
for two dimensions at least six points. In some degenerate situations
you may need even a larger number of points. If you have a lower
number of points, you can find an undefined number of perfect (stress
is zero) but different solutions. Conventional wisdom due to Kruskal
is that you should have $n > 4k + 1$ points for $k$ dimensions. A
typical symptom of insufficient data is that you have (nearly) zero
stress but no repeated solutions. In those cases you should reduce the
number of dimensions ($k$) and with very small data sets you should
not use `NMDS`, but rely on metric methods (`pco` and `wcmdscale` in
**vegan**, `cmdscale` in base **R**).

------------------------------------------------------------------------

### Zero dissimilarities in isoMDS

Function `metaMDS` uses function `monoMDS` as its default method for
NMDS, and this function can handle zero dissimilarities. Alternative
function `isoMDS` cannot handle zero dissimilarities. If you want to use
`isoMDS`, you can use argument `zerodist = "add"` in `metaMDS` to handle
zero dissimilarities. With this argument, zero dissimilarities are
replaced with a small positive value, and they can be handled in
`isoMDS`. This is a kluge, and some people do not like this. A more
principal solution is to remove duplicate sites using R command
`unique`. However, after some standardizations or with some
dissimilarity indices, originally non-unique sites can have zero
dissimilarity, and you have to resort to the kluge (or work harder with
your data). Usually it is better to use `monoMDS`.

------------------------------------------------------------------------

### I have heard that you cannot fit environmental vectors or surfaces to NMDS results which only have rank-order scores

Claims like this have indeed been at large in the Internet, but they
are based on grave misunderstanding and are plainly wrong. NMDS
ordination results are strictly metric, and in **vegan** `metaMDS` and
`monoMDS` they are even strictly Euclidean. The method is called
“non-metric” because there is non-metric relation from input
dissimilarities to the Euclidean ordination space. You can inspect
this non-linear step function using function `stressplot` in
**vegan**.  Because the ordination scores are strictly Euclidean, it
is correct to use **vegan** functions `envfit` and `ordisurf` with
NMDS results.

------------------------------------------------------------------------

### Why ordiellipses are much smaller than ggplot2 ellipses?

**vegan** `ordiellipse` and **ggplot2** `stat_ellipse` use nearly
identical methods, but there are three differences. In order of
importance: (1) Greatest difference is that they are different
ellipses. **Vegan** defaults to show standard deviation ellipses, but
**ggplot2** shows data ellipses. Data ellipses are intended to enclose
a given proportion of points. `ordiellipse` can also show data
ellipses with its `conf` argument. Setting `ordiellipse(...,
kind="sd", conf=0.95)` is similar to default **ggplot2** ellipse, and
setting `level=0.39` in `stat_ellipse` will give a reasonable good
approximation of SD ellipses. (2) **Vegan** uses standard
`stats::cov.wt` function to find the covariance matrices, whereas
**ggplot2** defaults to robust covariances with `MASS::cov.trob`
(which is not (yet?) implemented in **vegan**).  Setting `type="norm"`
in `ggplot2::stat_ellipse` will use the same covariance matrix as
`ordiellipse`. The use of robust covariances will influence both the
shape and location of ellipses. (3) `ordiellipse` uses asymptotic
confidence limits based on Chi-squared distribution whereas
`stat_ellipse` uses _F_-statistic based on bivariate Normal
distribution which takes into account group sizes and gives wider
radius for smaller groups. The differences are usually small, and
large groups are very similar.

------------------------------------------------------------------------

### Where can I find numerical scores of ordination axes?

Normally you can use function `scores` to extract ordination scores for
any ordination method. The `scores` function can also find ordination
scores for many non-**vegan** functions such as for `prcomp` and
`princomp` and for some **ade4** functions.

In some cases the ordination result object stores raw scores, and the
axes are also scaled appropriate when you access them with `scores`. For
instance, in `cca` and `rda` the ordination object has only so-called
normalized scores, and they are scaled for ordination plots or for other
use when they are accessed with `scores`.

------------------------------------------------------------------------

### How the RDA results are scaled?

The scaling or RDA results indeed differ from most other software
packages. The scaling of RDA is such a complicated issue that it cannot
be explained in this FAQ, but it is explained in a separate pdf document
on “Design decision and implementation details in vegan” that you can
read with command `browseVignettes("vegan")`.

------------------------------------------------------------------------

### Variance explained by ordination axes.

In general, **vegan** does not directly give any statistics on the
“variance explained” by ordination axes or by the constrained axes. This
is a design decision: I think this information is normally useless and
often misleading. In community ordination, the goal typically is not to
explain the variance, but to find the “gradients” or main trends in the
data. The “total variation” often is meaningless, and all proportions of
meaningless values also are meaningless. Often a better solution
explains a smaller part of “total variation”. For instance, in
non-standardized principal components analysis most of the variance is
generated by a small number of most abundant species, and they are easy
to “explain” because data really are not very multivariate. If you
standardize your data, all species are equally important. The first axes
explains much less of the “total variation”, but now they explain all
species equally, and results typically are much more useful for the
whole community. Correspondence analysis uses another measure of
variation (which is not variance), and again it typically explains a
“smaller proportion” than principal components but with a better result.
Detrended correspondence analysis and nonmetric multidimensional scaling
even do not try to “explain” the variation, but use other criteria. All
methods are incommensurable, and it is impossible to compare methods
using “explanation of variation”.

If you still want to get “explanation of variation” (or a deranged
editor requests that from you), it is possible to get this information
for some methods:

- Eigenvector methods: Functions `rda`, `cca`, `dbrda` and `capscale`
  give the variation of conditional (partialled), constrained
  (canonical) and residual components.  Function `eigenvals` extracts
  the eigenvalues, and `summary(eigenvals(ord))` reports the
  proportions explained in the result object `ord`, and also works
  with `decorana` and `wcmdscale`. Function `RsquareAdj` gives the
  R-squared and adjusted R-squared (if available) for constrained
  components.  Function `goodness` gives the same statistics for
  individual species or sites.  In addition, there is a special
  function `varpart` for unbiased partitioning of variance between up
  to four separate components.

- Nonmetric multidimensional scaling. NMDS is a method for
  nonlinear mapping, and the concept of of variation explained does
  not make sense. However, 1 - stress\^2 transforms nonlinear stress
  into quantity analogous to squared correlation coefficient. Function
  `stressplot` displays the nonlinear fit and gives this statistic.

------------------------------------------------------------------------

### Can I have random effects in constrained ordination or in `adonis`?

No. Strictly speaking, this is impossible. However, you can define
models that respond to similar goals as random effects models, although
they strictly speaking use only fixed effects.

Constrained ordination functions `cca`, `rda` and `dbrda` can have
`Condition()` terms in their formula. `Condition()` defines partial
terms that are fitted before other constraints and can be used to remove
the effects of background variables, and their contribution to
decomposing inertia (variance) is reported separately. These partial
terms are often regarded as similar to random effects, but they are
still fitted in the same way as other terms and strictly speaking they
are fixed terms.

Function `adonis2` can evaluate terms sequentially. In a model with
right-hand-side `~ A + B` the effects of `A` are evaluated first, and
the effects of `B` after removing the effects of `A`. Sequential tests
are also available in `anova` function for constrained ordination
results by setting argument `by = "term"`. In this way, the first terms
can serve in a similar role as random effects, although they are fitted
in the same way as all other terms, and strictly speaking they are fixed
terms.

All permutation tests in **vegan** are based on the **permute** package
that allows constructing various restricted permutation schemes. For
instance, you can set levels of `plots` or `blocks` for a factor
regarded as a random term.

A major reason why real random effects models are impossible in most
**vegan** functions is that their tests are based on the permutation of
the data. The data are given, that is fixed, and therefore permutation
tests are basically tests of fixed terms on fixed data. Random effect
terms would require permutations of data with a random component instead
of the given, fixed data, and such tests are not available in **vegan**.

------------------------------------------------------------------------

### Is it possible to have passive points in ordination?

**Vegan** does not have a concept of passive points, or a point that
should only little influence the ordination results. However, you can
add points to eigenvector methods using `predict` functions with
`newdata`. You can first perform an ordination without some species or
sites, and then you can find scores for all points using your complete
data as `newdata`. The `predict` functions are available for basic
eigenvector methods in **vegan** (`cca`, `rda`, `decorana`, for an
up-to-date list, use command `methods("predict")`).

You can add new points NMDS with function `MDSaddpoints`.

------------------------------------------------------------------------

### Class variables and dummies

You should define a class variable as an R `factor`, and **vegan** will
automatically handle them.

R (and **vegan**) knows both unordered and ordered factors. Unordered
factors are internally coded as dummy variables, but one redundant level
is removed or aliased. With default contrasts, the removed level is the
first one. Ordered factors are expressed as polynomial contrasts. Both
of these contrasts are explained in standard **R** documentation.

------------------------------------------------------------------------

### How are environmental arrows scaled?

The printed output of `envfit` gives the direction cosines which are the
coordinates of unit length arrows. For plotting, these are scaled by
their correlation (square roots of column `r2`). You can see the scaled
lengths of `envfit` arrows using command `scores`.

The scaled environmental vectors from `envfit` and the arrows for
continuous environmental variables in constrained ordination (`cca`,
`rda`, `dbrda`) are adjusted to fill the current graph. The lengths
of arrows do not have fixed meaning with respect to the points (species,
sites), but they can only compared against each other, and therefore
only their relative lengths are important.

If you want change to the scaling of the arrows, you can use `text`
(plotting arrows and text) or `points` (plotting only arrows) functions
for constrained ordination. These functions have argument `arrow.mul`
which sets the multiplier. The `plot` function for `envfit` also has the
`arrow.mul` argument to set the arrow multiplier.

------------------------------------------------------------------------

### I cannot see all variables in constrained ordination

You have some variables that have no use and information. These
variables are correlated with other constraining variables and have no
independent information. This can also concern only some levels of
factor variables. For instance, in **vegan** data `dune.env` we know
that `Manure` level is 0 in `Management` level `NM` and only there. If
both are used as constraints (or conditions), one level of `Manure` or
one level of `Management` is redundant. These variables or levels are
aliased, and **vegan** informs about this during fitting the model and
permanently when the short information of the result is printed.

**Vegan** function `alias` gives the defining equations for aliased
variables, or optionally their names.

------------------------------------------------------------------------

### Plotting aliased variables

You can fit vectors or class centroids for aliased variables using
`envfit` function. The `envfit` function uses weighted fitting, and
the fitted vectors are identical to the vectors in correspondence
analysis. In constrained analysis you must fit the centroids or
vectors to the constrained scores (LC scores), and use argument
`display = "lc"` in `envfit`.

------------------------------------------------------------------------

### Restricted permutations in **vegan**

**Vegan** uses **permute** package in all its permutation tests. The
**permute** package will allow restricted permutation designs for time
series, line transects, spatial grids and blocking factors. The
construction of restricted permutation schemes is explained in the
manual page `permutations` in **vegan** and in the documentation of the
**permute** package.

------------------------------------------------------------------------

### Why restricted permutation does not influence adonis results?

The permutation scheme influences the permutation distribution of the
statistics and significance levels, but does not influence the
calculation of the statistics.

------------------------------------------------------------------------

### How to use different plotting symbols in ordination graphics?

Ordination objects typically have many different kind of scores, and
there may be limitations of changing the way how they are
displayed. However, `plot` for constrained ordination results (`cca`,
`rda`, `dbrda`) can be fully configured since **vegan** 2.7-1. See
their help. Sometimes it is easier to compose the graph by layers
using pipes (`|>`) to add `text` or `points` of different scores. See
the help of ordination plot functions for examples.

------------------------------------------------------------------------

### How to avoid cluttered ordination graphs?

If there is a really high number of species or sites, the graphs often
are congested and many labels are overwritten. It may be impossible to
have complete readable graphics with some data sets. Below we give a
brief overview of tricks you can use. Gavin Simpson’s blog [From the
bottom of the heap](https://fromthebottomoftheheap.net) has a series
of articles on “decluttering ordination plots” with more detailed
discussion and examples.

Often the easiest solution is to use only `points`. If you need text
to identify items, most `text` and `plot` functions have argument
`optimize` and `bg`. With `optimize = TRUE` the exact location of the
score is marked with a point, and text label is written next to the
point trying to find a position that minimizes over-writing other
labels. This is impossible in very congested areas, but often helps in
moderate cases. Argument `bg` defines a background colour of
labels. For instance, `bg = "white"` will write text on white
background. These labels will cover other text below them, but at
least the uppermost are readable instead of both being messed
up. These arguments can be combined, and this often gives the best
result. These arguments can also be used in `text` functions in pipe
(`|>`). Finally, `text` functions take argument `labels` which allows
editing plotted text, and argument `select` to drop completely some
cases.

There is a special function `orditorp` (torp for text-or-point)
function that will use text only if it is not covered by other text
labels and points otherwise (with argument `priority` to define which
items are printed first).

Companion CRAN package **vegan3d** has function `orditkplot` that
provides editable plot where the points are fixed at their ordination
scores, but you can move text labels with mouse.

------------------------------------------------------------------------

### Can I flip an axis in ordination diagram?

Use `xlim` or `ylim` with flipped limits. If you have model
`mod <- cca(dune)` you can flip the first axis with
`plot(mod, xlim = c(3, -2))`.

------------------------------------------------------------------------

### Can I zoom into an ordination plot?

You can use `xlim` and `ylim` arguments in `plot` or `ordiplot` to zoom
into ordination diagrams. Normally you must set both `xlim` and `ylim`
because ordination plots will keep the equal aspect ratio of axes, and
they will fill the graph so that the longer axis will fit.

Dynamic zooming can be done with function `orditkplot` in CRAN package
**vegan3d**. You can directly save the edited `orditkplot` graph in
various graphic formats, or you can export the graph object back to R
session and use `plot` to display the results.

------------------------------------------------------------------------

Other analysis methods
----------------------

------------------------------------------------------------------------

### Is there TWINSPAN?

TWINSPAN for R is available in
[github](https://github.com/jarioksa/twinspan).

------------------------------------------------------------------------

% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
%\VignetteIndexEntry{Diversity analysis in vegan}
\documentclass[a4paper,10pt,twocolumn]{article}
\usepackage{vegan} %% vegan setup

%% TODO: SSarrhenius, adipart, beals update, betadisper
%% expansion (+ permutest), contribdiv, eventstar, multipart, refer to
%% FD, check Kindt reference to specaccum, check estimateR ref

\title{Vegan: ecological diversity} \author{Jari Oksanen}

\date{\footnotesize{
  processed with vegan \Sexpr{packageDescription("vegan", field="Version")}
  in \Sexpr{R.version.string} on \today}}

%% need no \usepackage{Sweave}
\begin{document}
\bibliographystyle{jss}

\SweaveOpts{strip.white=true}
<<echo=false>>=
par(mfrow=c(1,1))
options(width=55)
figset <- function() par(mar=c(4,4,1,1)+.1)
options(SweaveHooks = list(fig = figset))
options("prompt" = "> ", "continue" = "  ")
@

\maketitle
\begin{abstract}
  This document explains diversity related methods in
  \pkg{vegan}. The methods are briefly described, and the equations
  used them are given often in more detail than in their help
  pages. The methods discussed include common diversity indices and
  rarefaction, families of diversity indices, species abundance
  models, species accumulation models and beta diversity, extrapolated
  richness and probability of being a member of the species pool. The
  document is still incomplete and does not cover all diversity
  methods in \pkg{vegan}.
\end{abstract}
\tableofcontents


\noindent The \pkg{vegan} package has two major components:
multivariate analysis (mainly ordination), and methods for diversity
analysis of ecological communities.  This document gives an
introduction to the latter.  Ordination methods are covered in other
documents.  Many of the diversity functions were written by Roeland
Kindt, Bob O'Hara and P{\'e}ter S{\'o}lymos.

Most diversity methods assume that data are counts of individuals.
The methods are used with other data types, and some people argue that
biomass or cover are more adequate than counts of individuals of
variable sizes.  However, this document mainly uses a data set with
counts: stem counts of trees on $1$\,ha plots in the Barro Colorado
Island.  The following steps make these data available for the
document:
<<>>=
library(vegan)
data(BCI)
@

\section{Diversity indices}

Function \code{diversity} finds the most commonly used diversity
indices \citep{Hill73number}:
\begin{align}
H &= - \sum_{i=1}^S p_i \log_b  p_i & \text{Shannon--Weaver}\\
D_1 &= 1 - \sum_{i=1}^S p_i^2  &\text{Simpson}\\
D_2 &= \frac{1}{\sum_{i=1}^S p_i^2}  &\text{inverse Simpson}\,,
\end{align}
where $p_i$ is the proportion of species $i$, and $S$ is the number of
species so that $\sum_{i=1}^S p_i = 1$, and $b$ is the base of the
logarithm.  It is most common to use natural logarithms (and then we
mark index as $H'$), but $b=2$ has
theoretical justification. The default is to use natural logarithms.
Shannon index is calculated with:
<<>>=
H <- diversity(BCI)
@
which finds diversity indices for all sites.

\pkg{Vegan} does not have indices for evenness (equitability), but
the most common of these, Pielou's evenness $J = H'/\log(S)$ is easily
found as:
<<>>=
J <- H/log(specnumber(BCI))
@
where \code{specnumber} is a simple \pkg{vegan} function to find
the numbers of species.

\pkg{vegan} also can estimate series of R\'{e}nyi and Tsallis
diversities. R{\'e}nyi diversity of order $a$ is \citep{Hill73number}:
\begin{equation}
H_a = \frac{1}{1-a} \log \sum_{i=1}^S p_i^a \,,
\end{equation}
and the corresponding Hill number is $N_a = \exp(H_a)$.  Many common
diversity indices are special cases of Hill numbers: $N_0 = S$, $N_1 =
\exp(H')$, $N_2 = D_2$, and $N_\infty = 1/(\max p_i)$. The
corresponding R\'{e}nyi diversities are $H_0 = \log(S)$, $H_1 = H'$, $H_2 =
- \log(\sum p_i^2)$, and $H_\infty = - \log(\max p_i)$.
Tsallis diversity of order $q$ is \citep{Tothmeresz95}:
\begin{equation}
  H_q = \frac{1}{q-1} \left(1 - \sum_{i=1}^S p^q \right) \, .
\end{equation}
These correspond to common diversity indices: $H_0 = S-1$, $H_1 = H'$,
and $H_2 = D_1$, and can be converted to Hill numbers:
\begin{equation}
  N_q = (1 - (q-1) H_q )^\frac{1}{1-q} \, .
\end{equation}

We select a random subset of five sites for R\'{e}nyi diversities:
<<>>=
k <- sample(nrow(BCI), 6)
round(renyi(BCI[k,]), 3)
@
We can really regard a site more diverse if all of its R\'{e}nyi
diversities are higher than in another site.

Finally, the $\alpha$ parameter of Fisher's log-series can be used as
a diversity index \citep{FisherEtal43}:
<<>>=
alpha <- fisher.alpha(BCI)
@

\section{Rarefaction}

Species richness increases with sample size, and differences in
richness actually may be caused by differences in sample size.  To
solve this problem, we may try to rarefy species richness to the same
number of individuals.  Expected number of species in a community
rarefied from $N$ to $n$ individuals is \citep{Hurlbert71}:
\begin{equation}
\label{eq:rare}
\hat S_n = \sum_{i=1}^S (1 - q_i)\,, \quad\text{where }  q_i =
\frac{{N-x_i \choose n}}{{N \choose n}} \,.
\end{equation}
Here $x_i$ is the count of species $i$, and ${N \choose n}$ is the
binomial coefficient, or the number of ways we can choose $n$ from
$N$, and $q_i$ give the probabilities that species $i$ does \emph{not} occur in a
sample of size $n$.  This is positive only when $N-x_i \ge n$, but for
other cases $q_i = 0$ or the species is sure to occur in the sample.
The variance of rarefied richness is \citep{HeckEtal75}:
\begin{multline}
\label{eq:rarevar}
s^2 = q_i (1-q_i)  \\ + 2 \sum_{i=1}^S \sum_{j>i} \left[ \frac{{N- x_i - x_j
    \choose n}}{ {N
    \choose n}} - q_i q_j\right] \,.
\end{multline}
Equation~\ref{eq:rarevar} actually is of the same form as the variance
of sum of correlated variables:
\begin{equation}
\VAR \left(\sum x_i \right) = \sum \VAR (x_i) + 2 \sum_{i=1}^S
\sum_{j>i} \COV (x_i, x_j) \,.
\end{equation}

The number of stems per hectare varies in our
data set:
<<>>=
quantile(rowSums(BCI))
@
To express richness for the same number of individuals, we can use:
<<>>=
Srar <- rarefy(BCI, min(rowSums(BCI)))
@
Rarefaction curves often are seen as an objective solution for
comparing species richness with different sample sizes.  However, rank
orders typically differ among different rarefaction sample sizes,
rarefaction curves can cross.

As an extreme case we may rarefy sample size to two individuals:
<<>>=
S2 <- rarefy(BCI, 2)
@
This will not give equal rank order with the previous rarefaction
richness:
<<>>=
all(rank(Srar) == rank(S2))
@
Moreover, the rarefied richness for two individuals is a finite
sample variant of Simpson's diversity index \citep{Hurlbert71}\,--\,or
more precisely of $D_1 + 1$, and these two are almost identical in
BCI:
<<>>=
range(diversity(BCI, "simp") - (S2 -1))
@
Rarefaction is sometimes presented as an ecologically meaningful
alternative to dubious diversity indices \citep{Hurlbert71}, but the
differences really seem to be small.

\section{Taxonomic and functional diversity}

Simple diversity indices only consider species identity: all different
species are equally different. In contrast, taxonomic and functional
diversity indices judge the differences of species. Taxonomic and
functional diversities are used in different fields of science, but
they really have very similar reasoning, and either could be used
either with taxonomic or functional traits of species.

\subsection{Taxonomic diversity: average distance of traits}

The two basic indices are called taxonomic diversity $\Delta$ and
taxonomic distinctness $\Delta^*$ \citep{ClarkeWarwick98}:
\begin{align}
  \Delta &= \frac{\sum \sum_{i<j} \omega_{ij} x_i x_j}{n (n-1) / 2}\\
\Delta^* &= \frac{\sum \sum_{i<j} \omega_{ij} x_i x_j}{\sum \sum_{i<j}
  x_i x_j} \,.
\end{align}
These equations give the index values for a single site, and summation
goes over species $i$ and $j$, and $\omega$ are the taxonomic
distances among taxa, $x$ are species abundances, and $n$ is the total
abundance for a site.  With presence--absence data, both indices
reduce to the same index called $\Delta^+$, and for this it is
possible to estimate standard deviation. There are two indices
derived from $\Delta^+$: it can be multiplied with species
richness\footnote{This text normally uses upper case letter $S$ for
  species richness, but lower case $s$ is used here in accordance with
  the original papers on taxonomic diversity}
to give $s \Delta^+$, or it can be used to estimate an index of
variation in taxonomic distinctness $\Lambda^+$ \citep{ClarkeWarwick01}:
\begin{equation}
  \Lambda^+ = \frac{\sum \sum_{i<j} \omega_{ij}^2}{n (n-1) / 2} -
  (\Delta^+)^2 \,.
\end{equation}

We still need the taxonomic differences among species ($\omega$) to
calculate the indices. These can be any distance structure among
species, but usually it is found from established hierarchic
taxonomy. Typical coding is that differences among species in the same
genus is $1$, among the same family it is $2$ etc. However, the
taxonomic differences are scaled to maximum $100$ for easier
comparison between different data sets and taxonomies. Alternatively,
it is possible to scale steps between taxonomic level proportional to
the reduction in the number of categories \citep{ClarkeWarwick99}: if
almost all genera have only one species, it does not make a great
difference if two individuals belong to a different species or to a
different genus.

Function \code{taxondive} implements indices of taxonomic diversity,
and \code{taxa2dist} can be used to convert classification tables to
taxonomic distances either with constant or variable step lengths
between successive categories. There is no taxonomic table for the BCI
data in \pkg{vegan}\footnote{Actually I made such a classification,
  but taxonomic differences proved to be of little use in the Barro
  Colorado data: they only singled out sites with Monocots (palm
  trees) in the data.}
but there is such a table for the Dune meadow data (Fig.~\ref{fig:taxondive}):
<<>>=
data(dune)
data(dune.taxon)
taxdis <- taxa2dist(dune.taxon, varstep=TRUE)
mod <- taxondive(dune, taxdis)
@
\begin{figure}
<<fig=true,echo=false>>=
plot(mod)
@
\caption{Taxonomic diversity $\Delta^+$ for the dune meadow data. The
  points are diversity values of single sites, and the funnel is their
  approximate confidence intervals ($2 \times$ standard error).}
\label{fig:taxondive}
\end{figure}

\subsection{Functional diversity: the height of trait tree}

In taxonomic diversity the primary data were taxonomic trees which
were transformed to pairwise distances among species. In functional
diversity the primary data are species traits which are translated to
pairwise distances among species and then to clustering trees of
species traits. The argument for using trees is that in this way a
single deviant species will have a small influence, since its
difference is evaluated only once instead of evaluating its distance
to all other species \citep{PetcheyGaston06}.

Function \code{treedive} implements functional diversity defined as
the total branch length in a trait dendrogram connecting all species,
but excluding the unnecessary root segments of the tree
\citep{PetcheyGaston02, PetcheyGaston06}.  The example uses the
taxonomic distances of the previous chapter. These are first converted
to a hierarchic clustering (which actually were their original form
before \code{taxa2dist} converted them into distances)
<<>>=
tr <- hclust(taxdis, "aver")
mod <- treedive(dune, tr)
@

\section{Species abundance models}

Diversity indices may be regarded as variance measures of species
abundance distribution.  We may wish to inspect abundance
distributions more directly.  \pkg{Vegan} has functions for
Fisher's log-series and Preston's log-normal models, and in addition
several models for species abundance distribution.

\subsection{Fisher and Preston}

In Fisher's log-series, the expected number of species $\hat f$ with $n$
individuals is \citep{FisherEtal43}:
\begin{equation}
\hat f_n = \frac{\alpha x^n}{n} \,,
\end{equation}
where $\alpha$ is the diversity parameter, and $x$ is a nuisance
parameter defined by $\alpha$ and total number
of individuals $N$ in the site, $x = N/(N-\alpha)$.  Fisher's
log-series for a randomly selected plot is (Fig.~\ref{fig:fisher}):
<<>>=
k <- sample(nrow(BCI), 1)
fish <- fisherfit(BCI[k,])
fish
@
\begin{figure}
<<fig=true,echo=false>>=
plot(fish)
@
\caption{Fisher's log-series fitted to one randomly selected site
  (\Sexpr{k}).}
\label{fig:fisher}
\end{figure}
We already saw $\alpha$ as a diversity index.

Preston's log-normal model is the main challenger to Fisher's
log-series \citep{Preston48}.  Instead of plotting species by
frequencies, it bins species into frequency classes of increasing
sizes.  As a result, upper bins with high range of frequencies become
more common, and sometimes the result looks similar to Gaussian
distribution truncated at the left.

There are two alternative functions for the log-normal model:
\code{prestonfit} and \code{prestondistr}.  Function \code{prestonfit}
uses traditionally binning approach, and is burdened with arbitrary
choices of binning limits and treatment of ties. It seems that Preston
split ties between adjacent octaves: only half of the species observed
once were in the first octave, and half were transferred to the next
octave, and the same for all species at the octave limits occurring 2,
4, 8, 16\ldots times \citep{WilliamsonGaston05}. Function
\code{prestonfit} can either split the ties or keep all limit cases in
the lower octave.  Function \code{prestondistr} directly maximizes
truncated log-normal likelihood without binning data, and it is the
recommended alternative.  Log-normal models usually fit poorly to the
BCI data, but here our random plot (number \Sexpr{k}):
<<>>=
prestondistr(BCI[k,])
@

\subsection{Ranked abundance distribution}

An alternative approach to species abundance distribution is to plot
logarithmic abundances in decreasing order, or against ranks of
species \citep{Whittaker65}.  These are known as ranked abundance
distribution curves, species abundance curves, dominance--diversity
curves or Whittaker plots.  Function \code{radfit} fits some of the
most popular models \citep{Bastow91} using maximum likelihood
estimation:
\begin{align}
\hat a_r &= \frac{N}{S} \sum_{k=r}^S \frac{1}{k} &\text{brokenstick}\\
\hat a_r &= N \alpha (1-\alpha)^{r-1} & \text{preemption} \\
\hat a_r &= \exp \left[\log (\mu) + \log (\sigma) \Phi \right]
&\text{log-normal}\\
\hat a_r &= N \hat p_1 r^\gamma &\text{Zipf}\\
\hat a_r &= N c (r + \beta)^\gamma &\text{Zipf--Mandelbrot}
\end{align}
In all these, $\hat a_r$ is the expected abundance of species at rank $r$, $S$
is the number of species, $N$ is the number of individuals, $\Phi$ is
a standard normal function, $\hat p_1$ is the estimated proportion of
the most abundant species, and $\alpha$, $\mu$, $\sigma$, $\gamma$,
$\beta$ and $c$ are the estimated parameters in each model.

It is customary to define the models for proportions $p_r$ instead of
abundances $a_r$, but there is no reason for this, and \code{radfit}
is able to work with the original abundance data.  We have count data,
and the default Poisson error looks appropriate, and our example data
set gives (Fig.~\ref{fig:rad}):
<<>>=
rad <- radfit(BCI[k,])
rad
@
\begin{figure}
<<fig=true,echo=false>>=
print(radlattice(rad))
@
\caption{Ranked abundance distribution models for a random plot
  (no. \Sexpr{k}).  The best model has the lowest \textsc{aic}.}
\label{fig:rad}
\end{figure}

Function \code{radfit} compares the models using alternatively
Akaike's or Schwartz's Bayesian information criteria.  These are based
on log-likelihood, but penalized by the number of estimated
parameters.  The penalty per parameter is $2$ in \textsc{aic}, and
$\log S$ in \textsc{bic}.  Brokenstick is regarded as a null model and
has no estimated parameters in \pkg{vegan}.  Preemption model has
one estimated parameter ($\alpha$), log-normal and Zipf models two
($\mu, \sigma$, or $\hat p_1, \gamma$, resp.), and Zipf--Mandelbrot
model has three ($c, \beta, \gamma$).

Function \code{radfit} also works with data frames, and fits models
for each site. It is curious that log-normal model rarely is the
choice, although it generally is regarded as the canonical model, in
particular in data sets like Barro Colorado tropical forests.

\section{Species accumulation and beta diversity}

Species accumulation models and species pool models study collections
of sites, and their species richness, or try to estimate the number of
unseen species.

\subsection{Species accumulation models}

Species accumulation models are similar to rarefaction: they study the
accumulation of species when the number of sites increases.  There are
several alternative methods, including accumulating sites in the order
they happen to be, and repeated accumulation in random order.  In
addition, there are three analytic models.  Rarefaction pools
individuals together, and applies rarefaction equation (\ref{eq:rare})
to these individuals.  Kindt's exact accumulator resembles rarefaction
\citep{UglandEtal03}:
\begin{multline}
\label{eq:kindt}
\hat S_n = \sum_{i=1}^S (1 - p_i), \,\quad \text{where }
p_i = \frac{{N- f_i \choose n}}{{N \choose n}} \,,
\end{multline}
and $f_i$ is the frequency of species $i$.  Approximate variance
estimator is:
\begin{multline}
\label{eq:kindtvar}
s^2 = p_i (1 - p_i)  \\ + 2 \sum_{i=1}^S \sum_{j>i} \left( r_{ij}
  \sqrt{p_i(1-p_i)} \sqrt{p_j (1-p_j)}\right) \,,
\end{multline}
where $r_{ij}$ is the correlation coefficient between species $i$ and
$j$.  Both of these are unpublished: eq.~\ref{eq:kindt} was developed
by Roeland Kindt, and eq.~\ref{eq:kindtvar} by Jari Oksanen. The third
analytic method was suggested by \citet{Coleman82}:
\begin{equation}
\label{eq:cole}
S_n = \sum_{i=1}^S (1 - p_i), \quad \text{where }  p_i = \left(1 -
  \frac{1}{n}\right)^{f_i} \,,
\end{equation}
and the suggested variance is $s^2 = p_i (1-p_i)$ which ignores the
covariance component.  In addition, eq.~\ref{eq:cole} does not
properly handle sampling without replacement and underestimates the
species accumulation curve.

The recommended is Kindt's exact method (Fig.~\ref{fig:sac}):
<<a>>=
sac <- specaccum(BCI)
plot(sac, ci.type="polygon", ci.col="yellow")
@
\begin{figure}
<<fig=true,echo=false>>=
<<a>>
@
\caption{Species accumulation curve for the BCI data; exact method.}
\label{fig:sac}
\end{figure}

\subsection{Beta diversity}

\citet{Whittaker60} divided diversity into various components. The
best known are diversity in one spot that he called alpha diversity,
and the diversity along gradients that he called beta diversity. The
basic diversity indices are indices of alpha diversity. Beta diversity
should be studied with respect to gradients \citep{Whittaker60}, but
almost everybody understand that as a measure of general heterogeneity
\citep{Tuomisto10a, Tuomisto10b}: how many more species do you have in
a collection of sites compared to an average site.

The best known index of beta diversity is based on the ratio of total
number of species in a collection of sites $S$ and the average
richness per one site $\bar \alpha$ \citep{Tuomisto10a}:
\begin{equation}
  \label{eq:beta}
  \beta = S/\bar \alpha - 1 \,.
\end{equation}
Subtraction of one means that $\beta = 0$ when there are no excess
species or no heterogeneity between sites. For this index, no specific
functions are needed, but this index can be easily found with the help
of \pkg{vegan} function \code{specnumber}:
<<>>=
ncol(BCI)/mean(specnumber(BCI)) - 1
@

The index of eq.~\ref{eq:beta} is problematic because $S$ increases
with the number of sites even when sites are all subsets of the same
community.  \citet{Whittaker60} noticed this, and suggested the index
to be found from pairwise comparison of sites. If the number of shared
species in two sites is $a$, and the numbers of species unique to each
site are $b$ and $c$, then $\bar \alpha = (2a + b + c)/2$ and $S =
a+b+c$, and index~\ref{eq:beta} can be expressed as:
\begin{equation}
  \label{eq:betabray}
  \beta = \frac{a+b+c}{(2a+b+c)/2} - 1 = \frac{b+c}{2a+b+c} \,.
\end{equation}
This is the S{\o}rensen index of dissimilarity, and it can be found
for all sites using \pkg{vegan} function \code{vegdist} with
binary data:
<<>>=
beta <- vegdist(BCI, binary=TRUE)
mean(beta)
@

There are many other definitions of beta diversity in addition to
eq.~\ref{eq:beta}.  All commonly used indices can be found using
\code{betadiver} \citep{KoleffEtal03}. The indices in \code{betadiver}
can be referred to by subscript name, or index number:
<<>>=
betadiver(help=TRUE)
@
Some of these indices are duplicates, and many of them are well known
dissimilarity indices.
One of the more interesting indices is based
on the Arrhenius species--area model
\begin{equation}
  \label{eq:arrhenius}
  \hat S = c X^z\,,
\end{equation}
where $X$ is the area (size) of the patch or site, and $c$ and $z$ are
parameters. Parameter $c$ is uninteresting, but $z$ gives the
steepness of the species area curve and is a measure of beta
diversity. In islands typically  $z \approx 0.3$. This kind of
islands can be regarded as subsets of the same community, indicating
that we really should talk about gradient differences if $z \gtrapprox 0.3$. We
can find the value of $z$ for a pair of plots using function
\code{betadiver}:
<<>>=
z <- betadiver(BCI, "z")
quantile(z)
@
The size $X$ and parameter $c$ cancel out, and the index gives the
estimate $z$ for any pair of sites.

Function \code{betadisper} can be used to analyse beta diversities
with respect to classes or factors \citep{Anderson06, AndersonEtal06}.
There is no such classification available for the Barro Colorado
Island data, and the example studies beta diversities in the
management classes of the dune meadows (Fig.~\ref{fig:betadisper}):
<<>>=
data(dune)
data(dune.env)
z <- betadiver(dune, "z")
mod <- with(dune.env, betadisper(z, Management))
mod
@
\begin{figure}
<<fig=true,echo=false>>=
boxplot(mod)
@
\caption{Box plots of beta diversity measured as the average steepness
  ($z$) of the species area curve in the Arrhenius model $S = cX^z$ in
  Management classes of dune meadows.}
\label{fig:betadisper}
\end{figure}

\section{Species pool}
\subsection{Number of unseen species}

Species accumulation models indicate that not all species were seen in
any site.  These unseen species also belong to the species pool.
Functions \code{specpool} and \code{estimateR} implement some
methods of estimating the number of unseen species.  Function
\code{specpool} studies a collection of sites, and
\code{estimateR} works with counts of individuals, and can be used
with a single site.  Both functions assume that the number of unseen
species is related to the number of rare species, or species seen only
once or twice.

The incidence-based functions group species by their number of
occurrences $f_i = f_0, f_1, \ldots, f_N$, where $f$ is the number of
species occurring in exactly $i$ sites in the data: $f_N$ is the number
of species occurring on every $N$ site, $f_1$ the number of species
occurring once, and $f_0$ the number of species in the species pool
but not found in the sample. The total number of species in the pool
$S_p$ is
\begin{equation}
S_p = \sum_{i=0}^N f_i = f_0+ S_o \,,
\end{equation}
where $S_o = \sum_{i>0} f_i$ is the observed number of species.  The
sampling proportion $i/N$ is an estimate for the commonness of the
species in the community. When species is present in the community but
not in the sample, $i=0$ is an obvious under-estimate, and
consequently, for values $i>0$ the species commonness is
over-estimated \citep{Good53}. The models for the pool size estimate
the number of species missing in the sample $f_0$.

Function \code{specpool} implements the following models to estimate
the number of missing species $f_0$. Chao estimator  is \citep{Chao87, ChiuEtal14}:
\begin{equation}
\label{eq:chao}
\hat f_0 = \begin{cases}
    \frac{f_1^2}{2 f_2} \frac{N-1}{N} &\text{if } f_2 > 0 \\
\frac{f_1 (f_1 -1)}{2}  \frac{N-1}{N} & \text{if } f_2 = 0 \,.
\end{cases}
\end{equation}
The latter case for $f_2=0$ is known as the bias-corrected
form. \citet{ChiuEtal14} introduced the small-sample correction term
$\frac{N}{N-1}$, but it was not originally used \citep{Chao87}.

The first and second order jackknife estimators are
\citep{SmithVanBelle84}:
\begin{align}
\hat f_0 &=  f_1 \frac{N-1}{N}  \\
\hat f_0 & =  f_1 \frac{2N-3}{N}  + f_2 \frac{(N-2)^2}{N(N-1)} \,.
\end{align}
The bootstrap estimator is \citep{SmithVanBelle84}:
\begin{equation}
\hat f_0 =  \sum_{i=1}^{S_o} (1-p_i)^N \,.
\end{equation}
The idea in jackknife seems to be that we missed about as many species
as we saw only once, and the idea in bootstrap that if we repeat
sampling (with replacement) from the same data, we miss as many
species as we missed originally.

The variance estimaters only concern the estimated number of missing
species $\hat f_0$, although they are often expressed as they would
apply to the pool size $S_p$; this is only true if we assume that
$\VAR(S_o) = 0$.  The variance of the Chao estimate is \citep{ChiuEtal14}:
\begin{multline}
\label{eq:var-chao-basic}
\VAR(\hat f_0) = f_1 \left(A^2 \frac{G^3}{4} + A^2 G^2 + A \frac{G}{2} \right),\\
\text{where } A = \frac{N-1}{N}\;\text{and } G = \frac{f_1}{f_2} \,.
\end{multline}
%% The variance of bias-corrected Chao estimate can be approximated by
%% replacing the terms of eq.~\ref{eq:var-chao-basic} with the
%% corresponding terms of the bias-correcter form of in eq.~\ref{eq:chao}:
%% \begin{multline}
%% \label{eq:var-chao-bc}
%% s^2 = A \frac{f_1(f_1-1)}{2} + A^2 \frac{f_1(2 f_1+1)^2}{(f_2+1)^2}\\
%%  + A^2 \frac{f_1^2 f_2 (f_1 -1)^2}{4 (f_2 + 1)^4}
%% \end{multline}
For the bias-corrected form of eq.~\ref{eq:chao}  (case $f_2 = 0$), the variance is
\citep[who omit small-sample correction in some terms]{ChiuEtal14}:
\begin{multline}
\label{eq:var-chao-bc0}
\VAR(\hat f_0) = \tfrac{1}{4} A^2 f_1 (2f_1 -1)^2 + \tfrac{1}{2} A f_1
(f_1-1) \\- \tfrac{1}{4}A^2 \frac{f_1^4}{S_p} \,.
\end{multline}

The variance of the first-order jackknife is based on the number of
``singletons'' $r$ (species occurring only once in the data) in sample
plots \citep{SmithVanBelle84}:
\begin{equation}
\VAR(\hat f_0) = \left(\sum_{i=1}^N r_i^2 - \frac{f_1}{N}\right)
\frac{N-1}{N} \,.
\end{equation}
Variance of the second-order jackknife is not evaluated in
\code{specpool} (but contributions are welcome).

The variance of bootstrap estimator is\citep{SmithVanBelle84}:
\begin{multline}
\VAR(\hat f_0) = \sum_{i=1}^{S_o} q_i (1-q_i)  \\ +2 \sum_{i \neq
  j}^{S_o} \left[(Z_{ij}/N)^N - q_i q_j \right] \\
\text{where } q_i = (1-p_i)^N \, ,
\end{multline}
and $Z_{ij}$ is the number of sites where both species are absent.

The extrapolated richness values for the whole BCI data are:
<<>>=
specpool(BCI)
@
If the estimation of pool size really works, we should get the same
values of estimated richness if we take a random subset of a half of
the plots (but this is rarely true):
<<>>=
s <- sample(nrow(BCI), 25)
specpool(BCI[s,])
@

\subsection{Pool size from a single site}

The \code{specpool} function needs a collection of sites, but there
are some methods that estimate the number of unseen species for each
single site.  These functions need counts of individuals, and species
seen only once or twice, or other rare species, take the place of
species with low frequencies.  Function \code{estimateR} implements
two of these methods:
<<>>=
estimateR(BCI[k,])
@
In abundance based models $a_i$ denotes the number of species with $i$
individuals, and takes the place of $f_i$ of previous models.
Chao's method is similar as the bias-corrected model
eq.~\ref{eq:chao} \citep{Chao87, ChiuEtal14}:
\begin{equation}
  \label{eq:chao-bc}
  S_p = S_o + \frac{a_1 (a_1 - 1)}{2 (a_2 + 1)}\,.
\end{equation}
When $f_2=0$, eq.~\ref{eq:chao-bc} reduces to the bias-corrected form
of eq.~\ref{eq:chao}, but quantitative estimators are based on
abundances and do not use small-sample correction. This is not usually
needed because sample sizes are total numbers of individuals, and
these are usually high, unlike in frequency based models, where the
sample size is the number of sites \citep{ChiuEtal14}.

A commonly used approximate variance estimator of eq.~\ref{eq:chao-bc} is:
\begin{multline}
  \label{eq:var-chao-bc}
 s^2 = \frac{a_1(a_1-1)}{2} + \frac{a_1(2 a_1+1)^2}{(a_2+1)^2}\\
  + \frac{a_1^2 a_2 (a_1 -1)^2}{4 (a_2 + 1)^4} \,.
\end{multline}
However, \pkg{vegan} does not use this, but instead the following more
exact form which was directly derived from eq.~\ref{eq:chao-bc}
following \citet[web appendix]{ChiuEtal14}:
\begin{multline}
  s^2 = \frac{1}{4} \frac{1}{(a_2+1)^4 S_p} [a_1 (S_p a_1^3
      a_2 + 4 S_p a_1^2 a_2^2 \\+  2 S_p a_1 a_2^3 + 6 S_p a_1^2 a_2 + 2 S_p
      a_1 a_2^2 -2 S_p a_2^3 \\+ 4 S_p a_1^2 + S_p a_1 a_2 -5 S_p a_2^2 - a_1^3 - 2
      a_1^2 a_2\\ - a_1 a_2^2 - 2 S_p a_1 - 4 S_p a_2 - S_p ) ]\,.
\end{multline}
The variance estimators only concern the number of unseen species like previously.

The \textsc{ace} is estimator is defined as \citep{OHara05}:
\begin{equation}
\begin{split}
S_p &= S_\mathrm{abund} + \frac{S_\mathrm{rare}}{C_\mathrm{ACE}} +
\frac{a_1}{C_\mathrm{ACE}} \gamma^2\, , \quad \text{where}\\
C_\mathrm{ACE} &= 1 - \frac{a_1}{N_\mathrm{rare}}\\
\gamma^2 &= \frac{S_\mathrm{rare}}{C_\mathrm{ACE}} \sum_{i=1}^{10} i
(i-1) a_1 \frac{N_\mathrm{rare} - 1}{N_\mathrm{rare}}\,.
\end{split}
\end{equation}
Now $a_1$ takes the place of $f_1$ above, and means the number of
species with only one individual.
Here $S_\mathrm{abund}$ and $S_\mathrm{rare}$ are the numbers of
species of abundant and rare species, with an arbitrary upper limit of
10 individuals for a rare species, and $N_\mathrm{rare}$ is the total
number of individuals in rare species. The variance estimator uses
iterative solution, and it is best interpreted from the source code or
following \citet{OHara05}.

The pool size
is estimated separately for each site, but if input is a data frame,
each site will be analysed.

If log-normal abundance model is appropriate, it can be used to
estimate the pool size.  Log-normal model has a finite number of
species which can be found integrating the log-normal:
\begin{equation}
S_p = S_\mu \sigma \sqrt{2 \pi} \,,
\end{equation}
where $S_\mu$ is the modal height or the expected number of species at
maximum (at $\mu$), and $\sigma$ is the width.  Function
\code{veiledspec} estimates this integral from a model fitted either
with \code{prestondistr} or \code{prestonfit}, and fits the latter
if raw site data are given.  Log-normal model may fit poorly, but we
can try:
<<>>=
veiledspec(prestondistr(BCI[k,]))
veiledspec(BCI[k,])
@

\subsection{Probability of pool membership}

Beals smoothing was originally suggested as a tool of regularizing data
for ordination.  It regularizes data too strongly,
but it has been suggested as a method of estimating which of the
missing species could occur in a site, or which sites are suitable for
a species.  The probability for each species at each site is assessed
from other species occurring on the site.

Function \code{beals} implement Beals smoothing \citep{McCune87,
  DeCaceresLegendre08}:
<<>>=
smo <- beals(BCI)
@
We may see how the estimated probability of occurrence and observed
numbers of stems relate in one of the more familiar species. We study
only one species, and to avoid circular reasoning we do not include
the target species in the smoothing (Fig.~\ref{fig:beals}):
<<a>>=
j <- which(colnames(BCI) == "Ceiba.pentandra")
plot(beals(BCI, species=j, include=FALSE), BCI[,j],
     ylab="Occurrence", main="Ceiba pentandra",
     xlab="Probability of occurrence")
@
\begin{figure}
<<fig=true,echo=false>>=
<<a>>
@
\caption{Beals smoothing for \emph{Ceiba pentandra}.}
\label{fig:beals}
\end{figure}

\bibliography{vegan}

\end{document}

% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
%\VignetteIndexEntry{Design decisions and implementation}

\documentclass[a4paper,10pt,twocolumn]{article}
\usepackage{vegan} % package options and redefinitions

\author{Jari Oksanen}
\title{Design decisions and implementation details in vegan}

\date{\footnotesize{
  processed with vegan
\Sexpr{packageDescription("vegan", field="Version")}
in \Sexpr{R.version.string} on \today}}

%% need no \usepackage{Sweave}
\begin{document}
\bibliographystyle{jss}

\SweaveOpts{strip.white=true}

<<echo=false,results=hide>>=
figset <- function() par(mar=c(4,4,1,1)+.1)
options(SweaveHooks = list(fig = figset))
options("prompt" = "> ", "continue" = "  ")
options(width = 55)
require(vegan)
@
\maketitle

\begin{abstract}
  This document describes design decisions, and discusses implementation
and algorithmic details in some vegan functions. The proper FAQ is
another document.
\end{abstract}

\tableofcontents

\section{Parallel processing}

Several \pkg{vegan} functions can perform parallel processing using
the standard \R{} package \pkg{parallel}.
The \pkg{parallel} package in \R{} implements
the functionality of earlier contributed packages \pkg{multicore} and
\pkg{snow}.  The \pkg{multicore} functionality forks the analysis to
multiple cores, and \pkg{snow} functionality sets up a socket cluster
of workers.  The \pkg{multicore} functionality only works in unix-like
systems (such as MacOS and Linux), but \pkg{snow} functionality works
in all operating systems.  \pkg{Vegan} can use either method, but
defaults to \pkg{multicore} functionality when this is available,
because its forked clusters are usually faster.  This chapter
describes both the user interface and internal implementation for the
developers.

\subsection{User interface}
\label{sec:parallel:ui}

The functions that are capable of parallel processing have argument
\code{parallel}.  The normal default is \code{parallel = 1} which
means that no parallel processing is performed.  It is possible to set
parallel processing as the default in \pkg{vegan} (see
\S\,\ref{sec:parallel:default}).

For parallel processing, the \code{parallel} argument can be either

\begin{enumerate}
\item An integer in which case the given number of parallel processes
  will be launched (value $1$ launches non-parallel processing). In
  unix-like systems (\emph{e.g.}, MacOS, Linux) these will be forked
  \code{multicore} processes. In Windows socket clusters will be set up,
  initialized and closed.
\item A previously created socket cluster. This saves time as the
  cluster is not set up and closed in the function.  If the argument is a
  socket cluster, it will also be used in unix-like systems. Setting
  up a socket cluster is discussed in \S\,\ref{sec:parallel:socket}.
\end{enumerate}

\subsubsection{Using parallel processing as default}
\label{sec:parallel:default}

If the user sets option \code{mc.cores}, its value will be used as the
default value of the \code{parallel} argument in \pkg{vegan}
functions.  The following command will set up parallel processing to
all subsequent \pkg{vegan} commands:
<<eval=false>>=
options(mc.cores = 2)
@

The \code{mc.cores} option is defined in the \pkg{parallel} package,
but it is usually unset in which case \pkg{vegan} will default to
non-parallel computation.  The \code{mc.cores} option can be set by
the environmental variable \code{MC_CORES} when the \pkg{parallel}
package is loaded.

\R{} allows setting up a default socket cluster
(\code{setDefaultCluster}), but this will not be used in \pkg{vegan}.

\subsubsection{Setting up socket clusters}
\label{sec:parallel:socket}

If socket clusters are used (and they are the only alternative in
Windows), it is often wise to set up a cluster before calling
parallelized code and give the pre-defined cluster as the value of
the \code{parallel} argument in \pkg{vegan}.  If you want to use
socket clusters in unix-like systems (MacOS, Linux), this can be only
done with pre-defined clusters.

If socket cluster is not set up in Windows, \pkg{vegan} will create and
close the cluster within the function body. This involves following commands:
\begin{Schunk}
\begin{Soutput}
clus <- makeCluster(4)
## perform parallel processing
stopCluster(clus)
\end{Soutput}
\end{Schunk}
The first command sets up the cluster, in this case with four
cores, and the second command stops the cluster.

Most parallelized \pkg{vegan} functions work similarly in socket and
fork clusters, but in \code{oecosimu} the parallel processing is used
to evaluate user-defined functions, and their arguments and data must
be made known to the socket cluster.  For example, if you want to run
in parallel the \code{meandist} function of the \code{oecosimu}
example with a pre-defined socket cluster, you must use:
<<eval=false>>=
## start up and define meandist()
library(vegan)
data(sipoo)
meandist <-
    function(x) mean(vegdist(x, "bray"))
library(parallel)
clus <- makeCluster(4)
clusterEvalQ(clus, library(vegan))
mbc1 <- oecosimu(dune, meandist, "r2dtable",
                 parallel = clus)
stopCluster(clus)
@
Socket clusters are used for parallel processing in Windows, but you
do not need to pre-define the socket cluster in \code{oecosimu} if you
only need \pkg{vegan} commands.  However, if you need some other
contributed packages, you must pre-define the socket cluster also in
Windows with appropriate \code{clusterEvalQ} calls.

If you pre-set the cluster, you can also use \pkg{snow} style socket
clusters in unix-like systems.

\subsubsection{Random number generation}

\pkg{Vegan} does not use parallel processing in random number
generation, and you can set the seed for the standard random number
generator. Setting the seed for the parallelized generator (L'Ecuyer)
has no effect in \pkg{vegan}.

\subsubsection{Does it pay off?}

Parallelized processing has a considerable overhead, and the analysis
is faster only if the non-parallel code is really slow (takes several
seconds in wall clock time). The overhead is particularly large in
socket clusters (in Windows). Creating a socket cluster and evaluating
\code{library(vegan)} with \code{clusterEvalQ} can take two seconds or
longer, and only pays off if the non-parallel analysis takes ten
seconds or longer. Using pre-defined clusters will reduce the
overhead. Fork clusters (in unix-likes operating systems) have a
smaller overhead and can be faster, but they also have an overhead.

Each parallel process needs memory, and for a large number of
processes you need much memory.  If the memory is exhausted, the
parallel processes can stall and  take much longer than
non-parallel processes (minutes instead of seconds).

If the analysis is fast, and function runs in, say, less than five
seconds, parallel processing is rarely useful.  Parallel processing is
useful only in slow analyses: large number of replications or
simulations, slow evaluation of each simulation. The danger of memory
exhaustion must always be remembered.

The benefits and potential problems of parallel processing depend on
your particular system: it is best to rely on your own experience.

\subsection{Internals for developers}

The implementation of the parallel processing should accord with the
description of the user interface above (\S\,\ref{sec:parallel:ui}).
Function \code{oecosimu} can be used as a reference implementation,
and similar interpretation and order of interpretation of arguments
should be followed.  All future implementations should be consistent
and all must be changed if the call heuristic changes.

The value of the \code{parallel} argument can be \code{NULL}, a
positive integer or a socket cluster.  Integer $1$ means that no
parallel processing is performed.  The ``normal'' default is
\code{NULL} which in  the ``normal'' case is interpreted as $1$.  Here
``normal'' means that \R{} is run with default settings without
setting \code{mc.cores} or environmental variable \code{MC_CORES}.

Function \code{oecosimu} interprets the \code{parallel} arguments in
the following way:
\begin{enumerate}
\item \code{NULL}: The function is called with argument \code{parallel
    = getOption("mc.cores")}. The option \code{mc.cores} is normally
  unset and then the default is \code{parallel = NULL}.
\item Integer: An integer value is taken as the number of created
  parallel processes.  In unix-like systems this is the number of
  forked multicore processes, and in Windows this is the number of
  workers in socket clusters.  In Windows, the socket cluster is
  created, and if needed \code{library(vegan)} is evaluated in the
  cluster (this is not necessary if the function only uses internal
  functions), and the cluster is stopped after parallel processing.
\item Socket cluster: If a socket cluster is given, it will be used in
  all operating systems, and  the cluster is not stopped
  within the function.
\end{enumerate}

This gives the following precedence order for parallel processing
(highest to lowest):
\begin{enumerate}
  \item Explicitly given argument value of \code{parallel} will always
    be used.
  \item If \code{mc.cores} is set, it will be used. In Windows this
    means creating and stopping socket clusters. Please note
    that the \code{mc.cores} is only set from the environmental
    variable \code{MC_CORES} when you load the \pkg{parallel} package,
    and it is always unset before first
    \code{require(parallel)}.
 \item The fall back behaviour is no parallel processing.
\end{enumerate}

\section{Nestedness and Null models}

Some published indices of nestedness and null models of communities
are only described in general terms, and they could be implemented in
various ways. Here I discuss the implementation in \pkg{vegan}.

\subsection{Matrix temperature}

The matrix temperature is intuitively simple
(Fig. \ref{fig:nestedtemp}), but the the exact calculations were not
explained in the original publication \cite{AtmarPat93}.
\begin{figure}
<<fig=true,echo=false,results=hide>>=
data(sipoo)
mod <- nestedtemp(sipoo)
plot(mod, "i")
x <- mod$c["Falcsubb"]
y <- 1 - mod$r["Svartholm"]
points(x,y, pch=16, cex=1.5)
abline(x+y, -1, lty=2)
f <- function(x, p) (1-(1-x)^p)^(1/p)
cross <- function(x, a, p) f(x,p) - a + x
r <- uniroot(cross, c(0,1), a = x+y, p = mod$p)$root
arrows(x,y, r, f(r, mod$p), lwd=4)
@
\caption{Matrix temperature for \emph{Falco subbuteo} on Sibbo
  Svartholmen (dot). The curve is the fill line, and in a cold
  matrix, all presences (red squares) should be in the upper left
  corner behind the fill line. Dashed diagonal line of length $D$ goes
  through the point, and an arrow of length $d$ connects the point to
  the fill line. The ``surprise'' for this point is $u = (d/D)^2$ and
  the matrix temperature is based on the sum of surprises: presences
  outside the fill line or absences within the fill line.}
\label{fig:nestedtemp}
\end{figure}
The function can be implemented in many ways following the general
principles.  \citet{RodGir06} have seen the original code and reveal
more details of calculations, and their explanation is the basis of
the implementation in \pkg{vegan}.  However, there are still some open
issues, and probably \pkg{vegan} function \code{nestedtemp} will never
exactly reproduce results from other programs, although it is based on
the same general principles.\footnote{function \code{nestedness} in
  the \pkg{bipartite} package is a direct port of the original
  \proglang{BINMATNEST} program of \citet{RodGir06}.}  I try to give
main computation details in this document --- all details can be seen
in the source code of \code{nestedtemp}.

\begin{itemize}
\item Species and sites are put into unit square \citep{RodGir06}. The
  row and column coordinates will be $(k-0.5)/n$ for $k=1 \ldots n$,
  so that there are no points in the corners or the margins of the
  unit square, and a diagonal line can be drawn through any point. I
  do not know how the rows and columns are converted to the unit
  square in other software, and this may be a considerable source of
  differences among implementations.
  \item Species and sites are ordered alternately using indices
    \citep{RodGir06}:
    \begin{equation}
    \begin{split}
      s_j &= \sum_{i|x_{ij} = 1} i^2 \\
      t_j &= \sum_{i|x_{ij} = 0} (n-i+1)^2
    \end{split}
    \end{equation}
    Here $x$ is the data matrix, where $1$ is presence, and $0$ is
    absence, $i$ and $j$ are row and column indices, and $n$ is the
    number of rows. The equations give the indices for columns, but
    the indices can be reversed for corresponding row indexing.
    Ordering by $s$ packs presences to the top left corner, and
    ordering by $t$ pack zeros away from the top left corner. The final
    sorting should be ``a compromise'' \citep{RodGir06} between these
    scores, and \pkg{vegan} uses $s+t$.  The result should be cool,
    but the packing does not try to minimize the temperature
    \citep{RodGir06}.  I do not know how the ``compromise'' is
    defined, and this can cause some differences to other
    implementations.
  \item The following function is used to define the fill line:
    \begin{equation}
      y = (1-(1-x)^p)^{1/p}
    \end{equation}
    This is similar to the equation suggested by
    \citet[eq. 4]{RodGir06}, but omits all terms dependent on the
    numbers of species or sites, because I could not understand why
    they were needed. The differences are visible only in small data
    sets. The $y$ and $x$ are the coordinates in the unit square, and
    the parameter $p$ is selected so that the curve covers the same
    area as is the proportion of presences
    (Fig. \ref{fig:nestedtemp}). The parameter $p$ is found
    numerically using \proglang{R} functions \code{integrate} and
    \code{uniroot}.  The fill line used in the original matrix
    temperature software \citep{AtmarPat93} is supposed to be similar
    \citep{RodGir06}. Small details in the fill line combined with
    differences in scores used in the unit square (especially in the
    corners) can cause large differences in the results.
  \item A line with slope\,$= -1$ is drawn through the point and the $x$
    coordinate of the intersection of this line and the fill line is
    found using function \code{uniroot}. The difference of this
    intersection and the row coordinate gives the argument $d$ of matrix
    temperature (Fig. \ref{fig:nestedtemp}).
  \item In other software, ``duplicated'' species occurring on every
    site are removed, as well as empty sites and species after
    reordering \cite{RodGir06}. This is not done in \pkg{vegan}.
\end{itemize}


\section{Scaling in redundancy analysis}

This chapter discusses the scaling of scores (results) in redundancy
analysis and principal component analysis performed by function
\code{rda} in the \pkg{vegan} library.

Principal component analysis decomposes a centred data matrix
$\mathbf{X} = \{x_{ij}\}$ into $K$ orthogonal components so that
$x_{ij} = \sqrt{n-1} \sum_{k=1}^K u_{ik} \sqrt{\lambda_k} v_{jk}$,
where $u_{ik}$ and $v_{jk}$ are orthonormal coefficient matrices and
$\lambda_k$ are eigenvalues. In \pkg{vegan} the eigenvalues sum up to
variance of the data, and therefore we need to multiply with the
square root of degrees of freedom $n-1$.  Orthonormality means that
sums of squared columns is one and their cross-product is zero, or
$\sum_i u_{ik}^2 = \sum_j v_{jk}^2 = 1$, and
$\sum_i u_{ik} u_{il} = \sum_j v_{jk} v_{jl} = 0$ for $k \neq l$. This
is a decomposition, and the original matrix is found exactly from the
singular vectors and corresponding singular values, and first two
singular components give the rank $=2$ least squares estimate of the
original matrix.

The coefficients $u_{ik}$ and $v_{jk}$ are scaled to unit length for all
axes $k$. Eigenvalues $\lambda_k$ give
the information of the importance of axes, or the `axis lengths.'
Instead of the orthonormal coefficients, or equal length axes, it is
customary to scale species (column) or site (row) scores or both by
eigenvalues to display the importance of axes and to describe the true
configuration of points.  Table \ref{tab:scales} shows some
alternative scalings.  These alternatives apply to principal
components analysis in all cases, and in redundancy analysis, they
apply to species scores and constraints or linear combination scores;
weighted averaging scores have somewhat wider dispersion.

\begin{table*}[t]
  \centering
  \caption{\label{tab:scales} Alternative scalings for \textsc{rda} used
    in the functions \code{prcomp} and \code{princomp}, and the
    one used in the \pkg{vegan} function \code{rda}
    and the proprietary software \proglang{Canoco}
    scores in terms of orthonormal species ($v_{ik}$) and site scores
    ($u_{jk}$), eigenvalues ($\lambda_k$), number of sites  ($n$) and
    species standard deviations ($s_j$). In \code{rda},
    $\mathrm{const} = \sqrt[4]{(n-1) \sum \lambda_k}$.  Corresponding
    negative scaling in \pkg{vegan}
   % and corresponding positive scaling in \texttt{Canoco 3}
    is derived
    dividing each  species by its standard deviation $s_j$ (possibly
    with some additional constant multiplier).  }
 \begin{tabular}{lcc}
  \\
  \toprule
& \textbf{Site scores} $u_{ik}^*$ &
\textbf{Species scores} $v_{jk}^*$ \\
\midrule
\code{prcomp, princomp} &
$u_{ik} \sqrt{n-1} \sqrt{\lambda_k}$ &
$v_{jk}$ \\
\code{stats::biplot} &
$u_{ik}$ &
$v_{jk} \sqrt{n} \sqrt{\lambda_k}$ \\
\code{stats::biplot, pc.biplot=TRUE} &
$u_{ik} \sqrt{n-1}$ &
$v_{jk} \sqrt{\lambda_k}$\\
\code{rda, scaling="sites"} &
$u_{ik} \sqrt{\lambda_k/ \sum \lambda_k} \times \mathrm{const}$ &
$v_{jk} \times \mathrm{const}$
\\
\code{rda, scaling="species"} &
$u_{ik} \times \mathrm{const}$ &
$v_{jk} \sqrt{\lambda_k/ \sum \lambda_k} \times \mathrm{const}$  \\
\code{rda, scaling="symmetric"} &
$u_{ik} \sqrt[4]{\lambda_k/ \sum \lambda_k} \times \mathrm{const}$ &
$v_{jk} \sqrt[4]{\lambda_k/ \sum \lambda_k} \times \mathrm{const}$ \\
\code{rda, correlation=TRUE} &
$u_{ik}^*$ &
$\sqrt{\sum \lambda_k /(n-1)} s_j^{-1} v_{jk}^*$
\\
% \code{Canoco 3, scaling=-1} &
% $u_{ik} \sqrt{n-1} \sqrt{\lambda_k / \sum \lambda_k}$ &
% $v_{jk} \sqrt{n}$ \\
% \code{Canoco 3, scaling=-2} &
% $u_{ik} \sqrt{n-1}$ &
% $v_{jk} \sqrt{n} \sqrt{\lambda_k / \sum \lambda_k}$
% \\
% \code{Canoco 3, scaling=-3} &
% $u_{ik} \sqrt{n-1} \sqrt[4]{\lambda_k / \sum \lambda_k}$ &
% $v_{jk} \sqrt{n} \sqrt[4]{\lambda_k / \sum \lambda_k}$
\bottomrule
\end{tabular}
\end{table*}

In community ecology, it is common to plot both species and sites in
the same graph.  If this graph is a graphical display of \textsc{pca},
or a graphical, low-dimensional approximation of the data, the graph
is called a biplot.  The graph is a biplot if the transformed scores
satisfy $x_{ij} = c \sum_k u_{ij}^* v_{jk}^*$ where $c$ is a scaling
constant.  In functions \code{princomp}, \code{prcomp} and \code{rda}
with \code{scaling = "sites"}, the plotted scores define a biplot so that
the eigenvalues are expressed for sites, and species are left
unscaled.
% For \texttt{Canoco 3} $c = n^{-1} \sqrt{n-1}
% \sqrt{\sum \lambda_k}$ with negative \proglang{Canoco} scaling
% values. All these $c$ are constants for a matrix, so these are all
% biplots with different internal scaling of species and site scores
% with respect to each other.  For \proglang{Canoco} with positive scaling
% values and \pkg{vegan} with negative scaling values, no constant
% $c$ can be found, but the correction is dependent on species standard
% deviations $s_j$, and these scores do not define a biplot.

There is no natural way of scaling species and site scores to each
other.  The eigenvalues in redundancy and principal components
analysis are scale-dependent and change when the data are multiplied
by a constant.  If we have percent cover data, the eigenvalues are
typically very high, and the scores scaled by eigenvalues will have
much wider dispersion than the orthonormal set.  If we express the
percentages as proportions, and divide the matrix by $100$, the
eigenvalues will be reduced by factor $100^2$, and the scores scaled
by eigenvalues will have a narrower dispersion.  For graphical biplots
we should be able to fix the relations of row and column scores to be
invariant against scaling of data.  The solution in \proglang{R}
standard function \code{biplot} is to scale site and species scores
independently, and typically very differently
(Table~\ref{tab:scales}), but plot each independently to fill the
graph area.  The solution in \proglang{Canoco} and \code{rda} is to
use proportional eigenvalues $\lambda_k / \sum \lambda_k$ instead of
original eigenvalues.  These proportions are invariant with scale
changes, and typically they have a nice range for plotting two data
sets in the same graph.

The \textbf{vegan} package uses a scaling constant $c = \sqrt[4]{(n-1)
  \sum \lambda_k}$ in order to be able to use scaling by proportional
eigenvalues (like in \proglang{Canoco}) and still be able to have a
biplot scaling. Because of this, the scaling of \code{rda} scores is
non-standard. However, the \code{scores} function lets you to set
the scaling constant to any desired values. It is also possible to
have two separate scaling constants: the first for the species, and
the second for sites and friends, and this allows getting scores of
other software or \proglang{R} functions (Table \ref{tab:rdaconst}).

\begin{table*}[t]
  \centering
  \caption{\label{tab:rdaconst} Values of the \code{const} argument in
    \textbf{vegan} to get the scores that are equal to those from
    other functions and software. Number of sites (rows) is $n$,
    the number of species (columns) is $m$, and the sum of all
    eigenvalues is $\sum_k \lambda_k$ (this is saved as the item
    \code{tot.chi} in the \code{rda} result)}.
 \begin{tabular}{lccc}
  \\
  \toprule
& \textbf{Scaling} &\textbf{Species constant} & \textbf{Site constant} \\
\midrule
\pkg{vegan} & any  & $\sqrt[4]{(n-1) \sum \lambda_k}$ & $\sqrt[4]{(n-1) \sum \lambda_k}$\\
\code{prcomp}, \code{princomp} & \code{1} & $1$ & $\sqrt{(n-1) \sum_k \lambda_k}$\\
\proglang{Canoco\,v3} & \code{-1, -2, -3} & $\sqrt{n-1}$ & $\sqrt{n}$\\
\proglang{Canoco\,v4} & \code{-1, -2, -3} & $\sqrt{m}$ & $\sqrt{n}$\\
\bottomrule
\end{tabular}
\end{table*}

The scaling is controlled by three arguments in the \code{scores}
function in \pkg{vegan}:
\begin{enumerate}
  \item \code{scaling} with options \code{"sites"}, \code{"species"}
    and \code{"symmetric"} defines the set of scores which is scaled
    by eigenvalues (Table~\ref{tab:scales}).
  \item \code{const} can be used to set the numeric scaling constant
    to non-default values (Table~\ref{tab:rdaconst}).
  \item \code{correlation} can be used to modify species scores so
    that they show the relative change of species abundance, or their
    correlation with the ordination (Table~\ref{tab:scales}). This is
    no longer a biplot scaling.
\end{enumerate}

\section{Weighted average and linear combination scores}

Constrained ordination methods such as Constrained Correspondence
Analysis (CCA) and Redundancy Analysis (RDA) produce two kind of site
scores \cite{Braak86, Palmer93}:
\begin{itemize}
\item
LC or Linear Combination Scores which are linear combinations of
constraining variables.
\item
WA or Weighted Averages Scores which are such weighted averages of
species scores that are as similar to LC scores as possible.
\end{itemize}
Many computer programs for constrained ordinations give only or
primarily LC scores following recommendation of
\citet{Palmer93}.  However, functions \code{cca} and \code{rda} in
the \pkg{vegan} package use primarily WA scores. This chapter
explains the reasons for this choice.

Briefly, the main reasons are that
\begin{itemize}
\item LC scores \emph{are} linear combinations, so they give us only
  the (scaled) environmental variables. This means that they are
  independent of vegetation and cannot be found from the species
  composition.  Moreover, identical combinations of environmental
  variables give identical LC scores irrespective of vegetation.
\item \citet{McCune97} has demonstrated that noisy environmental
  variables result in deteriorated LC scores whereas WA scores
  tolerate some errors in environmental variables.  All environmental
  measurements contain some errors, and therefore it is safer to use
  WA scores.
\end{itemize}
This article studies mainly the first point.  The users of
\pkg{vegan} have a choice of either LC or WA (default) scores, but
after reading this article, I believe that most of them do not want to
use LC scores, because they are not what they were looking for in
ordination.

\subsection{LC Scores are Linear Combinations}

Let us perform a simple CCA analysis using only two environmental
variables so that we can see the constrained solution completely in
two dimensions:
<<>>=
library(vegan)
data(varespec)
data(varechem)
orig <- cca(varespec ~ Al + K, varechem)
@
Function \code{cca} in \pkg{vegan} uses WA scores as
default. So we must specifically ask for LC scores
(Fig. \ref{fig:ccalc}).
<<a,fig=false>>=
plot(orig, dis=c("lc","bp"))
@
\begin{figure}
<<fig=true,echo=false>>=
<<a>>
@
\caption{LC scores in CCA of the original data.}
\label{fig:ccalc}
\end{figure}

What would happen to linear combinations of LC scores if we shuffle
the ordering of sites in species data?  Function \code{sample()} below
shuffles the indices.
<<>>=
i <- sample(nrow(varespec))
shuff <- cca(varespec[i,] ~ Al + K, varechem)
@
\begin{figure}
<<fig=true,echo=false>>=
plot(shuff, dis=c("lc","bp"))
@
\caption{LC scores of shuffled species data.}
\label{fig:ccashuff}
\end{figure}
It seems that site scores are fairly similar, but oriented differently
(Fig. \ref{fig:ccashuff}).  We can use Procrustes rotation to see how
similar the site scores indeed are (Fig. \ref{fig:ccaproc}).
<<a,fig=false>>=
plot(procrustes(scores(orig, dis="lc"),
                scores(shuff, dis="lc")))
@
\begin{figure}
<<fig=true,echo=false>>=
<<a>>
@
\caption{Procrustes rotation of LC scores from CCA of original and shuffled data.}
\label{fig:ccaproc}
\end{figure}
There is a small difference, but this will disappear if we use
Redundancy Analysis (RDA) instead of CCA
(Fig. \ref{fig:rdaproc}). Here we use a new shuffling as well.
<<>>=
tmp1 <- rda(varespec ~ Al + K, varechem)
i <- sample(nrow(varespec)) # Different shuffling
tmp2 <- rda(varespec[i,] ~ Al + K, varechem)
@
\begin{figure}
<<fig=true,echo=false>>=
plot(procrustes(scores(tmp1, dis="lc"),
                scores(tmp2, dis="lc")))
@
\caption{Procrustes rotation of LC scores in RDA of the original and shuffled data.}
\label{fig:rdaproc}
\end{figure}

LC scores indeed are linear combinations of constraints (environmental
variables) and \emph{independent of species data}: You can
shuffle your species data, or change the data completely, but the LC
scores will be unchanged in RDA.  In CCA the LC scores are
\emph{weighted} linear combinations with site totals of species data
as weights. Shuffling species data in CCA changes the weights, and
this can cause changes in LC scores.  The magnitude of changes depends
on the variability of site totals.

The original data and shuffled data differ in their goodness of
fit:
<<>>=
orig
shuff
@
Similarly their WA scores will be (probably) very different
(Fig. \ref{fig:ccawa}).
\begin{figure}
<<fig=true,echo=false>>=
plot(procrustes(orig, shuff))
@
\caption{Procrustes rotation of WA scores of CCA with the original and
  shuffled data.}
\label{fig:ccawa}
\end{figure}

The example used only two environmental variables so that we can
easily plot all constrained axes.  With a larger number of
environmental variables the full configuration remains similarly
unchanged, but its orientation may change, so that two-dimensional
projections look different.  In the full space, the differences should
remain within numerical accuracy:
<<>>=
tmp1 <- rda(varespec ~ ., varechem)
tmp2 <- rda(varespec[i,] ~ ., varechem)
proc <- procrustes(scores(tmp1, dis="lc", choi=1:14),
                   scores(tmp2, dis="lc", choi=1:14))
max(residuals(proc))
@
In \code{cca} the difference would be somewhat larger than now
observed \Sexpr{format.pval(max(residuals(proc)))} because site
weights used for environmental variables are shuffled with the species
data.

\subsection{Factor constraints}

It seems that users often get confused when they perform constrained
analysis using  only one factor (class variable) as constraint.  The
following example uses the classical dune meadow data \cite{Jongman87}:
<<>>=
data(dune)
data(dune.env)
orig <- cca(dune ~ Moisture, dune.env)
@
When the results are plotted using LC scores, sample plots fall only
in four alternative positions (Fig. \ref{fig:factorlc}).
\begin{figure}
<<fig=TRUE,echo=false>>=
plot(orig, dis="lc")
@
\caption{LC scores of the dune meadow data using only one factor as a
  constraint.}
\label{fig:factorlc}
\end{figure}
In the previous chapter we saw that this happens because LC scores
\emph{are} the environmental variables, and they can be distinct only
if the environmental variables are distinct.  However, normally the user
would like to see how well the environmental variables separate the
vegetation, or inversely, how we could use the vegetation to
discriminate the environmental conditions.  For this purpose we should
plot WA scores, or LC scores and WA scores together:  The LC scores
show where the site \emph{should} be, the WA scores shows where the
site \emph{is}.

Function \code{ordispider} adds line segments to connect each WA
score with the corresponding LC (Fig.  \ref{fig:walcspider}).
<<a,fig=false>>=
plot(orig, display="wa", type="points")
ordispider(orig, col="red")
text(orig, dis="cn", col="blue")
@
\begin{figure}
<<fig=TRUE,echo=false>>=
<<a>>
@
\caption{A ``spider plot'' connecting WA scores to corresponding LC
  scores. The shorter the web segments, the better the ordination.}
\label{fig:walcspider}
\end{figure}
This is the standard way of displaying results of discriminant
analysis, too.  Moisture classes \code{1} and \code{2} seem to be
overlapping, and cannot be completely separated by their
vegetation. Other classes are more distinct, but there seems to be a
clear arc effect or a ``horseshoe'' despite using CCA.

\subsection{Conclusion}

LC scores are only the (weighted and scaled) constraints and
independent of vegetation. If you plot them, you plot only your
environmental variables. WA scores are based on vegetation data but
are constrained to be as similar to the LC scores as only
possible. Therefore \pkg{vegan} calls LC scores as
\code{constraints} and WA scores as \code{site scores}, and uses
primarily WA scores in plotting.  However, the user makes the ultimate
choice, since both scores are available.

\bibliography{vegan}

\end{document}

\encoding{UTF-8}
\name{dune.taxon}
\alias{dune.taxon}
\alias{dune.phylodis}
\docType{data}
\title{Taxonomic Classification and Phylogeny of Dune Meadow Species}
\description{
  Classification table of the species in the \code{\link{dune}} data
  set.
}
\usage{
  data(dune.taxon)
  data(dune.phylodis)
}
\format{
  \code{dune.taxon} is data frame with 30 species (rows) classified into
  five taxonomic levels (columns). \code{dune.phylodis} is a
  \code{\link{dist}} object of estimated coalescence ages extracted from
  \doi{10.5061/dryad.63q27} (Zanne et al. 2014) using tools in packages
  \pkg{ape} and \pkg{phylobase}.
}

\details{

  The families and orders are based on APG IV (2016) in vascular
  plants and on Hill et al. (2006) in mosses. The higher levels
  (superorder and subclass) are based on Chase & Reveal (2009). Chase
  & Reveal (2009) treat Angiosperms and mosses as subclasses of class
  Equisetopsida (land plants), but brylogists have traditionally used
  much more inflated levels which are adjusted here to match
  Angiosperm classification.

}
\references{

  APG IV [Angiosperm Phylogeny Group] (2016) An update of the
  Angiosperm Phylogeny Group classification for the orders and
  families of flowering plants: APG IV. \emph{Bot. J. Linnean Soc.}
  \strong{181}: 1--20.

  Chase, M.W. & Reveal, J. L. (2009) A phylogenetic classification of
  the land plants to accompany APG III. \emph{Bot. J. Linnean Soc.}
  \strong{161}: 122--127.

  Hill, M.O et al. (2006) An annotated checklist of the mosses of Europe
  and Macaronesia. \emph{J. Bryology} \strong{28}: 198--267.

  Zanne A.E., Tank D.C., Cornwell, W.K., Eastman J.M., Smith, S.A.,
  FitzJohn, R.G., McGlinn, D.J., O’Meara, B.C., Moles, A.T., Reich,
  P.B., Royer, D.L., Soltis, D.E., Stevens, P.F., Westoby, M., Wright,
  I.J., Aarssen, L., Bertin, R.I., Calaminus, A., Govaerts, R.,
  Hemmings, F., Leishman, M.R., Oleksyn, J., Soltis, P.S., Swenson,
  N.G., Warman, L. & Beaulieu, J.M. (2014) Three keys to the radiation
  of angiosperms into freezing environments. \emph{Nature}
  \strong{506}: 89--92.

}

\seealso{Functions \code{\link{taxondive}}, \code{\link{treedive}},
  and \code{\link{treedist}} use these data sets. }

\examples{ 
  data(dune.taxon) 
  data(dune.phylodis)
}
\keyword{datasets}

\encoding{UTF-8}
\name{varespec}
\alias{varechem}
\alias{varespec}
\docType{data}
\title{Vegetation and environment in lichen pastures}
\usage{
       data(varechem)
       data(varespec)
}
\description{
  The \code{varespec} data frame has 24 rows and 44 columns.  Columns
  are estimated cover values of 44 species.  The variable names are
  formed from the scientific names, and are self explanatory for anybody
  familiar with the vegetation type.
The \code{varechem} data frame has 24 rows and 14 columns, giving the
soil characteristics of the very same sites as in the \code{varespec}
data frame. The chemical measurements have obvious names.
\code{Baresoil} gives the estimated cover of bare soil, \code{Humdepth}
the thickness of the humus layer.
}
\references{
\enc{Väre}{Vare}, H., Ohtonen, R. and Oksanen, J. (1995) Effects of reindeer
grazing on understorey vegetation in dry Pinus sylvestris
forests. \emph{Journal of Vegetation Science} 6, 523--530.  
}
\examples{
data(varespec)
data(varechem)
}
\keyword{datasets}

\encoding{UTF-8}
\name{deviance.cca}
\alias{deviance.cca}
\alias{deviance.rda}
\alias{extractAIC.cca}
\title{ Statistics Resembling Deviance and AIC for Constrained Ordination}
\description{
  The functions extract statistics that resemble deviance and AIC from the
  result of constrained correspondence analysis \code{\link{cca}} or
  redundancy analysis \code{\link{rda}}.  These functions are rarely
  needed directly, but they are called by \code{\link{step}} in
  automatic model building.  Actually, \code{\link{cca}} and
  \code{\link{rda}} do not have \code{\link{AIC}} and these functions
  are certainly wrong.
}
\usage{
\method{deviance}{cca}(object, ...)

\method{extractAIC}{cca}(fit, scale = 0, k = 2, ...)
}

\arguments{
  \item{object}{the result of a constrained ordination
    (\code{\link{cca}} or \code{\link{rda}}). }
  \item{fit}{fitted model from constrained ordination.}
  \item{scale}{optional numeric specifying the scale parameter of the model,
    see \code{scale} in \code{\link{step}}.}
  \item{k}{numeric specifying the "weight" of the \emph{equivalent degrees of
    freedom} (=:\code{edf}) part in the AIC formula.}
  \item{\dots}{further arguments. }
}
\details{
  The functions find statistics that
  resemble \code{\link{deviance}} and \code{\link{AIC}} in constrained
  ordination.  Actually, constrained ordination methods do not have a
  log-Likelihood, which means that they cannot have AIC and deviance.
  Therefore you should not use these functions, and if you use them, you
  should not trust them.  If you use these functions, it remains as your
  responsibility to check the adequacy of the result.

  The deviance of \code{\link{cca}} is equal to the Chi-square of
  the residual data matrix after fitting the constraints.  The deviance
  of  \code{\link{rda}} is defined as the residual sum of squares. The
  deviance function of \code{rda} is also used for
  \code{\link{capscale}}. Function \code{extractAIC} mimics
  \code{extractAIC.lm} in translating deviance to AIC.
  
  There is little need to call these functions directly.  However, they
  are called implicitly in \code{\link{step}} function used in automatic
  selection of constraining variables.  You should check the resulting
  model with some other criteria, because the statistics used here are
  unfounded. In particular, the penalty \code{k} is not properly
  defined, and the default \code{k = 2} is not justified
  theoretically. If you have only continuous covariates, the \code{step}
  function will base the model building on magnitude of eigenvalues, and
  the value of \code{k} only influences the stopping point (but the
  variables with the highest eigenvalues are not necessarily the most
  significant in permutation tests in \code{\link{anova.cca}}). If you
  also have multi-class factors, the value of \code{k} will have a
  capricious effect in model building. The \code{\link{step}} function
  will pass arguments to \code{\link{add1.cca}} and
  \code{\link{drop1.cca}}, and setting \code{test = "permutation"}
  will provide permutation tests of each deletion and addition which
  can help in judging the validity of the model building.
 
}
\value{
  The \code{deviance} functions return \dQuote{deviance}, and
  \code{extractAIC} returns effective degrees of freedom and \dQuote{AIC}. 
}
\references{
  \enc{Godínez-Domínguez}{Godinez-Dominguez}, E. & Freire, J. (2003)
  Information-theoretic approach for selection of spatial and temporal
  models of community organization. \emph{Marine Ecology Progress
    Series} \strong{253}, 17--24. 
}
\author{ Jari  Oksanen }

\note{
  These functions are unfounded and untested and they should not be used
  directly or implicitly.  Moreover, usual caveats in using 
  \code{\link{step}} are very valid.
}

\seealso{\code{\link{cca}}, \code{\link{rda}}, \code{\link{anova.cca}},
    \code{\link{step}}, \code{\link{extractAIC}},
    \code{\link{add1.cca}}, \code{\link{drop1.cca}}. }
\examples{
# The deviance of correspondence analysis equals Chi-square
data(dune)
data(dune.env)
chisq.test(dune)
deviance(cca(dune))
# Stepwise selection (forward from an empty model "dune ~ 1")
ord <- cca(dune ~ ., dune.env)
step(cca(dune ~ 1, dune.env), scope = formula(ord))
}
\keyword{ multivariate }
\keyword{ models }

\encoding{UTF-8}
\name{betadisper}
\alias{betadisper}
\alias{scores.betadisper}
\alias{anova.betadisper}
\alias{plot.betadisper}
\alias{boxplot.betadisper}
\alias{TukeyHSD.betadisper}
\alias{eigenvals.betadisper}
\alias{print.betadisper}
\alias{ordimedian}

\title{Multivariate homogeneity of groups dispersions (variances)}
\description{
  Implements Marti Anderson's PERMDISP2 procedure for the analysis of
  multivariate homogeneity of group dispersions (variances).
  \code{betadisper} is a multivariate analogue of Levene's test for
  homogeneity of variances. Non-euclidean distances between objects and
  group centroids are handled by reducing the original distances to
  principal coordinates. This procedure has latterly been used as a
  means of assessing beta diversity. There are \code{anova},
  \code{scores}, \code{plot} and \code{boxplot} methods.

  \code{TukeyHSD.betadisper} creates a set of confidence intervals on
  the differences between the mean distance-to-centroid of the levels of
  the grouping factor with the specified family-wise probability of
  coverage.  The intervals are based on the Studentized range statistic,
  Tukey's 'Honest Significant Difference' method.
}
\usage{
betadisper(d, group, type = c("median","centroid"), bias.adjust = FALSE,
       sqrt.dist = FALSE, add = FALSE)

\method{anova}{betadisper}(object, \dots)

\method{scores}{betadisper}(x, display = c("sites", "centroids"),
       choices = c(1,2), \dots)

\method{eigenvals}{betadisper}(x, \dots)

\method{plot}{betadisper}(x, axes = c(1,2), cex = 0.7,
     pch = seq_len(ng), col = NULL, lty = "solid", lwd = 1, hull = TRUE,
     ellipse = FALSE, conf,
     segments = TRUE, seg.col = "grey", seg.lty = lty, seg.lwd = lwd,
     label = TRUE, label.cex = 1,
     ylab, xlab, main, sub, \dots)

\method{boxplot}{betadisper}(x, ylab = "Distance to centroid", ...)

\method{TukeyHSD}{betadisper}(x, which = "group", ordered = FALSE,
         conf.level = 0.95, \dots)

\method{print}{betadisper}(x, digits = max(3, getOption("digits") - 3),
                           neigen = 8, \dots)
}

\arguments{
  \item{d}{a distance structure such as that returned by 
    \code{\link[stats]{dist}}, \code{\link{betadiver}} or
    \code{\link{vegdist}}.}
  \item{group}{vector describing the group structure, usually a factor
    or an object that can be coerced to a factor using
    \code{\link{as.factor}}. Can consist of a factor with a single
    level (i.e., one group).}
  \item{type}{the type of analysis to perform. Use the spatial median or
    the group centroid? The spatial median is now the default.}
  \item{bias.adjust}{logical: adjust for small sample bias in beta
    diversity estimates?}
  \item{sqrt.dist}{Take square root of dissimilarities. This often
    euclidifies dissimilarities.}
  \item{add}{Add a constant to the non-diagonal dissimilarities such
    that all eigenvalues are non-negative in the underlying Principal
    Co-ordinates Analysis (see \code{\link{wcmdscale}} for
    details). Choice \code{"lingoes"} (or \code{TRUE}) use the
    recommended method of Legendre & Anderson (1999: \dQuote{method
    1}) and \code{"cailliez"} uses their \dQuote{method 2}.}
  \item{display}{character; partial match to access scores for
    \code{"sites"} or \code{"species"}.}
  \item{object, x}{an object of class \code{"betadisper"}, the result of a
    call to \code{betadisper}.}
  \item{choices, axes}{the principal coordinate axes wanted.}
  \item{hull}{logical; should the convex hull for each group be plotted?}
  \item{ellipse}{logical; should the standard deviation data ellipse for
    each group be plotted?}
  \item{conf}{Expected fractions of data coverage for data ellipses,
    e.g. 0.95. The default is to draw a 1 standard deviation data
    ellipse, but if supplied, \code{conf} is multiplied with the
    corresponding value found from the Chi-squared distribution with 2df
    to provide the requested coverage (probability contour).}
  \item{pch}{plot symbols for the groups, a vector of length equal to
    the number of groups.}
  \item{col}{colors for the plot symbols for the groups, a vector of
    length equal to the number of groups.}
  \item{lty, lwd}{linetype, linewidth for convex hulls and confidence
    ellipses.}
  \item{segments}{logical; should segments joining points to their
    centroid be drawn?}
  \item{seg.col}{colour to draw segments between points and their
    centroid. Can be a vector, in which case one colour per group.}
  \item{seg.lty, seg.lwd}{linetype and line width for segments.}
  \item{label}{logical; should the centroids by labelled with their
    respective factor label?}
  \item{label.cex}{numeric; character expansion for centroid labels.}
  \item{cex, ylab, xlab, main, sub}{graphical parameters. For details,
    see \code{\link{plot.default}}.}
  \item{which}{A character vector listing terms in the fitted model for
    which the intervals should be calculated. Defaults to the grouping
    factor.}
  \item{ordered}{logical; see \code{\link{TukeyHSD}}.}
  \item{conf.level}{A numeric value between zero and one giving the
    family-wise confidence level to use.}
  \item{digits, neigen}{numeric; for the \code{print} method, sets the number of digits to use (as per \code{\link{print.default}}) and the maximum number of axes to display eigenvalues for, repsectively.}
  \item{\dots}{arguments, including graphical parameters (for
    \code{plot.betadisper} and \code{boxplot.betadisper}), passed to
    other methods.}
}
\details{
  One measure of multivariate dispersion (variance) for a group of
  samples is to calculate the average distance of group members to the
  group centroid or spatial median (both referred to as 'centroid' from
  now on unless stated otherwise) in multivariate space. To test if the
  dispersions (variances) of one or more groups are different, the
  distances of group members to the group centroid are subject to
  ANOVA. This is a multivariate analogue of Levene's test for
  homogeneity of variances if the distances between group members and
  group centroids is the Euclidean distance.

  However, better measures of distance than the Euclidean distance are
  available for ecological data. These can be accommodated by reducing
  the distances produced using any dissimilarity coefficient to
  principal coordinates, which embeds them within a Euclidean space. The
  analysis then proceeds by calculating the Euclidean distances between
  group members and the group centroid on the basis of the principal
  coordinate axes rather than the original distances.
  
  Non-metric dissimilarity coefficients can produce principal coordinate
  axes that have negative Eigenvalues. These correspond to the
  imaginary, non-metric part of the distance between objects. If
  negative Eigenvalues are produced, we must correct for these imaginary
  distances.

  The distance to its centroid of a point is \deqn{z_{ij}^c =
  \sqrt{\Delta^2(u_{ij}^+, c_i^+) - \Delta^2(u_{ij}^-, c_i^-)},}{z[ij]^c
  = sqrt(Delta^2(u[ij]^+, c[i]^+) - Delta^2(u[ij]^-, c[i]^-)),} where
  \eqn{\Delta^2}{Delta^2} is the squared Euclidean distance between
  \eqn{u_{ij}}{u[ij]}, the principal coordinate for the \eqn{j}th
  point in the \eqn{i}th group, and \eqn{c_i}{c[i]}, the
  coordinate of the centroid for the \eqn{i}th group. The
  super-scripted \sQuote{\eqn{+}} and \sQuote{\eqn{-}} indicate the
  real and imaginary parts respectively. This is equation (3) in
  Anderson (2006). If the imaginary part is greater in magnitude than
  the real part, then we would be taking the square root of a negative
  value, resulting in NaN, and these cases are changed to zero distances
  (with a warning). This is in line with the behaviour of Marti Anderson's
  PERMDISP2 programme.
  
  To test if one or more groups is more variable than the others, ANOVA
  of the distances to group centroids can be performed and parametric
  theory used to interpret the significance of \eqn{F}. An alternative is to
  use a permutation test. \code{\link{permutest.betadisper}} permutes model
  residuals to generate a permutation distribution of \eqn{F} under the Null
  hypothesis of no difference in dispersion between groups.

  Pairwise comparisons of group mean dispersions can also be performed
  using \code{\link{permutest.betadisper}}. An alternative to the classical
  comparison of group dispersions, is to calculate Tukey's Honest
  Significant Differences between groups, via
  \code{TukeyHSD.betadisper}. This is a simple wrapper to
  \code{\link{TukeyHSD}}. The user is directed to read the help file
  for \code{\link{TukeyHSD}} before using this function. In particular,
  note the statement about using the function with 
  unbalanced designs.

  The results of the analysis can be visualised using the \code{plot}
  and \code{boxplot} methods.

  One additional use of these functions is in assessing beta diversity
  (Anderson \emph{et al} 2006). Function \code{\link{betadiver}}
  provides some popular dissimilarity measures for this purpose.

  As noted in passing by Anderson (2006) and in a related
  context by O'Neill (2000), estimates of dispersion around a
  central location (median or centroid) that is calculated from the same data
  will be biased downward. This bias matters most when comparing diversity
  among treatments with small, unequal numbers of samples.  Setting
  \code{bias.adjust=TRUE} when using \code{betadisper} imposes a 
  \eqn{\sqrt{n/(n-1)}}{sqrt(n/(n-1))} correction (Stier et al. 2013).
}
\value{
  The \code{anova} method returns an object of class \code{"anova"}
  inheriting from class \code{"data.frame"}.

  The \code{scores} method returns a list with one or both of the
  components \code{"sites"} and \code{"centroids"}.
  
  The \code{plot} function invisibly returns an object of class
  \code{"ordiplot"}, a plotting structure which can be used by
  \code{\link{identify.ordiplot}} (to identify the points) or other
  functions in the \code{\link{ordiplot}} family. 

  The \code{boxplot} function invisibly returns a list whose components
  are documented in \code{\link{boxplot}}.

  \code{eigenvals.betadisper} returns a named vector of eigenvalues.

  \code{TukeyHSD.betadisper} returns a list. See \code{\link{TukeyHSD}}
  for further details.

  \code{betadisper} returns a list of class \code{"betadisper"} with the
  following components:

  \item{eig}{numeric; the eigenvalues of the principal coordinates
    analysis.}
  \item{vectors}{matrix; the eigenvectors of the principal coordinates
    analysis.}
  \item{distances}{numeric; the Euclidean distances in principal
    coordinate space between the samples and their respective group
    centroid.}
  \item{group}{factor; vector describing the group structure}
  \item{centroids}{matrix; the locations of the group centroids on the
    principal coordinates.}
  \item{call}{the matched function call.}
}
\note{
  If \code{group} consists of a single level or group, then the
  \code{anova} and \code{permutest} methods are not appropriate and if
  used on such data will stop with an error.

  Missing values in either \code{d} or \code{group} will be removed
  prior to performing the analysis.
}
\section{Warning}{
  Stewart Schultz noticed that the permutation test for
  \code{type="centroid"} had the wrong type I error and was
  anti-conservative. As such, the default for \code{type} has been
  changed to \code{"median"}, which uses the spatial median as the group
  centroid. Tests suggests that the permutation test for this type of
  analysis gives the correct error rates.
}
\references{
  Anderson, M.J. (2006) Distance-based tests for homogeneity of
  multivariate dispersions. \emph{Biometrics} \strong{62}, 245--253.

  Anderson, M.J., Ellingsen, K.E. & McArdle, B.H. (2006) Multivariate
  dispersion as a measure of beta diversity. \emph{Ecology Letters}
  \strong{9}, 683--693.

  O'Neill, M.E. (2000) A Weighted Least Squares Approach to Levene's 
  Test of Homogeneity of Variance. \emph{Australian & New Zealand Journal of 
  Statistics} \strong{42}, 81-–100.

  Stier, A.C., Geange, S.W., Hanson, K.M., & Bolker, B.M. (2013) Predator 
  density and timing of arrival affect reef fish community
  assembly. \emph{Ecology} \strong{94}, 1057--1068.
}
\author{Gavin L. Simpson; bias correction by Adrian Stier and Ben Bolker.}
\seealso{\code{\link{permutest.betadisper}}, \code{\link[stats]{anova.lm}},
  \code{\link{scores}}, \code{\link{boxplot}},
  \code{\link{TukeyHSD}}. Further measure of beta diversity
  can be found in \code{\link{betadiver}}.}
\examples{
data(varespec)

## Bray-Curtis distances between samples
dis <- vegdist(varespec)

## First 16 sites grazed, remaining 8 sites ungrazed
groups <- factor(c(rep(1,16), rep(2,8)), labels = c("grazed","ungrazed"))

## Calculate multivariate dispersions
mod <- betadisper(dis, groups)
mod

## Perform test
anova(mod)

## Permutation test for F
permutest(mod, pairwise = TRUE, permutations = 99)

## Tukey's Honest Significant Differences
(mod.HSD <- TukeyHSD(mod))
plot(mod.HSD)

## Plot the groups and distances to centroids on the
## first two PCoA axes
plot(mod)

## with data ellipses instead of hulls
plot(mod, ellipse = TRUE, hull = FALSE) # 1 sd data ellipse
plot(mod, ellipse = TRUE, hull = FALSE, conf = 0.90) # 90% data ellipse

## can also specify which axes to plot, ordering respected
plot(mod, axes = c(3,1), seg.col = "forestgreen", seg.lty = "dashed")

## Draw a boxplot of the distances to centroid for each group
boxplot(mod)

## `scores` and `eigenvals` also work
scrs <- scores(mod)
str(scrs)
head(scores(mod, 1:4, display = "sites"))
# group centroids/medians 
scores(mod, 1:4, display = "centroids")
# eigenvalues from the underlying principal coordinates analysis
eigenvals(mod) 

## try out bias correction; compare with mod3
(mod3B <- betadisper(dis, groups, type = "median", bias.adjust=TRUE))
anova(mod3B)
permutest(mod3B, permutations = 99)

## should always work for a single group
group <- factor(rep("grazed", NROW(varespec)))
(tmp <- betadisper(dis, group, type = "median"))
(tmp <- betadisper(dis, group, type = "centroid"))

## simulate missing values in 'd' and 'group'
## using spatial medians
groups[c(2,20)] <- NA
dis[c(2, 20)] <- NA
mod2 <- betadisper(dis, groups) ## messages
mod2
permutest(mod2, permutations = 99)
anova(mod2)
plot(mod2)
boxplot(mod2)
plot(TukeyHSD(mod2))

## Using group centroids
mod3 <- betadisper(dis, groups, type = "centroid")
mod3
permutest(mod3, permutations = 99)
anova(mod3)
plot(mod3)
boxplot(mod3)
plot(TukeyHSD(mod3))

}
\keyword{methods}
\keyword{multivariate}
\keyword{hplot}

\encoding{UTF-8}
\name{betadiver}
\alias{betadiver}
\alias{scores.betadiver}
\alias{plot.betadiver}

\title{ Indices of beta Diversity }
\description{
  The function estimates any of the 24 indices of beta diversity
  reviewed by Koleff et al. (2003). Alternatively, it finds the
  co-occurrence frequencies for triangular plots (Koleff et
  al. 2003). 
}

\usage{
betadiver(x, method = NA, order = FALSE, help = FALSE, ...)
\method{plot}{betadiver}(x, ...)
\method{scores}{betadiver}(x, triangular = TRUE, ...)
}

\arguments{
  \item{x}{Community data matrix, or the \code{betadiver} result for
  \code{plot} and \code{scores} functions. }
  \item{method}{The index of beta diversity as defined in Koleff et al.
  (2003), Table 1. You can use either the subscript of \eqn{\beta} or
  the number of the index. See argument \code{help} below. }
  \item{order}{Order sites by increasing number of species. This will
  influence the configuration in the triangular plot and non-symmetric
  indices. }
  \item{help}{Show the numbers, subscript names and the defining
    equations of the indices and exit.}
  \item{triangular}{Return scores suitable for triangular plotting of
  proportions. If \code{FALSE}, returns a 3-column matrix of raw counts.}
  \item{\dots}{ Other arguments to functions. }
}

\details{ The most commonly used index of beta diversity is
  \eqn{\beta_w = S/\alpha - 1}, where \eqn{S} is the total number of
  species, and \eqn{\alpha} is the average number of species per site
  (Whittaker 1960). A drawback of this model is that \eqn{S} increases
  with sample size, but the expectation of \eqn{\alpha} remains
  constant, and so the beta diversity increases with sample size. A
  solution to this problem is to study the beta diversity of pairs of
  sites (Marion et al. 2017). If we denote the number of species
  shared between two sites as \eqn{a} and the numbers of unique
  species (not shared) as \eqn{b} and \eqn{c}, then \eqn{S = a + b +
  c} and \eqn{\alpha = (2 a + b + c)/2} so that \eqn{\beta_w =
  (b+c)/(2 a + b + c)}. This is the \enc{Sørensen}{Sorensen}
  dissimilarity as defined in \pkg{vegan} function
  \code{\link{vegdist}} with argument \code{binary = TRUE}. Many other
  indices are dissimilarity indices as well.

  Function \code{betadiver} finds all indices reviewed by Koleff et
  al. (2003). All these indices could be found with function
  \code{\link{designdist}}, but the current function provides a
  conventional shortcut. The function only finds the indices. The proper
  analysis must be done with functions such as \code{\link{betadisper}},
  \code{\link{adonis2}} or \code{\link{mantel}}.

  The indices are directly taken from Table 1 of Koleff et al. (2003),
  and they can be selected either by the index number or the subscript
  name used by Koleff et al. The numbers, names and defining equations
  can be seen using \code{betadiver(help = TRUE)}. In all cases where
  there are two alternative forms, the one with the term \eqn{-1} is
  used. There are several duplicate indices, and the number of distinct
  alternatives is much lower than 24 formally provided. The formulations
  used in functions differ occasionally from those in Koleff et
  al. (2003), but they are still mathematically equivalent. With
  \code{method = NA}, no index is calculated, but instead an object of
  class \code{betadiver} is returned. This is a list of elements
  \code{a}, \code{b} and \code{c}. Function \code{plot} can be used to
  display the proportions of these elements in triangular plot as
  suggested by Koleff et al. (2003), and \code{scores} extracts the
  triangular coordinates or the raw scores. Function \code{plot} returns
  invisibly the triangular coordinates as an \code{"\link{ordiplot}"}
  object. 
}

\value{ With \code{method = NA}, the function returns an object of
  class \code{"betadisper"} with elements \code{a}, \code{b}, and
  \code{c}. If \code{method} is specified, the function returns a
  \code{"dist"} object which can be used in any function analysing
  dissimilarities. For beta diversity, particularly useful functions
  are \code{\link{betadisper}} to study the betadiversity in groups,
  \code{\link{adonis2}} for any model, and \code{\link{mantel}} to
  compare beta diversities to other dissimilarities or distances
  (including geographical distances). Although \code{betadiver}
  returns a \code{"dist"} object, some indices are similarities and
  cannot be used as such in place of dissimilarities, but that is a
  severe user error. Functions 10 (\code{"j"}) and 11 (\code{"sor"})
  are two such similarity indices.
}

\references{

  Baselga, A. (2010) Partitioning the turnover and nestedness
  components of beta diversity. \emph{Global Ecology and Biogeography}
  19, 134--143.

  Koleff, P., Gaston, K.J. and Lennon, J.J. (2003) Measuring beta
  diversity for presence-absence data. \emph{Journal of Animal
  Ecology} 72, 367--382.

  Marion, Z.H., Fordyce, J.A. and Fitzpatrick, B.M. (2017) Pairwise
  beta diversity resolves an underappreciated source of confusion in
  calculating species turnover. \emph{Ecology} 98, 933--939.

  Whittaker, R.H. (1960) Vegetation of Siskiyou mountains, Oregon and
  California. \emph{Ecological Monographs} 30, 279--338.

 }

\author{Jari Oksanen }
\section{Warning }{Some indices return similarities instead of dissimilarities.} 

\seealso{ \code{\link{designdist}} can be used to implement all these
  functions, and also allows using notation with \code{alpha} and
  \code{gamma} diversities.  \code{\link{vegdist}} has some canned
  alternatives.  Functions \code{\link{betadisper}},
  \code{\link{adonis2}} and \code{\link{mantel}} can be used for
  analysing beta diversity objects. The returned dissimilarities can
  be used in any distance-based methods, such as
  \code{\link{metaMDS}}, \code{\link{capscale}} and
  \code{\link{dbrda}}. Functions \code{\link{nestedbetasor}} and
  \code{\link{nestedbetajac}} implement decomposition beta diversity
  measures (\enc{Sørensen}{Sorensen} and Jaccard) into turnover and
  nestedness components following Baselga (2010).  }


\examples{
## Raw data and plotting
data(sipoo)
m <- betadiver(sipoo)
plot(m)
## The indices
betadiver(help=TRUE)
## The basic Whittaker index
d <- betadiver(sipoo, "w")
## This should be equal to Sorensen index (binary Bray-Curtis in
## vegan)
range(d - vegdist(sipoo, binary=TRUE))
}

\keyword{ multivariate }

\encoding{UTF-8}
\name{multipart}
\alias{multipart}
\alias{multipart.default}
\alias{multipart.formula}
\title{Multiplicative Diversity Partitioning}

\description{
In multiplicative diversity partitioning, mean values of alpha diversity at lower levels of a sampling
hierarchy are compared to the total diversity in the entire data set or the pooled samples (gamma diversity).
}
\usage{
multipart(...)
\method{multipart}{default}(y, x, index=c("renyi", "tsallis"), scales = 1,
    global = FALSE, relative = FALSE, nsimul=99, method = "r2dtable", ...)
\method{multipart}{formula}(formula, data, index=c("renyi", "tsallis"), scales = 1,
    global = FALSE, relative = FALSE, nsimul=99, method = "r2dtable", ...)
}
\arguments{
  \item{y}{A community matrix.}

  \item{x}{A matrix with same number of rows as in \code{y}, columns
    coding the levels of sampling hierarchy. The number of groups within
    the hierarchy must decrease from left to right. If \code{x} is missing,
    two levels are assumed: each row is a group in the first level, and
    all rows are in the same group in the second level.}

  \item{formula}{A two sided model formula in the form \code{y ~ x},
    where \code{y} is the community data matrix with samples as rows and
    species as column. Right hand side (\code{x}) must be grouping
    variable(s) referring to levels of sampling hierarchy, terms from
    right to left will be treated as nested (first column is the lowest,
    last is the highest level). The formula will add a unique
    indentifier to rows and constant for the rows to always produce
    estimates of row-level alpha and overall gamma diversities. You must
    use non-formula interface to avoid this behaviour. Interaction terms
    are not allowed.}

  \item{data}{A data frame where to look for variables defined in the
    right hand side of \code{formula}. If missing, variables are looked
    in the global environment.}

  \item{index}{Character, the entropy index to be calculated (see Details).}

  \item{relative}{Logical, if \code{TRUE} then beta diversity is
    standardized by its maximum (see Details).}

  \item{scales}{Numeric, of length 1, the order of the generalized
    diversity index to be used.}

  \item{global}{Logical, indicates the calculation of beta diversity values,
    see Details.}

  \item{nsimul}{Number of permutations to use.  If \code{nsimul = 0},
    only the \code{FUN} argument is evaluated.
    It is thus possible to reuse the statistic values
    without a null model.}

  \item{method}{Null model method: either a name (character string) of
    a method defined in \code{\link{make.commsim}} or a
    \code{\link{commsim}} function.
    The default \code{"r2dtable"} keeps row sums and column sums fixed.
    See \code{\link{oecosimu}} for Details and Examples.}

  \item{\dots}{Other arguments passed to \code{\link{oecosimu}}, i.e.
    \code{method}, \code{thin} or \code{burnin}.}
}
\details{
Multiplicative diversity partitioning is based on Whittaker's (1972) ideas,
that has recently been generalised to one parametric diversity families
(i.e. \enc{Rényi}{Renyi} and Tsallis) by Jost (2006, 2007).
Jost recommends to use the numbers equivalents (Hill numbers),
instead of pure diversities, and proofs, that this satisfies the
multiplicative partitioning requirements.

The current implementation of \code{multipart} calculates Hill numbers
based on the functions \code{\link{renyi}} and \code{\link{tsallis}}
(provided as \code{index} argument).
If values for more than one \code{scales} are desired,
it should be done in separate runs, because it adds extra dimensionality
to the implementation, which has not been resolved efficiently.

Alpha diversities are then the averages of these Hill numbers for
each hierarchy levels, the global gamma diversity is the alpha value
calculated for the highest hierarchy level.
When \code{global = TRUE}, beta is calculated relative to the global gamma value:
\deqn{\beta_i = \gamma / \alpha_{i}}{beta_i = gamma / alpha_i}
when \code{global = FALSE}, beta is calculated relative to local
gamma values (local gamma means the diversity calculated for a particular
cluster based on the pooled abundance vector):
\deqn{\beta_ij = \alpha_{(i+1)j} / mean(\alpha_{ij})}{beta_ij = alpha_(i+1)j / mean(alpha_i)}
where \eqn{j} is a particular cluster at hierarchy level \eqn{i}.
Then beta diversity value for level \eqn{i} is the mean of the beta
values of the clusters at that level, \eqn{\beta_{i} = mean(\beta_{ij})}.

If \code{relative = TRUE}, the respective beta diversity values are
standardized by their maximum possible values (\eqn{mean(\beta_{ij}) / \beta_{max,ij}})
given as \eqn{\beta_{max,ij} = n_{j}} (the number of lower level units
in a given cluster \eqn{j}).

The expected diversity components are calculated \code{nsimul}
times by individual based randomization of the community data matrix.
This is done by the \code{"r2dtable"} method in \code{\link{oecosimu}} by default.
}
\value{
An object of class \code{"multipart"} with same structure as
\code{"oecosimu"} objects.
}
\references{
Jost, L. (2006). Entropy and diversity.
\emph{Oikos}, \bold{113}, 363--375.

Jost, L. (2007). Partitioning diversity into independent alpha and beta components.
\emph{Ecology}, \bold{88}, 2427--2439.

Whittaker, R. (1972). Evolution and measurement of species diversity.
\emph{Taxon}, \bold{21}, 213--251.
}
\author{\enc{Péter Sólymos}{Peter Solymos}, \email{solymos@ualberta.ca}}
\seealso{See \code{\link{adipart}} for additive diversity partitioning,
  \code{\link{hiersimu}} for hierarchical null model testing
  and \code{\link{oecosimu}} for permutation settings and calculating \eqn{p}-values.}
\examples{
## NOTE: 'nsimul' argument usually needs to be >= 99
## here much lower value is used for demonstration

data(mite)
data(mite.xy)
data(mite.env)
## Function to get equal area partitions of the mite data
cutter <- function (x, cut = seq(0, 10, by = 2.5)) {
    out <- rep(1, length(x))
    for (i in 2:(length(cut) - 1))
        out[which(x > cut[i] & x <= cut[(i + 1)])] <- i
    return(out)}
## The hierarchy of sample aggregation
levsm <- with(mite.xy, data.frame(
    l2=cutter(y, cut = seq(0, 10, by = 2.5)),
    l3=cutter(y, cut = seq(0, 10, by = 5))))
## Multiplicative diversity partitioning
multipart(mite, levsm, index="renyi", scales=1, nsimul=19)
multipart(mite ~ l2 + l3, levsm, index="renyi", scales=1, nsimul=19)
multipart(mite ~ ., levsm, index="renyi", scales=1, nsimul=19, relative=TRUE)
multipart(mite ~ ., levsm, index="renyi", scales=1, nsimul=19, global=TRUE)
}
\keyword{multivariate}

\encoding{UTF-8}
\name{dispweight}
\alias{dispweight}
\alias{gdispweight}
\alias{summary.dispweight}
\title{Dispersion-based weighting of species counts}

\description{Transform abundance data downweighting species that are 
  overdispersed to the Poisson error.}

\usage{
dispweight(comm, groups, nsimul = 999, nullmodel = "c0_ind",
    plimit = 0.05)
gdispweight(formula, data, plimit = 0.05)
\method{summary}{dispweight}(object, ...)
}

\arguments{
  \item{comm}{Community data matrix.}
  \item{groups}{Factor describing the group structure. If missing, all 
     sites are regarded as belonging to one group. \code{NA} values are 
     not allowed.}
  \item{nsimul}{Number of simulations.}
  \item{nullmodel}{The \code{\link{nullmodel}} used in
    \code{\link{commsim}} within \code{groups}. The default
    follows Clarke et al. (2006).}
  \item{plimit}{Downweight species if their \eqn{p}-value is at or
     below this limit.}
  \item{formula, data}{Formula where the left-hand side is the
    community data frame and right-hand side gives the explanatory
    variables. The explanatory variables are found in the data frame
    given in \code{data} or in the parent frame.}
  \item{object}{Result object from \code{dispweight} or
    \code{gdispweight}.}
  \item{\dots}{Other parameters passed to functions.}
}

\details{

The dispersion index (\eqn{D}) is calculated as ratio between variance
and expected value for each species.  If the species abundances follow
Poisson distribution, expected dispersion is \eqn{E(D) = 1}, and if
\eqn{D > 1}, the species is overdispersed. The inverse \eqn{1/D} can
be used to downweight species abundances.  Species are only
downweighted when overdispersion is judged to be statistically
significant (Clarke et al. 2006).

Function \code{dispweight} implements the original procedure of Clarke
et al. (2006). Only one factor can be used to group the sites and to
find the species means. The significance of overdispersion is assessed
freely distributing individuals of each species within factor
levels. This is achieved by using \code{\link{nullmodel}}
\code{"c0_ind"} (which accords to Clarke et al. 2006), but other
nullmodels can be used, though they may not be meaningful (see
\code{\link{commsim}} for alternatives). If a species is absent in
some factor level, the whole level is ignored in calculation of
overdispersion, and the number of degrees of freedom can vary among
species. The reduced number of degrees of freedom is used as a divisor
for overdispersion \eqn{D}, and such species have higher dispersion
and hence lower weights in transformation.

Function \code{gdispweight} is a generalized parametric version of
\code{dispweight}. The function is based on \code{\link{glm}} with
\code{\link{quasipoisson}} error \code{\link{family}}. Any
\code{\link{glm}} model can be used, including several factors or
continuous covariates. Function \code{gdispweight} uses the same test
statistic as \code{dispweight} (Pearson Chi-square), but it does not
ignore factor levels where species is absent, and the number of
degrees of freedom is equal for all species. Therefore transformation
weights can be higher than in \code{dispweight}. The
\code{gdispweight} function evaluates the significance of
overdispersion parametrically from Chi-square distribution
(\code{\link{pchisq}}).

Functions \code{dispweight} and \code{gdispweight} transform data, but
they add information on overdispersion and weights as attributes of
the result. The \code{summary} can be used to extract and print that
information.  
}

\value{
Function returns transformed data with the following new attributes:
    \item{D}{Dispersion statistic.}
    \item{df}{Degrees of freedom for each species.}
    \item{p}{\eqn{p}-value of the Dispersion statistic \eqn{D}.}
    \item{weights}{weights applied to community data.}
    \item{nsimul}{Number of simulations used to assess the \eqn{p}-value,
      or \code{NA} when simulations were not performed.}
    \item{nullmodel}{The name of \code{\link{commsim}} null model, or
      \code{NA} when simulations were not performed.}
}

\references{
Clarke, K. R., M. G. Chapman, P. J. Somerfield, and
H. R. Needham. 2006. Dispersion-based weighting of species counts in
assemblage analyses. \emph{Marine Ecology Progress Series}, 320,
11–27.
}

\author{
  Eduard Szöcs \email{eduardszoesc@gmail.com} wrote the original
  \code{dispweight}, Jari Oksanen significantly modified the code,
  provided support functions and developed \code{gdispweight}.
}


\examples{
data(mite, mite.env)
## dispweight and its summary
mite.dw <- with(mite.env, dispweight(mite, Shrub, nsimul = 99))
summary(mite.dw)
## generalized dispersion weighting
mite.dw <- gdispweight(mite ~ Shrub + WatrCont, data = mite.env)
rda(mite.dw ~ Shrub + WatrCont, data = mite.env)
}


\keyword{multivariate}
\keyword{manip}

\name{make.cepnames}
\alias{make.cepnames}

\title{Abbreviates a Botanical or Zoological Latin Name into an Eight-character Name} 
\description{
  A standard CEP name has four first letters of the generic name and
  four first letters of the specific epithet of a Latin name. The last
  epithet, that may be a subspecific name, is used in the current
  function. If the name has only one component, it is abbreviated to
  eight characters (see \code{\link{abbreviate}}).
  The returned names are made unique with function
  \code{\link{make.unique}} which adds numbers to the end of CEP names if needed.
}
\usage{
make.cepnames(names, seconditem = FALSE)
}
\arguments{
  \item{names}{The names to be formatted into CEP names. }
  \item{seconditem}{Take always the second item of the original name
    to the abbreviated name instead of the last original item
    (default).}
}

\details{ Cornell Ecology Programs (CEP) used eight-letter
  abbreviations for species and site names. In species, the names were
  formed by taking four first letters of the generic name and four
  first letters of the specific or subspecific epithet. The current
  function first makes valid \R names using \code{\link{make.names}},
  and then splits these into elements. The CEP name is made by taking
  the four first letters of the first element, and four first letters
  of the last (default) or the second element (with 
  \code{seconditem = TRUE}). If there was only one name element, it is
  \code{\link{abbreviate}}d to eight letters. Finally, the names are
  made unique which may add numbers to duplicated names.
  
  The CEP names were originally used, because old \code{FORTRAN IV}
  did not have \code{CHARACTER} data type, but text had to be stored
  in numerical variables, which in popular computers could hold four
  characters. In modern times, there is no reason for this limitation,
  but ecologists are used to these names, and they may be practical to
  avoid congestion in ordination plots.  
}

\value{
  Function returns CEP names.
}
\author{ Jari Oksanen }
\note{
  The function is simpleminded and rigid. You must write a better one if
  you need.  
}
\seealso{
  \code{\link{make.names}}, \code{\link{strsplit}},
  \code{\link{substring}}, \code{\link{paste}}, \code{\link{abbreviate}}.
  }
\examples{
make.cepnames(c("Aa maderoi", "Poa sp.", "Cladina rangiferina",
"Cladonia cornuta", "Cladonia cornuta var. groenlandica",
"Cladonia rangiformis", "Bryoerythrophyllum"))
data(BCI)
colnames(BCI) <- make.cepnames(colnames(BCI))
}
\keyword{ character }

\name{monoMDS}
\alias{monoMDS}
\alias{scores.monoMDS}
\alias{plot.monoMDS}
\alias{points.monoMDS}
\alias{text.monoMDS}

\title{ Global and Local Non-metric Multidimensional Scaling and
  Linear and Hybrid Scaling }

\description{ Function implements Kruskal's (1964a,b) non-metric
  multidimensional scaling (NMDS) using monotone regression and
  primary (\dQuote{weak}) treatment of ties. In addition to
  traditional global NMDS, the function implements local NMDS, linear
  and hybrid multidimensional scaling.
}

\usage{
monoMDS(dist, y, k = 2, model = c("global", "local", "linear", "hybrid"),
    threshold = 0.8, maxit = 200, weakties = TRUE, stress = 1,
    scaling = TRUE, pc = TRUE, smin = 1e-4, sfgrmin = 1e-7,
    sratmax=0.99999, ...)
\method{scores}{monoMDS}(x, choices = NA, ...)
\method{plot}{monoMDS}(x, choices = c(1,2), type = "t", ...)
\method{points}{monoMDS}(x, choices = c(1,2), select, ...)
\method{text}{monoMDS}(x, labels, choices = c(1,2), select, ...)
}

\arguments{
  \item{dist}{Input dissimilarities.}
  \item{y}{Starting configuration. A random configuration will be
    generated if this is missing.}

  \item{k}{Number of dimensions. NB., the number of points \eqn{n}
    should be \eqn{n > 2k + 1}{n > 2*k + 1}, and preferably higher in
    non-metric MDS.}

  \item{model}{MDS model: \code{"global"} is normal non-metric MDS
    with a monotone regression, \code{"local"} is non-metric MDS with
    separate regressions for each point, \code{"linear"} uses linear
    regression, and \code{"hybrid"} uses linear regression for
    dissimilarities below a threshold in addition to monotone
    regression. See Details.}

  \item{threshold}{Dissimilarity below which linear regression is
    used alternately with monotone regression. }

  \item{maxit}{Maximum number of iterations.}

  \item{weakties}{Use primary or weak tie treatment, where equal
    observed dissimilarities are allowed to have different fitted
    values. if \code{FALSE}, then secondary (strong) tie treatment is
    used, and tied values are not broken.}

  \item{stress}{Use stress type 1 or 2 (see Details).}

  \item{scaling}{Scale final scores to unit root mean squares.}

  \item{pc}{Rotate final scores to principal components.}

  \item{smin, sfgrmin, sratmax}{Convergence criteria: iterations stop
    when stress drops below \code{smin}, scale factor of the gradient
    drops below \code{sfgrmin}, or stress ratio between two iterations
    goes over \code{sratmax} (but is still \eqn{< 1}).}

  \item{x}{A \code{monoMDS} result.}

  \item{choices}{Dimensions returned or plotted. The default \code{NA}
    returns all dimensions. }

  \item{type}{The type of the plot: \code{"t"} for text, \code{"p"}
    for points, and \code{"n"} for none.}

  \item{select}{Items to be displayed.  This can either be a logical
          vector which is \code{TRUE} for displayed items or a vector of
          indices of displayed items.}

  \item{labels}{Labels to be use used instead of row names.}

  \item{\dots}{Other parameters to the functions (ignored in
    \code{monoMDS}, passed to graphical functions in \code{plot}.).}
}

\details{ There are several versions of non-metric multidimensional
  scaling in \R, but \code{monoMDS} offers the following unique
  combination of features:
  \itemize{

    \item \dQuote{Weak} treatment of ties (Kruskal 1964a,b), where tied
    dissimilarities can be broken in monotone regression. This is
    especially important for cases where compared sites share no species
    and dissimilarities are tied to their maximum value of one. Breaking
    ties allows these points to be at different distances and can help
    in recovering very long coenoclines (gradients).  Functions in the
    \CRANpkg{smacof} package also hav adequate tie treatment.

    \item Handles missing values in a meaningful way.

    \item Offers \dQuote{local} and \dQuote{hybrid} scaling in
    addition to usual \dQuote{global} NMDS (see below).

    \item Uses fast compiled code (\code{\link[MASS]{isoMDS}} of the
    \pkg{MASS} package also uses compiled code).

 }

  Function \code{monoMDS} uses Kruskal's (1964b) original monotone
  regression to minimize the stress. There are two alternatives of
  stress: Kruskal's (1964a,b) original or \dQuote{stress 1} and an
  alternative version or \dQuote{stress 2} (Sibson 1972). Both of
  these stresses can be expressed with a general formula

\deqn{s^2 = \frac{\sum (d - \hat d)^2}{\sum(d - d_0)^2}}{stress^2 = sum (d-dhat)^2/ sum (d-dnull)^2}

  where \eqn{d} are distances among points in ordination configuration,
  \eqn{\hat d}{dhat} are the fitted ordination distances, and
  \eqn{d_0}{dnull} are the ordination distances under null model.  For
  \dQuote{stress 1} \eqn{d_0 = 0}{dnull = 0}, and for \dQuote{stress 2}
  \eqn{d_0 = \bar{d}}{dnull = dbar} or mean distances. \dQuote{Stress 2}
  can be expressed as \eqn{s^2 = 1 - R^2}{stress^2 = 1 - R2},
  where\eqn{R^2}{R2} is squared correlation between fitted values and
  ordination distances, and so related to the \dQuote{linear fit} of
  \code{\link{stressplot}}.

  Function \code{monoMDS} can fit several alternative NMDS variants that
  can be selected with argument \code{model}.  The default \code{model =
  "global"} fits global NMDS, or Kruskal's (1964a,b) original NMDS
  similar to \code{\link[MASS]{isoMDS}} (\pkg{MASS}).  Alternative
  \code{model = "local"} fits local NMDS where independent monotone
  regression is used for each point (Sibson 1972).  Alternative
  \code{model = "linear"} fits a linear MDS. This fits a linear
  regression instead of monotone, and is not identical to metric scaling
  or principal coordinates analysis (\code{\link{cmdscale}}) that
  performs an eigenvector decomposition of dissimilarities (Gower
  1966). Alternative \code{model = "hybrid"} implements hybrid MDS that
  uses monotone regression for all points and linear regression for
  dissimilarities below or at a \code{threshold} dissimilarity in
  alternating steps (Faith et al. 1987). Function
  \code{\link{stressplot}} can be used to display the kind of regression
  in each \code{model}.

  Scaling, orientation and direction of the axes is arbitrary.
  However, the function always centres the axes, and the default
  \code{scaling} is to scale the configuration of unit root mean
  square and to rotate the axes (argument \code{pc}) to principal
  components so that the first dimension shows the major variation.
  It is possible to rotate the solution so that the first axis is
  parallel to a given environmental variable using function
  \code{\link{MDSrotate}}.

}

\section{Convergence Criteria}{

  NMDS is iterative, and the function stops when any of its
  convergence criteria is met. There is actually no criterion of
  assured convergence, and any solution can be a local optimum. You
  should compare several random starts (or use \code{monoMDS} via
  \code{\link{metaMDS}}) to assess if the solutions is likely a global
  optimum.

  The stopping criteria are:
  \describe{

     \item{\code{maxit}:}{ Maximum number of iterations. Reaching this
     criterion means that solutions was almost certainly not found,
     and \code{maxit} should be increased.}

     \item{\code{smin}:}{ Minimum stress. If stress is nearly zero,
     the fit is almost perfect. Usually this means that data set is
     too small for the requested analysis, and there may be several
     different solutions that are almost as perfect. You should reduce
     the number of dimensions (\code{k}), get more data (more
     observations) or use some other method, such as metric scaling
     (\code{\link{cmdscale}}, \code{\link{wcmdscale}}).}

     \item{\code{sratmax}:}{ Change in stress. Values close to one
     mean almost unchanged stress. This may mean a solution, but it
     can also signal stranding on suboptimal solution with flat stress
     surface.}

     \item{\code{sfgrmin}:}{ Minimum scale factor. Values close to
     zero mean almost unchanged configuration. This may mean a
     solution, but will also happen in local optima.}

  }

}

\value{ Returns an object of class \code{"monoMDS"}. The final scores
  are returned in item \code{points} (function \code{scores} extracts
  these results), and the stress in item \code{stress}. In addition,
  there is a large number of other items (but these may change without
  notice in the future releases). }

\references{

  Faith, D.P., Minchin, P.R and Belbin, L. 1987. Compositional
      dissimilarity as a robust measure of ecological
      distance. \emph{Vegetatio} 69, 57--68.

  Gower, J.C. (1966). Some distance properties of latent root and
      vector methods used in multivariate analysis. \emph{Biometrika}
      53, 325--328.

  Kruskal, J.B. 1964a. Multidimensional scaling by optimizing
      goodness-of-fit to a nonmetric hypothesis. \emph{Psychometrika}
      29, 1--28.

  Kruskal, J.B. 1964b. Nonmetric multidimensional scaling: a numerical
      method. \emph{Psychometrika} 29, 115--129.

  Minchin, P.R. 1987. An evaluation of relative robustness of
      techniques for ecological ordinations. \emph{Vegetatio} 69,
      89--107.

  Sibson, R. 1972. Order invariant methods for data
      analysis. \emph{Journal of the Royal Statistical Society B} 34,
      311--349.

}

\author{
Peter R. Michin (Fortran core) and Jari Oksanen (R interface).
}

\note{ This is the default NMDS function used in
   \code{\link{metaMDS}}. Function \code{\link{metaMDS}} adds support
   functions so that NMDS can be run like recommended by Minchin
   (1987).
}

\seealso{ \code{\link{metaMDS}} for the \pkg{vegan} way of running NMDS,
  and \code{\link[MASS]{isoMDS}} and \CRANpkg{smacof} for some
  alternative implementations of NMDS. }

\examples{
data(dune)
dis <- vegdist(dune)
m <- monoMDS(dis, model = "loc")
m
plot(m)
}
\keyword{ multivariate }

\name{nullmodel}
\alias{nullmodel}
\alias{simmat}
\alias{print.nullmodel}
\alias{simulate.nullmodel}
\alias{update.nullmodel}
\alias{str.nullmodel}
\alias{print.simmat}
\alias{smbind}
\title{
Null Model and Simulation
}
\description{
The \code{nullmodel} function creates an object,
which can serve as a basis for Null Model simulation
via the \code{\link{simulate}} method.
The \code{\link{update}} method updates the nullmodel
object without sampling (effective for sequential algorithms).
\code{smbind} binds together multiple \code{simmat} objects.
}
\usage{
nullmodel(x, method)
\method{print}{nullmodel}(x, ...)
\method{simulate}{nullmodel}(object, nsim = 1, seed = NULL,
    burnin = 0, thin = 1, ...)
\method{update}{nullmodel}(object, nsim = 1, seed = NULL, ...)
\method{print}{simmat}(x, ...)
smbind(object, ..., MARGIN, strict = TRUE)
}
\arguments{
  \item{x}{
A community matrix.
For the \code{print} method, it is an object to be printed.
}
  \item{method}{
Character, specifying one of the null model algorithms
listed on the help page of \code{\link{commsim}}.
It can be a user supplied object of class \code{commsim}.
}
  \item{object}{
An object of class \code{nullmodel} returned by
the function \code{nullmodel}.
In case of \code{smbind} it is a \code{simmat} object
as returned by the \code{update} or \code{simulate} methods.
}
  \item{nsim}{
Positive integer, the number of simulated matrices to return.
For the \code{update} method, it is the number of
burnin steps made for sequential algorithms
to update the status of the input model \code{object}.
}
  \item{seed}{
An object specifying if and how the random number
generator should be initialized ("seeded").
Either \code{NULL} or an integer that will be
used in a call to \code{\link{set.seed}} before
simulating the matrices.
If set, the value is saved as the
\code{"seed"} attribute of the returned value.
The default, \code{NULL} will not change the
random generator state, and return
\code{\link{.Random.seed}} as the \code{"seed"}
 attribute, see Value.
}
  \item{burnin}{
Nonnegative integer, specifying the number of steps
discarded before starting simulation.
Active only for sequential null model algorithms.
Ignored for non-sequential null model algorithms.
}
  \item{thin}{
Positive integer, number of simulation steps
made between each returned matrix.
Active only for sequential null model algorithms.
Ignored for non-sequential null model algorithms.
}
  \item{MARGIN}{
Integer, indicating the dimension over which multiple
\code{simmat} objects are to be bound together by \code{smbind}.
1: matrices are stacked (row bound), 2: matrices are column bound,
3: iterations are combined. Needs to be of length 1.
The other dimensions are expected to match across the objects.
}
  \item{strict}{
Logical, if consistency of the time series attributes
(\code{"start"}, \code{"end"}, \code{"thin"}, and number of simulated matrices)
of \code{simmat} objects are strictly enforced when
binding multiple objects together using \code{smbind}.
Applies only to input objects based on sequential
null model algorithms.
}
  \item{\dots}{
Additional arguments supplied to algorithms.
In case of \code{smbind} it can contain multiple \code{simmat} objects.
}
}
\details{
The purpose of the \code{nullmodel} function is to
create an object, where all necessary statistics of the
input matrix are calculated only once.
This information is reused, but not recalculated
in each step of the simulation process done by
the \code{simulate} method.

The \code{simulate} method carries out the simulation,
the simulated matrices are stored in an array.
For sequential algorithms, the method updates the state
of the input \code{nullmodel} object.
Therefore, it is possible to do diagnostic
tests on the returned \code{simmat} object,
and make further simulations, or use
increased thinning value if desired.

The \code{update} method makes burnin steps in case
of sequential algorithms to update the status of the
input model without any attempt to return matrices.
For non-sequential algorithms the method does nothing.

\code{update} is the preferred way of making burnin iterations
without sampling. Alternatively, burnin can be done
via the \code{simulate} method. For convergence
diagnostics, it is recommended to use the
\code{simulate} method without burnin.
The input nullmodel object is updated, so further
samples can be simulated if desired without having
to start the process all over again. See Examples.

The \code{smbind} function can be used to combine multiple
\code{simmat} objects. This comes handy when null model
simulations are stratified by sites (\code{MARGIN = 1})
or by species (\code{MARGIN = 2}), or in the case when
multiple objects are returned by identical/consistent settings
e.g. during parallel computations (\code{MARGIN = 3}).
Sanity checks are made to ensure that combining multiple
objects is sensible, but it is the user's responsibility
to check independence of the simulated matrices
and the null distribution has converged
in case of sequential null model algorithms.
The \code{strict = FALSE} setting can relax
checks regarding start, end, and thinning values
for sequential null models.
}
\value{
The function \code{nullmodel} returns an object of class \code{nullmodel}.
It is a set of objects sharing the same environment:
\itemize{
  \item{\code{data}: }{original matrix in integer mode.}
  \item{\code{nrow}: }{number of rows.}
  \item{\code{ncol}: }{number of columns.}
  \item{\code{rowSums}: }{row sums.}
  \item{\code{colSums}: }{column sums.}
  \item{\code{rowFreq}: }{row frequencies (number of nonzero cells).}
  \item{\code{colFreq}: }{column frequencies (number of nonzero cells).}
  \item{\code{totalSum}: }{total sum.}
  \item{\code{fill}: }{number of nonzero cells in the matrix.}
  \item{\code{commsim}: }{the \code{commsim} object as a result
    of the \code{method} argument.}
  \item{\code{state}: }{current state of the permutations,
    a matrix similar to the original.
    It is \code{NULL} for non-sequential algorithms.}
  \item{\code{iter}: }{current number of iterations
  for sequential algorithms.
    It is \code{NULL} for non-sequential algorithms.}
}

The \code{simulate} method returns an object of class \code{simmat}.
It is an array of simulated matrices (third dimension
corresponding to \code{nsim} argument).

The \code{update} method returns the current state (last updated matrix)
invisibly, and update the input object for sequential algorithms.
For non sequential algorithms, it returns \code{NULL}.

The \code{smbind} function returns an object of class \code{simmat}.
}
\author{
Jari Oksanen and Peter Solymos
}
\seealso{
\code{\link{commsim}}, \code{\link{make.commsim}},
\code{\link{permatfull}}, \code{\link{permatswap}}
}
\note{
Care must be taken when the input matrix only contains a single
row or column. Such input is invalid for swapping and hypergeometric
distribution (calling \code{\link{r2dtable}}) based algorithms.
This also applies to cases when the input is stratified into subsets.
}
\examples{
data(mite)
x <- as.matrix(mite)[1:12, 21:30]

## non-sequential nullmodel
(nm <- nullmodel(x, "r00"))
(sm <- simulate(nm, nsim=10))

## sequential nullmodel
(nm <- nullmodel(x, "swap"))
(sm1 <- simulate(nm, nsim=10, thin=5))
(sm2 <- simulate(nm, nsim=10, thin=5))

## sequential nullmodel with burnin and extra updating
(nm <- nullmodel(x, "swap"))
(sm1 <- simulate(nm, burnin=10, nsim=10, thin=5))
(sm2 <- simulate(nm, nsim=10, thin=5))

## sequential nullmodel with separate initial burnin
(nm <- nullmodel(x, "swap"))
nm <- update(nm, nsim=10)
(sm2 <- simulate(nm, nsim=10, thin=5))

## combining multiple simmat objects

## stratification
nm1 <- nullmodel(x[1:6,], "r00")
sm1 <- simulate(nm1, nsim=10)
nm2 <- nullmodel(x[7:12,], "r00")
sm2 <- simulate(nm2, nsim=10)
smbind(sm1, sm2, MARGIN=1)

## binding subsequent samples from sequential algorithms
## start, end, thin retained
nm <- nullmodel(x, "swap")
nm <- update(nm, nsim=10)
sm1 <- simulate(nm, nsim=10, thin=5)
sm2 <- simulate(nm, nsim=20, thin=5)
sm3 <- simulate(nm, nsim=10, thin=5)
smbind(sm3, sm2, sm1, MARGIN=3)

## 'replicate' based usage which is similar to the output
## of 'parLapply' or 'mclapply' in the 'parallel' package
## start, end, thin are set, also noting number of chains
smfun <- function(x, burnin, nsim, thin) {
    nm <- nullmodel(x, "swap")
    nm <- update(nm, nsim=burnin)
    simulate(nm, nsim=nsim, thin=thin)
}
smlist <- replicate(3, smfun(x, burnin=50, nsim=10, thin=5), simplify=FALSE)
smbind(smlist, MARGIN=3) # Number of permuted matrices = 30

\dontrun{
## parallel null model calculations
library(parallel)

if (.Platform$OS.type == "unix") {
## forking on Unix systems
smlist <- mclapply(1:3, function(i) smfun(x, burnin=50, nsim=10, thin=5))
smbind(smlist, MARGIN=3)
}

## socket type cluster, works on all platforms
cl <- makeCluster(3)
clusterEvalQ(cl, library(vegan))
clusterExport(cl, c("smfun", "x"))
smlist <- parLapply(cl, 1:3, function(i) smfun(x, burnin=50, nsim=10, thin=5))
stopCluster(cl)
smbind(smlist, MARGIN=3)
}
}
\keyword{ multivariate }
\keyword{ datagen }

\name{ordihull}
\alias{ordihull}
\alias{ordispider}
\alias{ordiellipse}
\alias{ordibar}
\alias{ordicluster}
\alias{summary.ordihull}
\alias{scores.ordihull}
\alias{summary.ordiellipse}
\alias{ordiareatest}

\title{Display Groups or Factor Levels in Ordination Diagrams}

\description{ Functions to add convex hulls, \dQuote{spider} graphs, ellipses
  or cluster dendrogram to ordination diagrams. The ordination
  diagrams can be produced by \code{vegan} \code{\link{plot.cca}},
  \code{\link{plot.decorana}} or \code{\link{ordiplot}}.  }

\usage{
ordihull(ord, groups, display = "sites", draw = c("lines","polygon", "none"),
         col = NULL, alpha = 127, show.groups, label = FALSE,
         border = NULL, lty = NULL, lwd = NULL, ...)
ordiellipse(ord, groups, display="sites", kind = c("sd","se", "ehull"),
         conf, draw = c("lines","polygon", "none"),
	 w = weights(ord, display), col = NULL, alpha = 127, show.groups,
	 label = FALSE, border = NULL, lty = NULL, lwd=NULL, ...)
ordibar(ord, groups, display = "sites", kind = c("sd", "se"), conf,
         w = weights(ord, display), col = 1, show.groups, label = FALSE,
	 lwd = NULL, length = 0,  ...)
ordispider(ord, groups, display="sites", w = weights(ord, display),
	 spiders = c("centroid", "median"),  show.groups,
         label = FALSE, col = NULL, lty = NULL, lwd = NULL, ...)
ordicluster(ord, cluster, prune = 0, display = "sites",
            w = weights(ord, display), col = 1, draw = c("segments", "none"),
            ...)
\method{summary}{ordihull}(object, ...)
\method{summary}{ordiellipse}(object, ...)
ordiareatest(ord, groups, area = c("hull", "ellipse"), kind = "sd",
         permutations = 999, parallel = getOption("mc.cores"), ...)
}

\arguments{
  \item{ord}{An ordination object or an \code{\link{ordiplot}} object. }
  \item{groups}{Factor giving the groups for which the graphical item is
    drawn. }
  \item{display}{Item to displayed. }

  \item{draw}{character; how should objects be represented on the plot?
    For \code{ordihull} and \code{ordiellipse} use either
    \code{\link{lines}} or \code{\link{polygon}} to draw the
    lines. For \code{ordicluster}, line segments are drawn using
    \code{\link{segments}}. To suppress plotting, use
    \code{"none"}. Graphical parameters are passed to both. The main
    difference is that \code{polygon}s may be filled and
    non-transparent. With \code{none} nothing is drawn, but the function
    returns the \code{\link{invisible}} plotting.}

  \item{col}{Colour of hull or ellipse lines (if \code{draw = "lines"})
    or their fills (if \code{draw = "polygon"}) in \code{ordihull} and
    \code{ordiellipse}.  When \code{draw = "polygon"}, the colour of
    bordering lines can be set with argument \code{border} of the
    \code{\link{polygon}} function. For other functions the effect
    depends on the underlining functions this argument is passed to.
    When multiple values of \code{col} are specified these are used
    for each element of \code{names(table(groups))} (in that order),
    shorter vectors are recycled. Function \code{ordicluster} has
    no \code{groups}, and there the argument will be recycled for
    points, and the colour of connecting lines is a mixture of point
    s in the cluster.}

  \item{alpha}{Transparency of the fill \code{col}our with \code{draw
    = "polygon"} in \code{ordihull} and \code{ordiellipse}.  The
    argument takes precedence over possible transparency definitions
    of the colour. The value must be in range \eqn{0...255}, and low
    values are more transparent.  Transparency is not available in all
    graphics devices or file formats.}

  \item{show.groups}{Show only given groups. This can be a vector, or
    \code{TRUE} if you want to show items for which condition is
    \code{TRUE}. This argument makes it possible to use different
    colours and line types for groups. The default is to show all groups. }

  \item{label}{Label the \code{groups} by their names in the centroid
    of the object. \code{ordiellipse} and \code{ordihull} use standard
    \code{\link{text}}, and others use \code{\link{ordilabel}}.}

  \item{w}{Weights used to find the average within group. Weights are
    used automatically for \code{\link{cca}}
    and \code{\link{decorana}} results, unless undone by the
    user. \code{w=NULL} sets equal weights to all points. }

  \item{kind}{Draw standard deviations of points (\code{sd}), standard
    errors (\code{se}) or ellipsoid hulls that enclose all points in
    the group (\code{ehull}).}

  \item{conf}{Confidence limit for ellipses, e.g. 0.95. If given, the
    corresponding \code{sd} or \code{se} is multiplied with the
    corresponding value found from the Chi-squared distribution with
    2df. }

  \item{spiders}{Are centres or spider bodies calculated either as
    centroids (averages) or spatial medians.}

  \item{cluster}{Result of hierarchic cluster analysis, such as
    \code{\link{hclust}} or \code{\link[cluster]{agnes}}.}

  \item{prune}{Number of upper level hierarchies removed from the
    dendrogram. If \code{prune} \eqn{>0}, dendrogram will be
    disconnected.}

  \item{object}{A result object from \code{ordihull} or
    \code{ordiellipse}. The result is \code{\link{invisible}}, but it
    can be saved, and used for summaries (areas etc. of hulls and
    ellipses). }

  \item{area}{Evaluate the area of convex hulls of \code{ordihull}, or of
    ellipses of \code{ordiellipse}.}

  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}

  \item{parallel}{Number of parallel processes or a predefined socket
    cluster.  With \code{parallel = 1} uses ordinary, non-parallel
    processing. The parallel processing is done with \pkg{parallel}
    package.}

  \item{lty, lwd, border}{Vectors of these parameters can be supplied
    and will be applied (if appropriate) for each element of
    \code{names(table(groups))} (in that order). Shorter vectors will be
    recycled.}

  \item{length}{Width (in inches) of the small (\dQuote{caps}) at the
    ends of the bar segment (passed to \code{\link{arrows}}).}

  \item{\dots}{Parameters passed to graphical functions or to
    \code{\link{scores}} to select axes and scaling etc. }
}

\details{

  Function \code{ordihull} draws \code{\link{lines}} or
  \code{\link{polygon}}s for the convex
  hulls found by function \code{\link{chull}} encircling
  the items in the groups.

  Function \code{ordiellipse} draws \code{\link{lines}} or
  \code{\link{polygon}}s for ellipses by \code{groups}. The function
  can either draw standard deviation of points (\code{kind="sd"}) or
  standard error of the (weighted) centroids (\code{kind="se"}), and
  the (weighted) correlation defines the direction of the principal
  axis of the ellipse. When \code{kind = "se"} is used together with
  argument \code{conf}, the ellipses will show the confidence regions
  for the locations of group centroids. With \code{kind="ehull"} the
  function draws an ellipse that encloses all points of a group using
  \code{\link[cluster]{ellipsoidhull}} (\pkg{cluster} package).

  Function \code{ordibar} draws crossed \dQuote{error bars} using
  either either standard deviation of point scores or standard error
  of the (weighted) average of scores. These are the principal axes of
  the corresponding \code{ordiellipse}, and are found by principal
  component analysis of the (weighted) covariance matrix.

  Functions \code{ordihull} and \code{ordiellipse} return invisibly an
  object that has a \code{summary} method that returns the coordinates
  of centroids and areas of the hulls or ellipses. Function
  \code{ordiareatest} studies the one-sided hypothesis that these
  areas are smaller than with randomized \code{groups}. Argument
  \code{kind} can be used to select the kind of ellipse, and has no
  effect with convex hulls.

  Function \code{ordispider} draws a \sQuote{spider} diagram where
  each point is connected to the group centroid with
  \code{\link{segments}}.  Weighted centroids are used in the
  correspondence analysis methods \code{\link{cca}} and
  \code{\link{decorana}} or if the user gives the weights in the
  call. If \code{ordispider} is called with \code{\link{cca}} or
  \code{\link{rda}} result without \code{groups} argument, the
  function connects each \sQuote{WA} scores to the corresponding
  \sQuote{LC} score. If the argument is a (\code{invisible})
  \code{ordihull} object, the function will connect the points of the
  hull to their centroid.

  Function \code{ordicluster} overlays a cluster dendrogram onto
  ordination. It needs the result from a hierarchic clustering such as
  \code{\link{hclust}} or \code{\link[cluster]{agnes}}, or other with
  a similar structure. Function \code{ordicluster} connects cluster
  centroids to each other with line \code{\link{segments}}. Function
  uses centroids of all points in the clusters, and is therefore
  similar to average linkage methods.
}

\note{These functions add graphical items to ordination graph: You
  must draw a graph first. To draw line segments, grids or arrows, see
  \code{\link{ordisegments}}, \code{\link{ordigrid}}
  and\code{\link{ordiarrows}}.  }

\value{

  Functions \code{ordihull}, \code{ordiellipse} and \code{ordispider}
  return the \code{\link{invisible}} plotting structure.

  Function \code{ordispider} return the coordinates to which each
  point is connected (centroids or \sQuote{LC} scores).

  Function \code{ordihull} and \code{ordiellipse} return invisibly an
  object that has a \code{summary} method that returns the coordinates
  of centroids and areas of the hulls or ellipses. Function
  \code{ordiareatest} studies the one-sided hypothesis that these
  areas are smaller than with randomized \code{groups}.

}

\author{ Jari Oksanen }

\seealso{The functions pass parameters to basic graphical functions,
  and you may wish to change the default values in
  \code{\link{lines}}, \code{\link{segments}} and
  \code{\link{polygon}}. You can pass parameters to
  \code{\link{scores}} as well. Underlying functions for
  \code{ordihull} is \code{\link{chull}}. The underlying function for
  ellipsoid hulls in \code{ordiellipse} is
  \code{\link[cluster]{ellipsoidhull}}. }

\examples{
data(dune)
data(dune.env)
mod <- cca(dune ~ Management, dune.env)
plot(mod, type="n", scaling = "symmetric")
## Catch the invisible result of ordihull...
pl <- with(dune.env, ordihull(mod, Management,
                              scaling = "symmetric", label = TRUE))
## ... and find centres and areas of the hulls
summary(pl)
## use more colours and add ellipsoid hulls
plot(mod, type = "n")
pl <- with(dune.env, ordihull(mod, Management,
                              scaling = "symmetric", col = 1:4,
                              draw="polygon", label =TRUE))
with(dune.env, ordiellipse(mod, Management, scaling = "symmetric",
                           kind = "ehull", col = 1:4, lwd=3))
## ordispider to connect WA and LC scores
plot(mod, dis=c("wa","lc"), type="p")
ordispider(mod)
## Other types of plots
plot(mod, type = "p", display="sites")
cl <- hclust(vegdist(dune))
ordicluster(mod, cl, prune=3, col = cutree(cl, 4))
## confidence ellipse: location of the class centroids
plot(mod, type="n", display = "sites")
with(dune.env, text(mod, display="sites", labels = as.character(Management),
                    col=as.numeric(Management)))
pl <- with(dune.env, ordiellipse(mod, Management, kind="se", conf=0.95, lwd=2,
                                 draw = "polygon", col=1:4, border=1:4,
                                 alpha=63))
summary(pl)
## add confidence bars
with(dune.env, ordibar(mod, Management, kind="se", conf=0.95, lwd=2, col=1:4,
                       label=TRUE))
}
\keyword{aplot }

\name{linestack}
\alias{linestack}

\title{Plots One-dimensional Diagrams without Overwriting Labels }
\description{
  Function \code{linestack} plots vertical one-dimensional plots for
  numeric vectors.  The plots are always labelled, but the labels
  are moved vertically to avoid overwriting.
}
\usage{
linestack(x, labels, cex = 0.8, side = "right", hoff = 2, air = 1.1,
          at = 0, add = FALSE, axis = FALSE, ...)
}

\arguments{
  \item{x}{Numeric vector to be plotted. }
  \item{labels}{Labels used instead of default (names of \code{x}). May
    be expressions to be drawn with \code{\link{plotmath}}.}
  \item{cex}{Size of the labels. }
  \item{side}{Put labels to the \code{"right"} or \code{"left"} of the
    axis.}
  \item{hoff}{Distance from the vertical axis to the label in units of
    the width of letter \dQuote{m}. }
  \item{air}{Multiplier to string height to leave empty space between
    labels.}
  \item{at}{Position of plot in horizontal axis. }
  \item{add}{Add to an existing plot. }
  \item{axis}{Add axis to the plot. }
  \item{\dots}{Other graphical parameters to labels.}
}
\value{
  The function returns invisibly the shifted positions of labels in
  user coordinates.
}
\author{Jari Oksanen with modifications by Gavin L. Simpson}
\note{ The function always draws labelled diagrams.  If you want to have
  unlabelled diagrams, you can use, e.g., \code{\link{plot}},
  \code{\link{stripchart}} or \code{\link{rug}}.
}

\examples{
## First DCA axis
data(dune)
ord <- decorana(dune)
linestack(scores(ord, choices=1, display="sp"))
linestack(scores(ord, choices=1, display="si"), side="left", add=TRUE)
title(main="DCA axis 1")

## Expressions as labels
N <- 10					# Number of sites
df <- data.frame(Ca = rlnorm(N, 2), NO3 = rlnorm(N, 4),
                 SO4 = rlnorm(N, 10), K = rlnorm(N, 3))
ord <- rda(df, scale = TRUE)
### vector of expressions for labels
labs <- expression(Ca^{2+phantom()},
                   NO[3]^{-phantom()},
                   SO[4]^{2-phantom()},
                   K^{+phantom()})
scl <- "sites"
linestack(scores(ord, choices = 1, display = "species", scaling = scl),
          labels = labs, air = 2)
linestack(scores(ord, choices = 1, display = "site", scaling = scl),
          side = "left", add = TRUE)
title(main = "PCA axis 1")
}
\keyword{ hplot }
\keyword{ aplot }

\encoding{UTF-8}
\name{commsim}
\alias{commsim}
\alias{make.commsim}
\alias{print.commsim}
\title{
Create an Object for Null Model Algorithms
}
\description{
The \code{commsim} function can be used to feed Null Model algorithms into
\code{\link{nullmodel}} analysis.
The \code{make.commsim} function returns various predefined algorithm types
(see Details).
These functions represent low level interface for community null model
infrastructure in \pkg{vegan} with the intent of extensibility,
and less emphasis on direct use by users.
}
\usage{
commsim(method, fun, binary, isSeq, mode)
make.commsim(method)
\method{print}{commsim}(x, ...)
}
\arguments{
  \item{method}{
Character, name of the algorithm.
}
  \item{fun}{
A function. For possible formal arguments of this function
see Details.
}
  \item{binary}{
Logical, if the algorithm applies to presence-absence or count matrices.
}
  \item{isSeq}{
Logical, if the algorithm is sequential (needs burnin and thinning) or not.
}
  \item{mode}{
Character, storage mode of the community matrix, either
\code{"integer"} or \code{"double"}.
}
  \item{x}{
An object of class \code{commsim}.
}
  \item{\dots}{
Additional arguments.
}
}
\details{
The function \code{fun} must return an array of \code{dim(nr, nc, n)},
and must take some of the following arguments:
\itemize{
  \item{\code{x}: }{input matrix,}
  \item{\code{n}: }{number of permuted matrices in output,}
  \item{\code{nr}: }{number of rows,}
  \item{\code{nc}: }{number of columns,}
  \item{\code{rs}: }{vector of row sums,}
  \item{\code{cs}: }{vector of column sums,}
  \item{\code{rf}: }{vector of row frequencies (non-zero cells),}
  \item{\code{cf}: }{vector of column frequencies (non-zero cells),}
  \item{\code{s}: }{total sum of \code{x},}
  \item{\code{fill}: }{matrix fill (non-zero cells),}
  \item{\code{thin}: }{thinning value for sequential algorithms,}
  \item{\code{...}: }{additional arguments.}
}

  You can define your own null model, but
  several null model algorithm are pre-defined and can be called by
  their name. The predefined algorithms are described in detail in the
  following chapters. The binary null models produce matrices of zeros
  (absences) and ones (presences) also when input matrix is
  quantitative. There are two types of quantitative data: Counts are
  integers with a natural unit so that individuals can be shuffled, but
  abundances can have real (floating point) values and do not have a
  natural subunit for shuffling. All quantitative models can handle
  counts, but only some are able to handle real values. Some of the null
  models are sequential so that the next matrix is derived from the
  current one. This makes models dependent from previous models, and usually
  you must thin these matrices and study the sequences for stability:
  see \code{oecosimu} for details and instructions.

  See Examples for structural constraints imposed by each algorithm and
  defining your own null model.

}
%% commsimulator

\section{Binary null models}{

  All binary null models preserve fill: number of presences or
  conversely the number of absences. The classic models may also
  preserve column (species) frequencies (\code{c0}) or row frequencies
  or species richness of each site (\code{r0}) and take into account
  commonness and rarity of species (\code{r1}, \code{r2}).  Algorithms
  \code{swap}, \code{tswap}, \code{curveball}, \code{quasiswap} and
  \code{backtracking} preserve both row and column frequencies. Three
  first ones are sequential but the two latter are non-sequential
  and produce independent matrices. Basic algorithms are reviewed by
  Wright et al. (1998).

\itemize{
  \item{\code{"r00"}: }{non-sequential algorithm for binary matrices
    that only  preserves the number of presences (fill).}

  \item{\code{"r0", "r0_old"}: }{non-sequential algorithm for binary
    matrices that preserves the site (row) frequencies.
    Methods \code{"r0"} and \code{"r0_old"} implement the
    same method, but use different random number sequences; use
    \code{"r0_old"} if you want to reproduce results in \pkg{vegan
    2.0-0} or older using \code{commsimulator} (now deprecated).}

  \item{\code{"r1"}: }{non-sequential algorithm for binary matrices
    that preserves the site (row) frequencies, but uses column marginal
    frequencies as probabilities of selecting species.}

  \item{\code{"r2"}: }{non-sequential algorithm for binary matrices
    that preserves the site (row) frequencies, and uses squared column
    marginal frequencies as as probabilities of selecting species.}

  \item{\code{"c0"}: }{non-sequential algorithm for binary matrices
    that preserves species frequencies (Jonsson 2001). }

  \item{\code{"swap"}: }{sequential algorithm for binary matrices that
    changes the matrix structure, but does not influence marginal sums
    (Gotelli & Entsminger 2003).  This inspects \eqn{2 \times 2}{2 by
    2} submatrices so long that a swap can be done.}

  \item{\code{"tswap"}: }{sequential algorithm for binary matrices.
    Same as the \code{"swap"} algorithm, but it tries a fixed
    number of times and performs zero to many swaps at one step
    (according to the thin argument in the call). This
    approach was suggested by \enc{Miklós}{Miklos} & Podani (2004)
    because they found that ordinary swap may lead to biased
    sequences, since some columns or rows are more easily swapped.}

  \item{\code{"curveball"}: }{sequential method for binary matrices that
    implements the \sQuote{Curveball} algorithm of Strona et
    al. (2014). The algorithm selects two random rows and finds the set
    of unique species that occur only in one of these rows. The
    algorithm distributes the set of unique species to rows preserving
    the original row frequencies.  Zero to several species are swapped
    in one step, and usually the matrix is perturbed more strongly than
    in other sequential methods.}

  \item{\code{"quasiswap"}: }{non-sequential algorithm for binary
    matrices that implements a method where matrix is first filled
    honouring row and column totals, but with integers that may be
    larger than one.  Then the method inspects random
    \eqn{2 \times 2}{2 by 2} matrices and performs a quasiswap on
    them. In addition to ordinary swaps, quasiswap can reduce numbers
    above one to ones preserving marginal totals (\enc{Miklós}{Miklos} &
    Podani 2004). The method is non-sequential, but it accepts
    \code{thin} argument: the convergence is checked at every
    \code{thin} steps. This allows performing several ordinary swaps in
    addition to fill changing swaps which helps in reducing or removing
    the bias.}

  \item{\code{"greedyqswap"}: }{A greedy variant of quasiswap. In
    greedy step, one element of the \eqn{2 \times 2}{2 by 2} matrix is
    taken from \eqn{> 1} elements. The greedy steps are biased, but
    the method can be thinned, and only the first of \code{thin}
    steps is greedy. Even modest thinning (say \code{thin = 20})
    removes or reduces the bias, and \code{thin = 100} (1\% greedy
    steps) looks completely safe and still speeds up simulation. The
    code is experimental and it is provided here for further scrutiny,
    and should be tested for bias before use.}

  \item{\code{"backtracking"}: }{non-sequential algorithm for binary
    matrices that implements a filling method with constraints both for
    row and column frequencies (Gotelli & Entsminger 2001).  The matrix
    is first filled randomly, but typically row and column sums are
    reached before all incidences are filled in. After this begins
    "backtracking", where some of the incidences are removed, and
    filling is started again, and this backtracking is done so many
    times that all incidences will be filled into matrix.  The results
    may be biased and should be inspected carefully before use.}

}
}

\section{Quantitative Models for Counts with Fixed Marginal Sums}{

  These models shuffle individuals of counts and keep marginal sums
  fixed, but marginal frequencies are not preserved. Algorithm
  \code{r2dtable} uses standard \R function \code{\link{r2dtable}} also
  used for simulated \eqn{P}-values in \code{\link{chisq.test}}.
  Algorithm \code{quasiswap_count} uses the same, but preserves the
  original fill. Typically this means increasing numbers of zero cells
  and the result is zero-inflated with respect to \code{r2dtable}.

\itemize{

  \item{\code{"r2dtable"}: }{non-sequential algorithm for count
    matrices.  This algorithm keeps matrix sum and row/column sums
    constant. Based on \code{\link{r2dtable}}.}

  \item{\code{"quasiswap_count"}: }{non-sequential algorithm for count
    matrices.  This algorithm is similar as Carsten Dormann's
    \code{\link[bipartite]{swap.web}} function in the package
    \pkg{bipartite}. First, a random matrix is generated by the
    \code{\link{r2dtable}} function preserving row and column sums.  Then
    the original matrix fill is reconstructed by sequential steps to
    increase or decrease matrix fill in the random matrix. These steps
    are based on swapping \eqn{2 \times 2}{2 x 2} submatrices (see
    \code{"swap_count"} algorithm for details) to maintain row and
    column totals. }
}
}

\section{Quantitative Swap Models}{

  Quantitative swap models are similar to binary \code{swap}, but they
  swap the largest permissible value. The models in this section all
  maintain the fill and perform a quantitative swap only if this can
  be done without changing the fill. Single step of swap often changes
  the matrix very little. In particular, if cell counts are variable,
  high values change very slowly. Checking the chain stability and
  independence is even more crucial than in binary swap, and very
  strong \code{thin}ning is often needed. These models should never be
  used without inspecting their properties for the current data. These
  null models can also be defined using \code{\link{permatswap}}
  function.

 \itemize{

   \item{\code{"swap_count"}: }{sequential algorithm for count matrices.
    This algorithm find \eqn{2 \times 2}{2 x 2} submatrices that can be
    swapped leaving column and row totals and fill unchanged. The
    algorithm finds the largest value in the submatrix that can be
    swapped (\eqn{d}). Swap means that the values in diagonal or
    antidiagonal positions are decreased by \eqn{d}, while remaining
    cells are increased by \eqn{d}. A swap is made only if fill does not
    change.  }

   \item{\code{"abuswap_r"}: }{sequential algorithm for count or
    nonnegative real valued matrices with fixed row frequencies (see
    also \code{\link{permatswap}}).  The algorithm is similar to
    \code{swap_count}, but uses different swap value for each row of the
    \eqn{2 \times 2}{2 x 2} submatrix. Each step changes the the
    corresponding column sums, but honours matrix fill, row sums, and
    row/column frequencies (Hardy 2008; randomization scheme 2x).}

  \item{\code{"abuswap_c"}: }{sequential algorithm for count or
    nonnegative real valued matrices with fixed column frequencies
    (see also \code{\link{permatswap}}).  The algorithm is similar as
    the previous one, but operates on columns.  Each step changes the
    the corresponding row sums, but honours matrix fill, column sums,
    and row/column frequencies (Hardy 2008; randomization scheme 3x).}
    } }

\section{Quantitative Swap and Shuffle Models}{

  Quantitative Swap and Shuffle methods (\code{swsh} methods) preserve
  fill and column and row frequencies, and also either row or column
  sums. The methods first perform a binary \code{quasiswap} and then
  shuffle original quantitative data to non-zero cells. The
  \code{samp} methods shuffle original non-zero cell values and can be
  used also with non-integer data. The \code{both} methods
  redistribute individuals randomly among non-zero cells and can only
  be used with integer data. The shuffling is either free over the
  whole matrix, or within rows (\code{r} methods) or within columns
  (\code{c} methods). Shuffling within a row preserves row sums, and
  shuffling within a column preserves column sums. These models can
  also be defined with \code{\link{permatswap}}.

\itemize{

  \item{\code{"swsh_samp"}: }{non-sequential algorithm for
    quantitative data (either integer counts or non-integer values).
    Original non-zero values values are shuffled.}

  \item{\code{"swsh_both"}: }{non-sequential algorithm for count data.
    Individuals are shuffled freely over non-zero cells.}

  \item{\code{"swsh_samp_r"}: }{non-sequential algorithm for
    quantitative data.  Non-zero values (samples) are shuffled
    separately for each row.}

  \item{\code{"swsh_samp_c"}: }{non-sequential algorithm for
    quantitative data.  Non-zero values (samples) are shuffled
    separately for each column.}

  \item{\code{"swsh_both_r"}: }{non-sequential algorithm for count matrices.
    Individuals are shuffled freely for non-zero values within each row.}

  \item{\code{"swsh_both_c"}: }{non-sequential algorithm for count matrices.
    Individuals are shuffled freely for non-zero values with each column.}
}
}

\section{Quantitative Shuffle Methods}{

  Quantitative shuffle methods are generalizations of binary models
  \code{r00}, \code{r0} and \code{c0}.  The \code{_ind} methods
  shuffle individuals so that the grand sum, row sum or column sums
  are preserved.  These methods are similar as \code{r2dtable} but
  with still slacker constraints on marginal sums. The \code{_samp}
  and \code{_both} methods first apply the corresponding binary model
  with similar restriction on marginal frequencies and then distribute
  quantitative values over non-zero cells. The \code{_samp} models
  shuffle original cell values and can therefore handle also non-count
  real values. The \code{_both} models shuffle individuals among
  non-zero values. The shuffling is over the whole matrix in
  \code{r00_}, and within row in \code{r0_} and within column in
  \code{c0_} in all cases.

\itemize{
  \item{\code{"r00_ind"}: }{non-sequential algorithm for count matrices.
    This algorithm preserves grand sum and
    individuals are shuffled among cells of the matrix.}

  \item{\code{"r0_ind"}: }{non-sequential algorithm for count matrices.
    This algorithm preserves row sums and
    individuals are shuffled among cells of each row of the matrix.}

  \item{\code{"c0_ind"}: }{non-sequential algorithm for count matrices.
    This algorithm preserves column sums and
    individuals are shuffled among cells of each column of the matrix.}

  \item{\code{"r00_samp"}: }{non-sequential algorithm for count
    or nonnegative real valued (\code{mode = "double"}) matrices.
    This algorithm preserves grand sum and
    cells of the matrix are shuffled.}

  \item{\code{"r0_samp"}: }{non-sequential algorithm for count
    or nonnegative real valued (\code{mode = "double"}) matrices.
    This algorithm preserves row sums and
    cells within each row are shuffled.}

  \item{\code{"c0_samp"}: }{non-sequential algorithm for count
    or nonnegative real valued (\code{mode = "double"}) matrices.
    This algorithm preserves column sums constant and
    cells within each column are shuffled.}

  \item{\code{"r00_both"}: }{non-sequential algorithm for count matrices.
    This algorithm preserves grand sum and
    cells and individuals among cells of the matrix are shuffled.}

  \item{\code{"r0_both"}: }{non-sequential algorithm for count matrices.
    This algorithm preserves grand sum and
    cells and individuals among cells of each row are shuffled.}

  \item{\code{"c0_both"}: }{non-sequential algorithm for count matrices.
    This algorithm preserves grand sum and
    cells and individuals among cells of each column are shuffled.}
}
}

\value{
An object of class \code{commsim} with elements
corresponding to the arguments (\code{method}, \code{binary},
\code{isSeq}, \code{mode}, \code{fun}).

If the input of \code{make.comsimm} is a \code{commsim} object,
it is returned without further evaluation. If this is not the case,
the character \code{method} argument is matched against
predefined algorithm names. An error message is issued
if none such is found. If the \code{method} argument is missing,
the function returns names of all currently available
null model algorithms as a character vector.
}
\references{
  Gotelli, N.J. & Entsminger, N.J. (2001). Swap and fill algorithms in
  null model analysis: rethinking the knight's tour. \emph{Oecologia}
  129, 281--291.

  Gotelli, N.J. & Entsminger, N.J. (2003). Swap algorithms in null model
  analysis. \emph{Ecology} 84, 532--535.

  Hardy, O. J. (2008) Testing the spatial phylogenetic structure of
  local communities: statistical performances of different null models
  and test statistics on a locally neutral community.  \emph{Journal of
  Ecology} 96, 914--926.

  Jonsson, B.G. (2001) A null model for randomization tests of
  nestedness in species assemblages. \emph{Oecologia} 127, 309--313.

  \enc{Miklós}{Miklos}, I. & Podani, J. (2004). Randomization of
  presence-absence matrices: comments and new algorithms. \emph{Ecology}
  85, 86--92.

  Patefield, W. M. (1981) Algorithm AS159.  An efficient method of
  generating r x c tables with given row and column totals.
  \emph{Applied Statistics} 30, 91--97.

  Strona, G., Nappo, D., Boccacci, F., Fattorini, S. &
  San-Miguel-Ayanz, J. (2014). A fast and unbiased procedure to
  randomize ecological binary matrices with fixed row and column
  totals. \emph{Nature Communications} 5:4114
  \doi{10.1038/ncomms5114}.

  Wright, D.H., Patterson, B.D., Mikkelson, G.M., Cutler, A. & Atmar,
  W. (1998). A comparative analysis of nested subset patterns of species
  composition. \emph{Oecologia} 113, 1--20.
}

\author{
Jari Oksanen and Peter Solymos
}

\seealso{ See \code{\link{permatfull}}, \code{\link{permatswap}} for
alternative specification of quantitative null models. Function
\code{\link{oecosimu}} gives a higher-level interface for applying null
models in hypothesis testing and analysis of models. Function
\code{\link{nullmodel}} and \code{\link{simulate.nullmodel}} are used to
generate arrays of simulated null model matrices.  }

\examples{
## write the r00 algorithm
f <- function(x, n, ...)
    array(replicate(n, sample(x)), c(dim(x), n))
(cs <- commsim("r00", fun=f, binary=TRUE,
    isSeq=FALSE, mode="integer"))

## retrieving the sequential swap algorithm
(cs <- make.commsim("swap"))

## feeding a commsim object as argument
make.commsim(cs)

## making the missing c1 model using r1 as a template
##   non-sequential algorithm for binary matrices
##   that preserves the species (column) frequencies,
##   but uses row marginal frequencies
##   as probabilities of selecting sites
f <- function (x, n, nr, nc, rs, cs, ...) {
    out <- array(0L, c(nr, nc, n))
    J <- seq_len(nc)
    storage.mode(rs) <- "double"
    for (k in seq_len(n))
        for (j in J)
            out[sample.int(nr, cs[j], prob = rs), j, k] <- 1L
    out
}
cs <- make.commsim("r1")
cs$method <- "c1"
cs$fun <- f

## structural constraints
diagfun <- function(x, y) {
    c(sum = sum(y) == sum(x),
        fill = sum(y > 0) == sum(x > 0),
        rowSums = all(rowSums(y) == rowSums(x)),
        colSums = all(colSums(y) == colSums(x)),
        rowFreq = all(rowSums(y > 0) == rowSums(x > 0)),
        colFreq = all(colSums(y > 0) == colSums(x > 0)))
}
evalfun <- function(meth, x, n) {
    m <- nullmodel(x, meth)
    y <- simulate(m, nsim=n)
    out <- rowMeans(sapply(1:dim(y)[3],
        function(i) diagfun(attr(y, "data"), y[,,i])))
    z <- as.numeric(c(attr(y, "binary"), attr(y, "isSeq"),
        attr(y, "mode") == "double"))
    names(z) <- c("binary", "isSeq", "double")
    c(z, out)
}
x <- matrix(rbinom(10*12, 1, 0.5)*rpois(10*12, 3), 12, 10)
algos <- make.commsim()
a <- t(sapply(algos, evalfun, x=x, n=10))
print(as.table(ifelse(a==1,1,0)), zero.print = ".")
}
\keyword{ multivariate }
\keyword{ datagen }

\encoding{UTF-8}
\name{designdist}
\alias{designdist}
\alias{chaodist}

\title{Design your own Dissimilarities }
\description{

  Function \code{designdist} lets you define your own dissimilarities
  using terms for shared and total quantities, number of rows and number
  of columns. The shared and total quantities can be binary, quadratic
  or minimum terms. In binary terms, the shared component is number of
  shared species, and totals are numbers of species on sites. The
  quadratic terms are cross-products and sums of squares, and minimum
  terms are sums of parallel minima and row totals. Function
  \code{chaodist} lets you define your own dissimilarities using terms
  that are supposed to take into account the \dQuote{unseen species}
  (see Chao et al., 2005 and Details in \code{\link{vegdist}}).

}
\usage{
designdist(x, method = "(A+B-2*J)/(A+B)",
           terms = c("binary", "quadratic", "minimum"), 
           abcd = FALSE, alphagamma = FALSE, name)
chaodist(x, method = "1 - 2*U*V/(U+V)", name)
}

\arguments{
  \item{x}{Input data. }
  \item{method}{Equation for your dissimilarities. This can use terms
    \code{J} for shared quantity, \code{A} and \code{B} for totals,
    \code{N} for the number of rows (sites) and \code{P} for the
    number of columns (species) or in \code{chaodist} it can use terms
    \code{U} and \code{V}. The equation can also contain any \R
    functions that accepts vector arguments and returns vectors of the
    same length. }
  \item{terms}{How shared and total components are found. For vectors
    \code{x} and \code{y} the  \code{"quadratic"} terms are \code{J = sum(x*y)},
    \code{A = sum(x^2)}, \code{B = sum(y^2)}, and \code{"minimum"} terms
    are \code{J = sum(pmin(x,y))}, \code{A = sum(x)} and \code{B = sum(y)}, 
    and \code{"binary"} terms are either of these after transforming
    data into binary form (shared number of species, and number of
    species for each row). }
  \item{abcd}{Use 2x2 contingency table notation for binary data:
    \eqn{a} is the number of shared species, \eqn{b} and \eqn{c} are the
    numbers of species occurring only one of the sites but not in both,
    and \eqn{d} is the number of species that occur on neither of the sites.}
  \item{alphagamma}{Use beta diversity notation with terms
    \code{alpha} for average alpha diversity for compared sites,
    \code{gamma} for diversity in pooled sites, and \code{delta} for the
    absolute value of difference of average \code{alpha} and alpha
    diversities of compared sites. Terms \code{A} and
    \code{B} refer to alpha diversities of compared sites.}
  \item{name}{The name you want to use for your index. The default is to
    combine the \code{method} equation and \code{terms} argument.}
}
\details{
  Most popular dissimilarity measures in ecology can be expressed with
  the help of terms \code{J}, \code{A} and \code{B}, and some also involve
  matrix dimensions \code{N} and \code{P}. Some examples you can define in
  \code{designdist} are:
  \tabular{lll}{
    \code{A+B-2*J} \tab \code{"quadratic"} \tab squared Euclidean \cr
    \code{A+B-2*J} \tab \code{"minimum"} \tab Manhattan \cr
    \code{(A+B-2*J)/(A+B)} \tab \code{"minimum"} \tab Bray-Curtis \cr
    \code{(A+B-2*J)/(A+B)} \tab \code{"binary"} \tab
    \enc{Sørensen}{Sorensen} \cr 
    \code{(A+B-2*J)/(A+B-J)} \tab \code{"binary"} \tab Jaccard \cr
    \code{(A+B-2*J)/(A+B-J)} \tab \code{"minimum"} \tab
    \enc{Ružička}{Ruzicka} \cr
    \code{(A+B-2*J)/(A+B-J)} \tab \code{"quadratic"} \tab
    (dis)similarity ratio \cr
    \code{1-J/sqrt(A*B)} \tab \code{"binary"} \tab Ochiai \cr
    \code{1-J/sqrt(A*B)} \tab \code{"quadratic"} \tab cosine
    complement \cr
    \code{1-phyper(J-1, A, P-A, B)} \tab \code{"binary"} \tab Raup-Crick (but see \code{\link{raupcrick}})
  }
  The function \code{designdist} can implement most dissimilarity
  indices in \code{\link{vegdist}} or elsewhere, and it can also be
  used to implement many other indices, amongst them, most of those
  described in Legendre & Legendre (2012). It can also be used to
  implement all indices of beta diversity described in Koleff et
  al. (2003), but there also is a specific function
  \code{\link{betadiver}} for the purpose.

  If you want to implement binary dissimilarities based on the 2x2
  contingency table notation, you can set \code{abcd = TRUE}. In this
  notation \code{a = J}, \code{b = A-J}, \code{c = B-J}, \code{d = P-A-B+J}. 
  This notation is often used instead of the more more
  tangible default notation for reasons that are opaque to me.

  With \code{alphagamma = TRUE} it is possible to use beta diversity
  notation with terms \code{alpha} for average alpha diversity and
  \code{gamma} for gamma diversity in two compared sites. The terms
  are calculated as \code{alpha = (A+B)/2}, \code{gamma = A+B-J} and
  \code{delta = abs(A-B)/2}.  Terms \code{A} and \code{B} are also
  available and give the alpha diversities of the individual compared
  sites.  The beta diversity terms may make sense only for binary
  terms (so that diversities are expressed in numbers of species), but
  they are calculated for quadratic and minimum terms as well (with a
  warning).

  Function \code{chaodist} is similar to \code{designgist}, but uses
  terms \code{U} and \code{V} of Chao et al. (2005). These terms are
  supposed to take into account the effects of unseen species. Both
  \code{U} and \code{V} are scaled to range \eqn{0 \dots 1}. They take
  the place of \code{A} and \code{B} and the product \code{U*V} is used
  in the place of \code{J} of \code{designdist}.  Function
  \code{chaodist} can implement any commonly used Chao et al. (2005)
  style dissimilarity:
  \tabular{ll}{
  \code{1 - 2*U*V/(U+V)} \tab \enc{Sørensen}{Sorensen} type \cr
  \code{1 - U*V/(U+V-U*V)} \tab Jaccard type \cr
  \code{1 - sqrt(U*V)} \tab Ochiai type \cr
  \code{(pmin(U,V) - U*V)/pmin(U,V)} \tab Simpson type
  }
  Function \code{\link{vegdist}} implements Jaccard-type Chao distance,
  and its documentation contains more complete discussion on the
  calculation of the terms.

}

\value{
  \code{designdist} returns an object of class \code{\link{dist}}.
}
\references{

  Chao, A., Chazdon, R. L., Colwell, R. K. and Shen, T. (2005) A new
  statistical approach for assessing similarity of species composition
  with incidence and abundance data. \emph{Ecology Letters} \strong{8},
  148--159.

  Koleff, P., Gaston, K.J. and Lennon, J.J. (2003) Measuring beta
  diversity for presence--absence data. \emph{J. Animal Ecol.}
  \strong{72}, 367--382. 
  
  Legendre, P. and Legendre, L. (2012) \emph{Numerical Ecology}. 3rd
  English ed. Elsevier
  }
\author{ Jari Oksanen }
\note{
  \code{designdist} does not use compiled code, but it is based on
  vectorized \R{} code. The \code{designdist} function can be much
  faster than \code{\link{vegdist}}, although the latter uses compiled
  code. However, \code{designdist} cannot skip missing values and uses
  much more memory during calculations.

  The use of sum terms can be numerically unstable. In particularly,
  when these terms are large, the precision may be lost. The risk is
  large when the number of columns is high, and particularly large with
  quadratic terms. For precise calculations it is better to use
  functions like \code{\link{dist}} and \code{\link{vegdist}} which are
  more robust against numerical problems.
}

\seealso{ \code{\link{vegdist}}, \code{\link{betadiver}}, \code{\link{dist}},
  \code{\link{raupcrick}}.}
\examples{
data(BCI)
## Four ways of calculating the same Sørensen dissimilarity
d0 <- vegdist(BCI, "bray", binary = TRUE)
d1 <- designdist(BCI, "(A+B-2*J)/(A+B)")
d2 <- designdist(BCI, "(b+c)/(2*a+b+c)", abcd = TRUE)
d3 <- designdist(BCI, "gamma/alpha - 1", alphagamma = TRUE)
## Arrhenius dissimilarity: the value of z in the species-area model
## S = c*A^z when combining two sites of equal areas, where S is the
## number of species, A is the area, and c and z are model parameters.
## The A below is not the area (which cancels out), but number of
## species in one of the sites, as defined in designdist().
dis <- designdist(BCI, "(log(A+B-J)-log(A+B)+log(2))/log(2)")
## This can be used in clustering or ordination...
ordiplot(cmdscale(dis))
## ... or in analysing beta diversity (without gradients)
summary(dis)
  }

\keyword{multivariate }

\encoding{UTF-8}
\name{vegdist}
\alias{vegdist}
\title{Dissimilarity Indices for Community Ecologists }
\description{
  The function computes dissimilarity indices that are useful for or
  popular with community ecologists. All indices use quantitative data,
  although they would be named by the corresponding binary index, but
  you can calculate the binary index using an appropriate argument.  If
  you do not find your favourite index here, you can see if it can be
  implemented using \code{\link{designdist}}.  Gower, Bray--Curtis,
  Jaccard and Kulczynski indices are good in detecting underlying
  ecological gradients (Faith et al. 1987). Morisita, Horn--Morisita,
  Binomial, Cao and Chao indices should be able to handle different
  sample sizes (Wolda 1981, Krebs 1999, Anderson & Millar 2004), and
  Mountford (1962) and Raup-Crick indices for presence--absence data
  should be able to handle unknown (and variable) sample sizes. Most of
  these indices are discussed by Krebs (1999) and Legendre & Legendre
  (2012), and their properties further compared by Wolda (1981) and
  Legendre & De \enc{Cáceres}{Caceres} (2012).
}

\usage{vegdist(x, method="bray", binary=FALSE, diag=FALSE, upper=FALSE,
        na.rm = FALSE, ...) } 
\arguments{
  \item{x}{Community data matrix.}
  \item{method}{Dissimilarity index, partial match to
    \code{"manhattan"}, \code{"euclidean"}, \code{"canberra"},
    \code{"clark"}, \code{"bray"}, \code{"kulczynski"},
    \code{"jaccard"}, \code{"gower"}, \code{"altGower"},
    \code{"morisita"}, \code{"horn"}, \code{"mountford"}, \code{"raup"},
    \code{"binomial"}, \code{"chao"}, \code{"cao"} or \code{"mahalanobis"}.}
  \item{binary}{Perform presence/absence standardization before analysis
    using \code{\link{decostand}}.}
  \item{diag}{Compute diagonals. }
  \item{upper}{Return only the upper diagonal. }
  \item{na.rm}{Pairwise deletion of missing observations when
    computing dissimilarities.}
  \item{\dots}{Other parameters.  These are ignored, except in
    \code{method ="gower"} which accepts \code{range.global} parameter of
    \code{\link{decostand}}. .}
}

\details{Jaccard (\code{"jaccard"}), Mountford (\code{"mountford"}),
  Raup--Crick (\code{"raup"}), Binomial and Chao indices are discussed
  later in this section.  The function also finds indices for presence/
  absence data by setting \code{binary = TRUE}. The following overview
  gives first the quantitative version, where \eqn{x_{ij}}{x[ij]}
  \eqn{x_{ik}}{x[ik]} refer to the quantity on species (column) \eqn{i}
  and sites (rows) \eqn{j} and \eqn{k}. In binary versions \eqn{A} and
  \eqn{B} are the numbers of species on compared sites, and \eqn{J} is
  the number of species that occur on both compared sites similarly as
  in \code{\link{designdist}} (many indices produce identical binary
  versions):
  
  \tabular{ll}{
    \code{euclidean}
    \tab \eqn{d_{jk} = \sqrt{\sum_i (x_{ij}-x_{ik})^2}}{d[jk] = sqrt(sum(x[ij]-x[ik])^2)}
    \cr \tab binary: \eqn{\sqrt{A+B-2J}}{sqrt(A+B-2*J)}
    \cr
    \code{manhattan}
    \tab \eqn{d_{jk}=\sum_i |x_{ij}-x_{ik}|}{d[jk] = sum(abs(x[ij] - x[ik]))}
    \cr \tab binary: \eqn{A+B-2J}{A+B-2*J}
    \cr
    \code{gower}
    \tab \eqn{d_{jk} = (1/M) \sum_i \frac{|x_{ij}-x_{ik}|}{\max x_i-\min
	x_i}}{d[jk] = (1/M) sum(abs(x[ij]-x[ik])/(max(x[i])-min(x[i])))}
    \cr \tab binary: \eqn{(A+B-2J)/M}{(A+B-2*J)/M}
    \cr
    \tab where \eqn{M} is the number of columns (excluding missing
    values)
    \cr
    \code{altGower}
    \tab \eqn{d_{jk} = (1/NZ) \sum_i |x_{ij} - x_{ik}|}{d[jk] = (1/NZ) sum(abs(x[ij] - x[ik]))}
    \cr
    \tab where \eqn{NZ} is the number of non-zero columns excluding
    double-zeros (Anderson et al. 2006).
    \cr \tab binary: \eqn{\frac{A+B-2J}{A+B-J}}{(A+B-2*J)/(A+B-J)}
    \cr
    \code{canberra}
    \tab \eqn{d_{jk}=\frac{1}{NZ} \sum_i
      \frac{|x_{ij}-x_{ik}|}{|x_{ij}|+|x_{ik}|}}{d[jk] = (1/NZ) sum (abs(x[ij]-x[ik])/(abs(x[ij])+abs(x[ik])))}
    \cr
    \tab where \eqn{NZ} is the number of non-zero entries.
    \cr \tab binary: \eqn{\frac{A+B-2J}{A+B-J}}{(A+B-2*J)/(A+B-J)}
    \cr
    \code{clark}
    \tab \eqn{d_{jk}=\sqrt{\frac{1}{NZ} \sum_i
      (\frac{x_{ij}-x_{ik}}{x_{ij}+x_{ik}})^2}}{d[jk] = sqrt( (1/NZ) sum (((x[ij]-x[ik])/(x[ij]+x[ik]))^2))}
    \cr
    \tab where \eqn{NZ} is the number of non-zero entries.
    \cr \tab binary: \eqn{\frac{A+B-2J}{A+B-J}}{(A+B-2*J)/(A+B-J)}
    \cr
    \code{bray}
    \tab \eqn{d_{jk} = \frac{\sum_i |x_{ij}-x_{ik}|}{\sum_i (x_{ij}+x_{ik})}}{d[jk] = (sum abs(x[ij]-x[ik]))/(sum (x[ij]+x[ik]))}
    \cr \tab binary: \eqn{\frac{A+B-2J}{A+B}}{(A+B-2*J)/(A+B)}
    \cr
    \code{kulczynski}
    \tab \eqn{d_{jk} = 1-0.5(\frac{\sum_i \min(x_{ij},x_{ik})}{\sum_i x_{ij}} +
      \frac{\sum_i \min(x_{ij},x_{ik})}{\sum_i x_{ik}} )}{d[jk] 1 - 0.5*(sum(min(x[ij],x[ik]))/(sum x[ij]) + sum(
      min(x[ij],x[ik]))/(sum x[ik]))}
    \cr \tab binary: \eqn{1-(J/A + J/B)/2}{1-(J/A + J/B)/2}
    \cr
    \code{morisita}
    \tab \eqn{d_{jk} =  1 - \frac{2 \sum_i x_{ij} x_{ik}}{(\lambda_j +
	  \lambda_k) \sum_i x_{ij} \sum_i
	  x_{ik}}}{d[jk] =  1 - 2*sum(x[ij]*x[ik])/((lambda[j]+lambda[k]) *
	sum(x[ij])*sum(x[ik]))}, where  
    \cr
    \tab \eqn{\lambda_j = \frac{\sum_i x_{ij} (x_{ij} - 1)}{\sum_i
	x_{ij} \sum_i (x_{ij} - 1)}}{lambda[j] = sum(x[ij]*(x[ij]-1))/sum(x[ij])*sum(x[ij]-1)}
    \cr \tab binary: cannot be calculated
    \cr
    \code{horn}
    \tab Like \code{morisita}, but \eqn{\lambda_j = \sum_i
      x_{ij}^2/(\sum_i x_{ij})^2}{lambda[j] = sum(x[ij]^2)/(sum(x[ij])^2)}
    \cr \tab binary: \eqn{\frac{A+B-2J}{A+B}}{(A+B-2*J)/(A+B)}
    \cr
    \code{binomial}
    \tab \eqn{d_{jk} = \sum_i [x_{ij} \log (\frac{x_{ij}}{n_i}) + x_{ik} \log
      (\frac{x_{ik}}{n_i}) - n_i \log(\frac{1}{2})]/n_i}{d[jk] =
      sum(x[ij]*log(x[ij]/n[i]) + x[ik]*log(x[ik]/n[i]) -
      n[i]*log(1/2))/n[i]},
    \cr
    \tab where \eqn{n_i = x_{ij} + x_{ik}}{n[i] = x[ij] + x[ik]}
    \cr \tab binary: \eqn{\log(2) \times (A+B-2J)}{log(2)*(A+B-2*J)}
    \cr
    \code{cao}
    \tab \eqn{d_{jk} = \frac{1}{S} \sum_i \log
    \left(\frac{n_i}{2}\right) - (x_{ij} \log(x_{ik}) + x_{ik}
    \log(x_{ij}))/n_i}{d[jk] = (1/S) * sum(log(n[i]/2) -
    (x[ij]*log(x[ik]) + x[ik]*log(x[ij]))/n[i])},
  \cr
  \tab where \eqn{S} is the number of species in compared sites and
    \eqn{n_i = x_{ij}+x_{ik}}{n[i] = x[ij] + x[ik]}
  }

  Jaccard index is computed as \eqn{2B/(1+B)}, where \eqn{B} is
  Bray--Curtis dissimilarity.

  Binomial index is derived from Binomial deviance under null hypothesis
  that the two compared communities are equal. It should be able to
  handle variable sample sizes. The index does not have a fixed upper
  limit, but can vary among sites with no shared species. For further
  discussion, see Anderson & Millar (2004).

  Cao index or CYd index (Cao et al. 1997) was suggested as a minimally
  biased index for high beta diversity and variable sampling intensity.
  Cao index does not have a fixed upper limit, but can vary among sites
  with no shared species.  The index is intended for count (integer)
  data, and it is undefined for zero abundances; these are replaced with
  arbitrary value \eqn{0.1} following Cao et al. (1997).  Cao et
  al. (1997) used \eqn{\log_{10}}{log10}, but the current function uses
  natural logarithms so that the values are approximately \eqn{2.30}
  times higher than with 10-based logarithms. Anderson & Thompson (2004)
  give an alternative formulation of Cao index to highlight its
  relationship with Binomial index (above).
  
  Mountford index is defined as \eqn{M = 1/\alpha} where \eqn{\alpha}
  is the parameter of Fisher's logseries assuming that the compared
  communities are samples from the same community
  (cf. \code{\link{fisherfit}}, \code{\link{fisher.alpha}}). The index
  \eqn{M} is found as the positive root of equation \eqn{\exp(aM) +
  \exp(bM) = 1 + \exp[(a+b-j)M]}{exp(a*M) + exp(b*M) = 1 +
  exp((a+b-j)*M)}, where \eqn{j} is the number of species occurring in
  both communities, and \eqn{a} and \eqn{b} are the number of species
  in each separate community (so the index uses presence--absence
  information). Mountford index is usually misrepresented in the
  literature: indeed Mountford (1962) suggested an approximation to be
  used as starting value in iterations, but the proper index is
  defined as the root of the equation above. The function
  \code{vegdist} solves \eqn{M} with the Newton method. Please note
  that if either \eqn{a} or \eqn{b} are equal to \eqn{j}, one of the
  communities could be a subset of other, and the dissimilarity is
  \eqn{0} meaning that non-identical objects may be regarded as
  similar and the index is non-metric. The Mountford index is in the
  range \eqn{0 \dots \log(2)}{0 \dots log(2)}, but the dissimilarities
  are divided by \eqn{\log(2)}{log(2)} so that the results will be in
  the conventional range \eqn{0 \dots 1}.

  Raup--Crick dissimilarity (\code{method = "raup"}) is a probabilistic
  index based on presence/absence data.  It is defined as \eqn{1 -
  prob(j)}, or based on the probability of observing at least \eqn{j}
  species in shared in compared communities.  The current function uses
  analytic result from hypergeometric distribution
  (\code{\link{phyper}}) to find the probabilities.  This probability
  (and the index) is dependent on the number of species missing in both
  sites, and adding all-zero species to the data or removing missing
  species from the data will influence the index.  The probability (and
  the index) may be almost zero or almost one for a wide range of
  parameter values.  The index is nonmetric: two communities with no
  shared species may have a dissimilarity slightly below one, and two
  identical communities may have dissimilarity slightly above zero. The
  index uses equal occurrence probabilities for all species, but Raup
  and Crick originally suggested that sampling probabilities should be
  proportional to species frequencies (Chase et al. 2011). A simulation
  approach with unequal species sampling probabilities is implemented in
  \code{\link{raupcrick}} function following Chase et al. (2011).  The
  index can be also used for transposed data to give a probabilistic
  dissimilarity index of species co-occurrence (identical to Veech
  2013).
  
  Chao index tries to take into account the number of unseen species
  pairs, similarly as in \code{method = "chao"} in
  \code{\link{specpool}}. Function \code{vegdist} implements a
  Jaccard, index defined as
  \eqn{1-\frac{U \times V}{U + V - U \times V}}{1 - U*V/(U + V - U*V)};
  other types can be defined with function \code{\link{chaodist}}. In Chao
  equation, \eqn{U = C_j/N_j + (N_k - 1)/N_k \times a_1/(2 a_2) \times
  S_j/N_j}{U = C[j]/N[j] + (N[k] -1)/N[k] * a1/(2*a2) * S[j]/N[j]},
  and \eqn{V} is similar except for site index
  \eqn{k}. \eqn{C_j}{C[j]} is the total number of individuals in the
  species of site \eqn{j} that are shared with site \eqn{k},
  \eqn{N_j}{N[j]} is the total number of individuals at site \eqn{j},
  \eqn{a_1}{a1} (and \eqn{a_2}{a2}) are the number of species
  occurring in site \eqn{j} that have only one (or two) individuals in
  site \eqn{k}, and \eqn{S_j}{S[j]} is the total number of individuals
  in the species present at site \eqn{j} that occur with only one
  individual in site \eqn{k} (Chao et al. 2005).

  Morisita index can be used with genuine count data (integers) only. Its
  Horn--Morisita variant is able to handle any abundance data.

  Mahalanobis distances are Euclidean distances of a matrix where
  columns are centred, have unit variance, and are uncorrelated.  The
  index is not commonly used for community data, but it is sometimes
  used for environmental variables. The calculation is based on
  transforming data matrix and then using Euclidean distances
  following Mardia et al. (1979). The Mahalanobis transformation
  usually fails when the number of columns is larger than the number
  of rows (sampling units). When the transformation fails, the
  distances are nearly constant except for small numeric noise. Users
  must check that the returned Mahalanobis distances are meaningful.

  Euclidean and Manhattan dissimilarities are not good in gradient
  separation without proper standardization but are still included for
  comparison and special needs.

  Bray--Curtis and Jaccard indices are rank-order similar, and some
  other indices become identical or rank-order similar after some 
  standardizations, especially with presence/absence transformation of
  equalizing site totals with \code{\link{decostand}}. Jaccard index is
  metric, and probably should be preferred instead of the default
  Bray-Curtis which is semimetric. 

  The naming conventions vary. The one adopted here is traditional
  rather than truthful to priority. The function finds either
  quantitative or binary variants of the indices under the same name,
  which correctly may refer only to one of these alternatives For
  instance, the Bray
  index is known also as Steinhaus, Czekanowski and \enc{Sørensen}{Sorensen} index.
  The quantitative version of Jaccard should probably called
  \enc{Ružička}{Ruzicka} index.
  The abbreviation \code{"horn"} for the Horn--Morisita index is
  misleading, since there is a separate Horn index. The abbreviation
  will be changed if that index is implemented in \code{vegan}. 
}
\value{
  Should provide a drop-in replacement for \code{\link{dist}} and
  return a distance object of the same type. 
}
\references{
  Anderson, M.J. and Millar, R.B. (2004). Spatial variation and effects
  of habitat on temperate reef fish assemblages in northeastern New
  Zealand.  \emph{Journal of Experimental Marine Biology and Ecology}
  305, 191--221.

  Anderson, M.J., Ellingsen, K.E. & McArdle, B.H. (2006). Multivariate
  dispersion as a measure of beta diversity. \emph{Ecology Letters} 
  9, 683--693.

  Anderson, M.J & Thompson, A.A. (2004). Multivariate control charts for
  ecological and environmental monitoring. \emph{Ecological
    Applications} 14, 1921--1935.

  Cao, Y., Williams, W.P. & Bark, A.W. (1997). Similarity measure bias
  in river benthic Auswuchs community analysis. \emph{Water
  Environment Research} 69, 95--106.

  Chao, A., Chazdon, R. L., Colwell, R. K. and Shen, T. (2005). A new
  statistical approach for assessing similarity of species composition
  with incidence and abundance data. \emph{Ecology Letters} 8, 148--159.

  Chase, J.M., Kraft, N.J.B., Smith, K.G., Vellend, M. and Inouye,
  B.D. (2011). Using null models to disentangle variation in community
  dissimilarity from variation in \eqn{\alpha}{alpha}-diversity.
  \emph{Ecosphere} 2:art24 \doi{10.1890/ES10-00117.1}
   
  Faith, D. P, Minchin, P. R. and Belbin, L. (1987).
  Compositional dissimilarity as a robust measure of ecological
  distance. \emph{Vegetatio} 69, 57--68.

  Gower, J. C. (1971). A general coefficient of similarity and some
  of its properties. \emph{Biometrics} 27, 623--637.

  Krebs, C. J. (1999). \emph{Ecological Methodology.} Addison Wesley
  Longman.

  Legendre, P. & De \enc{Cáceres}{Caceres}, M. (2012). Beta diversity as
  the variance of community data: dissimilarity coefficients and
  partitioning. \emph{Ecology Letters} 16, 951--963.
  \doi{10.1111/ele.12141}

  Legendre, P. and Legendre, L. (2012) \emph{Numerical Ecology}. 3rd English
  ed. Elsevier.

  Mardia, K.V., Kent, J.T. and Bibby, J.M. (1979). \emph{Multivariate analysis}.
  Academic Press.

  Mountford, M. D. (1962). An index of similarity and its application to
  classification problems. In: P.W.Murphy (ed.),
  \emph{Progress in Soil Zoology}, 43--50. Butterworths.

  Veech, J. A. (2013). A probabilistic model for analysing species
  co-occurrence. \emph{Global Ecology and Biogeography} 22, 252--260. 

  Wolda, H. (1981). Similarity indices, sample size and
  diversity. \emph{Oecologia} 50, 296--302.
}

\author{ Jari Oksanen, with contributions from Tyler Smith (Gower index)
  and Michael Bedward (Raup--Crick index). }

\note{The function is an alternative to \code{\link{dist}} adding some
  ecologically meaningful indices.  Both methods should produce similar
  types of objects which can be interchanged in any method accepting
  either.  Manhattan and Euclidean dissimilarities should be identical
  in both methods. Canberra index is divided by the number of variables
  in \code{vegdist}, but not in \code{\link{dist}}.  So these differ by
  a constant multiplier, and the alternative in \code{vegdist} is in
  range (0,1).  Function \code{\link[cluster]{daisy}} (package
  \pkg{cluster}) provides alternative implementation of Gower index that
  also can handle mixed data of numeric and class variables.  There are
  two versions of Gower distance (\code{"gower"}, \code{"altGower"})
  which differ in scaling: \code{"gower"} divides all distances by the
  number of observations (rows) and scales each column to unit range,
  but \code{"altGower"} omits double-zeros and divides by the number of
  pairs with at least one above-zero value, and does not scale columns
  (Anderson et al. 2006).  You can use \code{\link{decostand}} to add
  range standardization to \code{"altGower"} (see Examples). Gower
  (1971) suggested omitting double zeros for presences, but it is often
  taken as the general feature of the Gower distances. See Examples for
  implementing the Anderson et al. (2006) variant of the Gower index.

  Most dissimilarity indices in \code{vegdist} are designed for
  community data, and they will give misleading values if there are
  negative data entries.  The results may also be misleading or
  \code{NA} or \code{NaN} if there are empty sites.  In principle, you
  cannot study species composition without species and you should remove
  empty sites from community data.
}

\seealso{
 Function \code{\link{designdist}} can be used for defining your own
 dissimilarity index. Alternative dissimilarity functions include
 \code{\link{dist}} in base \R,
  \code{\link[cluster]{daisy}} (package \pkg{cluster}), and
  \code{\link[labdsv]{dsvdis}} (package \pkg{labdsv}).  Function
  \code{\link{betadiver}} provides indices intended for the analysis of
  beta diversity.}

\examples{
data(varespec)
vare.dist <- vegdist(varespec)
# Orlóci's Chord distance: range 0 .. sqrt(2)
vare.dist <- vegdist(decostand(varespec, "norm"), "euclidean")
# Anderson et al.  (2006) version of Gower
vare.dist <- vegdist(decostand(varespec, "log"), "altGower")
# Range standardization with "altGower" (that excludes double-zeros)
vare.dist <- vegdist(decostand(varespec, "range"), "altGower")
}
\keyword{ multivariate }

\name{pyrifos}
\alias{pyrifos} 
\docType{data}
\title{Response of Aquatic Invertebrates to Insecticide Treatment}
\description{
  The data are log transformed abundances of aquatic invertebrate in
  twelve ditches studied in eleven times before and after an insecticide
  treatment.
 
}
\usage{data(pyrifos)}
\format{
  A data frame with 132 observations on the log-transformed (\code{log(10*x + 1)}) abundances
  of 178 species. There are only twelve sites (ditches, mesocosms), but
  these were studied repeatedly in eleven occasions. The treatment
  levels, treatment times, or ditch ID's are not in the data frame, but
  the data are very regular, and the example below shows how to obtain
  these external variables.
}

\details{  This data set was obtained from an experiment in outdoor
  experimental ditches. Twelve mesocosms were allocated at random to
  treatments; four served as controls, and the remaining eight were 
  treated once with the insecticide chlorpyrifos, with nominal dose
  levels of 0.1, 0.9, 6, and 44  \eqn{\mu}{mu }g/ L in two mesocosms
  each. The example data set invertebrates.
  Sampling was done 11 times, from week -4 pre-treatment through
  week 24 post-treatment, giving a total of 132 samples (12 mesocosms
  times 11 sampling dates), see van den Brink & ter Braak (1999) for
  details. The data set contains only the species data,
  but the example below shows how to obtain the treatment, time and
  ditch ID variables. 
}
\source{
  CANOCO 4 example data, with the permission of Cajo J. F. ter Braak.
}
\references{
  van den Brink, P.J. & ter Braak, C.J.F. (1999). Principal response
  curves: Analysis of time-dependent multivariate responses of
  biological community to stress. Environmental Toxicology and
  Chemistry, 18, 138--148.
}
\examples{
data(pyrifos)
ditch <- gl(12, 1, length=132)
week <- gl(11, 12, labels=c(-4, -1, 0.1, 1, 2, 4, 8, 12, 15, 19, 24))
dose <- factor(rep(c(0.1, 0, 0, 0.9, 0, 44, 6, 0.1, 44, 0.9, 0, 6), 11))
}
\keyword{datasets}

\name{varpart}
\alias{varpart}
\alias{varpart2}
\alias{varpart3}
\alias{varpart4}
\alias{showvarparts}
\alias{plot.varpart}
\alias{plot.varpart234}
\alias{simpleRDA2}
\alias{simpleDBRDA}

\title{Partition the Variation of Community Matrix by 2, 3, or 4 Explanatory Matrices }

\description{ 

  The function partitions the variation in community data or community
  dissimilarities with respect to two, three, or four explanatory
  tables, using adjusted \eqn{R^2}{R-squared} in redundancy analysis
  ordination (RDA) or distance-based redundancy analysis. If response
  is a single vector, partitioning is by partial regression. Collinear
  variables in the explanatory tables do NOT have to be removed prior
  to partitioning.

}

\usage{
varpart(Y, X, ..., data, chisquare = FALSE, transfo, scale = FALSE,
    add = FALSE, sqrt.dist = FALSE, permutations)
showvarparts(parts, labels, bg = NULL, alpha = 63, Xnames,
    id.size = 1.2,  ...)
\method{plot}{varpart234}(x, cutoff = 0, digits = 1, ...)
}

\arguments{

\item{Y}{ Data frame or matrix containing the response data table or
  dissimilarity structure inheriting from \code{\link{dist}}. In
  community ecology, that table is often a site-by-species table or a
  dissimilarity object. }

\item{X}{Two to four explanatory models, variables or tables.  These can
  be defined in three alternative ways: (1) one-sided model formulae
  beginning with \code{~} and then defining the model, (2) name of a
  single numeric or factor variable, or (3) name of matrix with numeric
  or data frame with numeric and factor variables.  The model formulae
  can have factors, interaction terms and transformations of
  variables. The names of the variables in the model formula are found
  in data frame given in \code{data} argument, and if not found there,
  in the user environment.  Single variables, data frames or matrices
  are found in the user environment.  All entries till the next argument
  (\code{data} or \code{transfo}) are interpreted as explanatory models,
  and the names of these extra arguments cannot be abbreviated nor
  omitted.  }

\item{...}{Other parameters passed to functions. NB, arguments after
  dots cannot be abbreviated but they must be spelt out completely.}

\item{data}{The data frame with the variables used in the formulae in
  \code{X}.}

\item{chisquare}{Partition Chi-square or the inertia of Correspondence
  Analysis (\code{\link{cca}}).}

\item{transfo}{ Transformation for \code{Y} (community data) using
  \code{\link{decostand}}.  All alternatives in \code{decostand} can
  be used, and those preserving Euclidean metric include
  \code{"hellinger"}, \code{"chi.square"}, \code{"total"},
  \code{"norm"}. Ignored if \code{Y} are dissimilarities.}

\item{scale}{Should the columns of \code{Y} be standardized to unit
  variance. Ignored if \code{Y} are dissimilarities.}

\item{add}{Add a constant to the non-diagonal values to euclidify
  dissimilarities (see \code{\link{wcmdscale}} for details). Choice
  \code{"lingoes"} (or \code{TRUE}) use the recommended method of
  Legendre & Anderson (1999: \dQuote{method 1}) and \code{"cailliez"}
  uses their \dQuote{method 2}. The argument has an effect only when
  \code{Y} are dissimilarities.}

\item{sqrt.dist}{Take square root of dissimilarities. This often
  euclidifies dissimilarities. NB., the argument name cannot be
  abbreviated. The argument has an effect only when \code{Y} are
  dissimilarities.}

\item{permutations}{If \code{chisquare = TRUE}, the adjusted
    \eqn{R^2}{R-squared} is estimated by permutations, and this
    paramater can be a list of control values for the permutations as
    returned by the function \code{\link[permute]{how}}, or the number
    of permutations required, or a permutation matrix where each row
    gives the permuted indices.}

\item{parts}{Number of explanatory tables (circles) displayed.}
\item{labels}{Labels used for displayed fractions. Default is to use
  the same letters as in the printed output.}
\item{bg}{Fill colours of circles or ellipses.}
\item{alpha}{Transparency of the fill colour.  The argument takes
    precedence over possible transparency definitions of the
    colour. The value must be in range \eqn{0...255}, and low values
    are more transparent.  Transparency is not available in all
    graphics devices or file formats.}
  
\item{Xnames}{Names for sources of variation. Default names are \code{X1},
  \code{X2}, \code{X3} and \code{X4}. \code{Xnames=NA},
  \code{Xnames=NULL} and \code{Xnames=""} produce no names. The names
  can be changed to other names. It is often best to use short names. }

\item{id.size}{A numerical value giving the character expansion factor
  for the names of circles or ellipses. }

\item{x}{The \code{varpart} result.}
\item{cutoff}{The values below \code{cutoff} will not be displayed.}
\item{digits}{The number of significant digits; the number of decimal
  places is at least one higher.}
}

\details{

  The functions partition the variation in \code{Y} into components
  accounted for by two to four explanatory tables and their combined
  effects. If \code{Y} is a multicolumn data frame or matrix, the
  partitioning is based on redundancy analysis (RDA, see
  \code{\link{rda}}) or on constrained correspondence analysis if
  \code{chisquare = TRUE} (CCA, see \code{\link{cca}}).  If \code{Y}
  is a single variable, the partitioning is based on linear
  regression.  If \code{Y} are dissimilarities, the decomposition is
  based on distance-based redundancy analysis (db-RDA, see
  \code{\link{capscale}}) following McArdle & Anderson (2001). The
  input dissimilarities must be compatible to the results of
  \code{\link{dist}}. \pkg{Vegan} functions \code{\link{vegdist}},
  \code{\link{designdist}}, \code{\link{raupcrick}} and
  \code{\link{betadiver}} produce such objects, as do many other
  dissimilarity functions in \R packages. However, symmetric square
  matrices are not recognized as dissimilarities but must be
  transformed with \code{\link{as.dist}}.  Partitioning will be made
  to squared dissimilarities analogously to using variance with
  rectangular data -- unless \code{sqrt.dist = TRUE} was specified.

  The function primarily uses adjusted \eqn{R^2}{R-squared} to assess
  the partitions explained by the explanatory tables and their
  combinations (see \code{\link{RsquareAdj}}), because this is the
  only unbiased method (Peres-Neto et al., 2006). The raw
  \eqn{R^2}{R-squared} for basic fractions are also displayed, but
  these are biased estimates of variation explained by the explanatory
  table. In correspondence analysis (\code{chisquare = TRUE}), the
  adjusted \eqn{R^2}{R-squared} are found by permutation and they vary
  in repeated analyses.

  The identifiable fractions are designated by lower case alphabets. The
  meaning of the symbols can be found in the separate document (use
  \code{browseVignettes("vegan")}), or can be displayed graphically
  using function \code{showvarparts}.

  A fraction is testable if it can be directly expressed as an RDA or
  db-RDA model.  In these cases the printed output also displays the
  corresponding RDA model using notation where explanatory tables
  after \code{|} are conditions (partialled out; see \code{\link{rda}}
  for details). Although single fractions can be testable, this does
  not mean that all fractions simultaneously can be tested, since the
  number of testable fractions is higher than the number of estimated
  models. The non-testable components are found as differences of
  testable components. The testable components have permutation
  variance in correspondence analysis (\code{chisquare = TRUE}), and
  the non-testable components have even higher variance.

  An abridged explanation of the alphabetic symbols for the individual
  fractions follows, but computational details should be checked in the
  vignette (readable with \code{browseVignettes("vegan")}) or in the
  source code.

  With two explanatory tables, the fractions explained 
  uniquely by each of the two tables are \code{[a]} and
  \code{[c]}, and their joint effect
  is  \code{[b]} following Borcard et al. (1992).

  With three explanatory tables, the fractions explained uniquely
  by each of the three tables are  
  \code{[a]} to \code{[c]}, joint fractions between two tables are
  \code{[d]} to \code{[f]}, and the joint fraction between all three
  tables is \code{[g]}.

  With four explanatory tables, the fractions explained uniquely by each
  of the four tables are \code{[a]}
  to \code{[d]}, joint fractions between two tables are \code{[e]} to
  \code{[j]}, joint fractions between three variables are \code{[k]} to
  \code{[n]}, and the joint fraction between all four tables is
  \code{[o]}.

  There is a \code{plot} function that displays the Venn diagram and
  labels each intersection (individual fraction) with the adjusted R
  squared if this is higher than \code{cutoff}.  A helper function
  \code{showvarpart} displays the fraction labels. The circles and
  ellipses are labelled by short default names or by names defined by
  the user in argument \code{Xnames}. Longer explanatory file names can
  be written on the varpart output plot as follows: use option
  \code{Xnames=NA}, then add new names using the \code{text} function. A
  bit of fiddling with coordinates (see \code{\link{locator}}) and
  character size should allow users to place names of reasonably short
  lengths on the \code{varpart} plot.
  
}

\value{
  Function \code{varpart} returns an
  object of class \code{"varpart"} with items \code{scale} and
  \code{transfo} (can be missing) which hold information on
  standardizations, \code{tables} which contains names of explanatory
  tables, and \code{call} with the function \code{\link{call}}. The
  function \code{varpart} calls function \code{varpart2},
  \code{varpart3} or \code{varpart4} which return an object of class
  \code{"varpart234"} and saves its result in the item \code{part}.
  The items in this object are:
  \item{SS.Y}{Sum of squares of matrix \code{Y}.}
  \item{n}{Number of observations (rows).}
  \item{nsets}{Number of explanatory tables}
  \item{bigwarning}{Warnings on collinearity.}
  \item{fract}{Basic fractions from all estimated constrained models.}
  \item{indfract}{Individual fractions or all possible subsections in
    the Venn diagram (see \code{showvarparts}).}
  \item{contr1}{Fractions that can be found after conditioning on single
    explanatory table in models with three or four explanatory tables.}
  \item{contr2}{Fractions that can be found after conditioning on two
    explanatory tables in models with four explanatory tables.}
}
\section{Fraction Data Frames}{
  Items \code{fract},
  \code{indfract}, \code{contr1} and \code{contr2} are all data frames with
  items:
  \itemize{
  \item{\code{Df}: }{Degrees of freedom of numerator of the \eqn{F}-statistic
    for the fraction.}
  \item{\code{R.square}: }{Raw \eqn{R^2}{R-squared}. This is calculated only for
    \code{fract} and this is \code{NA} in other items.}
  \item{\code{Adj.R.square}: }{Adjusted \eqn{R^2}{R-squared}.}
  \item{\code{Testable}: }{If the fraction can be expressed as a (partial) RDA
    model, it is directly \code{Testable}, and this field is
    \code{TRUE}.  In that case the fraction label also gives the
    specification of the testable RDA model.}
  }
}

\references{ 

(a) References on variation partitioning

Borcard, D., P. Legendre & P. Drapeau. 1992. Partialling out the spatial
component of ecological variation. Ecology 73: 1045--1055.

Legendre, P. & L. Legendre. 2012. Numerical ecology, 3rd English edition.
Elsevier Science BV, Amsterdam.

(b) Reference on transformations for species data

Legendre, P. and E. D. Gallagher. 2001. Ecologically meaningful
transformations for ordination of species data. Oecologia 129: 271--280.

(c) Reference on adjustment of the bimultivariate redundancy statistic

Peres-Neto, P., P. Legendre, S. Dray and D. Borcard. 2006. Variation partitioning
of species data matrices: estimation and comparison of fractions.
Ecology 87: 2614--2625.

(d) References on partitioning of dissimilarities

Legendre, P. & Anderson, M. J. (1999). Distance-based redundancy
analysis: testing multispecies responses in multifactorial ecological
experiments. \emph{Ecological Monographs} 69, 1--24.

McArdle, B.H. & Anderson, M.J. (2001). Fitting multivariate models
to community data: a comment on distance-based redundancy
analysis. Ecology 82, 290-297.

}

\author{ Pierre Legendre, Departement de Sciences Biologiques, Universite de
Montreal, Canada.  Further developed by Jari Oksanen. }

\note{

  You can use command \code{browseVignettes("vegan")} to display
  document which presents Venn diagrams showing the fraction names in
  partitioning the variation of Y with respect to 2, 3, and 4 tables of
  explanatory variables, as well as the equations used in variation
  partitioning.

  The functions frequently give negative estimates of variation.
  Adjusted \eqn{R^2}{R-squared} can be negative for any fraction;
  unadjusted \eqn{R^2}{R-squared} of testable fractions of variances
  will be non-negative.  Non-testable fractions cannot be found
  directly, but by subtracting different models, and these subtraction
  results can be negative.  The fractions are orthogonal, or linearly
  independent, but more complicated or nonlinear dependencies can
  cause negative non-testable fractions. Any fraction can be negative
  for non-Euclidean dissimilarities because the underlying db-RDA model
  can yield negative eigenvalues (see \code{\link{capscale}},
  \code{\link{dbrda}}). These negative eigenvalues in the underlying
  analysis can be avoided with arguments \code{sqrt.dist} and \code{add}
  which have a similar effect as in \code{\link{capscale}}: the square
  roots of several dissimilarities do not have negative eigenvalues, and
  no negative eigenvalues are produced after Lingoes or Cailliez
  adjustment, which in effect add random variation to the
  dissimilarities.

  A simplified, fast version of RDA, CCA adn dbRDA are used (functions
  \code{simpleRDA2}, \code{simpleCCA} and \code{simpleDBRDA}).  The
  actual calculations are done in functions \code{varpart2} to
  \code{varpart4}, but these are not intended to be called directly by
  the user.

}

\seealso{ 
  For analysing testable fractions, see \code{\link{rda}} and
  \code{\link{anova.cca}}. For data transformation, see
  \code{\link{decostand}}. Function \code{\link{inertcomp}} gives
  (unadjusted) components of variation for each species or site
  separately.  Function \code{\link{rda}} displays unadjusted
  components in its output, but \code{\link{RsquareAdj}} will give
  adjusted \eqn{R^2}{R-squared} that are similar to the current
  function also for partial models.  
}

\examples{
data(mite)
data(mite.env)
data(mite.pcnm)

# Two explanatory data frames -- Hellinger-transform Y
mod <- varpart(mite, mite.env, mite.pcnm, transfo="hel")
mod

## Use fill colours
showvarparts(2, bg = c("hotpink","skyblue"))
plot(mod, bg = c("hotpink","skyblue"))
## Test fraction [a] using partial RDA, '~ .' in formula tells to use
## all variables of data mite.env.
aFrac <- rda(decostand(mite, "hel"), mite.env, mite.pcnm)
anova(aFrac)
## RsquareAdj gives the same result as component [a] of varpart
RsquareAdj(aFrac)

## Partition Bray-Curtis dissimilarities
varpart(vegdist(mite), mite.env, mite.pcnm)
## Three explanatory tables with formula interface
mod <- varpart(mite, ~ SubsDens + WatrCont, ~ Substrate + Shrub + Topo,
   mite.pcnm, data=mite.env, transfo="hel")
mod
showvarparts(3, bg=2:4)
plot(mod, bg=2:4)

## Use RDA to test fraction [a]
## Matrix can be an argument in formula
rda.result <- rda(decostand(mite, "hell") ~ SubsDens + WatrCont +
   Condition(Substrate + Shrub + Topo) +
   Condition(as.matrix(mite.pcnm)), data = mite.env)
anova(rda.result)

## Four explanatory tables
mod <- varpart(mite, ~ SubsDens + WatrCont, ~Substrate + Shrub + Topo,
  mite.pcnm[,1:11], mite.pcnm[,12:22], data=mite.env, transfo="hel")
mod
plot(mod, bg=2:5)
## Show values for all partitions by putting 'cutoff' low enough:
plot(mod, cutoff = -Inf, cex = 0.7, bg=2:5)
}

\keyword{ multivariate }

\encoding{UTF-8}
\name{bgdispersal}
\alias{bgdispersal}

\title{ Coefficients of Biogeographical Dispersal Direction }

\description{ This function computes coefficients of dispersal direction
between geographically connected areas, as defined by Legendre and
Legendre (1984), and also described in Legendre and Legendre (2012,
section 13.3.4). }

\usage{
bgdispersal(mat, PAonly = FALSE, abc = FALSE)
}

\arguments{
  \item{mat}{ Data frame or matrix containing a community composition
    data table (species presence-absence or abundance data). }
  \item{PAonly}{ \code{FALSE} if the four types of coefficients, DD1 to
    DD4, are requested; \code{TRUE} if \code{DD1} and \code{DD2} only are
    sought (see Details). }
  \item{abc}{If \code{TRUE}, return tables \code{a}, \code{b} and \code{c}
    used in \code{DD1} and \code{DD2}.}
  }
\details{
The signs of the DD coefficients indicate the 
direction of dispersal, provided that the 
asymmetry is significant. A positive sign 
indicates dispersal from the first (row in DD 
tables) to the second region (column); a negative 
sign indicates the opposite. A McNemar test of 
asymmetry is computed from the presence-absence 
data to test the hypothesis of a significant 
asymmetry between the two areas under comparison.

In the input data table, the rows are sites or 
areas, the columns are taxa. Most often, the taxa 
are species, but the coefficients can be computed 
from genera or families as well. DD1 and DD2 only 
are computed for presence-absence data. The four 
types of coefficients are computed for 
quantitative data, which are converted to 
presence-absence for the computation of DD1 and 
DD2. \code{PAonly = FALSE} indicates that the four types 
of coefficients are requested. \code{PAonly = TRUE} if DD1 
and DD2 only are sought. }


\value{
Function \code{bgdispersal} returns a list containing the following matrices:
\item{ DD1 }{ \eqn{DD1_{j,k} = (a(b - c))/((a + b + c)^2)}{DD1[j,k] = (a * (b - c))/((a + b + c)^2)} }
\item{ DD2 }{ \eqn{DD2_{j,k} = (2 a (b - c))/((2a + b + c)  (a + b +
    c))}{DD2[j,k] = (2*a * (b - c))/((2*a + b + c) * (a + b +
    c))}
  where \eqn{a}, \eqn{b}, and \eqn{c} have the 
same meaning as in the computation of binary 
similarity coefficients. }
\item{ DD3 }{ \eqn{DD3_{j,k} = {W(A-B) / (A+B-W)^2} }{DD3[j,k] = W*(A-B) / (A+B-W)^2} }
\item{ DD4 }{ \eqn{DD4_{j,k} = 2W(A-B) / ((A+B)(A+B-W))}{DD4[j,k] = 2*W*(A-B) / ((A+B)*(A+B-W))}
where \code{W = sum(pmin(vector1, vector2))}, \code{A = sum(vector1)},
\code{B = sum(vector2)} }

\item{ McNemar }{ McNemar chi-square statistic of asymmetry (Sokal and
  Rohlf 1995):
  \eqn{2(b \log(b) + c \log(c) - (b+c) \log((b+c)/2)) / q}{2*(b*log(b) + c*log(c) - (b+c)*log((b+c)/2)) / q},
  where \eqn{q = 1 + 1/(2(b+c))}{q = 1 + 1/(2*(b+c))}
  (Williams correction for continuity) }
\item{ prob.McNemar }{ probabilities associated 
with McNemar statistics, chi-square test. H0: no 
asymmetry in \eqn{(b-c)}. }

}


\references{ 
  Legendre, P. and V. Legendre. 1984. Postglacial dispersal of
  freshwater fishes in the Québec
  peninsula. \emph{Can. J. Fish. Aquat. Sci.} \strong{41}: 1781-1802.
  
  Legendre, P. and L. Legendre. 2012. \emph{Numerical ecology}, 3rd
  English edition. Elsevier Science BV, Amsterdam.
  
  Sokal, R. R. and F. J. Rohlf. 1995. \emph{Biometry. The principles and
  practice of statistics in biological research.} 3rd
  edn. W. H. Freeman, New York. 
}


\author{ Pierre Legendre, Departement de Sciences Biologiques,
  Universite de Montreal}

\note{The function uses a more powerful alternative for the McNemar test
  than the classical formula. The classical formula was constructed in
  the spirit of Pearson's Chi-square, but the formula in this function
  was constructed in the spirit of Wilks Chi-square or the \eqn{G}
  statistic. Function \code{\link{mcnemar.test}} uses the classical
  formula. The new formula was introduced in \pkg{vegan} version
  1.10-11, and the older implementations of \code{bgdispersal} used the
  classical formula.  }

\examples{
mat <- matrix(c(32,15,14,10,70,30,100,4,10,30,25,0,18,0,40,
  0,0,20,0,0,0,0,4,0,30,20,0,0,0,0,25,74,42,1,45,89,5,16,16,20),
  4, 10, byrow=TRUE)
bgdispersal(mat)
}

\keyword{ multivariate }
\keyword{ nonparametric }

\name{mso}
\alias{mso}
\alias{msoplot}

\title{ Functions for performing and displaying a spatial partitioning
of cca or rda results}

\description{ The function \code{mso} adds an attribute \code{vario} to
  an object of class \code{"cca"} that describes the spatial
  partitioning of the \code{\link{cca}} object and performs an optional
  permutation test for the spatial independence of residuals. The
  function \code{plot.mso} creates a diagnostic plot of the spatial
  partitioning of the \code{"cca"} object.  }

\usage{
mso(object.cca, object.xy, grain = 1, round.up = FALSE, permutations = 0)
msoplot(x, alpha = 0.05, explained = FALSE, ylim = NULL, legend = "topleft", ...)
}
\arguments{
  \item{object.cca}{ An object of class cca, created by the \code{\link{cca}} or
    \code{\link{rda}} function.}
   \item{object.xy}{ A vector, matrix or data frame with the spatial
    coordinates of the data represented by \code{object.cca}. The
    number of rows must match the number of observations (as given by
    \code{nobs}) in \code{cca.object}. Alternatively, interpoint
    distances can be supplied as a \code{\link{dist}} object. }
   \item{grain}{ Interval size for distance classes.}
  \item{round.up}{ Determines the choice of breaks. If false, distances
    are rounded to the nearest multiple of grain. If true, distances are
    rounded to the upper multiple of grain.} 
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}
  \item{x}{A result object of \code{mso}.}
  \item{alpha}{ Significance level for the two-sided permutation test of
    the Mantel statistic for spatial independence of residual inertia
    and for the point-wise envelope of the variogram of the total
    variance. A Bonferroni-type correction can be achieved by dividing
    the overall significance value (e.g. 0.05) by the number of distance
    classes.} 
  \item{explained}{ If false, suppresses the plotting of the variogram
    of explained variance.}
  \item{ylim}{Limits for y-axis.}
  \item{legend}{The x and y co-ordinates to be used to position the legend. 
    They can be specified by keyword or in any way which is accepted 
    by \code{\link{legend}}.}
  \item{\dots}{Other arguments passed to functions.}
}

\details{

  The Mantel test is an adaptation of the function
  \code{\link{mantel}} to the parallel testing of several distance
  classes and similar to multivariate \code{\link{mantel.correlog}}.
  It compares the mean inertia in each distance class to the pooled
  mean inertia of all other distance classes.

  If there are explanatory variables (RDA, CCA, pRDA, pCCA) and a
  significance test for residual autocorrelation was performed when
  running the function \code{mso}, the function \code{plot.mso} will
  print an estimate of how much the autocorrelation (based on
  significant distance classes) causes the global error variance of the
  regression analysis to be underestimated 

}
\value{
  The function \code{mso} returns an amended \code{cca} or \code{rda}
  object with the additional attributes \code{grain}, \code{H},
  \code{H.test} and \code{vario}.
  \item{grain}{The grain attribute defines the interval size of the
    distance classes .} 
  \item{H }{ H is an object of class 'dist' and contains the geographic
    distances between observations.} 
  \item{H.test }{ H.test contains a set of dummy variables that describe
    which pairs of observations (rows = elements of \code{object$H}) fall in
    which distance class (columns). } 
  \item{vario }{ The vario attribute is a data frame that contains some
    or all of the following components for the rda case (cca case in
      brackets):
    \describe{
      \item{\code{H}}{Distance class as multiples of grain.}
      \item{\code{Dist}}{ Average distance of pairs of observations in distance class H.}
      \item{n }{ Number of unique pairs of observations in distance class
	H.} 
      \item{\code{All}}{ Empirical (chi-square) variogram of total variance
	(inertia).} 
      \item{\code{Sum}}{ Sum of empirical (chi-square) variograms of explained
	and residual variance (inertia).} 
      \item{\code{CA}}{ Empirical (chi-square) variogram of residual variance
	(inertia).} 
      \item{\code{CCA}}{ Empirical (chi-square) variogram of explained variance
	(inertia).} 
      \item{\code{pCCA}}{ Empirical (chi-square) variogram of conditioned
	variance (inertia).} 
      \item{\code{se}}{ Standard error of the empirical (chi-square) variogram
	of total variance (inertia).} 
      \item{\code{CA.signif}}{P-value of permutation test for spatial
	independence of residual variance (inertia).} 
    }
  }
}
  
\references{ Wagner, H.H. 2004. Direct multi-scale ordination with
  canonical correspondence analysis. \emph{Ecology} 85: 342--351. } 
\author{ The responsible author was Helene Wagner.} 
\note{ The function is based on the code published in the Ecological
  Archives E085-006 (\doi{10.1890/02-0738}).  }

\seealso{ \code{\link{mantel.correlog}}. }

\examples{
## Reconstruct worked example of Wagner (submitted):
X <- matrix(c(1, 2, 3, 2, 1, 0), 3, 2)
Y <- c(3, -1, -2)
tmat <- c(1:3)
## Canonical correspondence analysis (cca):
Example.cca <- cca(X, Y)
Example.cca <- mso(Example.cca, tmat)
msoplot(Example.cca)
Example.cca$vario

## Correspondence analysis (ca):
Example.ca <- mso(cca(X), tmat)
msoplot(Example.ca)

## Unconstrained ordination with test for autocorrelation
## using oribatid mite data set as in Wagner (2004)
data(mite)
data(mite.env)
data(mite.xy)

mite.cca <- cca(log(mite + 1))
mite.cca <- mso(mite.cca, mite.xy, grain =  1, permutations = 99)
msoplot(mite.cca)
mite.cca

## Constrained ordination with test for residual autocorrelation
## and scale-invariance of species-environment relationships
mite.cca <- cca(log(mite + 1) ~ SubsDens + WatrCont + Substrate + Shrub + Topo, mite.env)
mite.cca <- mso(mite.cca, mite.xy, permutations = 99)
msoplot(mite.cca)
mite.cca
}
\keyword{ spatial }
\keyword{ multivariate }

\name{prc}
\alias{prc}
\alias{summary.prc}
\alias{plot.prc}

\title{Principal Response Curves for Treatments with Repeated Observations }

\description{ Principal Response Curves (PRC) are a special case of
  Redundancy Analysis (\code{\link{rda}}) for multivariate responses in
  repeated observation design. They were originally suggested for
  ecological communities. They should be easier to interpret than
  traditional constrained ordination. They can also be used to study how
  the effects of a factor \code{A} depend on the levels of a factor
  \code{B}, that is \code{A + A:B}, in a multivariate response
  experiment.
}

\usage{
prc(response, treatment, time, ...)
\method{summary}{prc}(object, axis = 1, scaling = "symmetric", const,
        digits = 4, correlation = FALSE, ...)
\method{plot}{prc}(x, species = TRUE, select, scaling = "symmetric",
     axis = 1, correlation = FALSE, const, type = "l", xlab, ylab, ylim,
     lty = 1:5, col = 1:6, pch, legpos, cex = 0.8, ...)  
}

\arguments{
  \item{response}{Multivariate response data. Typically these are
    community (species) data. If the data are counts, they probably
    should be log transformed prior to the analysis. }
  \item{treatment}{A factor for treatments. }
  \item{time}{ An unordered factor defining the observations times in
    the repeated design.}
  \item{object, x}{An \code{prc} result object.}
  \item{axis}{Axis shown (only one axis can be selected).}
  \item{scaling}{Scaling of species scores, identical to the
    \code{scaling} in \code{\link{scores.rda}}.

    The type of scores can also be specified as one of \code{"none"},
    \code{"sites"}, \code{"species"}, or \code{"symmetric"}, which
    correspond to the values \code{0}, \code{1}, \code{2}, and \code{3}
    respectively. Argument \code{correlation} can be used in combination
    with these character descriptions to get the corresponding negative
    value.
  }
  \item{const}{General scaling constant for species scores (see
    \code{\link{scores.rda}} for details). Lower values will reduce the
    range of species scores, but will not influence the regression
    coefficients.}
  \item{digits}{Number of significant digits displayed.}
  \item{correlation}{logical; if \code{scaling} is a character
    description of the scaling type, \code{correlation} can be used to
    select correlation-like scores for PCA. See argument \code{scaling}
    for details.}
  \item{species}{Display species scores.}
  \item{select}{Vector to select displayed species. This can be a vector
    of indices or a logical vector which is \code{TRUE} for the selected
    species}
  \item{type}{Type of plot: \code{"l"} for lines, \code{"p"} for points
    or \code{"b"} for both.}
  \item{xlab, ylab}{Text to replace default axis labels.}
  \item{ylim}{Limits for the vertical axis.}
  \item{lty, col, pch}{Line type, colour and plotting characters
    (defaults supplied).}
  \item{legpos}{The position of the \code{\link{legend}}. A guess is
    made if this is not supplied, and \code{NA} will suppress legend. }
  \item{cex}{Character expansion for symbols and species labels.}
  \item{\dots}{ Other parameters passed to functions. }
}

\details{ PRC is a special case of \code{\link{rda}} with a single
  factor for \code{treatment} and a single factor for \code{time} points
  in repeated observations. In \pkg{vegan}, the corresponding
  \code{\link{rda}} model is defined as \code{rda(response ~ treatment *
  time + Condition(time))}. Since the \code{time} appears twice in the
  model formula, its main effects will be aliased, and only the main
  effect of treatment and interaction terms are available, and will be
  used in PRC. Instead of usual multivariate ordination diagrams, PRC
  uses canonical (regression) coefficients and species scores for a
  single axis. All that the current functions do is to provide a special
  \code{summary} and \code{plot} methods that display the
  \code{\link{rda}} results in the PRC fashion. The current version only
  works with default contrasts (\code{\link{contr.treatment}}) in which
  the coefficients are contrasts against the first level, and the levels
  must be arranged so that the first level is the control (or a
  baseline). If necessary, you must change the baseline level with
  function \code{\link{relevel}}.  
  

  Function \code{summary} prints the species scores and the
  coefficients. Function \code{plot} plots coefficients against
  \code{time} using \code{\link{matplot}}, and has similar defaults.
  The graph (and PRC) is meaningful only if the first \code{treatment}
  level is the control, as the results are contrasts to the first level
  when unordered factors are used. The plot also displays species scores
  on the right vertical axis using function
  \code{\link{linestack}}. Typically the number of species is so high
  that not all can be displayed with the default settings, but users can
  reduce character size or padding (\code{air}) in
  \code{\link{linestack}}, or \code{select} only a subset of the
  species. A legend will be displayed unless suppressed with
  \code{legpos = NA}, and the functions tries to guess where to put the
  legend if \code{legpos} is not supplied.
  
}
\value{
  The function is a special case of \code{\link{rda}} and returns its
  result object (see \code{\link{cca.object}}). However, a special
  \code{summary} and \code{plot} methods display returns differently
  than in \code{\link{rda}}.
}
\references{
  van den Brink, P.J. & ter Braak, C.J.F. (1999). Principal response
  curves: Analysis of time-dependent multivariate responses of
  biological community to stress. Environmental Toxicology and
  Chemistry, 18, 138--148.
}
\author{ Jari Oksanen and Cajo ter Braak}

\section{Warning }{The first level of \code{treatment} must be the
  control: use function \code{\link{relevel}} to guarantee the correct
  reference level. The current version will ignore user setting of
  \code{\link{contrasts}} and always use treatment contrasts
  (\code{\link{contr.treatment}}). The \code{time} must be an unordered
  factor.  }

\seealso{\code{\link{rda}}, \code{\link{anova.cca}}.}
\examples{
## Chlorpyrifos experiment and experimental design: Pesticide
## treatment in ditches (replicated) and followed over from 4 weeks
## before to 24 weeks after exposure 
data(pyrifos)
week <- gl(11, 12, labels=c(-4, -1, 0.1, 1, 2, 4, 8, 12, 15, 19, 24))
dose <- factor(rep(c(0.1, 0, 0, 0.9, 0, 44, 6, 0.1, 44, 0.9, 0, 6), 11))
ditch <- gl(12, 1, length=132)
# PRC
mod <- prc(pyrifos, dose, week)
mod            # RDA
summary(mod)   # PRC
logabu <- colSums(pyrifos)
plot(mod, select = logabu > 100)
## Ditches are randomized, we have a time series, and are only
## interested in the first axis
ctrl <- how(plots = Plots(strata = ditch,type = "free"),
    within = Within(type = "series"), nperm = 99)
anova(mod, permutations = ctrl, first=TRUE)
}
\keyword{ multivariate }

\name{as.mlm.cca}
\alias{as.mlm}
\alias{as.mlm.cca}
\alias{as.mlm.rda}
\title{Refit Constrained Ordination as a Multiple Response Linear Model }
\description{
  Functions refit results of constrained ordination (\code{\link{cca}},
  \code{\link{rda}}, \code{\link{capscale}}) as a multiple response
  linear model (\code{\link{lm}}). This allows finding influence
  statistics (\code{\link{influence.measures}}).  This also allows
  deriving several other statistics, but most of these are biased and
  misleading, since refitting ignores a major component of variation in
  constrained ordination.
}
\usage{
as.mlm(x)
}
\arguments{
  \item{x}{Constrained ordination result. }
}
\details{
  Popular algorithm for constrained ordination is based on iteration
  with regression where weighted averages of sites are used as dependent
  variables and constraints as independent variables.
  Statistics of linear regression
  are a natural by-product in this algorithm. Constrained ordination in
  \pkg{vegan} uses different algorithm, but to obtain linear regression
  statistics you can refit an ordination result as a multiple response
  linear model (\code{\link{lm}}). This regression ignores residual
  unconstrained variation in the data, and therefore estimates of
  standard error are strongly biased and much too low. You can get
  statistics like \eqn{t}-values of coefficients, but you should not use
  these because of this bias. Some useful information you can get with
  refitted models are statistics for detecting influential observations
  (\code{\link{influence.measures}} including
  \code{\link{cooks.distance}}, \code{\link{hatvalues}}). 
}
\value{
  Function returns an object of multiple response linear model of class
  \code{"mlm"}  documented with \code{\link{lm}}.
}

\author{ Jari Oksanen }
\note{ 
  You can use these functions to find \eqn{t}-values of coefficients
  using \code{summary.mlm}, but you should not do this because the
  method ignores unconstrained residual variation. You also can find
  several other statistics for (multiple response) linear models with
  similar bias. This bias is not a unique feature in \pkg{vegan}
  implementation, but also applies to implementations in other
  software.

  Some statistics of linear models can be found without using
  these functions: \code{\link{coef.cca}} gives the regression
  coefficients, \code{\link{spenvcor}} the species-environment
  correlation, \code{\link{intersetcor}} the interset correlation,
  \code{\link{vif.cca}} the variance inflation factors.
}
\seealso{\code{\link{cca}}, \code{\link{rda}}, \code{\link{capscale}},
  \code{\link{cca.object}}, \code{\link{lm}}, \code{\link{summary.mlm}},
  \code{\link{influence.measures}}. }
\examples{
data(varespec)
data(varechem)
mod <- cca(varespec ~ Al + P + K, data=varechem)
lmod <- as.mlm(mod)
## Coefficients
lmod
coef(mod)
## Influential observations
influence.measures(lmod)
plot(mod, type = "n")
points(mod, cex = 10*hatvalues(lmod), pch=16, xpd = TRUE)
text(mod, display = "bp", col = "blue") 
}

\keyword{ models }
\keyword{ multivariate }

\encoding{UTF-8}
\name{mantel.correlog}
\alias{mantel.correlog}
\alias{plot.mantel.correlog}
\title{ Mantel Correlogram }

\description{
  Function \code{mantel.correlog} computes a multivariate
  Mantel correlogram. Proposed by Sokal (1986) and Oden and Sokal
  (1986), the method is also described in Legendre and Legendre (2012,
  pp. 819--821).
}

\usage{
mantel.correlog(D.eco, D.geo=NULL, XY=NULL, n.class=0, break.pts=NULL, 
cutoff=TRUE, r.type="pearson", nperm=999, mult="holm", progressive=TRUE)
\method{plot}{mantel.correlog}(x, alpha=0.05, ...)
}

\arguments{
  \item{D.eco}{ An ecological distance matrix, with class
  either \code{dist} or \code{matrix}. }
  
  \item{D.geo}{ A geographic distance matrix, with class either
  \code{dist} or \code{matrix}. Provide either \code{D.geo} or
  \code{XY}. Default: \code{D.geo=NULL}. }

  \item{XY}{ A file of Cartesian geographic coordinates of the
  points. Default: \code{XY=NULL}. }

  \item{n.class}{ Number of classes. If \code{n.class=0}, the Sturges
  equation will be used unless break points are provided. }

  \item{break.pts}{ Vector containing the break points of the distance
  distribution. Provide (n.class+1) breakpoints, that is, a list with
  a beginning and an ending point. Default: \code{break.pts=NULL}. }

  \item{cutoff}{ For the second half of the distance classes,
  \code{cutoff = TRUE} limits the correlogram to the distance classes
  that include all points. If \code{cutoff = FALSE}, the correlogram
  includes all distance classes. }

  \item{r.type}{ Type of correlation in calculation of the Mantel
  statistic. Default: \code{r.type="pearson"}.  Other choices are
  \code{r.type="spearman"} and \code{r.type="kendall"}, as in functions
  \code{\link{cor}} and \code{\link{mantel}}. }

  \item{nperm}{ Number of permutations for the tests of
  significance. Default: \code{nperm=999}. For large data files,
  permutation tests are rather slow. }

  \item{mult}{ Correct P-values for multiple testing. The correction
  methods are \code{"holm"} (default), \code{"hochberg"},
  \code{"sidak"}, and other methods available in the
  \code{\link{p.adjust}} function: \code{"bonferroni"} (best known, but
  not recommended because it is overly conservative), \code{"hommel"},
  \code{"BH"}, \code{"BY"}, \code{"fdr"}, and \code{"none"}. }

  \item{progressive}{ Default: \code{progressive=TRUE} for progressive
  correction of multiple-testing, as described in Legendre and Legendre
  (1998, p. 721). Test of the first distance class: no correction;
  second distance class: correct for 2 simultaneous tests; distance
  class k: correct for k simultaneous tests. \code{progressive=FALSE}:
  correct all tests for \code{n.class} simultaneous tests. }

  \item{x}{ Output of \code{mantel.correlog}. }

  \item{alpha}{ Significance level for the points drawn with black
  symbols in the correlogram. Default: \code{alpha=0.05}. }

  \item{...}{ Other parameters passed from other functions. }
}

\details{ A correlogram is a graph in which spatial correlation values
  are plotted, on the ordinate, as a function of the geographic distance
  classes among the study sites along the abscissa. In a Mantel
  correlogram, a Mantel correlation (Mantel 1967) is computed between a
  multivariate (e.g. multi-species) distance matrix of the user's choice
  and a design matrix representing each of the geographic distance
  classes in turn. The Mantel statistic is tested through a
  permutational Mantel test performed by \code{vegan}'s
  \code{\link{mantel}} function.

  When a correction for multiple testing is applied, more permutations
  are necessary than in the no-correction case, to obtain significant
  p-values in the higher correlogram classes.

  The \code{print.mantel.correlog} function prints out the
  correlogram. See examples.  }

\value{ 

  \item{mantel.res }{A table with the distance classes as rows and the
  class indices, number of distances per class, Mantel statistics
  (computed using Pearson's r, Spearman's r, or Kendall's tau), and
  p-values as columns. A positive Mantel statistic indicates positive
  spatial correlation. An additional column with p-values corrected for
  multiple testing is added unless \code{mult="none"}. }

  \item{n.class }{The n umber of distance classes. }
  
  \item{break.pts }{The break points provided by the user or computed by
    the program. }

  \item{mult }{The name of the correction for multiple testing. No
    correction: \code{mult="none"}. }  

  \item{progressive }{A logical (\code{TRUE}, \code{FALSE}) value
  indicating whether or not a progressive correction for multiple
  testing was requested. } 

  \item{n.tests }{The number of distance classes for which Mantel
  tests have been computed and tested for significance. }

  \item{call }{The function call. }  
}

\author{ Pierre Legendre, Université de Montréal }

\references{

  Legendre, P. and L. Legendre. 2012. Numerical ecology, 3rd English
  edition. Elsevier Science BV, Amsterdam.

  Mantel, N. 1967. The detection of disease clustering and a generalized
  regression approach. Cancer Res. 27: 209-220.

  Oden, N. L. and R. R. Sokal. 1986. Directional autocorrelation: an
  extension of spatial correlograms to two dimensions. Syst. Zool. 35:
  608-617.

  Sokal, R. R. 1986. Spatial data analysis and historical
  processes. 29-43 in: E. Diday et al. [eds.] Data analysis and
  informatics, IV. North-Holland, Amsterdam.
  
  Sturges, H. A. 1926. The choice of a class interval. Journal of the 
  American Statistical Association 21: 65–66.  }

\examples{   
# Mite data available in "vegan"
data(mite)        
data(mite.xy)  
mite.hel <- decostand(mite, "hellinger")

# Detrend the species data by regression on the site coordinates
mite.hel.resid <- resid(lm(as.matrix(mite.hel) ~ ., data=mite.xy))

# Compute the detrended species distance matrix
mite.hel.D <- dist(mite.hel.resid)

# Compute Mantel correlogram with cutoff, Pearson statistic
mite.correlog <- mantel.correlog(mite.hel.D, XY=mite.xy, nperm=49)
summary(mite.correlog)
mite.correlog   
# or: print(mite.correlog)
# or: print.mantel.correlog(mite.correlog)
plot(mite.correlog)

# Compute Mantel correlogram without cutoff, Spearman statistic
mite.correlog2 <- mantel.correlog(mite.hel.D, XY=mite.xy, cutoff=FALSE, 
   r.type="spearman", nperm=49)
summary(mite.correlog2)
mite.correlog2
plot(mite.correlog2)

# NOTE: 'nperm' argument usually needs to be larger than 49.
# It was set to this low value for demonstration purposes.

}

\keyword{ multivariate }

\name{eigenvals}
\Rdversion{1.1}
\alias{eigenvals}
\alias{eigenvals.default}
\alias{eigenvals.prcomp}
\alias{eigenvals.princomp}
\alias{eigenvals.cca}
\alias{eigenvals.wcmdscale}
\alias{eigenvals.pcnm}
\alias{eigenvals.dudi}
\alias{eigenvals.pca}
\alias{eigenvals.pco}
\alias{eigenvals.decorana}
\alias{summary.eigenvals}

\title{
  Extract Eigenvalues from an Ordination Object
}
\description{
  Function extracts eigenvalues from an object that has them. Many
  multivariate methods return such objects. 
}
\usage{
eigenvals(x, ...)
\method{eigenvals}{cca}(x, model = c("all", "unconstrained", "constrained"),
          constrained = NULL, ...)
\method{eigenvals}{decorana}(x, kind = c("additive", "axiswise", "decorana"),
           ...)
\method{summary}{eigenvals}(object, ...)
}

\arguments{
  \item{x}{
    An object from which to extract eigenvalues.
  }
  \item{object}{
    An \code{eigenvals} result object.
  }
  \item{model}{
    Which eigenvalues to return for objects that inherit from class
    \code{"cca"} only.
  }
  \item{constrained}{
    Return only constrained eigenvalues. Deprecated as of vegan
    2.5-0. Use \code{model} instead.
  }
  \item{kind}{
    Kind of eigenvalues returned for \code{\link{decorana}}. Only
    \code{"additive"} eigenvalues can be used for reporting importances
    of components in \code{summary}. \code{"axiswise"} gives the
    non-additive eigenvalues, and \code{"decorana"} the decorana values
    (see \code{\link{decorana}} for details).
  }
  \item{\dots}{
    Other arguments to the functions (usually ignored)
  }
}

\details{
  This is a generic function that has methods for \code{\link{cca}},
  \code{\link{wcmdscale}}, \code{\link{pcnm}}, \code{\link{prcomp}},
  \code{\link{princomp}}, \code{dudi} (of \pkg{ade4}), and 
  \code{pca} and \code{pco} (of 
  \pkg{labdsv}) result objects. The default method also
  extracts eigenvalues if the result looks like being from
  \code{\link{eigen}} or \code{\link{svd}}.  Functions
  \code{\link{prcomp}} and \code{\link{princomp}} contain square roots
  of eigenvalues that all called standard deviations, but
  \code{eigenvals} function returns their squares.  Function
  \code{\link{svd}} contains singular values, but function
  \code{eigenvals} returns their squares. For constrained ordination
  methods \code{\link{cca}}, \code{\link{rda}} and
  \code{\link{capscale}} the function returns the both constrained and
  unconstrained eigenvalues concatenated in one vector, but the partial
  component will be ignored. However, with argument 
  \code{constrained = TRUE} only constrained eigenvalues are returned. 

  The \code{summary} of \code{eigenvals} result returns eigenvalues,
  proportion explained and cumulative proportion explained. The result
  object can have some negative eigenvalues (\code{\link{wcmdscale}},
  \code{\link{dbrda}}, \code{\link{pcnm}}) which correspond to
  imaginary axes of Euclidean mapping of non-Euclidean distances
  (Gower 1985). In these case real axes (corresponding to positive
  eigenvalues) will "explain" proportion >1 of total variation, and
  negative eigenvalues bring the cumulative proportion to
  1. \code{\link{capscale}} will only find the positive eigenvalues
  and only these are used in finding proportions. For
  \code{\link{decorana}} the importances and cumulative proportions
  are only reported for \code{kind = "additive"}, because other
  alternatives do not add up to total inertia of the input data.

}

\value{
  An object of class \code{"eigenvals"}, which is a vector of
  eigenvalues.

  The \code{summary} method returns an object of class
  \code{"summary.eigenvals"}, which is a matrix.
}

\author{
 Jari Oksanen.
}

\references{
  Gower, J. C. (1985). Properties of Euclidean and non-Euclidean
  distance matrices. \emph{Linear Algebra and its Applications} 67,
  81--97.
}

\seealso{
 \code{\link{eigen}}, \code{\link{svd}}, \code{\link{prcomp}},
 \code{\link{princomp}}, \code{\link{cca}}, \code{\link{rda}},
 \code{\link{capscale}}, \code{\link{wcmdscale}},
 \code{\link{cca.object}}. 
}
\examples{
data(varespec)
data(varechem)
mod <- cca(varespec ~ Al + P + K, varechem)
ev <- eigenvals(mod)
ev
summary(ev)

## choose which eignevalues to return
eigenvals(mod, model = "unconstrained")
}
\keyword{ multivariate }

\encoding{UTF-8}
\name{adonis}
\alias{adonis2}

\title{Permutational Multivariate Analysis of Variance Using Distance
  Matrices}

\description{Analysis of variance using distance matrices --- for
  partitioning distance matrices among sources of variation and fitting
  linear models (e.g., factors, polynomial regression) to distance 
  matrices; uses a permutation test with pseudo-\eqn{F} ratios.}

\usage{
adonis2(formula, data, permutations = 999, method = "bray",
    sqrt.dist = FALSE, add = FALSE, by = NULL,
    parallel = getOption("mc.cores"), na.action = na.fail,
    strata = NULL, ...)
}

\arguments{

  \item{formula}{Model formula. The left-hand side (LHS) of the formula
    must be either a community data matrix or a dissimilarity matrix,
    e.g., from \code{\link{vegdist}} or \code{\link{dist}}.  If the LHS
    is a data matrix, function \code{\link{vegdist}} will be used to
    find the dissimilarities. The right-hand side (RHS) of the formula
    defines the independent variables. These can be continuous variables
    or factors, they can be transformed within the formula, and they can
    have interactions as in a typical \code{\link{formula}}.}
  \item{data}{ the data frame for the independent variables, with rows
    in the same order as the community data matrix or dissimilarity
    matrix named on the LHS of \code{formula}.}
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}
  \item{method}{ the name of any method used in \code{\link{vegdist}} to
    calculate pairwise distances if the left hand side of the
    \code{formula} was a data frame or a matrix. }
  \item{sqrt.dist}{Take square root of dissimilarities. This often
    euclidifies dissimilarities.}
  \item{add}{Add a constant to the non-diagonal dissimilarities such
    that all eigenvalues are non-negative in the underlying Principal
    Co-ordinates Analysis (see \code{\link{wcmdscale}} for
    details). Choice \code{"lingoes"} (or \code{TRUE}) use the
    recommended method of Legendre & Anderson (1999: \dQuote{method
    1}) and \code{"cailliez"} uses their \dQuote{method 2}.}
  \item{by}{\code{by = NULL} will assess the overall significance of all
    terms together, \code{by = "terms"} will assess significance for each
    term (sequentially from first to last), setting \code{by = "margin"}
    will assess the marginal effects of the terms (each marginal term
    analysed in a model with all other variables), \code{by = "onedf"}
    will analyse one-degree-of-freedom contrasts sequentially. The
    argument is passed on to \code{\link{anova.cca}}.}
  \item{parallel}{Number of parallel processes or a predefined socket
    cluster.  With \code{parallel = 1} uses ordinary, non-parallel
    processing. The parallel processing is done with \pkg{parallel}
    package.}
  \item{na.action}{Handling of missing values on the right-hand-side
    of the formula (see \code{\link{na.fail}} for explanation and
    alternatives). Missing values are not allowed on the
    left-hand-side. NB, argument \code{subset} is not implemented.}

  \item{strata}{Groups within which to constrain permutations. The
    traditional non-movable strata are set as Blocks in the
    \CRANpkg{permute} package, but some more flexible alternatives may
    be more appropriate.}

  \item{\dots}{Other arguments passed to \code{\link{vegdist}}.}
}

\details{

\code{adonis2} is a function for the analysis and partitioning sums of
squares using dissimilarities. The function is based on the principles
of McArdle & Anderson (2001) and can perform sequential, marginal and
overall tests. The function also allows using additive constants or
squareroot of dissimilarities to avoid negative eigenvalues, but can
also handle semimetric indices (such as Bray-Curtis) that produce
negative eigenvalues. The \code{adonis2} tests are identical to
\code{\link{anova.cca}} of \code{\link{dbrda}}. With Euclidean
distances, the tests are also identical to \code{\link{anova.cca}} of
\code{\link{rda}}.

The function partitions sums of squares of a multivariate data set,
and they are directly analogous to MANOVA (multivariate analysis of
variance). McArdle and Anderson (2001) and Anderson (2001) refer to
the method as \dQuote{permutational MANOVA} (formerly
\dQuote{nonparametric MANOVA}). Further, as the inputs are linear
predictors, and a response matrix of an arbitrary number of columns,
they are a robust alternative to both parametric MANOVA and to
ordination methods for describing how variation is attributed to
different experimental treatments or uncontrolled covariates. The
method is also analogous to distance-based redundancy analysis and
algorithmically similar to \code{\link{dbrda}} (Legendre and Anderson
1999), and provides an alternative to AMOVA (nested analysis of
molecular variance, Excoffier, Smouse, and Quattro, 1992; \code{amova}
in the \pkg{ade4} package) for both crossed and nested factors.

}

\value{

  The function returns an \code{\link{anova.cca}} result object with a
  new column for partial \eqn{R^2}{R-squared}: This is the proportion
  of sum of squares from the total, and in marginal models
  (\code{by = "margin"}) the \eqn{R^2}{R-squared} terms do not add up to
  1.

}

\note{Anderson (2001, Fig. 4) warns that the method may confound
  location and dispersion effects: significant differences may be caused
  by different within-group variation (dispersion) instead of different
  mean values of the groups (see Warton et al. 2012 for a general
  analysis). However, it seems that \code{adonis2} is less sensitive to
  dispersion effects than some of its alternatives (\code{\link{anosim}},
  \code{\link{mrpp}}). Function \code{\link{betadisper}} is a sister
  function to \code{adonis2} to study the differences in dispersion
  within the same geometric framework.
}

\references{
Anderson, M.J. 2001. A new method for non-parametric multivariate
analysis of variance. \emph{Austral Ecology}, \strong{26}: 32--46.

Excoffier, L., P.E. Smouse, and J.M. Quattro. 1992. Analysis of
molecular variance inferred from metric distances among DNA haplotypes:
Application to human mitochondrial DNA restriction data. \emph{Genetics},
\strong{131}:479--491.

Legendre, P. and M.J. Anderson. 1999. Distance-based redundancy
analysis: Testing multispecies responses in multifactorial ecological
experiments. \emph{Ecological Monographs}, \strong{69}:1--24.

McArdle, B.H.  and M.J. Anderson. 2001. Fitting multivariate models to
community data: A comment on distance-based redundancy
analysis. \emph{Ecology}, \strong{82}: 290--297.

Warton, D.I., Wright, T.W., Wang, Y. 2012. Distance-based multivariate
analyses confound location and dispersion effects. \emph{Methods in
Ecology and Evolution}, 3, 89--101.
}

\author{Martin Henry H. Stevens and Jari Oksanen.}

\seealso{ \code{\link{mrpp}}, \code{\link{anosim}},
  \code{\link{mantel}}, \code{\link{varpart}}. }
\examples{
data(dune)
data(dune.env)
## default is overall (omnibus) test
adonis2(dune ~ Management*A1, data = dune.env)
## sequential tests
adonis2(dune ~ Management*A1, data = dune.env, by = "terms")

### Example of use with strata, for nested (e.g., block) designs.
dat <- expand.grid(rep=gl(2,1), NO3=factor(c(0,10)),field=gl(3,1) )
dat
Agropyron <- with(dat, as.numeric(field) + as.numeric(NO3)+2) +rnorm(12)/2
Schizachyrium <- with(dat, as.numeric(field) - as.numeric(NO3)+2) +rnorm(12)/2
total <- Agropyron + Schizachyrium
dotplot(total ~ NO3, dat, jitter.x=TRUE, groups=field,
        type=c('p','a'), xlab="NO3", auto.key=list(columns=3, lines=TRUE) )

Y <- data.frame(Agropyron, Schizachyrium)
mod <- metaMDS(Y, trace = FALSE)
plot(mod)
### Ellipsoid hulls show treatment
with(dat, ordiellipse(mod, NO3, kind = "ehull", label = TRUE))
### Spider shows fields
with(dat, ordispider(mod, field, lty=3, col="red", label = TRUE))

### Incorrect (no strata)
adonis2(Y ~ NO3, data = dat, permutations = 199)
## Correct with strata
with(dat, adonis2(Y ~ NO3, data = dat, permutations = 199, strata = field))
}

\keyword{multivariate }
\keyword{nonparametric }

\encoding{UTF-8}
\name{simper}
\alias{simper}
\alias{summary.simper}
\title{Similarity Percentages}

\description{
  Discriminating species between two groups using
  Bray-Curtis dissimilarities
}

\usage{
simper(comm, group, permutations = 0, trace = FALSE, 
    parallel = getOption("mc.cores"), ...)
\method{summary}{simper}(object, ordered = TRUE,
    digits = max(3,getOption("digits") - 3), ...)
}

\arguments{
  \item{comm}{Community data matrix.}
  \item{group}{Factor describing the group structure. Must have at
    least 2 levels.}
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}
  \item{trace}{Trace permutations.}
  \item{object}{an object returned by \code{simper}.}
  \item{ordered}{Logical; Should the species be ordered by their
    average contribution?}
  \item{digits}{Number of digits in output.}
  \item{parallel}{Number of parallel processes or a predefined socket
    cluster.  With \code{parallel = 1} uses ordinary, non-parallel
    processing.}
  \item{...}{Parameters passed to other functions. In \code{simper} the
    extra parameters are passed to \code{\link[permute]{shuffleSet}} if
    permutations are used.}
}

\details{ Similarity percentage, \code{simper} (Clarke 1993) is based
  on the decomposition of Bray-Curtis dissimilarity index (see
  \code{\link{vegdist}}, \code{\link{designdist}}). The contribution
  of individual species \eqn{i} to the overall Bray-Curtis dissimilarity
  \eqn{d_{jk}}{d[jk]} is given by

  \deqn{d_{ijk} = \frac{|x_{ij}-x_{ik}|}{\sum_{i=1}^S (x_{ij}+x_{ik})}}{d[ijk] = abs(x[ij]-x[ik])/sum(x[ij]+x[ik])}
  
  where \eqn{x} is the abundance of species \eqn{i} in sampling units
  \eqn{j} and \eqn{k}. The overall index is the sum of the individual
  contributions over all \eqn{S} species 
  \eqn{d_{jk}=\sum_{i=1}^S d_{ijk}}{d[jk] = sum(i=1..S) d[ijk]}. 
  
  The \code{simper} functions performs pairwise comparisons of groups
  of sampling units and finds the contribution of each species to the
  average between-group Bray-Curtis dissimilarity. Although the method
  is called simper, it really studied dissimilarities instead of
  similarities (Clarke 1993).

  The function displays most important species for each pair of
  \code{groups}.  These species contribute at least to 70 \% of the
  differences between groups.  The function returns much more
  extensive results (including all species) which can be accessed
  directly from the result object (see section Value). Function
  \code{summary} transforms the result to a list of data frames. With
  argument \code{ordered = TRUE} the data frames also include the
  cumulative contributions and are ordered by species contribution.

  The results of \code{simper} can be very difficult to interpret and
  they are often misunderstood even in publications. The method gives
  the contribution of each species to overall dissimilarities, but
  these are caused by variation in species abundances, and only partly
  by differences among groups.  Even if you make groups that are
  copies of each other, the method will single out species with high
  contribution, but these are not contributions to non-existing
  between-group differences but to random noise variation in species
  abundances. The most abundant species usually have highest
  variances, and they have high contributions even when they do not
  differ among groups. Permutation tests study the differences among
  groups, and they can be used to find out the species for which the
  differences among groups is an important component of their
  contribution to dissimilarities.

}

\value{
  A list of class \code{"simper"} with following items:
  \item{species}{The species names.}
  \item{average}{Species contribution to average between-group dissimilarity.}
  \item{overall}{The average between-group dissimilarity. This is the sum of
    the item \code{average}.}
  \item{sd}{Standard deviation of contribution.} 
  \item{ratio}{Average to sd ratio.}
  \item{ava, avb}{Average abundances per group.}
  \item{ord}{An index vector to order vectors by their contribution or
    order \code{cusum} back to the original data order.}
  \item{cusum}{Ordered cumulative contribution. These are based on item
    \code{average}, but they sum up to total 1.}
  \item{p}{Permutation \eqn{p}-value. Probability of getting a larger
    or equal average contribution in random permutation of the group
    factor. These area only available if \code{permutations} were used
    (default: not calculated).}
}

\seealso{Function \code{\link{meandist}} shows the average between-group
   dissimilarities (as well as the within-group dissimilarities).}

\examples{
data(dune)
data(dune.env)
(sim <- with(dune.env, simper(dune, Management)))
summary(sim)
}
\author{
  Eduard Szöcs \email{eduardszoecs@gmail.com}
}

\references{
  Clarke, K.R. 1993. Non-parametric multivariate analyses of changes
    in community structure. \emph{Australian Journal of Ecology}, 18,
    117–143.
}
\keyword{multivariate}

\encoding{UTF-8}
\name{oecosimu}
\alias{oecosimu}
\alias{as.ts.oecosimu}
\alias{as.mcmc.oecosimu}

\title{Evaluate Statistics with Null Models of Biological Communities }

\description{
 
  Function evaluates a statistic or a vector of statistics in
  community and evaluates its significance in a series of simulated
  random communities.  The approach has been used traditionally for
  the analysis of nestedness, but the function is more general and can
  be used with any statistics evaluated with simulated
  communities. Function \code{oecosimu} collects and evaluates the
  statistics. The Null model communities are described in
  \code{\link{make.commsim}} and \code{\link{permatfull}}/
  \code{\link{permatswap}}, the definition of Null models in
  \code{\link{nullmodel}}, and nestedness statistics in
  \code{\link{nestednodf}} (which describes several alternative
  statistics, including nestedness temperature, \eqn{N0}, checker
  board units, nestedness discrepancy and NODF).

}

\usage{
oecosimu(comm, nestfun, method, nsimul = 99, burnin = 0, thin = 1,
   statistic = "statistic", alternative = c("two.sided", "less", "greater"), 
   batchsize = NA, parallel = getOption("mc.cores"), ...)
\method{as.ts}{oecosimu}(x, ...)
\method{as.mcmc}{oecosimu}(x)
}

\arguments{
  \item{comm}{Community data, or a Null model object generated by
    \code{\link{nullmodel}} or an object of class \code{simmat} (array
    of permuted matrices from \code{\link{simulate.nullmodel}}). If
    \code{comm} is a community data, null model simulation
    \code{method} must be specified.  If \code{comm} is a
    \code{\link{nullmodel}}, the simulation \code{method} is ignored,
    and if \code{comm} is a \code{simmat} object, all other arguments
    are ignored except \code{nestfun}, \code{statistic} and
    \code{alternative}.}
  \item{nestfun}{Function analysed. Some nestedness functions are
    provided in \pkg{vegan} (see \code{\link{nestedtemp}}), but any
    function can be used if it accepts the community as the first
    argument, and returns either a plain number or a vector or the
    result in list item with the name defined in argument
    \code{statistic}. See Examples for defining your own functions.}
  \item{method}{Null model method: either a name (character string) of
    a method defined in \code{\link{make.commsim}} or a
    \code{\link{commsim}} function. This argument is ignored if
    \code{comm} is a \code{\link{nullmodel}} or a \code{simmat}
    object. See Details and Examples.}
  \item{nsimul}{Number of simulated null communities (ignored if
    \code{comm} is a \code{simmat} object).}
  \item{burnin}{Number of null communities discarded before proper
    analysis in sequential methods (such as \code{"tswap"})
    (ignored with non-sequential methods or when \code{comm} is a
    \code{simmat} object).}
  \item{thin}{Number of discarded null communities between two
    evaluations of nestedness statistic in sequential methods (ignored
    with non-sequential methods or when \code{comm} is a \code{simmat}
    object).}
  \item{statistic}{The name of the statistic returned by
    \code{nestfun}.} 
  \item{alternative}{a character string specifying the alternative
    hypothesis, must be one of \code{"two.sided"} (default), \code{"greater"}
    or \code{"less"}. Please note that the \eqn{p}-value of two-sided
    test is approximately two times higher than in the corresponding
    one-sided test (\code{"greater"} or \code{"less"} depending on the
    sign of the difference).}
  \item{batchsize}{Size in Megabytes of largest simulation object. If
    a larger structure would be produced, the analysis is broken
    internally into batches. With default \code{NA} the analysis is
    not broken into batches.  See Details.}
  \item{parallel}{Number of parallel processes or a predefined socket
    cluster.  With \code{parallel = 1} uses ordinary, non-parallel
    processing. The parallel processing is done with \pkg{parallel}
    package.  If you define a \code{nestfun} in Windows that needs other
    \R packages than \pkg{vegan} or \pkg{permute}, you must set up a
    socket cluster before the call. }
  \item{x}{An \code{oecosimu} result object.}

  \item{\dots}{Other arguments to functions.}
}

\details{
  
  Function \code{oecosimu} is a wrapper that evaluates a statistic
  using function given by \code{nestfun}, and then simulates a series
  of null models based on \code{nullmodel}, and evaluates the
  statistic on these null models. The \pkg{vegan} packages contains
  some nestedness functions that are described separately
  (\code{\link{nestedchecker}}, \code{\link{nesteddisc}},
  \code{\link{nestedn0}}, \code{\link{nestedtemp}},
  \code{\link{nestednodf}}), but many other functions can be used as
  long as they are meaningful with simulated communities.  An
  applicable function must return either the statistic as a plain
  number or a vector, or as a list element \code{"statistic"} (like
  \code{\link{chisq.test}}), or in an item whose name is given in the
  argument \code{statistic}.  The statistic can be a single number
  (like typical for a nestedness index), or it can be a vector. The
  vector indices can be used to analyse site (row) or species (column)
  properties, see \code{\link{treedive}} for an example. Raup-Crick
  index (\code{\link{raupcrick}}) gives an example of using a
  dissimilarities.

  The Null model type can be given as a name (quoted character string)
  that is used to define a Null model in \code{\link{make.commsim}}.
  These include all binary models described by Wright et al. (1998),
  Jonsson (2001), Gotelli & Entsminger (2003), \enc{Miklós}{Miklos} &
  Podani (2004), and some others. There are several quantitative Null
  models, such those discussed by Hardy (2008), and several that are
  unpublished (see \code{\link{make.commsim}},
  \code{\link{permatfull}}, \code{\link{permatswap}} for
  discussion). The user can also define her own \code{\link{commsim}}
  function (see Examples).

  Function works by first defining a \code{\link{nullmodel}} with
  given \code{\link{commsim}}, and then generating a series of
  simulated communities with \code{\link{simulate.nullmodel}}. A
  shortcut can be used for any of these stages and the input can be
  \enumerate{
    \item Community data (\code{comm}), Null model function
      (\code{nestfun}) and the number of simulations (\code{nsimul}).
    \item A \code{\link{nullmodel}} object and the number of
      simulations, and argument \code{method} is ignored.
    \item A three-dimensional array of simulated communities generated
      with \code{\link{simulate.nullmodel}}, and arguments
      \code{method} and \code{nsimul} are ignored.  
  }
  The last case allows analysing several statistics with the same
  simulations.

  The function first generates simulations with given
  \code{\link{nullmodel}} and then analyses these using the
  \code{nestfun}.  With large data sets and/or large number of
  simulations, the generated objects can be very large, and if the
  memory is exhausted, the analysis can become very slow and the
  system can become unresponsive. The simulation will be broken into
  several smaller batches if the simulated \code{\link{nullmodel}}
  objective will be above the set \code{batchsize} to avoid memory
  problems (see \code{\link{object.size}} for estimating the size of
  the current data set). The parallel processing still increases the
  memory needs.  The parallel processing is only used for evaluating
  \code{nestfun}.  The main load may be in simulation of the
  \code{\link{nullmodel}}, and \code{parallel} argument does not help
  there.

  Function \code{as.ts} transforms the simulated results of sequential
  methods into a time series or a \code{\link{ts}} object. This allows
  using analytic tools for time series in studying the sequences (see
  examples). Function \code{as.mcmc} transforms the simulated results
  of sequential methods into an \code{\link[coda]{mcmc}} object of the
  \pkg{coda} package. The \pkg{coda} package provides functions for
  the analysis of stationarity, adequacy of sample size,
  autocorrelation, need of burn-in and much more for sequential
  methods, and summary of the results. Please consult the
  documentation of the \pkg{coda} package.

  Function \code{\link{permustats}} provides support to the standard
  \code{\link{density}}, \code{\link[lattice]{densityplot}},
  \code{\link{qqnorm}} and \code{\link[lattice]{qqmath}} functions for
  the simulated values.
}

\value{ 

  Function \code{oecosimu} returns an object of class
  \code{"oecosimu"}.  The result object has items \code{statistic} and
  \code{oecosimu}.  The \code{statistic} contains the complete object
  returned by \code{nestfun} for the original data.  The
  \code{oecosimu} component contains the following items:
  \item{statistic}{Observed values of the statistic.}
  \item{simulated}{Simulated values of the statistic.}
  \item{means}{Mean values of the statistic from simulations.}
  \item{z}{Standardized effect sizes (SES, a.k.a. the \eqn{z}-values)
     of the observed statistic based on simulations.}
  \item{pval}{The \eqn{P}-values of the statistic based on simulations.}
  \item{alternative}{The type of testing as given in argument \code{alternative}.}
  \item{method}{The \code{method} used in \code{\link{nullmodel}}.}
  \item{isSeq}{\code{TRUE} if \code{method} was sequential.}

}

\references{
  Hardy, O. J. (2008) 
  Testing the spatial phylogenetic structure of local communities: 
  statistical performances of different null models 
  and test statistics on a locally neutral community. 
  \emph{Journal of Ecology} 96, 914--926.

  Gotelli, N.J. & Entsminger, N.J. (2003). Swap algorithms in null model
  analysis. \emph{Ecology} 84, 532--535.

  Jonsson, B.G. (2001) A null model for randomization tests of
  nestedness in species assemblages. \emph{Oecologia} 127, 309--313.

  \enc{Miklós}{Miklos}, I. & Podani, J. (2004). Randomization of presence-absence
  matrices: comments and new algorithms. \emph{Ecology} 85, 86--92.

  Wright, D.H., Patterson, B.D., Mikkelson, G.M., Cutler, A. & Atmar,
  W. (1998). A comparative analysis of nested subset patterns of species
  composition. \emph{Oecologia} 113, 1--20.
}

\author{Jari Oksanen and Peter Solymos}

\note{
  If you wonder about the name of \code{oecosimu}, look at journal
  names in the References (and more in \code{\link{nestedtemp}}).  

  The internal structure of the function was radically changed in
  \pkg{vegan 2.2-0} with introduction of \code{\link{commsim}} and
  \code{\link{nullmodel}} and deprecation of
  \code{\link{commsimulator}}. However, the results and the basic user
  interface remain the same (except that \code{method = "r0_old"} must
  be used to reproduce the old results of \code{"method = r0"}).  
}

\seealso{Function \code{oecosimu} currently defines null models with
  \code{\link{commsim}} and generates the simulated null model
  communities with \code{\link{nullmodel}} and
  \code{\link{simulate.nullmodel}}. For other applications of
  \code{oecosimu}, see \code{\link{treedive}} and
  \code{\link{raupcrick}}.

  See also \code{\link{nestedtemp}} (that also discusses other
  nestedness functions) and \code{\link{treedive}} for another
  application.
}

\examples{
## Use the first eigenvalue of correspondence analysis as an index
## of structure: a model for making your own functions.
data(sipoo)
## Traditional nestedness statistics (number of checkerboard units)
oecosimu(sipoo, nestedchecker, "r0")
## sequential model, one-sided test, a vector statistic
out <- oecosimu(sipoo, decorana, "swap", burnin=100, thin=10, 
   statistic="evals", alt = "greater")
out
## Inspect the swap sequence as a time series object
plot(as.ts(out))
lag.plot(as.ts(out))
acf(as.ts(out))
## Density plot
densityplot(permustats(out), as.table = TRUE, layout = c(1,4))
## Use quantitative null models to compare
## mean Bray-Curtis dissimilarities
data(dune)
meandist <- function(x) mean(vegdist(x, "bray"))
mbc1 <- oecosimu(dune, meandist, "r2dtable")
mbc1

## Define your own null model as a 'commsim' function: shuffle cells
## in each row
foo <- function(x, n, nr, nc, ...) {
   out <- array(0, c(nr, nc, n))
   for (k in seq_len(n))
      out[,,k] <- apply(x, 2, function(z) sample(z, length(z)))
   out
}
cf <- commsim("myshuffle", foo, isSeq = FALSE, binary = FALSE, 
   mode = "double")
oecosimu(dune, meandist, cf)

## Use pre-built null model
nm <- simulate(nullmodel(sipoo, "curveball"), 99)
oecosimu(nm, nestedchecker)
## Several chains of a sequential model -- this can be generalized
## for parallel processing (see ?smbind)
nm <- replicate(5, simulate(nullmodel(sipoo, "swap"), 99,
   thin=10, burnin=100), simplify = FALSE)
## nm is now a list of nullmodels: use smbind to combine these into one
## nullmodel with several chains
nm <- smbind(nm, MARGIN = 3)
nm
oecosimu(nm, nestedchecker)
## After this you can use as.mcmc() and tools in the coda package to
## analyse the chains (these will show that thin, burnin and nsimul are
## all too low for real analysis).
}
\keyword{ multivariate }
\keyword{ datagen }
\keyword{ nonparametric }

\encoding{UTF-8}
\name{contribdiv}
\alias{contribdiv}
\alias{plot.contribdiv}
\title{Contribution Diversity Approach}
\description{
The contribution diversity approach is based in the differentiation of
within-unit and among-unit diversity by using additive diversity
partitioning and unit distinctiveness.
}
\usage{
contribdiv(comm, index = c("richness", "simpson"),
     relative = FALSE, scaled = TRUE, drop.zero = FALSE)
\method{plot}{contribdiv}(x, sub, xlab, ylab, ylim, col, ...)
}
\arguments{
  \item{comm}{The community data matrix with samples as rows and species as column.}
  \item{index}{Character, the diversity index to be calculated.}
  \item{relative}{Logical, if \code{TRUE} then contribution diversity
    values are expressed as their signed deviation from their mean. See details.}
  \item{scaled}{Logical, if \code{TRUE} then relative contribution diversity
    values are scaled by the sum of gamma values (if \code{index = "richness"})
    or by sum of gamma values times the number of rows in \code{comm}
    (if \code{index = "simpson"}). See details.}
  \item{drop.zero}{Logical, should empty rows dropped from the result?
    If empty rows are not dropped, their corresponding results will be \code{NA}s.}
  \item{x}{An object of class \code{"contribdiv"}.}
  \item{sub, xlab, ylab, ylim, col}{Graphical arguments passed to plot.}
  \item{\dots}{Other arguments passed to plot.}
}
\details{
This approach was proposed by Lu et al. (2007).
Additive diversity partitioning (see \code{\link{adipart}} for more references)
deals with the relation of mean alpha and the total (gamma) diversity. Although
alpha diversity values often vary considerably. Thus, contributions of the sites
to the total diversity are uneven. This site specific contribution is measured by
contribution diversity components. A unit that has e.g. many unique species will
contribute more to the higher level (gamma) diversity than another unit with the
same number of species, but all of which common.

Distinctiveness of species \eqn{j} can be defined as the number of sites where it
occurs (\eqn{n_j}), or the sum of its relative frequencies (\eqn{p_j}). Relative
frequencies are computed sitewise and \eqn{sum_j{p_ij}}s at site \eqn{i} sum up
to \eqn{1}.

The contribution of site \eqn{i} to the total diversity is given by
\eqn{alpha_i = sum_j(1 / n_ij)} when dealing with richness and
\eqn{alpha_i = sum(p_{ij} * (1 - p_{ij}))} for the Simpson index.

The unit distinctiveness of site \eqn{i} is the average of the species
distinctiveness, averaging only those species which occur at site \eqn{i}.
For species richness: \eqn{alpha_i = mean(n_i)} (in the paper, the second
equation contains a typo, \eqn{n} is without index). For the Simpson index:
\eqn{alpha_i = mean(n_i)}.

The Lu et al. (2007) gives an in-depth description of the different indices.
}
\value{
An object of class \code{"contribdiv"} inheriting from data frame.

Returned values are alpha, beta and gamma components for each sites (rows)
of the community matrix. The \code{"diff.coef"} attribute gives the
differentiation coefficient (see Examples).
}
\references{
Lu, H. P., Wagner, H. H. and Chen, X. Y. 2007. A contribution diversity
approach to evaluate species diversity.
\emph{Basic and Applied Ecology}, 8, 1--12.
}
\author{\enc{Péter Sólymos}{Peter Solymos}, \email{solymos@ualberta.ca}}
\seealso{\code{\link{adipart}}, \code{\link{diversity}}}
\examples{
## Artificial example given in
## Table 2 in Lu et al. 2007
x <- matrix(c(
1/3,1/3,1/3,0,0,0,
0,0,1/3,1/3,1/3,0,
0,0,0,1/3,1/3,1/3),
3, 6, byrow = TRUE,
dimnames = list(LETTERS[1:3],letters[1:6]))
x
## Compare results with Table 2
contribdiv(x, "richness")
contribdiv(x, "simpson")
## Relative contribution (C values), compare with Table 2
(cd1 <- contribdiv(x, "richness", relative = TRUE, scaled = FALSE))
(cd2 <- contribdiv(x, "simpson", relative = TRUE, scaled = FALSE))
## Differentiation coefficients
attr(cd1, "diff.coef") # D_ST
attr(cd2, "diff.coef") # D_DT
## BCI data set
data(BCI)
opar <- par(mfrow=c(2,2))
plot(contribdiv(BCI, "richness"), main = "Absolute")
plot(contribdiv(BCI, "richness", relative = TRUE), main = "Relative")
plot(contribdiv(BCI, "simpson"))
plot(contribdiv(BCI, "simpson", relative = TRUE))
par(opar)
}
\keyword{multivariate}

\encoding{UTF-8}
\name{metaMDS}
\alias{metaMDS}
\alias{metaMDSdist}
\alias{metaMDSiter}
\alias{metaMDSredist}
\alias{initMDS}
\alias{postMDS}
\alias{plot.metaMDS}
\alias{points.metaMDS}
\alias{text.metaMDS}
\alias{scores.metaMDS}

\title{Nonmetric Multidimensional Scaling with Stable Solution from
  Random Starts, Axis Scaling and Species Scores}

\description{Function \code{metaMDS} performs Nonmetric
  Multidimensional Scaling (NMDS), and tries to find a stable solution
  using several random starts. In addition, it standardizes the
  scaling in the result, so that the configurations are easier to
  interpret, and adds species scores to the site ordination. The
  \code{metaMDS} function does not provide actual NMDS, but it calls
  another function for the purpose. Currently \code{\link{monoMDS}} is
  the default choice, and it is also possible to call the
  \code{\link[MASS]{isoMDS}} (\pkg{MASS} package). }

\usage{
metaMDS(comm, distance = "bray", k = 2, try = 20, trymax = 20, 
    engine = c("monoMDS", "isoMDS"), autotransform =TRUE,
    noshare = (engine == "isoMDS"), wascores = TRUE, expand = TRUE, 
    trace = 1, plot = FALSE, previous.best,  ...)
\method{plot}{metaMDS}(x, display = c("sites", "species"), choices = c(1, 2),
    type = "p", shrink = FALSE,  ...)
\method{points}{metaMDS}(x, display = c("sites", "species"),
    choices = c(1,2), shrink = FALSE, select, ...)
\method{text}{metaMDS}(x, display = c("sites", "species"), labels, 
    choices = c(1,2), shrink = FALSE, select, ...)
\method{scores}{metaMDS}(x, display = c("sites", "species"), shrink = FALSE, 
    choices, ...)
metaMDSdist(comm, distance = "bray", autotransform = TRUE, 
    noshare = TRUE, trace = 1, commname, zerodist = "ignore", 
    distfun = vegdist, ...)
metaMDSiter(dist, k = 2, try = 20, trymax = 20, trace = 1, plot = FALSE, 
    previous.best, engine = "monoMDS", maxit = 200,
    parallel = getOption("mc.cores"), ...)   
initMDS(x, k=2)
postMDS(X, dist, pc=TRUE, center=TRUE, halfchange, threshold=0.8,
    nthreshold=10, plot=FALSE, ...)
metaMDSredist(object, ...)
}

\arguments{
  \item{comm}{Community data. Alternatively, dissimilarities either as
    a \code{\link{dist}} structure or as a symmetric square matrix. 
    In the latter case all other stages are skipped except random 
    starts and centring and pc rotation of axes. }
  \item{distance}{Dissimilarity index used in \code{\link{vegdist}}.}
  \item{k}{Number of dimensions.  NB., the number of points \eqn{n}
    should be \eqn{n > 2k + 1}{n > 2*k + 1}, and preferably higher in
    global non-metric MDS, and still higher in local NMDS.}
  \item{try, trymax}{Minimum and maximum numbers of random starts in
    search of stable solution. After \code{try} has been reached, the
    iteration will stop when two convergent solutions were found or
    \code{trymax} was reached.}
  \item{engine}{The function used for MDS. The default is to use the
    \code{\link{monoMDS}} function in \pkg{vegan}, but for backward
    compatibility it is also possible to use \code{\link{isoMDS}} of
    \pkg{MASS}.}

  \item{autotransform}{Use simple heuristics for possible data
    transformation of typical community data (see below). If you do
    not have community data, you should probably set
    \code{autotransform = FALSE}.}

  \item{noshare}{Triggering of calculation step-across or extended
    dissimilarities with function \code{\link{stepacross}}. The
    argument can be logical or a numerical value greater than zero
    and less than one. If \code{TRUE}, extended dissimilarities are
    used always when there are no shared species between some sites,
    if \code{FALSE}, they are never used. If \code{noshare} is a
    numerical value, \code{\link{stepacross}} is used when the
    proportion of site pairs with no shared species exceeds
    \code{noshare}. The number of pairs with no shared species is
    found with \code{\link{no.shared}} function, and \code{noshare}
    has no effect if input data were dissimilarities instead of
    community data.}

  \item{wascores}{Calculate species scores using function
    \code{\link{wascores}}.}
  \item{expand}{Expand weighted averages of species in
    \code{\link{wascores}}.}
  \item{trace}{Trace the function; \code{trace = 2} or higher will be
    more voluminous.}
  \item{plot}{Graphical tracing: plot interim results. You may want to set
    \code{par(ask = TRUE)} with this option.}
  \item{previous.best}{Start searches from a previous solution.}

  \item{x}{\code{metaMDS} result (or a dissimilarity structure for
  \code{initMDS}).}

  \item{choices}{Axes shown.}
  \item{type}{Plot type: \code{"p"} for points, \code{"t"} for text, and
   \code{"n"} for axes only.}
 \item{display}{Display \code{"sites"} or \code{"species"}.}
 \item{shrink}{Shrink back species scores if they were expanded
   originally.}
 \item{labels}{Optional test to be used instead of row names.}
 \item{select}{Items to be displayed.  This can either be a logical
   vector which is \code{TRUE} for displayed items or a vector of indices
   of displayed items.}
 \item{X}{Configuration from multidimensional scaling. }
 \item{commname}{The name of \code{comm}: should not be given if the
   function is called directly.}

 \item{zerodist}{Handling of zero dissimilarities: either
   \code{"fail"} or \code{"add"} a small positive value, or
   \code{"ignore"}. \code{\link{monoMDS}} accepts zero dissimilarities
   and the default is \code{zerodist = "ignore"}, but with
   \code{\link[MASS]{isoMDS}} you may need to set \code{zerodist = "add"}.}

 \item{distfun}{Dissimilarity function. Any function returning a
   \code{dist} object and accepting argument \code{method} can be used
   (but some extra arguments may cause name conflicts).}

 \item{maxit}{Maximum number of iterations in the single NMDS run;
   passed to the \code{engine} function \code{\link{monoMDS}} or
   \code{\link[MASS]{isoMDS}}.}

 \item{parallel}{Number of parallel processes or a predefined socket
   cluster.  If you use pre-defined socket clusters (say,
   \code{clus}), you must issue \code{clusterEvalQ(clus,
   library(vegan))} to make available internal \pkg{vegan}
   functions. With \code{parallel = 1} uses ordinary, non-parallel
   processing. The parallel processing is done with \pkg{parallel}
   package.}

 \item{dist}{Dissimilarity matrix used in multidimensional scaling. }
  \item{pc}{Rotate to principal components. }
  \item{center}{Centre the configuration. }
 \item{halfchange}{Scale axes to half-change units. This defaults
  \code{TRUE} when dissimilarities were evaluated within
  \code{metaMDS} and the dissimilarity index has an upper limit of
  \eqn{1}. If \code{FALSE}, the ordination dissimilarities are scaled
  to the same range as the input dissimilarities.}
 \item{threshold}{Largest dissimilarity used in half-change scaling. }
 \item{nthreshold}{ Minimum number of points in half-change scaling. }
 \item{object}{A result object from \code{metaMDS}.}

 \item{\dots}{Other parameters passed to functions. Function
   \code{metaMDS} passes all arguments to its component functions
   \code{metaMDSdist}, \code{metaMDSiter}, \code{postMDS}, and to
   \code{distfun} and \code{engine}.}  }

\details{ Non-metric Multidimensional Scaling (NMDS) is commonly
  regarded as the most robust unconstrained ordination method in
  community ecology (Minchin 1987).  Function \code{metaMDS} is a
  wrapper function that calls several other functions to combine
  Minchin's (1987) recommendations into one command. The complete
  steps in \code{metaMDS} are:
  
\enumerate{ 

  \item Transformation: If the data values are larger than common
    abundance class scales, the function performs a Wisconsin double
    standardization (\code{\link{wisconsin}}).  If the values look
    very large, the function also performs \code{\link{sqrt}}
    transformation. Both of these standardizations are generally found
    to improve the results. However, the limits are completely
    arbitrary (at present, data maximum 50 triggers \code{\link{sqrt}}
    and \eqn{>9} triggers \code{\link{wisconsin}}). If you want to
    have a full control of the analysis, you should set
    \code{autotransform = FALSE} and standardize and transform data
    independently. The \code{autotransform} is intended for community
    data, and for other data types, you should set
    \code{autotransform = FALSE}. This step is perfomed using
    \code{metaMDSdist}, and the step is skipped if input were
    dissimilarities.

  \item Choice of dissimilarity: For a good result, you should use
    dissimilarity indices that have a good rank order relation to
    ordering sites along gradients (Faith et al. 1987).  The default
    is Bray-Curtis dissimilarity, because it often is the test
    winner. However, any other dissimilarity index in
    \code{\link{vegdist}} can be used. Function
    \code{\link{rankindex}} can be used for finding the test winner
    for you data and gradients. The default choice may be bad if you
    analyse other than community data, and you should probably select
    an appropriate index using argument \code{distance}.  This step is
    performed using \code{metaMDSdist}, and the step is skipped if
    input were dissimilarities.

  \item Step-across dissimilarities: Ordination may be very difficult
    if a large proportion of sites have no shared species. In this
    case, the results may be improved with \code{\link{stepacross}}
    dissimilarities, or flexible shortest paths among all sites.  The
    default NMDS \code{engine} is \code{\link{monoMDS}} which is able
    to break tied values at the maximum dissimilarity, and this often
    is sufficient to handle cases with no shared species, and
    therefore the default is not to use \code{\link{stepacross}} with
    \code{\link{monoMDS}}.  Function \code{\link[MASS]{isoMDS}} does
    not handle tied values adequately, and therefore the default is to
    use \code{\link{stepacross}} always when there are sites with no
    shared species with \code{engine = "isoMDS"}. The
    \code{\link{stepacross}} is triggered by option \code{noshare}. If
    you do not like manipulation of original distances, you should set
    \code{noshare = FALSE}.  This step is skipped if input data were
    dissimilarities instead of community data. This step is performed
    using \code{metaMDSdist}, and the step is skipped always when
    input were dissimilarities.

  \item NMDS with random starts: NMDS easily gets trapped into local
    optima, and you must start NMDS several times from random starts
    to be confident that you have found the global solution. The
    strategy in \code{metaMDS} is to first run NMDS starting with the
    metric scaling (\code{\link{cmdscale}} which usually finds a good
    solution but often close to a local optimum), or use the
    \code{previous.best} solution if supplied, and take its solution
    as the standard (\code{Run 0}). Then \code{metaMDS} starts NMDS
    from several random starts (minimum number is given by \code{try}
    and maximum number by \code{trymax}). These random starts are
    generated by \code{initMDS}. If a solution is better (has a lower
    stress) than the previous standard, it is taken as the new
    standard. If the solution is better or close to a standard,
    \code{metaMDS} compares two solutions using Procrustes analysis
    (function \code{\link{procrustes}} with option
    \code{symmetric = TRUE}). If the solutions are very similar in their
    Procrustes \code{rmse} and the largest residual is very small, the
    solutions are regarded as convergent and the better one is taken
    as the new standard.  The conditions are stringent, and you may
    have found good and relatively stable solutions although the
    function is not yet satisfied. Setting \code{trace = TRUE} will
    monitor the final stresses, and \code{plot = TRUE} will display
    Procrustes overlay plots from each comparison. This step is
    performed using \code{metaMDSiter}. This is the first step
    performed if input data (\code{comm}) were dissimilarities. Random
    starts can be run with parallel processing (argument
    \code{parallel}).

  \item Scaling of the results: \code{metaMDS} will run \code{postMDS}
    for the final result. Function \code{postMDS} provides the
    following ways of \dQuote{fixing} the indeterminacy of scaling and
    orientation of axes in NMDS: Centring moves the origin to the
    average of the axes; Principal components rotate the configuration
    so that the variance of points is maximized on first dimension
    (with function \code{\link{MDSrotate}} you can alternatively
    rotate the configuration so that the first axis is parallel to an
    environmental variable); Half-change scaling scales the
    configuration so that one unit means halving of community
    similarity from replicate similarity.  Half-change scaling is
    based on closer dissimilarities where the relation between
    ordination distance and community dissimilarity is rather linear
    (the limit is set by argument \code{threshold}). If there are
    enough points below this threshold (controlled by the parameter
    \code{nthreshold}), dissimilarities are regressed on distances.
    The intercept of this regression is taken as the replicate
    dissimilarity, and half-change is the distance where similarity
    halves according to linear regression.  Obviously the method is
    applicable only for dissimilarity indices scaled to \eqn{0 \ldots
    1}, such as Kulczynski, Bray-Curtis and Canberra indices. If
    half-change scaling is not used, the ordination is scaled to the
    same range as the original dissimilarities. Half-change scaling is
    skipped by default if input were dissimilarities, but can be
    turned on with argument \code{halfchange = TRUE}. NB., The PC
    rotation only changes the directions of reference axes, and it
    does not influence the configuration or solution in general.

  \item Species scores: Function adds the species scores to the final
    solution as weighted averages using function
    \code{\link{wascores}} with given value of parameter
    \code{expand}.  The expansion of weighted averages can be undone
    with \code{shrink = TRUE} in \code{plot} or \code{scores}
    functions, and the calculation of species scores can be suppressed
    with \code{wascores = FALSE}. This step is skipped if input were
    dissimilarities and community data were unavailable. However, the
    species scores can be added or replaced with
    \code{\link{sppscores}}.
    
} 
}

\section{Convergence Problems}{

    The function tries hard to find two convergent solutions, but it
    may fail. With default \code{engine = "monoMDS"} the function will
    tabulate the stopping criteria used, so that you can see which
    criterion should be made more stringent. The criteria can be given
    as arguments to \code{metaMDS} and their current values are
    described in \code{\link{monoMDS}}. In particular, if you reach
    the maximum number of iterations, you should increase the value of
    \code{maxit}. You may ask for a larger number of random starts
    without losing the old ones giving the previous solution in
    argument \code{previous.best}.

    In addition to too slack convergence criteria and too low number
    of random starts, wrong number of dimensions (argument \code{k})
    is the most common reason for not finding convergent
    solutions. NMDS is usually run with a low number dimensions
    (\code{k=2} or \code{k=3}), and for complex data increasing
    \code{k} by one may help. If you run NMDS with much higher number
    of dimensions (say, \code{k=10} or more), you should reconsider
    what you are doing and drastically reduce \code{k}. For very
    heterogeneous data sets with partial disjunctions, it may help to
    set \code{stepacross}, but for most data sets the default
    \code{weakties = TRUE} is sufficient.

    Please note that you can give all arguments of other
    \code{metaMDS*} functions and NMDS engine (default
    \code{\link{monoMDS}}) in your \code{metaMDS} command,and you
    should check documentation of these functions for details.

}

\section{Common Wrong Claims}{

    NMDS is often misunderstood and wrong claims of its properties are
    common on the Web and even in publications. It is often claimed
    that the NMDS configuration is non-metric which means that you
    cannot fit environmental variables or species onto that
    space. This is a false statement. In fact, the result
    configuration of NMDS is metric, and it can be used like any other
    ordination result. In NMDS the rank orders of Euclidean distances
    among points in ordination have a non-metric monotone relationship
    to any observed dissimilarities. The transfer function from
    observed dissimilarities to ordination distances is non-metric
    (Kruskal 1964a, 1964b), but the ordination result configuration is
    metric and observed dissimilarities can be of any kind (metric or
    non-metric).

    The ordination configuration is usually rotated to principal
    components in \code{metaMDS}. The rotation is performed after
    finding the result, and it only changes the direction of the
    reference axes. The only invariant feature in the NMDS solution is
    the rank order of Euclidean ordination distances, and this does
    not change in rotation. Similarly, the rank order of distances
    does not change in uniform scaling or centring of configuration of
    points.

    Function \code{\link{stressplot}} displays the method graphically:
    it plots the observed dissimilarities against distances in
    ordination space, and also shows the non-metric monotone
    regression.

}

\value{ Function \code{metaMDS} returns an object of class
  \code{metaMDS}. The final site ordination is stored in the item
  \code{points}, and species ordination in the item \code{species},
  and the stress in item \code{stress} (NB, the scaling of the stress
  depends on the \code{engine}: \code{\link[MASS]{isoMDS}} uses
  percents, and \code{\link{monoMDS}} proportions in the range \eqn{0
  \ldots 1}). The other items store the information on the steps taken
  and the items returned by the \code{engine} function. The object has
  \code{print}, \code{plot}, \code{points} and \code{text} methods.
  Functions \code{metaMDSdist} and \code{metaMDSredist} return
  \code{\link{vegdist}} objects.  Function \code{initMDS} returns a
  random configuration which is intended to be used within
  \code{\link[MASS]{isoMDS}} only.  Functions \code{metaMDSiter} and
  \code{postMDS} returns the result of NMDS with updated
  configuration.  }


\references{
  Faith, D. P, Minchin, P. R. and Belbin, L. (1987).
  Compositional dissimilarity as a robust measure of ecological
  distance. \emph{Vegetatio} 69, 57--68.

  Kruskal, J.B. (1964a). Multidimensional scaling by optimizing
      goodness-of-fit to a nonmetric hypothesis. \emph{Psychometrika}
      29, 1--28.

  Kruskal, J.B. (1964b). Nonmetric multidimensional scaling: a numerical
      method. \emph{Psychometrika} 29, 115--129.

  Minchin, P.R. (1987). An evaluation of relative robustness
  of techniques for ecological ordinations. \emph{Vegetatio} 69,
  89--107.
}

\author{ Jari Oksanen }

\note{ Function \code{metaMDS} is a simple wrapper for an NMDS engine
  (either \code{\link{monoMDS}} or \code{\link[MASS]{isoMDS}}) and
  some support functions (\code{metaMDSdist},
  \code{\link{stepacross}}, \code{metaMDSiter}, \code{initMDS},
  \code{postMDS}, \code{\link{wascores}}).  You can call these support
  functions separately for better control of results.  Data
  transformation, dissimilarities and possible
  \code{\link{stepacross}} are made in function \code{metaMDSdist}
  which returns a dissimilarity result. Iterative search (with
  starting values from \code{initMDS} with \code{\link{monoMDS}}) is
  made in \code{metaMDSiter}.  Processing of result configuration is
  done in \code{postMDS}, and species scores added by
  \code{\link{wascores}}.  If you want to be more certain of reaching
  a global solution, you can compare results from several independent
  runs. You can also continue analysis from previous results or from
  your own configuration.  Function may not save the used
  dissimilarity matrix (\code{\link{monoMDS}} does), but
  \code{metaMDSredist} tries to reconstruct the used dissimilarities
  with original data transformation and possible
  \code{\link{stepacross}}.

  The \code{metaMDS} function was designed to be used with community
  data.  If you have other type of data, you should probably set some
  arguments to non-default values: probably at least \code{wascores},
  \code{autotransform} and \code{noshare} should be \code{FALSE}. If
  you have negative data entries, \code{metaMDS} will set the previous
  to \code{FALSE} with a warning.  
}

\section{Warning}{\code{metaMDS} uses \code{\link{monoMDS}} as its
  NMDS \code{engine} from \pkg{vegan} version 2.0-0, when it replaced
  the \code{\link[MASS]{isoMDS}} function. You can set argument
  \code{engine} to select the old engine.}

\seealso{\code{\link{monoMDS}} (and \code{\link[MASS]{isoMDS}}), 
  \code{\link{decostand}}, \code{\link{wisconsin}}, 
  \code{\link{vegdist}}, \code{\link{rankindex}}, \code{\link{stepacross}}, 
  \code{\link{procrustes}}, \code{\link{wascores}}, \code{\link{sppscores}},
  \code{\link{MDSrotate}}, \code{\link{ordiplot}}, \code{\link{stressplot}}.
}

\examples{
## The recommended way of running NMDS (Minchin 1987)
##
data(dune)
# Global NMDS using monoMDS
sol <- metaMDS(dune)
sol
plot(sol, type="t")
## Start from previous best solution
sol <- metaMDS(dune, previous.best = sol)
## Local NMDS and stress 2 of monoMDS
sol2 <- metaMDS(dune, model = "local", stress=2)
sol2
## Use Arrhenius exponent 'z' as a binary dissimilarity measure
sol <- metaMDS(dune, distfun = betadiver, distance = "z")
sol
}
\keyword{ multivariate }

\name{clamtest}
\alias{clamtest}
\alias{summary.clamtest}
\alias{plot.clamtest}

\title{
Multinomial Species Classification Method (CLAM)
}

\description{
The CLAM statistical approach for classifying generalists and
specialists in two distinct habitats is described in Chazdon et al. (2011).
}
\usage{
clamtest(comm, groups, coverage.limit = 10, specialization = 2/3, 
   npoints = 20, alpha = 0.05/20)
\method{summary}{clamtest}(object, ...)
\method{plot}{clamtest}(x, xlab, ylab, main,  pch = 21:24, col.points = 1:4, 
   col.lines = 2:4, lty = 1:3, position = "bottomright", ...)
}
\arguments{
  \item{comm}{
Community matrix, consisting of counts.
}
  \item{groups}{
A vector identifying the two habitats. Must have exactly
two unique values or levels. Habitat IDs in the grouping vector
must match corresponding rows in the community matrix \code{comm}.
}
  \item{coverage.limit}{
Integer, the sample coverage based correction 
is applied to rare species with counts below this limit. 
Sample coverage is calculated separately 
for the two habitats. Sample relative abundances are used for species 
with higher than or equal to \code{coverage.limit} total counts per habitat.
}
  \item{specialization}{
Numeric, specialization threshold value between 0 and 1.
The value of \eqn{2/3} represents \sQuote{supermajority} rule,
while a value of \eqn{1/2} represents a \sQuote{simple majority} rule
to assign shared species as habitat specialists.
}
  \item{npoints}{
Integer, number of points used to determine the boundary lines
in the plots.
}
\item{alpha}{ Numeric, nominal significance level for individual
  tests.  The default value reduces the conventional limit of
  \eqn{0.05} to account for overdispersion and multiple testing for
  several species simultaneously. However, the is no firm reason for
  exactly this limit.  }
  \item{x, object}{
Fitted model object of class \code{"clamtest"}.
}
  \item{xlab, ylab}{
Labels for the plot axes.
}
  \item{main}{
Main title of the plot.
}
  \item{pch, col.points}{
Symbols and colors used in plotting species groups.
}
  \item{lty, col.lines}{
Line types and colors for boundary lines in plot to separate species groups.
}
  \item{position}{
Position of figure legend, see \code{\link{legend}} for specification details.
Legend not shown if \code{position = NULL}.
}
  \item{\dots}{
Additional arguments passed to methods.
}
}

\details{ The method uses a multinomial model based on estimated
  species relative abundance in two habitats (A, B). It minimizes bias
  due to differences in sampling intensities between two habitat types
  as well as bias due to insufficient sampling within each
  habitat. The method permits a robust statistical classification of
  habitat specialists and generalists, without excluding rare species
  \emph{a priori} (Chazdon et al. 2011).  Based on a user-defined
  \code{specialization} threshold, the model classifies species into
  one of four groups: (1) generalists; (2) habitat A specialists; (3)
  habitat B specialists; and (4) too rare to classify with confidence.
  } 

\value{ A data frame (with class attribute \code{"clamtest"}),
  with columns: 
  \itemize{ 
    \item{\code{Species}:}{ species name (column names from \code{comm}),} 
    \item{\code{Total_*A*}:}{ total count in habitat A,} 
    \item{\code{Total_*B*}:}{ total count in habitat B,} 
    \item{\code{Classes}:}{ species classification, a factor with
       levels \code{Generalist}, \code{Specialist_*A*},
       \code{Specialist_*B*}, and \code{Too_rare}.}  
}
  \code{*A*} and \code{*B*} are placeholders for habitat names/labels found in the
  data.

The \code{summary} method returns descriptive statistics of the results.
The \code{plot} method returns values invisibly and produces a bivariate
scatterplot of species total abundances in the two habitats. Symbols and
boundary lines are shown for species groups.
}
\references{
Chazdon, R. L., Chao, A., Colwell, R. K., Lin, S.-Y., Norden, N., 
Letcher, S. G., Clark, D. B., Finegan, B. and Arroyo J. P.(2011). 
A novel statistical method for classifying habitat
generalists and specialists. \emph{Ecology} \bold{92}, 1332--1343.
}
\author{
Peter Solymos \email{solymos@ualberta.ca}
}
\note{
The code was tested against standalone CLAM software provided
on the website of Anne Chao (which were then at http://chao.stat.nthu.edu.tw/wordpress);
minor inconsistencies were found, especially for finding the
threshold for 'too rare' species.
These inconsistencies are probably due to numerical differences between the
two implementation. The current \R implementation uses 
root finding for iso-lines instead of iterative search.

The original method (Chazdon et al. 2011) has two major problems:
\enumerate{
  
  \item It assumes that the error distribution is multinomial. This is
    a justified choice if individuals are freely distributed, and
    there is no over-dispersion or clustering of individuals. In most
    ecological data, the variance is much higher than multinomial
    assumption, and therefore test statistic are too optimistic.

  \item The original authors suggest that multiple testing adjustment
    for multiple testing should be based on the number of points
    (\code{npoints}) used to draw the critical lines on the plot,
    whereas the adjustment should be based on the number of tests (i.e.,
    tested species). The function uses the same numerical values as
    the original paper, but there is no automatic connection between
    \code{npoints} and \code{alpha} arguments, but you must work out
    the adjustment yourself.
}
}
\examples{
data(mite)
data(mite.env)
sol <- with(mite.env, clamtest(mite, Shrub=="None", alpha=0.005))
summary(sol)
head(sol)
plot(sol)
}
\keyword{ htest }

\name{stressplot.wcmdscale}
\alias{stressplot.wcmdscale}
\alias{stressplot.cca}
\alias{stressplot.rda}
\alias{stressplot.capscale}
\alias{stressplot.dbrda}
\alias{stressplot.prcomp}
\alias{stressplot.princomp}

\title{
  Display Ordination Distances Against Observed Distances in Eigenvector Ordinations
}

\description{
  Functions plot ordination distances in given number of dimensions
  against observed distances or distances in full space in eigenvector
  methods. The display is similar as the Shepard diagram
  (\code{\link{stressplot}} for non-metric multidimensional scaling
  with \code{\link{metaMDS}} or \code{\link{monoMDS}}), but shows the
  linear relationship of the eigenvector ordinations. The
  \code{stressplot} methods are available for \code{\link{wcmdscale}},
  \code{\link{rda}}, \code{\link{cca}}, \code{\link{capscale}},
  \code{\link{dbrda}}, \code{\link{prcomp}} and \code{\link{princomp}}. 
}

\usage{
\method{stressplot}{wcmdscale}(object, k = 2, pch, p.col = "blue", l.col = "red",
    lwd = 2, ...)
}

\arguments{
  \item{object}{
    Result object from eigenvector ordination (\code{\link{wcmdscale}},
    \code{\link{rda}}, \code{\link{cca}}, \code{\link{dbrda}},
    \code{\link{capscale}})
}
  \item{k}{
    Number of dimensions for which the ordination distances are displayed.
}
  \item{pch, p.col, l.col, lwd}{
    Plotting character, point colour and line colour like in
    default \code{\link{stressplot}}
}
  \item{\dots}{
    Other parameters to functions, e.g. graphical parameters.
}
}

\details{ The functions offer a similar display for eigenvector
  ordinations as the standard Shepard diagram
  (\code{\link{stressplot}}) in non-metric multidimensional
  scaling. The ordination distances in given number of dimensions are
  plotted against observed distances. With metric distances, the
  ordination distances in full space (with all ordination axes) are
  equal to observed distances, and the fit line shows this
  equality. In general, the fit line does not go through the points,
  but the points for observed distances approach the fit line from
  below. However, with non-Euclidean distances (in
  \code{\link{wcmdscale}}, \code{\link{dbrda}} or
  \code{\link{capscale}}) with negative eigenvalues the ordination
  distances can exceed the observed distances in real dimensions; the
  imaginary dimensions with negative eigenvalues will correct these
  excess distances. If you have used \code{\link{dbrda}},
  \code{\link{capscale}} or \code{\link{wcmdscale}} with argument
  \code{add} to avoid negative eigenvalues, the ordination distances
  will exceed the observed dissimilarities.

  In partial ordination (\code{\link{cca}}, \code{\link{rda}}, and
  \code{\link{capscale}} with \code{Condition} in the formula), the
  distances in the partial component are included both in the observed
  distances and in ordination distances.  With \code{k=0}, the
  ordination distances refer to the partial ordination. The exception is
  \code{\link{dbrda}} where the distances in partial, constrained and
  residual components are not additive, and only the first of these
  components can be shown, and partial models cannot be shown at all.
}

\value{
  Functions draw a graph and return invisibly the ordination distances
  or the ordination distances.
}

\author{
  Jari Oksanen.
}

\seealso{
  \code{\link{stressplot}} and \code{\link{stressplot.monoMDS}} for
  standard Shepard diagrams.
}

\examples{
data(dune, dune.env)
mod <- rda(dune)
stressplot(mod)
mod <- rda(dune ~ Management, dune.env)
stressplot(mod, k=3)
}

\keyword{ multivariate }

\name{nobs.adonis}
\alias{nobs.adonis}
\alias{nobs.betadisper}
\alias{nobs.cca}
\alias{nobs.CCorA}
\alias{nobs.decorana}
\alias{nobs.isomap}
\alias{nobs.metaMDS}
\alias{nobs.pcnm}
\alias{nobs.procrustes}
\alias{nobs.rad}
\alias{nobs.varpart}
\alias{nobs.wcmdscale}

\title{
  Extract the Number of Observations from a vegan Fit.
}
\description{
  Extract the number of \sQuote{observations} from a \pkg{vegan} model fit.
}
\usage{
\method{nobs}{adonis}(object, ...)
}

\arguments{
  \item{object}{
    A fitted model object.
}
  \item{\dots}{
   Further arguments to be passed to methods.
}
}

\details{ Function \code{nobs} is generic in \R, and
  \pkg{vegan} provides methods for objects from \code{\link{adonis}},
  \code{\link{betadisper}}, \code{\link{cca}} and other related
  methods, \code{\link{CCorA}}, \code{\link{decorana}},
  \code{\link{isomap}}, \code{\link{metaMDS}}, \code{\link{pcnm}},
  \code{\link{procrustes}}, \code{\link{radfit}},
  \code{\link{varpart}} and \code{\link{wcmdscale}}.  }

\value{ A single number, normally an integer, giving the number of
  observations.  }

\author{
  Jari Oksanen
}

\keyword{models}

\name{raupcrick}
\alias{raupcrick}

\title{
  Raup-Crick Dissimilarity with Unequal Sampling Densities of Species
}

\description{ Function finds the Raup-Crick dissimilarity which is a
  probability of number of co-occurring species with species
  occurrence probabilities proportional to species frequencies.  }

\usage{
raupcrick(comm, null = "r1", nsimul = 999, chase = FALSE, ...)
}

\arguments{

  \item{comm}{Community data which will be treated as presence/absence data.}

  \item{null}{Null model used as the \code{method} in
     \code{\link{oecosimu}}.}

  \item{nsimul}{Number of null communities for assessing the
     dissimilarity index.}

  \item{chase}{Use the Chase et al. (2011) method of tie handling (not
     recommended except for comparing the results against the Chase
     script).}

 \item{\dots}{Other parameters passed to \code{\link{oecosimu}}.}

}

\details{Raup-Crick index is the probability that compared sampling
  units have non-identical species composition.  This probability can
  be regarded as a dissimilarity, although it is not metric: identical
  sampling units can have dissimilarity slightly above \eqn{0}, the
  dissimilarity can be nearly zero over a range of shared species, and
  sampling units with no shared species can have dissimilarity
  slightly below \eqn{1}. Moreover, communities sharing rare species
  appear as more similar (lower probability of finding rare species
  together), than communities sharing the same number of common
  species.

  The function will always treat the data as binary (presence/
  absence).

  The probability is assessed using simulation with
  \code{\link{oecosimu}} where the test statistic is the observed
  number of shared species between sampling units evaluated against a
  community null model (see Examples).  The default null model is
  \code{"r1"} where the probability of selecting species is
  proportional to the species frequencies.

  The \code{\link{vegdist}} function implements a variant of the
  Raup-Crick index with equal sampling probabilities for species using
  exact analytic equations without simulation. This corresponds to
  \code{null} model \code{"r0"} which also can be used with the
  current function.  All other null model methods of
  \code{\link{oecosimu}} can be used with the current function, but
  they are new unpublished methods.  }

\value{The function returns an object inheriting from
  \code{\link{dist}} which can be interpreted as a dissimilarity
  matrix.}

\references{
  Chase, J.M., Kraft, N.J.B., Smith, K.G., Vellend, M. and Inouye,
  B.D. (2011). Using null models to disentangle variation in community
  dissimilarity from variation in \eqn{\alpha}{alpha}-diversity.
  \emph{Ecosphere} 2:art24 \doi{10.1890/ES10-00117.1}
}

\author{The function was developed after Brian Inouye contacted us and
  informed us about the method in Chase et al. (2011), and the
  function takes its idea from the code that was published with their
  paper. The current function was written by Jari Oksanen.  }

\note{ The test statistic is the number of shared species, and this is
  typically tied with a large number of simulation results. The tied
  values are handled differently in the current function and in the
  function published with Chase et al. (2011). In \pkg{vegan}, the
  index is the number of simulated values that are smaller \emph{or
  equal} than the observed value, but smaller than observed value is
  used by Chase et al. (2011) with option \code{split = FALSE} in
  their script; this can be achieved with \code{chase = TRUE} in
  \pkg{vegan}.  Chase et al. (2011) script with \code{split = TRUE}
  uses half of tied simulation values to calculate a distance measure,
  and that choice cannot be directly reproduced in vegan (it is the
  average of \pkg{vegan} \code{raupcrick} results with 
  \code{chase = TRUE} and \code{chase = FALSE}).}

\seealso{The function is based on \code{\link{oecosimu}}. Function
  \code{\link{vegdist}} with {method = "raup"} implements a related
  index but with equal sampling densities of species, and
  \code{\link{designdist}} demonstrates its calculation.}

\examples{
## data set with variable species richness
data(sipoo)
## default raupcrick
dr1 <- raupcrick(sipoo)
## use null model "r0" of oecosimu
dr0 <- raupcrick(sipoo, null = "r0")
## vegdist(..., method = "raup") corresponds to 'null = "r0"'
d <- vegdist(sipoo, "raup")
op <- par(mfrow=c(2,1), mar=c(4,4,1,1)+.1)
plot(dr1 ~ d, xlab = "Raup-Crick with Null R1", ylab="vegdist")
plot(dr0 ~ d, xlab = "Raup-Crick with Null R0", ylab="vegdist")
par(op)

## The calculation is essentially as in the following oecosimu() call,
## except that designdist() is replaced with faster code
\dontrun{%
oecosimu(sipoo, function(x) designdist(x, "J", "binary"), method = "r1")
}
}

\keyword{ multivariate }

\encoding{UTF-8}
\name{kendall.global}
\alias{kendall.global}
\alias{kendall.post}

\title{ Kendall coefficient of concordance }

\description{ 
   Function \code{kendall.global} computes and tests the coefficient of
  concordance among several judges (variables, species) through a
  permutation test.

  Function \code{kendall.post} carries out \emph{a posteriori} tests
  of the contributions of individual judges (variables, species) to
  the overall concordance of their group through permutation tests.

  If several groups of judges are identified in the data table,
  coefficients of concordance (\code{kendall.global}) or a posteriori
  tests (\code{kendall.post}) will be computed for each group
  separately. Use in ecology: to identify significant species
  associations.
}
\usage{
kendall.global(Y, group, nperm = 999, mult = "holm")
kendall.post(Y, group, nperm = 999, mult = "holm")
}
\arguments{
  \item{Y}{ Data file (data frame or matrix) containing quantitative or
    semiquantitative data. Rows are objects and columns are judges
    (variables). In community ecology, that table is often a
    site-by-species table. }
  \item{group}{ A vector defining how judges should be divided into
    groups. See example below. If groups are not explicitly defined,
    all judges in the data file will be considered as forming a single
    group. }
  \item{nperm}{ Number of permutations to be performed. Default is
    999. }
  \item{mult}{Correct P-values for multiple testing using the
    alternatives described in \code{\link{p.adjust}} and in addition
    \code{"sidak"} (see Details). The Bonferroni correction is overly
    conservative; it is not recommended. It is included to allow
    comparisons with the other methods.}
}
\details{
  \code{Y} must contain quantitative data. They will be transformed to
  ranks within each column before computation of the coefficient of
  concordance.

  The search for species associations described in Legendre (2005)
  proceeds in 3 steps:

  (1) Correlation analysis of the species. A possible method is to
  compute Ward's agglomerative clustering of a matrix of correlations
  among the species. In detail: (1.1) compute a Pearson or Spearman
  correlation matrix (\code{correl.matrix}) among the species; (1.2)
  turn it into a distance matrix: \code{mat.D = as.dist(1-correl.matrix)}; 
  (1.3) carry out Ward's hierarchical
  clustering of that matrix using \code{hclust}: 
  \code{clust.ward = hclust(mat.D, "ward")}; (1.4) plot the dendrogram:
  \code{plot(clust.ward, hang=-1)}; (1.5) cut the dendrogram in two
  groups, retrieve the vector of species membership: 
  \code{group.2 = cutree(clust.ward, k=2)}. (1.6) After steps 2 and 3 below, 
  you may
  have to come back and try divisions of the species into k = \eqn{3, 4, 5, \dots} 
  groups.

  (2) Compute global tests of significance of the 2 (or more) groups
  using the function \code{kendall.global} and the vector defining the
  groups. Groups that are not globally significant must be refined or
  abandoned.

  (3) Compute a posteriori tests of the contribution of individual
  species to the concordance of their group using the function
  \code{kendall.post} and the vector defining the groups. If some
  species have negative values for "Spearman.mean", this means that
  these species clearly do not belong to the group, hence that group
  is too inclusive. Go back to (1.5) and cut the dendrogram more
  finely. The left and right groups can be cut separately,
  independently of the levels along the dendrogram; write your own
  vector of group membership if \code{cutree} does not produce the
  desired groups.

  The corrections used for multiple testing are applied to the list of
  P-values (P); they take into account the number of tests (k) carried
  out simultaneously (number of groups in \code{kendall.global}, or
  number of species in \code{kendall.post}). The corrections are
  performed using function \code{\link{p.adjust}}; see that function
  for the description of the correction methods. In addition, there is
  \enc{Šidák}{Sidak} correction which defined as 
  \eqn{P_{corr} = 1 -(1 - P)^k}.
}
\value{

  A table containing the following information in rows. The columns
  correspond to the groups of "judges" defined in vector "group". When
  function \code{Kendall.post} is used, there are as many tables as
  the number of predefined groups.

  \item{W }{Kendall's coefficient of concordance, W. }

  \item{F }{F statistic. F = W*(m-1)/(1-W) where m is the number of
  judges. }

  \item{Prob.F }{Probability associated with the F statistic, computed
  from the F distribution with nu1 = n-1-(2/m) and nu2 = nu1*(m-1); n is
  the number of objects. }

  \item{Corrected prob.F }{Probabilities associated with F, corrected
  using the method selected in parameter \code{mult}. Shown only if
  there are more than one group. }

  \item{Chi2 }{Friedman's chi-square statistic (Friedman 1937) used in
  the permutation test of W. }

  \item{Prob.perm }{Permutational probabilities, uncorrected. }

  \item{Corrected prob.perm }{Permutational probabilities corrected
  using the method selected in parameter \code{mult}. Shown only if
  there are more than one group. }

  \item{Spearman.mean }{Mean of the Spearman correlations between the
  judge under test and all the other judges in the same group. }

  \item{W.per.species }{Contribution of the judge under test to the
  overall concordance statistic for that group. }
}
\references{ 

Friedman, M. 1937. The use of ranks to avoid the assumption of normality
implicit in the analysis of variance. Journal of the American
Statistical Association 32: 675-701.

Kendall, M. G. and B. Babington Smith. 1939. The problem of m
rankings. Annals of Mathematical Statistics 10: 275-287.

Legendre, P. 2005. Species associations: the Kendall coefficient of
concordance revisited. Journal of Agricultural, Biological, and
Environmental Statistics 10: 226-245.

Legendre, P. 2009. Coefficient of concordance. In: Encyclopedia of
Research Design. SAGE Publications (in press).

Siegel, S. and N. J. Castellan, Jr. 1988. Nonparametric statistics for
the behavioral sciences. 2nd edition. McGraw-Hill, New York.
}

\seealso{\code{\link{cor}}, \code{\link{friedman.test}},
\code{\link{hclust}}, \code{\link{cutree}}, \code{\link{kmeans}},
\code{\link{cascadeKM}}, \code{\link[labdsv]{indval}}}

\author{ F. Guillaume Blanchet, University of Alberta, and Pierre
  Legendre, Université de Montréal }

\examples{
data(mite)
mite.hel <- decostand(mite, "hel")

# Reproduce the results shown in Table 2 of Legendre (2005), a single group
mite.small <- mite.hel[c(4,9,14,22,31,34,45,53,61,69),c(13:15,23)]
kendall.global(mite.small, nperm=49)
kendall.post(mite.small, mult="holm", nperm=49)

# Reproduce the results shown in Tables 3 and 4 of Legendre (2005), 2 groups
group <-c(1,1,2,1,1,1,1,1,2,1,1,1,1,1,1,2,1,2,1,1,1,1,2,1,2,1,1,1,1,1,2,2,2,2,2)
kendall.global(mite.hel, group=group, nperm=49)
kendall.post(mite.hel, group=group, mult="holm", nperm=49)

# NOTE: 'nperm' argument usually needs to be larger than 49.
# It was set to this low value for demonstration purposes.
}

\keyword{ multivariate }
\keyword{ nonparametric }

\name{anova.cca}
\alias{anova.cca}
%\alias{anova.ccanull}
%\alias{anova.ccabyaxis}
%\alias{anova.ccabyterm}
%\alias{anova.ccabymargin}
%\alias{anova.prc}
\alias{permutest}
%\alias{permutest.default}
\alias{permutest.cca}

\title{Permutation Test for Constrained Correspondence Analysis,
  Redundancy Analysis and Constrained Analysis of Principal Coordinates }

\description{
  The function performs an ANOVA like permutation test for Constrained
  Correspondence Analysis (\code{\link{cca}}), Redundancy Analysis
  (\code{\link{rda}}) or distance-based Redundancy Analysis (dbRDA,
  \code{\link{dbrda}}) to assess the significance of constraints.
}

\usage{
\method{anova}{cca}(object, ..., permutations = how(nperm=999),
     by = NULL, model = c("reduced", "direct", "full"),
     parallel = getOption("mc.cores"), strata = NULL,
     cutoff = 1, scope = NULL)
\method{permutest}{cca}(x, permutations = how(nperm = 99),
     model = c("reduced", "direct", "full"), by = NULL, first = FALSE,
     strata = NULL, parallel = getOption("mc.cores"), ...)
}

\arguments{

  \item{object}{One or several result objects from \code{\link{cca}},
    \code{\link{rda}}, \code{\link{dbrda}} or \code{\link{capscale}}. If
    there are several result objects, they are compared against each
    other in the order they were supplied. For a single object, a test
    specified in \code{by} or an overall test is given.}

  \item{x}{A single ordination result object.}

  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}

  \item{by}{Setting \code{by = "axis"} will assess significance for
    each constrained axis, and setting \code{by = "terms"} will assess
    significance for each term (sequentially from first to last), and
    setting \code{by = "margin"} will assess the marginal effects of
    the terms (each marginal term analysed in a model with all other
    variables), and \code{by = "onedf"} will assess sequentially
    one-degree-of-freedom contrasts of split factors.}

  \item{model}{Permutation model: \code{model="direct"} permutes
    community data, \code{model="reduced"} permutes residuals
    of the community data after Conditions (partial model),
    \code{model = "full"} permutes residuals after Conditions and
    Constraints.}

  \item{parallel}{Use parallel processing with the given number of
    cores.}

  \item{strata}{An integer vector or factor specifying the strata for
    permutation. If supplied, observations are permuted only within
    the specified strata. It is an error to use this when
    \code{permutations} is a matrix, or a \code{\link[permute]{how}}
    defines \code{blocks}. This is a legacy argument that will be
    deprecated in the future: use
    \code{permutations = how(\dots, blocks)} instead. }

  \item{cutoff}{Only effective with \code{by="axis"} where stops
    permutations after an axis equals or exceeds the \code{cutoff}
    \eqn{p}-value.}

  \item{scope}{Only effective with \code{by="margin"} where it can be
    used to select the marginal terms for testing. The default is to
    test all marginal terms in \code{\link{drop.scope}}.}

  \item{first}{Analyse only significance of the first axis.}

  \item{\dots}{Parameters passed to other functions.  \code{anova.cca}
    passes all arguments to \code{permutest.cca}. In \code{anova} with
    \code{by = "axis"} you can use argument \code{cutoff} (defaults
    \code{1}) which stops permutations after exceeding the given
    level. }
}

\details{

  Functions \code{anova.cca} and \code{permutest.cca} implement ANOVA
  like permutation tests for the joint effect of constraints in
  \code{\link{cca}}, \code{\link{rda}}, \code{\link{dbrda}} or
  \code{\link{capscale}}. Function \code{anova} is intended as a more
  user-friendly alternative to \code{permutest} (that is the real
  workhorse).

  Function \code{anova} can analyse a sequence of constrained
  ordination models. The analysis is based on the differences in
  residual deviance in permutations of nested models.

  The default test is for the sum of all constrained eigenvalues.
  Setting \code{first = TRUE} will perform a test for the first
  constrained eigenvalue.  Argument \code{first} can be set either in
  \code{anova.cca} or in \code{permutest.cca}.  It is also possible to
  perform significance tests for each axis or for each term
  (constraining variable) using argument \code{by} in \code{anova.cca}.
  Setting \code{by = "axis"} will perform separate significance tests
  for each constrained axis.  All previous constrained axes will be used
  as conditions (\dQuote{partialled out}) and a test for the first
  constrained eigenvalues is performed (Legendre et al. 2011).  You can
  stop permutation tests after exceeding a given significance level with
  argument \code{cutoff} to speed up calculations in large
  models. Setting \code{by = "terms"} will perform separate significance
  test for each term (constraining variable). The terms are assessed
  sequentially from first to last, and the order of the terms will
  influence their significances. Setting \code{by = "onedf"} will
  perform a similar sequential test for one-degree-of-freedom effects,
  where multi-level factors are split in their contrasts. Setting
  \code{by = "margin"} will perform separate significance test for each
  marginal term in a model with all other terms. The marginal test also
  accepts a \code{scope} argument for the \code{\link{drop.scope}} which
  can be a character vector of term labels that are analysed, or a
  fitted model of lower scope.  The marginal effects are also known as
  \dQuote{Type III} effects, but the current function only evaluates
  marginal terms. It will, for instance, ignore main effects that are
  included in interaction terms. In calculating pseudo-\eqn{F}, all
  terms are compared to the same residual of the full model.

  Community data are permuted with choice \code{model="direct"}, and
  residuals after partial CCA/ RDA/ dbRDA with choice
  \code{model="reduced"} (default).  If there is no partial CCA/ RDA/
  dbRDA stage, \code{model="reduced"} simply permutes the data and is
  equivalent to \code{model="direct"}.  The test statistic is
  \dQuote{pseudo-\eqn{F}}, which is the ratio of constrained and
  unconstrained total Inertia (Chi-squares, variances or something
  similar), each divided by their respective degrees of freedom.  If
  there are no conditions (\dQuote{partial} terms), the sum of all
  eigenvalues remains constant, so that pseudo-\eqn{F} and eigenvalues
  would give equal results.  In partial CCA/ RDA/ dbRDA, the effect of
  conditioning variables (\dQuote{covariables}) is removed before
  permutation, and the total Chi-square is not fixed, and test based on
  pseudo-\eqn{F} would differ from the test based on plain
  eigenvalues.

}

\value{
  The function \code{anova.cca} calls \code{permutest.cca} and fills an
  \code{\link{anova}} table. Additional attributes are
  \code{Random.seed} (the random seeds used),
  \code{control} (the permutation design, see \link[permute]{how}) and
  \code{F.perm} (the permuted test statistics).
}

\references{
  Legendre, P. and Legendre, L. (2012). \emph{Numerical Ecology}. 3rd
  English ed. Elsevier.

  Legendre, P., Oksanen, J. and ter Braak, C.J.F. (2011). Testing the
  significance of canonical axes in redundancy analysis.
  \emph{Methods in Ecology and Evolution} 2, 269--277.
}
\author{Jari Oksanen}

\seealso{\code{\link{anova.cca}}, \code{\link{cca}},
  \code{\link{rda}}, \code{\link{dbrda}} to get something to
  analyse. Function \code{\link{drop1.cca}} calls \code{anova.cca}
  with \code{by = "margin"}, and \code{\link{add1.cca}} an analysis
  for single terms additions, which can be used in automatic or
  semiautomatic model building (see \code{\link{deviance.cca}}). }

\examples{
data(dune, dune.env)
mod <- cca(dune ~ Moisture + Management, dune.env)
## overall test
anova(mod)
## tests for individual terms
anova(mod, by="term")
anova(mod, by="margin")
## sequential test for contrasts
anova(mod, by = "onedf")
## test for adding all environmental variables
anova(mod, cca(dune ~ ., dune.env))
}
\keyword{ multivariate }
\keyword{ htest }

\name{spantree}
\alias{spantree}
\alias{cophenetic.spantree}
\alias{as.hclust.spantree}
\alias{plot.spantree}
\alias{lines.spantree}
\alias{spandepth}

\title{Minimum Spanning Tree}
\description{
  Function \code{spantree} finds a minimum spanning tree
  connecting all points, but disregarding dissimilarities that are at or
  above the threshold or \code{NA}.
}
\usage{
spantree(d, toolong = 0)
\method{as.hclust}{spantree}(x, ...)
\method{cophenetic}{spantree}(x)
spandepth(x)
\method{plot}{spantree}(x, ord, cex = 0.7, type = "p", labels, dlim,
     FUN = sammon,  ...)
\method{lines}{spantree}(x, ord, display="sites", col = 1, ...)
}

\arguments{
  \item{d}{Dissimilarity data inheriting from class \code{dist} or
    a an object, such as a matrix, that can be converted to a
    dissimilarity matrix. Functions \code{\link{vegdist}} and
    \code{\link{dist}} are some functions producing suitable
    dissimilarity data.}
  \item{toolong}{ Shortest dissimilarity regarded as \code{NA}.
    The function uses a fuzz factor, so
    that dissimilarities close to the limit will be made \code{NA}, too.
    If \code{toolong = 0} (or negative), no dissimilarity is regarded
    as too long.
  }
  \item{x}{A \code{spantree} result object.}
  \item{ord}{An ordination configuration, or an ordination result known
    by \code{\link{scores}}.}
  \item{cex}{Character expansion factor.}
  \item{type}{Observations are plotted as points with
    \code{type="p"} or \code{type="b"}, or as text label with
    \code{type="t"}. The tree (lines) will always be plotted.}
  \item{labels}{Text used with \code{type="t"} or node names if this is
    missing.}
  \item{dlim}{A ceiling value used to highest \code{cophenetic} dissimilarity.}
  \item{FUN}{Ordination function to find the configuration from
    cophenetic dissimilarities. If the supplied \code{FUN} does not work,
    supply ordination result as argument \code{ord}. }
  \item{display}{Type of \code{\link{scores}} used for \code{ord}.}
  \item{col}{Colour of line segments. This can be a vector which is
    recycled for points, and the line colour will be a mixture of two
    joined points.}
\item{\dots}{Other parameters passed to functions.}
}
\details{

  Function \code{spantree} finds a minimum spanning tree for
  dissimilarities (there may be several minimum spanning trees, but the
  function finds only one). Dissimilarities at or above the threshold
  \code{toolong} and \code{NA}s are disregarded, and the spanning tree
  is found through other dissimilarities. If the data are disconnected,
  the function will return a disconnected tree (or a forest), and the
  corresponding link is \code{NA}. Connected subtrees can be identified
  using \code{\link{distconnected}}.

  Minimum spanning tree is closely related to single linkage
  clustering, a.k.a. nearest neighbour clustering, and in genetics as
  neighbour joining tree available in \code{\link{hclust}} and
  \code{\link[cluster]{agnes}} functions. The most important practical
  difference is that minimum spanning tree has no concept of cluster
  membership, but always joins individual points to each other. Function
  \code{as.hclust} can change the \code{spantree} result into a
  corresponding \code{\link{hclust}} object.

  Function \code{cophenetic} finds distances between all points along
  the tree segments. Function \code{spandepth} returns the depth of
  each node. The nodes of a tree are either leaves (with one link) or
  internal nodes (more than one link). The leaves are recursively
  removed from the tree, and the depth is the layer at with the leaf
  was removed. In disconnected \code{spantree} object (in a forest)
  each tree is analysed separately and disconnected nodes not in any
  tree have depth zero.

  Function \code{plot} displays the tree over a
  supplied ordination configuration, and \code{lines} adds a spanning
  tree to an ordination graph. If configuration is not supplied for \code{plot},
  the function ordinates the cophenetic dissimilarities of the
  spanning tree and overlays the tree on this result. The default
  ordination function is \code{\link[MASS]{sammon}} (package \pkg{MASS}),
  because Sammon scaling emphasizes structure in the neighbourhood of
  nodes and may be able to beautifully represent the tree (you may need
  to set \code{dlim}, and sometimes the results will remain
  twisted). These ordination methods do not work with disconnected
  trees, but you must supply the ordination configuration. Function
  \code{lines} will overlay the tree in an existing plot.

  Function \code{spantree} uses Prim's method
  implemented as priority-first search for dense graphs (Sedgewick
  1990). Function \code{cophenetic} uses function
  \code{\link{stepacross}} with option \code{path = "extended"}. The
  \code{spantree} is very fast, but \code{cophenetic} is slow in very
  large data sets.
}
\value{
  Function \code{spantree}
  returns an object of class \code{spantree} which is a
  list with two vectors, each of length \eqn{n-1}. The
  number of links in a tree is one less the number of observations, and
  the first item is omitted. The items are
  \item{kid }{The child node of the parent, starting from parent number
    two. If there is no link from the parent, value will be \code{NA}
    and tree is disconnected at the node.}
  \item{dist }{Corresponding distance. If \code{kid = NA}, then
    \code{dist = 0}.}
  \item{labels }{Names of nodes as found from the input dissimilarities.}
  \item{call}{The function call.}
}
\references{
 Sedgewick, R. (1990). \emph{Algorithms in C}. Addison Wesley.
}
\author{ Jari Oksanen }
\note{
  In principle, minimum spanning tree is equivalent to single linkage
  clustering that can be performed using \code{\link{hclust}} or
  \code{\link[cluster]{agnes}}. However, these functions combine
  clusters to each other and the information of the actually connected points
  (the \dQuote{single link}) cannot be recovered from the result. The
  graphical output of a single linkage clustering plotted with
  \code{\link{ordicluster}} will look very different from an equivalent
  spanning tree plotted with \code{lines.spantree}.
}


\seealso{\code{\link{vegdist}} or \code{\link{dist}} for getting
  dissimilarities,  and \code{\link{hclust}} or
    \code{\link[cluster]{agnes}} for single linkage clustering.
}
\examples{
data(dune)
dis <- vegdist(dune)
tr <- spantree(dis)
## Add tree to a metric scaling
plot(tr, cmdscale(dis), type = "t")
## Find a configuration to display the tree neatly
plot(tr, type = "t")
## Depths of nodes
depths <- spandepth(tr)
plot(tr, type = "t", label = depths)
## Plot as a dendrogram
cl <- as.hclust(tr)
plot(cl)
## cut hclust tree to classes and show in colours in spantree
plot(tr, col = cutree(cl, 5), pch=16)
}
\keyword{ multivariate}

\name{RsquareAdj}
\alias{RsquareAdj}
\alias{RsquareAdj.default}
\alias{RsquareAdj.rda}
\alias{RsquareAdj.cca}
\alias{RsquareAdj.lm}
\alias{RsquareAdj.glm}

\Rdversion{1.1}

\title{
Adjusted R-square
}
\description{
  The functions finds the adjusted R-square.
}
\usage{
\method{RsquareAdj}{default}(x, n, m, ...)
\method{RsquareAdj}{rda}(x, ...)
\method{RsquareAdj}{cca}(x, permutations = 1000, ...)
}

\arguments{

  \item{x}{ Unadjusted R-squared or an object from which the terms for
  evaluation or adjusted R-squared can be found.}
  
  \item{n, m}{Number of observations and number of degrees of freedom
  in the fitted model.}
  
  \item{permutations}{Number of permutations to use when computing the adjusted 
  R-squared for a cca. The permutations can be calculated in parallel by
  specifying the number of cores which is passed to \code{\link{permutest}}}

  \item{\dots}{ Other arguments (ignored) except in the case of cca in 
  which these arguments are passed to \code{\link{permutest}}.} 
}

\details{ The default method finds the adjusted \eqn{R^2}{R-squared}
  from the unadjusted \eqn{R^2}{R-squared}, number of observations, and
  number of degrees of freedom in the fitted model. The specific methods
  find this information from the fitted result object. There are
  specific methods for \code{\link{rda}}, \code{\link{cca}},
  \code{\link{lm}} and \code{\link{glm}}. Adjusted, or even unadjusted,
  \eqn{R^2}{R-squared} may not be available in some cases, and then the
  functions will return \code{NA}. There is no adjusted
  \eqn{R^2}{R-squared} in partial ordination, and \eqn{R^2}{R-squared}
  values are available only for \code{\link{gaussian}} models in
  \code{\link{glm}}.

  The adjusted, \eqn{R^2}{R-squared} of \code{cca} is computed using a
  permutation approach developed by Peres-Neto et al. (2006). By
  default 1000 permutations are used.
  
}

\value{ The functions return a list of items \code{r.squared} and
\code{adj.r.squared}.  
}

\references{
  Legendre, P., Oksanen, J. and ter Braak, C.J.F. (2011). Testing the
  significance of canonical axes in redundancy analysis. 
  \emph{Methods in Ecology and Evolution} 2, 269--277.
  
  Peres-Neto, P., P. Legendre, S. Dray and D. Borcard. 2006. Variation
  partitioning of species data matrices: estimation and comparison of
  fractions. \emph{Ecology} 87, 2614--2625.
}

\seealso{
  \code{\link{varpart}} uses \code{RsquareAdj}.
}
\examples{
data(mite)
data(mite.env)
## rda
m <- rda(decostand(mite, "hell") ~  ., mite.env)
RsquareAdj(m)
## cca
m <- cca(decostand(mite, "hell") ~  ., mite.env)
RsquareAdj(m)
## default method
RsquareAdj(0.8, 20, 5)
}
\keyword{ univar }
\keyword{ multivariate }

\name{cascadeKM}
\alias{cascadeKM}
\alias{cIndexKM}
\alias{plot.cascadeKM}
\alias{orderingKM}
\alias{pregraphKM}

\title{K-means partitioning using a range of values of K}
\description{
 This function is a wrapper for the \code{kmeans} function. It creates
 several partitions forming a cascade from a small to a large number of
 groups. 
}

\usage{
cascadeKM(data, inf.gr, sup.gr, iter = 100, criterion = "calinski")

cIndexKM(y, x, index = "all")

\method{plot}{cascadeKM}(x, min.g, max.g, grpmts.plot = TRUE, 
     sortg = FALSE, gridcol = NA, ...) 
}

\arguments{
  \item{ data }{ The data matrix. The objects (samples) are the rows.}
  \item{ inf.gr }{ The number of groups for the partition with the 
    smallest number of groups of the cascade (min).}
  \item{ sup.gr }{ The number of groups for the partition with the largest 	
    number of groups of the cascade (max).}
  \item{ iter }{ The number of random starting configurations for each value
    of \eqn{K}.}
  \item{ criterion }{ The criterion that will be used to select the best
    partition. The default value is \code{"calinski"}, which refers to
    the Calinski-Harabasz (1974) criterion. The simple structure index
    (\code{"ssi"}) is also available. Other indices are available in
    function \code{\link[cclust]{clustIndex}} (package \pkg{cclust}). In
    our experience, the two indices that work best and are most likely
    to return their maximum value at or near the optimal number of
    clusters are \code{"calinski"} and \code{"ssi"}. }
  \item{y}{Object of class \code{"kmeans"} returned by a clustering algorithm
    such as \code{\link{kmeans}}}
  \item{x}{Data matrix where columns correspond to variables and rows to
    observations, or the plotting object in \code{plot}}
  \item{index}{The available indices are: \code{"calinski"} and \code{"ssi"}. 
    Type \code{"all"} to obtain both indices. 
    Abbreviations of these names are also accepted.}
  \item{min.g, max.g}{The minimum and maximum numbers of groups to be
    displayed.}
  \item{grpmts.plot}{Show the plot (\code{TRUE} or \code{FALSE}).}
  \item{sortg}{Sort the objects as a function of their group membership
    to produce a more easily interpretable graph. See Details. The
    original object names are kept; they are used as labels in the
    output table \code{x}, although not in the graph.  If there were no
    row names, sequential row numbers are used to keep track of the
    original order of the objects.}
  \item{gridcol}{The colour of the grid lines in the plots. \code{NA},
    which is the default value, removes the grid lines.}
  \item{\dots}{Other parameters to the functions (ignored).}
    
}
\details{
  The function creates several partitions forming a cascade from a small
  to a large number of groups formed by \code{\link{kmeans}}.  Most
  of the work is performed by function \code{cIndex} which is based on the
  \code{\link[cclust]{clustIndex}} function (package \pkg{cclust}). 
  Some of the criteria were removed from this version because computation 
  errors were generated when only one object was found in a group.
  
  The default value is \code{"calinski"}, which refers to the well-known
  Calinski-Harabasz (1974) criterion. The other available index is the
  simple structure index \code{"ssi"} (Dolnicar et al. 1999).
  In the case of groups of equal
  sizes, \code{"calinski"} is generally a good criterion to indicate the
  correct number of groups. Users should not take its indications
  literally when the groups are not equal in size. Type \code{"all"} to
  obtain  both indices. The indices are defined as: 
  \describe{
    \item{calinski:}{
    \eqn{(SSB/(K-1))/(SSW/(n-K))}, where \eqn{n} is the
    number of data points and \eqn{K} is the number of clusters.
    \eqn{SSW} is the sum of squares within the clusters while
    \eqn{SSB} is the sum of squares among the clusters. This index
    is simply an \eqn{F} (ANOVA) statistic.}
 
    \item{ssi:}{
    the \dQuote{Simple Structure Index} multiplicatively combines
    several elements which influence the interpretability of a
    partitioning solution. The best partition is indicated by the
    highest SSI value.}
  }

  In a simulation study, Milligan and Cooper (1985) found
  that the Calinski-Harabasz criterion recovered the correct number of
  groups the most often. We recommend this criterion because, if the
  groups are of equal sizes, the maximum value of \code{"calinski"}
  usually indicates the correct number of groups. Another available
  index is the simple structure index \code{"ssi"}. Users should not
  take the indications of these indices literally when the groups are
  not equal in size and explore the groups corresponding to other values
  of \eqn{K}.
  
  Function \code{cascadeKM} has a \code{plot} method.  Two plots are
  produced. The graph on the left has the objects in 
  abscissa and the number of groups in ordinate. The groups are
  represented by colours. The graph on the right shows the values of the
  criterion (\code{"calinski"} or \code{"ssi"}) for determining the best
  partition. The highest value of the criterion is marked in red. Points
  marked in orange, if any, indicate partitions producing an increase in
  the criterion value as the number of groups increases; they may
  represent other interesting partitions.
  
  If \code{sortg=TRUE}, the objects are reordered by the following
  procedure: (1) a simple matching distance matrix is computed among the
  objects, based on the table of K-means assignments to groups, from
  \eqn{K} = \code{min.g} to \eqn{K} = \code{max.g}. (2) A principal
  coordinate analysis (PCoA, Gower 1966) is computed on the centred
  distance matrix. (3) The first principal coordinate is used as the new
  order of the objects in the graph. A simplified algorithm is used to
  compute the first principal coordinate only, using the iterative
  algorithm described in Legendre & Legendre (2012). The
  full distance matrix among objects is never computed; this avoids
  the problem of storing it when the number of objects is
  large. Distance values are computed as they are needed by the
  algorithm.
}

\value{ Function \code{cascadeKM} returns an object of class
  \code{cascadeKM} with items:
  \item{ partition }{ Table with the partitions found for different numbers 
    of groups \eqn{K}, from \eqn{K} = \code{inf.gr} to \eqn{K} =
    \code{sup.gr}. } 
  \item{ results }{ Values of the criterion to select the best
    partition. } 
  \item{ criterion }{ The name of the criterion used. }
  \item{ size }{ The number of objects found in each group, for all 
    partitions (columns). }

  Function \code{cIndex} returns a vector with the index values. The
  maximum value of these indices is supposed to indicate the best
  partition. These indices work best with groups of equal sizes. When
  the groups are not of equal sizes, one should not put too much faith
  in the maximum of these indices, and also explore the groups
  corresponding to other values of \eqn{K}.
}
\references{

  Calinski, T. and J. Harabasz. 1974. A dendrite method for cluster
  analysis. \emph{Commun. Stat.} \strong{3}: 1--27.

  Dolnicar, S., K. Grabler and J. A. Mazanec. 1999.  A tale of three
  cities: perceptual charting for analyzing destination images. Pp.
  39-62 in: Woodside, A. et al. [eds.] \emph{Consumer psychology of
  tourism, hospitality and leisure}. CAB International, New York.

  
  Gower, J. C. 1966. Some distance properties of latent root and vector
  methods used in multivariate analysis. \emph{Biometrika} \strong{53}:
  325--338.
  
  Legendre, P. & L. Legendre. 2012. \emph{Numerical ecology}, 3rd
  English edition. Elsevier Science BV, Amsterdam.
  
  Milligan, G. W. & M. C. Cooper. 1985. An examination of procedures for
  determining the number of clusters in a data set. \emph{Psychometrika}
  \strong{50}: 159--179.

  Weingessel, A., Dimitriadou, A. and Dolnicar, S. 2002. An examination
  of indexes for determining the number of clusters in binary data
  sets. \emph{Psychometrika} \strong{67}: 137--160.
}

\author{ Marie-Helene Ouellette
  \email{Marie-Helene.Ouellette@UMontreal.ca}, Sebastien Durand
  \email{Sebastien.Durand@UMontreal.ca} and Pierre Legendre
  \email{Pierre.Legendre@UMontreal.ca}. Edited for \pkg{vegan} by Jari
  Oksanen.
}

\seealso{\code{\link{kmeans}}, \code{\link[cclust]{clustIndex}}.}

\examples{
 # Partitioning a (10 x 10) data matrix of random numbers
 mat <- matrix(runif(100),10,10)
 res <- cascadeKM(mat, 2, 5, iter = 25, criterion = 'calinski') 
 toto <- plot(res)
 
 # Partitioning an autocorrelated time series
 vec <- sort(matrix(runif(30),30,1))
 res <- cascadeKM(vec, 2, 5, iter = 25, criterion = 'calinski')
 toto <- plot(res)
 
 # Partitioning a large autocorrelated time series
 # Note that we remove the grid lines
 vec <- sort(matrix(runif(1000),1000,1))
 res <- cascadeKM(vec, 2, 7, iter = 10, criterion = 'calinski')
 toto <- plot(res, gridcol=NA)
 
}
\keyword{ cluster }

\name{reorder.hclust}
\alias{reorder.hclust}
\alias{rev.hclust}
\alias{cutreeord}
\alias{scores.hclust}

\title{
Reorder a Hierarchical Clustering Tree
}

\description{

  Function takes a hierarchical clustering tree from
  \code{\link{hclust}} and a vector of values and reorders the
  clustering tree in the order of the supplied vector, maintaining the
  constraints on the tree. This is a method of generic function
  \code{\link{reorder}} and an alternative to reordering a
  \code{"dendrogram"} object with \code{\link{reorder.dendrogram}}

}

\usage{
\method{reorder}{hclust}(x, wts, 
   agglo.FUN = c("mean", "min", "max", "sum", "uwmean"), ...)
\method{rev}{hclust}(x)
\method{scores}{hclust}(x, display = "internal", ...)
cutreeord(tree, k = NULL, h = NULL)
}

\arguments{
  \item{x, tree}{
    hierarchical clustering from \code{\link{hclust}}.
}
  \item{wts}{
    numeric vector for reordering.
}
  \item{agglo.FUN}{
    a function for weights agglomeration, see below.
}
  \item{display}{
  return \code{"internal"} nodes or \code{"terminal"} nodes (also
  called \code{"leaves"}).
}
\item{k, h}{
  scalars or vectors giving the numbers of desired groups or the heights
  where the tree should be cut (passed to function
  \code{\link{cutree}}).
}
  \item{\dots}{
    additional arguments (ignored).
}
}

\details{
  
  Dendrograms can be ordered in many ways. The \code{reorder} function
  reorders an \code{\link{hclust}} tree and provides an alternative to
  \code{\link{reorder.dendrogram}} which can reorder a
  \code{\link{dendrogram}}. The current function will also work
  differently when the \code{agglo.FUN} is \code{"mean"}: the
  \code{\link{reorder.dendrogram}} will always take the direct mean of
  member groups ignoring their sizes, but this function will used
  \code{\link{weighted.mean}} weighted by group sizes, so that the
  group mean is always the mean of member leaves (terminal nodes). If
  you want to ignore group sizes, you can use unweighted mean with
  \code{"uwmean"}. 

  The function accepts only a limited list of \code{agglo.FUN}
  functions for assessing the value of \code{wts} for groups. The
  ordering is always ascending, but the order of leaves can be
  reversed with \code{rev}.

  Function \code{scores} finds the coordinates of nodes as a two-column
  matrix. For terminal nodes (leaves) this the value at which the item
  is merged to the tree, and the labels can still \code{hang} below this
  level (see \code{\link{plot.hclust}}).

  Function \code{cutreeord} cuts a tree to groups numbered from left to
  right in the tree. It is based on the standard function
  \code{\link{cutree}} which numbers the groups in the order they appear
  in the input data instead of the order in the tree.
}

\value{
  Reordered \code{\link{hclust}} result object with added item
  \code{value} that gives the value of the statistic at each merge
  level. 
}

\author{
  Jari Oksanen
}
\note{
  These functions should really be in base \R.
}


\seealso{
  \code{\link{hclust}} for getting clustering trees,
  \code{\link{as.hclust.spantree}} to change a \pkg{vegan} minimum
  spanning tree to an \code{\link{hclust}} object, and
  \code{\link{dendrogram}} and \code{\link{reorder.dendrogram}} for an
  alternative implementation.
}
\examples{
## reorder by water content of soil
data(mite, mite.env)
hc <- hclust(vegdist(wisconsin(sqrt(mite))))
ohc <- with(mite.env, reorder(hc, WatrCont))
plot(hc)
plot(ohc)

## label leaves by the observed value, and each branching point
## (internal node) by the cluster mean
with(mite.env, plot(ohc, labels=round(WatrCont), cex=0.7))
ordilabel(scores(ohc), label=round(ohc$value), cex=0.7)

## Slightly different from reordered 'dendrogram' which ignores group
## sizes in assessing means.
den <- as.dendrogram(hc)
den <- with(mite.env, reorder(den, WatrCont, agglo.FUN = mean))
plot(den)
}

\keyword{multivariate}

\name{CCorA}
\alias{CCorA}
\alias{biplot.CCorA}
\concept{ordination}

\title{Canonical Correlation Analysis}

\description{Canonical correlation analysis, following Brian McArdle's
unpublished graduate course notes, plus improvements to allow the
calculations in the case of very sparse and collinear matrices, and 
permutation test of Pillai's trace statistic. }

\usage{
CCorA(Y, X, stand.Y=FALSE, stand.X=FALSE, permutations = 0, ...)

\method{biplot}{CCorA}(x, plot.type="ov", xlabs, plot.axes = 1:2, int=0.5, 
   col.Y="red", col.X="blue", cex=c(0.7,0.9), ...)
}

\arguments{
  \item{Y}{ Left matrix (object class: \code{matrix} or \code{data.frame}). }
  \item{X}{ Right matrix (object class: \code{matrix} or \code{data.frame}). }
  \item{stand.Y}{ Logical; should \code{Y} be standardized? }
  \item{stand.X}{ Logical; should \code{X} be standardized? }
  
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}
  
  \item{x}{\code{CCoaR} result object.}
  \item{plot.type}{ A character string indicating which of the following 
    plots should be produced: \code{"objects"}, \code{"variables"}, \code{"ov"} 
    (separate graphs for objects and variables), or \code{"biplots"}. Any 
    unambiguous subset containing the first letters of these names can be used 
    instead of the full names. }
  \item{xlabs}{ Row labels. The default is to use row names, \code{NULL}
    uses row numbers instead, and \code{NA} suppresses plotting row names
    completely.}
  \item{plot.axes}{ A vector with 2 values containing the order numbers of 
    the canonical axes to be plotted. Default: first two axes. }
  \item{int}{ Radius of the inner circles plotted as visual references in 
    the plots of the variables. Default: \code{int=0.5}. With \code{int=0}, 
    no inner circle is plotted. }
  \item{col.Y}{ Color used for objects and variables in the first data 
    table (Y) plots. In biplots, the objects are in black. }
  \item{col.X}{ Color used for objects and variables in the second data 
    table (X) plots. }
  \item{cex}{ A vector with 2 values containing the size reduction factors 
    for the object and variable names, respectively, in the plots. 
    Default values: \code{cex=c(0.7,0.9)}. }
  \item{\dots}{ Other arguments passed to these functions. The function 
    \code{biplot.CCorA} passes graphical arguments to \code{\link{biplot}} 
    and \code{\link{biplot.default}}. \code{CCorA} currently ignores extra 
    arguments. } 
}

\details{
  Canonical correlation analysis (Hotelling 1936) seeks linear
  combinations of the variables of \code{Y} that are maximally
  correlated to linear combinations of the variables of \code{X}. The
  analysis estimates the relationships and displays them in graphs.
  Pillai's trace statistic is computed and tested parametrically (F-test);
  a permutation test is also available.

  Algorithmic note -- 
  The blunt approach would be to read the two matrices, compute the
  covariance matrices, then the matrix
  \code{S12 \%*\% inv(S22) \%*\% t(S12) \%*\% inv(S11)}.
  Its trace is Pillai's trace statistic. 
  This approach may fail, however, when there is heavy multicollinearity
  in very sparse data matrices. The safe approach is to replace all data
  matrices by their PCA object scores.

The function can produce different types of plots depending on the option 
chosen: 
\code{"objects"} produces two plots of the objects, one in the space 
of Y, the second in the space of X; 
\code{"variables"} produces two plots of the variables, one of the variables 
of Y in the space of Y, the second of the variables of X in the space of X; 
\code{"ov"} produces four plots, two of the objects and two of the variables; 
\code{"biplots"} produces two biplots, one for the first matrix (Y) and 
one for second matrix (X) solutions. For biplots, the function passes all arguments 
to \code{\link{biplot.default}}; consult its help page for configuring biplots.
}

\value{
  Function \code{CCorA} returns a list containing the following elements:
  \item{ Pillai }{ Pillai's trace statistic = sum of the canonical
    eigenvalues. } 
  \item{ Eigenvalues }{ Canonical eigenvalues. They are the squares of the
    canonical correlations. }
  \item{ CanCorr }{ Canonical correlations. }
  \item{ Mat.ranks }{ Ranks of matrices \code{Y} and \code{X}. }
  \item{ RDA.Rsquares }{ Bimultivariate redundancy coefficients
    (R-squares) of RDAs of Y|X and X|Y. } 
  \item{ RDA.adj.Rsq }{ \code{RDA.Rsquares} adjusted for \code{n} and the number 
    of explanatory variables. }
  \item{ nperm }{ Number of permutations. }
  \item{ p.Pillai }{ Parametric probability value associated with Pillai's trace. }
  \item{ p.perm }{ Permutational probability associated with Pillai's trace. }
  \item{ Cy }{ Object scores in Y biplot. }
  \item{ Cx }{ Object scores in X biplot. }
  \item{ corr.Y.Cy }{ Scores of Y variables in Y biplot, computed as cor(Y,Cy). }
  \item{ corr.X.Cx }{ Scores of X variables in X biplot, computed as cor(X,Cx). }
  \item{ corr.Y.Cx }{ cor(Y,Cy) available for plotting variables Y in space of X manually. }
  \item{ corr.X.Cy }{ cor(X,Cx) available for plotting variables X in space of Y manually. }
  \item{control}{A list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}.}
  \item{ call }{ Call to the CCorA function. }
}

\references{ 
  Hotelling, H. 1936. Relations between two sets of
  variates. \emph{Biometrika} \strong{28}: 321-377.
  
  Legendre, P. 2005. Species associations: the Kendall coefficient of 
  concordance revisited. \emph{Journal of Agricultural, Biological, and 
  Environmental Statistics} \strong{10}: 226-245.
}

\author{ Pierre Legendre, Departement de Sciences Biologiques,
  Universite de Montreal. Implemented in \pkg{vegan} with the help of
  Jari Oksanen. }

\examples{
# Example using two mite groups. The mite data are available in vegan
data(mite)
# Two mite species associations (Legendre 2005, Fig. 4)
group.1 <- c(1,2,4:8,10:15,17,19:22,24,26:30)
group.2 <- c(3,9,16,18,23,25,31:35)
# Separate Hellinger transformations of the two groups of species 
mite.hel.1 <- decostand(mite[,group.1], "hel")
mite.hel.2 <- decostand(mite[,group.2], "hel")
rownames(mite.hel.1) = paste("S",1:nrow(mite),sep="")
rownames(mite.hel.2) = paste("S",1:nrow(mite),sep="")
out <- CCorA(mite.hel.1, mite.hel.2)
out
biplot(out, "ob")                 # Two plots of objects
biplot(out, "v", cex=c(0.7,0.6))  # Two plots of variables
biplot(out, "ov", cex=c(0.7,0.6)) # Four plots (2 for objects, 2 for variables)
biplot(out, "b", cex=c(0.7,0.6))  # Two biplots
biplot(out, xlabs = NA, plot.axes = c(3,5))    # Plot axes 3, 5. No object names
biplot(out, plot.type="biplots", xlabs = NULL) # Replace object names by numbers

# Example using random numbers. No significant relationship is expected
mat1 <- matrix(rnorm(60),20,3)
mat2 <- matrix(rnorm(100),20,5)
out2 = CCorA(mat1, mat2, permutations=99)
out2
biplot(out2, "b")
}

\keyword{ multivariate }

\name{specaccum}
\alias{specaccum}
\alias{print.specaccum}
\alias{summary.specaccum}
\alias{plot.specaccum}
\alias{lines.specaccum}
\alias{boxplot.specaccum}
\alias{fitspecaccum}
\alias{plot.fitspecaccum}
\alias{lines.fitspecaccum}
\alias{predict.specaccum}
\alias{predict.fitspecaccum}
\alias{AIC.fitspecaccum}
\alias{deviance.fitspecaccum}
\alias{logLik.fitspecaccum}
\alias{nobs.fitspecaccum}
\alias{specslope}

\title{Species Accumulation Curves}
\description{
  Function \code{specaccum} finds species accumulation curves or the
  number of species for a certain number of sampled sites or
  individuals. 
}
\usage{
specaccum(comm, method = "exact", permutations = 100,
          conditioned =TRUE, gamma = "jack1",  w = NULL, subset, ...)
\method{plot}{specaccum}(x, add = FALSE, random = FALSE, ci = 2, 
    ci.type = c("bar", "line", "polygon"), col = par("fg"), lty = 1,
    ci.col = col, ci.lty = 1, ci.length = 0, xlab, ylab = x$method, ylim,
    xvar = c("sites", "individuals", "effort"), ...)
\method{boxplot}{specaccum}(x, add = FALSE, ...)
fitspecaccum(object, model, method = "random", ...)
\method{plot}{fitspecaccum}(x, col = par("fg"), lty = 1, xlab = "Sites", 
    ylab = x$method, ...) 
\method{predict}{specaccum}(object, newdata, interpolation = c("linear", "spline"), ...)
\method{predict}{fitspecaccum}(object, newdata, ...)
specslope(object, at)
}

\arguments{
  \item{comm}{Community data set.}
  \item{method}{Species accumulation method (partial match). Method
    \code{"collector"}
    adds sites in the order they happen to be in the data,
    \code{"random"} adds sites in random order, \code{"exact"} finds the
    expected (mean) species richness, \code{"coleman"} finds the
    expected richness following
    Coleman et al. 1982, and \code{"rarefaction"} finds the mean when
    accumulating individuals instead of sites.  }
  \item{permutations}{Number of permutations with \code{method = "random"}.
    Usually an integer giving the number permutations, but can also be a
    list of control values for the permutations as returned by the
    function \code{\link[permute]{how}}, or a permutation matrix where
    each row gives the permuted indices.
  }
  \item{conditioned}{ Estimation of standard deviation is conditional on
    the empirical dataset for the exact SAC}
  \item{gamma}{Method for estimating the total extrapolated number of species in the
    survey area by function \code{\link{specpool}}}
  \item{w}{Weights giving the sampling effort.}
  \item{subset}{logical expression indicating sites (rows) to keep: missing
    values are taken as \code{FALSE}.}
  \item{x}{A \code{specaccum} result object}
  \item{add}{Add to an existing graph.}
  \item{random}{Draw each random simulation separately instead of
    drawing their average and confidence intervals.}
  \item{ci}{Multiplier used to get confidence intervals from standard
    deviation (standard error of the estimate). Value \code{ci = 0}
    suppresses drawing confidence intervals.}
  \item{ci.type}{Type of confidence intervals in the graph: \code{"bar"}
    draws vertical bars, \code{"line"} draws lines, and
    \code{"polygon"} draws a shaded area.}
  \item{col}{Colour for drawing lines.}
  \item{lty}{line type (see \code{\link{par}}).}
  \item{ci.col}{Colour for drawing lines or filling the
    \code{"polygon"}.}
  \item{ci.lty}{Line type for confidence intervals or border of the
    \code{"polygon"}.}
  \item{ci.length}{Length of horizontal bars (in inches) at the end of
    vertical bars with \code{ci.type = "bar"}.}
  \item{xlab,ylab}{Labels for \code{x} (defaults \code{xvar}) and
    \code{y} axis.}
  \item{ylim}{the y limits of the plot.}
  \item{xvar}{Variable used for the horizontal axis:
    \code{"individuals"} can be used only with
    \code{method = "rarefaction"}. }
  \item{object}{Either a community data set or fitted \code{specaccum} model.}
  \item{model}{Nonlinear regression model (\code{\link{nls}}). See Details.}
  
  \item{newdata}{Optional data used in prediction interpreted as
    number of sampling units (sites). If missing, fitted values are
    returned. }
  
  \item{interpolation}{Interpolation method used with \code{newdata}.}

  \item{at}{Number of plots where the slope is evaluated. Can be a
    real number.}

  \item{...}{Other parameters to functions.}
}
\details{

  Species accumulation curves (SAC) are used to compare diversity
  properties of community data sets using different accumulator
  functions. The classic method is \code{"random"} which finds the mean
  SAC and its standard deviation from random permutations of the data,
  or subsampling without replacement (Gotelli & Colwell 2001).  The
  \code{"exact"} method finds the expected SAC using sample-based
  rarefaction method that has been independently developed numerous
  times (Chiarucci et al. 2008) and it is often known as Mao Tau
  estimate (Colwell et al. 2012).  The unconditional standard deviation
  for the exact SAC represents a moment-based estimation that is not
  conditioned on the empirical data set (sd for all samples > 0). The
  unconditional standard deviation is based on an estimation of the
  extrapolated number of species in the survey area (a.k.a. gamma
  diversity), as estimated by function \code{\link{specpool}}. The
  conditional standard deviation that was developed by Jari Oksanen (not
  published, sd=0 for all samples). Method \code{"coleman"} finds the
  expected SAC and its standard deviation following Coleman et
  al. (1982).  All these methods are based on sampling sites without
  replacement. In contrast, the \code{method = "rarefaction"} finds the
  expected species richness and its standard deviation by sampling
  individuals instead of sites.  It achieves this by applying function
  \code{\link{rarefy}} with number of individuals corresponding to
  average number of individuals per site.

  Methods \code{"random"} and \code{"collector"} can take weights
  (\code{w}) that give the sampling effort for each site.  The weights
  \code{w} do not influence the order the sites are accumulated, but
  only the value of the sampling effort so that not all sites are
  equal. The summary results are expressed against sites even when the
  accumulation uses weights (methods \code{"random"},
  \code{"collector"}), or is based on individuals
  (\code{"rarefaction"}).  The actual sampling effort is given as item
  \code{Effort} or \code{Individuals} in the printed result. For
  weighted \code{"random"} method the effort refers to the average
  effort per site, or sum of weights per number of sites. With
  weighted \code{method = "random"}, the averaged species richness is
  found from linear interpolation of single random permutations.
  Therefore at least the first value (and often several first) have
  \code{NA} richness, because these values cannot be interpolated in
  all cases but should be extrapolated.  The \code{plot} function
  defaults to display the results as scaled to sites, but this can be
  changed selecting \code{xvar = "effort"} (weighted methods) or
  \code{xvar = "individuals"} (with \code{method = "rarefaction"}).
 
  The \code{summary} and \code{boxplot} methods are available for
  \code{method = "random"}.

  Function \code{predict} for \code{specaccum} can return the values
  corresponding to \code{newdata}. With \code{method} \code{"exact"},
  \code{"rarefaction"} and \code{"coleman"} the function uses analytic
  equations for interpolated non-integer values, and for other methods
  linear (\code{\link{approx}}) or spline (\code{\link{spline}})
  interpolation. If \code{newdata} is not given, the function returns
  the values corresponding to the data. NB., the fitted values with
  \code{method="rarefaction"} are based on rounded integer counts, but
  \code{predict} can use fractional non-integer counts with
  \code{newdata} and give slightly different results.

  Function \code{fitspecaccum} fits a nonlinear (\code{\link{nls}})
  self-starting species accumulation model. The input \code{object}
  can be a result of \code{specaccum} or a community in data frame. In
  the latter case the function first fits a \code{specaccum} model and
  then proceeds with fitting the nonlinear model. The function can
  apply a limited set of nonlinear regression models suggested for
  species-area relationship (Dengler 2009). All these are
  \code{\link{selfStart}} models. The permissible alternatives are
  \code{"arrhenius"} (\code{\link{SSarrhenius}}), \code{"gleason"}
  (\code{\link{SSgleason}}), \code{"gitay"} (\code{\link{SSgitay}}),
  \code{"lomolino"} (\code{\link{SSlomolino}}) of \pkg{vegan}
  package. In addition the following standard \R models are available:
  \code{"asymp"} (\code{\link{SSasymp}}), \code{"gompertz"}
  (\code{\link{SSgompertz}}), \code{"michaelis-menten"}
  (\code{\link{SSmicmen}}), \code{"logis"} (\code{\link{SSlogis}}),
  \code{"weibull"} (\code{\link{SSweibull}}). See these functions for
  model specification and details. 

  When weights \code{w} were used the fit is based on accumulated
  effort and in \code{model = "rarefaction"} on accumulated number of
  individuals.  The \code{plot} is still based on sites, unless other
  alternative is selected with \code{xvar}.

  Function \code{predict} for \code{fitspecaccum} uses
  \code{\link{predict.nls}}, and you can pass all arguments to that
  function. In addition, \code{fitted}, \code{residuals}, \code{nobs},
  \code{coef}, \code{AIC}, \code{logLik} and \code{deviance} work on
  the result object.

  Function \code{specslope} evaluates the derivative of the species
  accumulation curve at given number of sample plots, and gives the
  rate of increase in the number of species. The function works with
  \code{specaccum} result object when this is based on analytic models
  \code{"exact"}, \code{"rarefaction"} or \code{"coleman"}, and with
  non-linear regression results of \code{fitspecaccum}.

  Nonlinear regression may fail for any reason, and some of the
  \code{fitspecaccum} models are fragile and may not succeed.  
}

\value{ Function \code{specaccum} returns an object of class
  \code{"specaccum"}, and \code{fitspecaccum} a model of class
  \code{"fitspecaccum"} that adds a few items to the
  \code{"specaccum"} (see the end of the list below):

  \item{call }{Function call.}
  \item{method}{Accumulator method.}
  \item{sites}{Number of sites.  For \code{method = "rarefaction"} this
    is the number of sites corresponding to a certain number of
    individuals and generally not an integer, and the average
    number of individuals is also returned in item \code{individuals}.} 
  \item{effort}{Average sum of weights corresponding to the number of
    sites when model was fitted with argument \code{w}}
  \item{richness}{The number of species corresponding to number of
    sites.  With \code{method = "collector"} this is the observed
    richness, for other methods the average or expected richness.}
  \item{sd}{The standard deviation of SAC (or its standard error). This
    is \code{NULL} in \code{method = "collector"}, and it
    is estimated from permutations in \code{method = "random"}, and from
    analytic equations in other methods.}
  \item{perm}{Permutation results with \code{method = "random"} and
    \code{NULL} in other cases. Each column in \code{perm} holds one
    permutation.}
  \item{weights}{Matrix of accumulated weights corresponding to the
    columns of the \code{perm} matrix when model was fitted with
    argument \code{w}.}
  \item{fitted, residuals, coefficients}{Only in \code{fitspecacum}:
     fitted values, residuals and nonlinear model coefficients. For
     \code{method = "random"} these are matrices with a column for
     each random accumulation.}

  \item{models}{Only in \code{fitspecaccum}: list of fitted
    \code{\link{nls}} models (see Examples on accessing these models).} 

}

\references{
  Chiarucci, A., Bacaro, G., Rocchini, D. & Fattorini,
  L. (2008). Discovering and rediscovering the sample-based rarefaction
  formula in the ecological literature. \emph{Commun. Ecol.} 9:
  121--123.

  Coleman, B.D, Mares, M.A., Willis, M.R. & Hsieh,
  Y. (1982). Randomness, area and species richness. \emph{Ecology} 63:
  1121--1133. 

  Colwell, R.K., Chao, A., Gotelli, N.J., Lin, S.Y., Mao, C.X., Chazdon,
  R.L. & Longino, J.T. (2012). Models and estimators linking
  individual-based and sample-based rarefaction, extrapolation and
  comparison of assemblages. \emph{J. Plant Ecol.} 5: 3--21.

  Dengler, J. (2009). Which function describes the species-area
  relationship best? A review and empirical evaluation. 
  \emph{Journal of Biogeography} 36, 728--744.

  Gotelli, N.J. & Colwell, R.K. (2001). Quantifying biodiversity:
  procedures and pitfalls in measurement and comparison of species
  richness. \emph{Ecol. Lett.} 4, 379--391.
}
\author{Roeland Kindt \email{r.kindt@cgiar.org} and Jari Oksanen.}
\note{
  The SAC with \code{method = "exact"} was
  developed by Roeland Kindt, and its standard deviation by Jari
  Oksanen (both are unpublished). The \code{method = "coleman"}
  underestimates the SAC because it does not handle properly sampling
  without replacement.  Further, its standard deviation does not take
  into account species correlations, and is generally too low. }

\seealso{\code{\link{rarefy}} and \code{\link{rrarefy}} are related
  individual based models. Other accumulation models are
  \code{\link{poolaccum}} for extrapolated richness, and
  \code{\link{renyiaccum}} and \code{\link{tsallisaccum}} for
  diversity indices.  Underlying graphical functions are
  \code{\link{boxplot}}, \code{\link{matlines}},
  \code{\link{segments}} and \code{\link{polygon}}.
}
\examples{
data(BCI)
sp1 <- specaccum(BCI)
sp2 <- specaccum(BCI, "random")
sp2
summary(sp2)
plot(sp1, ci.type="poly", col="blue", lwd=2, ci.lty=0, ci.col="lightblue")
boxplot(sp2, col="yellow", add=TRUE, pch="+")
## Fit Lomolino model to the exact accumulation
mod1 <- fitspecaccum(sp1, "lomolino")
coef(mod1)
fitted(mod1)
plot(sp1)
## Add Lomolino model using argument 'add'
plot(mod1, add = TRUE, col=2, lwd=2)
## Fit Arrhenius models to all random accumulations
mods <- fitspecaccum(sp2, "arrh")
plot(mods, col="hotpink")
boxplot(sp2, col = "yellow", border = "blue", lty=1, cex=0.3, add= TRUE)
## Use nls() methods to the list of models
sapply(mods$models, AIC)
}
\keyword{univar}
\keyword{models}

\name{ordistep}
\alias{ordistep}
\alias{ordiR2step}

\title{
  Choose a Model by Permutation Tests in Constrained Ordination
}
\description{
  Automatic stepwise model building for constrained ordination methods
  (\code{\link{cca}}, \code{\link{rda}}, \code{\link{capscale}}).
  The function \code{ordistep} is modelled after \code{\link{step}} and
  can do forward, backward and stepwise model selection using permutation tests.
  Function \code{ordiR2step} performs forward model choice solely on adjusted
  \eqn{R^2}{R2} and P-value, for ordination objects created by \code{\link{rda}} or \code{\link{capscale}}.
}
\usage{
ordistep(object, scope, direction = c("both", "backward", "forward"),
   Pin = 0.05, Pout = 0.1, permutations = how(nperm = 199), steps = 50,
   trace = TRUE, ...)
ordiR2step(object, scope, Pin = 0.05, R2scope = TRUE,
   permutations = how(nperm = 499), trace = TRUE, R2permutations = 1000, ...)
}

\arguments{
  \item{object}{
  In \code{ordistep}, an ordination object inheriting from
  \code{\link{cca}} or \code{\link{rda}}.
}

  \item{scope}{ Defines the range of models examined in the stepwise
  search.  This can be a list containing components \code{upper} and
  \code{lower}, both formulae. If it is a single item, it is interpreted
  the target scope, depending on the \code{direction}. If
  \code{direction} is \code{"forward"}, a single item is interpreted as
  the \code{upper} scope and the formula of the input \code{object} as
  the \code{lower} scope.  See \code{\link{step}} for details. In
  \code{ordiR2step}, this defines the upper scope; it can also be an
  ordination object from with the model is extracted.
  }

  \item{direction}{
  The mode of stepwise search, can be one of \code{"both"},
  \code{"backward"}, or \code{"forward"}, with a default of
  \code{"both"}.  If the \code{scope} argument is missing, the default
  for \code{direction} is \code{"backward"} in \code{ordistep} (and
  \code{ordiR2step} does not have this argument, but only works
  forward).
}
  \item{Pin, Pout}{
  Limits of permutation \eqn{P}-values for adding (\code{Pin}) a term to
  the model, or dropping (\code{Pout}) from the model. Term is added if
  \eqn{P \le}{P <=} \code{Pin}, and removed if \eqn{P >} \code{Pout}.
}

  \item{R2scope}{
  Use adjusted \eqn{R^2}{R2} as the stopping criterion: only models with
  lower adjusted \eqn{R^2}{R2} than scope are accepted.
  }

  \item{permutations}{a list of control values for the permutations as
    returned by the function \code{\link[permute]{how}}, or the number
    of permutations required, or a permutation matrix where each row
    gives the permuted indices. This is passed to
    \code{\link{anova.cca}}: see there for details.  }

  \item{steps}{
  Maximum number of iteration steps of dropping and adding terms.
}
  \item{trace}{
  If positive, information is printed during the model building. Larger
  values may give more information.
}

\item{R2permutations}{Number of permutations used in the estimation of
  adjusted \eqn{R^2}{R2} for \code{\link{cca}} using
  \code{\link{RsquareAdj}}.
}

  \item{\dots}{
  Any additional arguments to \code{\link{add1.cca}} and
  \code{\link{drop1.cca}}.
}
}
\details{
  The basic functions for model choice in constrained ordination are
  \code{\link{add1.cca}} and \code{\link{drop1.cca}}. With these functions,
  ordination models can be chosen with standard \R function
  \code{\link{step}} which bases the term choice on AIC. AIC-like
  statistics for ordination are provided by functions
  \code{\link{deviance.cca}} and \code{\link{extractAIC.cca}} (with
  similar functions for \code{\link{rda}}). Actually, constrained
  ordination methods do not have AIC, and therefore the \code{\link{step}}
  may not be trusted. This function provides an alternative using
  permutation \eqn{P}-values.

  Function \code{ordistep} defines the model, \code{scope} of models
  considered, and \code{direction} of the procedure similarly as
  \code{\link{step}}. The function alternates with \code{drop} and
  \code{add} steps and stops when the model was not changed during one
  step. The \code{-} and \code{+} signs in the summary table indicate
  which stage is performed.  It is often sensible to have \code{Pout}
  \eqn{>} \code{Pin} in stepwise models to avoid cyclic adds and drops
  of single terms.

  Function \code{ordiR2step} builds model forward so that it maximizes
  adjusted \eqn{R^2}{R2} (function \code{\link{RsquareAdj}}) at every
  step, and stopping when the adjusted \eqn{R^2}{R2} starts to decrease,
  or the adjusted \eqn{R^2}{R2} of the \code{scope} is exceeded, or the
  selected permutation \eqn{P}-value is exceeded (Blanchet et
  al. 2008). The second criterion is ignored with option
  \code{R2scope = FALSE}, and the third criterion can be ignored setting
  \code{Pin = 1} (or higher).
  The function cannot be used if adjusted \eqn{R^2}{R2}
  cannot be calculated, including partial models.  If the number of
  predictors is higher than the number of observations, adjusted
  \eqn{R^2}{R2} is also unavailable, but such models can be analysed
  with \code{R2scope = FALSE}.  The \eqn{R^2}{R2} of \code{\link{cca}}
  is based on simulations (see \code{\link{RsquareAdj}}) and different
  runs of \code{ordiR2step} can give different results.

  Functions \code{ordistep} (based on \eqn{P} values) and \code{ordiR2step}
  (based on adjusted \eqn{R^2}{R2} and hence on eigenvalues) can select
  variables in different order.
}

\value{
  Functions return the selected model with one additional
  component, \code{anova}, which contains brief information of steps
  taken. You can suppress voluminous output during model building by
  setting \code{trace = FALSE}, and find the summary of model history
  in the \code{anova} item.
}

\references{
  Blanchet, F. G., Legendre, P. & Borcard, D. (2008) Forward selection
  of explanatory variables. \emph{Ecology} 89, 2623--2632.
}

\author{
  Jari Oksanen
}

\seealso{

  The function handles constrained ordination methods
  \code{\link{cca}}, \code{\link{rda}}, \code{\link{dbrda}} and
  \code{\link{capscale}}. The underlying functions are
  \code{\link{add1.cca}} and \code{\link{drop1.cca}}, and the function
  is modelled after standard \code{\link{step}} (which also can be
  used directly but uses AIC for model choice, see
  \code{\link{extractAIC.cca}}). Function \code{ordiR2step} builds
  upon \code{\link{RsquareAdj}}.

}
\examples{
## See add1.cca for another example

### Dune data
data(dune)
data(dune.env)
mod0 <- rda(dune ~ 1, dune.env)  # Model with intercept only
mod1 <- rda(dune ~ ., dune.env)  # Model with all explanatory variables

## With scope present, the default direction is "both"
mod <- ordistep(mod0, scope = formula(mod1))
mod
## summary table of steps
mod$anova

## Example of ordistep, forward
ordistep(mod0, scope = formula(mod1), direction="forward")

## Example of ordiR2step (always forward)
## stops because R2 of 'mod1' exceeded
ordiR2step(mod0, mod1)
}


\keyword{ multivariate }
\keyword{ models }

\name{distconnected}
\alias{distconnected}
\alias{no.shared}

\title{Connectedness of Dissimilarities}
\description{
  Function \code{distconnected} finds groups that are connected
  disregarding dissimilarities that are at or above a threshold or
  \code{NA}. The function can be used to find groups that can be
  ordinated together or transformed by
  \code{\link{stepacross}}. Function \code{no.shared} returns a logical
  dissimilarity object, where \code{TRUE} means that sites have no
  species in common. This is a minimal structure for
  \code{distconnected} or can be used to set missing values to
  dissimilarities.
}
\usage{
distconnected(dis, toolong = 1, trace = TRUE)

no.shared(x)
}

\arguments{
  \item{dis}{Dissimilarity data inheriting from class \code{dist} or
    a an object, such as a matrix, that can be converted to a
    dissimilarity matrix. Functions \code{\link{vegdist}} and
    \code{\link{dist}} are some functions producing suitable
    dissimilarity data.}
  \item{toolong}{ Shortest dissimilarity regarded as \code{NA}.
    The function uses a fuzz factor, so
    that dissimilarities close to the limit will be made \code{NA}, too.
    If \code{toolong = 0} (or negative), no dissimilarity is regarded
    as too long.
  }
  \item{trace}{Summarize results of \code{distconnected}}
  \item{x}{Community data.}
  
}
\details{
  Data sets are disconnected if they have sample plots or groups of
  sample plots which share no species with other sites or groups of
  sites. Such data sets cannot be sensibly ordinated by any
  unconstrained method because these subsets cannot be related to each
  other. For instance, correspondence analysis will polarize these
  subsets with eigenvalue 1. Neither can such dissimilarities be
  transformed with \code{\link{stepacross}}, because there is no path
  between all points, and result will contain \code{NA}s. Function
  \code{distconnected} will find such subsets in dissimilarity
  matrices. The function will return a grouping vector that can be used
  for sub-setting the data. If data are connected, the result vector will
  be all \eqn{1}s. The connectedness between two points can be defined
  either by a threshold \code{toolong} or using input dissimilarities
  with \code{NA}s.

  Function \code{no.shared} returns a \code{dist} structure having value
  \code{TRUE} when two sites have nothing in common, and value
  \code{FALSE} when they have at least one shared species. This is a
  minimal structure that can be analysed with \code{distconnected}. The
  function can be used to select dissimilarities with no shared species
  in indices which do not have a fixed upper limit.
  
  Function \code{distconnected} uses depth-first search
  (Sedgewick 1990). 
}
\value{
  Function \code{distconnected} returns a vector for
  observations using integers to identify connected groups. If the data
  are connected, values will be all \code{1}. Function \code{no.shared}
  returns an object of class \code{\link{dist}}.
}
\references{
 Sedgewick, R. (1990). \emph{Algorithms in C}. Addison Wesley. 
}
\author{ Jari Oksanen }

\seealso{\code{\link{vegdist}} or \code{\link{dist}} for getting
  dissimilarities, \code{\link{stepacross}} for a case where you may need
  \code{distconnected}, and for connecting points \code{\link{spantree}}.
}

\examples{
## There are no disconnected data in vegan, and the following uses an
## extremely low threshold limit for connectedness. This is for
## illustration only, and not a recommended practice.
data(dune)
dis <- vegdist(dune)
gr <- distconnected(dis, toolong=0.4)
# Make sites with no shared species as NA in Manhattan dissimilarities
dis <- vegdist(dune, "manhattan")
is.na(dis) <- no.shared(dune)
}
\keyword{ multivariate}

\name{rankindex}
\alias{rankindex}

\title{Compares Dissimilarity Indices for Gradient Detection }
\description{
  Rank correlations between dissimilarity indices
  and gradient separation.
}
\usage{
rankindex(grad, veg, indices = c("euc", "man", "gow", "bra", "kul"),
          stepacross = FALSE, method = "spearman", 
	  metric = c("euclidean", "mahalanobis", "manhattan", "gower"),
	  ...)
}

\arguments{
  \item{grad}{The gradient variable or matrix. }
  \item{veg}{The community data matrix. }
  \item{indices}{Dissimilarity indices compared, partial matches to
    alternatives in \code{\link{vegdist}}.
    Alternatively, it can be a (named) list of functions returning
    objects of class 'dist'.}
  \item{stepacross}{Use \code{\link{stepacross}} to find
    a shorter path dissimilarity. The dissimilarities for site pairs
    with no shared species are set \code{NA} using
    \code{\link{no.shared}} so that indices with no fixed
    upper limit can also be analysed.}
  \item{method}{Correlation method used.}
  \item{metric}{Metric to evaluate the gradient separation. See Details.}
  \item{...}{Other parameters to \code{\link{stepacross}}.}
}
\details{
  A good dissimilarity index for multidimensional scaling should have
  a high rank-order similarity with gradient separation.  The function
  compares most indices in \code{\link{vegdist}} against gradient
  separation using rank correlation coefficients in
  \code{\link{cor}}. The gradient separation between each point is
  assessed using given \code{metric}. The default is to use Euclidean
  distance of continuous variables scaled to unit variance, or to use
  Gower metric for mixed data using function
  \code{\link[cluster]{daisy}} when \code{grad} has factors. The other
  alternatives are Mahalanabis distances which are based on
  \code{grad} matrix scaled so that columns are orthogonal
  (uncorrelated) and have unit variance, or Manhattan distances of
  \code{grad} variables scaled to unit range.

  The \code{indices} argument can accept any dissimilarity 
  indices besides the ones calculated by the 
  \code{\link{vegdist}} function. For this, the argument value
  should be a (possibly named) list of functions.
  Each function must return a valid 'dist' object with dissimilarities,
  similarities are not accepted and should be converted into dissimilarities
  beforehand.
}
\value{
  Returns a named vector of rank correlations.
}
\references{ Faith, F.P., Minchin, P.R. and Belbin,
  L. (1987).  Compositional dissimilarity as a robust measure of
    ecological distance. \emph{Vegetatio} 69, 57-68. }
\author{Jari Oksanen, with additions from Peter Solymos}
\note{
  There are several problems in using rank correlation coefficients.
  Typically there are very many ties when \eqn{n(n-1)/2} gradient
  separation values are derived from just \eqn{n} observations.
  Due to floating point arithmetics, many tied values differ by
  machine epsilon and are arbitrarily ranked differently by
  \code{\link{rank}} used in \code{\link{cor.test}}.  Two indices
  which are identical with certain
  transformation or standardization may differ slightly
  (magnitude \eqn{10^{-15}}) and this may lead into third or fourth decimal
  instability in rank correlations.  Small differences in rank
  correlations should not be taken too seriously.  Probably this method
  should be replaced with a sounder method, but I do not yet know
  which\ldots  You may experiment with \code{\link{mantel}},
  \code{\link{anosim}} or even \code{\link{protest}}.

  Earlier version of this function used \code{method = "kendall"}, but
  that is far too slow in large data sets.

  The functions returning dissimilarity objects should be self contained,
  because the \code{...} argument passes additional parameters
  to \code{\link{stepacross}} and not to the functions supplied
  via the \code{indices} argument.
}

\seealso{\code{\link{vegdist}}, \code{\link{stepacross}},
  \code{\link{no.shared}}, \code{\link{monoMDS}},
    \code{\link{cor}}, \code{\link{Machine}}, and for
    alternatives \code{\link{anosim}}, \code{\link{mantel}} and
    \code{\link{protest}}. }
  
\examples{
data(varespec)
data(varechem)
## The variables are automatically scaled
rankindex(varechem, varespec)
rankindex(varechem, wisconsin(varespec))
## Using non vegdist indices as functions
funs <- list(Manhattan=function(x) dist(x, "manhattan"),
    Gower=function(x) cluster:::daisy(x, "gower"),
    Ochiai=function(x) designdist(x, "1-J/sqrt(A*B)"))
rankindex(scale(varechem), varespec, funs)
}
\keyword{ multivariate }

\name{specpool}
\alias{specpool}
\alias{specpool2vect}
\alias{poolaccum}
\alias{summary.poolaccum}
\alias{plot.poolaccum}
\alias{estimateR}
\alias{estimateR.default}
\alias{estimateR.matrix}
\alias{estimateR.data.frame}
\alias{estaccumR}

\title{ Extrapolated Species Richness in a Species Pool}
\description{
  The functions estimate the extrapolated species richness in a species
  pool, or the number of unobserved species. Function \code{specpool}
  is based on incidences in sample sites, and gives a single estimate
  for a collection of sample sites (matrix).  Function \code{estimateR}
  is based on abundances (counts) on single sample site. 
}
\usage{
specpool(x, pool, smallsample = TRUE)
estimateR(x, ...)
specpool2vect(X, index = c("jack1","jack2", "chao", "boot","Species"))
poolaccum(x, permutations = 100, minsize = 3)
estaccumR(x, permutations = 100, parallel = getOption("mc.cores"))
\method{summary}{poolaccum}(object, display, alpha = 0.05, ...)
\method{plot}{poolaccum}(x, alpha = 0.05, type = c("l","g"), ...)
}

\arguments{
  \item{x}{Data frame or matrix with species data or the analysis result 
    for \code{plot} function.}
  \item{pool}{A vector giving a classification for pooling the sites in
    the species data. If missing, all sites are pooled together.}
  \item{smallsample}{Use small sample correction \eqn{(N-1)/N}, where
    \eqn{N} is the number of sites within the \code{pool}.}
  \item{X, object}{A \code{specpool} result object.}
  \item{index}{The selected index of extrapolated richness.}
  \item{permutations}{Usually an integer giving the number
    permutations, but can also be a list of control values for the
    permutations as returned by the function \code{\link[permute]{how}}, 
    or a permutation matrix where each row gives the permuted indices.}
  \item{minsize}{Smallest number of sampling units reported.}
  \item{parallel}{Number of parallel processes or a predefined socket
    cluster.  With \code{parallel = 1} uses ordinary, non-parallel
    processing. The parallel processing is done with \pkg{parallel}
    package.}
  \item{display}{Indices to be displayed.}
  \item{alpha}{Level of quantiles shown. This proportion will be left outside
    symmetric limits.}
  \item{type}{Type of graph produced in \code{\link[lattice]{xyplot}}.}
  \item{...}{Other parameters (not used).}
}
\details{
  Many species will always remain unseen or undetected in a collection
  of sample plots.  The function uses some popular ways of estimating
  the number of these unseen species and adding them to the observed
  species richness (Palmer 1990, Colwell & Coddington 1994).

  The incidence-based estimates in \code{specpool} use the frequencies
  of species in a collection of sites.
  In the following, \eqn{S_P} is the extrapolated richness in a pool,
  \eqn{S_0} is the observed number of species in the
  collection, \eqn{a_1}{a1} and \eqn{a_2}{a2} are the number of species
  occurring only in one or only in two sites in the collection, \eqn{p_i}
  is the frequency of species \eqn{i}, and \eqn{N} is the number of
  sites in the collection.  The variants of extrapolated richness in
  \code{specpool} are:
  \tabular{ll}{
     Chao
    \tab \eqn{S_P = S_0 + \frac{a_1^2}{2 a_2}\frac{N-1}{N}}{S_P = S_0 + a1^2/(2*a2) * (N-1)/N}
    \cr
    Chao bias-corrected
    \tab \eqn{S_P = S_0 + \frac{a_1(a_1-1)}{2(a_2+1)} \frac{N-1}{N}}{S_P = S_0 + a1*(a1-1)/(2*(a2+1)) * (N-1)/N}
    \cr
    First order jackknife
    \tab \eqn{S_P = S_0 + a_1 \frac{N-1}{N}}{S_P = S_0 + a1*(N-1)/N}
    \cr
    Second order jackknife
    \tab \eqn{S_P = S_0 + a_1 \frac{2N - 3}{N} - a_2 \frac{(N-2)^2}{N
	(N-1)}}{S_P = S_0 + a1*(2*N-3)/N - a2*(N-2)^2/N/(N-1)}
    \cr
    Bootstrap
    \tab \eqn{S_P = S_0 + \sum_{i=1}^{S_0} (1 - p_i)^N}{S_P = S_0 + Sum
      (1-p_i)^N}
    }
    \code{specpool} normally uses basic Chao equation, but when there
    are no doubletons (\eqn{a2=0}) it switches to bias-corrected
    version. In that case the Chao equation simplifies to
    \eqn{S_0 + \frac{1}{2} a_1 (a_1-1) \frac{N-1}{N}}{S_0 + (N-1)/N * a1*(a1-1)/2}.

    The abundance-based estimates in \code{estimateR} use counts
    (numbers of individuals) of species in a single site. If called for
    a matrix or data frame, the function will give separate estimates
    for each site.  The two variants of extrapolated richness in
    \code{estimateR} are bias-corrected Chao and ACE (O'Hara 2005, Chiu
    et al. 2014).  The Chao estimate is similar as the bias corrected
    one above, but \eqn{a_i} refers to the number of species with
    abundance \eqn{i} instead of number of sites, and the small-sample
    correction is not used. The ACE estimate is defined as:

    \tabular{ll}{
    ACE
    \tab \eqn{S_P = S_{abund} + \frac{S_{rare}}{C_{ace}}+ \frac{a_1}{C_{ace}}
      \gamma^2_{ace}}{S_P = S_abund + S_rare/C_ace + a1/C_ace * gamma^2}
    \cr
    where \tab
    \eqn{C_{ace} = 1 - \frac{a_1}{N_{rare}}}{C_{ace} = 1- a1/N_{rare}}
    \cr
    \tab \eqn{\gamma^2_{ace} = \max \left[ \frac{S_{rare} \sum_{i=1}^{10}
      i(i-1)a_i}{C_{ace} N_{rare} (N_{rare} - 1)}-1, 0 \right]}{gamma^2 = 
    max(S_rare/C_ace (sum[i=1..10] i*(i-1)*a_i) / N_rare/(N_rare-1) -1 , 0)}
    }
    Here \eqn{a_i} refers to number of species with abundance \eqn{i}
    and  \eqn{S_{rare}}{S_rare} is the number of rare
    species, 
    \eqn{S_{abund}}{S_abund} is the number of abundant species, with an
    arbitrary 
    threshold of abundance 10 for rare species, and \eqn{N_{rare}}{N_rare} is
    the number 
    of individuals in rare species.

    Functions estimate the standard errors of the estimates. These only
    concern the number of added species, and assume that there is no
    variance in the observed richness.  The equations of standard errors
    are too complicated to be reproduced in this help page, but they can
    be studied in the \R source code of the function and are discussed
    in the \code{\link{vignette}} that can be read with the
    \code{browseVignettes("vegan")}. The standard error are based on the
    following sources: Chiu et al. (2014) for the Chao estimates and
    Smith and van Belle (1984) for the first-order Jackknife and the
    bootstrap (second-order jackknife is still missing).  For the
    variance estimator of \eqn{S_{ace}}{S_ace} see O'Hara (2005).

  Functions \code{poolaccum} and \code{estaccumR} are similar to
  \code{\link{specaccum}}, but estimate extrapolated richness indices
  of \code{specpool} or \code{estimateR} in addition to number of
  species for random ordering of sampling units. Function
  \code{specpool} uses presence data and \code{estaccumR} count
  data. The functions share \code{summary} and \code{plot}
  methods. The \code{summary} returns quantile envelopes of
  permutations corresponding the given level of \code{alpha} and
  standard deviation of permutations for each sample size. NB., these
  are not based on standard deviations estimated within \code{specpool}
  or \code{estimateR}, but they are based on permutations. The
  \code{plot} function shows the mean and envelope of permutations
  with given \code{alpha} for models. The selection of models can be
  restricted and order changes using the \code{display} argument in
  \code{summary} or \code{plot}. For configuration of \code{plot}
  command, see \code{\link[lattice]{xyplot}}.
}

\value{
  Function \code{specpool} returns a data frame with entries for
  observed richness and each of the indices for each class in
  \code{pool} vector.  The utility function \code{specpool2vect} maps
  the pooled values into a vector giving the value of selected
  \code{index} for each original site. Function \code{estimateR}
  returns the estimates and their standard errors for each
  site. Functions \code{poolaccum} and \code{estimateR} return
  matrices of permutation results for each richness estimator, the
  vector of sample sizes and a table of \code{means} of permutations
  for each estimator.
}

\references{
  Chao, A. (1987). Estimating the population size for capture-recapture
  data with unequal catchability. \emph{Biometrics} 43, 783--791.

  Chiu, C.H., Wang, Y.T., Walther, B.A. & Chao, A. (2014). Improved
  nonparametric lower bound of species richness via a modified
  Good-Turing frequency formula. \emph{Biometrics} 70, 671--682.
  
  Colwell, R.K. & Coddington, J.A. (1994). Estimating terrestrial
  biodiversity through
  extrapolation. \emph{Phil. Trans. Roy. Soc. London} B 345, 101--118.

  O'Hara, R.B. (2005). Species richness estimators: how many species
  can dance on the head of a pin? \emph{J. Anim. Ecol.} 74, 375--386.

  Palmer, M.W. (1990). The estimation of species richness by
  extrapolation. \emph{Ecology} 71, 1195--1198.

  Smith, E.P & van Belle, G. (1984). Nonparametric estimation of
  species richness. \emph{Biometrics} 40, 119--129.
}
\author{Bob O'Hara (\code{estimateR}) and Jari Oksanen.}

\note{ The functions are based on assumption that there is a species
  pool: The community is closed so that there is a fixed pool size
  \eqn{S_P}.  In general, the functions give only the lower limit of
  species richness: the real richness is \eqn{S >= S_P}, and there is
  a consistent bias in the estimates. Even the bias-correction in Chao
  only reduces the bias, but does not remove it completely (Chiu et
  al. 2014).

  Optional small sample correction was added to \code{specpool} in
  \pkg{vegan} 2.2-0. It was not used in the older literature (Chao
  1987), but it is recommended recently (Chiu et al. 2014).

  See \url{http://viceroy.eeb.uconn.edu/EstimateS} for a more complete
  (and positive) discussion and alternative software for some platforms.
}
\seealso{\code{\link{veiledspec}}, \code{\link{diversity}}, \code{\link{beals}},
 \code{\link{specaccum}}. }
\examples{
data(dune)
data(dune.env)
pool <- with(dune.env, specpool(dune, Management))
pool
op <- par(mfrow=c(1,2))
boxplot(specnumber(dune) ~ Management, data = dune.env,
        col = "hotpink", border = "cyan3")
boxplot(specnumber(dune)/specpool2vect(pool) ~ Management,
        data = dune.env, col = "hotpink", border = "cyan3")
par(op)
data(BCI)
## Accumulation model
pool <- poolaccum(BCI)
summary(pool, display = "chao")
plot(pool)
## Quantitative model
estimateR(BCI[1:5,])
}
\keyword{ univar }

\name{orditkplot}
\alias{orditkplot}
\alias{plot.orditkplot}
\alias{scores.orditkplot}
\alias{points.orditkplot}
\alias{text.orditkplot}

\title{ Ordination Plot with Movable Labels }
\description{
  Function \code{orditkplot} produces an editable ordination plot with
  points and labels. The labels can be moved with mouse, and the edited
  plot can be saved as an encapsulated postscript file or exported via \R 
  \code{plot} function to other graphical formats, or saved in the \R session 
  for further processing.
}
\usage{
orditkplot(x, display = "species", choices = 1:2, width, xlim, ylim, 
   tcex = 0.8, tcol, pch = 1,  pcol, pbg, pcex = 0.7, labels,  ...)
\method{plot}{orditkplot}(x, ...)
\method{points}{orditkplot}(x, pch = x$args$pch, cex = x$args$pcex,
       col = x$args$pcol, bg = x$args$pbg, ...)
\method{text}{orditkplot}(x, cex = x$args$tcex, col = x$args$tcol,
     font = attr(x$labels, "font"), ...)
\method{scores}{orditkplot}(x, display, ...)
}

\arguments{ 
  \item{x}{An ordination result or any other object that
    \code{\link{scores}} can handle, or for the \code{plot} function the
    object dumped from the interactive \code{orditkplot} session.}
  \item{display}{Type of \code{\link{scores}} displayed. For ordination
    scores this typically is either \code{"species"} or \code{"sites"},
    and for \code{orditkplot} result it is either \code{"points"} or
    \code{"labels"}.}
  \item{choices}{Axes displayed.}
  \item{width}{Width of the plot in inches; defaults to the current
    width of the graphical device. }
  \item{xlim, ylim}{x and y limits for plots: points outside these
    limits will be completely removed.}
  \item{tcex}{Character expansion for text labels.}
  \item{tcol}{Colour of text labels.}
  \item{pch, pcol, pbg}{Point type and outline and fill colours. 
    Defaults \code{pcol="black"}  and \code{pbg="transparent"}. 
    Argument \code{pbg} has an effect only in filled plotting characters
    \code{pch = 21} to \code{25}.} 
  \item{pcex}{Expansion factor for point size.}  
  \item{labels}{Labels used instead of row names.}
  \item{cex, col, bg, font}{graphical parameters used in the
    \code{points} and \code{text} methods. See \code{\link{par}}.}
  \item{\dots}{Other arguments passed to the function. These can be
    graphical parameters (see \code{\link{par}}) used in the plot, or
    extra arguments to \code{\link{scores}}. These arguments are
    ignored in \code{plot}, but honoured in \code{text} and \code{points}. }  
}

\details{ Function \code{orditkplot} uses \pkg{tcltk} package to draw
  Tcl/Tk based ordination graphics with points and labels. The function
  opens an editable canvas with fixed points, but the labels can be
  dragged with mouse to better positions or edited. In addition, it is
  possible to zoom to a part of the graph.

  The function knows the following mouse operations:
  \itemize{
    \item \strong{Left mouse button} can be used to move labels to better
    positions. A line will connect a label to the corresponding point.
    \item \strong{Double clicking left mouse button} opens a window where the
    label can be edited. After editing the label, hit the Return key.
    \item \strong{Right mouse button} (or alternatively, Shift-Mouse button with
    one-button mouse) can be used for zooming to a part of the
    graph. Keeping the mouse button down and dragging will draw a box
    of the zoomed area, and after releasing the button, a new plot window
    will be created (this is still preliminary: all arguments are not
    passed to the new plot).
  }

  In addition there are buttons for the following tasks: \strong{Copy
  to EPS} copies the current plot to an encapsulated postscript (eps)
  file using standard Tcl/Tk utilities. The faithfulness of this copy
  is system dependent. Button \strong{Export plot} uses
  \code{plot.orditkplot} function to redraw the plot into graphical
  file formats. Depending on the system, the following graphical
  formats may be available: eps, pdf, svg, png, jpeg, tiff, bmp or
  xfig. Some of the output formats may be edited with external
  software: svg files with Illustrator or Inkscape, and xfig with the
  legacy program XFig. Button \strong{Save to R} writes the edited
  coordinates of labels and points to the \R session for further
  processing, and the \code{plot.orditkplot} function can be used to
  display the results. For faithful replication of the plot, the graph
  must have similar dimensions as the \code{orditkplot} canvas had
  originally. The \code{plot} function cannot be configured, but it
  uses the same settings as the original Tcl/Tk plot. However,
  \code{points} and \code{text} functions are fully configurable, but
  use the stored defaults for consistency with \code{plot.orditkplot}
  if none are supplied. Finally, button \strong{Close} closes the
  window.

  The produced plot will have equal aspect ratio. The width of the
  horizontal axis is fixed, but vertical axes will be scaled to needed
  height, and you can use scrollbar to move vertically if the whole
  canvas does not fit the window. If you use dumped labels in ordinary
  \R plots, your plot must have the same dimensions as the
  \code{orditkplot} canvas to have identical location of the labels.

  The function only displays one set of scores. However, you can use
  \code{\link{ordipointlabel}} to produce a result object that has
  different points and text types for several sets of scores and this
  can be further edited with \code{orditkplot}. For a good starting
  solution you need to scale the \code{\link{ordipointlabel}} result
  so that the points span over the whole horizontal axis. The function
  cannot show environmental variables or constraints, but it is
  limited to unconstrained ordination.

  The plot is a Tcl/Tk canvas, but the function tries to replicate
  standard graphical device of the platform, and it honours several
  graphical parameters (see \code{\link{par}}).  Many of the graphical
  parameters can be given on the command line, and they will be passed
  to the function without influencing other graphical devices in \R.
  At the moment, the
  following graphical parameters are honoured: 
  \code{pch} \code{bg},  \code{cex},
  \code{cex.axis}, \code{cex.lab}, \code{col} (for labels),
  \code{col.axis}, \code{col.lab}, \code{family} (for font faces),
  \code{fg}, \code{font}, \code{font.axis}, \code{font.lab},
  \code{lheight}, \code{lwd} (for the box), \code{mar}, \code{mex},
  \code{mgp}, \code{ps}, \code{tcl}. These can be set with
  \code{\link{par}}, and they also will influence other plots similarly.
  
  The \code{\link[tcltk]{tkcanvas}} text cannot be rotated, and
  therefore vertical axis is not labelled, and \code{las}
  \code{\link{par}}ameter will not be honoured in the Tcl/Tk plot, but
  it will be honoured in the exported \R plots and in
  \code{plot.orditkplot}.  }

\value{ Function returns nothing useful directly, but you can save the
  edited graph to a file or save the edited positions to an \R session
  for further processing and plotting.  
}

\author{ Jari Oksanen }
\note{
  You need \pkg{tcltk} package and \R must have been configured with
  \code{\link{capabilities}} for \code{tcltk}.
  Depending on your OS, you may need to start X11 and set the display
  before loading \pkg{tcltk} and starting the function (for instance,
  with \code{Sys.setenv("DISPLAY"=":0")}). See
  \code{\link[tcltk]{tcltk-package}}. 

}

\seealso{ Function \code{\link{ordipointlabel}} is an automatic
  procedure with similar goals of avoiding overplotting, and its
  output can be edited with \code{orditkplot}.  See
  \code{\link{ordiplot}}, \code{\link{plot.cca}},
  \code{\link[vegan3d]{ordirgl}} and \code{\link{orditorp}} for
  alternative ordination plots, and \code{\link{scores}} for
  extracting ordination scores.  }

\examples{
## The example needs user interaction and is not executed directly.
## It should work when pasted to the window.
\dontrun{
data(varespec)
ord <- cca(varespec)
## Do something with the graph and end by clicking "Dismiss"
orditkplot(ord, mar = c(4,4,1,1)+.1, font=3)
## Use ordipointlabel to produce a plot that has both species and site
## scores in different colors and plotting symbols
pl <- ordipointlabel(ord)
orditkplot(pl)
}
}
\keyword{ iplot }
\keyword{ dynamic }

\name{ordiarrows}
\alias{ordiarrows}
\alias{ordisegments}
\alias{ordigrid}

\title{Add Arrows and Line Segments to Ordination Diagrams}

\description{ Functions to add arrows, line segments, regular grids of
  points. The ordination diagrams can be produced by \code{vegan}
  \code{\link{plot.cca}}, \code{\link{plot.decorana}} or
  \code{\link{ordiplot}}.  }

\usage{
ordiarrows(ord, groups, levels, replicates, order.by, display = "sites",
         col = 1, show.groups, startmark, label = FALSE, length = 0.1, ...)
ordisegments(ord, groups, levels, replicates, order.by, display = "sites",
         col = 1, show.groups, label = FALSE, ...)
ordigrid(ord, levels, replicates, display = "sites",  lty = c(1,1), 
         col = c(1,1), lwd = c(1,1), ...)
}

\arguments{
  \item{ord}{An ordination object or an \code{\link{ordiplot}} object. }
  \item{groups}{Factor giving the groups for which the graphical item is
    drawn. }
  \item{levels, replicates}{Alternatively, regular
    groups can be defined with arguments \code{levels} and
    \code{replicates}, where \code{levels} gives the number of groups,
    and \code{replicates} the number of successive items at the same
    group.}
  \item{order.by}{Order points by increasing order of this variable
    within \code{groups}. Reverse sign of the variable for decreasing
    ordering.} 
  \item{display}{Item to displayed. }
  \item{show.groups}{Show only given groups. This can be a vector, or
    \code{TRUE} if you want to show items for which condition is
    \code{TRUE}. This argument makes it possible to use different
    colours and line types for groups. The default is to show all groups. }
  \item{label}{Label the \code{groups} by their names. In
    \code{ordiellipse}, \code{ordihull} and \code{ordispider} the the
    group name is in the centroid of the object, in \code{ordiarrows}
    in the start of the arrow, and in \code{ordisegments} at both
    ends. \code{ordiellipse} and \code{ordihull} use standard
    \code{\link{text}}, and others use \code{\link{ordilabel}}.}
  
  \item{startmark}{plotting character used to mark the first item. The
    default is to use no mark, and for instance, \code{startmark = 1}
    will draw a circle.  For other plotting characters, see \code{pch}
    in \code{\link{points}}. }

  \item{col}{Colour of lines, \code{label} borders and
    \code{startmark} in \code{ordiarrows} and
    \code{ordisegments}. This can be a vector recycled for
    \code{groups}. In \code{ordigrid} it can be a vector of length 2
    used for \code{levels} and \code{replicates}.}

  \item{length}{Length of edges of the arrow head (in inches).}

  \item{lty, lwd}{Line type, line width used for 
    \code{level}s and \code{replicate}s in \code{ordigrid}.}

  \item{\dots}{Parameters passed to graphical functions such as
    \code{\link{lines}}, \code{\link{segments}}, \code{\link{arrows}},
    or to \code{\link{scores}} to select axes and scaling etc. }
}

\details{ Function \code{ordiarrows} draws \code{\link{arrows}} and
  \code{ordisegments} draws line \code{\link{segments}} between
  successive items in the groups. Function \code{ordigrid} draws line
  \code{\link{segments}} both within the groups and for the
  corresponding items among the groups.

}

\note{These functions add graphical items to ordination graph: You must
  draw a graph first.
}
\author{ Jari Oksanen }

\seealso{The functions pass parameters to basic graphical functions, and
  you may wish to change the default values in \code{\link{arrows}},
  \code{\link{lines}} and \code{\link{segments}}. You can pass
  parameters to \code{\link{scores}} as well. 
}

\examples{
example(pyrifos)
mod <- rda(pyrifos)
plot(mod, type = "n")
## Annual succession by ditches, colour by dose
ordiarrows(mod, ditch, label = TRUE, col = as.numeric(dose))
legend("topright", levels(dose), lty=1, col=1:5, title="Dose")
## Show only control and highest Pyrifos treatment
plot(mod, type = "n")
ordiarrows(mod, ditch, label = TRUE, 
   show.groups = c("2", "3", "5", "11"))
ordiarrows(mod, ditch, label = TRUE, show = c("6", "9"),
   col = 2)
legend("topright", c("Control", "Pyrifos 44"), lty = 1, col = c(1,2))
}
\keyword{aplot }

\name{ordiArrowTextXY}
\alias{ordiArrowMul}
\alias{ordiArrowTextXY}

\title{Support Functions for Drawing Vectors}
\description{
  Support functions to assist with drawing of vectors (arrows) on
  ordination plots. \code{ordiArrowMul} finds the multiplier for the
  coordinates of the head of the vector such that they occupy
  \code{fill} proportion of the plot region. \code{ordiArrowTextXY}
  finds coordinates for the locations of \code{labels} to be drawn just
  beyond the head of the vector.
}
\usage{
ordiArrowTextXY(x, labels, display, choices = c(1,2),
                rescale = TRUE, fill = 0.75, at = c(0,0), \ldots)
ordiArrowMul(x, at = c(0,0), fill = 0.75,
             display, choices = c(1,2), \ldots)
}

\arguments{
  \item{x}{An R object, from which \code{\link{scores}} can determine
    suitable ordination scores or an object created by
    \code{\link{envfit}}, or a two-column matrix of coordinates of arrow
    heads on the two plot axes.}

  \item{labels}{Change plotting labels. A character vector of labels for
    which label coordinates are sought. If not supplied, these will be
    determined from the row names of \code{x}, or \code{scores(x, ...)}
    if required. If either of these are not defined, suitable labels
    will be generated.}

  \item{display}{a character string known to \code{\link{scores}} or one
    of its methods which indicates the type of scores to extract. In
    fitting functions these are ordinary site scores or linear
    combination scores (\code{"lc"}) in constrained ordination
    (\code{\link{cca}}, \code{\link{rda}}, \code{\link{capscale}}). If
    \code{x} was created by \code{envfit} then \code{display} can not be
    set by the user and takes the value \code{"vectors"}. Ignored if
    \code{x} is a matrix.}

  \item{choices}{Axes to be plotted.}

  \item{rescale}{logical; should the coordinates in or extracted from
    \code{x} be rescaled to fill \code{fill} proportion of the plot
    region? The default is to always rescale the coordinates as this is
    usually desired for objects \code{x} from which coordinates are
    retrieved. If supplying \code{x} a 2-column matrix that has already
    been rescaled, then set this to \code{FALSE}.}

  \item{fill}{numeric; the proportion of the plot to fill by the span of
    the arrows.}

  \item{at}{The origin of fitted arrows in the plot.  If you plot arrows
    in other places than origin, you probably have to specify
    \code{arrrow.mul}.}

  \item{...}{Parameters passed to \code{\link{scores}}, and
    \code{\link{strwidth}} and \code{\link{strheight}}.}
}

\details{
  \code{ordiArrowMul} finds a multiplier to scale a bunch of
  arrows to fill an ordination plot, and \code{ordiArrowTextXY} finds
  the coordinates for labels of these arrows. NB.,
  \code{ordiArrowTextXY} does not draw labels; it simply returns
  coordinates at which the labels should be drawn for use with another
  function, such as \code{\link{text}}.
}

\value{
  For \code{ordiArrowTextXY}, a 2-column matrix of coordinates for the
  label centres in the coordinate system of the currently active
  plotting device.

  For \code{ordiArrowMul}, a length-1 vector containing the scaling
  factor.
}

\author{Jari Oksanen, with modifications by Gavin L. Simpson}

\examples{
    ## Scale arrows by hand to fill 80% of the plot
    ## Biplot arrows by hand
    data(varespec, varechem)
    ord <- cca(varespec ~ Al + P + K, varechem)
    plot(ord, display = c("species","sites"))

    ## biplot scores
    bip <- scores(ord, choices = 1:2, display = "bp")

    ## scaling factor for arrows to fill 80% of plot
    (mul <- ordiArrowMul(bip, fill = 0.8))
    bip.scl <- bip * mul                    # Scale the biplot scores
    labs <- rownames(bip)                   # Arrow labels

    ## calculate coordinate of labels for arrows
    (bip.lab <- ordiArrowTextXY(bip.scl, rescale = FALSE, labels = labs))

    ## draw arrows and text labels
    arrows(0, 0, bip.scl[,1], bip.scl[,2], length = 0.1)
    text(bip.lab, labels = labs)

    ## Handling of ordination objects directly
    mul2 <- ordiArrowMul(ord, display = "bp", fill = 0.8)
    stopifnot(all.equal(mul, mul2))
}

\keyword{utilities}

\name{radfit}
\alias{radfit}
\alias{radfit.default}
\alias{radfit.data.frame}
\alias{AIC.radfit}
\alias{AIC.radfit.frame}
\alias{as.rad}
\alias{coef.radfit}
\alias{coef.radfit.frame}
\alias{deviance.radfit}
\alias{deviance.radfit.frame}
\alias{logLik, radfit}
\alias{logLik, radfit.frame}
\alias{fitted.radfit}
\alias{fitted.radfit.frame}
\alias{lines.radline}
\alias{lines.radfit}
\alias{plot.radfit.frame}
\alias{plot.radfit}
\alias{plot.radline}
\alias{plot.rad}
\alias{radlattice}
\alias{points.radline}
\alias{points.radfit}
\alias{summary.radfit.frame}
\alias{rad.preempt}
\alias{rad.lognormal}
\alias{rad.zipf}
\alias{rad.zipfbrot}
\alias{rad.null}
\alias{predict.radline}
\alias{predict.radfit}
\alias{predict.radfit.frame}

\title{ Rank -- Abundance or Dominance / Diversity Models}
\description{
  Functions construct rank -- abundance or dominance / diversity or
  Whittaker plots and fit brokenstick, preemption, log-Normal,
  Zipf and Zipf-Mandelbrot models of species abundance.
}
\usage{
\method{radfit}{default}(x, ...)
rad.null(x, family=poisson, ...)
rad.preempt(x, family = poisson, ...)
rad.lognormal(x, family = poisson, ...)
rad.zipf(x, family = poisson, ...)
rad.zipfbrot(x, family = poisson, ...)
\method{predict}{radline}(object, newdata, total, ...)
\method{plot}{radfit}(x, BIC = FALSE, legend = TRUE, ...)
\method{plot}{radfit.frame}(x, order.by, BIC = FALSE, model, legend = TRUE,
     as.table = TRUE, ...)
\method{plot}{radline}(x, xlab = "Rank", ylab = "Abundance", type = "b", ...)
radlattice(x, BIC = FALSE, ...)
\method{lines}{radfit}(x, ...)
\method{points}{radfit}(x, ...)
as.rad(x)
\method{plot}{rad}(x, xlab = "Rank", ylab = "Abundance", log = "y", ...)
}

\arguments{
  \item{x}{Data frame, matrix or a vector giving species abundances, or an object to
    be plotted.}

  \item{family}{Error distribution (passed to \code{\link{glm}}). All
    alternatives accepting \code{link = "log"} in \code{\link{family}}
    can be used, although not all make sense.}

  \item{object}{A fitted result object.}

  \item{newdata}{Ranks used for ordinations. All models can
    interpolate to non-integer \dQuote{ranks} (although this may be
    approximate), but extrapolation may fail}

  \item{total}{The new total used for predicting abundance. Observed
    total count is used if this is omitted.}

  \item{order.by}{A vector used for ordering sites in plots.}
  \item{BIC}{Use Bayesian Information Criterion, BIC, instead of
    Akaike's AIC. The penalty in BIC is \eqn{k = \log(S)}{k =
      log(S)}  where \eqn{S} is the number of species, whereas AIC uses
    \eqn{k = 2}.}
  \item{model}{Show only the specified model. If missing, AIC is used
    to select the model. The model names (which can be abbreviated)
    are \code{Null}, \code{Preemption}, \code{Lognormal}, \code{Zipf},
    \code{Mandelbrot}. }
  \item{legend}{Add legend of line colours.}
  \item{as.table}{Arrange panels starting from upper left corner (passed
    to \code{\link[lattice]{xyplot}}).}
  \item{xlab,ylab}{Labels for \code{x} and \code{y} axes.}
  \item{type}{Type of the plot, \code{"b"} for plotting both observed points
    and fitted lines, \code{"p"} for only points, \code{"l"} for only
    fitted lines, and \code{"n"} for only setting the frame. }
  \item{log}{Use logarithmic scale for given axis. The default
    \code{log = "y"} gives the traditional plot of community ecology
    where the preemption model is a straight line, and with
    \code{log = "xy"} Zipf model is a straight line. With
    \code{log = ""} both axes are in the original arithmetic scale.}
  \item{\dots}{Other parameters to functions. }
}
\details{

  Rank--Abundance Dominance (RAD) or Dominance/Diversity plots
  (Whittaker 1965) display logarithmic species abundances against
  species rank order. These plots are supposed to be effective in
  analysing types of abundance distributions in communities. These
  functions fit some of the most popular models mainly following
  Wilson (1991).

  Functions \code{rad.null}, \code{rad.preempt}, \code{rad.lognormal},
  \code{rad.zipf} and \code{zipfbrot} fit the individual models
  (described below) for a single vector (row of data frame), and
  function \code{radfit} fits all models. The argument of the function
  \code{radfit} can be either a vector for a single community or a data
  frame where each row represents a distinct community.

  Function \code{rad.null} fits a brokenstick model where the expected
  abundance of species at rank \eqn{r} is \eqn{a_r = (J/S)
  \sum_{x=r}^S (1/x)}{a[r] = J/S sum(from x=r to S) 1/x} (Pielou
  1975), where \eqn{J} is the total number of individuals (site total)
  and \eqn{S} is the total number of species in the community.  This
  gives a Null model where the individuals are randomly distributed
  among observed species, and there are no fitted parameters.
  Function \code{rad.preempt} fits the niche preemption model,
  a.k.a. geometric series or Motomura model, where the expected
  abundance \eqn{a} of species at rank \eqn{r} is \eqn{a_r = J \alpha
  (1 - \alpha)^{r-1}}{a[r] = J*alpha*(1-alpha)^(r-1)}. The only
  estimated parameter is the preemption coefficient \eqn{\alpha} which
  gives the decay rate of abundance per rank.  The niche preemption
  model is a straight line in a RAD plot.  Function
  \code{rad.lognormal} fits a log-Normal model which assumes that the
  logarithmic abundances are distributed Normally, or \eqn{a_r = \exp(
  \log \mu + \log \sigma N)}{a[r] = exp(log(mu) + log(sigma) * N)},
  where \eqn{N} is a Normal deviate.  Function \code{rad.zipf} fits
  the Zipf model \eqn{a_r = J p_1 r^\gamma}{a[r] = J*p1*r^gamma} where
  \eqn{p_1}{p1} is the fitted proportion of the most abundant species,
  and \eqn{\gamma} is a decay coefficient. The Zipf--Mandelbrot model
  (\code{rad.zipfbrot}) adds one parameter: \eqn{a_r = J c (r +
  \beta)^\gamma}{a[r] = J*c*(r+beta)^gamma} after which \eqn{p_1}{p1}
  of the Zipf model changes into a meaningless scaling constant
  \eqn{c}.

  Log-Normal and Zipf models are generalized linear models
  (\code{\link{glm}}) with logarithmic link function.  Zipf--Mandelbrot
  adds one nonlinear parameter to the Zipf model, and is fitted using
  \code{\link{nlm}} for the nonlinear parameter and estimating other
  parameters and log-Likelihood with \code{\link{glm}}. Preemption
  model is fitted as a purely nonlinear model. There are no estimated
  parameters in the Null model.

  The default \code{\link{family}} is \code{poisson} which is
  appropriate only for genuine counts (integers), but other families
  that accept \code{link = "log"} can be used. Families
  \code{\link{Gamma}} or \code{\link{gaussian}} may be appropriate for
  abundance data, such as cover. The best model is selected by
  \code{\link{AIC}}. Therefore \sQuote{quasi} families such as
  \code{\link{quasipoisson}} cannot be used: they do not have
  \code{\link{AIC}} nor log-Likelihood needed in non-linear models.

  All these functions have their own \code{plot} functions. When
  \code{radfit} was applied for a data frame, \code{plot} uses
  \code{\link[lattice]{Lattice}} graphics, and other \code{plot}
  functions use ordinary graphics. The ordinary graphics functions
  return invisibly an \code{\link{ordiplot}} object for observed points,
  and function \code{\link{identify.ordiplot}} can be used to label
  selected species.  Alternatively, \code{radlattice} uses
  \code{\link[lattice]{Lattice}} graphics to display each \code{radfit}
  model of a single site in a separate panel together with their AIC or
  BIC values.

  Function \code{as.rad} is a base function to construct ordered RAD
  data. Its \code{plot} is used by other RAD \code{plot} functions
  which pass extra arguments (such as \code{xlab} and \code{log}) to
  this function. The function returns an ordered vector of taxa
  occurring in a site, and a corresponding attribute \code{"index"} of
  included taxa.

}

\value{

  Functions \code{rad.null}, \code{rad.preempt}, \code{rad.lognormal},
  \code{zipf} and \code{zipfbrot} fit each a single RAD model to a
  single site. The result object has class \code{"radline"} and
  inherits from \code{\link{glm}}, and can be handled by some (but not
  all) \code{\link{glm}} methods.

  Function \code{radfit} fits all models either to a single site or to
  all rows of a data frame or a matrix. When fitted to a single site,
  the function returns an object of class \code{"radfit"} with items
  \code{y} (observed values), \code{\link{family}}, and \code{models}
  which is a list of fitted \code{"radline"} models.  When applied for a
  data frame or matrix, \code{radfit} function returns an object of
  class \code{"radfit.frame"} which is a list of \code{"radfit"}
  objects, each item names by the corresponding row name.

  All result objects (\code{"radline"}, \code{"radfit"},
  \code{"radfit.frame"}) can be accessed with same method functions.
  The following methods are available: \code{\link{AIC}},
  \code{\link{coef}}, \code{\link{deviance}}, \code{\link{logLik}}. In
  addition the fit results can be accessed with \code{\link{fitted}},
  \code{\link{predict}} and \code{\link{residuals}} (inheriting from
  \code{\link{residuals.glm}}). The graphical functions were discussed
  above in Details.

}

\references{
  Pielou, E.C. (1975) \emph{Ecological Diversity}. Wiley & Sons.

  Preston, F.W. (1948) The commonness and rarity of
  species. \emph{Ecology} 29, 254--283.

  Whittaker, R. H. (1965) Dominance and diversity in plant
  communities. \emph{Science} 147, 250--260.

  Wilson, J. B. (1991) Methods for fitting dominance/diversity
  curves. \emph{Journal of Vegetation Science} 2, 35--46.
}
\author{ Jari Oksanen }
\note{
  The RAD models are usually fitted for proportions instead of original
  abundances. However, nothing in these models seems to require division
  of abundances by site totals, and original observations are used in
  these functions. If you wish to use proportions, you must standardize
  your data by site totals, e.g. with \code{\link{decostand}} and use
  appropriate \code{\link{family}} such as \code{\link{Gamma}}.

  The lognormal model is fitted in a standard way, but I do think this is
  not quite correct -- at least it is not equivalent to fitting Normal
  density to log abundances like originally suggested (Preston 1948).

  Some models may fail. In particular, estimation of the Zipf-Mandelbrot
  model is difficult.  If the fitting fails, \code{NA} is returned.

  Wilson (1991) defined preemption model as \eqn{a_r = J p_1 (1
    - \alpha)^{r-1}}{a[r] = J*p1*(1 - alpha)^(r-1)}, where \eqn{p_1}{p1}
    is the fitted proportion of the first species. However, parameter
    \eqn{p_1}{p1} is completely defined by \eqn{\alpha} since the fitted
    proportions must add to one, and therefore I handle preemption as a
    one-parameter model.

    Veiled log-Normal model was included in earlier releases of this
    function, but it was removed because it was flawed: an implicit veil
    line also appears in the ordinary log-Normal. The latest release version
    with \code{rad.veil} was \code{1.6-10}.
}
\seealso{\code{\link{fisherfit}} and \code{\link{prestonfit}}.
  An alternative approach is to use
  \code{\link{qqnorm}} or  \code{\link{qqplot}} with any distribution.
  For controlling graphics: \code{\link[lattice]{Lattice}},
  \code{\link[lattice]{xyplot}}, \code{\link[lattice]{lset}}. }
\examples{
data(BCI)
mod <- rad.lognormal(BCI[5,])
mod
plot(mod)
mod <- radfit(BCI[1,])
## Standard plot overlaid for all models
## Preemption model is a line
plot(mod)
## log for both axes: Zipf model is a line
plot(mod, log = "xy")
## Lattice graphics separately for each model
radlattice(mod)
# Take a subset of BCI to save time and nerves
mod <- radfit(BCI[3:5,])
mod
plot(mod, pch=".")
}
\keyword{ univar }
\keyword{ distribution }

\encoding{UTF-8}
\name{diversity}
\alias{diversity}
\alias{simpson.unb}
\alias{fisher.alpha}
\alias{specnumber}

\title{Ecological Diversity Indices}

\description{ Shannon, Simpson, and Fisher diversity indices and species
  richness.  }

\usage{
diversity(x, index = "shannon", groups, equalize.groups = FALSE,
   MARGIN = 1, base = exp(1))
simpson.unb(x, inverse = FALSE)
fisher.alpha(x, MARGIN = 1, ...)
specnumber(x, groups, MARGIN = 1)
}

\arguments{
  \item{x}{Community data, a matrix-like object or a vector.}
  \item{index}{Diversity index, one of \code{"shannon"},
    \code{"simpson"} or \code{"invsimpson"}.}
  \item{MARGIN}{Margin for which the index is computed. }
  \item{base}{ The logarithm \code{base} used in \code{shannon}.}
  \item{inverse}{Use inverse Simpson similarly as in
     \code{diversity(x, "invsimpson")}.}
  \item{groups}{A grouping factor: if given, finds the diversity of
    communities pooled by the groups.}
  \item{equalize.groups}{Instead of observed abundances, standardize all
    communities to unit total.}
  \item{...}{Parameters passed to the function.}
}
\details{
  Shannon or Shannon--Weaver (or Shannon--Wiener) index is defined as
  \eqn{H' = -\sum_i p_i \log_{b} p_i}{H = -sum p_i log(b) p_i}, where
  \eqn{p_i} is the proportional abundance of species \eqn{i} and \eqn{b}
  is the base of the logarithm.  It is most popular to use natural
  logarithms, but some argue for base \eqn{b = 2} (which makes sense,
  but no real difference).

  Both variants of Simpson's index are based on \eqn{D = \sum p_i^2}{D =
    sum p_i^2}. Choice \code{simpson} returns \eqn{1-D} and
  \code{invsimpson} returns \eqn{1/D}.

  \code{simpson.unb} finds unbiased Simpson indices for discrete
  samples (Hurlbert 1971, eq. 5). These are less sensitive to sample
  size than the basic Simpson indices. The unbiased indices can be only
  calculated for data of integer counts.

  The \code{diversity} function can find the total (or gamma) diversity
  of pooled communities with argument \code{groups}. The average alpha
  diversity can be found as the mean of diversities by the same groups,
  and their difference or ratio is an estimate of beta diversity (see
  Examples). The pooling can be based either on the observed
  abundancies, or all communities can be equalized to unit total before
  pooling; see Jost (2007) for discussion. Functions
  \code{\link{adipart}} and \code{\link{multipart}} provide canned
  alternatives for estimating alpha, beta and gamma diversities in
  hierarchical settings.
 
  \code{fisher.alpha} estimates the \eqn{\alpha} parameter of
  Fisher's logarithmic series (see \code{\link{fisherfit}}). 
  The estimation is possible only for genuine
  counts of individuals.

  None of these diversity indices is usable for empty sampling units
  without any species, but some of the indices can give a numeric
  value. Filtering out these cases is left for the user.
  
  Function \code{specnumber} finds the number of species. With
  \code{MARGIN = 2}, it finds frequencies of species. If \code{groups}
  is given, finds the total number of species in each group (see 
  example on finding one kind of beta diversity with this option).
  
  Better stories can be told about Simpson's index than about
  Shannon's index, and still grander narratives about
  rarefaction (Hurlbert 1971).  However, these indices are all very
  closely related (Hill 1973), and there is no reason to despise one
  more than others (but if you are a graduate student, don't drag me in,
  but obey your Professor's orders). In particular, the exponent of the
  Shannon index is linearly related to inverse Simpson (Hill 1973)
  although the former may be more sensitive to rare species. Moreover,
  inverse Simpson is asymptotically equal to rarefied species richness
  in sample of two individuals, and Fisher's \eqn{\alpha} is very
  similar to inverse Simpson.

}

\value{
  A vector of diversity indices or numbers of species. 
}

\references{
  Fisher, R.A., Corbet, A.S. & Williams, C.B. (1943). The relation
  between the number of species and the number of individuals in a
  random sample of animal population. \emph{Journal of Animal Ecology}
  \strong{12}, 42--58.
  
  Hurlbert, S.H. (1971). The nonconcept of species diversity: a critique
  and alternative parameters. \emph{Ecology} \strong{52}, 577--586.

  Jost, L. (2007) Partitioning diversity into independent alpha and beta
  components. \emph{Ecology} \strong{88}, 2427--2439.
 
}

\seealso{These functions calculate only some basic indices, but many
  others can be derived with them (see Examples). Facilities related to
  diversity are discussed in a \pkg{vegan} vignette that can be read
  with \code{browseVignettes("vegan")}.  Functions \code{\link{renyi}}
  and \code{\link{tsallis}} estimate a series of generalized diversity
  indices. Function \code{\link{rarefy}} finds estimated number of
  species for given sample size. Beta diversity can be estimated with
  \code{\link{betadiver}}. Diversities can be partitioned with
  \code{\link{adipart}} and \code{\link{multipart}}. }

\author{ Jari Oksanen  and  Bob O'Hara (\code{fisher.alpha}).}

\examples{
data(BCI, BCI.env)
H <- diversity(BCI)
simp <- diversity(BCI, "simpson")
invsimp <- diversity(BCI, "inv")
## Unbiased Simpson
unbias.simp <- simpson.unb(BCI)
## Fisher alpha
alpha <- fisher.alpha(BCI)
## Plot all
pairs(cbind(H, simp, invsimp, unbias.simp, alpha), pch="+", col="blue")
## Species richness (S) and Pielou's evenness (J):
S <- specnumber(BCI) ## rowSums(BCI > 0) does the same...
J <- H/log(S)
## beta diversity defined as gamma/alpha - 1:
## alpha is the average no. of species in a group, and gamma is the
## total number of species in the group
(alpha <- with(BCI.env, tapply(specnumber(BCI), Habitat, mean)))
(gamma <- with(BCI.env, specnumber(BCI, Habitat)))
gamma/alpha - 1
## similar calculations with Shannon diversity
(alpha <- with(BCI.env, tapply(diversity(BCI), Habitat, mean))) # average
(gamma <- with(BCI.env, diversity(BCI, groups=Habitat))) # pooled
## additive beta diversity based on Shannon index
gamma-alpha
}
\keyword{ univar }

\name{cca.object}
\alias{cca.object}
\alias{ordConstrained} % a common interface, not really documented
\alias{ordiYbar}
\alias{model.matrix.cca}
\alias{model.matrix.rda}
\alias{model.frame.cca}
\alias{weights.cca}
\alias{weights.rda}
\alias{weights.decorana}
\alias{print.cca}

\title{Result Object from Constrained Ordination}

\description{

  Ordination methods \code{\link{cca}}, \code{\link{rda}},
  \code{\link{dbrda}} and \code{\link{capscale}} return similar result
  objects. All these methods use the same internal function
  \code{ordConstrained}. They differ only in (1) initial
  transformation of the data and in defining inertia, (2) weighting,
  and (3) the use of rectangular rows \eqn{\times}{x} columns data or
  symmetric rows \eqn{\times}{x} rows dissimilarities:
  \code{\link{rda}} initializes data to give variance or correlations
  as inertia, \code{\link{cca}} is based on double-standardized data
  to give Chi-square inertia and uses row and column weights,
  \code{\link{capscale}} maps the real part of dissimilarities to
  rectangular data and performs RDA, and \code{\link{dbrda}} performs
  an RDA-like analysis directly on symmetric dissimilarities.

  Function \code{ordConstrained} returns the same result components
  for all these methods, and the calling function may add some more
  components to the final result. However, you should not access these
  result components directly (using \code{$}): the internal structure
  is not regarded as stable application interface (API), but it can
  change at any release. If you access the results components
  directly, you take a risk of breakage at any \pkg{vegan} release.
  The \pkg{vegan} provides a wide set of accessor functions to those
  components, and these functions are updated when the result object
  changes. This documentation gives an overview of accessor functions
  to the \code{cca} result object.

}

\usage{

ordiYbar(x, model = c("CCA", "CA", "pCCA", "partial", "initial"))
\method{model.frame}{cca}(formula, ...)
\method{model.matrix}{cca}(object, ...)
\method{weights}{cca}(object, display = "sites", ...)

}

\arguments{

  \item{object, x, formula}{A result object from \code{\link{cca}},
    \code{\link{rda}}, \code{\link{dbrda}}, or
    \code{\link{capscale}}. }

  \item{model}{Show constrained (\code{"CCA"}), unconstrained
    (\code{"CA"}) or conditioned \dQuote{partial} (\code{"pCCA"})
    results. In \code{ordiYbar} the value can also be \code{"initial"}
    for the internal working input data, and \code{"partial"} for the
    internal working input data after removing the partial effects.}

  \item{display}{Display either \code{"sites"} or \code{"species"}.}

  \item{\dots}{Other arguments passed to the the function.}

}

\details{

  The internal (\dQuote{working}) form of the dependent (community)
  data can be accessed with function \code{ordiYbar}. The form depends
  on the ordination method: for instance, in \code{\link{cca}} the
  data are weighted and Chi-square transformed, and in
  \code{\link{dbrda}} they are Gower-centred dissimilarities. The
  input data in the original (\dQuote{response}) form can be accessed
  with \code{\link{fitted.cca}} and \code{\link{residuals.cca}}.
  Function \code{\link{predict.cca}} can return either working or
  response data, and also their lower-rank approximations.

  The model matrix of independent data (\dQuote{Constraints} and
  \dQuote{Conditions}) can be extracted with \code{model.matrix}. In
  partial analysis, the function returns a list of design matrices
  called \code{Conditions} and \code{Constraints}. If either component
  was missing, a single matrix is returned. The redundant (aliased)
  terms do not appear in the model matrix. These terms can be found
  with \code{\link{alias.cca}}. Function \code{model.frame} tries to
  reconstruct the data frame from which the model matrices were
  derived. This is only possible if the original model was fitted with
  \code{formula} and \code{data} arguments, and still fails if the
  \code{data} are unavailable.

  The number of observations can be accessed with
  \code{\link{nobs.cca}}, and the residual degrees of freedom with
  \code{\link{df.residual.cca}}. The information on observations with
  missing values can be accessed with \code{\link{na.action}}.  The
  terms and formula of the fitted model can be accessed with
  \code{\link{formula}} and \code{\link{terms}}.

  The weights used in \code{\link{cca}} can be accessed with
  \code{weights}. In unweighted methods (\code{\link{rda}}) all
  weights are equal.

  The ordination results are saved in separate components for partial
  terms, constraints and residual unconstrained ordination. There is
  no guarantee that these components will have the same internal names
  as currently, and you should be cautious when developing scripts and
  functions that directly access these components.

  The constrained ordination algorithm is based on QR decomposition of
  constraints and conditions (environmental data), and the QR
  component is saved separately for partial and constrained
  components.  The QR decomposition of constraints can be accessed
  with \code{\link{qr.cca}}. This will also include the residual
  effects of partial terms (Conditions), and it should be used
  together with \code{ordiYbar(x, "partial")}. The environmental data
  are first centred in \code{rda} or weighted and centred in
  \code{cca}.  The QR decomposition is used in many functions that
  access \code{cca} results, and it can be used to find many items
  that are not directly stored in the object.  For examples, see
  \code{\link{coef.cca}}, \code{\link{coef.rda}},
  \code{\link{vif.cca}}, \code{\link{permutest.cca}},
  \code{\link{predict.cca}}, \code{\link{predict.rda}},
  \code{\link{calibrate.cca}}. See \code{\link{qr}} for other possible
  uses of this component. For instance, the rank of the constraints
  can be found from the QR decomposition.

  The eigenvalues of the solution can be accessed with
  \code{\link{eigenvals.cca}}. Eigenvalues are not evaluated for
  partial component, and they will only be available for constrained
  and residual components.

  The ordination scores are internally stored as (weighted)
  orthonormal scores matrices. These results can be accessed with
  \code{\link{scores.cca}} and \code{\link{scores.rda}} functions. The
  ordination scores are scaled when accessed with \code{\link{scores}}
  functions, but internal (weighted) orthonormal scores can be
  accessed by setting \code{scaling = FALSE}. Unconstrained residual
  component has species and site scores, and constrained component has
  also fitted site scores or linear combination scores for sites and
  biplot scores and centroids for constraint variables. The biplot
  scores correspond to the \code{model.matrix}, and centroids
  are calculated for factor variables when they were used. The scores
  can be selected by defining the axes, and there is no direct way of
  accessing all scores of a certain component. The number of dimensions
  can be assessed from \code{\link{eigenvals}}. In addition, some
  other types can be derived from the results although not saved in
  the results. For instance, regression scores and model coefficients
  can be accessed with \code{\link{scores}} and \code{\link{coef}}
  functions. Partial component will have no scores.

  Distance-based methods (\code{\link{dbrda}}, \code{\link{capscale}})
  can have negative eigenvalues and associated imaginary axis
  scores. There is no way of accessing these imaginary scores.  In
  addition, species scores are initially missing in
  \code{\link{dbrda}} and they are accessory and found after analysis
  in \code{\link{capscale}} (and may be misleading). Function
  \code{\link{sppscores}} can be used to add species scores or replace
  them with more meaningful ones.

}

\seealso{

  The core function is \code{\link{ordConstrained}} which is called by
  \code{\link{cca}}, \code{\link{rda}}, \code{\link{dbrda}} and
  \code{\link{capscale}}. The basic class is \code{"cca"} for all
  methods, and the following functions are defined for this class:
  \Sexpr[results=rd,stage=build]{require(vegan, quietly=TRUE);
  paste0("\\\\code{\\\\link{", methods(class="cca"), "}}", collapse=", ")}.
  Other functions handling \code{"cca"} objects include \code{\link{inertcomp}},
  \code{\link{intersetcor}}, \code{\link{mso}}, \code{\link{ordiresids}},
  \code{\link{ordistep}} and \code{\link{ordiR2step}}.
}

\note{

  Saving of \dQuote{working} dependent (community) data changed in
  \pkg{vegan} version 2.5-0, and you should use \code{ordiYbar}
  function instead of direct access, or your scripts and functions
  will fail (\code{ordiYbar} has been available since \pkg{vegan}
  version 2.4-3, and it works both with the old and current result
  objects).

  The \code{model.matrix} returns the unweighted model matrix also for
  \code{\link{cca}}. Prior to \pkg{vegan} version 2.5-0 it returned
  the weighted model matrix

}

\references{
  Legendre, P. and Legendre, L. (2012) \emph{Numerical Ecology}. 3rd English
  ed. Elsevier.
}
\author{ Jari Oksanen }

\keyword{ models}
\keyword{multivariate}

\encoding{UTF-8}
\name{adipart}
\alias{adipart}
\alias{adipart.default}
\alias{adipart.formula}
\alias{hiersimu}
\alias{hiersimu.default}
\alias{hiersimu.formula}

\title{Additive Diversity Partitioning and Hierarchical Null Model Testing}
\description{
In additive diversity partitioning, mean values of alpha diversity at lower levels of a sampling
hierarchy are compared to the total diversity in the entire data set (gamma diversity).
In hierarchical null model testing, a statistic returned by a function is evaluated
according to a nested hierarchical sampling design (\code{hiersimu}).
}
\usage{
adipart(...)
\method{adipart}{default}(y, x, index=c("richness", "shannon", "simpson"),
    weights=c("unif", "prop"), relative = FALSE, nsimul=99,
    method = "r2dtable", ...)
\method{adipart}{formula}(formula, data, index=c("richness", "shannon", "simpson"),
    weights=c("unif", "prop"), relative = FALSE, nsimul=99,
    method = "r2dtable", ...)

hiersimu(...)
\method{hiersimu}{default}(y, x, FUN, location = c("mean", "median"),
    relative = FALSE, drop.highest = FALSE, nsimul=99,
    method = "r2dtable", ...)
\method{hiersimu}{formula}(formula, data, FUN, location = c("mean", "median"),
    relative = FALSE, drop.highest = FALSE, nsimul=99,
    method = "r2dtable", ...)
}
\arguments{
  \item{y}{A community matrix.}
  \item{x}{A matrix with same number of rows as in \code{y}, columns
    coding the levels of sampling hierarchy. The number of groups within
    the hierarchy must decrease from left to right. If \code{x} is missing,
    function performs an overall decomposition into alpha, beta and
    gamma diversities.}
  \item{formula}{A two sided model formula in the form \code{y ~ x},
    where \code{y} is the community data matrix with samples as rows and
    species as column. Right hand side (\code{x}) must be grouping variables
    referring to levels of sampling hierarchy, terms from right to left
    will be treated as nested (first column is the lowest, last is the
    highest level, at least two levels specified). Interaction terms are
    not allowed.}

  \item{data}{A data frame where to look for variables defined in the
    right hand side of \code{formula}. If missing, variables are looked
    in the global environment.}

  \item{index}{Character, the diversity index to be calculated (see Details).}

  \item{weights}{Character, \code{"unif"} for uniform weights,
    \code{"prop"} for weighting proportional to sample abundances to use
    in weighted averaging of individual alpha values within strata of a
    given level of the sampling hierarchy.}

  \item{relative}{Logical, if \code{TRUE} then alpha and beta diversity
    values are given relative to the value of gamma for function
    \code{adipart}.}

  \item{nsimul}{Number of permutations to use.  If \code{nsimul = 0},
    only the \code{FUN} argument is evaluated.
    It is thus possible to reuse the statistic values
    without a null model.}

  \item{method}{Null model method: either a name (character string) of
    a method defined in \code{\link{make.commsim}} or a
    \code{\link{commsim}} function.
    The default \code{"r2dtable"} keeps row sums and column sums fixed.
    See \code{\link{oecosimu}} for Details and Examples.}

  \item{FUN}{A function to be used by \code{hiersimu}. This must be
    fully specified, because currently other arguments cannot be passed
    to this function via \code{\dots}.}

  \item{location}{Character, identifies which function (mean or median)
    is to be used to calculate location of the samples.}

  \item{drop.highest}{Logical, to drop the highest level or not. When
    \code{FUN} evaluates only arrays with at least 2 dimensions, highest
    level should be dropped, or not selected at all.}

  \item{\dots}{Other arguments passed to functions, e.g. base of
    logarithm for Shannon diversity, or \code{method}, \code{thin} or
    \code{burnin} arguments for \code{\link{oecosimu}}.}
}

\details{

  Additive diversity partitioning means that mean alpha and beta
  diversities add up to gamma diversity, thus beta diversity is measured
  in the same dimensions as alpha and gamma (Lande 1996). This additive
  procedure is then extended across multiple scales in a hierarchical
  sampling design with \eqn{i = 1, 2, 3, \ldots, m} levels of sampling
  (Crist et al. 2003). Samples in lower hierarchical levels are nested
  within higher level units, thus from \eqn{i=1} to \eqn{i=m} grain size
  is increasing under constant survey extent. At each level \eqn{i},
  \eqn{\alpha_i} denotes average diversity found within samples.

  At the highest sampling level, the diversity components are calculated
  as \deqn{\beta_m  = \gamma -  \alpha_m}{beta_m = gamma -  alpha_m} For
  each  lower   sampling  level   as  \deqn{\beta_i  =   \alpha_{i+1}  -
  \alpha_i}{beta_i =  alpha_(i+1) - alpha_i} Then,  the additive partition
  of diversity is \deqn{\gamma  = \alpha_1 + \sum_{i=1}^m \beta_i}{gamma
  = alpha_1 + sum(beta_i)}

  Average alpha components can be weighted uniformly
  (\code{weight="unif"}) to calculate it as simple average, or
  proportionally to sample abundances (\code{weight="prop"}) to
  calculate it as weighted average as follows \deqn{\alpha_i =
  \sum_{j=1}^{n_i} D_{ij} w_{ij}}{alpha_i = sum(D_ij*w_ij)} where
  \eqn{D_{ij}} is the diversity index and \eqn{w_{ij}} is the weight
  calculated for the \eqn{j}th sample at the \eqn{i}th sampling level.

  The implementation of additive diversity partitioning in
  \code{adipart} follows Crist et al. 2003. It is based on species
  richness (\eqn{S}, not \eqn{S-1}), Shannon's and Simpson's diversity
  indices stated as the \code{index} argument.

  The expected diversity components are calculated \code{nsimul} times
  by individual based randomisation of the community data matrix. This
  is done by the \code{"r2dtable"} method in \code{\link{oecosimu}} by
  default.

  \code{hiersimu} works almost in the same way as \code{adipart}, but
  without comparing the actual statistic values returned by \code{FUN}
  to the highest possible value (cf. gamma diversity).  This is so,
  because in most of the cases, it is difficult to ensure additive
  properties of the mean statistic values along the hierarchy.

}
\value{

  An object of class \code{"adipart"} or \code{"hiersimu"} with same
  structure as \code{\link{oecosimu}} objects.

}

\references{

  Crist,   T.O.,   Veech,    J.A.,   Gering,   J.C.   and   Summerville,
  K.S.  (2003).  Partitioning species  diversity  across landscapes  and
  regions:  a hierarchical  analysis of  \eqn{\alpha},  \eqn{\beta}, and
  \eqn{\gamma}-diversity.  \emph{Am. Nat.}, \bold{162}, 734--743.

  Lande, R.  (1996). Statistics and partitioning of species diversity,
  and similarity among multiple communities.  \emph{Oikos}, \bold{76},
  5--13.

}

\author{

  \enc{Péter Sólymos}{Peter Solymos}, \email{solymos@ualberta.ca}}

\seealso{See \code{\link{oecosimu}} for permutation settings and
  calculating \eqn{p}-values. \code{\link{multipart}} for multiplicative
  diversity partitioning.}

\examples{
## NOTE: 'nsimul' argument usually needs to be >= 99
## here much lower value is used for demonstration

data(mite)
data(mite.xy)
data(mite.env)
## Function to get equal area partitions of the mite data
cutter <- function (x, cut = seq(0, 10, by = 2.5)) {
    out <- rep(1, length(x))
    for (i in 2:(length(cut) - 1))
        out[which(x > cut[i] & x <= cut[(i + 1)])] <- i
    return(out)}
## The hierarchy of sample aggregation
levsm <- with(mite.xy, data.frame(
    l1=1:nrow(mite),
    l2=cutter(y, cut = seq(0, 10, by = 2.5)),
    l3=cutter(y, cut = seq(0, 10, by = 5)),
    l4=cutter(y, cut = seq(0, 10, by = 10))))
## Let's see in a map
par(mfrow=c(1,3))
plot(mite.xy, main="l1", col=as.numeric(levsm$l1)+1, asp = 1)
plot(mite.xy, main="l2", col=as.numeric(levsm$l2)+1, asp = 1)
plot(mite.xy, main="l3", col=as.numeric(levsm$l3)+1, asp = 1)
par(mfrow=c(1,1))
## Additive diversity partitioning
adipart(mite, index="richness", nsimul=19)
adipart(mite ~ ., levsm, index="richness", nsimul=19)
## Hierarchical null model testing
## diversity analysis (similar to adipart)
hiersimu(mite, FUN=diversity, relative=TRUE, nsimul=19)
hiersimu(mite ~., levsm, FUN=diversity, relative=TRUE, nsimul=19)
## Hierarchical testing with the Morisita index
morfun <- function(x) dispindmorisita(x)$imst
hiersimu(mite ~., levsm, morfun, drop.highest=TRUE, nsimul=19)
}
\keyword{multivariate}

\name{envfit}
\alias{envfit}
\alias{envfit.default}
\alias{envfit.formula}
\alias{vectorfit}
\alias{factorfit}
\alias{plot.envfit}
\alias{scores.envfit}
\alias{labels.envfit}

\title{Fits an Environmental Vector or Factor onto an Ordination }
\description{
  The function fits environmental vectors or factors onto an
  ordination. The projections of points onto vectors have
  maximum correlation with corresponding environmental variables, and
  the factors show the averages of factor levels.
}
\usage{
\method{envfit}{default}(ord, env, permutations = 999, strata = NULL, 
   choices=c(1,2),  display = "sites", w  = weights(ord, display),
   na.rm = FALSE, ...)
\method{envfit}{formula}(formula, data, ...)
\method{plot}{envfit}(x, choices = c(1,2), labels, arrow.mul, at = c(0,0), 
   axis = FALSE, p.max = NULL, col = "blue", bg, add = TRUE, ...)
\method{scores}{envfit}(x, display, choices, ...)
vectorfit(X, P, permutations = 0, strata = NULL, w, ...)
factorfit(X, P, permutations = 0, strata = NULL, w, ...)
}

\arguments{
  \item{ord}{An ordination object or other structure from which the
    ordination \code{\link{scores}} can be extracted (including a data
    frame or matrix of scores).}
  \item{env}{Data frame, matrix or vector of environmental
    variables. The variables can be of mixed type (factors, continuous
    variables) in data frames.}
  \item{X}{Matrix or data frame of ordination scores.}
  \item{P}{Data frame, matrix or vector of environmental
    variable(s). These must be continuous for \code{vectorfit} and
    factors or characters for \code{factorfit}. }
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices. Set \code{permutations = 0} to skip
    permutations.}
  \item{formula, data}{Model  \code{\link{formula}} and data.  }
  \item{na.rm}{Remove points with missing values in ordination scores
    or environmental variables. The operation is casewise: the whole
    row of data is removed if there is a missing value and 
    \code{na.rm = TRUE}.}
    
  \item{x}{A result object from \code{envfit}. For \code{ordiArrowMul}
    and \code{ordiArrowTextXY} this must be a two-column matrix (or
    matrix-like object) containing the coordinates of arrow heads on
    the two plot axes, and other methods extract such a structure from
    the \code{envfit} results.}
    
  \item{choices}{Axes to plotted.}
    
  \item{labels}{Change plotting labels. The argument should be a list
    with elements \code{vectors} and \code{factors} which give the new
    plotting labels. If either of these elements is omitted, the
    default labels will be used. If there is only one type of elements
    (only \code{vectors} or only \code{factors}), the labels can be
    given as vector. The default labels can be displayed with
    \code{labels} command.}
  \item{arrow.mul}{Multiplier for vector lengths. The arrows are
    automatically scaled similarly as in \code{\link{plot.cca}} if this
    is not given and \code{add = TRUE}.}
  \item{at}{The origin of fitted arrows in the plot.  If you plot arrows
    in other places then origin, you probably have to specify
    \code{arrrow.mul}.}
  \item{axis}{Plot axis showing the scaling of fitted arrows.}
  \item{p.max}{Maximum estimated \eqn{P} value for displayed
    variables.  You must calculate \eqn{P} values with setting
    \code{permutations} to use this option. }
  \item{col}{Colour in plotting.}
  \item{bg}{Background colour for labels. If \code{bg} is set, the
    labels are displayed with \code{\link{ordilabel}} instead of
    \code{text}. See Examples for using semitransparent background.}
  \item{add}{Results added to an existing ordination plot.}
  \item{strata}{An integer vector or factor specifying the strata for
    permutation. If supplied, observations are permuted only within the
    specified strata.}
  \item{display}{In fitting functions these are ordinary site scores or
    linear combination scores 
    (\code{"lc"}) in constrained ordination (\code{\link{cca}},
    \code{\link{rda}}, \code{\link{capscale}}). In \code{scores}
    function they are either \code{"vectors"} or \code{"factors"}
    (with synonyms \code{"bp"} or \code{"cn"}, resp.).}
  \item{w}{Weights used in fitting (concerns mainly \code{\link{cca}}
    and \code{\link{decorana}} results which have nonconstant weights).}

  \item{...}{Parameters passed to \code{\link{scores}}.}
}
\details{
  Function \code{envfit} finds vectors or factor averages of
  environmental variables.  Function \code{plot.envfit} adds these in an
  ordination diagram.  If \code{X} is a \code{\link{data.frame}},
  \code{envfit}
  uses \code{factorfit} for \code{\link{factor}} variables and
  \code{vectorfit} for other variables.  If \code{X} is a matrix or a
  vector, \code{envfit} uses only \code{vectorfit}. Alternatively, the
  model can be defined a simplified model \code{\link{formula}}, where
  the left hand side must be an ordination result object or a matrix of
  ordination scores, and right hand
  side lists the environmental variables. The formula interface can be
  used for easier selection and/or transformation of environmental
  variables. Only the main effects will be analysed even if interaction
  terms were defined in the formula.

  The ordination results are extracted with \code{\link{scores}} and
  all extra arguments are passed to the \code{scores}. The fitted
  models only apply to the results defined when extracting the scores
  when using \code{envfit}. For instance, \code{scaling} in
  constrained ordination (see \code{\link{scores.rda}},
  \code{\link{scores.cca}}) must be set in the same way in
  \code{envfit} and in the \code{plot} or the ordination results (see
  Examples).

  The printed output of continuous variables (vectors) gives the
  direction cosines which are the coordinates of the heads of unit
  length vectors.  In \code{plot} these are scaled by their
  correlation (square root of the column \code{r2}) so that
  \dQuote{weak} predictors have shorter arrows than \dQuote{strong}
  predictors.  You can see the scaled relative lengths using command
  \code{scores}.  The \code{plot}ted (and scaled) arrows are further
  adjusted to the current graph using a constant multiplier: this will
  keep the relative \code{r2}-scaled lengths of the arrows but tries
  to fill the current plot.  You can see the multiplier using
  \code{ordiArrowMul(result_of_envfit)}, and set it with the
  argument \code{arrow.mul}.
  
  Functions \code{vectorfit} and \code{factorfit} can be called directly.
  Function \code{vectorfit} finds directions in the ordination space
  towards which the environmental vectors change most rapidly and to
  which they have maximal correlations with the ordination
  configuration.  Function \code{factorfit} finds averages of ordination
  scores for factor levels. Function \code{factorfit} treats ordered
  and unordered factors similarly.

  If \code{permutations} \eqn{> 0}, the significance of fitted vectors
  or factors is assessed using permutation of environmental variables.
  The goodness of fit statistic is squared correlation coefficient
  (\eqn{r^2}).
  For factors this is defined as \eqn{r^2 = 1 - ss_w/ss_t}, where
  \eqn{ss_w} and \eqn{ss_t} are within-group and total sums of
  squares. See \code{\link{permutations}} for additional details on
  permutation tests in Vegan.

  User can supply a vector of prior weights \code{w}. If the ordination
  object has weights, these will be used. In practise this means that
  the row totals are used as weights with \code{\link{cca}} or
  \code{\link{decorana}} results. If you do not like this, but want to
  give equal weights to all sites, you should set \code{w = NULL}.  The
  fitted vectors are similar to biplot arrows in constrained ordination
  only when fitted to LC scores (\code{display = "lc"}) and you set
  \code{scaling = "species"} (see \code{\link{scores.cca}}).  The
  weighted fitting gives similar results to biplot arrows and class
  centroids in \code{\link{cca}}.

  The lengths of arrows for fitted vectors are automatically adjusted
  for the physical size of the plot, and the arrow lengths cannot be
  compared across plots. For similar scaling of arrows, you must
  explicitly set the \code{arrow.mul} argument in the \code{plot}
  command; see \code{\link{ordiArrowMul}} and
  \code{\link{ordiArrowTextXY}}.
  
  The results can be accessed with \code{scores.envfit} function which
  returns either the fitted vectors scaled by correlation coefficient or
  the centroids of the fitted environmental variables.
}
\value{
  Functions \code{vectorfit} and \code{factorfit} return lists of
  classes \code{vectorfit} and \code{factorfit} which have a
  \code{print} method.  The result object have the following items:

  \item{arrows}{Arrow endpoints from \code{vectorfit}. The arrows are
    scaled to unit length.}
  \item{centroids}{Class centroids from \code{factorfit}.}
  \item{r}{Goodness of fit statistic: Squared correlation coefficient}
  \item{permutations}{Number of permutations.}
  \item{control}{A list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}.}
  \item{pvals}{Empirical P-values for each variable.}

  Function \code{envfit} returns a list of class \code{envfit} with
  results of \code{vectorfit} and \code{envfit} as items.
  
  Function \code{plot.envfit} scales the vectors by correlation.
}

\author{Jari Oksanen.  The permutation test derives from the code
  suggested by Michael Scroggie. }

\note{
  Fitted vectors have become the method of choice in displaying
  environmental variables in ordination.  Indeed, they are the optimal
  way of presenting environmental variables in Constrained
  Correspondence Analysis \code{\link{cca}}, since there they are the
  linear constraints.
  In unconstrained ordination the relation between external variables
  and ordination configuration may be less linear, and therefore other
  methods than arrows may be more useful.  The simplest is to adjust the
  plotting symbol sizes (\code{cex}, \code{\link{symbols}}) by
  environmental variables.
  Fancier methods involve smoothing and regression methods that
  abound in \R, and \code{\link{ordisurf}} provides a wrapper for some.
  }

\seealso{
  A better alternative to vectors may be \code{\link{ordisurf}}.    
  }

\examples{
data(varespec, varechem)
library(MASS)
ord <- metaMDS(varespec)
(fit <- envfit(ord, varechem, perm = 999))
scores(fit, "vectors")
plot(ord)
plot(fit)
plot(fit, p.max = 0.05, col = "red")
## Adding fitted arrows to CCA. We use "lc" scores, and hope
## that arrows are scaled similarly in cca and envfit plots
ord <- cca(varespec ~ Al + P + K, varechem)
plot(ord, type="p")
fit <- envfit(ord, varechem, perm = 999, display = "lc")
plot(fit, p.max = 0.05, col = "red")
## 'scaling' must be set similarly in envfit and in ordination plot
plot(ord, type = "p", scaling = "sites")
fit <- envfit(ord, varechem, perm = 0, display = "lc", scaling = "sites")
plot(fit, col = "red")

## Class variables, formula interface, and displaying the
## inter-class variability with ordispider, and semitransparent
## white background for labels (semitransparent colours are not
## supported by all graphics devices)
data(dune)
data(dune.env)
ord <- cca(dune)
fit <- envfit(ord ~ Moisture + A1, dune.env, perm = 0)
plot(ord, type = "n")
with(dune.env, ordispider(ord, Moisture, col="skyblue"))
with(dune.env, points(ord, display = "sites", col = as.numeric(Moisture),
                      pch=16))
plot(fit, cex=1.2, axis=TRUE, bg = rgb(1, 1, 1, 0.5))
## Use shorter labels for factor centroids
labels(fit)
plot(ord)
plot(fit, labels=list(factors = paste("M", c(1,2,4,5), sep = "")),
   bg = rgb(1,1,0,0.5))
}
\keyword{multivariate }
\keyword{aplot}
\keyword{htest}

\name{MDSrotate}
\alias{MDSrotate}

\title{
  Rotate First MDS Dimension Parallel to an External Variable
}

\description{ Function rotates a multidimensional scaling result so
  that its first dimension is parallel to an external (environmental
  variable). The function can handle the results from
  \code{\link{metaMDS}} or \code{\link{monoMDS}} functions.  }

\usage{
MDSrotate(object, vec, na.rm = FALSE, ...)
}

\arguments{
 
 \item{object}{ A result object from \code{\link{metaMDS}} or
    \code{\link{monoMDS}}.}

  \item{vec}{An environmental variable or a matrix of such
    variables. The number of variables must be lower than the number
    of dimensions, and the solution is rotated to these variables in
    the order they appear in the matrix. Alternatively \code{vec} can
    be a factor, and the solution is rotated to optimal separation of
    factor levels using \code{\link[MASS]{lda}}.}

  \item{na.rm}{ Remove missing values from the continuous variable
    \code{vec}.}

  \item{\dots}{ Other arguments (ignored). }

}

\details{ The orientation and rotation are undefined in multidimensional
  scaling.  Functions \code{\link{metaMDS}} and \code{\link{metaMDS}}
  can rotate their solutions to principal components so that the
  dispersion of the points is highest on the first dimension. Sometimes
  a different rotation is more intuitive, and \code{MDSrotate} allows
  rotation of the result so that the first axis is parallel to a given
  external variable or two first variables are completely in a
  two-dimensional plane etc. If several external variables are supplied,
  they are applied in the order they are in the matrix. First axis is
  rotated to the first supplied variable, and the second axis to the
  second variable. Because variables are usually correlated, the second
  variable is not usually aligned with the second axis, but it is
  uncorrelated to later dimensions. There must be at least one free
  dimension: the number of external variables must be lower than the
  number of dimensions, and all used environmental variables are
  uncorrelated with that free dimension.

  Alternatively the method can rotate to discriminate the levels of a
  factor using linear discriminant analysis
  (\code{\link[MASS]{lda}}). This is hardly meaningful for
  two-dimensional solutions, since all rotations in two dimensions
  have the same separation of cluster levels. However, the function
  can be useful in finding a two-dimensional projection of clusters
  from more than two dimensions. The last dimension will always show
  the residual variation, and for \eqn{k} dimensions, only \eqn{k-1}
  discrimination vectors are used.

}

\value{ Function returns the original ordination result, but with
  rotated scores (both site and species if available), and the
  \code{pc} attribute of scores set to \code{FALSE}.  
}

\note{Rotation to a factor variable is an experimental feature and may
  be removed. The discriminant analysis weights dimensions by their
  discriminating power, but \code{MDSrotate} performs a rigid
  rotation. Therefore the solution may not be optimal.}

\author{
  Jari Oksanen
}

\seealso{
  \code{\link{metaMDS}}, \code{\link{monoMDS}}.
}
\examples{
data(varespec)
data(varechem)
mod <- monoMDS(vegdist(varespec))
mod <- with(varechem, MDSrotate(mod, pH))
plot(mod)
ef <- envfit(mod ~ pH, varechem, permutations = 0)
plot(ef)
ordisurf(mod ~ pH, varechem, knots = 1, add = TRUE)
}

\keyword{multivariate}

\name{ordixyplot}
\alias{ordixyplot}
\alias{ordisplom}
\alias{ordicloud}
\alias{panel.ordi}
\alias{panel.ordiarrows}
\alias{panel.ordi3d}
\alias{prepanel.ordi3d}
\alias{ordilattice.getEnvfit}

\title{ Trellis (Lattice) Plots for Ordination }
\description{
  Functions \code{ordicloud}, \code{ordisplom} and \code{ordixyplot}
  provide an interface to plot ordination results using Trellis
  functions \code{\link[lattice]{cloud}}, \code{\link[lattice]{splom}}
  and \code{\link[lattice]{xyplot}} in package \pkg{lattice}. 
}
\usage{
ordixyplot(x, data = NULL, formula, display = "sites", choices = 1:3,
    panel = "panel.ordi", aspect = "iso", envfit,
    type = c("p", "biplot"), ...)
ordisplom(x, data=NULL, formula = NULL,  display = "sites", choices = 1:3,
    panel = "panel.ordi", type = "p",  ...)
ordicloud(x, data = NULL, formula, display = "sites", choices = 1:3, 
    panel = "panel.ordi3d", prepanel = "prepanel.ordi3d",  ...)
}
\arguments{
  \item{x}{ An ordination result that \code{\link{scores}} knows: any
    ordination result in \pkg{vegan} and many others. }
  \item{data}{ Optional data to amend ordination results. The ordination
    results are found from \code{x}, but you may give here data for other
    variables needed in plots. Typically these are environmental data.}
  \item{formula}{ Formula to define the plots. A default formula will be
    used if this is omitted. The
    ordination axes must be called by the same names as in the
    ordination results (and these names vary among methods). In
    \code{ordisplom}, special character \code{.} refers to the
    ordination result. }
  \item{display}{ The kind of scores: an argument passed to
    \code{\link{scores}}. }
  \item{choices}{ The axes selected: an argument passed to
    \code{\link{scores}}. } 
  \item{panel, prepanel}{ The names of the panel and prepanel
    functions. } 
  \item{aspect}{The aspect of the plot (passed to the \pkg{lattice}
    function).}
  \item{envfit}{Result of \code{\link{envfit}} function displayed in
    \code{ordixyplot}. Please note that this needs same \code{choices}
    as \code{ordixyplot}.}

  \item{type}{The type of plot. This knows the same alternatives as
    \code{\link[lattice]{panel.xyplot}}. In addition \code{ordixyplot}
    has alternatives \code{"biplot"}, \code{"arrows"} and
    \code{"polygon"}. The first displays fitted vectors and factor
    centroids of \code{envfit}, or in constrained ordination, the
    biplot arrows and factor centroids if \code{envfit} is not
    given. The second (\code{type = "arrows"}) is a trellis variant of
    \code{\link{ordiarrows}} and draws arrows by \code{groups}. The
    line parameters are controlled by
    \code{\link[lattice]{trellis.par.set}} for \code{superpose.line},
    and the user can set \code{length}, \code{angle} and \code{ends}
    parameters of \code{\link[lattice]{panel.arrows}}.  The last one
    (\code{type = "polygon"}) draws a polygon enclosing all points in
    a panel over a polygon enclosing all points in the data. The
    overall polygon is controlled by
    \code{\link[lattice]{trellis.par.set}} for \code{plot.polygon},
    and each panel polygon is controlled by \code{superpose.polygon}.}

  \item{\dots}{ Arguments passed to \code{\link{scores}} methods or
    \pkg{lattice} functions.}
}
\details{
  The functions provide an interface to the corresponding \pkg{lattice}
  functions.  All graphical parameters are passed to the \pkg{lattice}
  function so that these graphs are extremely configurable. See
  \code{\link[lattice]{Lattice}} and \code{\link[lattice]{xyplot}},
    \code{\link[lattice]{splom}} and \code{\link[lattice]{cloud}} for
  details, usage and possibilities.

  The argument \code{x} must always be an ordination result. The scores
  are extracted with \pkg{vegan} function \code{\link{scores}} so that
  these functions work with all \pkg{vegan} ordinations and many others.

  The \code{formula} is used to define the models. All functions have
  simple default formulae which are used if \code{formula} is missing. 
  If formula is omitted in \code{ordisplom} it
  produces a pairs plot of ordination axes and variables in
  \code{data}. If \code{formula} is given, ordination results must be
  referred to as \code{.} and other variables by their names. In other
  functions, the formula must use the names of ordination scores and names
  of \code{data}.

  The ordination scores are found from \code{x}, and \code{data} is
  optional. The \code{data} should contain other variables than
  ordination scores to be used in plots. Typically, they are
  environmental variables (typically factors) to define panels or plot
  symbols.

  The proper work is done by the panel function. The layout can be
  changed by defining own panel functions. See
  \code{\link[lattice]{panel.xyplot}},
  \code{\link[lattice]{panel.splom}} and
  \code{\link[lattice]{panel.cloud}} for details and survey of
  possibilities.

  Ordination graphics should always be isometric: same scale should be
  used in all axes. This is controlled (and can be changed) with
  argument \code{aspect} in \code{ordixyplot}. In \code{ordicloud} the
  isometric scaling is defined in \code{panel} and \code{prepanel}
  functions. You must replace these functions if you want to have
  non-isometric scaling of graphs. You cannot select isometric scaling
  in \code{ordisplom}.
}

\value{
  The function return \code{\link[lattice]{Lattice}} objects of class
  \code{"trellis"}.   
}
\author{Jari Oksanen }
 
\seealso{
  \code{\link[lattice]{Lattice}},
  \code{\link[lattice]{xyplot}},
  \code{\link[lattice]{splom}},
  \code{\link[lattice]{cloud}},
  \code{\link[lattice]{panel.splom}},
  \code{\link[lattice]{panel.cloud}}
}
\examples{
data(dune, dune.env)
ord <- cca(dune)
## Pairs plots
ordisplom(ord)
ordisplom(ord, data=dune.env, choices=1:2)
ordisplom(ord, data=dune.env, form = ~ . | Management, groups=Manure)
## Scatter plot with polygons
ordixyplot(ord, data=dune.env, form = CA1 ~ CA2 | Management,
  groups=Manure, type = c("p","polygon"))
## Choose a different scaling
ordixyplot(ord, scaling = "symmetric")
## ... Slices of third axis
ordixyplot(ord, form = CA1 ~ CA2 | equal.count(CA3, 4),
   type = c("g","p", "polygon"))
## Display environmental variables
ordixyplot(ord, envfit = envfit(ord ~ Management + A1, dune.env, choices=1:3))
## 3D Scatter plots
ordicloud(ord, form = CA2 ~ CA3*CA1, groups = Manure, data = dune.env)
ordicloud(ord, form = CA2 ~ CA3*CA1 | Management, groups = Manure,
   data = dune.env, auto.key = TRUE, type = c("p","h"))
}

\keyword{ hplot }

\name{goodness.metaMDS}
\alias{goodness.metaMDS}
\alias{goodness.monoMDS}
\alias{stressplot}
\alias{stressplot.default}
\alias{stressplot.monoMDS}

\title{Goodness of Fit and Shepard Plot for Nonmetric Multidimensional Scaling }
\description{
  Function \code{goodness.metaMDS} find goodness of fit measure for
  points in nonmetric multidimensional scaling, and function
  \code{stressplot} makes a \code{\link[MASS]{Shepard}} diagram.
}
\usage{
\method{goodness}{metaMDS}(object, dis, ...)
\method{stressplot}{default}(object, dis, pch, p.col = "blue", l.col = "red", 
    lwd = 2, ...) 
}

\arguments{
  \item{object}{A result object from \code{\link{metaMDS}}, 
    \code{\link{monoMDS}} or \code{\link[MASS]{isoMDS}}. }
  \item{dis}{Dissimilarities.  This should not be used with
    \code{\link{metaMDS}} or \code{\link{monoMDS}}, but must be used with
    \code{\link[MASS]{isoMDS}}.}
  \item{pch}{Plotting character for points.  Default is dependent on the
    number of points. }
  \item{p.col, l.col}{Point and line colours.}
  \item{lwd}{Line width. For \code{\link{monoMDS}} the default is
    \code{lwd = 1} if more than two lines are drawn, and \code{lwd = 2} 
    otherwise.}
  \item{\dots}{Other parameters to functions, e.g. graphical parameters.}
}
\details{
  Function \code{goodness.metaMDS} finds a goodness of fit statistic
  for observations (points).  This is defined so that sum of squared
  values is equal to squared stress.  Large values indicate poor fit.
  The absolute values of the goodness statistic depend on the
  definition of the stress: \code{\link[MASS]{isoMDS}} expresses
  stress in percents, and therefore its goodness values are 100 times
  higher than those of \code{\link{monoMDS}} which expresses the
  stress as a proportion.

  Function \code{stressplot} draws a Shepard diagram which is a plot
  of ordination distances and monotone or linear fit line against
  original dissimilarities.  In addition, it displays two
  correlation-like statistics on the goodness of fit in the graph.
  The nonmetric fit is based on stress \eqn{S} and defined as \eqn{R^2
  = 1-S^2}{R2 = 1-S*S}.  The \dQuote{linear fit} is the squared
  correlation between fitted values and ordination distances. For
  \code{\link{monoMDS}}, the \dQuote{linear fit} and \eqn{R^2}{R2}
  from \dQuote{stress type 2} are equal.

  Both functions can be used with \code{\link{metaMDS}},
  \code{\link{monoMDS}} and \code{\link[MASS]{isoMDS}}.  The original
  dissimilarities should not be given for \code{\link{monoMDS}} or
  \code{\link{metaMDS}} results (the latter tries to reconstruct the
  dissimilarities using \code{\link{metaMDSredist}} if
  \code{\link[MASS]{isoMDS}} was used as its engine).  With
  \code{\link[MASS]{isoMDS}} the dissimilarities must be given.  In
  either case, the functions inspect that dissimilarities are
  consistent with current ordination, and refuse to analyse
  inconsistent dissimilarities.  Function \code{goodness.metaMDS} is
  generic in \pkg{vegan}, but you must spell its name completely with
  \code{\link[MASS]{isoMDS}} which has no class.
} 

\value{ Function \code{goodness} returns a vector of values. Function
  \code{stressplot} returns invisibly an object with items for
  original dissimilarities, ordination distances and fitted values.  }

\author{Jari Oksanen. }

\seealso{\code{\link{metaMDS}},  \code{\link{monoMDS}}, 
  \code{\link[MASS]{isoMDS}}, \code{\link[MASS]{Shepard}}. Similar
  diagrams for eigenvector ordinations can be drawn with
  \code{\link{stressplot.wcmdscale}}, \code{\link{stressplot.cca}},
  \code{\link{stressplot.rda}} and \code{\link{stressplot.capscale}}.
}

\examples{
data(varespec)
mod <- metaMDS(varespec)
stressplot(mod)
gof <- goodness(mod)
gof
plot(mod, display = "sites", type = "n")
points(mod, display = "sites", cex = 2*gof/mean(gof))
}
\keyword{ multivariate }

\encoding{UTF-8}
\name{renyi}
\alias{renyi}
\alias{plot.renyi}
\alias{renyiaccum}
\alias{plot.renyiaccum}
\alias{persp.renyiaccum}

\title{Renyi and Hill Diversities and Corresponding Accumulation Curves }
\description{
  Function \code{renyi}  find \enc{Rényi}{Renyi} diversities with any
  scale or the corresponding Hill number (Hill 1973).  Function
  \code{renyiaccum} finds these statistics with accumulating sites.
}
\usage{
renyi(x, scales = c(0, 0.25, 0.5, 1, 2, 4, 8, 16, 32, 64, Inf),
   hill = FALSE)
\method{plot}{renyi}(x, ...)
renyiaccum(x, scales = c(0, 0.5, 1, 2, 4, Inf), permutations = 100,
    raw = FALSE, collector = FALSE, subset, ...)
\method{plot}{renyiaccum}(x, what = c("Collector", "mean", "Qnt 0.025", "Qnt 0.975"),
    type = "l",
    ...)
\method{persp}{renyiaccum}(x, theta = 220, col = heat.colors(100), zlim, ...)
}

\arguments{
  \item{x}{Community data matrix or plotting object. }
  \item{scales}{Scales of \enc{Rényi}{Renyi} diversity.}
  \item{hill}{Calculate Hill numbers.}
  \item{permutations}{Usually an integer giving the number
    permutations, but can also be a list of control values for the
    permutations as returned by the function \code{\link[permute]{how}},
    or a permutation matrix where each row gives the permuted indices.}
  \item{raw}{if \code{FALSE} then return summary statistics of
    permutations, and if \code{TRUE} then returns the individual
    permutations.}
  \item{collector}{Accumulate the diversities in the order the sites are
    in the data set, and the collector curve can be plotted against
    summary of permutations. The argument is ignored if \code{raw = TRUE}.
  }
  \item{subset}{logical expression indicating sites (rows) to keep: missing
    values are taken as \code{FALSE}.}
  \item{what}{Items to be plotted.}
  \item{type}{Type of plot, where \code{type = "l"} means lines.}
  \item{theta}{Angle defining the viewing direction (azimuthal) in
    \code{\link{persp}}.}
  \item{col}{Colours used for surface. Single colour will be passed on,
    and vector colours will be
    selected by the midpoint of a rectangle in \code{\link{persp}}. }
  \item{zlim}{Limits of vertical axis.}
  \item{\dots}{Other arguments which are passed to \code{renyi} and
    to graphical functions.}

}
\details{
  Common \code{\link{diversity}} indices are special cases of
  \enc{Rényi}{Renyi} diversity
  \deqn{H_a = \frac{1}{1-a} \log \sum p_i^a}{H.a = 1/(1-a) log
    sum(p^a)}
  where \eqn{a} is a scale parameter, and Hill (1975) suggested to
  use so-called \sQuote{Hill numbers} defined as \eqn{N_a = \exp(H_a)}{N.a =
    exp(H.a)}.  Some Hill numbers are the number of species with
  \eqn{a = 0}, \eqn{\exp(H')}{exp(H')} or the exponent of Shannon
  diversity with \eqn{a = 1}, inverse Simpson with \eqn{a = 2} and
  \eqn{1/ \max(p_i)}{1/max(p)} with \eqn{a = \infty}{a = Inf}. According
  to the theory of diversity ordering, one community can be regarded as
  more diverse than another only if its \enc{Rényi}{Renyi} diversities are all higher
  (\enc{Tóthmérész}{Tothmeresz}  1995).

  The \code{plot} method for \code{renyi} uses \pkg{lattice} graphics,
  and displays the diversity values against each scale in separate panel
  for each site together with minimum, maximum and median values in the
  complete data.

  Function \code{renyiaccum} is similar to \code{\link{specaccum}} but
  finds \enc{Rényi}{Renyi} or Hill diversities at given \code{scales}
  for random permutations of accumulated sites.  Its \code{plot}
  function uses \pkg{lattice} function \code{\link[lattice]{xyplot}}
  to display the accumulation curves for each value of \code{scales}
  in a separate panel.  In addition, it has a \code{persp} method to
  plot the diversity surface against scale and number and
  sites. Similar dynamic graphics can be made with
  \code{\link[vegan3d]{rgl.renyiaccum}} in \pkg{vegan3d} package.

}
\value{
  Function \code{renyi} returns a data frame of selected
  indices.  Function \code{renyiaccum} with argument \code{raw = FALSE}
  returns a three-dimensional array, where the first dimension are the
  accumulated sites, second dimension are the diversity scales, and
  third dimension are the summary statistics \code{mean}, \code{stdev},
  \code{min}, \code{max}, \code{Qnt 0.025} and \code{Qnt 0.975}.  With
  argument \code{raw = TRUE} the statistics on the third dimension are
  replaced with individual permutation results.
}
\references{

   Hill, M.O. (1973). Diversity and evenness: a unifying notation and its
   consequences. \emph{Ecology} 54, 427--473.

   Kindt, R., Van Damme, P., Simons, A.J. (2006). Tree diversity in western
   Kenya: using profiles to characterise richness and
   evenness. \emph{Biodiversity and Conservation} 15, 1253--1270.

   \enc{Tóthmérész}{Tothmeresz}, B. (1995). Comparison of different methods for diversity
   ordering. \emph{Journal of Vegetation Science} 6, 283--290.
}

\author{ Roeland Kindt and Jari Oksanen }

\seealso{\code{\link{diversity}} for diversity indices, and
  \code{\link{specaccum}} for ordinary species accumulation curves, and
  \code{\link[lattice]{xyplot}}, \code{\link{persp}} and
  \code{\link[vegan3d]{rgl.renyiaccum}}.
}
\examples{
data(BCI)
i <- sample(nrow(BCI), 12)
mod <- renyi(BCI[i,])
plot(mod)
mod <- renyiaccum(BCI[i,])
plot(mod, as.table=TRUE, col = c(1, 2, 2))
persp(mod)
}
\keyword{univar }

\name{anosim}
\alias{anosim}
\alias{summary.anosim}
\alias{plot.anosim}

\title{ Analysis of Similarities }
\description{
  Analysis of similarities (ANOSIM) provides a way to test statistically
  whether there is a significant difference between two or more groups
  of sampling units.
}
\usage{
anosim(x, grouping, permutations = 999, distance = "bray", strata = NULL,
    parallel = getOption("mc.cores"))
}

\arguments{
  \item{x}{Data matrix or data frame in which rows are samples and
    columns are response variable(s), or a dissimilarity object or a
    symmetric square matrix of dissimilarities.}
  \item{grouping}{Factor for grouping observations.}
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}
  \item{distance}{Choice of distance metric that measures the
    dissimilarity between two observations. See \code{\link{vegdist}} for
    options.  This will be used if \code{x} was not a dissimilarity
    structure or a symmetric square matrix.}  
  \item{strata}{An integer vector or factor specifying the strata for
    permutation. If supplied, observations are permuted only within the
    specified strata.}
  \item{parallel}{Number of parallel processes or a predefined socket
    cluster.  With \code{parallel = 1} uses ordinary, non-parallel
    processing. The parallel processing is done with \pkg{parallel}
    package.}
}
\details{
  Analysis of similarities (ANOSIM) provides a way to test statistically
  whether there is a significant difference between two or more groups
  of sampling units.  Function \code{anosim} operates directly on a
  dissimilarity matrix.  A suitable dissimilarity matrix is produced by
  functions \code{\link{dist}} or \code{\link{vegdist}}.  The
  method is philosophically allied with NMDS ordination
  (\code{\link{monoMDS}}), in that it uses only the rank order of
  dissimilarity values.

  If two groups of sampling units are really different in their species
  composition, then compositional dissimilarities between the groups
  ought to be greater than those within the groups.  The \code{anosim}
  statistic \eqn{R} is based on the difference of mean ranks between
  groups (\eqn{r_B}) and within groups (\eqn{r_W}):

  \deqn{R = (r_B - r_W)/(N (N-1) / 4)}

  The divisor is chosen so that \eqn{R} will be in the interval
  \eqn{-1 \dots +1}, value \eqn{0} indicating completely random
  grouping.

  The statistical significance of observed \eqn{R} is assessed by
  permuting the grouping vector to obtain the empirical distribution
  of \eqn{R} under null-model.  See \code{\link{permutations}} for
  additional details on permutation tests in Vegan. The distribution
  of simulated values can be inspected with the \code{\link{permustats}}
  function.

  The function has \code{summary} and \code{plot} methods.  These both
  show valuable information to assess the validity of the method:  The
  function assumes that all ranked dissimilarities within groups 
  have about equal median and range.  The \code{plot} method uses
  \code{\link{boxplot}} with options \code{notch=TRUE} and
  \code{varwidth=TRUE}. 
}
\value{
  The function returns a list of class \code{"anosim"} with following
  items: 
  \item{call }{Function call.}
  \item{statistic}{The value of ANOSIM statistic \eqn{R}}
  \item{signif}{Significance from permutation.}
  \item{perm}{Permutation values of \eqn{R}. The distribution of
    permutation values can be inspected with function \code{\link{permustats}}.}
  \item{class.vec}{Factor with value \code{Between} for dissimilarities
    between classes and class name for corresponding dissimilarity
    within class.}
  \item{dis.rank}{Rank of dissimilarity entry.}
  \item{dissimilarity}{The name of the dissimilarity index: the
    \code{"method"} entry of the \code{dist} object.}
  \item{control}{A list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}.}
}
\references{
  Clarke, K. R. (1993). Non-parametric multivariate analysis of changes
  in community structure. \emph{Australian Journal of Ecology} 18,
  117--143.
  
  Warton, D.I., Wright, T.W., Wang, Y. 2012. Distance-based multivariate
  analyses confound location and dispersion effects. \emph{Methods in
  Ecology and Evolution}, 3, 89--101
  
}
\author{Jari Oksanen, with a help from Peter R. Minchin.}
\note{

  The \code{anosim} function can confound the differences between groups
  and dispersion within groups and the results can be difficult to
  interpret (cf. Warton et al. 2012).  The function returns a lot of
  information to ease studying its performance. Most \code{anosim}
  models could be analysed with \code{\link{adonis2}} which seems to be a
  more robust alternative.

}

\seealso{\code{\link{mrpp}} for a similar function using original
dissimilarities instead of their ranks. 
\code{\link{dist}} and \code{\link{vegdist}} for obtaining
  dissimilarities, and \code{\link{rank}} for ranking real values.  For
  comparing dissimilarities against continuous variables, see
  \code{\link{mantel}}. Function \code{\link{adonis2}} is a more robust
  alternative that should preferred. }

\examples{
data(dune)
data(dune.env)
dune.dist <- vegdist(dune)
dune.ano <- with(dune.env, anosim(dune.dist, Management))
summary(dune.ano)
plot(dune.ano)
}
\keyword{multivariate }
\keyword{ nonparametric }
\keyword{ htest }

\name{decorana}
\alias{decorana}
\alias{summary.decorana}
\alias{print.summary.decorana}
\alias{plot.decorana}
\alias{downweight}
\alias{scores.decorana}
\alias{points.decorana}
\alias{text.decorana}

\title{Detrended Correspondence Analysis and Basic Reciprocal Averaging }
\description{
  Performs detrended correspondence analysis and basic reciprocal
  averaging or orthogonal correspondence analysis.
}
\usage{
decorana(veg, iweigh=0, iresc=4, ira=0, mk=26, short=0,
         before=NULL, after=NULL)

\method{plot}{decorana}(x, choices=c(1,2), origin=TRUE,
     display=c("both","sites","species","none"),
     cex = 0.8, cols = c(1,2), type, xlim, ylim, ...)

\method{text}{decorana}(x, display = c("sites", "species"), labels,
     choices = 1:2, origin = TRUE, select,  ...)

\method{points}{decorana}(x, display = c("sites", "species"),
       choices=1:2, origin = TRUE, select, ...)

## deprecated: use 'scores' for scores, 'weights' for weights
\method{summary}{decorana}(object, digits=3, origin=TRUE,
        display=c("both", "species","sites","none"), ...)

\method{print}{summary.decorana}(x, head = NA, tail = head, ...)

downweight(veg, fraction = 5)

\method{scores}{decorana}(x, display="sites", choices=1:4,
       origin=TRUE, tidy=FALSE, ...)
}

\arguments{
  \item{veg}{Community data, a matrix-like object. }
  \item{iweigh}{Downweighting of rare species (0: no). }
  \item{iresc}{Number of rescaling cycles (0: no rescaling). }
  \item{ira}{Type of analysis (0: detrended, 1: basic reciprocal averaging). }
  \item{mk}{Number of segments in rescaling. }
  \item{short}{Shortest gradient to be rescaled. }
  \item{before}{Hill's piecewise transformation: values before transformation. }
  \item{after}{Hill's piecewise transformation: values after
  transformation -- these must correspond to values in \code{before}.}
  \item{x, object}{A \code{decorana} result object.}
  \item{choices}{Axes shown.}
  \item{origin}{Use true origin even in detrended correspondence analysis.}
  \item{display}{Display only sites, only species, both or neither.}
  \item{cex}{Plot character size.}
  \item{cols}{Colours used for sites and species.}
  \item{type}{Type of plots, partial match to \code{"text"},
    \code{"points"} or \code{"none"}.}
  \item{labels}{Optional text to be used instead of row names.}
  \item{select}{Items to be displayed.  This can either be a logical
    vector which is \code{TRUE} for displayed items or a vector of indices
    of displayed items.}
  \item{xlim, ylim}{the x and y limits (min,max) of the plot.}
  \item{digits}{Number of digits in summary output.}
  \item{head, tail}{Number of rows printed from the head and tail of
    species and site scores. Default \code{NA} prints all.}
  \item{fraction}{Abundance fraction where downweighting begins.}
  \item{tidy}{Return scores that are compatible with \CRANpkg{ggplot2}:
    all scores are in a single \code{data.frame}, score type is
    identified by factor variable \code{score} (\code{"sites"},
    \code{"species"}), the names by variable \code{label}. These scores
    are incompatible with conventional \code{plot} functions, but they can
    be used in \pkg{ggplot2}.}
  \item{...}{Other arguments for \code{plot} function.}
  }

\details{
  In late 1970s, correspondence analysis became the method of choice for
  ordination in vegetation science, since it seemed better able to cope 
  with non-linear species responses than principal components
  analysis. However, even correspondence analysis can produce an arc-shaped
  configuration of a single gradient. Mark Hill developed detrended
  correspondence analysis to correct two assumed \sQuote{faults} in 
  correspondence analysis: curvature of straight gradients and packing
  of sites at the ends of the gradient.  

  The curvature is removed by replacing the orthogonalization of axes
  with detrending.  In orthogonalization successive axes are made
  non-correlated, but detrending should remove all systematic dependence
  between axes.  Detrending is performed using a smoothing window on
  \code{mk} segments.  The packing of sites at the ends of the gradient
  is undone by rescaling the axes after extraction.  After rescaling,
  the axis is supposed to be scaled by \sQuote{SD} units, so that the
  average width of Gaussian species responses is supposed to be one over
  whole axis. Other innovations were the piecewise linear transformation
  of species abundances and downweighting of rare species which were
  regarded to have an unduly high influence on ordination axes.

  It seems that detrending actually works by twisting the ordination
  space, so that the results look non-curved in two-dimensional
  projections (\sQuote{lolly paper effect}).  As a result, the points
  usually have an easily recognized triangular or diamond shaped
  pattern, obviously an artefact of detrending.  Rescaling works
  differently than commonly presented, too. \code{decorana} does not
  use, or even evaluate, the widths of species responses.  Instead, it
  tries to equalize the weighted standard deviation of species scores on
  axis segments (parameter \code{mk} has no effect, since
  \code{decorana} finds the segments internally). Function
  \code{\link{tolerance}} returns this internal criterion and can be
  used to assess the success of rescaling.

  The \code{plot} method plots species and site scores.  Classical
  \code{decorana} scaled the axes so that smallest site score was 0 (and
  smallest species score was negative), but \code{summary}, \code{plot}
  and \code{scores} use the true origin, unless \code{origin = FALSE}.

  In addition to proper eigenvalues, the function reports
  \sQuote{decorana values} in detrended analysis. These \sQuote{decorana
  values} are the values that the legacy code of \code{decorana} returns
  as eigenvalues. They are estimated during iteration, and describe the
  joint effects of axes and detrending. The \sQuote{decorana values} are
  estimated before rescaling and do not show its effect on
  eigenvalues. The proper eigenvalues are estimated after extraction of
  the axes and they are the ratio of weighted sum of squares of site and
  species scores even in detrended and rescaled solutions. These
  eigenvalues are estimated for each axis separately, but they are not
  additive, because higher \code{decorana} axes can show effects already
  explained by prior axes. \sQuote{Additive eigenvalues} are cleansed
  from the effects of prior axes, and they can be assumed to add up to
  total inertia (scaled Chi-square). For proportions and cumulative
  proportions explained you can use \code{\link{eigenvals.decorana}}.

}
\value{
  \code{decorana} returns an object of class \code{"decorana"}, which
  has \code{print}, \code{summary}, \code{scores}, \code{plot},
  \code{points} and \code{text} methods, and support functions
  \code{\link{eigenvals}}, \code{\link{bstick}}, \code{screeplot},
  \code{predict} and \code{\link{tolerance}}. \code{downweight} is an
  independent function that can also be used with other methods than
  \code{decorana}.
}

\references{
  Hill, M.O. and Gauch, H.G. (1980). Detrended correspondence analysis:
  an improved ordination technique. \emph{Vegetatio} \strong{42},
  47--58.

  Oksanen, J. and Minchin, P.R. (1997). Instability of ordination
  results under changes in input data order: explanations and
  remedies. \emph{Journal of Vegetation Science} \strong{8}, 447--454.
}
\author{Mark O. Hill wrote the original Fortran code, the \R port was by
  Jari Oksanen. }
\note{
  \code{decorana} uses the central numerical engine of the
  original Fortran code (which is in the public domain), or about 1/3 of
  the original program.  I have tried to implement the original
  behaviour, although a great part of preparatory steps were written in
  \R language, and may differ somewhat from the original code. However,
  well-known bugs are corrected and strict criteria used (Oksanen &
  Minchin 1997). 

  Please note that there really is no need for piecewise transformation
  or even downweighting within \code{decorana}, since there are more
  powerful and extensive alternatives in \R, but these options are
  included for compliance with the original software.  If a different
  fraction of abundance is needed in downweighting, function
  \code{downweight} must be applied before \code{decorana}.  Function
  \code{downweight} indeed can be applied prior to correspondence
  analysis, and so it can be used together with \code{\link{cca}}, too.

  Github package \pkg{natto} has an \R{} implementation of
  \code{decorana} which allows easier inspection of the
  algorithm and also easier development of the function.
}

\seealso{

  For unconstrained ordination, non-metric multidimensional scaling in
  \code{\link{monoMDS}} may be more robust (see also
  \code{\link{metaMDS}}).  Constrained (or \sQuote{canonical})
  correspondence analysis can be made with \code{\link{cca}}.
  Orthogonal correspondence analysis can be made with \code{decorana} or
  \code{\link{cca}}, but the scaling of results vary (and the one in
  \code{decorana} corresponds to \code{scaling = "sites"} and \code{hill
  = TRUE} in \code{\link{cca}}.).  See \code{\link{predict.decorana}}
  for adding new points to an ordination.

}

\examples{
data(varespec)
vare.dca <- decorana(varespec)
vare.dca
plot(vare.dca)

### the detrending rationale:
gaussresp <- function(x,u) exp(-(x-u)^2/2)
x <- seq(0,6,length=15) ## The gradient
u <- seq(-2,8,len=23)   ## The optima
pack <- outer(x,u,gaussresp)
matplot(x, pack, type="l", main="Species packing")
opar <- par(mfrow=c(2,2))
plot(scores(prcomp(pack)), asp=1, type="b", main="PCA")
plot(scores(decorana(pack, ira=1)), asp=1, type="b", main="CA")
plot(scores(decorana(pack)), asp=1, type="b", main="DCA")
plot(scores(cca(pack ~ x), dis="sites"), asp=1, type="b", main="CCA")

### Let's add some noise:
noisy <- (0.5 + runif(length(pack)))*pack
par(mfrow=c(2,1))
matplot(x, pack, type="l", main="Ideal model")
matplot(x, noisy, type="l", main="Noisy model")
par(mfrow=c(2,2))
plot(scores(prcomp(noisy)), type="b", main="PCA", asp=1)
plot(scores(decorana(noisy, ira=1)), type="b", main="CA", asp=1)
plot(scores(decorana(noisy)), type="b", main="DCA", asp=1)
plot(scores(cca(noisy ~ x), dis="sites"), asp=1, type="b", main="CCA")
par(opar)
}
\keyword{ multivariate }

\name{vegemite}
\alias{vegemite}
\alias{tabasco}
\alias{coverscale}

\title{Display Compact Ordered Community Tables }
\description{
  Functions \code{vegemite} and \code{tabasco} display compact
  community tables.  Function \code{vegemite} prints text tables where
  species are rows, and each site takes only one column without
  spaces.  Function \code{tabasco} provides interface for
  \code{\link{heatmap}} for a colour \code{\link{image}} of the
  data. The community table can be ordered by explicit indexing, by
  environmental variables or results from an ordination or cluster
  analysis.
}
\usage{
vegemite(x, use, scale, sp.ind, site.ind, zero=".", select ,...)
tabasco(x, use, sp.ind = NULL, site.ind = NULL, select,
    Rowv = TRUE, Colv = TRUE, labRow = NULL, labCol = NULL,
    scale, col = heat.colors(12), ...)
coverscale(x, scale=c("Braun.Blanquet", "Domin", "Hult", "Hill", "fix","log"),
           maxabund, character = TRUE)
}

\arguments{
  \item{x}{Community data. }
  \item{use}{Either a vector, or an object from \code{cca},
    \code{decorana} \emph{etc.} or \code{hclust} or a
    \code{\link{dendrogram}} for ordering sites and species.}
  \item{sp.ind, site.ind}{Species and site indices. In \code{tabasco},
    these can also be \code{\link{hclust}} tree,
    \code{\link[cluster]{agnes}} clusterings or
    \code{\link{dendrogram}}s. }
  \item{zero}{Character used for zeros. }
  \item{select}{Select a subset of sites.  This can be a logical vector
    (\code{TRUE} for selected sites), or a vector of indices of selected
    sites.  The order of indices does not influence results, but you
    must specify \code{use} or \code{site.ind} to reorder sites.
  }
  \item{Rowv, Colv}{Re-order dendrograms for the rows (sites) or
    columns (species) of \code{x}.  If the \code{Rowv = TRUE}, row
    dendrograms are ordered by the first axis of correspondence
    analysis, and when \code{Colv = TRUE} column dendrograms by the
    weighted average (\code{\link{wascores}}) of the row order.
    Alternatively, the arguments can be vectors that are used to
    reorder the dendrogram. }

  \item{labRow, labCol}{character vectors with row and column labels
    used in the \code{\link{heatmap}} instead of the default. NB., the
    input matrix is transposed so that row labels will be used for data
    columns.}

  \item{scale}{In \code{vegemite} and \code{coverscale}: cover scale
    used (can be abbreviated). In \code{tabasco}: scaling of colours
    in \code{\link{heatmap}}. The alternatives of \code{coverscale}
    can be used in \code{tabasco}, and in addition \code{"column"} or
    \code{"row"} scale columns or rows to equal maxima (NB., these
    refer to the transposed data of the \code{\link{heatmap}}), while
    \code{"none"} uses original values. }

  \item{col}{A vector of colours used for above-zero abundance values.}

  \item{maxabund}{Maximum abundance used with \code{scale = "log"}.
    Data maximum in the \code{select}ed subset will be used if this is
    missing.}

  \item{character}{Return character codes suitable for
    \code{vegemite}. If \code{FALSE}, returns corresponding
    integers.}

  \item{\dots}{Arguments passed to \code{coverscale} (i.e., \code{maxabund}) in
    \code{vegemite} and to \code{\link{heatmap}} in \code{tabasco}.}
}
\details{
  The function \code{vegemite} prints a traditional community table.
  The display is transposed, so that species are in rows and sites in
  columns.  The table is printed in compact form: only one character
  can be used for abundance, and there are no spaces between
  columns. Species with no occurrences are dropped from the table.

  Function \code{tabasco} produces a similar table as \code{vegemite}
  using \code{\link{heatmap}}, where abundances are coded by
  colours. The function scales the abundances to equal intervals for
  colour palette, but either rows or columns can be scaled to equal
  maxima, or the \code{coverscale} class systems can be used. The
  function can also display dendrograms for sites (columns) or species
  if these are given as an argument (\code{use} for sites,
  \code{sp.ind} for species).

  The parameter \code{use} will be used to re-order output. The
  \code{use} can be a vector or an object from \code{\link{hclust}} or
  \code{\link[cluster]{agnes}}, a \code{\link{dendrogram}} or any
  ordination result recognized by \code{\link{scores}} (all ordination
  methods in \pkg{vegan} and some of those not in \pkg{vegan}). The
  \code{\link{hclust}}, \code{\link[cluster]{agnes}} and
  \code{\link{dendrogram}} must be for sites. The dendrogram is
  displayed above the sites in \code{tabasco}, but is not shown in
  \code{vegemite}.  No dendrogram for species is displayed, except
  when given in \code{sp.ind}.

  If \code{use} is a vector, it is used for ordering sites.  If
  \code{use} is an object from ordination, both sites and species are
  arranged by the first axis (provided that results are available both
  also for species).  When \code{use} is an object from
  \code{\link{hclust}}, \code{\link[cluster]{agnes}} or a
  \code{\link{dendrogram}}, the sites are ordered similarly as in the
  cluster dendrogram.  Function \code{tabasco} re-orders the dendrogram
  if \code{Rowv = TRUE} or \code{Rowv} is a vector. Such re-ordering is
  not available for \code{vegemite}, but it can be done by hand using
  \code{\link{reorder.dendrogram}} or \code{\link{reorder.hclust}}.
  Please note that \code{\link{dendrogram}} and \code{\link{hclust}}
  reordering can differ: unweighted means of merged branches are used in
  \code{\link{dendrogram}}, but weighted means (= means of leaves of the
  cluster) are used in \code{\link{reorder.hclust}}.  In all cases where
  species scores are missing, species are ordered by their weighted
  averages (\code{\link{wascores}}) on site order.

  Species and sites can be ordered explicitly giving their indices or
  names in parameters \code{sp.ind} and \code{site.ind}.  If these are
  given, they take precedence over \code{use}. A subset of sites can
  be displayed using argument \code{select}, but this cannot be used
  to order sites, but you still must give \code{use} or
  \code{site.ind}.  However, \code{tabasco} makes two exceptions:
  \code{site.ind} and \code{select} cannot be used when \code{use} is
  a dendrogram (clustering result). In addition, the \code{sp.ind} can
  be an \code{\link{hclust}} tree, \code{\link[cluster]{agnes}}
  clustering or a \code{\link{dendrogram}}, and in that case the
  dendrogram is plotted on the left side of the
  \code{\link{heatmap}}. Phylogenetic trees cannot be directly used,
  \code{\link[ape]{as.hclust.phylo}} (package \pkg{ape}) can transform
  these to \code{\link{hclust}} trees.

  If \code{scale} is given, \code{vegemite} calls \code{coverscale} to
  transform percent cover scale or some other scales into traditional
  class scales used in vegetation science (\code{coverscale} can be
  called directly, too). Function \code{tabasco} can also use these
  traditional class scales, but it treats the transformed values as
  corresponding integers.  Braun-Blanquet and Domin scales are
  actually not strict cover scales, and the limits used for codes
  \code{r} and \code{+} are arbitrary.  Scale \code{Hill} may be
  inappropriately named, since Mark O. Hill probably never intended
  this as a cover scale.  However, it is used as default \dQuote{cut levels}
  in his \code{TWINSPAN}, and surprisingly many users stick to this
  default, and this is a \emph{de facto} standard in publications.
  All traditional scales assume that values are cover percentages with
  maximum 100.  However, non-traditional alternative \code{log} can be
  used with any scale range.  Its class limits are integer powers of
  1/2 of the maximum (argument \code{maxabund}), with \code{+} used
  for non-zero entries less than 1/512 of the maximum (\code{log}
  stands alternatively for logarithmic or logical).  Scale \code{fix}
  is intended for \dQuote{fixing} 10-point scales: it truncates scale values
  to integers, and replaces 10 with \code{X} and positive values below
  1 with \code{+}.  } \value{ The functions are used mainly to display
  a table, but they return (invisibly) a list with items:
  \item{species}{Ordered species indices} \item{sites}{Ordered site
  indices}

  These items can be used as arguments \code{sp.ind} and \code{site.ind}
  to reproduce the table. In addition to the proper table, \code{vegemite}
  prints the numbers of species and sites and the name of the used cover
  scale at the end.
}
\references{ The cover scales are presented in many textbooks of vegetation
  science; I used:

  Shimwell, D.W. (1971) \emph{The Description and Classification of
  Vegetation}. Sidgwick & Jackson.
}
\author{Jari Oksanen}

\seealso{\code{\link{cut}} and \code{\link{approx}} for making your
  own \sQuote{cover scales} for \code{vegemite}.  Function
  \code{tabasco} is based on \code{\link{heatmap}} which in turn is
  based on \code{\link{image}}. Both functions order species with
  weighted averages using \code{\link{wascores}}.  }

\note{

  The name \code{vegemite} was chosen because the output is so
  compact, and the \code{tabasco} because it is just as compact, but
  uses heat colours.

}
\examples{
data(varespec)
## Print only more common species
freq <- apply(varespec > 0, 2, sum)
vegemite(varespec, scale="Hult", sp.ind = freq > 10)
## Order by correspondence analysis, use Hill scaling and layout:
dca <- decorana(varespec)
vegemite(varespec, dca, "Hill", zero="-")
## Show one class from cluster analysis, but retain the ordering above
clus <- hclust(vegdist(varespec))
cl <- cutree(clus, 3)
sel <- vegemite(varespec, use=dca, select = cl == 3, scale="Br")
## Re-create previous
vegemite(varespec, sp=sel$sp, site=sel$site, scale="Hult")
## Re-order clusters by ordination
clus <- as.dendrogram(clus)
clus <- reorder(clus, scores(dca, choices=1, display="sites"), agglo.FUN = mean)
vegemite(varespec, clus, scale = "Hult")

## Abundance values have such a wide range that they must be rescaled
tabasco(varespec, dca, scale="Braun")

## Classification trees for species
data(dune, dune.taxon)
taxontree <- hclust(taxa2dist(dune.taxon))
plotree <- hclust(vegdist(dune), "average")
## Automatic reordering of clusters
tabasco(dune, plotree, sp.ind = taxontree)
## No reordering of taxonomy
tabasco(dune, plotree, sp.ind = taxontree, Colv = FALSE)
## Species cluster: most dissimilarity indices do a bad job when
## comparing rare and common species, but Raup-Crick makes sense
sptree <- hclust(vegdist(t(dune), "raup"), "average")
tabasco(dune, plotree, sptree)
}
\keyword{print}
\keyword{manip}
\keyword{hplot}

\name{pcnm}
\alias{pcnm}
\alias{scores.pcnm}
\title{ Principal Coordinates of Neighbourhood Matrix }
\description{
  This function computed classical PCNM by the principal coordinate
  analysis of a truncated distance matrix. These are commonly used to
  transform (spatial) distances to rectangular data that suitable for
  constrained ordination or regression.
}
\usage{
pcnm(dis, threshold, w, dist.ret = FALSE)
}

\arguments{
  \item{dis}{ A distance matrix. }
  \item{threshold}{ A threshold value or truncation distance. If
    missing, minimum distance giving connected network will be
    used. This is found as the longest distance in the minimum spanning
    tree of \code{dis}. }
  \item{w}{Prior weights for rows.}
  \item{dist.ret}{Return the distances used to calculate the PCNMs.}
}

\details{
  Principal Coordinates of Neighbourhood Matrix (PCNM) map distances
  between rows onto rectangular matrix on rows using a truncation
  threshold for long distances (Borcard & Legendre 2002). If original
  distances were Euclidean distances in two dimensions (like normal
  spatial distances), they could be mapped onto two dimensions if there
  is no truncation of distances. Because of truncation, there will be a
  higher number of principal coordinates. The selection of truncation
  distance has a huge influence on the PCNM vectors. The default is to
  use the longest distance to keep data connected. The distances above
  truncation threshold are given an arbitrary value of 4 times
  threshold.  For regular data, the first PCNM vectors show a wide scale
  variation and later PCNM vectors show smaller scale variation (Borcard
  & Legendre 2002), but for irregular data the interpretation is not as
  clear.

  The PCNM functions are used to express distances in rectangular form
  that is similar to normal explanatory variables used in, e.g.,
  constrained ordination (\code{\link{rda}}, \code{\link{cca}} and
  \code{\link{capscale}}) or univariate regression (\code{\link{lm}})
  together with environmental variables (row weights should be supplied
  with \code{\link{cca}}; see Examples). This is regarded as a more
  powerful method than forcing rectangular environmental data into
  distances and using them in partial mantel analysis
  (\code{\link{mantel.partial}}) together with geographic distances
  (Legendre et al. 2008, but see Tuomisto & Ruokolainen 2008).

  The function is based on \code{pcnm} function in Dray's unreleased
  \pkg{spacemakeR} package. The differences are that the current
  function uses \code{\link{spantree}} as an internal support
  function. The current function also can use prior weights for rows by
  using weighted metric scaling of \code{\link{wcmdscale}}. The use of
  row weights allows finding orthonormal PCNMs also for correspondence
  analysis (e.g., \code{\link{cca}}).
  }

\value{
  A list of the following elements:
  \item{values }{Eigenvalues obtained by the principal coordinates
    analysis.}
  \item{vectors }{Eigenvectors obtained by the principal coordinates
    analysis. They are scaled to unit norm. The vectors can be extracted
    with \code{scores} function. The default is to return all PCNM vectors,
    but argument \code{choices} selects the given vectors.}
 \item{threshold}{Truncation distance.}
 \item{dist}{The distance matrix where values above \code{threshold}
    are replaced with arbitrary value of four times the
    threshold. String \code{"pcnm"} is added to the \code{method}
    attribute, and new attribute \code{threshold} is added to the
    distances. This is returned only when \code{dist.ret = TRUE}.  }
}

\references{
  Borcard D. and Legendre P. (2002) All-scale spatial analysis of
  ecological data by means of principal coordinates of neighbour
  matrices. \emph{Ecological Modelling} \bold{153}, 51--68.

  Legendre, P., Borcard, D and Peres-Neto, P. (2008) Analyzing or
  explaining beta diversity? Comment. \emph{Ecology} \bold{89},
  3238--3244.

  Tuomisto, H. & Ruokolainen, K. (2008) Analyzing or explaining beta
  diversity? A reply. \emph{Ecology} \bold{89}, 3244--3256.
}

\author{Jari Oksanen, based on the code of Stephane Dray.}
\seealso{ \code{\link{spantree}}. }
\examples{
## Example from Borcard & Legendre (2002)
data(mite.xy)
pcnm1 <- pcnm(dist(mite.xy))
op <- par(mfrow=c(1,3))
## Map of PCNMs in the sample plot
ordisurf(mite.xy, scores(pcnm1, choi=1), bubble = 4, main = "PCNM 1")
ordisurf(mite.xy, scores(pcnm1, choi=2), bubble = 4, main = "PCNM 2")
ordisurf(mite.xy, scores(pcnm1, choi=3), bubble = 4, main = "PCNM 3")
par(op)
## Plot first PCNMs against each other
ordisplom(pcnm1, choices=1:4)
## Weighted PCNM for CCA
data(mite)
rs <- rowSums(mite)/sum(mite)
pcnmw <- pcnm(dist(mite.xy), w = rs)
ord <- cca(mite ~ scores(pcnmw))
## Multiscale ordination: residual variance should have no distance
## trend
msoplot(mso(ord, mite.xy))
}
\keyword{ spatial }
\keyword{ multivariate }

\name{taxondive}
\alias{taxondive}
\alias{summary.taxondive}
\alias{plot.taxondive}
\alias{taxa2dist}

\title{ Indices of Taxonomic Diversity and Distinctness }
\description{
  Function finds indices of taxonomic diversity and distinctness,
  which are averaged taxonomic distances among species or individuals in
  the community (Clarke & Warwick 1998, 2001)
}
\usage{
taxondive(comm, dis, match.force = FALSE)
taxa2dist(x, varstep = FALSE, check = TRUE, labels)
}

\arguments{
  \item{comm}{Community data. }
  \item{dis}{Taxonomic distances among taxa in \code{comm}. This should
    be a \code{\link{dist}} object or a symmetric square matrix. }
  \item{match.force}{Force matching of column names in \code{comm} and
    labels in \code{dis}. If \code{FALSE}, matching only happens when
    dimensions differ, and in that case the species must be in identical
    order in both.}
  \item{x}{Classification table with a row for each species or other
    basic taxon, and columns
    for identifiers of its classification at higher levels.}
  \item{varstep}{Vary step lengths between successive levels
    relative to proportional loss of the number of distinct classes.}
  \item{check}{If \code{TRUE}, remove all redundant levels which are
    different for all rows or constant for all rows and regard each row
    as a different basal taxon (species). If \code{FALSE} all
    levels are retained and basal taxa (species) also must be coded as
    variables (columns). You will get a warning if species are not
    coded, but you can ignore this if that was your intention.}
  \item{labels}{The \code{labels} attribute of taxonomic distances. Row
    names will be used if this is not given. Species will be matched by
    these labels in \code{comm} and \code{dis} in \code{taxondive} if
    these have different dimensions.}
}
\details{
  Clarke & Warwick (1998, 2001) suggested several alternative indices of
  taxonomic diversity or distinctness. Two basic indices are called
  taxonomic diversity (\eqn{\Delta}) and distinctness (\eqn{\Delta^*}):
  \tabular{c}{
    \eqn{\Delta = (\sum \sum_{i<j} \omega_{ij} x_i x_j)/(n (n-1) / 2)} \cr
    \eqn{\Delta^* = (\sum \sum_{i<j} \omega_{ij} x_i x_j)/(\sum \sum_{i<j} x_i x_j) }
  }
  The equations give the index value for a single site, and summation
  goes over species \eqn{i} and \eqn{j}. Here \eqn{\omega} are taxonomic
  distances among taxa, and \eqn{x} are species abundances, and \eqn{n}
  is the total abundance for a site. 
  With presence/absence  data both indices reduce to the same index
  \eqn{\Delta^+}, and for this index Clarke & Warwick (1998) also have
  an estimate of its standard deviation. Clarke & Warwick (2001) 
  presented two new indices: \eqn{s\Delta^+} is the product of species
  richness and \eqn{\Delta^+}, and index of variation in
  taxonomic distinctness (\eqn{\Lambda^+}) defined as
  \tabular{c}{
    \eqn{\Lambda^+ = (\sum \sum_{i<j} \omega_{ij}^2)/(n (n-1) / 2) - (\Delta^+)^2}
  }

  The \code{dis} argument must be species dissimilarities. These must be
  similar to dissimilarities produced by \code{\link{dist}}. It is
  customary to have integer steps of taxonomic hierarchies, but other
  kind of dissimilarities can be used, such as those from phylogenetic
  trees or genetic differences.  Further, the \code{dis} need not be
  taxonomic, but other species classifications can be used. 

  Function \code{taxa2dist} can produce a suitable \code{dist} object
  from a classification table. Each species (or basic taxon) corresponds
  to a row of the classification table, and columns give the
  classification at different levels. With \code{varstep = FALSE} the
  successive levels will be separated by equal steps, and with
  \code{varstep = TRUE} the step length is relative to the proportional
  decrease in the number of classes (Clarke & Warwick 1999).
  With \code{check = TRUE}, the function removes classes which are distinct for all
  species or which combine all species into one class, and assumes that
  each row presents a distinct basic taxon. The function scales
  the distances so that longest path length between
  taxa is 100 (not necessarily when \code{check = FALSE}). 
  
  Function \code{plot.taxondive} plots \eqn{\Delta^+} against Number of
  species, together with expectation and its approximate 2*sd
  limits. Function \code{summary.taxondive} finds the \eqn{z} values and
  their significances from Normal distribution for \eqn{\Delta^+}.
}
\value{
Function returns an object of class \code{taxondive} with following items:
  \item{Species }{Number of species for each site.}
  \item{D, Dstar, Dplus, SDplus, Lambda}{\eqn{\Delta}, \eqn{\Delta^*},
    \eqn{\Delta^+},  \eqn{s\Delta^+} and \eqn{\Lambda^+} 
    for each site.}
  \item{sd.Dplus}{Standard deviation of \eqn{\Delta^+}.}
  \item{ED, EDstar, EDplus}{Expected values of corresponding
    statistics.}

  Function \code{taxa2dist} returns an object of class \code{"dist"}, with
  an attribute \code{"steps"} for the step lengths between successive levels.
}
\references{
 Clarke, K.R & Warwick, R.M. (1998) A taxonomic distinctness index and
 its statistical properties. \emph{Journal of Applied Ecology} 35,
 523--531.

 Clarke, K.R. & Warwick, R.M. (1999) The taxonomic distinctness measure
 of biodiversity: weighting of step lengths between hierarchical
 levels. \emph{Marine Ecology Progress Series} 184: 21--29.

 Clarke, K.R. & Warwick, R.M. (2001) A further biodiversity index
 applicable to species lists: variation in taxonomic
 distinctness. \emph{Marine Ecology Progress Series} 216, 265--278.
}
\author{ Jari Oksanen }
\note{
  The function is still preliminary and may change. The scaling of
  taxonomic dissimilarities influences the results. If you multiply
  taxonomic distances (or step lengths) by a constant, the values of all
  Deltas will be multiplied with the same constant, and the value of
  \eqn{\Lambda^+} by the square of the constant.
}
\seealso{\code{\link{diversity}}. }
\examples{
## Preliminary: needs better data and some support functions
data(dune)
data(dune.taxon)
# Taxonomic distances from a classification table with variable step lengths.
taxdis <- taxa2dist(dune.taxon, varstep=TRUE)
plot(hclust(taxdis), hang = -1)
# Indices
mod <- taxondive(dune, taxdis)
mod
summary(mod)
plot(mod)
}

\keyword{ univar }

\name{dune}
\alias{dune}
\alias{dune.env}
\docType{data}
\title{Vegetation and Environment in Dutch Dune Meadows. }
\usage{
  data(dune)
  data(dune.env)
}
\description{
  The dune meadow vegetation data, \code{dune}, has cover class values
  of 30 species on 20 sites. The corresponding environmental data frame
  \code{dune.env} has following entries: } 

\format{
  \code{dune} is a data frame of observations of 30 species at 20
  sites. The species names are abbreviated to 4+4 letters (see
  \code{\link{make.cepnames}}). The following names are changed from
  the original source (Jongman et al. 1987): \emph{Leontodon
  autumnalis} to \emph{Scorzoneroides}, and \emph{Potentilla
  palustris} to \emph{Comarum}.
  
  \code{dune.env} is a data frame of 20 observations on the following
  5 variables:
  \describe{
    \item{A1:}{a numeric vector of thickness of soil A1 horizon.}
    \item{Moisture:}{an ordered factor with levels: \code{1} < \code{2} <
      \code{4} < \code{5}.}
    %\item{Moisture}{\code{1} < \code{2} < \code{4} < \code{5}}
    \item{Management:}{a factor with levels: \code{BF} (Biological
      farming), \code{HF} (Hobby farming), \code{NM} (Nature
      Conservation Management), and \code{SF} (Standard Farming).}
    %\item{Management}{\code{BF}: Biological Farming  }
    %\item{Management}{\code{HF}: Hobby Farming }
    %\item{Management}{\code{NM}: Nature Conservation Management }
    %\item{Management}{\code{SF}: Standard Farming }
    \item{Use:}{an ordered factor of land-use with levels: \code{Hayfield}
      < \code{Haypastu} < \code{Pasture}.}
    %\item{Use}{\code{Hayfield} < \code{Haypastu} < \code{Pasture}}
    \item{Manure:}{an ordered factor with levels: \code{0} < \code{1} <
      \code{2} < \code{3} < \code{4}.} 
    %\item{Manure}{\code{0} < \code{1} < \code{2} < \code{3} < \code{4}}
  }
}
\source{
  Jongman, R.H.G, ter Braak, C.J.F & van Tongeren,
  O.F.R. (1987). \emph{Data Analysis in Community and Landscape
    Ecology}. Pudoc, Wageningen.
}
\examples{
data(dune)
data(dune.env)
}
\keyword{datasets}

\name{scores}
\alias{scores}
\alias{scores.default}
\alias{scores.lda}

\title{ Get Species or Site Scores from an Ordination }

\description{
  Function to access either species or site scores for specified axes
  in some ordination methods. The \code{scores} function is generic in
  \pkg{vegan}, and \pkg{vegan} ordination functions have their own
  \code{scores} functions that are documented separately with the
  method (see e.g. \code{\link{scores.cca}},
  \code{\link{scores.metaMDS}}, \code{\link{scores.decorana}}). This
  help file documents the default \code{scores} method that is only
  used for non-\pkg{vegan} ordination objects.
}

\usage{
\method{scores}{default}(x, choices, display=c("sites", "species"), ...)
}

\arguments{
  \item{x}{ An ordination result. }
  \item{choices}{ Ordination axes.  If missing, default method returns all axes.}
  \item{display}{ Partial match to access scores for \code{sites} or
    \code{species}.}
  \item{...}{ Other parameters (unused). }
}
\details{
  Function \code{scores} is a generic method in \pkg{vegan}. Several
  \pkg{vegan} functions have their own \code{scores} methods with their
  own defaults and with some new arguments. This help page describes
  only the default method. For other methods, see, e.g.,
  \code{\link{scores.cca}}, \code{\link{scores.rda}},
  \code{\link{scores.decorana}}.

  All \pkg{vegan} ordination functions should have a \code{scores}
  method which should be used to extract the scores instead of
  directly accessing them. Scaling and transformation of scores should
  also happen in the \code{scores} function. If the \code{scores}
  function is available, the results can be plotted using
  \code{\link{ordiplot}}, \code{\link{ordixyplot}} etc., and the
  ordination results can be compared in \code{\link{procrustes}}
  analysis.
  
  The \code{scores.default} function is used to extract scores from
  non-\pkg{vegan} ordination results.  Many standard ordination
  methods of libraries do not have a specific \code{class}, and no
  specific method can be written for them.  However,
  \code{scores.default} guesses where some commonly used functions
  keep their site scores and possible species scores.

  If \code{x} is a matrix, \code{scores.default} returns the chosen
  columns of that matrix, ignoring whether species or sites were
  requested (do not regard this as a bug but as a feature, please).
  Currently the function seems to work at least for \code{\link[MASS]{isoMDS}},
  \code{\link{prcomp}}, \code{\link{princomp}} and some \pkg{ade4} objects. 
  It may work in other cases or fail mysteriously.
}
\value{
  The function returns a matrix of scores.
}
\author{Jari Oksanen }

\seealso{
  Specific \code{scores} functions include (but are not limited to)
  \code{\link{scores.cca}}, \code{\link{scores.rda}},
  \code{\link{scores.decorana}}, \code{\link{scores.envfit}},
  \code{\link{scores.metaMDS}}, \code{\link{scores.monoMDS}} and
  \code{\link{scores.pcnm}}.  These have somewhat different interface
  -- \code{\link{scores.cca}} in particular -- but all work with
  keywords \code{display="sites"} and return a matrix. However, they
  may also return a list of matrices, and some other \code{scores}
  methods will have quite different arguments.
}

\examples{
data(varespec)
vare.pca <- prcomp(varespec)
scores(vare.pca, choices=c(1,2))
}
\keyword{ multivariate }

\name{humpfit}
\alias{humpfit-deprecated}
\alias{summary.humpfit}
\alias{lines.humpfit}
\alias{plot.humpfit}
\alias{points.humpfit}
\alias{predict.humpfit}
\alias{profile.humpfit}

\title{No-interaction Model for Hump-backed Species Richness vs. Biomass }
\description{
  Function \code{humpfit} fits a no-interaction model for species
  richness vs. biomass data (Oksanen 1996). This is a null model that
  produces a hump-backed response as an artifact of plant size and
  density.

  This function is \code{\link{.Deprecated}}: It was very rarely used,
  and it was problematic to fit robustly. The function was transferred
  to the \pkg{natto} package and is still available from
  \url{https://github.com/jarioksa/natto/}.
}
\usage{
humpfit(mass, spno, family = poisson, start)
\method{summary}{humpfit}(object, ...)
\method{predict}{humpfit}(object, newdata = NULL, ...) 
\method{plot}{humpfit}(x, xlab = "Biomass", ylab = "Species Richness", lwd = 2, 
    l.col = "blue", p.col = 1, type = "b", ...)
\method{points}{humpfit}(x, ...)
\method{lines}{humpfit}(x, segments=101,  ...)
\method{profile}{humpfit}(fitted, parm = 1:3, alpha = 0.01, maxsteps = 20, del = zmax/5, ...)
}

\arguments{
  \item{mass}{Biomass. }
  \item{spno}{Species richness.}
  \item{start}{Vector of starting values for all three parameters.}
  \item{family}{Family of error distribution. Any \code{\link{family}}
    can be used, but the link function is always Fisher's diversity
    model, and other \code{link} functions are silently ignored. }
  \item{x, object, fitted}{Result object of \code{humpfit}}
  \item{newdata}{Values of \code{mass} used in \code{predict}. The
    original data values are used if missing.}
  \item{xlab,ylab}{Axis labels in \code{plot}}
  \item{lwd}{Line width}
  \item{l.col, p.col}{Line and point colour in \code{plot}}
  \item{type}{Type of \code{plot}: \code{"p"} for observed points,
    \code{"l"} for fitted lines, \code{"b"} for both, and \code{"n"} for
    only setting axes.}
  \item{segments}{Number of segments used for fitted lines.}
  \item{parm}{Profiled parameters.}
  \item{alpha, maxsteps, del}{Parameters for profiling range and density.}
  \item{...}{Other parameters to functions.}
}
\details{
  The no-interaction model assumes that the humped species richness
  pattern along biomass gradient is an artifact of plant size and
  density (Oksanen 1996). For low-biomass sites, it assumes that plants
  have a fixed size, and biomass increases with increasing number of
  plants. When the sites becomes crowded, the number of plants and
  species richness reaches the maximum. Higher biomass is reached by
  increasing the plant size, and then the number of plants and species
  richness will decrease. At biomasses below the hump, plant number and
  biomass are linearly related, and above the hump, plant number is
  proportional to inverse squared biomass. The number of plants is
  related to the number of species by the relationship (\code{link}
  function) from Fisher's log-series (Fisher et al. 1943).

  The parameters of the model are:
  \enumerate{
    \item \code{hump}: the location of the hump on the biomass gradient.
    \item \code{scale}: an arbitrary multiplier to translate the biomass
    into virtual number of plants.
    \item \code{alpha}: Fisher's \eqn{\alpha}{alpha} to translate the
    virtual number of plants into number of species.
  }
  The parameters \code{scale} and \code{alpha} are intermingled and this
  function should not be used for estimating Fisher's
  \eqn{\alpha}{alpha}.  Probably the only meaningful and interesting
  parameter is the location of the \code{hump}.

  Function may be very
  difficult to fit and easily gets trapped into local solutions, or
  fails with non-Poisson families, and function \code{profile} should be
  used to inspect the fitted models. If you have loaded \code{package}
  \pkg{MASS}, you can use functions
  \code{\link[MASS]{plot.profile}},
  \code{\link[MASS]{pairs.profile}} for graphical inspection of the
  profiles, and \code{\link[MASS]{confint.profile.glm}} for the profile
  based confidence intervals. 

  The original model intended to show that there is no need to
  speculate about \dQuote{competition} and \dQuote{stress}
  (Al-Mufti et al. 1977),
  but humped response can be produced as an artifact of using fixed
  plot size for varying plant sizes and densities.  } \value{ The
  function returns an object of class \code{"humpfit"} inheriting from
  class \code{"glm"}. The result object has specific \code{summary},
  \code{predict}, \code{plot}, \code{points} and \code{lines}
  methods. In addition, it can be accessed by the following methods
  for \code{\link{glm}} objects: \code{\link{AIC}},
  \code{\link{extractAIC}}, \code{\link{deviance}},
  \code{\link{coef}}, \code{\link{residuals.glm}} (except 
  \code{type = "partial"}), \code{\link{fitted}}, and perhaps some others. 
  In addition, function \code{\link[ellipse]{ellipse.glm}} (package
  \pkg{ellipse}) can be used to draw approximate confidence ellipses
  for pairs of parameters, if the normal assumptions look appropriate.
  } 
\references{ 
  Al-Mufti, M.M., Sykes, C.L, Furness, S.B., Grime, J.P
  & Band, S.R. (1977) A quantitative analysis of shoot phenology and
  dominance in herbaceous vegetation. \emph{Journal of Ecology}
  65,759--791.
  
  Fisher, R.A., Corbet, A.S. & Williams, C.B. (1943) The relation
  between the number of species and the number of individuals in a
  random sample of of an animal population. \emph{Journal of Animal
    Ecology} 12, 42--58.
  
  Oksanen, J. (1996) Is the humped relationship between species richness
  and biomass an artefact due to plot size? \emph{Journal of Ecology}
  84, 293--295.
}
\author{ Jari Oksanen }
\note{
  The function is a replacement for the original \code{GLIM4} function
  at the archive of Journal of Ecology.  There the function was
  represented as a mixed \code{\link{glm}} with one non-linear
  parameter (\code{hump}) and a special one-parameter link function from
  Fisher's log-series.  The current function directly applies non-linear
  maximum likelihood fitting using function \code{\link{nlm}}.  Some
  expected problems with the current approach are:
  \itemize{
    \item The function is discontinuous at \code{hump} and may be
    difficult to optimize in some cases (the lines will always join, but
    the derivative jumps).
    \item The function does not try very hard to find sensible starting
    values and can fail. The user may supply starting values in
    argument \code{start} if fitting fails.
    \item The estimation is unconstrained, but both \code{scale} and
    \code{alpha} should always be positive.  Perhaps they should be
    fitted as logarithmic. Fitting \code{\link{Gamma}}
    \code{\link{family}} models might become easier, too.
    }
}


\seealso{\code{\link{fisherfit}}, \code{\link[MASS]{profile.glm}},
  \code{\link[MASS]{confint.glm}}. }
\examples{
##
## Data approximated from Al-Mufti et al. (1977)
##
\donttest{
mass <- c(140,230,310,310,400,510,610,670,860,900,1050,1160,1900,2480)
spno <- c(1,  4,  3,  9, 18, 30, 20, 14,  3,  2,  3,  2,  5,  2)
sol <- humpfit(mass, spno)
summary(sol) # Almost infinite alpha...
plot(sol)
# confint is in MASS, and impicitly calls profile.humpfit.
# Parameter 3 (alpha) is too extreme for profile and confint, and we
# must use only "hump" and "scale".
library(MASS)
plot(profile(sol, parm=1:2))
confint(sol, parm=c(1,2))
}
}
\keyword{models }
\keyword{regression }
\keyword{nonlinear}

\encoding{UTF-8}
\name{permat}
\alias{permatfull}
\alias{permatswap}
\alias{summary.permat}
\alias{print.summary.permat}
\alias{print.permat}
\alias{plot.permat}
\alias{lines.permat}
\alias{as.ts.permat}
\alias{as.mcmc.permat}

\title{Matrix Permutation Algorithms for Presence-Absence and Count Data}

\description{ Individual (for count data) or incidence (for
presence-absence data) based null models can be generated for
community level simulations. Options for preserving characteristics of
the original matrix (rows/columns sums, matrix fill) and
restricted permutations (based on strata) are discussed in the
Details section.}

\usage{
permatfull(m, fixedmar = "both", shuffle = "both", strata = NULL, 
    mtype = "count", times = 99, ...)
permatswap(m, method = "quasiswap", fixedmar="both", shuffle = "both",
    strata = NULL, mtype = "count", times = 99, 
    burnin = 0, thin = 1, ...)
\method{print}{permat}(x, digits = 3, ...)
\method{summary}{permat}(object, ...)
\method{print}{summary.permat}(x, digits = 2, ...)
\method{plot}{permat}(x, type = "bray", ylab, xlab, col, lty,
    lowess = TRUE, plot = TRUE, text = TRUE, ...)
\method{lines}{permat}(x, type = "bray", ...)
\method{as.ts}{permat}(x, type = "bray", ...)
\method{as.mcmc}{permat}(x)
}
\arguments{
  \item{m}{A community data matrix with plots (samples) as rows and
    species (taxa) as columns.} 
  \item{fixedmar}{character, stating which of the row/column sums should
    be preserved (\code{"none", "rows", "columns", "both"}).} 
  \item{strata}{Numeric vector or factor with length same as
    \code{nrow(m)} for grouping rows within strata for restricted
    permutations. Unique values or levels are used.} 
  \item{mtype}{Matrix data type, either \code{"count"} for count data,
    or \code{"prab"} for presence-absence type incidence data.} 
  \item{times}{Number of permuted matrices.} 
  \item{method}{Character for method used for the swap algorithm
    (\code{"swap"}, \code{"tswap"}, \code{"quasiswap"},
    \code{"backtrack"}) as described for function
    \code{\link{make.commsim}}. If \code{mtype="count"} the
    \code{"quasiswap"}, \code{"swap"}, \code{"swsh"} and
    \code{"abuswap"} methods are available (see details).} 
  \item{shuffle}{Character, indicating whether individuals
    (\code{"ind"}), samples (\code{"samp"}) or both (\code{"both"})
    should be shuffled, see details.} 
  \item{burnin}{Number of null communities discarded before proper
    analysis in sequential (\code{"swap", "tswap"}) methods.} 
  \item{thin}{Number of discarded permuted matrices between two
    evaluations in sequential (\code{"swap", "tswap"}) methods.} 
  \item{x, object}{Object of class \code{"permat"}} 
  \item{digits}{Number of digits used for rounding.}
  \item{ylab, xlab, col, lty}{graphical parameters for the \code{plot}
    method.} 
  \item{type}{Character, type of plot to be displayed: \code{"bray"} for
    Bray-Curtis dissimilarities, \code{"chisq"} for Chi-squared values.} 
  \item{lowess, plot, text}{Logical arguments for the \code{plot}
    method, whether a locally weighted regression curve should be drawn,
    the plot should be drawn, and statistic values should be printed on
    the plot.} 
  \item{\dots}{Other arguments passed to \code{\link{simulate.nullmodel}} 
    or methods.}
}

\details{
  The function \code{permatfull} is useful when matrix fill is
  allowed to vary, and matrix type is \code{count}.  The \code{fixedmar}
  argument is used to set constraints for permutation.  If \code{none}
  of the margins are fixed, cells are randomised within the matrix.  If
  \code{rows} or \code{columns} are fixed, cells within rows or columns
  are randomised, respectively.  If \code{both} margins are fixed, the
  \code{\link{r2dtable}} function is used that is based on Patefield's
  (1981) algorithm. For presence absence data, matrix fill should be
  necessarily fixed, and \code{permatfull} is a wrapper for the function
  \code{\link{make.commsim}}. The \code{r00, r0, c0, quasiswap}
  algorithms of \code{\link{make.commsim}} are used for \code{"none",
  "rows", "columns", "both"} values of the \code{fixedmar} argument,
  respectively

  The \code{shuffle} argument only have effect if the \code{mtype =
  "count"} and \code{permatfull} function is used with \code{"none",
  "rows", "columns"} values of \code{fixedmar}. All other cases for
  count data are individual based randomisations. The \code{"samp"} and
  \code{"both"} options result fixed matrix fill. The \code{"both"}
  option means that individuals are shuffled among non zero cells
  ensuring that there are no cell with zeros as a result, then cell
  (zero and new valued cells) are shuffled.

  The function \code{permatswap} is useful when with matrix fill
  (i.e. the proportion of empty cells) and row/columns sums should be
  kept constant. \code{permatswap} uses different kinds of swap
  algorithms, and row and columns sums are fixed in all cases.  For
  presence-absence data, the \code{swap} and \code{tswap} methods of
  \code{\link{make.commsim}} can be used.  For count data, a special
  swap algorithm ('swapcount') is implemented that results in permuted
  matrices with fixed marginals and matrix fill at the same time.

  The 'quasiswapcount' algorithm (\code{method="quasiswap"} and
  \code{mtype="count"}) uses the same trick as Carsten Dormann's
  \code{\link[bipartite]{swap.web}} function in the package
  \pkg{bipartite}. First, a random matrix is generated by the
  \code{\link{r2dtable}} function retaining row and column sums. Then
  the original matrix fill is reconstructed by sequential steps to
  increase or decrease matrix fill in the random matrix. These steps are
  based on swapping 2x2 submatrices (see 'swapcount' algorithm for
  details) to maintain row and column totals. This algorithm generates
  independent matrices in each step, so \code{burnin} and \code{thin}
  arguments are not considered. This is the default method, because this
  is not sequential (as \code{swapcount} is) so independence of subsequent
  matrices does not have to be checked.

  The \code{swapcount} algorithm (\code{method="swap"} and
  \code{mtype="count"}) tries to find 2x2 submatrices (identified by 2
  random row and 2 random column indices), that can be swapped in order
  to leave column and row totals and fill unchanged. First, the
  algorithm finds the largest value in the submatrix that can be swapped
  (\eqn{d}) and whether in diagonal or antidiagonal way. Submatrices
  that contain values larger than zero in either diagonal or
  antidiagonal position can be swapped. Swap means that the values in
  diagonal or antidiagonal positions are decreased by \eqn{d}, while
  remaining cells are increased by \eqn{d}. A swap is made only if fill
  doesn't change. This algorithm is sequential, subsequent matrices are
  not independent, because swaps modify little if the matrix is
  large. In these cases many burnin steps and thinning is needed to get
  independent random matrices. Although this algorithm is implemented in
  C, large burnin and thin values can slow it down
  considerably. WARNING: according to simulations, this algorithm seems
  to be biased and non random, thus its use should be avoided!

  The algorithm \code{"swsh"} in the function \code{permatswap} is a
  hybrid algorithm. First, it makes binary quasiswaps to keep row and
  column incidences constant, then non-zero values are modified
  according to the \code{shuffle} argument (only \code{"samp"} and
  \code{"both"} are available in this case, because it is applied only
  on non-zero values). It also recognizes the \code{fixedmar}
  argument which cannot be \code{"both"} (\pkg{vegan} versions <= 2.0
  had this algorithm with \code{fixedmar = "none"}).

  The algorithm \code{"abuswap"} produces two kinds of null models
  (based on \code{fixedmar="columns"} or \code{fixedmar="rows"}) as
  described in Hardy (2008; randomization scheme 2x and 3x,
  respectively).  These preserve column and row occurrences, and column
  or row sums at the same time. (Note that similar constraints
  can be achieved by the non sequential \code{"swsh"} algorithm
  with \code{fixedmar} argument set to \code{"columns"} or
  \code{"rows"}, respectively.)

  Constraints on row/column sums, matrix fill, total sum and sums within
  strata can be checked by the \code{summary} method. \code{plot} method
  is for visually testing the randomness of the permuted matrices,
  especially for the sequential swap algorithms. If there are any
  tendency in the graph, higher \code{burnin} and \code{thin} values can
  help for sequential methods.  New lines can be added to existing plot
  with the \code{lines} method.

  Unrestricted and restricted permutations: if \code{strata} is
  \code{NULL}, functions perform unrestricted permutations. Otherwise,
  it is used for restricted permutations. Each strata should contain at
  least 2 rows in order to perform randomization (in case of low row
  numbers, swap algorithms can be rather slow). If the design is not
  well balanced (i.e. same number of observations within each stratum),
  permuted matrices may be biased because same constraints are forced on
  submatrices of different dimensions. This often means, that the number
  of potential permutations will decrease with their dimensions.  So the
  more constraints we put, the less randomness can be expected.

  The \code{plot} method is useful for graphically testing for trend and
  independence of permuted matrices. This is especially important when
  using sequential algorithms (\code{"swap", "tswap", "abuswap"}).

  The \code{as.ts} method can be used to extract Bray-Curtis
  dissimilarities or Chi-squared values as time series. This can further
  used in testing independence (see Examples). The method \code{as.mcmc}
  is useful for accessing diagnostic tools available in the \pkg{coda}
  package.  }

\value{Functions \code{permatfull} and \code{permatswap} return an
  object of class \code{"permat"} containing the the function call
  (\code{call}), the original data matrix used for permutations
  (\code{orig}) and a list of permuted matrices with length \code{times}
  (\code{perm}).

  The \code{summary} method returns various statistics as a list
  (including mean Bray-Curtis dissimilarities calculated pairwise among
  original and permuted matrices, Chi-square statistics, and check
  results of the constraints; see Examples). Note that when
  \code{strata} is used in the original call, summary calculation may
  take longer.

  The \code{plot} creates a plot as a side effect.

  The \code{as.ts} method returns an object of class \code{"ts"}.  }


\references{ Original references for presence-absence algorithms are
  given on help page of \code{\link{make.commsim}}.

  Hardy, O. J. (2008) Testing the spatial phylogenetic structure of
  local communities: statistical performances of different null models
  and test statistics on a locally neutral community. Journal of Ecology
  96, 914--926. 

  Patefield, W. M. (1981) Algorithm AS159. An efficient method of
  generating r x c tables with given row and column totals.  
  Applied Statistics 30, 91--97.
}

\author{\enc{Péter Sólymos}{Peter Solymos},
\email{solymos@ualberta.ca} and Jari Oksanen}

\seealso{ For other functions to permute matrices:
\code{\link{make.commsim}}, \code{\link{r2dtable}},
\code{\link{sample}}, \code{\link[bipartite]{swap.web}}.

For the use of these permutation algorithms: \code{\link{oecosimu}},
\code{\link{adipart}}, \code{\link{hiersimu}}.

For time-series diagnostics: \code{\link{Box.test}},
\code{\link{lag.plot}}, \code{\link{tsdiag}}, \code{\link{ar}},
\code{\link{arima}} 

For underlying low level implementation:
\code{\link{commsim}} and \code{\link{nullmodel}}.}

\examples{
## A simple artificial community data matrix.
m <- matrix(c(
    1,3,2,0,3,1,
    0,2,1,0,2,1,
    0,0,1,2,0,3,
    0,0,0,1,4,3
    ), 4, 6, byrow=TRUE)
## Using the quasiswap algorithm to create a 
## list of permuted matrices, where
## row/columns sums and matrix fill are preserved:
x1 <- permatswap(m, "quasiswap")
summary(x1)
## Unrestricted permutation retaining
## row/columns sums but not matrix fill:
x2 <- permatfull(m)
summary(x2)
## Unrestricted permutation of presence-absence type
## not retaining row/columns sums:
x3 <- permatfull(m, "none", mtype="prab")
x3$orig  ## note: original matrix is binarized!
summary(x3)
## Restricted permutation,
## check sums within strata:
x4 <- permatfull(m, strata=c(1,1,2,2))
summary(x4)

## NOTE: 'times' argument usually needs to be >= 99
## here much lower value is used for demonstration

## Not sequential algorithm
data(BCI)
a <- permatswap(BCI, "quasiswap", times=19)
## Sequential algorithm
b <- permatswap(BCI, "abuswap", fixedmar="col",
    burnin=0, thin=100, times=19)
opar <- par(mfrow=c(2,2))
plot(a, main="Not sequential")
plot(b, main="Sequential")
plot(a, "chisq")
plot(b, "chisq")
par(opar)
## Extract Bray-Curtis dissimilarities
## as time series
bc <- as.ts(b)
## Lag plot
lag.plot(bc)
## First order autoregressive model
mar <- arima(bc, c(1,0,0))
mar
## Ljung-Box test of residuals
Box.test(residuals(mar))
## Graphical diagnostics
tsdiag(mar)
}

\keyword{multivariate}
\keyword{datagen}

\name{permutations}
\alias{permutations}

\title{Permutation tests in Vegan}
\description{
  From version 2.2-0, \pkg{vegan} has significantly improved access to
  restricted permutations which brings it into line with those offered
  by Canoco. The permutation designs are modelled after the permutation
  schemes of Canoco 3.1 (ter Braak, 1990).

  \pkg{vegan} currently provides for the following features within
  permutation tests:
  \enumerate{
    \item{Free permutation of \emph{DATA}, also known as randomisation,}
    \item{Free permutation of \emph{DATA} within the levels of a
      grouping variable,}
    \item{Restricted permutations for line transects or time series,}
    \item{Permutation of groups of samples whilst retaining the
      within-group ordering,}
    \item{Restricted permutations for spatial grids,}
    \item{Blocking, samples are never permuted \emph{between} blocks,
      and}
    \item{Split-plot designs, with permutation of whole plots, split
      plots, or both.}
  }
  Above, we use \emph{DATA} to mean either the observed data themselves
  or some function of the data, for example the residuals of an
  ordination model in the presence of covariables.

  These capabilities are provided by functions from the \pkg{permute}
  package. The user can request a particular type of permutation by
  supplying the \code{permutations} argument of a function with an
  object returned by \code{\link{how}}, which defines how samples should
  be permuted. Alternatively, the user can simply specify the required
  number of permutations and a simple randomisation procedure will be
  performed. Finally, the user can supply a matrix of permutations (with
  number of rows equal to the number of permutations and number of
  columns equal to the number of observations in the data) and
  \pkg{vegan} will use these permutations instead of generating new
  permutations.

  The majority of functions in \pkg{vegan} allow for the full range of
  possibilities outlined above. Exceptions include
  \code{\link{kendall.post}} and \code{\link{kendall.global}}.

  The Null hypothesis for the first two types of permutation test listed
  above assumes free exchangeability of \emph{DATA} (within the levels
  of the grouping variable, if specified). Dependence between
  observations, such as that which arises due to spatial or temporal
  autocorrelation, or more-complicated experimental designs, such as
  split-plot designs, violates this fundamental assumption of the test
  and requires more complex restricted permutation test designs. It is
  these designs that are available via the \pkg{permute} package and to
  which \pkg{vegan} provides access from version 2.2-0 onwards.

  Unless otherwise stated in the help pages for specific functions,
  permutation tests in \pkg{vegan} all follow the same format/structure:
  \enumerate{
    \item{An appropriate test statistic is chosen. Which statistic is
      chosen should be described on the help pages for individual
      functions.}
    \item{The value of the test statistic is evaluate for the observed
      data and analysis/model and recorded. Denote this value
      \eqn{x_0}{x[0]}.}
    \item{The \emph{DATA} are randomly permuted according to one of the
      above schemes, and the value of the test statistic for this
      permutation is evaluated and recorded.}
    \item{Step 3 is repeated a total of \eqn{n} times, where \eqn{n} is
      the number of permutations requested. Denote these values as
      \eqn{x_i}{x[i]}, where \eqn{i = 1, ..., n}{{i = 1, \ldots, n}.}}
    \item{Count the number of values of the test statistic,
      \eqn{x_i}{x[i]}, in the Null distribution that are as extreme as
      test statistic for the observed data \eqn{x_0}{x[0]}. Denote this
      count as \eqn{N}.

      We use the phrase \emph{as extreme} to include cases where a
      two-sided test is performed and large negative values of the test
      statistic should be considered.}
    \item{The permutation p-value is computed as
      \deqn{p = \frac{N + 1}{n + 1}}{(N + 1) / (n + 1)}}
  }

  The above description illustrates why the default number of
  permutations specified in \pkg{vegan} functions takes values of 199 or
  999 for example. Pretty \emph{p} values are achieved because the
  \eqn{+ 1} in the denominator results in division by 200 or 1000, for
  the 199 or 999 random permutations used in the test.

  The simple intuition behind the presence of \eqn{+ 1} in the numerator
  and denominator is that these represent the inclusion of the observed
  value of the statistic in the Null distribution (e.g. Manly 2006).
  Phipson & Smyth (2010) present a more compelling explanation for the
  inclusion of \eqn{+ 1} in the numerator and denominator of the
  \emph{p} value calculation.

  Fisher (1935) had in mind that a permutation test would involve
  enumeration of all possible permutations of the data yielding an exact
  test. However, doing this complete enumeration may not be feasible in
  practice owing to the potentially vast number of arrangements of the
  data, even in modestly-sized data sets with free permutation of
  samples. As a result we evaluate the \emph{p} value as the tail
  probability of the Null distribution of the test statistic directly
  from the random sample of possible permutations. Phipson & Smyth
  (2010) show that the naive calculation of the permutation \emph{p}
  value is

  \deqn{p = \frac{N}{n}}{p = N / n}

  which leads to an invalid test with incorrect type I error rate. They
  go on to show that by replacing the unknown tail probability (the
  \emph{p} value) of the Null distribution with the biased estimator

  \deqn{p = \frac{N + 1}{n + 1}}{p = (N + 1) / (n + 1)}

  that the positive bias induced is of just the right size to
  account for the  uncertainty in the estimation of the tail probability
  from the set of randomly sampled permutations to yield a test with the
  correct type I error rate.

  The estimator described above is correct for the situation where
  permutations of the data are samples randomly \emph{without}
  replacement. This is not strictly what happens in \pkg{vegan} because
  permutations are drawn pseudo-randomly independent of one
  another. Note that the actual chance of this happening is practice is
  small but the functions in \pkg{permute} do not guarantee to generate
  a unique set of permutations unless complete enumeration of
  permutations is requested. This is not feasible for all but the
  smallest of data sets or restrictive of permutation designs, but in
  such cases the chance of drawing a set of permutations with repeats is
  lessened as the sample size, and thence the size of set of all
  possible permutations, increases.

  Under the situation of sampling permutations with replacement then,
  the tail probability \eqn{p} calculated from the biased estimator
  described above is somewhat \strong{conservative}, being too large by
  an amount that depends on the number of possible values that the test
  statistic can take under permutation of the data (Phipson & Smyth,
  2010). This represents a slight loss of statistical power for the
  conservative \emph{p} value calculation used here. However, unless
  sample sizes are small and the the permutation design such that the
  set of values that the test statistic can take is also small, this
  loss of power is unlikely to be critical.

  The minimum achievable p-value is

  \deqn{p_{\mathrm{min}} = \frac{1}{n + 1}}{p[min] = 1 / (n + 1)}

  and hence depends on the number of permutations evaluated. However,
  one cannot simply increase the number of permutations (\eqn{n}) to
  achieve a potentially lower p-value unless the number of observations
  available permits such a number of permutations. This is unlikely to
  be a problem for all but the smallest data sets when free permutation
  (randomisation) is valid, but in restricted permutation designs with a
  low number of observations, there may not be as many unique
  permutations of the data as you might desire to reach the required
  level of significance.

  It is currently the responsibility of the user to determine the total
  number of possible permutations for their \emph{DATA}. The number of
  possible permutations allowed under the specified design can be
  calculated using \code{\link[permute]{numPerms}} from the
  \pkg{permute} package. Heuristics employed within the
  \code{\link[permute]{shuffleSet}} function used by \pkg{vegan} can be
  triggered to generate the entire set of permutations instead of a
  random set. The settings controlling the triggering of the complete
  enumeration step are contained within a permutation design created
  using \code{link[permute]{how}} and can be set by the user. See
  \code{\link[permute]{how}} for details.

  Limits on the total number of permutations of \emph{DATA} are more
  severe in temporally or spatially ordered data or experimental designs
  with low replication. For example, a time series of \eqn{n = 100}
  observations has just 100 possible permutations \strong{including} the
  observed ordering.

  In situations where only a low number of permutations is possible due
  to the nature of \emph{DATA} or the experimental design, enumeration
  of all permutations becomes important and achievable computationally.

  Above, we have provided only a brief overview of the capabilities of
  \pkg{vegan} and \pkg{permute}. To get the best out of the new
  functionality and for details on how to set up permutation designs
  using \code{\link[permute]{how}}, consult the vignette
  \emph{Restricted permutations; using the permute package} supplied
  with \pkg{permute} and accessible via \code{vignette("permutations",
  package = "permute").}
}

\section{Random Number Generation}{

  The permutations are based on the random number generator provided
  by \R. This may change in \R{} releases and change the permutations
  and \pkg{vegan} test results. One such change was in \R{} release
  3.6.0. The new version is clearly better for permutation tests and
  you should use it. However, if you need to reproduce old results,
  you can set the \R{} random number generator to a previous version
  with \code{\link{RNGversion}}.

} % section RNG

\seealso{
  \code{\link{permutest}} for the main interface in \pkg{vegan}. See
  also \code{\link[permute]{how}} for details on permutation design
  specification, \code{\link[permute]{shuffleSet}} for the code used to
  generate a set of permutations, \code{\link[permute]{numPerms}} for
  a function to return the size of the set of possible permutations
  under the current design.
}

\references{

  Manly, B. F. J. (2006). \emph{Randomization, Bootstrap and Monte Carlo
  Methods in Biology}, Third Edition. Chapman and Hall/CRC.

  Phipson, B., & Smyth, G. K. (2010). Permutation P-values should never
  be zero: calculating exact P-values when permutations are randomly
  drawn. \emph{Statistical Applications in Genetics and Molecular
    Biology}, \strong{9}, Article 39. DOI: 10.2202/1544-6115.1585

  ter Braak, C. J. F. (1990). \emph{Update notes: CANOCO version
    3.1}. Wageningen: Agricultural Mathematics Group. (UR).

  See also:

  Davison, A. C., & Hinkley, D. V. (1997). \emph{Bootstrap Methods and
    their Application}. Cambridge University Press.
}
\author{ Gavin L. Simpson }
\keyword{multivariate}

\name{model.matrix.cca}
\Rdversion{1.1}
\alias{model.matrix.cca}
\alias{model.frame.cca}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
 Reconstruct Model Frame and Model Matrices of Constrained Ordination
}
\description{
  Function \code{model.frame.cca} reconstructs a \code{\link{data.frame}}
  with the variables used in the constrained ordination method 
  (\code{\link{cca}}, \code{\link{rda}} or \code{\link{capscale}}. 
  Function \code{model.matrix.cca} creates a list of design matrices used 
  in constrained ordination. The items of the list are called 
  \code{Conditions} and \code{Constraints}. If either partial 
  (\code{Conditions}) or constrained component was missing, 
  a single matrix is returned. 
}
\usage{
\method{model.frame}{cca}(formula, ...)
\method{model.matrix}{cca}(object, ...)
}

\arguments{
  \item{formula, object}{
  A constrained ordination result object from which the needed
  information is extracted.
}
  \item{\dots}{
  Other arguments passed to the default method of the function.
}
}
\details{
  The constrained ordination method objects do not save data on model 
  frame or design matrix, and the functions must reconstruct the
  information in the session. This will fail if the data sets and variables
  of the original model are unavailable.
}
\value{
  Returns a data frame (\code{model.frame}) or an unnamed matrix or a 
  list of two matrices called \code{Constraints} and \code{Conditions} 
  (\code{model.matrix}). 
}
\author{
 Jari Oksanen
}
\seealso{
 \code{\link{model.frame}}, \code{\link{model.matrix}}.
}
\examples{
data(dune)
data(dune.env)
mod <- cca(dune ~  poly(A1, 2) + Management + Use, dune.env)
model.frame(mod)
model.matrix(mod)
}
\keyword{ models }
\keyword{ multivariate }

\encoding{UTF-8}
\name{beals}
\alias{beals}
\alias{swan}

\title{Beals Smoothing and Degree of Absence}
\description{
  Beals smoothing replaces each entry in the community data with a
  probability of a target species occurring in that particular site, based
  on the joint occurrences of the target species with the species that
  actually occur in the site. Swan's (1970) degree of absence applies
  Beals smoothing to zero items so long that all zeros are replaced
  with smoothed values. 
}
\usage{
beals(x, species = NA, reference = x, type = 0, include = TRUE)
swan(x, maxit = Inf, type = 0)
}
\arguments{
  \item{x}{Community data frame or matrix. }
  \item{species}{ Column index used to compute Beals function for a single species. 
  The default (\code{NA}) indicates that the function will be computed for all species.} 
  \item{reference}{ Community data frame or matrix to be used to compute
  joint occurrences. By default, \code{x} is used as reference to
  compute the joint occurrences.} 
  \item{type}{Numeric. Specifies if and how abundance values have to be 
  used in function \code{beals}. See details for more explanation.}  
  \item{include}{This logical flag indicates whether the target species has to be
  included when computing the mean of the conditioned probabilities. The
  original Beals (1984) definition is equivalent to \code{include=TRUE},
  while the formulation of \enc{Münzbergová}{Munzbergova} and Herben is
  equal to \code{include=FALSE}.}

  \item{maxit}{Maximum number of iterations. The default \code{Inf}
    means that iterations are continued until there are no zeros or
    the number of zeros does not change. Probably only 
    \code{maxit = 1} makes sense in addition to the default.}  
}

\details{

  Beals smoothing is the estimated probability \eqn{p_{ij}}{p[ij]} that
  species \eqn{j} occurs at site \eqn{i}. It is defined as \eqn{p_{ij}
  = \frac{1}{S_i} \sum_k \frac{N_{jk} I_{ik}}{N_k}}{p[ij] = 1/S[i]
  Sum(k) N[jk] I[ik] / N[k]}, where \eqn{S_i}{S[i]} is the number of
  species at site \eqn{i}, \eqn{N_{jk}}{N[jk]} is the number of joint
  occurrences of species \eqn{j} and \eqn{k}, \eqn{N_k}{N[k]} is the
  number of occurrences of species \eqn{k}, and \eqn{I} is the incidence
  (0 or 1) of species (this last term is usually omitted from the
  equation, but it is necessary). As \eqn{N_{jk}}{N[jk]} can be
  interpreted as a mean of conditional probability, the \code{beals}
  function can be interpreted as a mean of conditioned probabilities (De
  \enc{Cáceres}{Caceres} & Legendre 2008). The present function is
  generalized to abundance values (De \enc{Cáceres}{Caceres} & Legendre
  2008).
  
  The \code{type} argument specifies if and how abundance values have to be
  used. \code{type = 0} presence/absence mode. \code{type = 1}
  abundances in \code{reference} (or \code{x}) are used to compute
  conditioned probabilities. \code{type = 2} abundances in \code{x} are
  used to compute weighted averages of conditioned
  probabilities. \code{type = 3} abundances are used to compute both
  conditioned probabilities and weighted averages.

  Beals smoothing was originally suggested as a method of data
  transformation to remove excessive zeros (Beals 1984, McCune 1994).
  However, it is not a suitable method for this purpose since it does
  not maintain the information on species presences: a species may have
  a higher probability of occurrence at a site where it does not occur
  than at sites where it occurs. Moreover, it regularizes data too
  strongly. The method may be useful in identifying species that belong
  to the species pool (Ewald 2002) or to identify suitable unoccupied
  patches in metapopulation analysis (\enc{Münzbergová}{Munzbergova} &
  Herben 2004). In this case, the function should be called with
  \code{include=FALSE} for cross-validation smoothing for species;
  argument \code{species} can be used if only one species is studied.

  Swan (1970) suggested replacing zero values with degrees of absence of
  a species in a community data matrix. Swan expressed the method in
  terms of a similarity matrix, but it is equivalent to applying Beals
  smoothing to zero values, at each step shifting the smallest initially
  non-zero item to value one, and repeating this so many times that
  there are no zeros left in the data. This is actually very similar to
  extended dissimilarities (implemented in function
  \code{\link{stepacross}}), but very rarely used. 
}
\value{
  The function returns a transformed data matrix or a vector if Beals smoothing 
  is requested for a single species.
}
\references{
  
Beals, E.W. 1984. Bray-Curtis ordination: an effective strategy for
analysis of multivariate ecological data. Pp. 1--55 in: MacFadyen, A. &
E.D. Ford [eds.] \emph{Advances in Ecological Research, 14}. Academic
Press, London.

De \enc{Cáceres}{Caceres}, M. & Legendre, P. 2008. Beals smoothing
revisited. \emph{Oecologia} 156: 657--669.

Ewald, J. 2002. A probabilistic approach to estimating species pools
from large compositional matrices. \emph{J. Veg. Sci.} 13: 191--198.

McCune, B. 1994. Improving community ordination with the Beals smoothing
function. \emph{Ecoscience} 1: 82--86.

\enc{Münzbergová}{Munzbergova}, Z. & Herben, T. 2004. Identification of
suitable unoccupied 
habitats in metapopulation studies using co-occurrence of species. \emph{Oikos}
105: 408--414.

Swan, J.M.A. 1970. An examination of some ordination problems by use of
simulated vegetational data. \emph{Ecology} 51: 89--102. 

}
\author{Miquel De \enc{Cáceres}{Caceres} and Jari Oksanen}

\seealso{\code{\link{decostand}} for proper standardization methods,
  \code{\link{specpool}} for an attempt to assess the size of species
  pool. Function \code{\link{indpower}} assesses the power of each species
  to estimate the probabilities predicted by \code{beals}. }
\examples{
data(dune)
## Default
x <- beals(dune)
## Remove target species
x <- beals(dune, include = FALSE)
## Smoothed values against presence or absence of species
pa <- decostand(dune, "pa")
boxplot(as.vector(x) ~ unlist(pa), xlab="Presence", ylab="Beals")
## Remove the bias of tarbet species: Yields lower values.
beals(dune, type =3, include = FALSE)
## Uses abundance information.
## Vector with beals smoothing values corresponding to the first species
## in dune.
beals(dune, species=1, include=TRUE) 
}
\keyword{ manip }
\keyword{ smooth }

\name{ordilabel}
\alias{ordilabel}

\title{Add Text on Non-transparent Label to an Ordination Plot. }

\description{ Function \code{ordilabel} is similar to
  \code{\link{text}}, but the text is on an opaque label. This can help
  in crowded ordination plots: you still cannot see all text labels, but
  at least the uppermost are readable. Argument \code{priority} helps to
  make the most important labels visible.  }

\usage{
ordilabel(x, display, labels, choices = c(1, 2), priority, select, 
    cex = 0.8, fill = "white", border = NULL, col = NULL, xpd = TRUE, ...)
}

\arguments{
  \item{x}{An ordination object an any object known to
    \code{\link{scores}}. } 
  \item{display}{Kind of scores displayed (passed to
    \code{\link{scores}}). } 
  \item{labels}{Optional text used in plots. If this is not given, the
     text is found from the ordination object.}
  
  \item{choices}{Axes shown (passed to \code{\link{scores}}). }
  
  \item{priority}{Vector of the same length as the number of
    labels. The items with high priority will be plotted uppermost.}
  
  \item{select}{Items to be displayed. This can either be a logical
    vector which is \code{TRUE} for displayed items or a vector of
    indices of displayed items.}
  
  \item{cex}{ Character expansion for the text (passed to
    \code{\link{text}}). } 
  \item{fill}{ Background colour of the labels (the \code{col} argument
    of \code{\link{polygon}}).}
  \item{border}{The colour and visibility of the border of the label as
    defined in \code{\link{polygon}}.}
  \item{col}{Text colour. Default \code{NULL} will give the value of
    \code{border} or \code{par("fg")} if \code{border} is \code{NULL}.}
  \item{xpd}{Draw labels also outside the plot region (see 
    \code{\link{par}}).} 
  \item{\dots}{Other arguments (passed to \code{\link{text}}). }
}

\details{ The function may be useful with crowded ordination plots, in
  particular together with argument \code{priority}. You will not see
  all text labels, but at least some are readable. Other alternatives to
  crowded plots are \code{\link{identify.ordiplot}},
  \code{\link{orditorp}} and \code{\link{orditkplot}}.  }

\author{ Jari Oksanen }

\seealso{ \code{\link{scores}}, \code{\link{polygon}},
 \code{\link{text}}. The function is modelled after
 \code{\link[ade4]{s.label}} in \pkg{ade4} package.} 
\examples{
data(dune)
ord <- cca(dune)
plot(ord, type = "n")
ordilabel(ord, dis="sites", cex=1.2, font=3, fill="hotpink", col="blue")
## You may prefer separate plots, but here species as well
ordilabel(ord, dis="sp", font=2, priority=colSums(dune))
}
\keyword{ aplot }

\name{plot.cca}
\alias{plot.cca}
\alias{text.cca}
\alias{points.cca}
\alias{scores.cca}
\alias{scores.rda}
\alias{summary.cca}
\alias{print.summary.cca}
\alias{head.summary.cca}
\alias{tail.summary.cca}

\title{Plot or Extract Results of Constrained Correspondence Analysis
  or Redundancy Analysis}
\description{
  Functions to plot or extract results of constrained correspondence analysis
  (\code{\link{cca}}), redundancy analysis (\code{\link{rda}}) or
  constrained analysis of principal coordinates (\code{\link{capscale}}).
}
\usage{
\method{plot}{cca}(x, choices = c(1, 2), display = c("sp", "wa", "cn"),
     scaling = "species", type, xlim, ylim, const,
     correlation = FALSE, hill = FALSE, ...)
\method{text}{cca}(x, display = "sites", labels, choices = c(1, 2),
     scaling = "species", arrow.mul, head.arrow = 0.05, select, const,
     axis.bp = FALSE, correlation = FALSE, hill = FALSE, ...)
\method{points}{cca}(x, display = "sites", choices = c(1, 2),
       scaling = "species", arrow.mul, head.arrow = 0.05, select, const,
       axis.bp = FALSE, correlation = FALSE, hill = FALSE, ...)
\method{scores}{cca}(x, choices = c(1,2), display = c("sp","wa","cn"),
       scaling = "species", hill = FALSE, ...)
\method{scores}{rda}(x, choices = c(1,2), display = c("sp","wa","cn"),
       scaling = "species", const, correlation = FALSE, ...)
\method{summary}{cca}(object, scaling = "species", axes = 6,
        display = c("sp", "wa", "lc", "bp", "cn"),
        digits = max(3, getOption("digits") - 3),
        correlation = FALSE, hill = FALSE, ...)
\method{print}{summary.cca}(x, digits = x$digits, head = NA, tail = head, ...)
\method{head}{summary.cca}(x, n = 6, tail = 0, ...)
\method{tail}{summary.cca}(x, n = 6, head = 0, ...)
}

\arguments{
  \item{x, object}{A \code{cca} result object.}
    \item{choices}{Axes shown.}
  \item{display}{Scores shown.  These must include some of the
    alternatives \code{species} or \code{sp} for species scores,
    \code{sites} or \code{wa} for site scores, \code{lc} for linear
    constraints or LC scores, or \code{bp} for biplot arrows or
    \code{cn} for centroids of factor constraints instead of an arrow,
    and \code{reg} for regression coefficients (a.k.a. canonical
    coefficients). }
  \item{scaling}{Scaling for species and site scores. Either species
    (\code{2}) or site (\code{1}) scores are scaled by eigenvalues, and
    the other set of scores is left unscaled, or with \code{3} both are
    scaled symmetrically by square root of eigenvalues.  Corresponding
    negative values can be used in \code{cca} to additionally multiply
    results with \eqn{\sqrt(1/(1-\lambda))}.  This scaling is know as Hill
    scaling (although it has nothing to do with Hill's rescaling of
    \code{\link{decorana}}). With corresponding negative values
    in \code{rda}, species scores are divided by standard deviation of each
    species and multiplied with an equalizing constant. Unscaled raw
    scores stored in the result can be accessed with \code{scaling = 0}.

    The type of scores can also be specified as one of \code{"none"},
    \code{"sites"}, \code{"species"}, or \code{"symmetric"}, which
    correspond to the values \code{0}, \code{1}, \code{2}, and \code{3}
    respectively. Arguments \code{correlation} and \code{hill} in
    \code{scores.rda} and \code{scores.cca} respectively can be used in
    combination with these character descriptions to get the
    corresponding negative value.
  }
  \item{correlation, hill}{logical; if \code{scaling} is a character
    description of the scaling type, \code{correlation} or \code{hill}
    are used to select the corresponding negative scaling type; either
    correlation-like scores or Hill's scaling for PCA/RDA and CA/CCA
    respectively. See argument \code{scaling} for details.}
  \item{type}{Type of plot: partial match to \code{text}
    for text labels, \code{points} for points, and \code{none} for
    setting frames only.  If omitted, \code{text} is selected for
    smaller data sets, and \code{points} for larger.}
  \item{xlim, ylim}{the x and y limits (min,max) of the plot.}
  \item{labels}{Optional text to be used instead of row names.}
  \item{arrow.mul}{Factor to expand arrows in the graph.  Arrows will be
    scaled automatically to fit the graph if this is missing.}
  \item{head.arrow}{Default length of arrow heads.}
  \item{select}{Items to be displayed.  This can either be a logical
    vector which is \code{TRUE} for displayed items or a vector of indices
    of displayed items.}
  \item{const}{General scaling constant to \code{rda} scores. The
    default is to use a constant that gives biplot scores, that is,
    scores that approximate original data (see \code{\link{vignette}}
    on \sQuote{Design Decisions} with \code{browseVignettes("vegan")}
    for details and discussion). If \code{const} is a vector of two
    items, the first is used for species, and the second item for site
    scores.}
  \item{axis.bp}{Draw \code{\link{axis}} for biplot arrows.}
  \item{axes}{Number of axes in summaries.}
  \item{digits}{Number of digits in output.}
  \item{n, head, tail}{Number of rows printed from the head and tail of
    species and site scores.  Default \code{NA} prints all.}
  \item{...}{Parameters passed to other functions.}
}

\details{
  Same \code{plot} function will be used for \code{\link{cca}} and
  \code{\link{rda}}. This produces a quick, standard plot with current
  \code{scaling}.

  The \code{plot} function sets colours (\code{col}), plotting
  characters (\code{pch}) and character sizes (\code{cex}) to
  certain standard values. For a fuller control of produced plot, it is
  best to call \code{plot} with \code{type="none"} first, and then add
  each plotting item separately using \code{text.cca} or
  \code{points.cca} functions. These use the default settings of standard
  \code{\link{text}} and \code{\link{points}} functions and accept all
  their parameters, allowing  a full user control of produced plots.

  Environmental variables receive a special treatment. With
  \code{display="bp"}, arrows will be drawn. These are labelled with
  \code{text} and unlabelled with \code{points}. The arrows have
  basically unit scaling, but if sites were scaled (\code{scaling}
  \code{"sites"} or \code{"symmetric"}), the scores of requested axes
  are adjusted relative to the axis with highest eigenvalue.  With
  \code{scaling = "species"} or \code{scaling = "none"}, the arrows will
  be consistent with vectors fitted to linear combination scores
  (\code{display = "lc"} in function \code{\link{envfit}}), but with
  other scaling alternatives they will differ. The basic \code{plot}
  function uses a simple heuristics for adjusting the unit-length arrows
  to the current plot area, but the user can give the expansion factor
  in \code{mul.arrow}.  With \code{display="cn"} the centroids of levels
  of \code{\link{factor}} variables are displayed (these are available
  only if there were factors and a formula interface was used in
  \code{\link{cca}} or \code{\link{rda}}). With this option continuous
  variables still are presented as arrows and ordered factors as arrows
  and centroids. With \code{display = "reg"} arrows will be drawn for
  regression coefficients (a.k.a. canonical coefficients) of constraints
  and conditions. Biplot arrows can be interpreted individually, but
  regression coefficients must be interpreted all together: the LC score
  for each site is the sum of regressions displayed by arrows. The
  partialled out conditions are zero and not shown in biplot arrows, but
  they are shown for regressions, and show the effect that must be
  partialled out to get the LC scores. The biplot arrows are more
  standard and more easily interpreted, and regression arrows should be
  used only if you know that you need them.

  If you want to have a better control of plots, it is best to
  construct the plot \code{text} and \code{points} commands which
  accept graphical parameters. It is important to remember to use the
  same \code{scaling}, \code{correlation} and \code{hill} arguments
  in all calls. The \code{plot.cca} command returns invisibly an
  \code{\link{ordiplot}} result object, and this will have consistent
  scaling for all its elements. The easiest way for full control of
  graphics is to first set up the plot frame using \code{plot} with
  \code{type = "n"} and all needed scores in \code{display} and save
  this result. The \code{points} and \code{text} commands for
  \code{\link{ordiplot}} will allow full graphical control (see
  section Examples).

  Function \code{summary} lists all scores and the output can be very
  long.  You can suppress scores by setting \code{axes = 0} or
  \code{display = NA} or \code{display = NULL}. You can display some
  first or last (or both) rows of scores by using \code{head} or
  \code{tail} or explicit \code{print} command for the \code{summary}.
  
  Palmer (1993) suggested using linear constraints (\dQuote{LC scores})
  in ordination diagrams, because these gave better results in
  simulations and site scores (\dQuote{WA scores}) are a step from
  constrained to unconstrained analysis.  However, McCune (1997) showed
  that noisy environmental variables (and all environmental measurements
  are noisy) destroy \dQuote{LC scores} whereas \dQuote{WA scores} were
  little affected.  Therefore the \code{plot} function uses site scores
  (\dQuote{WA scores}) as the default. This is consistent with the usage
  in statistics and other functions in \R (\code{\link[MASS]{lda}},
  \code{\link{cancor}}).  } \value{ The \code{plot} function returns
  invisibly a plotting structure which can be used by function
  \code{\link{identify.ordiplot}} to identify the points or other
  functions in the \code{\link{ordiplot}} family.  }

\author{Jari Oksanen }

\seealso{\code{\link{cca}}, \code{\link{rda}} and \code{\link{capscale}}
  for getting something
  to plot, \code{\link{ordiplot}} for an alternative plotting routine
  and more support functions, and \code{\link{text}},
  \code{\link{points}} and \code{\link{arrows}} for the basic routines.  }

\examples{
data(dune)
data(dune.env)
mod <- cca(dune ~ A1 + Moisture + Management, dune.env)
## better control -- remember to set scaling etc identically
plot(mod, type="n", scaling="sites")
text(mod, dis="cn", scaling="sites")
points(mod, pch=21, col="red", bg="yellow", cex=1.2, scaling="sites")
text(mod, "species", col="blue", cex=0.8, scaling="sites")
## catch the invisible result and use ordiplot support - the example
## will make a biplot with arrows for species and correlation scaling
pca <- rda(dune)
pl <- plot(pca, type="n", scaling="sites", correlation=TRUE)
with(dune.env, points(pl, "site", pch=21, col=1, bg=Management))
text(pl, "sp", arrow=TRUE, length=0.05, col=4, cex=0.6, xpd=TRUE)
with(dune.env, legend("bottomleft", levels(Management), pch=21, pt.bg=1:4, bty="n"))
## Limited output of 'summary'
head(summary(mod), tail=2)
## Scaling can be numeric or more user-friendly names
## e.g. Hill's scaling for (C)CA
scrs <- scores(mod, scaling = "sites", hill = TRUE)
## or correlation-based scores in PCA/RDA
scrs <- scores(rda(dune ~ A1 + Moisture + Management, dune.env),
               scaling = "sites", correlation = TRUE)
}
\keyword{hplot}
\keyword{aplot}

\name{predict.cca}
\alias{fitted.cca}
\alias{fitted.rda}
\alias{fitted.capscale}
\alias{fitted.dbrda}
\alias{residuals.cca}
\alias{predict.cca}
\alias{predict.rda}
\alias{predict.decorana}
\alias{coef.cca}
\alias{coef.rda}
\alias{calibrate.cca}
\alias{calibrate}

\title{Prediction Tools for [Constrained] Ordination (CCA,
  RDA, DCA, CA, PCA) }
\description{
  Function \code{predict} can be used to find site and species scores or
  estimates of the response data with new data sets, Function
  \code{calibrate} estimates values of constraints with new data set.
  Functions \code{fitted} and \code{residuals} return estimates of
  response data. 
}
\usage{
\method{fitted}{cca}(object, model = c("CCA", "CA", "pCCA"),
    type =  c("response", "working"), ...)
\method{fitted}{capscale}(object, model = c("CCA", "CA", "pCCA", "Imaginary"),
    type = c("response", "working"), ...)
\method{residuals}{cca}(object, ...)
\method{predict}{cca}(object, newdata, type = c("response", "wa", "sp", "lc", "working"),
        rank = "full", model = c("CCA", "CA"), scaling = "none",
        hill = FALSE, ...)
\method{predict}{rda}(object, newdata, type = c("response", "wa", "sp", "lc", "working"),
        rank = "full", model = c("CCA", "CA"), scaling = "none",
        correlation = FALSE, const, ...)
\method{calibrate}{cca}(object, newdata, rank = "full", ...)
\method{coef}{cca}(object, norm = FALSE, ...)
\method{predict}{decorana}(object, newdata, type = c("response", "sites", "species"),
    rank = 4, ...)
}

\arguments{

  \item{object}{A result object from \code{\link{cca}},
    \code{\link{rda}}, \code{\link{dbrda}}, \code{\link{capscale}} or
    \code{\link{decorana}}. }
  
  \item{model}{Show constrained (\code{"CCA"}), unconstrained
    (\code{"CA"}) or conditioned \dQuote{partial} (\code{"pCCA"})
    results. For \code{fitted} method of \code{\link{capscale}} this can
    also be \code{"Imaginary"} for imaginary components with negative
    eigenvalues }

  \item{newdata}{New data frame to be used in prediction or in
    calibration.  Usually this a new community data frame, but with
    \code{type = "lc"} and for constrained component with \code{type =
    "response"} and \code{type = "working"} it must be a data frame of
    constraints.  The \code{newdata} must have the same number of rows
    as the original community data for a \code{\link{cca}} result with
    \code{type = "response"} or \code{type = "working"}.  If the
    original model had row or column names, then new data must contain
    rows or columns with the same names (row names for species scores,
    column names for \code{"wa"} scores and constraint names of
    \code{"lc"} scores). In other cases the rows or columns must match
    directly. }
  \item{type}{The type of prediction, fitted values or residuals:
    \code{"response"} scales results so that the same ordination gives
    the same results, and \code{"working"} gives the values used
    internally, that is after Chi-square standardization in
    \code{\link{cca}} and scaling and centring in
    \code{\link{rda}}. In \code{\link{capscale}} and
    \code{\link{dbrda}} the \code{"response"} gives the
    dissimilarities, and \code{"working"} the internal data structure
    analysed in the ordination. Alternative \code{"wa"} gives the site
    scores as weighted averages of the community data, \code{"lc"} the
    site scores as linear combinations of environmental data, and
    \code{"sp"} the species scores. In \code{predict.decorana} the
    alternatives are scores for \code{"sites"} or \code{"species"}.}
  \item{rank}{The rank or the number of axes used in the approximation.
    The default is to use all axes (full rank) of the \code{"model"} or
    all available four axes in \code{predict.decorana}.}
  \item{scaling}{logical, character, or numeric; Scaling or predicted
    scores with the same meaning as in \code{\link{cca}},
    \code{\link{rda}}, \code{\link{dbrda}}, and
    \code{\link{capscale}}. See \code{scores.cca} for further details
    on acceptable values.}
  \item{correlation, hill}{logical; correlation-like scores or Hill's
    scaling as appropriate for RDA and CCA respectively. See
    \code{\link{scores.cca}} for additional details.}
  \item{const}{Constant multiplier for  RDA scores. This will be used
    only when \code{scaling} is not \code{FALSE}, and the default value
    will give similar scaling as in \code{\link{scores.rda}}.}
  \item{norm}{Coefficients for variables that are centred and scaled
    to unit norm.}

  \item{\dots}{Other parameters to the functions.}
}
\details{

  Function \code{fitted} gives the approximation of the original data
  matrix or dissimilarities from the ordination result either in the
  scale of the response or as scaled internally by the function.
  Function \code{residuals} gives the approximation of the original data
  from the unconstrained ordination.  With argument \code{type =
  "response"} the \code{fitted.cca} and \code{residuals.cca} function
  both give the same marginal totals as the original data matrix, and
  fitted and residuals do not add up to the original data.  Functions
  \code{fitted} and \code{residuals} for \code{\link{dbrda}} and
  \code{\link{capscale}} give the dissimilarities with \code{type =
  "response"}, but these are not additive.  However, the
  \code{"working"} scores are additive for \code{\link{capscale}} (but
  not for \code{\link{dbrda}}). The \code{fitted} and \code{residuals}
  for \code{\link{capscale}} and \code{\link{dbrda}} will include the
  additive constant if that was requested in the function call.  All
  variants of \code{fitted} and \code{residuals} are defined so that for
  model \code{mod <- cca(y ~ x)}, \code{cca(fitted(mod))} is equal to
  constrained ordination, and \code{cca(residuals(mod))} is equal to
  unconstrained part of the ordination.

  Function \code{predict} can find the estimate of the original data
  matrix or dissimilarities (\code{type = "response"}) with any rank.
  With \code{rank = "full"} it is identical to \code{fitted}.  In
  addition, the function can find the species scores or site scores from
  the community data matrix for \code{\link{cca}} or \code{\link{rda}}.
  The function can be used with new data, and it can be used to add new
  species or site scores to existing ordinations.  The function returns
  (weighted) orthonormal scores by default, and you must specify
  explicit \code{scaling} to add those scores to ordination
  diagrams. With \code{type = "wa"} the function finds the site scores
  from species scores. In that case, the new data can contain new sites,
  but species must match in the original and new data.  With \code{type="sp"} 
  the function finds species scores from site constraints
  (linear combination scores). In that case the new data can contain new
  species, but sites must match in the original and new data. With
  \code{type = "lc"} the function finds the linear combination scores
  for sites from environmental data. In that case the new data frame
  must contain all constraining and conditioning environmental variables
  of the model formula. With \code{type = "response"} or 
  \code{type = "working"} the new data must contain environmental variables 
  if constrained component is desired, and community data matrix if
  residual or unconstrained component is desired.  With these types, the
  function uses \code{newdata} to find new \code{"lc"} (constrained) or
  \code{"wa"} scores (unconstrained) and then finds the response or
  working data from these new row scores and species scores.  The
  original site (row) and species (column) weights are used for
  \code{type = "response"} and \code{type = "working"} in correspondence
  analysis (\code{\link{cca}}) and therefore the number of rows must
  match in the original data and \code{newdata}.

  If a completely new data frame is created, extreme care is needed
  defining variables similarly as in the original model, in particular
  with (ordered) factors. If ordination was performed with the formula
  interface, the \code{newdata} can be a data frame or matrix, but
  extreme care is needed that the columns match in the original and
  \code{newdata}.

  Function \code{calibrate.cca} finds estimates of constraints from
  community ordination or \code{"wa"} scores from \code{\link{cca}},
  \code{\link{rda}} and \code{\link{capscale}}. This is often known as
  calibration, bioindication or environmental reconstruction.
  Basically, the method is similar to projecting site scores onto
  biplot arrows, but it uses regression coefficients.  The function
  can be called with \code{newdata} so that cross-validation is
  possible.  The \code{newdata} may contain new sites, but species
  must match in the original and new data.  The function does not work
  with \sQuote{partial} models with \code{Condition} term, and it
  cannot be used with \code{newdata} for \code{\link{capscale}} or
  \code{\link{dbrda}} results.  The results may only be interpretable
  for continuous variables.
  
  Function \code{coef} will give the regression coefficients from centred
  environmental variables (constraints and conditions) to linear
  combination scores. The coefficients are for unstandardized environmental
  variables. The coefficients will be \code{NA} for aliased effects.

  Function \code{predict.decorana} is similar to \code{predict.cca}.
  However, \code{type = "species"} is not available in detrended
  correspondence analysis  (DCA), because detrending destroys the mutual
  reciprocal averaging (except for the first axis when rescaling is not
  used). Detrended CA does not attempt to approximate the original data
  matrix, so \code{type = "response"} has no meaning in detrended
  analysis (except with \code{rank = 1}).
}

\value{
  The functions return matrices, vectors or dissimilarities as is appropriate.
}

\references{
  Greenacre, M. J. (1984). Theory and applications of correspondence
  analysis. Academic Press, London.
}

\author{Jari Oksanen.}


\seealso{\code{\link{cca}}, \code{\link{rda}}, \code{\link{dbrda}},
  \code{\link{capscale}}, \code{\link{decorana}},
  \code{\link{goodness.cca}}. }

\examples{
data(dune)
data(dune.env)
mod <- cca(dune ~ A1 + Management + Condition(Moisture), data=dune.env)
# Definition of the concepts 'fitted' and 'residuals'
mod
cca(fitted(mod))
cca(residuals(mod))
# Remove rare species (freq==1) from 'cca' and find their scores
# 'passively'.
freq <- specnumber(dune, MARGIN=2)
freq
mod <- cca(dune[, freq>1] ~ A1 + Management + Condition(Moisture), dune.env)
predict(mod, type="sp", newdata=dune[, freq==1], scaling="species")
# New sites
predict(mod, type="lc", new=data.frame(A1 = 3, Management="NM", Moisture="2"), scal=2)
# Calibration and residual plot
mod <- cca(dune ~ A1 + Moisture, dune.env)
pred <- calibrate(mod)
pred
with(dune.env, plot(A1, pred[,"A1"] - A1, ylab="Prediction Error"))
abline(h=0)
}
\keyword{ multivariate }

\name{vegan-deprecated}

\alias{adonis}
\alias{orditkplot}
\alias{plot.orditkplot}
\alias{scores.orditkplot}
\alias{points.orditkplot}
\alias{text.orditkplot}
\alias{as.mcmc.oecosimu}
\alias{as.mcmc.permat}
\alias{vegan-deprecated}

%------ NOTE:  ../R/vegan-deprecated.R   must be synchronized with this!
\title{Deprecated Functions in vegan package}
%------ PLEASE: one \alias{.} for EACH ! (+ one \usage{} & \arguments{} for all)

\description{
  These functions are provided for compatibility with older versions of
  \pkg{vegan} only, and may be defunct as soon as the next release.
}

\usage{
## use adonis2 instead
adonis(formula, data, permutations = 999, method = "bray",
    strata = NULL, contr.unordered = "contr.sum",
    contr.ordered = "contr.poly", parallel = getOption("mc.cores"), ...)
    orditkplot(x, display = "species", choices = 1:2, width, xlim, ylim, 
   tcex = 0.8, tcol, pch = 1,  pcol, pbg, pcex = 0.7, labels,  ...)
## moved to vegan3d package: install from CRAN
orditkplot(x, display = "species", choices = 1:2, width, xlim, ylim, 
   tcex = 0.8, tcol, pch = 1,  pcol, pbg, pcex = 0.7, labels,  ...)
\method{plot}{orditkplot}(x, ...)
\method{points}{orditkplot}(x, pch = x$args$pch, cex = x$args$pcex,
       col = x$args$pcol, bg = x$args$pbg, ...)
\method{text}{orditkplot}(x, cex = x$args$tcex, col = x$args$tcol,
     font = attr(x$labels, "font"), ...)
\method{scores}{orditkplot}(x, display, ...)
## use toCoda instead
as.mcmc.oecosimu(x)
as.mcmc.permat(x)
}

\arguments{
  \item{formula, data, permutations, method, parallel, \dots}{See
    \code{\link{adonis2}}.}
  \item{strata}{groups (strata) within which to constrain permutations.}
  \item{contr.unordered, contr.ordered}{contrasts used for design matrix.}
  \item{x}{ordination result \code{orditkplot} or object to be
     tranformed \code{as.mcmc}.}
  \item{display, choices}{kind of scores and number of axes.}
  \item{width}{size of Tcl/Tk window in inches.}

  \item{xlim, ylim, tcex, tcol, pch, pcol, pbg, pcex, labels, cex,
     col, bg, font}{graphical parameters similar to standard
     \code{plot} functions; those applied only to points preceded with
     \code{p}, and those to text with \code{t}.}

}

\details{

  \code{\link{adonis2}} replaces \code{adonis} with extended
  functionality and completely new internal design. The shared arguments
  of \code{adonis} are similar as in \code{\link{adonis2}}, but
  arguments \code{contr.unordered} and \code{contr.ordered} can set the
  contrasts within \code{adonis}.

  \code{orditkplot} was moved to \CRANpkg{vegan3d} (version
  1.3-0). Install that package from CRAN and use in the old way.

}

\seealso{
  \code{\link{Deprecated}}
}
\keyword{misc}

\name{capscale}
\alias{capscale}
\alias{dbrda}

\title{[Partial] Distance-based Redundancy Analysis }
\description{
  Distance-based redundancy analysis (dbRDA) is an ordination method
  similar to Redundancy Analysis (\code{\link{rda}}), but it allows
  non-Euclidean dissimilarity indices, such as Manhattan or
  Bray--Curtis distance. Despite this non-Euclidean feature, the analysis
  is strictly linear and metric. If called with Euclidean distance,
  the results are identical to \code{\link{rda}}, but dbRDA
  will be less efficient. Functions \code{capscale} and \code{dbrda} are
  constrained versions of metric scaling, a.k.a. principal coordinates
  analysis, which are based on the Euclidean distance but can be used,
  and are more useful, with other dissimilarity measures. The functions
  can also perform unconstrained principal coordinates analysis,
  optionally using extended dissimilarities.
}
\usage{
capscale(formula, data, distance = "euclidean", sqrt.dist = FALSE,
    comm = NULL, add = FALSE,  dfun = vegdist, metaMDSdist = FALSE,
    na.action = na.fail, subset = NULL, ...)
dbrda(formula, data, distance = "euclidean", sqrt.dist = FALSE,
    add = FALSE, dfun = vegdist, metaMDSdist = FALSE,
    na.action = na.fail, subset = NULL, ...)
}

\arguments{
  \item{formula}{Model formula. The function can be called only with the
    formula interface. Most usual features of \code{\link{formula}} hold,
    especially as defined in \code{\link{cca}} and \code{\link{rda}}. The
    LHS must be either a community data matrix or a dissimilarity matrix,
    e.g., from
    \code{\link{vegdist}} or \code{\link{dist}}.
    If the LHS is a data matrix, function \code{\link{vegdist}} or
    function given in \code{dfun}
    will be used to find the dissimilarities. The RHS defines the
    constraints. The constraints can be continuous variables or factors,
    they can be transformed within the formula, and they can have
    interactions as in a typical \code{\link{formula}}. The RHS can have a
    special term \code{Condition} that defines variables to be
    \dQuote{partialled out} before constraints, just like in \code{\link{rda}}
    or \code{\link{cca}}. This allows the use of partial dbRDA.}
  \item{data}{ Data frame containing the variables on the right hand side of
    the model formula. }
  \item{distance}{The name of the dissimilarity (or distance) index if
    the LHS of the \code{formula} is a data frame instead of
    dissimilarity matrix.}
  \item{sqrt.dist}{Take square roots of dissimilarities. See section
    \code{Details} below.}

  \item{comm}{ Community data frame which will be used for finding
    species scores when the LHS of the \code{formula} was a
    dissimilarity matrix. This is not used if the LHS is a data
    frame. If this is not supplied, the \dQuote{species scores} are
    unavailable when dissimilarities were supplied. N.B., this is
    only available in \code{capscale}: \code{dbrda} does not return
    species scores. Function \code{\link{sppscores}} can be used to add
    species scores if they are missing.}

  \item{add}{Add a constant to the non-diagonal dissimilarities such
     that all eigenvalues are non-negative in the underlying Principal
     Co-ordinates Analysis (see \code{\link{wcmdscale}} for
     details). \code{"lingoes"} (or \code{TRUE}) uses the
     recommended method of Legendre & Anderson (1999: \dQuote{method
     1}) and \code{"cailliez"} uses their \dQuote{method 2}. The
     latter is the only one in \code{\link{cmdscale}}.}
  \item{dfun}{Distance or dissimilarity function used. Any function
     returning standard \code{"dist"} and taking the index name as the
     first argument can be used. }
   \item{metaMDSdist}{Use \code{\link{metaMDSdist}} similarly as in
     \code{\link{metaMDS}}. This means automatic data transformation and
     using extended flexible shortest path dissimilarities (function
     \code{\link{stepacross}}) when there are many dissimilarities based on
     no shared species.}
   \item{na.action}{Handling of missing values in constraints or
     conditions. The default (\code{\link{na.fail}}) is to stop
     with missing values. Choices \code{\link{na.omit}} and
     \code{\link{na.exclude}} delete rows with missing values, but
     differ in representation of results. With \code{na.omit} only
     non-missing site scores are shown, but \code{na.exclude} gives
     \code{NA} for scores of missing observations. Unlike in
     \code{\link{rda}}, no WA scores are available for missing
     constraints or conditions.}
   \item{subset}{Subset of data rows. This can be a logical vector
     which is \code{TRUE} for kept observations, or a logical
     expression which can contain variables in the working
     environment, \code{data} or species names of the community data
     (if given in the formula or as \code{comm} argument).}
   \item{\dots}{Other parameters passed to underlying functions (e.g.,
     \code{\link{metaMDSdist}}).  }
}
\details{

  Functions \code{capscale} and \code{dbrda} provide two alternative
  implementations of dbRDA. Function \code{capscale} is based on
  Legendre & Anderson (1999): the dissimilarity data are first
  ordinated using metric scaling, and the ordination results are
  analysed as \code{\link{rda}}. Function \code{dbrda} is based on
  McArdle & Anderson (2001) and directly decomposes
  dissimilarities. It does not use \code{\link{rda}} but a parallel
  implementation adapted for analysing dissimilarities and returns a
  subset of \code{\link{rda}} items. With Euclidean distances both
  results are identical to \code{\link{rda}}.  Other dissimilarities
  may give negative eigenvalues associated with imaginary
  axes. Negative eigenvalues are handled differently: \code{capscale}
  ignores imaginary axes and analyses only real axes with positive
  eigenvalues, and \code{dbrda} directly analyses dissimilarities and
  can give negative eigenvalues in any component.

  If the user supplied a community data frame instead of
  dissimilarities, the functions will find dissimilarities using
  \code{\link{vegdist}} or distance function given in \code{dfun} with
  specified \code{distance}. The functions will accept distance
  objects from \code{\link{vegdist}}, \code{\link{dist}}, or any other
  method producing compatible objects. The constraining variables can be
  continuous or factors or both, they can have interaction terms, or
  they can be transformed in the call. Moreover, there can be a
  special term \code{Condition} just like in \code{\link{rda}} and
  \code{\link{cca}} so that \dQuote{partial} analysis can be performed.

  Function \code{dbrda} does not return species scores, and they can
  also be missing in \code{capscale}, but they can be added after the
  analysis using function \code{\link{sppscores}}.

  Non-Euclidean dissimilarities can produce negative eigenvalues
  (Legendre & Anderson 1999, McArdle & Anderson 2001).  If there are
  negative eigenvalues, the printed output of \code{capscale} will add
  a column with sums of positive eigenvalues and an item of sum of
  negative eigenvalues, and \code{dbrda} will add a column giving the
  number of real dimensions with positive eigenvalues.  If negative
  eigenvalues are disturbing, functions let you to distort the
  dissimilarities so that only non-negative eigenvalues will be
  produced with argument \code{add = TRUE}. Alternatively, with
  \code{sqrt.dist = TRUE}, square roots of dissimilarities will be
  used which may help in avoiding negative eigenvalues (Legendre &
  Anderson 1999).

  The functions can be also used to perform ordinary metric scaling
  a.k.a. principal coordinates analysis by using a formula with only a
  constant on the left hand side, or \code{comm ~ 1}. With
  \code{metaMDSdist = TRUE}, the function can do automatic data
  standardization and use extended dissimilarities using function
  \code{\link{stepacross}} similarly as in non-metric multidimensional
  scaling with \code{\link{metaMDS}}.

}
\value{

  The functions return an object of class \code{capscale} or
  \code{dbrda} which inherits from \code{\link{rda}}. See
  \code{\link{cca.object}} for description of the result object.

}
\references{
  Anderson, M.J. & Willis, T.J. (2003). Canonical analysis of principal
  coordinates: a useful method of constrained ordination for
  ecology. \emph{Ecology} 84, 511--525.

  Gower, J.C. (1985). Properties of Euclidean and non-Euclidean
  distance matrices. \emph{Linear Algebra and its Applications} 67, 81--97.

  Legendre, P. & Anderson, M. J. (1999). Distance-based redundancy
  analysis: testing multispecies responses in multifactorial ecological
  experiments. \emph{Ecological Monographs} 69, 1--24.

  Legendre, P. & Legendre, L. (2012).  \emph{Numerical Ecology}. 3rd English
  Edition. Elsevier.

  McArdle, B.H. & Anderson, M.J. (2001). Fitting multivariate models
  to community data: a comment on distance-based redundancy
  analysis. \emph{Ecology} 82, 290--297.
}

\author{ Jari Oksanen }

\note{ The function \code{capscale} was originally developed as a
  variant of constrained analysis of proximities (Anderson & Willis
  2003), but these developments made it similar to dbRDA. However, it
  discards the imaginary dimensions with negative eigenvalues and
  ordination and significance tests area only based on real dimensions
  and positive eigenvalues.

  The inertia is named after the dissimilarity index as defined in the
  dissimilarity data, or as \code{unknown distance} if such
  information is missing. If the largest original dissimilarity was
  larger than 4, \code{capscale} handles input similarly as \code{rda}
  and bases its analysis on variance instead of sum of
  squares. Keyword \code{mean} is added to the inertia in these cases,
  e.g. with Euclidean and Manhattan distances.  Inertia is based on
  squared index, and keyword \code{squared} is added to the name of
  distance, unless data were square root transformed (argument
  \code{sqrt.dist=TRUE}). If an additive constant was used with
  argument \code{add}, \code{Lingoes} or \code{Cailliez adjusted} is
  added to the the name of inertia, and the value of the constant is
  printed.}


\seealso{\code{\link{rda}}, \code{\link{cca}}, \code{\link{plot.cca}},
  \code{\link{anova.cca}}, \code{\link{vegdist}},
  \code{\link{dist}}, \code{\link{cmdscale}}, \code{\link{wcmdscale}}
  for underlying and related functions. Function \code{\link{sppscores}}
  can add species scores or replace existing species scores.

  The function returns similar result object as \code{\link{rda}} (see
  \code{\link{cca.object}}). This section for \code{\link{rda}} gives a
  more complete list of functions that can be used to access and
  analyse dbRDA results.

}
\examples{
data(varespec)
data(varechem)
## Basic Analysis
vare.cap <- capscale(varespec ~ N + P + K + Condition(Al), varechem,
                     dist="bray")
vare.cap
plot(vare.cap)
anova(vare.cap)
## Avoid negative eigenvalues with additive constant
capscale(varespec ~ N + P + K + Condition(Al), varechem,
                     dist="bray", add =TRUE)
## Avoid negative eigenvalues by taking square roots of dissimilarities
capscale(varespec ~ N + P + K + Condition(Al), varechem,
                     dist = "bray", sqrt.dist= TRUE)
## Principal coordinates analysis with extended dissimilarities
capscale(varespec ~ 1, dist="bray", metaMDS = TRUE)
## dbrda
dbrda(varespec ~ N + P + K + Condition(Al), varechem,
                     dist="bray")
## avoid negative eigenvalues also with Jaccard distances
dbrda(varespec ~ N + P + K + Condition(Al), varechem,
                     dist="jaccard")
}
\keyword{ multivariate }

\name{sppscores}
\alias{sppscores}
\alias{sppscores<-}
\alias{sppscores<-.dbrda}
\alias{sppscores<-.capscale}
\alias{sppscores<-.metaMDS}

\title{
Add or Replace Species Scores in Distance-Based Ordination
}

\description{

  Distance-based ordination (\code{\link{dbrda}},
  \code{\link{capscale}}, \code{\link{metaMDS}}) have no information
  on species, but some methods may add species scores if community
  data were available. However, the species scores may be missing (and
  they always are in \code{\link{dbrda}}), or they may not have a
  close relation to used dissimilarity index. This function will add
  the species scores or replace the existing species scores in
  distance-based methods.

}

\usage{
sppscores(object) <- value
}

\arguments{
  \item{object}{Ordination result.}
  \item{value}{Community data to find the species scores.}
}

\details{

  Distances have no information on species (columns, variables), and
  hence distance-based ordination has no information on species
  scores. However, the species scores can be added as supplementary
  information after the analysis to help the interpretation of
  results. Some ordination methods (\code{\link{capscale}},
  \code{\link{metaMDS}}) can supplement the species scores during the
  analysis if community data was available in the analysis.

  In \code{\link{capscale}} the species scores are found by projecting
  the community data to site ordination (linear combination scores),
  and the scores are accurate if the analysis used Euclidean
  distances. If the dissimilarity index can be expressed as Euclidean
  distances of transformed data (for instance, Chord and Hellinger
  Distances), the species scores based on transformed data will be
  accurate, but the function still finds the dissimilarities with
  untransformed data. Usually community dissimilarities differ in two
  significant ways from Euclidean distances: They are bound to maximum
  1, and they use absolute differences instead of squared
  differences. In such cases, it may be better to use species scores
  that are transformed so that their Euclidean distances have a good
  linear relation to used dissimilarities. It is often useful to
  standardize data so that each row has unit total, and perform
  squareroot transformation to damp down the effect of squared
  differences (see Examples).

  Function \code{\link{dbrda}} never finds the species scores, but it
  is mathematically similar to \code{\link{capscale}}, and similar
  rules should be followed when supplementing the species scores.

  Function \code{\link{metaMDS}} uses weighted averages
  (\code{\link{wascores}}) to find the species scores. These have a
  better relationship with most dissimilarities than the projection
  scores used in metric ordination, but similar transformation of the
  community data should be used both in dissimilarities and in species
  scores.

}

\value{

  Replacement function adds the species scores or replaces the old
  scores in the ordination object.

}


\author{
  Jari Oksanen
}

\seealso{

  Function \code{\link{envfit}} finds similar scores, but based on
  correlations. The species scores for non-metric ordination use
  \code{\link{wascores}} which can also used directly on any
  ordination result.
 
}

\examples{
data(BCI, BCI.env)
mod <- dbrda(vegdist(BCI) ~ Habitat, BCI.env)
## add species scores
sppscores(mod) <- BCI
## Euclidean distances of BCI differ from used dissimilarity
plot(vegdist(BCI), dist(BCI))
## more linear relationship
plot(vegdist(BCI), dist(sqrt(decostand(BCI, "total"))))
## better species scores
sppscores(mod) <- sqrt(decostand(BCI, "total"))
}

\keyword{ multivariate }

\name{stepacross}
\alias{stepacross}

\title{Stepacross as Flexible Shortest Paths or Extended Dissimilarities } 
\description{
  Function \code{stepacross} tries to replace dissimilarities with
  shortest paths stepping across intermediate 
  sites while regarding dissimilarities above a threshold as missing
  data (\code{NA}). With \code{path = "shortest"} this is the flexible shortest
  path (Williamson 1978, Bradfield & Kenkel 1987),
  and with \code{path = "extended"} an
  approximation known as extended dissimilarities (De'ath 1999).
  The use of \code{stepacross} should improve the ordination with high
  beta diversity, when there are many sites with no species in common.
}
\usage{
stepacross(dis, path = "shortest", toolong = 1, trace = TRUE, ...)
}
\arguments{
  \item{dis}{Dissimilarity data inheriting from class \code{dist} or
    a an object, such as a matrix, that can be converted to a
    dissimilarity matrix. Functions \code{\link{vegdist}} and
    \code{\link{dist}} are some functions producing suitable
    dissimilarity data. }
  \item{path}{The method of stepping across (partial match)
    Alternative \code{"shortest"} finds the shortest paths, and
    \code{"extended"}  their approximation known as extended
    dissimilarities.} 
  \item{toolong}{Shortest dissimilarity regarded as \code{NA}.
    The function uses a fuzz factor, so
    that dissimilarities close to the limit will be made \code{NA}, too. }
  \item{trace}{ Trace the calculations.}
  \item{\dots}{Other parameters (ignored).}
}
\details{
  Williamson (1978) suggested using flexible shortest paths to estimate
  dissimilarities between sites which have nothing in common, or no shared
  species. With \code{path = "shortest"} function \code{stepacross}
  replaces dissimilarities that are
  \code{toolong} or longer with \code{NA}, and tries to find shortest
  paths between all sites using remaining dissimilarities. Several
  dissimilarity indices are semi-metric which means that they do not
  obey the triangle inequality \eqn{d_{ij} \leq d_{ik} + d_{kj}}{d[ij] <=
    d[ik] + d[kj]}, and shortest path algorithm can replace these
  dissimilarities as well, even when they are shorter than
  \code{toolong}. 

  De'ath (1999) suggested a simplified method known as extended
  dissimilarities, which are calculated with \code{path = "extended"}. 
  In this method, dissimilarities that are
  \code{toolong} or longer are first made \code{NA}, and then the function
  tries to replace these \code{NA} dissimilarities with a path through
  single stepping stone points. If not all \code{NA} could be 
  replaced with one pass, the function will make new passes with updated
  dissimilarities as long as
  all \code{NA} are replaced with extended dissimilarities. This mean
  that in the second and further passes, the remaining \code{NA}
  dissimilarities are allowed to have more than one stepping stone site,
  but previously replaced dissimilarities are not updated. Further, the
  function does not consider dissimilarities shorter than \code{toolong},
  although some of these could be replaced with a shorter path in
  semi-metric indices, and used as a part of other paths. In optimal
  cases, the extended dissimilarities are equal to shortest paths, but
  they may be longer.  

  As an alternative to defining too long dissimilarities with parameter
  \code{toolong}, the input dissimilarities can contain \code{NA}s. If
  \code{toolong} is zero or negative, the function does not make any
  dissimilarities into \code{NA}. If there are no \code{NA}s in the
  input  and \code{toolong = 0}, \code{path = "shortest"}
  will find shorter paths for semi-metric indices, and \code{path = "extended"} 
  will do nothing. Function \code{\link{no.shared}} can be
  used to set dissimilarities to \code{NA}.
  
  If the data are disconnected or there is no path between all points,
  the result will
  contain \code{NA}s and a warning is issued. Several methods cannot
  handle \code{NA} dissimilarities, and this warning should be taken
  seriously. Function \code{\link{distconnected}} can be used to find
  connected groups and remove rare outlier observations or groups of
  observations.

  Alternative \code{path = "shortest"} uses Dijkstra's method for
  finding flexible shortest paths, implemented as priority-first search
  for dense graphs (Sedgewick 1990). Alternative \code{path = "extended"} 
  follows De'ath (1999), but implementation is simpler
  than in his code.
  
}
\value{
  Function returns an object of class \code{dist} with extended
  dissimilarities (see functions \code{\link{vegdist}} and
  \code{\link{dist}}). 
  The value of \code{path} is appended to the \code{method} attribute.
}
\references{
  Bradfield, G.E. & Kenkel, N.C. (1987). Nonlinear ordination using
  flexible shortest path adjustment of ecological
  distances. \emph{Ecology} 68, 750--753.
  
  De'ath, G. (1999). Extended dissimilarity: a method of robust
  estimation of ecological distances from high beta diversity
  data. \emph{Plant Ecol.} 144, 191--199.

  Sedgewick, R. (1990). \emph{Algorithms in C}. Addison Wesley. 

  Williamson, M.H. (1978). The ordination of incidence
  data. \emph{J. Ecol.} 66, 911-920.
}
\author{ Jari Oksanen}
\note{
  The function changes the original dissimilarities, and not all
  like this. It may be best to  use  the
  function only when you really \emph{must}:  extremely high
  beta diversity where a large proportion of dissimilarities are at their
  upper limit (no species in common). 

  Semi-metric indices vary in their degree of violating the triangle
  inequality. Morisita and Horn--Morisita indices of
  \code{\link{vegdist}} may be very strongly semi-metric, and shortest
  paths can change these indices very much. Mountford index violates
  basic rules of dissimilarities: non-identical sites have zero
  dissimilarity if species composition of the poorer site is a subset of
  the richer. With Mountford index, you can find three sites \eqn{i, j,
    k} so that \eqn{d_{ik} = 0}{d[ik] = 0} and \eqn{d_{jk} = 0}{d[jk] =
    0}, but \eqn{d_{ij} > 0}{d[ij] > 0}. The results of \code{stepacross}
  on Mountford index can be very weird. If \code{stepacross} is needed,
  it is best to try to use it with more metric indices only.
}

\seealso{
  Function \code{\link{distconnected}} can find connected groups in
  disconnected data, and function \code{\link{no.shared}} can be used to
  set dissimilarities as \code{NA}.  See \code{\link{swan}} for an
  alternative approach. Function \code{stepacross} is an essential
  component in \code{\link{isomap}} and \code{\link{cophenetic.spantree}}.
 }
\examples{
# There are no data sets with high beta diversity in vegan, but this
# should give an idea.
data(dune)
dis <- vegdist(dune)
edis <- stepacross(dis)
plot(edis, dis, xlab = "Shortest path", ylab = "Original")
## Manhattan distance have no fixed upper limit.
dis <- vegdist(dune, "manhattan")
is.na(dis) <- no.shared(dune)
dis <- stepacross(dis, toolong=0)
}
\keyword{multivariate }

\name{mrpp}
\alias{mrpp}
\alias{meandist}
\alias{summary.meandist}
\alias{plot.meandist}

\title{Multi Response Permutation Procedure and Mean Dissimilarity Matrix}

\description{ Multiple Response Permutation Procedure (MRPP) provides a
test of whether there is a significant difference between two or more
groups of sampling units. Function \code{meandist} finds the mean within
and between block dissimilarities.}

\usage{
mrpp(dat, grouping, permutations = 999, distance = "euclidean",
     weight.type = 1, strata = NULL, parallel = getOption("mc.cores"))
meandist(dist, grouping, ...)
\method{summary}{meandist}(object, ...)
\method{plot}{meandist}(x, kind = c("dendrogram", "histogram"),  cluster = "average", 
     ylim, axes = TRUE, ...)
}

\arguments{
  \item{dat}{data matrix or data frame in which rows are samples and
    columns are response variable(s), or a dissimilarity object or a
    symmetric square matrix of dissimilarities.} 
  \item{grouping}{ Factor or numeric index for grouping observations.}
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices. These are used to assess
    the significance of the MRPP statistic, \eqn{delta}.} 
  \item{distance}{Choice of distance metric that measures the
    dissimilarity between two observations . See \code{\link{vegdist}} for
    options.  This will be used if \code{dat} was not a dissimilarity
    structure of a symmetric square matrix.}  
  \item{weight.type}{ choice of group weights. See Details below for options.}
  \item{strata}{An integer vector or factor specifying the strata for
    permutation. If supplied, observations are permuted only within the
    specified strata.}
  \item{parallel}{Number of parallel processes or a predefined socket
    cluster.  With \code{parallel = 1} uses ordinary, non-parallel
    processing. The parallel processing is done with \pkg{parallel}
    package.}
  \item{dist}{A \code{\link{dist}} object of dissimilarities, such as
    produced by functions \code{\link{dist}}, \code{\link{vegdist}} or
    \code{\link{designdist}}.}.
  \item{object, x}{A \code{meandist} result object.}
  \item{kind}{Draw a dendrogram or a histogram; see Details.}
  \item{cluster}{A clustering method for the \code{\link{hclust}}
    function for \code{kind = "dendrogram"}. 
    Any \code{hclust} method can be used, but perhaps only
    \code{"average"} and \code{"single"} make sense.}
  \item{ylim}{Limits for vertical axes (optional).}
  \item{axes}{Draw scale for the vertical axis.}
  \item{\dots}{Further arguments passed to functions.}
}

\details{

  Multiple Response Permutation Procedure (MRPP) provides a test of
  whether there is a significant difference between two or more groups
  of sampling units. This difference may be one of location (differences
  in mean) or one of spread (differences in within-group distance;
  cf. Warton et al. 2012). Function \code{mrpp} operates on a
  \code{data.frame} matrix where rows are observations and responses
  data matrix. The response(s) may be uni- or multivariate. The method
  is philosophically and mathematically allied with analysis of
  variance, in that it compares dissimilarities within and among
  groups. If two groups of sampling units are really different (e.g. in
  their species composition), then average of the within-group
  compositional dissimilarities ought to be less than the average of the
  dissimilarities between two random collection of sampling units drawn
  from the entire population.

  The mrpp statistic \eqn{\delta} is the overall weighted mean of
  within-group means of the pairwise dissimilarities among sampling
  units. The choice of group weights is currently not clear. The
  \code{mrpp} function offers three choices: (1) group size (\eqn{n}),
  (2) a degrees-of-freedom analogue (\eqn{n-1}), and (3) a weight that
  is the number of unique distances calculated among \eqn{n} sampling
  units (\eqn{n(n-1)/2}).

  The \code{mrpp} algorithm first calculates all pairwise distances in
  the entire dataset, then calculates \eqn{\delta}. It then permutes the
  sampling units and their associated pairwise distances, and
  recalculates \eqn{\delta} based on the permuted data. It repeats the
  permutation step \code{permutations} times. The significance test is
  the fraction of permuted deltas that are less than the observed delta,
  with a small sample correction. The function also calculates the
  change-corrected within-group agreement \eqn{A = 1 -\delta/E(\delta)},
  where \eqn{E(\delta)} is the expected \eqn{\delta} assessed as the
  average of dissimilarities.

  If the first argument \code{dat} can be interpreted as
  dissimilarities, they will be used directly. In other cases the
  function treats \code{dat} as observations, and uses
  \code{\link{vegdist}} to find the dissimilarities.  The default
  \code{distance} is Euclidean as in the traditional use of the method,
  but other dissimilarities in \code{\link{vegdist}} also are available.

  Function \code{meandist} calculates a matrix of mean within-cluster
  dissimilarities (diagonal) and between-cluster dissimilarities
  (off-diagonal elements), and an attribute \code{n} of \code{grouping}
  counts. Function \code{summary} finds the within-class, between-class
  and overall means of these dissimilarities, and the MRPP statistics
  with all \code{weight.type} options and the Classification Strength,
  CS (Van Sickle and Hughes, 2000). CS is defined for dissimilarities as
  \eqn{\bar{B} - \bar{W}}{Bbar-Wbar}, where \eqn{\bar{B}}{Bbar} is the
  mean between cluster dissimilarity and \eqn{\bar{W}}{Wbar} is the mean
  within cluster dissimilarity with \code{weight.type = 1}. The function
  does not perform significance tests for these statistics, but you must
  use \code{mrpp} with appropriate \code{weight.type}. There is
  currently no significance test for CS, but \code{mrpp} with
  \code{weight.type = 1} gives the correct test for \eqn{\bar{W}}{Wbar}
  and a good approximation for CS.  Function \code{plot} draws a
  dendrogram or a histogram of the result matrix based on the
  within-group and between group dissimilarities. The dendrogram is
  found with the method given in the \code{cluster} argument using
  function \code{\link{hclust}}. The terminal segments hang to
  within-cluster dissimilarity. If some of the clusters are more
  heterogeneous than the combined class, the leaf segment are reversed.
  The histograms are based on dissimilarities, but ore otherwise similar
  to those of Van Sickle and Hughes (2000): horizontal line is drawn at
  the level of mean between-cluster dissimilarity and vertical lines
  connect within-cluster dissimilarities to this line.  }

\value{
The function returns a list of class mrpp with following items:
  \item{call }{	Function call.}
  \item{delta }{The overall weighted mean of group mean distances.}
  \item{E.delta}{expected delta, under the null hypothesis of no group
    structure. This is the mean of original dissimilarities.}
  \item{CS}{Classification strength (Van Sickle and Hughes,
    2000). Currently not implemented and always \code{NA}.}
  \item{n}{Number of observations in each class.}
  \item{classdelta}{Mean dissimilarities within classes. The overall
    \eqn{\delta} is the weighted average of these values with given
    \code{weight.type}}. 
  \item{Pvalue}{Significance of the test.}
  \item{A}{A chance-corrected estimate of the proportion of the distances
    explained by group identity; a value analogous to a coefficient of 
    determination in a linear model.} 
  \item{distance}{Choice of distance metric used; the "method" entry of
    the dist object.}
  \item{weight.type}{The choice of group weights used.}
  \item{boot.deltas}{The vector of "permuted deltas," the deltas
    calculated from each of the permuted datasets. The distribution of
    this item can be inspected with \code{\link{permustats}} function.}
  \item{permutations}{The number of permutations used.}
  \item{control}{A list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}.}
}
\references{
  B. McCune and J. B. Grace. 2002. \emph{Analysis of Ecological
  Communities.} MjM  Software Design, Gleneden Beach, Oregon, USA.

  P. W. Mielke and K. J. Berry. 2001. \emph{Permutation Methods: A
  Distance  Function Approach.} Springer Series in
  Statistics. Springer.  

  J. Van Sickle and R. M. Hughes 2000. Classification strengths of
  ecoregions, catchments, and geographic clusters of aquatic vertebrates
  in Oregon. \emph{J. N. Am. Benthol. Soc.} 19:370--384.

  Warton, D.I., Wright, T.W., Wang, Y. 2012. Distance-based multivariate
  analyses confound location and dispersion effects. \emph{Methods in
  Ecology and Evolution}, 3, 89--101

}
\author{
M. Henry H. Stevens \email{HStevens@muohio.edu} and Jari Oksanen.
}
\note{ 
This difference may be one of location (differences in mean) or one of
spread (differences in within-group distance). That is, it may find a
significant difference between two groups simply because one of those
groups has a greater dissimilarities among its sampling units. Most
\code{mrpp} models can be analysed with \code{\link{adonis2}} which seems
not suffer from the same problems as \code{mrpp} and is a more robust
alternative.
}
\seealso{
  \code{\link{anosim}} for a similar test based on ranks, and
  \code{\link{mantel}} for comparing dissimilarities against continuous
  variables, and
  \code{\link{vegdist}} for obtaining dissimilarities,
  \code{\link{adonis2}} is a more robust alternative in most cases.
}
\examples{
data(dune)
data(dune.env)
dune.mrpp <- with(dune.env, mrpp(dune, Management))
dune.mrpp

# Save and change plotting parameters
def.par <- par(no.readonly = TRUE)
layout(matrix(1:2,nr=1))

plot(dune.ord <- metaMDS(dune), type="text", display="sites" )
with(dune.env, ordihull(dune.ord, Management))

with(dune.mrpp, {
  fig.dist <- hist(boot.deltas, xlim=range(c(delta,boot.deltas)), 
                 main="Test of Differences Among Groups")
  abline(v=delta); 
  text(delta, 2*mean(fig.dist$counts), adj = -0.5,
     expression(bold(delta)), cex=1.5 )  }
)
par(def.par)
## meandist
dune.md <- with(dune.env, meandist(vegdist(dune), Management))
dune.md
summary(dune.md)
plot(dune.md)
plot(dune.md, kind="histogram")
}
\keyword{ multivariate }
\keyword{ nonparametric }
\keyword{htest}

\name{biplot.rda}
\alias{biplot.rda}
\alias{biplot.cca}

\title{PCA biplot}
\description{
  Draws a PCA biplot with species scores indicated by biplot arrows
}
\usage{
\method{biplot}{rda}(x, choices = c(1, 2), scaling = "species",
       display = c("sites", "species"), type, xlim, ylim, col = c(1,2), 
       const, correlation = FALSE, ...)
}

\arguments{
  \item{x}{A \code{\link{rda}} result object.}
  \item{choices}{Axes to show.}
   \item{scaling}{Scaling for species and site scores. Either species
    (\code{2}) or site (\code{1}) scores are scaled by eigenvalues, and
    the other set of scores is left unscaled, or with \code{3} both are
    scaled symmetrically by square root of eigenvalues. With negative
    scaling values in \code{rda}, species scores are divided by standard
    deviation of each species and multiplied with an equalizing
    constant. Unscaled raw scores stored in the result can be accessed
    with \code{scaling = 0}.

    The type of scores can also be specified as one of \code{"none"},
    \code{"sites"}, \code{"species"}, or \code{"symmetric"}, which
    correspond to the values \code{0}, \code{1}, \code{2}, and \code{3}
    respectively. Argument \code{correlation} can be used in combination
    with these character descriptions to get the corresponding negative
    value.
  }
  \item{correlation}{logical; if \code{scaling} is a character
    description of the scaling type, \code{correlation} can be used to
    select correlation-like scores for PCA. See argument \code{scaling}
    for details.}
  \item{display}{Scores shown.  These must some of the alternatives
    \code{"species"} for species scores, and/or \code{"sites"} for site
    scores.}
  \item{type}{Type of plot: partial match to \code{text}
    for text labels, \code{points} for points, and \code{none} for
    setting frames only. If omitted, \code{text} is selected for
    smaller data sets, and \code{points} for larger. Can be of length 2
    (e.g. \code{type = c("text", "points")}), in which case the first
    element describes how species scores are handled, and the second how
    site scores are drawn.}
  \item{xlim, ylim}{the x and y limits (min, max) of the plot.}
  \item{col}{Colours used for sites and species (in this order). If only
    one colour is given, it is used for both.}
  \item{const}{General scaling constant for \code{\link{scores.rda}}.}
  \item{\dots}{Other parameters for plotting functions.}
}
\details{
  Produces a plot or biplot of the results of a call to
  \code{\link{rda}}. It is common for the "species" scores in a PCA to
  be drawn as biplot arrows that point in the direction of increasing
  values for that variable. The \code{biplot.rda} function provides a
  wrapper to \code{plot.cca} to allow the easy production of such a
  plot.

  \code{\link{biplot.rda}} is only suitable for unconstrained models. If
  used on an ordination object with constraints, an error is issued.

  If species scores are drawn using \code{"text"}, the arrows are drawn
  from the origin to 0.85 * species score, whilst the labels are
  drawn at the species score. If the type used is \code{"points"}, then
  no labels are drawn and therefore the arrows are drawn from the origin
  to the actual species score.
}
\value{
  The \code{plot} function returns invisibly a plotting structure which
  can be used by \code{\link{identify.ordiplot}} to identify
  the points or other functions in the \code{\link{ordiplot}} family.
}
\author{Gavin Simpson, based on \code{\link{plot.cca}} by Jari Oksanen.}
\seealso{\code{\link{plot.cca}}, \code{\link{rda}} for something to
  plot, \code{\link{ordiplot}} for an alternative plotting routine
  and more support functions, and \code{\link{text}},
  \code{\link{points}} and \code{\link{arrows}} for the basic routines.}
\examples{
data(dune)
mod <- rda(dune, scale = TRUE)
biplot(mod, scaling = "symmetric")

## different type for species and site scores
biplot(mod, scaling = "symmetric", type = c("text", "points"))
}
\keyword{hplot}

\name{wascores}
\alias{wascores}
\alias{eigengrad}

\title{ Weighted Averages Scores for Species }
\description{
  Computes Weighted Averages scores of species for ordination
  configuration or for environmental variables.
}
\usage{
wascores(x, w, expand=FALSE)
eigengrad(x, w)
}

\arguments{
  \item{x}{Environmental variables or ordination scores.}
  \item{w}{Weights: species abundances.}
  \item{expand}{Expand weighted averages so that they have the same
    weighted variance as the corresponding environmental variables.  }
}
\details{
  Function \code{wascores} computes weighted averages. Weighted averages
  \dQuote{shrink}: they cannot be more extreme than values used for
  calculating the averages. With \code{expand = TRUE}, the function
  \dQuote{deshrinks} the weighted averages by making their biased
  weighted variance equal to the biased weighted variance of the
  corresponding environmental variable.  Function \code{eigengrad}
  returns the inverses of squared expansion factors or the attribute
  \code{shrinkage} of the \code{wascores} result for each environmental
  gradient.  This is equal to the constrained eigenvalue of
  \code{\link{cca}} when only this one gradient was used as a
  constraint, and describes the strength of the gradient.
}
\value{
  Function \code{wascores} returns a matrix where species define rows
  and ordination axes or environmental variables define columns. If
  \code{expand = TRUE}, attribute \code{shrinkage} has the inverses of
  squared expansion factors or \code{\link{cca}} eigenvalues for the
  variable.  Function \code{eigengrad} returns only the \code{shrinkage}
  attribute.
}

\author{ Jari Oksanen }

\seealso{ \code{\link{monoMDS}}, \code{\link{cca}}. }

\examples{
data(varespec)
data(varechem)
vare.dist <- vegdist(wisconsin(varespec))
vare.mds <- monoMDS(vare.dist)
vare.points <- postMDS(vare.mds$points, vare.dist)
vare.wa <- wascores(vare.points, varespec)
plot(scores(vare.points), pch="+", asp=1)
text(vare.wa, rownames(vare.wa), cex=0.8, col="blue")
## Omit rare species (frequency <= 4)
freq <- apply(varespec>0, 2, sum)
plot(scores(vare.points), pch="+", asp=1)
text(vare.wa[freq > 4,], rownames(vare.wa)[freq > 4],cex=0.8,col="blue")
## Works for environmental variables, too.
wascores(varechem, varespec)
## And the strengths of these variables are:
eigengrad(varechem, varespec)
}
\keyword{ multivariate }
\keyword{ univar }

\name{bioenv}
\alias{bioenv}
\alias{bioenv.default}
\alias{bioenv.formula}
\alias{summary.bioenv}
\alias{bioenvdist}

\title{Best Subset of Environmental Variables with
  Maximum (Rank) Correlation with Community Dissimilarities }
\description{
  Function finds the best subset of environmental variables, so that
  the Euclidean distances of scaled environmental variables have the
  maximum (rank) correlation with community dissimilarities.  
}
\usage{
\method{bioenv}{default}(comm, env, method = "spearman", index = "bray",
       upto = ncol(env), trace = FALSE, partial = NULL, 
       metric = c("euclidean", "mahalanobis", "manhattan", "gower"),
       parallel = getOption("mc.cores"), ...)
\method{bioenv}{formula}(formula, data, ...)
bioenvdist(x, which = "best")
}

\arguments{
  \item{comm}{Community data frame or a dissimilarity object or a square
    matrix that can be interpreted as dissimilarities. }
  \item{env}{Data frame of continuous environmental variables. }
  \item{method}{The correlation method used in \code{\link{cor}}.}
  \item{index}{The dissimilarity index used for community data (\code{comm}) 
    in \code{\link{vegdist}}. This is ignored if \code{comm} are dissimilarities.}
  \item{upto}{Maximum number of parameters in studied subsets.}
  \item{formula, data}{Model \code{\link{formula}} and data.}
  \item{trace}{Trace the calculations }
  \item{partial}{Dissimilarities partialled out when inspecting
    variables in \code{env}.}
  \item{metric}{Metric used for distances of environmental distances. See 
    Details.}
  \item{parallel}{Number of parallel processes or a predefined socket
    cluster.  With \code{parallel = 1} uses ordinary, non-parallel
    processing. The parallel processing is done with \pkg{parallel}
    package.}
  \item{x}{\code{bioenv} result object.}
  \item{which}{The number of the model for which the environmental
    distances are evaluated, or the \code{"best"} model.}
  \item{...}{Other arguments passed to \code{\link{cor}}.}
}
\details{
  
  The function calculates a community dissimilarity matrix using
  \code{\link{vegdist}}.  Then it selects all possible subsets of
  environmental variables, \code{\link{scale}}s the variables, and
  calculates Euclidean distances for this subset using
  \code{\link{dist}}.  The function finds the correlation between
  community dissimilarities and environmental distances, and for each
  size of subsets, saves the best result.  There are \eqn{2^p-1}
  subsets of \eqn{p} variables, and an exhaustive search may take a
  very, very, very long time (parameter \code{upto} offers a partial
  relief).

  The argument \code{metric} defines distances in the given set of
  environmental variables.  With \code{metric = "euclidean"}, the
  variables are scaled to unit variance and Euclidean distances are
  calculated. With \code{metric = "mahalanobis"}, the Mahalanobis
  distances are calculated: in addition to scaling to unit variance,
  the matrix of the current set of environmental variables is also
  made orthogonal (uncorrelated). With \code{metric = "manhanttan"},
  the variables are scaled to unit range and Manhattan distances are
  calculated, so that the distances are sums of differences of
  environmental variables.  With \code{metric = "gower"}, the Gower
  distances are calculated using function
  \code{\link[cluster]{daisy}}. This allows also using factor
  variables, but with continuous variables the results are equal to
  \code{metric = "manhattan"}.

  The function can be called with a model \code{\link{formula}} where
  the LHS is the data matrix and RHS lists the environmental variables.
  The formula interface is practical in selecting or transforming
  environmental variables.

  With argument \code{partial} you can perform \dQuote{partial}
  analysis. The partializing item must be a dissimilarity object of
  class \code{\link{dist}}. The
  \code{partial} item can be used with any correlation \code{method},
  but it is strictly correct only for Pearson.

  Function \code{bioenvdist} recalculates the environmental distances
  used within the function. The default is to calculate distances for
  the best model, but the number of any model can be given.
  
  Clarke & Ainsworth (1993) suggested this method to be used for
  selecting the best subset of environmental variables in interpreting
  results of nonmetric multidimensional scaling (NMDS). They recommended a
  parallel display of NMDS of community dissimilarities and NMDS of
  Euclidean distances from the best subset of scaled environmental
  variables.  They warned against the use of Procrustes analysis, but
  to me this looks like a good way of comparing these two ordinations.

  Clarke & Ainsworth wrote a computer program BIO-ENV giving the name to
  the current function. Presumably BIO-ENV
  was later incorporated in Clarke's PRIMER software (available for
  Windows).  In addition, Clarke & Ainsworth suggested a novel method of
  rank correlation which is not available in the current function.
}

\value{
  The function returns an object of class \code{bioenv} with a
  \code{summary} method.
}

\references{
  Clarke, K. R & Ainsworth, M. 1993. A method of linking multivariate
  community structure to environmental variables. \emph{Marine Ecology
    Progress Series}, 92, 205--219.
}
\author{ Jari Oksanen }

\note{ If you want to study the \sQuote{significance} of \code{bioenv}
  results, you can use function \code{\link{mantel}} or
  \code{\link{mantel.partial}} which use the same definition of
  correlation.  However, \code{bioenv} standardizes environmental
  variables depending on the used metric, and you must do the same in
  \code{\link{mantel}} for comparable results (the standardized data are
  returned as item \code{x} in the result object). It is safest to use
  \code{bioenvdist} to extract the environmental distances that really
  were used within \code{bioenv}. NB., \code{bioenv} selects variables
  to maximize the Mantel correlation, and significance tests based on
  \emph{a priori} selection of variables are biased.  }

\seealso{\code{\link{vegdist}}, \code{\link{dist}}, \code{\link{cor}}
  for underlying routines, \code{\link{monoMDS}} and
  \code{\link{metaMDS}} for ordination, \code{\link{procrustes}} for
  Procrustes analysis, \code{\link{protest}} for an alternative, and
  \code{\link{rankindex}} for studying alternatives to the default
  Bray-Curtis index.}

\examples{
# The method is very slow for large number of possible subsets.
# Therefore only 6 variables in this example.
data(varespec)
data(varechem)
sol <- bioenv(wisconsin(varespec) ~ log(N) + P + K + Ca + pH + Al, varechem)
sol
summary(sol)
}
\keyword{ multivariate }

\encoding{UTF-8}
\name{permustats}
\alias{permustats}
\alias{permustats.adonis}
\alias{permustats.anosim}
\alias{permustats.anova.cca}
\alias{permustats.CCorA}
\alias{permustats.envfit}
\alias{permustats.factorfit}
\alias{permustats.mantel}
\alias{permustats.mrpp}
\alias{permustats.mso}
\alias{permustats.oecosimu}
\alias{permustats.ordiareatest}
\alias{permustats.permutest.betadisper}
\alias{permustats.permutest.cca}
\alias{permustats.protest}
\alias{permustats.vectorfit}
\alias{summary.permustats}
\alias{c.permustats}
\alias{densityplot.permustats}
\alias{density.permustats}
\alias{qqnorm.permustats}
\alias{qqmath.permustats}
\alias{boxplot.permustats}

\title{
  Extract, Analyse and Display Permutation Results
}

\description{
  The \code{permustats} function extracts permutation results of
  \pkg{vegan} functions. Its support functions can find quantiles and
  standardized effect sizes, plot densities and Q-Q plots.
}

\usage{
permustats(x, ...)
\method{summary}{permustats}(object, interval = 0.95, alternative, ...)
\method{densityplot}{permustats}(x, data, xlab = "Permutations", ...)
\method{density}{permustats}(x, observed = TRUE, ...)
\method{qqnorm}{permustats}(y, observed = TRUE, ...)
\method{qqmath}{permustats}(x, data, observed = TRUE, sd.scale = FALSE,
    ylab = "Permutations", ...)
\method{boxplot}{permustats}(x, scale = FALSE, names, ...)
}

\arguments{
  \item{object, x, y}{The object to be handled.}
  \item{interval}{numeric; the coverage interval reported.}
  \item{alternative}{A character string specifying the limits used for
    the \code{interval} and the direction of the test when evaluating
    the \eqn{p}-values. Must be one of \code{"two.sided"} (both upper
    and lower limit), \code{"greater"} (upper limit), \code{"less"}
    (lower limit). Usually \code{alternative} is given in the result
    object, but it can be specified with this argument.}
  \item{xlab, ylab}{Arguments of
    \code{\link[lattice]{densityplot}} and
    \code{\link[lattice]{qqmath}} functions.}
  \item{observed}{Add observed statistic among permutations.}
  \item{sd.scale}{Scale permutations to unit standard deviation and observed
    statistic to standardized effect size.}
  \item{data}{Ignored.}
  \item{scale}{Use standardized effect size (SES).}
  \item{names}{Names of boxes (default: names of statistics).}
  \item{\dots}{ Other arguments passed to the function. In
    \code{density} these are passed to \code{\link{density.default}},
    and in \code{boxplot} to \code{\link{boxplot.default}}.}
}

\details{

  The \code{permustats} function extracts permutation results and
  observed statistics from several \pkg{vegan} functions that perform
  permutations or simulations.

  The \code{summary} method of \code{permustats} estimates the
  standardized effect sizes (SES) as the difference of observed
  statistic and mean of permutations divided by the standard deviation
  of permutations (also known as \eqn{z}-values). It also prints the
  the mean, median, and limits which contain \code{interval} percent
  of permuted values. With the default (\code{interval = 0.95}), for
  two-sided test these are (2.5\%, 97.5\%) and for one-sided tests
  either 5\% or 95\% quantile and the \eqn{p}-value depending on the
  test direction. The mean, quantiles and \eqn{z} values are evaluated
  from permuted values without observed statistic, but the
  \eqn{p}-value is evaluated with the observed statistic. The
  intervals and the \eqn{p}-value are evaluated with the same test
  direction as in the original test, but this can be changed with
  argument \code{alternative}. Several \code{permustats} objects can
  be combined with \code{c} function. The \code{c} function checks
  that statistics are equal, but performs no other sanity tests.

  The \code{density} and \code{densityplot} methods display the
  kernel density estimates of permuted values. When observed value of
  the statistic is included in the permuted values, the
  \code{densityplot} method marks the observed statistic as a vertical
  line. However the \code{density} method uses its standard \code{plot}
  method and cannot mark the observed value.

  The \code{qqnorm} and \code{qqmath} display Q-Q plots of
  permutations, optionally together with the observed value (default)
  which is shown as horizontal line in plots. \code{qqnorm} plots
  permutation values against standard Normal variate. \code{qqmath}
  defaults to the standard Normal as well, but can accept other
  alternatives (see standard \code{\link[lattice]{qqmath}}). The
  \code{qqmath} function can also plot observed statistic as
  standardized effect size (SES) with standandized permutations
  (argument \code{sd.scale}). The permutations are standardized
  without the observed statistic, similarly as in \code{summary}.

  Functions \code{\link{density}} and \code{\link{qqnorm}} are based
  on standard \R methods and accept their arguments. They only handle
  one statistic, and cannot be used when several test statistic were
  evaluated. The \code{\link[lattice]{densityplot}} and
  \code{\link[lattice]{qqmath}} are \pkg{lattice} graphics, and can be
  used either for one or for several statistics.  All these functions
  pass arguments to their underlying functions; see their
  documentation. Functions \code{\link[lattice]{qqmath}} and
  \code{\link[lattice]{densityplot}} default to use same axis scaling
  in all subplots of the lattice. You can use argument \code{scales} to
  set independent scaling for subplots when this is appropriate (see
  \code{\link[lattice]{xyplot}} for an exhaustive list of arguments).

  Function \code{boxplot} draws the box-and-whiskers plots of effect
  size, or the difference of permutations and observed statistic. If
  \code{scale = TRUE}, permutations are standardized to unit standard
  deviation, and the plot will show the standardized effect sizes.

  The \code{permustats} can extract permutation statistics from the
  results of \code{\link{adonis2}}, \code{\link{adonis}},
  \code{\link{anosim}}, \code{\link{anova.cca}}, \code{\link{mantel}},
  \code{\link{mantel.partial}}, \code{\link{mrpp}},
  \code{\link{oecosimu}}, \code{\link{ordiareatest}},
  \code{\link{permutest.cca}}, \code{\link{protest}}, and
  \code{\link{permutest.betadisper}}.

}

\value{
  The \code{permustats} function returns an object of class
  \code{"permustats"}. This is a list of items \code{"statistic"} for
  observed statistics, \code{permutations} which contains permuted
  values, and \code{alternative} which contains text defining the
  character of the test (\code{"two.sided"}, \code{"less"} or
  \code{"greater"}). The \code{\link{qqnorm}} and
  \code{\link{density}} methods return their standard result objects.
}

\author{
  Jari Oksanen with contributions from Gavin L. Simpson
  (\code{permustats.permutest.betadisper} method and related
  modifications to \code{summary.permustats} and the \code{print}
  method) and Eduard \enc{Szöcs}{Szoecs} (\code{permustats.anova.cca).}
}

\seealso{
  \code{\link{density}}, \code{\link[lattice]{densityplot}},
  \code{\link{qqnorm}}, \code{\link[lattice]{qqmath}}.
}

\examples{
data(dune, dune.env)
mod <- adonis2(dune ~ Management + A1, data = dune.env)
## use permustats
perm <- permustats(mod)
summary(perm)
densityplot(perm)
qqmath(perm)
boxplot(perm, scale=TRUE, lty=1, pch=16, cex=0.6, col="hotpink", ylab="SES")
abline(h=0, col="skyblue")
## example of multiple types of statistic
mod <- with(dune.env, betadisper(vegdist(dune), Management))
pmod <- permutest(mod, nperm = 99, pairwise = TRUE)
perm <- permustats(pmod)
summary(perm, interval = 0.90)
}

\keyword{ distribution }
\keyword{ smooth }

\name{add1.cca}
\alias{add1.cca}
\alias{drop1.cca}

\title{Add or Drop  Single Terms to a Constrained Ordination Model }
\description{
Compute all single terms that can be added to or dropped from a
constrained ordination model.
}
\usage{
\method{add1}{cca}(object, scope, test = c("none", "permutation"),
    permutations = how(nperm=199), ...)
\method{drop1}{cca}(object, scope, test = c("none", "permutation"), 
    permutations = how(nperm=199), ...)
}

\arguments{
  \item{object}{ A constrained ordination object from
  \code{\link{cca}}, \code{\link{rda}} or \code{\link{capscale}}. }
  \item{scope}{ A formula giving the terms to be considered for adding
  or dropping; see \code{\link{add1}} for details.}
  \item{test}{ Should a permutation test be added using \code{\link{anova.cca}}. }
 
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}

  \item{\dots}{Other arguments passed to \code{\link{add1.default}},
  \code{\link{drop1.default}}, and \code{\link{anova.cca}}.}
}

\details{ 
  With argument \code{test = "none"} the functions will only call
  \code{\link{add1.default}} or \code{\link{drop1.default}}. With
  argument \code{test = "permutation"} the functions will add test
  results from \code{\link{anova.cca}}. Function \code{drop1.cca} will
  call \code{\link{anova.cca}} with argument \code{by = "margin"}.
  Function \code{add1.cca} will implement a test for single term
  additions that is not directly available in \code{\link{anova.cca}}.

  Functions are used implicitly in \code{\link{step}},
  \code{\link{ordiR2step}} and \code{\link{ordistep}}. The
  \code{\link{deviance.cca}} and \code{\link{deviance.rda}} used in
  \code{\link{step}} have no firm basis, and setting argument \code{test
  = "permutation"} may help in getting useful insight into validity of
  model building. Function \code{\link{ordistep}} calls alternately
  \code{drop1.cca} and \code{add1.cca} with argument 
  \code{test = "permutation"} and selects variables by their permutation
  \eqn{P}-values.  Meticulous use of \code{add1.cca} and
  \code{drop1.cca} will allow more judicious model building.

  The default number of \code{permutations} is set to a low value, because
  permutation tests can take a long time. It should be sufficient to
  give a impression on the significances of the terms, but higher
  values of \code{permutations} should be used if \eqn{P} values really
  are important.  

}

\value{
  Returns a similar object as \code{\link{add1}} and \code{\link{drop1}}.
}

\author{Jari Oksanen}

\seealso{ \code{\link{add1}}, \code{\link{drop1}} and
  \code{\link{anova.cca}} for basic methods. You probably need these
  functions with \code{\link{step}} and \code{\link{ordistep}}. Functions
  \code{\link{deviance.cca}} and \code{\link{extractAIC.cca}} are used
  to produce the other arguments than test results in the
  output. Functions \code{\link{cca}}, \code{\link{rda}} and
  \code{\link{capscale}} produce result objects for these functions.}

\examples{
data(dune)
data(dune.env)
## Automatic model building based on AIC but with permutation tests
step(cca(dune ~  1, dune.env), reformulate(names(dune.env)), test="perm")
## see ?ordistep to do the same, but based on permutation P-values
\dontrun{
ordistep(cca(dune ~  1, dune.env), reformulate(names(dune.env)))
}
## Manual model building
## -- define the maximal model for scope
mbig <- rda(dune ~  ., dune.env)
## -- define an empty model to start with
m0 <- rda(dune ~ 1, dune.env)
## -- manual selection and updating
add1(m0, scope=formula(mbig), test="perm")
m0 <- update(m0, . ~ . + Management)
add1(m0, scope=formula(mbig), test="perm")
m0 <- update(m0, . ~ . + Moisture)
## -- included variables still significant?
drop1(m0, test="perm")
add1(m0, scope=formula(mbig), test="perm")
}
\keyword{multivariate }
\keyword{ models}

\encoding{UTF-8}
\name{sipoo}
\alias{sipoo}
\alias{sipoo.map}
\docType{data}
\title{ Birds in the Archipelago of Sipoo (Sibbo and Borgå)}
\description{
Land birds on islands covered by 
coniferous forest in the Sipoo Archipelago, southern Finland.
}
\usage{
  data(sipoo)
  data(sipoo.map)
}

\format{
  The \code{sipoo} data frame contains data of occurrences of 50 land
  bird species on 18 islands in the Sipoo Archipelago (Simberloff &
  Martin, 1991, Appendix 3). The species are referred by 4+4 letter
  abbreviation of their Latin names (but using five letters in two
  species names to make these unique).

  The \code{sipoo.map} data contains the geographic coordinates of the
  islands in the ETRS89-TM35FIN coordinate system (EPSG:3067) and the
  areas of islands in hectares.
}

\source{
  Simberloff, D. & Martin, J.-L.  (1991).  Nestedness of insular
  avifaunas: simple summary statistics masking complex species patterns.
  \emph{Ornis Fennica} 68:178--192.
}

\examples{
data(sipoo)
data(sipoo.map)
plot(N ~ E, data=sipoo.map, asp = 1)
}

\keyword{datasets}

\name{rarefy}
\alias{rarefy}
\alias{rrarefy}
\alias{drarefy}
\alias{rarecurve}
\alias{rareslope}

\title{Rarefaction Species Richness}

\description{ Rarefied species richness for community ecologists.  }

\usage{
rarefy(x, sample, se = FALSE, MARGIN = 1)
rrarefy(x, sample)
drarefy(x, sample)
rarecurve(x, step = 1, sample, xlab = "Sample Size", ylab = "Species",
          label = TRUE, col, lty, ...)
rareslope(x, sample)
}

\arguments{
  \item{x}{Community data, a matrix-like object or a vector.}
  \item{MARGIN}{Margin for which the index is computed. }
  \item{sample}{Subsample size for rarefying community, either a single
    value or a vector.}
  \item{se}{Estimate standard errors.}
  \item{step}{Step size for sample sizes in rarefaction curves.}
  \item{xlab, ylab}{Axis labels in plots of rarefaction curves.}
  \item{label}{Label rarefaction curves by rownames of \code{x}
    (logical).}
  \item{col, lty}{plotting colour and line type, see
    \code{\link{par}}. Can be a vector of length \code{nrow(x)}, one per
    sample, and will be extended to such a length internally.}
  \item{...}{Parameters passed to \code{\link{nlm}}, or to \code{\link{plot}},
    \code{\link{lines}} and \code{\link{ordilabel}} in \code{rarecurve}.}
}
\details{

  Function \code{rarefy} gives the expected species richness in random
  subsamples of size \code{sample} from the community. The size of
  \code{sample} should be smaller than total community size, but the
  function will work for larger \code{sample} as well (with a warning)
  and return non-rarefied species richness (and standard error =
  0). If \code{sample} is a vector, rarefaction of all observations is
  performed for each sample size separately.  Rarefaction can be
  performed only with genuine counts of individuals.  The function
  \code{rarefy} is based on Hurlbert's (1971) formulation, and the
  standard errors on Heck et al. (1975).

  Function \code{rrarefy} generates one randomly rarefied community
  data frame or vector of given \code{sample} size. The \code{sample}
  can be a vector giving the sample sizes for each row.  If the
  \code{sample} size is equal to or smaller than the observed number
  of individuals, the non-rarefied community will be returned.  The
  random rarefaction is made without replacement so that the variance
  of rarefied communities is rather related to rarefaction proportion
  than to the size of the \code{sample}. Random rarefaction is
  sometimes used to remove the effects of different sample
  sizes. This is usually a bad idea: random rarefaction discards valid
  data, introduces random error and reduces the quality of the data
  (McMurdie & Holmes 2014). It is better to use normalizing
  transformations (\code{\link{decostand}} in \pkg{vegan}) possible
  with variance stabilization (\code{\link{decostand}} and
  \code{\link{dispweight}} in \pkg{vegan}) and methods that are not
  sensitive to sample sizes.

  Function \code{drarefy} returns probabilities that species occur in
  a rarefied community of size \code{sample}. The \code{sample} can be
  a vector giving the sample sizes for each row. If the \code{sample}
  is equal to or smaller than the observed number of individuals, all
  observed species will have sampling probability 1.

  Function \code{rarecurve} draws a rarefaction curve for each row of
  the input data. The rarefaction curves are evaluated using the
  interval of \code{step} sample sizes, always including 1 and total
  sample size.  If \code{sample} is specified, a vertical line is
  drawn at \code{sample} with horizontal lines for the rarefied
  species richnesses.

  Function \code{rareslope} calculates the slope of \code{rarecurve}
  (derivative of \code{rarefy}) at given \code{sample} size; the
  \code{sample} need not be an integer.

}

\value{
  A vector of rarefied species richness values. With a single
  \code{sample} and \code{se = TRUE}, function \code{rarefy} returns a
  2-row matrix with rarefied richness (\code{S}) and its standard error
  (\code{se}). If \code{sample} is a vector in \code{rarefy}, the
  function returns a matrix with a column for each \code{sample} size,
  and if \code{se = TRUE}, rarefied richness and its standard error are
  on consecutive lines.

  Function \code{rarecurve} returns \code{\link{invisible}} list of
  \code{rarefy} results corresponding each drawn curve.
}

\references{
  Heck, K.L., van Belle, G. & Simberloff, D. (1975). Explicit
  calculation of the rarefaction diversity measurement and the
  determination of sufficient sample size. \emph{Ecology} \strong{56},
  1459--1461.

  Hurlbert, S.H. (1971). The nonconcept of species diversity: a critique
  and alternative parameters. \emph{Ecology} \strong{52}, 577--586.

  McMurdie, P.J. & Holmes, S. (2014). Waste not, want not: Why
  rarefying microbiome data is inadmissible. \emph{PLoS Comput Biol}
  \strong{10(4):} e1003531. \doi{10.1371/journal.pcbi.1003531}

}

\seealso{Use \code{\link{specaccum}} for species accumulation curves
  where sites are sampled instead of individuals. \code{\link{specpool}}
  extrapolates richness to an unknown sample size.}

\author{Jari Oksanen}

\examples{
data(BCI)
S <- specnumber(BCI) # observed number of species
(raremax <- min(rowSums(BCI)))
Srare <- rarefy(BCI, raremax)
plot(S, Srare, xlab = "Observed No. of Species", ylab = "Rarefied No. of Species")
abline(0, 1)
rarecurve(BCI, step = 20, sample = raremax, col = "blue", cex = 0.6)
}
\keyword{ univar }

\name{vegan-internal}
\alias{vegan-internal}
\alias{ordiParseFormula}
\alias{ordiNAexclude}
\alias{ordiNApredict}
\alias{getPermuteMatrix}
\alias{howHead}
\alias{centroids.cca}
\alias{ordiTerminfo}
\alias{pasteCall}
\alias{ordiArgAbsorber}
\alias{veganCovEllipse}
\alias{hierParseFormula}
\alias{veganMahatrans}
\alias{GowerDblcen}
\alias{addLingoes}
\alias{addCailliez}

\title{Internal vegan functions}

\description{
  Internal vegan functions that are not intended to be called directly,
  but only within other functions.
}
\usage{
ordiParseFormula(formula, data, xlev = NULL,  na.action = na.fail,
    subset = NULL, X)
ordiTerminfo(d, data)
ordiNAexclude(x, excluded)
ordiNApredict(omit, x)
ordiArgAbsorber(..., shrink, origin, scaling, triangular,
                display, choices, const, truemean, FUN)
centroids.cca(x, mf, wt)
getPermuteMatrix(perm, N, strata = NULL)
howHead(x, ...)
pasteCall(call, prefix = "Call:")
veganCovEllipse(cov, center = c(0, 0), scale = 1, npoints = 100)
veganMahatrans(x, s2, tol = sqrt(.Machine$double.eps))
hierParseFormula(formula, data)
GowerDblcen(x, na.rm = TRUE)
addLingoes(d)
addCailliez(d)
}

\details{ The description here is only intended for \pkg{vegan}
  developers: these functions are not intended for users, but they
  only should be used within functions.  In general, these functions
  are not exported to the namespace, but you must use
  \code{\link{get}} or \code{\link{:::}} to directly call these
  functions.
  
  \code{ordiParseFormula} returns a list of three matrices (dependent
  variables, and \code{\link{model.matrix}} of constraints and
  conditions, possibly \code{NULL}) needed in constrained
  ordination. Argument \code{xlev} is passed to
  \code{\link{model.frame}}. If the left-hand-side was already
  evaluated in calling code, it can be given as argument \code{X} and
  will not be re-evaluated. \code{ordiTermInfo} finds the term
  information for constrained ordination as described in
  \code{\link{cca.object}}. \code{ordiNAexclude} implements
  \code{na.action = na.exclude} for constrained ordination finding WA
  scores of CCA components and site scores of unconstrained component
  from \code{excluded} rows of observations. Function
  \code{ordiNApredict} pads the result object with these or with WA
  scores similarly as \code{\link{napredict}}.

  \code{ordiArgAbsorber} absorbs arguments of \code{\link{scores}}
  function of \pkg{vegan} so that these do not cause superfluous
  warnings in graphical function \code{FUN}. If you implement
  \code{scores} functions with new arguments, you should update
  \code{ordiArgAbsorber}. 

  \code{centroids.cca} finds the weighted centroids of variables.

  \code{getPermuteMatrix} interprets user input and returns a
  permutation matrix where each row gives indices of observations for
  a permutation. The input \code{perm} can be a single number for the
  number of simple permutations, a result of
  \code{\link[permute]{how}} defining a permutation scheme or a
  permutation matrix. Function \code{permuted.index} was used earlier
  to generate permutations, but it is now deprecated.

  \code{howHead} formats the permutation scheme of
  \code{\link[permute]{how}} for display. The formatting is more
  compact than the one used in \code{print} in the \pkg{permute}
  package, and shows only non-default choices. This output is normally
  used when printing the results of \pkg{vegan} permutations.

  \code{pasteCall} prints the function call so that it is nicely wrapped
  in \code{\link[utils]{Sweave}} output.

  \code{veganCovEllipse} finds the coordinates for drawing a
  covariance ellipse.

  \code{veganMahatrans} transforms data matrix so that its Euclidean
  distances are Mahalanobis distances. The input data \code{x} must be
  a matrix centred by columns, and \code{s2} its covariance matrix. If
  \code{s2} is not given, covariance matrix is found from \code{x}
  within the function.

  \code{hierParseFormula} returns a list of one matrix (left hand side)
  and a model frame with factors representing hierarchy levels 
  (right hand side) to be used in \code{\link{adipart}}, 
  \code{\link{multipart}} and \code{\link{hiersimu}}.

  \code{GowerDblcen} performs the Gower double centring of a matrix of
  dissimilarities. Similar function was earlier available as a compiled
  code in \pkg{stats}, but it is not a part of official API, and
  therefore we have this poorer replacement.

  \code{addLingoes} and \code{addCailliez} find the constant added to
  non-diagonal (squared) dissimilarities to make all eigenvalues
  non-negative in Principal Co-ordinates Analysis
  (\code{\link{wcmdscale}}, \code{\link{capscale}}). Function
  \code{\link{cmdscale}} implements the Cailliez method. The argument
  is a matrix of dissimilarities.
}

\keyword{internal }

\name{mantel}
\alias{mantel}
\alias{mantel.partial}

\title{Mantel and Partial Mantel Tests for Dissimilarity Matrices }
\description{
  Function \code{mantel}  finds the Mantel statistic as a matrix
  correlation between two dissimilarity matrices, and function
  \code{mantel.partial} finds the partial Mantel statistic as the
  partial matrix correlation between three dissimilarity matrices.  The
  significance of the statistic is evaluated by permuting rows and
  columns of the first dissimilarity matrix. 

}
\usage{
mantel(xdis, ydis, method="pearson", permutations=999, strata = NULL,
    na.rm = FALSE, parallel = getOption("mc.cores"))
mantel.partial(xdis, ydis, zdis, method = "pearson", permutations = 999, 
    strata = NULL, na.rm = FALSE, parallel = getOption("mc.cores"))
}

\arguments{
  \item{xdis, ydis, zdis}{ Dissimilarity matrices or a \code{dist} objects. }
  \item{method}{ Correlation method, as accepted by \code{\link{cor}}:
    \code{"pearson"}, \code{"spearman"} or \code{"kendall"}. }
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}
  \item{strata}{An integer vector or factor specifying the strata for
    permutation. If supplied, observations are permuted only within the
    specified strata.}
  \item{na.rm}{Remove missing values in calculation of Mantel
    correlation. Use this option with care: Permutation tests can
    be biased, in particular if two matrices had missing values in
    matching positions.}
  \item{parallel}{Number of parallel processes or a predefined socket
    cluster.  With \code{parallel = 1} uses ordinary, non-parallel
    processing. The parallel processing is done with \pkg{parallel}
    package.}

}
\details{
  Mantel statistic is simply a correlation between entries of two
  dissimilarity matrices (some use cross products, but these are linearly
  related).  However, the significance cannot be directly assessed,
  because there are \eqn{N(N-1)/2} entries for just \eqn{N} observations.
  Mantel developed asymptotic test, but here we use permutations of
  \eqn{N} rows and columns of dissimilarity matrix.  See
  \code{\link{permutations}} for additional details on permutation tests
  in Vegan.

  Partial Mantel statistic uses partial correlation
  conditioned on the third matrix. Only the first matrix is permuted so
  that the correlation structure between second and first matrices is
  kept constant. Although \code{mantel.partial} silently accepts other
  methods than \code{"pearson"}, partial correlations will probably be
  wrong with other methods.

  The function uses \code{\link{cor}}, which should accept
  alternatives \code{pearson} for product moment correlations and
  \code{spearman} or \code{kendall} for rank correlations.
}
\value{
  The function returns a list of class \code{mantel} with following
  components: 
  \item{Call }{Function call.}
  \item{method }{Correlation method used, as returned by
    \code{\link{cor.test}}.}
  \item{statistic}{The Mantel statistic.}
  \item{signif}{Empirical significance level from permutations.}
  \item{perm}{A vector of permuted values. The distribution of
    permuted values can be inspected with \code{\link{permustats}} 
    function.}
  \item{permutations}{Number of permutations.}
  \item{control}{A list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}.}
}
\references{ The test is due to Mantel, of course, but the
  current implementation is based on Legendre and Legendre.

  Legendre, P. and Legendre, L. (2012) \emph{Numerical Ecology}. 3rd English
  Edition. Elsevier.
  
}

\note{
  Legendre & Legendre (2012, Box 10.4) warn against using partial
  Mantel correlations.
}

\author{Jari Oksanen }


\seealso{\code{\link{cor}} for correlation coefficients,
  \code{\link{protest}} (Procrustes test) for an alternative with
  ordination diagrams, \code{\link{anosim}}  and \code{\link{mrpp}}
  for comparing dissimilarities against 
  classification.  For dissimilarity matrices, see \code{\link{vegdist}}
  or \code{\link{dist}}.  See \code{\link{bioenv}} for selecting
  environmental variables. }

\examples{
## Is vegetation related to environment?
data(varespec)
data(varechem)
veg.dist <- vegdist(varespec) # Bray-Curtis
env.dist <- vegdist(scale(varechem), "euclid")
mantel(veg.dist, env.dist)
mantel(veg.dist, env.dist, method="spear")
}
\keyword{ multivariate }
\keyword{ htest }

\name{simulate.rda}
\alias{simulate.rda}
\alias{simulate.cca}
\alias{simulate.capscale}
\title{ Simulate Responses with Gaussian Error or Permuted Residuals for Constrained Ordination }

\description{ Function simulates a response data frame so that it adds
 Gaussian error to the fitted responses of Redundancy Analysis
 (\code{\link{rda}}), Constrained Correspondence Analysis
 (\code{\link{cca}}) or distance-based RDA (\code{\link{capscale}}).
 The function is a special case of generic \code{\link{simulate}}, and
 works similarly as \code{simulate.lm}.  }

\usage{
\method{simulate}{rda}(object, nsim = 1, seed = NULL, indx = NULL,
    rank = "full", correlated = FALSE, ...)
}
\arguments{

  \item{object}{an object representing a fitted \code{\link{rda}},
    \code{\link{cca}} or \code{\link{capscale}} model.}

  \item{nsim}{number of response matrices to be simulated. Only one
    dissimilarity matrix is returned for \code{\link{capscale}}, and
    larger \code{nsim} is an error.}

  \item{seed}{an object specifying if and how the random number
    generator should be initialized (\sQuote{seeded}). See 
    \code{\link{simulate}} for details. }

  \item{indx}{Index of residuals added to the fitted values, such as
    produced by \code{\link[permute]{shuffleSet}} or
    \code{\link{sample}}.  The index can have duplicate entries so
    that bootstrapping is allowed. If \code{nsim} \eqn{>1}, the output
    should be compliant with \code{\link[permute]{shuffleSet}} with
    one line for each simulation.  If \code{nsim} is missing, the
    number of rows of \code{indx} is used to define the number of
    simulations, but if \code{nsim} is given, it should match number
    of rows in \code{indx}. If null, parametric simulation is used and
    Gaussian error is added to the fitted values.}

  \item{rank}{The rank of the constrained component: passed to
    \code{\link{predict.rda}} or \code{\link{predict.cca}}. }

  \item{correlated}{Are species regarded as correlated in parametric
    simulation or when \code{indx} is not given? If
    \code{correlated = TRUE}, multivariate Gaussian random error is
    generated, and if \code{FALSE}, Gaussian random error is generated
    separately for each species. The argument has no effect in
    \code{\link{capscale}} which has no information on species.}

  \item{\dots}{additional optional arguments (ignored). }
}

\details{ The implementation follows \code{"lm"} method of
  \code{\link{simulate}}, and adds Gaussian (Normal) error to the fitted
  values (\code{\link{fitted.rda}}) using function \code{\link{rnorm}}
  if \code{correlated = FALSE} or \code{\link[MASS]{mvrnorm}} if
  \code{correlated = TRUE}. The standard deviations (\code{\link{rnorm}})
  or covariance matrices for species (\code{\link[MASS]{mvrnorm}}) are
  estimated from the residuals after fitting the constraints.
  Alternatively, the function can take a permutation index that is used
  to add permuted residuals (unconstrained component) to the fitted
  values. Raw data are used in \code{\link{rda}}. Internal Chi-square
  transformed data are used in \code{\link{cca}} within the function,
  but the returned matrix is similar to the original input data. The
  simulation is performed on internal metric scaling data in
  \code{\link{capscale}}, but the function returns the Euclidean
  distances calculated from the simulated data.  The simulation uses
  only the real components, and the imaginary dimensions are ignored.  }

\value{ If \code{nsim = 1}, returns a matrix or dissimilarities (in
  \code{\link{capscale}}) with similar additional arguments on random
  number seed as \code{\link{simulate}}. If \code{nsim > 1}, returns a
  similar array as returned by \code{\link{simulate.nullmodel}} with
  similar attributes.  }

\author{Jari Oksanen}


\seealso{ \code{\link{simulate}} for the generic case and for
  \code{\link{lm}} objects, and \code{\link{simulate.nullmodel}} for
  community null model simulation. Functions \code{\link{fitted.rda}}
  and \code{\link{fitted.cca}} return fitted values without the error
  component. See \code{\link{rnorm}} and \code{\link[MASS]{mvrnorm}}
  (\pkg{MASS} package) for simulating Gaussian random error. }

\examples{
data(dune)
data(dune.env)
mod <- rda(dune ~  Moisture + Management, dune.env)
## One simulation
update(mod, simulate(mod) ~  .)
## An impression of confidence regions of site scores
plot(mod, display="sites")
for (i in 1:5) lines(procrustes(mod, update(mod, simulate(mod) ~ .)), col="blue")
## Simulate a set of null communities with permutation of residuals
simulate(mod, indx = shuffleSet(nrow(dune), 99))
}
\keyword{ models }
\keyword{ datagen }
\keyword{ multivariate }

\encoding{UTF-8}
\name{decostand}
\alias{decostand}
\alias{wisconsin}
\alias{decobackstand}

\title{Standardization Methods for Community Ecology}
\description{
The function provides some popular (and effective) standardization
methods for community ecologists.
}
\usage{
decostand(x, method, MARGIN, range.global, logbase = 2, na.rm=FALSE, ...)
wisconsin(x)
decobackstand(x, zap = TRUE)
}

\arguments{
  \item{x}{Community data, a matrix-like object. For
    \code{decobackstand} standardized data. }
  \item{method}{Standardization method. See Details for available options.}
  \item{MARGIN}{Margin, if default is not acceptable. \code{1} = rows,
    and \code{2} = columns of \code{x}.}
  \item{range.global}{Matrix from which the range is found in
    \code{method = "range"}.  This allows using same ranges across
    subsets of data.  The dimensions of \code{MARGIN} must match with
    \code{x}. }
  \item{logbase}{The logarithm base used in \code{method = "log"}.}
  \item{na.rm}{Ignore missing values in row or column
    standardizations. The \code{NA} values remain as \code{NA}, but they
    are ignored in standardization of other values.}
  \item{zap}{Make near-zero values exact zeros to avoid negative
    values and exaggerated estimates of species richness.}
  \item{\dots}{Other arguments to the function (ignored).}
}
\details{
  The function offers following standardization methods for community
  data:
  \itemize{
    \item \code{total}: divide by margin total (default \code{MARGIN = 1}).
    \item \code{max}: divide by margin maximum (default \code{MARGIN = 2}).
    \item \code{frequency}: divide by margin total and multiply by the
    number of non-zero items, so that the average of non-zero entries is
    one (Oksanen 1983; default \code{MARGIN = 2}).
    \item \code{normalize}: make margin sum of squares equal to one (default
    \code{MARGIN = 1}).
    \item \code{range}: standardize values into range 0 \dots 1 (default
    \code{MARGIN = 2}).  If all values are constant, they will be
    transformed to 0.
    
    \item \code{rank, rrank}: \code{rank} replaces abundance values by
    their increasing ranks leaving zeros unchanged, and \code{rrank} is
    similar but uses relative ranks with maximum 1 (default
    \code{MARGIN = 1}). Average ranks are used for tied values.

    \item \code{standardize}: scale \code{x} to zero mean and unit variance
    (default \code{MARGIN = 2}).
    \item \code{pa}: scale \code{x} to presence/absence scale (0/1).
    \item \code{chi.square}: divide by row sums and square root of
    column sums, and adjust for square root of matrix total
    (Legendre & Gallagher 2001). When used with the Euclidean
    distance, the distances should be similar to the
    Chi-square distance used in correspondence analysis. However, the
    results from \code{\link{cmdscale}} would still differ, since
    CA is a weighted ordination method (default \code{MARGIN = 1}).
    \item \code{hellinger}: square root of \code{method = "total"}
    (Legendre & Gallagher 2001).
    \item \code{log}: logarithmic transformation as suggested by
     Anderson et al. (2006): \eqn{\log_b (x) + 1}{log_b (x) + 1} for
     \eqn{x > 0}, where \eqn{b} is the base of the logarithm; zeros are
     left as zeros. Higher bases give less weight to quantities and more
     to presences, and \code{logbase = Inf} gives the presence/absence
     scaling. Please note this is \emph{not} \eqn{\log(x+1)}{log(x+1)}.
     Anderson et al. (2006) suggested this for their (strongly) modified
     Gower distance (implemented as \code{method = "altGower"} in 
     \code{\link{vegdist}}), but the standardization can be used 
     independently of distance indices.

     \item \code{alr}: Additive log ratio ("alr") transformation
     (Aitchison 1986) reduces data skewness and compositionality
     bias. The transformation assumes positive values, pseudocounts can
     be added with the argument \code{pseudocount}. One of the
     rows/columns is a reference that can be given by \code{reference}
     (name of index). The first row/column is used by default
     (\code{reference = 1}).  Note that this transformation drops one
     row or column from the transformed output data. The \code{alr}
     transformation is defined formally as follows:
     \deqn{alr = [log\frac{x_1}{x_D}, ..., log\frac{x_{D-1}}{x_D}]}{%
       alr = [log(x_i/x_D), ..., log(x_{-D}/x_D)]}
     where the denominator sample \eqn{x_D} can be chosen
     arbitrarily. This transformation is often used with pH and other
     chemistry measurenments. It is also commonly used as multinomial
     logistic regression. Default \code{MARGIN = 1} uses row as the
     \code{reference}.

     \item \code{clr}: centered log ratio ("clr") transformation proposed by
     Aitchison (1986) and it is used to reduce data skewness and compositionality bias.
     This transformation has frequent applications in microbial ecology
     (see e.g. Gloor et al., 2017). The clr transformation is defined as:
     \deqn{clr = log\frac{x}{g(x)} = log x - log g(x)}{%
           clr = log(x/g(x)) = log x - log g(x)}     
     where \eqn{x} is a single value, and g(x) is the geometric mean of
     \eqn{x}.
     The method can operate only with positive data;
     a common way to deal with zeroes is to add pseudocount
     (e.g. the smallest positive value in the data), either by
     adding it manually to the input data, or by using the argument
     \code{pseudocount} as in
     \code{decostand(x, method = "clr", pseudocount = 1)}. Adding
     pseudocount will inevitably introduce some bias; see
     the \code{rclr} method for one available solution.

     \item \code{rclr}: robust clr ("rclr") is similar to regular clr
     (see above) but allows data that contains zeroes. This method
     does not use pseudocounts, unlike the standard clr.
     The robust clr (rclr) divides the values by geometric mean
     of the observed features; zero values are kept as zeroes, and not
     taken into account. In high dimensional data,
     the geometric mean of rclr approximates the true
     geometric mean; see e.g. Martino et al. (2019)
     The \code{rclr} transformation is defined formally as follows:
     \deqn{rclr = log\frac{x}{g(x > 0)}}{%
     rclr = log(x/g(x > 0))}
     where \eqn{x} is a single value, and \eqn{g(x > 0)} is the geometric 
     mean of sample-wide values \eqn{x} that are positive (> 0).
  }
  
  Standardization, as contrasted to transformation, means that the
  entries are transformed relative to other entries.

  All methods have a default margin. \code{MARGIN=1} means rows (sites
  in a normal data set) and \code{MARGIN=2} means columns (species in a
  normal data set).

  Command \code{wisconsin} is a shortcut to common Wisconsin double
  standardization where species (\code{MARGIN=2}) are first standardized
  by maxima (\code{max}) and then sites (\code{MARGIN=1}) by
  site totals (\code{tot}).

  Most standardization methods will give nonsense results with
  negative data entries that normally should not occur in the community
  data. If there are empty sites or species (or constant with
  \code{method =  "range"}), many standardization will change these into
  \code{NaN}.

  Function \code{decobackstand} can be used to transform standardized
  data back to original. This is not possible for all standardization
  and may not be implemented to all cases where it would be
  possible. There are round-off errors and back-transformation is not
  exact, and it is wise not to overwrite the original data. With
  \code{zap=TRUE} original zeros should be exact.
}

\value{
  Returns the standardized data frame, and adds an attribute
  \code{"decostand"} giving the name of applied standardization
  \code{"method"} and attribute \code{"parameters"} with appropriate
  transformation parameters.  }

\author{Jari Oksanen, Etienne \enc{Laliberté}{Laliberte}
  (\code{method = "log"}), Leo Lahti (\code{alr}, 
  \code{"clr"} and \code{"rclr"}).}
  
\note{Common transformations can be made with standard \R functions.}

\references{ 

  Aitchison, J. The Statistical Analysis of Compositional Data (1986).
  London, UK: Chapman & Hall.

  Anderson, M.J., Ellingsen, K.E. & McArdle, B.H. (2006) Multivariate
  dispersion as a measure of beta diversity. \emph{Ecology Letters} 
  \strong{9}, 683--693.

  Egozcue, J.J., Pawlowsky-Glahn, V., Mateu-Figueras, G.,
  Barcel'o-Vidal, C. (2003) Isometric logratio transformations for
  compositional data analysis. \emph{Mathematical Geology}
  \strong{35}, 279--300.

  Gloor, G.B., Macklaim, J.M., Pawlowsky-Glahn, V. & Egozcue, J.J. (2017)
  Microbiome Datasets Are Compositional: And This Is Not Optional.
  \emph{Frontiers in Microbiology} \strong{8}, 2224. 

  Legendre, P. & Gallagher, E.D. (2001) Ecologically meaningful
  transformations for ordination of species data. \emph{Oecologia}
  \strong{129}, 271--280.

  Martino, C., Morton, J.T., Marotz, C.A., Thompson, L.R., Tripathi, A.,
  Knight, R. & Zengler, K. (2019) A novel sparse compositional technique
  reveals microbial perturbations.
  \emph{mSystems} \strong{4}, 1.

  Oksanen, J. (1983) Ordination of boreal heath-like vegetation with
  principal component analysis, correspondence analysis and
  multidimensional scaling. \emph{Vegetatio} \strong{52}, 181--189.
  }

\examples{
data(varespec)
sptrans <- decostand(varespec, "max")
apply(sptrans, 2, max)
sptrans <- wisconsin(varespec)

# CLR transformation for rows, with pseudocount
varespec.clr <- decostand(varespec, "clr", pseudocount=1)

# ALR transformation for rows, with pseudocount and reference sample
varespec.alr <- decostand(varespec, "alr", pseudocount=1, reference=1)

## Chi-square: PCA similar but not identical to CA.
## Use wcmdscale for weighted analysis and identical results.
sptrans <- decostand(varespec, "chi.square")
plot(procrustes(rda(sptrans), cca(varespec)))
}
\keyword{ multivariate}
\keyword{ manip }

\encoding{UTF-8}
\name{dispindmorisita}
\alias{dispindmorisita}
\title{Morisita index of intraspecific aggregation}
\description{
Calculates the Morisita index of dispersion, standardized index values, and the so called clumpedness and uniform indices.
}
\usage{
dispindmorisita(x, unique.rm = FALSE, crit = 0.05, na.rm = FALSE)
}
\arguments{
  \item{x}{community data matrix, with sites (samples) as rows and
    species as columns.}
  \item{unique.rm}{logical, if \code{TRUE}, unique species (occurring
    in only one sample) are removed from the result.}
  \item{crit}{two-sided p-value used to calculate critical
    Chi-squared values.}
  \item{na.rm}{logical.
    Should missing values (including \code{NaN}) be omitted from the
    calculations?}
}
\details{
The Morisita index of dispersion is defined as (Morisita 1959, 1962):

\code{Imor = n * (sum(xi^2) - sum(xi)) / (sum(xi)^2 - sum(xi))}

where \eqn{xi} is the count of individuals in sample \eqn{i}, and
\eqn{n} is the number of samples (\eqn{i = 1, 2, \ldots, n}).
\eqn{Imor} has values from 0 to \eqn{n}. In uniform (hyperdispersed)
patterns its value falls between 0 and 1, in clumped patterns it falls
between 1 and \eqn{n}. For increasing sample sizes (i.e. joining
neighbouring quadrats), \eqn{Imor} goes to \eqn{n} as the
quadrat size approaches clump size. For random patterns,
\eqn{Imor = 1} and counts in the samples follow Poisson
frequency distribution.

The deviation from random expectation (null hypothesis)
can be tested using critical values of the Chi-squared
distribution with \eqn{n-1} degrees of freedom.
Confidence intervals around 1 can be calculated by the clumped
\eqn{Mclu} and uniform \eqn{Muni} indices (Hairston et al. 1971, Krebs
1999) (Chi2Lower and Chi2Upper refers to e.g. 0.025 and 0.975 quantile
values of the Chi-squared distribution with \eqn{n-1} degrees of
freedom, respectively, for \code{crit = 0.05}):

\code{Mclu = (Chi2Lower - n + sum(xi)) / (sum(xi) - 1)}

\code{Muni = (Chi2Upper - n + sum(xi)) / (sum(xi) - 1)}

Smith-Gill (1975) proposed scaling of Morisita index from [0, n]
interval into [-1, 1], and setting up -0.5 and 0.5 values as
confidence limits around random distribution with rescaled value 0. To
rescale the Morisita index, one of the following four equations apply
to calculate the standardized index \eqn{Imst}:

(a) \code{Imor >= Mclu > 1}: \code{Imst = 0.5 + 0.5 (Imor - Mclu) / (n - Mclu)},

(b) \code{Mclu > Imor >= 1}: \code{Imst = 0.5 (Imor - 1) / (Mclu - 1)},

(c) \code{1 > Imor > Muni}: \code{Imst = -0.5 (Imor - 1) / (Muni - 1)},

(d) \code{1 > Muni > Imor}: \code{Imst = -0.5 + 0.5 (Imor - Muni) / Muni}.
}

\value{ Returns a data frame with as many rows as the number of columns
in the input data, and with four columns. Columns are: \code{imor} the
unstandardized Morisita index, \code{mclu} the clumpedness index,
\code{muni} the uniform index, \code{imst} the standardized Morisita
index, \code{pchisq} the Chi-squared based probability for the null
hypothesis of random expectation.
}

\references{

Morisita, M. 1959. Measuring of the dispersion of individuals and
analysis of the distributional patterns.  \emph{Mem. Fac. Sci. Kyushu
Univ. Ser. E} 2, 215--235.

Morisita, M. 1962. Id-index, a measure of dispersion of individuals.
\emph{Res. Popul. Ecol.} 4, 1--7.

Smith-Gill, S. J. 1975. Cytophysiological basis of disruptive pigmentary
patterns in the leopard frog, \emph{Rana pipiens}. II.  Wild type and
mutant cell specific patterns. \emph{J. Morphol.} 146, 35--54.

Hairston, N. G., Hill, R. and Ritte, U. 1971. The interpretation of
aggregation patterns. In: Patil, G. P., Pileou, E. C. and Waters,
W. E. eds. \emph{Statistical Ecology 1: Spatial Patterns and Statistical
Distributions}. Penn. State Univ. Press, University Park.

Krebs, C. J. 1999. \emph{Ecological Methodology}. 2nd ed. Benjamin
Cummings Publishers.  }

\author{\enc{Péter Sólymos}{Peter Solymos}, \email{solymos@ualberta.ca}}

\note{ A common error found in several papers is that when standardizing
as in the case (b), the denominator is given as \code{Muni - 1}. This
results in a hiatus in the [0, 0.5] interval of the standardized
index. The root of this typo is the book of Krebs (1999), see the Errata
for the book (Page 217,
\url{https://www.zoology.ubc.ca/~krebs/downloads/errors_2nd_printing.pdf}).
}

\examples{
data(dune)
x <- dispindmorisita(dune)
x
y <- dispindmorisita(dune, unique.rm = TRUE)
y
dim(x) ## with unique species
dim(y) ## unique species removed
}
\keyword{ multivariate }
\keyword{ spatial }

\name{ordiresids}
\alias{ordiresids}
 
\title{Plots of Residuals and Fitted Values for Constrained Ordination}
\description{
  The function provides \code{\link{plot.lm}} style diagnostic plots for
  the results of constrained ordination from \code{\link{cca}},
  \code{\link{rda}} and \code{\link{capscale}}. Normally you do not need
  these plots, because ordination is descriptive and does not
  make assumptions on the distribution of the residuals. However, if
  you permute residuals in significance tests (\code{\link{anova.cca}}),
  you may be interested in inspecting that the residuals really are
  exchangeable and independent of fitted values.
}
\usage{
ordiresids(x, kind = c("residuals", "scale", "qqmath"),
   residuals = "working", type = c("p", "smooth", "g"),
   formula, ...)
}
\arguments{
  \item{x}{Ordination result from \code{\link{cca}}, \code{\link{rda}}
    or \code{\link{capscale}}. }
  \item{kind}{The type of plot: \code{"residuals"} plot residuals
    against fitted values, \code{"scale"} the square root of absolute
    residuals against fitted values, and \code{"qqmath"} the residuals
    against expected distribution (defaults \code{\link{qnorm}}),
    unless defined differently in the \code{formula}  argument. }
  \item{residuals}{The kind of residuals and fitted values, with alternatives
    \code{"working"}, \code{"response"}, \code{"standardized"} and
    \code{"studentized"} (see Details).}
  \item{type}{The type of plot. The argument is passed on to
    \pkg{lattice} functions. }
  \item{formula}{Formula to override the default plot. The formula can
    contain items \code{Fitted}, \code{Residuals}, \code{Species} and
    \code{Sites} (provided that names of species and sites are available
    in the ordination result).  }
  \item{\dots}{Other arguments passed to \pkg{lattice} functions. }
}

\details{
  The default plots are similar as in \code{\link{plot.lm}}, but they
  use \code{\link[lattice]{Lattice}} functions
  \code{\link[lattice]{xyplot}} and \code{\link[lattice]{qqmath}}. The
  alternatives have default formulae but these can be replaced by the
  user. The elements available in formula or in the \code{groups} argument
  are \code{Fitted}, \code{Residuals}, \code{Species} and \code{Sites}.

  With \code{residuals = "response"} and \code{residuals = "working"}
  the fitted values and residuals are found with functions
  \code{\link{fitted.cca}} and \code{\link{residuals.cca}}. With
  \code{residuals = "standardized"} the residuals are found with
  \code{\link{rstandard.cca}}, and with \code{residuals = "studentized"}
  they are found with \code{\link{rstudent.cca}}, and in both cases the
  fitted values are standardized with \code{\link{sigma.cca}}.
}

\value{
  The function returns a \code{\link[lattice]{Lattice}} object that can
  displayed as plot.
}

\author{Jari Oksanen }
\seealso{

  \code{\link{plot.lm}}, \code{\link{fitted.cca}},
  \code{\link{residuals.cca}}, \code{\link{rstandard.cca}},
  \code{\link{rstudent.cca}}, \code{\link{sigma.cca}},
  \code{\link[lattice]{Lattice}}, \code{\link[lattice]{xyplot}},
  \code{\link[lattice]{qqmath}}.  }


\examples{
data(varespec)
data(varechem)
mod <- cca(varespec ~ Al + P + K, varechem)
ordiresids(mod)
ordiresids(mod, formula = Residuals ~ Fitted | Species, residuals="standard",
   cex = 0.5)
}
\keyword{ hplot }

\name{mite}
\alias{mite}
\alias{mite.env}
\alias{mite.pcnm}
\alias{mite.xy}
\docType{data}
\title{Oribatid Mite Data with Explanatory Variables
  }
\description{
Oribatid mite data. 70 soil cores collected by Daniel Borcard in 1989.
See Borcard et al. (1992, 1994) for details.
}
\usage{
data(mite)
data(mite.env)
data(mite.pcnm)
data(mite.xy)
}
\format{
  There are three linked data sets: \code{mite} that contains the data
  on 35 species of Oribatid mites, \code{mite.env} that contains
  environmental data in the same sampling sites, \code{mite.xy}
  that contains geographic coordinates,  and \code{mite.pcnm}
  that contains 22 PCNM base functions (columns) computed from the geographic
  coordinates of the 70 sampling sites (Borcard & Legendre 2002).
  The whole sampling area was 2.5 m x 10 m in size.

  The fields in the environmental data are:
  \describe{
    \item{SubsDens}{Substrate density (g/L)}
    \item{WatrCont}{Water content of the substrate (g/L)}
    \item{Substrate}{Substrate type, factor with levels \code{Sphagn1,
	Sphagn2 Sphagn3 Sphagn Litter Barepeat Interface} }
    \item{Shrub}{Shrub density, an ordered factor with levels \code{1} <
      \code{2} < \code{3}}
    \item{Topo}{Microtopography, a factor with levels \code{Blanket} and \code{Hummock}}
  }
}
\source{
  Pierre Legendre
}
\references{
Borcard, D., P. Legendre and P. Drapeau. 1992. Partialling out the
spatial component of ecological variation. Ecology 73: 1045-1055.

Borcard, D. and P. Legendre. 1994. Environmental control and spatial
structure in ecological communities: an example using Oribatid mites
(Acari, Oribatei). Environmental and Ecological Statistics 1: 37-61.

Borcard, D. and P. Legendre. 2002. All-scale spatial analysis of
ecological data by means of principal coordinates of neighbour
matrices. Ecological Modelling 153: 51-68.
}
\examples{
data(mite)
}
\keyword{datasets}

\name{influence.cca}
\alias{hatvalues.cca}
\alias{hatvalues.rda}
\alias{sigma.cca}
\alias{rstandard.cca}
\alias{rstudent.cca}
\alias{cooks.distance.cca}
\alias{SSD.cca}
\alias{vcov.cca}
\alias{qr.cca}
\alias{df.residual.cca}

\title{Linear Model Diagnostics for Constrained Ordination}

\description{

  This set of function extracts influence statistics and some other
  linear model statistics directly from a constrained ordination result
  object from \code{\link{cca}}, \code{\link{rda}},
  \code{\link{capscale}} or \code{\link{dbrda}}. The constraints are
  linear model functions and these support functions return identical
  results as the corresponding linear models (\code{\link{lm}}), and you
  can use their documentation. The main functions for normal usage are
  leverage values (\code{\link{hatvalues}}), standardized residuals
  (\code{\link{rstandard}}), studentized or leave-one-out residuals
  (\code{\link{rstudent}}), and Cook's distance
  (\code{\link{cooks.distance}}).  In addition, \code{\link{vcov}}
  returns the variance-covariance matrix of coefficients, and its
  diagonal values the variances of coefficients. Other functions are
  mainly support functions for these, but they can be used directly.

}

\usage{

\method{hatvalues}{cca}(model, ...)
\method{rstandard}{cca}(model, type = c("response", "canoco"), ...)
\method{rstudent}{cca}(model, type = c("response", "canoco"), ...)
\method{cooks.distance}{cca}(model, type = c("response", "canoco"), ...)

\method{sigma}{cca}(object, type = c("response", "canoco"), ...)
\method{vcov}{cca}(object, type = "canoco", ...)
\method{SSD}{cca}(object, type = "canoco", ...)

\method{qr}{cca}(x, ...)
\method{df.residual}{cca}(object, ...)

}

\arguments{

  \item{model, object, x}{A constrained ordination result object.}

  \item{type}{Type of statistics used for extracting raw residuals and
    residual standard deviation (\code{sigma}). Either
    \code{"response"} for species data or difference of WA and LC
    scores for \code{"canoco"}.}

  \item{\dots}{Other arguments to functions (ignored).}

}

\details{

  The \pkg{vegan} algorithm for constrained ordination uses linear model
  (or weighted linear model in \code{\link{cca}}) to find the fitted
  values of dependent community data, and constrained ordination is
  based on this fitted response (Legendre & Legendre 2012). The
  \code{\link{hatvalues}} give the leverage values of these constraints,
  and the leverage is independent on the response data. Other influence
  statistics (\code{\link{rstandard}}, \code{\link{rstudent}},
  \code{\link{cooks.distance}}) are based on leverage, and on the raw
  residuals and residual standard deviation (\code{\link{sigma}}). With
  \code{type = "response"} the raw residuals are given by the
  unconstrained component of the constrained ordination, and influence
  statistics are a matrix with dimensions no. of observations times
  no. of species. For \code{\link{cca}} the statistics are the same as
  obtained from the \code{\link{lm}} model using Chi-square standardized
  species data (see \code{\link{decostand}}) as dependent variable, and
  row sums of community data as weights, and for \code{\link{rda}} the
  \code{\link{lm}} model uses non-modified community data and no
  weights.

  The algorithm in the CANOCO software constraints the results during
  iteration by performing a linear regression of weighted averages (WA)
  scores on constraints and taking the fitted values of this regression
  as linear combination (LC) scores (ter Braak 1984). The WA scores are
  directly found from species scores, but LC scores are linear
  combinations of constraints in the regression. With \code{type =
  "canoco"} the raw residuals are the differences of WA and LC scores,
  and the residual standard deviation (\code{\link{sigma}}) is taken to
  be the axis sum of squared WA scores minus one. These quantities have
  no relationship to residual component of ordination, but they rather
  are methodological artefacts of an algorithm that is not used in
  \pkg{vegan}. The result is a matrix with dimensions no. of
  observations times no. of constrained axes.

  Function \code{\link{vcov}} returns the matrix of variances and
  covariances of regression coefficients. The diagonal values of this
  matrix are the variances, and their square roots give the standard
  errors of regression coefficients. The function is based on
  \code{\link{SSD}} that extracts the sum of squares and crossproducts
  of residuals. The residuals are defined similarly as in influence
  measures and with each \code{type} they have similar properties and
  limitations, and define the dimensions of the result matrix. 

}

\references{

  Legendre, P. and Legendre, L. (2012) \emph{Numerical Ecology}. 3rd
  English ed. Elsevier.
  
  ter Braak, C.J.F. (1984--): CANOCO -- a FORTRAN program for
  \emph{cano}nical \emph{c}ommunity \emph{o}rdination by [partial]
  [detrended] [canonical] correspondence analysis, principal components
  analysis and redundancy analysis. \emph{TNO Inst. of Applied Computer
  Sci., Stat. Dept. Wageningen, The Netherlands}.

  
}

\note{

  Function \code{\link{as.mlm}} casts an ordination object to a multiple
  linear model of class \code{"mlm"} (see \code{\link{lm}}), and similar
  statistics can be derived from that modified object as with this set
  of functions. However, there are some problems in the \R{}
  implementation of the further analysis of multiple linear model
  objects. When the results differ, the current set of functions is more
  probable to be correct. The use of \code{as.mlm} objects should be
  avoided.
  
}

\author{Jari Oksanen}

\seealso{Corresponding \code{\link{lm}} methods and
  \code{\link{as.mlm.cca}}. Function \code{\link{ordiresids}} provides
  lattice graphics for residuals.}

\examples{

data(varespec, varechem)
mod <- cca(varespec ~ Al + P + K, varechem)
## leverage
hatvalues(mod)
plot(hatvalues(mod), type = "h")
## ordination plot with leverages: points with high leverage have
## similar LC and WA scores
plot(mod, type = "n")
ordispider(mod)       # segment from LC to WA scores
points(mod, dis="si", cex=5*hatvalues(mod), pch=21, bg=2) # WA scores
text(mod, dis="bp", col=4)

## deviation and influence
head(rstandard(mod))
head(cooks.distance(mod))

## Influence measures from lm
y <- decostand(varespec, "chi.square") # needed in cca
y1 <- with(y, Cladstel)         # take one species for lm
lmod1 <- lm(y1 ~ Al + P + K, varechem, weights = rowSums(varespec))
## numerically identical within 2e-15
range(cooks.distance(lmod1) - cooks.distance(mod)[, "Cladstel"])

## t-values of regression coefficients based on type = "canoco"
## residuals
coef(mod)
coef(mod)/sqrt(diag(vcov(mod, type = "canoco")))
}

\keyword{ models }
\keyword{ multivariate }

\encoding{UTF-8}
\name{eventstar}
\alias{eventstar}

\title{
Scale Parameter at the Minimum of the Tsallis Evenness Profile
}
\description{
The function \code{eventstar} finds the minimum (\eqn{q^*}{q*}) of the 
evenness profile based on the Tsallis entropy. This scale factor
of the entropy represents a specific weighting of species
relative frequencies that leads to minimum evenness of the
community (Mendes et al. 2008).
}
\usage{
eventstar(x, qmax = 5)
}
\arguments{
  \item{x}{
A community matrix or a numeric vector.
}
  \item{qmax}{
Maximum scale parameter of the Tsallis entropy to be used in 
finding the minimum of Tsallis based evenness
in the range \code{c(0, qmax)}.
}
}
\details{
The function \code{eventstar} finds a characteristic value of the scale 
parameter \eqn{q} of the Tsallis entropy corresponding to
minimum of the evenness (equitability) profile based on Tsallis entropy.
This value was proposed by Mendes et al. (2008) as \eqn{q^*}{q*}.

The \eqn{q^\ast}{q*} index represents the scale parameter of
the one parameter Tsallis diversity family that leads to
the greatest deviation from the maximum equitability given the relative 
abundance vector of a community.

The value of \eqn{q^\ast}{q*} is found by identifying the minimum
of the evenness profile over scaling factor \eqn{q}{q} by
one-dimensional minimization. Because evenness profile is
known to be a convex function, it is guaranteed that underlying
\code{\link{optimize}} function will find a unique solution
if it is in the range \code{c(0, qmax)}.

The scale parameter value \eqn{q^\ast}{q*} is used to 
find corresponding values of diversity (\eqn{H_{q^\ast}}{H.q*}), 
evenness (\eqn{H_{q^\ast}(\max)}{H.q*(max)}),
and numbers equivalent (\eqn{D_{q^\ast}}{D.q*}). For calculation
details, see \code{\link{tsallis}} and Examples below.

Mendes et al. (2008) advocated the use of \eqn{q^\ast}{q*}
and corresponding diversity, evenness, and Hill numbers, because
it is a unique value representing the diversity profile, and is
is positively associated with rare species in the community,
thus it is a potentially useful indicator of certain
relative abundance distributions of the communities.
}
\value{
A data frame with columns:
\itemize{
  \item{\code{qstar}}{ scale parameter value \eqn{q\ast}{q*}
    corresponding to minimum value of Tsallis based evenness profile.}
  \item{\code{Estar}}{ Value of evenness based on normalized Tsallis 
    entropy at \eqn{q^\ast}{q*}.}
  \item{\code{Hstar}}{ Value of Tsallis entropy at \eqn{q^\ast}{q*}.}
  \item{\code{Dstar}}{ Value of Tsallis entropy at \eqn{q^\ast}{q*} 
    converted to numbers equivalents
    (also called as Hill numbers, effective number of species, 
    \sQuote{true} diversity; cf. Jost 2007).}
}
See \code{\link{tsallis}} for calculation details.
}
\references{
Mendes, R.S., Evangelista, L.R., Thomaz, S.M.,
  Agostinho, A.A. and Gomes, L.C. (2008) A unified
  index to measure ecological diversity and species
  rarity. \emph{Ecography} \bold{31}, 450--456.

Jost, L. (2007) Partitioning diversity into independent alpha and beta components.
  \emph{Ecology} \bold{88}, 2427--2439.

Tsallis, C. (1988) Possible generalization of Boltzmann-Gibbs statistics. 
  \emph{J. Stat. Phis.} \bold{52}, 479--487.
}
\note{
Values for \eqn{q^\ast}{q*} found by Mendes et al. (2008) ranged
from 0.56 and 1.12 presenting low variability, so an
interval between 0 and 5 should safely encompass
the possibly expected \eqn{q^\ast}{q*} values in practice,
but profiling the evenness and changing the value of
the \code{qmax} argument is advised if output values
near the range limits are found.
}
\author{
Eduardo Ribeiro Cunha \email{edurcunha@gmail.com} and 
Heloisa Beatriz Antoniazi Evangelista \email{helobeatriz@gmail.com}, 
with technical input of Péter Sólymos.
}
\seealso{
Tsallis entropy: \code{\link{tsallis}}
}
\examples{
data(BCI)
(x <- eventstar(BCI[1:5,]))
## profiling
y <- as.numeric(BCI[10,])
(z <- eventstar(y))
q <- seq(0, 2, 0.05)
Eprof <- tsallis(y, scales=q, norm=TRUE)
Hprof <- tsallis(y, scales=q)
Dprof <- tsallis(y, scales=q, hill=TRUE)
opar <- par(mfrow=c(3,1))
plot(q, Eprof, type="l", main="Evenness")
abline(v=z$qstar, h=tsallis(y, scales=z$qstar, norm=TRUE), col=2)
plot(q, Hprof, type="l", main="Diversity")
abline(v=z$qstar, h=tsallis(y, scales=z$qstar), col=2)
plot(q, Dprof, type="l", main="Effective number of species")
abline(v=z$qstar, h=tsallis(y, scales=z$qstar, hill=TRUE), col=2)
par(opar)
}
\keyword{ optimize }
\keyword{ multivariate }
\keyword{ utilities }

% Based on:
% File src/library/stats/man/cmdscale.Rd
% Part of the R package, http://www.R-project.org
% Copyright 1995-2007 R Core Development Team
% Distributed under GPL 2 or later

\name{wcmdscale}
\alias{wcmdscale}
\alias{scores.wcmdscale}
\alias{plot.wcmdscale}

\title{Weighted Classical (Metric) Multidimensional Scaling}
\usage{
wcmdscale(d, k, eig = FALSE, add = FALSE, x.ret = FALSE, w)
\method{plot}{wcmdscale}(x, choices = c(1, 2), type = "t", ...)
\method{scores}{wcmdscale}(x, choices = NA, ...)
}
\description{
  Weighted classical multidimensional scaling,
  also known as weighted \emph{principal coordinates analysis}.
}
\arguments{
  \item{d}{a distance structure such as that returned by \code{dist}
    or a full symmetric matrix containing the dissimilarities.}
  \item{k}{the dimension of the space which the data are to be
    represented in; must be in \eqn{\{1,2,\ldots,n-1\}}{{1,2,\ldots,n-1}}.
    If missing, all dimensions with above zero eigenvalue.}
  \item{eig}{indicates whether eigenvalues should be returned.}
  \item{add}{an additive constant \eqn{c} is added to the non-diagonal
    dissimilarities such that all \eqn{n-1} eigenvalues are
    non-negative. Alternatives are \code{"lingoes"} (default, also
    used with \code{TRUE}) and \code{"cailliez"} (which is the only
    alternative in \code{\link{cmdscale}}). See Legendre & Anderson
    (1999).}
  \item{x.ret}{indicates whether the doubly centred symmetric distance
    matrix should be returned.}
  \item{w}{Weights of points.}
  \item{x}{The \code{wcmdscale} result object when the function was
    called with options \code{eig = TRUE} or \code{x.ret = TRUE} (See
    Details).}
  \item{choices}{Axes to be returned; \code{NA} returns all real axes.}
  \item{type}{Type of graph which may be \code{"t"}ext, \code{"p"}oints
    or \code{"n"}one.}
  \item{\dots}{Other arguments passed to graphical functions.}
}
\details{
  Function \code{wcmdscale} is based on function
  \code{\link{cmdscale}} (package \pkg{stats} of base \R), but it uses
  point weights. Points with high weights will have a stronger
  influence on the result than those with low weights. Setting equal
  weights \code{w = 1} will give ordinary multidimensional scaling.

  With default options, the function returns only a matrix of scores
  scaled by eigenvalues for all real axes. If the function is called
  with \code{eig = TRUE} or \code{x.ret = TRUE}, the function returns
  an object of class \code{"wcmdscale"} with \code{print},
  \code{plot}, \code{scores}, \code{\link{eigenvals}} and
  \code{\link{stressplot}} methods, and described in section Value.

  The method is Euclidean, and with non-Euclidean dissimilarities some
  eigenvalues can be negative. If this disturbs you, this can be
  avoided by adding a constant to non-diagonal dissimilarities making
  all eigenvalues non-negative. The function implements methods
  discussed by Legendre & Anderson (1999): The method of Lingoes
  (\code{add="lingoes"}) adds the constant \eqn{c} to squared
  dissimilarities \eqn{d} using \eqn{\sqrt{d^2 + 2 c}}{sqrt(d^2 + 2*c)}
  and the method of Cailliez (\code{add="cailliez"}) to
  dissimilarities using \eqn{d + c}. Legendre & Anderson (1999)
  recommend the method of Lingoes, and base \R{} function
  \code{\link{cmdscale}} implements the method of Cailliez.
}

\value{ If \code{eig = FALSE} and \code{x.ret = FALSE} (default), a
  matrix with \code{k} columns whose rows give the coordinates of
  points corresponding to positive eigenvalues.  Otherwise, an object
  of class \code{wcmdscale} containing the components that are mostly
  similar as in \code{\link{cmdscale}}:

  \item{points}{a matrix with \code{k} columns whose rows give the
    coordinates of the points chosen to represent the
    dissimilarities.}
  \item{eig}{the \eqn{n-1} eigenvalues computed during the scaling
    process if \code{eig} is true.}
  \item{x}{the doubly centred and weighted distance matrix if
    \code{x.ret} is true.}
  \item{ac, add}{additive constant and adjustment method used to avoid
    negative eigenvalues. These are \code{NA} and \code{FALSE} if no
    adjustment was done.}
  \item{GOF}{Goodness of fit statistics for \code{k} axes. The first
    value is based on the sum of absolute values of all eigenvalues,
    and the second value is based on the sum of positive eigenvalues}
  \item{weights}{Weights.}
  \item{negaxes}{A matrix of scores for axes with negative eigenvalues
     scaled by the absolute eigenvalues similarly as
     \code{points}. This is \code{NULL} if there are no negative
     eigenvalues or \code{k} was specified, and would not include
     negative eigenvalues.}
  \item{call}{Function call.}
}

\references{
  Gower, J. C. (1966)
  Some distance properties of latent root and vector
  methods used in multivariate analysis.
  \emph{Biometrika} \bold{53}, 325--328.

  Legendre, P. & Anderson, M. J. (1999). Distance-based redundancy
    analysis: testing multispecies responses in multifactorial
    ecological experiments. \emph{Ecology} \bold{69}, 1--24.

  Mardia, K. V., Kent, J. T. and Bibby, J. M. (1979).  Chapter 14 of
  \emph{Multivariate Analysis}, London: Academic Press.
}

\seealso{The function is modelled after \code{\link{cmdscale}}, but adds
  weights (hence name) and handles negative eigenvalues differently.
  \code{\link{eigenvals.wcmdscale}} and
  \code{\link{stressplot.wcmdscale}} are some specific methods. Other
  multidimensional scaling methods are \code{\link{monoMDS}}, and
  \code{\link[MASS]{isoMDS}} and \code{\link[MASS]{sammon}} in package
  \pkg{MASS}.  }

\examples{
## Correspondence analysis as a weighted principal coordinates
## analysis of Euclidean distances of Chi-square transformed data
data(dune)
rs <- rowSums(dune)/sum(dune)
d <- dist(decostand(dune, "chi"))
ord <- wcmdscale(d, w = rs, eig = TRUE)
## Ordinary CA
ca <- cca(dune)
## Eigevalues are numerically similar
ca$CA$eig - ord$eig
## Configurations are similar when site scores are scaled by
## eigenvalues in CA
procrustes(ord, ca, choices=1:19, scaling = "sites")
plot(procrustes(ord, ca, choices=1:2, scaling="sites"))
## Reconstruction of non-Euclidean distances with negative eigenvalues
d <- vegdist(dune)
ord <- wcmdscale(d, eig = TRUE)
## Only positive eigenvalues:
cor(d, dist(ord$points))
## Correction with negative eigenvalues:
cor(d, sqrt(dist(ord$points)^2 - dist(ord$negaxes)^2))
}
\keyword{multivariate}

\name{cca}
\alias{cca}
\alias{cca.default}
\alias{cca.formula}
\alias{rda}
\alias{rda.default}
\alias{rda.formula}

\title{ [Partial] [Constrained] Correspondence Analysis and Redundancy
  Analysis } 
\description{
  Function \code{cca} performs correspondence analysis, or optionally
  constrained correspondence analysis (a.k.a. canonical correspondence
  analysis), or optionally partial constrained correspondence
  analysis. Function \code{rda} performs redundancy analysis, or
  optionally principal components analysis.
  These are all very popular ordination techniques in community ecology.
}
\usage{
\method{cca}{formula}(formula, data, na.action = na.fail, subset = NULL,
  ...)
\method{rda}{formula}(formula, data, scale=FALSE, na.action = na.fail,
  subset = NULL, ...)
\method{cca}{default}(X, Y, Z, ...)
\method{rda}{default}(X, Y, Z, scale=FALSE, ...)
}

\arguments{
  \item{formula}{Model formula, where the left hand side gives the
    community data matrix, right hand side gives the constraining variables,
    and conditioning variables can be given within a special function
    \code{Condition}.}
  \item{data}{Data frame containing the variables on the right hand side
    of the model formula.}
  \item{X}{ Community data matrix. }
  \item{Y}{ Constraining matrix, typically of environmental variables.
    Can be missing. If this is a \code{data.frame}, it will be
    expanded to a \code{\link{model.matrix}} where factors are
    expanded to contrasts (\dQuote{dummy variables}). It is better to
    use \code{formula} instead of this argument, and some further
    analyses only work when \code{formula} was used.}
  \item{Z}{ Conditioning matrix, the effect of which is removed
    (\dQuote{partialled out}) before next step. Can be missing. If this is a
    \code{data.frame}, it is expanded similarly as constraining
    matrix.}
\item{scale}{Scale species to unit variance (like correlations).}
  \item{na.action}{Handling of missing values in constraints or
    conditions. The default (\code{\link{na.fail}}) is to stop with
    missing value. Choice \code{\link{na.omit}} removes all rows with
    missing values. Choice \code{\link{na.exclude}} keeps all
    observations but gives \code{NA} for results that cannot be
    calculated. The WA scores of rows may be found also for missing
    values in constraints. Missing values are never allowed in
    dependent community data. }
  \item{subset}{Subset of data rows. This can be a logical vector which
    is \code{TRUE} for kept observations, or a logical expression which
    can contain variables in the working environment, \code{data} or
    species names of the community data.}
  \item{...}{Other arguments for \code{print} or \code{plot} functions
    (ignored in other functions).}
}
\details{
  Since their introduction (ter Braak 1986), constrained, or canonical,
  correspondence analysis and its spin-off, redundancy analysis, have
  been the most popular ordination methods in community ecology.
  Functions \code{cca} and \code{rda} are  similar to popular
  proprietary software \code{Canoco}, although the implementation is
  completely different.  The functions are based on Legendre &
  Legendre's (2012) algorithm: in \code{cca}
  Chi-square transformed data matrix is subjected to weighted linear
  regression on constraining variables, and the fitted values are
  submitted to correspondence analysis performed via singular value
  decomposition (\code{\link{svd}}). Function \code{rda} is similar, but uses
  ordinary, unweighted linear regression and unweighted SVD. Legendre &
  Legendre (2012), Table 11.5 (p. 650) give a skeleton of the RDA
  algorithm of \pkg{vegan}. The algorithm of CCA is similar, but
  involves standardization by row and column weights.

  The functions can be called either with matrix-like entries for
  community data and constraints, or with formula interface.  In
  general, the formula interface is preferred, because it allows a
  better control of the model and allows factor constraints. Some
  analyses of ordination results are only possible if model was fitted
  with formula (e.g., most cases of \code{\link{anova.cca}}, automatic
  model building).

  In the following sections, \code{X}, \code{Y} and \code{Z}, although
  referred to as matrices, are more commonly data frames.

  In the matrix interface, the
  community data matrix \code{X} must be given, but the other data
  matrices may be omitted, and the corresponding stage of analysis is
  skipped.  If matrix \code{Z} is supplied, its effects are removed from
  the community matrix, and the residual matrix is submitted to the next
  stage.  This is called partial correspondence or redundancy
  analysis.  If matrix
  \code{Y} is supplied, it is used to constrain the ordination,
  resulting in constrained or canonical correspondence analysis, or
  redundancy analysis.
  Finally, the residual is submitted to ordinary correspondence
  analysis (or principal components analysis).  If both matrices
  \code{Z} and \code{Y} are missing, the
  data matrix is analysed by ordinary correspondence analysis (or
  principal components analysis).

  Instead of separate matrices, the model can be defined using a model
  \code{\link{formula}}.  The left hand side must be the
  community data matrix (\code{X}).  The right hand side defines the
  constraining model.
  The constraints can contain ordered or unordered factors,
  interactions among variables and functions of variables.  The defined
  \code{\link{contrasts}} are honoured in \code{\link{factor}}
  variables.  The constraints can also be matrices (but not data
  frames).
  The formula can include a special term \code{Condition}
  for conditioning variables (\dQuote{covariables}) partialled out before
  analysis.  So the following commands are equivalent:
  \code{cca(X, Y, Z)},  \code{cca(X ~ Y + Condition(Z))}, where \code{Y}
  and \code{Z} refer to constraints and conditions matrices respectively.

  Constrained correspondence analysis is indeed a constrained method:
  CCA does not try to display all variation in the
  data, but only the part that can be explained by the used constraints.
  Consequently, the results are strongly dependent on the set of
  constraints and their transformations or interactions among the
  constraints.  The shotgun method is to use all environmental variables
  as constraints.  However, such exploratory problems are better
  analysed with
  unconstrained methods such as correspondence analysis
  (\code{\link{decorana}}, \code{\link[MASS]{corresp}}) or non-metric
  multidimensional scaling (\code{\link{metaMDS}}) and
  environmental interpretation after analysis
  (\code{\link{envfit}}, \code{\link{ordisurf}}).
  CCA is a good choice if the user has
  clear and strong \emph{a priori} hypotheses on constraints and is not
  interested in the major structure in the data set.  

  CCA is able to correct the curve artefact commonly found in
  correspondence analysis by forcing the configuration into linear
  constraints.  However, the curve artefact can be avoided only with a
  low number of constraints that do not have a curvilinear relation with
  each other.  The curve can reappear even with two badly chosen
  constraints or a single factor.  Although the formula interface makes it
  easy to include polynomial or interaction terms, such terms often
  produce curved artefacts (that are difficult to interpret), these
  should probably be avoided.

  According to folklore, \code{rda} should be used with ``short
  gradients'' rather than \code{cca}. However, this is not based
  on research which finds methods based on Euclidean metric as uniformly
  weaker than those based on Chi-squared metric.  However, standardized
  Euclidean distance may be an appropriate measures (see Hellinger
  standardization in \code{\link{decostand}} in particular).
  
  Partial CCA (pCCA; or alternatively partial RDA) can be used to remove
  the effect of some
  conditioning or background or random variables or
  covariables before CCA proper.  In fact, pCCA compares models
  \code{cca(X ~ Z)} and \code{cca(X ~ Y + Z)} and attributes their
  difference to the effect of \code{Y} cleansed of the effect of
  \code{Z}.  Some people have used the method for extracting
  \dQuote{components of variance} in CCA.  However, if the effect of
  variables together is stronger than sum of both separately, this can
  increase total Chi-square after partialling out some
  variation, and give negative \dQuote{components of variance}.  In general,
  such components of \dQuote{variance} are not to be trusted due to
  interactions between two sets of variables.
  
  The functions have \code{summary} and \code{plot} methods which are
  documented separately (see \code{\link{plot.cca}}, \code{\link{summary.cca}}).

}
\value{
  Function \code{cca} returns a huge object of class \code{cca}, which
  is described separately in \code{\link{cca.object}}.

  Function \code{rda} returns an object of class \code{rda} which
  inherits from class \code{cca} and is described in \code{\link{cca.object}}.
  The scaling used in \code{rda} scores is described in a separate
  vignette with this package.
}
\references{ The original method was by ter Braak, but the current
  implementation follows Legendre and Legendre.

  Legendre, P. and Legendre, L. (2012) \emph{Numerical Ecology}. 3rd English
  ed. Elsevier.

  McCune, B. (1997) Influence of noisy environmental data on canonical
  correspondence analysis. \emph{Ecology} \strong{78}, 2617-2623.
  
  Palmer, M. W. (1993) Putting things in even better order: The
  advantages of canonical correspondence analysis.  \emph{Ecology}
  \strong{74},2215-2230. 
  
  Ter Braak, C. J. F. (1986) Canonical Correspondence Analysis: a new
  eigenvector technique for multivariate direct gradient
  analysis. \emph{Ecology} \strong{67}, 1167-1179.
  
}
\author{
  The responsible author was Jari Oksanen, but the code borrows heavily
  from Dave Roberts (Montana State University, USA).
}

\seealso{
  
  This help page describes two constrained ordination functions,
  \code{cca} and \code{rda}.  A related method, distance-based
  redundancy analysis (dbRDA) is described separately
  (\code{\link{capscale}}). All these functions return similar objects
  (described in \code{\link{cca.object}}). There are numerous support
  functions that can be used to access the result object. In the list
  below, functions of type \code{cca} will handle all three constrained
  ordination objects, and functions of \code{rda} only handle \code{rda}
  and \code{\link{capscale}} results.

  The main plotting functions are \code{\link{plot.cca}} for all
  methods, and \code{\link{biplot.rda}} for RDA and dbRDA.  However,
  generic \pkg{vegan} plotting functions can also handle the results.
  The scores can be accessed and scaled with \code{\link{scores.cca}},
  and summarized with \code{\link{summary.cca}}. The eigenvalues can
  be accessed with \code{\link{eigenvals.cca}} and the regression
  coefficients for constraints with \code{\link{coef.cca}}.  The
  eigenvalues can be plotted with \code{\link{screeplot.cca}}, and the
  (adjusted) \eqn{R^2}{R-squared} can be found with
  \code{\link{RsquareAdj.rda}}. The scores can be also calculated for
  new data sets with \code{\link{predict.cca}} which allows adding
  points to ordinations.  The values of constraints can be inferred
  from ordination and community composition with
  \code{\link{calibrate.cca}}.

  Diagnostic statistics can be found with \code{\link{goodness.cca}},
  \code{\link{inertcomp}}, \code{\link{spenvcor}},
  \code{\link{intersetcor}}, \code{\link{tolerance.cca}}, and
  \code{\link{vif.cca}}.  Function \code{\link{as.mlm.cca}} refits the
  result object as a multiple \code{\link{lm}} object, and this allows
  finding influence statistics (\code{\link{lm.influence}},
  \code{\link{cooks.distance}} etc.).
  
  Permutation based significance for the overall model, single
  constraining variables or axes can be found with
  \code{\link{anova.cca}}.  Automatic model building with \R{}
  \code{\link{step}} function is possible with
  \code{\link{deviance.cca}}, \code{\link{add1.cca}} and
  \code{\link{drop1.cca}}.  Functions \code{\link{ordistep}} and
  \code{\link{ordiR2step}} (for RDA) are special functions for
  constrained ordination. Randomized data sets can be generated with
  \code{\link{simulate.cca}}.

  Separate methods based on constrained ordination model are principal
  response curves (\code{\link{prc}}) and variance partitioning between
  several components (\code{\link{varpart}}).

  Design decisions are explained in \code{\link{vignette}}
  on \dQuote{Design decisions} which can be accessed with
  \code{browseVignettes("vegan")}.

  Package \pkg{ade4} provides alternative constrained ordination
  function \code{\link[ade4]{pcaiv}}.

}

\examples{
data(varespec)
data(varechem)
## Common but bad way: use all variables you happen to have in your
## environmental data matrix
vare.cca <- cca(varespec, varechem)
vare.cca
plot(vare.cca)
## Formula interface and a better model
vare.cca <- cca(varespec ~ Al + P*(K + Baresoil), data=varechem)
vare.cca
plot(vare.cca)
## Partialling out and negative components of variance
cca(varespec ~ Ca, varechem)
cca(varespec ~ Ca + Condition(pH), varechem)
## RDA
data(dune)
data(dune.env)
dune.Manure <- rda(dune ~ Manure, dune.env)
plot(dune.Manure) 
}
\keyword{ multivariate }

\name{vegan-package}
\alias{vegan-package}
\alias{vegan}
\docType{package}
\title{
Community Ecology Package: Ordination, Diversity and Dissimilarities
}

\description{ The \pkg{vegan} package provides tools for descriptive
community ecology. It has most basic functions of diversity analysis,
community ordination and dissimilarity analysis. Most of its
multivariate tools can be used for other data types as well.  }

\details{The functions in the \pkg{vegan} package contain tools for
diversity analysis, ordination methods and tools for the analysis of
dissimilarities. Together with the \pkg{labdsv} package, the \pkg{vegan}
package provides most standard tools of descriptive community
analysis. Package \pkg{ade4} provides an alternative comprehensive
package, and several other packages complement \pkg{vegan} and provide
tools for deeper analysis in specific fields. Package
\pkg{BiodiversityR} provides a GUI for a large subset of \pkg{vegan}
functionality.

The \pkg{vegan} package is developed at GitHub
(\url{https://github.com/vegandevs/vegan/}).  GitHub provides up-to-date
information and forums for bug reports.

Most important changes in \pkg{vegan} documents can be read with
\code{news(package="vegan")} and vignettes can be browsed with
\code{browseVignettes("vegan")}. The vignettes include a \pkg{vegan}
FAQ, discussion on design decisions, short introduction to ordination
and discussion on diversity methods.
%A tutorial of the package at
%\url{http://cc.oulu.fi/~jarioksa/opetus/metodi/vegantutor.pdf} provides
%a more thorough introduction to the package.

To see the preferable citation of the package, type
\code{citation("vegan")}.  
}

\author{ The \pkg{vegan} development team is Jari Oksanen,
F. Guillaume Blanchet, Roeland Kindt, Pierre Legendre, Peter
R. Minchin, R. B. O'Hara, Gavin L. Simpson, Peter Solymos, M. Henry
H. Stevens, Helene Wagner.  Many other people have contributed to
individual functions: see credits in function help pages.
}
\examples{
### Example 1: Unconstrained ordination
## NMDS
data(varespec)
data(varechem)
ord <- metaMDS(varespec)
plot(ord, type = "t")
## Fit environmental variables
ef <- envfit(ord, varechem)
ef
plot(ef, p.max = 0.05)
### Example 2: Constrained ordination (RDA)
## The example uses formula interface to define the model
data(dune)
data(dune.env)
## No constraints: PCA
mod0 <- rda(dune ~ 1, dune.env)
mod0
plot(mod0)
## All environmental variables: Full model
mod1 <- rda(dune ~ ., dune.env)
mod1
plot(mod1)
## Automatic selection of variables by permutation P-values
mod <- ordistep(mod0, scope=formula(mod1))
mod
plot(mod)
## Permutation test for all variables
anova(mod)
## Permutation test of "type III" effects, or significance when a term
## is added to the model after all other terms
anova(mod, by = "margin")
## Plot only sample plots, use different symbols and draw SD ellipses 
## for Managemenet classes
plot(mod, display = "sites", type = "n")
with(dune.env, points(mod, disp = "si", pch = as.numeric(Management)))
with(dune.env, legend("topleft", levels(Management), pch = 1:4,
  title = "Management"))
with(dune.env, ordiellipse(mod, Management, label = TRUE))
## add fitted surface of diversity to the model
ordisurf(mod, diversity(dune), add = TRUE)
### Example 3: analysis of dissimilarites a.k.a. non-parametric
### permutational anova
adonis(dune ~ ., dune.env)
adonis(dune ~ Management + Moisture, dune.env)
}

\keyword{ package }
\keyword{ multivariate }
\keyword{ univar }
\keyword{ models }
\keyword{ spatial }
\keyword{ nonparametric }
\keyword{ htest }
\keyword{ regression }

\encoding{UTF-8}
\name{MOStest}
\alias{MOStest}
\alias{plot.MOStest}
\alias{fieller.MOStest}
\alias{profile.MOStest}
\alias{confint.MOStest}

\title{ Mitchell-Olds \& Shaw Test for the Location of Quadratic Extreme }
\description{
  Mitchell-Olds & Shaw test concerns the location of the highest (hump)
  or lowest (pit) value of a quadratic curve at given points. Typically,
  it is used to study whether the quadratic hump or pit is located
  within a studied interval. The current test is generalized so that it
  applies generalized linear models (\code{\link{glm}}) with link
  function instead of simple quadratic curve.  The test was popularized
  in ecology for the analysis of humped species richness patterns
  (Mittelbach et al. 2001), but it is more general. With logarithmic
  link function, the quadratic response defines the Gaussian response
  model of ecological gradients (ter Braak & Looman 1986), and the test
  can be used for inspecting the location of Gaussian optimum within a
  given range of the gradient. It can also be used to replace Tokeshi's
  test of \dQuote{bimodal} species frequency distribution. 
}
\usage{
MOStest(x, y, interval, ...)
\method{plot}{MOStest}(x, which = c(1,2,3,6), ...)
fieller.MOStest(object, level = 0.95)
\method{profile}{MOStest}(fitted, alpha = 0.01, maxsteps = 10, del = zmax/5, ...)
\method{confint}{MOStest}(object, parm = 1, level = 0.95, ...)
}

\arguments{
  \item{x}{The independent variable or plotting object in \code{plot}. }
  \item{y}{The dependent variable. }
  \item{interval}{The two points at which the test statistic is
    evaluated. If missing, the extremes of \code{x} are used. }
  \item{which}{Subset of plots produced. Values \code{which=1} and
    \code{2} define plots specific to \code{MOStest} (see Details), and
    larger values select graphs of \code{\link{plot.lm}} (minus 2). }
  \item{object, fitted}{A result object from \code{MOStest}.}
  \item{level}{The confidence level required.}
  \item{alpha}{Maximum significance level allowed.}
  \item{maxsteps}{Maximum number of steps in the profile.}
  \item{del}{A step length parameter for the profile (see code).}
  \item{parm}{Ignored.}
  \item{\dots}{ Other variables passed to functions. Function
    \code{MOStest} passes these to \code{\link{glm}} so that
    these can include \code{\link{family}}. The other functions pass
    these to underlying graphical functions. }
}

\details{

  The function fits a quadratic curve \eqn{\mu = b_0 + b_1 x + b_2
  x^2} with given \code{\link{family}} and link function.  If \eqn{b_2
  < 0}, this defines a unimodal curve with highest point at \eqn{u =
  -b_1/(2 b_2)} (ter Braak & Looman 1986). If \eqn{b_2 > 0}, the
  parabola has a minimum at \eqn{u} and the response is sometimes
  called \dQuote{bimodal}.  The null hypothesis is that the extreme
  point \eqn{u} is located within the interval given by points
  \eqn{p_1} and \eqn{p_2}. If the extreme point \eqn{u} is exactly at
  \eqn{p_1}, then \eqn{b_1 = 0} on shifted axis \eqn{x - p_1}.  In the
  test, origin of \code{x} is shifted to the values \eqn{p_1} and
  \eqn{p_2}, and the test statistic is based on the differences of
  deviances between the original model and model where the origin is
  forced to the given location using the standard
  \code{\link{anova.glm}} function (Oksanen et al. 2001).
  Mitchell-Olds & Shaw (1987) used the first degree coefficient with
  its significance as estimated by the \code{\link{summary.glm}}
  function.  This give identical results with Normal error, but for
  other error distributions it is preferable to use the test based on
  differences in deviances in fitted models.

  The test is often presented as a general test for the location of the
  hump, but it really is dependent on the quadratic fitted curve. If the
  hump is of different form than quadratic, the test may be
  insignificant.

  Because of strong assumptions in the test, you should use the support
  functions to inspect the fit. Function \code{plot(..., which=1)}
  displays the data points, fitted quadratic model, and its approximate
  95\% confidence intervals (2 times SE). Function \code{plot} with
  \code{which = 2} displays the approximate confidence interval of
  the polynomial coefficients, together with two lines indicating the
  combinations of the coefficients that produce the evaluated points of
  \code{x}. Moreover, the cross-hair shows the approximate confidence
  intervals for the polynomial coefficients ignoring their
  correlations. Higher values of \code{which} produce corresponding
  graphs from \code{\link{plot.lm}}. That is, you must add 2 to the
  value of \code{which} in \code{\link{plot.lm}}.

  Function \code{fieller.MOStest} approximates the confidence limits
  of the location of the extreme point (hump or pit) using Fieller's
  theorem following ter Braak & Looman (1986). The test is based on
  quasideviance except if the \code{\link{family}} is \code{poisson}
  or \code{binomial}. Function \code{profile} evaluates the profile
  deviance of the fitted model, and \code{confint} finds the profile
  based confidence limits following Oksanen et al. (2001).

  The test is typically used in assessing the significance of diversity
  hump against productivity gradient (Mittelbach et al. 2001). It also
  can be used for the location of the pit (deepest points) instead of
  the Tokeshi test. Further, it can be used to test the location of the
  the Gaussian optimum in ecological gradient analysis (ter Braak &
  Looman 1986, Oksanen et al. 2001).
}

\value{
  The function is based on \code{\link{glm}}, and it returns the result
  of object of \code{glm} amended with the result of the test. The new
  items in the \code{MOStest} are: 
  \item{isHump }{\code{TRUE} if the response is a
    hump.}
  \item{isBracketed}{\code{TRUE} if the hump or the pit is bracketed by
    the evaluated points.} 
  \item{hump}{Sorted vector of location of the hump or the pit and the
    points where the test was evaluated.}
  \item{coefficients}{Table of test statistics and their significances.}
}

\references{
Mitchell-Olds, T. & Shaw, R.G. 1987. Regression analysis of natural
selection: statistical inference and biological
interpretation. \emph{Evolution} 41, 1149--1161.

Mittelbach, G.C. Steiner, C.F., Scheiner, S.M., Gross, K.L., Reynolds,
H.L., Waide, R.B., Willig, R.M., Dodson, S.I. & Gough, L. 2001. What is
the observed relationship between species richness and productivity?
\emph{Ecology} 82, 2381--2396.

Oksanen, J., Läärä, E., Tolonen, K. & Warner, B.G. 2001. Confidence
intervals for the optimum in the Gaussian response
function. \emph{Ecology} 82, 1191--1197.

ter Braak, C.J.F & Looman, C.W.N 1986. Weighted averaging, logistic
regression and the Gaussian response model. \emph{Vegetatio} 65,
3--11. 
}
\author{Jari Oksanen }

\note{ 
Function \code{fieller.MOStest} is based on package \pkg{optgrad} in
the Ecological Archives
(\url{https://figshare.com/articles/Full_Archive/3521975})
accompanying Oksanen et al. (2001). The Ecological Archive package
\pkg{optgrad} also contains profile deviance method for the location
of the hump or pit, but the current implementation of \code{profile}
and \code{confint} rather follow the example of
\code{\link[MASS]{profile.glm}} and \code{\link[MASS]{confint.glm}} in
the \pkg{MASS} package.
}

\seealso{The no-interaction model can be fitted with \code{\link{humpfit}}. }
\examples{
## The Al-Mufti data analysed in humpfit():
mass <- c(140,230,310,310,400,510,610,670,860,900,1050,1160,1900,2480)
spno <- c(1,  4,  3,  9, 18, 30, 20, 14,  3,  2,  3,  2,  5,  2)
mod <- MOStest(mass, spno)
## Insignificant
mod
## ... but inadequate shape of the curve
op <- par(mfrow=c(2,2), mar=c(4,4,1,1)+.1)
plot(mod)
## Looks rather like log-link with Poisson error and logarithmic biomass
mod <- MOStest(log(mass), spno, family=quasipoisson)
mod
plot(mod)
par(op)
## Confidence Limits
fieller.MOStest(mod)
confint(mod)
plot(profile(mod))
}

\keyword{ models }
\keyword{ regression }

\name{SSarrhenius}
\alias{SSarrhenius}
\alias{SSlomolino}
\alias{SSgitay}
\alias{SSgleason}

\title{
  Self-Starting nls Species-Area Models
}

\description{
  These functions provide self-starting species-area models for
  non-linear regression (\code{\link{nls}}). They can also be used for
  fitting species accumulation models in
  \code{\link{fitspecaccum}}. These models (and many more) are reviewed
  by Dengler (2009).
}

\usage{
SSarrhenius(area, k, z)
SSgleason(area, k, slope)
SSgitay(area, k, slope)
SSlomolino(area, Asym, xmid, slope)
}

\arguments{
  \item{area}{
    Area or size of the sample: the independent variable.
  }
  \item{k, z, slope, Asym, xmid}{
    Estimated model parameters: see Details.
  }
}
\details{
  All these functions are assumed to be used for species richness
  (number of species) as the independent variable, and area or sample
  size as the independent variable. Basically, these define least
  squares models of untransformed data, and will differ from models
  for transformed species richness or models with non-Gaussian error.

  The Arrhenius model (\code{SSarrhenius}) is the expression
  \code{k*area^z}. This is the most classical model that can be found in
  any textbook of ecology (and also in Dengler 2009). Parameter \code{z}
  is the steepness of the species-area curve, and \code{k} is the
  expected number of species in a unit area.

  The Gleason model (\code{SSgleason}) is a linear expression 
  \code{k + slope*log(area)} (Dengler 200). This is a linear model,  
  and starting values give the final estimates; it is provided to 
  ease comparison with other models.

  The Gitay model (\code{SSgitay}) is a quadratic logarithmic expression
  \code{(k + slope*log(area))^2} (Gitay et al. 1991, Dengler
  2009). Parameter \code{slope} is the steepness of the species-area
  curve, and \code{k} is the square root of expected richness in a unit
  area. 

  The Lomolino model (\code{SSlomolino}) is
  \code{Asym/(1 + slope^log(xmid/area))} (Lomolino 2000, Dengler 2009).
  Parameter \code{Asym} is the asymptotic maximum number of species,
  \code{slope} is the maximum slope of increase of richness, and
  \code{xmid} is the  area where half of the maximum richness is
  achieved. 

  In addition to these models, several other models studied by Dengler
  (2009) are available in standard \R self-starting models:
  Michaelis-Menten (\code{\link{SSmicmen}}), Gompertz
  (\code{\link{SSgompertz}}), logistic (\code{\link{SSlogis}}), Weibull
  (\code{\link{SSweibull}}), and some others that may be useful.
    
}
\value{
  Numeric vector of the same length as \code{area}. It is the value of
  the expression of each model. If all arguments are names of objects
  the gradient matrix with respect to these names is attached as an
  attribute named \code{gradient}. 
}
\references{
  Dengler, J. (2009) Which function describes the species-area
  relationship best? A review and empirical evaluation. \emph{Journal of
    Biogeography} 36, 728--744.

  Gitay, H., Roxburgh, S.H. & Wilson, J.B. (1991) Species-area
  relationship in a New Zealand tussock grassland, with implications for
  nature reserve design and for community structure. \emph{Journal of
  Vegetation Science} 2, 113--118.

  Lomolino, M. V. (2000) Ecology's most general, yet protean pattern:
  the species-area relationship. \emph{Journal of Biogeography} 27,
  17--26. 
}
\author{
  Jari Oksanen.
}

\seealso{
  \code{\link{nls}}, \code{\link{fitspecaccum}}. 
}
\examples{
## Get species area data: sipoo.map gives the areas of islands
data(sipoo, sipoo.map)
S <- specnumber(sipoo)
plot(S ~ area, sipoo.map,  xlab = "Island Area (ha)",
  ylab = "Number of Species", ylim = c(1, max(S)))
## The Arrhenius model
marr <- nls(S ~ SSarrhenius(area, k, z), data=sipoo.map)
marr
## confidence limits from profile likelihood
confint(marr)
## draw a line
xtmp <- with(sipoo.map, seq(min(area), max(area), len=51))
lines(xtmp, predict(marr, newdata=data.frame(area = xtmp)), lwd=2)
## The normal way is to use linear regression on log-log data,
## but this will be different from the previous:
mloglog <- lm(log(S) ~ log(area), data=sipoo.map)
mloglog
lines(xtmp, exp(predict(mloglog, newdata=data.frame(area=xtmp))),
   lty=2)
## Gleason: log-linear
mgle <- nls(S ~ SSgleason(area, k, slope), sipoo.map)
lines(xtmp, predict(mgle, newdata=data.frame(area=xtmp)),
  lwd=2, col=2)
## Gitay: quadratic of log-linear
mgit <- nls(S ~ SSgitay(area, k, slope), sipoo.map)
lines(xtmp, predict(mgit, newdata=data.frame(area=xtmp)),
  lwd=2, col = 3)
## Lomolino: using original names of the parameters (Lomolino 2000):
mlom <- nls(S ~ SSlomolino(area, Smax, A50, Hill), sipoo.map)
mlom
lines(xtmp, predict(mlom, newdata=data.frame(area=xtmp)),
  lwd=2, col = 4)
## One canned model of standard R:
mmic <- nls(S ~ SSmicmen(area, slope, Asym), sipoo.map)
lines(xtmp, predict(mmic, newdata = data.frame(area=xtmp)),
  lwd =2, col = 5)
legend("bottomright", c("Arrhenius", "log-log linear", "Gleason", "Gitay", 
  "Lomolino", "Michaelis-Menten"), col=c(1,1,2,3,4,5), lwd=c(2,1,2,2,2,2), 
   lty=c(1,2,1,1,1,1))
## compare models (AIC)
allmods <- list(Arrhenius = marr, Gleason = mgle, Gitay = mgit, 
   Lomolino = mlom, MicMen= mmic)
sapply(allmods, AIC)
}
\keyword{ models }

\name{vegan-defunct}
\alias{vegan-defunct}

\alias{getNumObs}
\alias{permuted.index2}
\alias{metaMDSrotate}
\alias{density.adonis}
\alias{density.anosim}
\alias{density.mantel}
\alias{density.mrpp}
\alias{density.permutest.cca}
\alias{density.protest}
\alias{plot.vegandensity}
\alias{densityplot.adonis}
\alias{density.oecosimu}
\alias{densityplot.oecosimu}
\alias{commsimulator}

%------ NOTE:  ../R/vegan-defunct.R   must be synchronized with this!
\title{Defunct Functions in Package \pkg{vegan}}
%------ PLEASE: one \alias{.} for EACH ! (+ one \usage{} & \arguments{} for all)
\description{
  The functions or variables listed here are no longer part of
  \pkg{vegan} as they are no longer needed.
}
\usage{
## defunct in vegan 2.5-0
commsimulator(x, method, thin=1)

## defunct in vegan 2.4-0
\method{density}{adonis}(x, ...)
\method{plot}{vegandensity}(x, main = NULL, xlab = NULL, ylab = "Density", 
   type = "l", zero.line = TRUE, obs.line = TRUE, ...)
\method{densityplot}{adonis}(x, data, xlab = "Null", ...)

## defunct in vegan 2.2-0
metaMDSrotate(object, vec, na.rm = FALSE, ...)

## defunct in vegan 2.0-0
getNumObs(object, \dots)
permuted.index2(n, control = permControl())

}

\details{

  Function \code{commsimulator} is replaced with
  \code{\link{make.commsim}} which defines the Null models, and
  functions \code{\link{nullmodel}} and
  \code{\link{simulate.nullmodel}} that check the input data and
  generate the Null model communities.

  The deprecated \code{density} and \code{densityplot} methods are
  replaced with similar methods for \code{\link{permustats}}. The
  \code{\link{permustats}} offers more powerful analysis tools for
  permutations, including \code{\link{summary.permustats}} giving
  \eqn{z} values (a.k.a. standardized effect sizes, SES), and Q-Q
  plots (\code{\link{qqnorm.permustats}},
  \code{\link{qqmath.permustats}}).

  Function \code{metaMDSrotate} is replaced with
  \code{\link{MDSrotate}} which can handle \code{\link{monoMDS}}
  results in addition to \code{\link{metaMDS}}.

  The permutation functions were moved to the \pkg{permute} package,
  and they are documented there.  The \pkg{permute} package replaces
  \code{permuted.index2} with \code{\link[permute]{shuffle}} and
  \code{getNumObs} with its specific
  \code{\link[permute]{nobs-methods}}.

}

\seealso{
  \code{\link{Defunct}}, \code{\link{vegan-deprecated}}
}

\keyword{internal}

\name{read.cep}
\alias{read.cep}

\title{Reads a CEP (Canoco) data file }
\description{
  \code{read.cep} reads a file formatted with relaxed strict CEP format
  used in Canoco software, among others.
}
\usage{
read.cep(file, positive=TRUE)
}

\arguments{
  \item{file}{File name (character variable). }
  \item{positive}{Only positive entries, like in community data.}
}
\details{

  Cornell Ecology Programs (CEP) introduced several data formats
  designed for punched cards.  One of these was the \sQuote{condensed
  strict} format which was adopted by popular software DECORANA and
  TWINSPAN. A relaxed variant of this format was later adopted in
  Canoco software (ter Braak 1984). Function \code{read.cep} reads
  legacy files written in this format.

  The condensed CEP and CANOCO formats have:
  \itemize{
    \item Two or three title cards, most importantly specifying the format
      and the number of items per record.
    \item Data in condensed format: First number on the line is the
      site identifier (an integer), and it is followed by pairs
      (\sQuote{couplets}) of numbers identifying the species and its
      abundance (an integer and a floating point number).
    \item Species and site names, given in Fortran format \code{(10A8)}:
      Ten names per line, eight columns for each.
  }

  With option \code{positive = TRUE} the function removes all rows and
  columns with zero or negative marginal sums.  In community data
  with only positive entries, this removes empty sites and species.
  If data entries can be negative, this ruins data, and such data sets
  should be read in with option \code{positive = FALSE}.
}
\value{
  Returns a data frame, where columns are species and rows are
  sites. Column and row names are taken from the CEP file, and changed
  into unique \R names by \code{\link{make.names}} after stripping the blanks.
}
\references{ 
  ter Braak, C.J.F. (1984--): CANOCO -- a FORTRAN program for \emph{cano}nical
  \emph{c}ommunity \emph{o}rdination by [partial] [detrended] [canonical]
  correspondence analysis, principal components analysis and redundancy
  analysis. \emph{TNO Inst. of Applied Computer Sci., Stat. Dept. Wageningen,
  The Netherlands}. 
}
\author{ Jari Oksanen }

\note{

  Function \code{read.cep} used Fortran to read data in \pkg{vegan}
  2.4-5 and earlier, but Fortran I/O is no longer allowed in CRAN
  packages, and the function was re-written in \R. The original
  Fortran code was more robust, and there are several legacy data sets
  that may fail with the current version, but could be read with the
  previous Fortran version. CRAN package \pkg{cepreader} makes
  available the original Fortran-based code run in a separate
  subprocess. The \pkg{cepreader} package can also read \sQuote{free}
  and \sQuote{open} Canoco formats that are not handled in this
  function.

  The function is based on \code{\link[utils]{read.fortran}}. If the
  \code{REAL} format defines a decimal part for species abundances
  (such as \code{F5.1}), \code{\link[utils]{read.fortran}} divides the
  input with the corresponding power of 10 even when the input data
  had explicit decimal separator. With \code{F5.1}, 100 would become
  10, and 0.1 become 0.01. Function \code{read.cep} tries to undo this
  division, but you should check the scaling of results after reading
  the data, and if necessary, multiply results to the original scale.

}


\examples{
## Provided that you have the file "dune.spe"
\dontrun{
theclassic <- read.cep("dune.spe")}
}
\keyword{ IO }
\keyword{ file }

\name{treedive}
\alias{treedive}
\alias{treeheight}
\alias{treedist}  

\title{Functional Diversity and Community Distances from Species Trees}

\description{ Functional diversity is defined as the total branch
  length in a trait dendrogram connecting all species, but excluding
  the unnecessary root segments of the tree (Petchey and Gaston
  2006). Tree distance is the increase in total branch length when
  combining two sites.  }

\usage{
treedive(comm, tree, match.force = TRUE, verbose = TRUE)
treeheight(tree)
treedist(x, tree, relative = TRUE, match.force = TRUE, ...)
}

\arguments{
  \item{comm, x}{Community data frame or matrix.}
  \item{tree}{A dendrogram which for \code{treedive} must be for species
    (columns).}
  \item{match.force}{Force matching of column names in data
    (\code{comm}, \code{x}) and labels in \code{tree}. If \code{FALSE},
    matching only happens when dimensions differ (with a warning or
    message). The order of data must match to the order in \code{tree}
    if matching by names is not done.}
  \item{verbose}{Print diagnostic messages and warnings.}
  \item{relative}{Use distances relative to the height of combined tree.}
  \item{\dots}{Other arguments passed to functions (ignored).}
}
\details{
  Function \code{treeheight} finds the sum of lengths of connecting
  segments in a dendrogram produced by \code{\link{hclust}}, or other
  dendrogram that can be coerced to a correct type using
  \code{\link{as.hclust}}. When applied to a clustering of species
  traits, this is a measure of functional diversity (Petchey and Gaston
  2002, 2006), and when applied to phylogenetic trees this is
  phylogenetic diversity.

  Function \code{treedive} finds the \code{treeheight} for each site
  (row) of a community matrix. The function uses a subset of
  dendrogram for those species that occur in each site, and excludes
  the tree root if that is not needed to connect the species (Petchey
  and Gaston 2006). The subset of the dendrogram is found by first
  calculating \code{\link{cophenetic}} distances from the input
  dendrogram, then reconstructing the dendrogram for the subset of the
  cophenetic distance matrix for species occurring in each
  site. Diversity is 0 for one species, and \code{NA} for empty
  communities.

  Function \code{treedist} finds the dissimilarities among
  trees. Pairwise dissimilarity of two trees is found by combining
  species in a common tree and seeing how much of the tree height is
  shared and how much is unique. With \code{relative = FALSE} the
  dissimilarity is defined as \eqn{2 (A \cup B) - A - B}{2AB - A - B}, where
  \eqn{A} and \eqn{B} are heights of component trees and
  \eqn{A \cup B}{AB} is the height of the combined tree. With \code{relative = TRUE}
  the dissimilarity is \eqn{(2(A \cup B)-A-B)/(A \cup B)}{(2AB - A - B)/(AB)}. 
  Although the latter formula is similar to
  Jaccard dissimilarity (see \code{\link{vegdist}},
  \code{\link{designdist}}), it is not in the range \eqn{0 \ldots 1}{0
  \dots 1}, since combined tree can add a new root. When two zero-height
  trees are combined into a tree of above zero height, the relative
  index attains its maximum value \eqn{2}. The dissimilarity is zero
  from a combined zero-height tree.

  The functions need a dendrogram of species traits or phylogenies as an
  input. If species traits contain \code{\link{factor}} or
  \code{\link{ordered}} factor variables, it is recommended to use Gower
  distances for mixed data (function \code{\link[cluster]{daisy}} in
  package \pkg{cluster}), and usually the recommended clustering method
  is UPGMA (\code{method = "average"} in function \code{\link{hclust}})
  (Podani and Schmera 2006). Phylogenetic trees can be changed into
  dendrograms using \code{\link[ape]{as.hclust.phylo}} (package
  \pkg{ape})

  It is possible to analyse the non-randomness of tree diversity
  using \code{\link{oecosimu}}. This needs specifying an adequate Null
  model, and the results will change with this choice.
}

\value{ 
  A vector of diversity values or a single tree height, or a
  dissimilarity structure that inherits from \code{\link{dist}} and
  can be used similarly.  
}

\references{
  Lozupone, C. and Knight, R. 2005. UniFrac: a new phylogenetic method
  for comparing microbial communities. \emph{Applied and Environmental
  Microbiology} 71, 8228--8235.

  Petchey, O.L. and Gaston, K.J. 2002. Functional diversity (FD), species
  richness and community composition. \emph{Ecology Letters} 5,
  402--411.
  
  Petchey, O.L. and Gaston, K.J. 2006. Functional diversity: back to
  basics and looking forward. \emph{Ecology Letters} 9, 741--758.

  Podani J. and Schmera, D. 2006. On dendrogram-based methods of
  functional diversity. \emph{Oikos} 115, 179--185.
}

\author{Jari Oksanen}

\seealso{
  Function \code{treedive} is similar to the phylogenetic
  diversity function \code{\link[picante]{pd}} in \pkg{picante}, but
  excludes tree root if that is not needed to connect species. Function
  \code{treedist} is similar to the phylogenetic similarity
  \code{\link[picante]{phylosor}} in \pkg{picante}, but excludes
  unneeded tree root and returns distances instead of similarities.

  \code{\link{taxondive}} is something very similar from another world.
}

\examples{

## There is no data set on species properties yet, and we demonstrate
## the methods using phylogenetic trees
data(dune)
data(dune.phylodis)
cl <- hclust(dune.phylodis)
treedive(dune, cl)
## Significance test using Null model communities.
## The current choice fixes numbers of species and picks species
## proportionally to their overall frequency
oecosimu(dune, treedive, "r1", tree = cl, verbose = FALSE)
## Phylogenetically ordered community table
dtree <- treedist(dune, cl)
tabasco(dune, hclust(dtree), cl)
## Use tree distances  in capscale
capscale(dtree ~ 1, comm=dune)
}

\keyword{ univar }

\name{isomap}
\alias{isomap}
\alias{isomapdist}
\alias{plot.isomap}
\alias{summary.isomap}

\title{ Isometric Feature Mapping Ordination }
\description{
  The function performs isometric feature mapping which consists of
  three simple steps: (1) retain only some of the shortest
  dissimilarities among objects, (2) estimate all dissimilarities as
  shortest path distances, and (3) perform metric scaling (Tenenbaum et
  al. 2000).
}
\usage{
isomap(dist, ndim=10, ...)
isomapdist(dist, epsilon, k, path = "shortest", fragmentedOK =FALSE, ...)
\method{summary}{isomap}(object, axes = 4, ...)
\method{plot}{isomap}(x, net = TRUE, n.col = "gray", type = "points", ...)
}

\arguments{
  \item{dist}{Dissimilarities. }
  \item{ndim}{Number of axes in metric scaling (argument \code{k} in 
   \code{\link{cmdscale}}).}
  \item{epsilon}{Shortest dissimilarity retained. }
  \item{k}{Number of shortest dissimilarities retained for a point. If
   both \code{epsilon} and \code{k} are given, \code{epsilon} will be used.  }
  \item{path}{Method used in \code{\link{stepacross}} to estimate the
    shortest path, with alternatives \code{"shortest"} and \code{"extended"}. }
  \item{fragmentedOK}{What to do if dissimilarity matrix is
    fragmented. If \code{TRUE}, analyse the largest connected group,
    otherwise stop with error. }
  \item{x, object}{An \code{isomap} result object.}
  \item{axes}{Number of axes displayed.}
  \item{net}{Draw the net of retained dissimilarities.}
  \item{n.col}{Colour of drawn net segments. This can also be a vector
    that is recycled for points, and the colour of the net segment is
    a mixture of joined points.}
  \item{type}{Plot observations either as \code{"points"},
    \code{"text"} or use \code{"none"} to plot no observations. The
    \code{"text"} will use \code{\link{ordilabel}} if \code{net = TRUE} 
    and \code{\link{ordiplot}} if \code{net = FALSE}, and pass
    extra arguments to these functions.}
  \item{\dots}{Other parameters passed to functions. }
}
\details{
  The function \code{isomap} first calls function \code{isomapdist} for
  dissimilarity transformation, and then performs metric scaling for the
  result. All arguments to \code{isomap} are passed to
  \code{isomapdist}. The functions are separate so that the
  \code{isompadist} transformation could be easily used with other
  functions than simple linear mapping of \code{\link{cmdscale}}.
  
  Function \code{isomapdist} retains either dissimilarities equal or shorter to
  \code{epsilon}, or if \code{epsilon} is not given, at least \code{k}
  shortest dissimilarities for a point.  Then a complete dissimilarity
  matrix is reconstructed using \code{\link{stepacross}} using either
  flexible shortest paths or extended dissimilarities (for details, see
  \code{\link{stepacross}}).

  De'ath (1999) actually published essentially the same method before
  Tenenbaum et al. (2000), and De'ath's function is available in function
  \code{xdiss} in non-CRAN package \pkg{mvpart}. The differences are that
  \code{isomap} introduced the \code{k} criterion, whereas De'ath only
  used \code{epsilon} criterion.  In practice, De'ath also retains
  higher proportion of dissimilarities than typical \code{isomap}.

  The \code{plot} function uses internally \code{\link{ordiplot}},
  except that it adds text over net using \code{\link{ordilabel}}. The
  \code{plot} function passes extra arguments to these functions.  In
  addition, \pkg{vegan3d} package has function
  \code{\link[vegan3d]{rgl.isomap}} to make dynamic 3D plots that can
  be rotated on the screen.
}

\value{
  Function \code{isomapdist} returns a dissimilarity object similar to
  \code{dist}. Function \code{isomap} returns an object of class
  \code{isomap} with \code{plot} and \code{summary} methods. The
  \code{plot} function returns invisibly an object of class
  \code{\link{ordiplot}}. Function \code{\link{scores}} can extract
  the ordination scores.
}
\references{
    De'ath, G. (1999)  Extended dissimilarity: a method of robust
     estimation of ecological distances from high beta diversity data.
     \emph{Plant Ecology} 144, 191--199

     Tenenbaum, J.B., de Silva, V. & Langford, J.C. (2000) A global
     network framework for nonlinear dimensionality
     reduction. \emph{Science} 290, 2319--2323.
}
\author{ Jari Oksanen }
\note{ 
  Tenenbaum et al. (2000) justify \code{isomap} as a tool of unfolding a
  manifold (e.g. a 'Swiss Roll'). Even with a manifold structure, the
  sampling must be even and dense so
  that dissimilarities along a manifold are shorter than across the
  folds. If data do not have such a manifold structure, the results are
  very sensitive to parameter values. 
}

\seealso{The underlying functions that do the proper work are
  \code{\link{stepacross}}, \code{\link{distconnected}} and
  \code{\link{cmdscale}}.  Function \code{\link{metaMDS}} may trigger
  \code{\link{stepacross}} transformation, but usually only for
  longest dissimilarities.  The \code{plot} method of \pkg{vegan}
  minimum spanning tree function (\code{\link{spantree}}) has even
  more extreme way of isomapping things. }

\examples{
## The following examples also overlay minimum spanning tree to
## the graphics in red.
op <- par(mar=c(4,4,1,1)+0.2, mfrow=c(2,2))
data(BCI)
dis <- vegdist(BCI)
tr <- spantree(dis)
pl <- ordiplot(cmdscale(dis), main="cmdscale")
lines(tr, pl, col="red")
ord <- isomap(dis, k=3)
ord
pl <- plot(ord, main="isomap k=3")
lines(tr, pl, col="red")
pl <- plot(isomap(dis, k=5), main="isomap k=5")
lines(tr, pl, col="red")
pl <- plot(isomap(dis, epsilon=0.45), main="isomap epsilon=0.45")
lines(tr, pl, col="red")
par(op)
## colour points and web by the dominant species
dom <- apply(BCI, 1, which.max)
## need nine colours, but default palette  has only eight
op <- palette(c(palette("default"), "sienna"))
plot(ord, pch = 16, col = dom, n.col = dom) 
palette(op)
}
\keyword{ multivariate}

\encoding{UTF-8}
\name{indpower}
\alias{indpower}

\title{Indicator Power of Species}

\description{ Indicator power calculation of Halme et al.  (2009)
  or the congruence between indicator and target species.  
}

\usage{
indpower(x, type = 0)
}
\arguments{
  \item{x}{Community data frame or matrix. }
  \item{type}{The type of
  statistic to be returned. See Details for explanation.}  
}
\details{

  Halme et al. (2009) described an index of indicator power defined as
  \eqn{IP_I = \sqrt{a \times  b}}{IP_I = sqrt(a*b)}, where \eqn{a = S / O_I} and
  \eqn{b = 1 - (O_T - S) / (N - O_I)}. 
  \eqn{N} is the number of sites,
  \eqn{S} is the number of shared occurrences of the indicator (\eqn{I})
  and the target (\eqn{T}) species. \eqn{O_I} and \eqn{O_T} are number
  of occurrences of the indicator and target species. The \code{type}
  argument in the function call enables to choose which statistic to
  return. \code{type = 0} returns \eqn{IP_I}, \code{type = 1} returns 
  \eqn{a}, \code{type = 2} returns \eqn{b}.
  Total indicator power (TIP) of an indicator species is the column mean
  (without its own value, see examples).  
  Halme et al. (2009) explain how to calculate confidence
  intervals for these statistics, see Examples.
}
\value{
  A matrix with indicator species as rows and
  target species as columns (this is indicated by the first letters of the
  row/column names).
}
\references{
Halme, P., \enc{Mönkkönen}{Monkkonen}, M., Kotiaho, J. S, 
\enc{Ylisirniö}{Ylisirnio}, A-L. 2009. Quantifying the indicator power
of an indicator species. \emph{Conservation Biology} 23: 1008--1016.
}
\author{Peter Solymos}

\seealso{ \code{\link[labdsv]{indval}} (package \pkg{labdsv}) for the
indicator species analysis of \enc{Dufrêne}{Dufrene} & Legendre. 
Function \code{\link{beals}} estimates individual cell probabilities
of species occurrences. }

\examples{
data(dune)
## IP values
ip <- indpower(dune)
## and TIP values
diag(ip) <- NA
(TIP <- rowMeans(ip, na.rm=TRUE))

## p value calculation for a species
## from Halme et al. 2009
## i is ID for the species
i <- 1
fun <- function(x, i) indpower(x)[i,-i]
## 'c0' randomizes species occurrences
os <- oecosimu(dune, fun, "c0", i=i, nsimul=99)
## get z values from oecosimu output
z <- os$oecosimu$z
## p-value
(p <- sum(z) / sqrt(length(z)))
## 'heterogeneity' measure
(chi2 <- sum((z - mean(z))^2))
pchisq(chi2, df=length(z)-1)
## Halme et al.'s suggested output
out <- c(TIP=TIP[i], 
    significance=p,
    heterogeneity=chi2,
    minIP=min(fun(dune, i=i)),
    varIP=sd(fun(dune, i=i)^2))
out
}
\keyword{ multivariate }

\encoding{UTF-8}
\name{BCI}
\alias{BCI}
\alias{BCI.env}
\docType{data}
\title{Barro Colorado Island Tree Counts}
\description{
  Tree counts in 1-hectare plots in the Barro Colorado Island and
  associated site information.
}
\usage{
data(BCI)
data(BCI.env)
}
\format{
  
  A data frame with 50 plots (rows) of 1 hectare with counts of trees on
  each plot with total of 225 species (columns). Full Latin names are
  used for tree species. The names were updated against
  \url{http://www.theplantlist.org} and Kress et al. (2009) which allows
  matching 207 of species against \doi{10.5061/dryad.63q27} (Zanne et
  al., 2014). The original species names are available as attribute
  \code{original.names} of \code{BCI}. See Examples for changed names.

  For \code{BCI.env}, a data frame with 50 plots (rows) and nine site
  variables derived from Pyke et al. (2001) and Harms et al. (2001):

  \describe{
    \item{\code{UTM.EW}: }{UTM coordinates (zone 17N) East-West.}
    \item{\code{UTM.NS}: }{UTM coordinates (zone 17N) North-South.}
    \item{\code{Precipitation}: }{Precipitation in mm per year.}
    \item{\code{Elevation}: }{Elevation in m above sea level.}
    \item{\code{Age.cat}: }{Forest age category.}
    \item{\code{Geology}: }{The Underlying geological formation.}

    \item{\code{Habitat}: }{Dominant habitat type based on the map of
    habitat types in 25 grid cells in each plot (Harms et al. 2001,
    excluding streamside habitat). The habitat types are \code{Young}
    forests (\emph{ca.} 100 years), old forests on > 7 degree slopes
    (\code{OldSlope}), old forests under 152 m elevation
    (\code{OldLow}) and at higher elevation (\code{OldHigh}) and
    \code{Swamp} forests.}

    \item{\code{River}: }{\code{"Yes"} if there is streamside habitat
    in the plot.}
    \item{\code{EnvHet}: }{Environmental Heterogeneity assessed as the
    Simpson diversity of frequencies of \code{Habitat} types in 25
    grid cells in the plot.}

}

 }
\details{

  Data give the numbers of trees at least 10 cm in diameter at breast
  height (DBH) in each one hectare quadrat in the 1982 BCI
  plot. Within each plot, all individuals were tallied and are
  recorded in this table. The full survey included smaller trees with
  DBH 1 cm or larger, but the \code{BCI} dataset is a subset of larger
  trees as compiled by Condit et al. (2002). The full data with
  thinner trees has densities above 4000 stems per hectare, or about
  ten times more stems than these data. The dataset \code{BCI} was
  provided (in 2003) to illustrate analysis methods in
  \pkg{vegan}. For scientific research on ecological issues we
  strongly recommend to access complete and more modern data (Condit
  et al. 2019) with updated taxonomy (Condit et al. 2020).

  The data frame contains only the Barro Colorado Island subset of the
  full data table of Condit et al. (2002).

  The quadrats are located in a regular grid. See \code{BCI.env} for the
  coordinates.

  A full description of the site information in \code{BCI.env} is
  given in Pyke et al. (2001) and Harms et al. (2001). \emph{N.B.}
  Pyke et al. (2001) and Harms et al. (2001) give conflicting
  information about forest age categories and elevation.

}
\source{

  \url{https://www.science.org/doi/10.1126/science.1066854} for
  community data and References for environmental data. For updated
  complete data (incl. thinner trees down to 1 cm), see Condit et
  al. (2019).

}

\seealso{

  Extra-CRAN package \pkg{natto}
  (\url{https://github.com/jarioksa/natto}) has data set
  \code{BCI.env2} with original grid data of Harms et al. (2001)
  habitat classification, and data set \code{BCI.taxon} of APG III
  classification of tree species.

}

\references{
  
  Condit, R, Pitman, N, Leigh, E.G., Chave, J., Terborgh, J., Foster,
  R.B., \enc{Nuñez}{Nunez}, P., Aguilar, S., Valencia, R., Villa, G.,
  Muller-Landau, H.C., Losos, E. & Hubbell, S.P. (2002).
  Beta-diversity in tropical forest trees. \emph{Science} 295,
  666--669.

  Condit R., Pérez, R., Aguilar, S., Lao, S., Foster, R. & Hubbell,
  S. (2019). Complete data from the Barro Colorado 50-ha plot: 423617
  trees, 35 years [Dataset].  \emph{Dryad}.
  \doi{https://doi.org/10.15146/5xcp-0d46}

  Condit, R., Aguilar, S., Lao, S., Foster, R., Hubbell,
  S. (2020). BCI 50-ha Plot Taxonomy [Dataset].  \emph{Dryad}.
  \doi{https://doi.org/10.15146/R3FH61}

  Harms K.E., Condit R., Hubbell S.P. & Foster R.B. (2001) Habitat
  associations of trees and shrubs in a 50-ha neotropical forest
  plot. \emph{J. Ecol.} 89, 947--959.

  Kress W.J., Erickson D.L, Jones F.A., Swenson N.G, Perez R., Sanjur
  O. & Bermingham E. (2009) Plant DNA barcodes and a community
  phylogeny of a tropical forest dynamics plot in Panama. \emph{PNAS}
  106, 18621--18626.

  Pyke, C. R., Condit, R., Aguilar, S., & Lao, S. (2001). Floristic
  composition across a climatic gradient in a neotropical lowland
  forest. \emph{Journal of Vegetation Science} 12, 553--566.
  \doi{10.2307/3237007}

  Zanne A.E., Tank D.C., Cornwell, W.K., Eastman J.M., Smith, S.A.,
  FitzJohn, R.G., McGlinn, D.J., O’Meara, B.C., Moles, A.T., Reich,
  P.B., Royer, D.L., Soltis, D.E., Stevens, P.F., Westoby, M., Wright,
  I.J., Aarssen, L., Bertin, R.I., Calaminus, A., Govaerts, R.,
  Hemmings, F., Leishman, M.R., Oleksyn, J., Soltis, P.S., Swenson,
  N.G., Warman, L. & Beaulieu, J.M. (2014) Three keys to the radiation
  of angiosperms into freezing environments. \emph{Nature} 506,
  89--92.  \doi{10.1038/nature12872} (published online Dec 22, 2013).

}

\examples{
data(BCI, BCI.env)
head(BCI.env)
## see changed species names
oldnames <- attr(BCI, "original.names")
taxa <- cbind("Old Names" = oldnames, "Current Names" = names(BCI))
noquote(taxa[taxa[,1] != taxa[,2], ])
}
\keyword{datasets}

\name{permutest.betadisper}
\alias{permutest.betadisper}

\title{Permutation test of multivariate homogeneity of groups dispersions
  (variances)}
\description{
  Implements a permutation-based test of multivariate homogeneity of
  group dispersions (variances) for the results of a call to
  \code{\link{betadisper}}.
}
\usage{
\method{permutest}{betadisper}(x, pairwise = FALSE,
          permutations = 999,
          parallel = getOption("mc.cores"),
          \dots)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{an object of class \code{"betadisper"}, the result of a
    call to \code{betadisper}.}
  \item{pairwise}{logical; perform pairwise comparisons of group means?}
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}
  \item{parallel}{Number of parallel processes or a predefined socket
    cluster.  With \code{parallel = 1} uses ordinary, non-parallel
    processing.}
  \item{\dots}{Arguments passed to other methods.}
}
\details{
  To test if one or more groups is more variable than the others, ANOVA
  of the distances to group centroids can be performed and parametric
  theory used to interpret the significance of F. An alternative is to
  use a permutation test. \code{permutest.betadisper} permutes model
  residuals to generate a permutation distribution of F under the Null
  hypothesis of no difference in dispersion between groups.

  Pairwise comparisons of group mean dispersions can be performed by
  setting argument \code{pairwise} to \code{TRUE}. A classical t test
  is performed on the pairwise group dispersions. This is combined with a
  permutation test based on the t statistic calculated on pairwise group
  dispersions. An alternative to the classical comparison of group
  dispersions, is to calculate Tukey's Honest Significant Differences
  between groups, via \code{\link{TukeyHSD.betadisper}}.
}
\value{
  \code{permutest.betadisper} returns a list of class
  \code{"permutest.betadisper"} with the following components:

  \item{tab}{the ANOVA table which is an object inheriting from class
    \code{"data.frame"}.}
  \item{pairwise}{a list with components \code{observed} and
    \code{permuted} containing the observed and permuted p-values for
    pairwise comparisons of group mean distances (dispersions or variances).}
  \item{groups}{character; the levels of the grouping factor.}
  \item{control}{a list, the result of a call to
    \code{\link{how}}.}
}
\references{
  Anderson, M.J. (2006) Distance-based tests for homogeneity of
  multivariate dispersions. \emph{Biometrics} \strong{62(1)}, 245--253.

  Anderson, M.J., Ellingsen, K.E. & McArdle, B.H. (2006) Multivariate
  dispersion as a measure of beta diversity. \emph{Ecology Letters}
  \strong{9(6)}, 683--693.
}
\author{Gavin L. Simpson}
\seealso{For the main fitting function see \code{\link{betadisper}}. For
  an alternative approach to determining which groups are more variable,
  see \code{\link{TukeyHSD.betadisper}}.}
\examples{
data(varespec)

## Bray-Curtis distances between samples
dis <- vegdist(varespec)

## First 16 sites grazed, remaining 8 sites ungrazed
groups <- factor(c(rep(1,16), rep(2,8)), labels = c("grazed","ungrazed"))

## Calculate multivariate dispersions
mod <- betadisper(dis, groups)
mod

## Perform test
anova(mod)

## Permutation test for F
pmod <- permutest(mod, permutations = 99, pairwise = TRUE)

## Tukey's Honest Significant Differences
(mod.HSD <- TukeyHSD(mod))
plot(mod.HSD)

## Has permustats() method
pstat <- permustats(pmod)
densityplot(pstat, scales = list(x = list(relation = "free")))
qqmath(pstat, scales = list(relation = "free"))
}
\keyword{methods}
\keyword{multivariate}

\name{vegandocs}
\alias{vegandocs}

\title{Display vegan Documentation}

\description{Deprecated function to display news and vignettes in
  \pkg{vegan}. You should use \code{\link[utils]{news}} or
  \code{\link[utils]{browseVignettes}} instead. }

\usage{
vegandocs(doc = c("NEWS", "ONEWS", "FAQ-vegan", "intro-vegan",
    "diversity-vegan", "decision-vegan", "partitioning", "permutations"))
}

\arguments{
  \item{doc}{The name of the document (partial match, case sensitive). }
}

\author{Jari Oksanen }

\note{The function is deprecated.}

\seealso{\code{\link[utils]{browseVignettes}},
\code{\link[utils]{vignette}}, \code{\link[utils]{news}}. }

\keyword{ utilities }
\keyword{ documentation }

\name{screeplot.cca}
\alias{screeplot.cca}
\alias{screeplot.princomp}
\alias{screeplot.prcomp}
\alias{screeplot.decorana}
\alias{bstick}
\alias{bstick.default}
\alias{bstick.cca}
\alias{bstick.prcomp}
\alias{bstick.princomp}
\alias{bstick.decorana}
\title{Screeplots for Ordination Results and Broken Stick Distributions}
\description{
  Screeplot methods for plotting variances of ordination axes/components
  and overlaying broken stick distributions. Also, provides alternative
  screeplot methods for \code{\link{princomp}} and \code{\link{prcomp}}.
}
\usage{
\method{screeplot}{cca}(x, bstick = FALSE, type = c("barplot", "lines"),
         npcs = min(10, if (is.null(x$CCA) || x$CCA$rank == 0) x$CA$rank else x$CCA$rank),
         ptype = "o", bst.col = "red", bst.lty = "solid",
         xlab = "Component", ylab = "Inertia",
         main = deparse(substitute(x)), legend = bstick,
         \dots)

\method{screeplot}{decorana}(x, bstick = FALSE, type = c("barplot", "lines"),
         npcs = 4,
         ptype = "o", bst.col = "red", bst.lty = "solid",
         xlab = "Component", ylab = "Inertia",
         main = deparse(substitute(x)),
         \dots)

\method{screeplot}{prcomp}(x, bstick = FALSE, type = c("barplot", "lines"),
         npcs = min(10, length(x$sdev)),
         ptype = "o", bst.col = "red", bst.lty = "solid",
         xlab = "Component", ylab = "Inertia",
         main = deparse(substitute(x)), legend = bstick,
         \dots)

\method{screeplot}{princomp}(x, bstick = FALSE, type = c("barplot", "lines"),
         npcs = min(10, length(x$sdev)),
         ptype = "o", bst.col = "red", bst.lty = "solid",
         xlab = "Component", ylab = "Inertia",
         main = deparse(substitute(x)), legend = bstick,
         \dots)

bstick(n, \dots)

\method{bstick}{default}(n, tot.var = 1, \dots)

\method{bstick}{cca}(n, \dots)

\method{bstick}{prcomp}(n, \dots)

\method{bstick}{princomp}(n, \dots)

\method{bstick}{decorana}(n, \dots)
}
\arguments{
  \item{x}{an object from which the component variances can be determined.}
  \item{bstick}{logical; should the broken stick distribution be drawn?}
  \item{npcs}{the number of components to be plotted.}
  \item{type}{the type of plot.}
  \item{ptype}{if \code{type == "lines"} or \code{bstick = TRUE}, a
    character indicating the type of plotting used for the lines;
    actually any of the \code{type}s as in \code{\link{plot.default}}.} 
  \item{bst.col, bst.lty}{the colour and line type used to draw the
    broken stick distribution.}
  \item{xlab, ylab, main}{graphics parameters.}
  \item{legend}{logical; draw a legend?}
  \item{n}{an object from which the variances can be extracted or the
    number of variances (components) in the case of
    \code{bstick.default}.}
  \item{tot.var}{the total variance to be split.}
  \item{\dots}{arguments passed to other methods.}
}
\details{
  The functions provide screeplots for most ordination methods in
  \pkg{vegan} and enhanced versions with broken stick for
  \code{\link{prcomp}} and \code{\link{princomp}}. 

  Function \code{bstick} gives the brokenstick values which are ordered
  random proportions, defined as  \eqn{p_i = (tot/n) \sum_{x=i}^n 
    (1/x)}{p[i] = tot/n sum(from x=i to n) 1/x} (Legendre & Legendre 2012), where
  \eqn{tot} is the total  and \eqn{n} is the number of brokenstick
  components (cf. \code{\link{radfit}}).  Broken stick has
  been recommended as a stopping rule in principal component analysis
  (Jackson 1993): principal components should be retained as long as
  observed eigenvalues are higher than corresponding random broken stick
  components.

  The \code{bstick} function is generic. The default needs the number of
  components and the total, and specific methods extract this
  information from ordination results.  There also is a \code{bstick}
  method for \code{\link{cca}}.  However, the broken stick model is not
  strictly valid for correspondence analysis (CA), because eigenvalues
  of CA are defined to be \eqn{\leq 1}{<=1}, whereas brokenstick
  components have no such restrictions. The brokenstick components are not
  available for \code{\link{decorana}} where the sum of eigenvalues
  (total inertia) is unknown, and the eigenvalues of single axes are not
  additive in detrended analysis.
  }
\value{
  Function \code{screeplot} draws a plot on the currently active device, 
  and returns invisibly the \code{\link{xy.coords}} of the points or
  bars for the eigenvalues.

  Function \code{bstick} returns a numeric vector of broken stick
  components. 
}
\references{
  Jackson, D. A. (1993). Stopping rules in principal components
  analysis: a comparison of heuristical and statistical
  approaches. \emph{Ecology} 74, 2204--2214.

  Legendre, P. and Legendre, L. (2012) \emph{Numerical Ecology}. 3rd English
  ed. Elsevier.
  }

\author{Gavin L. Simpson}
\seealso{
  \code{\link{cca}}, \code{\link{decorana}}, \code{\link{princomp}} and
  \code{\link{prcomp}} for the ordination functions, and
  \code{\link{screeplot}} for the stock version.  }
\examples{
data(varespec)
vare.pca <- rda(varespec, scale = TRUE)
bstick(vare.pca)
screeplot(vare.pca, bstick = TRUE, type = "lines")
}
\keyword{multivariate}

\encoding{UTF-8}
\name{nestedtemp}
\alias{nestedtemp}
\alias{nestedchecker}
\alias{nestedn0}
\alias{nesteddisc}
\alias{nestednodf}
\alias{nestedbetasor}
\alias{nestedbetajac}
\alias{plot.nestedtemp}
\alias{plot.nestednodf}

\title{ Nestedness Indices for Communities of Islands or Patches }

\description{
  Patches or local communities are regarded as nested if they all could
  be subsets of the same community. In general, species poor communities
  should be subsets of species rich communities, and rare species should
  only occur in species rich communities.
}

\usage{
nestedchecker(comm)
nestedn0(comm)
nesteddisc(comm, niter = 200)
nestedtemp(comm, ...)
nestednodf(comm, order = TRUE, weighted = FALSE, wbinary = FALSE)
nestedbetasor(comm)
nestedbetajac(comm)
\method{plot}{nestedtemp}(x, kind = c("temperature", "incidence"),
    col=rev(heat.colors(100)),  names = FALSE, ...)
\method{plot}{nestednodf}(x, col = "red", names = FALSE, ...)
}

\arguments{
  \item{comm}{Community data.}
  \item{niter}{Number of iterations to reorder tied columns.}
  \item{x}{Result object for a \code{plot}.}
  \item{col}{Colour scheme for matrix temperatures.}
  \item{kind}{The kind of plot produced.}
  \item{names}{Label columns and rows in the plot using names in \code{comm}.
    If it is a logical vector of length 2, row and column labels are
    returned accordingly.}
  \item{order}{Order rows and columns by frequencies.}
  \item{weighted}{Use species abundances as weights of interactions.}
  \item{wbinary}{Modify original method so that binary data give the same
    result in weighted and and unweighted analysis. }
  \item{\dots}{Other arguments to functions.}
}

\details{
  The nestedness functions evaluate alternative indices of nestedness.
  The functions are intended to be used together with Null model
  communities and used as an argument in \code{\link{oecosimu}} to analyse
  the non-randomness of results.

  Function \code{nestedchecker} gives the number of checkerboard units,
  or 2x2 submatrices where both species occur once but on different
  sites (Stone & Roberts 1990).

  Function \code{nestedn0} implements
  nestedness measure N0 which is the number of absences from the sites
  which are richer than the most pauperate site species occurs
  (Patterson & Atmar 1986).

  Function \code{nesteddisc} implements discrepancy index which is the
  number of ones that should be shifted to fill a row with ones in a
  table arranged by species frequencies (Brualdi & Sanderson
  1999). The original definition arranges species (columns) by their
  frequencies, but did not have any method of handling tied
  frequencies.  The \code{nesteddisc} function tries to order tied
  columns to minimize the discrepancy statistic but this is rather
  slow, and with a large number of tied columns there is no guarantee
  that the best ordering was found (argument \code{niter} gives the
  maximum number of tried orders). In that case a warning of tied
  columns will be issued.

  Function \code{nestedtemp} finds the matrix temperature which is
  defined as the sum of \dQuote{surprises} in arranged matrix.  In
  arranged unsurprising matrix all species within proportion given by
  matrix fill are in the upper left corner of the matrix, and the
  surprise of the absence or presences is the diagonal distance from the
  fill line (Atmar & Patterson 1993). Function tries to pack species and
  sites to a low temperature (\enc{Rodríguez-Gironés}{Rodriguez-Girones}
  & Santamaria 2006), but this is an iterative procedure, and the
  temperatures usually vary among runs.  Function \code{nestedtemp} also
  has a \code{plot} method which can display either incidences or
  temperatures of the surprises. Matrix temperature was rather vaguely
  described (Atmar & Patterson 1993), but
  \enc{Rodríguez-Gironés}{Rodriguez-Girones} & Santamaria (2006) are
  more explicit and their description is used here. However, the results
  probably differ from other implementations, and users should be
  cautious in interpreting the results. The details of calculations are
  explained in the \code{\link{vignette}} \emph{Design decisions and
  implementation} that you can read using functions
  \code{\link{browseVignettes}}. Function
  \code{\link[bipartite]{nestedness}} in the \pkg{bipartite} package is
  a direct port of the BINMATNEST programme of
  \enc{Rodríguez-Gironés}{Rodriguez-Girones} & Santamaria (2006).

  Function \code{nestednodf} implements a nestedness metric based on
  overlap and decreasing fill (Almeida-Neto et al., 2008). Two basic
  properties are required for a matrix to have the maximum degree of
  nestedness according to this metric: (1) complete overlap of 1's
  from right to left columns and from down to up rows, and (2)
  decreasing marginal totals between all pairs of columns and all
  pairs of rows. The nestedness statistic is evaluated separately for
  columns (\code{N columns}) for rows (\code{N rows}) and combined for
  the whole matrix (\code{NODF}).  If you set \code{order = FALSE},
  the statistic is evaluated with the current matrix ordering allowing
  tests of other meaningful hypothesis of matrix structure than
  default ordering by row and column totals (breaking ties by total
  abundances when \code{weighted = TRUE}) (see Almeida-Neto et
  al. 2008). With \code{weighted = TRUE}, the function finds the
  weighted version of the index (Almeida-Neto & Ulrich,
  2011). However, this requires quantitative null models for adequate
  testing. Almeida-Neto & Ulrich (2011) say that you have positive
  nestedness if values in the first row/column are higher than in the
  second.  With this condition, weighted analysis of binary data will
  always give zero nestedness. With argument \code{wbinary = TRUE},
  equality of rows/columns also indicates nestedness, and binary data
  will give identical results in weighted and unweighted analysis.
  However, this can also influence the results of weighted analysis so
  that the results may differ from Almeida-Neto & Ulrich (2011).

  Functions \code{nestedbetasor} and \code{nestedbetajac} find
  multiple-site dissimilarities and decompose these into components of
  turnover and nestedness following Baselga (2012); the pairwise
  dissimilarities can be found with \code{\link{designdist}}. This can
  be seen as a decomposition of beta diversity (see
  \code{\link{betadiver}}).  Function \code{nestedbetasor} uses
  \enc{Sørensen}{Sorensen} dissimilarity and the turnover component is
  Simpson dissimilarity (Baselga 2012), and \code{nestedbetajac} uses
  analogous methods with the Jaccard index. The functions return a
  vector of three items: turnover, nestedness and their sum which is
  the multiple \enc{Sørensen}{Sorensen} or Jaccard dissimilarity. The
  last one is the total beta diversity (Baselga 2012). The functions
  will treat data as presence/absence (binary) and they can be used
  with binary \code{\link{nullmodel}}. The overall dissimilarity is
  constant in all \code{\link{nullmodel}}s that fix species (column)
  frequencies (\code{"c0"}), and all components are constant if row
  columns are also fixed (e.g., model \code{"quasiswap"}), and the
  functions are not meaningful with these null models.

}

\value{
  The result returned by a nestedness function contains an item called
  \code{statistic}, but the other components differ among functions. The
  functions are constructed so that they can be handled by
  \code{\link{oecosimu}}.
}

\references{

  Almeida-Neto, M., \enc{Guimarães}{Guimaraes}, P.,
  \enc{Guimarães}{Guimaraes}, P.R., Loyola, R.D. & Ulrich, W. (2008). A
  consistent metric for nestedness analysis in ecological systems:
  reconciling concept and measurement. \emph{Oikos} 117, 1227--1239.

  Almeida-Neto, M. & Ulrich, W. (2011). A straightforward
  computational approach for measuring nestedness using quantitative
  matrices. \emph{Env. Mod. Software} 26, 173--178.

  Atmar, W. & Patterson, B.D. (1993). The measurement of order and
  disorder in the distribution of species in fragmented
  habitat. \emph{Oecologia} 96, 373--382.

  Baselga, A. (2012). The relationship between species replacement,
  dissimilarity derived from nestedness, and nestedness. \emph{Global
  Ecol. Biogeogr.} 21, 1223--1232.

  Brualdi, R.A. & Sanderson, J.G. (1999). Nested species subsets, gaps,
  and discrepancy. \emph{Oecologia} 119, 256--264.

  Patterson, B.D. & Atmar, W. (1986). Nested subsets and the structure
  of insular mammalian faunas and archipelagos. \emph{Biol. J. Linnean
    Soc.} 28, 65--82.

  \enc{Rodríguez-Gironés}{Rodriguez-Girones}, M.A.  & Santamaria, L.
  (2006). A new algorithm to calculate the nestedness temperature of
  presence-absence matrices. \emph{J. Biogeogr.} 33, 924--935.

  Stone, L. & Roberts, A. (1990). The checkerboard score and species
  distributions. \emph{Oecologia} 85, 74--79.

  Wright, D.H., Patterson, B.D., Mikkelson, G.M., Cutler, A. & Atmar,
  W. (1998). A comparative analysis of nested subset patterns of species
  composition. \emph{Oecologia} 113, 1--20.
  }
\author{ Jari Oksanen and Gustavo Carvalho (\code{nestednodf}). }

\seealso{
  In general, the functions should be used with \code{\link{oecosimu}}
  which generates Null model communities to assess the non-randomness of
  nestedness patterns.
 }
\examples{
data(sipoo)
## Matrix temperature
out <- nestedtemp(sipoo)
out
plot(out)
plot(out, kind="incid")
## Use oecosimu to assess the non-randomness of checker board units
nestedchecker(sipoo)
oecosimu(sipoo, nestedchecker, "quasiswap")
## Another Null model and standardized checkerboard score
oecosimu(sipoo, nestedchecker, "r00", statistic = "C.score")
}
\keyword{ univar }

\name{procrustes}
\alias{procrustes}
\alias{summary.procrustes}
\alias{plot.procrustes}
\alias{points.procrustes}
\alias{text.procrustes}
\alias{lines.procrustes}
\alias{residuals.procrustes}
\alias{fitted.procrustes}
\alias{predict.procrustes}
\alias{protest}

\title{Procrustes Rotation of Two Configurations and PROTEST }
\description{
 Function \code{procrustes} rotates a configuration to maximum similarity
  with another configuration. Function \code{protest} tests the
  non-randomness (significance) between two configurations.
}
\usage{
procrustes(X, Y, scale = TRUE, symmetric = FALSE, scores = "sites", ...)
\method{summary}{procrustes}(object, digits = getOption("digits"), ...)
\method{plot}{procrustes}(x, kind=1, choices=c(1,2), to.target = TRUE, 
    type = "p", xlab, ylab, main, ar.col = "blue", len=0.05, 
    cex = 0.7, ...)
\method{points}{procrustes}(x, display = c("target", "rotated"),
    choices = c(1,2), truemean = FALSE, ...)
\method{text}{procrustes}(x, display = c("target", "rotated"),
    choices = c(1,2), labels, truemean = FALSE, ...)
\method{lines}{procrustes}(x, type = c("segments", "arrows"),
    choices = c(1, 2), truemean = FALSE, ...)
\method{residuals}{procrustes}(object, ...)
\method{fitted}{procrustes}(object, truemean = TRUE, ...)
\method{predict}{procrustes}(object, newdata, truemean = TRUE, ...)
protest(X, Y, scores = "sites", permutations = how(nperm = 999), ...)
}

\arguments{
  \item{X}{Target matrix}
  \item{Y}{Matrix to be rotated.}
  \item{scale}{Allow scaling of axes of \code{Y}.}
  \item{symmetric}{Use symmetric Procrustes statistic (the rotation will
    still be non-symmetric).}
  \item{scores}{Kind of scores used. This is the \code{display} argument
  used with the corresponding \code{scores} function: see
  \code{\link{scores}}, \code{\link{scores.cca}} and
  \code{\link{scores.cca}} for alternatives.}
  \item{x, object}{An object of class \code{procrustes}.}
  \item{digits}{Number of digits in the output.}
  \item{kind}{For \code{plot} function, the kind of plot produced:
    \code{kind = 1} plots shifts in two configurations, \code{kind = 0}
    draws a corresponding empty plot, and \code{kind = 2}
    plots an impulse diagram of residuals.}
  \item{choices}{Axes (dimensions) plotted.}
  \item{xlab, ylab}{Axis labels, if defaults unacceptable.}
  \item{main}{Plot title, if default unacceptable.}
  \item{display}{Show only the \code{"target"} or \code{"rotated"}
    matrix as points.}
  \item{to.target}{Draw arrows to point to target.}
  \item{type}{The type of plot drawn. In \code{plot}, the \code{type}
    can be \code{"points"} or \code{"text"} to select the marker for
    the tail of the arrow, or \code{"none"} for drawing an empty
    plot. In \code{lines} the \code{type} selects either
    \code{\link{arrows}} or line \code{\link{segments}} to connect
    target and rotated configuration.}
  \item{truemean}{Use the original range of target matrix instead of
    centring the fitted values. Function \code{plot.procrustes} needs
    \code{truemean = FALSE}, and adding graphical items to the plots
    from the original results may need \code{truemean = TRUE}.}
  \item{newdata}{Matrix of coordinates to be rotated and translated to
     the target.}
  \item{permutations}{a list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}, or the
    number of permutations required, or a permutation matrix where each
    row gives the permuted indices.}
  \item{ar.col}{Arrow colour.}
  \item{len}{Width of the arrow head.}
  \item{labels}{Character vector of text labels. Rownames of the result 
    object are used as default.}
  \item{cex}{Character expansion for points or text.}
  \item{...}{Other parameters passed to functions. In \code{procrustes}
    and \code{protest} parameters are passed to \code{\link{scores}}, in
    graphical functions to underlying graphical functions.}
}
\details{
  Procrustes rotation rotates a matrix to maximum similarity with a
  target matrix minimizing sum of squared differences.  Procrustes
  rotation is typically used in comparison of ordination results.  It is
  particularly useful in comparing alternative solutions in
  multidimensional scaling.  If \code{scale=FALSE}, the function only
  rotates matrix \code{Y}. If \code{scale=TRUE}, it scales linearly
  configuration \code{Y} for maximum similarity.  Since \code{Y} is scaled
  to fit \code{X}, the scaling is non-symmetric. However, with
  \code{symmetric=TRUE}, the configurations are scaled to equal
  dispersions and  a symmetric version of the Procrustes statistic
  is computed.

  Instead of matrix, \code{X} and \code{Y} can be results from an
  ordination from which \code{\link{scores}} can extract results.
  Function \code{procrustes} passes extra arguments to
  \code{\link{scores}}, \code{\link{scores.cca}} etc. so that you can
  specify arguments such as \code{scaling}. 

  Function \code{plot} plots a \code{procrustes} object and returns
  invisibly an \code{ordiplot} object so that function
  \code{\link{identify.ordiplot}} can be used for identifying
  points. The items in the \code{ordiplot} object are called
  \code{heads} and \code{points} with \code{kind=1} (ordination
  diagram) and \code{sites} with \code{kind=2} (residuals).  In
  ordination diagrams, the arrow heads point to the target
  configuration if \code{to.target = TRUE}, and to rotated
  configuration if \code{to.target = FALSE}.  Target and original
  rotated axes are shown as cross hairs in two-dimensional Procrustes
  analysis, and with a higher number of dimensions, the rotated axes
  are projected onto plot with their scaled and centred
  range. Function \code{plot} passes parameters to underlying plotting
  functions.  For full control of plots, you can draw the axes using
  \code{plot} with \code{kind = 0}, and then add items with
  \code{points} or \code{lines}.  These functions pass all parameters
  to the underlying functions so that you can select the plotting
  characters, their size, colours etc., or you can select the width,
  colour and type of line \code{\link{segments}} or arrows, or you can
  select the orientation and head width of \code{\link{arrows}}.

  Function \code{residuals} returns the pointwise
  residuals, and \code{fitted} the fitted values, either centred to zero
  mean (if \code{truemean=FALSE}) or with the original scale (these
  hardly make sense if \code{symmetric = TRUE}). In
  addition, there are \code{summary} and \code{print} methods.

  If matrix \code{X} has a lower number of columns than matrix
  \code{Y}, then matrix \code{X} will be filled with zero columns to
  match dimensions. This means that the function can be used to rotate
  an ordination configuration to an environmental variable (most
  practically extracting the result with the \code{fitted}
  function). Function \code{predict} can be used to add new rotated
  coordinates to the target. The \code{predict} function will always
  translate coordinates to the original non-centred matrix. The
  function cannot be used with \code{newdata} for \code{symmetric}
  analysis.

  Function \code{protest} performs symmetric Procrustes analysis
  repeatedly to estimate the significance of the Procrustes
  statistic. Function \code{protest} uses a correlation-like statistic
  derived from the symmetric Procrustes sum of squares \eqn{ss} as
  \eqn{r =\sqrt{1-ss}}{r = sqrt(1-ss)}, and also prints the sum of
  squares of the symmetric analysis, sometimes called
  \eqn{m_{12}^2}{squared m12}. Function \code{protest} has own
  \code{print} method, but otherwise uses \code{procrustes}
  methods. Thus \code{plot} with a \code{protest} object yields a
  Procrustean superimposition plot.  }

\value{
  Function \code{procrustes} returns an object of class
  \code{procrustes} with items. Function \code{protest} inherits from
  \code{procrustes}, but amends that with some new items:
  \item{Yrot }{Rotated matrix \code{Y}.}
  \item{X}{Target matrix.}
  \item{ss }{Sum of squared differences between \code{X} and \code{Yrot}.}
  \item{rotation}{Orthogonal rotation matrix.}
  \item{translation}{Translation of the origin.}
  \item{scale}{Scaling factor.}
  \item{xmean}{The centroid of the target.}
  \item{symmetric}{Type of \code{ss} statistic.}
  \item{call}{Function call.}
  \item{t0}{This and the following items are only in class
    \code{protest}:  Procrustes correlation from non-permuted solution.}
  \item{t}{Procrustes correlations from permutations. The distribution
    of these correlations can be inspected with \code{\link{permustats}}
    function.}
  \item{signif}{Significance of \code{t}}
  \item{permutations}{Number of permutations.}
  \item{control}{A list of control values for the permutations
    as returned by the function \code{\link[permute]{how}}.}
  \item{control}{the list passed to argument \code{control} describing
    the permutation design.}
}
\references{
  Mardia, K.V., Kent, J.T. and Bibby,
  J.M. (1979). \emph{Multivariate Analysis}. Academic Press.

  Peres-Neto, P.R. and Jackson, D.A. (2001). How well do multivariate
  data sets match? The advantages of a Procrustean superimposition
  approach over the Mantel test. \emph{Oecologia} 129: 169-178.
  
}
\author{Jari Oksanen }

\note{The function \code{protest} follows Peres-Neto & Jackson (2001),
  but the implementation is still after Mardia \emph{et al.}
  (1979).}

\seealso{\code{\link{monoMDS}},  for obtaining
objects for \code{procrustes}, and \code{\link{mantel}} for an
alternative to \code{protest} without need of dimension reduction. See
\code{\link[permute]{how}} for details on specifying the type of
permutation required.} 

\examples{
data(varespec)
vare.dist <- vegdist(wisconsin(varespec))
mds.null <- monoMDS(vare.dist, y = cmdscale(vare.dist))
mds.alt <- monoMDS(vare.dist)
vare.proc <- procrustes(mds.alt, mds.null)
vare.proc
summary(vare.proc)
plot(vare.proc)
plot(vare.proc, kind=2)
residuals(vare.proc)
}
\keyword{multivariate }
\keyword{htest}

\encoding{UTF-8}
\name{goodness.cca}
\alias{goodness}
\alias{goodness.cca}
\alias{inertcomp}
\alias{spenvcor}
\alias{intersetcor}
\alias{vif.cca}
\alias{alias.cca}

\title{Diagnostic Tools for [Constrained] Ordination (CCA,
  RDA, DCA, CA, PCA) }
\description{
  Functions \code{goodness} and \code{inertcomp} can
  be used to assess the goodness of fit for individual sites or
  species. Function \code{vif.cca} and \code{alias.cca} can be used to
  analyse linear dependencies among constraints and conditions. In
  addition, there are some other diagnostic tools (see 'Details').

}
\usage{
\method{goodness}{cca}(object, choices, display = c("species", "sites"),
    model = c("CCA", "CA"), summarize = FALSE, addprevious = FALSE, ...)
inertcomp(object, display = c("species", "sites"),
    unity = FALSE, proportional = FALSE)
spenvcor(object)
intersetcor(object)
vif.cca(object)
\method{alias}{cca}(object, names.only = FALSE, ...)
}

\arguments{
  \item{object}{A result object from \code{\link{cca}},
    \code{\link{rda}}, \code{\link{dbrda}} or \code{\link{capscale}}. }

  \item{display}{Display \code{"species"} or \code{"sites"}. Species
    are not available in \code{\link{dbrda}} and \code{\link{capscale}}. }

  \item{choices}{Axes shown. Default is to show all axes of the
    \code{"model"}. }

  \item{model}{Show constrained (\code{"CCA"}) or unconstrained
    (\code{"CA"}) results. }

  \item{summarize}{Show only the accumulated total.}

  \item{addprevious}{Add the variation explained by previous components
     when \code{statistic="explained"}. For \code{model = "CCA"} add
     conditioned (partialled out) variation, and for \code{model = "CA"}
     add both conditioned and constrained variation. This will give
     cumulative explanation. The argument has no effect when
     \code{statistic="distance"}, but this will always show the residual
     distance after current axis and all previous components.
   }

  \item{unity}{Scale inertia components to unit sum (sum of all items is
    1).}
  \item{proportional}{Give the inertia components as proportional for
    the corresponding total of the item (sum of each row is 1). This
    option takes precedence over \code{unity}.}
  \item{names.only}{Return only names of aliased variable(s) instead of
    defining equations.}
  \item{\dots}{Other parameters to the functions.}
}
\details{

  Function \code{goodness} gives cumulative proportion of inertia
  accounted by species up to chosen axes. The proportions can be
  assessed either by species or by sites depending on the argument
  \code{display}, but species are not available in distance-based
  \code{\link{dbrda}}. The function is not implemented for
  \code{\link{capscale}}.

  Function \code{inertcomp} decomposes the inertia into partial,
  constrained and unconstrained components for each site or species.
  Legendre & De \enc{Cáceres}{Caceres} (2012) called these inertia
  components as local contributions to beta-diversity (LCBD) and
  species contributions to beta-diversity (SCBD), and they give these
  as relative contributions summing up to unity (argument
  \code{unity = TRUE}). For this interpretation, appropriate dissimilarity
  measures should be used in \code{\link{dbrda}} or appropriate
  standardization in \code{\link{rda}} (Legendre & De
  \enc{Cáceres}{Caceres} 2012). The function is not implemented for
  \code{\link{capscale}}.

  Function \code{spenvcor} finds the so-called \dQuote{species --
    environment correlation} or (weighted) correlation of
  weighted average scores and linear combination scores.  This is a bad
  measure of goodness of ordination, because it is sensitive to extreme
  scores (like correlations are), and very sensitive to overfitting or
  using too many constraints. Better models often have poorer
  correlations. Function \code{\link{ordispider}} can show the same
  graphically.

  Function \code{intersetcor} finds the so-called \dQuote{interset
    correlation} or (weighted) correlation of weighted averages scores
  and constraints.  The defined contrasts are used for factor
  variables.  This is a bad measure since it is a correlation.  Further,
  it focuses on correlations between single contrasts and single axes
  instead of looking at the multivariate relationship.  Fitted vectors
  (\code{\link{envfit}}) provide a better alternative.  Biplot scores
  (see \code{\link{scores.cca}}) are a multivariate alternative for
  (weighted) correlation between linear combination scores and
  constraints.

  Function \code{vif.cca} gives the variance inflation factors for each
  constraint or contrast in factor constraints. In partial ordination,
  conditioning variables are analysed together with constraints. Variance
  inflation is a diagnostic tool to identify useless constraints. A
  common rule is that values over 10 indicate redundant
  constraints. If later constraints are complete linear combinations of
  conditions or previous constraints, they will be completely removed
  from the estimation, and no biplot scores or centroids are calculated
  for these aliased constraints. A note will be printed with default
  output if there are aliased constraints. Function \code{alias} will
  give the linear coefficients defining the aliased constraints, or
  only their names with argument \code{names.only = TRUE}.
}

\value{
  The functions return matrices or vectors as is appropriate.
}
\references{
  Greenacre, M. J. (1984). Theory and applications of correspondence
  analysis. Academic Press, London.

  Gross, J. (2003). Variance inflation factors. \emph{R News} 3(1),
  13--15.

  Legendre, P. & De \enc{Cáceres}{Caceres}, M. (2012). Beta diversity as
  the variance of community data: dissimilarity coefficients and
  partitioning. \emph{Ecology Letters} 16, 951--963.
  \doi{10.1111/ele.12141}
}
\author{Jari Oksanen. The \code{vif.cca} relies heavily on the code by
  W. N. Venables. \code{alias.cca} is a simplified version of
  \code{\link{alias.lm}}.}

\note{

  It is a common practise to use \code{goodness} statistics to remove
  species from ordination plots, but this may not be a good idea, as the
  total inertia is not a meaningful concept in \code{cca}, in particular
  for rare species.

}

\seealso{\code{\link{cca}}, \code{\link{rda}}, \code{\link{dbrda}},
  \code{\link{capscale}}. }

\examples{
data(dune)
data(dune.env)
mod <- cca(dune ~ A1 + Management + Condition(Moisture), data=dune.env)
goodness(mod, addprevious = TRUE)
goodness(mod, addprevious = TRUE, summ = TRUE)
# Inertia components
inertcomp(mod, prop = TRUE)
inertcomp(mod)
# vif.cca
vif.cca(mod)
# Aliased constraints
mod <- cca(dune ~ ., dune.env)
mod
vif.cca(mod)
alias(mod)
with(dune.env, table(Management, Manure))
# The standard correlations (not recommended)
spenvcor(mod)
intersetcor(mod)
}
\keyword{ multivariate }

\name{ordiplot}
\alias{ordiplot}
\alias{identify.ordiplot}
\alias{scores.ordiplot}
\alias{points.ordiplot}
\alias{text.ordiplot}

\title{ Alternative plot and identify Functions for Ordination }

\description{
  Function \code{ordiplot} is an alternative plotting function which
  can be worked with any \pkg{vegan} ordination result and many
  non-\pkg{vegan} results. In addition, \code{plot} functions for
  \pkg{vegan} ordinations return invisibly an \code{"ordiplot"} result
  object, and this allows using \code{ordiplot} support functions with
  this result: \code{identify} can be used to add labels to selected
  site, species or constraint points, and \code{points} and
  \code{text} can add elements to the plot.
}

\usage{
ordiplot(ord, choices = c(1, 2), type="points", display, xlim, ylim,
     cex = 0.7, ...)
\method{identify}{ordiplot}(x, what, labels,  ...)
\method{points}{ordiplot}(x, what, select, arrows = FALSE, ...)
\method{text}{ordiplot}(x, what, labels, select, arrows = FALSE, ...)
}

\arguments{
  \item{ord}{A result from an ordination.}
  \item{choices}{Axes shown. }
  \item{type}{The type of graph which may be \code{"points"},
    \code{"text"} or \code{"none"} for any ordination method.}
  \item{display}{Display only "sites" or "species". The default for most
    methods is to display both, but for \code{\link{cca}},
    \code{\link{rda}} and \code{\link{capscale}} it is the same as in
    \code{\link{plot.cca}}.  }
  \item{xlim, ylim}{the x and y limits (min,max) of the plot.}
  \item{cex}{Character expansion factor for points and text.}
  \item{\dots}{Other graphical parameters. }
  \item{x}{A result object from \code{ordiplot}.}
  \item{what}{Items identified in the ordination plot. The types depend
    on the kind of plot used. Most methods know \code{sites} and
    \code{species}, functions \code{\link{cca}} and \code{\link{rda}}
    know in addition 
    \code{constraints} (for LC scores), \code{centroids} and
    \code{biplot}, and \code{\link{plot.procrustes}} ordination plot has
    \code{heads} and \code{points}.}
  \item{labels}{Optional text used for labels. Row names will be used if
    this is missing.}
  \item{arrows}{Draw arrows from the origin. This will always be
    \code{TRUE} for biplot scores and its value will be ignored. Setting
    this \code{TRUE} will draw arrows for any type of scores. This
    allows, e.g, using biplot arrows for species. The arrow head will be
  at the value of scores, and possible text is moved outwards.}
  \item{select}{Items to be displayed.  This can either be a logical
    vector which is \code{TRUE} for displayed items or a vector of indices
    of displayed items.}

}
\details{
  Function \code{ordiplot} draws an ordination diagram using black circles for
  sites and red crosses for species.  It returns invisibly an object of
  class \code{ordiplot} which can be used by \code{identify.ordiplot}
  to label selected sites or species, or constraints in
  \code{\link{cca}} and \code{\link{rda}}.

  The function can handle output from several alternative ordination
  methods. For \code{\link{cca}}, \code{\link{rda}} and
  \code{\link{decorana}} it uses their \code{plot} method with option
  \code{type = "points"}. In addition, the \code{plot} functions of
  these methods return invisibly an \code{ordiplot} object which can
  be used by \code{identify.ordiplot} to label points. For other
  ordinations it relies on \code{\link{scores}} to extract the scores.

  For full user control of plots, it is best to call \code{ordiplot}
  with \code{type = "none"} and save the result, and then add sites and
  species using \code{points.ordiplot} or \code{text.ordiplot} which
  both pass all their arguments to the corresponding default graphical
  functions.
}
\value{
  
  Function \code{ordiplot} returns invisibly an object of class
  \code{ordiplot} with used scores. In general, \pkg{vegan} \code{plot}
  functions for ordination results will also return an invisible
  \code{ordiplot} object. If the \code{plot(..., type = "n")} was used
  originally, the plot is empty, and items can be added with the
  invisible object. Functions \code{points} and \code{text} will return
  their input object without modification, which allows chaining these
  commands, for instance with \pkg{magrittr} pipes. Function
  \code{identify.ordiplot} uses this object to label the point.

}

\author{
  Jari Oksanen
}
\note{
  The purpose of these functions is to provide similar functionality as
  the \code{plot}, \code{plotid} and \code{specid} methods in library
  \code{labdsv}. The functions are somewhat limited in parametrization,
  but you can call directly the standard \code{\link{identify}} and
  \code{\link{plot}} functions for a better user control.
}

\seealso{ \code{\link{identify}} for basic operations, \code{\link{plot.cca}},
  \code{\link{plot.decorana}}, \code{\link{plot.procrustes}} which also
  produce objects for
  \code{identify.ordiplot} and \code{\link{scores}} for extracting
  scores from non-\code{vegan} ordinations.   
}

\examples{
# Draw a plot for a non-vegan ordination (cmdscale).
data(dune)
dune.dis <- vegdist(wisconsin(dune))
dune.mds <- cmdscale(dune.dis, eig = TRUE)
dune.mds$species <- wascores(dune.mds$points, dune, expand = TRUE)
pl <- ordiplot(dune.mds, type = "none")
points(pl, "sites", pch=21, col="red", bg="yellow")
text(pl, "species", col="blue", cex=0.9)
# Default plot of the previous using identify to label selected points
\dontrun{
pl <- ordiplot(dune.mds)
identify(pl, "spec")}

}
\keyword{ hplot }
\keyword{ iplot }
\keyword{ aplot }

\name{orditorp}
\alias{orditorp}

\title{ Add Text or Points to Ordination Plots }
\description{
  The function adds \code{\link{text}} or \code{\link{points}} to
  ordination plots.  Text will be used if this can be done without
  overwriting other text labels, and points will be used otherwise. The
  function can help in reducing clutter in ordination graphics, but
  manual editing may still be necessary.
}
\usage{
orditorp(x, display, labels, choices = c(1, 2), priority,
    select, cex = 0.7, pcex, col = par("col"), pcol,
    pch = par("pch"), air = 1, ...)
}

\arguments{
  \item{x}{A result object from ordination or an \code{\link{ordiplot}}
    result. }
  \item{display}{Items to be displayed in the plot.  Only one
    alternative is allowed. Typically this is \code{"sites"} or
    \code{"species"}.  }
  \item{labels}{ Optional text used for labels. Row names will be used if
  this is missing. }
  \item{choices}{ Axes shown.}
  \item{priority}{ Text will be used for items with higher priority
    if labels overlap.  This should be vector of the same
    length as the number of items plotted.}
  \item{select}{Items to be displayed.  This can either be a logical
    vector which is \code{TRUE} for displayed items or a vector of indices
    of displayed items. If a logical vector is used, it must have the
    same length as the scores plotted.}
  \item{cex, pcex}{Text and point sizes, see \code{\link{plot.default}}..}
  \item{col, pcol}{Text and point colours, see \code{\link{plot.default}}.}
  \item{pch}{Plotting character, see \code{\link{points}}.}
  \item{air}{Amount of empty space between text labels. Values <1 allow
    overlapping text.}
  \item{\dots}{ Other arguments to \code{\link{scores}} (and its various
    methods), \code{\link{text}} and \code{\link{points}}.}
}
\details{
  Function \code{orditorp} will add either text or points to an existing
  plot. The items with high \code{priority} will be added first
  and \code{\link{text}} will be used if this can be done without
  overwriting previous labels,and \code{\link{points}} will be used
  otherwise. If \code{priority} is missing, labels will be added from the
  outskirts to the centre. Function \code{orditorp} can be used
  with most ordination results, or plotting results from
  \code{\link{ordiplot}} or ordination plot functions
  (\code{\link{plot.cca}}, \code{\link{plot.decorana}},
  \code{\link{plot.metaMDS}}).

  Arguments can be passed to the relevant \code{\link{scores}} method
  for the ordination object (\code{x}) being drawn. See the relevant
  \code{\link{scores}} help page for arguments that can be used.
}
\value{
  The function returns invisibly a logical vector where \code{TRUE}
  means that item was labelled with text and \code{FALSE} means that it
  was marked with a point. The returned vector can be used as the
  \code{select} argument in ordination \code{text} and \code{points}
  functions.
}

\author{ Jari Oksanen }


\examples{
## A cluttered ordination plot :
data(BCI)
mod <- cca(BCI)
plot(mod, dis="sp", type="t")
# Now with orditorp and abbreviated species names
cnam <- make.cepnames(names(BCI))
plot(mod, dis="sp", type="n")
stems <- colSums(BCI)
orditorp(mod, "sp", label = cnam, priority=stems, pch="+", pcol="grey")

## show select in action
set.seed(1)
take <- sample(ncol(BCI), 50)
plot(mod, dis="sp", type="n")
stems <- colSums(BCI)
orditorp(mod, "sp", label = cnam, priority=stems, select = take,
         pch="+", pcol="grey")
\dontshow{
## example(orditorp) should not set random seed in the user session
rm(.Random.seed)
}
}
\keyword{ aplot }
\keyword{ hplot }

\name{ordisurf}
\alias{ordisurf}
\alias{ordisurf.default}
\alias{ordisurf.formula}
\alias{calibrate.ordisurf}
\alias{plot.ordisurf}

\title{ Fit and Plot Smooth Surfaces of Variables on Ordination. }
\description{
  Function \code{ordisurf} fits a smooth surface for given variable and
  plots the result on ordination diagram.
}
\usage{
\method{ordisurf}{default}(x, y, choices = c(1, 2), knots = 10,
         family = "gaussian", col = "red", isotropic = TRUE,
         thinplate = TRUE, bs = "tp", fx = FALSE, add = FALSE,
         display = "sites", w = weights(x, display), main, nlevels = 10,
         levels, npoints = 31, labcex = 0.6, bubble = FALSE,
         cex = 1, select = TRUE, method = "REML", gamma = 1,
         plot = TRUE, lwd.cl = par("lwd"), ...)

\method{ordisurf}{formula}(formula, data, ...)

\method{calibrate}{ordisurf}(object, newdata, ...)

\method{plot}{ordisurf}(x, what = c("contour","persp","gam"),
     add = FALSE, bubble = FALSE, col = "red", cex = 1,
     nlevels = 10, levels, labcex = 0.6, lwd.cl = par("lwd"), \dots)
}

\arguments{
  \item{x}{For \code{ordisurf} an ordination configuration, either a
    matrix or a result known by \code{\link{scores}}. For
    \code{plot.ordisurf} an object of class \code{"ordisurf"} as
    returned by \code{ordisurf}.}
  \item{y}{Variable to be plotted / modelled as a function of the
    ordination scores.}
  \item{choices}{Ordination axes. }
  \item{knots}{Number of initial knots in \code{\link[mgcv]{gam}} (one
    more than degrees of freedom). If \code{knots = 0} or
    \code{knots = 1}  the function will fit a linear trend surface, and
    if \code{knots = 2} the function  will fit a quadratic trend surface
    instead of a smooth surface. A vector of length 2 is allowed when
    \code{isotropic = FALSE}, with the first and second elements of
    \code{knots} referring to the first and second of ordination
    dimensions (as indicated by \code{choices}) respectively.}
  \item{family}{Error distribution in \code{\link[mgcv]{gam}}.}
  \item{col}{ Colour of contours. }
  \item{isotropic, thinplate}{Fit an isotropic smooth surface (i.e. same
    smoothness in both ordination dimensions) via
    \code{\link[mgcv]{gam}}. Use of \code{thinplate} is deprecated and
    will be removed in a future version of the package.}
  \item{bs}{a two letter character string indicating the smoothing basis
    to use. (e.g. \code{"tp"} for thin plate regression spline,
    \code{"cr"} for cubic regression spline). One of \code{c("tp", "ts",
      "cr", "cs", "ds", "ps", "ad")}. See
    \code{\link[mgcv]{smooth.terms}} for an over view of what these
    refer to. The default is to use thin plate splines: \code{bs = "tp"}.}
  \item{fx}{indicates whether the smoothers are fixed degree of freedom
    regression splines (\code{fx = FALSE}) or penalised regression
    splines (\code{fx = TRUE}). Can be a vector of length 2 for
    anisotropic surfaces (\code{isotropic = FALSE}). It doesn't make
    sense to use \code{fx = TRUE} \strong{and} \code{select = TRUE} and
    it is an \strong{error} to do so. A warning is issued if you specify
    \code{fx = TRUE} and forget to use \code{select = FALSE} though
    fitting continues using \code{select = FALSE}.}
  \item{add}{Add contours to an existing diagram or draw a new plot?}
  \item{display}{Type of scores known by \code{\link{scores}}: typically
    "sites" for ordinary site scores or "lc" for linear combination scores.}
  \item{w}{Prior weights on the data. Concerns mainly \code{\link{cca}}
    and \code{\link{decorana}} results which have nonconstant weights.}
  \item{main}{The main title for the plot, or as default the name of
    plotted variable in a new plot.}
  \item{nlevels, levels}{Either a vector of \code{levels} for which contours
    are drawn, or suggested number of contours in \code{nlevels} if
    \code{levels} are not supplied.}
  \item{npoints}{numeric; the number of locations at which to evaluate
    the fitted surface. This represents the number of locations in each
    dimension.}
  \item{labcex}{Label size in contours.  Setting this zero will suppress
    labels.}
  \item{bubble}{Use a \dQuote{bubble plot} for points, or vary the point
    diameter by the value of the plotted variable. If \code{bubble} is
    numeric, its value is used for the maximum symbol size (as in
    \code{cex}), or if \code{bubble = TRUE}, the value of \code{cex} gives
    the maximum. The minimum size will always be \code{cex = 0.4}.  The
    option only has an effect if \code{add = FALSE}.}
  \item{cex}{Character expansion of plotting symbols.}
  \item{select}{Logical; specify \code{\link[mgcv]{gam}} argument
    \code{"select"}. If this is \code{TRUE} then \code{\link[mgcv]{gam}} can
    add an extra  penalty to each term so that it can be penalized to
    zero. This means that the smoothing parameter estimation that is part
    of fitting can completely remove terms from the model. If the
    corresponding smoothing parameter is estimated as zero then the extra
    penalty has no effect.}
  \item{method}{character; the smoothing parameter estimation
    method. Options allowed are: \code{"GCV.Cp"} uses GCV for models with
    unknown scale parameter and Mallows' Cp/UBRE/AIC for models with
    known scale; \code{"GACV.Cp"} as for \code{"GCV.Cp"} but uses GACV
    (Generalised Approximate CV) instead of GCV; \code{"REML"} and
    \code{"ML"} use restricted maximum likelihood or maximum likelihood
    estimation for both known and unknown scale; and \code{"P-REML"} and
    \code{"P-ML"} use REML or ML estimation but use a Pearson estimate
    of the scale.}
  \item{gamma}{Multiplier to inflate model degrees of freedom in GCV or
    UBRE/AIC score by. This effectively places an extra penalty on
    complex models. An oft-used value is \code{gamma = 1.4}.}
  \item{plot}{logical; should any plotting be done by
    \code{ordisurf}? Useful if all you want is the fitted response
    surface model.}
  \item{lwd.cl}{numeric; the \code{lwd} (line width) parameter to use
    when drawing the contour lines.}
  \item{formula, data}{Alternative definition of the fitted model as
    \code{x ~ y}, where left-hand side is the ordination \code{x} and
    right-hand side the single fitted continuous variable
    \code{y}. The variable \code{y} must be in the working environment
    or in the data frame or environment given by \code{data}. All
    other arguments of are passed to the default method.}
  \item{object}{An \code{ordisurf} result object.}
  \item{newdata}{Coordinates in two-dimensional ordination for new
    points.}
  \item{what}{character; what type of plot to produce. \code{"contour"}
    produces a contour plot of the response surface, see
    \code{\link{contour}} for details. \code{"persp"} produces a
    perspective plot of the same, see \code{\link{persp}} for
    details. \code{"gam"} plots the fitted GAM model, an object that
    inherits from class \code{"gam"} returned by \code{ordisurf}, see
    \code{\link[mgcv]{plot.gam}}.}
  \item{\dots}{Other parameters passed to \code{\link{scores}}, or
    to the graphical functions. See Note below for exceptions.}
}

\details{

  Function \code{ordisurf} fits a smooth surface using penalised
  splines (Wood 2003) in \code{\link[mgcv]{gam}}, and uses
  \code{\link[mgcv]{predict.gam}} to find fitted values in a regular
  grid. The smooth surface can be fitted with an extra penalty that
  allows the entire smoother to be penalized back to 0 degrees of
  freedom, effectively removing the term from the model (see Marra &
  Wood, 2011). The addition of this extra penalty is invoked by
  setting argument \code{select} to \code{TRUE}. An alternative is to
  use a spline basis that includes shrinkage (\code{bs = "ts"} or
  \code{bs = "cs"}).

  \code{ordisurf()} exposes a large number of options from
  \code{\link[mgcv]{gam}} for specifying the basis functions used for
  the surface. If you stray from the defaults, do read the
  \strong{Notes} section below and relevant documentation in
  \code{\link[mgcv]{s}} and \code{\link[mgcv]{smooth.terms}}.

  The function plots the fitted contours with convex hull of data points
  either over an existing ordination diagram or draws a new plot. If
  \code{select = TRUE} and the smooth is effectively penalised out of
  the model, no contours will be plotted.

  \code{\link[mgcv]{gam}} determines the degree of smoothness for the
  fitted response surface during model fitting, unless \code{fx =
  TRUE}. Argument \code{method} controls how \code{\link[mgcv]{gam}}
  performs this smoothness selection. See \code{\link[mgcv]{gam}} for
  details of the available options. Using \code{"REML"} or \code{"ML"}
  yields p-values for smooths with the best coverage properties if such
  things matter to you.

  The function uses \code{\link{scores}} to extract ordination scores,
  and \code{x} can be any result object known by that function.

  The user can supply a vector of prior weights \code{w}. If the
  ordination object has weights, these will be used. In practise this
  means that the row totals are used as weights with \code{\link{cca}}
  or \code{\link{decorana}} results. If you do not like this, but want
  to give equal weights to all sites, you should set \code{w =
  NULL}. The behaviour is consistent with \code{\link{envfit}}. For
  complete accordance with constrained \code{\link{cca}}, you should set
  \code{display = "lc"}.

  Function \code{calibrate} returns the fitted values of the response
  variable. The \code{newdata} must be coordinates of points for which
  the fitted values are desired. The function is based on
  \code{\link[mgcv]{predict.gam}} and will pass extra arguments to
  that function.
}

\value{
  \code{ordisurf} is usually called for its side effect of drawing the
  contour plot. The function returns a result object of class
  \code{"ordisurf"} that inherits from \code{\link[mgcv]{gam}} used
  internally to fit the surface, but adds an item \code{grid} that
  contains the data for the grid surface. The item \code{grid} has
  elements \code{x} and \code{y} which are vectors of axis coordinates,
  and element \code{z} that is a matrix of fitted values for
  \code{\link{contour}}. The values outside the convex hull of observed
  points are indicated as \code{NA} in \code{z}. The
  \code{\link[mgcv]{gam}} component of the result can be used for
  further analysis like predicting new values (see
  \code{\link[mgcv]{predict.gam}}).
}

\author{ Dave Roberts, Jari Oksanen and Gavin L. Simpson }
\note{
  The default is to use an isotropic smoother via
  \code{\link[mgcv]{s}} employing thin plate regression splines
  (\code{bs = "tp"}). These make sense in ordination as they have
  equal smoothing in all directions and are rotation invariant. However,
  if different degrees of smoothness along dimensions are required, an
  anisotropic smooth surface may be more applicable. This can be
  achieved through the use of \code{isotropic = FALSE}, wherein the
  surface is fitted via a tensor product smoother via
  \code{\link[mgcv]{te}} (unless \code{bs = "ad"}, in which case
  separate splines for each dimension are fitted using
  \code{\link[mgcv]{s}}).

  Cubic regression splines and P splines can \strong{only} be used with
  \code{isotropic = FALSE}.

  Adaptive smooths (\code{bs = "ad"}), especially in two dimensions,
  require a large number of observations; without many hundreds of
  observations, the default complexities for the smoother will exceed
  the number of observations and fitting will fail.

  To get the old behaviour of \code{ordisurf} use \code{select = FALSE},
  \code{method = "GCV.Cp"}, \code{fx = FALSE}, and \code{bs = "tp"}. The
  latter two options are the current defaults.

  Graphical arguments supplied to \code{plot.ordisurf} are passed on to
  the underlying plotting functions, \code{contour}, \code{persp}, and
  \code{\link[mgcv]{plot.gam}}. The exception to this is that arguments
  \code{col} and \code{cex} can not currently be passed to
  \code{\link[mgcv]{plot.gam}} because of a bug in the way that function
  evaluates arguments when arranging the plot.

  A work-around is to call \code{\link[mgcv]{plot.gam}} directly on the
  result of a call to \code{ordisurf}. See the Examples for an
  illustration of this.
}

\section{Warning}{
  The fitted GAM is a regression model and has the usual assumptions of
  such models. Of particular note is the assumption of independence of
  residuals. If the observations are not independent (e.g. they are
  repeat measures on a set of objects, or from an experimental design,
  \emph{inter alia}) do not trust the \emph{p}-values from the GAM
  output.

  If you need further control (i.e. to add additional fixed effects to
  the model, or use more complex smoothers), extract the ordination
  scores using the \code{scores} function and then generate your own
  \code{\link[mgcv]{gam}} call.
}

\references{

  Marra, G.P & Wood, S.N. (2011) Practical variable selection for
  generalized additive models. \emph{Comput. Stat. Data Analysis} 55,
  2372--2387.

  Wood, S.N. (2003) Thin plate regression splines.
  \emph{J. R. Statist. Soc. B} 65, 95--114.

}

\seealso{ For basic routines \code{\link[mgcv]{gam}},
  and \code{\link{scores}}. Function
  \code{\link{envfit}} provides a more traditional and compact
  alternative. }

\examples{
data(varespec)
data(varechem)
vare.dist <- vegdist(varespec)
vare.mds <- monoMDS(vare.dist)
ordisurf(vare.mds ~ Baresoil, varechem, bubble = 5)

## as above but without the extra penalties on smooth terms,
## and using GCV smoothness selection (old behaviour of `ordisurf()`):
ordisurf(vare.mds ~ Baresoil, varechem, col = "blue", add = TRUE,
                        select = FALSE, method = "GCV.Cp")

## Cover of Cladina arbuscula
fit <- ordisurf(vare.mds ~ Cladarbu, varespec, family=quasipoisson)
## Get fitted values
calibrate(fit)

## Variable selection via additional shrinkage penalties
## This allows non-significant smooths to be selected out
## of the model not just to a linear surface. There are 2
## options available:
##  - option 1: `select = TRUE` --- the *default*
ordisurf(vare.mds ~ Baresoil, varechem, method = "REML", select = TRUE)
##  - option 2: use a basis with shrinkage
ordisurf(vare.mds ~ Baresoil, varechem, method = "REML", bs = "ts")
## or bs = "cs" with `isotropic = FALSE`

## Plot method
plot(fit, what = "contour")

## Plotting the "gam" object
plot(fit, what = "gam") ## 'col' and 'cex' not passed on
## or via plot.gam directly
library(mgcv)
plot.gam(fit, cex = 2, pch = 1, col = "blue")
## 'col' effects all objects drawn...

### controlling the basis functions used
## Use Duchon splines
ordisurf(vare.mds ~ Baresoil, varechem, bs = "ds")

## A fixed degrees of freedom smooth, must use 'select = FALSE'
ordisurf(vare.mds ~ Baresoil, varechem, knots = 4,
                        fx = TRUE, select = FALSE)

## An anisotropic smoother with cubic regression spline bases
ordisurf(vare.mds ~ Baresoil, varechem, isotropic = FALSE,
                        bs = "cr", knots = 4)

## An anisotropic smoother with cubic regression spline with
## shrinkage bases & different degrees of freedom in each dimension
ordisurf(vare.mds ~ Baresoil, varechem, isotropic = FALSE,
                        bs = "cs", knots = c(3,4), fx = TRUE,
                        select = FALSE)
}
\keyword{ multivariate }
\keyword{ aplot }

\encoding{UTF-8}
\name{tsallis}
\alias{tsallis}
\alias{tsallisaccum}
\alias{persp.tsallisaccum}
\title{Tsallis Diversity and Corresponding Accumulation Curves}
\description{
Function \code{tsallis} find Tsallis diversities with any scale or the corresponding evenness measures. Function \code{tsallisaccum} finds these statistics with accumulating sites.
}
\usage{
tsallis(x, scales = seq(0, 2, 0.2), norm = FALSE, hill = FALSE)
tsallisaccum(x, scales = seq(0, 2, 0.2), permutations = 100, 
   raw = FALSE, subset, ...)
\method{persp}{tsallisaccum}(x, theta = 220, phi = 15, col = heat.colors(100), zlim, ...)
}

\arguments{
  \item{x}{Community data matrix or plotting object. }
  \item{scales}{Scales of Tsallis diversity.}

  \item{norm}{Logical, if \code{TRUE} diversity values are normalized
    by their maximum (diversity value at equiprobability conditions).}

  \item{hill}{Calculate Hill numbers.}

  \item{permutations}{Usually an integer giving the number
    permutations, but can also be a list of control values for the
    permutations as returned by the function \code{\link[permute]{how}}, 
    or a permutation matrix where each row gives the permuted indices.}

  \item{raw}{If \code{FALSE} then return summary statistics of
    permutations, and if TRUE then returns the individual
    permutations.}

  \item{subset}{logical expression indicating sites (rows) to keep:
    missing values are taken as \code{FALSE}.}

  \item{theta, phi}{angles defining the viewing
    direction. \code{theta} gives the azimuthal direction and
    \code{phi} the colatitude.}
  
  \item{col}{Colours used for surface.}  \item{zlim}{Limits of
  vertical axis.}  

  \item{\dots}{Other arguments which are passed to \code{tsallis} and
    to graphical functions.}

} 

\details{ The Tsallis diversity (also equivalent to Patil and Taillie
diversity) is a one-parametric generalised entropy function, defined
as:

\deqn{H_q = \frac{1}{q-1} (1-\sum_{i=1}^S p_i^q)}{H.q = 1/(q-1)(1-sum(p^q))}

where \eqn{q} is a scale parameter, \eqn{S} the number of species in
the sample (Tsallis 1988, Tothmeresz 1995). This diversity is concave
for all \eqn{q>0}, but non-additive (Keylock 2005). For \eqn{q=0} it
gives the number of species minus one, as \eqn{q} tends to 1 this
gives Shannon diversity, for \eqn{q=2} this gives the Simpson index
(see function \code{\link{diversity}}).

If \code{norm = TRUE}, \code{tsallis} gives values normalized by the
maximum:

\deqn{H_q(max) = \frac{S^{1-q}-1}{1-q}}{H.q(max) = (S^(1-q)-1)/(1-q)}

where \eqn{S} is the number of species. As \eqn{q} tends to 1, maximum
is defined as \eqn{ln(S)}.

If \code{hill = TRUE}, \code{tsallis} gives Hill numbers (numbers
equivalents, see Jost 2007):

\deqn{D_q = (1-(q-1) H)^{1/(1-q)}}{D.q = (1-(q-1)*H)^(1/(1-q))}

Details on plotting methods and accumulating values can be found on
the help pages of the functions \code{\link{renyi}} and
\code{\link{renyiaccum}}.  
}

\value{ 
Function \code{tsallis} returns a data frame of selected
indices. Function \code{tsallisaccum} with argument \code{raw = FALSE}
returns a three-dimensional array, where the first dimension are the
accumulated sites, second dimension are the diversity scales, and
third dimension are the summary statistics \code{mean}, \code{stdev},
\code{min}, \code{max}, \code{Qnt 0.025} and \code{Qnt 0.975}. With
argument \code{raw = TRUE} the statistics on the third dimension are
replaced with individual permutation results.  }

\references{

Tsallis, C. (1988) Possible generalization of Boltzmann-Gibbs
  statistics.  \emph{J. Stat. Phis.} 52, 479--487.

Tothmeresz, B. (1995) Comparison of different methods for diversity
  ordering. \emph{Journal of Vegetation Science} \bold{6}, 283--290.

Patil, G. P. and Taillie, C. (1982) Diversity as a concept and its
  measurement.  \emph{J. Am. Stat. Ass.} \bold{77}, 548--567.

Keylock, C. J. (2005) Simpson diversity and the Shannon-Wiener index
  as special cases of a generalized entropy.  \emph{Oikos} \bold{109},
  203--207.

Jost, L (2007) Partitioning diversity into independent alpha and beta
  components.  \emph{Ecology} \bold{88}, 2427--2439.
}

\author{\enc{Péter Sólymos}{Peter Solymos},
\email{solymos@ualberta.ca}, based on the code of Roeland Kindt and
Jari Oksanen written for \code{renyi}}

\seealso{ Plotting methods and accumulation routines are based on
functions \code{\link{renyi}} and \code{\link{renyiaccum}}. An object
of class 'tsallisaccum' can be used with function
\code{\link[vegan3d]{rgl.renyiaccum}} as well. See also settings for
\code{\link{persp}}.  }

\examples{
data(BCI)
i <- sample(nrow(BCI), 12)
x1 <- tsallis(BCI[i,])
x1
diversity(BCI[i,],"simpson") == x1[["2"]]
plot(x1)
x2 <- tsallis(BCI[i,],norm=TRUE)
x2
plot(x2)
mod1 <- tsallisaccum(BCI[i,])
plot(mod1, as.table=TRUE, col = c(1, 2, 2))
persp(mod1)
mod2 <- tsallisaccum(BCI[i,], norm=TRUE)
persp(mod2,theta=100,phi=30)
}
\keyword{multivariate}

\name{fisherfit}
\alias{fisherfit}
\alias{as.fisher}
\alias{plot.fisherfit}
\alias{prestonfit}
\alias{prestondistr}
\alias{as.preston}
\alias{plot.prestonfit}
\alias{lines.prestonfit}
\alias{plot.preston}
\alias{lines.preston}
\alias{plot.fisher}
\alias{veiledspec}

\title{Fit Fisher's Logseries and Preston's Lognormal Model to Abundance Data}
\description{
  Function \code{fisherfit} fits Fisher's logseries to abundance
  data. Function \code{prestonfit} groups species frequencies into
  doubling octave classes and fits Preston's lognormal model, and
  function \code{prestondistr} fits the truncated lognormal model
  without pooling the data into octaves.
}
\usage{
fisherfit(x, ...)
prestonfit(x, tiesplit = TRUE, ...)
prestondistr(x, truncate = -1, ...)
\method{plot}{prestonfit}(x, xlab = "Frequency", ylab = "Species", bar.col = "skyblue", 
    line.col = "red", lwd = 2, ...)
\method{lines}{prestonfit}(x, line.col = "red", lwd = 2, ...)
veiledspec(x, ...)
as.fisher(x, ...)
\method{plot}{fisher}(x, xlab = "Frequency", ylab = "Species", bar.col = "skyblue",
             kind = c("bar", "hiplot", "points", "lines"), add = FALSE, ...)
as.preston(x, tiesplit = TRUE, ...)
\method{plot}{preston}(x, xlab = "Frequency", ylab = "Species", bar.col = "skyblue", ...)
\method{lines}{preston}(x, xadjust = 0.5, ...)
}

\arguments{
  \item{x}{Community data vector for fitting functions or their result
    object for \code{plot} functions.}
  \item{tiesplit}{Split frequencies \eqn{1, 2, 4, 8} etc between adjacent 
    octaves.}
  \item{truncate}{Truncation point for log-Normal model, in log2
    units. Default value \eqn{-1} corresponds to the left border of zero
    Octave. The choice strongly influences the fitting results.}
  \item{xlab, ylab}{Labels for \code{x} and \code{y} axes.}
  \item{bar.col}{Colour of data bars.}
  \item{line.col}{Colour of fitted line.}
  \item{lwd}{Width of fitted line.}
  \item{kind}{Kind of plot to drawn: \code{"bar"} is similar bar plot
    as in \code{plot.fisherfit}, \code{"hiplot"} draws vertical lines
    as with \command{plot(..., type="h")}, and \code{"points"} and
    \code{"lines"} are obvious.}
  \item{add}{Add to an existing plot.}
  \item{xadjust}{Adjustment of horizontal positions in octaves.}
  \item{\dots}{Other parameters passed to functions. Ignored in 
    \code{prestonfit} and \code{tiesplit} passed to \code{as.preston} in
    \code{prestondistr}. }
}
\details{
  In Fisher's logarithmic series the expected
  number of species \eqn{f} with \eqn{n} observed individuals is
  \eqn{f_n = \alpha x^n / n} (Fisher et al. 1943).
  The estimation is possible only for genuine
  counts of individuals. The parameter \eqn{\alpha} is used as a
  diversity index, and \eqn{\alpha} and its standard error can be
  estimated with a separate function \code{\link{fisher.alpha}}. The
  parameter \eqn{x} is taken as a nuisance parameter which is not
  estimated separately but taken to be \eqn{n/(n+\alpha)}. Helper
  function \code{as.fisher} transforms abundance data into Fisher
  frequency table.

  Preston (1948) was not satisfied with Fisher's model which seemed to
  imply infinite species richness, and postulated that rare species is
  a diminishing class and most species are in the middle of frequency
  scale. This was achieved by collapsing higher frequency classes into
  wider and wider \dQuote{octaves} of doubling class limits: 1, 2, 3--4,
  5--8, 9--16 etc. occurrences. It seems that Preston regarded
  frequencies 1, 2, 4, \emph{etc.}. as \dQuote{tied} between octaves
  (Williamson & Gaston 2005). This means that only half of the species
  with frequency 1 are shown in the lowest octave, and the rest are
  transferred to the second octave. Half of the species from the
  second octave are transferred to the higher one as well, but this is
  usually not as large a number of species. This practise makes data
  look more lognormal by reducing the usually high lowest
  octaves. This can be achieved by setting argument \code{tiesplit = TRUE}. 
  With \code{tiesplit = FALSE} the frequencies are not split,
  but all ones are in the lowest octave, all twos in the second, etc.
  Williamson & Gaston (2005) discuss alternative definitions in
  detail, and they should be consulted for a critical review of
  log-Normal model.

  Any logseries data will look like lognormal when plotted in
  Preston's way. The expected frequency \eqn{f} at abundance octave
  \eqn{o} is defined by \eqn{f_o = S_0 \exp(-(\log_2(o) -
  \mu)^2/2/\sigma^2)}{f = S0 exp(-(log2(o)-mu)^2/2/sigma^2)}, where
  \eqn{\mu} is the location of the mode and \eqn{\sigma} the width,
  both in \eqn{\log_2}{log2} scale, and \eqn{S_0}{S0} is the expected
  number of species at mode. The lognormal model is usually truncated
  on the left so that some rare species are not observed. Function
  \code{prestonfit} fits the truncated lognormal model as a second
  degree log-polynomial to the octave pooled data using Poisson (when
  \code{tiesplit = FALSE}) or quasi-Poisson (when \code{tiesplit = TRUE})
  error.  Function \code{prestondistr} fits left-truncated
  Normal distribution to \eqn{\log_2}{log2} transformed non-pooled
  observations with direct maximization of log-likelihood. Function
  \code{prestondistr} is modelled after function
  \code{\link[MASS]{fitdistr}} which can be used for alternative
  distribution models. 

  The functions have common \code{print}, \code{plot} and \code{lines}
  methods. The \code{lines} function adds the fitted curve to the
  octave range with line segments showing the location of the mode and
  the width (sd) of the response. Function \code{as.preston}
  transforms abundance data to octaves.  Argument \code{tiesplit} will
  not influence the fit in \code{prestondistr}, but it will influence
  the barplot of the octaves. 

  The total extrapolated richness from a fitted Preston model can be
  found with function \code{veiledspec}. The function accepts results
  both from \code{prestonfit} and from \code{prestondistr}. If
  \code{veiledspec} is called with a species count vector, it will
  internally use \code{prestonfit}. Function \code{\link{specpool}}
  provides alternative ways of estimating the number of unseen
  species. In fact, Preston's lognormal model seems to be truncated at
  both ends, and this may be the main reason why its result differ
  from lognormal models fitted in Rank--Abundance diagrams with
  functions \code{\link{rad.lognormal}}.  
}

\value{ The function \code{prestonfit} returns an object with fitted
  \code{coefficients}, and with observed (\code{freq}) and fitted
  (\code{fitted}) frequencies, and a string describing the fitting
  \code{method}. Function \code{prestondistr} omits the entry
  \code{fitted}.  The function \code{fisherfit} returns the result of
  \code{\link{nlm}}, where item \code{estimate} is \eqn{\alpha}. The
  result object is amended with the \code{nuisance} parameter and item
  \code{fisher} for the observed data from \code{as.fisher}
}
\references{
  Fisher, R.A., Corbet, A.S. & Williams, C.B. (1943). The relation
  between the number of species and the number of individuals in a
  random sample of animal population. \emph{Journal of Animal Ecology}
  12: 42--58.

  Preston, F.W. (1948) The commonness and rarity of
  species. \emph{Ecology} 29, 254--283.

  Williamson, M. & Gaston, K.J. (2005). The lognormal distribution is
  not an appropriate null hypothesis for the species--abundance
  distribution. \emph{Journal of Animal Ecology} 74, 409--422.
}

\author{Bob O'Hara and Jari Oksanen. }

\seealso{\code{\link{diversity}}, \code{\link{fisher.alpha}},
  \code{\link{radfit}}, \code{\link{specpool}}. Function
  \code{\link[MASS]{fitdistr}} of \pkg{MASS} package was used as the
  model for \code{prestondistr}. Function \code{\link{density}} can be used for
  smoothed non-parametric estimation of responses, and
  \code{\link{qqplot}} is an alternative, traditional and more effective
  way of studying concordance of observed abundances to any distribution model.
}
\examples{
data(BCI)
mod <- fisherfit(BCI[5,])
mod
# prestonfit seems to need large samples
mod.oct <- prestonfit(colSums(BCI))
mod.ll <- prestondistr(colSums(BCI))
mod.oct
mod.ll
plot(mod.oct)  
lines(mod.ll, line.col="blue3") # Different
## Smoothed density
den <- density(log2(colSums(BCI)))
lines(den$x, ncol(BCI)*den$y, lwd=2) # Fairly similar to mod.oct
## Extrapolated richness
veiledspec(mod.oct)
veiledspec(mod.ll)
}
\keyword{ univar }
\keyword{ distribution }

\name{ordipointlabel}
\alias{ordipointlabel}
\alias{plot.ordipointlabel}

\title{ Ordination Plots with Points and Optimized Locations for Text }
\description{
  The function \code{ordipointlabel} produces ordination plots with
  points and text label to the points. The points are in the exact
  location given by the ordination, but the function tries to optimize
  the location of the text labels to minimize overplotting text. The
  function may be useful with moderately crowded ordination plots.
}
\usage{
ordipointlabel(x, display = c("sites", "species"), choices = c(1, 2),
   col = c(1, 2),  pch = c("o", "+"), font = c(1, 1), 
   cex = c(0.8, 0.8), add = FALSE, select, ...)

\method{plot}{ordipointlabel}(x, ...)
}

\arguments{
  \item{x}{For \code{ordipointlabel()} a result object from an
    ordination function. For \code{plot.ordipointlabel} an object
    resulting from a call to \code{ordipointlabel()}.}
  \item{display}{Scores displayed in the plot. }
  \item{choices}{Axes shown. }
  \item{col, pch, font, cex}{Colours, point types, font style and
     character expansion for each kind of scores displayed in the
     plot. These should be vectors of the same length as the number of
     items in \code{display}.}
  \item{add}{ Add to an existing plot. }
  \item{select}{Items to be displayed.  This can either be a logical
    vector which is \code{TRUE} for displayed items or a vector of indices
    of displayed items. \code{select} is only used if a single set of
    scores is being plotted (i.e. \code{length(display) == 1}),
    otherwise it is ignored and a warning issued. If a logical vector is
    used, it must have the same length as the scores plotted.}
  \item{\dots}{Other arguments passed to \code{\link{points}} and
  \code{\link{text}}.}
}
\details{
  The function uses simulated annealing (\code{\link{optim}},
  \code{method = "SANN"}) to optimize the location of the text labels
  to the points. There are eight possible locations: up, down, sides
  and corners. There is a weak preference to text right above the
  point, and a weak avoidance of corner positions. The exact locations
  and the goodness of solution varies between runs, and there is no
  guarantee of finding the global optimum. The optimization can take a
  long time in difficult cases with a high number of potential
  overlaps. Several sets of scores can be displayed in one plot. 

  The function is modelled after \code{\link[maptools]{pointLabel}} in
  \pkg{maptools} package (which has chained dependencies of \code{S4}
  packages). 
}
\value{
  The function returns invisibly an object of class
  \code{ordipointlabel} with items \code{xy} for coordinates of
  points, \code{labels} for coordinates of labels, items \code{pch},
  \code{cex} and \code{font} for graphical parameters of each point or
  label. In addition, it returns the result of \code{\link{optim}} as
  an attribute \code{"optim"}. The unit of overlap is the area
  of character \code{"m"}, and with variable \code{cex} it is the
  smallest alternative.

  There is a \code{plot} method based on \code{orditkplot} but which
  does not alter nor reset the graphical parameters via \code{par}.
  
  The result object from \code{ordipointlabel} inherits from
  \code{\link{orditkplot}}, and can also be replotted with its
  \code{plot} method. It may be possible to further edit the result
  object with \code{\link{orditkplot}}, but for good results it is
  necessary that the points span the whole horizontal axis without empty
  margins. 
}
\references{ See \code{\link[maptools]{pointLabel}} for references. }
\author{ Jari Oksanen }
\note{ 
  The function is designed for ordination graphics, and the
  optimization works properly with plots of isometric aspect ratio.
}
\seealso{ \code{\link[maptools]{pointLabel}} for the model
  implementation, and \code{\link{optim}} for the optimization. }
\examples{
data(dune)
ord <- cca(dune)
plt <- ordipointlabel(ord)

## set scaling - should be no warnings!
ordipointlabel(ord, scaling = "sites")

## plot then add
plot(ord, scaling = "symmetric", type = "n")
ordipointlabel(ord, display = "species", scaling = "symm", add = TRUE)
ordipointlabel(ord, display = "sites", scaling = "symm", add = TRUE)

## redraw plot without rerunning SANN optimisation
plot(plt)
}
\keyword{ hplot }
\keyword{ aplot }

\name{tolerance}
\alias{tolerance}
\alias{tolerance.cca}

\title{Species tolerances and sample heterogeneities}
\usage{
tolerance(x, \dots)

\method{tolerance}{cca}(x, choices = 1:2, which = c("species","sites"),
          scaling = "species", useN2 = TRUE, hill = FALSE, \dots)
}
\description{
  Species tolerances and sample heterogeneities.
}
\details{
  Function to compute species tolerances and site heterogeneity measures
  from unimodal ordinations (CCA & CA). Implements Eq 6.47 and 6.48 from
  the Canoco 4.5 Reference Manual (pages 178--179).
}
\value{
  Matrix of tolerances/heterogeneities with some additional
  attributes: \code{which}, \code{scaling}, and \code{N2}, the latter of
  which will be \code{NA} if \code{useN2 = FALSE}.
}
\author{Gavin L. Simpson}
\arguments{
  \item{x}{object of class \code{"cca"}.}
  \item{choices}{numeric; which ordination axes to compute
    tolerances and heterogeneities for. Defaults to axes 1 and 2.}
  \item{which}{character; one of \code{"species"} or \code{"sites"},
    indicating whether species tolerances or sample heterogeneities
    respectively are computed.}
  \item{scaling}{character or numeric; the ordination scaling to
    use. See \code{\link{scores.cca}} for details.}
  \item{hill}{logical; if \code{scaling} is a character,
    these control whether Hill's scaling is used for (C)CA
    respectively. See \code{\link{scores.cca}} for details.}
  \item{useN2}{logical; should the bias in the tolerances /
    heterogeneities be reduced via scaling by Hill's N2?}
  \item{\dots}{arguments passed to other methods.}
}
\examples{
data(dune)
data(dune.env)
mod <- cca(dune ~ ., data = dune.env)

## defaults to species tolerances
tolerance(mod)

## sample heterogeneities for CCA axes 1:6
tolerance(mod, which = "sites", choices = 1:6)
}

\encoding{UTF-8}
\name{avgdist}
\alias{avgdist}
\title{Averaged Subsampled Dissimilarity Matrices}
\description{
  The function computes the dissimilarity matrix of a dataset multiple
  times using \code{\link{vegdist}} while randomly subsampling the
  dataset each time. All of the subsampled iterations are then averaged
  (mean) to provide a distance matrix that represents the average of
  multiple subsampling iterations. This emulates the behavior of the
  distance matrix calculator within the Mothur microbial ecology toolkit.
}

\usage{
avgdist(x, sample, distfun = vegdist, meanfun = mean,
    transf = NULL, iterations = 100, dmethod = "bray", ...)
}
\arguments{
  \item{x}{Community data matrix.}
  \item{sample}{The subsampling depth to be used in each iteration. Samples that do not meet this threshold will be removed from the analysis, and their identify returned to the user in stdout.}
  \item{distfun}{The dissimilarity matrix function to be used. Default is the vegan \code{\link{vegdist}}}
  \item{meanfun}{The calculation to use for the average (mean or median).}
  \item{transf}{Option for transforming the count data before calculating the distance matrix. Any base transformation option can be used (e.g. \code{\link{sqrt}})}
  \item{iterations}{The number of random iterations to perform before averaging. Default is 100 iterations.}
  \item{dmethod}{Dissimilarity index to be used with the specified dissimilarity matrix function. Default is Bray-Curtis}
  \item{...}{Any additional arguments to add to the distance function or mean/median function specified.}
}

\author{ Geoffrey Hannigan. }

\note{

  The function builds on the function \code{\link{rrarefy}} and and
  additional distance matrix function (e.g. \code{\link{vegdist}}) to
  add more meaningful representations of distances among randomly
  subsampled datasets by presenting the average of multiple random
  iterations. This function runs using the \code{\link{vegdist}}. This
  functionality has been utilized in the Mothur standalone microbial
  ecology toolkit
  \href{https://www.mothur.org/wiki/Dist.shared#subsample}{here}.

}

\seealso{
 This function utilizes the \code{\link{vegdist}} and \code{\link{rrarefy}}
 functions.}

\examples{
# Import an example count dataset
data(BCI)
# Test the base functionality
mean.avg.dist <- avgdist(BCI, sample = 50, iterations = 10)
# Test the transformation function
mean.avg.dist.t <- avgdist(BCI, sample = 50, iterations = 10, transf = sqrt)
# Test the median functionality
median.avg.dist <- avgdist(BCI, sample = 50, iterations = 10, meanfun = median)
# Print the resulting tables
head(as.matrix(mean.avg.dist))
head(as.matrix(mean.avg.dist.t))
head(as.matrix(median.avg.dist))
# Run example to illustrate low variance of mean, median, and stdev results
# Mean and median std dev are around 0.05
sdd <- avgdist(BCI, sample = 50, iterations = 100, meanfun = sd)
summary(mean.avg.dist)
summary(median.avg.dist)
summary(sdd)
# Test for when subsampling depth excludes some samples
# Return samples that are removed for not meeting depth filter
depth.avg.dist <- avgdist(BCI, sample = 450, iterations = 10)
# Print the result
depth.avg.dist
}
\keyword{ multivariate }
